#!/usr/bin/perl

## @file userconfig.pl
# @brief This script provides accounts management for applications
# @details
# @param help      print this help
# @param adduser   add new user account
# @li     --username      - name of the user (mandatory)
# @li     --type          - type of the user ('Linux_User', 'Linux_FTP_User', 'Linux_SFTP_User', 'Linux_FTP_SFTP_User', 'UCM_User')
# @li     --pwd           - user's password (MD5 hash)
# @li     --nopasswd      - create user with blank password
# @li     --shell         - if this option is defined then restricted bash shell is used, otherwise /sbin/nologin.
# @li     --initial_group - initial group user belongs to
# @li     --supp_group    - list of supplementary groups, delimited by commas
# @li     --app_name      - name of the application user belongs to.
# @li     --uid           - user's UID
# @li     --exp_date      - expire date
# @param deluser   delete user accounts
# @li     --username      - name of the user for deletion (mandatory)
# @li     --type          - type of the user (by default is 'Linux_User')
# @param modifyuser change user's properties
# @li     --username      - name of the user (mandatory)
# @li     --pwd           - new password (MD5 hash)
# @li     --initial_group - initial group user belongs to
# @li     --supp_group    - list of supplementary groups, delimited by commas
# @li     --app_name      - name of the application user belongs to.
# @li     --exp_date      - expire date
# @author Evgeny Lazarev
# @date December 2007
# @version 1.00 - Initial version
# @version 1.01 - Changes in input parameters (--shell is a key)
# @version 1.02 - Alexander Khoroshilov: added --exp_date and --nopasswd options
# @note if uid is less than 500, then the user must be nologin.
# @version 1.03 - support of the key '--quiet' has been added.

=pod

=head1 NAME

B<userconfig> - tool for the managing user accounts.

=head1 SYNOPSIS

B<userconfig> [B<help>]|[B<--adduser>]|[B<--deluser>]|[B<--modifyuser>]

B<userconfig> B<adduser> | [B<--username> I<username>] | [B<--type> I<type>] | [B<--pwd> I<password>] | [B<--initial_group>
I<initial_group>] | [B<--supp_group> I<supp_group>] | [B<--app_name> I<app_name>] | [B<--shell>] | [B<--uid> I<uid>] |
[B<--exp_date> I<exp_date>] | [B<--nopasswd>]

B<userconfig> B<deluser> | [B<--username> I<username>] | [B<--type> I<type>] 

B<userconfig> B<modifyuser> | [B<--username> I<username>] | [B<--pwd> I<password>] | [B<--initial_group>
I<initial_group>] | [B<--supp_group> I<supp_group>] | [B<--app_name> I<app_name>] | [B<--shell> I<shell>] | [B<--exp_date> I<exp_date>]

=head1 DESCRIPTION

This command is able to create, modify, delete user accounts, change password. Home directories of users accounts' created via this tool are
saved/restore during backup/restore.

=head1 OPTIONS

=over 4

=item B<help>

Print this help message and exit.

=item B<adduser>

Add new user account.

=item B<deluser>

Delete the user created by this command.

=item B<modifyuser>

Change user account's properties.

=item I<adduser options are:>

=over 4

=item B<--username>

The name of a new user.

=item B<--type>

User type (optionally property). Have several available options: 'Linux_User', 'Linux_FTP_User', 'Linux_SFTP_User', 'Linux_FTP_SFTP_User'. By default is 'Linux_FTP_SFTP_User'.

=item B<--pwd>

The user's password (MD5 hash). If it is not specified and --nopasswd option is not used, then prompt to enter a new password is printed.

=item B<--nopasswd>

Create user with blank password. Used to skip prompt to enter a new password if no --pwd option specified.

=item B<--initial_group>

The name of the initial group for a new user. This group must exist.

=item B<--supp_group>

The list of supplementary groups, delimited by commas. These groups must exist also.

=item B<--app_name>

The name of the application user belongs to. If name is not specified then app_name is set to 'base'.

=item B<--shell>

The default shell user logins to. If it is not specified then user will have /sbin/nologin. If user specifiy '--shell', then will have restricted bash shell.

=item B<--uid>

Users ID. If the UID is less then 500, then this user must be nologin. If user's primary group GID is less then 500, then UID must greater or
equal 500.

=item B<--exp_date>

The date on which the user account will be disabled. The date is specified in the format YYYY-MM-DD.

=back

=item I<deluser options are:>

=over 4

=item B<--username>

The name of the user for deletion. Mandatory property.

=back

=item I<modifyuser options are:>

=over 4

=item B<--username>

The name of a user which properties would be changed. Mandatory property.

=item B<--pwd>

The user's new password (MD5 hash). 

=item B<--initial_group>

New initial group for the specified user. This group must exist.

=item B<--supp_group>

The list of new supplemenatry groups for the specified user. Groups names must be separated by commas. These groups must exist.

=item B<--app_name>

New name of the application user belongs to.

=item B<--shell>

The default shell user logins to. If it is not specified then user will have /sbin/nologin. If user specifiy '--shell', then will have restricted bash shell.

=item B<--quiet>

The quiet mode flag. If it is not specified then some messages will be displayed on the screen. Otherwise no messages are printed.

=item B<--exp_date>

The date on which the user account will be disabled. The date is specified in the format YYYY-MM-DD.

=back

=back

=head1 EXAMPLES

=over 4

=item B<$ userconfig adduser --username test_user --initial_group sysadm --app_name nrsm --shell>

User will be prompted to enter password. After invoking new account with name test_user with initial group sysadm will be created. User will login to restricted bash shell. This account is linked with application nrsm;

=item B<$ userconfig adduser --username test_user2 --initial_group sysadm --app_name nrsm --pwd '$1$C7NadBfy$KGTqgljQhi0XUcdJTCT8L.'>

User with specified password (MD5) and /sbin/nologin shell will be created .

=item B<$ userconfig deluser --username test_user>

Account test_user and home directory will be removed.

=item B<$ userconfig modifyuser --username test_user --pwd '$1$C7NadBfy$KGTqgljQhi0XUcdJTCT8L.' --initial_group backupadmin>

Changes password of the test_user and set a initial group to backupadmin.

=back

=cut

use warnings;
use strict;

use lib "/opt/nortel/base/lib/";
use common_variables;
use common_functions;
use baselog;
use base_system;
use base_accounts;
use Getopt::Long;

## @skipline
sub userconfig();

exit(userconfig());

## @fn sub userconfig ()
# @brief Main function of the module.
# @details This function parses input parameter and starts sudo_nortel with
#          required parameters.
# @return $OK  - management operation successfully finished.
# @return $ERROR - some errors happened.
# @author Evgeny Lazarev
# @date November 2007
# @note
sub userconfig()
{
   my $retval = $TRUE;
   my $action = undef;

   if (warning_if_not_root_uid())
   {
      return $ERROR;
   }

   my @tmp = @ARGV;
   my $pos = pos_in_array(\@tmp, '--pwd');
   if ($pos >= 0)
   {
      $tmp[$pos + 1] = '######';
   }
   baselog($LOG_INFO, "'userconfig' is started with the following options: @tmp");

   if (scalar(@ARGV) == 0)
   {
      print_pod_usage("No parameters are specified.");
      $retval = $FALSE;
   }
   else
   {
      $action = shift @ARGV;
      if ($action eq $ADD_USER)
      {
         my $user_info = {};
         if (GetOptions($user_info, 'username=s', 'pwd=s', 'initial_group=s', 'supp_group=s', 'shell', 'home=s', 'type=s', 'app_name=s', 'uid=i', 'exp_date=s', 'nopasswd', 'quiet'))
         {
            if (defined($user_info->{'shell'}) && $user_info->{'shell'})
            {
               $user_info->{'shell'} = $TRUE;
            }
            if (!$user_info->{'quiet'})
            {
               $user_info->{'quiet'} = $FALSE;
            }
            $user_info->{'manual'} = $TRUE;
            $retval = base_accounts::add_user($user_info);
         } ## end if (GetOptions($user_info...
         else
         {
            baselog($LOG_ERROR, "Could not parse input parameters.");
            $retval = $FALSE;
         }
      } ## end if ($action eq $ADD_USER...
      elsif ($action eq $DEL_USER)
      {
         my $user_info = {};
         if (GetOptions($user_info, 'username=s', 'quiet'))
         {
            if (!$user_info->{'quiet'})
            {
               $user_info->{'quiet'} = $FALSE;
            }
            $retval = base_accounts::del_user($user_info->{'username'}, $user_info->{'quiet'});
         } ## end if (GetOptions($user_info...
         else
         {
            baselog($LOG_ERROR, "Could not parse input parameters.");
            $retval = $FALSE;
         }
      } ## end elsif ($action eq $DEL_USER...
      elsif ($action eq $MODIFY_USER)
      {
         my $user_info = {};
         if (GetOptions($user_info, 'username=s', 'pwd=s', 'initial_group=s', 'supp_group=s', 'shell', 'home=s', 'type=s', 'app_name=s', 'exp_date=s', 'quiet'))
         {
            if (defined($user_info->{'shell'}) && $user_info->{'shell'})
            {
               $user_info->{'shell'} = $TRUE;
            }
            if (!$user_info->{'quiet'})
            {
               $user_info->{'quiet'} = $FALSE;
            }
            $retval = base_accounts::modify_user($user_info);
         } ## end if (GetOptions($user_info...
         else
         {
            baselog($LOG_ERROR, "Could not parse input parameters.");
            $retval = $FALSE;
         }
      } ## end elsif ($action eq $MODIFY_USER...
      elsif ($action eq 'help')
      {
         print_pod_usage();
      }
      else
      {
         baselog($LOG_ERROR, "Unknown action '$action'.");
         print_pod_usage("Unknown action '$action'.");
         $retval = $FALSE;
      } ## end else [ if ($action eq $ADD_USER...
   } ## end else [ if (scalar(@ARGV) == 0...
   if ($retval == $FALSE)
   {
      $retval = $ERROR;
   }
   else
   {
      $retval = $OK;
   }
   return $retval;
} ## end sub userconfig()
