## @file CommonInstallUtils.pm
# @brief Common data and interfaces for Linux Base tools
# @details
# @version 1.00 - Initial creation
# @version 1.01 - getTimeValue routine added
# @version 1.10 - common USB routines added
# @version 1.20 - following functions were moved here from PreInstall.pl:
#   @li check_backup_archive
#   @li getSFTPArchive
#   @li getUSBBackupData
# @version 1.21 - the mount option "nosuid" added.
# @version 1.22 - Two colons were removed before shared functions and variables.
# @version 1.23 - use internal date/hostname functions instead of external
#    commands for generation of backup name
# @version 1.24 - Modified under the perlcheck warnings cleanup activity.
# @version 1.25 - Enhancement for showListByPage function.
# @version 1.26 - isCPPM() function has been deleted.
# @version 1.27 -  function showListByPage() has been modified to use none-selective input variable.
# @version 1.28 -  $LOCAL_IP was added.
# @version 1.29 - function validateRemoteCIQInfo() is modified to print local network
#    information during the installation only
# @version 1.30 - function validatePassword() is changed to "getPassword" and now performs
#                 all user input & validation. getCrpytPw() now retrieves encrypted password
#                 from appropriate location instead of generating it
# @version 1.31 - "header" function is updated to get clear screen parameter
# @version 1.32 - Function getFileNameResponse()
# @note
package CommonInstallUtils;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;
use baselog;
use base_system;
use Exporter ();

our @ISA    = qw(Exporter);
our @EXPORT = qw(&stopInstall &getYNResponse &mountAdmin &umountAdmin
  &clear &getValidIP &getTextResponse &getLogin &getCryptPW &andIpAddrs
  &validateIP &getTimeValue &craphack &validateRemoteCIQInfo
  &header &generateBackupName
  &showListByPage &getSFTPArchive &getUSBBackupData &umountUSB &calcBcastAddr
  &getPassword &defDev
  showListByPage getSFTPArchive getUSBBackupData formatUSB umountUSB calcBcastAddr
  $INTERNAL_CLOCK $EXTERNAL_CLOCK $PRIMARY $SECONDARY $OTHER $LOCAL_IP
  $UNCONFIGURED $usbMountPoint getFileNameResponse
  );

our $VERSION = '1.32';

my $USBDeviceNode;
my $USBFirstPartitionNode;
our $usbMountPoint = "/tmp/usb/backup";
my $gen_backup_name = "backup_archive.tar.gz";      # general name for retrieved backup archive
my $gomesg          = "Press Enter to continue.\n";

my $LINES_PER_PAGE = 23;
our $UNCONFIGURED = 0;

our $PRIMARY   = 0;
our $SECONDARY = 1;
our $OTHER     = 2;

our $EXTERNAL_CLOCK = 0;
our $INTERNAL_CLOCK = 1;

our $LOCAL_IP = '127.127.1.0';

my $clearRsp = '';

# next line is used for salt in crypt
my $randString = './0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
my $USE_MD5    = $TRUE;
my $clearDBG   = $FALSE;                                                             # if true, no clears are issued - useful for debugging
sub checkBackupSizeAvail($);

#################################################
#             Initialization block              #
#        /to define global parameters/          #
#################################################
INIT
{
   system_installation_init();
}

##------------------------------------------------------------------
# Purpose:
# - To retrieve a valid password for the given account
# @param   $account - the user account to get a password for
# @param   $inf - boolean, whether or not we should ask indefinitely
#                 until we get a good password, or stop after 3 tries
# @return  $TRUE if password was changed successfully, $FALSE otherwise
##------------------------------------------------------------------
sub getPassword
{
   my ($userid, $inf) = @_;
   my $tmp;

   header('Password Configuration');
   print <<HERE;
For security reasons, password entry keystrokes will not be shown as they
are typed. Please ensure you type the correct password and remember it for
future reference. Once the installation is started, you will not be prompted
for the password again.

Please note that a valid password must contain at least 8 characters,
6 of which are UNIQUE from all 4 character classes (lowercase, uppercase,
digits, other characters) to be considered valid.
Your password should not contain words from any dictionary in any
language or jargon, and should not be based on any personal
or login information.
   
Press ENTER to continue...
HERE
   $tmp = <STDIN>;
   while (($tmp = system("passwd $userid 2>&1")) && $inf) { }

   if ($tmp > 0)
   {
      return $FALSE;
   }
   return $TRUE;

} ## end sub getPassword

################################################################################
## @fn getCryptPW
# @brief To retrieve the encrypted password for the given account.
# @param $account - account name
# @return $crypt_pw - the encrypted password stored in the passwd file
################################################################################
sub getCryptPW($)
{
   my ($login) = @_;
   my $crypt_pw = "";

   my $password_handler;

   unless (open($password_handler, '<', $PASSWDFILE))
   {
      baselog($LOG_ERROR, "Cannot open file  $PASSWDFILE : $!\n", $TRUE);
      return $FALSE;
   }

   while (my $password_str = readline($password_handler))
   {
      if ($password_str =~ /\A$login:([^:]*):(.*)/)
      {
         $crypt_pw = $1;
      }
   } ## end while (my $password_str =...

   unless (close($password_handler))
   {
      baselog($LOG_ERROR, "Cannot close file  $PASSWDFILE : $!\n", $TRUE);
      return $FALSE;
   }

   return $crypt_pw;

} ## end sub getCryptPW($)

##------------------------------------------------------------------
# Purpose:
# - To clear the screen - we can cache the output from clear and
#   then just print it - keeps us from calling system every
#   time.
##-------------------------------------------------------------------
sub clear
{
   if (!$clearDBG)
   {
      if ($clearRsp eq "")
      {
         $clearRsp = `$CLEAR`;
      }
      print $clearRsp;
   } ## end if (!$clearDBG)
} ## end sub clear

##------------------------------------------------------------------
# Purpose:
# - Used during debugging to show who called the function
##-------------------------------------------------------------------
sub whowasi
{
   my $caller = (caller(2))[3];
   my $line   = (caller(2))[2];
   return ("$caller:$line");
}

##------------------------------------------------------------------
# Purpose:
# - Mount the admin partition on /tmp (used during %pre)
##-------------------------------------------------------------------
sub mountAdmin
{
   my ($device, $dir) = @_;

   $dir = "/tmp/admin" if !$dir;
   if (!($device =~ /^\//))
   {
      $device = "/tmp/$device";
   }

   #
   # Create /tmp/<device>2, if it doesn't already exist
   #
   if (!-e $device)
   {
      if (!do_system([$MKNOD, $device]))
      {
         baselog($LOG_ERROR, "Cannot create '$device' node.", $TRUE);
      }
   } ## end if (!-e $device)

   #
   # Create & mount /tmp/admin, if it doesn't already exist
   #
   if (!-e $dir)
   {
      if (!do_system([$MKDIR, $dir]))
      {
         baselog($LOG_ERROR, "Cannot create '$dir' directory.", $TRUE);
      }
   } ## end if (!-e $dir)

   do_system([$MOUNT, $device, $dir]);
} ## end sub mountAdmin

##------------------------------------------------------------------
# Purpose:
# - Unmount the admin directory (used during %pre)
##-------------------------------------------------------------------
sub umountAdmin
{
   my ($adminDir) = @_;

   $adminDir = "/tmp/admin" if !$adminDir;

   baselog($LOG_INFO, "Unmounting Admin!!! $UMOUNT $adminDir");
   do_system([$UMOUNT, $adminDir]);
} ## end sub umountAdmin

##------------------------------------------------------------------
# Purpose:
# - Ensure the we have a valid IPv4 IP
##-------------------------------------------------------------------
sub validateIP
{

   # check that in right dot notation format
   my ($IPaddr) = $_[0] =~ /^(\d{1,3}(\.\d{1,3}){3})$/;
   return $FALSE unless $IPaddr;

   # Check that bytes are in range
   my $zcnt = 0;

   for (split /\./, $IPaddr)
   {
      $zcnt++ if ($_ == 0);  # keep track of the # of 0's
      return $FALSE if $_ < 0 or $_ > 255;
   }

   #
   # don't allow an ip with only 0's in it
   #
   if ($zcnt == 4)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub validateIP

##------------------------------------------------------------------
# Purpose:
# - Prompt the user until they give us a valid IP
##-------------------------------------------------------------------
sub getValidIP
{
   my ($prompt, $hostIP) = @_;
   my $newIP = "";

   my $valid = $FALSE;
   while (!$valid)
   {
      if ($hostIP)
      {
         print "$prompt [$hostIP]: ";
      }
      else
      {
         print "$prompt: ";
      }

      chomp($newIP = <STDIN>);

      if ($newIP eq "")
      {
         if ($hostIP)
         {
            $newIP = $hostIP if $newIP eq "";  # default
         }
         else
         {
            print "\nERROR: Invalid entry. Please try again.\n\n";
            next;
         }
      } ## end if ($newIP eq "")
      if ($newIP =~ /^0+([0-9]+)/ || $newIP =~ /\.0+([0-9]+)/)
      {
         $newIP =~ s/^0+([0-9]+)/$1/;
         $newIP =~ s/\.0+([0-9]+)/\.$1/g;
         baselog($LOG_WARNING, "The leading zero will be removed from IP address: $newIP\n", $TRUE);
      } ## end if ($newIP =~ /^0+([0-9]+)/...
      if (validateIP($newIP))
      {
         $valid = $TRUE;
      }
      else
      {
         print "\nERROR: \"$newIP\" is in an invalid format. Please try again.\n\n";
      }
   } ## end while (!$valid)

   return $newIP;
} ## end sub getValidIP

##------------------------------------------------------------------
# Purpose:
# - Common routine to prompt for a Y/N question
##-------------------------------------------------------------------
sub getYNResponse
{
   my ($query, $default) = @_;

   my $opt;
   while (1)
   {
      print "$query (Y/N) [$default]? ";
      chomp($opt = <STDIN>);
      $opt = "$default" if $opt eq "";

      if ($opt =~ /^y$/i)
      {
         return $TRUE;
      }
      elsif ($opt =~ /^n$/i)
      {
         return $FALSE;
      }
      else
      {
         print "\nERROR: Invalid entry. Please try again.\n";
      }
   } ## end while (1)
} ## end sub getYNResponse

##------------------------------------------------------------------
# Purpose:
# - Common routine to prompt for a question that requires text input
##-------------------------------------------------------------------
sub getTextResponse
{
   my $default = $FALSE;
   my ($query, $defaultText) = @_;
   if (!defined($defaultText))
   {
      $defaultText = "";
   }
   my $opt;
   while ($TRUE)
   {
      if ($defaultText eq "")
      {
         print "$query: ";
      }
      else
      {
         $default = $TRUE;
         print "$query [$defaultText]: ";
      }
      chomp($opt = <STDIN>);
      if ($opt eq "")
      {
         if (!$default)
         {
            print "\nERROR: Invalid response entered. Please try again.\n\n";
         }
         else
         {
            return $defaultText;
         }
      } ## end if ($opt eq "")
      else
      {
         return $opt;
      }
   } ## end while ($TRUE)
} ## end sub getTextResponse

###############################################################################
## @fn getFileNameResponse($query, $defaultFile, $mustExist)
# @brief Prompt for a valid file name
# @param  $query - question to ask
# @param  $defaultFile - default answer (optional)
# @param  $mustExist - the file must exist
# @return  $filename - the file name
sub getFileNameResponse($;$$)
{
   my ($query, $defaultFile, $mustExist) = @_;

   my $default = $FALSE;
   if (defined($defaultFile))
   {
      $query   = "$query [$defaultFile]";
      $default = $TRUE;
   }

   if (not defined $mustExist or $mustExist != $TRUE)
   {
      $mustExist = $FALSE;
   }

   my ($filename, $dirname);
   while ($TRUE)
   {
      print "$query: ";
      chomp($filename = <STDIN>);
      if ($filename eq "")
      {
         if ($default)
         {
            $filename = $defaultFile;
         }
         else
         {
            print "\nERROR: Invalid response entered. Please try again.\n\n";
            next;
         }
      } ## end if ($filename eq "")
      $dirname = get_dirname($filename);
      if (not -d $dirname)
      {
         print "\nERROR: There is no such directory '$dirname'. Please try again.\n\n";
      }
      elsif ($mustExist and not -f $filename)
      {
         print "\nERROR: There is no such file '$filename'. Please try again.\n\n";
      }
      else
      {
         return $filename;
      }
   }  ## while
} ## end sub getFileNameResponse($;$$)

##------------------------------------------------------------------
# Purpose:
# - Common routine to prompt login. Login must match with regex
#       [a-zA-Z0-9_.][a-zA-Z0-9_.-]{0,30}[a-zA-Z0-9_.$-]?
#   this information can be found in the source code of the shadow-utils
#   package
#  INPUT:  $query       - prompt
#          $defaultText - default value
#  OUTPUT: $user        - entered login
##-------------------------------------------------------------------
sub getLogin
{
   my $user = "";
   my ($query, $defaultText) = @_;

   while ($TRUE)
   {
      $user = getTextResponse($query, $defaultText);
      last
        if ($user =~ /^[a-zA-Z0-9_\.][a-zA-Z0-9_\.-]{0,30}[a-zA-Z0-9_\.\$-]?$/);
      print "\nERROR: Invalid login entered. Please try again.\n\n";
   } ## end while ($TRUE)

   return $user;
} ## end sub getLogin

##------------------------------------------------------------------
# Purpose:
# - Performs a bit-wise and operation on two
#   IP addresses.
##-------------------------------------------------------------------
sub andIpAddrs
{
   my ($ip1) = shift(@_);
   my ($ip2) = shift(@_);

   my (@ip1);
   my (@ip2);
   my (@result);

   if ($ip1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/)
   {
      $ip1[0] = int $1;
      $ip1[1] = int $2;
      $ip1[2] = int $3;
      $ip1[3] = int $4;
   } ## end if ($ip1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/...
   else
   {
      printf("Malformed ip1 \"$ip1\".\n");
      exit(0);
   }

   if ($ip2 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/)
   {
      $ip2[0] = int $1;
      $ip2[1] = int $2;
      $ip2[2] = int $3;
      $ip2[3] = int $4;
   } ## end if ($ip2 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/...
   else
   {
      printf("Malformed ip1 \"$ip1\".\n");
      exit(0);
   }

   for (my $i = 0 ; $i <= $#ip1 ; $i++)
   {
      $result[$i] = $ip1[$i] & $ip2[$i];
   }
   my ($result) = join('.', @result);
   return $result;
} ## end sub andIpAddrs

##------------------------------------------------------------------
# Purpose:
# - Given a network mask, and an IP address, calculates
#   the broadcast address.
##-------------------------------------------------------------------
sub calcBcastAddr
{
   my ($ipAddr) = shift(@_);
   my ($mask)   = shift(@_);

   my (@ipAddr);
   my (@mask);
   my (@bcast);

   if ($ipAddr =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/)
   {
      $ipAddr[0] = int $1;
      $ipAddr[1] = int $2;
      $ipAddr[2] = int $3;
      $ipAddr[3] = int $4;
   } ## end if ($ipAddr =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/...
   else
   {
      printf("Error-calcBcast-1: Malformed ip address \"$ipAddr\".\n");
      exit(0);
   }

   if ($mask =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/)
   {
      $mask[0] = int $1;
      $mask[1] = int $2;
      $mask[2] = int $3;
      $mask[3] = int $4;
   } ## end if ($mask =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/...
   else
   {
      printf("Error-calcBcast-2: Malformed netmask \"$mask\".\n");
      exit(0);
   }

   for (my $i = 0 ; $i <= $#mask ; $i++)
   {
      if ($mask[$i] == 255)
      {

         # Mask is all 1's, so just use this byte of the IP
         # address.
         $bcast[$i] = $ipAddr[$i];
      } ## end if ($mask[$i] == 255)
      else
      {

         # Base address of subnet addr space for this byte.
         my ($base) = $mask[$i] & $ipAddr[$i];

         # Invert the mask (fill host addr portion of this byte
         # with 1's).
         my ($filler) = $mask[$i] ^ 255;

         # This type of broadcast addr.
         $bcast[$i] = $base + $filler;
      } ## end else [ if ($mask[$i] == 255)
   } ## end for (my $i = 0 ; $i <= ...
   my ($result) = join('.', @bcast);
   return ($result);
} ## end sub calcBcastAddr

##------------------------------------------------------------------
# Purpose:
# - An unrecoverable error has occurred - the install must be stopped
##-------------------------------------------------------------------
sub stopInstall
{
   print "$_[0]\n\n";
   print "This Installation has been halted.\n";

   #print "Press the ENTER key to shutdown the system\n";
   #<STDIN>;
   #system("chvt 1");
   map {`kill $1` if (/^\s*(\d+).+anaconda/)} (`ps -ef`);
   exit(1);
} ## end sub stopInstall

##------------------------------------------------------------------
# Purpose:
# - A common routine used when querying for date and time parameters
#   INPUT:
#          $default: default value for parameter
#          $change:
#          $prompt: string which will be shown to user
#          $min: bottom value acceptable for parameter
#          $max: top value acceptable for parameter
#          $sign: symbol that could be used as additional value
##-------------------------------------------------------------------
sub getTimeValue
{
   my ($default, $change, $prompt, $min, $max, $sign) = @_;
   my ($retval,  $opt);

   while ($TRUE)
   {
      print "$prompt [", $default + $change, "]: ";
      chomp($opt = <STDIN>);
      if ($opt eq "")
      {
         $retval = $default;
         last;
      }
      elsif ($opt eq $sign)
      {
         $retval = $sign;
         last;
      }
      elsif (   ($opt ne "")
             && ($opt =~ /^\d+$/)
             && ($opt >= $min)
             && ($opt <= $max))
      {
         $retval = $opt - $change;
         last;
      } ## end elsif (($opt ne "") && ($opt...
      else
      {
         print "\nERROR: Invalid Entry - please enter a value between $min and $max\n\n";
      }
   } ## end while ($TRUE)
   return $retval;
} ## end sub getTimeValue

#------------------------------------------------
sub craphack($)
{
   my $retval = '';
   map {$retval .= chr(ord($_))} (split '', shift @_);
   return $retval;
}

##------------------------------------------------------------------
# Purpose:
# - Validate information for remote file
##-------------------------------------------------------------------
sub validateRemoteCIQInfo
{
   my (%ftpData) = @_;
   header('Remote Configuration File Validation');

   my $indent = "";
   if (is_install_stage())
   {
      print "       Local machine IP: $ftpData{'machineIP'}\n";
      print "  Local machine netmask: $ftpData{'netmask'}\n";
      print "                Gateway: $ftpData{'gateway'}\n\n";
      $indent = "       ";
   } ## end if (is_install_stage()...
   print "$indent  SFTP server IP: $ftpData{'remoteIP'}\n";
   print "$indent     SFTP userid: $ftpData{'user'}\n";
   print "$indent   SFTP password: **********\n";
   print "$indent  SFTP directory: $ftpData{'remoteDir'}\n\n";

   if (getYNResponse("Is this information correct", 'Y'))
   {
      return $TRUE;
   }

   return $FALSE;
} ## end sub validateRemoteCIQInfo

###############################################################################
# @fn header($head, $noclear)
# @brief A common routine to clear the screen, print a header and then
#   print a line of dashes the size of the header.
# @param $head - header text
# @param $noclear - if screen clean up is required or not (optional).
#        clear screen takes place by default if $noclear is not $TRUE
# @return none
# @note
sub header($;$)
{
   my ($head, $noclear) = @_;
   if ((!defined $noclear) or !$noclear)
   {
      clear();
   }

   print "$head\n";
   print "-" x length($head);
   print "\n";
} ## end sub header($;$)

##------------------------------------------------------------------
# Purpose:
#   - To print out a list by $LINES_PER_PAGE lines per page
#     Before to print new page - wait user's input (to let him/her
#     to read information )
#     Each line is numbered to support number selection
# Arguments:
#     $ref - reference to list,
#     $exit_key - set exit key for prompt, default is '0'.
#     $none_selective - if $TRUE - the list will be shown as none selective list
##-------------------------------------------------------------------
sub showListByPage
{
   my ($ref, $exit_key, $none_selective) = @_;
   unless (defined($exit_key))
   {
      $exit_key = '0';
   }
   unless (defined($none_selective))
   {
      $none_selective = $FALSE;
   }
   if (defined($ref) && scalar(@$ref))
   {
      my $number = 0;
      my @list   = @$ref;
      my $size   = scalar(@list);
      chomp(@list);
      foreach my $line (@list)
      {
         $number++;
         if ($none_selective)
         {
            print "- $line\n";
         }
         else
         {
            print "$number) $line\n";
         }

         # check if it is the time to stop and if there are
         # more lines to print out
         if (($number % $LINES_PER_PAGE == 0) and ($number < $size))
         {
            print $gomesg;
            <STDIN>;
            clear();
         } ## end if (($number % $LINES_PER_PAGE...
      } ## end foreach my $line (@list)
      unless ($none_selective)
      {
         print "Select ($exit_key,1-$number):";
      }
   } ## end if (defined($ref) && scalar...
   else
   {
      baselog($LOG_ERROR, "Empty list.", $TRUE);
   }
} ## end sub showListByPage

##------------------------------------------------------------------
# Purpose:
#   Generate path for retrieving archive file
#   Check if input is not .tar.gz archive then treat it as
#   directory and add general name for recovering archive.
#   INPUT:   $dest - path to valid check
#
#   OUTPUT:  $dest - new path if check was successful or nothing
#                    if provided directory does not exists
##------------------------------------------------------------------
sub gen_backup_path
{
   my ($dest) = @_;
   if (substr($dest, -1) eq "/")
   {  # path is directory
      if (-d $dest)
      {
         $dest .= $gen_backup_name;
      }
      else
      {
         $dest = "";
         print "Input directory $dest does not exists.";
      }
   } ## end if (substr($dest, -1) ...
   elsif (substr($dest, -7) ne ".tar.gz")
   {
      if (-d $dest)
      {
         $dest .= "/$gen_backup_name";
      }
      else
      {
         $dest = "";
         print "Input directory $dest does not exists.";
      }
   } ## end elsif (substr($dest, -7) ...
   return $dest;
} ## end sub gen_backup_path

##-----------------------------------------------------------------------------
#  Purpose:
#      Check if provided archive is created by the backup system
#      (check if cookie and configuration file are present in the archive)
#      if this is valid archive (or file in case of config. file - txt):
#   INPUT:    $ArchiveFile - archive file name (including abs. path)
#   OUTPUT:   $TRUE - if this file is valid
#             $FALSE - if this file is invalid
##-----------------------------------------------------------------------------
sub check_backup_archive($)
{
   my $ArchiveFile = shift @_;
   my $retval      = $FALSE;
   if ((`$TAR -ztf $ArchiveFile | $GREP "backup_cookie" | $WC -l`) == 1)
   {
      if ((`$TAR -ztf $ArchiveFile | $GREP "userinfo.txt" | $WC -l`) == 1)
      {
         $retval = $TRUE;
      }
      else
      {
         print "There is no (or more than one) configuration file found in this archive.\n";
      }
   } ## end if ((`$TAR -ztf $ArchiveFile | $GREP "backup_cookie" | $WC -l`...
   else
   {
      print "There is no (or more than one) cookie file in this archive.\n";
   }
   return $retval;
} ## end sub check_backup_archive($)

##------------------------------------------------------------------
# Purpose:
#   Function to provide user info about progress of SFTP downloading
#
##-------------------------------------------------------------------
sub SFTP_progress_callback
{
   my ($sftp, $data, $offset, $size) = @_;
   print "\rRead $offset of $size bytes";
   baselog($LOG_DEBUG, "Read $offset of $size bytes\n");
}
##------------------------------------------------------------------
# Purpose:
#   Download selected file from the defined SFTP-server
#
#   INPUT:    $dest - directory where downloaded file should be put
#             %ftp  - structure with parameters to access to the SFTP
#                     server confirmed by user
#   OUTPUT:   name (including path) of downloaded file
#             if error occurred then empty string will be returned
##-------------------------------------------------------------------
sub getSFTPArchive
{
   my ($dest, %ftpData) = @_;
   my $file_path      = "";
   my $noninteractive = $FALSE;
   if (defined $ftpData{'noninteractive'})
   {
      $noninteractive = $ftpData{'noninteractive'};
   }

   $dest = gen_backup_path($dest);
   baselog($LOG_INFO, "Retrieving file listing from $ftpData{'remoteIP'}...\n", !$noninteractive);

   #ignore data input from user
   do_co_stdin("close");

   # Get the directory listing of the remote directory
   my %args;
   $args{'user'}     = $ftpData{'user'};
   $args{'password'} = $ftpData{'passwd'};
   $args{'debug'}    = 0;

   my $sftp = eval {Net::SFTP->new($ftpData{'remoteIP'}, %args)};

   if (!$sftp)
   {  # error occurred
      do_co_stdin('open');
      baselog($LOG_ERROR, "\nAn error occurred while trying to access the supplied SFTP server.\n", !$noninteractive);
      if (!$noninteractive)
      {
         print $gomesg;
         <STDIN>;
      }
   } ## end if (!$sftp)
   else
   {  #SFTP-connection initialized successfully
      if (!$sftp->do_stat($ftpData{'remoteDir'}))
      {
         do_co_stdin('open');
         baselog($LOG_ERROR, "\nAn error occurred while trying to access the supplied SFTP directory.\n", !$noninteractive);
         if (!$noninteractive)
         {
            print $gomesg;
            <STDIN>;
         }
      } ## end if (!$sftp->do_stat($ftpData...
      else
      {  # access to SFTP dir was successful
           # Prompt the user for the file they want to use
         my @files;
         my @data = eval {$sftp->ls($ftpData{'remoteDir'})};
         foreach (@data)
         {  # the files are returned as a hash - just want the filename
            my $file  = $_->{filename};
            my $perms = $_->{longname};

            # ignore directories and . files and show only archive files
            if (    ($file !~ /^\.$|^\.\.$|^\..+$/ && $perms !~ /^d/)
                and (substr($file, length($file) - 6) eq "tar.gz"))
            {
               push(@files, $file . "\n");
            }
         }  #list with archive files generated
         do_co_stdin('open');
         if (!scalar(@files))
         {
            baselog($LOG_INFO, "Cannot find any files to download.\n", !$noninteractive);
            if (!$noninteractive)
            {
               print $gomesg;
               <STDIN>;
            }
         } ## end if (!scalar(@files))
         else
         {
            my $answer = "";

            # Show the list of found files and wait user's decision
            my $is_end = $FALSE;
            while ($is_end == $FALSE)
            {
               my $opt;
               if ($noninteractive)
               {
                  $is_end = $TRUE;
                  my $filenum = 0;
                  foreach my $file (@files)
                  {
                     $filenum = $filenum + 1;
                     chomp($file);
                     if ($file eq $ftpData{'archname'})
                     {
                        $opt = $filenum;
                     }
                  } ## end foreach my $file (@files)
                  unless ($opt)
                  {
                     baselog($LOG_ERROR, "Backup archive file $ftpData{'archname'} is not found.");
                  }
               } ## end if ($noninteractive)
               else
               {
                  print "Please select one of files (0 means exit):\n";
                  showListByPage(\@files);
                  chomp($opt = <STDIN>);
               } ## end else [ if ($noninteractive)
               if ((defined $opt) and ($opt =~ /^\d+$/))
               {
                  if ($opt eq "0")
                  {
                     $is_end = $TRUE;
                  }
                  elsif (($opt > 0) and ($opt < (scalar(@files) + 1)))
                  {
                     chomp($answer = $files[$opt - 1]);
                     if ($answer ne "")
                     {
                        if (-e $dest)
                        {
                           unlink $dest;
                        }
                        do_co_stdin("close");

                        #do_start returns structure with file attributes
                        my $remoteFileSize = $sftp->do_stat("$ftpData{'remoteDir'}/$answer")->size();
                        if (checkBackupSizeAvail($remoteFileSize) == $TRUE)
                        {
                           baselog($LOG_INFO, "Operation in progress. Please wait.\n", !$noninteractive);

                           if (!$noninteractive)
                           {
                              $sftp->get("$ftpData{'remoteDir'}/$answer", $dest, \&SFTP_progress_callback);
                              print "\n";
                           }
                           else
                           {
                              $sftp->get("$ftpData{'remoteDir'}/$answer", $dest);
                           }
                           do_co_stdin("open");
                           if (-e $dest)
                           {

                              # If we had a permission problem, the file still gets transferred, but
                              # as a 0 length file (arghhh...)
                              my @inodeInfo = stat($dest);
                              if ($inodeInfo[7] > 0)
                              {  # file size is element #7
                                 if (check_backup_archive($dest) == $TRUE)
                                 {
                                    $file_path = $dest;
                                    $is_end    = $TRUE;
                                 }
                                 else
                                 {
                                    unlink $dest;
                                    if (!$noninteractive)
                                    {
                                       print "This is not a valid backup archive.\n$gomesg";
                                       <STDIN>;
                                    }
                                    else
                                    {
                                       baselog($LOG_ERROR, "This is not a valid backup archive.\n");
                                    }
                                 } ## end else [ if (check_backup_archive...
                              } ## end if ($inodeInfo[7] > 0)
                              else
                              {
                                 unlink $dest;
                                 if (!$noninteractive)
                                 {
                                    print "\nERROR: Permission Denied - Ensure that the file \"$answer\"\n";
                                    print "       has the correct access permissions for the SFTP user \"$ftpData{'user'}\".\n $gomesg";
                                    <STDIN>;
                                 } ## end if (!$noninteractive)
                                 else
                                 {
                                    baselog($LOG_ERROR, "Permission Denied - Ensure that the file \"$answer\" has the correct access permissions for the SFTP user \"$ftpData{'user'}\"");
                                 }
                              } ## end else [ if ($inodeInfo[7] > 0)
                           } ## end if (-e $dest)
                           else
                           {
                              baselog($LOG_ERROR, "$ftpData{'remoteDir'}/$answer failed. Cannot write $dest file.\n", !$noninteractive);
                           }  # file in the target dir or fault occurred
                        } ## end if (checkBackupSizeAvail...
                        else
                        {
                           baselog($LOG_ERROR, "There is not enough space to keep backup archive.\n", !$noninteractive);
                           sleep(3);
                        }
                     } ## end if ($answer ne "")
                  } ## end elsif (($opt > 0) and ($opt...
               } ## end if ((defined $opt) and...
            } ## end while ($is_end == $FALSE)

         } ## end else [ if (!scalar(@files))
      }  # access to SFTP dir was successful
   }  # end of working with SFTP-session
   return $file_path;
} ## end sub getSFTPArchive

##------------------------------------------------------------------
# Purpose:
#   Check if there is enough space on the local disk to keep
#   remote backup archive
#
#   INPUT:    $remoteFileSize - remote backup archive size
#   OUTPUT:   $TRUE - there is enough space on the local disk
#             $FALSE - free space is not enough
##-------------------------------------------------------------------
sub checkBackupSizeAvail($)
{

   # The passed parameter is inside of double quotes - it specifies type exactly.
   my $remoteFileSize = "$_[0]";
   my $retval         = $FALSE;
   if (!defined($remoteFileSize))
   {
      baselog($LOG_ERROR, "Cannot define size of the remote file.\n", $TRUE);
      return $FALSE;
   }

   my $freespace = get_device_free_space($ADMIN_PARTITION);

   if (!$freespace)
   {
      baselog($LOG_ERROR, "Cannot define free space on the local disk.\n", $TRUE);
      return $FALSE;
   }
   if (int($freespace * 1024) > $remoteFileSize)
   {
      baselog($LOG_DEBUG, "Free space on the /admin is $freespace; remote file size is $remoteFileSize");
      return $TRUE;
   }

   return $FALSE;
} ## end sub checkBackupSizeAvail($)

##------------------------------------------------------------------
# Purpose:
#   Get free space on a device
#
#   INPUT:    $dev - device name or mount point
#   OUTPUT:   free device space in kilobytes
#             undef - error condition
##-------------------------------------------------------------------
sub get_device_free_space()
{
   my ($dev) = @_;
   my $retval = undef;

   if (!$dev)
   {
      baselog($LOG_ERROR, "Device is not defined. Wrong parameter passed in function call.");
   }
   else
   {
      my $devinfo = qx{$DF $dev 2> /dev/null | $GREP $dev};
      if ($? != 0)
      {
         baselog($LOG_ERROR, "Cannot get device $dev space usage information.");
      }
      else
      {
         my @info_descr = split /\s+/, $devinfo;

         # check if it is an integer
         $retval = $info_descr[3];
      } ## end else [ if ($? != 0)

   } ## end else [ if (!$dev)
   return $retval;
} ## end sub get_device_free_space()

##-----------------------------------------------------------------------------
# Purpose:
#   Mount USB device
#   INPUT:  None
#   OUTPUT: $TRUE  - operation succeeded
#           $FALSE - operation failed
##-----------------------------------------------------------------------------
sub mount_USB_device()
{
   my $retval = $FALSE;
   if (check_attached_USB() == $TRUE)
   {
      if (get_USB_node_name() == $TRUE)
      {

         #Free (umount) the node first because this user can try to use
         #several USB devices (sequentially - plug/unplug device)
         umountUSB("/dev/$USBDeviceNode");
         umountUSB($usbMountPoint);

         # try to mount usb device.
         if (!-d $usbMountPoint)
         {
            if (!do_system([$MKDIR, '-p', $usbMountPoint]))
            {
               baselog($LOG_ERROR, "Cannot create $usbMountPoint directory.");
            }
         } ## end if (!-d $usbMountPoint...

         my @comm = ();
         if ((getpwuid($<))[2])
         {
            push @comm, $SUDO;
         }

         # the US DoD requires to use "nosuid" mount option for removable media
         push @comm, $MOUNT, "-o", "nosuid", "/dev/$USBFirstPartitionNode", $usbMountPoint;
         if ((do_system(\@comm)) and (`$MOUNT | $GREP "$usbMountPoint" | $WC -l` == 1))
         {
            $retval = $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "Cannot mount USB device to $usbMountPoint\n", $TRUE);
         }
      } ## end if (get_USB_node_name(...
      else
      {
         baselog($LOG_ERROR, "Cannot define USB node name.\n");
      }
   } ## end if (check_attached_USB...
   else
   {
      baselog($LOG_ERROR, "Cannot detect the USB device.");
   }
   return $retval;
} ## end sub mount_USB_device()

##-----------------------------------------------------------------------------
# Purpose:
#  Get last attached USB device node name from dmesg bus
#  Initialize variables
#                       $USBDeviceNode - device name and
#                       $USBFirstPartitionNode - first partition node name
#  Create nodes in /dev dir for this device
#  INPUT:  None
#  OUTPUT:
#          $TRUE  - device nodes defined and created successfully
#          $FALSE - function failed
##-----------------------------------------------------------------------------
sub get_USB_node_name()
{
   my $retval = $TRUE;
   $USBDeviceNode         = "";
   $USBFirstPartitionNode = "";

   #get last attached usb-storage device name
   my $infoline = `dmesg | $GREP \"Attached scsi removable disk\" | $TAIL -n 1`;
   baselog($LOG_DEBUG, "calling: dmesg | $GREP \"Attached scsi removable disk\" | $TAIL -n 1 returned: $infoline annd $?");

   # ignore the 'sd' at the very beginning of the output
   my $start = index($infoline, "sd", 5);
   if ($start != -1)
   {
      my $nodename = substr($infoline, $start, 3);
      if ($nodename ne "")
      {
         $USBDeviceNode         = $nodename;
         $USBFirstPartitionNode = $USBDeviceNode . "1";
         my $majorNumber = (ord(substr($nodename, 2, 1)) - ord('a')) * 16;
         if ($majorNumber > -1)
         {

            # Create nodes for /dev directory - to introduce block device
            # If node creation will be unsuccessful then it will be handled
            # during mount command (mount command will be failed)
            if (!(-e "/dev/$USBDeviceNode"))
            {
               if (do_system("$MKNOD /dev/$USBDeviceNode b 8 $majorNumber") != $TRUE)
               {
                  baselog($LOG_ERROR, "Cannot create a node $USBDeviceNode for a USB device.\n", $TRUE);
                  $retval = $FALSE;
               }
            } ## end if (!(-e "/dev/$USBDeviceNode"...
            $majorNumber = $majorNumber + 1;
            if (!(-e "/dev/$USBFirstPartitionNode"))
            {
               if (do_system("$MKNOD /dev/$USBFirstPartitionNode b 8 $majorNumber") != $TRUE)
               {
                  baselog($LOG_ERROR, "Cannot create a node /dev/$USBFirstPartitionNode for a USB device.\n", $TRUE);
                  $retval = $FALSE;
               }
            } ## end if (!(-e "/dev/$USBFirstPartitionNode"...
         } ## end if ($majorNumber > -1)
         else
         {
            baselog($LOG_ERROR, "Incorrect major number for block device node.\n", $TRUE);
            $retval = $FALSE;
         }

      } ## end if ($nodename ne "")
      else
      {
         baselog($LOG_ERROR, "Cannot extract a device node name from dmesg.\n", $TRUE);
         $retval = $FALSE;
      }
   } ## end if ($start != -1)
   else
   {
      baselog($LOG_ERROR, "Cannot get a device node name from dmesg.\n", $TRUE);
      $retval = $FALSE;
   }
   return $retval;
} ## end sub get_USB_node_name()

##-----------------------------------------------------------------------------
# Purpose:
#  Check if there is USB device attached to the system and if it's only one
#
##-----------------------------------------------------------------------------
sub check_attached_USB()
{
   my $retval         = $TRUE;
   my $usbDevQuantity = (`$CAT /proc/bus/usb/devices | $GREP "usb-storage" | $WC -l`);

   # Check if device not already plugged in or if there are more then one USB
   # device then shows a message about limitations. If user do not want to
   # follow these rules then some undesired behavior could take place -
   # for example wrong device will be used for recovery/backup
   if ($usbDevQuantity != 1)
   {
      print "Please do not plug in more than one USB device.\n";
      print "Plug in your device to any USB port and then press \"Enter\"\n";
      <STDIN>;
      $usbDevQuantity = (`$CAT /proc/bus/usb/devices | $GREP "usb-storage" | $WC -l`);
   } ## end if ($usbDevQuantity !=...
   else
   {
      print "USB storage detected.\n";
      sleep(2);  #get user time to see the message
   }

   # This loop if forcing the user to plug in only one USB device.
   # If there are more then 1 device  - then we can not predict
   # which one will be used. If there is no USB devices at all then
   # we will got an error during mount.
   while ($usbDevQuantity != 1)
   {
      if ($usbDevQuantity < 1)
      {
         print "Cannot find USB device in the system. ";
         print "Please attach one and try again.\n";
      }
      else
      {
         print "There are more than one USB device attached to the system.\n";
         print "Please remove all attached USB devices and then plug in one.\n";
      }
      print "To exit type \"Exit\"\n";
      chomp(my $opt = <STDIN>);
      $opt =~ tr/[A-Z]/[a-z]/;
      if ($opt eq "exit")
      {
         $retval = $FALSE;
         last;
      }
      else
      {
         $usbDevQuantity = (`$CAT /proc/bus/usb/devices | $GREP "usb-storage" | $WC -l`);
         if ($usbDevQuantity == 1)
         {
            print "USB storage detected.\n";
            sleep(2);  #get user time to see the message
            last;
         } ## end if ($usbDevQuantity ==...
      } ## end else [ if ($opt eq "exit")
   } ## end while ($usbDevQuantity !=...
   return $retval;
} ## end sub check_attached_USB()

###############################################################################
## @fn formatUSB($deviceNode)
# @brief     It formats a USB storage device and creates one
#            fat32 partition on this device
# @details
# @param    $deviceNode - device node name
# @return   $FALSE - USB disk format failed
# @return   $TRUE  - format operation succeed
# @author   Kate Bereznyakova
# @date     2007
sub formatUSB($)
{
   my $deviceNode;
   my $sudocmd;
   if (is_install_stage())
   {
      my $retval = get_USB_node_name();
      if ($retval != $TRUE)
      {
         baselog($LOG_ERROR, "Could not define or create usb device node.");
         return $FALSE;
      }
      $deviceNode = "/dev/$USBDeviceNode";
      $sudocmd    = "";
   } ## end if (is_install_stage()...
   else
   {
      $deviceNode = shift @_;
      $sudocmd    = "$SUDO ";
   }
   umountUSB($deviceNode);

   # workaround for parted 1.8.1 since scriptmode is broken (always returns false!)
   # let's provide input from re-directed stdin file for now...
   # (bug fixed in parted 1.8.7 but package not currently available for RHEL5)
   my $tmpcmds = "/tmp/usbcmds";
   my $USBCMDS;

   unless (open($USBCMDS, ">$tmpcmds"))
   {
      baselog($LOG_ERROR, "Could not open usbcmds!");
      return $FALSE;
   }

   my $cmd;

   print $USBCMDS "y\nmsdos\n\n";
   $cmd = $sudocmd . "$PARTED -i $deviceNode mklabel msdos >/dev/null 2>&1 < $tmpcmds";
   my $ret1 = (system($cmd)) >> 8;
   baselog($LOG_INFO, "$cmd returned $ret1");

   #create a first primary partition and formatting it as fat32
   $cmd = $sudocmd . "$PARTED -i $deviceNode mkpart primary 0 100%";
   my $ret2 = (system($cmd)) >> 8;
   baselog($LOG_INFO, "$cmd returned $ret2");
   seek $USBCMDS, 0, 0;
   print $USBCMDS "y\n1\nfat32\n\n";
   $cmd = $sudocmd . "$PARTED -i $deviceNode mkfs 1 fat32 >/dev/null 2>&1 < $tmpcmds";
   my $ret3 = (system($cmd)) >> 8;
   baselog($LOG_INFO, "$cmd returned $ret3");
   close($USBCMDS)  or baselog($LOG_WARNING, "Could not close usbcmds file!");
   unlink($tmpcmds) or baselog($LOG_WARNING, "Could not unlink usbcmds file!");

   if ($ret1 || $ret2 || $ret3)
   {
      baselog($LOG_ERROR, "Could not format USB device.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub formatUSB($)

##-----------------------------------------------------------------------------
#  Purpose:
#    - To allow the user to use backed up data from USB device
#
#  Functional Description:
#
#  Scan USB device searching for backed up files
#  If such files are found and one of them have chosen by user
#  then it will be copied to target directory and can be used
#  for system\application recovery
#
#   INPUT:    $dest - directory where downloaded file should be put
#
#   OUTPUT:   name (including path) of copied file
##-----------------------------------------------------------------------------
sub getUSBBackupData
{
   my ($dest) = @_;
   my $file_path = "";
   $dest = gen_backup_path($dest);

   # mounting a USB device
   if (mount_USB_device() == $FALSE)
   {
      my $msg = "Cannot mount USB device.\n";
      baselog($LOG_ERROR, $msg, $TRUE);
      print $gomesg;
      <STDIN>;
   } ## end if (mount_USB_device()...
   else
   {  # device mounted
      print "Scanning the device...\n\n";

      # if any backed_up data files have found on the device in /sysinfo directory
      # then let user choose one and use it to recover the system
      my @files = (`$LS $usbMountPoint/sysinfo`);

      if (scalar(@files) == 0)
      {
         print "Sorry. No backed up data files are found on the device.\n";
         umountUSB($usbMountPoint);
         print "You can unplug your device now.\n $gomesg";
         <STDIN>;
      } ## end if (scalar(@files) == ...
      else
      {  # Show the list of found files and wait user's decision
         my $is_end = $FALSE;
         while ($is_end == $FALSE)
         {
            print "Please select one of files (0 means exit):\n";
            showListByPage(\@files);
            chomp(my $opt = <STDIN>);
            if (($opt =~ /^\d+$/))
            {
               if ($opt eq "0")
               {
                  $is_end = $TRUE;
               }
               elsif (($opt > 0) and ($opt < (scalar(@files) + 1)))
               {
                  chomp(my $userFile = $files[$opt - 1]);
                  if (substr($userFile, -6) eq "tar.gz")
                  {
                     if (check_backup_archive("$usbMountPoint/sysinfo/$userFile") == $FALSE)
                     {
                        print "This is not valid backup archive.\n$gomesg";
                        <STDIN>;
                     }
                     else
                     {
                        my $filesize = (stat("$usbMountPoint/sysinfo/$userFile"))[7];
                        if (checkBackupSizeAvail($filesize) == $TRUE)
                        {
                           if (do_system([$CP, "$usbMountPoint/sysinfo/$userFile", $dest]))
                           {
                              $file_path = $dest;
                              $is_end    = $TRUE;
                           }
                           else
                           {
                              print "Transfer backup file to local partition failed.\n";
                           }
                        } ## end if (checkBackupSizeAvail...
                        else
                        {
                           print "There is not enough space to keep backup archive.\n";
                           sleep(3);
                        }
                     } ## end else [ if (check_backup_archive...
                  } ## end if (substr($userFile, ...
                  else
                  {
                     print "This file cannot be used for recovery.\n";
                  }
               } ## end elsif (($opt > 0) and ($opt...
            } ## end if (($opt =~ /^\d+$/))
         }  # end of  recovery file selection block
         umountUSB($usbMountPoint);
      } ## end else [ if (scalar(@files) == ...
   }  # end of device mounted block
   return $file_path;
} ## end sub getUSBBackupData

###-----------------------------------------------------------------
# Purpose
#  - unmount the USB device
#    INPUT: can be device name or mounting point
#
###-----------------------------------------------------------------
sub umountUSB($)
{
   my $node = shift @_;
   my @comm = ();
   if ((getpwuid($<))[2])
   {
      push @comm, $SUDO;
   }
   push @comm, $UMOUNT, $node;

   do_system(\@comm);
} ## end sub umountUSB($)

###############################################################################
## @fn formatUSB($deviceNode)
# @brief     To generate name of backup archive to be created with current date
# @details
# @param    $hostname - server hostname (optional)
# @param    $suffix - suffix to add (optional)
# @return   $archive_name - name of archive file

sub generateBackupName(@)
{
   my ($sec, $min, $hour, $mday, $mon, $year) = localtime();
   my $date = sprintf("%04d_%02d_%02d-%02d_%02d_%02d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);
   my $archive_name;
   if (scalar(@_) == 2)
   {
      my ($hostname, $suffix) = @_;
      $archive_name = $hostname . "-" . $suffix . "-" . $date;
   }
   elsif (scalar(@_) == 0)
   {
      my $hostname = get_hostname();
      $archive_name = $hostname . "-" . $date;
   }
   else
   {
      baselog($LOG_ERROR, "Incorrect parameters are provided");
   }
   return $archive_name;
} ## end sub generateBackupName(@)

##------------------------------------------------------------------
# Purpose:
#   Define device name of the appropriate partition by partition label
#   INPUT:  $label    - label name of the appropriate partition
#           $dev      - file name of device
#   OUTPUT: $partdev  - name of the partition device
#           undef     - error condition
##------------------------------------------------------------------
sub defDev($)
{
   my $label         = shift @_;
   my @fdisk         = `$FDISK -l`;
   my $dev_partition = undef;
   my $partdev       = undef;
   foreach my $fdisk_str (@fdisk)
   {
      if ($fdisk_str =~ /(\/dev\/\w+)\s+.*/)
      {
         my $curlab = $1;
         $dev_partition = `$E2LABEL $curlab 2>/dev/null`;
         chomp($dev_partition);
         unless ($? == 0) {$dev_partition = undef;}
         if ((defined $dev_partition) and ($dev_partition =~ /$label\d*$/))
         {
            $partdev = $curlab;
         }
      } ## end if ($fdisk_str =~ /(\/dev\/\w+)\s+.*/...
   } ## end foreach my $fdisk_str (@fdisk...
   return $partdev;
} ## end sub defDev($)

1;
