###############################################################################
## @file   Recovery.pm
# @brief   The recovery console main module.
# @details This module introduces common variables and functions
#          for creating of the recovery console main menu and 
#          dynamical loading of the recovery console functional modules.
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version
# @version 1.01 - Modified to use is_cppm() function.
# @note

package Recovery;

use 5.008005;
use strict;
use warnings;
use common_variables;
use common_functions;
use plugin_mgr;
use CommonInstallUtils;
use hwRecognition;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ('all' => [qw()]);
our @EXPORT_OK = (@{$EXPORT_TAGS{'all'}});
our @EXPORT = qw(
  callRecoveryMenu
  );
our $VERSION = '1.01';

###############################################################################
## @fn sub callRecoveryMenu()
# @brief   Dynamical creating a menu.
# @details Dynamical loading modules from list in the menu.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008
# @note

sub callRecoveryMenu()
{
   my @IncludedModules = ();
   my @LoadedModules = (); 
   my $ModuleCount = 0;
   my $modDir = undef;
   my $rcvr   = "recovery_pm";
   if (is_cppm())
   {
      $modDir = "/tmp/cf/scripts/$rcvr";
   }
   else
   {
      $modDir = "/mnt/source/scripts/$rcvr";
   }
  
   my $manager = plugin_mgr->new($modDir);
   @IncludedModules = $manager->get_modules();  #creating the list of modules
   while ($TRUE)
   {
      $ModuleCount = 0;
      #Clear screen
      clear();
      print "Recovery Console: Please, be careful. You may break your system.\n\n";
      print " You may choose to do one of the following:\n";
      foreach my $module (@IncludedModules)
      {
         my %hash = $manager->get_action($module, 'menu');
         if (%hash)
         {
            $ModuleCount++;
            push @LoadedModules, $module;
            print " ", $ModuleCount, ") ", $hash{'string'},"\n";
         }    
      } ## end foreach my $module...

      print " ", $ModuleCount + 1, ") ", "Reboot system ", "\n";
      my $selected = $FALSE; 
      while ($selected == $FALSE)
      {
         print "Select an option (1-", $ModuleCount + 1, "):";
         my $choice = <STDIN>;
         chomp($choice);
         if ($choice =~ /^\s*(\d+)\s*$/)
         {
            $choice = $1;
      
            if (($choice <= $ModuleCount)&&($choice > 0))
            {
               my $hash = prepare_action_hash($LoadedModules[$choice - 1], 'DefaultEntryPoint');
               $manager->try_action($hash);
               $selected = $TRUE;
            }
            elsif ($choice == $ModuleCount + 1)
            {
               Reboot(); #reboot system
               $selected = $TRUE;
            }  
            else
            {
               print "Wrong choice. Please repeat\n";
            } 
         }
      } 
   }  ##end while($TRUE)
}  ##end sub callRecoveryMenu

###############################################################################
## @fn sub Reboot()
# @brief   System reboot.
# @details This function allows users to reboot the system and
#          provides users with interface for system rebooting
#          via the recovery console.
# @param   None
# @return  TRUE (system will be rebooted) or FALSE (system will not be rebooted)
# @author  Ekaterina Bazhanova
# @date    February  2008
# @note

sub Reboot()
{
   my $query = 'Would you like to reboot system now?';
   clear();
   if (getYNResponse($query,'N'))
   {
      system "$REBOOT -f";
      return  $TRUE;
   }
   else
   {
      print "System will not reboot\n";
      print "Press <Enter> to continue:";
      <STDIN>;
      return $FALSE;
   }

} ## end sub Reboot
###############################################################################
## @fn sub prepare_action_hash($module, $action)
# @brief   Prepare a hash to use it with an instance of the Plug-in manager.
# @details This method takes the passed parameters and fills the hash. This
#          hash could be used with such method of the Plug-in manager as
#          do_action and try_action.
# @param   $module - name of module/plug-in. Could be undefined.
# @param   $action - action.  Could be undefined.
# @return  \%hash  - reference to the action hash
sub prepare_action_hash(@)
{  
   my %hash = ();
   $hash{'module'} = shift @_;
   $hash{'action'}  = shift @_;
   return \%hash;
}
1;
__END__
