# ############################################################
# @file RouteControl.pm
# @brief Routes manipualting interface module
# @details
#  This module is the LinuxBase interface to manage
# Kernel IP routing table.
# It stores user configured routes (file routes_usercfg.xml)
# and provides interface to manipulate it.
# Methods: route->add route->del route->show route->apply_all
#
# @author Peter Kascheev
# @author Pavel Sergeev
# @author Alex Arkhipov
#
# @date  July 2007
#
# @version 1.00 - Initial version
# @version 1.01 - apply_all subroutine is changed, parameter ($cmd)
#          is added
# @version 1.10 - the '-host' option support is added
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.20 - new "-tag <tag_name>" option support is added
# @version 1.21 - the print_user_routes() function has been changed to use printf command
# ############################################################
package RouteControl;

use warnings;
use strict;

use Data::Dumper;
use XML::Writer;
use IO::File;
use English;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use install_common;
use ifxml;
use baselog;
use base_system;

use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw($ALL_ROUTES $DEFAULT_ROUTE_TAG);

# Subroutines declarations
sub read_route_xml();
sub save_xml_file($);
sub apply_route($$);

sub new();
sub add($$);
sub del($$);
sub show($$$);
sub apply_all($$);

our $VERSION = '1.21';

our $ALL_ROUTES        = 'all_routes';
our $DEFAULT_ROUTE_TAG = 'manual';

#####################################################################
########################## Private Data #############################
my $NET_MASK   = '255.255.255.255';
my $DEFAULT_GW = '0.0.0.0';

my $routexml_name = 'routes_usercfg.xml';                   #Route DB XML file
my $route_xml     = $SYSCONFIG_ROOT . '/' . $routexml_name;
my $section       = 'routes';                               #main tag in route xml file
my $tag           = 'route';                                #tag in route xml file

my $TAG_OPTION = 'tag';
my $CLASS_NAME = 'RouteControl';

INIT
{
   unless (-e $route_xml)
   {
      baselog($LOG_INFO, "'$route_xml' file is not exist. Creating.");
      my $fh;
      unless (open $fh, '>', $route_xml)
      {
         baselog($LOG_ERROR, "Could not create '$route_xml' file\n", $TRUE);
         exit($ERROR);
      }
      print $fh "<$section></$section>\n";
      unless (close $fh)
      {
         baselog($LOG_ERROR, "Error closing '$route_xml' file\n", $TRUE);
         exit($ERROR);
      }
   } ## end unless (-e $route_xml)
} ## end INIT

#####################################################################
########################## Private Methods ##########################

#####################################################################
# @fn read_route_xml
# @brief This subroutine reads route xml file and creates array of hashes
# using the object of ifxml class.
# @param none
#
# @return array reference
#          undef - if error
#####################################################################
sub read_route_xml()
{
   my $ret;

   #Create parser
   my $xml_parser = ifxml::new($SYSCONFIG_ROOT, $routexml_name);

   #Read xml file
   if ($xml_parser)
   {
      $ret = $xml_parser->find_data_by_xpath($GETDATA, "/$section/$tag");
   }
   else {return undef;}
   undef $xml_parser;
   return $ret;

} ## end sub read_route_xml()

#####################################################################
# @fn save_xml_file
# @brief This subroutine overwrites route xml file
# @param @data - array reference with data to be written to xml file
#
# @return $TRUE - no errors
#         $FALSE - errors
#####################################################################
sub save_xml_file($)
{
   my @data = @{shift @_};
   my ($output, $xml, $hash);

   $output = new IO::File(">$route_xml");
   unless (defined $output)
   {
      baselog($LOG_ERROR, "Cannot open $route_xml file");
      return $FALSE;
   }
   $xml = new XML::Writer(DATA_MODE => 'true', DATA_INDENT => 3, OUTPUT => $output);
   if (!$xml)
   {
      baselog($LOG_ERROR, "Cannot get XML::Writer");
      $output->close();
      return $FALSE;
   } ## end if (!$xml)

   $xml->startTag($section);  #OPEN section
   foreach $hash (@data)
   {
      if (defined($hash))
      {
         $xml->emptyTag($tag, %{$hash});
      }
   } ## end foreach $hash (@data)
   $xml->endTag($section);    #CLOSE section

   $xml->end();
   $output->close();

   return $TRUE;

} ## end sub save_xml_file($)

#####################################################################
# @fn apply_route
# @brief This subroutine tries to configure route using the $ROUTE command
#
# @param $data - hash reference with data
# @param $cmd  - add or del
#
# @return $TRUE - no errors $FALSE - errors
#####################################################################
sub apply_route($$)
{
   my ($data, $cmd) = @_;

   # @route_cmd is for command line data
   my @route_cmd;

   # filling @route_cmd array with route command and arguments
   if ($data->{'net'})
   {
      push(@route_cmd, $ROUTE, $cmd, '-net', $data->{'net'}, 'gw', $data->{gw}, 'netmask', $data->{'netmask'}, $data->{'dev'});
   }
   else
   {
      push(@route_cmd, $ROUTE, $cmd, '-host', $data->{'host'});
      if (exists($data->{'gw'}))
      {
         push(@route_cmd, 'gw', $data->{gw});
      }
      push(@route_cmd, $data->{'dev'});
   } ## end else [ if ($data->{'net'})

   my @err_mes;
   if (not do_system(\@route_cmd, {$dsERR => \@err_mes}))
   {

      #error message may contain possible usage of /sbin/route command
      #it is neseccary not to show this message to user
      my @err = " ";
      foreach (@err_mes)
      {

         #cut everything after word <Usage:>
         last if (/^Usage:/);
         push @err, $_;
      } ## end foreach (@err_mes)
      print @err;
      baselog($LOG_ERROR, "Manipulate route operation failed\n");
      return $FALSE;
   } ## end if (not do_system(\@route_cmd...
   return $TRUE;
} ## end sub apply_route($$)

#####################################################################
########################## Public Methods ###########################

#####################################################################
# @fn new
# @brief This subroutine is object constructor. Object is an array of hashes
#
# @param none
# @return class instance or undef if error
#####################################################################
sub new()
{

   #initialization
   my $class = shift;
   my $self  = [];

   if (warning_if_not_root_uid())
   {
      return undef;
   }

   # Class variables initialization
   $class = $CLASS_NAME unless $class;

   $self = read_route_xml();
   unless ($self)
   {
      baselog($LOG_ERROR, "Could not read '$route_xml'\n", $TRUE);
      return undef;
   }

   # Declaring an object
   bless $self, $class;

   # Assign default tag name if not assigned yet
   # (for files migrated from previous release)
   foreach my $entry (@$self)
   {
      if (not exists($entry->{$TAG_OPTION}))
      {
         $entry->{$TAG_OPTION} = $DEFAULT_ROUTE_TAG;
      }
   } ## end foreach my $entry (@$self)

   return $self;
} ## end sub new()

#####################################################################
# @fn add
# @brief This subroutine addes route to route array of hashes and
# invokes applying and save functions
# @param $data - hash reference with data
# @param $add  - hash reference with data to be added
#
# @return $TRUE  - no errors
#         $FALSE - errors
#####################################################################
sub add($$)
{
   my ($data, $add) = @_;

   if (apply_route($add, 'add'))
   {

      # Add default 'gw' if not provided
      if (not exists($add->{'gw'}))
      {
         $add->{'gw'} = $DEFAULT_GW;
      }

      # Add route
      push @$data, $add;

      # Save routes
      unless (save_xml_file($data))
      {
         baselog($LOG_ERROR, "Could not save the '$route_xml' file\n", $TRUE);
         print "This change will not survive the reboot\n";
         return $FALSE;
      } ## end unless (save_xml_file($data...

   } ## end if (apply_route($add, ...
   else
   {
      return $FALSE;
   }
   print "Route succesfully added\n";
   return $TRUE;
} ## end sub add($$)

#####################################################################
# @method del
# @brief This subroutine deletes route from array of hashes of routes and
# invokes applying and save functions
#
# @param $data - hash reference with data
# @param $del  - hash reference with data to be deleted
#               OR array index of data to be deleted (must be > 0)
#
# @return $TRUE  - no errors
#         $FALSE - erorrs
#####################################################################
sub del($$)
{
   my ($data, $del) = @_;
   my @inds = ();

   if (is_hash($del))
   {
      my $i = 0;
      if (not exists($del->{$TAG_OPTION}))
      {

         # Define array index that will be deleted
         foreach my $dat (@$data)
         {

            # Add tag value to make hashes equal
            $del->{$TAG_OPTION} = $dat->{$TAG_OPTION};
            if (cmp_hashes($dat, $del))
            {
               push @inds, $i;
               last;
            }
            $i++;
         } ## end foreach my $dat (@$data)
         if (scalar(@inds) == 0)
         {
            baselog($LOG_ERROR, "Could not find route that should be deleted\n", $TRUE);
            return $FALSE;
         }
      } ## end if (not exists($del->{...
      else
      {
         if (scalar(keys(%$del)) > 1)
         {
            baselog($LOG_ERROR, "No other options allowed together with -tag options for del command.\n", $TRUE);
            return $FALSE;
         }

         # Remove all rules with provided tag
         foreach my $dat (@$data)
         {
            if ($dat->{$TAG_OPTION} eq $del->{$TAG_OPTION})
            {
               push @inds, $i;
            }
            $i++;
         } ## end foreach my $dat (@$data)
         if (scalar(@inds) == 0)
         {
            baselog($LOG_INFO, "No routes with tag '$del->{$TAG_OPTION}' found\n", $TRUE);
            return $TRUE;
         }
      } ## end else [ if (not exists($del->{...
   } ## end if (is_hash($del))
   else
   {
      if (($del <= 0) or ($del > scalar(@$data)))
      {
         baselog($LOG_ERROR, "There is no route with index $del\n", $TRUE);
         return $FALSE;
      }
      push @inds, $del - 1;
   } ## end else [ if (is_hash($del))
   my $real_ind = 0;
   foreach my $ind (@inds)
   {
      if (defined $data->[$ind])
      {
         $real_ind = $ind + 1;
         if (apply_route($data->[$ind], 'del'))
         {

            # Delete route
            $data->[$ind] = undef;
            baselog($LOG_INFO, "Route($real_ind) has been succesfully deleted\n", $TRUE);
         } ## end if (apply_route($data->...
         else
         {
            baselog($LOG_ERROR, "Route deletion failed ($real_ind)\n", $TRUE);
         }
      } ## end if (defined $data->[$ind...
      else
      {
         print "Can't find route: $ind\n";
      }
   } ## end foreach my $ind (@inds)

   # Delete undef values from array
   @{$data} = grep {defined $_} @{$data};

   unless (save_xml_file($data))
   {
      baselog($LOG_ERROR, "Could not save the '$route_xml' file\n", $TRUE);
      print "This change will not survive the reboot\n";
      return $FALSE;
   } ## end unless (save_xml_file($data...

   return $TRUE;
} ## end sub del($$)

#####################################################################
# @fn apply_all
# @brief This subroutine makes action provided on the all user routes
#
# @param $data - hash reference with data
# @param $cmd  - "del", "add" command
#
# @return $TRUE  - no errors
#         $FALSE - error
#####################################################################
sub apply_all($$)
{
   my $data = shift;
   my $cmd  = shift;
   my $ret  = $TRUE;
   foreach my $dat (@$data)
   {
      unless (apply_route($dat, $cmd))
      {
         baselog($LOG_ERROR, "Can not manipulate route " . Data::Dumper::Dumper($dat) . "\n");
         $ret = $FALSE;
      }
   } ## end foreach my $dat (@$data)
   return $ret;
} ## end sub apply_all($$)

#####################################################################
# @method show
# @brief This subroutine prints routes
#
# @param $data - hash reference with data
# @param $arg  - application name to print or $ALL_ROUTES for all routes
# @param $opt - "-n" or undef - prints in numeric or in symbolic mode
#               effective only for $APP_ROUTES
#
# @return $TRUE  - no errors
#         $FALSE - erorrs
#####################################################################
sub show($$$)
{
   my ($data, $arg, $opt) = @_;

   # Put empty line
   print "\n";

   if ((defined $arg) && ($arg eq $ALL_ROUTES))
   {
      my @route_cmd = ($ROUTE);
      if (defined $opt)
      {
         push @route_cmd, $opt;
      }
      system(@route_cmd);
      if ($? >> 8)
      {
         print "Can not get information about current routes\n";
         return $FALSE;
      }
   } ## end if ((defined $arg) && ...
   else
   {
      if (scalar(@$data) > 0)
      {
         my $i          = 0;
         my $is_printed = $FALSE;
         foreach my $dat (@$data)
         {
            $i++;
            if (defined($arg) and ($arg ne $dat->{$TAG_OPTION}))
            {
               next;
            }

            # There is at least one route printed
            $is_printed = $TRUE;

            if ($dat->{'net'})
            {
               print_user_routes($i, $dat->{'net'}, $dat->{'gw'}, $dat->{'netmask'}, $dat->{'dev'}, $dat->{$TAG_OPTION});
            }
            else
            {
               print_user_routes($i, $dat->{'host'}, $dat->{'gw'}, $NET_MASK, $dat->{'dev'}, $dat->{$TAG_OPTION});
            }
         } ## end foreach my $dat (@$data)
         if ((not $is_printed) and defined($arg))
         {
            print "The system has no routes with tag: $arg\n";
         }
      } ## end if (scalar(@$data) > 0...
      else
      {
         print "The system has no user configured routes.\n";
      }
   } ## end else [ if ((defined $arg) && ...
   return $TRUE;
} ## end sub show($$$)

#####################################################################
# @fn print_user_routes
# @brief This sub prints data in the specified format
#
# @param  @_ - data to print
# @return none
#####################################################################
sub print_user_routes(@)
{

   my $route_header = "Kernel IP routing table (user configured routes)\n\n";

   my ($route_num, $dest, $gateway, $netmask, $iface, $tag ) = @_;
   if ($route_num == 1)
   {
      print $route_header;
      printf ("%-6s  %-15s  %-15s  %-15s  %-5s  %-21s\n", 'ROUTE#', 'Destination', 'Gateway', 'Netmask', 'Iface', 'Tag');
   }
   printf ("%-3s     %-15s  %-15s  %-15s  %-5s  %-21s\n", $route_num, $dest, $gateway, $netmask, $iface, $tag);

} ## end sub print_user_routes(@)

#####################################################################
# @fn DESTROY
# @brief Object destructor is used to reduce warnings output.
# @param none
# @return none
# @note This function name is defined in Perl and must not be changed.
#####################################################################
sub DESTROY
{
   return;
}

1;

__END__
