## @file XMLRef.pm
# @brief XML reference interface module
# @details This module allows to get data from XML files by using XPath
#           references.
#
# @author: Alexander Kolotov
#
# @version 1.00 - Initial version
# @version 1.10 get_referenced_value method is added by Alex Arkhipov
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.12 - 'dirname' and 'basename' are now replaced with appropriate 
#                 subroutines from common_functions lib.
# @version 1.13 - function get_value() has been updated to check existence of the XML file
#############################################################
package XMLRef;
use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;

our $VERSION = "1.13";

use warnings;
use strict;
use ifxml;
use baselog;

#####################################################################
#################### Private Methods Declaration ####################

## @skipline
sub new(;$);
## @skipline
sub DESTROY;
## @skipline
sub remove_parser($$);
## @skipline
sub get_parser($$);

#####################################################################
##################### Public Methods Declaration ####################

## @skipline
sub get_value($$$);
## @skipline
sub get_referenced_value($$$$);

#####################################################################
########################## Private Methods ##########################

## @method new ($class)
# @brief XMLRef object constructor.
# @param  $class - class name
# @return $self - The reference to the entity of the class
sub new(;$)
{

   #Parser initialization.
   my $class = shift @_;

   my $self = {};

   #Class attributes initialization
   $class = "XMLRef" unless $class;
   $self->{'PARSERS'} = {};

   #Declaring an object
   bless $self, $class;

   return $self;
} ## end sub new

## @method  DESTROY
# @brief Destrucitor. It cleans the owned data.
# @param  None
# @return None
sub DESTROY
{
   my $self = shift @_;

   my $filename;

   #Clean the hash with XML parsers
   foreach $filename (keys %{$self->{'PARSERS'}})
   {
      baselog($LOG_DEBUG, "Try to clean XML parser for '$filename'");
      $self->remove_parser($filename);
   }

   return;
} ## end sub DESTROY

## @method  remove_parser($self, $filename)
# @brief Remove one XML parser from the %PARSERS hash.
# @param  $self - reference to object.
# @param  $filename - file name which XML parser is to be removed for.
# @return $TRUE  - parser is removed sucessfully.
# @return $FALSE - there is no parser for this file.
sub remove_parser($$)
{
   my $self     = shift @_;
   my $filename = shift @_;

   if (defined $self->{'PARSERS'}->{$filename})
   {
      undef $self->{'PARSERS'}->{$filename};
   }
   else
   {
      baselog($LOG_WARNING, "XML parser is not found for '$filename'");
      return $FALSE;
   }
   return $TRUE;
} ## end sub remove_parser($$)

## @method  get_parser($self, $reffile)
# @brief This subroutine creates the object of ifxml class in the case if
#    \li the object was not created before for the given XML file. The created
#    \li object is put into the hash 'PARSERS' in order to re-use this
#    \li object later
# @param  $self - reference to object.
# @param  $reffile - path to XML file
# @return $parser - the object of ifxml class if the XML file is read
#               \li successfully otherwise undefined value.

sub get_parser($$)
{
   my $self    = shift @_;
   my $reffile = shift @_;

   my ($dirname, $basename);
   my $parser;

   baselog($LOG_DEBUG, "Get XML parser for '$reffile'");

   $dirname  = get_dirname($reffile);
   $basename = get_basename($reffile);

   #If parser has already been created (or it has already been
   #determined that the XML file can not be read)
   if (exists $self->{'PARSERS'}->{$reffile})
   {
      baselog($LOG_DEBUG, "XML parser is re-used");
      $parser = $self->{'PARSERS'}->{$reffile};
   }
   else
   {

      #Create parser
      $parser = ifxml::new($dirname, $basename);

      #If some errors happen (file can not be found, opened, read)
      unless ($parser)
      {
         $parser = undef;
         baselog($LOG_ERROR, "Can not get xml parser. Mark it as undefined");
      }
      else
      {
         baselog($LOG_DEBUG, "XML parser is initialized");
      }

      #Put the parser to the hash in order to be used in the future
      $self->{'PARSERS'}->{$reffile} = $parser;
   } ## end else [ if (exists $self->{'PARSERS'...

   return $parser;
} ## end sub get_parser($$)

#####################################################################
########################### Public Methods ##########################

## @method get_value ($self,$reffile, $refxpath) 
# @brief The subroutine looks for a referenced value by using XML file
#        and string which contains a path where this referenced value is
#        situated in this XML file, e.g. '/serverConfig/ncs/ncsPrimaryPort'.
# @param $self - ref to the class object
# @param $reffile - name of XML file
# @param $refxpath - path where the referenced value is situated
# @return $refvalue - The dereferenced value or undef if it can not be found.

sub get_value($$$)
{
   my $self = shift @_;
   my ($reffile, $refxpath) = @_;

   my $refvalue = undef;
   my $xml_parser;

   if (!defined $reffile) 
   {
      baselog($LOG_ERROR, 'Name of XML file is undefined');
      return undef; 
   }
   if (! -e $reffile) 
   {
      baselog($LOG_INFO, "XML file '$reffile' does not exist"); 
      return undef;
   }
 
   #Get XML parser for XML reference-file
   $xml_parser = $self->get_parser($reffile);
   if (defined $xml_parser)
   {

      #De-reference by using XPath
      $refvalue = $xml_parser->find_data_by_xpath($GETTEXT, $refxpath);

      if (!defined $refvalue)
      {
         baselog($LOG_WARNING, "Some tag from XPath '$refxpath' is not found in '$reffile'");
      }
      elsif ((!defined $refvalue->[0]) || ($refvalue->[0] eq ''))
      {
         $refvalue = undef;
         baselog($LOG_WARNING, "There is nothing found by XPath '$refxpath' in '$reffile'");
      }
      else
      {
         $refvalue = $refvalue->[0];
         baselog($LOG_DEBUG, "File: '$reffile', XPath: '$refxpath', Value: '$refvalue'");
      }
   } ## end if (defined $xml_parser...
   else
   {
      baselog($LOG_WARNING, "Can not read XML file '$reffile'");
   }

   return $refvalue;
} ## end sub get_value($$$)

## @method get_referenced_value
# @brief check parameters and return referenced value or default if not present
# @param $self - ref to the class object
# @param $reffile - path to the reference file
# @param $xpath - xpath to the valu in the reference file
# @param $def_val - default value if file not found or there is no data by xpath
# @return referenced value (either from reffile or default)
# @return undef - if incorrect input parameters provided
# @author Alex Arkhipov
# @date August 2008
sub get_referenced_value($$$$)
{
   my ($self, $reffile, $xpath, $def_val) = @_;

   if (not defined($self))
   {
      baselog($LOG_ERROR, "Object is not provided");
      return undef;
   }

   if (not defined($def_val))
   {
      baselog($LOG_ERROR, "The default value is missing.");
      return undef;
   }

   my $retval = $def_val;

   if (not(defined $reffile) || not(defined $xpath))
   {
      return $retval;
   }
   my $refvalue = $self->get_value($reffile, $xpath);
   if (defined $refvalue)
   {
      $retval = $refvalue;
   }
   else
   {
      baselog($LOG_WARNING, "The rule does not contain a value for de-referencing.");
   }

   return $retval;
} ## end sub get_referenced_value($$$$)

1;
__END__

