## @file appupgrade.pm
# @brief This module checks current installation for possible upgrade
# and performs uninstallation if it is needed.
# @author Ilya Teterev
# @return  $FALSE if upgrade is needed but user cancels it
# @return  Otherwise $TRUE
# @version 1.00 - Initial version
# @version 1.01 - Checking applications scripts before execution added.
#                 Handle situations when scripts absent
# @version 1.02 - Protection from invoking "appstartup" during
#                 application installation is added.
# @version 1.03 - usage of 'su' command is replaced by invoking
#                 prepare_command_to_run_as subroutine.
# @version 1.04 - add support of a interactive and noninteractive installation.
# @version 1.05 - uninstall_prepare() function modified to execute scripts
#                 related to uninstalling applications only. Reverse order
#                 execution  for uninstall scripts added.
# @version 1.06 - Added support of users' accounts.
# @version 1.07 - logging output during deployment operatins has been added.
# @version 1.06 - Added support of users' accounts.
# @version 1.07 - logging output during deployment operatins has been added.
# @version 1.08 - Two colons were removed before shared functions and variables.
# @version 1.09 - patches uninstallation is moved just before RPMs uninstallation.
#                 Four new error codes are used: "CNSA", "PRUF", "DMPF", "CNRP".
# @version 1.10 - Added "sp" param to the poos() call in check_patches()
#                 Added the "piControl" folder to the "reserved_dirs" list in
#                 the application_cleanup()
# @version 1.11 - Add localpatchlibrary and patchlibrary directories to %reserved_dirs.
# @version 1.12 - Added the "base_harden" folder to the "reserved_dirs" list in
#                 the application_cleanup() 

package appinstall::appupgrade;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use install_common;
use patchutil::patches_target;
use CommonInstallUtils;

use ifxml;
use XML::Parser;
use Data::Dumper;
use base_semctl;
use baselog;
use appinstall::listgen;
use patchutil::patching;
use base_apps_service;
use base_system;
use base_ftp;

use Exporter ();
@ISA    = qw(Exporter);
@EXPORT = qw(&check_upgrade &application_cleanup);

our $VERSION = 1.12;

## @skipline
sub fill_forms();
## @skipline
sub preupdate();
## @skipline
sub uninstall_prepare();
## @skipline
sub check_upgrade();

our %rpmtree    = ();  #this variable is used for @rpmlist production
our @rpmlist    = ();  #RPMs installation order
our @applist    = ();  #applications installation order
our %preplist   = ();  #applications uninstallation scripts
our $workconfig = "";  #installed configuration

###############################################################################
## @fn check_upgrade()
# @brief   This function performs upgrade steps if user answers 'yes'.
# @details
# @return  $TRUE  - if previous installation is removed.
# @return  $FALSE - some error happened
sub check_upgrade()
{
   my $params = use_confver($CONFVER);
   unless ($params)
   {
      baselog($LOG_ERROR, "Can not read the configuration file.");
      return $FALSE;
   }

   #re-installation is used by default
   my $ansv              = 1;
   my $is_match_old_appl = $FALSE;

   # get current installed applciations
   my $appconfig = $params->{'APP_INSTALLED'};

   if ($INTERACTIVE)
   {
      print "\nIn order to continue, the previously installed applications\n";
      print "have to be removed. Also, you have to be sure that nobody else\n";
      print "is interactive with the system in anyway\n\n";
      return $FALSE unless (getYNResponse("Do you agree?", 'N'));
   } ## end if ($INTERACTIVE)
   foreach (@appinstall::listgen::configs)
   {
      if ($_ eq $appconfig)
      {
         $is_match_old_appl = $TRUE;
         last;
      }
   } ## end foreach (@appinstall::listgen::configs...

   $INSTALL_TYPE = "upgrade";
   unless ($is_match_old_appl)
   {
      if ($INTERACTIVE)
      {
         print "\nWarning: Media does not contain configuration, which is currently installed.\n";
         print "New application configuration will be installed (previous app-related data\n";
         print "might be lost)\n";
         return $FALSE unless (getYNResponse("Do you want to continue?", 'N'));
      } ## end if ($INTERACTIVE)
      else
      {
         $INSTALL_TYPE = "re-install";
      }
   } ## end unless ($is_match_old_appl...
   else
   {
      if ($INTERACTIVE)
      {
         while ($TRUE)
         {
            print "\nPlease choose one of the following:\n";
            print "1. Application re-installation (previous app-related data might be lost)\n";
            print "2. Application upgrade\n";
            print "Enter your choice (q to exit) [2]: ";
            $ansv = <STDIN>;
            chomp $ansv;
            $ansv = 2 if ($ansv eq "");
            return $FALSE if ($ansv =~ /^[Qq]$/);
            last          if ($ansv =~ /^[12]$/);
            print "Incorrect input.\n";
         } ## end while ($TRUE)
         $INSTALL_TYPE = ("re-install", "upgrade")[$ansv - 1];
      } ## end if ($INTERACTIVE)
   } ## end else
   if ($INTERACTIVE)
   {
      $INSTALL_TYPE = ("re-install", "upgrade")[$ansv - 1];
   }

   #let's prepare to upgrade
   #First reset $LASTERR value from UPGR state
   $LASTERR = "NULL";

   #update STATUS field in 'installedconfig' file for Nortel applications
   unless ($baseAppsmode)
   {
      return $FALSE unless (update_app_installation_status($CONFVER_STATES{'UNINSTALLING'}));
   }

   fill_forms();
   if ($LASTERR ne "NULL")
   {
      return $FALSE;  #The parent function will take care about the exit
   }

   print_header("Preuninstall stage");

   #Next step is to stop all applications:

   print "Stopping applications...\n";
   my $stop_val = $TRUE;
   my $app = new base_apps_service({'type' => $TYPE_NORTEL_APPL});
   unless (defined $app)
   {
      baselog($LOG_ERROR, "Can not create base_apps_service object");
      $stop_val = $FALSE;
   }
   else
   {
      unless ($app->stop())
      {
         $stop_val = $FALSE;
      }
   } ## end else
   unless ($stop_val)
   {
      baselog($LOG_ERROR, "Cannot stop applications. The 'appinstall' stops.", $TRUE);
      print " $CONTACT_MESSAGE\n";
      $LASTERR = "CNSA";
      undef $app;
      return $FALSE;
   } ## end unless ($stop_val)
   undef $app;

   #Don't invoke preupdate scripts if re-install
   if ($INSTALL_TYPE ne "re-install")
   {
      print "Preupdate stage...\n";

      # call preupdate scripts
      if (!preupdate())
      {
         baselog($LOG_ERROR, "Cannot proceed application upgrade. The 'appinstall' stops.", $TRUE);
         print " $CONTACT_MESSAGE\n";
         system("$RM -rf $BACKUPDIR/*");
         baselog($LOG_INFO, "Removing backup directory\n");
         $LASTERR = "PRUF";
         return $FALSE;
      } ## end if (!preupdate())
   } ## end if ($INSTALL_TYPE ne "re-install"...

   # Invoke uninstall scripts
   print "Preparing the uninstallation...\n";

   # Try to disable FTP mount points first.
   my @users_list = ();
   foreach my $app (@applist)
   {
      my $parser = ifxml::new("$APP_CFG_ROOT/$app", $APPLCFG);
      unless ($parser)
      {
         baselog($LOG_ERROR, "Could nor read $APPLCFG file of the '$app' application.");
         next;
      }
      my $users_inform = $parser->find_data_by_xpath($GETDATA, '/application/ftp/dir', $NOATTR);
      if (defined($users_inform))
      {
         for my $user (@$users_inform)
         {
            if (defined($user->{'user'}) && (pos_in_array(\@users_list, $user->{'user'}) < 0))
            {
               push @users_list, $user->{'user'};
            }
         } ## end for my $user (@$users_inform...
      } ## end if (defined($users_inform...
      undef $parser;
   } ## end foreach my $app (@applist)

   baselog($LOG_INFO, 'Deactivate mount points.');
   if (not base_ftp::deactivate_ftp_mounts(\@users_list))
   {
      baselog($LOG_ERROR, 'Cannot deactivate extended FTP access.');
      $LASTERR = "DMPF";
      return $FALSE;
   } ## end if (not base_ftp::deactivate_ftp_mounts...

   # call preuinstall scripts
   if (!uninstall_prepare())
   {
      baselog($LOG_ERROR, "Cannot un-install applications. The 'appinstall' stops.", $TRUE);
      print " $CONTACT_MESSAGE\n";
      $LASTERR = "STOP";
      return $FALSE;
   } ## end if (!uninstall_prepare...
   print "Preuninstall stage finished\n";

   #Uninstall application patches
   print "Uninstalling patches...\n";
   unless (check_patches())
   {
      baselog($LOG_ERROR, "Cannot remove applications patches. The 'appinstall' stops.", $TRUE);
      $LASTERR = "CNRP";
      return $FALSE;
   } ## end unless (check_patches())

   print_header("Uninstall RPM packages");

   uninstall_rpms(@rpmlist);
   if ($LASTERR ne "NULL")
   {
      return $FALSE;  #The parent function will take care about the exit
   }

   #Clean up application stuff
   print "Application cleanup...\n";
   application_cleanup();

   print "Uninstallation finished\n";
   return $TRUE;
} ## end sub check_upgrade()

###############################################################################
## @fn check_patches()
# @brief Checks if uninstalled applications have patches in service.
#        If so then "poos" and "pout" all of them.
# @return   TRUE  - operation successful
# @return   FALSE - some error happens
sub check_patches()
{
   baselog($LOG_INFO, "Patch uninstallation:\n");
   my $retval   = $TRUE;
   my $patching = new patchutil::patching;
   if (not defined($patching))
   {
      baselog($LOG_ERROR, 'Cannot create patching object');
      return $FALSE;
   }
   else
   {
      # "sp" will ignore start/stop/reboot tags in patch & not prompt user (non-interactive)
      my %params = ('app', undef, 'interact', $FALSE, 'appstop', $FALSE, 'sp', $TRUE);
      foreach my $appl (@applist)
      {
         $params{'app'} = $appl;
         unless ($patching->poos(\%params))
         {
            baselog($LOG_ERROR, 'Cannot put the patch out of service for ' . $appl . ' application');
            $retval = $FALSE;
         }
         unless ($patching->pout(\%params))
         {
            baselog($LOG_ERROR, 'Cannot unloaded the patch ' . $appl . ' application');
            $retval = $FALSE;
         }
      } ## end foreach my $appl (@applist)
      $patching = undef;
   } ## end else [ if (not defined($patching...
   baselog($LOG_INFO, "Patch uninstallation complete\n");
   return $retval;
} ## end sub check_patches()

###############################################################################
## @fn application_cleanup()
# @brief Cleanup applications directories.
# @details Cleanup the following directories after
# application uninstallation (if not empty):
#    \li /var/opt/nortel/"app_name"/
#    \li  /etc/opt/nortel/"app_name"/
#    \li /opt/nortel/"app_name"/
# This subroutine does not remove Base Applicarions directories.
# @return None
# @author Evgeny Lazarev
# @date April 2008
sub application_cleanup()
{
   my @app_dirs = ("/var/opt/nortel/", "/etc/opt/nortel/", "/opt/nortel/");
   my %reserved_dirs = ("base", "", ".", "", "..", "", "patch", "", "rpmdb", "", "deployment", "", "base-apps", "", "common", "", "groups", "", "ssl", "", "piControl", "", "localpatchlibrary", "", "patchlibrary", "", "base_harden", "");

   baselog($LOG_INFO, "Application cleanup:\n");
   foreach my $app_dir (@app_dirs)
   {
      my $dir_handle;
      opendir($dir_handle, $app_dir) || next;
      my @dirs = readdir($dir_handle);
      closedir($dir_handle);
      foreach my $dir (@dirs)
      {

         # remove directories specified in the array @app_dirs except reserved and Base Applications directories
         if ((-d "${app_dir}${dir}") and (!defined($reserved_dirs{$dir})) and (!isBaseApplication($dir)))
         {
            system("$RM -rf ${app_dir}${dir}");
            baselog($LOG_INFO, "$RM -rf ${app_dir}${dir}\n$?\n");
         }
      } ## end foreach my $dir (@dirs)
   } ## end foreach my $app_dir (@app_dirs...
   unlink $CONFVER;
   unlink $ARCHVER;

   #remove app license files
   if (system($RM, '-rf', $LICENSAPP) >> 8)
   {
      baselog($LOG_ERROR, "Could not delete license files");
   }

   baselog($LOG_INFO, "Application cleanup complete\n");
} ## end sub application_cleanup()

###############################################################################
## @fn preupdate()
# @brief   Calls application "preupdate" scripts
# @details
# @return  $TRUE  - operation successful
# @return  $FALSE - if one of "preupdate" script is absent or has wrong
#                     permissions
sub preupdate()
{
   if (!check_scripts('preupdate', \@applist))
   {
      return $FALSE;
   }
   my @revapplist = reverse @applist;
   baselog($LOG_INFO, "Preupdate stage:\n");
   foreach $appl (@revapplist)
   {
      foreach $entry (@{$preplist{$appl}})
      {
         my ($cfile, $cas) = ($entry->{'file'}, $entry->{'as'});

         #Let execute this file:
         my @cmdarr = @{prepare_command_to_run_as($cfile, $cas)};
         if (scalar @cmdarr != 0)
         {
            my $retval = $OK;
            if (!$INTERACTIVE)
            {
               do_system(\@cmdarr, {$dsRETVAL => \$retval, $dsCHDIR => $dsHOMEDIR, $dsOUTMODE => '>>', $dsERRMODE => '>>', $dsERRFILE => $APPINSTALL_ERRFILE, $dsOUTFILE => $APPINSTALL_OUTFILE});
            }
            else
            {
               do_system(\@cmdarr, {$dsRETVAL => \$retval, $dsINTERACT => $TRUE, $dsCHDIR => $dsHOMEDIR});
            }
            baselog($LOG_INFO, "'@cmdarr': $retval");
         } ## end if (scalar @cmdarr != ...
         else
         {
            baselog($LOG_ERROR, "Script '$cfile' can not be executed.");
         }
      } ## end foreach $entry (@{$preplist...
   } ## end foreach $appl (@revapplist)
   baselog($LOG_INFO, "Preupdate stage complete\n");
   return $TRUE;
} ## end sub preupdate()

###############################################################################
## @fn uninstall_prepare()
# @brief   Calls application "preuninstall" scripts.
# @details Only applications that will be uninstalled are subjects for checks
#          and scripts execution.
#     do not block the uninstall process as data integrity could be
# broken - some of the applications already have passed uninstall
# prepare stage.
# @return  TRUE - operation successful
# @return  FALSE - if validation check of the applications preuninstall scripts
#                  failed.
# @note If any of the applications uninstall script of just one of them  cannot
#       be called the it would not fail uninstall_prepare. It would be captured
#       in the log file only.
# @todo If none of the required applications uninstall script cannot be called
#       then  uninstall_prepare() must return FALSE to indicate a serious
#       issue.
sub uninstall_prepare()
{
   my $retval = $TRUE;
   if (!check_scripts('preuninstall', \@applist))
   {
      return $FALSE;
   }
   baselog($LOG_INFO, 'Preuninstall stage:');
   my @revapplist = reverse @applist;
   foreach my $app (@revapplist)
   {
      my $parser = ifxml::new("$APP_CFG_ROOT/$app", $APPLCFG);
      if (!$parser)
      {
         baselog($LOG_WARNING, "Cannot initialize parser for '$app'.");
         baselog($LOG_WARNING, "Cannot detect preuninstall scripts for '$app' applications.", $TRUE);
      }
      else
      {
         my $tag_info = undef;
         $tag_info = $parser->get_preuninstall_script();
         foreach my $script (@{$tag_info})
         {
            my ($cfile, $cas) = ($script->{'file'}, $script->{'as'});

            #Let execute this file:
            my @cmdarr = @{prepare_command_to_run_as($cfile, $cas)};
            if (scalar @cmdarr != 0)
            {
               my $retval = $OK;
               if (!$INTERACTIVE)
               {
                  do_system(\@cmdarr, {$dsRETVAL => \$retval, $dsCHDIR => $dsHOMEDIR, $dsOUTMODE => '>>', $dsERRMODE => '>>', $dsERRFILE => $APPINSTALL_ERRFILE, $dsOUTFILE => $APPINSTALL_OUTFILE});
               }
               else
               {
                  do_system(\@cmdarr, {$dsRETVAL => \$retval, $dsINTERACT => $TRUE, $dsCHDIR => $dsHOMEDIR});
               }
               baselog($LOG_INFO, "'@cmdarr': $retval");
            } ## end if (scalar @cmdarr != ...
            else
            {
               baselog($LOG_ERROR, "Script '$cfile' can not be executed.");
            }
         } ## end foreach my $script (@{$tag_info...
      } ## end else [ if (!$parser)
      undef $parser;
   } ## end foreach my $app (@revapplist...

   baselog($LOG_INFO, "Preuninstall stage complete\n");
   return $retval;
} ## end sub uninstall_prepare()

###############################################################################
## @fn fill_forms()
# @brief  This function parses old install.xml and configure.xml files.
# @details
sub fill_forms()
{
   my $params = use_confver($CONFVER);
   return unless $params;

   #Get installed SW version
   $workconfig = $params->{'APP_INSTALLED'};
   return unless $workconfig;

   #Parsing installed
   my $parser = new XML::Parser(Style => 'Subs', Pkg => 'get_order');
   $parser->parsefile($ARCHVER);
   print_xml_dump($INSTLOG, "Parsing $ARCHVER:\n", Dumper(\@applist));

   #Let's generate rpmlist
   foreach $app (@applist)
   {
      foreach $file (@{$rpmtree{$app}})
      {
         push @rpmlist, $file;
      }
   } ## end foreach $app (@applist)
   print_xml_dump($INSTLOG, Dumper(\@rpmlist));

   #Let's check configure.xml files for uninstall scripts
   parse_configs("get_prepare");

   print_xml_dump($INSTLOG, "Parsing configure.xml files:\n", Dumper(\%preplist));
} ## end sub fill_forms()

{

   package get_order;
   use appinstall::appupgrade;
   use common_variables;

   my $myconfig = $FALSE;
   my $appname  = "";

   sub supportedConfig()
   {
      my $name = {@_}->{'name'};
      if ($name eq $workconfig)
      {
         $myconfig = $TRUE;
      }
   } ## end sub supportedConfig()

   sub supportedConfig_()
   {
      $myconfig = $FALSE;
   }

   sub application()
   {
      $appname = {@_}->{'name'};
      if ($myconfig)
      {

         #Store application order
         push @applist, $appname;
      } ## end if ($myconfig)
   } ## end sub application()

   sub application_()
   {
      $appname = "";
   }

   sub file()
   {

      #Store RPM order
      push @{$rpmtree{$appname}}, {@_}->{'name'};
   }
}

{

   package get_prepare;
   use appinstall::appupgrade;
   use common_variables;

   my $appname = "";

   sub application()
   {
      $appname = {@_}->{'name'};
      $preplist{$appname} = ();
   }

   sub application_()
   {
      $appname = "";
   }

   sub preupdate()
   {
      $section = $TRUE;
   }

   sub preupdate_()
   {
      $section = $FALSE;
   }

   sub script()
   {
      if ($section)
      {
         push @{$preplist{$appname}}, {'file' => {@_}->{'file'}, 'as' => {@_}->{'as'}};
      }
   } ## end sub script()
}

1;
__END__

=head1 NAME

appupgrade - removes all data and prepares system for new installation

=head1 SYNOPSIS

use appupgrade;

=head1 FUNCTIONS

=head2 check_upgrade()

Checks for possible upgrade story

=head1 LIMITATIONS

=head1 SEE ALSO

