# ############################################################
# Description:
# This module contains application media check functions
# Input/Output:
# Returns current operration status (see codes in errors.pm)
# Author:
# Ilya Teterev
#
# 1.4 - To export shared functions 'do_cdcheck' and 'do_swcheck'.
#       Two colons before shared variables were removed.
# 1.3 - Added support for alphanumeric characters in rpm name
# 1.2 - Progress bar is added
# 1.1 - Initial version
# ############################################################
package appinstall::check_cd;

use lib "/opt/nortel/base/lib";

use common_variables;
use install_common;

use warnings;
use strict;

use Digest;
use ifxml;
use baselog;

use Exporter;
our @ISA=qw(Exporter);
our @EXPORT=qw(&do_cdcheck &do_swcheck);

our $VERSION = "1.4";

sub do_cdcheck()
{
   my $olddir;

   #go to CD
   chomp($olddir = qx/$PWD/);
   chdir $MNTPOINT;

   my $retval = chk_files();

   chdir $olddir;
   return $retval;
} ## end sub do_cdcheck()

sub chk_files()
{
   my ($md5digest, $filename, $line);
   print "It may take some time to perform checking.\n";
   print "Please wait...\n";

   #1. Checking for .diskinfo existence.
   if (!(-f "./$DINFO"))
   {
      $LASTERR = "NDSK";
      return $FALSE;
   }

   #2. Checking md5sums
   unless (open(DiskInfo, $DINFO))
   {
      $LASTERR = "MCOR";
      baselog($LOG_ERROR, "Can not open .diskinfo file");
      return $FALSE;
   } ## end unless (open(DiskInfo, $DINFO...
   $LASTERR = "NULL";

   #save previous value of AUTOFLUSH
   my $prv_val = $|;

   #next line tells perl to flush after each write
   $| = 1;
   my $i     = 1;
   my @lines = <DiskInfo>;
   close(DiskInfo);
   my $files_number = scalar @lines;
   foreach $line (@lines)
   {
      ($md5digest, $filename) = split(" ", $line);
      print "Checking file $i of $files_number ...\r";
      if (!chk_md5($filename, $md5digest))
      {
         last;
      }
      $i++;
      baselog($LOG_DEBUG, "File $filename was checked successfully");
   } ## end foreach $line (@lines)
   if ($LASTERR ne "NULL")
   {
      return $FALSE;
   }
   $| = $prv_val;
   print "\nThe media is OK.\n";
   return $TRUE;
} ## end sub chk_files()

##################################################
# This sub calculates the Md5 digest of the file and compares it with
# the given digest. If the digests are different or if the file can not
# be read the sub reports about the error.
#
# INPUT:  $_[0]  - file name
#         $_[1]  - MD5 digest in hexadecimal form
#
# OUTPUT: $TRUE  - calculated MD5 digest is the same
#         $FALSE - the digests are diferent or the file can not be read
##################################################
sub chk_md5($$)
{
   my ($file, $md5_of_file) = @_;

   my $md5 = Digest->new("MD5");

   # 1. Open the file
   unless (open(CheckedFile, $file))
   {
      $LASTERR = "MCOR";
      baselog($LOG_ERROR, "Can not open $file file");
      return $FALSE;
   } ## end unless (open(CheckedFile, ...

   # 2. Geting MD5 digest of the file
   eval {$md5->addfile('CheckedFile')};
   if ($@)
   {
      $LASTERR = "MCOR";
      baselog($LOG_ERROR, "File $file can not be read");
      return $FALSE;
   } ## end if ($@)
   close(CheckedFile);

   # 2. Comparing digests
   if ($md5->hexdigest ne $md5_of_file)
   {
      $LASTERR = "MD5F";
      baselog($LOG_ERROR, "Wrong MD5 sum of $file file");
      return $FALSE;
   } ## end if ($md5->hexdigest ne...

   return $TRUE;
} ## end sub chk_md5($$)

sub do_swcheck()
{
   my $olddir;
   chomp($olddir = qx/$PWD/);
   chdir $MNTPOINT;

   my $retval = chk_swver();

   chdir $olddir;
   return $retval;
} ## end sub do_swcheck()

sub chk_swver()
{
   my $basedir = (split("\n", qx/$FIND . -name \'$REL_NAME*\' -printf \"%f\n\"/))[0];
   if (!defined($basedir))
   {
      print "No data found.\n";
      $LASTERR = "NDIR";
      return $FALSE;
   } ## end if (!defined($basedir)...

   $INSTALL_FOLDER = "$MNTPOINT/$basedir";
   $INSTVER        = "$INSTALL_FOLDER/$INSTCFG";
   $LASTERR        = "NULL";

   #Check Media for the SW version
   my $parser = ifxml::new($INSTALL_FOLDER, $INSTCFG);
   if (!$parser)
   {
      print "ERROR: Can not find $INSTCFG file.\n";
      $LASTERR = "WCFG";
      return $FALSE;
   } ## end if (!$parser)

   my $entries = $parser->get_base();
   @$entries[0]->{'version'} =~ /(\d+\.\d+)\.\d+/;
   my $sw_ver = $1;

   #Define Linux Base release version
   $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   if (!$parser)
   {
      print "ERROR: Wrong format of $APPLCFG file.\n";
      $LASTERR = "WFLS";
      return $FALSE;
   } ## end if (!$parser)

   $entries = $parser->get_application({name => 'base'});
   @$entries[0]->{'version'} =~ /(^\d+\.\d+)\.\d+/;
   my $base_ver = $1;

   #calling destructor for xml parser;
   undef $parser;

   if ($sw_ver ne $base_ver)
   {
      print "ERROR: Wrong version of the application media ($sw_ver).\n";
      print "       Must be equal LinuxBase version ($base_ver).\n";
      $LASTERR = "WVER";
      return $FALSE;
   } ## end if ($sw_ver ne $base_ver...

   return $TRUE;
} ## end sub chk_swver()

1;
__END__

=head1 NAME

check_cd - performs initial application Media checking

=head1 SYNOPSIS

use check_cd;

=head1 FUNCTIONS

=head2 do_cdcheck

Prepares the environment and calls chk_files.

=head2 chk_files

Looks for .diskinfo file and checks md5sum for all files on Media.

=head2 chk_md5

Gets MD5 digest of the file and compares it with the given.

=head2 do_swcheck

Prepares the environment and calls chk_swver.

=head2 chk_swver

Checks application media for folder which contains files for installed base release and checks install.xml format and presence.

=head1 LIMITATIONS

=head1 SEE ALSO

