## @file errors.pm
# @brief This module contains error messages hash and functions which control
#   operations with log files
# @details
# @author Ilya Teterev
# @author Kate Bazhanova
# @author Lazarev Evgeny
# @author Vladimir Ivanov
# @date November 2008
# @version 1.00 - Initial version
# @version 1.01 - Unnecessary function definitions deleted
# @version 1.02 - "Wrong format of install.xml file" is changed on "Cannot find install.xml file";
# @version 1.10 - new hash ERRCODES were added. New errors types: ANIN, AAIN, UNER, UNPA, NEPA were added.
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.12 - Four new error codes were added: CNSA, PRUF, DMPF, CNRP.
# @version 1.13 - Modified under the perlcheck warnings cleanup activity.
# @version 1.14 - New error code is added: RSBLK.
# @version 1.15 - RPMDB added. 
# @version 1.16 - RPMUF added.
# @version 1.17 -  New error code is added:'WAPPVER'. 
# @note

package appinstall::errors;

use lib "/opt/nortel/base/lib";

use install_common;

use Exporter;
use baselog;
our @ISA    = qw(Exporter);
our @EXPORT = qw(%ERRCODES %ERRORDB &error_decoder);

our $VERSION = '1.17';

our %ERRORDB = (
                AAIN    => "Applications are already installed\n",
                ANIN    => "Applications are not installed\n",
                BKUP    => "Cannot create backup archive\n",
                BKUPIPN => "Backup server IP is undefined.\n",
                CFGV    => "The media contains old install.xml\n",
                CLDF    => "Clearing directory is failed.\n",
                CNCL    => "Application installation is cancelled\n",
                CNRP    => "Cannot remove applications patches\n",
                CNSA    => "Cannot stop applications\n",
                CRDF    => "Creating directory is failed.\n",
                DANF    => "Defining backup/restore archive name is failed.\n",
                DMPF    => "Deactivation of FTP/SFTP mount points failed.\n",
                EMNT    => "Cannot mount application media.\n",
                INPF    => "Input parameter(s) validation is failed.\n",
                INSF    => "Cannot copy installation xml\n",
                INVIP   => "Invalid IP\n",
                KEYCD   => "Could not get keycode file.\n",
                LICNS   => "Cannot get license data.\n",
                MCOR    => "Application media is corrupted. File(s) can not be checked.\n",
                MD5F    => "MD5 sum check failed.\n",
                MNTC    => "Cannot mount application media.\n",
                NCFG    => "Cannot find install.xml for installed release.\n",
                NDIR    => "The media doesn't contain any files for installed release.\n",
                NDSK    => "The .diskinfo file is not found.\n",
                NEPA    => "Not enough parameters\n",
                NFLS    => "No configure.xml files found\n",
                NULL    => "No errors\n",
                NUSR    => "Please login as $INSTALL_USERNAME.\n",
                PRECFG  => "Cannot get preconfig data.\n",
                PREDEPF => "Pre-deployment failed.\n",
                PREP    => "Installation data preparations error\n",
                PRUF    => "Pre-update stage of applications installation failed.\n",
                RESF    => "Restore of applications was failed\n",
                RPMDB   => "RPM database is corrupted",
                RPMF    => "RPM installation failed\n",
		RPMUF   => "RPM uninstallation failed\n",
                RSBLK   => "Common resource is blocked by another process.\n",
                RSWF    => "Remote script is failed.\n",
                STOP    => "Application installation is stopped\n",
                TRNSF   => "Transfer failed.\n",
                UNER    => "Undeployment error\n",
                UNKCF   => "Keycode file is undefined\n",
                UNMTHD  => "Method is undefined\n",
                UNPA    => "Undefined parameter\n",
                UNPTH   => "Path is undefined.\n",
                UNPWD   => "Password is undefined\n",
                UNUSR   => "Undefined user\n",
                UPGR    => "Previous configurations needs to be removed for the upgrade\n",
                WAPPVER => "Incorrect application configuration or version\n",
                WCFG    => "Cannot find install.xml file\n",
                WFLS    => "Wrong format of configure.xml\n",
                WVER    => "Application media version mismatches Linux Base version\n",
                XMLE    => "Cannot create XML file.\n",
               );
our %ERRCODES = (
                 NULL    => 0,
                 MNTC    => 2,
                 NDSK    => 3,
                 NUSR    => 4,
                 MD5F    => 5,
                 MCOR    => 6,
                 NDIR    => 7,
                 NCFG    => 8,
                 EMNT    => 9,
                 XMLE    => 10,
                 NFLS    => 11,
                 CFGV    => 12,
                 UPGR    => 13,
                 WCFG    => 14,
                 WFLS    => 15,
                 WVER    => 16,
                 CNCL    => 17,
                 STOP    => 18,
                 ANIN    => 19,
                 AAIN    => 20,
                 UNER    => 21,
                 UNPA    => 22,
                 NEPA    => 23,
                 RPMF    => 24,
                 INSF    => 25,
                 PREP    => 26,
                 PRECFG  => 27,
                 RESF    => 28,
                 BKUP    => 29,
                 INVIP   => 30,
                 DANF    => 31,
                 INPF    => 32,
                 RSWF    => 33,
                 CLDF    => 34,
                 CRDF    => 35,
                 PREDEPF => 36,
                 BKUPIPN => 37,
                 TRNSF   => 38,
                 UNKCF   => 39,
                 UNMTHD  => 40,
                 UNPTH   => 41,
                 UNPWD   => 42,
                 UNUSR   => 43,
                 CNSA    => 44,
                 CNRP    => 45,
                 DMPF    => 46,
                 PRUF    => 47,
                 LICNS   => 48,
                 RSBLK   => 49,
                 RPMDB   => 50,
                 RPMUF   => 51,
                 WAPPVER => 52
                );

###############################################################################
## @fn       error_decoder($error_code)
# @brief     Get the abbreviation of the error by error code.
# @details   None.
# @param     $error_code - error code getting by appinstall
# @return    $error   - abbreviation of the error
# @return    undef    - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
# @note      None
sub error_decoder($)
{
   my $error_code = shift @_;
   if (!defined $error_code)
   {
      baselog($LOG_ERROR, 'Error code is undefined');
      return undef;
   }
   foreach my $error (keys %ERRCODES)
   {
      if ($ERRCODES{$error} eq $error_code)
      {
         return $error;
      }
   } ## end foreach my $error (keys %ERRCODES...
   baselog($LOG_ERROR, "Cannot define appinstall error by error code [ $error_code ]");
   return undef;
} ## end sub error_decoder($)

1;
__END__

=head1 NAME

errors - error messages DB.

=head1 SYNOPSIS

use errors;

=head1 FUNCTIONS

=head1 LIMITATIONS

=head1 SEE ALSO

