## @file listgen.pm
# @brief This module reads install.xml
# @details
# @author Ilya Teterev
# @author Lazarev Evgeny
# @author Vladimir Ivanov
# @date November 2008
# @version 1.00 - Initial version
# @version 1.01 - To export shared functions and variables (get_package, @configs);
#  two colons before shared variables were removed.
# @version 1.02 - Correct handle of situation when during non-interactive upgrade
#  function check_upgrade() returns $FALSE.
# @version 1.03 - reload monit if applications were removed
# @version 1.04 - Modified under the perlcheck warnings cleanup activity.
# @version 1.05 - 'get_app_rpms' function is implemented.
# @note
package appinstall::listgen;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use install_common;
use base_monit;

use XML::Parser;
use Data::Dumper;
use appinstall::appupgrade;
use appinstall::show_app_info;
use baselog;
use ifxml;
use recovery::backup;
use CommonInstallUtils;
use appinstall::update_sys_data;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&get_package @configs &get_app_rpms);
our $VERSION = '1.05';

my %deplistscan = ();  #list of applications RPMs. application name => [rpm names]
my %apporder    = ();  #application installation order
our @configs = ();     #supported configs order
my $textStatus = $FALSE;  #is required for baseVersion tag
my $iversion   = "";      #supported config version

my $version        = '';  #install.xml version
my @banner         = ();  #banner message from install.xml
my $BANNER_CFG_VER = 1.2; #install.xml version with banner supporting

## @fn       get_package()
# @brief     returnes information from the install.xml file in specific format.
# @return    array with references to %deplistscan and $apporder{selected_configuration}.
# @author    Ilya Teterev
# @details   Format:
# \li %deplistscan: key - application's name. Value - reference to array with RPM names.
# \li $apporder: key - supported configuration's name. Value - reference to array with
#     applications' names belonging to selected configuration.
sub get_package()
{
   my $usersel = 0;

   #get package list
   if (not -f $INSTVER)
   {
      install_common::print_xml_dump($INSTLOG, "Can not find $INSTVER file.\n");
      $LASTERR = "WCFG";
      return;
   } ## end if (not -f $INSTVER)
   my $parser = new XML::Parser(Style => 'Subs', Pkg => 'listgen_tags');
   $parser->setHandlers(Char => \&listgen_tags::get_text);
   $parser->parsefile($INSTVER);
   install_common::print_xml_dump($INSTLOG, "Parsing install.xml:\n", Dumper(\%deplistscan));

   my $parser_ifxml = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   return if (!$parser_ifxml);
   my $table = $parser_ifxml->get_application();

   #array with installed applications
   my @oldlist;

   foreach my $entry (@$table)
   {
      push @oldlist, " ", ($entry->{'name'}, " ", $entry->{'version'}), "\n";
   }
   my $params = use_confver($CONFVER);
   if ($params)
   {
      print_xml_dump($INSTLOG, "The following applications are found:\n", $params->{'APP_INSTALLED'}, " ", $params->{'APP_VERSION'}, "\n", @oldlist);
      $LASTERR = "UPGR";
   }

   #print the package list to the log file
   install_common::print_xml_dump($INSTLOG, "Configurations:\n", Dumper(\@configs), "Applications:\n", Dumper(\%apporder));
   my $break_loop = $FALSE;
   my $retval     = $ERROR;
   if ($LASTERR ne "NULL")
   {

      if ($LASTERR eq "UPGR")
      {
         if ($INTERACTIVE)
         {
            showAppInfo("all");
         }
         $retval = check_upgrade();
      } ## end if ($LASTERR eq "UPGR"...
      if (!$retval)
      {
         print "Can't perform the installation\n";
         $break_loop = $TRUE;
      }
   } ## end if ($LASTERR ne "NULL"...
   if ($INTERACTIVE)
   {
      while (!$break_loop)
      {
         print_header("Installation stage");
         if (($version >= $BANNER_CFG_VER) && (scalar(@banner) != 0))
         {

            #removes last element from banner message (usually spaces or tabs)
            pop(@banner) if ($banner[scalar(@banner) - 1] =~ /^[\ \t]+$/);
            print @banner;
         } ## end if (($version >= $BANNER_CFG_VER...
         else
         {
            print "The following supported configurations are found: \n";
            baselog($LOG_WARNING, "$version install.xml version doesn't have a <banner> tag\n")
              unless ($version < $BANNER_CFG_VER);
         } ## end else [ if (($version >= $BANNER_CFG_VER...
         my $i = 1;
         if ($INSTALL_TYPE eq 'upgrade')
         {
            for ($i = 0 ; $i <= $#configs ; $i++)
            {
               if ($params->{'APP_INSTALLED'} eq $configs[$i])
               {
                  $usersel = ++$i;
                  last;
               }
            } ## end for ($i = 0 ; $i <= $#configs...
            $break_loop = $TRUE;
         } ## end if ($INSTALL_TYPE eq 'upgrade'...
         else
         {
            foreach $name (@configs)
            {
               print $i++ . ". " . $name . " $iversion ", "\n";
            }
            print "Please select the supported configuration # to install (q for exit): ";

            chomp($usersel = <STDIN>);

            if (($usersel =~ /^\d+$/) && ($configs[$usersel - 1]))
            {
               $break_loop = $TRUE;
            }
            if ($usersel =~ /^[Qq]$/)
            {
               $usersel    = 0;
               $break_loop = $TRUE;
               if ($INSTALL_TYPE eq "re-install")
               {

                  #restart monit with configuration(monitrc) reload
                  unless (appinstall::update_sys_data::update_sdb())
                  {
                     baselog($LOG_ERROR, "Cannot reload configuration data.");
                  }
                  else
                  {
                     baselog($LOG_DEBUG, "restart monit.");
                     restart_monit();
                  }
               } ## end if ($INSTALL_TYPE eq "re-install"...
               $LASTERR = "CNCL";
            } ## end if ($usersel =~ /^[Qq]$/...
         } ## end else [ if ($INSTALL_TYPE eq 'upgrade'...
      } ## end while (!$break_loop)
   } ## end if ($INTERACTIVE)
   else
   {

      # if check_upgrade returned $FALSE do not install applications (get_package()
      # will return two empty arrays). Otherwise install the first configuration
      # from the install.xml file
      if (!$break_loop)
      {
         $usersel = 1;
      }
   } ## end else [ if ($INTERACTIVE)

   #preparing result array
   my @retval = ();
   push @retval, (($usersel) ? \%deplistscan : 0);
   push @retval, (($usersel) ? $apporder{$configs[$usersel - 1]} : 0);

   #store selected configuration name
   my $cfgname = $configs[$usersel - 1];

   if ($usersel)
   {
      $CONFVER_CONTENT->{'APP_INSTALLED'}=$cfgname;
      $CONFVER_CONTENT->{'APP_VERSION'}=$iversion;
      unless ($baseAppsmode)
      {
         $CONFVER_CONTENT->{'STATUS'}=$CONFVER_STATES{'STARTED'};
      }      
      write_file($CONFVER,$CONFVER_CONTENT,$TRUE);
      chmod 0644, $CONFVER;
      if (-e $UPGRADE_RESTORE)
      {

         #check default upgrade data
         my ($app_config, $app_ver) = getBackupConfiguration($UPGRADE_RESTORE);
         if ($app_config and $app_ver)
         {
            if ($INTERACTIVE)
            {
               my $warning = "\nRestore data is found for application\n Configuration: $app_config\n Version: $app_ver\nDo you want to restore data from the archive?";
               if (!getYNResponse($warning, "Y"))
               {
                  if (system($RM, "-rf", $UPGRADE_RESTORE) != 0)
                  {
                     baselog($LOG_WARNING, "Cannot remove default upgrade restore archive.");
                  }
               } ## end if (!getYNResponse($warning...
            } ## end if ($INTERACTIVE)
         } ## end if ($app_config and $app_ver...
         else
         {
            baselog($LOG_ERROR, "Default upgrade back up data found. But configuration data cannot be retrieved.\n", $TRUE);
         }
      } ## end if (-e $UPGRADE_RESTORE...

   } ## end if ($usersel)
   return @retval;
} ## end sub get_package()

## @fn       get_app_rpms($app_set)
# @brief     returnes referense to array with rpms for specifyed application set.
# @details   variable $INSTVER from install_common must be defined first!
# @param     $app_set - name of set of Nortel applications(value of APP_INSTALLED 
#                       field in installedconfig file).  
# @return    referense to array.
# @return    undef if some errors happens.
sub get_app_rpms($)
{
   my $app_set=shift @_;
   my $app_rpms=undef;
   #get package list
   unless (defined $INSTVER)
   {
      baselog($LOG_ERROR,"Variable 'INSTVER' must be initialized first.");
      return undef;
   }
   if (not -f $INSTVER)
   {
      install_common::print_xml_dump($INSTLOG, "Can not find $INSTVER file.\n");
      return undef;
   } ## end if (not -f $INSTVER)
   my $parser = new XML::Parser(Style => 'Subs', Pkg => 'listgen_tags');
   unless(defined $parser)
   {
      baselog($LOG_ERROR,"XML parser is not creted.");
      return undef;
   }
   $parser->setHandlers(Char => \&listgen_tags::get_text);
   $parser->parsefile($INSTVER);

   foreach my $app_name (@{$apporder{$app_set}})
   {
      push @{$app_rpms},@{$deplistscan{$app_name}};
   }
   undef $parser;
   return $app_rpms;
}

{

   package listgen_tags;
   use appinstall::listgen;
   use install_common;
   use common_variables;
   our $currpkg = "none";  #current package
   our $currapp = "none";  #current application
   our $section;

   sub get_text()
   {
      push @banner, $_[1] if (defined($section) && $section eq 'banner');
   }

   sub install()
   {
      $version = {@_}->{'version'};
      if ($version < $MIN_CFGVER)
      {
         $LASTERR = "CFGV";
      }
   } ## end sub install()

   sub install_()
   {
      return;
   }

   sub supportedConfig()
   {

      #check the presence of the application
      $currpkg = {@_}->{'name'};
      if (!$apporder{$currpkg})
      {
         $apporder{$currpkg} = ();
         push @configs, $currpkg;
      }
   } ## end sub supportedConfig()

   sub supportedConfig_()
   {
      $currpkg = "none";
   }

   sub base()
   {
      $iversion = {@_}->{'version'};
      $iversion =~ s/nortel-cs1000-//;
   }

   sub application()
   {
      $currapp = {@_}->{'name'};

      #if we are not in supportedConfig section
      #let's scan for RPM order, else
      #let's scan for application order
      if ($currpkg eq "none")
      {
         if (!$deplistscan{$currapp})
         {
            $deplistscan{$currapp} = [];
         }
      } ## end if ($currpkg eq "none"...
      else
      {

         #looking for duplicate entries
         my %tmparr = map {$_, 1} @{$apporder{$currpkg}};
         if (!$tmparr{$currapp})
         {
            push @{$apporder{$currpkg}}, ($currapp);
         }
      } ## end else [ if ($currpkg eq "none"...
   } ## end sub application()

   sub application_()
   {
      $currapp = "none";
      return;
   }

   sub banner()
   {
      $section = "banner";
      return;
   }

   sub banner_()
   {
      $section = undef;
      return;
   }

   sub file()
   {

      #TODO: CHECK that RPM packages are not installed
      #
      my $name = {@_}->{'name'};
      push @{$deplistscan{$currapp}}, ($name);
   } ## end sub file()
}

1;
__END__

=head1 NAME

listgen - generates list of packages ready to install.

=head1 SYNOPSIS

use listgen;

=head1 FUNCTIONS

=head2 get_package()

Returns the user selected package to install and generate dependency order to get this package

=head1 LIMITATIONS

=head1 SEE ALSO

