#############################################################
## @file show_app_info.pm 
# @brief This module contains the function, which show application
#        information (name and version)
# @author Peter Kascheev
# @version 1.00 - Initial version
# @version 1.01 - showAppInfo() was modified by Ekaterina Bazhanova
# @version 1.02 - get rid of global variables $APP_INSTALLED and $APP_VERSION.
# @version 1.03 - Two colons were removed before shared functions and variables.
# @version 1.04 - Base product info changed for clarity
# @version 1.05 - SU information no longer displayed associated with applications
# @note
package appinstall::show_app_info;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use patchutil::patching;
use patchutil::patches_target;
use patchutil::patches_public;
use install_common;

our $VERSION = "1.05";
use warnings;
use strict;

use baselog;
use basedb;
use ifxml;

use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(&showAppInfo);

## @fn  showAppInfo
# @brief print information about application installed (name and version)
# @details
# @param $_[0] - 'app'  - get versions of applications only
#             or 'base' - get version of base only
#             or 'all'  - get versions of base and applications
# @return $TRUE - all information is successfuly printed
#         $FALSE - some error happens.
# @author
# @date
sub showAppInfo
{
   my $mode = shift(@_);
   my ($app_installed, $app_version) = ('undefined', 'undefined');

   if (($mode eq 'base') or ($mode eq 'all'))
   {
      my $baseapps = getBaseApplicationsList();
      my $params   = use_confver($BASEAPPCONFVER);
      if ($params)
      {
         if ($params->{'APP_INSTALLED'})
         {
            $app_installed = $params->{'APP_INSTALLED'};
         }
         if ($params->{'APP_VERSION'})
         {
            $app_version = $params->{'APP_VERSION'};
         }
      } ## end if ($params)
      print "Product Release: $app_version\n";
      print "$app_installed\n";

      ## Show Linux Base version
      # (this part of the script should be deleted then Linux Base will be a base application
      my @base = ('base');
      unless (show(\@base))
      {
         baselog($LOG_ERROR, "Cannot show Linux Base version");
         return $FALSE;
      }
      ## end Show Linux Base version

      unless (show($baseapps))
      {
         baselog($LOG_ERROR, "Cannot show Base Configuration version");
         return $FALSE;
      }
   } ## end if (($mode eq 'base') ...

   if (($mode eq 'app') or ($mode eq 'all'))
   {
      my $packages   = getInstalledPackagesList();
      my $nortelapps = getNortelApplicationsList();

      #Reading installation data

      ($app_installed, $app_version) = ('undefined', 'undefined');
      my $params = use_confver($CONFVER);
      if ($params)
      {
         if ($params->{'APP_INSTALLED'})
         {
            $app_installed = $params->{'APP_INSTALLED'};
         }
         if ($params->{'APP_VERSION'})
         {
            $app_version = $params->{'APP_VERSION'};
         }
      } ## end if ($params)

      print "Application configuration: $app_installed\n";
      if (defined @$packages[0])
      {
         print "Packages:\n";
         foreach my $package (@$packages)
         {
            print "$package\n";
         }
      } ## end if (defined @$packages...
      else
      {
         baselog($LOG_WARNING, "No installed packages found");
      }
      print "Configuration version:     $app_version\n";

      unless (show($nortelapps))
      {
         baselog($LOG_ERROR, "Cannot show Application Configuration version");
         return $FALSE;
      }
   } ## end if (($mode eq 'app') or...

   return $TRUE;
} ## end sub showAppInfo

## @fn  show
# @brief print information about application installed (name and version)
# @details
# @param $applist - list of applications
# @return $TRUE - all information is successfuly printed
#         $FALSE - some error happens.
# @author
# @date
sub show($)
{
   my $applist = shift;
   my ($name, $version) = ();
   unless ($applist)
   {
      baselog($LOG_ERROR, "Application list is undefined.\n");
      return $FALSE;
   }

   format VERSION_SHOW =
   @<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<
   $name,                       $version
.
   my $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   if (!$parser)
   {
      baselog($LOG_ERROR, "No configuration file ($APPLCFG) found.\n");
      return $FALSE;
   }

   my $format = $~;
   $~ = "VERSION_SHOW";

   foreach my $app (@$applist)
   {
      unless (is_app_installed($app))
      {
         baselog($LOG_ERROR, "Application $app is not installed.");
         next;
      }
      my $table = $parser->get_application({name => $app});

      foreach my $entry (@$table)
      {
         ($name, $version) = ($entry->{'name'}, $entry->{'version'});
         $version = "n/a" if (!defined($version));
         write;
      } ## end foreach my $entry (@$table)
   } ## end foreach my $app (@$applist)

   $~ = $format;
   undef $parser;
   return $TRUE;
} ## end sub show($)

1;

__END__

=head1 NAME

show_app_info - show name and version of application installed.
