# ############################################################
# @file update_sys_sdb
# @brief This module updates system DB from configure.xml files
#
# @author Ilya Teterev
# @author Alexander Kolotov
# @author Alex Arkhipov
# @author Vladimir Ivanov
# @date November 2008
# @version 1.00 - Initial version
# @version 1.01 - The support of references within 'iptables' section is added.#
# @version 1.10 - The support of resources testing within 'monit' section is added.
# @version 1.20 - The support of the 'sysctl' tag is added.
# @version 1.25 - The support of the 'order' attribute is added.
# @version 1.30 - The support of references within 'init' section is added.
# @version 1.31 - Monit attribute saving for init tags is added into savedb function.
# @version 1.32 - Two colons were removed before shared functions and variables;
#   The shared function update_sdb was exported.
# @version 1.40 - Move MONIT related functionality to base_monit module (Alex Arkhipov)
# @version 1.41 - One app restart behavior inconsistency was fixed.
#   The attribute 'app_restart_handler' support was added into the init section.

package appinstall::update_sys_data;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use baselog;

use install_common;
use base_monit;

use XML::Parser;
use Data::Dumper;
use XML::Writer;
use IO::File;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&update_sdb);

our $VERSION = "1.41";

my $SUDOERS      = "/etc/sudoers";  # SUDO config file
my $SUDOERS_MODE = 0440;            # File permission of /etc/sudoers file

my @configlist;
my %baseinfo;

## @skipline
sub config_lkup();
## @skipline
sub savedb($$);
## @skipline
sub update_sudo($);

# For testing purposes (will be skipped during installation)
$LASTERR = "NULL";

my %configdata = ();

#
#  %configdata format:
#  %configdata{section}->{application}->{field => value}

{

   package config_tags;

   use common_variables;
   use baselog;
   use base_monit;

   my $section                 = "";
   my $appname                 = "";
   my $reffile                 = "";
   my $monit_system_wide_tests = $FALSE;
   my $current_pid             = "";
   my $script_number           = 0;
   my $app_restart_handler     = "";

   sub application
   {
      $appname = {@_}->{'name'};
   }

   sub application_
   {
      $appname = "";
   }

   sub iptables
   {
      $section = "iptables";

      #initialize section
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }

      #remember reference file for this application
      my %tmp = (@_[2 .. $#_]);
      $reffile = $tmp{'reference'} if defined $tmp{'reference'};
   } ## end sub iptables

   sub iptables_
   {
      $section = "";
      $reffile = "";
   }

   sub sysctl
   {
      $section = "sysctl";

      #initialize section
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }

      #remember reference file for this application
      my %tmp = (@_[2 .. $#_]);
      $reffile = $tmp{'reference'} if defined $tmp{'reference'};
   } ## end sub sysctl

   sub sysctl_
   {
      $section = "";
      $reffile = "";
   }

   sub ipSec
   {
      $section = "ipSec";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }
   } ## end sub ipSec

   sub ipSec_
   {
      $section = "";
   }

   sub init
   {
      $section = "init";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }

      $script_number = 0;

      # Remember reference file for this application
      my %tmp = (@_[2 .. $#_]);
      if (defined($tmp{'reference'}))
      {
         $reffile = $tmp{'reference'};
         baselog($LOG_DEBUG, 'Init reference: ' . $appname);
      }

      #default behavior during restart action
      $app_restart_handler = $PER_SCRIPT_KEY;
      if (defined($tmp{'app_restart_handler'}))
      {
         if ($tmp{'app_restart_handler'} eq $PER_APPL_KEY or $tmp{'app_restart_handler'} eq $PER_SCRIPT_KEY)
         {
            $app_restart_handler = $tmp{'app_restart_handler'};
         }
         else
         {
            baselog($LOG_WARNING, 'Unknown application restart handler: ' . $tmp{'app_restart_handler'});
         }
      } ## end if (defined($tmp{'app_restart_handler'...
      baselog($LOG_DEBUG, 'Init app_restart_handler: ' . $app_restart_handler);
   } ## end sub init

   sub init_
   {
      $section             = "";
      $reffile             = "";
      $app_restart_handler = "";
   }

   #########################################################
   # To handle opening 'monit' tag.
   #   If there was no processing of 'monit' tags before - initialize
   #   section.
   #########################################################
   sub monit
   {
      $section = "monit";
      unless ($configdata{$section})
      {
         $configdata{$section} = {};
         $configdata{$section}->{$NAME_OF_MONIT_GLOBAL} = {};
      }
      $configdata{$section}->{$appname} = {@_[2 .. $#_]};
      $monit_system_wide_tests          = $TRUE;
      $current_pid                      = "";
      $script_number                    = 0;
   } ## end sub monit

   sub monit_
   {
      $section                 = "";
      $monit_system_wide_tests = $FALSE;
      $current_pid             = "";
   }

   sub sudo
   {
      $section = "sudo";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }
   } ## end sub sudo

   sub sudo_
   {
      $section = "";
   }

   sub config
   {
      $section = "config";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }
   } ## end sub config

   sub config_
   {
      $section = "";
   }

   sub postupdate
   {
      $section = "postupdate";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }
   } ## end sub postupdate

   sub postupdate_
   {
      $section = "";
   }

   sub restore
   {
      $section = "restore";
      unless ($configdata{$section})
      {
         $configdata{$section} = ();
      }
   } ## end sub restore

   sub restore_
   {
      $section = "";
   }

   sub deps
   {
   }

   sub deps_
   {
   }

   sub rule
   {
      my $skip_rule = $FALSE;

      my %tmphash = ('appl', $appname, @_[2 .. $#_]);

      if ($reffile)
      {

         #if there is an reference file and there is at least one '*_ref'
         #attribute, add reference file as attribute for the rule
         foreach my $key (keys %tmphash)
         {
            if ($key =~ /_ref$/)
            {
               $tmphash{'reference'} = $reffile;
               last;
            }
         } ## end foreach my $key (keys %tmphash...
      } ## end if ($reffile)

      #Check whether the rule is unique (only for sysctl data)
      if ($section eq "sysctl")
      {

         #Grep creates array from hashes if the parameter is the same as
         #the current one.
         my @tmparr = grep {($tmphash{'param'} eq $_->{'param'})} @{$configdata{$section}};

         #If the array contains element, the parameter does not unique.
         if (scalar(@tmparr) > 0)
         {
            baselog($LOG_EMERG, "Duplicated rule for parameter '$tmphash{'param'}' is found. The rule for application '$appname' is skipped.");
            $skip_rule = $TRUE;
         }
      } ## end if ($section eq "sysctl"...

      if (!$skip_rule)
      {
         push @{$configdata{$section}}, {%tmphash};
      }

   } ## end sub rule

   sub script
   {
      my %tmphash = ('appl', $appname, @_[2 .. $#_]);
      $script_number++;
      if ($reffile)
      {

         # If there is a reference file and there is at least one '*_ref'
         # attribute, add reference file as attribute for the rule
         foreach my $key (keys %tmphash)
         {
            if ($key =~ /_ref$/)
            {
               $tmphash{'reference'} = $reffile;
               baselog($LOG_DEBUG, 'Adding reffile: ' . $reffile);
               last;
            } ## end if ($key =~ /_ref$/)
         } ## end foreach my $key (keys %tmphash...
      } ## end if ($reffile)

      # The $app_restart_handler variable will be initialized only in <init> section.
      # All other cases will be skipped.
      if ($app_restart_handler ne "")
      {
         $tmphash{'app_restart_handler'} = $app_restart_handler;
      }

      # Set default values
      if (not exists($tmphash{'pmark'}))
      {
         $tmphash{'pmark'} = $script_number;
      }

      if (not exists($tmphash{'noparams'}))
      {
         $tmphash{'noparams'} = 0;
      }

      if (not exists($tmphash{'monit'}))
      {
         $tmphash{'monit'} = 0;
      }

      push @{$configdata{$section}}, {%tmphash};
   } ## end sub script

   sub after
   {
   }

   #########################################################
   # To handle opening 'pid' tag. All the following test-tags will be
   # attached to this certain process.
   #########################################################
   sub pid
   {
      my %tmp = (@_[2 .. $#_]);

      $monit_system_wide_tests = $FALSE;
      $script_number++;

      $current_pid = $tmp{'file'};
      if (not exists($tmp{'pmark'}))
      {
         $tmp{'pmark'} = $script_number;
      }
      $configdata{$section}->{$appname}->{'pids'}->{$current_pid} = {%tmp};
   } ## end sub pid

   #########################################################
   # To handle closing 'pid' tag. All the following 'system' and
   # 'device' tags will be attached to system wide section.
   #########################################################
   sub pid_
   {
      $monit_system_wide_tests = $TRUE;
   }

   #########################################################
   # To handle opening 'port' tag.
   #########################################################
   sub port
   {
      push @{$configdata{$section}->{$appname}->{'pids'}->{$current_pid}->{'tests'}}, {('tag', 'port', @_[2 .. $#_])};
   }

   #########################################################
   # To handle opening 'lock' tag.
   #########################################################
   sub lock
   {
      push @{$configdata{$section}->{$appname}->{'pids'}->{$current_pid}->{'tests'}}, {('tag', 'lock', @_[2 .. $#_])};
   }

   #########################################################
   # To handle opening 'system' tag. The system-wide tests are attached
   # to the separate section.
   #########################################################
   sub system
   {
      my @tmparr = ('tag', 'system', @_[2 .. $#_]);
      if ($monit_system_wide_tests)
      {
         push @{$configdata{$section}->{$NAME_OF_MONIT_GLOBAL}->{'tests'}}, {('owner', $appname, @tmparr)};
      }
      else
      {
         push @{$configdata{$section}->{$appname}->{'pids'}->{$current_pid}->{'tests'}}, {@tmparr};
      }
   } ## end sub system

   #########################################################
   # To handle opening 'device' tag. The system-wide tests are attached
   # to the separate section.
   #########################################################
   sub device
   {
      my @tmparr = ('tag', 'device', @_[2 .. $#_]);
      if ($monit_system_wide_tests)
      {
         push @{$configdata{$section}->{$NAME_OF_MONIT_GLOBAL}->{'tests'}}, {('owner', $appname, @tmparr)};
      }
      else
      {
         push @{$configdata{$section}->{$appname}->{'pids'}->{$current_pid}->{'tests'}}, {@tmparr};
      }
   } ## end sub device
}

## @fn update_sdb()
# @brief updates LinuxBase data files
# @param none
# @return reference to hash with data
sub update_sdb()
{
   %configdata = ();

   config_lkup();
   if ($LASTERR ne "NULL")
   {
      return;  #And pass the status to top function
   }
   print_xml_dump($INSTLOG, "Parsing configure.xml files:\n", Data::Dumper::Dumper(\%configdata));

   #
   #   DB files are available only for iptables, ipSec and init sections
   #
   foreach my $section (@DBSECTIONS)
   {
      savedb(\%configdata, $section);
      if ($LASTERR ne "NULL")
      {
         return;  #And pass the status to top function
      }
   } ## end foreach my $section (@DBSECTIONS...

   #
   #   Update sudo
   #
   update_sudo(\%configdata);

   #
   #   Update monit
   #   Warning: it changes %configdata
   #
   update_monit(\%configdata);

   return \%configdata;
} ## end sub update_sdb()

##################################################
# @fn update_sudo($configdata)
# @brief updates $SUDOERS file
# @param $configdata - reference to configuration data
# @return none
##################################################
sub update_sudo($)
{
   my $configdata = shift(@_);
   my $content    = '';

   unless (defined $configdata->{'sudo'})
   {
      baselog($LOG_ERROR, "Skip empty section: 'sudo'.");
      return;
   }

   # Prepare content
   $content .= "# Do not modify this file manually, it will be re-generated by Linux Base scripts!\n";
   $content .= "Defaults listpw=always\n";
   $content .= "Defaults rootpw\n";
   $content .= "Defaults timestamp_timeout=0\n";
   foreach my $r (@{$configdata->{'sudo'}})
   {
      my %rule = %{$r};
      my $entry;
      if ($rule{'who'})
      {
         $content .= "$rule{'who'}  ALL=($rule{'as'}) NOPASSWD: $rule{'file'}\n";
      }
      if ($rule{'group'})
      {
         $content .= "\%$rule{'group'}  ALL=($rule{'as'}) NOPASSWD: $rule{'file'}\n";
      }
   } ## end foreach my $r (@{$configdata...

   # Update sudoers file
   if (not write_file($SUDOERS, $content))
   {
      baselog($LOG_ERROR, "Cannot update $SUDOERS file!");
   }
   else
   {
      baselog($LOG_DEBUG, "The $SUDOERS file has been updated.");
   }
} ## end sub update_sudo($)

##################################################
# @fn config_lkup
# @brief reads configure.xml files and fills out $configdata hash
# @param none
# @return none
##################################################
sub config_lkup()
{
   my $retval = parse_configs('config_tags');
   if (!$retval)
   {
      $LASTERR = "NFLS";
   }
} ## end sub config_lkup()

##################################################
# @fn savedb($configdata, $section)
# @brief updates DB xml files
#   (stores writing order for the 'init' section)
# @param $configdata - reference to configuration data
# @param $section - xml section name
# @return none
##################################################
sub savedb($$)
{
   my $configdata = shift(@_);
   my $section    = shift(@_);
   unless (defined $configdata->{$section})
   {
      baselog($LOG_ERROR, "Skip empty section: '$section'.");
      return;
   }

   my @data    = @{$configdata->{$section}};
   my $dstfile = ">" . $SYSDB_ROOT . "/" . $section . ".xml\n";
   my $output  = new IO::File($dstfile);
   my $tag     = undef;
   my $xml     =
     new XML::Writer(
                     DATA_MODE   => 'true',
                     DATA_INDENT => 3,
                     OUTPUT      => $output
                    );
   if (!$xml)
   {
      $LASTERR = "XMLE";
      return;
   }
   $xml->startTag($section);  #OPEN section
                              #applying rules order for init section
   if ($section eq "init")
   {
      my $order     = 1;
      my %app_monit = ();

      # First pass. Define monitoring applications
      foreach $tag (@data)
      {
         if ($configdata->{'monit'}->{$tag->{'appl'}}->{'enable'} or $tag->{'monit'})
         {
            $app_monit{$tag->{'appl'}} = $TRUE;
         }
      } ## end foreach $tag (@data)

      # Second pass. Write init data to the file
      foreach $tag (@data)
      {
         $tag->{'order'} = $order;
         if (exists($app_monit{$tag->{'appl'}}))
         {
            $tag->{'monit'} = $TRUE;
         }
         $xml->emptyTag('rule', %{$tag});
         $order++;
      } ## end foreach $tag (@data)
   } ## end if ($section eq "init"...
   else
   {
      foreach $tag (@data)
      {
         $xml->emptyTag('rule', %{$tag});
      }
   } ## end else [ if ($section eq "init"...
   $xml->endTag($section);  #CLOSE section
   $xml->end();
} ## end sub savedb($$)

1;
__END__

=head1 NAME

update_sys_data - configure.xml parser.

=head1 DESCRIPTION

This module looks for all configure.xml in the applications
configuration root and updates the base installation DB 
with info from these files.

=head1 SYNOPSIS

use update_sys_data;

=head1 FUNCTIONS

=head2 update_sdb

This function is exported interface of this module. It returns 0 if system DB is updated successfully. Otherwise it returns 1 and $LASTERR will contain error code.

=head2 config_lkup

This function searches configure.xml files in the application configuration root directory.

=head2 parsefiles

It parses XML files and generates %baseinfo structure.

=head2 savedb

It saves the %configdata structure into system DB XML files.

=head1 PACKAGES

=head2 config_tags

This package contains a set of functions for each tag of configure.xml file.

=head1 LIMITATIONS

=head1 SEE ALSO

