#############################################################
## @file baseParams.pm
# @brief Module for managing Linux Base parameters
# @details Provides Read/Write interfaces to userinfo.txt
#          Allows to change system parameter and
#          syncronize them with userinfo.txt file
# @author Alex Arkhipov
# @date December 2006
# @version 1.00 - Initial version
# @version 1.01 - The following subroutines were
#                          added by Kate Pestova:
#   - validateClockSourceType()
#   - validateClockType()
#   - validateClockIPs()
#   - validateYesNo()
#   Also function setupNTP() was changed.
# @version 1.02 - applyTimeFromCS parameter is
#                         added by Maxim Uvarov
# @version 1.03 - Two colons were removed before shared functions and variables.
# @version 1.04 - Implemented automatic restart of NTP service
#                 removed applyTimeFromCS parameter
# @version 1.05 - Subroutine validatePriKey() was added by Kate Pestova.
# @version 1.06 - Subroutine validatePeer() was added by Kate Pestova.
# @version 1.07 - Subroutine validateElanTlanGw() was added.
# @version 1.08 - Support of "panic" option is added
# @version 1.09 - Subroutines validateNTPconfiguration() and validateKeyID()
#                 were added by Kate Pestova.
# @version 1.10 - Subroutine validateNTPconfiguration() was modified.
# @version 1.11 - The uniqueness check for external IP addresses was added
# @version 1.20 - Functions 'getTimezonesInfo' and 'getTimezoneDescr' has been added.
#                 'setupTimeZone', 'getParams' and 'convert' has been rewritten.
#                 'timezone.xml' is used instead of 'timezone'.
#                 'userinfo.txt' has been modified. 'userinfo.txt' version has been
#                 changed to '2.0'. Conversion function 'convert1to2' has been added.
# @version 1.21 - new function 'unconfigureDNS' has been added.
# @version 1.22 - '--peer' support has been removed (now default values are applied).
#                 New function 'getPeer' has been added. 'validatePeer'
#                 has been removed. Version of the 'userinfo.txt' has been changed to
#                 the version '3.0'
# @version 1.23 - new functions have been added: 'convertDNSData', 'convert3to4',
#                 'validateListOfParams'.
# @version 1.24 - setupDNS function was reworked. "127.0.0.1" ip will be add always
#                 to /etc/resolv.conf
# @version 1.25 - new variable '$MAX_EXTERNAL_IPS_COUNT' has been added.
# @version 1.26 - Added some NTP settings that are required to support new version of
#                 ntp RPM (ntp-4.2.4p8). These settings will be written to /etc/ntp.conf
# @note
#############################################################
package baseParams;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;

use CommonInstallUtils;
use baselog;
use base_system;
use base_xml;

# Access management
use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(&getParams &setParams &convert $TZFILE
  $USERINFO_VERSION $USERINFO_FILE $UNCONFIGURED $MAX_EXTERNAL_IPS_COUNT
  &setupDateTime &setupNetwork &setupDNS &setupTimeZone
  &setupNTP &validatePriKey &validateParam &validateTimeZone &validateAllParams
  &validateNTPconfiguration &getTimezonesInfo &getTimezoneDescr &unconfigureDNS
  &validateListOfParams
  );

our $VERSION = "1.26";

our $USERINFO_VERSION = "4.0";  # current version of userinfo file

our $UNCONFIGURED = 0;          # unconfigured ip address

my %fullData = (
                'userInfoVer'           => "0.0",         #initial version
                'hostname'              => "",
                'machineIP'             => "",
                'gateway'               => "",
                'netmask'               => "",
                'machineIPTlan'         => "",
                'gatewayTlan'           => "",
                'netmaskTlan'           => "",
                'defaultGateway'        => "",
                'fqdnTlan'              => "",
                'timezone'              => "",
                'rootpw'                => "",
                'nortelpw'              => "",
                'clockSourceType'       => "",
                'clockIPs'              => "",
                'clockType'             => "",
                'ntpSecurityMode'       => "",
                'ntpSecurityModeKeyID'  => "",
                'ntpSecurityModePriKey' => "",
                'primaryDNSIP'          => "",
                'secondaryDNSIP'        => "",
                'tertiaryDNSIP'         => "$LOCALHOST",  #default value
                'consolePort'           => "",
                'consoleSpeed'          => "",
                'enablePanic'           => ""
               );

# validate subroutines
my %validateData = (
                    'userInfoVer'           => \&validateVersion,
                    'hostname'              => \&validateHostname,
                    'machineIP'             => \&validateIPAddr,
                    'gateway'               => \&validateIPAddr,
                    'netmask'               => \&validateIPAddr,
                    'machineIPTlan'         => \&validateIPAddr,
                    'gatewayTlan'           => \&validateIPAddr,
                    'netmaskTlan'           => \&validateIPAddr,
                    'defaultGateway'        => \&validateElanTlanGw,
                    'fqdnTlan'              => \&validateFQDN,
                    'timezone'              => \&validateTimeZone,
                    'rootpw'                => \&validateNothing,
                    'nortelpw'              => \&validateNothing,
                    'clockSourceType'       => \&validateClockSourceType,
                    'clockIPs'              => \&validateClockIPs,
                    'clockType'             => \&validateClockType,
                    'ntpSecurityMode'       => \&validateYesNo,
                    'ntpSecurityModeKeyID'  => \&validateKeyID,
                    'ntpSecurityModePriKey' => \&validatePriKey,
                    'primaryDNSIP'          => \&validateIPAddrOpt,
                    'secondaryDNSIP'        => \&validateIPAddrOpt,
                    'tertiaryDNSIP'         => \&validateIPAddrOpt,
                    'consolePort'           => \&validateNothing,
                    'consoleSpeed'          => \&validateNothing,
                    'enablePanic'           => \&validateNothing
                   );

# userinfo.txt conversion functions
my %convertData = (
                   0 => \&convert0to1,
                   1 => \&convert1to2,
                   2 => \&convert2to3,
                   3 => \&convert3to4,
                  );

#Declarations
## @skipline
sub validateNTPconfiguration($);
## @skipline
sub validateVersion($);
## @skipline
sub validateIPAddrOpt($);
## @skipline
sub validateIPAddr($);
## @skipline
sub validateElanTlanGw($);
## @skipline
sub validateClockSourceType($);
## @skipline
sub validateClockType($);
## @skipline
sub validateClockIPs($);
## @skipline
sub validateYesNo($);
## @skipline
sub validateKeyID($);
## @skipline
sub validatePriKey($);
## @skipline
sub validatePeer($);
## @skipline
sub validateNothing(;$);
## @skipline
sub validateHostname($);
## @skipline
sub validateFQDN($);
## @skipline
sub validateTimeZone($);
## @skipline
sub convert0to1($);
## @skipline
sub convert1to2($);
## @skipline
sub convert2to3($);
## @skipline
sub readInfo($;$);
## @skipline
sub writeInfo($;$);
## @skipline
sub convertNetworkData($);
## @skipline
sub convertDNSData($);
## @skipline
sub convert($);
## @skipline
sub validateParam($$);
## @skipline
sub validateAllParams($);
## @skipline
sub getDefaultParams;
## @skipline
sub getParams($;$);
## @skipline
sub setParams($;$);
## @skipline
sub setupDateTime;
## @skipline
sub setupNetwork($);
## @skipline
sub updateDNSHost($);
## @skipline
sub setupDNS($;$);
## @skipline
sub setupTimeZone($);
## @skipline
sub setupNTP($);
## @skipline
sub getTimezonesInfo(;$);
## @skipline
sub getTimezoneDescr($);
## @skipline
sub unconfigureDNS($);
## @skipline
sub validateListOfParams($$);

our $USERINFO_FILE = "/admin/userinfo.txt";  # default userinfo.txt file

our $TZFILE = "/opt/nortel/base/data/timezone.xml";  # default time zone file

our $MAX_EXTERNAL_IPS_COUNT = 11;

my $NSSWITCH = "/usr/local/bin/configNSSwitch.pl";   # default nsswitch file

system_installation_init();

################################################################################
######################## Private Functions #####################################

################################################################################
## @fn validateVersion($ver)
# @brief Checks the version of the userinfo.txt.
# @param $ver - version
# @return $TRUE - version is correct
# @return $FALSE - version is not correct
# @author Kate Pestova
# @date December 2008
################################################################################
sub validateVersion($)
{
   my $ver = shift(@_);
   if ($ver eq $USERINFO_VERSION)
   {
      return $TRUE;
   }
   baselog($LOG_WARNING, "Incorrect version: $ver.");
   return $FALSE;
} ## end sub validateVersion($)

################################################################################
## @fn validateIPAddrOpt($ip)
# @brief Checks the IP address that can be unconfigured.
# @param $ip - IP address for check
# @return $TRUE - IP is valid
# @return $FALSE - IP is invalid
# @author Kate Pestova
# @date December 2008
################################################################################
sub validateIPAddrOpt($)
{
   my $ip = shift(@_);
   return $TRUE if ($ip eq $UNCONFIGURED);
   return validateIPAddr($ip);
}

################################################################################
## @fn validateIP($ip)
# @brief Checks the IP address.
# @param $ip - IP address for check
# @return $TRUE - IP is valid
# @return $FALSE - IP is invalid
# @author Kate Pestova
# @date December 2008
################################################################################
sub validateIPAddr($)
{
   my $ip = shift(@_);
   return validateIP($ip);
}

################################################################################
## @fn validateElanTlanGw($type)
# @brief Checks a value of ELAN_GW or TLAN_GW. Also checkes if the default GW
#        may contain an IP for backwards compatibility.
# @details Type can be:
#   \li 0 - ELAN_GW
#   \li 1 - TLAN_GW
# @param $type - value
# @return $TRUE, if input is 0/1 or an IPAddress
# @return $FALSE, if input is not 0/1 or an IPAddress
# @author Kefei Wei
# @date December 2008
# @note None
################################################################################

sub validateElanTlanGw($)
{
   my $type = shift(@_);
   if (($type =~ /^($ELAN_GW|$TLAN_GW)$/) or (validateIPAddr($type)))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validateElanTlanGw($)

################################################################################
## @fn validateClockSourceType($type)
# @brief Checks a value of clock source type
# @details TYpe can be:
#   \li 0 - Primary
#   \li 1 - Secondary
#   \li 2 - Not a clock server
# @param $type - clock source type
# @return $TRUE or $FALSE
# @author Kate Pestova
# @date December 2007
# @note None
###############################################################################

sub validateClockSourceType($)
{
   my $type = shift(@_);
   if ($type =~ /^($PRIMARY|$SECONDARY|$OTHER)$/)
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validateClockSourceType($)

################################################################################
## @fn validateClockType($type)
# @brief Checks a value of clock type
# @details Type can be:
#   \li 0 - EXTERNAL
#   \li 1 - INTERNAL
# @param $type - clock type
# @return $TRUE or $FALSE
# @author Kate Pestova
# @date December 2007
# @note None
################################################################################

sub validateClockType($)
{
   my $type = shift(@_);
   if ($type =~ /^($EXTERNAL_CLOCK|$INTERNAL_CLOCK)$/)
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validateClockType($)

################################################################################
## @fn validateClockIPs($str)
# @brief Checks a value of clock IPs
# @details Clock IPs are separated by '|' character
# @param $str - clock IPs
# @return $TRUE or $FALSE
# @author Kate Pestova
# @date December 2007
# @note None
################################################################################

sub validateClockIPs($)
{
   my $str   = shift(@_);
   my $valid = $TRUE;
   my @IPs   = split /\|/, $str;
   if (scalar(@IPs) > $MAX_EXTERNAL_IPS_COUNT)
   {
      return $FALSE;
   }
   foreach my $ip (@IPs)
   {
      if (!validateIP($ip))
      {
         $valid = $FALSE;
         last;
      }
   } ## end foreach my $ip (@IPs)
   return $valid;
} ## end sub validateClockIPs($)

################################################################################
## @fn validateYesNo($value)
# @brief Checks if parameter has value y/n.
# @details None
# @param $value - parameter value
# @return $TRUE or $FALSE
# @author Kate Pestova
# @date December 2007
# @note None
################################################################################

sub validateYesNo($)
{
   my $value = shift(@_);
   if ($value =~ /^(y|n)$/i)
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validateYesNo($)

################################################################################
## @fn validateKeyID($value)
# @brief Checks a value of public key.
# @param $value - public key
# @return $TRUE - public key is valid
# @return $FALSE - public key is invalid
# @author Kate Pestova
# @date December 2008
################################################################################

sub validateKeyID($)
{
   my $value = shift(@_);
   if ($value eq "" or ($value >= 1 and $value <= 65535))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validateKeyID($)

################################################################################
## @fn validatePriKey($value)
# @brief Checks a value of private key.
# @param $value - private key
# @return $TRUE - private key is valid
# @return $FALSE - private key is invalid
# @author Kate Pestova
# @date November 2008
################################################################################

sub validatePriKey($)
{
   my $value = shift(@_);
   if (length($value) <= 16 and $value !~ /[\s#']/)
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub validatePriKey($)

################################################################################
## @fn validateNTPconfiguration($data)
# @brief Checks correctness of NTP configuration
# @param $data - reference to hash with data
# @return $TRUE - configuration is valid
# @return $FALSE - configuration is invalid
# @author Kate Pestova
# @date December 2008
################################################################################

sub validateNTPconfiguration($)
{
   my $data  = shift(@_);
   my $print = $FALSE;

   if (!is_install_stage())
   {
      $print = $TRUE;
   }

   if (!defined($data))
   {
      baselog($LOG_ERROR, "Incorrect data.");
      return $FALSE;
   }

   if (!validateClockSourceType($data->{'clockSourceType'}))
   {
      baselog($LOG_ERROR, "Clock Source Type is not defined.", $print);
      return $FALSE;
   }

   if (!validateYesNo($data->{'ntpSecurityMode'}))
   {
      baselog($LOG_ERROR, "Security mode is not defined.", $print);
      return $FALSE;
   }

   if ($data->{'ntpSecurityMode'} eq "y" or $data->{'ntpSecurityMode'} eq "Y")
   {
      if ($data->{'ntpSecurityModeKeyID'} eq "" or $data->{'ntpSecurityModePriKey'} eq "")
      {
         baselog($LOG_ERROR, "Key ID and private key are mandatory for secure mode.", $print);
         return $FALSE;
      }
      if (!validateKeyID($data->{'ntpSecurityModeKeyID'}))
      {
         baselog($LOG_ERROR, "Key ID is incorrect.", $print);
         return $FALSE;
      }
      if (!validatePriKey($data->{'ntpSecurityModePriKey'}))
      {
         baselog($LOG_ERROR, "Private key is incorrect.", $print);
         return $FALSE;
      }
   } ## end if ($data->{'ntpSecurityMode'...
   else
   {
      if ($data->{'ntpSecurityModeKeyID'} ne "" or $data->{'ntpSecurityModePriKey'} ne "")
      {
         baselog($LOG_ERROR, "Key ID and private key are not required for insecure mode.", $print);
         return $FALSE;
      }
   } ## end else [ if ($data->{'ntpSecurityMode'...

   if ($data->{'clockSourceType'} ne $PRIMARY)
   {
      if (is_install_stage())
      {
         if ($data->{'clockType'} ne $EXTERNAL_CLOCK)
         {
            baselog($LOG_ERROR, "Secondary and Other servers must be configured with external clock.", $TRUE);
            return $FALSE;
         }
      } ## end if (is_install_stage()...
      else
      {
         if ($data->{'clockType'} ne "")
         {
            baselog($LOG_ERROR, "Clock type parameter is not supported for Secondary and Other servers.", $TRUE);
            return $FALSE;
         }
         $data->{'clockType'} = $EXTERNAL_CLOCK;
      } ## end else [ if (is_install_stage()...
   } ## end if ($data->{'clockSourceType'...
   else
   {
      if (!validateClockType($data->{'clockType'}))
      {
         baselog($LOG_ERROR, "Clock Type is not defined.\n", $print);
         return $FALSE;
      }

      my %hash = ('machineIP' => '');
      getParams(\%hash);

      if (is_install_stage())
      {
         if ($data->{'clockType'} eq $INTERNAL_CLOCK and $data->{'clockIPs'} ne $LOCAL_IP)
         {
            baselog($LOG_ERROR, "Clock IPs are incorrect.\n");
            return $FALSE;
         }
      } ## end if (is_install_stage()...
      else
      {
         if ($data->{'clockType'} eq $INTERNAL_CLOCK)
         {
            if ($data->{'clockIPs'} ne "")
            {
               print "Parameter clIPs is not supported for the server with Internal clock.\n";
               return $FALSE;
            }
            $data->{'clockIPs'} = $LOCAL_IP;
         } ## end if ($data->{'clockType'...
      } ## end else [ if (is_install_stage()...
   } ## end else [ if ($data->{'clockSourceType'...

   if ($data->{'clockType'} eq $EXTERNAL_CLOCK)
   {
      if ($data->{'clockIPs'} eq "")
      {
         baselog($LOG_ERROR, "External clock IP addresses are not defined.", $print);
         return $FALSE;
      }
      elsif (not validateClockIPs($data->{'clockIPs'}))
      {
         baselog($LOG_ERROR, "External clock IP addresses are not valid.", $print);
         return $FALSE;
      }
      elsif (not is_array_unique([split(/\|/, $data->{'clockIPs'})]))
      {
         baselog($LOG_ERROR, "External clock IP addresses are to be unique.", $print);
         return $FALSE;
      }

      my @IPs;
      @IPs = split /\|/, $data->{'clockIPs'};

      if ($data->{'clockSourceType'} eq $OTHER)
      {
         if (scalar(@IPs) > 2)
         {
            $data->{'clockIPs'} = $IPs[0] . "|" . $IPs[1];
         }

         if (defined $IPs[1] and $IPs[0] eq $IPs[1])
         {
            $data->{'clockIPs'} = $IPs[0];
         }
      } ## end if ($data->{'clockSourceType'...
   } ## end if ($data->{'clockType'...
   return $TRUE;
} ## end sub validateNTPconfiguration($)

################################################################################
## @fn validateNothing($value)
# @brief Validates nothing.
# @param Any
# @return $TRUE - always
################################################################################
sub validateNothing(;$)
{
   return $TRUE;
}

################################################################################
## @fn validateHostname($hostname)
# @brief Makes sure the user entered hostname follows the defined hostname rules
#        taken from network.py in the '/usr/lib/anaconda' directory
# @param $hostname - hostname to check
# @return $TRUE - hostname is correct
# @return $FALSE - hostname is not correct
################################################################################
sub validateHostname($)
{
   my $hostname = shift(@_);

   # Hostname must start with a valid character in the range 'a-z' or 'A-Z'
   if ($hostname !~ /^[a-zA-Z]/)
   {
      return $FALSE;

   }

   # Hostnames can only contain the characters 'a-z', 'A-Z', '0-9', '-'
   if ($hostname =~ /[^a-zA-Z-0-9]/)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub validateHostname($)

################################################################################
## @fn validateFQDN($fqdn)
# @brief Makes sure the user entered FQDN follows the defined hostname rules
#        taken from network.py in the '/usr/lib/anaconda' directory.
#        It must have at least two dots inside.
# @param $fqdn - FQDN to check
# @return $TRUE - FQDN is correct
# @return $FALSE - FQDN is not correct
################################################################################
sub validateFQDN($)
{
   my $fqdn = shift(@_);

   # Hostname must start with a valid character in the range 'a-z' or 'A-Z'
   if ($fqdn !~ /^[a-zA-Z]/)
   {
      return $FALSE;

   }

   # fqdn can only contain the characters 'a-z', 'A-Z', '0-9', '-', or '.'
   if ($fqdn =~ /[^a-zA-Z-\.0-9]/)
   {
      return $FALSE;
   }

   my @parts = split(/\./, $fqdn);

   # There sould be at least two dots in fqdn (Quantum requirement)
   if (scalar(@parts) < 3)
   {
      if ((scalar(@parts) == 2) and ($parts[1] eq 'localdomain'))
      {  # 'localdomain' is exception
         return $TRUE;
      }
      return $FALSE;
   } ## end if (scalar(@parts) < 3...

   if ($fqdn =~ /\.\./)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub validateFQDN($)

################################################################################
## @method validateTimeZone($timezone)
# @brief Ensure that the timezone is valid
# @param $timezone - string defining the timezone
# @return $TRUE - Valid.
# @return $FALSE - Not Valid.
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub validateTimeZone($)
{
   my $timezone = shift(@_);
   if (!defined($timezone))
   {
      baselog($LOG_ERROR, "Timezone is not specified.");
      return $FALSE;
   }
   my $zones_info = getTimezonesInfo({'zone' => $timezone});
   if (!defined($zones_info) || !scalar(@$zones_info))
   {
      baselog($LOG_INFO, "Timezone is not valid.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub validateTimeZone($)

#################################################################################
## @method convert0to1($data)
# @brief convert function from unversioned userinfo.txt file to file with version 1.0
# @details Just returns $FALSE because conversion can not be done
# @param $data - data hash reference to convert
# @return $FALSE - Conversion is impossible.
#################################################################################
sub convert0to1($)
{
   baselog($LOG_INFO, "Conversion impossible");
   return $FALSE;
}

################################################################################
## @method convert1to2($data)
# @brief convert function from userinfo.txt version 1.0 to file with version 2.0
# @details Timezone conversion is performed, Gateway changes, DNS settings.
# @param $data - data hash reference to convert
# @return $TRUE - Conversion is successful
# @return $FALSE - Conversion is impossible
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub convert1to2($)
{
   my $data = shift(@_);
   baselog($LOG_INFO, "Convert userinfo.txt from version 1.0 to 2.0");
   convertNetworkData($data);
   convertDNSData($data);
   my $tzsinfo = getTimezonesInfo({'LinuxZone' => $data->{'timezone'}});
   if (!defined($tzsinfo) || !scalar(@$tzsinfo))
   {
      baselog($LOG_WARNING, "The timezone format has been changed.\nCannot find an appropriate conversion for '$data->{'timezone'}'.\nPlease select another timezone when prompted.", $TRUE);
      $data->{'timezone'} = '';
      return $FALSE;
   } ## end if (!defined($tzsinfo)...
   $data->{'timezone'}    = $tzsinfo->[0]->{'zone'};
   $data->{'userInfoVer'} = '2.0';
   baselog($LOG_INFO, "Conversion of the userinfo.txt from version 1.0 to 2.0 finished successfully.");
   return $TRUE;
} ## end sub convert1to2($)

################################################################################
## @method convert2to3($data)
# @brief convert function from userinfo.txt version 2.0 to file with version 3.0
# @details 'otherClockSource' is removed.
# @param $data - data hash reference to convert
# @return $TRUE - always
sub convert2to3($)
{
   my $data = shift(@_);
   if (exists $data->{'otherClockSource'})
   {
      delete $data->{'otherClockSource'};
   }
   convertNetworkData($data);
   convertDNSData($data);
   $data->{'userInfoVer'} = '3.0';
   baselog($LOG_INFO, "Conversion of the userinfo.txt from version 2.0 to 3.0 finished successfully.");
   return $TRUE;
} ## end sub convert2to3($)

################################################################################
## @method convert3to4($data)
# @brief convert function from userinfo.txt version 3.0 to file with version 4.0
# @details Gateway changes, DNS settings;
# @param $data - data hash reference to convert
# @return $TRUE - Conversion is successful
# @return $FALSE - Conversion is impossible
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub convert3to4($)
{
   my $data = shift(@_);
   baselog($LOG_INFO, "Convert userinfo.txt from version 3.0 to 4.0");
   convertNetworkData($data);
   convertDNSData($data);
   $data->{'userInfoVer'} = '4.0';
   baselog($LOG_INFO, "Conversion of the userinfo.txt from version 3.0 to 4.0 finished successfully.");
   return $TRUE;
} ## end sub convert3to4($)

################################################################################
## @method readInfo($data, $file)
# @brief Reads parameters defined in $data from the $file file.
# @param $data - hash reference to fill
# @param $file - file to read from ($USERINFO_FILE by default)
# @return $TRUE - file has been read
# @return $FALSE - can not read file
################################################################################
sub readInfo($;$)
{
   my ($data, $file) = (@_);
   my $fh;
   if (!defined($file))
   {
      $file = $USERINFO_FILE;
   }

   unless (open($fh, "<$file"))
   {
      baselog($LOG_ERROR, "Can not open $file file");
      return $FALSE;
   }
   while (<$fh>)
   {
      chomp();
      baselog($LOG_DEBUG, "Read: $_");
      next if (/^\s*\#/);  # skip comments

      my ($key, $value) = split(/=/);

      #Skip unknown keys
      if (!exists($fullData{$key}))
      {
         baselog($LOG_WARNING, "Skip unknown key: $key\n");
         next;
      }

      # save value only if it is defined in %$data
      $data->{$key} = $value if (defined($data->{$key}));
   } ## end while (<$fh>)
   close($fh);
   return $TRUE;
} ## end sub readInfo($;$)

################################################################################
## @method writeInfo($data, $file)
# @brief Writes parameters from the $data hash to the $file file.
# @param $data - hash reference with data
# @param $file - file to write to ($USERINFO_FILE by default)
# @return $TRUE - file has been written
# @return $FALSE - can not write to file
################################################################################
sub writeInfo($;$)
{
   my ($data, $file) = (@_);
   my $fh;
   if (!defined($file))
   {
      $file = $USERINFO_FILE;
   }

   unless (open($fh, ">$file.new"))
   {
      baselog($LOG_ERROR, "Can not open $file.new file");
      return $FALSE;
   }
   foreach my $key (sort keys %$data)
   {

      # Skip unknown keys
      if (!exists($fullData{$key}))
      {
         baselog($LOG_WARNING, "Skip unknown key: $key");
         next;
      }
      baselog($LOG_DEBUG, "Write data: $key=$data->{$key}");
      print $fh "$key=$data->{$key}\n";
   } ## end foreach my $key (sort keys ...
   close($fh);
   unless (rename("$file.new", $file))
   {
      baselog($LOG_ERROR, "Rename failed: $file.new -> $file");
      unlink("$file.new");
      return $FALSE;
   } ## end unless (rename("$file.new"...
   baselog($LOG_INFO, "File $file is updated");
   return $TRUE;
} ## end sub writeInfo($;$)

################################################################################
## @method convertNetworkData($data)
# @brief Update the older network data in userinfo.txt with the equivalent new
#        values
# @details None.
# @param $data - Hash reference with data
# @return $TRUE
################################################################################
sub convertNetworkData($)
{
   my $data = shift(@_);
   my $dg   = $TLAN_GW;

   if (($data->{'defaultGateway'} eq $data->{'gateway'}) || ($data->{'defaultGateway'} eq $ELAN_GW))
   {
      baselog($LOG_INFO, "Setting defaultGateway to ELAN_GW[$ELAN_GW]");
      $dg = $ELAN_GW;
   }
   elsif (($data->{'defaultGateway'} eq $data->{'gatewayTlan'}) || ($data->{'defaultGateway'} eq $TLAN_GW))
   {
      baselog($LOG_INFO, "Setting defaultGateway to TLAN_GW[$TLAN_GW]");
   }
   else
   {
      baselog($LOG_WARNING, "Unrecognized defaultGateway value, setting to default: TLAN_GW[$TLAN_GW]");
   }

   $data->{'defaultGateway'} = $dg;
   return $TRUE;
} ## end sub convertNetworkData($)

################################################################################
## @fn convertDNSData($data)
# @brief Update the DNS settings. If the Primary is not configured and the
#        Secondary is configured then swap them.
# @details None.
# @param  $data - Hash reference with data
# @return $TRUE - always
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub convertDNSData($)
{
   my $data = shift(@_);
   if ($data->{'primaryDNSIP'} eq $UNCONFIGURED and $data->{'secondaryDNSIP'} ne $UNCONFIGURED)
   {
      baselog($LOG_INFO, "Swap the Primary '$data->{'primaryDNSIP'}' and the Secondary '$data->{'secondaryDNSIP'}' servers.");
      $data->{'primaryDNSIP'}   = $data->{'secondaryDNSIP'};
      $data->{'secondaryDNSIP'} = $UNCONFIGURED;
   } ## end if ($data->{'primaryDNSIP'...
   return $TRUE;
} ## end sub convertDNSData($)

################################################################################
################################ Public Functions ##############################

################################################################################
## @method convert($data)
# @brief try to convert parameters from one version to another.
# @param $data - data hash reference to convert
# @return $TRUE - Conversion is successful
# @return $FALSE - Conversion failed
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub convert($)
{
   my $data = shift(@_);
   my ($from) = ($data->{'userInfoVer'} =~ /(\d*)\.\d/);
   my ($to)   = ($USERINFO_VERSION      =~ /(\d*)\.\d/);
   if ($from == $to)
   {
      return $TRUE;
   }
   my $retval = $TRUE;

   for (my $version = $from ; $version < $to ; $version++)
   {
      if (!defined($convertData{$version}))
      {
         baselog($LOG_ERROR, "Can not convert from version '$version.0'.");
         $retval = $FALSE;
      }
      else
      {
         if (!$convertData{$version}->($data))
         {
            $retval = $FALSE;
         }
      } ## end else [ if (!defined($convertData...
   } ## end for (my $version = $from...
   if ($retval)
   {
      $data->{'userInfoVer'} = $USERINFO_VERSION;
      baselog($LOG_INFO, "userinfo.txt has been converted successfully.");
   }
   else
   {
      baselog($LOG_ERROR, "Can not convert userinfo.txt file to a new version.", $TRUE);
   }
   return $retval;
} ## end sub convert($)

################################################################################
## @method validateParam($param, $value)
# @brief validates parameter based on %validateData.
# @param $param - name of the parameter
# @return $TRUE - parameter is valid.
# @return $FALSE - parameter is invalid.
################################################################################
sub validateParam($$)
{
   my ($param, $value) = @_;
   if (!defined($validateData{$param}))
   {
      baselog($LOG_WARNING, "Unknown parameter: $param");
      return $FALSE;
   }
   return $validateData{$param}->($value);
} ## end sub validateParam($$)

################################################################################
## @fn    validateAllParams($data)
# @brief  validate all params in hash
# @param  $data - Hash reference
# @return $TRUE - all parameters are valid
# @return $FALSE - at least one parameter is invalid
################################################################################
sub validateAllParams($)
{
   my $data = shift(@_);
   foreach my $key (keys %$data)
   {
      baselog($LOG_DEBUG, "Validating: $key=$data->{$key}\n");
      if (!validateParam($key, $data->{$key}))
      {
         baselog($LOG_ERROR, "Validating failed: $key=$data->{$key}\n");
         return $FALSE;
      }
   } ## end foreach my $key (keys %$data...
   baselog($LOG_INFO, "Validation successful");
   return $TRUE;
} ## end sub validateAllParams($)

################################################################################
## @fn    getDefaultParams()
# @brief  Returns reference to the hash with default data
# @param  None
# @return $copy - reference to the hash with default data.
################################################################################
sub getDefaultParams
{
   my %copy = %fullData;
   return \%copy;
}

################################################################################
## @fn getParams($;$)
# @brief Returns data from userinfo.txt
# @param $data - hash reference to fill
# @param $file - file to read from ($USERINFO_FILE by default)
# @return $TRUE  - information successfully retrieved
# @return $FALSE - information retrieved but file can not be converted, or some
#         values are not correct.
# @return undef  - can not read $USERINFO_FILE file
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub getParams($;$)
{
   my ($data, $file) = (@_);
   my %d = %fullData;  # create a hash copy
   if (!defined($file))
   {
      $file = $USERINFO_FILE;
   }
   unless (readInfo(\%d, $file))
   {
      return undef;
   }

   # copy data to the destination hash
   for my $key (keys %d)
   {
      $data->{$key} = $d{$key};
   }
   if (!validateVersion($data->{'userInfoVer'}))
   {
      if (!convert($data))
      {
         return $FALSE;
      }
   } ## end if (!validateVersion($data...
   if (!validateAllParams($data))
   {
      return $FALSE;
   }
   return $TRUE;
} ## end sub getParams($;$)

################################################################################
## @fn setParams($;$)
# @brief set parameters and updates file $USERINFO_FILE
# @param $data - hash reference to update from
# @param $file - file to write to ($USERINFO_FILE by default)
# @return $TRUE  - $USERINFO_FILE successfully updated
# @return $FALSE - $USERINFO_FILE can not be updated
################################################################################
sub setParams($;$)
{
   my ($data, $file) = (@_);
   if (!defined($file))
   {
      $file = $USERINFO_FILE;
   }
   my $d_full = getDefaultParams();
   if (!readInfo($d_full, $file))
   {
      baselog($LOG_WARNING, "Can not extract data from: $file");

      # if there is no file then it is installation stage
      # So just get default parameters
      $d_full = getDefaultParams();
   } ## end if (!readInfo($d_full,...
   foreach my $key (keys %$data)
   {
      if (defined($d_full->{$key}))
      {
         baselog($LOG_DEBUG, "Updating: $key => $data->{$key}");
         $d_full->{$key} = $data->{$key};
      }
      else
      {
         baselog($LOG_WARNING, "Skip unknown key: $key\n");
      }
   } ## end foreach my $key (keys %$data...
   unless (writeInfo($d_full, $file)) {return $FALSE;}
   baselog($LOG_INFO, "Parameters updated successfully\n");
   return $TRUE;
} ## end sub setParams($;$)

################################################################################
## @fn setupDateTime($date)
# @brief set system date and time
# @param $date - system date and time in the format of standard Linux "data"
#        utility
# @return $TRUE  - time has changed successfully
# @return $FALSE - error during time change
################################################################################
sub setupDateTime($)
{
   my $date = shift(@_);
   if (!defined($date))
   {
      baselog($LOG_WARNING, "Missing mandatory argument");
      return $FALSE;
   }
   my $retVal = $FALSE;
   my (@output, @err) = ((), ());
   if (do_system([$DATE, $date], {$dsOUT => \@output, $dsERR => \@err}))
   {

      # Synchronize hardware clock with the system one
      do_system([$HWCLOCK, '--systohc']);
      baselog($LOG_INFO, "Set new date: @output");
      $retVal = $TRUE;
   } ## end if (do_system([$DATE, ...
   else
   {
      baselog($LOG_ERROR, "Can not set date: " . join(', ', @err));
   }
   return $retVal;
} ## end sub setupDateTime($)

################################################################################
## @fn setupNetwork($)
# @brief Setup network settings
# @details Modify network files:
#   \li /etc/sysconfig/network
#   \li /etc/sysconfig/network-scripts
#   \li /etc/hosts
# @param $data - hash reference with new data
# @return $TRUE  - network setup correctly
# @return $FALSE - error during setu
################################################################################
sub setupNetwork($)
{
   my $data = shift(@_);
   if (   !defined($data->{'machineIP'})
       or !defined($data->{'gateway'})
       or !defined($data->{'netmask'})
       or !defined($data->{'hostname'})
       or !defined($data->{'fqdnTlan'})
       or !defined($data->{'machineIPTlan'})
       or !defined($data->{'gatewayTlan'})
       or !defined($data->{'netmaskTlan'})
       or !defined($data->{'defaultGateway'}))
   {
      baselog($LOG_ERROR, "Incorrect data parameter. Can not setup Network");
      return $FALSE;
   } ## end if (!defined($data->{'machineIP'...
   unless (validateAllParams($data))
   {
      baselog($LOG_ERROR, "Validation FAILED");
      return $FALSE;
   }
   my ($bcast, $attachedSubnet, $file1, $file2, $file3, $file4, $fh);
   $file1 = "/etc/sysconfig/network";
   $file2 = "/etc/sysconfig/network-scripts/ifcfg-eth0";
   $file3 = "/etc/sysconfig/network-scripts/ifcfg-eth1";
   $file4 = "/etc/hosts";

   if (open($fh, ">$file1.new"))
   {
      print $fh "NETWORKING=yes\n";
      print $fh "HOSTNAME=$data->{'fqdnTlan'}\n";

      # The default gateway
      if ($data->{'defaultGateway'} eq $ELAN_GW)
      {
         print $fh "GATEWAY=$data->{'gateway'}\n";
         print $fh "GATEWAYDEV=eth0\n";
      }
      elsif ($data->{'defaultGateway'} eq $TLAN_GW)
      {
         print $fh "GATEWAY=$data->{'gatewayTlan'}\n";
         print $fh "GATEWAYDEV=eth1\n";
      }
      else
      {
         baselog($LOG_ERROR, "Invalid defaultGateway data field --- $data->{'defaultGateway'}");
      }  ## enf if ($data->{'gatewayTlan'}...
      close($fh);
   } ## end if (open($fh, ">$file1.new"...
   else
   {
      baselog($LOG_ERROR, "Could not open $file1.new: $!");
      return $FALSE;
   }
   baselog($LOG_INFO, "     /etc/sysconfig/network configured");

   #
   # Setup the eth0 network interface
   #
   if (open($fh, ">$file2.new"))
   {
      if (-e "/admin/.vmware_machine")
      {  # have to use DHCP here or IS yells at us
         print $fh "DEVICE=eth0\n";
         print $fh "ONBOOT=yes\n";
         print $fh "BOOTPROTO=dhcp\n";
         print $fh "check_link_down () {\n\treturn 1;\n}\n";
      } ## end if (-e "/admin/.vmware_machine"...
      else
      {
         print $fh "DEVICE=eth0\n";
         print $fh "BOOTPROTO=static\n";
         $bcast = calcBcastAddr($data->{'machineIP'}, $data->{'netmask'});
         print $fh "BROADCAST=$bcast\n";
         print $fh "IPADDR=$data->{'machineIP'}\n";
         print $fh "NETMASK=$data->{'netmask'}\n";
         $attachedSubnet = andIpAddrs($data->{'machineIP'}, $data->{'netmask'});
         print $fh "NETWORK=$attachedSubnet\n";
         print $fh "ONBOOT=yes\n";
      } ## end else [ if (-e "/admin/.vmware_machine"...
      close($fh);
   } ## end if (open($fh, ">$file2.new"...
   else
   {
      baselog($LOG_ERROR, "Could not open $file2.new: $!");
      unlink("$file1.new");
      return $FALSE;
   } ## end else [ if (open($fh, ">$file2.new"...
   baselog($LOG_INFO, "     Network eth0 configured");

   #
   # Setup the eth1 network interface
   #
   if (open($fh, ">$file3.new"))
   {
      if (-e "/admin/.vmware_machine")
      {  # have to use DHCP here or IS yells at us
         print $fh "DEVICE=eth1\n";
         print $fh "ONBOOT=yes\n";
         print $fh "BOOTPROTO=dhcp\n";
         print $fh "check_link_down () {\n\treturn 1;\n}\n";
      } ## end if (-e "/admin/.vmware_machine"...
      else
      {
         print $fh "DEVICE=eth1\n";
         print $fh "BOOTPROTO=static\n";
         $bcast = calcBcastAddr($data->{'machineIPTlan'}, $data->{'netmaskTlan'});
         print $fh "BROADCAST=$bcast\n";
         print $fh "IPADDR=$data->{'machineIPTlan'}\n";
         print $fh "NETMASK=$data->{'netmaskTlan'}\n";
         $attachedSubnet = andIpAddrs($data->{'machineIPTlan'}, $data->{'netmaskTlan'});
         print $fh "NETWORK=$attachedSubnet\n";
         print $fh "ONBOOT=yes\n";
      } ## end else [ if (-e "/admin/.vmware_machine"...
      close($fh);
   } ## end if (open($fh, ">$file3.new"...
   else
   {
      baselog($LOG_ERROR, "Could not open $file3.new: $!");
      unlink("$file1.new");
      unlink("$file2.new");
      return $FALSE;
   } ## end else [ if (open($fh, ">$file3.new"...
   baselog($LOG_INFO, "     Network eth1 configured");

   if (!-e "/admin/.vmware_machine")
   {
      if (open($fh, "<$file4"))
      {
         my @new_host_str = "";

         push @new_host_str, "##################################################\n";
         push @new_host_str, "## DO NOT MANUALLY MODIFY THIS FILE\n";
         push @new_host_str, "## Auto generated by LinuxBase facility\n";
         push @new_host_str, "##################################################\n";
         push @new_host_str, "$LOCALHOST\t\tlocalhost.localdomain localhost\n";
         push @new_host_str, "$data->{'machineIPTlan'}\t\t$data->{'fqdnTlan'} $data->{'hostname'}\n";
         push @new_host_str, "################## DNS Servers ###################\n";

         my @host_str = <$fh>;
         close($fh);

         my $i = 0;
         foreach my $str (@host_str)
         {
            $i++;
            next if ($i < 8);  ## Skip the first 7 lines
            push @new_host_str, $str;
         } ## end foreach my $str (@host_str)

         if (open($fh, ">$file4.new"))
         {
            print $fh @new_host_str;
            close($fh);
         }
         else
         {
            baselog($LOG_ERROR, "Could not open $file4.new: $!");
            unless (unlink("$file1.new")) {baselog($LOG_ERROR, "Failed to unlink $file1.new")}
            unless (unlink("$file2.new")) {baselog($LOG_ERROR, "Failed to unlink $file2.new")}
            unless (unlink("$file3.new")) {baselog($LOG_ERROR, "Failed to unlink $file3.new")}
            return $FALSE;
         } ## end else [ if (open($fh, ">$file4.new"...
      } ## end if (open($fh, "<$file4"...
      else
      {
         baselog($LOG_ERROR, "Could not open $file4: $!");
         unless (unlink("$file1.new")) {baselog($LOG_ERROR, "Failed to unlink $file1.new")}
         unless (unlink("$file2.new")) {baselog($LOG_ERROR, "Failed to unlink $file2.new")}
         unless (unlink("$file3.new")) {baselog($LOG_ERROR, "Failed to unlink $file3.new")}
         return $FALSE;
      } ## end else [ if (open($fh, "<$file4"...
   } ## end if (!-e "/admin/.vmware_machine"...

   if (setParams($data))
   {
      system($CHMOD, "--reference=$file1", "$file1.new");
      rename("$file1.new", $file1);
      system($CHMOD, "--reference=$file2", "$file2.new");
      rename("$file2.new", $file2);
      system($CHMOD, "--reference=$file3", "$file3.new");
      rename("$file3.new", $file3);
      if (-f "$file4.new")
      {
         system($CHMOD, "--reference=$file4", "$file4.new");
         rename("$file4.new", $file4);
         my %dnsip_data = (
                           'primaryDNSIP'   => '',
                           'secondaryDNSIP' => '',
                           'tertiaryDNSIP'  => ''
                          );

         unless (getParams(\%dnsip_data))
         {
            baselog($LOG_ERROR, "Can not get current data\n", $TRUE);
            return $FALSE;
         }
         unless (updateDNSHost(\%dnsip_data))
         {
            baselog($LOG_ERROR, "Can not update DNS IP addresses\n", $TRUE);
            return $FALSE;
         }

      } ## end if (-f "$file4.new")
      baselog($LOG_INFO, "Network configured successfully");
      return $TRUE;
   } ## end if (setParams($data))

   unlink("$file1.new");
   unlink("$file2.new");
   unlink("$file3.new");
   unlink("$file4.new");
   baselog($LOG_ERROR, "Can not save parameters");
   return $FALSE;
} ## end sub setupNetwork($)

################################################################################
## @fn updateDNSHost($data)
# @brief Setup DNS ip information
# @details Updates DNS ip information in the '/etc/hosts' file
# @param $data - hash reference with new data
# @return $TRUE  - DNS setup correctly
# @return $FALSE - error during DNS setup
################################################################################
sub updateDNSHost($)
{
   my $data = shift(@_);

   #by default DNS IP's(/etc/hosts file) have "127.0.0.1" value regarding to
   #not allow connections from other hosts, if DNS is not configured
   my %host_data = (
                    'primaryDNSIP'   => '127.0.0.1',
                    'secondaryDNSIP' => '127.0.0.1',
                    'tertiaryDNSIP'  => '127.0.0.1'
                   );
   foreach (keys %host_data)
   {
      if ($data->{$_} ne $UNCONFIGURED && $data->{$_} ne "")
      {
         $host_data{$_} = $data->{$_};
      }
   } ## end foreach (keys %host_data)

   #update /etc/hosts file
   my $etc_host_file = "/etc/hosts";
   my %host_check;  #to check what DNS IP(primary, secon,..) is changed
   my $is_update = $FALSE;  #to check /etc/hosts file is updated or not

   open IN, "<$etc_host_file"
     or baselog($LOG_ERROR, "Could not open $etc_host_file: $!"), return $FALSE;

   my @host_str = <IN>;
   close IN;

   my $comment_flag = $FALSE;
   foreach my $str (@host_str)
   {
      if ($str =~ "User Configured Hosts")
      {
         $comment_flag = $TRUE;
         next;
      }
      next if ($str =~ /^\s*\#/);
      if ($str =~ /(([0-9]{1,3}\.){3}[0-9]{1,3})\s+(\w+)/)
      {
         if (exists $host_data{$3})
         {
            my $old_dns_ip = $1;
            my $new_dns_ip = $host_data{$3};
            $str =~ s/$old_dns_ip/$new_dns_ip/;
            $is_update = $TRUE;
         } ## end if (exists $host_data{...
         $host_check{$3} = 1;
      } ## end if ($str =~ /(([0-9]{1,3}\.){3}[0-9]{1,3})\s+(\w+)/...
   } ## end foreach my $str (@host_str)

   foreach (keys %host_data)
   {
      unless (exists $host_check{$_})
      {
         push @host_str, $host_data{$_}, "    ", $_, "\n";
         baselog($LOG_DEBUG, "Adding $host_data{$_} $_ to /etc/hosts file");
         $is_update = $TRUE;
      } ## end unless (exists $host_check...
   } ## end foreach (keys %host_data)

   if (!$comment_flag)
   {
      push @host_str, "############# User Configured Hosts ##############\n";
   }

   my $tmpfile = $etc_host_file . $$;

   if ($is_update)
   {
      open OUT, ">$tmpfile"
        or baselog($LOG_ERROR, "Can't open $tmpfile file for writing: $!"), return $FALSE;

      print OUT @host_str;
      close OUT;
      system($CHMOD, "--reference=$etc_host_file", "$tmpfile");
      rename($tmpfile, $etc_host_file);
      baselog($LOG_INFO, "The $etc_host_file file is updated");
   } ## end if ($is_update)
   return $TRUE;
} ## end sub updateDNSHost($)

################################################################################
## @fn setupDNS($data, $configNS)
# @brief Setup DNS servers
# @details Updates the following files:
#   \li /etc/resolv.conf
#   \li /etc/nsswitch.conf
#   \li /etc/hosts (updateDNSHost function)
# @param $data - hash reference with new data
# @param $configNS - path to nsswitch setup file ($NSSWITCH by default)
# @return $TRUE  - DNS setup correctly
# @return $FALSE - error during setup
################################################################################
sub setupDNS($;$)
{
   my ($data, $configNS) = (@_);
   if (   !defined($data->{'primaryDNSIP'})
       or !defined($data->{'secondaryDNSIP'})
       or !defined($data->{'tertiaryDNSIP'}))
   {
      baselog($LOG_ERROR, "Incorrect data parameter. Can not setup DNS");
      return $FALSE;
   } ## end if (!defined($data->{'primaryDNSIP'...
   unless (validateAllParams($data))
   {
      baselog($LOG_ERROR, "Validation FAILED");
      return $FALSE;
   }
   if (!defined($configNS))
   {
      $configNS = $NSSWITCH;
   }

   unless (updateDNSHost($data))
   {
      baselog($LOG_ERROR, "Can not update DNS IP addresses\n", $TRUE);
      return $FALSE;
   }

   my $file        = "/etc/resolv.conf";
   my $isDNS       = 0;                  # Flag if DNS configured or not
   my @dns_records = ();
   if (!-e "/admin/.vmware_machine")
   {
      if ($data->{'primaryDNSIP'} ne $UNCONFIGURED)
      {
         $isDNS = 1;
         push @dns_records, "nameserver $data->{'primaryDNSIP'}\n";
      }
      if ($data->{'secondaryDNSIP'} ne $UNCONFIGURED)
      {
         $isDNS = 1;
         push @dns_records, "nameserver $data->{'secondaryDNSIP'}\n";
      }

      if ($data->{'tertiaryDNSIP'} ne $UNCONFIGURED)
      {
         if ($data->{'tertiaryDNSIP'} ne '127.0.0.1' and $isDNS == 0)
         {
            push @dns_records, "nameserver $data->{'tertiaryDNSIP'}\n";
         }
      } ## end if ($data->{'tertiaryDNSIP'...
      push @dns_records, "nameserver 127.0.0.1\n";
   } ## end if (!-e "/admin/.vmware_machine"...

   if (setParams($data))
   {
      if (not write_file($file, \@dns_records))
      {
         baselog($LOG_ERROR, "Cannot write data into '$file'");
         return $FALSE;
      }
      baselog($LOG_INFO, "DNS records are added: @dns_records");

      # Configure /etc/nsswitch.conf file
      do_system([$PERL, $configNS, '1']);
      baselog($LOG_INFO, "The /etc/nsswitch.conf file is updated");
      return $TRUE;
   } ## end if (setParams($data))

   baselog($LOG_ERROR, "Can not save parameters");
   return $FALSE;
} ## end sub setupDNS($;$)

################################################################################
## @fn setupTimeZone($data)
# @brief Configures timezone information:
#  \li remove /etc/localtime
#  \li create a softlink from /etc/localtime to /usr/share/zoneinfo/<tz_file>
#  \li edit /etc/sysconfig/clock
# @param $data - hash reference with new data.
# @return $TRUE  - TimeZone setup correctly.
# @return $FALSE - error during setup
################################################################################
sub setupTimeZone($)
{
   my $data = shift(@_);

   if (!defined($data->{'timezone'}))
   {
      baselog($LOG_ERROR, "Incorrect data parameter. Can not setup TimeZone.");
      return $FALSE;
   }
   unless (validateAllParams($data))
   {
      baselog($LOG_ERROR, "Validation FAILED");
      return $FALSE;
   }

   my $zones_info = getTimezonesInfo({'zone' => $data->{'timezone'}});
   if (!defined($zones_info))
   {
      baselog($LOG_ERROR, "Incorrect data parameter. Can not setup TimeZone.");
      return $FALSE;
   }

   my $LinuxZone = $zones_info->[0]->{'LinuxZone'};
   if (!defined($LinuxZone))
   {
      baselog($LOG_ERROR, "Can not find match 'LinuxZone' for '$data->{'timezone'}' timezone.");
      return $FALSE;
   }

   # Save date to prevent its changing due to timezone change
   my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
   my ($fh, $file, $date);
   $file = "/etc/sysconfig/clock";
   if (!write_file("$file.new", ["ZONE=$LinuxZone\n", "UTC=false\n", "ARC=false\n"]))
   {
      baselog($LOG_ERROR, "Could not write to '$file.new' file.");
      return $FALSE;
   }
   if (!do_system([$LN, '-fs', "/usr/share/zoneinfo/$LinuxZone", '/etc/localtime']))
   {
      baselog($LOG_ERROR, "Can not apply zone changes.", $TRUE);
      unlink("$file.new");
      return $FALSE;
   } ## end if (!do_system([$LN, '-fs'...

   # restore date
   $date = sprintf "%02d%02d%02d%02d%4d\n", $mon + 1, $mday, $hour, $min, $year + 1900;
   qx{$DATE $date};

   if (setParams($data))
   {
      system($CHMOD, "--reference=$file", "$file.new");
      rename("$file.new", $file);
      baselog($LOG_INFO, "Timezone configured");
      return $TRUE;
   } ## end if (setParams($data))
   unlink("$file.new");
   baselog($LOG_ERROR, "Can not save parameters");
   return $FALSE;
} ## end sub setupTimeZone($)

################################################################################
## @fn setupNTP($data)
# @brief configures NTP
# @param $data - hash reference with new data.
# @return $TRUE  - NTP setup correctly.
# @return $FALSE - error during setup
################################################################################
sub setupNTP($)
{
   my $data = shift(@_);
   if (   !defined($data->{'clockSourceType'})
       or !defined($data->{'clockType'})
       or !defined($data->{'ntpSecurityMode'})
       or !defined($data->{'ntpSecurityModeKeyID'})
       or !defined($data->{'clockIPs'})
       or !defined($data->{'ntpSecurityModePriKey'}))
   {
      baselog($LOG_ERROR, "Incorrect data parameter\n", $TRUE);
      print "Can not setup NTP\n";
      return $FALSE;
   } ## end if (!defined($data->{'clockSourceType'...

   unless (validateAllParams($data))
   {
      baselog($LOG_ERROR, "Validation FAILED");
      return $FALSE;
   }
   my $ntp_file     = "/etc/ntp.conf";
   my $ntp_key_file = "/etc/ntp/keys";
   my $fh;
   my $i           = 0;
   my $private_key = $data->{'ntpSecurityModePriKey'};

   # Open the ntp.conf file
   do_system("$CP -f $ntp_file $ntp_file.orig");
   do_system("$CHMOD --reference=$ntp_file $ntp_file.orig");
   unless (open($fh, ">", "$ntp_file.tmp"))
   {
      baselog($LOG_ERROR, "Could not open $ntp_file.tmp: $!");
      unlink("$ntp_file.orig");
      return $FALSE;
   } ## end unless (open($fh, ">", "$ntp_file.tmp"...

   baselog($LOG_INFO, "Generating NTP config file ($ntp_file)...");

   if ($data->{'clockSourceType'} =~ /^$PRIMARY$|^$SECONDARY$/)
   {
      print $fh "# $ntp_file\n";
      print $fh "\n";
      print $fh "# Permit time synchronization with our time source, but do not\n";
      print $fh "# permit the source to query or modify the service on this system.\n";
      print $fh "restrict default kod nomodify notrap nopeer noquery\n";
      print $fh "restrict -6 default kod nomodify notrap nopeer noquery\n\n";
      print $fh "# Permit all access over the loopback interface.  This could\n";
      print $fh "# be tightened as well, but to do so would effect some of\n";
      print $fh "# the administrative functions.\n";
      print $fh "restrict 127.0.0.1\n";
      print $fh "restrict -6 ::1\n\n";

      if ($data->{'clockType'} == $EXTERNAL_CLOCK)
      {  #external clock source
           #disable panic if time delay is too big
         if (not exists $data->{'enablePanic'} or not $data->{'enablePanic'})
         {
            print $fh "\ntinker panic 0\n";
         }

         my @clocks = split(/\|/, $data->{'clockIPs'});
         if (@clocks)
         {
            print $fh "\n\n#Using External Time Source\n\n";
            if ($data->{'ntpSecurityMode'} eq "Y")
            {
               print $fh "server ", shift(@clocks), " key ", $data->{'ntpSecurityModeKeyID'}, " prefer\n";
            }
            else
            {
               print $fh "server ", shift(@clocks), " prefer\n";
            }
            foreach (@clocks)
            {
               if ($data->{'ntpSecurityMode'} eq "Y")
               {
                  print $fh "server $_ key $data->{'ntpSecurityModeKeyID'}\n";
               }
               else
               {
                  print $fh "server $_\n";
               }
            } ## end foreach (@clocks)
         } ## end if (@clocks)
           # Add local sync entries
         print $fh "server $LOCAL_IP\n";
         print $fh "fudge $LOCAL_IP stratum 10\n";
      } ## end if ($data->{'clockType'...
      else
      {
         print $fh "\n\n#Using internal clock as Time Source\n\n";
         if ($data->{'ntpSecurityMode'} eq "Y")
         {
            print $fh "server $data->{'clockIPs'} key ";
            print $fh "$data->{'ntpSecurityModeKeyID'} prefer\n";
         }
         else
         {
            print $fh "server $data->{'clockIPs'} prefer\n";
         }
         print $fh "fudge $data->{'clockIPs'} stratum 0\n";
      } ## end else [ if ($data->{'clockType'...

      my $peer = getPeer($data);
      if ($peer)
      {
         print $fh "\n\n#Peer Server Information\n";
         if ($data->{'ntpSecurityMode'} eq "Y")
         {
            print $fh "peer $peer key ";
            print $fh "$data->{'ntpSecurityModeKeyID'}\n\n";
         }
         else
         {
            print $fh "peer $peer\n\n";
         }
      } ## end if ($peer)
      print $fh "broadcast 224.0.1.1 ttl 4\n\n";
      print $fh "enable auth monitor\n";
      print $fh "driftfile /etc/ntp/ntp.drift\n";
      print $fh "statsdir /etc/ntp/ntpstats/\n";
      print $fh "filegen peerstats file peerstats type day enable\n";
      print $fh "filegen loopstats file loopstats type day enable\n";
      print $fh "filegen clockstats file clockstats type day enable\n\n";
   } ## end if ($data->{'clockSourceType'...
   else
   {
      print $fh "# $ntp_file\n";
      if (not exists $data->{'enablePanic'} or not $data->{'enablePanic'})
      {
         print $fh "\ntinker panic 0\n";
      }

      my @clocks = split(/\|/, $data->{'clockIPs'});
      if ($data->{'ntpSecurityMode'} eq "Y")
      {
         print $fh "server ", shift(@clocks), " key ", $data->{'ntpSecurityModeKeyID'}, " prefer\n";
         foreach (@clocks)
         {
            print $fh "server $_ key $data->{'ntpSecurityModeKeyID'}\n";
         }
      } ## end if ($data->{'ntpSecurityMode'...
      else
      {
         print $fh "server ", shift(@clocks), " prefer\n";
         foreach (@clocks)
         {
            print $fh "server $_\n";
         }
      } ## end else [ if ($data->{'ntpSecurityMode'...

      print $fh "\n";
      print $fh "driftfile /etc/ntp/ntp.drift\n";
   } ## end else [ if ($data->{'clockSourceType'...

   #
   # If the NTP is configured as MD5 Security Transfer Mode
   #
   if ($data->{'ntpSecurityMode'} eq "Y")
   {
      print $fh "trustedkey $data->{'ntpSecurityModeKeyID'}\n";
      print $fh "keys $ntp_key_file\n";
      close($fh);

      system("$CP $ntp_key_file $ntp_key_file.orig");

      # Open the ntp/keys file
      my $fh1;
      if (open($fh1, ">>$ntp_key_file"))
      {
         print $fh1 "\n$data->{'ntpSecurityModeKeyID'}\t";
         print $fh1 "M\t";
         print $fh1 "$private_key\n";
         close($fh1);
      } ## end if (open($fh1, ">>$ntp_key_file"...
      else
      {
         baselog($LOG_ERROR, "Could not open $ntp_key_file: $!");
         unlink("$ntp_file.orig");
         unlink("$ntp_file.tmp");
         return $FALSE;
      } ## end else [ if (open($fh1, ">>$ntp_key_file"...
   } ## end if ($data->{'ntpSecurityMode'...

   close($fh);
   rename("$ntp_file.tmp", $ntp_file);
   do_system("$CHMOD --reference=$ntp_file.orig $ntp_file");
   if (setParams($data))
   {
      baselog($LOG_INFO, "NTP setup is done.");
      unlink("$ntp_file.orig");
      return $TRUE;
   } ## end if (setParams($data))
   rename("$ntp_file.orig",     $ntp_file);
   rename("$ntp_key_file.orig", $ntp_key_file);
   baselog($LOG_ERROR, "Can not save parameters");
   return $FALSE;
} ## end sub setupNTP($)

################################################################################
## @fn getTimezoneInfo($attrs)
# @brief Read specified xml file and return reference to array with hashes which
#        contain information about timezones that satisfies specified conditions
# @param $attrs  - reference to the hash with attributes that must be satisfied.
# @return undef  - information can not be read.
# @return reference to array - information successfully retrieved.
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub getTimezonesInfo(;$)
{
   my $attrs = shift(@_);
   my @ret   = ();

   my $parser = base_xml->new($TZFILE);
   if (!defined($parser))
   {
      baselog($LOG_ERROR, "Can not read timezone file '$TZFILE'.");
      return undef;
   }
   my $elements = $parser->findElements('zoneentry', $attrs);
   if (!defined($elements))
   {
      baselog($LOG_ERROR, "Cannot find elements.");
      undef $parser;
      return undef;
   } ## end if (!defined($elements...
   for my $elem (@$elements)
   {
      push @ret, $parser->getAttrsForElement($elem);
   }
   undef $parser;
   return \@ret;
} ## end sub getTimezonesInfo(;$)

################################################################################
## @fn getTimezoneDescr($zones_info)
# @brief get timezone description from the hash of attributes.
# @param $zones_info - reference to the array of hashes with attributes or the
#        reference to a hash with attributes
# @return string - timezone description
# @return 'N/A' - timezone description can not be identified.
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub getTimezoneDescr($)
{
   my $zones_info = shift(@_);
   my $hash;
   my $retval = 'N/A';
   if (!defined($zones_info))
   {
      baselog($LOG_WARNING, "Timezones information is not specified.");
      return $retval;
   }
   if (is_array($zones_info))
   {
      if (!scalar(@$zones_info))
      {
         baselog($LOG_WARNING, "Array does not contain information about timezones.");
         return $retval;
      }
      $hash = $zones_info->[0];
   } ## end if (is_array($zones_info...
   elsif (is_hash($zones_info))
   {
      $hash = $zones_info;
   }
   else
   {
      baselog($LOG_ERROR, "Invalid argument passed.");
      return $retval;
   }
   if (!defined($hash->{'displayname'}) || !defined($hash->{'DST'}))
   {
      baselog($LOG_ERROR, "Required attributes are not defined.");
      return $retval;
   }
   return "[DST=$hash->{'DST'}] $hash->{'displayname'}";
} ## end sub getTimezoneDescr($)

################################################################################
## @fn unconfigureDNS($data)
# @brief Unconfigures DNS settings.
# @param $data - hash reference with data.
# @return $TRUE  - DNS unconfigured successfully.
# @return $FALSE - passed argument is not correct.
# @author Evgeny Lazarev
# @date January 2009
################################################################################
sub unconfigureDNS($)
{
   my $data = shift(@_);
   if (!defined($data))
   {
      baselog($LOG_ERROR, "Passed argument is incorrect");
      return $FALSE;
   }

   #By default the tertiaryDNS will be the internal DNS Server.
   $data->{'primaryDNSIP'}   = $UNCONFIGURED;
   $data->{'secondaryDNSIP'} = $UNCONFIGURED;
   $data->{'tertiaryDNSIP'}  = $LOCALHOST;
   baselog($LOG_INFO, 'DNS unconfigured.');
   return $TRUE;
} ## end sub unconfigureDNS($)

###############################################################################
## @fn getPeer($data)
# @brief returns value for the peer depending on NTP configuration
# @param $data - hash reference with data.
# @return string - peer.
# @author Evgeny Lazarev
# @date February 2009
sub getPeer($)
{
   my $data   = shift(@_);
   my $retval = '';
   if ($data->{'clockSourceType'} eq $PRIMARY)
   {
      $retval = $data->{'machineIP'};
   }
   elsif ($data->{'clockSourceType'} eq $SECONDARY)
   {
      my @IPs = split /\|/, $data->{'clockIPs'};
      $retval = $IPs[0];
   }
   return $retval;
} ## end sub getPeer($)

################################################################################
## @fn validateListOfParams($data, $list)
# @brief Validates list of the parameters in the passed hash.
#        If the value is not correct it is cleared.
# @param $data - hash reference with data to verify.
# @param $list - reference to the hash with options for validation.
# @return $TRUE  - all parameters are correct.
# @return $FALSE - at least one parameter is invalid.
# @author Evgeny Lazarev
# @date February 2009
################################################################################
sub validateListOfParams($$)
{
   my ($data, $list) = (@_);
   my $retval = $TRUE;
   if (!is_hash($data))
   {
      baselog($LOG_ERROR, "Passed argument is not a hash.");
      return $FALSE;
   }
   if (!is_array($list))
   {
      baselog($LOG_ERROR, "Passed argument is incorrect.");
      return $FALSE;
   }

   for my $param (@$list)
   {
      if (defined($data->{$param}) && !validateParam($param, $data->{$param}))
      {
         baselog($LOG_ERROR, "Parameter '$param' is incorrect.\n", $TRUE);
         $data->{$param} = '';
         $retval = $FALSE;
      } ## end if (defined($data->{$param...
   } ## end for my $param (@$list)
   return $retval;
} ## end sub validateListOfParams($$)

1;

__END__

=head1 NAME

baseParams - API for linux base parameters management

=head1 SYNOPSIS

use baseParams;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

