#############################################################
## @file baseParamsUI.pm
# @brief User interface for LinuxBase
# @details Module for simple textbased user interface
#          for changing and managing Linux Base
#          parameters
# @author Alex Arkhipov
# @date December 2006
# @version 1.00 - Initial version
# @version 1.01 - Function print_ntp_config() was changed
#                 by Kate Pestova
# @version 1.02 - applyTimeFromCS parameter was added
#               - Function configClockSourceNTP() was change
# @version 1.03 - Two colons were removed before shared functions and variables.
# @version 1.04 - Printing of error message was added into multiColumnDisplay function
#                 if wrong timezone was selected by user.
# @version 1.05 - Function configNetwork() was changed, doxygen comments was added by Ekaterina Bazhanova
# @version 1.06 - Correcting functions getValidNTPKeyID() and print_ntp_config().
# @version 1.07 - Remove CSSync question from configNTP
#                 Remove applyTimeFromCS parameter
# @version 1.08 - Call function validatePriKey() in getNTPPriKey().
# @version 1.09 - Set empty peer value for OTHER server.
# @version 1.10 - Adding more checks for Network Configuration
# @version 1.11 - Adding fn convertNetworkData
# @version 1.12 - Support of "panic" option is added
# @version 1.13 - Function configClockSourceNTP() was modified by Kate Pestova:
#                 1) removing of Seconday-Internal configuration
#                 2) default value for peer
# @version 1.14 - The uniqueness check for external IP addresses was added
# @version 1.20 - Configuration of the Timezone has been  modified. Now GMT offset is configured first,
#               then Timezone is chosen. 'configTimeZone' has been rewritten.
#               'TimeZone.xml' file is used instead of 'timezone'. 'multiColumnDisplay' has been
#               rewritten: only one parameter is accepted - array of data to display.
# @version 1.21 - 'otherClockSource' has been removed.
# @version 1.22 - 'convertNetworkData' has been moved to the baseParams.pm module.
# @version 1.23 - Change the query for configuring a domain name and set the default gateway
#                 to TLAN by default in the interactive mode (install-time and run-time).
# @version 1.24 - 'configClockSourceNTP' has been changed. Parameter
#               verification (number of external IPs) has been changed.
# @note
#############################################################
package baseParamsUI;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;

use baseParams;
use CommonInstallUtils;
use baselog;

# Access management
use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(
  &configNTP &configTimeZone &configDateTime
  &configNetwork &configDNSServer &rebootPrompt
  &print_network_config &print_dns_config
  &print_ntp_config &print_baseparam_config
  %paramDescr);

our $VERSION = "1.24";

our %paramDescr = (
                   'machineIP'             => 'ELAN IP Address',
                   'gateway'               => 'ELAN Gateway IP Address',
                   'netmask'               => 'ELAN Netmask',
                   'machineIPTlan'         => 'TLAN IP Address ',
                   'gatewayTlan'           => 'TLAN Gateway IP Address',
                   'netmaskTlan'           => 'TLAN Netmask',
                   'defaultGateway'        => 'Default Gateway',
                   'hostname'              => 'Hostname',
                   'fqdnTlan'              => 'Fully Qualified Domain Name',
                   'primaryDNSIP'          => 'Primary DNS Server IP Address',
                   'secondaryDNSIP'        => 'Secondary DNS Server IP Address',
                   'tertiaryDNSIP'         => 'Default DNS Server IP Address',
                   'clockSourceType'       => 'NTP Clock Source',
                   'clockType'             => 'NTP Clock Type',
                   'ntpSecurityMode'       => 'NTP Server is in secure MD5 Transfer Mode',
                   'ntpSecurityModeKeyID'  => 'NTP Key ID',
                   'ntpSecurityModePriKey' => 'NTP Private Key',
                   'clockIPs'              => 'Clock Source IP Address',
                   'timezone'              => 'Timezone'
                  );

my $MAX_PER_DISPLAY = 63;  # good for normal 25x80 terminal

my @clockSourceTypes = ('Primary', 'Secondary', 'Other');

## @skipline
sub multiColumnDisplay(@);
## @skipline
sub configClockSourceNTP($$);
## @skipline
sub getValidNTPKeyID($$);
## @skipline
sub getNTPPriKey();
## @skipline
sub configDNSServer($);
## @skipline
sub configNTP($);
## @skipline
sub configNetwork($);
## @skipline
sub configTimeZone($);
## @skipline
sub configDateTime();
## @skipline
sub rebootPrompt();
## @skipline
sub print_network_config($);
## @skipline
sub print_dns_config($);
## @skipline
sub print_ntp_config($);
## @skipline
sub print_baseparam_config($);

sub INIT
{
   system_installation_init();
}

################################################################################
################################ Private Functions #############################

################################################################################
## @fn multiColumnDisplay(@values)
# @brief Provides a method of displaying multiple columns to the user for now
#        this is used for timezones
# @details None.
# @param @values - array of values
# @return None
################################################################################
sub multiColumnDisplay(@)
{
   my (@values) = @_;
   my ($prompt1, $prompt2, $start, $end, $i, $done, $failed, $opt);
   my %nums;

   my $numVals = scalar @values;

   # display the amount that will fit on the first screen
   $start = 1;
   if ($numVals > $MAX_PER_DISPLAY)
   {
      $end = $MAX_PER_DISPLAY;
   }
   else
   {
      $end = $numVals;
   }

   $prompt1 = "GMT Offset Selection";
   $prompt2 = "GMT Offset";

   # Keep prompting until we get a valid value
   $done   = 0;
   $failed = $FALSE;
   while (!$done)
   {
      clear();
      if ($failed)
      {
         print "Please, input correct data(number between $start and $end values).\n";
         $failed = $FALSE;
      }
      print "$prompt1\n";

      # for a 25x80 screen, 3 columns is all we can handle - make sure we have
      # a valid value before printing
      for ($i = $start ; $i <= $end ; $i += 3)
      {
         printf("%3d) %-20s ", $i, substr($values[$i - 1], 0, 20));
         $nums{$i} = $values[$i - 1];

         if ($i + 1 <= $end)
         {
            printf("%3d) %-20s ", $i + 1, substr($values[$i], 0, 20));
            $nums{$i + 1} = $values[$i];
         }

         if ($i + 2 <= $end)
         {
            printf("%3d) %-20s", $i + 2, substr($values[$i + 1], 0, 20));
            $nums{$i + 2} = $values[$i + 1];
         }

         print "\n";
      } ## end for ($i = $start ; $i <=...

      # If we have more than can fit on one screen, let user know there
      # are more items
      if ($numVals > $MAX_PER_DISPLAY)
      {
         print "Enter $prompt2 ($start-$end) or press \"Enter\" for more options: ";
      }
      else
      {
         print "Enter $prompt2 ($start-$end): ";
      }

      # Ok user pick somethin' good now...
      chomp($opt = <STDIN>);

      # If the user hit enter with entering anything, display either
      # the next screen or the current screen, depending on how many
      # there were to begin with
      if ($opt eq "")
      {
         if (($start > 1) || ($numVals > $end))
         {
            if ($end == $numVals)
            {  # go back to the beginning
               $start = 1;
               $end   = $MAX_PER_DISPLAY;
            }
            else
            {
               $start = $end + 1;
               $end   = $numVals;
            }
         } ## end if (($start > 1) || ($numVals...
         else
         {
            $failed = $TRUE;
         }
      } ## end if ($opt eq "")
      else
      {
         if (($opt !~ /^\d+$/))  # only digits are acceptable
         {
            $failed = $TRUE;
            next;
         }
         $opt = int($opt);

         # validate the input - if not valid, redisplay the menu
         if (($opt <= $end) && ($opt >= $start))
         {
            $done = 1;
            return "$nums{$opt}";
         }
         else
         {
            $failed = $TRUE;
         }
      } ## end else [ if ($opt eq "")
   } ## end while (!$done)
} ## end sub multiColumnDisplay(@)

################################################################################
## @fn       configClockSourceNTP($data, $clockIPref)
# @brief     Configure the actual clock source data
# @details   None.
# @param     $data   - hash with data to save
# @param     $clockIPref  - clock ip refferences
# @return    None
################################################################################
sub configClockSourceNTP($$)
{
   my ($data, $clockIPref) = (@_);
   my @clockIPs        = @$clockIPref;
   my $valid           = $FALSE;
   my @defaultClockIPs = ();
   my $defaultType     = "";
   my $type            = "";
   my $default;

   if (@clockIPs)
   {
      @defaultClockIPs = @clockIPs;
      @clockIPs        = ();        # clear out the clockIPs
   }

   $data->{'enablePanic'} = $TRUE;

   if ($data->{'clockSourceType'} != $OTHER)
   {
      header('NTP Clock Source Configuration');
      print "The $clockSourceTypes[$data->{'clockSourceType'}] Clock Source server ";
      print "requires the use of an external clock.\n\n";
      if ($data->{'clockSourceType'} == $PRIMARY)
      {
         print "Select External Clock for time source(s) external to this server.\n";
         print "Select Internal Clock to use the local system clock as the time source.\n\n";

         print "   E - External Clock Source (IP Addresses)\n";
         print "   I - Internal Clock (Unreliable)\n";

         if ($data->{'clockType'} eq $INTERNAL_CLOCK)
         {
            $defaultType = 'I';
         }
         elsif ($data->{'clockType'} eq $EXTERNAL_CLOCK)
         {
            $defaultType = 'E';
         }
      } ## end if ($data->{'clockSourceType'...

      while (!$valid)
      {
         $valid = $TRUE;
         if ($data->{'clockSourceType'} == $PRIMARY)
         {
            print "\nSelect an option (E, I)";

            if ($defaultType ne '')
            {
               print " [$defaultType]: ";
            }
            else
            {
               print ": ";
            }

            chomp($type = <STDIN>);

            if ($type eq '')
            {
               $type = $defaultType;
            }
         } ## end if ($data->{'clockSourceType'...

         if ($type =~ /^e$/i or $data->{'clockSourceType'} == $SECONDARY)
         {  # External device (reached via IP)

            #disable ntpd panic on big time delta
            $data->{'enablePanic'} = $FALSE;

            if ($data->{'clockType'} eq $INTERNAL_CLOCK)
            {
               @defaultClockIPs = ();
            }
            $data->{'clockType'} = $EXTERNAL_CLOCK;

            my $default_cnt = scalar(@defaultClockIPs);
            if ($default_cnt == 0)
            {
               $default_cnt = 1;
            }
            while ($TRUE)
            {
               print "How many external clock sources would you like to reference (1-$MAX_EXTERNAL_IPS_COUNT) [$default_cnt]? ";
               chomp(my $cnt = <STDIN>);

               if ($cnt eq '')
               {
                  $cnt = $default_cnt;
               }

               if (is_integer($cnt) && $cnt >= 1 && $cnt <= $MAX_EXTERNAL_IPS_COUNT)
               {
                  for (my $i = 1 ; $i <= $cnt ; $i++)
                  {
                     if (defined($defaultClockIPs[$i - 1]))
                     {
                        $default = $defaultClockIPs[$i - 1];
                     }
                     else
                     {
                        $default = "";
                     }
                     if ($i == 1 and $data->{'clockSourceType'} == $SECONDARY)
                     {
                        push(@clockIPs, getValidIP("Primary IP Address", $default));
                     }
                     else
                     {

                        # To avoid duplicate IP addresses
                        while ($TRUE)
                        {
                           my $tmpIP = getValidIP("External $paramDescr{'clockIPs'} \#$i", $default);
                           if ($i == 1 or ($i >= 2 and pos_in_array([@clockIPs[0 .. $i - 2]], $tmpIP) < 0))
                           {
                              push(@clockIPs, $tmpIP);
                              last;
                           }
                           else
                           {
                              print "\nERROR: Duplicated entry. Please try again.\n\n";
                              if ($default eq $tmpIP)
                              {
                                 $default = "";
                              }
                           } ## end else [ if ($i == 1 or ($i >= ...
                        } ## end while ($TRUE)
                     } ## end else [ if ($i == 1 and $data->...
                  }  # end of for()
                  last;
               } ## end if (is_integer($cnt) &&...
               else
               {
                  print "\nInvalid Entry - please enter a value between 1 and $MAX_EXTERNAL_IPS_COUNT\n\n";
               }
            }  # end of while($TRUE)
         } ## end if ($type =~ /^e$/i or...
         elsif ($type =~ /^i$/i)
         {     # Internal clock
            $data->{'clockType'} = $INTERNAL_CLOCK;
            $clockIPs[0] = $LOCAL_IP;
         }
         else
         {
            $valid = $FALSE;
            print "\nERROR: Invalid entry. Please try again.\n";
         }
      } ## end while (!$valid)

      if ($data->{'clockSourceType'} == $PRIMARY)
      {
         my %hash = ('machineIP' => '');
         getParams(\%hash);
      }
   } ## end if ($data->{'clockSourceType'...
   else
   {
      $data->{'clockType'} = $EXTERNAL_CLOCK;

      header('NTP Configuration');

      #disable panic on big time delta
      $data->{'enablePanic'} = $FALSE;

      print "Provide the IP addresses of the Primary and/or Secondary Clock Source Server(s)\n\n";

      my $prompt;
      $prompt = "Enter MACHINE IP of the Primary Clock Source Server";

      my $default = "";
      if (defined($defaultClockIPs[0]))
      {
         $default = $defaultClockIPs[0];
      }
      $clockIPs[0] = getValidIP($prompt, $default);

      $prompt  = "Enter MACHINE IP of the Secondary Clock Source Server";
      $default = "";
      if (defined($defaultClockIPs[1]))
      {
         $default = $defaultClockIPs[1];
      }
      else
      {

         # The 2nd source is same as first
         $default = $clockIPs[0];
      } ## end else [ if (defined($defaultClockIPs...
      my $temp = getValidIP($prompt, $default);
      if ($temp ne $clockIPs[0])
      {
         $clockIPs[1] = $temp;
      }
   } ## end else [ if ($data->{'clockSourceType'...
   $data->{'clockIPs'} = join("|", @clockIPs);
} ## end sub configClockSourceNTP($$)

###############################################################################
## @fn       getValidNTPKeyID($prompt, $ntpKeyID)
# @brief     Prompt the user until he gives us a valid NTP key
#            ID between 1 ~ 65535
# @details   None.
# @param     $prompt    - prompt
# @param     $ntpKeyID  - default ntp key ID
# @return    $newKeyID  - NTP key ID
###############################################################################
sub getValidNTPKeyID($$)
{
   my ($prompt, $ntpKeyID) = @_;
   my $newKeyID = "";

   my $valid = $FALSE;
   while (!$valid)
   {
      if ($ntpKeyID)
      {
         print "$prompt [$ntpKeyID]: ";
      }
      else
      {
         print "$prompt: ";
      }

      chomp($newKeyID = <STDIN>);

      if ($newKeyID eq "")
      {
         if ($ntpKeyID)
         {
            $newKeyID = $ntpKeyID if $newKeyID eq "";  # default
         }
         else
         {
            print "\nERROR: Invalid entry. Please try again.\n\n";
            next;
         }
      } ## end if ($newKeyID eq "")

      if ($newKeyID !~ /^\d+$/)
      {
         print "\nERROR: NTP key ID should be numeric. Please try again.\n\n";
         next;
      }

      if ($newKeyID >= 1 and $newKeyID <= 65535)
      {
         $valid = $TRUE;
      }
      else
      {
         print "\nERROR: NTP key ID should be between 1 ~ 65535. Please try again.\n\n";
      }
   } ## end while (!$valid)

   return $newKeyID;
} ## end sub getValidNTPKeyID($$)

###############################################################################
## @fn       getNTPPriKey()
# @brief     Prompt the user until he gives us a valid NTP
#            private key
# @details   None.
# @param     None.
# @return    $pw2 - NTP private key
###############################################################################
sub getNTPPriKey()
{
   my ($pw1, $pw2);
   my $valid = $FALSE;

   while (!$valid)
   {
      print "\n\nPrivate key is a printable ASCII string 16 characters or less in length excluding space, # and '.\n";
      print "\nEnter the NTP private key: ";
      system "$STTY -echo";
      chomp($pw1 = <STDIN>);
      if (!validatePriKey($pw1))
      {
         print "\n\nERROR: Invalid NTP private key. Please try again.\n\n";
      }
      else
      {

         # See if this is a valid key
         print "\nEnter the NTP private key again: ";
         chomp($pw2 = <STDIN>);
         system "$STTY echo";

         if ($pw1 eq $pw2)
         {
            $valid = $TRUE;
         }
         else
         {
            print "\n\nERROR: Private Keys do not match. Please try again.\n\n";
         }
      } ## end else [ if (!validatePriKey($pw1...
   } ## end while (!$valid)

   return $pw2;
} ## end sub getNTPPriKey()

##################################################################
######################## Public Functions ########################

###############################################################################
## @fn       configDNSServer($data)
# @brief     Configure DNS servers
# @details   None.
# @param     $data    -  Hash refference to save data in
# @return    $TRUE
###############################################################################
sub configDNSServer($)
{
   my $data = shift(@_);
   header('DNS Server Configuration');

   my $query = 'Do you wish to configure the Primary DNS Server IP Address';
   if (getYNResponse($query, 'N'))
   {
      print "\n";
      $data->{'primaryDNSIP'} = getValidIP('Enter the Primary DNS Server IP Address', $data->{'primaryDNSIP'});

      my $query = 'Do you wish to configure the Secondary DNS Server IP Address';
      if (getYNResponse($query, 'N'))
      {
         print "\n";
         $data->{'secondaryDNSIP'} = getValidIP('Enter the Secondary DNS Server IP Address', $data->{'secondaryDNSIP'});
      }
      else
      {
         $data->{'secondaryDNSIP'} = "$UNCONFIGURED";
      }
   } ## end if (getYNResponse($query...
   else
   {
      $data->{'primaryDNSIP'}   = "$UNCONFIGURED";
      $data->{'secondaryDNSIP'} = "$UNCONFIGURED";
   }

   #By default the tertiaryDNS will be the internal DNS Server.
   $data->{'tertiaryDNSIP'} = $LOCALHOST;
   return $TRUE;
} ## end sub configDNSServer($)

###############################################################################
## @fn       configNTP($data)
# @brief     Configure NTP
# @details   None.
# @param     $data    -  Hash refference to save data in
# @return    $TRUE
###############################################################################
sub configNTP($)
{
   my $data    = shift(@_);
   my $default = $FALSE;
   my $opt;
   my $tempClockSourceType = $data->{'clockSourceType'};
   my @clockIPs = split(/\|/, $data->{'clockIPs'});
   header('Network Time Protocol (NTP) Configuration');

   # First, determine system-wide NTP transfer mode
   print "\nPlease determine NTP transfer mode within your whole system:\n\n";
   my $query = '  Do you wish to configure NTP in secure MD5 transfer mode?';
   if (getYNResponse($query, 'Y'))
   {
      print "\n";
      $data->{'ntpSecurityMode'}       = 'Y';
      $data->{'ntpSecurityModeKeyID'}  = getValidNTPKeyID('   Enter the NTP (in secure MD5 transfer mode) key ID', $data->{'ntpSecurityModeKeyID'});
      $data->{'ntpSecurityModePriKey'} = getNTPPriKey();
   } ## end if (getYNResponse($query...
   else
   {
      $data->{'ntpSecurityMode'}       = 'N';
      $data->{'ntpSecurityModeKeyID'}  = '';
      $data->{'ntpSecurityModePriKey'} = '';
   } ## end else [ if (getYNResponse($query...

   # Determine the Clock Source fuction of the Linux system
   while ($TRUE)
   {
      print "\nPlease indicate the Clock Source function of this Linux system:\n\n";
      print "  1) Primary Clock Source server (This is the Primary NTP server)\n";
      print "  2) Secondary Clock Source server (another one is the Primary NTP server)\n";
      print "  3) This Linux system is NOT a Clock Source server \n";

      if ($data->{'clockSourceType'} eq '')
      {
         print "\nSelect an option (1-3): ";
      }
      else
      {
         $default = $TRUE;
         if ($data->{'clockSourceType'} == $PRIMARY)
         {
            print "\nSelect an option (1-3) [1]: ";
         }
         elsif ($data->{'clockSourceType'} == $SECONDARY)
         {
            print "\nSelect an option (1-3) [2]: ";
         }
         else
         {
            print "\nSelect an option (1-3) [3]: ";
         }
      } ## end else [ if ($data->{'clockSourceType'...
      chomp($opt = <STDIN>);

      if ($opt eq '1')
      {
         $data->{'clockSourceType'} = $PRIMARY;
         last;
      }
      elsif ($opt eq '2')
      {
         $data->{'clockSourceType'} = $SECONDARY;
         last;
      }
      elsif ($opt eq '3')
      {
         $data->{'clockSourceType'} = $OTHER;
         last;
      }
      elsif ($default && ($opt eq ''))
      {
         last;
      }
      else
      {
         print "Incorrect entry. Please try again.\n";
      }
   } ## end while ($TRUE)

   # If we just switched server types (1/2 -> 3 or vice versa), then
   # we should clear out some of the data so we don't have strange
   # defaults in the following NTP questions
   if ($tempClockSourceType ne '')
   {
      if ($tempClockSourceType ne $data->{'clockSourceType'})
      {  # change occurred
         @clockIPs = ();
         $data->{'clockType'} = '';
      }
   } ## end if ($tempClockSourceType...

   configClockSourceNTP($data, \@clockIPs);
   return $TRUE;
} ## end sub configNTP($)

###############################################################################
## @fn       configNetwork($data)
# @brief     This will query the network settings for this machine
# @details   None.
# @param     $data    -  Hash refference to save data in
# @return    $TRUE
###############################################################################
sub configNetwork($)
{
   my $data        = shift(@_);
   my $oldhostname = $data->{'hostname'};
   header('Network Configuration');

   # ELAN IP configurations
   while ($TRUE)
   {
      $data->{'machineIP'} = getValidIP("Enter $paramDescr{'machineIP'}", $data->{'machineIP'});
      $data->{'netmask'}   = getValidIP("Enter $paramDescr{'netmask'}",   $data->{'netmask'});
      $data->{'gateway'}   = getValidIP("Enter $paramDescr{'gateway'}",   $data->{'gateway'});

      # ELAN IP and its Gateway shoule be within the same network
      if (andIpAddrs($data->{'machineIP'}, $data->{'netmask'}) eq andIpAddrs($data->{'gateway'}, $data->{'netmask'}))
      {
         last;
      }
      else
      {
         print <<HERE;

ERROR:     ELAN IP Address and its Gateway must be within the same subnet 
           Please try again.

HERE
      } ## end else [ if (andIpAddrs($data->...
   } ## end while ($TRUE)
     # TLAN IP configurations
   print("FQDN (Fully Qualified Domain Name) = Hostname + Domain Name \n");
   while ($TRUE)
   {
      my $hostname = getTextResponse("Enter $paramDescr{'hostname'}", $data->{'hostname'});

      # to check host name doesn't contain dot or dots and other checking
      if ($hostname !~ /\./ && validateParam('hostname', $hostname))
      {
         $hostname =~ tr/A-Z/a-z/;
         $data->{'hostname'} = $hostname;
         last;
      } ## end if ($hostname !~ /\./ ...
      else
      {
         print <<HERE;

ERROR: Hostnames must begin with a valid character in the range 'a-z' or 'A-Z'
       and can only contain the characters 'a-z', 'A-Z', '0-9', '-'. 
       Please try again.

HERE
      } ## end else [ if ($hostname !~ /\./ ...
   } ## end while ($TRUE)
   my $query = 'Do you wish to configure a Domain Name';
   if (getYNResponse($query, 'Y'))
   {
      while ($TRUE)
      {
         if ($data->{'fqdnTlan'} ne ""
             && !($data->{'fqdnTlan'} =~ /localdomain$/))
         {
            $data->{'fqdnTlan'} =~ s/^$oldhostname\.//;
         }
         else
         {
            $data->{'fqdnTlan'} = "";
         }
         my $fqdnTlan = getTextResponse("Enter TLAN port Domain Name", $data->{'fqdnTlan'});
         if (validateParam('fqdnTlan', $data->{'hostname'} . "." . $fqdnTlan)
             && ($fqdnTlan !~ /^$data->{'hostname'}\./))
         {
            $fqdnTlan =~ tr/A-Z/a-z/;
            $data->{'fqdnTlan'} = $data->{'hostname'} . "." . $fqdnTlan;
            last;
         } ## end if (validateParam('fqdnTlan'...
         else
         {
            print <<HERE;

ERROR: Domain Name must begin with a valid character in the range 'a-z' or 'A-Z'
       and can only contain the characters 'a-z', 'A-Z', '0-9', '-', or '.'.
       Domain Name must have at least one '.' (dot) in it.
       Also, Hostname can not be the first part of Domain Name. Please try again.

HERE
         } ## end else [ if (validateParam('fqdnTlan'...
      } ## end while ($TRUE)
   } ## end if (getYNResponse($query...
   else
   {
      $data->{'fqdnTlan'} = $data->{'hostname'} . ".localdomain";
   }
   
   while ($TRUE)
   {
      # TLAN IP and ELAN IP can not be the same
      while ($TRUE)
      {
         $data->{'machineIPTlan'} = getValidIP("Enter $paramDescr{'machineIPTlan'}", $data->{'machineIPTlan'});
         if ($data->{'machineIPTlan'} eq $data->{'machineIP'})
         {
            print <<HERE;

ERROR:     ELAN IP Address and TLAN IP Address can not be the same
           Please try again.

HERE
         } ## end if ($data->{'machineIPTlan'...
         else
         {
            last;
         }
      } ## end while ($TRUE)
      $data->{'netmaskTlan'} = getValidIP("Enter $paramDescr{'netmaskTlan'}", $data->{'netmaskTlan'});
      $data->{'gatewayTlan'} = getValidIP("Enter $paramDescr{'gatewayTlan'}", $data->{'gatewayTlan'});

      # TLAN IP and its Gateway shoule be within the same network
      if (andIpAddrs($data->{'machineIPTlan'}, $data->{'netmaskTlan'}) eq andIpAddrs($data->{'gatewayTlan'}, $data->{'netmaskTlan'}))
      {
         last;
      }
      else
      {
         print <<HERE;

ERROR:     TLAN IP Address and its Gateway must be within the same subnet 
           Please try again.

HERE
      } ## end else [ if (andIpAddrs($data->...
   } ## end while ($TRUE)

   print "\n";

   #In the interactive mode (and during install-time), 
   #the defaultGateway will always be defaulted to "1" - TLAN
   print "The Default Gateway will be set to the TLAN Gateway.\n";
   print "Press ENTER to continue.\n";
   $data->{'defaultGateway'} = 1;
   <STDIN>;
   return $TRUE;
} ## end sub configNetwork($)

##############################################################################
## @fn       configTimeZone($data)
# @brief     Query for the timezone - unfortunately.
# @details   None.
# @param     $data    -  Hash refference to save data in
# @return    $TRUE
###############################################################################
sub configTimeZone($)
{
   my ($data) = (@_);
   my @items;
   my %tmp_hash = ();
   my $selected_offset;
   my $zones_info = [];

   header('TimeZone Configuration');
   if (validateTimeZone($data->{'timezone'}))
   {
      my $tzdescr = getTimezoneDescr(getTimezonesInfo({'zone' => $data->{'timezone'}}));
      print "Previously configured timezone is:\n $tzdescr\n";
      my $query = "Do you want to use this timezone?";
      if (getYNResponse($query, 'Y'))
      {
         return $TRUE;
      }
   } ## end if (validateTimeZone($data...

   $zones_info = getTimezonesInfo();
   if (!defined($zones_info))
   {
      baselog($LOG_ERROR, "Can not configure TimeZone.");
      return $FALSE;
   }

   for my $record (@$zones_info)
   {
      $tmp_hash{$record->{'UTCOFFSET'}} = $TRUE;
   }

   my @offsets = sort keys %tmp_hash;

   # Until something valid is entered, query the user
   # first for the UTC offset and then for the Timezone
   my $valid = $FALSE;
   while (!$valid)
   {
      $selected_offset = multiColumnDisplay(@offsets);

      my @disp_names = ();
      for my $record (@$zones_info)
      {
         if ($record->{'UTCOFFSET'} eq $selected_offset)
         {
            push @disp_names, getTimezoneDescr($record);
         }
      } ## end for my $record (@$zones_info...
      showListByPage(\@disp_names);
      my $select = <STDIN>;

      # if user entered a number
      if ($select =~ /^\d+$/)
      {
         if ($select >= 1 && ($select <= scalar(@disp_names)))
         {
            my $selected_tz = $disp_names[$select - 1];
            for my $record (@$zones_info)
            {
               if ($selected_tz eq getTimezoneDescr($record))
               {
                  $data->{'timezone'} = $record->{'zone'};
                  $valid = $TRUE;
               }
            } ## end for my $record (@$zones_info...
         } ## end if ($select >= 1 && ($select...
      } ## end if ($select =~ /^\d+$/...
      else
      {
         print "Please, enter a number from 0 to " . scalar(@disp_names) . ".\n";
      }
   } ## end while (!$valid)
   return $TRUE;
} ## end sub configTimeZone($)

###############################################################################
## @fn       configDateTime()
# @brief     Queries the user for the date and time - allows
#            the user to keep the current date and time or
#            allows him to set it
# @details   None.
# @param     None.
# @return    $TRUE
###############################################################################
sub configDateTime()
{
   header('Date and Time Configuration');

   # show the current date and time and see if they want to change it
   while ($TRUE)
   {
      my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
      printf "Current Date and Time: %02d:%02d:%02d %d/%d/%d\n\n", $hour, $min, $sec, $mon + 1, $mday, $year + 1900;
      if (!getYNResponse("Do you want to keep this date and time", 'Y'))
      {
         my ($umon, $uday, $uyear, $uhour, $umin) = ($mon, $mday, $year, $hour, $min);
         print "\n";

         $umon  = getTimeValue($umon,  1,    "Enter the month (1-12)",   1,         12,        "");
         $uday  = getTimeValue($uday,  0,    "Enter the day (1-31)",     1,         31,        "");
         $uyear = getTimeValue($uyear, 1900, "Enter the year (YYYY)",    $MIN_YEAR, $MAX_YEAR, "");
         $uhour = getTimeValue($uhour, 0,    "Enter the hour (0-23)",    0,         23,        "");
         $umin  = getTimeValue($umin,  0,    "Enter the minutes (0-59)", 0,         59,        "");

         printf "\n\nYou entered: %02d:%02d %d/%d/%d\n\n", $uhour, $umin, $umon + 1, $uday, $uyear + 1900;
         if (getYNResponse("Do you want to keep this date and time", 'Y'))
         {
            my $date = sprintf "%02d%02d%02d%02d%4d", $umon + 1, $uday, $uhour, $umin, $uyear + 1900;
            unless (setupDateTime($date))
            {
               baselog($LOG_ERROR, "Date set failed\n", $TRUE);
               return $FALSE;
            }
            last;
         } ## end if (getYNResponse("Do you want to keep this date and time"...
         print "\n";
      } ## end if (!getYNResponse("Do you want to keep this date and time"...
      else
      {
         last;
      }
   } ## end while ($TRUE)
   return $TRUE;
} ## end sub configDateTime()

###############################################################################
## @fn       rebootPrompt()
# @brief     Ask user to confirm that system will be rebooted
# @details   None.
# @param     None.
# @return    $TRUE - if user agreed
# @return    $FALSE - if user disagreed
###############################################################################
sub rebootPrompt()
{
   return ((getYNResponse("Do you want to save changes (the system will be rebooted)", 'N')) ? $TRUE : $FALSE);
}

###############################################################################
## @fn       print_network_config($data)
# @brief     print parameters of network configuration
# @details   None.
# @param     $data - hash of parameters
# @return    $TRUE
###############################################################################
sub print_network_config($)
{
   my $data = shift(@_);
   print "\nNetwork configuration:\n\n";
   printf "%35s: $data->{'machineIP'}\n", $paramDescr{'machineIP'};
   printf "%35s: $data->{'netmask'}\n",   $paramDescr{'netmask'};
   printf "%35s: $data->{'gateway'}\n\n", $paramDescr{'gateway'};

   printf "%35s: $data->{'hostname'}\n", $paramDescr{'hostname'};
   unless ($data->{'fqdnTlan'} =~ /localdomain$/)
   {
      printf "%35s: $data->{'fqdnTlan'}\n\n", $paramDescr{'fqdnTlan'};
   }
   else
   {
      printf "%35s: not configured\n\n", $paramDescr{'fqdnTlan'};
   }

   printf "%35s: $data->{'machineIPTlan'}\n", $paramDescr{'machineIPTlan'};
   printf "%35s: $data->{'netmaskTlan'}\n",   $paramDescr{'netmaskTlan'};
   printf "%35s: $data->{'gatewayTlan'}\n\n", $paramDescr{'gatewayTlan'};

   if ($data->{'defaultGateway'} eq $ELAN_GW)
   {
      printf "%35s: $data->{'gateway'}\n", $paramDescr{'defaultGateway'};
   }
   else
   {
      printf "%35s: $data->{'gatewayTlan'}\n", $paramDescr{'defaultGateway'};
   }

   return $TRUE;
} ## end sub print_network_config($)

###############################################################################
## @fn       print_dns_config($data)
# @brief     print parameters of DNS server configuration
# @details   None.
# @param     $data - hash of parameters
# @return    $TRUE
###############################################################################
sub print_dns_config($)
{
   my $data = shift(@_);
   print "\nDNS server configuration:\n\n";
   if ($data->{'primaryDNSIP'} eq $UNCONFIGURED)
   {
      printf "%35s: not configured\n", $paramDescr{'primaryDNSIP'};
   }
   else
   {
      printf "%35s: $data->{'primaryDNSIP'}\n", $paramDescr{'primaryDNSIP'};
   }
   if ($data->{'secondaryDNSIP'} eq $UNCONFIGURED)
   {
      printf "%35s: not configured\n", $paramDescr{'secondaryDNSIP'};
   }
   else
   {
      printf "%35s: $data->{'secondaryDNSIP'}\n", $paramDescr{'secondaryDNSIP'};
   }
   return $TRUE;
} ## end sub print_dns_config($)

###############################################################################
## @fn       print_ntp_config($data)
# @brief     print parameters of NTP server configuration
# @details   None.
# @param     $data - hash of parameters
# @return    $TRUE
###############################################################################
sub print_ntp_config($)
{
   my $data = shift(@_);
   print "\nNTP server configuration:\n\n";
   if ($data->{'ntpSecurityMode'} eq 'Y')
   {
      print "  NTP is configured in secure MD5 transfer mode:\n";
      printf "%35s: $data->{'ntpSecurityModeKeyID'}\n", $paramDescr{'ntpSecurityModeKeyID'};
      printf "%35s: ********\n\n",                      $paramDescr{'ntpSecurityModePriKey'};
   } ## end if ($data->{'ntpSecurityMode'...
   else
   {
      print "  NTP is not configured in secure MD5 transfer mode:\n";
   }

   printf "%35s: $clockSourceTypes[$data->{'clockSourceType'}]\n", $paramDescr{'clockSourceType'};

   if ($data->{'clockType'} == $INTERNAL_CLOCK)
   {
      if ($data->{'clockSourceType'} != $OTHER)
      {
         printf "%35s: Internal (Unreliable)\n", $paramDescr{'clockType'};
      }
   } ## end if ($data->{'clockType'...
   elsif ($data->{'clockType'} == $EXTERNAL_CLOCK)
   {
      if ($data->{'clockSourceType'} != $OTHER)
      {
         printf "%35s: External\n", $paramDescr{'clockType'};
      }
      if (defined($data->{'clockIPs'}))
      {
         my $temp = $data->{'clockIPs'};
         $temp =~ s/\|/, /g;
         printf "%35s: $temp\n", "External IP(s)";
      } ## end if (defined($data->{'clockIPs'...

   } ## end elsif ($data->{'clockType'...
   return $TRUE;
} ## end sub print_ntp_config($)

###############################################################################
## @fn       print_baseparam_config($data)
# @brief     print all parameters of system configuration
# @details   None.
# @param     $data - reference to the hash of parameters
# @return    $TRUE
###############################################################################
sub print_baseparam_config($)
{
   my $data = shift @_;
   print_network_config($data);
   print_dns_config($data);
   print_ntp_config($data);
   print "\nDate, Time and Zone configuration:\n";
   my $tzdescr = getTimezoneDescr(getTimezonesInfo({'zone' => $data->{'timezone'}}));
   printf "%35s: $tzdescr\n", $paramDescr{'timezone'};
   my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
   printf "%35s: %02d:%02d:%02d %d/%d/%d\n\n", "Current Date and Time", $hour, $min, $sec, $mon + 1, $mday, $year + 1900;
   return $TRUE;
} ## end sub print_baseparam_config($)

1;

__END__

=head1 NAME

baseParamsUI - UI API for linux base parameters setting

=head1 SYNOPSIS

use baseParamsUI;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

