## @file base_accounts.pm
# @brief This module contains basic functions required for user management
# @details All these functions are started by sudo_nortel, because most of them
#          requires root privileges. This module allows to add, remove accounts,
#          change password for account, save information about created user in
#          accounts.xml file in directory /var/opt/nortel/base/appconfig/.
# @author Evgeny Lazarev
# @author Alexander Khoroshilov
# @author Vladimir Ivanov
# @author Peter Kascheev
# @date October 2008
# @version 1.00 - Initial version
# @version 1.10 - new variables and functions added:
#   @li $LINUX_FTP_USER - user FTP account for file transfer via FTP. Could not be used
#     for login via ssh.
#   @li $LINUX_SFTP_USER - user SFTP account for file transfer via SFTP. Could not be used
#     for login via ssh.
#   @li $LINUX_FTP_SFTP_USER - User account for both FTP and SFTP transfer.
# @version 1.20 - several changes and fixes:
#   @li Changed handle of the --shell parameter. Now it is a key.
#   @li Umount extended access points of user before user removal.
#   @li For FTP and SFTP users shell is set automatically.
#   @li All users have 'nortel' as an initial groups.
#   @li File locks have been added.
# @version 1.21 - Two colons were removed before shared functions and variables.
# @version 1.25 - The following functions have been added:
#   @li get_user_type
#   @li get_added_usernames
#   @li get_ftp_accounts
#   @li get_accounts_by_type
#   @li get_sftp_accounts
# @version 1.26 - Change sequence of baselog and common_functions
# @version 1.27 - Alexander Khoroshilov:
#   @li added handle of the --exp_date and --nopasswd parameters,
#   @li changed permissions for user profile files in case restricted shell chosen
#   @li  moved is_valid_group function to common_function.pm
# @version 1.28 - Modified under the perlcheck warnings cleanup activity.
# @version 1.30 -
#   @li Protection with semaphore SEM_ACCOUNTS has been added.
#   @li Name of the temporary file for "accounts.xml" is created with PID.
#   @li "use Fcntl ':flock'" has been removed.
# @version 1.31
#   @li Support for new user type 'UCM_User' has been added.
#   @li $ETC_PASSWD moved to common_variables module. "$ETC_PASSWD" -> "$PASSWDFILE"
# @version 1.32 - faillog options are added during user creating.
# @version 1.33 - password expiration date setting is added during user creation
# @version 1.34 - support of the '-quiet' has been added. Checks for username have
#                 been added to the 'modify_user' and 'del_user' functions.
# @version 1.35 - ftp configuration file is removed when user is deleted.
# @version 1.36 - Disable SFTP accounts.
# @version 1.37 - Remove password expiry and lockout policies for UCM users
#                 These policies are to be controlled from UCM only

package base_accounts;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;
use install_common;
use baselog;
use ifxml;
use CommonInstallUtils;
use XML::Writer;
use IO::File;
use recovery::backup;
use base_system;
use base_ftp;
use base_semctl;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT =
  qw(&add_user &add_users &del_users &append_user_to_accounts_xml &get_added_users &del_user
  &change_pwd &delete_user_from_accounts_xml &get_user_type &get_added_usernames &get_ftp_accounts
  &get_accounts_by_type &get_sftp_accounts &is_ftp_account
  $LINUX_USER $LINUX_FTP_USER $LINUX_SFTP_USER $LINUX_FTP_SFTP_USER $UCM_USER
  $ADD_USER $DEL_USER $ADD_USERS $DEL_ALL_USERS $MODIFY_USER $ALL_USERS $BASE_USERS $NORTEL_USERS
  );

our $VERSION = '1.36';

# user's account types
our $LINUX_USER          = 'Linux_User';
our $LINUX_FTP_USER      = 'Linux_FTP_User';
our $LINUX_SFTP_USER     = 'Linux_SFTP_User';
our $LINUX_FTP_SFTP_USER = 'Linux_FTP_SFTP_User';
our $UCM_USER            = 'UCM_User';

# minimal user account UID
my $MIN_USER_UID = 500;

# time to wait for semaphore
my $TIME_TO_WAIT = 5;

# nologin shell
my $NOLOGIN_SHELL = '/sbin/nologin';

# list of profile files
my @PROFILE_FILES = ('.bash_profile', '.bashrc');

# Applications types
our $ALL_USERS    = 'AllUsers';
our $BASE_USERS   = 'BaseUsers';
our $NORTEL_USERS = 'NortelUsers';

# arguments for userconfig.pl
our $ADD_USER    = 'adduser';
our $ADD_USERS   = 'add_users';
our $DEL_USER    = 'deluser';
our $MODIFY_USER = 'modifyuser';

# Public functions
## @skipline
sub add_user($);
## @skipline
sub add_users($);
## @skipline
sub del_users($);
## @skipline
sub del_user($;$);
## @skipline
sub delete_user_from_accounts_xml($);
## @skipline
sub get_accounts_by_type($);
## @skipline
sub get_sftp_accounts();
## @skipline
sub get_ftp_accounts();
## @skipline
sub is_empty_group($);
## @skipline
sub modify_user($);

# Private functions
## @skipline
sub append_user_to_accounts_xml($);
## @skipline
sub do_add_user($);
## @skipline
sub do_del_user($);
## @skipline
sub get_added_usernames($);
## @skipline
sub get_added_users($$);
## @skipline
sub get_user_type($);
## @skipline
sub is_shared_user($);
## @skipline
sub is_system_user($);
## @skipline
sub make_command($$);
##### Public functions #####

## @fn sub add_user($user)
# @brief This function is a wrapper for adding users' accounts. It runs
#        different function in order to create Linux accounts.
# @details Before adding user it checks user for existence, if it is not
#          existed, then call function "do_add_user", which adds account and
#          calls function append_user_to_accounts_xml,which updates accounts.xml
# @param $user - hash reference to information about user. It can contain such
#                keys:
#   @li username - new user's name;
#   @li pwd - new user's password (hash, not plain text!). If password is
#           not specified and no --nopasswd used, when user is prompted to enter it;
#   @li nopasswd - crate user with blank password;
#   @li initial_group - initial user's group;
#   @li supp_group - list of supplementary groups (separated by commas);
#   @li shell - if this option is defined then restricted bash shell is used, otherwise /sbin/nologin.
#   @li home - home directory;
#   @li type - user's type.
#   @li app_name - application's name, which creates this user;
#   @li uid - user's UID;
#   @li exp_date - the date on which the user account will be disabled.
#           The date is specified in the format YYYY-MM-DD
#   @li quiet - enable quiet mode;
# @return $TRUE  - if user(and all required groups) has been added successfully
# @return $FALSE - some errors happened
# @author Evgeny Lazarev
# @date May 2008
sub add_user($)
{
   my $user    = shift @_;
   my $retval  = $TRUE;
   my $verbose = $TRUE;
   if ($user->{'quiet'})
   {
      $verbose = $FALSE;
   }

   if (!$user->{'username'})
   {
      baselog($LOG_ERROR, "Username is not specified.\n", $verbose);
      return $FALSE;
   }
   if ((defined $user->{'app_name'}) && ($user->{'app_name'}))
   {
      if (!is_app_installed($user->{'app_name'}))
      {
         baselog($LOG_ERROR, "Application '$user->{'app_name'}' is not installed.\n", $verbose);
         return $FALSE;
      }
   } ## end if ((defined $user->{'app_name'...
   if (!defined($user->{'app_name'}))
   {
      $user->{'app_name'} = $NORTEL_LINUX_BASE_APP_NAME;
   }
   if (!defined($user->{'initial_group'}))
   {
      baselog($LOG_INFO, "User's group is not specified. '$NORTEL_GRP' is set by default.", $verbose);
      $user->{'initial_group'} = $NORTEL_GRP;
   }
   if (!defined($user->{'manual'}))
   {
      $user->{'manual'} = $FALSE;
   }

   # User types list will be extended with insecure accounts later to exclude
   # user auto creation during application installation.
   if ((!defined $user->{'type'}) || (!$user->{'type'}))
   {
      $user->{'type'} = $LINUX_FTP_SFTP_USER;
   }
   if (($user->{'type'} eq $LINUX_USER) || ($user->{'type'} eq $LINUX_FTP_USER) || ($user->{'type'} eq $LINUX_SFTP_USER) || ($user->{'type'} eq $LINUX_FTP_SFTP_USER) || ($user->{'type'} eq $UCM_USER))
   {
      if ($user->{'username'} =~ /^NT_S_*/)
      {

         #Disable NT_S_* user
         $user->{'pwd'}      = "";
         $user->{'nopasswd'} = 1;
      } ## end if ($user->{'username'...
      if ($user->{'type'} eq $LINUX_FTP_USER)
      {
         $user->{'shell'} = 0;
      }
      elsif (($user->{'type'} eq $LINUX_SFTP_USER) || ($user->{'type'} eq $LINUX_FTP_SFTP_USER) || ($user->{'type'} eq $UCM_USER))
      {
         $user->{'shell'} = 1;
      }
      if (is_user_exist($user->{'username'}))
      {
         baselog($LOG_INFO, "User '$user->{'username'}' is already exists.\n", $verbose);
         $retval = $TRUE;
      }
      else
      {
         if (defined($user->{'uid'}))
         {
            if ($user->{'uid'} < $MIN_USER_UID)
            {
               if (defined($user->{'shell'}) && ($user->{'shell'}))
               {
                  baselog($LOG_ERROR, "It is prohibited to create a login system user.\n", $verbose);
                  $retval = $FALSE;
               }
            } ## end if ($user->{'uid'} < $MIN_USER_UID...
         } ## end if (defined($user->{'uid'...
         my (@list_of_groups, @tmp_list) = (($user->{'initial_group'}), ());
         if (defined($user->{'supp_group'}))
         {
            push @list_of_groups, split(/,/, $user->{'supp_group'});
         }
         for my $grp (@list_of_groups)
         {
            unless (is_valid_group($grp))
            {
               if ($grp =~ /^\d+$/)
               {

                  # it is group id
                  my @lst = sort values %GROUPS;
                  baselog($LOG_ERROR, "Group id $grp is not in the list of the group ids: @lst.\n", $verbose);
                  $retval = $FALSE;
               } ## end if ($grp =~ /^\d+$/)
               else
               {

                  # it is group
                  my @lst = keys %GROUPS;
                  baselog($LOG_ERROR, "Group $grp is not in the list of the groups: @lst.\n", $verbose);
                  $retval = $FALSE;
               } ## end else [ if ($grp =~ /^\d+$/)
            } ## end unless (is_valid_group($grp...

            if (($grp ne $NORTEL_GRP) && ($grp ne $NORTEL_GID))
            {
               push @tmp_list, $grp;
            }
         } ## end for my $grp (@list_of_groups...
           # set initial group to 'nortel'. Set other specified groups to supplementary groups
         if (scalar(@tmp_list) != 0)
         {
            $user->{'supp_group'} = join(',', @tmp_list);
         }
         else
         {
            $user->{'supp_group'} = undef;
         }
         $user->{'initial_group'} = $NORTEL_GID;
         if (($retval != $FALSE) && do_add_user($user))
         {
            if (!append_user_to_accounts_xml($user))
            {
               baselog($LOG_ERROR, "Error during updating accounts configuration file.\nUser will be deleted.", $verbose);
               do_del_user($user->{'username'});
               $retval = $FALSE;
            } ## end if (!append_user_to_accounts_xml...
            else
            {

               # mount additional directories required for UCM users
               if ($user->{'type'} eq $UCM_USER)
               {
                  base_ftp::mount_sftp_jail_dirs([$user->{'username'}]);
                  base_ftp::mount_UCM_dirs([$user->{'username'}]);
               }

               # mount sftp-jail directories for manually created users (using 'userconfig.pl') which use SFTP
               if ((($user->{'type'} eq $LINUX_SFTP_USER) || ($user->{'type'} eq $LINUX_FTP_SFTP_USER)) && ($user->{'manual'}))
               {
                  base_ftp::mount_sftp_jail_dirs([$user->{'username'}]);
               }

            } ## end else [ if (!append_user_to_accounts_xml...
            baselog($LOG_INFO, "User '$user->{'username'}' added successfully.\n", $verbose);
         } ## end if (($retval != $FALSE...
         else
         {
            $retval = $FALSE;
         }
      } ## end else [ if (is_user_exist($user...
   } ## end if (($user->{'type'} eq...
   else
   {
      baselog($LOG_ERROR, "Unknown type '$user->{'type'}' of user '$user->{'username'}'.\n", $verbose);
      $retval = $FALSE;
   }

   if ($retval)
   {

      #change faillog options for accounts
      if ($user->{'type'} =~ /^($LINUX_FTP_USER|$LINUX_SFTP_USER|$LINUX_FTP_SFTP_USER|$UCM_USER)$/)
      {
         if (not do_system([$FAILLOG, '-u', $user->{'username'}, '-m', '0']))
         {
            baselog($LOG_ERROR, "Can not setup login limit to unlimited for " . $user->{'username'} . " user");
            $retval = $FALSE;
         }
         if (not do_system([$LCHAGE, '-M', $PASS_UNLIM_VALUE, $user->{'username'}]))
         {
            baselog($LOG_ERROR, "Applying password expiration date for account $user->{'username'} falied.");
            $retval = $FALSE;
         }

      } ## end if ($user->{'type'} =~...
      else
      {
         if (not do_system([$FAILLOG, '-u', $user->{'username'}, '-m', '3']))
         {
            baselog($LOG_ERROR, "Can not setup login limit for " . $user->{'username'} . " user");
            $retval = $FALSE;
         }

      } ## end else [ if ($user->{'type'} =~...
   } ## end if ($retval)

   return $retval;
} ## end sub add_user($)

## @fn sub add_users($)
# @brief Adds users, defined in configure.xml files.
# @param $type - type of the users: could be $NORTEL_USERS, $BASE_USERS or $ALL_USERS
# @return $TRUE  - all users added successfully.
# @return $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date November 2007
sub add_users($)
{
   my $type         = shift @_;
   my $users_inform = undef;
   my $retval       = $TRUE;
   my $parser       = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   my $app_list     = ();

   unless ($type)
   {
      baselog($LOG_ERROR, "Users type is not specified.");
      return $FALSE;
   }
   if ($type eq $BASE_USERS)
   {
      $app_list = getBaseApplicationsList();
   }
   elsif ($type eq $NORTEL_USERS)
   {
      $app_list = getNortelApplicationsList();
   }
   elsif ($type eq $ALL_USERS)
   {
      $app_list = [@{getBaseApplicationsList()}, @{getNortelApplicationsList()}];
   }
   else
   {
      baselog($LOG_ERROR, "Undefined type of the users: '$type'");
      return $FALSE;
   }
   unless ($app_list)
   {
      baselog($LOG_INFO, "There is no installed applications of type '$type'.");
      return $TRUE;
   }
   for my $app (@$app_list)
   {
      $users_inform = $parser->find_data_by_xpath($GETDATA, '/application/addusers/user', $NOATTR, {name => $app});
      if (!defined $users_inform || !scalar @{$users_inform})
      {
         baselog($LOG_DEBUG, "There is no 'addusers' section in $APPLCFG.");
      }
      else
      {
         for my $user (@$users_inform)
         {
            $user->{'app_name'} = $app;
            if (!add_user($user))
            {
               $retval = $FALSE;
            }
         } ## end for my $user (@$users_inform...
      } ## end else [ if (!defined $users_inform...
   } ## end for my $app (@$app_list...
   return $retval;
} ## end sub add_users($)

## @fn sub modify_user($user)
# @brief Modifies parameters of the specified users.
# @details This function changes parameters for specified account, which is
#          present in accounts.xml file.
# @param $user - hash reference to information about user.
# @return $TRUE  - user's option changed successfully.
# @return $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date November 2007
sub modify_user($)
{
   my $user           = shift @_;
   my $retval         = $TRUE;
   my @out            = ();
   my @list_of_groups = ();
   my $cmd            = undef;
   my $verbose        = $TRUE;
   if ($user->{'quiet'})
   {
      $verbose = $FALSE;
   }
   if (!$user->{'username'})
   {
      baselog($LOG_ERROR, "Username is not specified.\n", $verbose);
      return $FALSE;
   }

   push @list_of_groups, $user->{'initial_group'};
   if ((defined $user->{'supp_group'}) && $user->{'supp_group'})
   {
      push @list_of_groups, split(/,/, $user->{'supp_group'});  # add supplementary groups to list of groups
   }

   $cmd = make_command($USERMOD, $user);
   unless ($cmd)
   {
      baselog($LOG_ERROR, "Can not create command.");
      return $FALSE;
   }

   if (!(defined $user->{'app_name'}) || !$user->{'app_name'})
   {
      $user->{'app_name'} = $NORTEL_LINUX_BASE_APP_NAME;
   }
   else
   {
      if (!is_app_installed($user->{'app_name'}))
      {
         baselog($LOG_ERROR, "Application '$user->{'app_name'}' is not installed.\n", $verbose);
         return $FALSE;
      }
   } ## end else [ if (!(defined $user->{...

   if (!defined($user->{'username'}) || !$user->{'username'})
   {
      baselog($LOG_ERROR, "User's name is not specified.\n", $verbose);
      $retval = $FALSE;
   }
   elsif (is_user_exist($user->{'username'}))
   {
      if (!is_system_user($user->{'username'}))
      {
         if (scalar(@$cmd) > 2)
         {
            my $result = do_system($cmd, {$dsOUT => \@out});
            baselog($LOG_DEBUG, "The following command is executed: @$cmd.");
            $$cmd[4] = "<some password>";
            baselog($LOG_INFO, "Modifying user account with the following command: @$cmd");

            # if some system user's parameters are changed (not only app_name)
            if ($result)
            {
               $retval = append_user_to_accounts_xml($user);
               baselog($LOG_INFO, "Properties for user '$user->{'username'}' changed successfully\n", $verbose);
            }
            else
            {
               baselog($LOG_ERROR, "Can not change properties for user '$user->{'username'}': @out.\n", $verbose);
               $retval = $FALSE;
            }
         } ## end if (scalar(@$cmd) > 2)
         else
         {

            # only application's name is changed
            $retval = append_user_to_accounts_xml($user);
            baselog($LOG_INFO, "Properties for user '$user->{'username'}' changed successfully\n", $verbose);
         } ## end else [ if (scalar(@$cmd) > 2)
      } ## end if (!is_system_user($user...
      else
      {
         baselog($LOG_ERROR, "Can not modify parameters for '$user->{'username'}': it is a system user.\n", $verbose);
         $retval = $FALSE;
      }
   } ## end elsif (is_user_exist($user...
   else
   {
      baselog($LOG_ERROR, "Can not modify parameters for '$user->{'username'}': such user does not exist.\n", $verbose);
      $retval = $FALSE;
   }
   return $retval;
} ## end sub modify_user($)

## @fn sub del_all_users($mode)
# @brief Deletes all users, defined in accounts.xml file.
# @details Also delete accounts.xml file.
# @param $type - type of the users: could be $NORTEL_USERS, $BASE_USERS or $ALL_USERS
# @return $TRUE  - all users successfully deleted.
# @return $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date May 2008
sub del_users($)
{
   my $type         = shift @_;
   my $users_inform = undef;
   my $retval       = $TRUE;
   my $flag         = $FALSE;

   unless ($type)
   {
      baselog($LOG_ERROR, "User's type is not specified.");
      return $FALSE;
   }
   if (($type ne $ALL_USERS) && ($type ne $BASE_USERS) && ($type ne $NORTEL_USERS))
   {
      baselog($LOG_INFO, "Undefined type of users: '$type'.");
      return $FALSE;
   }

   $users_inform = get_added_users("", $type);
   if ((!defined $users_inform) || !(scalar @{$users_inform}))
   {
      baselog($LOG_INFO, "'addusers' section in $ACCOUNTS_CFG is empty");
      @{$users_inform} = ();
   }
   for my $user (@$users_inform)
   {
      if (defined $user)
      {
         if (!del_user($user->{'username'}))
         {
            $retval = $FALSE;
         }
      } ## end if (defined $user)
   } ## end for my $user (@$users_inform...
   if (($type eq $ALL_USERS) && (-f "$SYSDB_ROOT/$ACCOUNTS_CFG"))
   {
      if (unlink "$SYSDB_ROOT/$ACCOUNTS_CFG")
      {
         baselog($LOG_INFO, "$ACCOUNTS_CFG file successfully removed.");
      }
      else
      {
         baselog($LOG_ERROR, "Can not remove $ACCOUNTS_CFG file.\n");
      }
   } ## end if (($type eq $ALL_USERS...
   return $retval;
} ## end sub del_users($)

## @fn sub del_user($user_name, $quiet)
# @brief This function is a wrapper for deletion users' accounts.
# @details It performs check for user existence before deletion.
# @param $user_name - user name for deletion.
# @param $quiet - enable quiet mode. By default it is disabled.
# @return $TRUE  - if user has been deleted successfully
# @return $FALSE - some errors happened
# @author Evgeny Lazarev
# @date May 2008
sub del_user($;$)
{
   my ($user_name, $quiet) = (@_);
   my $retval  = $TRUE;
   my $verbose = $TRUE;
   if ($quiet)
   {
      $verbose = $FALSE;
   }
   if (!$user_name)
   {
      baselog($LOG_ERROR, "Username is not specified.\n", $verbose);
      return $FALSE;
   }

   if ((!defined $user_name) || (!$user_name))
   {
      baselog($LOG_ERROR, "Username is not specified\n", $verbose);
      $retval = $FALSE;
   }
   if (is_user_exist($user_name))
   {
      if (is_system_user($user_name))
      {
         baselog($LOG_ERROR, "Can not delete '$user_name': this is a system user.\n", $verbose);
      }
      elsif (!is_shared_user($user_name))
      {
         if (do_del_user($user_name))
         {
            delete_user_from_accounts_xml($user_name);
            baselog($LOG_INFO, "User '$user_name' deleted successfully.\n", $verbose);
         }
         else
         {
            baselog($LOG_ERROR, "Can not delete user '$user_name'\n", $verbose);
            $retval = $FALSE;
         }
      } ## end elsif (!is_shared_user($user_name...
      else
      {
         baselog($LOG_INFO, "Can not delete user '$user_name': it is used by several applications.\n", $verbose);
         $retval = $FALSE;
      }
   } ## end if (is_user_exist($user_name...
   else
   {
      baselog($LOG_ERROR, "Can not delete user '$user_name': such user does not exist.\n", $verbose);
      $retval = $FALSE;
   }
   return $retval;
} ## end sub del_user($;$)

##### Private functions #####

## @fn sub append_user_to_accounts_xml($user_data)
# @brief Updates accounts.xml file.
# @details This function adds information about created user, or update
#          information for already existed user in accounts.xml file.
# @param $user_data - hash reference to information about user.
# @return $TRUE  - if information about user is successfully added to accounts.xml.
# @return $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date November 2007
sub append_user_to_accounts_xml($)
{
   my $user_data = shift @_;
   my $output_fd;
   my $users_inform = undef;
   my $writer       = undef;
   my $retval       = $TRUE;

   my $semmngr = base_semctl::new($base_semctl::SBS_USERCONFIG);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }

   if (!$semmngr->sem_take([$SEM_ACCOUNTS], $WAIT_FLAG, $TIME_TO_WAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (!-f "$SYSDB_ROOT/$ACCOUNTS_CFG")
   {
      baselog($LOG_INFO, "$ACCOUNTS_CFG does not exist. It will be created.");
   }
   else
   {
      $users_inform = get_added_users("", $ALL_USERS);
      if ((!defined $users_inform) || (!scalar @{$users_inform}))
      {
         baselog($LOG_INFO, "'addusers' section in $ACCOUNTS_CFG is empty");
         @{$users_inform} = ();
      }
   } ## end else [ if (!-f "$SYSDB_ROOT/$ACCOUNTS_CFG"...

   my $tmp_filename = "$SYSDB_ROOT/$ACCOUNTS_CFG.$$";
   $output_fd = new IO::File(">$tmp_filename");
   $writer = new XML::Writer(DATA_MODE => 'true', OUTPUT => $output_fd, DATA_INDENT => 3);
   $writer->startTag("addusers");

   # $ref - reference to hash which describe user with name $user_data->{'username'}.
   my $ref = undef;
   if (defined($users_inform))
   {
      for (my $ind = 0 ; $ind < scalar(@$users_inform) ; $ind++)
      {

         # try to find description for user in accounts.xml and store it in $ref
         if ($users_inform->[$ind]->{'username'} eq $user_data->{'username'})
         {
            $ref = $users_inform->[$ind];
            last;
         }
      } ## end for (my $ind = 0 ; $ind...
   } ## end if (defined($users_inform...
   else
   {
      $users_inform->[0] = $ref;
   }
   if (!defined($ref))
   {

      # if user does not present in accounts.xml then add information about it
      push @$users_inform, $user_data;
   } ## end if (!defined($ref))
   else
   {

      # update information for existed user
      for my $key (keys(%$user_data))
      {
         if (defined $user_data->{$key})
         {
            $ref->{$key} = $user_data->{$key};
         }
      } ## end for my $key (keys(%$user_data...
   } ## end else [ if (!defined($ref))
   for my $user (@$users_inform)
   {
      if (defined $user)
      {
         $writer->emptyTag('user', %$user);
      }
   } ## end for my $user (@$users_inform...
   $writer->endTag("addusers");
   $writer->end();
   $output_fd->close();
   unless (do_system([$MV, $tmp_filename, "$SYSDB_ROOT/$ACCOUNTS_CFG"]))
   {
      print "Can not modify account's configuration file.\n";
      $retval = $FALSE;
   }
   else
   {
      if (!do_system([$CHMOD, "--reference=$PASSWDFILE", "$SYSDB_ROOT/$ACCOUNTS_CFG"]))
      {
         baselog($LOG_ERROR, "Can not change file permissions.");
      }
   } ## end else
   if (!$semmngr->sem_give($SEM_ACCOUNTS))
   {
      baselog($LOG_ERROR, "Can not release semaphore.");
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_give...
   undef $semmngr;

   return $retval;
} ## end sub append_user_to_accounts_xml($)

## @fn sub delete_user_from_accounts_xml($user_name)
# @brief Delete user from accounts.xml.
# @details This function deletes information about user from accounts.xm
# @param $user_name - user's name for deletion from accounts.xml.
# @return $TRUE  - information successfully deleted.
# @return $FALSE - some errors happened
# @author Evgeny Lazarev
# @date November 2007
sub delete_user_from_accounts_xml($)
{
   my $user_name = shift @_;
   my $output_fd;
   my $users_inform = undef;
   my $writer       = undef;
   my $retval       = $TRUE;

   my $semmngr = base_semctl::new($base_semctl::SBS_USERCONFIG);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take([$SEM_ACCOUNTS], $WAIT_FLAG, $TIME_TO_WAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (-f "$SYSDB_ROOT/$ACCOUNTS_CFG")
   {
      $users_inform = get_added_users("", $ALL_USERS);
      if ((!defined $users_inform) || (!scalar @{$users_inform}))
      {
         baselog($LOG_INFO, "'addusers' section in $ACCOUNTS_CFG is empty.");
         @{$users_inform} = ();
      }
   } ## end if (-f "$SYSDB_ROOT/$ACCOUNTS_CFG"...

   my $tmp_filename = "$SYSDB_ROOT/$ACCOUNTS_CFG.$$";
   if (defined($users_inform))
   {
      $output_fd = new IO::File(">$tmp_filename");
      $writer = new XML::Writer(DATA_MODE => 'true', OUTPUT => $output_fd, DATA_INDENT => 3);
      $writer->startTag("addusers");

      # find and delete reference to hash which describe receding user
      for (my $ind = 0 ; $ind < scalar(@$users_inform) ; $ind++)
      {

         # try to find description for user in accounts.xml
         if ($users_inform->[$ind]->{'username'} eq $user_name)
         {
            $users_inform->[$ind] = $users_inform->[scalar(@$users_inform) - 1];
            pop @$users_inform;
            last;
         } ## end if ($users_inform->[$ind...
      } ## end for (my $ind = 0 ; $ind...
      for my $user (@$users_inform)
      {
         if (defined $user)
         {
            $writer->emptyTag('user', %$user);
         }
      } ## end for my $user (@$users_inform...
      $writer->endTag("addusers");
      $writer->end();
      $output_fd->close();
   } ## end if (defined($users_inform...

   if (!do_system([$MV, $tmp_filename, "$SYSDB_ROOT/$ACCOUNTS_CFG"]))
   {
      print "Can not modify account's configuration file.\n";
      $retval = $FALSE;
   }
   else
   {
      if (!do_system([$CHMOD, "--reference=$PASSWDFILE", "$SYSDB_ROOT/$ACCOUNTS_CFG"]))
      {
         baselog($LOG_ERROR, "Can not change file permissions.");
      }
   } ## end else [ if (!do_system([$MV, $tmp_filename...
   if (!$semmngr->sem_give($SEM_ACCOUNTS))
   {
      baselog($LOG_ERROR, "Can not release semaphore.");
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_give...
   undef $semmngr;
   return $retval;
} ## end sub delete_user_from_accounts_xml($)

## @fn sub do_add_user($user)
# @brief Adds new user.
# @details this function actually creates new Linux account with specified
#          parameters.
# @param $user - hash reference to information about user.
# @return $TRUE  - if user(and all required groups) has been added successfully
# @return $FALSE - some errors happened
# @author Evgeny Lazarev
# @date November 2007
sub do_add_user($)
{
   my $user           = shift @_;
   my $retval         = $TRUE;
   my $cmd            = undef;
   my @list_of_groups = ();
   my @out            = ();
   my $verbose        = $TRUE;
   if ($user->{'quiet'})
   {
      $verbose = $FALSE;
   }

   push @list_of_groups, $user->{'initial_group'};
   if ((defined $user->{'supp_group'}) && $user->{'supp_group'})
   {
      push @list_of_groups, split(/,/, $user->{'supp_group'});  # add supplementary groups to list of groups
   }

   $cmd = make_command($USERADD, $user);
   unless ($cmd)
   {
      baselog($LOG_ERROR, "Can not create command.");
      return $FALSE;
   }
   my $result = do_system($cmd, {$dsOUT => \@out});
   $$cmd[4] = "<some password>";
   baselog($LOG_INFO, "Adding new user account with the following command: @$cmd");
   if ($result)
   {

      # Change permissions for user profile files in case restricted shell chosen
      # in order to prevent breaking out of restricted shell
      if ((defined $user->{'shell'}) && ($user->{'shell'}))
      {
         my $home = (getpwnam($user->{'username'}))[7];
         if (-d $home)
         {
            for my $file (@PROFILE_FILES)
            {
               if (do_system([$TOUCH, "$home/$file"]))
               {
                  if (change_permissions_and_ownership(0640, $ROOT_UID, $user->{'initial_group'}, ["$home/$file"]))
                  {
                     baselog($LOG_DEBUG, "Permissions for $home/$file is set.");
                  }
                  else
                  {
                     baselog($LOG_ERROR, "Can not change permissions for $home/$file.");
                  }
               } ## end if (do_system([$TOUCH,...
               else
               {
                  baselog($LOG_ERROR, "Can not create $home/$file file.");
               }
            } ## end for my $file (@PROFILE_FILES...
         } ## end if (-d $home)
         else
         {
            baselog($LOG_ERROR, "Directory $home is absent.");
         }

         my $bashrc;
         if (open($bashrc, ">>", "$home/$PROFILE_FILES[1]"))
         {
            print $bashrc "unalias vi which 2>/dev/null\n";
         }
         else
         {
            baselog($LOG_ERROR, "Cannot open $home/$PROFILE_FILES[1] for writing.");
         }
         unless (close $bashrc)
         {
            baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         }
      } ## end if ((defined $user->{'shell'...

      if ((!(defined $user->{'pwd'}) || !$user->{'pwd'}) && !(defined $user->{'nopasswd'}))
      {
         if (!do_system([$PASSWD, $user->{'username'}], {$dsOUT => \@out, $dsINTERACT => $TRUE}))
         {
            baselog($LOG_ERROR, "Can not change '$user->{'username'}' password: @out.");
            print "User is not created.\n";
            do_del_user($user->{'username'});
            $retval = $FALSE;
         } ## end if (!do_system([$PASSWD...
         else
         {
            my (undef, $passwd) = getpwnam($user->{'username'});

            # add password to the account.xml file
            append_user_to_accounts_xml({'username' => $user->{'username'}, 'pwd' => $passwd});
         } ## end else [ if (!do_system([$PASSWD...
      } ## end if ((!(defined $user->...
   } ## end if ($result)
   else
   {
      print @out;
      baselog($LOG_ERROR, "Can not create user '$user->{'username'}'.\n", $verbose);
      $retval = $FALSE;
   } ## end else [ if ($result)
   return $retval;
} ## end sub do_add_user($)

## @fn sub do_del_user($user_name)
# @brief This function actually deletes specified user
# @details User's home directory is also deleted.
# @param $user_name - user's name for deletion.
# @return $TRUE  - user has been successfully deleted
# @return $FALSE - user has not been deleted.
# @author Evgeny Lazarev
# @date November 2007
sub do_del_user($)
{
   my $user_name = shift @_;
   my $retval    = $TRUE;
   my @out       = ();
   my $ret       = 0;
   my $home      = (getpwnam($user_name))[7];
   my $usertype  = get_user_type($user_name);

   if ($usertype eq $LINUX_FTP_USER)
   {
      base_ftp::deactivate_ftp_mounts([$user_name]);
      base_ftp::remove_user_ftp_config_file($user_name);
   }
   if (($usertype eq $LINUX_SFTP_USER) || ($usertype eq $LINUX_FTP_SFTP_USER))
   {
      base_ftp::umount_sftp_jail_dirs([$user_name]);
      base_ftp::deactivate_ftp_mounts([$user_name]);
   }
   if ($usertype eq $UCM_USER)
   {
      base_ftp::umount_sftp_jail_dirs([$user_name]);
      base_ftp::umount_UCM_dirs([$user_name]);
   }
   if (!do_system([$USERDEL, '-r', $user_name], {$dsERR => \@out}))
   {
      baselog($LOG_ERROR, "$USERDEL can not delete user '$user_name': @out.");
      $retval = $FALSE;
   }
   return $retval;
} ## end sub do_del_user($)

## @fn sub get_added_users($)
# @brief Get information about users from accounts.xml.
# @details Returns the reference to array of hashes with information about users
# @param $directory - directory there accounts.xml is situated. If this parameter is
#                empty (i.e. is equal ''), then directory is $SYSDB_ROOT;
# @param $type - type of users ($NORTEL_USERS, $BASE_USERS, $ALL_USERS)
# @return $result - reference to array of hashes with information about users
# @author Evgeny Lazarev
# @date June 2008
sub get_added_users($$)
{
   my ($directory, $type) = (@_);
   my $users_inform = undef;
   my $result       = undef;
   if (!$directory)
   {
      $directory = $SYSDB_ROOT;
   }
   unless ($type)
   {
      baselog($LOG_ERROR, "Users type is not specified.");
      return undef;
   }
   my $parser = ifxml::new($directory, $ACCOUNTS_CFG);
   return undef if (!$parser);

   $users_inform = $parser->find_data_by_xpath($GETDATA, '/addusers/user');
   if ($type ne $ALL_USERS)
   {
      for my $user (@$users_inform)
      {
         if (   ((isBaseApplication($user->{'app_name'}) || ($user->{'app_name'} eq $NORTEL_LINUX_BASE_APP_NAME)) && ($type eq $BASE_USERS))
             || ((!isBaseApplication($user->{'app_name'}) && ($user->{'app_name'} ne $NORTEL_LINUX_BASE_APP_NAME)) && ($type eq $NORTEL_USERS)))
         {
            push @$result, $user;
         }
      } ## end for my $user (@$users_inform...
   } ## end if ($type ne $ALL_USERS...
   else
   {
      return $users_inform;
   }
   return $result;
} ## end sub get_added_users($$)

## @fn sub get_added_usernames($)
# @brief Get list of the usernames defined in accounts.xml.
# @param $type - type of users ($NORTEL_USERS, $BASE_USERS, $ALL_USERS)
# @return $result - reference to array with names of users
# @return undef - user's type is not specified
# @author Evgeny Lazarev
# @date September 2008
sub get_added_usernames($)
{
   my $type = shift @_;
   unless ($type)
   {
      baselog($LOG_ERROR, "User's type is not specified.");
      return undef;
   }
   my $users_inform = get_added_users("", $type);
   my $result       = [];

   if (is_array($users_inform))
   {
      for my $user (@$users_inform)
      {
         push @$result, $user->{'username'};
      }
   } ## end if (is_array($users_inform...
   else
   {
      baselog($LOG_WARNING, "Hash with information about users is not defined.");
   }
   return $result;
} ## end sub get_added_usernames($)

## @fn get_sftp_accounts()
# @brief returns reference to list names of Linux_SFTP_User and
#        Linux_FTP_SFTP_User users
# @return reference to list
# @author Evgeny Lazarev
# @date September 2008
sub get_sftp_accounts()
{
   return [@{get_accounts_by_type($LINUX_SFTP_USER)}, @{get_accounts_by_type($LINUX_FTP_SFTP_USER)}, @{get_accounts_by_type($UCM_USER)}];
}

## @fn get_ftp_accounts()
# @brief returns reference to list names of Linux_FTP_User and
#        Linux_FTP_SFTP_User users
# @return reference to list
# @author Evgeny Lazarev
# @date September 2008
sub get_ftp_accounts()
{
   return [@{get_accounts_by_type($LINUX_FTP_USER)}, @{get_accounts_by_type($LINUX_FTP_SFTP_USER)}];
}

## @fn sub is_shared_user($check_name)
# @brief Checks if specified user is used by several applications.
# @details This function checks if two or more configure.xml contain sections
#          "addusers" with tag "user" with value of an attribute "username"
#          equal to $check_name.
# @param $check_name - user's name for check.
# @return $TRUE  - specified user is used by more than one application
# @return $FALSE - specified user is used by single application
# @author Evgeny Lazarev
# @date April 2007
sub is_shared_user($)
{
   my $check_name   = shift @_;
   my $users_inform = undef;
   my $counter      = 0;
   my $retval       = $FALSE;
   my $parser       = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   return $FALSE if (!$parser);

   $users_inform = $parser->find_data_by_xpath($GETDATA, "/addusers/user");
   undef $parser;
   if ((defined $users_inform) && (scalar @{$users_inform}))
   {
      foreach my $h_ref (@$users_inform)
      {
         if ((defined $h_ref->{'username'}) && ($h_ref->{'username'} eq $check_name))
         {
            $counter++;
         }
      } ## end foreach my $h_ref (@$users_inform...
   } ## end if ((defined $users_inform...
   return $counter > 1;
} ## end sub is_shared_user($)

## @fn sub is_system_user($user_name)
# @brief Checks if user is a system.
# @details This function checks accounts.xml file. If it does not contain
#          information about user $user_name, then this user is considered
#          a system account.
# @param $user_name - user's name for check.
# @return
#   - $TRUE  - if specified user is a system account;
#   - $FALSE - otherwise.
# @author Evgeny Lazarev
# @date November 2007
sub is_system_user($)
{
   my $user_name          = shift @_;
   my $added_users_inform = get_added_users("", $ALL_USERS);
   my $retval             = $TRUE;

   if (!defined($user_name) || !$user_name)
   {
      baselog($LOG_ERROR, "User's name is not specified.\n");
      $retval = $FALSE;
   }
   else
   {
      if (defined($added_users_inform))
      {
         for my $user_info (@$added_users_inform)
         {
            if ($user_name eq $user_info->{'username'})
            {
               $retval = $FALSE;
               last;
            }
         } ## end for my $user_info (@$added_users_inform...
      } ## end if (defined($added_users_inform...
   } ## end else [ if (!defined($user_name...
   return $retval;
} ## end sub is_system_user($)

## @fn sub make_command($command, $user)
# @brief Create array with parameters that can be directly executed by the do_system (modify and add new user).
# @param $command - name of the command to execute
# @param $user - a reference to the hash with user's parameters
# @return $TRUE  - command successfully created
# @return undef - some error happened
# @author Evgeny Lazarev
# @date June 2008
sub make_command($$)
{
   my ($command, $user) = (@_);
   my @cmd = ();
   unless ($command)
   {
      baselog($LOG_ERROR, "Command is not defined.");
      return undef;
   }
   unless ($user)
   {
      baselog($LOG_ERROR, "User is not defined.");
      return undef;
   }

   push @cmd, $command;
   if ((defined $user->{'initial_group'}) && $user->{'initial_group'})
   {
      push @cmd, '-g', $user->{'initial_group'};
   }
   if ((defined $user->{'supp_group'}) && $user->{'supp_group'})
   {
      push @cmd, '-G', $user->{'supp_group'};
   }
   if ((defined $user->{'home'}) && $user->{'home'})
   {
      push @cmd, '-d', $user->{'home'};
   }
   if ((defined $user->{'pwd'}) && $user->{'pwd'})
   {
      push @cmd, '-p', $user->{'pwd'};
   }
   if (!(defined $user->{'app_name'}) || !$user->{'app_name'})
   {
      $user->{'app_name'} = $NORTEL_LINUX_BASE_APP_NAME;
   }
   if (defined($user->{'uid'}))
   {
      push @cmd, '-u', $user->{'uid'};
   }
   if ($command ne $USERMOD)
   {
      push @cmd, '-m', '-k', $SKEL_DIR;
   }
   if ((defined $user->{'shell'}) && ($user->{'shell'}))
   {
      push @cmd, '-s', $RBASH;
   }
   else
   {
      push @cmd, '-s', $NOLOGIN_SHELL;
   }
   if ((defined $user->{'exp_date'}) && ($user->{'exp_date'}))
   {
      push @cmd, '-e', $user->{'exp_date'};
   }
   push @cmd, $user->{'username'};
   baselog($LOG_DEBUG, "The following command has been generated: @cmd.");
   return \@cmd;
} ## end sub make_command($$)

## @fn get_user_type($username)
# @brief returns user's type
# @param $username - user's name for check
# @return user's type
# @return undef value in case of error
# @author Evgeny Lazarev
# @date September 2008
sub get_user_type($)
{
   my $username = shift @_;
   my $retval   = undef;
   unless (defined($username))
   {
      baselog($LOG_ERROR, "User's name is not specified.");
   }
   else
   {
      my $users = get_added_users("", $ALL_USERS);
      for my $user (@$users)
      {
         if ($user->{'username'} eq $username)
         {
            $retval = $user->{'type'};
            last;
         }
      } ## end for my $user (@$users)
   } ## end else
   return $retval;
} ## end sub get_user_type($)

## @fn get_accounts_by_type($)
# @brief returns reference to list of user names of specified type.
# @param $type - type of users for retrieve. Could be one of the:
#        $LINUX_USER, $LINUX_FTP_USER,  $LINUX_SFTP_USER, $LINUX_FTP_SFTP_USER,
#        $UCM_USER.
# @return $result reference to list
# @return undef - user's type is not specified
# @author Evgeny Lazarev
# @date September 2008
sub get_accounts_by_type($)
{
   my $type   = shift @_;
   my $result = [];
   if (!defined($type))
   {
      baselog($LOG_ERROR, "User's type is not specified.");
      return [];
   }
   my $users = get_added_users("", $ALL_USERS);
   for my $user (@$users)
   {
      if ($user->{'type'} eq $type)
      {
         push @$result, $user->{'username'};
      }
   } ## end for my $user (@$users)
   return $result;
} ## end sub get_accounts_by_type($)

## @fn sub is_ftp_account($username)
# @brief Checks if specified user is a Linux_FTP_User user or not.
# @param $username - a reference to the hash with user's parameters
# @return $TRUE - specified user is FTP user
# @return $FALSE - specified user is not FTP user or user name is not specified
# @author Evgeny Lazarev
# @date August 2008
# @note if the user's type is Linux_FTP_SFTP_User then this function will return
#       $FALSE!
sub is_ftp_account($)
{
   my $username  = shift(@_);
   my $ftp_users = undef;
   my $retval    = $TRUE;
   unless (defined($username))
   {
      baselog($LOG_ERROR, "User name is not specified.");
      return $FALSE;
   }
   $ftp_users = get_accounts_by_type($LINUX_FTP_USER);
   if (pos_in_array($ftp_users, $username) < 0)
   {
      $retval = $FALSE;
   }
   return $retval;
} ## end sub is_ftp_account($)

1;
