## @file base_apps_service.pm
# @brief User interface for LinuxBase
# @details Module is the interface to manage applications startup,
#          stop and status commands. The xml databases (install.xml and
#          init.xml files) are used to contain startup and stop sequence.
# @author Peter Kascheev, Pavel Sergeev
# @author Alexander Kolotov
# @author Evgeny Lazarev
# @author Alex Arkhipov
# @author Vladimir Ivanov
# @date November 2008
# @version 1.00 - Initial version
# @version 1.10 - usage of 'su' command is replaced by invoking
#          prepare_command_to_run_as subroutine. priority_cmd is
#          reworked and renamed to prepare_command_to_run_with_priority
#          It invokes prepare_command_to_run_as.
#          All invocations of system are replaced by do_system.
# @version 1.20 - Base Applications are started first, then Nortel applications
#          are started.
# @version 1.30 - Management of the Base and Nortel Applications is separated.
# @version 1.31 - get rid of global variables $APP_INSTALLED and $APP_VERSION.
# @version 1.40 - Support for init reference files is added by Alex Arkhipov
# @version 1.41 - Monit status applying is fixed in the app_call function.
# @version 1.42 - Two colons were removed before shared functions and variables
# @version 1.43 - PAM support added
# @version 1.50 - Unmonitor application before start it and monitor after start
#          monit functionality is moved to base_monit.pm
# @version 1.60 - Add multiple app control. New function app_management is added
# @version 1.65 - support of iptables-restore functions
# @version 1.67 - one app restart behavior inconsistency was fixed; subroutine
#          is_restart_per_appl() was added.
# @version 1.68 - the function get_app_states() was updated to check the statuses
#          of the given applications (optional behaviour)
# @version 1.69 - support for 'norestartonsysrestore' flag in init.xml
# @version 1.70 - checking "noparams" attribute was added.
# @version 1.71 - re-worked to perform firewall actions thru base_firewall object
# @version 1.72 - re-worked some other actions for better performance
# @version 1.73 - flushing iptables rules was added if applications were stoped and
#                 START/RESTART operation is invoked
# @version 1.74 - appstart lock file support is added
# @version 1.75 - support of the 'APPSTART_IGNORE' environment variable is added
# @version 1.76 - RESTART operation has been changed to STOP in case of disabled application.
# @version 1.77 - subroutine convert_to_relative_priority added
# @version 1.78 - message indication for disabled init scripts is added.
# @note APP_START semaphore must be taken outside prior to this object
#       creation and usage. Also semaphore must be released after usage.
package base_apps_service;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;
use install_common;

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw($VERSION $TYPE_NORTEL_APPL $TYPE_BASE_APPL $TYPE_ALL_APPL $APPL_TYPE);

use Data::Dumper;
use Term::ANSIColor;
use XML::Parser;

use appinstall::errors;
use appinstall::update_sys_data;
use baselog;
use base_firewall;
use hostControl;
use base_sysctl;
use ifxml;
use XMLRef;
use base_system;
use Authen::PAM;
use base_semctl;
use base_monit;

our $VERSION = '1.78';

#####################################################################
########################## Public methods ###########################
## @skipline
sub app_management(@);
## @skipline
sub base_app_start();
## @skipline
sub base_app_stop();
## @skipline
sub reload($);
## @skipline
sub start($);
## @skipline
sub status($);
## @skipline
sub stop($);

#####################################################################
########################## Private methods ##########################
## @skipline
sub app_call(@);
## @skipline
sub check_disabled_scripts($);
## @skipline
sub get_application_scripts($$;$);
## @skipline
sub get_stopped_BaseApps($);
## @skipline
sub is_app_started($);
## @skipline
sub net_configuration_reapply(;$$);
## @skipline
sub prepare($);
## @skipline
sub prepare_command_to_run_with_priority($$$);
## @skipline
sub priority_check($);
## @skipline
sub get_app_states($;$$);
## @skipline
sub convert_to_relative_priority($);
#####################################################################

# Key for hash with data
our $APPL_TYPE = 'type';

# Types of the applications for management
our $TYPE_NORTEL_APPL = 'nortel';
our $TYPE_BASE_APPL   = 'base';
our $TYPE_ALL_APPL    = 'all';
my $APPSTARTUP_LOCK_FILE = "/var/lock/subsys/appstartup";

## @method new
# @brief   constructor, Object is an hash of hashes
# @details
# @param   reference to the hash with parameters. There is one key: $APPL_TYPE
# with values $TYPE_NORTEL_APPL, $TYPE_BASE_APPL or $TYPE_ALL_APPL
# which specifies type of the applications for management.
# @return  class instance or undef if error
# @author  Peter Kascheev
# @date    November 2007
sub new
{

   #initialization
   my ($class, $params) = @_;
   my $self = {};  #Class instance
   my $pamh;       #PAM object

   #set default value
   $self->{$APPL_TYPE} = $TYPE_NORTEL_APPL;

   if (defined($params))
   {
      if (defined($params->{$APPL_TYPE}))
      {
         if (($params->{$APPL_TYPE} eq $TYPE_NORTEL_APPL) || ($params->{$APPL_TYPE} eq $TYPE_BASE_APPL) || ($params->{$APPL_TYPE} eq $TYPE_ALL_APPL))
         {
            $self->{$APPL_TYPE} = $params->{$APPL_TYPE};
         }
         else
         {
            baselog($LOG_ERROR, "Unknown type of applications '$params->{$APPL_TYPE}'");
            return undef;
         }
      } ## end if (defined($params->{...
   } ## end if (defined($params))
   baselog($LOG_DEBUG, "Create base_apps_service instance for management of the $self->{$APPL_TYPE} applications.");

   if (warning_if_not_root_uid())
   {
      baselog($LOG_ERROR, "base_app_service must be used under $ROOT_ACCT account. " . (getpwuid($<))[0] . " is not allowed to do this.");
      return undef;
   }

   unless (check_scripts("init"))
   {
      baselog($LOG_ERROR, "Could not read applications xml files\n", $TRUE);
      return undef;
   }

   #Class variables initialization
   $class = "base_apps_service" unless $class;

   # Update files of system data
   $LASTERR = 'NULL';
   appinstall::update_sys_data::update_sdb();
   if ($LASTERR ne 'NULL')
   {
      baselog($LOG_ERROR, "$appinstall::errors::ERRORDB{$LASTERR}");
   }

   if ($self->{$APPL_TYPE} eq $TYPE_ALL_APPL)
   {
      my ($tmpinstorder, $tmpapporder) = ([], []);

      # add Base Applications first. After them add Nortel Applications
      ($self->{'instorder'}, $self->{'apporder'}) = prepare($TYPE_BASE_APPL);
      unless (defined($self->{'instorder'}))
      {
         baselog($LOG_ERROR, "Could not read xml files\n", $TRUE);
         return undef;
      }
      ($tmpinstorder, $tmpapporder) = prepare($TYPE_NORTEL_APPL);
      unless (defined($tmpinstorder))
      {
         baselog($LOG_ERROR, "Could not read xml files\n", $TRUE);
         return undef;
      }
      push @{$self->{'instorder'}}, @$tmpinstorder;
      push @{$self->{'apporder'}},  @$tmpapporder;
   } ## end if ($self->{$APPL_TYPE...
   else
   {
      ($self->{'instorder'}, $self->{'apporder'}) = prepare($self->{$APPL_TYPE});
   }
   unless ($self->{'instorder'} or $self->{'apporder'})
   {
      baselog($LOG_ERROR, "Could not read xml files\n", $TRUE);
      return undef;
   }

   $pamh = new Authen::PAM("system-auth", "root");
   if (not ref($pamh))
   {
      baselog($LOG_ERROR, "Error during PAM initializing\n");
      return undef;
   }
   my $res = $pamh->pam_open_session(PAM_SILENT);
   if ($res != PAM_SUCCESS)
   {
      baselog($LOG_ERROR, "Error opening session for user 'root'\n");
      return undef;
   }
   $res = $pamh->pam_setcred(PAM_SILENT | PAM_ESTABLISH_CRED);
   if ($res != PAM_SUCCESS)
   {
      baselog($LOG_ERROR, "Cannot set credentials for user 'root'\n");
      return undef;
   }

   $self->{'pamh'} = $pamh;

   # Create base_firewall object ones
   $self->{'firewall'} = new base_firewall;
   unless (defined $self->{'firewall'})
   {
      baselog($LOG_ERROR, "Cannot create base_firewall object.");
      return undef;
   }

   bless $self, $class;  #Declaring an object

   return $self;
} ## end sub new

## @method DESTROY
# @brief   Object destructor
# @details
# @param   none
# @return  none
# @author  Peter Kascheev
# @date    November 2007
sub DESTROY(@)
{
   my $self = shift(@_);
   if (ref($self->{'pamh'}))
   {
      my $res = $self->{'pamh'}->pam_close_session(PAM_SILENT);
      if ($res != PAM_SUCCESS)
      {
         baselog($LOG_ERROR, "Error during closing PAM session");
      }
   } ## end if (ref($self->{'pamh'...
   return;
} ## end sub DESTROY(@)

## @method  net_configuration_reapply($states, $apps)
# @brief   Restart the firewall and synchronize user defined hosts
# @details
# @param   $states - application states hash (optional)
# @param   $apps - an array containing applications names (optional)
# @return  $TRUE - Success
# @return  $FALSE - Failure
# @author
# @date
sub net_configuration_reapply(;$$)
{
   my $data = shift @_;
   return $FALSE unless (defined $data);
   my $states = shift @_;
   my $apps   = shift @_;

   unless ($data->{'firewall'}->firewall_restart($states, $apps))
   {
      baselog($LOG_ERROR, "Cannot restart firewall");
      return $FALSE;
   }

   my $host = new hostControl;
   unless (defined $host)
   {
      baselog($LOG_ERROR, "Cannot create hostControl object");
      return $FALSE;
   }
   unless ($host->sync())
   {
      baselog($LOG_ERROR, "Cannot synchronize hosts");
      print "Could not synchronize /etc/hosts with user defined hosts\n";
   }
   undef $host;
   return $TRUE;
} ## end sub net_configuration_reapply(;$$)

## @fn     prepare
# @brief   It parses baseapps-install.xml or install.xml and init.xml files
# @details
# @param   $type type of the Applications for management
# @return  two references to two hashes with data
# @author
# @date
sub prepare($)
{
   my $type = shift(@_);

   my $supportedConfigs = [];
   my $instorder        = [];

   if ($type eq $TYPE_BASE_APPL)
   {
      $instorder = getBaseApplicationsList();
   }
   else
   {
      $instorder = getNortelApplicationsList();
   }

   print_xml_dump($STARTLOG, "Startup order:\n", Dumper($instorder));

   my $parser = ifxml::new($SYSDB_ROOT, $INIT_CFG);
   if (!$parser)
   {
      baselog($LOG_ERROR, "Can not find file $INIT_CFG");
      print "Installation file is not found\n";
      return undef;
   } ## end if (!$parser)

   my %isItBaseApplication = list2hash(getBaseApplicationsList(),  $TRUE);
   my $tmpapporder         = $parser->find_data_by_xpath($GETDATA, "/init/rule");
   my $apporder            = [];
   for my $rule (@$tmpapporder)
   {
      if ($isItBaseApplication{$rule->{appl}})
      {

         # add script if $type is not 'nortel', i.e. is 'base' or 'all'
         if ($type ne $TYPE_NORTEL_APPL)
         {
            push @$apporder, $rule;
         }
      } ## end if ($isItBaseApplication...
      else
      {

         # add script if $type is not 'base', i.e. is 'nortel' or 'all'
         if ($type ne $TYPE_BASE_APPL)
         {
            push @$apporder, $rule;
         }
      } ## end else [ if ($isItBaseApplication...
   } ## end for my $rule (@$tmpapporder...
   print_xml_dump($STARTLOG, "INIT list:\n", Dumper($apporder));

   # Check for disabled application scripts
   check_disabled_scripts($apporder);

   foreach my $script (@$apporder)
   {
      $script->{'as'} = $NORTEL_ACCT unless $script->{'as'};
   }
   baselog($LOG_INFO, "Preparation completed\n");
   return ($instorder, $apporder);

} ## end sub prepare($)

## @fn check_disabled_scripts
# @brief Check reference files to see if script is enabled or not
# @details Adds enl attribute to $rule hash
# @details It is $FALSE if disabled or $TRUE if enabled
# @param $script_rules - reference to array of application rules
# @return none
# @author Alex Arkhipov
# @date August 2008
sub check_disabled_scripts($)
{
   my $script_rules = shift(@_);

   # Create instance of XMLRef class.
   my $xmlref = XMLRef::new();

   foreach my $rule (@$script_rules)
   {

      # Script is enabled by default
      my $enl = $TRUE;

      my $reffile = $rule->{'reference'};
      my $xpath   = $rule->{'enl_ref'};
      my $def_enl = (defined $rule->{'enl'}) ? $rule->{'enl'} : $enl;
      my $enl_tmp;

      $enl_tmp = $xmlref->get_referenced_value($reffile, $xpath, $def_enl);

      if (defined($enl_tmp))
      {

         # Convert to $TRUE or $FALSE
         $enl = ($enl_tmp eq '0') ? $FALSE : $TRUE;
      } ## end if (defined($enl_tmp))
      $rule->{'enl'} = $enl;
   } ## end foreach my $rule (@$script_rules...
   $xmlref = undef;
} ## end sub check_disabled_scripts($)

## @method monitinit
# @brief   Performs monit initialization operation
# @details Checks for enable flag for all applications and enable/disable monit for them
# @param   none
# @return $FALSE if error
# @return $TRUE  if success
# @author Alex Arkhipov
# @date October 20, 2008
sub monitinit($)
{
   my $data = shift;
   return $FALSE unless (defined $data);

   foreach my $app (@{$data->{'instorder'}})
   {
      my $scripts = get_application_scripts($data->{'apporder'}, $app);

      foreach my $script (@$scripts)
      {
         if (not $script->{'enl'})
         {

            # do unmonitor
            app_monit_control($UNMONITOR, $TRUE, $script->{'appl'}, $script->{'pmark'});
            baselog($LOG_INFO, "Unmonitor $script->{'file'} script.");
         } ## end if (not $script->{'enl'...
      } ## end foreach my $script (@$scripts...
   } ## end foreach my $app (@{$data->{...

   return $TRUE;
} ## end sub monitinit($)

## @method start
# @brief   Performs applications startup
# @details In order to start applications it invokes applications scripts
#          one by one in startup sequence.
# @details If there is no scripts for particular application
#            it is considered as started successfully
# @details If all enabled application scripts return success
#            it is considered as started successfully
# @details If all application scripts are disabled
#            it is considered as not started
# @details If at least one script is failed to start
#            it is considered as not started
# @details If application is not started then
#            all FW ports for the application are closed
#            sysctl parameters are not applied
# @param   none
# @return $FALSE if error
# @return $TRUE if success
# @author
# @date
sub start($)
{
   my $data = shift;
   return $FALSE unless (defined $data);
   my $cmdstr;
   my $succeeded;
   my $enabled;
   my @apps         = @{getBaseApplicationsList()};
   my $noparams_enl = $FALSE;

   # check if one or more Base Applications are not started then exit with error.
   my $not_started = get_stopped_BaseApps($data);
   if (scalar(@$not_started) > 0)
   {
      my $tmp = join(',', @$not_started);
      baselog($LOG_ERROR, "These Base Applications are not started: $tmp.", $TRUE);
      return $FALSE;
   } ## end if (scalar(@$not_started...

   if ($data->{$APPL_TYPE} ne $TYPE_BASE_APPL)
   {
      push @apps, @{$data->{'instorder'}};
   }
   my %states = ();

   # Do not get application statuses
   # assign $TRUE for specified applications
   foreach my $appname (@apps)
   {
      $states{$appname} = $TRUE;
   }
   $data->net_configuration_reapply(\%states);

   foreach my $app (@{$data->{'instorder'}})
   {
      baselog($LOG_INFO, "Starting application: $app\n");

      # The 'undef' means that there are no init scripts at all
      $succeeded = undef;

      # Flag that shows if there is at least one script that is enabled
      $enabled = $FALSE;

      #flag that shows if there is at least one script has "noparams" enabled
      $noparams_enl = $FALSE;

      my $scripts = get_application_scripts($data->{'apporder'}, $app);

      # Count disabled scripts per application
      my $skipped_scripts = 0;

      foreach my $script (@$scripts)
      {
         if (not defined($succeeded))
         {

            # There is at least one application script
            $succeeded = $TRUE;
         } ## end if (not defined($succeeded...

         # Skip disabled script
         if (not $script->{'enl'})
         {
            $skipped_scripts++;
            baselog($LOG_INFO, "Skipping $script->{'file'} script.");
            next;
         } ## end if (not $script->{'enl'...

         next if skip_on_restore($script);
         next unless priority_check($script);

         $enabled = $TRUE;
         baselog($LOG_DEBUG, "Start file: $script->{'file'} noparams: $script->{'noparams'} appl: $script->{'appl'} monit: $script->{'monit'}");

         $cmdstr = $script->{'file'};
         if ($script->{'noparams'})
         {
            $noparams_enl = $TRUE;
         }
         else
         {
            $cmdstr .= " $SRV_START";
         }
         my @cmdarr = @{prepare_command_to_run_with_priority($cmdstr, $script->{'as'}, $script)};
         if (scalar @cmdarr != 0)
         {
            baselog($LOG_INFO, "@cmdarr");

            my $cmd_retval;
            if (not do_system(\@cmdarr, {$dsRETVAL => \$cmd_retval, $dsINTERACT => $TRUE, $dsCHDIR => $dsHOMEDIR, $dsSIGDEFAULT => $TRUE}))
            {
               baselog($LOG_ERROR, "Script $script->{'file'} returned $cmd_retval");
               $succeeded = $FALSE;
               print color 'bold red';
               print "Application " . $app . " (" . $script->{'file'} . ") $SRV_START returned $cmd_retval.\n";
               print color 'reset';
            } ## end if (not do_system(\@cmdarr...
         } ## end if (scalar @cmdarr != ...
         else
         {
            baselog($LOG_ERROR, "Script $script->{'file'} can not be executed.");
            $succeeded = $FALSE;
            print color 'bold red';
            print "Application " . $app . " (" . $script->{'file'} . ") is not started.\n";
            print color 'reset';
         } ## end else [ if (scalar @cmdarr != ...
      } ## end foreach my $script (@$scripts...

      if ($skipped_scripts > 0)
      {
         if ($enabled)
         {

            #at least one of application's scripts was called
            baselog($LOG_WARNING, "Some of the components of the '$app' application  have not been started (possibly not configured).\n", $TRUE);
         } ## end if ($enabled)
         else
         {

            # none of applications scripts were called.
            baselog($LOG_WARNING, "Application '$app' was not started (possibly not configured).\n", $TRUE);
         } ## end else [ if ($enabled)
      } ## end if ($skipped_scripts >...

      if ((not defined($succeeded)) || ($enabled && $succeeded))
      {

         # Application is successfully started or no init scripts provided
         # Apply system parameters (by sysctl)
         apply_parameters($app);
      } ## end if ((not defined($succeeded...
      else
      {
         if (not $noparams_enl)
         {
            unless ($data->{'firewall'}->disable_app_rules($app))
            {
               baselog($LOG_ERROR, "Disabling $app rules failed");
            }
         } ## end if (not $noparams_enl)

      } ## end else [ if ((not defined($succeeded...

      baselog($LOG_INFO, "$app start finished");
   } ## end foreach my $app (@{$data->{...
   restart_monit();
   baselog($LOG_INFO, "monit updated");
   if ($data->{$APPL_TYPE} ne $TYPE_BASE_APPL)
   {
      if (not do_system([$TOUCH, $APPSTARTUP_LOCK_FILE]))
      {
         baselog($LOG_WARNING, "Could not create $APPSTARTUP_LOCK_FILE", $TRUE);
      }
   } ## end if ($data->{$APPL_TYPE...
   return $TRUE;
} ## end sub start($)

## @method stop
# @brief   Performs the applications shutdown
# @details In order to stop applications
#          it invokes applications scripts one by one in stop sequence.
# @param   $data - object reference
# @return  $FALSE if error, $TRUE if success
# @author
# @date
sub stop($)
{
   my $data = shift;
   return $FALSE unless $data;
   my @reverseorder = reverse @{$data->{'instorder'}};
   my ($active_init, $clear_alarm, $scripts);
   my @sortedinitlist;
   my @todeleterulelist = ();
   my $noparams_enl     = $FALSE;
   foreach my $app (@reverseorder)
   {
      $scripts = get_application_scripts($data->{'apporder'}, $app);
      @sortedinitlist = sort {$b->{'order'} <=> $a->{'order'}} @{$scripts};

      $clear_alarm  = $FALSE;
      $active_init  = $FALSE;
      $noparams_enl = $FALSE;
      baselog($LOG_INFO, "Stopping application: $app\n");
      foreach my $script (@sortedinitlist)
      {
         baselog($LOG_DEBUG, "Stop file: $script->{'file'} noparams: $script->{'noparams'} appl: $script->{'appl'} monit: $script->{'monit'}");

         # All scripts with "noparams" flag will be skipped
         # because we can't pass them "stop" parameter
         if (!$script->{'noparams'})
         {
            next if skip_on_restore($script);
            $active_init = $TRUE;
            if ($script->{'monit'})
            {
               $clear_alarm = $TRUE;
               app_monit_control($UNMONITOR, $TRUE, $script->{'appl'}, $script->{'pmark'});
            }

            my @cmdarr = @{prepare_command_to_run_as("$script->{'file'} $SRV_STOP", $script->{'as'})};
            if (scalar @cmdarr != 0)
            {
               baselog($LOG_INFO, "@cmdarr");

               my $cmd_retval;
               if (not do_system(\@cmdarr, {$dsRETVAL => \$cmd_retval, $dsINTERACT => $TRUE, $dsCHDIR => $dsHOMEDIR, $dsSIGDEFAULT => $TRUE}))
               {
                  baselog($LOG_ERROR, "Script $script->{'file'} returned $cmd_retval. The system parameters are restored for '$script->{'appl'}' application anyway.");
                  print color 'bold red';
                  print "Application " . $app . " (" . $script->{'file'} . ") $SRV_STOP returned $cmd_retval.\n";
                  print color 'reset';
               } ## end if (not do_system(\@cmdarr...
            } ## end if (scalar @cmdarr != ...
            else
            {
               baselog($LOG_ERROR, "Script $script->{'file'} can not be executed. " . "The system parameters are restored for '$script->{'appl'}' application anyway.");
               print color 'bold red';
               print "Application " . $app . " (" . $script->{'file'} . ") is not stopped.\n";
               print color 'reset';
            } ## end else [ if (scalar @cmdarr != ...
         } ## end if (!$script->{'noparams'...
         else
         {
            $noparams_enl = $TRUE;
         }
      } ## end foreach my $script (@sortedinitlist...
      if ($clear_alarm)
      {
         clear_alarm($app);
      }
      if ($active_init)
      {

         # Restore system parameters (by sysctl) in both cases:
         # either application's script returns OK or ERROR. In the
         # second case the system anyway in incorrect state, so
         # there are no hindrances to restore parameters
         restore_parameters($app);
      } ## end if ($active_init)

      # If at least one script exist, then remove firewall rules
      if (scalar(@sortedinitlist) > 0)
      {
         if (not $noparams_enl)
         {

            # postpone real rules deletion for better 'appstart stop' performance.
            push @todeleterulelist, $app;
         } ## end if (not $noparams_enl)
      } ## end if (scalar(@sortedinitlist...
      baselog($LOG_INFO, "$app stopped\n");
   } ## end foreach my $app (@reverseorder...
     # now perform delete_rule_by_name for apropriate apps name
   unless ($data->{'firewall'}->delete_rules_by_name(\@todeleterulelist))
   {
      baselog($LOG_ERROR, "Disabling @todeleterulelist rules failed");
   }
   if ($data->{$APPL_TYPE} ne $TYPE_BASE_APPL)
   {
      if (-f $APPSTARTUP_LOCK_FILE && !unlink($APPSTARTUP_LOCK_FILE))
      {
         baselog($LOG_WARNING, "Cannot remove appstart lock file");
      }
   } ## end if ($data->{$APPL_TYPE...
   return $TRUE;
} ## end sub stop($)

## @method status
# @brief   It calls application scripts with 'status' parameter
# @details
# @param   none
# @return  $TRUE
# @author
# @date
sub status($)
{
   my $data = shift;
   return $FALSE unless $data;
   get_app_states($data, $TRUE);
   return $TRUE;
} ## end sub status($)

## @method reload
# @brief
# @details It invokes stop method, then reloads system data, then
#          starts applications
# @param   none
# @return  $FALSE if error, $TRUE if data reloaded and application
#          restarted
# @author
# @date
sub reload($)
{
   my $data = shift;
   return $FALSE unless $data;
   my $retval;

   # Stop application. If stop() returns $TRUE it means that all
   # the system parameters were restored.
   $retval = stop($data);

   if ($retval == $TRUE)
   {
      $retval = $FALSE;

      # Restore system parameters for Linux Base.
      if (restore_parameters('base'))
      {

         # Remove backup DB
         if (remove_sysctl_backup())
         {

            # Apply system parameters for Linux Base.
            if (apply_parameters('base'))
            {
               $retval = start($data);
               if ($retval == $FALSE)
               {
                  baselog($LOG_ERROR, "Cannot start applications");
               }
            } ## end if (apply_parameters('base'...
            else
            {
               baselog($LOG_ERROR, "Cannot apply the system parameters");
            }
         } ## end if (remove_sysctl_backup...
      } ## end if (restore_parameters...
      else
      {
         baselog($LOG_ERROR, "Cannot restore the system parameters");
      }
   } ## end if ($retval == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot stop applications");
   }
   return $retval;
} ## end sub reload($)

## @method app_call
# @brief  It invokes application script with parameter provided
# @details invokes specific command for one particular application or script
# @param $data - base_app_service object
# @param $params - params:
#          $param->[0] - application name or process name (see base_monit.pm module)
#          $param->[1] - command (start, stop, status, reload, etc.)
#          $param->[2] - parameters to pass to application script
# @return $FALSE - error
# @return $TRUE  - success
sub app_call(@)
{
   my $data = shift(@_);
   return $FALSE unless $data;
   my $parm = shift(@_);
   my ($process, $cmd, @args) = @{$parm};
   my $script;
   my $succeeded = undef;
   my $enabled;
   my $clear_alarm     = $FALSE;
   my $noparams_enl    = $FALSE;
   my $skipped_scripts = 0;

   # Let's check the input parameters

   # Get application name and process name
   my ($app, $script_name) = get_application_script_name($process);

   if (!grep {$_ eq $app} @{$data->{'instorder'}})
   {
      baselog($LOG_ERROR, "Unknown application: $app\n", $TRUE);
      return $FALSE;
   }
   my $scripts = get_application_scripts($data->{'apporder'}, $app, $script_name);
   if (scalar(@$scripts) == 0)
   {
      if (defined($script_name))
      {
         baselog($LOG_ERROR, "Unknown application script: $script_name\n", $TRUE);
         return $FALSE;
      }
      else
      {
         baselog($LOG_ERROR, "Application $app has no init scripts\n", $TRUE);
         return $TRUE;
      }
   } ## end if (scalar(@$scripts) ...

   # if a Nortel Applications is going to be started then check that all Base
   # Applications are started. Write warning to log.
   if ((!isBaseApplication($app)) && ($cmd eq $SRV_START))
   {
      my $notstarted = get_stopped_BaseApps($data);
      if (scalar(@$notstarted) > 0)
      {
         my $tmp = join(',', @$notstarted);
         baselog($LOG_WARNING, "These Base Applications are not started: $tmp.");
      }
   } ## end if ((!isBaseApplication...

   if ((($cmd eq $SRV_START) || ($cmd eq $SRV_RESTART)) and (not is_app_started($app)))
   {
      if (not $data->{'firewall'}->delete_rules_by_name([$app], $TRUE))
      {
         baselog($LOG_ERROR, "Flushing $app rules failed.");
      }
      elsif (not $data->{'firewall'}->enable_app_rules($app))
      {
         baselog($LOG_ERROR, "Enabling $app rules failed");
      }
   } ## end if ((($cmd eq $SRV_START...

   # Let's pass parameters to application's scripts
   my $argstr = join " ", @args;

   my $monitenabled   = $FALSE;
   my @sortedinitlist = @$scripts;
   if ($cmd eq $SRV_STOP)
   {
      @sortedinitlist = sort {$b->{'order'} <=> $a->{'order'}} @$scripts;
   }
   foreach $script (@sortedinitlist)
   {
      my $tmpcmd = $cmd;

      # "app_call" should not invoke scripts with 'noparam' attribute set
      if ($script->{'noparams'})
      {
         $noparams_enl = $TRUE;
         next;
      }

      next unless priority_check($script);

      # There is at least one init script
      if (not defined($succeeded))
      {
         $succeeded = $TRUE;
      }

      if ($script->{'monit'})
      {
         $monitenabled = $TRUE;
      }

      if (($cmd eq $SRV_STOP) || ($cmd eq $SRV_RESTART) || ($cmd eq $SRV_START))
      {

         # Check for monit control
         if ($monitenabled)
         {
            $clear_alarm = $TRUE;

            # Do unmonitor even for 'start' to prevent monit 'overcontroling'
            app_monit_control($UNMONITOR, $TRUE, $script->{'appl'}, $script->{'pmark'});
         } ## end if ($monitenabled)
      } ## end if (($cmd eq $SRV_STOP...

      if (not $script->{'enl'})
      {
         $skipped_scripts++;
         if ($cmd eq $SRV_START)
         {
            baselog($LOG_INFO, "Application $script->{'appl'} is not enabled. Skipping $script->{'file'} script.");
            next;
         }
         elsif ($cmd eq $SRV_RESTART)
         {
            $tmpcmd = $SRV_STOP;
            baselog($LOG_INFO, "Application $script->{'appl'} is not enabled. Only STOP action is applied.");
         }
      } ## end if (not $script->{'enl'...
      else
      {
         $enabled = $TRUE;
      }

      my @cmdarr = @{prepare_command_to_run_with_priority("$script->{'file'} $tmpcmd $argstr", $script->{'as'}, $script)};
      baselog($LOG_INFO, "@cmdarr") if (scalar @cmdarr != 0);

      if (
          (scalar @cmdarr == 0)
          or (
              not do_system(
                            \@cmdarr,
                            {
                             $dsINTERACT => $TRUE,
                             $dsCHDIR    => $dsHOMEDIR,
                             $dsNOERROR  => ($tmpcmd eq $SRV_STATUS) ? $TRUE : $FALSE
                            }
                           )
             )
         )
      {
         baselog($LOG_ERROR, "Script $script->{'file'} failed.");
         $succeeded = $FALSE;
         print color 'bold red';
         print "Application " . $app . " (" . $script->{'file'} . ") $tmpcmd $argstr failed.\n";
         print color 'reset';
      } ## end if ((scalar @cmdarr ==...
        # Start monitoring again even if application start failed
      if (($tmpcmd eq $SRV_RESTART) || ($tmpcmd eq $SRV_START))
      {
         if ($monitenabled)
         {
            app_monit_control($MONITOR, $TRUE, $script->{'appl'}, $script->{'pmark'});
         }
      } ## end if (($tmpcmd eq $SRV_RESTART...

   } ## end foreach $script (@sortedinitlist...
   if (($cmd eq $SRV_START) || ($cmd eq $SRV_RESTART))
   {
      if ((not defined($succeeded)) || ($succeeded && $enabled))
      {

         # Apply system parameters (by sysctl)
         apply_parameters($app);
      } ## end if ((not defined($succeeded...
      else
      {
         if (not $noparams_enl)
         {
            unless ($data->{'firewall'}->delete_rules_by_name([$app]))
            {
               baselog($LOG_ERROR, "Disabling $app rules failed");
            }
         } ## end if (not $noparams_enl)
      } ## end else [ if ((not defined($succeeded...
      if ($skipped_scripts > 0)
      {
         if ($enabled)
         {

            #at least one of application's scripts was called
            baselog($LOG_WARNING, "Some of the components of the '$app' application  have not been started (possibly not configured).\n", $TRUE);
         } ## end if ($enabled)
         else
         {

            # none of applications scripts were called.
            baselog($LOG_WARNING, "Application '$app' was not started (possibly not configured).\n", $TRUE);
         } ## end else [ if ($enabled)
      } ## end if ($skipped_scripts >...

   } ## end if (($cmd eq $SRV_START...
   elsif ($cmd eq $SRV_STOP)
   {

      # Restore system parameters (by sysctl) in both cases:
      # either application's script returns OK or ERROR. In the
      # second case the system anyway in incorrect state, so
      # there are no hindrances to restore parameters.
      restore_parameters($app);

      # Clear application alarm only if the whole application is stopped
      if ((not defined($script_name)) and $clear_alarm)
      {
         clear_alarm($app);
      }
      if (not $noparams_enl)
      {
         unless ($data->{'firewall'}->delete_rules_by_name([$app]))
         {
            baselog($LOG_ERROR, "Disabling $app rules failed");
         }
      } ## end if (not $noparams_enl)
   } ## end elsif ($cmd eq $SRV_STOP)

   return $TRUE;
} ## end sub app_call(@)

## @method app_management
# @brief  It invokes application scripts with parameter provided
# @details
# @param $data - base_app_service object
# @param $params - params:
#          $params->[0-..] - application or process name lists
#          $params->[scalar(@$params-1] - parameter
# @return $TRUE - can execute application scripts (no success indication)
# @return $FALSE - wrong arguments
# @author Alex Arkhipov
# @date November 2008
sub app_management(@)
{
   my $data = shift(@_);
   return $FALSE unless $data;
   my $params   = shift(@_);
   my @app_list = @$params;
   my $cmd      = pop(@app_list);

   if (!defined($cmd))
   {
      baselog($LOG_ERROR, "Command expected\n", $TRUE);
      return $FALSE;
   }

   # Remove duplicate elements
   {
      my %seen = ();
      @app_list = grep {!$seen{$_}++} @app_list;
   }

   baselog($LOG_DEBUG, "app_management: $cmd - " . join(' ', @app_list));

   $data->set_application_list(\@app_list);

   # skip applications name cheking if environment variable is set
   if (!$ENV{'APPSTART_IGNORE'})
   {

      # Check for incorrect input provided
      if (not $data->check_incorrect_app_names(\@app_list))
      {
         return $FALSE;
      }
   } ## end if (!$ENV{'APPSTART_IGNORE'...
   else
   {
      baselog($LOG_INFO, 'Applications name checking is skipped.');
   }

   @app_list = @{$data->{'app_list'}};

   # If list is empty - just exit
   if (scalar(@app_list) == 0)
   {
      baselog($LOG_WARNING, "No applications provided to manage", $TRUE);
      return $TRUE;
   }

   $data->{'appstates'} = get_app_states($data, $FALSE, \@app_list);

   if ($cmd ne $SRV_STATUS)
   {
      $data->net_configuration_reapply($data->{'appstates'}, \@app_list);
   }

   # Special handling for RESTART cmd
   if ($cmd eq $SRV_RESTART and $data->is_restart_per_appl(\@app_list))
   {
      baselog($LOG_DEBUG, "$cmd means '$SRV_STOP' and '$SRV_START'\n");
      $data->app_list_cmd(\@app_list, $SRV_STOP, $TRUE);
      $data->app_list_cmd(\@app_list, $SRV_START);
   } ## end if ($cmd eq $SRV_RESTART...
   else
   {
      $data->app_list_cmd(\@app_list, $cmd, ($cmd eq $SRV_STOP));
   }

   return $TRUE;
} ## end sub app_management(@)

## @method is_restart_per_appl
# @brief  Checks if the restart method is set in 'per_appl' value
# @details Checks if we handle only one application, if so gets the application
#          scripts and checks the parameter 'app_restart_handler'
#          For more than one application returns $TRUE
# @param $data - base_app_service object
# @param $param - reference to application list
# @return $TRUE - application restart handle set to 'per_appl'
#                 or there are more then one application given
# @return $FALSE - application restart handle set to 'per_script'
# @author Vladimir Ivanov
# @date November 2008
sub is_restart_per_appl($$)
{
   my $data = shift(@_);
   unless (defined $data)
   {
      baselog($LOG_ERROR, "The first parameter must be the reference to the object.");
      return $FALSE;
   }
   my $params   = shift(@_);
   my @app_list = @$params;

   my $ret = $TRUE;
   if (scalar @app_list == 1)
   {
      $ret = $FALSE;
      my $scripts = get_application_scripts($data->{'apporder'}, $app_list[0]);
      if (scalar @$scripts == 0)
      {  #default behavior
         $ret = $TRUE;
      }
      else
      {  #at least one script
         $ret = $TRUE if ($scripts->[0]->{'app_restart_handler'} eq $PER_APPL_KEY);
      }
   } ## end if (scalar @app_list ==...
   return $ret;
} ## end sub is_restart_per_appl($$)

## @method set_application_list
# @brief  Set app_list member of base_app_service object
# @details Goes through 'instorder' list and set 'app_list' with
#          applications that present in $app_list only
# @param $data - base_app_service object
# @param $param - reference to application list
# @return none
# @author Alex Arkhipov
# @date November 2008
# @note This sub looks for application names only. Application scripts
#       are not supported
sub set_application_list($$)
{
   my $data = shift(@_);
   return unless $data;
   my $params   = shift(@_);
   my @app_list = @$params;
   $data->{'app_list'} = [];

   # If only one element then do not check for application
   # This is done to allow script execution (e.g. sigServerShare___pbx)
   if (scalar(@app_list) == 1)
   {
      push(@{$data->{'app_list'}}, @app_list);
      return;
   }

   # Go through application list first to preserve the order
   foreach my $entry (@{$data->{'instorder'}})
   {
      foreach my $list_item (@app_list)
      {
         if ($entry eq $list_item)
         {
            push(@{$data->{'app_list'}}, $list_item);
            last;
         }
      } ## end foreach my $list_item (@app_list...
   } ## end foreach my $entry (@{$data->...

} ## end sub set_application_list($$)

## @method check_incorrect_app_names()
# @brief  Check if there is at least one incorrect application provided
# @details Compares $data->{'app_list'} with 'app_list' and print error if necessary
# @param $data - base_app_service object
# @param $param - reference to application list to check
# @return $TRUE  - Everything is OK
# @return $FALSE - There is at least one unknown appliaction
# @author Alex Arkhipov
# @date November 2008
sub check_incorrect_app_names($)
{
   my $data  = shift(@_);
   my $param = shift(@_);
   return unless $param;
   my @app_list = @{$param};
   my ($len1, $len2) = (scalar(@app_list), scalar(@{$data->{'app_list'}}));

   if ($len1 == $len2)
   {
      return $TRUE;
   }

   # Unknown applications found
   my @unknown_apps = ();
   foreach my $app (@app_list)
   {
      if (pos_in_array($data->{'app_list'}, $app) < 0)
      {
         push(@unknown_apps, $app);
      }
   } ## end foreach my $app (@app_list)
   baselog($LOG_ERROR, "Unknown applications provided: @unknown_apps", $TRUE);
   return $FALSE;
} ## end sub check_incorrect_app_names($)

## @method app_list_cmd
# @brief Invoke specific command
# @details Call app_call for each applications from the list
#          list of application van be reversed
#          if is_reverse flag set to $TRUE
# @param $data - base_app_service object
# @param $param - reference to application list to execute $cmd
# @param $cmd - command to invoke
# @param $is_reverse - (optional) reverse application list or not
# @return none
# @author Alex Arkhipov
# @date November 2008
sub app_list_cmd($$$;$)
{
   my $data = shift(@_);
   return unless $data;
   my $param = shift(@_);
   return unless $param;
   my @app_list = @$param;
   my ($cmd, $is_reverse) = @_;
   return unless $cmd;
   $is_reverse = $FALSE if (not defined($is_reverse));

   if ($is_reverse)
   {
      @app_list = reverse(@app_list);
   }

   foreach my $item (@app_list)
   {
      $data->app_call([$item, $cmd]);
   }
} ## end sub app_list_cmd($$$;$)

## @fn sub base_app_start()
# @brief This function starts Base Applications
# @details
# @param None
# @return
#   - $TRUE  - Applications started successfully
#   - $FALSE - some errors happened
# @author Evgeny Lazarev
# @date April 2008
# @note
sub base_app_start()
{
   my $retval = $TRUE;
   my $app = new base_apps_service({$APPL_TYPE => $TYPE_BASE_APPL});
   if (!defined($app))
   {
      $retval = $FALSE;
      baselog($LOG_ERROR, "Can not create base_apps_service object.");
   }
   else
   {
      if (!$app->start())
      {
         baselog($LOG_ERROR, "Base Applications have not been started successfully.");
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "Base Applications have been successfully started.");
      }
   } ## end else [ if (!defined($app))
   return $retval;
} ## end sub base_app_start()

## @fn sub base_app_stop()
# @brief This function stops Base Applications
# @details
# @param None
# @return
#   - $TRUE  - Applications stopped successfully
#   - $FALSE - some errors happened
# @author Evgeny Lazarev
# @date April 2008
# @note
sub base_app_stop()
{
   my $retval = $TRUE;
   my $app = new base_apps_service({$APPL_TYPE => $TYPE_BASE_APPL});
   if (!defined($app))
   {
      $retval = $FALSE;
      baselog($LOG_ERROR, "Can not create base_apps_service object.");
   }
   else
   {
      if (!$app->stop())
      {
         baselog($LOG_ERROR, "Base Applications have not been stopped successfully.");
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "Base Applications have been successfully stopped.");
      }
   } ## end else [ if (!defined($app))
   return $retval;
} ## end sub base_app_stop()

## @fn sub prepare_command_to_run_with_priority($cmd, $user, $priority)
# @brief This subroutine and generates an array containing the priority
#        settings and a command for using them in the command system.
# @details The priority settings are based on the content of the
#          $priority hash. The array can be used both in system and in
#          do_system.
# @param $cmd      - the command to be run
# @param $user     - the login name of user which is used for the
#                    command running
# @param $priority - the hash containing the following fields:
#                    'scheduler' - scheduler policy
#                    'priority'  - priority number
#                    'ulimit'    - the set of options for ulimit
#                                  command
# @return @res - a reference to the array with priority part, runuser
#                part and the command. The array is empty, if some error
#                happened.
# @author
# @date
# @note
sub prepare_command_to_run_with_priority($$$)
{
   my ($cmd, $user, $priority) = @_;

   my @res = ();

   if ($priority->{'scheduler'} eq $SCHED_POLICIES[0])
   {
      my $nice = convert_to_relative_priority($priority->{'priority'});
      if (!defined($nice))
      {
         push @res, $NICE, "-n", 0;
         baselog($LOG_ERROR, "Incorrect nice priority is skipped");
      }
      else
      {
         push @res, $NICE, "-n", $nice;
      }
   } ## end if ($priority->{'scheduler'...
   else
   {
      push @res, $CHRT, {@SCHED_POLICIES}->{"$priority->{'scheduler'}"}, $priority->{'priority'};
   }

   my $ulimitstr = "";
   if ($priority->{'ulimit'} ne '')
   {
      $ulimitstr = "ulimit $priority->{'ulimit'} ;";

      # Screen the command by double quotes.
      $cmd = qq|"$cmd"|;

      my @cmdarr = @{prepare_command_to_run_as($cmd, $user)};
      if (scalar @cmdarr == 0)
      {
         @res = ();
      }
      else
      {

         # it will equal the string "$NICE|$CHRT priority $SH -c '$ulimitstr $cmdarr'"
         push @res, $SH, "-c", $ulimitstr . join(" ", @cmdarr);
      } ## end else [ if (scalar @cmdarr == ...

   } ## end if ($priority->{'ulimit'...
   else
   {
      my @cmdarr = @{prepare_command_to_run_as($cmd, $user)};
      if (scalar @cmdarr != 0)
      {
         push @res, @cmdarr;
      }
      else
      {
         @res = ();
      }
   } ## end else [ if ($priority->{'ulimit'...

   baselog($LOG_DEBUG, "prepare_command_to_run_with_priority: @res") if (scalar @res != 0);

   return \@res;
} ## end sub prepare_command_to_run_with_priority($$$)

## @fn     priority_check
# @brief   sets the default priority values and makes check
# @details
# @param   $_[0] - application script
# @return  $TRUE - Success, $FALSE - Failure
# @author
# @date
sub priority_check($)
{
   my $script = shift @_;

   #set default values if they are empty
   $script->{'priority'}  = $PRIORITY_DEFAULT     if (!defined($script->{'priority'}));
   $script->{'scheduler'} = $SCHED_POLICY_DEFAULT if (!defined($script->{'scheduler'}));
   $script->{'ulimit'}    = ''                    if (!defined($script->{'ulimit'}));

   baselog($LOG_DEBUG, "Scheduler: '$script->{'scheduler'}', priority: '$script->{'priority'}', ulimits: '$script->{'ulimit'}'");

   # check the scheduler
   unless (defined({@SCHED_POLICIES}->{"$script->{'scheduler'}"}))
   {
      baselog($LOG_ERROR, "The policy is not correct.\n", $TRUE);
      return $FALSE;
   }

   # check the priority
   my %prts =
     ($script->{'scheduler'} eq $SCHED_POLICY_DEFAULT)
     ? map {$_ => ''} @NICE_LIMITS
     : map {$_ => ''} @RT_LIMITS;
   unless (defined($prts{"$script->{priority}"}))
   {
      baselog($LOG_ERROR, "The priority mismatches a range of the chosen policy.\n", $TRUE);
      return $FALSE;
   }

   if ($script->{'ulimit'} ne '')
   {

      # syntax check of ulimit
      my $cmd_retval;
      do_system([$SH, "-c", "ulimit $script->{'ulimit'}"], {$dsRETVAL => \$cmd_retval});
      if ($cmd_retval == 2)
      {
         baselog($LOG_ERROR, "Incorrect ulimit parameter - '$script->{'ulimit'}'.\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($script->{'ulimit'...
   undef %prts;
   return $TRUE;
} ## end sub priority_check($)

## @fn get_app_states
# @brief  gets applications states
# @details if application does not have any "init" scripts at all, then it is considered as 'running'
# @details if application does have one 'init' script with 'noparams' flag set, then it is considered as 'running'
# @details if application does have multiple 'init' scripts, then check status of only enabled scripts
# @details if application does have multiple 'init' scripts and all of them are disabled, then it is considered as 'not running'
# @param $data        - hash reference with list of application init scripts
# @param $interact    - control interactive of do_system ($TRUE or $FALSE) (optional, default is $FALSE)
# @param $applist_ref - the reference to the array with applications' list (optional)
# @return hash reference with data 'app_name' => 'status' ($TRUE or $FALSE)
# @author Pavel Sergeev
# @date April 2008
sub get_app_states($;$$)
{
   my ($data, $interact, $applist_ref) = @_;
   return undef unless (defined $data);
   $interact = $FALSE unless (defined $interact);
   if (defined $applist_ref)
   {
      unless (is_array($applist_ref))
      {
         baselog($LOG_ERROR, "The 3d parameter type must be an array reference.");
         $applist_ref = undef;
      }
   } ## end if (defined $applist_ref...

   my %statehash = ();

   # @var $start_flag
   # @brief describes if application was started successfully or not
   my $start_flag;

   # @var $enl_flag
   # @brief describes if application is enabled or not
   # @details application is considered enabled if
   #           at least one 'init' script is enabled
   my $enl_flag;

   # @var $initexist
   # @brief describes if there is at least one init script
   my $initexist;

   # @var $noparams_enl
   # @brief flag to detect that at least one init-scripts has "noparams=1"
   my $noparams_enl = $FALSE;

   foreach my $app (@{$data->{instorder}})
   {
      if (defined $applist_ref)
      {
         next if (pos_in_array($applist_ref, $app) < 0);
      }
      $start_flag   = $TRUE;
      $enl_flag     = $FALSE;
      $initexist    = $FALSE;
      $noparams_enl = $FALSE;
      foreach my $script (@{$data->{apporder}})
      {
         if ($script->{appl} eq $app)
         {

            #All scripts with "noparams" flag will be skipped
            #because we can't pass them "status" parameter
            if (!$script->{'noparams'})
            {
               $initexist = $TRUE;
               my @cmdarr = @{prepare_command_to_run_as("$script->{'file'} $SRV_STATUS", $script->{'as'})};
               if (scalar @cmdarr != 0)
               {
                  my $cmd_retval;
                  do_system(\@cmdarr, {$dsRETVAL => \$cmd_retval, $dsINTERACT => $interact, $dsCHDIR => $dsHOMEDIR, $dsNOERROR => $TRUE});

                  # Skip script if disabled
                  # We still invoke status command to print info to the user
                  if (not $script->{'enl'})
                  {
                     next;
                  }

                  $enl_flag = $TRUE;

                  if ($cmd_retval)
                  {
                     baselog($LOG_INFO, "'@cmdarr' returned $cmd_retval");
                     $start_flag = $FALSE;
                  }
               } ## end if (scalar @cmdarr != ...
               else
               {
                  baselog($LOG_ERROR, "Script $script->{'file'} can not be executed");
               }
            } ## end if (!$script->{'noparams'...
            else
            {
               $noparams_enl = $TRUE;
            }
         } ## end if ($script->{appl} eq...
      } ## end foreach my $script (@{$data...
      if ((not $enl_flag) && $initexist)
      {

         # All application 'init' scripts are disabled
         $start_flag = $FALSE;
      } ## end if ((not $enl_flag) &&...
      if ($noparams_enl)
      {

         #if at least one init-script with noparams enabled
         $start_flag = $TRUE;
      } ## end if ($noparams_enl)
      $statehash{$app} = $start_flag;
   } ## end foreach my $app (@{$data->{...
   return \%statehash;
} ## end sub get_app_states($;$$)

## @fn     is_app_started($app_name)
# @brief   checks if specified applications is started.
# @details Applications is considered started if all the "init" scripts
#          of the applications are working.
# @param   $app_name - name of the application for check.
# @return  $TRUE - specified application is started
#          $FALSE - specified application is not started
# @author  Evgeny Lazarev
# @date May 2008
sub is_app_started($)
{
   my $app_name     = shift(@_);
   my $retval       = $TRUE;
   my $noparams_enl = $FALSE;

   my $parser = ifxml::new($SYSDB_ROOT, $INIT_CFG);
   if (!$parser)
   {
      baselog($LOG_ERROR, "Can not find file $INIT_CFG");
      print "Installation file is not found\n";
      return $FALSE;
   } ## end if (!$parser)

   my $apporder = $parser->find_data_by_xpath($GETDATA, "/init/rule");
   my @scripts = ();
   for my $rule (@$apporder)
   {
      if ($rule->{appl} eq $app_name)
      {
         if ($rule->{'noparams'})
         {

            #if at least on init script has "noparams" enabled
            $noparams_enl = $TRUE;
            next;
         } ## end if ($rule->{'noparams'...
         push @scripts, $rule->{file};
      } ## end if ($rule->{appl} eq $app_name...
   } ## end for my $rule (@$apporder...
   for my $scr (@scripts)
   {
      if (!do_system([$scr, 'status']))
      {
         baselog($LOG_INFO, "Script '$scr' is not started.");
         $retval = $FALSE;
      }
   } ## end for my $scr (@scripts)
   if ($noparams_enl)
   {
      $retval = $TRUE;
   }
   return $retval;
} ## end sub is_app_started($)

## @fn     get_stopped_BaseApps()
# @brief   returns the reference to the list of stopped Base applications
# @details Applications is considered started if all the "init" scripts
#          of the applications are working.
# @param   $data reference to the hash with class data
# @return  $not_started reference to the list of the stopped Base Applications
# @author  Evgeny Lazarev
# @date May 2008
sub get_stopped_BaseApps($)
{
   my $data        = shift(@_);
   my $baseapp     = getBaseApplicationsList();
   my $not_started = [];
   if ($data->{$APPL_TYPE} eq $TYPE_NORTEL_APPL)
   {
      for my $app (@$baseapp)
      {
         if (!is_app_started($app))
         {
            push @$not_started, $app;
         }
      } ## end for my $app (@$baseapp)
   } ## end if ($data->{$APPL_TYPE...
   return $not_started;
} ## end sub get_stopped_BaseApps($)

## @fn     get_application_scripts()
# @brief   returns the reference to the list of all $app <init> scripts
# @param   $scripts - reference to array with scripts
# @param   $app - name of application
# @param   $sc - script pmark that identifies particular application script (optional)
# @return  $list - reference to the list of all application scripts (might be empty)
sub get_application_scripts($$;$)
{
   my ($scripts, $app, $sc) = @_;
   my @list = grep {(($_->{'appl'} eq $app) and ((defined($sc) ? ($_->{'pmark'} eq $sc) : $TRUE)))} @$scripts;
   return \@list;
}

## @fn     skip_on_restore
# @brief   determines if restart is appropriate for this application
# @param   $script - reference to hash with application data
# @details
# @return  $FALSE  - we can restart this application.
#          $TRUE - we should not restart this application this time.
# @date    Dec 2008
sub skip_on_restore($)
{
   my $script = shift @_;
   if ($script->{'norestartonsysrestore'} and base_semctl::is_subsystem_active($SBS_SYSRESTORE))
   {
      baselog($LOG_INFO, "Skip the script $script->{'file'}.");
      return $TRUE;
   }
   return $FALSE;
} ## end sub skip_on_restore($)

## @fn     convert_to_relative_priority($new_priority)
# @brief   calculates offset of nice value relative to the current process priority
# @param   $new_priority - absolute nice value for process
# @return  $retval - offset of nice value to get the requested priority
#          $FALSE - can't determine nice value of the current process
sub convert_to_relative_priority($)
{
   my $new_priority = shift(@_);
   my $old_priority;
   if (!defined($new_priority))
   {
      baselog($LOG_ERROR, "undefined new_priority");
      return undef;
   }
   if ($new_priority < -20 or $new_priority > 19)
   {
      baselog($LOG_ERROR, "new_priority is incorrect");
      return undef;
   }
   my $pid = $$;
   open(FILE, "/proc/$pid/stat");
   $old_priority = <FILE>;
   close(FILE);
   if (!defined $old_priority)
   {
      return undef;
   }
   my @log = split(/ /, $old_priority);
   $old_priority = $log[18];
   if ($new_priority < -20 or $new_priority > 19)
   {
      return undef;
   }
   my $retval = $new_priority - $old_priority;
   return $retval;
} ## end sub convert_to_relative_priority($)
1;
