## @file base_crontab.pm
# @brief API module for crontab utility
# @details This module can add/remove crontab task for a user.
# @author Vlad Ivanov
# @author Alexander Kolotov
# @date
# @version 2.10 - Function getCrontabTask is implemented
# @version 2.01 - Two colons were removed before shared functions and variables.
# @version 2.00 - a semaphore is used to prevent changing of one crontab
#          at the same time, usage of do_system is introduced, module
#          was reworked in order to provide more common API
# @version 1.00 - Initial version
#
# @note
package base_crontab;

use lib "/opt/nortel/base/lib";

use common_variables;
use basedb;

### Access management
use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&addCrontabTask &removeCrontabTask &getCrontabTask);
our $VERSION = 2.10;

use warnings;
use strict;

use base_semctl;
use base_system;
use baselog;

my $CRON_SPOOL  = "/var/spool/cron/";
my $REMOVE_TASK = "remove";
my $ADD_TASK    = "add";

sub modifyCrontab;
sub doAction;
sub getCrontabCfg;

sub addCrontabTask($$$);
sub removeCrontabTask($$);
sub getCrontabTask($;$);

#####################################################################
########################### Private Methods #########################

## @fn sub modifyCrontab($cron_file, $action, $user, $task_time, $task_cmd)
# @brief Modifies cron jobs (add or remove) for a user.
# @details Reads all cron jobs from the crontab file, add or removes a
#    job, prepares the new crontab file and applies it
# @param $cron_file - the path to crontab
# @param $action    - "add" or "remove" task
# @param $user      - user
# @param $task_time - date and time in crontab notation, can be empty
#                     in case of removing
# @param $task_cmd  - command for crontab task
# @return $TRUE, if no errors happen during crontab modification,
#         otherwise $FALSE
# @author
# @date
# @note
sub modifyCrontab
{

   #parameters
   my ($cron_file, $action, $user, $task_time, $task_cmd) = @_;

   my $crontab_file;

   #read the current crontab
   if (!open($crontab_file, "<", $cron_file))
   {
      baselog($LOG_ERROR, "Cannot open '$cron_file' for reading. Check permissions.");
      return $FALSE;
   }
   my @rules = <$crontab_file>;
   if (!close($crontab_file))
   {
      baselog($LOG_ERROR, "Cannot close '$cron_file'");
      return $FALSE;
   }

   #remove $task from crontab file
   my $removed = $FALSE;
   foreach my $rule (@rules)
   {
      baselog($LOG_DEBUG, "rule: '$rule'");
      if ($rule =~ /$task_cmd/)
      {
         $rule    = undef;
         $removed = $TRUE;
         baselog($LOG_DEBUG, "The rule removed");
      } ## end if ($rule =~ /$task_cmd/...
   } ## end foreach my $rule (@rules)

   #it is unnecessary to modify crontab
   if (($action eq $REMOVE_TASK) && (not $removed))
   {
      baselog($LOG_INFO, "There is no '$task_cmd' in '$cron_file'");
      return $TRUE;
   }

   if ($action eq $ADD_TASK)
   {

      #add new task to the context of crontab
      push @rules, $task_time . " " . $task_cmd . "\n";
   } ## end if ($action eq $ADD_TASK...

   #update the crontab file
   my $tmpfile = "$cron_file.tmp";
   baselog($LOG_DEBUG, "Creating new '$tmpfile'");
   if (!open($crontab_file, ">", $tmpfile))
   {
      baselog($LOG_ERROR, "Cannot open '$tmpfile' for writing. Check permissions.");
      return $FALSE;
   }

   foreach my $rule (@rules)
   {

      #do not take into account deleted lines
      print $crontab_file $rule if (defined $rule);
   } ## end foreach my $rule (@rules)

   if (!close($crontab_file))
   {
      baselog($LOG_ERROR, "Cannot close '$tmpfile'");
      return $FALSE;
   }

   do_system([$CHOWN, "--reference=$cron_file", $tmpfile]);
   do_system([$CHMOD, "--reference=$cron_file", $tmpfile]);
   if (not rename($tmpfile, $cron_file))
   {
      baselog($LOG_ERROR, "Cannot move '$tmpfile' to '$cron_file'");
      unlink $tmpfile;
      return $FALSE;
   } ## end if (not rename($tmpfile...

   #Remove the previous crontab from cron pool
   if (-e $CRON_SPOOL . $user)
   {
      baselog($LOG_DEBUG, "Try to remove crontab for '$user'");
      if (not do_system([$CRONTAB, "-u", $user, "-r"]))
      {
         baselog($LOG_ERROR, "Cannot remove crontab for '$user'");
         return $FALSE;
      }
   } ## end if (-e $CRON_SPOOL . $user...

   #Apply new crontab
   if (-s $cron_file)
   {
      baselog($LOG_DEBUG, "Update cron pool");
      if (not do_system([$CRONTAB, "-u", $user, $cron_file]))
      {
         baselog($LOG_ERROR, "Cannot apply crontab for '$user'");
         return $FALSE;
      }
   } ## end if (-s $cron_file)

   #Reread crontabs from cron pool
   baselog($LOG_DEBUG, "Reload crontabs '$user'");
   if (not do_system([$SERVICE, "crond", $SRV_RESTART]))
   {
      baselog($LOG_ERROR, "'$SERVICE crond $SRV_RESTART' failed");
      return $FALSE;
   }

   return $TRUE;
} ## end sub modifyCrontab

## @fn sub doAction($action, $user, $task_time, $task_cmd)
# @brief This subroutine performs preparation actions prior to modifying
#        a crontab file
# @details Checks input parameters, and invokes modifyCrontab to apply the
#    changes. The semaphore SEM_CRONTAB is used to prevent modifying
#    the crontabs by several processes.
# @param $action    - "add" or "remove" task
# @param $user      - user
# @param $task_time - date and time in crontab notation, can be empty
#                     in case of removing
# @param $task_cmd  - command for crontab task
# @return $TRUE, if no errors happen during crontab modification,
#         otherwise $FALSE
# @author
# @date
# @note
sub doAction
{

   #parameters
   my ($action, $user, $task_time, $task_cmd) = @_;

   baselog($LOG_DEBUG, "action: '$action'; user: '$user'; date and time: '$task_time'; task: '$task_cmd'");

   #crontab configuration file for the $user
   my $cron_cfg = getCrontabCfg($user);
   return $FALSE if ($cron_cfg eq "");

   if ($task_cmd eq "")
   {
      baselog($LOG_ERROR, "Command is empty.");
      return $FALSE;
   }

   # build a rule for a crontab task. This rule is for adding.
   if ($action eq $ADD_TASK)
   {
      if ($task_time eq "")
      {
         baselog($LOG_ERROR, "Date and time are not specified.");
         return $FALSE;
      }
      my @datetime = split /\s+/, $task_time;
      if (scalar(@datetime) != 5)
      {
         baselog($LOG_ERROR, "Date and time are in wrong format.");
         return $FALSE;
      }

      baselog($LOG_DEBUG, "New crontab task: '$task_time $task_cmd'");
   } ## end if ($action eq $ADD_TASK...

   #create semaphore manager object
   baselog($LOG_DEBUG, "Try to take the semaphore '$SEM_CRONTAB'");
   my $semmngr = base_semctl::new("CRONTAB");
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_CRONTAB, IPC_NOWAIT))
   {
      undef $semmngr;
      return $FALSE;
   }

   my $retval = modifyCrontab($cron_cfg, $action, $user, $task_time, $task_cmd);

   baselog($LOG_DEBUG, "Try to release the semaphore '$SEM_CRONTAB'");
   if (!$semmngr->sem_give($SEM_CRONTAB))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_CRONTAB semaphore.");
   }

   # destroy the object
   undef $semmngr;

   return $retval;
} ## end sub doAction

## @fn sub getCrontabCfg($user)
# @brief This subroutine gets a crontab file name
# @details Returns crontab configuration file name for the $user or an
#    empty string in the case of non interactive user. If the crontab
#    configuration file is absence in the home directory, it is created
# @param $user      - user
# @return $cron_cfg - empty string if there is no home directory for
#                     the $user
# @author
# @date
# @note
sub getCrontabCfg
{
   my $user     = shift @_;
   my $cron_cfg = "";

   #get user home directory from /etc/passwd
   my $home_dir = (getpwnam($user))[7];

   baselog($LOG_DEBUG, "HOMEDIR for '$user': '$home_dir'");

   if (-e $home_dir)
   {

      #crontab configuration file for the $user
      $cron_cfg = "$home_dir/.$user.cron";

      do_system([$TOUCH, $cron_cfg]) if (!-e $cron_cfg);
      do_system([$CHOWN, $user, $cron_cfg]);
      do_system([$CHMOD, "640", $cron_cfg]);
   } ## end if (-e $home_dir)
   else
   {
      baselog($LOG_ERROR, "User without home dir invoked cron API");
   }
   return $cron_cfg;
} ## end sub getCrontabCfg

#####################################################################
########################### Public Methods ##########################

## @method sub addCrontabTask($user, $task_time, $task_cmd)
# @brief This method adds a job into crontab for a user
# @details Adds the task into crontab for $user. If such task already
#    exists in the crontab (exact matching), it will be replaced.
# @param $user      - user
# @param $task_time - date and time in crontab notation
# @param $task_cmd  - command for crontab task
# @return $TRUE, if no errors happen during crontab modification,
#         otherwise $FALSE
# @author
# @date
# @note
sub addCrontabTask($$$)
{
   my ($user, $task_time, $task_cmd) = @_;

   return doAction($ADD_TASK, $user, $task_time, $task_cmd);
}

## @method sub removeCrontabTask($user, $task_cmd)
# @brief This method removes a job from crontab
# @details Removes the task running the $task_cmd from crontab for the
#    $user.
#    If a string with the task consist of several pieces and at least
#    one of them is not constant, you have to invoke this subroutine
#    prior to adding new task. For example, the task is a command with
#    arguments: 'sysbackup_task backuptoUSB' or
#    'sysbackup_task backuptoFTP', so, to remove such task $task_cmd has
#    to be 'sysbackup_task'.
# @param $user     - user
# @param $task_cmd - command for crontab task.
# @return $TRUE, if no errors happen during crontab modification,
#         otherwise $FALSE
# @author
# @date
# @note
sub removeCrontabTask($$)
{
   my ($user, $task_cmd) = @_;

   return doAction($REMOVE_TASK, $user, "", $task_cmd);
}

## @method getCrontabTask($user, $task_cmd)
# @brief This function returns the array containing crontab tasks
# @details This function returns an array containing crontab tasks
#          for the given user. If the optional parameter $task_cmd
#          is provided, the array contains only tasks matching with it
# @param $user      - user
# @param $task_cmd  - pattern to scheduled task (optional)
# @return $crontab  - undefined if error occurred
#                   - reference to array with zero length.
#                   - reference to array of links to hashes contains scheduled task
# @author Ilya Kudelin
# @date
# @note
sub getCrontabTask($;$)
{
   my ($user, $task_cmd) = @_;
   if (not defined $task_cmd)
   {
      $task_cmd = ".*";
   }
   my $cron_cfg = getCrontabCfg($user);
   my $cron_task;  #link to hash
   my $crontab = [];  #array of crontab tasks

   if ($cron_cfg eq "")
   {
      return undef;
   }
   if (!open(CRON, "<", $cron_cfg))
   {
      baselog($LOG_ERROR, "Cannot open crontab file for user $user");
      return undef;
   }
   while (my $line = <CRON>)
   {
      if ($line =~ /$task_cmd/)
      {
         my ($min, $hour, $dom, $month, $dow, $task) = split(' ', $line);
         $cron_task = {"min" => $min, "hour" => $hour, "dom" => $dom, "month" => $month, "dow" => $dow, "task" => $task};
         push @$crontab, $cron_task;
      } ## end if ($line =~ /$task_cmd/...
   } ## end while (my $line = <CRON>)
   close(CRON);
   return $crontab;
} ## end sub getCrontabTask($;$)

1;

__END__

=head1 NAME

base_crontab - API for crontab modifications.

=head1 SYNOPSIS

use base_crontab;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
