## @file base_deploy.pm
# @brief This module is Linux Base deploy management library.
# @details
# @author Kate Bereznyakova
# @author Ekaterina Bazhanova
# @date March 2008
# @version 1.00 - Initial version
# @version 1.10 - The following functions are added:
#   @li is_src_and_dst_the_same
#   @li get_ds_preconfig_data
#   @li get_rmt_preconfig_data
#   @li transfer_file function is enhanced to make a local copy
# command if target
# and local host is the same.
# @version 1.20 - The following function is added:
#   @li prepare_deploy_dir
# @version 1.21 - Two colons were removed before shared functions and variables.
# @version 1.22 - The following functions are added:
#   @li op_stat_sync
#   @li rmt_rmv_installxml
#   @li reset_status
# @version 1.23 -  The following functions are added:
#   @li get_rmt_archive_data
#   @li create_archive_data
#     The following functions are removed:
#   @li get_rmt_preconfig_data
#   @li get_ds_preconfig_data
#   @li rmt_clean_up
#   @li rmt_rmv_installxml
# @version 1.24 - Modified under the perlcheck warnings cleanup activity.
# @version 1.25 - function call_remote_cmd was modified (ssh connection timeout was decreased)
# @version 1.26 - function update_section is updated to
#                 to keep old content in the section.
# @version 1.27 - $SSH (call_remote_cmd) & $SCP (transfer_file) -o options moved to common_variables
# @version 1.28 - move -o options back to this file (Q01964307)
# @version 1.29 - ssh keep-alive timeout was increased
# @version 1.30 - new  'operStop' flag is added to deployment object to indicate
#                 major issues to stop the operation and do not try to get
#                 remote data.
#                 New make_deploy_dir() function is added.
# @version 1.31 - "ping" has been substituted by a "ssh" call in the keep_alive_host function.
# @version 1.32 - change call_remote_cmd function to support NOERROR mode.
# @version 1.33 - get_deploy_logs(),  get_deploy_log_files, cleanup_logs_dir  and
#                 get_error_code() functions are added.
# @version 1.34 - prepare_deploy_data()  is modified to add the keycode file in the filelist.
# @version 1.40 - update_deploy_error() function is updated to support new "keep" option.
# @version 1.50 - transfer_files function is updated to provide more information about transfer process

package base_deploy;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use base_xml;
use property;
use install_common;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&gen_deploy_install_xml &get_param
  $SCTAG $DEPLVER $APPLOADPATH $INSTALLDIR &transfer_file
  $DEPLOYEDVER $SERVSTAT $DEPAPP &transfer_deploy_xml $DEPLERROR
  $DEPLOYEDHOSTPATH &get_deploy_file_list &DESTROY $DEPSTATUS
  &retrieve_deploy_xml &update_deploy_status &op_stat_sync &reset_status
  &prepare_file_transfer &call_remote_cmd &is_local_deploy
  &change_deploy_status &transfer_files &prepare_deploy_data &keep_alive_host
  %DEPLOYSTATUS &clean_up $INSTALLDIR &cleanup_ds_tmp_data &prepare_deploy_dir
  &update_deploy_backup %DEPLOYMODE &update_section &get_deploy_cfg_path $KEEPOLDCNT $DEPERRORPATH
  $PRECONFIGDIR $LICENSEDIR $PRECONFIGARCH $LICENSEARCH &create_archive_data &get_rmt_archive_data
  $DEPERRORMSGPATH $KEYCODEFILE $KEYCODEARCH &make_deploy_dir $noERROR &get_deploy_logs
  &get_error_code);

use strict;
use warnings;
use baselog;
use XML::LibXML;
use CommonInstallUtils;
use base_system;
use appinstall::errors;
use File::Basename;

our $VERSION = '1.50';

our $DEPLOYROOT       = '/var/opt/nortel/deployment';
our $DEPLOYEDHOSTPATH = "$DEPLOYROOT/deployed";
our $APPLOADPATH      = "$DEPLOYROOT/app_loads";
our $INSTALLDIR       = 'install';
our $PRECONFIGDIR     = 'preconfig';
our $LICENSEDIR       = 'license';
our $KEYCODEFILE      = 'keycode.kcd';
our $LOGDIR           = 'logs';
my $TMPDIR = 'tmp';  # directory with temporary data.
our $PRECONFIGARCH = 'preconfig.tar';
our $LICENSEARCH   = 'license.tar';
our $KEYCODEARCH   = 'keycode.tar';
our $noERROR       = $TRUE;

#### deployment.xml file tags
our $SCTAG           = 'supportedConfig';
our $PACKAGE         = 'packageToDeploy';
our $DEPLVER         = 'versionToDeploy';
our $DEPLERROR       = 'error';
our $DEPSTATUS       = 'deploymentStatus';
our $DEPERRORPATH    = "/target/$DEPLERROR";
our $DEPERRORMSGPATH = "/target/errorMsg";

#### baseOS.properties variables
our $DEPLOYEDVER = 'deployedVersion';
our $DEPAPP      = 'deployedApplications';
our $SERVSTAT    = 'serverStatus';

#Hash with possible complete statuses of deployment operation.
our %DEPLOYSTATUS = (
                     deployed   => 'Deployed',
                     undeployed => 'Undeployed',
                     none       => 'None'
                    );

our %DEPLOYMODE = (
                   DM => 'DM',
                   BS => 'BS'
                  );

# Flag for update_section function to keep old
# content while adding new information.
our $KEEPOLDCNT = $TRUE;

# ssh options
my @ssh_connection_options =
  ("-q", "-o", "ServerAliveInterval=$SSH_ALIVE_INTERVAL", "-o", "ServerAliveCountMax=$SSH_ALIVE_COUNT_MAX", "-o", "TCPKeepAlive=$SSH_TCP_KEEP_ALIVE", "-o", "ConnectTimeout=$SSH_CONNECT_TO");

## @skipline
sub new($;$);
## @skipline
sub DESTROY;
## @skipline
sub init_object($);
## @skipline
sub get_deploy_cfg_path($);
## @skipline
sub get_param($$);
## @skipline
sub get_ip($);
## @skipline
sub gen_deploy_install_xml($);
## @skipline
sub get_deploy_file_list($);
## @skipline
sub transfer_file($$$;$);
## @skipline
sub update_deploy_status($$);
## @skipline
sub prepare_file_transfer($);
## @skipline
sub call_remote_cmd($$;$$);
## @skipline
sub cleanup_ds_tmp_data($);
## @skipline
sub is_local_deploy($);
## @skipline
sub transfer_deploy_xml($);
## @skipline
sub retrieve_deploy_xml($);
## @skipline
sub clean_up ($);
## @skipline
sub keep_alive_host($$);
## @skipline
sub transfer_files($$);
## @skipline
sub create_slinks($);
## @skipline
sub prepare_deploy_data($);
## @skipline
sub update_deploy_error($$;$);
## @skipline
sub remove_packages_sctag($$);
## @skipline
sub remove_packages_packtag($$);
## @skipline
sub is_src_and_dst_the_same($$);
## @skipline
sub prepare_deploy_dir($);
## @skipline
sub update_section($$$;$);
## @skipline
sub update_deploy_backup($$);
## @skipline
sub op_stat_sync();
## @skipline
sub get_rmt_archive_data($$);
## @skipline
sub reset_status($);
## @skipline
sub create_archive_data($$);
## @skipline
sub make_deploy_dir($$;$);
## @skipline
sub get_deploy_logs($);
## @skipline
sub get_error_code($);
## @skipline
sub get_deploy_log_files($);
## @skipline
sub cleanup_logs_dir($);

###############################################################################
## @method new($hostname, $ip)
# @brief   Deploy object constructor.
# @details This class method creates an object and fills necessary object
#          fields.  Deploy object is tight with a deployment.xml file which
#          is a major data holder and interface between Deployment Manager and
#          LinuxBase APIs.
# @param   $hostname - hostname associated with a new object.
# @param   $ip - IP address associated with the host. This parameter is
#          optional. IP address in mandatory only for the remote deployment.
#          Local deployment and remote-side API's need only hostname to find a
#          deployment data.
# @return  $self - reference to object
# @return  undef - indicates an error condition.
# @author  Kate Bereznyakova
# @date    March 2008
sub new($;$)
{
   my ($hostname, $ip) = @_;
   if ((!defined $hostname) or ($hostname eq ''))
   {
      baselog($LOG_ERROR, 'Hostname is undefined.');
      return undef;
   }

   if (!defined $ip)
   {
      baselog($LOG_INFO, 'IP address is undefined.');
   }

   #initialization
   my %self = (
               'parser'         => undef,         # reference to a parser
               'doc'            => undef,         # parsing document
               'deploymentCfg'  => undef,         # path to deployment.xml file
               'deploymentHome' => undef,         # path to deployment data
               'installHome'    => undef,         # path to install directory
               'hostname'       => $hostname,
               'ip'             => $ip,           # ip-address associated with the object
               'deployAcct'     => $DEPLOY_ACCT,  # default deployment account.
               'params'         => {},            # parameters from deployment.xml
               'operStop'       => $FALSE         # flag to stop the operation
              );

   bless \%self;                         #Declaring an object
   if (!init_object(\%self))
   {
      baselog($LOG_ERROR, 'Cannot initialize an object.');
      undef %self;
      return undef;
   } ## end if (!init_object(\%self...
   return \%self;
} ## end sub new($;$)

###############################################################################
## @method DESTROY($self)
# @brief   Object destructor.
# @details Destructor releases a defined parser.
# @param   $self  - reference to object.
# @return  $TRUE - always returns true.
# @author  Kate Bereznyakova
# @date    March 2008
sub DESTROY
{
   my $self = shift @_;
   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
   }
   elsif (defined $self->{'parser'})
   {

      # To Do: check the release operation.
      baselog($LOG_DEBUG, 'Parser is defined.');
      undef $self->{'parser'};
      baselog($LOG_DEBUG, 'Parser is released.');
      if (defined $self->{'doc'})
      {
         baselog($LOG_DEBUG, 'Object doc is defined.');
         undef $self->{'doc'};
         baselog($LOG_DEBUG, 'Object doc is released.');
      } ## end if (defined $self->{'doc'...
   } ## end elsif (defined $self->{'parser'...
   else
   {
      baselog($LOG_INFO, 'Object parser is undefined.');
   }
   return $TRUE;
} ## end sub DESTROY

###############################################################################
## @method init_object($self)
# @brief   Object initializer.
# @details It sets up object properties according to identified key(hostname).
#          The following properties will be defined:
#   \li deploymentHome:  full path to deployment data (the same on the
#                      deployment server and target host).
#   \li deploymentCfg:   full path to deployment.xml file associated with the
#                      hostname.
#   \li parser:          parser object (to parse deployment.xml file).
#   \li doc:             xml document object associated with  deployment.xml file.
# @param   $self  - reference to object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author  Kate Bereznyakova
# @date    March 2008
sub init_object($)
{
   my $self   = shift @_;
   my $retval = $TRUE;
   baselog($LOG_DEBUG, 'Object initializer.');
   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return $FALSE;
   }

   $self->{'deploymentHome'} = "$DEPLOYEDHOSTPATH/$self->{'hostname'}";
   if (!-d $self->{'deploymentHome'})
   {

      # todo: test this case.
      baselog($LOG_ERROR, "Deploy home directory '$self->{'deploymentHome'}' does not exist.");
      return $FALSE;
   } ## end if (!-d $self->{'deploymentHome'...

   $self->{'deploymentCfg'} = "$self->{'deploymentHome'}/$DEPLOYMENTCFG";

   if (!-e $self->{'deploymentCfg'})
   {
      baselog($LOG_ERROR, "Cannot create object parser: cannot find $self->{'deploymentCfg'} file.");
      return $FALSE;
   }
   $self->{'installHome'} = "$self->{'deploymentHome'}/$INSTALLDIR";

   if (defined $self->{'parser'})
   {
      baselog($LOG_INFO, 'Parser is defined. Re-initialization.');
   }

   # ! todo: Check if these are valid destroy operations.
   undef $self->{'parser'};
   undef $self->{'doc'};

   my $parser = XML::LibXML->new();
   if (!defined $parser)
   {
      baselog($LOG_ERROR, 'Cannot create parser object.');
      $retval = $FALSE;
   }
   else
   {
      $self->{'parser'} = $parser;
      baselog($LOG_DEBUG, 'Parser object is created.');
      my $doc;

      # todo: test this!!!
      # trying to handle die and croak events here.
      eval {$doc = $parser->parse_file($self->{'deploymentCfg'})};
      my $error = $@;
      if ($error)
      {
         baselog($LOG_ERROR, "Cannot create a document object ('$error').");
         undef $self->{'parser'};
         undef $parser;
         $retval = $FALSE;
      } ## end if ($error)
      else
      {
         $self->{'doc'} = $doc;
         baselog($LOG_DEBUG, 'Document object is created.');
      }
   } ## end else [ if (!defined $parser)
   return $retval;
} ## end sub init_object($)

###############################################################################
## @method get_param($self, $nodePath)
# @brief   Retrieves text data from requested section of a deployment.xml file.
# @details If parameters was retrieved earlier then it will be in params hash.
#          This hash will be checked first.
# @param   $self  - reference to deploy object.
# @param   $nodePath - deployment.xml file section(path) to get data from.
# @return  $targetData  - requested text data.
# @return  undef - some error happens
# @author  Kate Bereznyakova
# @date    March 2008
sub get_param($$)
{
   my ($self, $nodePath) = @_;
   if ((!defined $nodePath) or ($nodePath eq ''))
   {
      baselog($LOG_ERROR, 'Parameter path is undefined.');
      return undef;
   }

   baselog($LOG_DEBUG, "Getting data from '$nodePath' section.");

   # todo: test this case.
   if (not exists $self->{'params'}->{$nodePath} and not defined $self->{'params'}->{$nodePath})
   {
      if (!defined $self->{'doc'})
      {
         baselog($LOG_ERROR, 'Document object must be initialized.');
         return undef;
      }

      # note: getElementsByTagName($nodePath)->[0]->textContent() could produce
      # not user friendly case. Should be handled somehow.
      $self->{'params'}->{$nodePath} = $self->{'doc'}->getElementsByTagName($nodePath)->[0]->textContent();
   } ## end if (not exists $self->...
   return $self->{'params'}->{$nodePath};
} ## end sub get_param($$)

###############################################################################
## @method  prepare_file_transfer($self)
# @brief    It prepares the target host for deployment operation.
#           exists. If it does not exist, it will be created.
# @details  None.
# @param    $self  - reference to deploy object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     April 2008
sub prepare_file_transfer($)
{
   my $self    = shift @_;
   my $retval  = $TRUE;
   my $command = "$RMT_DEPLOY_CMD -prepare -hostname $self->{'hostname'}";
   baselog($LOG_DEBUG, "Preparing file transfer (cmd: '$command').");

   my $rcmd = $self->call_remote_cmd($command);

   if ((!defined $rcmd) or ($rcmd != $OK))
   {
      baselog($LOG_ERROR, 'Cannot prepare the remote host.');
      $retval = $FALSE;
   }
   return $retval;
} ## end sub prepare_file_transfer($)

###############################################################################
## @method  gen_deploy_install_xml($self)
# @brief    Generation of install.xml files
# @details  None.
# @param    $self  - reference to object.
# @return   $TRUE - if generation was successful
# @return   $FALSE  - in error case
# @author   Ekaterina Bazhanova
# @date     April 2008
# @note     Function is changed to re-use object's data (by Kate Bereznyakova)
sub gen_deploy_install_xml($)
{
   my $self = shift @_;

   if (!defined $self->{'hostname'})
   {
      baselog($LOG_ERROR, 'Deployment manager property(hostname) is undefined.');
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Getting file list to deploy '$self->{'hostname'}' host.");

   if (!defined $self->{'doc'})
   {
      baselog($LOG_ERROR, 'Document object must be initialized.');
      return $FALSE;
   }

   if (!defined $self->{'installHome'})
   {
      baselog($LOG_ERROR, "Cannot find installation directory.");
      return $FALSE;
   }

   my $requiredConfig = $self->get_param($SCTAG);
   if ((!defined $requiredConfig) or ($requiredConfig eq ''))
   {
      baselog($LOG_ERROR, "Cannot find selected configuration to deploy to '$self->{'hostname'}'.");
      return $FALSE;
   }
   baselog($LOG_INFO, "Requested configuration: '$requiredConfig'");
   my $load = $self->get_param($DEPLVER);
   if ((!defined $load) or ($load eq ''))
   {
      baselog($LOG_ERROR, "Cannot find deploy version for '$self->{'hostname'}'.");
      return $FALSE;
   }
   baselog($LOG_INFO, "Requested version: '$load'");

   my $outFile = "$self->{'installHome'}/$INSTCFG";
   if (!-d $self->{'installHome'})
   {
      if (!mkdir($self->{'installHome'}))
      {
         baselog($LOG_ERROR, 'Cannot create install directory.');
         return $FALSE;
      }
   } ## end if (!-d $self->{'installHome'...
   my $inputFile = "$APPLOADPATH/$load/$INSTCFG";

   my $applications;
   my @requiredApp;
   my $configs;
   my $packages;
   my $required = $FALSE;
   my $install = base_xml->new($inputFile, $outFile);
   if (!defined $install)
   {
      baselog($LOG_ERROR, 'Cannot create base_xml object.');
      return $FALSE;
   }

   #delete unrequired <supportedConfig>
   $configs = $install->findElements('/install/supportedConfig', undef);
   foreach my $config (@$configs)
   {

      #if the current <supportedConfig> element is required create list of the required application
      #in other case remove  the current <supportedConfig> element from install.xml
      if ($config->getAttribute('name') eq $requiredConfig)
      {
         push @requiredApp, $config->getChildrenByTagName('application');
      }
      else
      {
         my $parent = $config->parentNode();
         $parent->removeChild($config);
      }
   } ## end foreach my $config (@$configs...

   #delete unrequired <package>
   if ($self->{'doc'}->getElementsByTagName($PACKAGE)->[0])
   {
      remove_packages_packtag($self->{'hostname'}, $install);
   }
   else
   {
      remove_packages_sctag($install, $requiredConfig);
   }

   #delete unrequired <application>
   $applications = $install->findElements("/install/application");
   foreach my $app (@$applications)
   {
      $required = $FALSE;

      # check if application is required set $required flag
      foreach my $reqApp (@requiredApp)
      {
         if ($app->getAttribute('name') eq $reqApp->getAttribute('name'))
         {
            $required = $TRUE;  # set $required flag
            last;
         }
      }  ##end foreach my $reqApp...

      #if application is not required
      #delete the appropriate <application> element from install.xml
      unless ($required)
      {
         my $parent = $app->parentNode();
         $parent->removeChild($app);
      }
   } ## end foreach my $app (@$applications...

   $install->save();
   return $TRUE;
} ## end sub gen_deploy_install_xml($)

###############################################################################
## @fn       remove_packages_packtag ($hostname, $install)
# @brief     Remove all unrequired packages using information from
#            "packageToDeploy" tag of deployment.xml file
# @details   None.
# @param     $hostname - hostname
# @param     $install - reference to base_xml object
# @return    $TRUE - packages is removed
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
sub remove_packages_packtag($$)
{
   my ($hostname, $install) = @_;
   my $retval = $TRUE;
   my $packages;
   my $required_pack_nodes;
   my @required_packages = ();

   unless ($hostname)
   {
      baselog($LOG_ERROR, 'Hostname is undefined');
      return $FALSE;
   }
   unless ($install)
   {
      baselog($LOG_ERROR, 'Reference to base_xml object is undefined');
      return $FALSE;
   }

   my $deployment_xml = "$DEPLOYEDHOSTPATH/$hostname/$DEPLOYMENTCFG";
   my $doc            = base_xml->new($deployment_xml);

   $required_pack_nodes = $doc->findElements("/target/packageToDeploy");
   foreach my $node (@$required_pack_nodes)
   {
      push @required_packages, $node->textContent();
   }

   #delete unrequired <package>
   $packages = $install->findElements("/install/package");
   foreach my $package (@$packages)
   {
      my $pack_name = $package->getAttribute('name');
      my $required  = $FALSE;
      foreach my $required_package (@required_packages)
      {
         if ($required_package eq $pack_name)
         {
            $required = $TRUE;
         }
      } ## end foreach my $required_package...

      #if the current <package> element is not appropriate to required configuration
      # delete the current <package> element from install.xml file
      if ($required)
      {
         $package->setAttribute('deployed', $TRUE);
      }
      else
      {
         my $parent = $package->parentNode();
         $parent->removeChild($package);
      }
   } ## end foreach my $package (@$packages...

   return $retval;
} ## end sub remove_packages_packtag($$)

###############################################################################
## @fn       remove_packages_sctag ($install, $requiredConfig )
# @brief     Remove all unrequired packages using information from
#            required configuration string
# @details   None.
# @param     $install - reference to base_xml object
# @param     $requiredConfig - required configuration string
# @return    $TRUE - packages is removed
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
sub remove_packages_sctag($$)
{
   my ($install, $requiredConfig) = @_;
   my $retval = $TRUE;
   my $packages;
   my $required_pack_nodes;
   my @required_packages = ();

   unless ($install)
   {
      baselog($LOG_ERROR, 'Reference to base_xml object is undefined');
      return $FALSE;
   }

   #delete unrequired <package>
   $packages = $install->findElements("/install/package");
   foreach my $package (@$packages)
   {
      my $pack_name = $package->getAttribute('name');
      my $required  = $FALSE;

      if ($requiredConfig =~ /(\W|\A)$pack_name(\W|\Z)/)
      {
         $required = $TRUE;
      }

      #if the current <package> element is not appropriate to required configuration
      # delete the current <package> element from install.xml file
      if ($required)
      {
         $package->setAttribute('deployed', $TRUE);
      }
      else
      {
         my $parent = $package->parentNode();
         $parent->removeChild($package);
      }
   } ## end foreach my $package (@$packages...

   return $retval;
} ## end sub remove_packages_sctag($$)

###############################################################################
## @method  call_remote_cmd($self, $command, $output, $noerror)
# @brief    It calls remote command and puts error output to a log file.
# @details  None.
# @param    $self  - reference to object.
# @param    $command - command to call.
# @param    $output - reference to array where output should be put. This
#                     parameter is optional.
# @param    $noerror - if $TRUE - call do_system with $dsNOERROR flag. This
#                     parameter is optional.
# @return   undef  - error condition
# @return   $retval - value returned by a remote command.
# @author   Kate Bereznyakova
# @date     April 2008
sub call_remote_cmd($$;$$)
{
   my ($self, $command, $output, $noerror) = @_;
   my $retval = undef;

   if (!defined $command)
   {
      baselog($LOG_ERROR, 'Command is undefined');
      return $retval;
   }

   baselog($LOG_DEBUG, "Remote command call: '$command'");

   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Deploy object is undefined');
      return $retval;
   }

   if ((!defined $noerror) or (!($noerror eq $TRUE)))
   {
      $noerror = $FALSE;
   }
   my $ip = $self->{'ip'};
   if (!defined $ip)
   {
      baselog($LOG_ERROR, 'IP address is undefined. Cannot use SSH call.');
      return $retval;
   }

   if (!defined $self->{'deployAcct'})
   {
      baselog($LOG_ERROR, 'Deploy account is undefined. Cannot use SSH call.');
      return $retval;
   }

   my $stdout;
   my $stderr;
   my $is_local = $self->is_local_deploy();

   # if type of operation: local or remote cannot be identified - use remote
   # by default.
   if ((defined $is_local) and ($is_local))
   {
      baselog($LOG_INFO, 'Local operation is requested: switching to a regular call.');
      do_system($command, {$dsRETVAL => \$retval, $dsOUT => \@$stdout, $dsERR => \@$stderr, $dsNOERROR => $noerror});
   }
   else
   {
      if (keep_alive_host($self->{'deployAcct'}, $ip))
      {
         baselog($LOG_INFO, 'Using ssh-call to execute requested command.');
         do_system([$SSH, @ssh_connection_options, "$self->{'deployAcct'}\@$ip", $command], {$dsRETVAL => \$retval, $dsOUT => \@$stdout, $dsERR => \@$stderr, $dsNOERROR => $noerror});
      }
      else
      {
         baselog($LOG_WARNING, 'Cannot check remote server status.');
         return $retval;
      }

   } ## end else [ if ((defined $is_local...
   if (scalar @$stderr)
   {
      baselog($LOG_WARNING, "@$stderr");
      if ($output)
      {
         push @$output, @$stderr;
      }
   } ## end if (scalar @$stderr)
   if (scalar @$stdout)
   {
      baselog($LOG_DEBUG, "@$stdout");
      if ($output)
      {
         push @$output, @$stdout;
      }
   } ## end if (scalar @$stdout)

   if (defined $retval)
   {
      baselog($LOG_DEBUG, "Remote command returned: '$retval'.");
   }
   return $retval;
} ## end sub call_remote_cmd($$;$$)

###############################################################################
## @method  transfer_file($self, $srcFile, $dstDir, $download)
# @brief    It copies a file to a remote host.
# @details  None.
# @param    $self  - reference to object.
# @param    $srcFile - source file that needs to be transferred.
# @param    $dstDir  - destination - where file should be transferred.
# @param    $download - optional parameter. Defined download direction:
#                       if it set to $TRUE (by default), upload operation
#                       will be performed
#                       if it set to $FALSE, download operation will be
#                       performed.  upload: file from remote machine will be
#                       uploaded.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     April 2008
sub transfer_file($$$;$)
{
   my ($self, $srcFile, $dstDir, $download) = @_;
   my $retval = $TRUE;
   my $cmd_retval;

   if (!defined $srcFile)
   {
      baselog($LOG_ERROR, 'Source file is undefined.');
      return $FALSE;
   }

   if (!defined $dstDir)
   {
      baselog($LOG_ERROR, 'Destination directory is undefined.');
      return $FALSE;
   }

   baselog($LOG_INFO, "Transfering '$srcFile' file.");

   if (!defined $download)
   {
      $download = $TRUE;
   }

   if (!defined $self->{'deployAcct'})
   {
      baselog($LOG_ERROR, 'Deploy account is undefined.');
      return $FALSE;
   }

   if (!defined $self->{'ip'})
   {
      baselog($LOG_ERROR, 'IP address is undefined.');
      return $FALSE;
   }

   if ($download and (!-e $srcFile))
   {
      baselog($LOG_ERROR, 'Source file does not exist.');
      return $FALSE;
   }

   # if this is a local operation then use a "cp" command
   # if source and target is the same then skip copy operation.
   # if type of operation: local or remote cannot be identified - use remote
   # by default.
   my $is_local = $self->is_local_deploy();
   if ((defined $is_local) and ($is_local))
   {
      if (is_src_and_dst_the_same($srcFile, $dstDir))
      {
         baselog($LOG_INFO, "Do not copy file to itself ('$srcFile'). Ignore the operation.");
      }
      elsif (!do_system([$CP, '-p', $srcFile, $dstDir]))
      {
         baselog($LOG_ERROR, "Failed to copy '$srcFile' file to '$dstDir' directory.");
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_DEBUG, "'$srcFile' file successfully copied to '$dstDir' directory.");
      }
   } ## end if ((defined $is_local...
   else
   {
      my ($src, $dst);
      if ($download)
      {
         $dst = $self->{'deployAcct'} . '@' . $self->{'ip'} . ":$dstDir";
         $src = $srcFile;
      }
      else
      {
         $src = $self->{'deployAcct'} . '@' . $self->{'ip'} . ":$srcFile";
         $dst = $dstDir;
      }

      if (!do_system([$SCP, @ssh_connection_options, $src, $dst], {$dsRETVAL => \$cmd_retval}))
      {
         baselog($LOG_ERROR, "Failed to transfer '$src' file to '$dst' [$cmd_retval].");

         #change error code at deployment.xml file only if it is clean. Do not replace
         #existing error code (if final sync operation fail).
         my $temp_error = get_error_code($self->{'deploymentCfg'});
         if ((defined $temp_error) and ($temp_error eq ''))
         {
            update_section($self->{'deploymentCfg'}, $DEPERRORPATH, 'TRNSF');
         }
         update_section($self->{'deploymentCfg'}, $DEPERRORMSGPATH, "Failed to transfer $srcFile to target.", $KEEPOLDCNT);
         $retval = $FALSE;
      } ## end if (!do_system([$SCP, ...
   } ## end else [ if ((defined $is_local...
   return $retval;
} ## end sub transfer_file($$$;$)

###############################################################################
## @fn   is_src_and_dst_the_same($src, $dst)
# @brief   It checks if the source file is the same as a destination.
# @details This function can be used to avoid attempt to copy the file to the
#          same directory under the same name.
# @param   $src  - source file with the full path
# @param   $dst  - destination directory of file with the full path
# @return  $TRUE - the source and destination is the same.
# @return  $FALSE - source and destination are not the same.
# @author  Kate Bereznyakova
# @date    May 2008
sub is_src_and_dst_the_same($$)
{
   my ($src, $dst) = @_;
   my $retval = $FALSE;
   if ($src eq $dst)
   {
      $retval = $TRUE;
   }
   else
   {
      my $filename = basename($src);
      if ($src eq "$dst/$filename")
      {
         $retval = $TRUE;
      }
   } ## end else [ if ($src eq $dst)
   return $retval;
} ## end sub is_src_and_dst_the_same($$)

###############################################################################
## @method   get_deploy_file_list($self)
# @brief   Get list of rpm files for deploy
# @details None.
# @param   $self  - reference to object.
# @return  $filelist - reference to a list of rpm files for deploy
# @return  undef - indicates an error condition.
# @author  Ekaterina Bazhanova
# @date    April 2008
sub get_deploy_file_list($)
{
   my $self = shift @_;
   baselog($LOG_DEBUG, "Getting file list to deploy for '$self->{'hostname'}' host.");
   my @file_nodes = ();
   my @filelist   = ();
   my $full_file_path;
   my $doc;

   if (!defined $self->{'doc'})
   {
      baselog($LOG_ERROR, 'Document object must be initialized.');
      return undef;
   }

   my $load = $self->get_param($DEPLVER);
   if (!defined $load)
   {
      baselog($LOG_ERROR, "Cannot find deploy version for '$self->{'hostname'}'.");
      return undef;
   }

   if (!defined $self->{'installHome'})
   {
      baselog($LOG_ERROR, "Cannot find installation directory.");
      return undef;
   }
   my $deploymentCfg = "$self->{'installHome'}/$INSTCFG";

   unless (-e $deploymentCfg)
   {
      baselog($LOG_ERROR, "Cannot find $deploymentCfg");
      return undef;
   }

   my $retval = undef;

   my $parser = XML::LibXML->new();
   eval {$doc = $parser->parse_file($deploymentCfg)};
   my $error = $@;
   if ($error)
   {
      baselog($LOG_ERROR, "Cannot create document object ($error).");
   }
   else
   {
      @file_nodes = $doc->getElementsByTagName('file');
      foreach my $file_node (@file_nodes)
      {
         if ($file_node->getAttribute('name'))
         {
            my $file = $file_node->getAttribute('name');
            $full_file_path = "$APPLOADPATH/$load/$file";
            push @filelist, $full_file_path;
         } ## end if ($file_node->getAttribute...
      } ## end foreach my $file_node (@file_nodes...

      if (scalar @filelist)
      {
         push @filelist, $deploymentCfg;
         $retval = \@filelist;
      }
      else
      {
         baselog($LOG_ERROR, 'There is no files to deploy.');
      }
   } ## end else [ if ($error)
   undef $doc;
   return $retval;
} ## end sub get_deploy_file_list($)

###############################################################################
## @method   create_slinks($self)
# @brief   Get list of rpm files for deploy and create
#          symbolic links to these files.
# @details None.
# @param   $self  - reference to object.
# @return  @filelist - list of rpm files for deploy
# @return  undef - indicates an error condition.
# @author  Ekaterina Bazhanova
# @date    April 2008
sub create_slinks($)
{
   my $self = shift @_;
   baselog($LOG_DEBUG, "Getting file list to deploy for '$self->{'hostname'}' host..");
   my @file_nodes = ();
   my @filelist   = ();
   my @linklist   = ();  #list of created links for roll-back purpose
   my $full_file_path;
   my $doc;

   if (!defined $self->{'doc'})
   {
      baselog($LOG_ERROR, 'Document object must be initialized.');
      return undef;
   }

   my $load = $self->get_param($DEPLVER);
   if (!defined $load)
   {
      baselog($LOG_ERROR, "Cannot find deploy version for '$self->{'hostname'}'.");
      return undef;
   }
   if (!defined $self->{'installHome'})
   {
      baselog($LOG_ERROR, "Cannot find installation directory.");
      return undef;
   }

   # create link to license data
   if (-d "$APPLOADPATH/$load/$LICENSEDIR")
   {
      if (do_system([$LN, '-s', "$APPLOADPATH/$load/$LICENSEDIR", "$self->{'installHome'}/$LICENSEDIR"]))
      {
         baselog($LOG_DEBUG, 'Link to license data has been created sucessully.');
      }
      else
      {
         baselog($LOG_ERROR, 'Failed to create link to license data.');
         return undef;
      }
   } ## end if (-d "$APPLOADPATH/$load/$LICENSEDIR"...
   my $deploymentCfg = "$self->{'installHome'}/$INSTCFG";
   my $parser        = XML::LibXML->new();

   unless (-e $deploymentCfg)
   {
      baselog($LOG_ERROR, "Cannot find $deploymentCfg");
      return undef;
   }

   eval {$doc = $parser->parse_file($deploymentCfg)};
   my $error = $@;
   if ($error)
   {
      baselog($LOG_ERROR, "Cannot create document object ($error).");
      return undef;

   } ## end if ($error)

   @file_nodes = $doc->getElementsByTagName('file');
   foreach my $file_node (@file_nodes)
   {
      if ($file_node->getAttribute('name'))
      {
         my $file = $file_node->getAttribute('name');
         my $link = "$self->{'installHome'}/$file";
         $full_file_path = "$APPLOADPATH/$load/$file";
         push @filelist, $full_file_path;
         push @linklist, $link;

         # create symbolic links
         my $symlink_exists = eval {symlink("$full_file_path", "$link"); 1};
         unless ($symlink_exists)
         {
            baselog($LOG_ERROR, "Cannot create $link: all created links will be deleted");
            foreach my $created_link (@linklist)
            {
               if (do_system([$RM, '-rf', $created_link]))
               {
                  baselog($LOG_INFO, "$created_link was deleted");
               }
            } ## end foreach my $created_link (@linklist...
            return undef;
         } ## end unless ($symlink_exists)
      } ## end if ($file_node->getAttribute...
   } ## end foreach my $file_node (@file_nodes...

   if (scalar @filelist)
   {
      push @filelist, $deploymentCfg;
      return @filelist;
   }
   else
   {
      baselog($LOG_ERROR, 'There is no files to deploy.');
      return undef;
   }

} ## end sub create_slinks($)

###############################################################################
## @method  update_deploy_status($self, $status)
# @brief    It updates deploy status information in deployment.xml file.
# @details  None.
# @param    $self  - reference to object.
# @param    $status - new deployment status (string).
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Ekaterina Bazhanova
# @date     April 2008
sub update_deploy_status($$)
{
   my ($self, $status) = @_;
   return update_section($self->{'deploymentCfg'}, "/target/deploymentStatus", $status);
}

###############################################################################
## @method  update_deploy_error($self, $error_code, $keep)
# @brief    It updates deploy error information in deployment.xml file.
# @details  None.
# @param    $self  - file to update.
# @param    $error_code  - error description.
# @param    $keep - parameter to define if old code has a priority over the
#           new one or not.
#           If set to TRUE then old code has higher priority then new one.
#           If set to FALSE then new code has higher priority then old one.
#           If parameter is not defined (skipped) then
#           new error code has a top priority and replaces the old one.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bazhanova
# @date     April 2008
sub update_deploy_error($$;$)
{
   my ($self, $error_code, $keep) = @_;
   if (defined $keep and $keep)
   {
      my $temp_error = get_error_code($self->{'deploymentCfg'});
      if ((defined $temp_error) and ($temp_error ne ''))
      {
         baselog($LOG_INFO, "Error code already set to '$temp_error'. Skipping update.");
         return $FALSE;
      }
   } ## end if (defined $keep and ...
   return update_section($self->{'deploymentCfg'}, $DEPERRORPATH, $error_code);
} ## end sub update_deploy_error($$;$)

###############################################################################
## @fn  update_section($file, $section, $newval, $add)
# @brief    It updates section in xml file with provided value.
# @details  None.
# @param    $file  - file to update.
# @param    $section - section to update
# @param    $newval  - new value
# @param    $add     - parameter to defined if keep an old content and add
#                      new to existing one or to replace old content.
#                 \li if defined and set to TRUE then action is add content.
#                 \li otherwise to replace old content with new on.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
sub update_section($$$;$)
{
   my ($file, $section, $newval, $add) = @_;
   my $retval = $TRUE;

   if (!defined $file)
   {
      baselog($LOG_ERROR, "File name is undefined.");
      return $FALSE;
   }

   if (!-e $file)
   {
      baselog($LOG_ERROR, "File '$file' does not exist.");
      return $FALSE;
   }

   if (!defined $section)
   {
      baselog($LOG_ERROR, 'Section is undefined.');
      return $FALSE;
   }

   if (!defined $newval)
   {
      baselog($LOG_ERROR, 'New value is undefined.');
      return $FALSE;
   }

   my $doc = base_xml->new($file);
   unless ($doc)
   {
      baselog($LOG_ERROR, 'Cannot create base_xml object.');
      return $FALSE;
   }

   # make a new value as combination of old one and new one.
   if ((defined $add) and $add)
   {
      my $update_content = $doc->getText($section);
      if ((defined $update_content) and ($update_content ne ""))
      {
         $newval = $update_content . " $newval";
      }

      baselog($LOG_DEBUG, "New content is: $newval");
   } ## end if ((defined $add) and...

   if ($doc->changeText($section, undef, $newval))
   {
      if ($doc->save())
      {
         baselog($LOG_INFO, "'$section' section in '$file' file has been changed to '$newval' successfully.");
      }
      else
      {
         baselog($LOG_ERROR, "Cannot save changes ('$newval') in '$section' section at '$file' file");
         $retval = $FALSE;
      }
   } ## end if ($doc->changeText($section...
   else
   {
      baselog($LOG_ERROR, "Failed to update '$section' section in '$file' file with '$newval' value");
      $retval = $FALSE;
   }
   return $retval;
} ## end sub update_section($$$;$)
###############################################################################
## @method  update_deploy_backup($self, $archive_name)
# @brief    It updates deploy backup information in deployment.xml file.
# @details  None.
# @param    $self  - reference to object.
# @param    $archive_name  - name of the backup archive.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bazhanova
# @date     April 2008
sub update_deploy_backup($$)
{
   my ($self, $archive_name) = @_;
   return update_section($self->{'deploymentCfg'}, "/target/lastBackupName", $archive_name);
}

###############################################################################
## @fn op_stat_sync()
# @brief This function updates 'deploymentStatus', 'supportedConfig' and
# 'versionToDeploy' in the deployment.xml file related to the current host
# with data from baseOs.properties file.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub op_stat_sync()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, 'Updating deployment.xml file (op_stat_sync).');

   # get current host name
   my $hostname = undef;
   if (!do_system([$HOSTNAME, '-s'], {$dsOUT => \@$hostname}))
   {
      baselog($LOG_ERROR, 'Cannot define hostname.');
      return $FALSE;
   }

   chomp($hostname = @$hostname[0]);
   my $deployment_xml = get_deploy_cfg_path($hostname);

   #sync-ing deployment status
   my $prop = property::get_property_value($BASEOSPROP, $SERVSTAT);
   if (!defined $prop)
   {
      baselog($LOG_ERROR, "Deployed status in '$BASEOSPROP' file at '$hostname' is unknown. Failed to update status in '$deployment_xml'.");
      return $FALSE;
   }

   if (!update_section($deployment_xml, "/target/deploymentStatus", $prop))
   {
      baselog($LOG_ERROR, "Failed to update '$deployment_xml' with new status: '$prop'.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "'$deployment_xml' file updated with new status: '$prop'.");

   #sync-ing data about installed applications.
   $prop = property::get_property_value($BASEOSPROP, $DEPAPP);
   if (!defined $prop)
   {
      baselog($LOG_ERROR, "'$DEPAPP' in '$BASEOSPROP' file at '$hostname' is unknown. Failed to update installed applications in '$deployment_xml'.");
      return $FALSE;
   }
   if (!update_section($deployment_xml, "/target/supportedConfig", $prop))
   {
      baselog($LOG_ERROR, "Failed to update '$deployment_xml' with '$prop'.");
      return $FALSE;
   }

   $prop = property::get_property_value($BASEOSPROP, $DEPLOYEDVER);
   if (!defined $prop)
   {
      baselog($LOG_ERROR, "'$DEPLOYEDVER' in '$BASEOSPROP' file at '$hostname' is unknown. Failed to update installed application version in '$deployment_xml'.");
      return $FALSE;
   }
   if (!update_section($deployment_xml, "/target/versionToDeploy", $prop))
   {
      baselog($LOG_ERROR, "Failed to update '$deployment_xml' with '$prop'.");
      return $FALSE;
   }

   return $retval;
} ## end sub op_stat_sync()

###############################################################################
## @fn      change_deploy_status($hostname)
# @brief    It updates deploy status information in deployment.xml
#           and baseOs.properties files.
# @details  None.
# @param    $hostname - hostname
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bazhanova
# @date     April 2008
sub change_deploy_status($)
{
   my $hostname       = shift @_;
   my $retval         = $TRUE;
   my $deployment_xml = get_deploy_cfg_path($hostname);
   my $tmp_baseOs     = dirname($deployment_xml) . "/baseOs_tmp.$$";

   baselog($LOG_INFO, 'Updating baseOs.properties file (change_deploy_status).');

   # get information about installed nortel applications
   my ($app_installed, $app_version) = ('', '');
   my $dep_status = $DEPLOYSTATUS{undeployed};

   my $params = use_confver($CONFVER);
   if ($params)
   {
      $dep_status = $DEPLOYSTATUS{deployed};
      if (defined($params->{'APP_INSTALLED'}))
      {
         $app_installed = $params->{'APP_INSTALLED'};
      }
      if (defined($params->{'APP_VERSION'}))
      {
         $app_version = $params->{'APP_VERSION'};
      }
   } ## end if ($params)

   baselog($LOG_DEBUG, "Updating '$tmp_baseOs' with: $dep_status, $app_installed, $app_version.");

   # make a temp. copy of baseOS.properties file
   if (!do_system([$CP, "-p", $BASEOSPROP, $tmp_baseOs]))
   {
      baselog($LOG_ERROR, "Failed to create temporary copy: '$tmp_baseOs'. Can not update '$BASEOSPROP' file.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Temporary copy: '$tmp_baseOs' is created.");

   unless (property::set_property_value($tmp_baseOs, $SERVSTAT, $dep_status))
   {
      $retval = $FALSE;
   }
   unless (property::set_property_value($tmp_baseOs, $DEPLOYEDVER, $app_version))
   {
      $retval = $FALSE;
   }
   unless (property::set_property_value($tmp_baseOs, $DEPAPP, $app_installed))
   {
      $retval = $FALSE;
   }

   # replacing original baseOS.properties file with updated one
   if (!do_system([$MV, $tmp_baseOs, $BASEOSPROP]))
   {
      baselog($LOG_ERROR, "Failed to replace '$BASEOSPROP' with updated copy '$tmp_baseOs'.");
      $retval = $FALSE;
   }

   if ($retval)
   {
      baselog($LOG_INFO, "Deployment status of '$hostname' has been updated successfully.");
   }
   else
   {
      baselog($LOG_WARNING, "Deployment status of '$hostname' was not updated successfully.");
   }

   return $retval;

} ## end sub change_deploy_status($)

###############################################################################
## @method  is_local_deploy($self)
# @brief    It checks if deploy operation is local or remote.
# @details  If the hostname defined in the deployment manager object matches to
#           the current host's name then local operation must be performed.
# @param    $self  - reference to object.
# @return   $TRUE  - deploy target server is local host.
# @return   $FALSE - deploy target server is remote host.
# @return   undef - check failed.
# @author   Kate Bereznyakova
# @date     April 2008
sub is_local_deploy($)
{
   my $self   = shift @_;
   my $retval = $FALSE;
   if (!defined $self->{'hostname'})
   {
      baselog($LOG_ERROR, 'Target hostname is undefined.');
      $retval = undef;
   }
   else
   {
      my $hostname = undef;
      baselog($LOG_DEBUG, "Target host is '$self->{'hostname'}'.");
      if (!do_system([$HOSTNAME, '-s'], {$dsOUT => \@$hostname}))
      {
         baselog($LOG_ERROR, 'Cannot define hostname.');
         $retval = undef;
      }
      else
      {
         chomp($hostname = @$hostname[0]);
         baselog($LOG_DEBUG, "Current host is '$hostname'.");
         if ($self->{'hostname'} eq $hostname)
         {
            baselog($LOG_DEBUG, 'This is a Local deployment.');
            $retval = $TRUE;
         }
         else
         {
            baselog($LOG_DEBUG, 'This is a remote deployment.');
         }
      } ## end else [ if (!do_system([$HOSTNAME...
   } ## end else [ if (!defined $self->{'hostname'...
   return $retval;
} ## end sub is_local_deploy($)

###############################################################################
## @method  transfer_deploy_xml($self)
# @brief    It copies deployment.xml file to the target host.
# @details  None.
# @param    $self  - reference to object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     April 2008
# @note     Assumption: remote target directory exists.
sub transfer_deploy_xml($)
{
   my $self   = shift @_;
   my $retval = $FALSE;

   my $is_local = $self->is_local_deploy();
   if ((defined $is_local) and ($is_local))
   {
      baselog($LOG_INFO, 'Local operation. Transfer action is skipped.');
      return $TRUE;
   }

   if (!defined $self->{'deploymentCfg'})
   {
      baselog($LOG_ERROR, 'Deployment config. file is not defined.');
      return $retval;
   }
   if (!defined $self->{'deploymentHome'})
   {
      baselog($LOG_ERROR, 'Deployment home directory is not defined.');
      return $retval;
   }

   baselog($LOG_DEBUG, 'Uploading deployment.xml file to the remote host.');
   unless (-e $self->{'deploymentCfg'})
   {
      baselog($LOG_ERROR, 'Deployment configuration file does not exist.');
      return $retval;
   }

   # check if host directory exists on the target
   # and create it if required
   my $cmd = $self->call_remote_cmd("$TEST -d '$self->{'deploymentHome'}'", undef, $noERROR);
   if (!defined $cmd)
   {
      baselog($LOG_ERROR, 'Cannot check remote side home.');
      return $FALSE;
   }
   elsif ($cmd ne $OK)
   {
      my $make_dir = $self->call_remote_cmd("$MKDIR -p $self->{'deploymentHome'}");
      if ((defined $make_dir) and ($make_dir eq $OK))
      {
         baselog($LOG_INFO, 'Host directory has been created.');
      }
      else
      {
         baselog($LOG_ERROR, 'Cannot create host directory.');
         return $FALSE;
      }
   } ## end elsif ($cmd ne $OK)
   if (!$self->transfer_file($self->{'deploymentCfg'}, $self->{'deploymentHome'}))
   {
      baselog($LOG_ERROR, "Cannot copy $self->{'deploymentCfg'} to the target.");
   }
   else
   {
      baselog($LOG_INFO, "File $self->{'deploymentCfg'} is copied to the target.");
      $retval = $TRUE;
   }
   return $retval;
} ## end sub transfer_deploy_xml($)

###############################################################################
## @method  retrieve_deploy_xml($self)
# @brief    It gets deployment.xml file from the target host.
# @details  None.
# @param    $self  - reference to object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     April 2008
sub retrieve_deploy_xml($)
{
   my $self   = shift @_;
   my $retval = $FALSE;

   my $is_local = $self->is_local_deploy();
   if ((defined $is_local) and ($is_local))
   {
      baselog($LOG_INFO, 'Local operation. Retrieve action is skipped.');
      return $TRUE;
   }

   if (!defined $self->{'deploymentCfg'})
   {
      baselog($LOG_ERROR, 'Deployment config. file is not defined.');
      return $retval;
   }
   if (!defined $self->{'deploymentHome'})
   {
      baselog($LOG_ERROR, 'Deployment home directory is not defined.');
      return $retval;
   }

   if (!-d $self->{'deploymentHome'})
   {
      baselog($LOG_WARNING, "$self->{'deploymentHome'} directory does not exist.");
      if (!mkdir($self->{'deploymentHome'}))
      {
         baselog($LOG_ERROR, 'Cannot create deployment home directory.');
         return $FALSE;
      }
   } ## end if (!-d $self->{'deploymentHome'...
   baselog($LOG_DEBUG, 'Downloading deployment.xml file from the remote host.');

   if (!$self->transfer_file($self->{'deploymentCfg'}, $self->{'deploymentHome'}, $FALSE))
   {
      baselog($LOG_ERROR, "Cannot get $self->{'deploymentCfg'}.");
   }
   else
   {
      baselog($LOG_INFO, "File $self->{'deploymentCfg'} is downloaded.");
      $retval = $TRUE;
   }
   return $retval;
} ## end sub retrieve_deploy_xml($)

###############################################################################
## @fn       clean_up($hostname)
# @brief     Clearing of the deployment directory
# @details   Delete all files which can be deleted from the deployment directory
#            excepting of install.xml file
# @param     $hostname - host name
# @return    $TRUE - if the directory was cleared
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
sub clean_up ($)
{
   my $hostname = shift @_;
   my $directory;
   my $retval = $TRUE;

   if ($hostname)
   {
      $directory = "$DEPLOYEDHOSTPATH/$hostname/install";
   }
   else
   {
      baselog($LOG_ERROR, 'Hostname is undefined');
      return $FALSE;
   }

   if ($directory)
   {
      baselog($LOG_INFO, "Removig install directory...");
      if (do_system([$RM, '-rf', $directory]))
      {
         baselog($LOG_INFO, 'Install directory has been removed successfully.');
      }
      else
      {
         baselog($LOG_ERROR, "Failed to remove install directory.");
      }
   } ## end if ($directory)
   else
   {
      baselog($LOG_DEBUG, 'Install directory is already removed.');
   }
   return $retval;
} ## end sub clean_up ($)

###############################################################################
## @method  transfer_files($self, $filelist)
# @brief    It copies files to a remote host.
# @param    $self - reference to a deployment manager object.
# @param    $filelist - reference to a file list that need to copy.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     April 2008
sub transfer_files($$)
{
   my ($self, $filelist) = @_;
   my $retval = $TRUE;

   # check required parameters.
   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Deployment manager object is undefined.');
      return $FALSE;
   }
   if (!defined $self->{'hostname'})
   {
      baselog($LOG_ERROR, 'Deployment manager property(hostname) is undefined.');
      return $FALSE;
   }

   if (!defined $self->{'installHome'})
   {
      baselog($LOG_ERROR, 'Installation directory is undefined.');
      return $FALSE;
   }
   ##end check required parameters.

   if (!$self->update_deploy_status('File transfer is in progress...'))
   {
      baselog($LOG_ERROR, 'Cannot change status in the deployment.xml file.');
   }

   # target installation directory to use in appinstall
   my $dst = $self->{'installHome'};

   if (!$self->prepare_file_transfer())
   {
      baselog($LOG_ERROR, 'Cannot create a installation directory on the remote host.');
      return $FALSE;
   }

   my $filenumber = 0;
   my $totalnum   = scalar(@$filelist);

   foreach my $file (@$filelist)
   {
      if (!$self->transfer_file($file, $dst))
      {
         baselog($LOG_ERROR, "Cannot transfer '$file' to the target host.");
         $retval = $FALSE;
         last;
      } ## end if (!$self->transfer_file...
      else
      {
         $filenumber++;
         if (!$self->update_deploy_status("File transfer is in progress ... ($filenumber out of $totalnum completed)."))
         {
            baselog($LOG_ERROR, 'Cannot change status in the deployment.xml file.');
         }

         baselog($LOG_INFO, "Transferred file: '$file'.");
      } ## end else [ if (!$self->transfer_file...
   } ## end foreach my $file (@$filelist...
   return $retval;
} ## end sub transfer_files($$)

###############################################################################
## @method pre_deploy_clean_up($self)
# @brief It cleans up "old" deploy data on the deployment server side.
# @details  Deployment uses /var/opt/nortel/deployment/deployed/<HostX>/install/
#           directory to keep most of dynamically generated installation data:
#           soft links to rpms and preconfig.tar archive (in local deploy case),
#           install.xml file. These data must be removed before generate new
#           content.
# @param    $self  - reference to deployment manager object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author  Kate Bereznyakova
# @date    June 2008
sub pre_deploy_clean_up($)
{
   my $self   = shift @_;
   my $retval = $FALSE;
   if (!defined $self->{'installHome'})
   {
      baselog($LOG_ERROR, "Cannot find installation directory.");
   }
   elsif (!-d $self->{'installHome'})
   {
      baselog($LOG_DEBUG, "$self->{'installHome'} does not exist.");
      $retval = $TRUE;
   }
   else
   {
      if (!do_system([$RM, '-rf', $self->{'installHome'}]))
      {
         baselog($LOG_ERROR, "Fail to clean up install directory ($self->{'installHome'}).");
      }
      else
      {
         baselog($LOG_DEBUG, "Successful clean up of install directory ($self->{'installHome'}).");
         if (do_system([$MKDIR, '-p', $self->{'installHome'}]))
         {
            baselog($LOG_DEBUG, 'Deployment directory has been created successfully');
            $retval = $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, 'Deployment directory has not been created');
         }
      } ## end else [ if (!do_system([$RM, '-rf'...
   } ## end else [ if (!defined $self->{'installHome'...
   return $retval;
} ## end sub pre_deploy_clean_up($)

###############################################################################
## @method prepare_deploy_data($self)
# @brief   It prepares data for the deployment operation.
# @details Application installation requires some data for deployment.
# All required for installation/deployment data must be under
# /var/opt/nortel/deployment/deployed/"hostnameX"/install/ directory.
# New  install.xml file with a single supported configuration will be generated
# under target directory.
# There two supported modes:
#      - Local deployment: do not need to copy files to the remote host.  Also
#      set of symbolic links to required rpms will be added.
#      - Regular deployment: all required files will be generated and copied to
#        the remote machine to target directory.
# @param    $self  - reference to deployment manager object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author  Kate Bereznyakova
# @date    April 2008
sub prepare_deploy_data($)
{
   my $self   = shift @_;
   my $retval = $FALSE;

   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Deployment manager object is undefined.');
      return $FALSE;
   }
   my $load = $self->get_param($DEPLVER);
   if (!defined $load)
   {
      baselog($LOG_ERROR, "Cannot find deploy version for '$self->{'hostname'}'.");
      return $FALSE;
   }
   if (!$self->update_deploy_status('Content Initialization'))
   {
      baselog($LOG_ERROR, 'Cannot change status in the deployment.xml file.');
   }
   if (!$self->pre_deploy_clean_up())
   {

      # log message will be printed by pre_deploy_clean_up method.
      return $FALSE;
   } ## end if (!$self->pre_deploy_clean_up...
   ### generate list of data to deploy

   # Create new install.xml
   if (!$self->gen_deploy_install_xml())
   {
      baselog($LOG_ERROR, 'Cannot create new install.xml file.');
      return $FALSE;
   }

   baselog($LOG_INFO, 'New install.xml file has been generated.');

   my @filelist;  # list of files required for deployment (that need to be
                  # transferred to the target machine).
   my $is_local = $self->is_local_deploy();
   if (!defined $is_local)
   {
      baselog($LOG_ERROR, 'Cannot detect operation type: local or remote.');
      return $FALSE;
   }

   # Get list of files to deploy
   if (!$is_local)
   {
      my $listref = $self->get_deploy_file_list();
      @filelist = @$listref;
   }
   else
   {
      @filelist = $self->create_slinks();
   }

   # undef will cause this allways true. need to change called functions.
   if (!scalar @filelist)
   {
      baselog($LOG_ERROR, 'Cannot get file list to deploy.');
      $retval = $FALSE;
   }
   else
   {
      baselog($LOG_INFO, 'File list to deploy is identified.');
      if ($is_local)
      {
         baselog($LOG_DEBUG, 'Local installation. File transfer stage is skipped.');
         $retval = $TRUE;
      }
      else
      {

         # add pre-config data to list
         my $pre_config_arch = "$self->{'deploymentHome'}/$TMPDIR/$PRECONFIGARCH";
         my $pre_config_dir  = "$self->{'deploymentHome'}/$PRECONFIGDIR";
         my $pre_config      = create_archive_data($pre_config_dir, $pre_config_arch);
         if ($pre_config)
         {
            push @filelist, $pre_config_arch;
         }

         # add license data to list
         my $license_dir  = "$APPLOADPATH/$load/$LICENSEDIR";
         my $license_arch = "$self->{'deploymentHome'}/$TMPDIR/$LICENSEARCH";
         my $license      = create_archive_data($license_dir, $license_arch);
         if ($license)
         {
            push @filelist, $license_arch;
         }

         # add keycode file to list
         my $keycode_file = "$self->{'deploymentHome'}/$KEYCODEFILE";
         my $keycode_arch = "$self->{'deploymentHome'}/$TMPDIR/$KEYCODEARCH";
         my $keycode      = create_archive_data($keycode_file, $keycode_arch);
         if ($keycode)
         {
            push @filelist, $keycode_arch;
         }

         baselog($LOG_DEBUG, 'Remote installation. File transfer stage...');
         if (!$self->transfer_files(\@filelist))
         {
            baselog($LOG_ERROR, 'File transfer stage failed.');
            $retval = $FALSE;
         }
         else
         {

            # copy deployment.xml file to remote machine.
            if (!$self->transfer_deploy_xml())
            {
               baselog($LOG_ERROR, 'Fail to transfer deployment.xml file.');
               $retval = $FALSE;
            }
            else
            {
               baselog($LOG_INFO, 'File transfer stage completed successfully.');
               $retval = $TRUE;
            }
         } ## end else [ if (!$self->transfer_files...

         # fail of this function is not critical and will be captured in log file.
         $self->cleanup_ds_tmp_data();
      } ## end else [ if ($is_local)
   } ## end else [ if (!scalar @filelist)
   return $retval;
} ## end sub prepare_deploy_data($)
###############################################################################
## @fn       prepare_deploy_dir($hostname)
# @brief     Prepare deployment directory for deploy
# @details   If deployment directory exists it will be cleared
#            otherwise it will be created.
# @param     $hostname - host name
# @return    $TRUE    - deployment directory has been prepared for deploy
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      May 2008

sub prepare_deploy_dir($)
{
   my $hostname = shift @_;
   my $retval   = $TRUE;

   if ((!defined $hostname) or ($hostname eq ''))
   {
      baselog($LOG_ERROR, 'Hostname is undefined.');
      return $FALSE;
   }

   my $installHome = "$DEPLOYEDHOSTPATH/$hostname/$INSTALLDIR";

   if (clean_up($hostname))
   {
      baselog($LOG_INFO, 'Deployment directory has been cleared');
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, 'Deployment directory has not been cleared');
      $retval = $FALSE;
   }

   if (do_system([$MKDIR, '-p', $installHome]))
   {
      baselog($LOG_INFO, 'Deployment directory has been created successfully');
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, 'Deployment directory has not been created');
      $retval = $FALSE;
   }
   return $retval;
} ## end sub prepare_deploy_dir($)

###############################################################################
## @fn     create_archive_data($data, $archive )
# @brief   It detects data on the server and creates an archive with existing data.
# @details If an old archive is found then it will be removed.
# @param    $data - full path to data that should be archived.
# @param    $archive - full archive name.
# @return   $TRUE  - data archive has been created.
# @return   $FALSE - data does not exist (need not to create archive).
# @return   undef - some error happened.
# @author  Kate Bazhanova
# @date    October 2008
# @note    none
sub create_archive_data($$)
{
   my ($data, $archive) = @_;
   if (!defined $data)
   {
      baselog($LOG_ERROR, 'Required data to archive is undefined.');
      return undef;
   }
   if (-e $data)
   {
      baselog($LOG_DEBUG, "Required data to archive has been detected:[$data]");
   }
   else
   {
      baselog($LOG_DEBUG, "Required data to archive does not exist:[$data]");
      return $FALSE;
   }
   if (!defined $archive)
   {
      baselog($LOG_ERROR, 'Archive name is undefined.');
      return undef;
   }
   my $arcdir = dirname($archive);
   if (!defined $arcdir)
   {
      baselog($LOG_ERROR, 'Invalide archive name.');
      return undef;
   }
   unless (-d $arcdir)
   {
      if (!mkdir($arcdir))
      {
         baselog($LOG_ERROR, 'Failed to create an archive directory.');
         return undef;
      }
   } ## end unless (-d $arcdir)
   else
   {
      if (-e $archive)
      {
         baselog($LOG_DEBUG, 'Old data archive has been found. Removing.');
         if (!unlink($archive))
         {
            baselog($LOG_ERROR, 'Failed to remove old data archive.');
            return undef;
         }
      } ## end if (-e $archive)
   } ## end else
   my $datadir = dirname($data);
   my $bndata  = basename($data);
   if (!do_system([$TAR, '-czf', $archive, '-C', $datadir, $bndata]))
   {
      baselog($LOG_ERROR, "Failed to create required archive:[$archive]");
      return undef;
   }
   else
   {
      baselog($LOG_INFO, "Required archive has been created: [$archive]");
      return $TRUE;
   }
} ## end sub create_archive_data($$)
###############################################################################
## @method cleanup_ds_tmp_data($self)
# @brief   It removes temporary directory from deployed directory on the
#          Deployment Server.
# @param    $self  - reference to deployment manager object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author  Kate Bereznyakova
# @date    May 2008
# @note    none
sub cleanup_ds_tmp_data($)
{
   my $self   = shift @_;
   my $retval = $TRUE;
   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Deployment manager object is undefined.');
      $retval = $FALSE;
   }
   else
   {
      my $tmp_dir = "$self->{'deploymentHome'}/$TMPDIR";
      if (-d $tmp_dir)
      {
         baselog($LOG_INFO, 'Removing temporary data.');
         if (!do_system([$RM, '-rf', $tmp_dir]))
         {
            baselog($LOG_ERROR, 'Failed to remove temporary data.');
            $retval = $FALSE;
         }
      } ## end if (-d $tmp_dir)
   } ## end else [ if (!defined $self)
   return $retval;
} ## end sub cleanup_ds_tmp_data($)

###############################################################################
## @method get_rmt_archive_data($self)
# @brief   It detects archive data on the target host and extracts the
#          data.
# @details
# @param    $archive - archive path
# @param    $extdir  - directory to extract an archive.
# @return   $TRUE  - there are no archive data or archive has been extracted.
# @return   $FALSE - some error happens
# @author  Kate Bereznyakova
# @date    May 2008
# @note    none
sub get_rmt_archive_data($$)
{
   my ($archive, $extdir) = @_;
   my $retval = $FALSE;
   if (!-e $archive)
   {
      baselog($LOG_DEBUG, "No archive data $archive has been found.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "Archive data $archive have been found.");
      if (!do_system([$TAR, '-xzf', $archive, '-C', $extdir, '--recursive-unlink']))
      {
         baselog($LOG_ERROR, "Failed extract archive data $archive.");
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "Archive $archive successfully extracted.");
         $retval = $TRUE;
      }
   } ## end else [ if (!-e $archive)
   return $retval;
} ## end sub get_rmt_archive_data($$)

###############################################################################
## @fn get_deploy_cfg_path($hostname)
# @brief   It returns path to the configuration file (deployment.xml)
#          associated with host.
# @details
# @param    $hostname  - hostname.
# @return   undef  - hostname is undefined
# @return   path to the deployment.xml file.
sub get_deploy_cfg_path($)
{
   my $hostname = shift @_;

   if (!defined $hostname)
   {
      baselog($LOG_ERROR, 'Hostname is undefined.');
      return undef;
   }

   return "$DEPLOYEDHOSTPATH/$hostname/$DEPLOYMENTCFG";
} ## end sub get_deploy_cfg_path($)

###############################################################################
## @fn keep_alive_host($account, $ip)
# @brief  Checks if the host is alive using remote call of the "exit 0" command
# @param   $account - account name for the ssh connection
# @param   $ip - host IP address
# @return  $TRUE  - host is alive
# @return  $FALSE - some error happens or server does not respond
sub keep_alive_host($$)
{
   my ($account, $ip) = @_;
   my $retval = $FALSE;
   my $stdout = [];
   my $stderr = [];
   if ((!defined $ip) or ($ip eq ''))
   {
      baselog($LOG_ERROR, "IP address is not defined.");
      return $FALSE;
   }
   if ((!defined $account) or ($account eq ''))
   {
      baselog($LOG_ERROR, "Account name is not defined.");
      return $FALSE;
   }
   if (do_system([$SSH, @ssh_connection_options, "$account\@$ip", 'exit'], {$dsOUT => $stdout, $dsERR => $stderr}))
   {
      baselog($LOG_INFO, "Server '$ip' is alive.");
      $retval = $TRUE;
   }

   if (scalar @$stderr)
   {
      baselog($LOG_WARNING, "@$stderr");
   }
   if (scalar @$stdout)
   {
      baselog($LOG_DEBUG, "@$stdout");
   }

   return $retval;
} ## end sub keep_alive_host($$)

###############################################################################
## @fn reset_status($cfg_file)
# @brief  It checks if operation status set to incomplete state
#        then reset operation status in initial one - None.
# @param $cfg_file - full patch to configuration file to update
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
sub reset_status($)
{
   my $cfg_file = shift @_;

   # get current operation status from deployment.xml file
   my $doc = base_xml->new($cfg_file);
   unless ($doc)
   {
      baselog($LOG_ERROR, 'Cannot create base_xml object.');
      return $FALSE;
   }

   my $dep_status = $doc->getText($DEPSTATUS);
   undef $doc;

   baselog($LOG_DEBUG, "Current status: $dep_status.");

   # Keys in hash start with lower case.
   $dep_status =~ tr/[A-Z]/[a-z]/;

   if (exists $DEPLOYSTATUS{$dep_status})
   {
      baselog($LOG_DEBUG, "Deployment status set correctly at '$cfg_file'.");
      return $TRUE;
   }

   if (!defined $cfg_file)
   {
      baselog($LOG_ERROR, 'Full path to deployment.xml file is not defined.');
      return $FALSE;
   }

   if (update_section($cfg_file, "/target/deploymentStatus", $DEPLOYSTATUS{none}))
   {
      baselog($LOG_INFO, "Deployment status reset to '$DEPLOYSTATUS{none}' (at $cfg_file).");
      return $TRUE;
   }
   return $FALSE;
} ## end sub reset_status($)

###############################################################################
## @fn make_deploy_dir($deployMng, $directory, $error_mesg)
# @brief   It checks if directory exists. If there is no such directory
#          then is will be created.
# @details None.
# @param   $deployMng - deployment object.
# @param   $directory - directory to create
# @param   $error_mesg - optional error message with extra details for
#          deployment.xml file in case of error condition.
# @return  $TRUE    - directory exists or successfully created.
# @return  $FALSE -  failed to check or create directory.

sub make_deploy_dir($$;$)
{
   my ($deployMng, $directory, $error_mesg) = @_;

   my $cmd;
   my $retval = $TRUE;

   if ((!defined $deployMng) or (!defined $directory))
   {
      baselog($LOG_ERROR, 'Required parameters are not defined.');
      return $FALSE;
   }

   $cmd = $deployMng->call_remote_cmd("$TEST -d '$directory'", undef, $noERROR);
   if (!defined $cmd)
   {
      baselog($LOG_ERROR, "Cannot check '$directory' directory.");
      $retval = $FALSE;
   }
   elsif ($cmd ne $OK)
   {
      my $make_dir = $deployMng->call_remote_cmd("$MKDIR -p $directory");
      if ((defined $make_dir) and ($make_dir eq $OK))
      {
         baselog($LOG_INFO, "'$directory' directory has been created.");
      }
      else
      {
         baselog($LOG_ERROR, "Cannot create '$directory' directory.");
         $retval = $FALSE;
      }
   } ## end elsif ($cmd ne $OK)

   if (!$retval)
   {
      update_section($deployMng->{'deploymentCfg'}, $DEPERRORPATH, 'CRDF');
      if ((defined $error_mesg) and ($error_mesg ne ''))
      {
         update_section($deployMng->{'deploymentCfg'}, $DEPERRORMSGPATH, $error_mesg, $KEEPOLDCNT);
      }
   } ## end if (!$retval)

   return $retval;
} ## end sub make_deploy_dir($$;$)

###############################################################################
## @fn get_error_code($cfg_file)
# @brief   It gets appinstall logs and report from target to Deployment Server
#          directory.
# @param    $cfg_file - full patch to deployment.xml configuration file
# @return   $error_tmp  - value of "error" section in the file.
# @return   undef - some error happened
sub get_error_code($)
{
   my $cfg_file  = shift @_;
   my $error_tmp = undef;

   my $doc = base_xml->new($cfg_file);
   if ($doc)
   {
      $error_tmp = $doc->getText($DEPERRORPATH);
      undef $doc;
   }
   else
   {
      baselog($LOG_ERROR, 'Cannot create base_xml object.');
   }

   return $error_tmp;
} ## end sub get_error_code($)

###############################################################################
## @fn get_deploy_logs($deployMng)
# @brief   It prepares logs directory and gets appinstall logs and report from
#          target to Deployment Server directory.
# @param   $deployMng - deployment object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happened
sub get_deploy_logs($)
{
   my $deployMng = shift(@_);
   my $retval    = $TRUE;
   if (!defined $deployMng)
   {
      baselog($LOG_ERROR, 'Required parameters are not defined.');
      return $FALSE;
   }

   #prepare logs directory on the Deployment Server
   my $dst_dir = "$deployMng->{'deploymentHome'}/$LOGDIR";
   if (!-d $dst_dir)
   {
      baselog($LOG_INFO, "Log directory '$dst_dir' does not exist. Creating.");
      if (do_system([$MKDIR, '-p', $dst_dir]))
      {
         baselog($LOG_INFO, "'$dst_dir' directory has been created successfully.");
      }
      else
      {
         baselog($LOG_ERROR, "Cannot create '$dst_dir' directory.");
         update_section($deployMng->{'deploymentCfg'}, $DEPERRORMSGPATH, 'Cannot create deployment log directory.', $KEEPOLDCNT);

         #change error code at deployment.xml file only if it is clean. Do not replace
         #existing error code.
         my $temp_error = get_error_code($deployMng->{'deploymentCfg'});
         if ((defined $temp_error) and ($temp_error eq ''))
         {
            update_section($deployMng->{'deploymentCfg'}, $DEPERRORPATH, 'CRDF');
         }
         return $FALSE;
      } ## end else [ if (do_system([$MKDIR,...
   } ## end if (!-d $dst_dir)

   #logs directory is ready now to get log files
   $retval = $deployMng->get_deploy_log_files();

   return $retval;
} ## end sub get_deploy_logs($)

###############################################################################
## @fn cleanup_logs_dir($deployMng)
# @brief   It removes old deploy logs files from host's subdirectory.
# @details Operation has the following logic:
#          If log file exists then try to remove it.
#          If file cannot be removed then add appropriate error message to
#          errorMgs section of deployment.xml file and stop clean up operation.
# @param   $deployMng - deployment object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happened
sub cleanup_logs_dir($)
{
   my $deployMng = shift(@_);

   my $filename = get_basename($APPINSTALL_REPORTFILE);
   if ($filename eq '')
   {
      baselog($LOG_ERROR, 'Cannot get log filename.');
      update_section($deployMng->{'deploymentCfg'}, $DEPERRORMSGPATH, 'Cannot get old log file name.', $KEEPOLDCNT);
      return $FALSE;
   } ## end if ($filename eq '')

   if (-e "$deployMng->{'deploymentHome'}/$LOGDIR/$filename")
   {
      baselog($LOG_INFO, "Removing '$filename'.");
      if (!do_system([$RM, '-f', "$deployMng->{'deploymentHome'}/$LOGDIR/$filename"]))
      {
         baselog($LOG_ERROR, "Fail to remove '$filename'.");
         update_section($deployMng->{'deploymentCfg'}, $DEPERRORMSGPATH, 'Cannot remove old log file.', $KEEPOLDCNT);
         return $FALSE;
      } ## end if (!do_system([$RM, '-f'...
   } ## end if (-e "$deployMng->{'deploymentHome'}/$LOGDIR/$filename"...

   return $TRUE;
} ## end sub cleanup_logs_dir($)

###############################################################################
## @fn get_deploy_log_files($deployMng)
# @brief   It copies appinstall logs and report from target to Deployment Server
#          directory.
# @param   $deployMng - deployment object.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happened
sub get_deploy_log_files($)
{
   my $deployMng = shift(@_);

   my $retval = $TRUE;
   unless ($deployMng->transfer_file($APPINSTALL_REPORTFILE, "$deployMng->{'deploymentHome'}/$LOGDIR/", $FALSE))
   {
      $retval = $FALSE;
   }

   return $retval;
} ## end sub get_deploy_log_files($)

1;

__END__

=head1 NAME
base_deploy - Linux Base deploy management library.

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=over 4

=item new


=over 4

=item *

=head1 AUTHOR

Kate Bereznyakova
Ekaterina Bazhanova

=back

