## @file base_firewall.pm
# @brief This module is the LinuxBase interface for updating firewall rules.
# @details It reads data from the database (file iptables.xml) and adds these
#   rules to the iptables.
#   Methods: firewall_restart, firewall_status, firewall_stop, check_rules,
#            get_iptables, save_iptables
# @author Ilya Teterev
# @author Alexander Kolotov
# @author Vladimir Ivanov
# @author Pavel Sergeev
# @author Kushal Agarwal
# @version 1.00 - Initial version
# @version 2.00 - ifxml module is used instead of XML::Parser. The support of
#   references is added.
# @version 2.10 - small rework of the script - some code splitted to functions,
#   check functions for the rule's attributes were added
# @version 3.00 - TOS support was added
# @version 3.10 - the temporary way to handle ECN setup was removed, now ECN
#   feature can be handled as one of the system parameters
# @version 3.20 - enable_app_rules, disable_app_rules subroutines are added.
#   States hash support. "redirectport", "priority", "localredirectport"
#   options support is added.
# @version 3.21 - Two colons were removed before shared functions and variables.
# @version 3.22 - flushing of mangle and nat tables added to firewall_restart sub
# @version 3.23 - Change sequence of baselog and common_functions
# @version 3.24 - Modified under the perlcheck warnings cleanup activity.
# @version 3.25 - fixed bug with dirname and basename commands
# @version 3.30 - new functions added for iptables-restore support
# @version 3.31 - are_rules_applied function added, enable_app_rules checks for
#   rules existence before adding.
# @version 3.32 - added consultation to masterfirewall when generating iptables rules
# @version 3.33 - Now get_basename and get_dirname are used instead of File::Basename
# @version 4.00 -
#   @li Protection with semaphore SEM_FIREWALL has been added.
#   @li Name of the temporary file for "iptables.xml" is created with PID.
#   @li This module was re-worked into the class: constructor new() was added.
# @version 4.01 - read_applied_rules optimization
# @version 4.10 - Reject rule proccessing is added. IKudelin.
#   @li check_reject($) is added.
#   @li delete_rules_by_name($) is updated.
#   @li apply_rules($;$) is updated.
#   @li firewall_restart(;$$) is updated. Reject rules branch is added.
#   @li add_reject_rules_by_name($$) is added.Private method.
#   @li get_priority($) is added.
#   @li collect_app_names() is added.
#   @li generate_rule($;$$) is enhanced
# @version 4.11 - firewall_restart now by default just returns $TRUE instead
#   of starting previously stopped firewall.

package base_firewall;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use common_variables;
use common_functions;
use baselog;
use install_common;
use ifxml;
use base_system;
use base_semctl;
use Data::Dumper;
use XML::Writer;
use IO::File;
use master_firewall;
use base_snmp;

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&firewall_restart
  &firewall_status
  &firewall_stop
  &check_rule
  &get_iptables
  &save_iptables
  &get_status
  &disable_app_rules
  &enable_app_rules
  &delete_rules_by_name
  &take_firewall_semaphore
  &release_firewall_semaphore);

our $VERSION = '4.11';
#####################################################################
########################## Private Data #############################

# Hash with refs to check function
my %check_attr = (
                  'service'           => \&check_service,
                  'proto'             => \&check_proto,
                  'port'              => \&check_port,
                  'redirectport'      => \&check_redirect_port,
                  'localredirectport' => \&check_redirect_port,
                  'on'                => \&check_on,
                  'srcport'           => \&check_srcport,
                  'icmptype'          => \&check_icmptype,
                  'if'                => \&check_if,
                  'host'              => \&check_host,
                  'localhost'         => \&check_localhost,
                  'eth'               => \&check_eth,
                  'appl'              => \&check_appl,
                  'reference'         => \&check_reference,
                  'dscp'              => \&check_dscp,
                  'priority'          => \&check_priority,
                  'reject'            => \&check_reject
                 );

#rule generating variables
my $ACTION = 3;
my $CHAIN  = 4;
my $TABLE  = 2;

my $_accept            = 'ACCEPT';
my $_reject            = 'REJECT';
my $_drop              = 'DROP';
my $filter_INPUT       = 0;
my $mangle_POSTROUTING = 0;
my $mangle_PREROUTING  = 1;
my $nat_PREROUTING     = 2;
my $nat_OUTPUT         = 3;

#priority hash
my %PRIORITIES_HASH = ('high' => "FREQUENTLY_USED_RULES", 'normal' => "REGULAR_USED_RULES", 'low' => "RARELY_USED_RULES");

#path to the nortel firewall rules file
my $NORTEL_IPTABLES_FILE = "/var/opt/nortel/base/nortel.iptables.save";

#iptables-save file path
my $IPTABLES_SAVE_FILE = "/etc/sysconfig/iptables";

# time to wait for semaphore
my $TIME_TO_WAIT = 5;

###############################################################################
#################### Private Methods Declaration ##############################
## @skipline
sub get_xml_parser($);
## @skipline
sub get_value_by_xpath($$);
## @skipline
sub prepare_rule($);
## @skipline
sub generate_rule($;$$);
## @skipline
sub apply_rules($;$);

## @skipline
sub add_rules_by_name($$;$);
## @skipline
sub get_master_firewall_object();
## @skipline
sub add_reject_rules_by_name($$);
## @skipline
sub collect_app_names();

###############################################################################
#################### Private Functions Declaration ############################
## @skipline
sub get_priority($$);
## @skipline
sub check_attr($$);
## @skipline
sub check_attr_exist($);

## @skipline
sub check_proto($);
## @skipline
sub check_port($);
## @skipline
sub check_redirect_port($);
## @skipline
sub check_on($);
## @skipline
sub check_service($);
## @skipline
sub check_srcport($);
## @skipline
sub check_icmptype($);
## @skipline
sub check_if($);
## @skipline
sub check_host($);
## @skipline
sub check_localhost($);
## @skipline
sub check_eth($);
## @skipline
sub check_appl($);
## @skipline
sub check_reference($);
## @skipline
sub check_dscp($);
## @skipline
sub check_priority($);
## @skipline
sub check_reject($);

## @skipline
sub read_applied_rules(;$);
## @skipline
sub write_rules_hash($);
## @skipline
sub update_firewall_rules();
## @skipline
sub are_rules_applied($);

###############################################################################
################ Public Methods and Functions Declaration #####################
## @skipline
sub new();
## @skipline
sub get_iptables($$);
## @skipline
sub save_iptables($$$);
## @skipline
sub firewall_restart(;$$$);
## @skipline
sub firewall_status();
## @skipline
sub firewall_stop();
## @skipline
sub get_status();
## @skipline
sub check_rule($);

## @skipline
sub enable_app_rules($);
## @skipline
sub disable_app_rules($);
## @skipline
sub delete_rules_by_name($;$);

## @skipline
sub take_firewall_semaphore();
## @skipline
sub release_firewall_semaphore($);

###############################################################################
########################## Private Methods ####################################

###############################################################################
## @method get_xml_parser($reffile)
# @brief Creates XML parser for the given XML file.
# @details Creates the object of ifxml class in the case if the object
#   was not created before for the given XML file. The created object
#   is put into the %xml_parsers hash in order to re-use this
# @param  $self - reference to the object
# @param  $reffile - path to XML file
# @return $parser  - the object of ifxml class if the XML file is read
#                    successfully otherwise undefined value.
###############################################################################
sub get_xml_parser($)
{
   my $self    = shift @_;
   my $reffile = shift @_;

   my ($dirname, $basename);
   my $parser;

   baselog($LOG_DEBUG, "Get XML parser for '$reffile'");

   $dirname  = get_dirname($reffile);
   $basename = get_basename($reffile);

   #If parser has already been created (or it has already been
   #determined that the XML file can not be read)
   if (exists $self->{'xml_parsers'}->{$reffile})
   {
      baselog($LOG_DEBUG, "XML parser is re-used");
      $parser = $self->{'xml_parsers'}->{$reffile};
   }
   else
   {

      #Create parser
      $parser = ifxml::new($dirname, $basename);

      #If some errors happen (file can not be found, opened, read)
      unless ($parser)
      {
         $parser = undef;
         baselog($LOG_DEBUG, "Can not get xml parser. Mark it as undefined");
      }
      else
      {
         baselog($LOG_DEBUG, "XML parser is initialized");
      }

      #Put the parser to the hash in order to be used in the future
      $self->{'xml_parsers'}->{$reffile} = $parser;
   } ## end else [ if (exists $self->{'xml_parsers'...

   return $parser;
} ## end sub get_xml_parser($)

###############################################################################
## @fn get_value_by_xpath($xml_parser, $refxpath)
# @brief Looks for the value by using XPath and the object of ifxml class.
# @param  $xml_parser - the object of ifxml class
# @param  $refxpath   - path to the tag (XPath) in XML file in the form:
#                        Tag1/Tag2/.../TagN
# @return $retvalue - the found value. If some tag can not be found or the value
#                      is empty, then undefined value is returned
###############################################################################
sub get_value_by_xpath($$)
{
   my ($xml_parser, $refxpath) = @_;
   my ($ref_to_sub, $expression, $refvalue);

   #Replace slash in XPath notation by underscore
   $ref_to_sub = $refxpath;
   $ref_to_sub =~ tr|/|_|;

   #De-reference
   $expression = '$refvalue = $xml_parser->get' . $ref_to_sub . '_text();';
   eval $expression;
   baselog($LOG_DEBUG, "Expression for eval: '$expression'; refvalue = " . Data::Dumper::Dumper($refvalue));

   if ((defined $refvalue) && (exists ${$refvalue}[0]) && (${$refvalue}[0] ne ""))
   {
      $refvalue = ${$refvalue}[0];
   }
   else
   {
      $refvalue = undef;
   }

   return $refvalue;
} ## end sub get_value_by_xpath($$)

###############################################################################
## @method prepare_rule($rule_ref)
# @brief Looks for the attributes-references and de-reference them
# @param  $self - reference to the object
# @param  $rule_ref - the reference to the rule-hash
# @return $rule_ref - the reference to the new rule-hash if there is no errors,
#                     otherwise the undefined value
###############################################################################
sub prepare_rule($)
{
   my $self = shift @_;
   my %rule = %{shift @_};
   my $xml_parser;
   my $reffile;
   my $attr;
   my $refxpath;
   my $refvalue;
   my $resulting_attr_exists;

   #Process rules with attribute 'reference' only
   if (defined $rule{'reference'})
   {
      $reffile = $rule{'reference'};

      $xml_parser = $self->get_xml_parser($reffile);
      unless (defined $xml_parser)
      {
         baselog($LOG_WARNING, "Application '$rule{'appl'}': Can not read XML file '$reffile'.");
      }

      $resulting_attr_exists = $TRUE;

      #look for attributes-references ('*_ref')
      foreach $attr (keys %rule)
      {
         if ($attr =~ /_ref$/)
         {
            $refxpath = $rule{$attr};

            #Get the name of resulting attribute
            $attr =~ s/(.*)_ref$/$1/;
            $resulting_attr_exists = (defined $rule{$attr}) ? $TRUE : $FALSE;

            #If the file can not be read, it is necessary to create rule
            #by using default attributes. But if there are no default
            #attributes - such rule should not be generated.
            unless (defined $xml_parser)
            {
               unless ($resulting_attr_exists)
               {
                  baselog($LOG_WARNING, "Application '$rule{'appl'}': There is no default attribute '$attr'.");
                  last;
               }
               else
               {
                  baselog($LOG_DEBUG, "Default value is used for attribute '$attr': '$rule{$attr}'.");
               }
            } ## end unless (defined $xml_parser...
            else
            {
               baselog($LOG_DEBUG, "reference: '$reffile'; attribute: '$attr'; XPath: '$refxpath'");

               #De-reference by using XPath
               $refvalue = get_value_by_xpath($xml_parser, $refxpath);

               unless (defined $refvalue)
               {
                  baselog($LOG_WARNING, "Application '$rule{'appl'}': The value for attribute '$attr' is not found by using XPath '$refxpath'.");

                  #If reference is not valid and the default value is
                  #not defined mark the rule as bad
                  unless ($resulting_attr_exists)
                  {
                     baselog($LOG_WARNING, "Application '$rule{'appl'}': There is no default value for attribute '$attr'.");
                     last;
                  }
                  else
                  {
                     baselog($LOG_DEBUG, "Default value is used for attribute '$attr': '$rule{$attr}'.");
                  }
               } ## end unless (defined $refvalue)
               else
               {
                  baselog($LOG_DEBUG, "attribute: '$attr'; XPath '$refxpath'; value = '$refvalue'");

                  #Create the attribute with the new value
                  $rule{$attr} = $refvalue;

                  #Resulting attribute (without "_ref") exists now
                  $resulting_attr_exists = $TRUE;
               } ## end else
            } ## end else
         } ## end if ($attr =~ /_ref$/)
      } ## end foreach $attr (keys %rule)

      # Rule is skipped when there is warning message and there is no default attribute
      if (!$resulting_attr_exists)
      {
         baselog($LOG_WARNING, "Application '$rule{'appl'}': Rule is skipped");
         return undef;
      }
   } ## end if (defined $rule{'reference'...

   return \%rule;
} ## end sub prepare_rule($)

###############################################################################
## @fn get_priority($rule, $action)
# @brief Return name of input chain where rule must be placed.
# @param  $rule   - reference to hash with rule from iptables.xml.
# @param  $action - generate priority for specified action (ACEPT/REJECT/DROP).
# @return $retval  - name of the chain. undef in error case.
###############################################################################
sub get_priority($$)
{
   my $rule   = shift @_;
   my $action = shift @_;
   my $retval;

   if (not is_hash($rule))
   {
      baselog($LOG_ERROR, "Incorrect input parameter. Must be a reference to hash.");
      return undef;
   }

   if (exists $rule->{'priority'})
   {
      $retval = $PRIORITIES_HASH{$rule->{'priority'}};
   }
   else
   {
      if ($action eq $_accept)
      {
         $retval = $PRIORITIES_HASH{'normal'};
      }
      elsif ($action eq $_reject)
      {
         $retval = $PRIORITIES_HASH{'low'};
      }
      elsif ($action eq $_drop)
      {
         $retval = $PRIORITIES_HASH{'low'};
      }
      else
      {
         baselog($LOG_ERROR, "Incorrect second parameter. Must be one of:" . join(' ', ($_accept, $_reject, $_drop)));
         return undef;
      }
   } ## end else [ if (exists $rule->{'priority'...
   return $retval;
} ## end sub get_priority($$)
###############################################################################
## @fn check_attr($attr, $value)
# @brief Validates the attribute value.
# @param  $attr   - the attribute name
# @param  $value  - the value of the attribute
# @return $TRUE  - the value can be used to generate rule
# @return $FALSE - the value can not be used.
###############################################################################
sub check_attr($$)
{
   my ($attr, $value) = @_;
   my $retval = $FALSE;
   if (defined $value)
   {
      $retval = $TRUE;
      if ($value eq "")
      {

         #Empty value of the attribute means "any". It must be used for
         #unnecessary attributes together with reference in order to
         #show that default attribute exists.
         $retval = $FALSE;
      } ## end if ($value eq "")
      else
      {
         if (exists $check_attr{$attr})
         {
            $retval = $check_attr{$attr}->($value);
         }
         else
         {
            $retval = $FALSE;
         }
      } ## end else [ if ($value eq "")
   } ## end if (defined $value)
   return $retval;
} ## end sub check_attr($$)

###############################################################################
## @fn check_attr_exist($rule_ref)
# @brief Checks the existence of any one of attributes (defined within this sub
#   as array of @one_must_exist) for a certain rule.
# @param  $rule_ref - the reference with the rule hash
# @return $TRUE  - at least one of attributes exists
# @return $FALSE - none of those attributes exists
###############################################################################
sub check_attr_exist($)
{
   my %rule   = %{shift @_};
   my $retval = $TRUE;

   #This array specifies the set of attributes which must be in the
   #iptable rule (one of them at least)
   my @one_must_exist = ('port', 'srcport', 'icmptype', 'localhost');

   $retval = $FALSE;
   foreach (@one_must_exist)
   {
      if (check_attr($_, $rule{$_}))
      {
         $retval = $TRUE;
         last;
      }
   } ## end foreach (@one_must_exist)

   return $retval;
} ## end sub check_attr_exist($)

###############################################################################
## @fn check_rule($rule_ref)
# @brief Checks the sufficiency of the attributes and their correctness.
# @param  $rule_ref - the reference with the rule hash
# @return $TRUE  - the rule is valid
# @return $FALSE - the rule is not valid
###############################################################################
sub check_rule($)
{
   my %rule   = %{shift @_};
   my $retval = $TRUE;
   my $attr;

   #Only LinuxBase supported attributes must be present in a rule
   foreach (keys(%rule))
   {

      #Reference attributes are also supported
      $attr = ($_ =~ /(.*)_ref$/) ? $1 : $_;
      unless (exists $check_attr{$attr})
      {
         baselog($LOG_ERROR, "Application '$rule{'appl'}': Unsupported attribute '$attr'");
         $retval = $FALSE;
      }
   } ## end foreach (keys(%rule))
   unless ($retval)
   {
      baselog($LOG_ERROR, "Application '$rule{'appl'}': Rule is skipped due to unsupported attribute(s)");
      return $retval;
   }

   #Check all attributes values except *_ref attributes
   foreach (keys(%rule))
   {
      unless ($_ =~ /(.*)_ref$/)
      {
         unless (check_attr($_, $rule{$_}))
         {
            baselog($LOG_ERROR, "Application '$rule{'appl'}': Wrong value of the attribute'$_': '$rule{$_}'");
            $retval = $FALSE;
         }
      } ## end unless ($_ =~ /(.*)_ref$/)
   } ## end foreach (keys(%rule))
   unless ($retval)
   {
      baselog($LOG_ERROR, "Application '$rule{'appl'}': Rule is skipped due to wrong value of attribute(s)");
      return $retval;
   }

   #for any protocol (except ipv6-crypt, esp and ah) one of the following
   #attributes must be defined: port
   #                            srcport
   #                            icmptype
   #                            localhost
   if (check_attr('proto', $rule{'proto'}) && ($rule{'proto'} ne 'ipv6-crypt') && ($rule{'proto'} ne 'esp') && ($rule{'proto'} ne 'ah'))
   {
      unless (check_attr_exist(\%rule))
      {
         baselog($LOG_WARNING, "Application '$rule{'appl'}': Too few arguments in the rule. Rule is skipped");
      }
   } ## end if (check_attr('proto'...

   #for protocols esp or ah, any of the following
   #attributes must not be defined: port
   #                                srcport
   #                                icmptype
   #                                localhost
   if (check_attr('proto', $rule{'proto'}) && (($rule{'proto'} eq 'esp') || ($rule{'proto'} eq 'ah')))
   {
      if (check_attr_exist(\%rule))
      {
         $retval = $FALSE;
         baselog($LOG_WARNING, "Application '$rule{'appl'}'with $rule{'proto'}: Too many arguments in this rule. Rule is skipped");
      }
   } ## end if (check_attr('proto'...

   return $retval;
} ## end sub check_rule($)

###############################################################################
## @method generate_rule($rule_hash_ref, $for_iptables_file, $action_is)
# @brief Generates lines with firewall rule commands from the rule-hash.
# @details If it is necessary it de-references the attributes-references
# @param  $self - reference to the object
# @param  $rule_hash_ref - the reference to the rule-hash
# @param  $for_iptables_file - if $TRUE rules will be generated for using in
#   the iptables save file; command path and -t argument will be deleted;
#   if $FALSE - full rules generating (optional).
# @param  $action_is - Generate rule for specified action(ACCEPT/REJECT). Optional.
#                      By default rule will be generated for ACCEPT action
# @return @commands - the array with firewall rules or an empty array if error
# @note It uses the master class object from the $self object
###############################################################################
sub generate_rule($;$$)
{
   my $self              = shift @_;
   my %rule              = %{shift @_};
   my $for_iptables_file = shift @_;
   my $action_is         = shift @_;
   my @commands          = ([]);

   #set default action if it is not provided. if action is provided, check it.
   if (not defined $action_is)
   {
      $action_is = $_accept;
   }
   elsif (($action_is ne $_accept) and ($action_is ne $_reject))
   {
      baselog($LOG_ERROR, "Incorrect 3rd argument. Must be '$_accept' or '$_reject'");
      return ();
   }

   #ifpos - stores interface declaration position in array
   #destpos - stores destination port declaration position in array
   my ($ifpos, $destpos) = (0, 0);

   my $dscp_value = undef;

   my $tmpref = $self->prepare_rule(\%rule);

   return () unless ($tmpref);

   %rule = %{$tmpref};

   return () unless (check_rule(\%rule));

   #Creating rules
   #Consult master firewall Control
   if (check_attr('service', $rule{'service'}))
   {
      if (not $self->{'mfc_object'}->service_check($rule{'service'}))
      {
         baselog($LOG_WARNING, "The firewall rule '$rule{'service'}' is not permitted, as per Master Firewall Control");
         return ();
      }
   } ## end if (check_attr('service'...
   elsif ($rule{'appl'} ne "usercfg")
   {
      baselog($LOG_WARNING, "Service tag is not defined, skipping rule! Application: $rule{'appl'}\n");
      return ();
   }

   if (not defined($for_iptables_file) or $for_iptables_file == $FALSE)
   {
      push @{$commands[$filter_INPUT]}, $IPTABLES, "-t", "filter", "-A", get_priority(\%rule, $action_is), "-m", "comment", "--comment", "$rule{appl}";
      $TABLE = 2;
      $CHAIN = 4;
   } ## end if (not defined($for_iptables_file...
   else
   {
      push @{$commands[$filter_INPUT]}, "filter", "-A", get_priority(\%rule, $action_is), "-m", "comment", "--comment", "$rule{appl}";
      $CHAIN = 2;
      $TABLE = 0;
   } ## end else [ if (not defined($for_iptables_file...
   if (check_attr('proto', $rule{'proto'}))
   {
      push @{$commands[$filter_INPUT]}, "-p", $rule{'proto'};
   }
   if (check_attr('if', $rule{'if'}))
   {
      $ifpos = scalar(@{$commands[$filter_INPUT]});
      push @{$commands[$filter_INPUT]}, "-i", $rule{'if'};
   }
   elsif (check_attr('eth', $rule{'eth'}))
   {
      $ifpos = scalar(@{$commands[$filter_INPUT]});
      push @{$commands[$filter_INPUT]}, "-i", "eth$rule{'eth'}";
   }
   if (check_attr('host', $rule{'host'}))
   {
      push @{$commands[$filter_INPUT]}, "-s", $rule{'host'};
   }
   if (check_attr('proto', $rule{'proto'}) && ($rule{'proto'} eq "icmp"))
   {
      push @{$commands[$filter_INPUT]}, "--icmp-type", $rule{'icmptype'};
   }
   if (check_attr('port', $rule{'port'}))
   {
      $destpos = scalar(@{$commands[$filter_INPUT]});
      push @{$commands[$filter_INPUT]}, "--destination-port", $rule{'port'};
   }
   if (check_attr('srcport', $rule{'srcport'}))
   {
      push @{$commands[$filter_INPUT]}, "--source-port", $rule{'srcport'};
   }
   if (check_attr('dscp', $rule{'dscp'}))
   {
      $dscp_value = $rule{'dscp'} >> 2;
      push @{$commands[$filter_INPUT]}, "-j", "DSCP", "--set-dscp", $dscp_value;
   }
   elsif (check_attr('redirectport', $rule{'redirectport'}))
   {
      @{$commands[$mangle_PREROUTING]} = @{$commands[$nat_PREROUTING]} = @{$commands[$filter_INPUT]};

      $commands[$mangle_PREROUTING][$TABLE] = "mangle";
      $commands[$mangle_PREROUTING][$CHAIN] = "PREROUTING";

      #set mark rule
      push @{$commands[$mangle_PREROUTING]}, "-j", "MARK", "--set-mark", "1";

      if ($ifpos)
      {
         $destpos -= 2;
      }
      if (check_attr('localredirectport', $rule{'localredirectport'}))
      {
         @{$commands[$nat_OUTPUT]} = @{$commands[$filter_INPUT]};

         #change interface declaration for loop interface
         if ($ifpos)
         {
            $commands[$nat_OUTPUT][$ifpos] = "-o";
            $commands[$nat_OUTPUT][$ifpos + 1] = "lo";
         }
         $commands[$nat_OUTPUT][$TABLE] = "nat";
         $commands[$nat_OUTPUT][$CHAIN] = "OUTPUT";

         #redirect rule
         push @{$commands[$nat_OUTPUT]}, "-j", "DNAT", "--to", ":$rule{'localredirectport'}";
      } ## end if (check_attr('localredirectport'...

      #deleting interface declaration
      if ($ifpos)
      {
         splice(@{$commands[$nat_PREROUTING]}, $ifpos, 2);
      }
      $commands[$nat_PREROUTING][$TABLE] = "nat";
      $commands[$nat_PREROUTING][$CHAIN] = "PREROUTING";

      #redirect rule
      push @{$commands[$nat_PREROUTING]}, "-j", "DNAT", "--to", ":$rule{'redirectport'}";

      #deleting interface declaration
      if ($ifpos)
      {
         splice(@{$commands[$filter_INPUT]}, $ifpos, 2);
      }

      #replace '-' with ':' for destination port
      if ($destpos)
      {
         $rule{'redirectport'} =~ s/-/:/;
         $commands[$filter_INPUT][$destpos + 1] = $rule{'redirectport'};
      }

      #check mark rule
      #accept/reject
      if ($action_is eq $_accept)
      {
         push @{$commands[$filter_INPUT]}, "-m", "mark", "--mark", "1", "-j", (('on' eq $rule{'on'}) ? $_accept : $_drop);
      }
      else
      {

         #check for protocol type. If protocol is not TCP then reject with default icmp message
         my $reject_with = 'icmp-port-unreachable';

         #reject with TCP reset message
         $reject_with = 'tcp-reset' if ($rule{'proto'} =~ m/tcp/i);
         push @{$commands[$filter_INPUT]}, "-m", "mark", "--mark", "1", "-m", "limit", "--limit", "2/sec", "-j", $_reject, "--reject-with", $reject_with;
      } ## end else [ if ($action_is eq $_accept...

   } ## end elsif (check_attr('redirectport'...

   if (check_attr('localredirectport', $rule{'localredirectport'}) and (not check_attr('redirectport', $rule{'redirectport'})))
   {
      if ($ifpos)
      {
         $commands[$filter_INPUT][$ifpos] = "-o";
         $commands[$filter_INPUT][$ifpos + 1] = "lo";
      }
      $commands[$filter_INPUT][$TABLE] = "nat";
      $commands[$filter_INPUT][$CHAIN] = "OUTPUT";

      #redirect rule
      push @{$commands[$filter_INPUT]}, "-j", "DNAT", "--to", ":$rule{'localredirectport'}";
   } ## end if (check_attr('localredirectport'...

   if (defined($dscp_value))
   {
      $commands[$mangle_POSTROUTING][$TABLE] = "mangle";
      $commands[$mangle_POSTROUTING][$CHAIN] = "POSTROUTING";
   }
   elsif (
          check_attr('on', $rule{'on'})
          and (not(check_attr('redirectport', $rule{'redirectport'}))
               and (not check_attr('localredirectport', $rule{'localredirectport'})))
         )
   {

      #check for ACCEPT/REJECT actions.
      if ($action_is eq $_accept)
      {
         push @{$commands[$filter_INPUT]}, "-j", (('on' eq $rule{'on'}) ? "ACCEPT" : "DROP");
      }
      else
      {

         #check for protocol type. If protocol is not TCP then reject with default icmp message
         my $reject_with = 'icmp-port-unreachable';

         #reject with TCP reset message
         $reject_with = 'tcp-reset' if ($rule{'proto'} =~ m/tcp/i);
         push @{$commands[$filter_INPUT]}, "-m", "limit", "--limit", "2/sec", "-j", $_reject, "--reject-with", $reject_with;
      } ## end else [ if ($action_is eq $_accept...
   } ## end elsif (check_attr('on', $rule...

   return @commands;
} ## end sub generate_rule($;$$)

###############################################################################
# @method apply_rules($rules_ref, $action)
# @brief Applies firewall rules
# @details none
# @param $self      - reference to the object
# @param $rules_ref - the reference to the rules array
# @param $action    - action (optional); "del" - to delete firewall rules
# @return $FALSE - if any error occurred
# @return $TRUE  - if no errors
###############################################################################
sub apply_rules($;$)
{
   my $self   = shift @_;
   my @rules  = @{shift @_};
   my $action = shift @_;
   my ($rulehash, @commands, $rule);
   my $failed = $FALSE;
   my $retval = $TRUE;

   if (is_stopped())
   {
      baselog($LOG_DEBUG, "Cannot apply firewall rules to stopped firewall.");
      return $TRUE; # since we should not process rules for stopped firewall, just ignore this
   }

   unless ($self->get_master_firewall_object())
   {
      baselog($LOG_ERROR, "Cannot apply firewall rules.");
      return $FALSE;
   }

   foreach $rulehash (@rules)
   {
      @commands = $self->generate_rule($rulehash);
      if ((defined($action) and ($action eq 'del')))
      {
         for $rule (@commands)
         {
            $$rule[$ACTION] = "-D";
            do_system($rule);
         }
      } ## end if ((defined($action) ...
      else
      {
         $failed = $FALSE;
         for $rule (@commands)
         {
            baselog($LOG_DEBUG, "Applying firewall rule for application '$rulehash->{'appl'}'");
            if (not(do_system($rule)))
            {
               $retval = $FALSE;
               if (@commands > 1)
               {
                  $failed = $TRUE;
                  last;
               }
            } ## end if (not(do_system($rule...
         } ## end for $rule (@commands)
         if ($failed)
         {
            for $rule (@commands)
            {
               $$rule[$ACTION] = "-D";
               do_system($rule);
            }
         } ## end if ($failed)
      } ## end else [ if ((defined($action) ...
   } ## end foreach $rulehash (@rules)
   return $retval;
} ## end sub apply_rules($;$)

# Set of check functions
# All of them accept parameter value and return $TRUE or $FALSE
sub check_proto($)
{
   my $proto = shift @_;
   return (($proto =~ /^(tcp|udp|icmp|ipv6-crypt|esp|ah)$/i) ? $TRUE : $FALSE);
}

sub check_port($)
{
   my $port   = shift @_;
   my $retval = $FALSE;
   my @list;

   if ($port =~ /^\d+(:\d+)?$/)
   {
      $retval = $TRUE;
      if (defined $1)
      {
         @list = split(/:/, $port);
      }
      else
      {
         push @list, $port;
      }
      foreach (@list)
      {
         $retval = $FALSE if ($_ > 65535);
      }
   } ## end if ($port =~ /^\d+(:\d+)?$/...
   return $retval;
} ## end sub check_port($)

#############################################################
# @fn check_redirect_port($port_range)
# @brief checks redirect port range
# @param $port_range redirect port or redirect port range
# @return $FALSE - invalid port range
#         $TRUE - port range is valid
# @author Pavel Sergeev
# @date April 2008
#############################################################
sub check_redirect_port($)
{
   my $port_range = shift @_;

   #split port_range in two parts ($` and $'). Delimiter is "-" symbol
   if ($port_range =~ /\-/)
   {
      if ((not check_port($`)) or (not check_port($')))
      {
         return $FALSE;
      }
   } ## end if ($port_range =~ /\-/...
   elsif (not(check_port($port_range)))
   {
      return $FALSE;
   }
   return $TRUE;
} ## end sub check_redirect_port($)

sub check_on($)
{
   return $TRUE;
}

sub check_service($)
{
   my $service = shift @_;
   if (!defined($service) or ($service eq ""))
   {
      baselog($LOG_ERROR, "Undefined or empty service tag in iptable rule");
      return $FALSE;
   }
   return $TRUE;
} ## end sub check_service($)

sub check_srcport($)
{
   my $srcport = shift @_;
   return check_port($srcport);
}

sub check_icmptype($)
{
   return $TRUE;
}

sub check_if($)
{
   my $interface = shift @_;
   return (($interface =~ /^(eth(\d+)|lo)$/) ? $TRUE : $FALSE);
}

sub check_host($)
{
   return $TRUE;
}

sub check_localhost($)
{
   my $localhost = shift @_;
   return (($localhost == 1) ? $TRUE : $FALSE);
}

sub check_eth($)
{
   my $eth = shift @_;
   return (($eth =~ /^\d+$/) ? $TRUE : $FALSE);
}

sub check_appl($)
{
   return $TRUE;
}

sub check_reference($)
{
   return $TRUE;
}

sub check_dscp($)
{
   my $dscp   = shift @_;
   my $retval = $FALSE;
   if ($dscp =~ /^\d+$/)
   {
      $retval = $TRUE unless ($dscp > 255);
   }
   return $retval;
} ## end sub check_dscp($)

#############################################################
# @fn check_priority($prior)
# @brief checks rule priority
# @param $prior rule priority
# @return $FALSE invalid priority
#         $TRUE priority is valid
# @author Pavel Sergeev
# @date April 2008
#############################################################
sub check_priority($)
{
   my $prior = shift @_;
   return (exists($PRIORITIES_HASH{$prior})) ? $TRUE : $FALSE;
}

#############################################################
# @fn check_reject($reject)
# @brief checks value of reject flag
# @param $reject value of reject flag
# @return $FALSE invalid reject flag value
#         $TRUE reject flag value is valid
#############################################################
sub check_reject($)
{
   my $reject = shift @_;
   return $TRUE if ($reject == 1 or $reject == 0);
   return $FALSE;
}

###############################################################################
############################# Public Methods ##################################

###############################################################################
## @fn new()
# @brief   Object constructor. Object is a hash.
# @param none
# @return $self - reference to the base_firewall class object
###############################################################################
sub new()
{

   #initialization
   my %self = (
               'xml_parsers' => undef,  # hash with saved ifxml parsers
               'mfc_object'  => undef   # master_firewall object
              );

   bless \%self;               #Declaring an object
   return \%self;

} ## end sub new()

###############################################################################
## @fn get_iptables($dir, $cfg)
# @brief Reads iptables rules from the given file.
# @param $dir  - the directory where cfg is placed
# @param $cfg  - the iptables configuration file
# @return $rules - the reference to the list of the hashes with iptables rules
# @return undef  - the configuration file could not be read
# @date November 2008
# @note This function must be protected with SEM_FIREWALL semaphore
###############################################################################
sub get_iptables($$)
{
   my ($dir, $cfg) = @_;
   my $rules = undef;

   my $xml_parser = ifxml::new($dir, $cfg);
   if ($xml_parser)
   {
      $rules = $xml_parser->get_iptables_rule();
   }
   undef $xml_parser;
   return $rules;
} ## end sub get_iptables($$)

###############################################################################
## @fn save_iptables($data_ref, $dir, $cfg)
# @brief Saves iptables rules into the given file
# @param $data_ref - the reference to the list of iptables rules hashes
# @param $dir      - the directory where configuration file is placed
# @param $cfg      - the iptables configuration file
# @return $TRUE  - iptables have been saved successfully
# @return $FALSE - in case of error
# @note This function must be protected with SEM_FIREWALL semaphore
###############################################################################
sub save_iptables($$$)
{
   my @data = @{shift @_};
   my ($dir, $cfg) = @_;
   my ($output, $xml, $hash);
   my $section = "iptables";

   if (!-f "$dir/$cfg")
   {
      baselog($LOG_INFO, "File $dir/$cfg does not exist. It will be created.");
   }

   #temporary file to safely write into
   my $tmp_file = "$dir/$cfg.$$";

   $output = IO::File->new(">$tmp_file");
   unless (defined $output)
   {
      baselog($LOG_ERROR, "Cannot open temporary file $tmp_file");
      return $FALSE;
   }
   $xml = XML::Writer->new(DATA_MODE => 'true', DATA_INDENT => 3, OUTPUT => $output);
   if (!$xml)
   {
      baselog($LOG_ERROR, "Cannot get XML::Writer");
      $output->close();
      return $FALSE;
   } ## end if (!$xml)

   $xml->startTag($section);  #OPEN section
   foreach $hash (@data)
   {
      if (defined($hash))
      {
         $xml->emptyTag('rule', %{$hash});
      }
   } ## end foreach $hash (@data)
   $xml->endTag($section);    #CLOSE section

   $xml->end();
   $output->close();

   unless (do_system([$MV, $tmp_file, "$dir/$cfg"]))
   {
      baselog($LOG_ERROR, "Cannot modify iptables configuration file", $TRUE);
      return $FALSE;
   }

   return $TRUE;
} ## end sub save_iptables($$$)

###############################################################################
# @method firewall_restart($statehash, $apps)
# @brief Manages firewall rules
# @details Flushes all firewall rules and applies them again; if there are no
#   parameters, it update only current applied rules
# @param $self      - reference to the object
# @param $statehash - applications state hash (optional)
#            key - application name
#            value - application state ($TRUE - active, $FALSE - inactive)
# @param $apps - an array of applications names needed to be updated (optional)
# @param $force - should the firewall be started if it was stopped
# @return  $TRUE   - iptables rules have been updated successfully
# @return  $FALSE  - error
###############################################################################
sub firewall_restart(;$$$)
{
   my $self = shift @_;
   $self = new() unless (defined $self);

   my $statehash = shift @_;
   my $apps      = shift @_;
   my $force     = shift @_;
   my $rules;
   my %tables   = ('*filter' => [], '*nat' => [], '*mangle' => []);
   my $app_name = '';

   if (is_stopped())
   {
      if ($force)
      {
         unless (rename("$IPTABLES_SAVE_FILE.last", $IPTABLES_SAVE_FILE))
         {
            baselog($LOG_ERROR, "Cannot rename '$IPTABLES_SAVE_FILE.last' to $IPTABLES_SAVE_FILE. Firewall cannot be started!");
            return $FALSE;
         }
         send_snmp_trap("SEC0109");
      } ## end if ($force)
      else
      {
         baselog($LOG_DEBUG, "Attempt of restarting stopped firewall.");
         return $TRUE;  # we should not restart stopped firewall by default,
                        # so we can just ignore all attempts of this.
      }
   } ## end if (is_stopped())

   my $semmgr = take_firewall_semaphore();
   return $FALSE unless (defined $semmgr);

   if (not defined $statehash)
   {

      #generate app state hash with states 'STOP'
      baselog($LOG_INFO, "Aplication states for managment are not provided. Generate them regarding linux kernel iptable.");
      $statehash = $self->collect_app_names();
      if (not defined $statehash)
      {
         baselog($LOG_ERROR, "Cannot fill applications state hash with default values.");
         return $FALSE;
      }
      %tables = %{read_applied_rules()};

      #check for application states.
      #Search only in FILTER table, because finally decision accepts at this point
      foreach my $line (@{$tables{'*filter'}})
      {

         #If accept rule was applied on firewall application is consider as running.
         $line =~ /"(.*)"/;
         $app_name = $1;
         if (defined $app_name)
         {
            $statehash->{$app_name} = $TRUE if ($line !~ /$_reject/);
         }
      } ## end foreach my $line (@{$tables...
      %tables = ('*filter' => [], '*nat' => [], '*mangle' => []);
   } ## end if (not defined $statehash...
   elsif (not is_hash($statehash))
   {
      baselog($LOG_ERROR, "Incorrect 1st argument. Must be a hash reference.");
      return $FALSE;
   }

   if (not defined $apps)
   {

      #set policy for INPUT chain
      push @{$tables{'*filter'}}, ":INPUT ACCEPT";

      #create priority chains
      push @{$tables{'*filter'}}, ":FREQUENTLY_USED_RULES - [0:0]";
      push @{$tables{'*filter'}}, ":REGULAR_USED_RULES - [0:0]";
      push @{$tables{'*filter'}}, ":RARELY_USED_RULES - [0:0]";
      push @{$tables{'*filter'}}, "-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
-A INPUT -j FREQUENTLY_USED_RULES
-A INPUT -j REGULAR_USED_RULES
-A INPUT -j RARELY_USED_RULES\n";

      #it is necessary to apply base rules and user-defined
      $statehash->{$NORTEL_LINUX_BASE_APP_NAME} = $TRUE if (not exists $statehash->{$NORTEL_LINUX_BASE_APP_NAME});
      $statehash->{$USERCFG}                    = $TRUE if (not exists $statehash->{$USERCFG});
   } ## end if (not defined $apps)
   elsif (is_array($apps))
   {

      #update rules only for specified applications
      %tables = %{read_applied_rules($apps)};
      unless (%tables)
      {
         baselog($LOG_ERROR, "Error during reading firewall rules");
         release_firewall_semaphore($semmgr);
         return $FALSE;
      } ## end unless (%tables)
        #remove applications are not listed in received applications list for managment.
      my $copy_appstate = {};
      foreach my $app (@$apps)
      {
         $copy_appstate->{$app} = $statehash->{$app} if (exists $statehash->{$app});
      }
      $statehash = $copy_appstate;
      undef $copy_appstate;
   } ## end elsif (is_array($apps))
   else
   {
      baselog($LOG_ERROR, "Wrong second parameters. Must be an array reference.");
      release_firewall_semaphore($semmgr);
      return $FALSE;
   } ## end else [ if (not defined $apps)

   my $retval = $FALSE;
   if (not $self->add_rules_by_name(\%tables, $statehash))
   {
      baselog($LOG_ERROR, "Firewall rules adding failed");
   }
   elsif (not $self->add_reject_rules_by_name(\%tables, $statehash))
   {
      baselog($LOG_ERROR, "Firewall reject rules adding failed.");
   }
   elsif (not write_rules_hash(\%tables))
   {
      baselog($LOG_ERROR, "An error occurred during writing iptables file");
   }
   else
   {
      print "Updating iptables rules:";
      if (not update_firewall_rules())
      {
         baselog($LOG_ERROR, "Error updating firewall rules");
      }
      else
      {
         &status_success;
         $retval = $TRUE;
      }
   } ## end else [ if (not $self->add_rules_by_name...
   $retval = $FALSE unless (release_firewall_semaphore($semmgr));
   return $retval;
} ## end sub firewall_restart(;$$)

###############################################################################
## @fn is_stopped()
# @brief Returns whether the firewall was stopped
# @param none
# @return $TRUE  - firewall was stopped
# @return $FALSE - firewall was not stopped
###############################################################################
sub is_stopped()
{
   if (-f "$IPTABLES_SAVE_FILE.last")
   {
      my $ret;
      # invoke /sbin/iptables-save to check is firewall is active (so retval is $OK)
      # or not (so retval is not $OK)
      do_system([$IPTABLES_SAVE], {$dsNOERROR => $TRUE, $dsRETVAL => \$ret});
      if ($ret == $OK)
      {
         baselog($LOG_ERROR, "The firewall should be stopped now, but actually it is working now.");
      } # if ($ret == $OK)
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_stopped()

###############################################################################
## @fn get_status()
# @brief Returns the firewall status
# @param none
# @return $TRUE  - firewall is started
# @return $FALSE - otherwise
###############################################################################
sub get_status()
{
   my $ret;
   do_system([$SERVICE, $IPTABLES_SERVICE, $SRV_STATUS], {$dsNOERROR => $TRUE, $dsRETVAL => \$ret});
   if ($ret == $OK)
   {
      return $TRUE;
   } # if ($ret == $OK)
   return $FALSE;
}

###############################################################################
## @fn firewall_status()
# @brief Prints the firewall status
# @param none
# @return $TRUE  - always
###############################################################################
sub firewall_status()
{
   my $stderr;
   do_system([$SERVICE, $IPTABLES_SERVICE, $SRV_STATUS], {$dsSTDOUT => $TRUE, $dsNOERROR => $TRUE, $dsERR => \@$stderr});
   if (scalar @$stderr)
   {
      baselog($LOG_ERROR, "@$stderr");
   }
   return $TRUE;
}

###############################################################################
## @fn firewall_stop()
# @brief Stops the firewall
# @param none
# @return $TRUE  - is no errors
# @return $FALSE - in case of any error
###############################################################################
sub firewall_stop()
{
   if (get_status())
   {
      if (is_stopped())
      {
         baselog($LOG_ERROR, "The firewall should be stopped now, but actually it is working now.");
      }
      do_system([$SERVICE, $IPTABLES_SERVICE, $SRV_STOP], {$dsSTDOUT => $TRUE});
      baselog($LOG_INFO, "The firewall is now stopped.");
      send_snmp_trap("SEC0110");

      # we should delete /etc/sysconfig/iptables to prevent the firewall from
      # auto-restart since only 'basefirewallconfig start' is legitimate method
      # for this
      unless (rename($IPTABLES_SAVE_FILE, "$IPTABLES_SAVE_FILE.last"))
      {
         baselog($LOG_ERROR, "Cannot rename $IPTABLES_SAVE_FILE file, so firewall can be unexpectedly auto-started.");
         return $FALSE;
      }
   } ## end if (get_status())
   else
   {
      baselog($LOG_INFO, "Cannot stop the firewall since it is already stopped.");
   }
   return $TRUE;
} ## end sub firewall_stop()

###############################################################################
# @method enable_app_rules($applname)
# @brief Enables application firewall rules
# @param $self     - reference to the object
# @param $applname - an application name
# @return  $TRUE - enabling succeeded
# @return  $FALSE - enabling failed
# @author Pavel Sergeev
# @date April 2008
###############################################################################
sub enable_app_rules($)
{
   my $self     = shift @_;
   my $applname = shift @_;
   my @commands;

   if (are_rules_applied($applname))
   {
      baselog($LOG_INFO, "Firewall rules for $applname are already applied");
      return $TRUE;
   }

   my $semmgr = take_firewall_semaphore();
   return $FALSE unless (defined $semmgr);

   my $parser = $self->get_xml_parser("$SYSDB_ROOT/$IPTABLES_CFG");
   unless ($parser)
   {
      baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
      baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
      release_firewall_semaphore($semmgr);
      return $FALSE;
   } ## end unless ($parser)

   my $rules = $parser->get_rule({'appl' => $applname});
   unless ($self->apply_rules($rules))
   {
      baselog($LOG_ERROR, "Error occurred during applying firewall rules for $applname");
      release_firewall_semaphore($semmgr);
      return $FALSE;
   } ## end unless ($self->apply_rules...
   return $FALSE unless (release_firewall_semaphore($semmgr));
   return $TRUE;
} ## end sub enable_app_rules($)

###############################################################################
# @method disable_app_rules($applname)
# @brief Disables application firewall rules
# @param $self     - reference to the object
# @param $applname - an application name
# @return $TRUE - disabling succeeded
# @return $FALSE - disabling failed
# @author Pavel Sergeev
# @date April 2008
###############################################################################
sub disable_app_rules($)
{
   my $self     = shift @_;
   my $applname = shift @_;
   my @commands;

   if (is_stopped())
   {
      baselog($LOG_DEBUG, "Cannot apply firewall rules to stopped firewall.");
      return $TRUE; # since we should not process rules for stopped firewall, just ignore this
   }

   my $semmgr = take_firewall_semaphore();
   return $FALSE unless (defined $semmgr);

   my $parser = $self->get_xml_parser("$SYSDB_ROOT/$IPTABLES_CFG");
   unless ($parser)
   {
      baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
      baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
      release_firewall_semaphore($semmgr);
      return $FALSE;
   } ## end unless ($parser)

   my $rules = $parser->get_rule({'appl' => $applname});
   $self->apply_rules($rules, 'del');

   #apply reject rules on the system
   if (not is_array($rules))
   {
      baselog($LOG_ERROR, "Cannot collect firewall rules for application '$applname'");
      return $FALSE;
   }
   foreach my $rule (@{$rules})
   {

      #check for reject attribute
      if ((exists $rule->{'reject'}) and ($rule->{'reject'} == 1))
      {

         #packet must be rejected
         my @commands = $self->generate_rule($rule, $FALSE, $_reject);
         foreach my $comm (@commands)
         {
            if (not do_system($comm))
            {
               baselog($LOG_ERROR, "Cannot apply reject rule for '$applname'");
            }
         } ## end foreach my $comm (@commands...
      } ## end if ((exists $rule->{'reject'...
   } ## end foreach my $rule (@{$rules}...

   return $FALSE unless (release_firewall_semaphore($semmgr));
   return $TRUE;
} ## end sub disable_app_rules($)

###############################################################################
# @method add_rules_by_name($tables, $app_state, $need_update)
# @brief Adds new rules to the rules hash using an array of application names
# @param $self  - reference to the object
# @param $tables  - rules hash reference. Hash keys are table names
# @param $app_state  - reference to hash with applications states
# @param $need_update - if $TRUE, this sub will updates firewall rules,
#    otherwise it only adds rules to hash (optional, default is $FALSE)
# @return $TRUE - rules successfully added
# @return $FALSE - an error occurred.
# @author Pavel Sergeev
# @date November 2008
# @note This method uses the object of the master_firewall class
###############################################################################
sub add_rules_by_name($$;$)
{
   my $self        = shift @_;
   my $tables      = shift @_;
   my $app_state   = shift @_;
   my $need_update = shift @_;
   my @app_names   = ();

   if (not is_hash($app_state))
   {
      baselog($LOG_ERROR, "Incorrect input parameter. Must be a hash reference.");
      return $FALSE;
   }

   #collect app names for managment
   foreach my $app (keys %{$app_state})
   {
      push @app_names, $app if ($app_state->{$app} == $TRUE);
   }
   if (not scalar(@app_names))
   {
      baselog($LOG_DEBUG, "Nothing to ACCEPT.");
      return $TRUE;
   }

   # Load iptables rules from the iptables configuration file
   my $parser = $self->get_xml_parser("$SYSDB_ROOT/$IPTABLES_CFG");
   unless ($parser)
   {
      baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
      baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
      return $FALSE;
   } ## end unless ($parser)

   unless ($self->get_master_firewall_object())
   {
      baselog($LOG_ERROR, "Cannot add firewall rules");
      return $FALSE;
   }

   #generating rules and add them to the hash
   my $table_name = "";
   my $rules      = undef;
   foreach my $app (@app_names)
   {
      if ($app eq $USERCFG)
      {
         $rules = get_iptables($SYSCONFIG_ROOT, $IPTABLES_USERCFG);
         unless ($rules)
         {
            baselog($LOG_DEBUG, "Could not load rules from $SYSCONFIG_ROOT/$IPTABLES_USERCFG");
         }
         for my $tmp (@$rules)
         {
            my @commands = $self->generate_rule($tmp, $TRUE);
            for my $rule (@commands)
            {

               #deleteing two array items (iptables action string, chain name)
               #after this the first item in array is a table name
               $table_name = '*' . shift(@$rule);
               push @{$tables->{$table_name}}, join(" ", @{$rule});
            } ## end for my $rule (@commands...
         } ## end for my $tmp (@$rules)
      } ## end if ($app eq $USERCFG)
      else
      {
         $rules = $parser->get_rule({'appl' => $app});
         unless ($rules)
         {
            baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
            baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
            return $FALSE;
         } ## end unless ($rules)
         for my $tmp (@$rules)
         {
            my @commands = $self->generate_rule($tmp, $TRUE);
            for my $rule (@commands)
            {
               $table_name = '*' . shift(@$rule);
               push @{$tables->{$table_name}}, join(" ", @{$rule});
            }
         } ## end for my $tmp (@$rules)
      } ## end else [ if ($app eq $USERCFG)
   } ## end foreach my $app (@app_names...
   if (defined $need_update and $need_update)
   {
      if (not write_rules_hash($tables))
      {
         baselog($LOG_ERROR, "An error occurred during writing iptables file");
         return $FALSE;
      }
      if (not update_firewall_rules())
      {
         baselog($LOG_ERROR, "Error updating firewall rules");
         return $FALSE;
      }
   } ## end if (defined $need_update...
   return $TRUE;
} ## end sub add_rules_by_name($$;$)

###############################################################################
# @method add_reject_rules_by_name($tables, $app_state)
# @brief Adds new rules to the rules hash using an array of application names
# @details This function receives list of application, ports for these applications must be open.
#          It collects names of all applications and do 'received list' XOR 'collected list'.
# @param $self  - reference to the object
# @param $tables  - rules hash reference. Hash keys are table names
# @param $app_state  - Application names with states.
# @return $TRUE - rules successfully added
# @return $FALSE - an error occurred.
###############################################################################
sub add_reject_rules_by_name($$)
{
   my $self      = shift @_;
   my $tables    = shift @_;
   my $app_state = shift @_;

   if (not is_hash($tables))
   {
      baselog($LOG_ERROR, "Incorrect or undefined first parameter.Must be a hash reference.");
      return $FALSE;
   }
   elsif (not is_hash($app_state))
   {
      baselog($LOG_ERROR, "Incorrect or undefined second parameter.Must be a hash reference.");
      return $FALSE;
   }

   #collecting stopped applications
   my @stopped_app = ();
   foreach my $app (keys %{$app_state})
   {
      push @stopped_app, $app if ($app_state->{$app} == $FALSE);
   }

   if (scalar(@stopped_app) < 1)
   {
      baselog($LOG_DEBUG, "Nothing to reject.");
      return $TRUE;
   }

   #generate reject rules for sropped applications.
   my $parser = $self->get_xml_parser("$SYSDB_ROOT/$IPTABLES_CFG");
   if ($parser)
   {

      #is required by generate_rule method.
      unless ($self->get_master_firewall_object())
      {
         baselog($LOG_ERROR, "Cannot generate reject rules. MFC is undefined.");
         return $FALSE;
      }

      foreach my $app (@stopped_app)
      {
         my $rules = $parser->get_rule({'appl' => $app});
         if (not is_array($rules))
         {
            baselog($LOG_ERROR, "Cannot collect firewall rules for application '$app'.");
            return $FALSE;
         }
         foreach my $rule (@{$rules})
         {
            if ((exists $rule->{'reject'}) and ($rule->{'reject'} == 1))
            {
               my @commands = $self->generate_rule($rule, $TRUE, $_reject);

               #add reject rules into iptables hash.
               foreach my $comm (@commands)
               {
                  my $table = "*" . shift(@{$comm});
                  push @{$tables->{$table}}, join(" ", @{$comm});
               }
            } ## end if ((exists $rule->{'reject'...
         } ## end foreach my $rule (@{$rules}...
      } ## end foreach my $app (@stopped_app...

      #release parser
      undef $parser;
   } ## end if ($parser)
   else
   {
      baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
      baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
      return $FALSE;
   } ## end else [ if ($parser)

   return $TRUE;
} ## end sub add_reject_rules_by_name($$)

###############################################################################
# @method delete_rules_by_name($appnames, $flush)
# @brief - Deletes applications firewall rules
# @details Reads applied firewall rules, except rules specified by applications
#   names. Then it applies these rules. If rule for this application has reject
#   flag in configure.xml then packets on this port will be rejected.
# @param $self     - reference to the object
# @param $appnames - reference to an array of applications names
# @param $no_reject    - flush firewall rules($TRUE) or accept reject parameters
#                    from configure.xml($FALSE). (optional, default is $FALSE)
# @return  $FALSE if an error occurred, $TRUE - otherwise
# @author Pavel Sergeev
# @date November 2008
###############################################################################
sub delete_rules_by_name($;$)
{
   my $self      = shift @_;
   my $appnames  = shift @_;
   my $no_reject = shift @_;

   if (is_stopped())
   {
      baselog($LOG_DEBUG, "Cannot apply firewall rules to stopped firewall.");
      return $TRUE; # since we should not process rules for stopped firewall, just ignore this
   }

   $no_reject = $FALSE if (not defined $no_reject);

   my $semmgr = take_firewall_semaphore();
   return $FALSE unless (defined $semmgr);

   my $retval = $FALSE;
   my $tables = read_applied_rules($appnames);
   if (not $tables)
   {
      baselog($LOG_ERROR, "Error reading firewall rules");
   }
   else
   {

      #check for reject
      if (not $no_reject)
      {

         #add_reject_rules_by_name receives hash with application states.
         #generate it.
         my %state_hash = map {$_ => $FALSE} @{$appnames};
         if (not $self->add_reject_rules_by_name($tables, \%state_hash))
         {
            baselog($LOG_ERROR, "Cannot add reject rules for applications:" . join(" ", @{$appnames}));
         }
      } ## end if (not $no_reject)

      if (not write_rules_hash($tables))
      {
         baselog($LOG_ERROR, "An error occurred during writing iptables file");
      }
      elsif (not update_firewall_rules())
      {
         baselog($LOG_ERROR, "Error updating firewall rules");
      }
      else
      {
         $retval = $TRUE;
      }
   } ## end else [ if (not $tables)
   $retval = $FALSE unless (release_firewall_semaphore($semmgr));
   return $retval;
} ## end sub delete_rules_by_name($;$)

###############################################################################
# @method collect_app_names()
# @brief - Collect applications name from iptables.xml
# @details - Collect applications list from iptables.xml, state of applications
#            is set into false(is not launched).
# @param $self     - reference to the object
# @return  reference to hash or undef if error was occured
###############################################################################
sub collect_app_names()
{
   my $self = shift @_;

   my $app_state = {};

   #collect application names from iptables.xml
   my $parser = $self->get_xml_parser("$SYSDB_ROOT/$IPTABLES_CFG");
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Iptables rules file is not found or empty.\n", $TRUE);
      baselog($LOG_ERROR, "Could not load rules from $SYSDB_ROOT/$IPTABLES_CFG");
      return undef;
   } ## end if (not defined $parser...

   #extract all applications names from iptables.xml.
   my $full_app_list = $parser->find_data_by_xpath($GETDATA, "/iptables/rule/appl");
   if (not defined $full_app_list)
   {
      baselog($LOG_ERROR, "Some error happens while extracting applications names from iptables.xml.");
      undef $parser;
      return undef;
   } ## end if (not defined $full_app_list...
   undef $parser;

   #delete duplicated string from the array
   #now all applications names will be stored as hash keys
   %{$app_state} = map {$_ => $FALSE} @{$full_app_list};

   return $app_state;
} ## end sub collect_app_names()

###############################################################################
# @fn read_applied_rules($app_names)
# @brief Reads current applied firewall rules from file
# @details Invokes "iptables-save" command, which displays current
#   iptables rules; returns them in the rules hash
# @param  $app_names - reference to an array containing application names
# @return  the reference to the rules hash, undef - if an error occurred
# @author Pavel Sergeev
# @date November 2008
###############################################################################
sub read_applied_rules(;$)
{
   my $app_names = shift @_;
   $app_names = [] if not defined $app_names;

   # save current iptables rules to the @rules variable
   my @rules = ();
   unless (do_system([$IPTABLES_SAVE], {$dsOUT => \@rules}))
   {
      baselog($LOG_ERROR, "Error with iptables-save reading");
      return undef;
   }

   my %tables = ('*filter' => [], '*nat' => [], '*mangle' => []);

   #generate hash of applied rules except specified applications
   my $tname     = '';
   my $rule      = '';
   my $skip_rule = $FALSE;
   foreach $rule (@rules)
   {
      $skip_rule = $FALSE;
      chomp($rule) if $rule;
      if ($rule =~ /^\*/)
      {
         $tname = $rule;
         next;
      }
      elsif ($tname ne '' and $rule ne "COMMIT")
      {
         for my $appname (@$app_names)
         {
            if ($rule =~ /\"$appname\"/)
            {
               $skip_rule = $TRUE;
               last;
            }
         } ## end for my $appname (@$app_names...
         if (not $skip_rule)
         {
            push @{$tables{$tname}}, $rule;
         }
      } ## end elsif ($tname ne '' and $rule...
   } ## end foreach $rule (@rules)
   return \%tables;
} ## end sub read_applied_rules(;$)

###############################################################################
# @fn write_rules_hash($tables)
# @brief Writes rules to the file for iptables-restore
# @param $self   - reference to the object
# @param $tables - reference to the hash containing firewall rules
# @return $TRUE - no errors, $FALSE - any error occurred
# @author Pavel Sergeev
# @date November 2008
#################################################################
sub write_rules_hash($)
{
   my $tables = shift @_;
   if (not is_hash($tables))
   {
      base_log($LOG_ERROR, "Invalid parameter type");
      return $FALSE;
   }
   else
   {
      my @file_content = ();

      for my $table_name (keys %{$tables})
      {
         push @file_content, "$table_name\n";
         for my $rule (@{$tables->{$table_name}})
         {
            push @file_content, $rule . "\n";
         }
         push @file_content, "COMMIT\n";
      } ## end for my $table_name (keys...
      if (not write_file($NORTEL_IPTABLES_FILE, \@file_content))
      {
         baselog($LOG_ERROR, "Error writing to $NORTEL_IPTABLES_FILE file");
         return $FALSE;
      }
   } ## end else [ if (not is_hash($tables...
   return $TRUE;
} ## end sub write_rules_hash($)

###############################################################################
# @fn update_firewall_rules()
# @brief Updates firewall rules from file using iptables-restore
# @param none
# @return $TRUE - firewall rules updated
# @return $FALSE - any errors occurred
# @author Pavel Sergeev
# @date November 2008
####################################################################
sub update_firewall_rules()
{
   if (is_stopped())
   {
      baselog($LOG_DEBUG, "Cannot update firewall rules of stopped firewall.");
      return $TRUE; # since we should not process rules for stopped firewall, just ignore this
   }
   signals_ignore();

   #chain default policy on INPUT chain to ACCEPT
   if (not(do_system([$IPTABLES, "-P", "INPUT", "ACCEPT"])))
   {
      baselog($LOG_ERROR, "Changing INPUT policy to 'ACCEPT' failed");
      signals_default();
      return $FALSE;
   } ## end if (not(do_system([$IPTABLES...

   if (system("$CAT $NORTEL_IPTABLES_FILE | $IPTABLES_RESTORE") != 0)
   {
      baselog($LOG_ERROR, "Error restoring iptables rules");
   }

   #change default policy on INPUT chain
   if (not(do_system([$IPTABLES, "-P", "INPUT", "DROP"])))
   {
      baselog($LOG_ERROR, "Changing INPUT policy to 'DROP' failed");
      signals_default();
      return $FALSE;
   } ## end if (not(do_system([$IPTABLES...

   #save firewall changes
   do_system([$SERVICE, $IPTABLES_SERVICE, "save"]);

   signals_default();
   return $TRUE;
} ## end sub update_firewall_rules()

###############################################################################
# @fn are_rules_applied($appname)
# @brief Checks that firewall rules are applied for the specified application
# @param $appname - application name
# @return $TRUE  - rules are applied
# @return $FALSE - rules are not applied
# @author Pavel Sergeev
# @date November 2008
###############################################################################
sub are_rules_applied($)
{
   my $appname = shift @_;
   if (not defined($appname) or $appname eq "")
   {
      baselog($LOG_ERROR, "Application name is not specified");
      return $FALSE;
   }
   my @rules = ();
   do_system([$SERVICE, $IPTABLES_SERVICE, $SRV_STATUS], {$dsOUT => \@rules});
   foreach my $rule (@rules)
   {
      if ($rule =~ /\* $appname \*/)
      {
         return $TRUE;
      }
   } ## end foreach my $rule (@rules)
   return $FALSE;
} ## end sub are_rules_applied($)

#####################################################
# @fn take_firewall_semaphore()
# @brief takes SEM_FIREWALL semaphore
# @return $semmgr - the reference to a semaphore manager object
# @return undef   - the semaphore cannot be taken
# @author Vladimir Ivanov
# @date January 2009
#####################################################
sub take_firewall_semaphore()
{
   my $semmgr = base_semctl::new($base_semctl::SBS_FIREWALL);
   if (!defined($semmgr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
   }
   elsif (!$semmgr->sem_take([$SEM_FIREWALL], $WAIT_FLAG, $TIME_TO_WAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphore $SEM_FIREWALL.");
      undef $semmgr;
   }
   return $semmgr;
} ## end sub take_firewall_semaphore()

#####################################################
# @fn release_firewall_semaphore($semmgr)
# @brief releases SEM_FIREWALL semaphore
# @param $semmgr - the reference to semaphore manager object
# @return $TRUE - the semaphore is released successfully
# @return $FALSE - the semaphore cannot be released
# @author Vladimir Ivanov
# @date January 2009
#####################################################
sub release_firewall_semaphore($)
{
   my $semmgr = shift @_;
   if (!defined $semmgr)
   {
      baselog($LOG_ERROR, "Undefined parameter.");
      return $FALSE;
   }

   my $retval = $TRUE;
   if (!$semmgr->sem_give($SEM_FIREWALL))
   {
      baselog($LOG_ERROR, "Cannot release semaphore $SEM_FIREWALL.");
      $retval = $FALSE;
   }
   undef $semmgr;
   return $retval;
} ## end sub release_firewall_semaphore($)

###############################################################################
# @method get_master_firewall_object()
# @brief Creates the master_firewall class object
# @details After the first creation it saves this object into $self object,
#   then, if $self->{'mfc_object'} is already defined, it returns $TRUE
# @param $self - reference to the object
# @return $TRUE - the master_firewall class object is created successfully
#                 or it already exists.
# @return $FALSE - the master_firewall class object is not created.
# @author Vladimir Ivanov
# @date January 2009
###############################################################################
sub get_master_firewall_object()
{
   my $self   = shift @_;
   my $retval = $TRUE;
   if (defined $self->{'mfc_object'})
   {
      baselog($LOG_DEBUG, "master_firewall object already exists. Re-use it.");
   }
   else
   {
      baselog($LOG_DEBUG, "master_firewall object does not exist. Create it.");
      $self->{'mfc_object'} = master_firewall->new();
      unless (defined $self->{'mfc_object'})
      {
         baselog($LOG_ERROR, "Cannot create master_firewall object.");
         $retval = $FALSE;
      }
   } ## end else [ if (defined $self->{'mfc_object'...
   return $retval;
} ## end sub get_master_firewall_object()

1;

__END__

=head1 NAME

base_firewall - LinuxBase interface to manage firewall

=head1 SYNOPSIS

use base_firewall;

my $fw_obj = new base_firewall();
$fw_obj->firewall_restart();

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
