## @file base_ftp.pm
# @brief This module is Linux Base File transfer management library.
# @details This module provides a basic functionality to setup an access
#          to pre-defined in configure.xml files directories.
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02
# @li Two functions were added: activate_sftp_mounts, deactivate_sftp_mounts
# @li Two functions were changed in order to support list of users as a parameter:
#     deactivate_ftp_mounts, activate_ftp_mounts.
# @version 1.03 - The following functions have been moved to hardening_common.pm module:
#  enable_service_firewall, disable_service_firewall, change_firewall_port_status.
# @version 1.04 -
# @li Fixed issue with "grep" command.
# @li Fixed issue when "undefined variable" error is printed if one of users was removed manually.
# @li Add restriction that FTP users only can use aliasDir equal to "/"
# @version 1.50 -
# @li - 'get_mount_counter' has been moved to common_functions module
# @li the following functions have been added: mount_sftp_jail_dirs, umount_sftp_jail_dirs,
#     mount_UCM_dirs, umount_UCM_dirs, get_ftp_dirs_for_user, is_all_directories_mounted,
#     check_ftp_dirs_for_user
# @version 1.51 - new functions have been added: create_user_ftp_config_file,
#                 remove_user_ftp_config_file . New attribute 'chrootDir' has been added.
# @version 1.52 - Q02018655-01 - fixed umount_UCM_dirs to only delete the actual dir
#                 associated with a mount point in case multiple mount points share a
#                 common first-level dir
# @version 1.53 - Q02030805. Fixed issue with mounting directories for UCM users.

package base_ftp;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&prepare_ftp_homes &activate_ftp_mounts &deactivate_ftp_mounts
  &close_ftp_firewall &close_tftp_firewall &open_tftp_firewall &activate_sftp_mounts
  &deactivate_sftp_mounts &open_ftp_firewall $FTP_SRV &mount_sftp_jail_dirs &umount_sftp_jail_dirs
  &mount_UCM_dirs &umount_UCM_dirs &get_ftp_dirs_for_user &is_all_directories_mounted
  &check_ftp_dirs_for_user $VSFTPD_USER_CONF $FTP_ROOT &remove_user_ftp_config_file);

our $VERSION = 1.53;

use warnings;
use strict;

use baselog;
use install_common;
use base_system;
use ifxml;
use base_accounts;
use hardening::hardening_common;

our $FTP_SRV          = 'vsftpd';
our $VSFTPD_USER_CONF = '/etc/vsftpd_user_conf';

# configure.xml attributes.
my $Csrc    = 'srcDir';
my $Cdst    = 'aliasDir';
my $Cuser   = 'user';
my $Cchroot = 'chrootDir';

# UCM - users additional directories hash. key - name of the group, value - reference to array with two elements.
# The first element - name of the source directory for mount. The second - is the hame of the directory which will
# be created under user's home directory where source directory will be mounted.
my %UCM_ADDITIONAL_DIRS = (
                           'securityadmin' => ['var/opt/nortel/Jboss-Quantum/conf/elementRegistry', 'var/opt/nortel/Jboss-Quantum/conf/elementRegistry'],
                           'logadmin'      => ['var/log/nortel',                                    'log'],
                           'patchadmin'    => ['var/opt/nortel/patch',                              'patch'],
                           'maintadmin'    => ['var/opt/nortel/dfoTools/pcap',                      'var/opt/nortel/dfoTools/pcap']
                          );

## @skipline
sub prepare_ftp_homes($);
## @skipline
sub prepare_app_ftp_home($);
## @skipline
sub create_mount_path($$);
## @skipline
sub activate_ftp_mounts(;$);
## @skipline
sub deactivate_ftp_mounts(;$);
## @skipline
sub get_mount_counter($);
## @skipline
sub close_ftp_firewall();
## @skipline
sub open_tftp_firewall();
## @skipline
sub open_ftp_firewall();
## @skipline
sub close_tftp_firewall();
## @skipline
sub mount_sftp_jail_dirs(;$);
## @skipline
sub umount_sftp_jail_dirs(;$);
## @skipline
sub mount_UCM_dirs(;$);
## @skipline
sub umount_UCM_dirs(;$);
## @skipline
sub get_ftp_dirs_for_user($);
## @skipline
sub is_all_directories_mounted($);
## @skipline
sub check_ftp_dirs_for_user($);
## @skipline
sub create_user_ftp_config_file($;$);
## @skipline
sub remove_user_ftp_config_file($);

###############################################################################
## @fn  prepare_ftp_homes($applist)
# @brief   It creates a home directories and mount points for provided list
#          of applications.
# @details
# @param    $applist - reference to an application list or application name
#       to perform an action with.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub prepare_ftp_homes($)
{
   my $applist = shift @_;
   if (!defined $applist)
   {
      baselog($LOG_ERROR, "'applist' is undefined.");
      return $FALSE;
   }

   my $retval = $TRUE;
   if (is_array($applist))
   {
      foreach my $app (@{$applist})
      {
         if (!prepare_app_ftp_home($app))
         {
            baselog($LOG_ERROR, "Failed to setup FTP home for '$app' application.");
            $retval = $FALSE;
         }
      } ## end foreach my $app (@{$applist...
   } ## end if (is_array($applist)...
   else
   {
      if (!prepare_app_ftp_home($applist))
      {
         baselog($LOG_ERROR, "Failed to setup FTP home for '$applist' application.");
         $retval = $FALSE;
      }
   } ## end else [ if (is_array($applist)...
   return $retval;
} ## end sub prepare_ftp_homes($)

###############################################################################
## @fn  prepare_app_ftp_homes($app)
# @brief    It creates a home directory and mount points for provided
#          application.
# @details
# @param    $app - application's name
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub prepare_app_ftp_home($)
{
   my $app      = shift @_;
   my $retval   = $FALSE;
   my $dir_refs = undef;

   if ((!defined $app) or ($app eq ''))
   {
      baselog($LOG_ERROR, 'Application name is not defined.');
   }
   else
   {
      my $parser = ifxml::new("$APP_CFG_ROOT/$app/", $APPLCFG);

      if (!$parser)
      {
         baselog($LOG_ERROR, "Parser (for '$app' application) is undefined.");
         return $FALSE;
      }
      $retval = $TRUE;
      $dir_refs = $parser->find_data_by_xpath($GETDATA, "/$FTP/dir");
      if (!defined $dir_refs)
      {
         baselog($LOG_INFO, "Application $app have no rules.");
         return $TRUE;

      } ## end if (!defined $dir_refs...

      foreach my $rule_ref (@$dir_refs)
      {
         if (!defined $rule_ref->{$Cdst})
         {
            baselog($LOG_WARNING, "'$Cdst' parameter is not defined. Rule will be skipped.");
            next;
         }

         if (!defined $rule_ref->{$Cuser})
         {
            baselog($LOG_WARNING, "'$Cuser' parameter is not defined. Rule will be skipped.");
            next;
         }

         if (!create_mount_path($rule_ref->{$Cdst}, $rule_ref->{$Cuser}))
         {
            baselog($LOG_ERROR, "Failed to prepare home for '$rule_ref->{$Cuser}' and '$rule_ref->{$Cdst}' extention.");
            $retval = $FALSE;
         }
      } ## end foreach my $rule_ref (@$dir_refs...
      undef $parser;
   } ## end else [ if ((!defined $app) or...

   return $retval;
} ## end sub prepare_app_ftp_home($)

###############################################################################
## @fn      create_mount_path($dst, $user)
# @brief    It creates an "entry point" to extended part
#           FTP sandbox.
# @details
# @param    $user    - user (owner) of new mount path
# @param    $dst - "entry point" path as user will see it from
#           FTP sandbox.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub create_mount_path($$)
{
   my ($dst, $user) = @_;
   my $retval = $FALSE;
   my (undef, undef, undef, undef, undef, undef, undef, $mount_path, undef) = getpwnam($user);

   unless (defined $mount_path)
   {
      baselog($LOG_ERROR, "User '$user' does not exist. Path '$dst' will not be created for this user.\n", $TRUE);
      return $FALSE;
   }

   if (!-d $mount_path)
   {
      baselog($LOG_ERROR, "Root directory '$mount_path' does not exist.");
      return $FALSE;
   }

   if ($dst eq '/')
   {
      baselog($LOG_WARNING, "Attention: Destination directory  home that can be overwritten by another rule.");
      return $TRUE;
   }

   my $old_dirs       = $mount_path;
   my @dirs_list      = split("/", $dst);
   my $tmp_mount_path = $mount_path . $dst;
   baselog($LOG_DEBUG, "Directories: @dirs_list");
   baselog($LOG_INFO,  "Creating mount path '$tmp_mount_path'.");
   if (-d $tmp_mount_path)
   {
      baselog($LOG_DEBUG, "Mount path '$tmp_mount_path' already exists.");
      return $TRUE;
   }

   foreach my $dir_add (@dirs_list)
   {
      $old_dirs = "$old_dirs/$dir_add";
      if (!-d $old_dirs)
      {
         last;
      }
   } ## end foreach my $dir_add (@dirs_list...
   baselog($LOG_DEBUG, "Adding new directory structure ('$old_dirs').");
   my $cmd_retval;

   if (!do_system([$MKDIR, '-p', $tmp_mount_path], {$dsRETVAL => \$cmd_retval}))
   {
      baselog($LOG_ERROR, "Cannot create '$tmp_mount_path' mount directory. (Return: $cmd_retval)");
   }
   else
   {
      baselog($LOG_DEBUG, "Mount path '$tmp_mount_path' successfully created.");

      #change ownership
      if (!do_system([$CHOWN, '-R', "$user:", $old_dirs], {$dsRETVAL => \$cmd_retval}))
      {
         baselog($LOG_ERROR, "Cannot create '$tmp_mount_path' mount directory. (Return: $cmd_retval)");
      }
      else
      {
         $retval = $TRUE;
      }
      unless (do_system([$CHMOD, '-R', '700', $old_dirs], {$dsRETVAL => \$cmd_retval}))
      {
         baselog($LOG_ERROR, "Cannot create '$tmp_mount_path' mount directory. (Return: $cmd_retval)");
         $retval = $FALSE;
      }
   } ## end else [ if (!do_system([$MKDIR...
   return $retval;
} ## end sub create_mount_path($$)

###############################################################################
## @fn  activate_ftp_mounts(;$)
# @brief    It mounts configured in configure.xml files aliases.
# @param    $users_list - reference to array with names of users which aliases
#           should be mounted. If not specified, then activate FTP mount points
#           only.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @note  This version can not handle changes in the configure.xml files as
#        account and home creation and is a separate action.
sub activate_ftp_mounts(;$)
{
   my ($users_list) = (@_);
   if (defined($users_list))
   {
      if (!is_array($users_list))
      {
         baselog($LOG_ERROR, "Wrong argument passed. Activation is stopped.");
         return $FALSE;
      }
   } ## end if (defined($users_list...
   else
   {
      $users_list = base_accounts::get_ftp_accounts();
   }
   my $retval   = $TRUE;
   my $dir_refs = undef;
   my $parser   = ifxml::new($APP_CFG_ROOT, $APPLCFG);

   if (!$parser)
   {
      baselog($LOG_ERROR, 'Parser is undefined.');
      return $FALSE;
   }

   $dir_refs = $parser->find_data_by_xpath($GETDATA, "/$FTP/dir");
   unless (is_array($dir_refs))
   {
      baselog($LOG_ERROR, 'Failed to check rules.');
      return $FALSE;

   } ## end unless (is_array($dir_refs...

   foreach my $rule_ref (@$dir_refs)
   {
      if (!defined $rule_ref->{$Cuser})
      {
         baselog($LOG_WARNING, "'$Cuser' parameter is not defined. Rule will be skipped.");
         next;
      }

      #handle FTP accounts which want to change the chroot directory
      if (base_accounts::is_ftp_account($rule_ref->{$Cuser}))
      {
         if ($rule_ref->{$Cchroot} || (defined $rule_ref->{$Cdst} && $rule_ref->{$Cdst} eq '/'))
         {
            my $chroot_dir = $rule_ref->{$Cchroot};
            if (!$chroot_dir)
            {

               # as account need to get access to the $Csrc directory the chroot directory
               # will be this directory
               $chroot_dir = $rule_ref->{$Csrc};
               if (!-d $chroot_dir)
               {
                  baselog($LOG_WARNING, "Directory '$chroot_dir' does not exist. Skip the rule.");
                  next;
               }
            } ## end if (!$chroot_dir)
            if (!create_user_ftp_config_file($rule_ref->{$Cuser}, $chroot_dir))
            {
               baselog($LOG_ERROR, "Can not create FTP configuration file for user '$rule_ref->{$Cuser}'.");
            }
            next;
         } ## end if ($rule_ref->{$Cchroot...
      } ## end if (base_accounts::is_ftp_account...

      if (!defined $rule_ref->{$Cdst})
      {
         baselog($LOG_WARNING, "'$Cdst' parameter is not defined. Rule will be skipped.");
         next;
      }

      if (!defined $rule_ref->{$Csrc})
      {
         baselog($LOG_WARNING, "'$Csrc' parameter is not defined. Rule will be skipped.");
         next;
      }

      if (pos_in_array($users_list, $rule_ref->{$Cuser}) < 0)
      {
         next;
      }

      if ($rule_ref->{$Cdst} eq '/')
      {
         baselog($LOG_WARNING, "It is not allowed to mount directly to home directory.");
         next;
      }

      if (!-d $rule_ref->{$Csrc})
      {
         baselog($LOG_WARNING, "Source directory ('$rule_ref->{$Csrc}') does not exist. Skip the rule.");
         next;
      }

      my (undef, undef, undef, undef, undef, undef, undef, $mount_path, undef) = getpwnam($rule_ref->{$Cuser});
      if (!defined($mount_path))
      {
         baselog($LOG_WARNING, "Home directory for user '$rule_ref->{$Cuser}' could not be retrieved. Rule is skipped.");
         next;
      }
      $mount_path = $mount_path . $rule_ref->{$Cdst};

      if (!-d $mount_path)
      {
         baselog($LOG_WARNING, "Alias directory ('$mount_path') does not exist. Skip the rule.");
         next;
      }

      # check if mount point is already in use
      if (get_mount_counter($mount_path) == 0)
      {
         if (!do_system([$MOUNT, '--bind', '-o', 'nosuid', $rule_ref->{$Csrc}, $mount_path]))
         {
            baselog($LOG_ERROR, "Cannot mount '$rule_ref->{$Csrc}' to '$mount_path' mount directory.");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_INFO, "FTP path '$mount_path' is activated.");
         }
      } ## end if (get_mount_counter(...
      else
      {
         baselog($LOG_DEBUG, "FTP path '$mount_path' is already in use.");
      }
   } ## end foreach my $rule_ref (@$dir_refs...

   undef $parser;

   return $retval;
} ## end sub activate_ftp_mounts(;$)

###############################################################################
## @fn  deactivate_ftp_mounts($;$)
# @brief  It de-mounts configure.xml files aliases.
# @param  $users_list - reference to array with names of users which aliases
#         should be unmounted. If not specified, then deactivate FTP mount
#         points only.
# @return $TRUE  - operation successful
# @return $FALSE - some error happens
# @note  This version can not handle changes in the configure.xml files as
#        account and home creation and is a separate action.
sub deactivate_ftp_mounts(;$)
{
   my ($users_list) = (@_);
   if (defined($users_list))
   {
      if (!is_array($users_list))
      {
         baselog($LOG_ERROR, "Wrong argument passed. Deactivation is stopped.");
         return $FALSE;
      }
   } ## end if (defined($users_list...
   else
   {
      $users_list = base_accounts::get_accounts_by_type($base_accounts::LINUX_FTP_USER);
   }

   my $retval    = $TRUE;
   my $dir_refs  = undef;
   my $parser    = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   my $user_type = undef;

   if (!$parser)
   {
      baselog($LOG_ERROR, 'Parser is undefined.');
      return $FALSE;
   }

   $dir_refs = $parser->find_data_by_xpath($GETDATA, "/$FTP/dir");
   if (!defined $dir_refs)
   {
      baselog($LOG_ERROR, 'Failed to check rules.');
      return $FALSE;

   } ## end if (!defined $dir_refs...

   if (!defined $dir_refs)
   {
      baselog($LOG_ERROR, 'Failed to check rules.');
      return $FALSE;

   } ## end if (!defined $dir_refs...

   foreach my $rule_ref (@$dir_refs)
   {
      if ($rule_ref->{$Cchroot} || (defined($rule_ref->{$Cdst}) && $rule_ref->{$Cdst} eq '/'))
      {
         baselog($LOG_INFO, "There is nothing to un-mount.");
         next;
      }
      if (!defined $rule_ref->{$Cdst})
      {
         baselog($LOG_WARNING, "Destination parameter is not defined. Rule is skipped.");
         next;
      }

      if (!defined $rule_ref->{$Cuser})
      {
         baselog($LOG_WARNING, "'$Cuser' parameter is not defined. Rule will be skipped.");
         next;
      }

      if (pos_in_array($users_list, $rule_ref->{$Cuser}) < 0)
      {
         next;
      }

      my (undef, undef, undef, undef, undef, undef, undef, $mount_path, undef) = getpwnam($rule_ref->{$Cuser});
      if (!defined($mount_path))
      {
         baselog($LOG_WARNING, "Home directory for user '$rule_ref->{$Cuser}' could not be retrieved. Rule is skipped.");
         next;
      }
      $mount_path = $mount_path . $rule_ref->{$Cdst};

      if (!-d $mount_path)
      {
         baselog($LOG_WARNING, "Alias directory ('$mount_path') does not exist. Skip the rule.");
         next;
      }

      # check how many records for the mount point is in use.
      my $mount_counter = get_mount_counter($mount_path);

      if ($mount_counter == 0)
      {
         baselog($LOG_DEBUG, "'$mount_path' record already unregistered.");
         next;
      }

      while ($mount_counter > 0)
      {
         if (!do_system([$UMOUNT, '-l', $mount_path]))
         {
            baselog($LOG_WARNING, "Cannot unregister '$mount_path'.");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_INFO, "'$mount_path' unregistered successfully,");
         }
         $mount_counter--;
      } ## end while ($mount_counter > 0...
   } ## end foreach my $rule_ref (@$dir_refs...
   return $retval;
} ## end sub deactivate_ftp_mounts(;$)
###############################################################################
## @fn  open_ftp_firewall()
# @brief    It opens FTP firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.
sub open_ftp_firewall()
{
   my $retval = $FALSE;

   if (!enable_service_firewall($FTP))
   {
      baselog($LOG_ERROR, 'Cannot enable firewall settings for FTP service.');
   }
   else
   {
      baselog($LOG_INFO, 'FTP service is enabled.');
      $retval = $TRUE;
   }

   return $retval;
} ## end sub open_ftp_firewall()

###############################################################################
## @fn open_tftp_firewall()
# @brief    It opens TFTP firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub open_tftp_firewall()
{
   my $retval = $FALSE;

   if (!enable_service_firewall($TFTP))
   {
      baselog($LOG_ERROR, 'Cannot enable firewall settings for TFTP service.');
   }
   else
   {
      baselog($LOG_INFO, 'TFTP service is enabled.');
      $retval = $TRUE;
   }

   return $retval;
} ## end sub open_tftp_firewall()

###############################################################################
## @fn  close_ftp_firewall()
# @brief    It closes FTP firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub close_ftp_firewall()
{
   my $retval = $TRUE;

   if (disable_service_firewall($FTP))
   {
      baselog($LOG_DEBUG, 'FTP firewall disabled.');
   }
   else
   {
      baselog($LOG_DEBUG, 'Cannot disable FTP firewall.');
      $retval = $FALSE;
   }

   return $retval;
} ## end sub close_ftp_firewall()

###############################################################################
## @fn  close_tftp_firewall()
# @brief    It closes TFTP firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub close_tftp_firewall()
{
   my $retval = $TRUE;

   if (disable_service_firewall($TFTP))
   {
      baselog($LOG_DEBUG, 'TFTP firewall disabled.');
   }
   else
   {
      baselog($LOG_DEBUG, 'Cannot disable TFTP firewall.');
      $retval = $FALSE;
   }

   return $retval;
} ## end sub close_tftp_firewall()

###############################################################################
## @fn  activate_sftp_mounts()
# @brief    Enables SFTP mount points
# @details  Enables mount points for Linux_SFTP_User and Linux_FTP_SFTP_User users.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
sub activate_sftp_mounts()
{
   baselog($LOG_INFO, "Activate mount points for sftp users.");
   return activate_ftp_mounts(base_accounts::get_sftp_accounts());
}

###############################################################################
## @fn  deactivate_sftp_mounts()
# @brief    Disables SFTP mount points
# @details  Disables mount points for Linux_SFTP_User and Linux_FTP_SFTP_User users.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
sub deactivate_sftp_mounts()
{
   baselog($LOG_INFO, "Deactivate mount points for sftp users.");
   return deactivate_ftp_mounts(base_accounts::get_sftp_accounts());
}

###############################################################################
## @fn mount_sftp_jail_dirs(;$users)
# @brief Mount directories required for SFTP access to the system
# @param $users - reference to the list with names of users. If not defined,
#        then directories will be mounted for all accounts those use SFTP.
# @return $TRUE - operation is successfull.
# @return $FALSE - otherwise
# @date December 2008
sub mount_sftp_jail_dirs(;$)
{
   my $retval     = $TRUE;
   my $users      = shift(@_);
   my $sftp_users = base_accounts::get_sftp_accounts();
   if (!defined($sftp_users) || scalar(@$sftp_users) == 0)
   {
      baselog($LOG_INFO, "There are no configured SFTP users.");
      return $TRUE;
   }
   if (!defined($users))
   {
      baselog($LOG_DEBUG, "Users list is not defined. Mount directories for all users those use SFTP.");
      $users = $sftp_users;
   }
   else
   {
      if (!is_array($users))
      {
         baselog($LOG_ERROR, "Specified parameter is not a reference to a list.");
         return $FALSE;
      }
   } ## end else [ if (!defined($users))
   for my $user (@$users)
   {

      # check that user is SFTP
      if (pos_in_array($sftp_users, $user) < 0)
      {
         baselog($LOG_DEBUG, "User '$user' is not SFTP.");
         next;
      }
      my $home = (getpwnam($user))[7];
      if (!defined($home))
      {
         baselog($LOG_ERROR, "Home directory for user $user is not defined.");
         $retval = $FALSE;
         next;
      } ## end if (!defined($home))
      my $prep_result = $TRUE;
      for my $jaildir (@JAIL_DIRS)
      {
         if (!-d "$home/$jaildir")
         {
            if (!do_system([$MKDIR, '-p', '-m', '755', "$home/$jaildir"]))
            {
               baselog($LOG_ERROR, "Could not create '$home/$jaildir' directory.");
               $prep_result = $FALSE;
               next;
            } ## end if (!do_system([$MKDIR...
         } ## end if (!-d "$home/$jaildir"...

         # check that directory is not mounted yet
         if (get_mount_counter("$home/$jaildir") != 0)
         {
            baselog($LOG_DEBUG, "Directory '$home/$jaildir' is already mounted.");
            next;
         }
         my @err = ();
         if (!do_system([$MOUNT, '--rbind', "$JAILTEMPLATEDIR/$jaildir", "$home/$jaildir"], {$dsERR => \@err}))
         {
            baselog($LOG_ERROR, "Could not mount '$JAILTEMPLATEDIR/$jaildir' directory: @err.");
            $prep_result = $FALSE;
            next;
         } ## end if (!do_system([$MOUNT...
      } ## end for my $jaildir (@JAIL_DIRS...

      if (!$prep_result)
      {
         baselog($LOG_ERROR, "Could not prepare the jail for user '$user'.");
         $retval = $FALSE;
         next;
      } ## end if (!$prep_result)

      # add user $user to jail. Change home directory and shell
      my $fd;
      my $entry = "";
      if (!open($fd, '<', $PASSWDFILE))
      {
         baselog($LOG_ERROR, "Could not open '$PASSWDFILE' file.");
         $retval = $FALSE;
         next;
      } ## end if (!open($fd, '<', $PASSWDFILE...
      while (chomp($entry = <$fd>))
      {
         if ($entry =~ /^$user:/)
         {
            $entry =~ s/$RBASH/$BASH/;
            last;
         }
      } ## end while (chomp($entry = <$fd>...
      if (!close($fd))
      {
         baselog($LOG_ERROR, "Could not close '$PASSWDFILE' file.");
         $prep_result = $FALSE;
         next;
      } ## end if (!close($fd))

      if (!open($fd, '>', "$home/$PASSWDFILE"))
      {
         baselog($LOG_ERROR, "Could not open '$home/$PASSWDFILE' file.");
         $retval = $FALSE;
         next;
      } ## end if (!open($fd, '>', "$home/$PASSWDFILE"...
      print $fd $entry;
      if (!close($fd))
      {
         baselog($LOG_ERROR, "Could not close '$PASSWDFILE' file.");
         $retval = $FALSE;
         next;
      } ## end if (!close($fd))
      if (!do_system([$CHMOD, '644', "$home/$PASSWDFILE"]))
      {
         baselog($LOG_ERROR, "Can not change file permissions.");
      }

      # copy /etc/group file to jail
      if (!do_system([$CP, '--preserve', $GROUPFILE, "$home/$GROUPFILE"]))
      {
         baselog($LOG_ERROR, "Could not copy '$GROUPFILE'.");
         $retval = $FALSE;
         next;
      } ## end if (!do_system([$CP, '--preserve'...

      if ($prep_result)
      {
         baselog($LOG_DEBUG, "Jail for user '$user' prepared successfully.");
      }
      else
      {
         $retval = $FALSE;
      }
   } ## end for my $user (@$users)

   return $retval;
} ## end sub mount_sftp_jail_dirs(;$)

###############################################################################
## @fn umount_sftp_jail_dirs(;$users)
# @brief Umount directories required for SFTP access from the system
# @param $users - reference to the list with names of users. If not defined,
#        then directories will be umounted for all accounts those use SFTP.
# @return $TRUE - operation is successfull.
# @return $FALSE - otherwise
# @data December 2008
sub umount_sftp_jail_dirs(;$)
{
   my $retval     = $TRUE;
   my $users      = shift(@_);
   my $sftp_users = base_accounts::get_sftp_accounts();
   if (!defined($sftp_users) || scalar(@$sftp_users) == 0)
   {
      baselog($LOG_INFO, "There are no configured SFTP users.");
      return $TRUE;
   }
   if (!defined($users))
   {
      baselog($LOG_DEBUG, "Users list is not defined. Umount directories for all users those use SFTP.");
      $users = $sftp_users;
   }
   else
   {
      if (!is_array($users))
      {
         baselog($LOG_ERROR, "Specified parameter is not a reference to a list.");
         return $FALSE;
      }
   } ## end else [ if (!defined($users))
   for my $user (@$users)
   {

      # check that user is SFTP
      if (pos_in_array($sftp_users, $user) < 0)
      {
         baselog($LOG_DEBUG, "User '$user' is not SFTP.");
         next;
      }

      my $home = (getpwnam($user))[7];
      if (!defined($home))
      {
         baselog($LOG_ERROR, "Home directory for user $user is not defined.");
         $retval = $FALSE;
         next;
      } ## end if (!defined($home))
      my $prep_result = $TRUE;
      for my $jaildir (@JAIL_DIRS)
      {
         if (-d "$home/$jaildir")
         {

            # check that directory is not mounted yet
            if (get_mount_counter("$home/$jaildir") == 0)
            {
               baselog($LOG_DEBUG, "Directory '$home/$jaildir' is not mounted.");
               next;
            }
            my @err = ();
            if (!do_system([$UMOUNT, '-l', "$home/$jaildir"], {$dsERR => \@err}))
            {
               baselog($LOG_ERROR, "Could not umount '$home/$jaildir' directory: @err.");
               $prep_result = $FALSE;
            }
            else
            {
               if (!do_system([$RM, '-rf', "$home/$jaildir"]))
               {
                  baselog($LOG_ERROR, "Can not remove '$home/$jaildir' directory.");
               }
            } ## end else [ if (!do_system([$UMOUNT...
         } ## end if (-d "$home/$jaildir"...
      } ## end for my $jaildir (@JAIL_DIRS...
      if ($prep_result)
      {
         baselog($LOG_DEBUG, "Jail for user '$user' has not been cleaned successfully.");
      }
      else
      {
         baselog($LOG_DEBUG, "Jail for user '$user' cleanup failed.");
         $retval = $FALSE;
      }
   } ## end for my $user (@$users)

   return $retval;
} ## end sub umount_sftp_jail_dirs(;$)

###############################################################################
## @fn mount_UCM_dirs(;$users)
# @brief Mount directories required for UCM accounts
# @param $users - reference to the list with names of users. If not defined,
#        then directories will be mounted for all UCM accounts.
# @return $TRUE - operation is successfull.
# @return $FALSE - otherwise
# @date December 2008
sub mount_UCM_dirs(;$)
{
   my $retval    = $TRUE;
   my $users     = shift(@_);
   my $UCM_users = base_accounts::get_accounts_by_type($base_accounts::UCM_USER);
   if (!defined($UCM_users) || scalar(@$UCM_users) == 0)
   {
      baselog($LOG_INFO, "There are no configured UCM users.");
      return $TRUE;
   }
   if (!defined($users))
   {
      baselog($LOG_DEBUG, "Users list is not defined. Mount directories for all UCM users.");
      $users = $UCM_users;
   }
   else
   {
      if (!is_array($users))
      {
         baselog($LOG_ERROR, "Specified parameter is not a reference to a list.");
         return $FALSE;
      }
   } ## end else [ if (!defined($users))
   my %users_hash = ();  # hash with keys - group names, values - the reference
                         # to array with users those belong to group
   for my $groupname (keys(%UCM_ADDITIONAL_DIRS))
   {
      my @arr = split(/ /, (getgrnam($groupname))[3]);
      $users_hash{$groupname} = \@arr;
   }

   for my $user (@$users)
   {

      # check that user is UCM user
      if (pos_in_array($UCM_users, $user) < 0)
      {
         baselog($LOG_DEBUG, "User '$user' is not UCM.");
         next;
      }
      my $home = (getpwnam($user))[7];
      if (!defined($home))
      {
         baselog($LOG_ERROR, "Home directory for user '$user' is not defined. User is skipped.");
         $retval = $FALSE;
         next;
      } ## end if (!defined($home))
      my $prep_result = $TRUE;
      for my $group (keys(%UCM_ADDITIONAL_DIRS))
      {
         if (pos_in_array($users_hash{$group}, $user) < 0)
         {
            next;
         }
         if (get_mount_counter("$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]") != 0)
         {
            baselog($LOG_DEBUG, "Directory '$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' is already mounted.");
            next;
         }
         if (!do_system([$MKDIR, '-p', '-m', '660', "$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]"]))
         {
            baselog($LOG_ERROR, "Could not create '$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' directory.");
            $prep_result = $FALSE;
            next;
         } ## end if (!do_system([$MKDIR...
         my @err = ();
         if (!do_system([$MOUNT, '--rbind', "/@{$UCM_ADDITIONAL_DIRS{$group}}[0]", "$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]"], {$dsERR => \@err}))
         {
            baselog($LOG_ERROR, "Could not mount '/@{$UCM_ADDITIONAL_DIRS{$group}}[0]' directory: @err.");
            $prep_result = $FALSE;
            next;
         } ## end if (!do_system([$MOUNT...
      } ## end for my $group (keys(%UCM_ADDITIONAL_DIRS...

      if ($prep_result)
      {
         baselog($LOG_DEBUG, "Additional directories for UCM user '$user' mounted successfully.");
      }
      else
      {
         $retval = $FALSE;
      }
   } ## end for my $user (@$users)

   return $retval;
} ## end sub mount_UCM_dirs(;$)

###############################################################################
## @fn umount_UCM_dirs(;$users)
# @brief Umount directories required for UCM accounts
# @param $users - reference to the list with names of users. If not defined,
#        then directories will be umounted for all UCM accounts.
# @return $TRUE - operation is successfull.
# @return $FALSE - otherwise
# @date December 2008
sub umount_UCM_dirs(;$)
{
   my $retval    = $TRUE;
   my $users     = shift(@_);
   my @err       = ();
   my $UCM_users = base_accounts::get_accounts_by_type($base_accounts::UCM_USER);
   if (!defined($UCM_users) || scalar(@$UCM_users) == 0)
   {
      baselog($LOG_INFO, "There are no configured UCM users.");
      return $TRUE;
   }
   if (!defined($users))
   {
      baselog($LOG_DEBUG, "Users list is not defined. Umount directories for all UCM users.");
      $users = $UCM_users;
   }
   else
   {
      if (!is_array($users))
      {
         baselog($LOG_ERROR, "Specified parameter is not a reference to a list.");
         return $FALSE;
      }
   } ## end else [ if (!defined($users))
   my %users_hash = ();  # hash with keys - group names, values - the reference
                         # to array with users those belong to group
   for my $groupname (keys(%UCM_ADDITIONAL_DIRS))
   {
      my @arr = split(/ /, (getgrnam($groupname))[3]);
      $users_hash{$groupname} = \@arr;
   }

   for my $user (@$users)
   {

      # check that user is UCM user
      if (pos_in_array($UCM_users, $user) < 0)
      {
         baselog($LOG_DEBUG, "User '$user' is not UCM.");
         next;
      }
      my $home = (getpwnam($user))[7];
      if (!defined($home))
      {
         baselog($LOG_ERROR, "Home directory for user '$user' is not defined. User is skipped.");
         $retval = $FALSE;
         next;
      } ## end if (!defined($home))
      my $prep_result = $TRUE;
      for my $group (keys(%UCM_ADDITIONAL_DIRS))
      {
         if (pos_in_array($users_hash{$group}, $user) < 0)
         {
            next;
         }
         if (get_mount_counter("$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]") == 0)
         {
            baselog($LOG_DEBUG, "Directory '$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' is not mounted.");
            next;
         }
         my @err = ();
         if (!do_system([$UMOUNT, '-l', "$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]"], {$dsERR => \@err}))
         {
            baselog($LOG_ERROR, "Could not umount '/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' directory: @err.");
            $prep_result = $FALSE;
         }
         else
         {
            if (!do_system([$RM, '-rf', "$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]"]))
            {
               baselog($LOG_ERROR, "Could not remove  '$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' directory.");
               $prep_result = $FALSE;
            }
         } ## end else [ if (!do_system([$UMOUNT...
      } ## end for my $group (keys(%UCM_ADDITIONAL_DIRS...

      if ($prep_result)
      {
         baselog($LOG_DEBUG, "Additional directories for UCM user '$user' umounted successfully.");
      }
      else
      {
         $retval = $FALSE;
      }
   } ## end for my $user (@$users)

   return $retval;
} ## end sub umount_UCM_dirs(;$)

###############################################################################
## @fn is_all_directories_mounted($username)
# @brief Checks that all the directories required for this account are mounted:
#        sftp jail directories, extended access point directories, UCM accounts
#        specific directories.
# @param $username - name of the user for check.
# @return $TRUE - all the directories are mounted
# @return $FALSE - otherwise
# @date January 2009
sub is_all_directories_mounted($)
{
   my $username = shift(@_);
   my $retval   = $TRUE;

   unless (defined($username))
   {
      baselog($LOG_ERROR, "User's name is not defined.");
      return $FALSE;
   }

   my $usertype = base_accounts::get_user_type($username);
   baselog($LOG_DEBUG, "'$username' is of type '$usertype'.");
   if (!defined($usertype))
   {
      baselog($LOG_ERROR, "User's '$username' type is unknown.");
      return $FALSE;
   }

   my $home = (getpwnam($username))[7];
   if (!defined($home))
   {
      baselog($LOG_ERROR, "Home directory for user '$username' is not defined.");
      return $FALSE;
   }

   if (   ($usertype eq $base_accounts::LINUX_SFTP_USER)
       || ($usertype eq $base_accounts::LINUX_FTP_SFTP_USER))
   {

      # check that all the directories required for sftp jail are mounted
      for my $jaildir (@JAIL_DIRS)
      {
         if (get_mount_counter("$home/$jaildir") == 0)
         {
            baselog($LOG_ERROR, "'$home/$jaildir' is not mounted for user '$username'.");
            $retval = $FALSE;
         }
      } ## end for my $jaildir (@JAIL_DIRS...
        # check that all the FTP/SFTP directories are mounted
      if (!check_ftp_dirs_for_user($username))
      {
         $retval = $FALSE;
      }
   } ## end if (($usertype eq $base_accounts::LINUX_SFTP_USER...
   elsif ($usertype eq $base_accounts::UCM_USER)
   {
      for my $jaildir (@JAIL_DIRS)
      {
         if (get_mount_counter("$home/$jaildir") == 0)
         {
            baselog($LOG_INFO, "'$home/$jaildir' is not mounted for user '$username'.");
            $retval = $FALSE;
         }
      } ## end for my $jaildir (@JAIL_DIRS...
      my %users_hash = ();  # hash with keys - group names, values - the reference
                            # to array with users those belong to group
      for my $groupname (keys(%UCM_ADDITIONAL_DIRS))
      {
         my @arr = split(/ /, (getgrnam($groupname))[3]);
         $users_hash{$groupname} = \@arr;
      }

      for my $group (keys(%UCM_ADDITIONAL_DIRS))
      {
         if (pos_in_array($users_hash{$group}, $username) >= 0)
         {
            if (get_mount_counter("$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]") == 0)
            {
               baselog($LOG_ERROR, "Directory '$home/@{$UCM_ADDITIONAL_DIRS{$group}}[1]' is not mounted.");
               $retval = $FALSE;
            }
         } ## end if (pos_in_array($users_hash...
      } ## end for my $group (keys(%UCM_ADDITIONAL_DIRS...
   } ## end elsif ($usertype eq $base_accounts::UCM_USER...
   elsif ($usertype eq $base_accounts::LINUX_FTP_USER)
   {
      $retval = check_ftp_dirs_for_user($username);
   }
   elsif ($usertype eq $base_accounts::LINUX_USER)
   {
      baselog($LOG_DEBUG, "There are no required directories for '$base_accounts::LINUX_USER' user.");
   }
   else
   {
      baselog($LOG_ERROR, "User's '$username' type is unknown.");
      $retval = $FALSE;
   }
   return $retval;
} ## end sub is_all_directories_mounted($)

###############################################################################
## @fn get_ftp_dirs_for_user($username)
# @brief return information about FTP directories for specified user.
# @param $username - user's name for which information about FTP directories is
#        required.
# @return $dirs - reference to the hash: key - name of the source, value - name
#         of the alias directory.
# @return undef- user's name is undefined, XML parser is undefined or parsing
#         of the XML file is not correct.
# @date January 2009
sub get_ftp_dirs_for_user($)
{
   my $username = shift(@_);
   if (!defined($username))
   {
      baselog($LOG_ERROR, "User's name is not defined.");
      return undef;
   }
   if (!-f "$SYSDB_ROOT/$ACCOUNTS_CFG")
   {
      baselog($LOG_DEBUG, "'$SYSDB_ROOT/$ACCOUNTS_CFG' does not exist.");
      return [];
   }
   my $dir_refs = undef;
   my $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   if (!$parser)
   {
      baselog($LOG_ERROR, 'Parser is undefined.');
      return undef;
   }

   $dir_refs = $parser->find_data_by_xpath($GETDATA, "/$FTP/dir");
   unless (is_array($dir_refs))
   {
      baselog($LOG_ERROR, 'Failed to check rules.');
      return undef;
   }

   my $dirs = {};
   foreach my $rule_ref (@$dir_refs)
   {
      if (defined($rule_ref->{$Cuser}) && ($rule_ref->{$Cuser} eq $username))
      {
         $dirs->{$rule_ref->{$Csrc}} = $rule_ref->{$Cdst};
      }
   } ## end foreach my $rule_ref (@$dir_refs...
   return $dirs;
} ## end sub get_ftp_dirs_for_user($)

###############################################################################
## @fn check_ftp_dirs_for_user($username)
# @brief check that all the FTP directories are mounted for specified user
# @param $username - user's name for which information about FTP directories is
#        to be checked.
# @return $TRUE - all the directories are mounted or user is not required any
#         directories
# @date January 2009
sub check_ftp_dirs_for_user($)
{
   my $username = shift(@_);
   if (!defined($username))
   {
      baselog($LOG_ERROR, "User's name is not specified.");
      return $FALSE;
   }
   my $home = (getpwnam($username))[7];
   if (!defined($home))
   {
      baselog($LOG_ERROR, "Home directory for user '$username' is not defined.");
      return $FALSE;
   }

   my $retval   = $TRUE;
   my $ftp_dirs = get_ftp_dirs_for_user($username);
   if (defined($ftp_dirs))
   {
      for my $srcDir (keys(%$ftp_dirs))
      {
         if (defined($ftp_dirs->{$srcDir}))
         {
            if (get_mount_counter("$home/$ftp_dirs->{$srcDir}") == 0)
            {
               baselog($LOG_INFO, "'$home/$ftp_dirs->{$srcDir}' is not mounted for user '$username'.");
               $retval = $FALSE;
            }
         } ## end if (defined($ftp_dirs->...
      } ## end for my $srcDir (keys(%$ftp_dirs...
   } ## end if (defined($ftp_dirs)...
   return $retval;
} ## end sub check_ftp_dirs_for_user($)

###############################################################################
## @fn      create_user_ftp_config_file($user, $chroot_dir)
# @brief    It adds user's FTP configuration file under the
#           /etc/vsftpd_user_conf/ directory.
# @details  none
# @param    $user - user account.
# @param    $chroot_dir - full path to a chroot dir. By default /home/$user is
#           used.
# @return   $TRUE  - operation successful
# @return   $FALSE - some errors happened
# @author   Evgeny Lazarev
# @date     April 2009
###############################################################################
sub create_user_ftp_config_file($;$)
{
   my $user       = shift @_;
   my $chroot_dir = shift @_;
   my $retval     = $TRUE;
   if (!$user)
   {
      baselog($LOG_ERROR, 'User name is not defined.');
      return $FALSE;
   }
   if (!$chroot_dir)
   {
      baselog($LOG_INFO, "Chroot directory for user '$user' is not defined.");
      return $FALSE;
   }
   if (!-d $VSFTPD_USER_CONF)
   {
      baselog($LOG_INFO, "'$VSFTPD_USER_CONF' directory does not exist. Creating...");
      if (!mkdir($VSFTPD_USER_CONF, 0755))
      {
         baselog($LOG_ERROR, "Cannot create '$VSFTPD_USER_CONF' directory: $!.");
         return $FALSE;
      }
   } ## end if (!-d $VSFTPD_USER_CONF...

   baselog($LOG_DEBUG, "Create '$user' user FTP configuration file.");
   if (-e "$VSFTPD_USER_CONF/$user")
   {
      baselog($LOG_DEBUG, "Configuration file '$VSFTPD_USER_CONF/$user' already exists.");
   }
   else
   {
      my @data = ("local_root=$chroot_dir\n");
      if (!write_file("$VSFTPD_USER_CONF/$user", \@data))
      {
         baselog($LOG_ERROR, "Cannot save the '$VSFTPD_USER_CONF/$user' file.");
         $retval = $FALSE;
      }
   } ## end else [ if (-e "$VSFTPD_USER_CONF/$user"...
   return $retval;
} ## end sub create_user_ftp_config_file($;$)

###############################################################################
## @fn      remove_user_ftp_config_file($user)
# @brief    It removes user's FTP configuration file under /the
#           etc/vsftpd_user_conf/ directory.
# @details  none
# @param    $user - user account.
# @return   $TRUE  - operation successful
# @return   $FALSE - some errors happened
# @author   Evgeny Lazarev
# @date     April 2009
###############################################################################
sub remove_user_ftp_config_file($)
{
   my $user = shift @_;
   if (!$user)
   {
      baselog($LOG_ERROR, 'User name is not defined.');
      return $FALSE;
   }

   if (!unlink "$VSFTPD_USER_CONF/$user")
   {
      baselog($LOG_WARNING, "Can not remove '$VSFTPD_USER_CONF/$user' file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub remove_user_ftp_config_file($)

1;
__END__
