## @file base_monit.pm
# @brief This module handles monit functionality
#
# @author Alex Arkhipov
# @date September 2008
# @version 1.00 - Initial version
# @version 1.01 - handle_device_section(@) function has been modified
#                 to support mountpoint attribute of device section
# @version 1.02 - New is_monit_already_trying() function is added
# @version 1.03 - The restart action in handle_* subroutines is
#                 reworked to invoke 'monitappstartup'. Fixed issue
#                 with 'every 0 cycles'.
# @version 1.04 - The 'hostname` functions call has been replaced with
#                 get_hostname() function from common_functions
# @version 1.05 - using "-b $device" instead of 
#                 "do_system([$STAT, '--format=%F', $device])"

package base_monit;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use baselog;
use base_system;
use install_common;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&restart_monit &update_monit $MONITRC
  $MONIT $NAME_OF_MONIT_GLOBAL
  &get_monit_process_name &get_application_script_name
  &is_monitored &do_monit $MONITOR $UNMONITOR
  &app_monit_control &is_run_by_monit &clear_alarm
  &is_monit_already_trying
  );

our $VERSION = '1.05';

# Variables
our $MONIT   = '/usr/bin/monit';                # monit tool
our $MONITRC = '/etc/opt/nortel/base/monitrc';  # monit config file

my $MONITTMP = '/etc/opt/nortel/base/monitrc.tmp';  # monit temporary file
my $MONITPID = '/var/run/monit.pid';                # monit pid file

our $NAME_OF_MONIT_GLOBAL = 'system_wide';

# Keys for app_monit_confrol function
our $MONITOR   = 'monitor';
our $UNMONITOR = 'unmonitor';

# Soft link to appstartup to be run by 'monit'
my $MONITAPPSTARTUP_NAME = 'monitappstartup';
my $MONITAPPSTARTUP      = '/opt/nortel/base/appinstall/bin/' . $MONITAPPSTARTUP_NAME;

my $MONITBK                = '/etc/opt/nortel/base/monitrc.bak';
my $MONITRC_PERMISSIONS    = 0600;
my $MONIT_FILTERING_SCRIPT = $ALARM;
my $DEVICE_DIR             = "/dev";

## @var $SECONDS_PER_CYCLE
# @brief how many seconds in one monit cycle
my $SECONDS_PER_CYCLE = 5;

## @var $DEFAULT_SECONDS_NUMBER
# @brief how many seconds is the default monitor period
my $DEFAULT_SECONDS_NUMBER = 300;  # Every 300 seconds (5 min)

## @var $DEFAULT_SECONDS_NUMBER
# @brief how many seconds is the default monitor period
my $DEFAULT_CYCLE_NUMBER = int($DEFAULT_SECONDS_NUMBER / $SECONDS_PER_CYCLE);

my $APPLICATION_SCRIPT_SEPARATOR = '___';

## @var $MONIT_RESTART_ACTION
# @brief action passed to the monit configuration for a process restart
my $MONIT_RESTART_ACTION = 'restart';

#Declarations
## @skipline
sub is_run_by_monit();
## @skipline
sub clear_alarm($);
## @skipline
sub do_monit(@);
## @skipline
sub update_monit($);
## @skipline
sub restart_monit();
## @skipline
sub monit_convert($);
## @skipline
sub handle_port_section($$);
## @skipline
sub handle_system_section(@);
## @skipline
sub handle_device_section(@);
## @skipline
sub get_default_header();
## @skipline
sub get_monit_process_name($;$);
## @skipline
sub get_application_script_name($);
## @skipline
sub is_monitored($;$);
## @skipline
sub app_monit_control($$$;$);
## @skipline
sub is_monit_already_trying($);

## @fn     is_run_by_monit
# @brief   determines if monit current program is run by monit
# @details analyse the program name. if it matches to $MONITAPPSTARTUP or not
# @param   none
# @return  $TRUE - if current program is run by monit
#          $FALSE - if current program is not run by monit
sub is_run_by_monit()
{
   return (($0 eq $MONITAPPSTARTUP) ? $TRUE : $FALSE);
}

## @fn     is_monitored
# @brief   determines if monit monitors application or script
# @details check for one particular process if $_[1] is provided
# @details If checks for the whole application then returns $TRUE
#          only if ALL application pids are monitored
# @details If there is no such application at all then return $FALSE
# @param   $_[0] - application name
# @param   $_[1] - script name or undef if necessary to check all application scripts
# @return  $TRUE - monit monitors script or all application scripts
#          $FALSE - monit does not monitors script or
#                   at least one application script is not monitored
# @return  undef - if there is not any information about this script
#                  or application in monit configuration
# @author Alex Arkhipov
# @date September 2008
sub is_monitored($;$)
{
   my ($appl, $script) = @_;
   my $unmonitor_mark = "not monitored";
   my @look;
   my $retval = undef;
   my $process = get_monit_process_name($appl, $script);
   do_system([$MONIT, "-c", $MONITRC, "summary"], {$dsOUT => \@look});
   foreach my $str (@look)
   {

      if (defined($script))
      {

         # Check for exact name
         if ($str =~ /\'$process\'/)
         {
            if ($str =~ /$unmonitor_mark$/)
            {
               return $FALSE;
            }
            return $TRUE;
         } ## end if ($str =~ /\'$process\'/...
      } ## end if (defined($script))
      else
      {
         if ($str =~ /\'$appl$APPLICATION_SCRIPT_SEPARATOR\w+\'/)
         {
            if ($str =~ /$unmonitor_mark$/)
            {
               return $FALSE;
            }

            # There is at least one application script that is supposed to be monitored
            $retval = $TRUE;
         } ## end if ($str =~ /\'$appl$APPLICATION_SCRIPT_SEPARATOR\w+\'/...
      } ## end else [ if (defined($script))
   } ## end foreach my $str (@look)

   return $retval;
} ## end sub is_monitored($;$)

# @fn update_monit()
# @brief Generate the new monitrc file (monitrc.tmp) according with 'monit' section.
# @details  First, the system wide section is generated;
#           next, the process sections for each application;
#           last, device sections (system wide and for processes)
# @param $configdata - reference to hash with configuration data
# @return none

sub update_monit($)
{
   my $configdata = shift(@_);
   my @filelist   = ();        #List of application control files
   my ($appl, $current_pid, $process, $test, $headlines);
   my %devices = ();
   my ($fname, $fpath, $fptr, $tmp, $tag, $mfh, $cycles);

   # Create a backup copy of monitrc file
   if (-f $MONITRC)
   {
      do_system([$CP, $MONITRC, $MONITBK]);
   }

   unless (open($mfh, ">", $MONITTMP))
   {
      baselog($LOG_ERROR, "Can't open temporary file for monit configuration.", $TRUE);
      return;
   }

   # Put default header in the file
   print $mfh get_default_header();

   # Take into account all enabled monit sections
   monit_convert($configdata);

   # Process global section
   my $hostname = get_hostname($TRUE);

   # Header for system wide tests
   my @monit_system_lines = ("\n# System wide tests\n", "check system $hostname\n", "    every $DEFAULT_CYCLE_NUMBER cycles\n");
   $headlines = scalar @monit_system_lines;

   # Generate system wide tests: "check system" section of monitrc
   foreach $test (@{$configdata->{'monit'}->{$NAME_OF_MONIT_GLOBAL}->{'tests'}})
   {

      # Do not process tests which are from application with disabled monit
      next unless ($configdata->{'monit'}->{$test->{'owner'}}->{'enable'});

      if ($test->{'tag'} eq 'system')
      {
         $tmp = handle_system_section($test);
         push @monit_system_lines, $tmp if ($tmp);
      }
      elsif ($test->{'tag'} eq 'device')
      {
         handle_device_section(\%devices, $test);
      }
   } ## end foreach $test (@{$configdata...

   # Write generated tests
   if (scalar(@monit_system_lines) > $headlines)
   {
      print $mfh @monit_system_lines;
   }

   # Process process sections
   foreach $appl (keys %{$configdata->{'monit'}})
   {

      # Do not process system wide tests
      next if ($appl eq $NAME_OF_MONIT_GLOBAL);

      # Do not process tests for application with disabled monit
      next unless ($configdata->{'monit'}->{$appl}->{'enable'});

      foreach $current_pid (keys %{$configdata->{'monit'}->{$appl}->{'pids'}})
      {
         $process = $configdata->{'monit'}->{$appl}->{'pids'}->{$current_pid};

         my $pname = get_monit_process_name($appl, $process->{'pmark'});

         if (not exists($process->{'every'}))
         {
            $process->{'every'} = $DEFAULT_SECONDS_NUMBER;
         }

         $cycles = int($process->{'every'} / $SECONDS_PER_CYCLE);
         if ($cycles < 1)
         {
            baselog($LOG_WARNING, "Wrong value of 'every' ('$process->{'every'}') for the process '$pname'. It is re-set to 1 monit cycle.");
            $cycles = 1;
         }

         # Header
         my @monit_process_lines = (
                                    "\n# Process: $pname\n",
                                    "check process $pname with pidfile $current_pid\n",
                                    "    every $cycles cycles\n",
                                    "    start program = \"$MONITAPPSTARTUP $pname $SRV_START\"\n",
                                    "    stop program  = \"$MONITAPPSTARTUP $pname $SRV_STOP\"\n"
                                   );

         # Generate the process' tests
         foreach $test (@{$process->{'tests'}})
         {
            $tag = $test->{'tag'};

            baselog($LOG_DEBUG, "appl='$appl', current_pid='$current_pid', tag='$tag'");

            if ($tag eq 'lock')
            {
               $fname = $test->{'file'};
               push @monit_process_lines, "    depends $fname\n";

               # Assign "every" attribute for <lock> tag
               $test->{'every'} = $process->{'every'};
               push @filelist, $test;
               next;
            } ## end if ($tag eq 'lock')
            elsif ($tag eq 'port')
            {
               $tmp = handle_port_section($test, $pname);
               push @monit_process_lines, $tmp if $tmp;
            }
            elsif ($tag eq 'system')
            {
               $tmp = handle_system_section($test, $pname);
               push @monit_process_lines, $tmp if $tmp;
            }
            elsif ($tag eq 'device')
            {
               handle_device_section(\%devices, $test, $pname, $cycles);
            }
         } ## end foreach $test (@{$process->...
         print $mfh @monit_process_lines;

      } ## end foreach $current_pid (keys ...
   } ## end foreach $appl (keys %{$configdata...

   # Generate check device" sections of monitrc
   foreach my $key (keys %devices)
   {
      print $mfh "\n";
      print $mfh $devices{$key}->{'comment'};
      print $mfh $devices{$key}->{'service'};
      if (defined($devices{$key}->{'start'}))
      {
         print $mfh "    " . $devices{$key}->{'start'};
         print $mfh "    " . $devices{$key}->{'stop'};
      }
      my $count = $devices{$key}->{'count'};
      for (my $i = 1 ; $i <= $count ; $i++)
      {
         print $mfh "    " . $devices{$key}->{"$i"};
      }
   } ## end foreach my $key (keys %devices...

   # Application control files (<lock> tags) handlers
   foreach $fptr (@filelist)
   {
      $fname  = $fptr->{'file'};
      $fpath  = $fptr->{'path'};
      $cycles = int($fptr->{'every'} / $SECONDS_PER_CYCLE);

      print $mfh "\ncheck file $fname with path $fpath/$fname\n";
      print $mfh "    every $cycles cycles\n";
      print $mfh "    if size < 1 B then unmonitor\n";
   } ## end foreach $fptr (@filelist)

   close $mfh;
} ## end sub update_monit($)

#########################################################
# @fn monit_convert()
# @brief converts monit attribute to enable attribute
# @details Extract 'monit' attribute from 'script' tag ('init' section) and put
#          it as 'enable' attribute into 'monit' tag.
# @derails The rule for forming of the resulting 'enable' tag is following:
#          monit->enable |   1   |   0   | undef |
#               \        |-----------------------|
#          script->monit |-----------------------|
#               1        |   1   |   1   |   1   |
#               0        |   0   |   0   |   0   |
#             undef      |   1   |   0   |   0   |
# @param  $configdata - reference to hash with configuration data
# @return none
#########################################################
sub monit_convert($)
{
   my $configdata = shift(@_);
   my $monit_flag;

   foreach my $script (@{$configdata->{'init'}})
   {
      my $monit_init   = "";
      my $monit_enable = "";

      $monit_flag = $FALSE;
      if (defined $configdata->{'monit'}->{$script->{'appl'}}->{'enable'})
      {
         $monit_flag = ($configdata->{'monit'}->{$script->{'appl'}}->{'enable'}) ? $TRUE : $FALSE;
         $monit_enable = $monit_flag;
      }
      if ((defined $script->{'monit'}) && ($script->{'monit'} == 0))
      {
         $monit_flag = $FALSE;
         $monit_init = $monit_flag;
      }
      elsif ($script->{'monit'})
      {
         $monit_flag = $TRUE;
         $monit_init = $monit_flag;
      }

      baselog($LOG_DEBUG, "Application: '" . $script->{'appl'} . "', script monit='$monit_init', monit enable='$monit_enable'");

      $configdata->{'monit'}->{$script->{'appl'}}->{'enable'} = $monit_flag;
   } ## end foreach my $script (@{$configdata...
} ## end sub monit_convert($)

#########################################################
# @fn handle_port_section()
# @brief This subroutine handles port section and generates the line containing
#        the test for monit.
# @details The common form of port-tag:
#          <port id="<port number>" [if="<interface>"] [timeout="<timeout, sec>"] [type="<TCP|UDP>"] action="restart" />
#          will be translated into:
#          if failed host 47.11.65.27 port 1719 type UDP with timeout 5 seconds then /opt/nortel/base/appinstall/bin/monitappstartup <APPLNAME> restart
# @param  $test - A hash which is a results of XML parsing
# @param  $appl - the application name used for handling the restart action 
# @return $monit_line - generated line
#########################################################
sub handle_port_section($$)
{
   my ($test, $appl) = @_;

   my $monit_line = "";
   my ($iface, $type, $timeout) = ("", "", "");

   my $id    = $test->{'id'};
   my $act   = $test->{'action'};
   my $iftmp = "";

   $monit_line = "    if failed ";

   #Optional attribute 'if'
   if (defined $test->{'if'})
   {
      $iface = $test->{'if'};
      $iftmp = $iface;
      $iface = qx{$IFCONFIG $test->{'if'} 2>/dev/null \| $GREP 'inet addr'};
      $iface =~ /^.*r:(\d*\.\d*\.\d*\.\d*).*$/;
      $iface = $1;
      if (defined($iface) and $iface)
      {
         $monit_line .= "host $iface ";
      }
      else
      {
         baselog($LOG_WARNING, "Test of port '$id' is skipped: there is no if='$iftmp'");
         return '';
      }
   } ## end if (defined $test->{'if'...
   $monit_line .= "port $id ";

   #Optional attribute 'type'
   if (defined $test->{'type'})
   {
      $type = $test->{'type'};
      $monit_line .= "type $type ";
   }

   #Optional attribute 'timeout'
   if (defined $test->{'timeout'})
   {
      $timeout = $test->{'timeout'};
      $monit_line .= "with timeout $timeout seconds ";
   }
   
   #Assign action
   if ($act eq $MONIT_RESTART_ACTION)
   {
      $monit_line .= "then exec \"$MONITAPPSTARTUP $appl $SRV_RESTART\"\n";
   }
   else
   {
      $monit_line .= "then $act\n";
   }

   baselog($LOG_DEBUG, "Tests of port: appl='$appl', id='$id', if='$iftmp', " . "type='$type', timeout='$timeout', act='$act'");

   return $monit_line;
} ## end sub handle_port_section($)

#########################################################
# @fn handle_system_section()
# @brief This subroutine handles system section and generates
#        the line containing the test for monit.
# @details The common form of system-tag:
#          <system type="<type>" value="<value>" {action="<action>"|mark_id="<mark_id>" msg="<msg>"} />
#             where type    : cpu or mem
#                   value   : <op val>
#                             op   - one of "gt", "lt", "eq", "ne"
#                             val  - an integer
#                   action  : start or stop or restart
#                   mark_id : unique (per application) mark identifier
#                             (e.g. syscpu, sysmem, cpu_proc1 etc)
#                   msg     : type of message, either "CRIT" or "WARN"
#          will be translated into:
#            for system wide tests
#          if cpu() gt 70% then exec "/some/predifined/script WARN cpu syscpu gt 70"
#          if memory lt 50% then exec "/some/predifined/script CRIT mem sysmem lt 50"
#            for certain process
#          if cpu gt 90% then /opt/nortel/base/appinstall/bin/monitappstartup <APPLNAME> restart
#          if totalmemory gt 50% then stop
# @param  $test - A hash which is a results of XML parsing
# @param  $appl - the application name. It is used as a flag that this
#                 is not a system-wide test.
# @return $monit_line - generated line
#########################################################
sub handle_system_section(@)
{
   my ($test, $appl) = @_;

   my $monit_line     = "";
   my $is_system_wide = (defined $appl) ? $FALSE : $TRUE;

   my $type    = $test->{'type'};
   my $value   = $test->{'value'};
   my $mark_id = (defined($test->{'mark_id'})) ? $test->{'mark_id'} : '';
   my $msg     = (defined($test->{'msg'})) ? $test->{'msg'} : '';
   my $action  = (defined($test->{'action'})) ? $test->{'action'} : '';

   unless ($is_system_wide)
   {
      baselog($LOG_DEBUG, "Tests of $type: appl='$appl', value='$value', mark_id='$mark_id', msg='$msg', action='$action'");
   }
   else
   {
      baselog($LOG_DEBUG, "System-wide tests of $type: value='$value', mark_id='$mark_id', msg='$msg'");
   }

   if ($type eq 'cpu')
   {

      #tests for resource 'cpu' are different for the system wide
      #section and for the process sections
      if ($is_system_wide)
      {
         $monit_line = "    if cpu() ";
      }
      else
      {
         $monit_line = "    if cpu ";
      }
   } ## end if ($type eq 'cpu')
   elsif ($type eq 'mem')
   {

      #tests for resource 'mem' are different for the system wide
      #section and for the process sections
      if ($is_system_wide)
      {
         $monit_line = "    if memory ";
      }
      else
      {
         $monit_line = "    if totalmemory ";
      }
   } ## end elsif ($type eq 'mem')
   else
   {
      baselog($LOG_WARNING, "Mnemonic '$type' is unknown");
      return '';
   }
   $monit_line .= "$value% then ";

   #'mark_id' and 'action' are mutual exclusive
   if (($mark_id ne '') && ($msg ne ''))
   {
      $monit_line .= qq/exec "$MONIT_FILTERING_SCRIPT $msg $type $mark_id $value %"/;
   }
   elsif ($action ne '')
   {

      #'action' can not be defined for system wide tests because there
      #is no controlling resource
      if ($is_system_wide)
      {
         baselog($LOG_WARNING, "Test of $type is skipped: actions are not allowed for system resources");
         return '';
      }

      #Assign action
      if ($action eq $MONIT_RESTART_ACTION)
      {
         $monit_line .= "exec \"$MONITAPPSTARTUP $appl $SRV_RESTART\"\n";
      }
      else
      {
         $monit_line .= $action;
      }
   } ## end elsif ($action ne '')
   else
   {
      baselog($LOG_WARNING, "Insufficient set of attributes");
      return '';
   }
   $monit_line .= "\n";

   return $monit_line;
} ## end sub handle_system_section(@)

#########################################################
# @fn handle_device_section()
# @brief This subroutine handles device section and generates
#        the line containing the test for monit.
# @details The common form of device-tag:
#          <device dev="<device>" value="<value>" {action="<action>"|mark_id="<mark_id>" msg="<msg>"} />
#            where device  : name of the device block special file (e.g. sda8)
#                  value   : <op val>
#                            op  - one of "gt", "lt", "eq", "ne"
#                            val - an integer
#                  mark_id : unique (per application) mark identifier
#                            (e.g. testvar, root1 etc)
#                  action  : start or stop or restart
#                  msg     : type of message, either "CRIT" or "WARN"
#          will be translated into:
#            if space lt 25% then exec "/some/predifined/script WARN /dev/sda8 fs1 lt 25"
#            if space gt 30% then exec "/some/predifined/script CRIT /dev/sda10 dev1 gt 25"
# @details Moreover this subroutine generates header: comment, control line,
#          and lines of service starting/stopping.
# @param  $dev_hash - a hash in which each key is a device name and the
#                    value of the key is a hash with tests for this
#                    device.
# @param  $test - A hash which is a results of XML parsing
# @param  $appl - the application name. It is used as a flag that this
#                 is not a system-wide test.
# @param  $cycles - the number of monit-checks cycles for this device, if it is
#                   not specified, default $DEFAULT_CYCLE_NUMBER will be used.
# @return $TRUE  - everiting is Ok
# @return $FALSE - There is something wrong
#########################################################
sub handle_device_section(@)
{
   my ($dev_hash, $test, $appl, $cycles) = @_;

   my $monit_line     = "";
   my $is_system_wide = (defined $appl) ? $FALSE : $TRUE;
   my $device         = "";
   my $key            = $test->{'dev'};
   my $value          = $test->{'value'};
   my $mark_id        = (defined($test->{'mark_id'})) ? $test->{'mark_id'} : '';
   my $msg            = (defined($test->{'msg'})) ? $test->{'msg'} : '';
   my $action         = (defined($test->{'action'})) ? $test->{'action'} : '';

   $cycles = (defined $cycles) ? $cycles : $DEFAULT_CYCLE_NUMBER;

   if (defined $test->{'dev'})
   {
      $device = "$DEVICE_DIR/" . $test->{'dev'};
   }
   elsif (defined $test->{'mountpoint'})
   {
      my $mntpoint = $test->{'mountpoint'};
      $key = get_mntdevice($mntpoint);
      unless ($key)
      {
         baselog($LOG_WARNING, "Test of $mntpoint is skipped: there is no such mount point");
         return $FALSE;
      }
      $device = "$DEVICE_DIR/" . $key;
   } ## end elsif (defined $test->{'mountpoint'...
   unless ($is_system_wide)
   {
      baselog($LOG_DEBUG, "Tests of $device: appl='$appl', value='$value', mark_id='$mark_id', msg='$msg', action='$action'");
   }
   else
   {
      baselog($LOG_DEBUG, "System-wide tests of $device: value='$value', mark_id='$mark_id', msg='$msg'");
   }

   # Check if the device is a block special file
   if (-e $device)
   {
      unless (-b $device)
      {
         baselog($LOG_WARNING, "Test of $device is skipped: this is not block special file");
         return $FALSE;
      }
   } ## end if (-e $device)
   else
   {
      baselog($LOG_WARNING, "Test of $device is skipped: there is no device with such path");
      return $FALSE;
   }
   $monit_line .= "if space $value% then ";

   # 'mark_id' and 'action' are mutual exclusive
   if (($mark_id ne '') && ($msg ne ''))
   {
      $monit_line .= qq{exec "$MONIT_FILTERING_SCRIPT $msg $device $mark_id $value %"};
   }
   elsif ($action ne '')
   {

      #'action' can not be defined for system wide tests because there
      #is no controlling resource
      if ($is_system_wide)
      {
         baselog($LOG_WARNING, "Test of $device is skipped: actions are not allowed for system resources");
         return $FALSE;
      }

      #Assign action
      if ($action eq $MONIT_RESTART_ACTION)
      {
         $monit_line .= "exec \"$MONITAPPSTARTUP $appl $SRV_RESTART\"\n";
      }
      else
      {
         $monit_line .= $action;
      }
   } ## end elsif ($action ne '')
   else
   {

      baselog($LOG_WARNING, "Insufficient set of attributes");
      return $FALSE;
   } ## end else [ if (($mark_id ne '') &&...
   $monit_line .= "\n";
   $key =~ s|/|_|g;
   $key = $key . "_" . $appl unless $is_system_wide;

   # Add new test line
   if (defined($dev_hash->{$key}))
   {
      my $num = $dev_hash->{$key}->{'count'};
      $num++;
      $dev_hash->{$key}->{"$num"}  = $monit_line;
      $dev_hash->{$key}->{'count'} = $num;
   } ## end if (defined($dev_hash->...
   else
   {
      
      # If the hash has not been initialized before - do it: generate
      # controlling line and lines for service starting/stopping
      $dev_hash->{$key}              = {};
      $dev_hash->{$key}->{'service'} = "check device $key with path $device\n" . "    every $cycles cycles\n";
      $dev_hash->{$key}->{'count'}   = 1;
      $dev_hash->{$key}->{'1'}       = $monit_line;
      unless ($is_system_wide)
      {
         $dev_hash->{$key}->{'comment'} = "# Tests of device $device for application $appl\n";
         $dev_hash->{$key}->{'start'}   = qq/start program = "$MONITAPPSTARTUP $appl $SRV_START"\n/;
         $dev_hash->{$key}->{'stop'}    = qq/stop program = "$MONITAPPSTARTUP $appl $SRV_STOP"\n/;
      } ## end unless ($is_system_wide)
      else
      {
         $dev_hash->{$key}->{'comment'} = "# System wide tests of check device $device\n";
      }
   } ## end else [ if (defined($dev_hash->...

   return $TRUE;

} ## end sub handle_device_section(@)

## @fn get_default_header()
# @brief Returns default header for monitrc file
# @param  none
# @return header stuff
sub get_default_header()
{

   # Use this dummy variable because otherwise RPMBUILD considers 'use' word
   # as a useage of perl module 'address' and includes it in dependancy list
   # of linuxbase RPM
   my $address = 'use address localhost';
   return <<HERE;
set daemon $SECONDS_PER_CYCLE
set logfile syslog facility log_local7
set httpd port 2812 and
    $address
    allow localhost
    allow admin:monit

check process sshd with pidfile /var/run/sshd.pid
    every $DEFAULT_CYCLE_NUMBER cycles
    start program = "/etc/init.d/sshd start"
    stop program  = "/etc/init.d/sshd stop"
    if failed port 22 then restart

#
# Include configurations specified by other applications
#
include /etc/monit.d/*.inc

HERE
} ## end sub get_default_header()

## @fn restart_monit()
# @brief Rename monitrc to monitrc.tmp and stops monit
# @details monit will be restarted again by init process
# @param  none
# @return $TRUE  - monit has been killed
# @return $FALSE - There is no new file monitrc.tmp
sub restart_monit()
{
   if (-e $MONITTMP)
   {

      # Quit monit
      # It will be started automatically by initd daemon
      # It is done to perform a little delay between app start
      # script and monit daemon
      do_monit(['quit'], $TRUE);

      # Change file permission
      chmod $MONITRC_PERMISSIONS, $MONITTMP;

      # Move temporary monit config file
      rename($MONITTMP, $MONITRC);
      baselog($LOG_DEBUG, "Renamed $MONITTMP to $MONITRC");
   } ## end if (-e $MONITTMP)
} ## end sub restart_monit()

## @fn      get_application_script_name()
# @brief   returns the application and script name if applicable
# @details If application contains more that one script than it has to
#          set attribute "pname". Then during monitrc file generation
#          it will be converted to process name and applcation name
#          separated by $APPLICATION_SCRIPT_SEPARATOR
#          There is additional check performed:
#             if script is a number this means that it has been
#             assigned automatically and it is dangerous
#             to rely on script order.
#             Thus, in such a case $script is set to "undef"
# @param   $name - string that has to be parsed
# @return  ($app_name, $pname)
# @author  Alex Arkhipov
# @date September 2008
sub get_application_script_name($)
{
   my $name = shift(@_);
   my ($app, $script) = split(/$APPLICATION_SCRIPT_SEPARATOR/, $name);

   if (defined($script) and ($script =~ /^\d+$/))
   {
      $script = undef;
   }

   return ($app, $script);
} ## end sub get_application_script_name($)

## @fn      get_monit_process_name()
# @brief   returns the application and script name if applicable
# @details If application contains more that one script than it has to
#          set attribute "pname". This function converts them to
#          pname by joining application name and script with $APPLICATION_SCRIPT_SEPARATOR
# @param   $app - application name
# @param   $script - script name
# @return  $name - service name
# @author  Alex Arkhipov
# @date September 2008
sub get_monit_process_name($;$)
{
   my ($app, $script) = @_;
   if (not defined($script))
   {
      return $app;
   }
   my $pname = $app . $APPLICATION_SCRIPT_SEPARATOR . $script;

   return $pname;
} ## end sub get_monit_process_name($;$)

## @fn     app_monit_control
# @brief   Provides an interface to monit (un)monitor command
# @details
# @param   $_[0] - monit command $MONITOR $UNMONITOR
# @param   $_[1] - this flag is 'true' if the output is not needed.
# @param   $_[2] - application name
# @param   $_[3] - script name (if missing then stop all application scripts)
# @return  $TRUE - command execution successful
# @return  $FALSE - something wrong
# @author Alex Arkhipov
# @date September 2008
sub app_monit_control($$$;$)
{
   my ($command, $silent, $app, $script) = @_;
   if (($command ne $MONITOR) and ($command ne $UNMONITOR))
   {
      baselog($LOG_ERROR, "Unknown command: $command");
      return $FALSE;
   }

   # Check if application monitored or not
   if (defined($script))
   {
      my $status = is_monitored($app, $script);
      if (not defined($status))
      {
         baselog($LOG_WARNING, "$app - $script is not supposed to be monitored at all");
         return $FALSE;
      }
      if ($command eq $MONITOR)
      {
         if ($status)
         {
            baselog($LOG_DEBUG, "$app - $script is already monitored");

            # It is already monitored. Exiting...
            return $TRUE;
         } ## end if ($status)
      } ## end if ($command eq $MONITOR...
      else
      {
         if (not $status)
         {
            baselog($LOG_DEBUG, "$app $script is not monitored");

            # It is already unmonitored. Exiting...
            return $TRUE;
         } ## end if (not $status)
      } ## end else [ if ($command eq $MONITOR...
   } ## end if (defined($script))

   my $process = get_monit_process_name($app, $script);

   # Add list for all application scripts

   do_monit([$command, $process], $silent);

   return $TRUE;
} ## end sub app_monit_control($$$;$)

## @fn     do_monit
# @brief   Provides an interface to monit (un)monitor command
# @details
# @param   $_[0] - a reference to an array with the action and application
#                  name
# @param   $_[1] - this flag is 'true' if the output is not needed.
#                  Default is 'false'
# @return  none
sub do_monit(@)
{
   my ($parm, $silent) = @_;

   my $retval;
   my %output_options;
   if ((defined $silent) and ($silent == $TRUE))
   {
      %output_options = ($dsRETVAL => \$retval, $dsCHDIR => $dsHOMEDIR);
   }
   else
   {
      %output_options = ($dsRETVAL => \$retval, $dsINTERACT => $TRUE, $dsCHDIR => $dsHOMEDIR);
   }

   if (-e $MONITPID)
   {
      my @cmdarr;
      push @cmdarr, $MONIT, "-c", $MONITRC, @$parm;
      do_system(\@cmdarr, \%output_options);
      baselog($LOG_INFO, "'@cmdarr' returned $retval");
   } ## end if (-e $MONITPID)
   else
   {
      baselog($LOG_INFO, "monit is not running.");
   }
} ## end sub do_monit(@)

## @fn     clear_alarm
# @brief   Clears alarms for the specific applications
# @param   $_[0] - application name to clear alarms
# @return  none
sub clear_alarm($)
{
   my $app = shift(@_);
   if (not defined($app))
   {
      baselog($LOG_ERROR, "Application is not privided");
   }
   do_system([$MONIT_FILTERING_SCRIPT, '--clear', $app]);
} ## end sub clear_alarm($)

## @fn     is_monit_already_trying($script_name)
# @brief   Checks if monit is already trying to start script
# @details Check 'ps -C monitappstartup'
# @param   $script_name - application script to start
# @return  $TRUE  - monit is already trying to start this script
# @return  $FALSE - monit is not trying to start the script
sub is_monit_already_trying($)
{
   my $script_name = shift(@_);
   my $ret_val     = $FALSE;

   # Check if it was run under monit
   if (not is_run_by_monit())
   {
      return $FALSE;
   }
   my @cmd = ($PS, 'h', '-C', $MONITAPPSTARTUP_NAME);
   my @output;
   my $ret;

   if (not do_system(\@cmd, {$dsOUT => \@output, $dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, 'Cannot check if it is running or not:' . join(' ', @cmd));
      return $FALSE;
   }

   # Search for our script
   my @script_lines = grep(/$script_name/, @output);

   # Check for numberr 2 because every time there are two
   # processes in the 'ps' list (one for shell and one for perl)
   if (scalar(@script_lines) > 2)
   {
      baselog($LOG_INFO, "The $script_name is already trying to be started");
      $ret_val = $TRUE;
   }
   return $ret_val;
} ## end sub is_monit_already_trying($)

1;
