#################################################################
## @file base_pm.pm
# @brief This module contains Linux Base API for Patch Manager
# @details Linux Base Perl API for patch manager. Wrappers around patch management command.
# @author Sergey Urlin
# @author Roman Orazmagomedov
# @date July 2008
# @version 1.00 - Initial version
# @version 1.10 - Corrected in accordance with defects raised in the IDCE
# @version 1.20 - cSwVersionShow() modified to output the results to the separate xml file (swversionshow.xml)
# @version 1.30 - getTagValue() and writeXML() functions were added, getTextElement() modified to return XML::LibXML::Node object
# @version 1.31 - Two colons were removed before shared functions and variables.
# @version 1.41 - checkFile() function is added to check specified file can be re-written, this function is called from
#                 cPstat, cSwVersionShow, cSPload and cSPins functions
#               - cSPload and cSPins functions are added to process spload and spins commands accordingly
#               - patch.xml file is modified: two new tags introduced <type> - can contain SP or PATCH and <spname> - SP file name
#                 <patchdetails> tag is moved under <patches> tag
#               - <category> tag is removed from pstat.xml file.
# @version 1.51 - cSPout and cIssp functions added.
#		- cPstat function and pstat.xml file format are changed
# @version 1.52 - fix of Q01961866
# @version 1.53 - Removed -q option from ssh to capture error outputs
# @version 1.54 - fix for Q01970607 - patch.xml is updated incorrectly
# @version 1.55 - Fixes for Q01970612 - patch.xml is not updated properly during faulire
#		  Q01971326 - LB creates empty pstat.xml file even if there patches available on the system.
# @version 1.57 - fixes for Q01970616, Q01970619 and Q01974348
# @version 1.58 - fix for Q01976410 - Improper pstat.xml format and typo in header.xml of service pack
# @version 1.59 - fix for Q01979486 - Patch Manager requires SP information in pstat.xml
# @version 1.60 - fix for Q01983732 - Patch Manager requires empty tags in pstat.xml when not patches installed case
# @version 1.61 - fix for Q01985628 - PM LB api does not work when executed on a Member server locally
# @version 1.62 - fix for Q02013371 - When Central PM is patching the same element then should not use SSH
# @version 1.63 - work around for Q02015942 - SSH connection hangs when running centralized patching commands
#                 -- "-die" flag added to "pins", "poos" and "spins" to trigger workaround due to a ssh hang on exit bug.
# @version 1.64 - pload,pins,poos and pout operations for remote host via patch manager were merge in one transaction for
#	          any counts of patches.
# @version 1.65 - pload,pins,poos and pout operations were updated to get status from the status string
# @version 1.66 - cPins,cPout,cPload,cPoos functions is updated, right sequence of patches for current operation is
#                 passed to remote(local) pP* commands.
# @version 1.67	- fix for Q02046509-01 - PM - Misleading information does not identify errors found in patch activation
# @version 1.68	- fix for Q02084020 - After Upgrade "sp" folder is missing and target is unpatchable
# @note

package base_pm;

use lib "/opt/nortel/base/lib";
use strict;
use warnings;

use common_variables;
use patchutil::patches_target;
use base_system;
use baselog;
use File::Basename qw(dirname);
use XML::LibXML;
use Exporter;

our @ISA    = qw(Exporter);
our @EXPORT = qw(cPcmd);

our $VERSION = 1.68;

my $class        = "base_pm";
my $XML_VERSION  = "1.0";
my $XML_ENCODING = "UTF-8";

my $PATCHLIB_DIRECTORY     = "/var/opt/nortel/patchlibrary";
my $PATCH_XML_FILE         = "patch.xml";
my $PSTAT_XML_FILE         = "pstat.xml";
my $SWVERSIONSHOW_XML_FILE = "swversionshow.xml";
my $SPLOAD_TEXT_FILE       = "spload.txt";
my $SPINS_TEXT_FILE        = "spins.txt";
my $ISSP_TEXT_FILE         = "MIRCC.txt";
my $PATCH_UTILS_DIR        = "/opt/nortel/base/patchutil/bin";
my $BIN_DIR                = "/opt/nortel/base/bin";
my $DEF_VALUE              = "None";
my $LOCAL_HOST             = "127.0.0.1";

my %PATCHCOMMANDS = (
                     pPload        => "$PATCH_UTILS_DIR/pPload -die -patches",
                     pPins         => "$PATCH_UTILS_DIR/pPins -die -patches",  # don't ask anything
                     pPoos         => "$PATCH_UTILS_DIR/pPoos -die -patches",
                     pPout         => "$PATCH_UTILS_DIR/pPout -die -patches",
                     spload        => "$PATCH_UTILS_DIR/spload -r",
                     spins         => "$PATCH_UTILS_DIR/spins -die -r",
                     spout         => "$PATCH_UTILS_DIR/spout -r",
                     issp          => "$PATCH_UTILS_DIR/issp",
                     pstat         => "$PATCH_UTILS_DIR/pstat -a",
                     spstat        => "$PATCH_UTILS_DIR/spstat",
                     swVersionShow => "$BIN_DIR/swVersionShow",
                    );

# Tag names:
# tags of patch.xml
my %PATCH_TAG = (
                 PATCH         => "patch",
                 PATCHFILEPATH => "patchfilepath",
                 PCOMMAND      => "pcommand",
                 TYPE          => "type",
                 SPNAME        => "spname",
                 PATCHES       => "patches",
                 PATCHDETAILS  => "patchdetails",
                 PNAME         => "pname",
                 PID           => "pid",
                 PSTATUS       => "pstatus",
                 PERROR        => "perror",
                 OVERALLERROR  => "overallerror",
                 OVERALLSTATUS => "overallstatus",
                );

# tags of pstat.xml
my %PSTAT_TAG = (
   CPSTAT       => "cpstat",        # Root tag of pstat.xml
   PATCHDETAILS => "patchdetails",  # Patchdetails tag
   PNAME        => "pname",         # Patch name as loaded on the system
   PID          => "pid",           # Patch handle/id of the loaded patch
   TYPE         => "type",          # Patch type, whether it is a patch / service update
   PSTAT    => "pstat",     # Patch current status on the system: Patch is out-of-service | Patch will be in service after restart | Patch will be out of service after restart | Patch is in-service
   SUBTYPE  => "subtype",   # WAR|JAR|BIN|FRU for patch or RPM for SU
   ENGINEER => "engineer",  # Name of engineer
   LOADED_DATE    => "loaded_date",     # Patch loaded date on the system
   CREATED        => "created",         # Patch creation date
   OPERATION_DATE => "operation_date",  # Patch in service or out of service date depending on status
   SERVICE_FOR    => "service_for",     # Patch in/out of service for, depending on status
   PRE_SCRIPT     => "pre_script",      # Patch pre script existence flag: YES|NO
   POST_SCRIPT    => "post_script",     # Patch post script existence flag: YES|NO
   SPECINST       => "specinst",        # Patch special instructions existence flag: YES|NO
   REBOOT         => "reboot",          # Shows if reboot is needed to install/uninstall this patch/SU: YES|NO
   APP_STOP       => "app_stop",        # List of application to stop
   APP_START      => "app_start",       # List of applications to start
   DEPENDENT      => "dependent",       # Dependent patch/SU
   CR             => "cr",              # CR number
   RPM_NAME       => "rpm_name",        # RPM Name that patch pertains to
   LOADED_SP      => "loaded_sp",       # Loaded SP tag
   INSTALLED_SP   => "installed_sp",    # Installed SP tag
   SP_DATE        => "date",            # Date of loading/installation of Service Pack
   SP_NAME        => "name",            # Service Pack name
                );

# tags of swversionshow.xml
my %SWVERSIONSHOW_TAG = (
                         CSWVERS     => "cswversions",
                         APPLICATION => "application",
                         NAME        => "name",
                         VERSION     => "version",
                        );

###############################################################################
## @method initPatchParser($hostname, $ip)
# @brief   Initializes patch parser.
# @details This class method creates an object and fills necessary object
#          fields. Patch object is binded with a patch.xml file which
#          is interface between Patch Manager and LinuxBase APIs.
# @param   $_[0] - hostname associated with a new object.
# @param   $_[1] - IP address associated with the host.
# @return  $self - reference to a hash which contains information related to patch.xml
# @return  undef - if something wrong with patch.xml or patch.xml parser can't be initialized. For example,
#                  the process can't read/write to patch.xml.
# @date    July 2008
sub new(@)
{
   my ($hostname, $ip) = @_;
   if ((not defined $hostname) or ($hostname eq ''))
   {
      baselog($LOG_ERROR, "Hostname is undefined.");
      return undef;
   }
   if ((not defined $ip) or ($ip eq ''))
   {
      baselog($LOG_ERROR, "IP address is undefined.");
      return undef;
   }

   my %self;
   $self{"targetname"} = $hostname;
   $self{"targetip"}   = $ip;
   $self{"patch_xml"}  = "$PATCHLIB_DIRECTORY/$self{'targetname'}/$PATCH_XML_FILE";

   #
   # We will make use of the "ifconfig" utility to determine if we are going
   # to perform PM patching on a local target or a remote target
   #
   my ($tmp, $rc);
   my @out;
   if (not do_system([$IFCONFIG], {$dsOUT => \@out, $dsRETVAL => \$rc}))
   {
      baselog($LOG_ERROR, "$IFCONFIG Error rc=$rc");
      return undef;
   }
   $tmp = join(" ", @out);
   if ($tmp =~ /.*inet addr:$ip\s.*/)
   {
      baselog($LOG_INFO, "ip=$ip - LOCAL.");
      $self{"is_local"} = $TRUE;
   }
   elsif ($ip =~ /^$LOCAL_HOST$/)
   {
      baselog($LOG_INFO, "ip=$ip - LOCAL.");
      $self{"is_local"} = $TRUE;
   }
   else
   {
      baselog($LOG_INFO, "ip=$ip - REMOTE.");
      $self{"is_local"} = $FALSE;
   }

   # Check if patch.xml file exists
   if (not -f $self{"patch_xml"})
   {
      baselog($LOG_ERROR, "File $self{patch_xml} doesn't exist");
      undef %self;
      return undef;
   } ## end if (not -f $self{"patch_xml"...

   # Check if patch.xml is readable
   if (not -r $self{"patch_xml"})
   {
      baselog($LOG_ERROR, "Can't read $self{patch_xml}");
      undef %self;
      return undef;
   } ## end if (not -r $self{"patch_xml"...
     # Check if patch.xml is writtable
   if (not -w $self{"patch_xml"})
   {
      baselog($LOG_ERROR, "Can't write to $self{patch_xml}");
      undef %self;
      return undef;
   } ## end if (not -w $self{"patch_xml"...
   my $parser = XML::LibXML->new();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Can't create parser object.");
      undef %self;
      return undef;
   } ## end if (not defined $parser...
   $self{"patch_xml_doc"} = eval {$parser->parse_file($self{"patch_xml"})};
   if ($@)
   {
      baselog($LOG_ERROR, $@);
      undef $parser;
      undef %self;
      return undef;
   } ## end if ($@)
   $self{"root"} = $self{"patch_xml_doc"}->documentElement();
   if (not defined $self{"root"})
   {
      baselog($LOG_ERROR, "Can't retrieve root element of $PATCH_XML_FILE.");
      undef $parser;
      undef $self{"patch_xml_doc"};
      undef %self;
      return undef;
   } ## end if (not defined $self{...

   $self{$PATCH_TAG{"OVERALLERROR"}}  = getTextElement($self{"root"}, $PATCH_TAG{"OVERALLERROR"});
   $self{$PATCH_TAG{"OVERALLSTATUS"}} = getTextElement($self{"root"}, $PATCH_TAG{"OVERALLSTATUS"});
   $self{$PATCH_TAG{"PATCHES"}}       = getTextElement($self{"root"}, $PATCH_TAG{"PATCHES"});
   $self{"actionsFailed"}             = 0;
   $self{"actionsCount"}              = 0;
   bless(\%self, $class);
   return \%self;
} ## end sub new(@)

###############################################################################
## @method DESTROY($self)
# @brief   Object destructor.
# @details Destroys instance of patch object.
# @param   $self - reference to a hash which contains information related to patch.xml
# @date    August 2008
sub DESTROY
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
   }
   else
   {
      if (defined $self->{"patch_xml_doc"})
      {
         baselog($LOG_INFO, "Destroying document object");
         undef $self->{"patch_xml_doc"};
      }
   } ## end else [ if (not defined $self)
   baselog($LOG_INFO, "Remote patch object destroyed.");
   return;
} ## end sub DESTROY

###############################################################################
## @method updatePatchXML($self)
# @brief   Updates patch.xml file on the file system.
# @details none.
# @param   $self - reference to patch object.
# @return  $TRUE - on success
# @return  $FALSE - on error
# @date    July 2008
sub updatePatchXML($)
{
   my $self = shift @_;
   return writeXML($self->{"patch_xml_doc"}, $self->{"patch_xml"});
}

###############################################################################
## @method writeXML($document, $fileName)
# @brief   Saves document object into file.
# @details none.
# @param   $document - XML document
# @param   $fileName - file name being saved.
# @return  $TRUE - on success
# @return  $FALSE - on error
# @date    August 2008
sub writeXML($$)
{
   my ($doc, $fileName) = @_;
   my $dirName = dirname($fileName);
   if (not -d $dirName)
   {
      baselog($LOG_ERROR, "Directory $dirName doesn't exist. Unable to save $fileName.");
      return $FALSE;
   }
   if (not -w $dirName)
   {
      baselog($LOG_ERROR, "Can't write into $dirName, directory isn't writtable. Unable to save $fileName.");
      return $FALSE;
   }
   baselog($LOG_INFO, "writeXML(): Updating $fileName");

   $doc->toFile($fileName, 1);
   return $TRUE;
} ## end sub writeXML($$)

###############################################################################
## @method scpPut($file $user, $host, $path, $resOut, $resErr)
# @brief   Does 'scp -q $file $user@$host:$path' and places all output
#          to $resOut.
# @details none.
# @param   $file - file to be copied.
# @param   $user - remote user.
# @param   $host - IP address of the remote host.
# @param   $path - directory on the remote host.
# @param   $is_local - shows if API is called from local PM or not.
# @param   $resOut - is reference to an array which contains command output.
# @param   $resErr - is reference to an array which contains error
# @return  TRUE - on success
# @return  FALSE - if scp command completed with error.
# @date    July 2008
sub scpPut($$$$$$$)
{
   my ($file, $user, $host, $path, $is_local, $resOut, $resErr) = @_;
   my $rc;
   my $patch_command;

   my @sshOut = ();
   my @sshErr = ();
   my $output = undef;

   doSSH("$FILE $path", $is_local, \@sshOut, \@sshErr, $user, $host);
   $output = join('', @sshOut);
   if (defined $output)
   {
      if ($output =~ /:\s*directory/)
      {
         baselog($LOG_INFO, "Directory $path already exists.");
      }
      else
      {
         if ($output =~ /ERROR/)
         {
            baselog($LOG_ERROR, "Directory $path does not exist on the host $host.");
         }
         else
         {
            baselog($LOG_ERROR, "File $path is not a directory. Trying to remove $path from host $host...");
            doSSH("$RM $path", $is_local, \@sshOut, \@sshErr, $user, $host);

         } ## end else [ if ($output =~ /ERROR/...

         if (doSSH("$MKDIR $path", $is_local, \@sshOut, \@sshErr, $user, $host))
         {
            baselog($LOG_INFO, "Directory $path has been created successfully on the host $host.");
         }
         else
         {
            baselog($LOG_ERROR, "Directory $path has not been created  on the host $host.");
         }
      } ## end else [ if ($output =~ /:\s*directory/...
   } ## end if (defined $output)
   else
   {
      baselog($LOG_ERROR, "Getting $path directory info from the host $host has been failed.");
   }

   if ($is_local)
   {
      $patch_command = [$CP, "$file", "$path"];
   }
   else
   {
      $patch_command = [$SCP, "-o", "ServerAliveInterval=$SSH_ALIVE_INTERVAL", "-o", "TCPKeepAlive=$SSH_TCP_KEEP_ALIVE", "-o", "ConnectTimeout=$SSH_CONNECT_TO", '-q', $file, "$user\@$host:$path"];
   }

   my $str = "scpPut: " . join(" ", @$patch_command);
   baselog($LOG_INFO, $str);

   # redirection 2>&1 doesn't work
   if (not do_system($patch_command, {$dsRETVAL => \$rc, $dsOUT => \@$resOut, $dsERR => \@$resErr}))
   {
      my $error = "scpPut: '" . join(" ", @$patch_command) . "' failed.";
      $error .= " Returned $rc." if defined $rc;
      baselog($LOG_ERROR, $error);
      return $FALSE;
   } ## end if (not do_system($patch_command...
   return $TRUE;
} ## end sub scpPut($$$$$$$)

###############################################################################
## @method doSSH($cmd, $is_local, $resOut, $resErr, $user, $host)
# @brief   Does 'ssh -q -l $remUser $remHost $remCmd' on the $remHost and
#          places all output into $refToOutput in case of remote patching.
#	   If local patching operation is taking place, invokes corresponding
#	   patch command directly (without ssh).
# @details none.
# @param   $cmd - command to be executed on the remote host.
# @param   $is_local - shows if API is called from local PM or not.
# @param   $resOut - is reference to an array which contains command output.
# @param   $resErr - is reference to an array which contains error
# @param   $user - remote user.
# @param   $host - IP address of the remote host.
# @return  TRUE - on success
# @return  FALSE - if ssh command completed with error.
# @date    July 2008
sub doSSH($$$$;$$)
{
   my ($cmd, $is_local, $resOut, $resErr, $user, $host) = @_;
   my $rc;
   my $patch_command;

   if ($is_local)
   {
      $cmd =~ s/-die//;
      $patch_command = [split(' ', $cmd)];
   }
   else
   {
      unless (defined $user and defined $host)
      {
         baselog($LOG_ERROR, "Undefined remote user or IP address of the remote host. These arguments are mandatory for remote patching.");
         return $FALSE;
      }
      $patch_command = [$SSH, "-o", "ServerAliveInterval=$SSH_ALIVE_INTERVAL", "-o", "TCPKeepAlive=$SSH_TCP_KEEP_ALIVE", "-o", "ConnectTimeout=$SSH_CONNECT_TO", '-l', $user, $host, $cmd];
   } ## end else [ if ($is_local)

   my $str = "doSSH: " . join(" ", @$patch_command);
   baselog($LOG_INFO, $str);

   if (not do_system($patch_command, {$dsRETVAL => \$rc, $dsOUT => \@$resOut, $dsERR => \@$resErr}))
   {
      my $error = "Command: '" . join(" ", @$patch_command) . "' failed.";
      $error .= " Returned $rc." if defined $rc;
      baselog($LOG_ERROR, $error);
      return $FALSE;
   } ## end if (not do_system($patch_command...
   return $TRUE;
} ## end sub doSSH($$$$;$$)

###############################################################################
## @method getTagValue($tag)
# @brief   Retrieves element value.
# @details none.
# @param   $tag - Node object.
# @return  undef - on error condition
# @return  Text content of tag.
# @date    August 2008
sub getTagValue($)
{
   my $tag = shift @_;
   if (not defined $tag)
   {
      baselog($LOG_DEBUG, "Tag is not defined.");
      return undef;
   }
   return $tag->textContent();
} ## end sub getTagValue($)

###############################################################################
## @method getTextElement($parentNode, $elementName)
# @brief   Retrieves element by name
# @details none.
# @param   $parentNode - parent node.
# @param   $elementName - name of element.
# @return  undef - on error condition
# @return  NodeList object which contains named tag.
# @date    July 2008
sub getTextElement($$)
{
   my ($parent, $name) = @_;
   if (not defined $parent)
   {
      baselog($LOG_ERROR, "Invalid input parameter.");
      return undef;
   }
   my $nlist = $parent->getElementsByTagName($name);
   unless (defined $nlist)
   {
      baselog($LOG_ERROR, "Can't get elements by tag name: $name.");
      return undef;
   }
   if ($nlist->size() == 0)
   {
      my $new_elem = XML::LibXML::Element->new($name);
      my $child    = undef;
      if (not defined $new_elem)
      {
         baselog($LOG_ERROR, "Can't create XML element.");
         return undef;
      }
      $child = $parent->addChild($new_elem);
      if (not defined $child)
      {
         baselog($LOG_ERROR, "Can't add child node.");
         return undef;
      }
      else
      {
         $nlist->push($child);
      }
   } ## end if ($nlist->size() == ...
   elsif ($nlist->size() > 1)
   {

      # abnormal situation. got more then 1 tag. taking first one and report into log
      baselog($LOG_ERROR, $nlist->size() . " tags detected where expected only one. Please check $PATCH_XML_FILE.");

   } ## end elsif ($nlist->size() > 1...
   return $nlist->get_node(1);
} ## end sub getTextElement($$)

###############################################################################
## @method setTextData($node, $data)
# @brief   Sets text data for $node.
# @details If $node doesn't exist then it will be added.
# @param   $node - Node object.
# @param   $data - text data to set.
# @return  FALSE - if error occured
# @return  TRUE - on success
# @date    July 2008
sub setTextData($$)
{
   my ($node, $data) = @_;
   if (not defined $node)
   {
      baselog($LOG_ERROR, "Node object is not defined.");
      return $FALSE;
   }
   if (not defined $node->firstChild)
   {
      my $textElem = XML::LibXML::Text->new($data);
      if (not defined $textElem)
      {
         baselog($LOG_ERROR, "Can't create LibXML::Text element.");
         return $FALSE;
      }
      else
      {

         # Need to provide explanation
         baselog($LOG_DEBUG, "Adding new text element.");
         $node->addChild($textElem);
      } ## end else [ if (not defined $textElem...
   } ## end if (not defined $node->firstChild...
   else
   {
      $node->firstChild->setData($data);
   }
   return $TRUE;
} ## end sub setTextData($$)

###############################################################################
## @method removeTextData($node)
# @brief   Removes text data from node.
# @details none.
# @param   $node - Node object.
# @return  FALSE - if node is undefined.
# @return  TRUE - on success
# @date    July 2008
sub removeTextData($)
{
   my $node = shift @_;
   unless (defined $node)
   {
      baselog($LOG_ERROR, "Node object is undefined.");
      return $FALSE;
   }
   $node->removeChildNodes();
   return $TRUE;
} ## end sub removeTextData($)

###############################################################################
## @method addApplicationNode($self, $appName, $version)
# @brief   Inserts <application> tag into patch.xml before <overallerror> tag.
# @details <application> tag consists of 2 tags: <name> and <version>.
# @param   $parentNode - root Node of the swversionshow.xml
# @param   $appName - application name.
# @param   $version - application version.
# @return  TRUE - on success (application tag has been added to the swverionshow.xml)
# @return  FALSE - on error
# @date    July 2008
sub addApplicationNode($$$)
{
   my ($parentNode, $appName, $version) = @_;
   my $appNode = XML::LibXML::Element->new($SWVERSIONSHOW_TAG{"APPLICATION"});
   if (not defined $appNode)
   {
      baselog($LOG_ERROR, "Can't create $SWVERSIONSHOW_TAG{APPLICATION} tag");
      return $FALSE;
   }
   $parentNode->addChild($appNode);
   $appNode->appendTextChild($SWVERSIONSHOW_TAG{"NAME"},    $appName);
   $appNode->appendTextChild($SWVERSIONSHOW_TAG{"VERSION"}, $version);
   return $TRUE;
} ## end sub addApplicationNode($$$)

###############################################################################
## @method removeChildrenByName($node, $name)
# @brief   Removes child nodes by node name.
# @details none.
# @param   $node - parent node.
# @param   $name - name of nodes are being removed.
# @return  TRUE - on success
# @return  FALSE - on error
# @date    July 2008
sub removeChildrenByName($$)
{
   my ($node, $name) = @_;
   if (defined $node)
   {
      foreach my $childNode ($node->getChildrenByTagName($name))
      {
         $node->removeChild($childNode);
      }
      return $TRUE;
   } ## end if (defined $node)
   else
   {
      baselog($LOG_ERROR, "Can't remove child nodes as node object is undefined.");
      return $FALSE;
   }
} ## end sub removeChildrenByName($$)

###############################################################################
## @method fillOverallErrorAndOverallStatus($self, $action)
# @brief   Fills <overallerror> and <overallstatus> tags of the patch.xml file.
# @details <overallstatus> can contain "Failed", "Succeed" or "Partially completed".
#          <overallerror> can contain "3 out of 5 $action succeed".
# @param   $self - reference to patch object.
# @param   $action - contains "transfers" or "patches".
# @return  FALSE - if error occured
# @return  TRUE - <overallerror> and <overallstatus> tags were updated successfully
# @date    July 2008
sub fillOverallErrorAndOverallStatus($$)
{
   my ($self, $action) = @_;
   my $retValue = $TRUE;
   if ($self->{"actionsFailed"} == 0)
   {
      unless (setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed"))
      {
         baselog($LOG_ERROR, "Can't set 'Succeed' value into $PATCH_TAG{OVERALLSTATUS} tag.");
         $retValue = $FALSE;
      }
      unless (setTextData($self->{$PATCH_TAG{"OVERALLERROR"}}, $DEF_VALUE))
      {
         baselog($LOG_ERROR, "Can't set $DEF_VALUE value into $PATCH_TAG{OVERALLERROR} tag of $PATCH_XML_FILE.");
         $retValue = $FALSE;
      }
   } ## end if ($self->{"actionsFailed"...
   else
   {
      my ($error, $status);
      if ($self->{"actionsFailed"} == $self->{"actionsCount"})
      {
         $status = "Failed";
         $error  = "All $action failed";
      }
      else
      {
         $status = "Partially completed";
         $error  = ($self->{"actionsCount"} - $self->{"actionsFailed"}) . " out of $self->{'actionsCount'} $action succeed";
      }
      unless (setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, $status))
      {
         baselog($LOG_ERROR, "Can't set $status value into $PATCH_TAG{OVERALLSTATUS} tag.");
         $retValue = $FALSE;
      }
      unless (setTextData($self->{$PATCH_TAG{"OVERALLERROR"}}, $error))
      {
         baselog($LOG_ERROR, "Can't set $error value into $PATCH_TAG{OVERALLERROR} tag.");
         $retValue = $FALSE;
      }
   } ## end else [ if ($self->{"actionsFailed"...
   return $retValue;
} ## end sub fillOverallErrorAndOverallStatus($$)

###############################################################################
## @method fillPErrorAndPStatus($patchDetailsTag, $exitStatus, $command, $refToOutput, $refToError)
# @brief   Fills <perror> and <pstatus> individual patch tags accordingly to $exitStatus.
# @details Example:
#          my @out;
#          my $rc = doSSH("pins 0", $self->{"is_local"}, \@out, \@err, "nortel", "host");
#          fillPErrorAndPStatus($patchDetailsTag, $rc, $command, \@out, \@err);
# @param   $patchDetailsTag - <patchdetails> tag for particular patch.
# @param   $exitStatus - exit status of the executed command (scp or ssh).
# @param   $refToError - reference to captured error.
# @param   $command - command executed on the remote host.
# @param   $refToOutput - reference to captured command output.
# @return  FALSE - if error occured
# @return  TRUE - <perror> and <pstatus> tags were updated successfully
# @date    July 2008
sub fillPErrorAndPStatus($$$;$$)
{
   my ($patchDetails, $exitStatus, $err, $command, $out) = @_;
   my $perror = getTextElement($patchDetails, $PATCH_TAG{"PERROR"});
   if (not defined $perror)
   {
      baselog($LOG_ERROR, "Can't retrieve $PATCH_TAG{PERROR} element from document.");
      return $FALSE;
   }
   my $pstatus = getTextElement($patchDetails, $PATCH_TAG{"PSTATUS"});
   if (not defined $pstatus)
   {
      baselog($LOG_ERROR, "Can't retrieve $PATCH_TAG{PSTATUS} element from document.");
      return $FALSE;
   }
   my $retValue = $TRUE;
   if ($exitStatus == $TRUE)
   {
      unless (setTextData($pstatus, "Succeed"))
      {
         baselog($LOG_ERROR, "Can't set 'Succeed' vaule into $PATCH_TAG{PSTATUS} tag of $PATCH_XML_FILE.");
         $retValue = $FALSE;
      }
      unless (setTextData($perror, $DEF_VALUE))
      {
         baselog($LOG_ERROR, "Can't set $DEF_VALUE value into $PATCH_TAG{PERROR} tag of $PATCH_XML_FILE.");
         $retValue = $FALSE;
      }
   } ## end if ($exitStatus == $TRUE...
   else
   {
      unless (setTextData($pstatus, "Failed"))
      {
         baselog($LOG_ERROR, "Can't set 'Failed' value into $PATCH_TAG{PSTATUS} tag of $PATCH_XML_FILE.");
         $retValue = $FALSE;
      }
      my $error_message = constructMessage($err, $command, $out);
      unless (setTextData($perror, $error_message))
      {
         baselog($LOG_ERROR, "Can't set $error_message value into $PATCH_TAG{PERROR} tag of $PATCH_XML_FILE.");
         $retValue = $FALSE;
      }
   } ## end else [ if ($exitStatus == $TRUE...
   return $retValue;
} ## end sub fillPErrorAndPStatus($$$;$$)

###############################################################################
## @method constructMessage($refToError, $command, $refToError)
# @brief   Constructs message based on $refToError and $refToOutput parameters.
# @param   $refToError - reference to captured error.
# @param   $command - command executed on the remote host.
# @param   $refToOutput - reference to captured command output.
# @return  FALSE - if error occured
# @return  TRUE - <perror> and <pstatus> tags were updated successfully
# @date    December 2008
sub constructMessage($;$$)
{
   my ($err, $cmd, $out) = @_;

   # Constructing error message
   my @msg = ();
   if ((not defined $out) or (not scalar(@$out)))
   {
      if ((not defined $err) or (not scalar(@$err)))
      {
         push @msg, $DEF_VALUE;
      }
      else
      {
         chomp(@$err);
         @msg = @$err;
      }
   } ## end if ((not defined $out)...
   else
   {
      if (defined $cmd)
      {
         push @msg, "Command '$cmd' failed.\n";
         push @msg, "Command output: @$out";
      }
      else
      {
         chomp(@$out);
         @msg = @$out;
      }
   } ## end else [ if ((not defined $out)...

   return join("\n", @msg);
} ## end sub constructMessage($;$$)

###############################################################################
## @function checkFile($fileName)
# @brief   Checks if provided file can be re-written.
# @param   $fileName - file name.
# @return  error - if directory contained the file provided doesn't exist or is not writable or file
#          exists and is not writable
# @return  undef - if file can be re-written
# @date    October 2008
sub checkFile($)
{
   my $fileName = shift @_;
   my $error;
   if (-e $fileName and not -w $fileName)
   {
      $error = "$fileName: Permission denied";
   }
   elsif (not -e $fileName)
   {
      my $dirName = dirname($fileName);
      if (not -e $dirName)
      {
         $error = "$dirName: No such directory";
      }
      elsif (not -w $dirName)
      {
         $error = "$dirName: Permission denied";
      }
   } ## end elsif (not -e $fileName)
   return $error;
} ## end sub checkFile($)

###############################################################################
## @function cPcmd($self)
# @brief   Analyzes <pcommand> tag of patch.xml file and calls associated function.
# @param   $self - reference to patch object.
# @return  FALSE - if something wrong with patch.xml file or associated command returned FALSE
# @return  TRUE - if associated command returned TRUE
# @date    July 2008
sub cPcmd($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      return $FALSE;
   }

   my $pcmd = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"PCOMMAND"}));
   if (not defined $pcmd)
   {
      baselog($LOG_ERROR, "Can't retrieve patch command. Please check <pcommand> tag of " . $self->{"patch_xml"} . ".");
      return $FALSE;
   }
   elsif ($pcmd =~ /^(cPatchTransfer|cPload|cPins|cPoos|cPout|cPstat|cSwVersionShow|cSPload|cSPins|cSPout|cIssp)$/)
   {
      baselog($LOG_INFO, "Invoking $pcmd function.");
      return eval "$1(\$self)";
   }
   else
   {
      baselog($LOG_ERROR, "Wrong command in <pcommand> tag: '$pcmd'");

      # updating <overallerror> and <overallstatus> tags
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  "Wrong command in <pcommand> tag: '$pcmd'");
      updatePatchXML($self);
      return $FALSE;
   } ## end else [ if (not defined $pcmd)
} ## end sub cPcmd($)

###############################################################################
## @function cPatchTransfer($self)
# @brief   Transfers patches SUs or Service Pack to the host specified.
# @details If <type> tag contains "PATCH" it copies patches specified in the <patchdetails> tags of the patch.xml file
#          from the PM server to the specific targets patch directory. If "SP" value is specified in the
#          <type> tag, then Service Pack which name is taken from the <spname> tag will be transferred to the
#          remote host. The copying of the patches will make use the user of the SSH Secure Copy (scp) utility
#          and is initiated from the PM Server side (Primary Security Server).
#          cPatchTransfer also updates the patch.xml file with the patch delivery
#          status. The PM thus polls the specific patch.xml to get the status of the patch transfer.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    July 2008
sub cPatchTransfer($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }

   my $type = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"TYPE"}));
   if (not defined $type)
   {
      baselog($LOG_ERROR, "Can't retrieve type. Please check <type> tag of " . $self->{"patch_xml"} . ".");
      return $FALSE;
   }
   my $patchFilePath = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"PATCHFILEPATH"}));
   if ((not defined $patchFilePath) or ($patchFilePath eq ""))
   {
      baselog($LOG_ERROR, "Can't locate directory with patches.");
      return $FALSE;
   }
   my @scpOut;
   my @scpErr;
   my $rc = $FALSE;

   #type is SP
   if ($type =~ m/^sp$/i)
   {
      baselog($LOG_INFO, "cPatchTransfer():: Processing $type ");

      # need to transfer one SP file
      my $spname = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"SPNAME"}));

      # check if spname defined and is not empty
      if ((not defined $spname) or ($spname eq ""))
      {
         my $error = "Not defined or empty Service Pack name.";
         baselog($LOG_ERROR, $error);

         # that's it, filling <overallstatus> and <overallerror> and exit.
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      } ## end if ((not defined $spname...
      else
      {
         if (not -f $patchFilePath . "/" . $spname)
         {
            @scpErr = "SP file $spname specified in $PATCH_XML_FILE doesn't exist.";
            baselog($LOG_ERROR, @scpErr);
         }
         else
         {

            # $SRV_PACK_DIR is defined in patches_target.pm
            $rc = scpPut("$patchFilePath/$spname", $NORTEL_ACCT, $self->{"targetip"}, $SRV_PACK_DIR, $self->{"is_local"}, \@scpOut, \@scpErr);
         } ## end else [ if (not -f $patchFilePath...

         # scp completed with error
         if (!$rc)
         {
            my $error_msg = constructMessage(\@scpErr, $SCP, \@scpOut);
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error_msg);
         } ## end if (!$rc)

         # scp completed successfully, so SP has been transferred.
         else
         {
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $DEF_VALUE);
         }
      } ## end else [ if ((not defined $spname...
   } ## end if ($type =~ m/^sp$/i)

   # type is PATCH
   elsif ($type =~ m/^patch$/i)
   {
      baselog($LOG_INFO, "cPatchTransfer():: Processing $type ");

      foreach my $patchDetails ($self->{$PATCH_TAG{"PATCHES"}}->getChildrenByTagName($PATCH_TAG{"PATCHDETAILS"}))
      {
         my $pname = getTagValue(getTextElement($patchDetails, $PATCH_TAG{"PNAME"}));
         if ((not defined $pname) or ($pname eq ""))
         {
            my @error = "Not defined or empty patch name tag.";
            baselog($LOG_ERROR, @error);
            $self->{"actionsCount"}++;
            $self->{"actionsFailed"}++;
            return $FALSE unless fillPErrorAndPStatus($patchDetails, $FALSE, \@error);
            next;
         } ## end if ((not defined $pname...

         if (not -f $patchFilePath . "/" . $pname)
         {
            @scpErr = "Patch file $patchFilePath/$pname specified in $PATCH_XML_FILE doesn't exist.";
            baselog($LOG_ERROR, @scpErr);
         }
         else
         {
            $rc = scpPut("$patchFilePath/$pname", $NORTEL_ACCT, $self->{"targetip"}, $PATCHDIR, $self->{"is_local"}, \@scpOut, \@scpErr);
         }

         $self->{"actionsFailed"}++ unless $rc;
         $self->{"actionsCount"}++;

         # If can't update <perror> or/and <pstatus> tags of patch.xml then return FALSE.
         return $FALSE unless fillPErrorAndPStatus($patchDetails, $rc, \@scpErr, $SCP, \@scpOut);
      } ## end foreach my $patchDetails ($self...
      return $FALSE unless fillOverallErrorAndOverallStatus($self, "transfers");
   } ## end elsif ($type =~ m/^patch$/i...

   # <type> tag contains wrong value
   else
   {
      baselog($LOG_ERROR, "Wrong value in <type> tag: '$type'");

      # updating <overallerror> and <overallstatus> tags
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  "Wrong value in <type> tag: '$type'");
   } ## end else [ if ($type =~ m/^sp$/i)
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPatchTransfer($)

###############################################################################
## @function cPload($self)
# @brief   Loads the patches specified in the patch.xml file into the target database
#          with the specific patch information.
# @details This command invokes pload <patch name> on the remote host to load particular
#          patch. All patches specified in the <patchdetails> tags of the patch.xml are
#          loaded sequentially. Also updates <pid> tag in the patch.xml file with the i
#          patch handles for each loaded patch.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    July 2008
sub cPload($)
{
   my $self        = shift @_;
   my %patches     = ();
   my %p_details   = ();
   my @patch_queue = ();
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   foreach my $patchDetails (@{$self->{$PATCH_TAG{"PATCHES"}}->getChildrenByTagName($PATCH_TAG{"PATCHDETAILS"})})
   {
      my $pname = getTagValue(getTextElement($patchDetails, $PATCH_TAG{"PNAME"}));
      if ((not defined $pname) or ($pname eq ""))
      {
         my @error = "Not defined or empty patch name.";
         baselog($LOG_ERROR, @error);
         $self->{"actionsCount"}++;
         $self->{"actionsFailed"}++;
         if (not fillPErrorAndPStatus($patchDetails, $FALSE, \@error))
         {
            baselog($LOG_ERROR, "Cannot update patch.xml for '$pname'");
         }
      } ## end if ((not defined $pname...
      else
      {
         $patches{$pname} = {'status' => undef, 'out' => [], 'err' => [], 'pid' => -1, 'rc' => $FALSE};
         push @patch_queue, $pname;
         $p_details{$pname} = $patchDetails;
         $self->{"actionsCount"}++;
      } ## end else [ if ((not defined $pname...
   } ## end foreach my $patchDetails (@...
   my @sshOut;
   my @sshErr;
   my $pnames = join(',', @patch_queue);
   doSSH("$PATCHCOMMANDS{pPload} $pnames", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});

   #Initially let's think that all pathces were failed, parsing data for every patch will change this assumption
   $self->{"actionsFailed"} = $self->{"actionsCount"};
   my $pname;
   foreach my $str (@sshOut)
   {

      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {

         #to start parse output for single patch
         $pname = $1;
      } ## end if ($str =~ /$PATCH_str\s*(.+)\s*/...
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         my $status;
         while ($status = pop @{$patches{$pname}->{'out'}})
         {
            if ($status =~ /$STATUS_str/)
            {
               $status =~ s/$STATUS_str//;
               last;
            }
         } ## end while ($status = pop @{$patches...
         if (not defined $status)
         {
            baselog($LOG_ERROR, "Cannot get status string for patch $pname");
            $status = " $pERROR -1\n";
         }

         my $pid;
         ($status, $pid) = split(' ', $status);
         if ($status =~ /$pOK/)
         {
            $patches{$pname}->{'rc'} = $TRUE;
            $self->{"actionsFailed"} = $self->{"actionsFailed"} - 1;
         }
         $patches{$pname}->{'status'} = $status;
         $patches{$pname}->{'pid'}    = $pid;
      } ## end elsif ($str =~ /$PM_SEPARATOR/...
      else
      {
         push @{$patches{$pname}->{'out'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshOut)
   foreach my $str (@sshErr)
   {
      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {
         $pname = $1;
      }
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         next;
      }
      else
      {
         push @{$patches{$pname}->{'err'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshErr)

   foreach my $patch (keys %patches)
   {
      my $pid = getTextElement($p_details{$patch}, $PATCH_TAG{"PID"});
      setTextData($pid, $patches{$patch}->{'pid'});
      if (not fillPErrorAndPStatus($p_details{$patch}, $patches{$patch}->{'rc'}, $patches{$patch}->{'err'}, "$PATCHCOMMANDS{pPload} $patch", $patches{$patch}->{'out'}))
      {
         baselog($LOG_ERROR, "Cannot update patch.xml for '$patch'.");
      }
   } ## end foreach my $patch (keys %patches...
   return $FALSE unless fillOverallErrorAndOverallStatus($self, "patches");
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPload($)

###############################################################################
## @function cSPload($self)
# @brief   Loads the Service pack specified in the patch.xml file into the target database
#          with the specific patch information.
# @details This command invokes spload <SP name> on the remote host to load particular
#          SP. SP name specified in <spname> tag of patch.xml file is loaded into patch
#          database on the remote host.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    October 2008
sub cSPload($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   my $spload_txt = "$PATCHLIB_DIRECTORY/$self->{'targetname'}/$SPLOAD_TEXT_FILE";
   my $error      = checkFile($spload_txt);

   # spload.txt file is not accessable if defined $error
   if (defined $error)
   {
      baselog($LOG_ERROR, $error);
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      updatePatchXML($self);
      return $FALSE;
   } ## end if (defined $error)

   # retrieving service pack name
   my $spname    = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"SPNAME"}));
   my $overerror = $DEF_VALUE;

   if ((not defined $spname) or ($spname eq ""))
   {
      $error = "Not defined or empty Service Pack name.";
      baselog($LOG_ERROR, $error);
   }
   else
   {
      my @sshOut;
      my @sshErr;

      # executing spload command on the target host.
      my $rc = doSSH("$PATCHCOMMANDS{spload} $spname", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});

      #if  we got an error then try to check on "partially completed" status first
      if (!$rc)
      {
         if ((@sshOut) and scalar(@sshOut))
         {

            # get fault status string as it should be generated by target.
            # Note!: function get_fault_sp_str() should be identical on the
            # primary and member servers!
            my $status_str = patchutil::patches_target::get_fault_sp_str('spload', $spname);
            foreach my $str (@sshOut)
            {
               if (index($str, $status_str) > -1)
               {
                  $overerror = $status_str;
                  baselog($LOG_INFO, 'Minor issue detected on the target side. Set as Succeed status.');

                  # set return value as Success.
                  $rc = $TRUE;
                  last;
               } ## end if (index($str, $status_str...
            } ## end foreach my $str (@sshOut)
         } ## end if ((@sshOut) and scalar...
      } ## end if (!$rc)

      if (!$rc)
      {

         # ssh exits with error
         $error = constructMessage(\@sshErr, "$PATCHCOMMANDS{spload} $spname", \@sshOut);
      } ## end if (!$rc)
      else
      {

         # creating spload.txt file based on console output
         my $FH;

         # this should not happen
         unless (open($FH, ">", "$spload_txt"))
         {

            # actually ssh completed successfully, but result can not be saved into spload.txt file
            baselog($LOG_ERROR, "Can't open $spload_txt file.");
            my @msg = ("Can't open $spload_txt file.", "Command '$PATCHCOMMANDS{spload} $spname' completed successfully.", "Command output:  @sshOut");
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Partially Completed");
            return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}}, join("\n", @msg));
            return (updatePatchXML($self) ? $TRUE : $FALSE);
         } ## end unless (open($FH, ">", "$spload_txt"...
         print $FH @sshOut;

         baselog($LOG_ERROR, "Cannot close the file descriptor ($spload_txt)") unless close($FH);

      } ## end else [ if (!$rc)
   } ## end else [ if ((not defined $spname...

   # error was met, updating patch.xml accordingly
   if (defined $error)
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
   }

   # there is no error, cSPload completed successfully
   else
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $overerror);
   }
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cSPload($)

###############################################################################
## @function cSPins($self)
# @brief   Puts Service Pack specified in patch.xml file into service on the remote host.
# @details This command invokes spins on the remote host to put particular
#          SP in service.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    October 2008
sub cSPins($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   my $spins_txt = "$PATCHLIB_DIRECTORY/$self->{'targetname'}/$SPINS_TEXT_FILE";
   my $error     = checkFile($spins_txt);

   # spins.txt file is not accessable if defined $error
   if (defined $error)
   {
      baselog($LOG_ERROR, $error);
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      updatePatchXML($self);
      return $FALSE;
   } ## end if (defined $error)

   # don't need Service Pack name for spins command
   my @sshOut;
   my @sshErr;
   my $overerror = $DEF_VALUE;

   # executing spins command on the target host.
   my $rc = doSSH("$PATCHCOMMANDS{spins}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   if (!$rc)
   {

      # ssh exits with error
      $error = constructMessage(\@sshErr, "$PATCHCOMMANDS{spins}", \@sshOut);
   } ## end if (!$rc)
   else
   {

      # creating spins.txt file based on console output
      my $FH;

      # this should not happen
      unless (open($FH, ">", "$spins_txt"))
      {

         # actually ssh completed successfully, but result can not be saved into spins.txt file
         baselog($LOG_ERROR, "Can't open $spins_txt file.");
         my @msg = ("Can't open $spins_txt file.", "Command '$PATCHCOMMANDS{spins}' completed successfully.", "Command output:  @sshOut");
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Partially Completed");
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}}, join("\n", @msg));
         return (updatePatchXML($self) ? $TRUE : $FALSE);
      } ## end unless (open($FH, ">", "$spins_txt"...
      print $FH @sshOut;

      baselog($LOG_ERROR, "Cannot close the file descriptor ($spins_txt)") unless close($FH);

      my $spname = getTagValue(getTextElement($self->{"root"}, $PATCH_TAG{"SPNAME"}));
      if ((not defined $spname) or ($spname eq ""))
      {
         baselog($LOG_ERROR, 'Not defined or empty Service Pack name.');
      }
      else
      {

         # get fault status string as it should be generated by target.
         # Note!: function get_fault_sp_str() should be identical on the
         # primary and member servers!
         my $status_str = patchutil::patches_target::get_fault_sp_str('spins', $spname);
         foreach my $str (@sshOut)
         {
            if (index($str, $status_str) > -1)
            {
               $overerror = $status_str;
               baselog($LOG_INFO, 'Minor issue detected on the target side.');
               last;
            } ## end if (index($str, $status_str...
         } ## end foreach my $str (@sshOut)
      } ## end else [ if ((not defined $spname...

   } ## end else [ if (!$rc)

   # cSPins completed, updating patch.xml
   if (defined $error)
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
   }
   else
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $overerror);
   }
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cSPins($)

###############################################################################
## @function cPins($self)
# @brief   Puts patches specified in patch.xml file into service on the remote host.
# @details This command invokes pins <patch id> on the remote host to load particular
#          patch. All patches specified in the <patchdetails> tags of the patch.xml are
#          inserted sequentially (one by one). <patch id> information provided cPload
#          command and obtained from <pid> tag of patch.xml for every patch.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    July 2008
sub cPins($)
{
   my $self        = shift @_;
   my %patches     = ();
   my %p_details   = ();
   my @patch_queue = ();
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   foreach my $patchDetails (@{$self->{$PATCH_TAG{"PATCHES"}}->getChildrenByTagName($PATCH_TAG{"PATCHDETAILS"})})
   {
      my $pname = getTagValue(getTextElement($patchDetails, $PATCH_TAG{"PNAME"}));
      if ((not defined $pname) or ($pname eq ""))
      {
         my @error = "Not defined or empty patch ID.";
         baselog($LOG_ERROR, @error);
         $self->{"actionsCount"}++;
         $self->{"actionsFailed"}++;
         if (not fillPErrorAndPStatus($patchDetails, $FALSE, \@error))
         {
            baselog($LOG_ERROR, "Cannot update patch.xml for '$pname'");
         }
      } ## end if ((not defined $pname...
      else
      {
         push @patch_queue, $pname;
         $patches{$pname} = {'status' => undef, 'out' => [], 'err' => [], 'pid' => -1, 'rc' => $FALSE};
         $p_details{$pname} = $patchDetails;
         $self->{"actionsCount"}++;
      } ## end else [ if ((not defined $pname...
   } ## end foreach my $patchDetails (@...
   my @sshOut;
   my @sshErr;
   my $pnames = join(',', @patch_queue);
   my $rc = doSSH("$PATCHCOMMANDS{pPins} $pnames", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   $self->{"actionsFailed"} = $self->{"actionsCount"};
   my $pname;
   foreach my $str (@sshOut)
   {

      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {

         #to start parse output for single patch
         $pname = $1;
      } ## end if ($str =~ /$PATCH_str\s*(.+)\s*/...
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         baselog($LOG_DEBUG, "Get status for '$pname'.");
         my $status;
         while ($status = pop @{$patches{$pname}->{'out'}})
         {
            if ($status =~ /$STATUS_str/)
            {
               $status =~ s/$STATUS_str//;
               last;
            }
         } ## end while ($status = pop @{$patches...
         if (not defined $status)
         {
            baselog($LOG_ERROR, "Cannot get status string for patch $pname");
            $status = " $pERROR -1\n";
         }

         my $pid;
         ($status, $pid) = split(' ', $status);
         if ($status =~ /$pOK/)
         {
            $patches{$pname}->{'rc'} = $TRUE;
            $self->{"actionsFailed"} = $self->{"actionsFailed"} - 1;
         }
         $patches{$pname}->{'status'} = $status;
         $patches{$pname}->{'pid'}    = $pid;
      } ## end elsif ($str =~ /$PM_SEPARATOR/...
      else
      {
         push @{$patches{$pname}->{'out'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshOut)
   foreach my $str (@sshErr)
   {
      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {

         #if e
         $pname = $1;
      } ## end if ($str =~ /$PATCH_str\s*(.+)\s*/...
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         next;
      }
      else
      {
         push @{$patches{$pname}->{'err'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshErr)

   foreach my $patch (keys %patches)
   {
      if (not fillPErrorAndPStatus($p_details{$patch}, $patches{$patch}->{'rc'}, $patches{$patch}->{'err'}, "$PATCHCOMMANDS{pPins} $patch", $patches{$patch}->{'out'}))
      {
         baselog($LOG_ERROR, "Cannot update patch.xml for '$patch'.");
      }
   } ## end foreach my $patch (keys %patches...
   return $FALSE unless fillOverallErrorAndOverallStatus($self, "patches");
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPins($)

###############################################################################
## @function cPoos($self)
# @brief   Removes patches specified in the <patchdetails> tags of the patch.xml
#          file from service.
# @details This function invokes poos <patch id> command on the remote host to
#          remove particular patch from service. All patches specified in the
#          patch.xml are handled sequentially (one by one). <patch id> information
#          provided by cPload command and obtained from <pid> tag of patch.xml for
#          every patch.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    July 2008
sub cPoos($)
{
   my $self        = shift @_;
   my %patches     = ();
   my %p_details   = ();
   my @patch_queue = ();
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   foreach my $patchDetails (@{$self->{$PATCH_TAG{"PATCHES"}}->getChildrenByTagName($PATCH_TAG{"PATCHDETAILS"})})
   {
      my $pname = getTagValue(getTextElement($patchDetails, $PATCH_TAG{"PNAME"}));
      if ((not defined $pname) or ($pname eq ""))
      {
         my @error = "Not defined or empty patch name.";
         baselog($LOG_ERROR, @error);
         $self->{"actionsCount"}++;
         $self->{"actionsFailed"}++;
         if (not fillPErrorAndPStatus($patchDetails, $FALSE, \@error))
         {
            baselog($LOG_ERROR, "Cannot update patch.xml for '$pname'");
         }
         next;
      } ## end if ((not defined $pname...
      else
      {
         push @patch_queue, $pname;
         $patches{$pname} = {'status' => undef, 'out' => [], 'err' => [], 'pid' => -1, 'rc' => $FALSE};
         $p_details{$pname} = $patchDetails;
         $self->{"actionsCount"}++;
      } ## end else [ if ((not defined $pname...
   } ## end foreach my $patchDetails (@...
   my @sshOut;
   my @sshErr;
   my $pnames = join(',', @patch_queue);
   my $rc = doSSH("$PATCHCOMMANDS{pPoos} $pnames", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   $self->{"actionsFailed"} = $self->{"actionsCount"};
   my $pname;
   foreach my $str (@sshOut)
   {

      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {

         #to start parse output for single patch
         $pname = $1;
      } ## end if ($str =~ /$PATCH_str\s*(.+)\s*/...
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         baselog($LOG_DEBUG, "Get status for '$pname'");
         my $status;
         while ($status = pop @{$patches{$pname}->{'out'}})
         {
            if ($status =~ /$STATUS_str/)
            {
               $status =~ s/$STATUS_str//;
               last;
            }
         } ## end while ($status = pop @{$patches...
         if (not defined $status)
         {
            baselog($LOG_ERROR, "Cannot get status string for patch $pname");
            $status = " $pERROR -1\n";
         }

         my $pid;
         ($status, $pid) = split(' ', $status);
         if ($status =~ /$pOK/)
         {
            $patches{$pname}->{'rc'} = $TRUE;
            $self->{"actionsFailed"} = $self->{"actionsFailed"} - 1;
         }
         $patches{$pname}->{'status'} = $status;
         $patches{$pname}->{'pid'}    = $pid;
      } ## end elsif ($str =~ /$PM_SEPARATOR/...
      else
      {
         push @{$patches{$pname}->{'out'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshOut)
   foreach my $str (@sshErr)
   {
      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {
         $pname = $1;
      }
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         next;
      }
      else
      {
         push @{$patches{$pname}->{'err'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshErr)
   foreach my $patch (keys %patches)
   {
      if (not fillPErrorAndPStatus($p_details{$patch}, $patches{$patch}->{'rc'}, $patches{$patch}->{'err'}, "$PATCHCOMMANDS{pPoos} $patch", $patches{$patch}->{'out'}))
      {
         baselog($LOG_ERROR, "Cannot update patch.xml for '$patch'.");
      }
   } ## end foreach my $patch (keys %patches...
   return $FALSE unless fillOverallErrorAndOverallStatus($self, "patches");
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPoos($)

###############################################################################
## @function cPout($self)
# @brief   Unloads patches specified in the patch.xml file from target host.
# @details This command invokes pout <patch id> on the remote host to unload particular patch
#          that was loaded with the pload command and cleanup the on-switch database of
#          the specific patch information.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    July 2008
sub cPout($)
{
   my $self        = shift @_;
   my %patches     = ();
   my %p_details   = ();
   my @patch_queue = ();
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   foreach my $patchDetails (@{$self->{$PATCH_TAG{"PATCHES"}}->getChildrenByTagName($PATCH_TAG{"PATCHDETAILS"})})
   {
      my $pname = getTagValue(getTextElement($patchDetails, $PATCH_TAG{"PNAME"}));
      if ((not defined $pname) or ($pname eq ""))
      {
         my @error = "Not defined or empty patch name.";
         baselog($LOG_ERROR, @error);
         $self->{"actionsCount"}++;
         $self->{"actionsFailed"}++;
         if (not fillPErrorAndPStatus($patchDetails, $FALSE, \@error))
         {
            baselog($LOG_ERROR, "Cannot update patch.xml for '$pname'");
         }
         next;
      } ## end if ((not defined $pname...
      else
      {
         push @patch_queue, $pname;
         $patches{$pname} = {'status' => undef, 'out' => [], 'err' => [], 'pid' => -1, 'rc' => $FALSE};
         $p_details{$pname} = $patchDetails;
         $self->{"actionsCount"}++;
      } ## end else [ if ((not defined $pname...
   } ## end foreach my $patchDetails (@...
   my $pnames = join(',', @patch_queue);
   my @sshOut;
   my @sshErr;
   my $rc = doSSH("$PATCHCOMMANDS{pPout} $pnames", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   $self->{"actionsFailed"} = $self->{"actionsCount"};
   my $pname;
   foreach my $str (@sshOut)
   {

      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {

         #to start parse output for single patch
         $pname = $1;
      } ## end if ($str =~ /$PATCH_str\s*(.+)\s*/...
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         my $status;
         while ($status = pop @{$patches{$pname}->{'out'}})
         {
            if ($status =~ /$STATUS_str/)
            {
               $status =~ s/$STATUS_str//;
               last;
            }
         } ## end while ($status = pop @{$patches...
         if (not defined $status)
         {
            baselog($LOG_ERROR, "Cannot get status string for patch $pname");
            $status = " $pERROR -1\n";
         }

         my $pid;
         ($status, $pid) = split(' ', $status);
         if ($status =~ /$pOK/)
         {
            $patches{$pname}->{'rc'} = $TRUE;
            $self->{"actionsFailed"} = $self->{"actionsFailed"} - 1;
         }
         $patches{$pname}->{'status'} = $status;
         $patches{$pname}->{'pid'}    = $pid;
      } ## end elsif ($str =~ /$PM_SEPARATOR/...
      else
      {
         push @{$patches{$pname}->{'out'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshOut)
   foreach my $str (@sshErr)
   {
      if ($str =~ /$PATCH_str\s*(.+)\s*/)
      {
         $pname = $1;
      }
      elsif ($str =~ /$PM_SEPARATOR/)
      {
         next;
      }
      else
      {
         push @{$patches{$pname}->{'err'}}, $str if ($pname);
      }
   } ## end foreach my $str (@sshErr)

   foreach my $patch (keys %patches)
   {
      setTextData($p_details{$patch}, "-1");
      if (not fillPErrorAndPStatus($p_details{$patch}, $patches{$patch}->{'rc'}, $patches{$patch}->{'err'}, "$PATCHCOMMANDS{pPout} $patch", $patches{$patch}->{'out'}))
      {
         baselog($LOG_ERROR, "Cannot update patch.xml for '$patch'.");
      }
   } ## end foreach my $patch (keys %patches...
   return $FALSE unless fillOverallErrorAndOverallStatus($self, "patches");
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPout($)

###############################################################################
## @function cSPout($self)
# @brief   This command is used to cancel a spload command on the target host.
# @details cSPout invokes spout on the remote host to remove particular
#          SP from the patch database.
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    October 2008
sub cSPout($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }

   # don't need Service Pack name for spout
   my @sshOut;
   my @sshErr;

   # executing spout command on the target host.
   my $rc = doSSH("$PATCHCOMMANDS{spout}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});

   # error was met, updating patch.xml accordingly
   if (not $rc)
   {
      my $error = constructMessage(\@sshErr, "$PATCHCOMMANDS{spout}", \@sshOut);
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
   } ## end if (not $rc)

   # there is no error, cSPout completed successfully
   else
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $DEF_VALUE);
   }
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cSPout($)

###############################################################################
## @function cIssp($self)
# @brief   Obtains information as to what is installed on the target (all RPMs, SUs and patches) host.
# @details This command invokes issp on the remote host
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml file.
# @date    October 2008
sub cIssp($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   my $issp_txt = "$PATCHLIB_DIRECTORY/$self->{'targetname'}/$ISSP_TEXT_FILE";
   my $error    = checkFile($issp_txt);

   # issp.txt file is not accessable if defined $error
   if (defined $error)
   {
      baselog($LOG_ERROR, $error);
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      updatePatchXML($self);
      return $FALSE;
   } ## end if (defined $error)
   my @sshOut;
   my @sshErr;

   # executing issp command on the target host.
   my $rc = doSSH("$PATCHCOMMANDS{issp}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   if (!$rc)
   {

      # ssh exits with error
      $error = constructMessage(\@sshErr, "$PATCHCOMMANDS{issp}", \@sshOut);
   } ## end if (!$rc)
   else
   {

      # creating issp.txt file based on console output
      my $FH;

      # this should not happen
      unless (open($FH, ">", "$issp_txt"))
      {

         # actually ssh completed successfully, but result can not be saved into issp.txt file
         $error = "Command $PATCHCOMMANDS{issp} completed successfully.\nCan't open $issp_txt file to save the result.";
         baselog($LOG_ERROR, $error);
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Partially Completed");
         return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
         return (updatePatchXML($self) ? $TRUE : $FALSE);
      } ## end unless (open($FH, ">", "$issp_txt"...
      print $FH @sshOut;

      baselog($LOG_ERROR, "Cannot close the file descriptor ($issp_txt)") unless close($FH);
   } ## end else [ if (!$rc)
     # cIssp completed, updating patch.xml
   if (defined $error)
   {
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
   }
   else
   {

      # command ssh completed successfully and issp output saved into issp.txt file.
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $DEF_VALUE);
   } ## end else [ if (defined $error)

   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cIssp($)

###############################################################################
## @function cPstat($self)
# @brief   Executes 'pstat -a' and 'spstat' commands on the specified host and fills pstat.xml file.
# @details Is responsible for retrieving patch and SP information from the target host and create
#          pstat.xml file to be used by PM. It makes two function calls: 'pstat -a' and 'cpstat' on
#          the remote host.
#          Patch information for each patch looks like:
#             Handle:   0
#             Type:     PATCH
#             Filename: p19000_1.ntl
#             Dependency: None
#             CR number: Q000000000
#             Engineer: Ana Alvarez
#             Created: Tue Sep 16 15:49:33 2008
#             Loaded: Tue Sep 23 12:21:59 2008
#             Patch is in-service
#             In-service date: 10/03/07  01:24:08
#             Special Instructions: no
#             Applications stopped: None
#             Applications started: None
#             Requires reboot: No
#             PrePatch script: No
#             PostPatch script: No
#             Patch member type: JAR
#             RPM: nortel-cs1000-vtrk-6.00.00-00.i386.rpm
#          SP information looks like:
#             The following SP is in loaded status: sp2.ntl
#             Loaded by user nortel on Tue Jan 13 11:51:52 2009.
#
#             The following SP is in service: sp3.ntl
#             Put in service by user nortel on Sun Jan 11 14:52:38 2009.
#             spins command completed with no errors detected..
#          Then this output is parsed and pstat.xml file created. This file has the following format:
#		<?xml version="1.0" encoding="UTF-8"?>
#		<cpstat>
#			<loaded_sp>
#				<date>Mon Jan 12 11:23:50 2009</date>
#				<name>sp2.ntl</name>
#			</loaded_sp>
#			<installed_sp>
#				<date>Mon Jan 12 12:26:20 2009</date>
#				<name>sp4.ntl</name>
#			</installed_sp>
#			<patchdetails>
#				<pname>p12345_1.ntl</pname>
#				<pid>0</pid>
#				<type>Patch</type>
#				<subtype>WAR</subtype>
#				<loaded_date>Tue Sep 23 12:21:59 2008</loaded_date>
#				<created>Tue Sep 16 15:49:33 2008</created>
#				<pstat>In service</pstat>
#				<operation_date>10/03/07  01:24:08</operation_date>
#				<pre_script>YES</pre_script>
#				<specinst>YES</specinst>
#				<post_script>NO</post_script>
#				<reboot>NO</reboot>
#				<app_stop>vtrk</app_stop>
#				<app_start>vtrk</app_start>
#				<dependent>p12344.ntl/dependent>
#				<cr>Q999999</cr>
#				<rpm_name></rpm_name>
#			</patchdetails>
#		</cpstat>
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml or pstat.xml file.
# @date    July 2008
sub cPstat($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   my $pstat_xml = "$PATCHLIB_DIRECTORY/$self->{'targetname'}/$PSTAT_XML_FILE";
   my $error     = checkFile($pstat_xml);
   if (defined $error)
   {
      baselog($LOG_ERROR, $error);
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      updatePatchXML($self);
      return $FALSE;
   } ## end if (defined $error)
   my $pstat_xml_doc = XML::LibXML::Document->new($XML_VERSION, $XML_ENCODING);
   if (not defined $pstat_xml_doc)
   {
      baselog($LOG_ERROR, "$pstat_xml: Can't create document.");
      return $FALSE;
   }
   my $rootElem = XML::LibXML::Element->new($PSTAT_TAG{"CPSTAT"});
   if (not defined $rootElem)
   {
      baselog($LOG_ERROR, "$pstat_xml: Can't create root element.");
      return $FALSE;
   }
   $pstat_xml_doc->setDocumentElement($rootElem);

   my @sshOut;
   my @sshErr;

   # making 'pstat -a' function call on the remote host
   my $rc = doSSH("$PATCHCOMMANDS{pstat}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   if (!$rc)
   {
      $error = constructMessage(\@sshErr, $PATCHCOMMANDS{"pstat"}, \@sshOut);
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      return (updatePatchXML($self) ? $TRUE : $FALSE);
   } ## end if (!$rc)
   my $patchElem;
   foreach (@sshOut)
   {

      if (/^\s*Handle:\s*(\d+)/)
      {
         $patchElem = XML::LibXML::Element->new($PSTAT_TAG{"PATCHDETAILS"});
         if (not defined $patchElem)
         {
            baselog($LOG_ERROR, "$pstat_xml: Can't create $PSTAT_TAG{PATCHDETAILS} tag.");
            return $FALSE;
         }

         $rootElem->addChild($patchElem);
         $patchElem->appendTextChild($PSTAT_TAG{"PID"}, $1);
      } ## end if (/^\s*Handle:\s*(\d+)/...
      elsif (/^\s*In system patches:\s*(.*?)\s*$/)
      {
         if ($1 eq '0')
         {
            $patchElem = XML::LibXML::Element->new($PSTAT_TAG{"PATCHDETAILS"});
            $rootElem->addChild($patchElem);
         }
      } ## end elsif (/^\s*In system patches:\s*(.*?)\s*$/...
      elsif (/^\s*Type:\s*(.*?)\s$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"TYPE"}, $1);
      }
      elsif (/^\s*Filename:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PNAME"}, $1);
      }
      elsif (/^\s*Dependency:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"DEPENDENT"}, $1);
      }
      elsif (/^\s*Engineer:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"ENGINEER"}, $1);
      }
      elsif (/^\s*CR number:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"CR"}, $1);
      }
      elsif (/^\s*Created:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"CREATED"}, $1);
      }
      elsif (/^\s*Loaded:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"LOADED_DATE"}, $1);
      }
      elsif (/^\s*Patch member type:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"SUBTYPE"}, $1);
      }
      elsif (/^\s*Special Instructions:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"SPECINST"}, $1);
      }
      elsif (/^\s*Applications stopped:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"APP_STOP"}, $1);
      }
      elsif (/^\s*Applications started:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"APP_START"}, $1);
      }
      elsif (/^\s*PrePatch script:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PRE_SCRIPT"}, $1);
      }
      elsif (/^\s*PostPatch script:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"POST_SCRIPT"}, $1);
      }
      elsif (/^\s*Requires reboot:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"REBOOT"}, $1);
      }
      elsif (/^\s*RPM:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"RPM_NAME"}, $1);
      }
      elsif (/^\s*(?:In-|Out of )service date:\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"OPERATION_DATE"}, $1);
      }
      elsif (/^\s*In-service for\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"SERVICE_FOR"}, $1);
      }
      elsif (/^\s*Out of service for\s*(.*?)\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"SERVICE_FOR"}, $1);
      }
      elsif (/^\s*Patch is in-service\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PSTAT"}, "In service");
      }
      elsif (/^\s*Patch will be in service after restart\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PSTAT"}, "Will be in service after restart");
      }
      elsif (/^\s*Patch will be out of service after restart\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PSTAT"}, "Will be out of service after restart");
      }
      elsif (/^\s*Patch is out-of-service\s*$/)
      {
         $patchElem->appendTextChild($PSTAT_TAG{"PSTAT"}, "Out of service");
      }
   } ## end foreach (@sshOut)

   # completed with 'pstat -a'.
   # making 'spstat' function call on the remote host
   $rc = doSSH("$PATCHCOMMANDS{spstat}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   if (!$rc)
   {
      $error = constructMessage(\@sshErr, $PATCHCOMMANDS{"spstat"}, \@sshOut);
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      return (updatePatchXML($self) ? $TRUE : $FALSE);
   } ## end if (!$rc)
   my $spElem;
   foreach (@sshOut)
   {
      if (/^\s*The following SP is in loaded status:\s*(.*?)$/)
      {
         $spElem = XML::LibXML::Element->new($PSTAT_TAG{"LOADED_SP"});
         if (not defined $spElem)
         {
            baselog($LOG_ERROR, "$pstat_xml: Can't create $PSTAT_TAG{LOADED_SP} tag.");
            return $FALSE;
         }

         $rootElem->addChild($spElem);
         $spElem->appendTextChild($PSTAT_TAG{"SP_NAME"}, $1);
      } ## end if (/^\s*The following SP is in loaded status:\s*(.*?)$/...
      elsif (/^\s*Loaded by user\s*(.*?)\s*(?:on)\s*(.*?)\.$/)
      {
         $spElem->appendTextChild($PSTAT_TAG{"SP_DATE"}, $2);
      }
      elsif (/^\s*There is no SP in loaded status/)
      {
         $spElem = XML::LibXML::Element->new($PSTAT_TAG{"LOADED_SP"});
         if (not defined $spElem)
         {
            baselog($LOG_ERROR, "$pstat_xml: Can't create $PSTAT_TAG{LOADED_SP} tag.");
            return $FALSE;
         }

         $rootElem->addChild($spElem);
         $spElem->appendTextChild($PSTAT_TAG{"SP_NAME"}, $DEF_VALUE);
         $spElem->appendTextChild($PSTAT_TAG{"SP_DATE"}, $DEF_VALUE);
      } ## end elsif (/^\s*There is no SP in loaded status/...
      elsif (/^\s*The following SP is in service:\s*(.*?)$/)
      {
         $spElem = XML::LibXML::Element->new($PSTAT_TAG{"INSTALLED_SP"});
         if (not defined $spElem)
         {
            baselog($LOG_ERROR, "$pstat_xml: Can't create $PSTAT_TAG{INSTALLED_SP} tag.");
            return $FALSE;
         }

         $rootElem->addChild($spElem);
         $spElem->appendTextChild($PSTAT_TAG{"SP_NAME"}, $1);
      } ## end elsif (/^\s*The following SP is in service:\s*(.*?)$/...
      elsif (/^\s*Put in service by user\s*(.*?)\s*on\s*(.*?)\.$/)
      {
         $spElem->appendTextChild($PSTAT_TAG{"SP_DATE"}, $2);
      }
      elsif (/^\s*There is no SP in service/)
      {
         $spElem = XML::LibXML::Element->new($PSTAT_TAG{"INSTALLED_SP"});
         if (not defined $spElem)
         {
            baselog($LOG_ERROR, "$pstat_xml: Can't create $PSTAT_TAG{INSTALLED_SP} tag.");
            return $FALSE;
         }

         $rootElem->addChild($spElem);
         $spElem->appendTextChild($PSTAT_TAG{"SP_NAME"}, $DEF_VALUE);
         $spElem->appendTextChild($PSTAT_TAG{"SP_DATE"}, $DEF_VALUE);
      } ## end elsif (/^\s*There is no SP in service/...
   } ## end foreach (@sshOut)

   return $FALSE unless writeXML($pstat_xml_doc, $pstat_xml);
   return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
   return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $DEF_VALUE);
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cPstat($)

###############################################################################
## @function cSwVersionShow($self)
# @brief   Retrieves the baseline versions of all applications installed of the specific target.
# @details Executes swVersionShow on the specified host and creates swversionshow.xml
#          swversionshow.xml has the following format:
#          <cswversions>
#             <application>
#                 <name>sps</name>
#                 <version>6.0.2</version>
#             </application>
#             <application>
#             ...
#             </application>
#          </cswversions>
# @param   $self - reference to patch object.
# @return  TRUE - if function completed successfully (patch.xml file updated)
# @return  FALSE - indicates an error condition. Something wrong with patch.xml or swversionshow.xml file.
# @date    July 2008
sub cSwVersionShow($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Remote patch object is undefined.");
      return $FALSE;
   }
   my $swversionshow_xml = "$PATCHLIB_DIRECTORY/$self->{'targetname'}/$SWVERSIONSHOW_XML_FILE";
   my $error             = checkFile($swversionshow_xml);
   if (defined $error)
   {
      baselog($LOG_ERROR, $error);
      setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      updatePatchXML($self);
      return $FALSE;
   } ## end if (defined $error)

   my @sshOut;
   my @sshErr;
   my $rc = doSSH("$PATCHCOMMANDS{swVersionShow}", $self->{"is_local"}, \@sshOut, \@sshErr, $NORTEL_ACCT, $self->{"targetip"});
   if (!$rc)
   {
      $error = constructMessage(\@sshErr, $PATCHCOMMANDS{"swVersionShow"}, \@sshOut);
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Failed");
      return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $error);
      return (updatePatchXML($self) ? $TRUE : $FALSE);
   } ## end if (!$rc)
   my $swversionshow_xml_doc = XML::LibXML::Document->new($XML_VERSION, $XML_ENCODING);
   if (not defined $swversionshow_xml_doc)
   {
      baselog($LOG_ERROR, "$swversionshow_xml: Can't create document.");
      return $FALSE;
   }
   my $rootElem = XML::LibXML::Element->new($SWVERSIONSHOW_TAG{"CSWVERS"});
   if (not defined $rootElem)
   {
      baselog($LOG_ERROR, "$swversionshow_xml: Can't create root element.");
      return $FALSE;
   }
   $swversionshow_xml_doc->setDocumentElement($rootElem);

   foreach my $config (@sshOut)
   {
      next unless ($config =~ m/\d/);  # ingore lines without version
      $config =~ s/^\s+//;             # remove leading spaces
      next if ($config =~ m/^Configuration version:/);
      next if ($config =~ m/^(?:Base|Application) configuration:/);
      next if ($config =~ m/^Base Applications/);

      my ($appName, $version) = split(/\s+/, $config);
      return $FALSE unless addApplicationNode($rootElem, $appName, $version);
   } ## end foreach my $config (@sshOut...
   return $FALSE unless writeXML($swversionshow_xml_doc, $swversionshow_xml);
   return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLSTATUS"}}, "Succeed");
   return $FALSE unless setTextData($self->{$PATCH_TAG{"OVERALLERROR"}},  $DEF_VALUE);
   return (updatePatchXML($self) ? $TRUE : $FALSE);
} ## end sub cSwVersionShow($)

1;

# template of patch.xml
#<!This file is maintained for individual target, it contains the target
#  details and the list of patch files name that needs to be transferred-->
#<?xml version="1.0" encoding="UTF-8"?>
#<patch>
#   <patchfilepath>
#      <!-- Patch file path -->
#      /var/opt/nortel/patchlibrary/nortel-cs1000-6.00.40
#   </patchfilepath>
#   <pcommand>
#      <!-- Command to execute on the target system-->
#      <!-- values: patchTransfer/cPload/cPins/cPoos/cPout/cPstat/None, cSPload | cSPins, default:None-->
#      cPins
#   </pcommand>
#   <!-- For SP, the PM only provides patchFilePath, command (cSPload or cSPins), type (SP) and spname to the Linux Base API.
#   In case of SUs & patches, the patchdetails groups will be added and updated by PM based on the number of the patch selected from the PM GUI. During patching operation process Base will update the individual patch status, pid and error. -->
#   <type>SP | PATCH</type>
#   <spname>none|something</spname>
#   <patches>
#      <patchdetails>
#         <pname>
#            <!-- Patch name as stored on MPL -->
#            p12345_1.ntl
#         </pname>
#         <pid>
#            <!-- Patch handle, values:0/positive integers/None, default:None-->
#            0
#         </pid>
#         <pstatus>
#            <!-- Patch transfer status -->
#            <!-- Possible values:None/transfer completed/transfer failed, default: None -->
#            transfer completed
#         </pstatus>
#         <perror>
#            <!-- Concise error message for individual patch transfer failure-->
#            Example:Target Unreachable
#         </perror>
#      </patchdetails>
#      <patchdetails>
#         ......
#      </patchdetails>
#      <patchdetails>
#         ......
#      </patchdetails>
#   </patches>
#   <overallerror>
#      <!-- This error tag will be the reason for overall patch transfer failure. If any one of the patch transfer fails then the overall error will be the same error as the error mentioned in the individual patch error tag. -->
#      Example:Target unreachable (or) Permission denied
#   </overallerror>
#   <overallstatus>
#      <!--Overall patch transfer status -->
#      <!-- Values:Started/Loaded/Failed/Completed/None, default:None -->
#      None
#   </overallstatus>
#</patch>

