###############################################################################
## @file base_semctl.pm
# @brief Linux Base critical sections management interface
# @details
# @author Kate Bereznyakova, Kate Pestova
# @date June 2008
# @version 1.00 - Initial version
# @version 1.01 - bugfix in destructor(incorrect appearance of error message)
# @version 1.02 - add $SEM_CRONTAB
# @version 1.10 - add the waiting semaphores support
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.12 - From now on the 'semaphores' database will be associated with
#                 'nortel' group instead of 'root'.
# @verions 1.13 - Modified the log message with which operation is blocked
# @version 1.20 - New semaphore 'SEM_ACCOUNTS' has been added
# @version 1.21 - New is_subsystem_active() procedure
# @version 1.22 - New semaphore 'SEM_FIREWALL' has been added
# @version 1.23 - New semaphore 'SEM_BIOSUPGRADE' has been added
# @note
package base_semctl;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use basedb;

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
  $SEM_BACKUP
  $SEM_PATCH
  $SEM_APPINSTALL
  $SEM_APPSTART
  $SEM_CRONTAB
  $SEM_HARD
  $SEM_ACCOUNTS
  IPC_NOWAIT
  $NOWAIT_FLAG
  $WAIT_FLAG
  $SEMS_DB
  $SBS_USERCONFIG
  $SBS_SYSRESTORE
  $SEM_FIREWALL
  $SBS_FIREWALL
  $SEM_BIOSUPGRADE
  $SBS_BIOSUPGRADE
  );

our $VERSION = '1.23';
our $SEMS_DB = "$PATH_DB/semaphores";

### Used modules
use IPC::SysV qw(IPC_RMID IPC_EXCL IPC_NOWAIT IPC_STAT IPC_CREAT IPC_PRIVATE S_IRWXU S_IRWXG S_IRWXO GETPID);

### Default flags
# S_IRWXU          /* Read, write, execute: owner */
# S_IRWXG          /* Read, write, execute: group */
# S_IRWXO          /* Read, write, execute: other */
# IPC_CREAT        /* Create entry if key doesn't exist. */
# IPC_EXCL         /* Fail if key exists. */
# IPC_NOWAIT       /* Error if request must wait. */
# IPC_STAT         /* Get resource options. */
# IPC_RMID         /* Remove resource. */

use IPC::Semaphore;
use baselog;
use warnings;
use strict;

#####################################################################
##################### Public Data Declaration #######################

our $SEM_BACKUP      = 'SEM_BACKUP';
our $SEM_APPINSTALL  = 'SEM_APPINSTALL';
our $SEM_APPSTART    = 'SEM_APPSTART';
our $SEM_CRONTAB     = 'SEM_CRONTAB';
our $SEM_HARD        = 'SEM_HARD';
our $SEM_PATCH       = 'SEM_PATCH';
our $SEM_ACCOUNTS    = 'SEM_ACCOUNTS';
our $SEM_FIREWALL    = 'SEM_FIREWALL';
our $SEM_BIOSUPGRADE = 'SEM_BIOSUPGRADE';
our $SBS_BIOSUPGRADE = 'SBS_BIOSUPGRADE';

our $SEM_NO_FLAG    = '0';
our $NOWAIT_FLAG    = IPC_NOWAIT;
our $WAIT_FLAG      = 0;
our $SBS_USERCONFIG = 'USERCONFIG';
our $SBS_SYSRESTORE = 'SYSRESTORE';
our $SBS_FIREWALL   = 'SBS_FIREWALL';

#####################################################################
##################### Public Methods Declaration ####################

sub new($);          # constructor
sub sems_create($);  # create all semaphores defined by %BASE_SEMS
sub sems_delete($);  # remove all semaphores defined by %BASE_SEMS
sub sem_take(@);
sub sem_give(@);
sub sem_test($);
sub sem_show();
sub sem_print_base_status($);
sub sem_get_stat($);
sub get_sem_attr($);
sub is_subsystem_active(@);

#####################################################################
##################### Private Data Declaration ######################
#internal semaphore to block all other semaphore operations until
#current the one is not finished.
my $SEM_BLOCK = 0;

my $SEMSET = 200700;  # unique id for the semaphores set

my %BASE_SEMS = (
                 $SEM_BACKUP      => 1,
                 $SEM_PATCH       => 2,
                 $SEM_APPINSTALL  => 3,
                 $SEM_APPSTART    => 4,
                 $SEM_CRONTAB     => 5,
                 $SEM_HARD        => 6,
                 $SEM_ACCOUNTS    => 7,
                 $SEM_FIREWALL    => 8,
                 $SEM_BIOSUPGRADE => 9
                );

my %sem_fields = (
                  'SEMID'     => '0',
                  'PID'       => 'n/a',
                  'SUBSYSTEM' => 'n/a',
                  'BUSY'      => '0'
                 );
my $NA = "n/a";

my $ZERO_OPERATION = 0;   # "wait-for-zero" operation: if semaphore value is zero,
                          # the operation can immediately proceed
my $INC_OPERATION  = 1;   # this operation adds "1" to the semaphore value
my $DEC_OPERATION  = -1;  # this operation decreases semaphore value by "1"

#####################################################################
##################### Private Methods Declaration ###################
sub take_block_sem();
sub release_block_sem();
sub semdb_lock($);
sub semdb_unlock($);
sub semdb_create();
sub semdb_init($);
sub sem_get_info(@);
sub sem_set_info(@);
sub hash_to_str($);
sub sems_create_all();
sub sems_delete_all();

sub is_semname_valid($);
sub get_array_from_ref($);
sub sem_get_val($);
sub sem_check_alive($);
sub is_sems_exists();
sub get_sem(@);
sub sem_up(@);
sub sem_down($);
sub get_new_sem($);
sub who_blocks_sem($$);

#####################################################################
########################### Private Methods #########################

#--------------------------------------------------------------------
# Self blocking code.
# All actions on the semaphores should be performed  as an atomic
# operation (to allow to work on the list of semaphores).
#--------------------------------------------------------------------

#####################################################################
## @fn sub take_block_sem()
# @brief   Takes 'SEM_BLOCK' semaphore in set
# @param   none
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub take_block_sem()
{
   my $retval = $TRUE;
   my $sem    = get_sem($SEMSET);
   if (!defined($sem))
   {
      $retval = $FALSE;
   }
   else
   {

      # $SEM_BLOCK is taken with $NOWAIT_FLAG, because this function is used
      # for semaphores table show
      $retval = $sem->op($SEM_BLOCK, $ZERO_OPERATION, $NOWAIT_FLAG, $SEM_BLOCK, $INC_OPERATION, $NOWAIT_FLAG);
   } ## end else [ if (!defined($sem))
   return $retval;
} ## end sub take_block_sem()

#####################################################################
## @fn sub release_block_sem()
# @brief   Releases 'SEM_BLOCK' semaphore in set
# @param   none
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub release_block_sem()
{
   my $retval = $TRUE;
   my $sem    = get_sem($SEMSET);
   if (!defined($sem))
   {
      $retval = $FALSE;
   }
   else
   {
      $retval = $sem->op($SEM_BLOCK, $DEC_OPERATION, $NOWAIT_FLAG);
   }
   return $retval;
} ## end sub release_block_sem()

#--------------------------------------------------------------------
# Semaphore DB management block.
# DB has the following structure: key => value
# where: key - semaphore name (key from %BASE_SEMS)
# value - packed string with associated attributes separated with "\t"
# Attributes defined by %sem_fields hash.
#--------------------------------------------------------------------
#####################################################################
## @fn sub semdb_lock()
# @brief   Locks semaphore database
# @param   $self - reference to the hash with data
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub semdb_lock($)
{
   my $self = shift @_;

   # Lock semaphores DB file and get the hash linked with this DB
   ($self->{'basedbDESC'}, $self->{'hash_db'}) = basedb_lock($SEMS_DB);
   if ((!$self->{'basedbDESC'}) || (!$self->{'hash_db'}))
   {

      #first message is problem indication for user.
      #second message - detail description for problem analysis.
      baselog($LOG_ERROR, "Problem with access to semaphore's data base.\n", $TRUE);
      baselog($LOG_ERROR, 'Cannot initialize DB hash!');
      return $FALSE;
   } ## end if ((!$self->{'basedbDESC'...
   return $TRUE;
} ## end sub semdb_lock($)

#####################################################################
## @fn sub semdb_unlock()
# @brief   Unlocks semaphore database
# @param   $self - reference to the hash with data
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub semdb_unlock($)
{
   my $self = shift @_;
   unless (basedb_unlock($self->{'basedbDESC'}, $self->{'hash_db'}))
   {

      #first message is problem indication for user.
      #second message - detail description for problem analysis.
      baselog($LOG_ERROR, "Problem with access to semaphore's data base.\n", $TRUE);
      baselog($LOG_ERROR, 'Cannot unlock DB file!');
      return $FALSE;
   } ## end unless (basedb_unlock($self...

   return $TRUE;
} ## end sub semdb_unlock($)

#####################################################################
## @fn sub semdb_create()
# @brief   Creates semaphore database
# @param   none
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub semdb_create()
{
   my $retval = $TRUE;
   if (!basedb_create($SEMS_DB, $NORTEL_UID, $NORTEL_GID, 0660))
   {
      $retval = $FALSE;
   }
   return $retval;
} ## end sub semdb_create()

#####################################################################
## @fn sub semdb_init()
# @brief   Creates semaphore database and initializes it
# with default data.
# @param   $self - reference to the hash with data
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub semdb_init($)
{
   my $self   = shift @_;
   my $retval = $self->semdb_lock();

   # get DB descriptor and hash with records.
   if ($retval)
   {
      my %dbrecord = %sem_fields;

      my $sem = get_sem($SEMSET);

      # for each supported semaphore create a record in DB
      foreach my $semname (keys(%BASE_SEMS))
      {

         if (defined $sem)
         {

            #get semaphore id in set
            $dbrecord{'SEMID'} = $BASE_SEMS{$semname};
            $self->{'hash_db'}{$semname} = hash_to_str(\%dbrecord);
         } ## end if (defined $sem)
         else
         {
            $retval = $FALSE;
         }
      } ## end foreach my $semname (keys(%BASE_SEMS...

      $retval &= $self->semdb_unlock();
   } ## end if ($retval)
   return $retval;
} ## end sub semdb_init($)

#####################################################################
## @fn sub sem_get_info()
# @brief   Retrieves data from semaphore database
# @param   $self - reference to the hash with data
# @param   $semname - semaphore name as defined in %BASE_SEMS
# @param   $dbrecord - reference to resulting hash
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_get_info(@)
{
   my ($self, $semname, $dbrecord) = @_;
   my $retval = $self->semdb_lock();
   if ($retval)
   {
      if ((not defined $semname) or (not exists $self->{'hash_db'}{$semname}))
      {
         baselog($LOG_ERROR, "There is no record for the semaphore in DB.");
         $retval = $FALSE;
      }
      else
      {
         %{$dbrecord} = eval($self->{'hash_db'}{$semname});
      }
      $retval &= $self->semdb_unlock();
   } ## end if ($retval)
   return $retval;
} ## end sub sem_get_info(@)

#####################################################################
## @fn sub sem_set_info()
# @brief   Updates semaphore database
# @param   $self - reference to the hash with data
# @param   $semname - semaphore name as defined in %BASE_SEMS
# @param   $res - reference to hash with data to update
# @return  TRUE - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_set_info(@)
{
   my ($self, $semname, $res) = @_;
   my %dbrecord;
   my $retval = $self->semdb_lock();
   if ($retval)
   {
      if (not exists $self->{'hash_db'}{$semname})
      {
         baselog($LOG_ERROR, "There is no record for $semname semaphore in DB.");
         $retval = $FALSE;
      }
      else
      {
         %dbrecord = eval($self->{'hash_db'}{$semname});

         # update record with new values
         foreach my $key (keys(%{$res}))
         {
            if (exists $dbrecord{$key})
            {
               $dbrecord{$key} = $res->{$key};
            }
            else
            {
               baselog($LOG_WARNING, "There is no attribute: $key");
               $retval = $FALSE;
            }
         } ## end foreach my $key (keys(%{$res...
         $self->{'hash_db'}{$semname} = hash_to_str(\%dbrecord);
      } ## end else [ if (not exists $self->...
      $retval &= $self->semdb_unlock();
      $retval &= $self->sem_get_info($semname, \%dbrecord);
   } ## end if ($retval)
   return $retval;
} ## end sub sem_set_info(@)

####################################################################
## @fn sub hash_to_str()
# @brief   Prepares a string describing a hash
# @param   $source - reference to the hash with data
# @return  $str - resulting string
#####################################################################
sub hash_to_str($)
{
   my $source = shift @_;
   my $str    = "(";
   foreach my $key (keys(%$source))
   {
      $str = $str . " $key => '$source->{$key}',";
   }
   $str = substr($str, 0, -1) . " )";
   return $str;
} ## end sub hash_to_str($)

#####################################################################
## @fn sub who_blocks_sem()
# @brief   Prints information about subsystem that blocks specified
#          semaphore
# @param   $self - reference to the hash with data
# @param   $semname - name of the semaphore
# @return  none
#####################################################################
sub who_blocks_sem($$)
{
   my ($self, $semname) = @_;
   if (not defined $semname)
   {
      baselog($LOG_ERROR, "Semaphore name is not defined.");
      return;
   }
   my %dbrecord;
   if (($self->sem_get_info($semname, \%dbrecord)) and ($dbrecord{'SUBSYSTEM'} ne $NA))
   {
      baselog($LOG_WARNING, "Operation is momentarily blocked by $dbrecord{'SUBSYSTEM'} subsystem.\nTry again later if action is still required.\n", $TRUE);
   }
   else
   {
      baselog($LOG_WARNING, "Operation is momentarily blocked by another subsystem.\nTry again later if operation is still action.\n", $TRUE);
   }
} ## end sub who_blocks_sem($$)

#--------------------------------------------------------------------
# General semaphore management functions
#--------------------------------------------------------------------
#####################################################################
## @fn sub get_sem()
# @brief   Creates a semaphore object, assosiated with existing semaphore
# @param   $semkey - key of the semaphore that should be used to initialize an object.
# @param   $silent - error output mode: if defined then print errors to the screen.
#          Otherwise put them to log file only.
# @return  $sem - to created object
# @return  undef - some error happens
#####################################################################
sub get_sem(@)
{
   my ($semkey, $silent) = @_;
   my $sem;

   # Parameters for "new":
   #       $semkey - key of the semaphore in the system table
   #       second parameter - number of semaphores in the set
   #       $SEM_NO_FLAG - If semaphore does not exist
   #                                      then do not try to create one and return undef.
   my $nsems = keys(%BASE_SEMS) + 1;
   $sem = IPC::Semaphore->new($semkey, $nsems, $SEM_NO_FLAG);
   if (not defined $sem)
   {
      my $error_descr = $!;
      baselog($LOG_ERROR, "Error description: $error_descr.");
      baselog($LOG_ERROR, "Cannot get information about semaphore. Semaphore does not exist.");

      if (not defined $silent)
      {
         print "Cannot get information about semaphore. Semaphore does not exist.\n";
      }

      return undef;
   } ## end if (not defined $sem)
   return $sem;
} ## end sub get_sem(@)

#####################################################################
## @fn sub get_new_sem()
# @brief   Creates a semaphore object assosiated with new semaphore
# @param   $semkey - key of the semaphore that should be used to initialize an object.
# @return  $sem - reference to created object
# @return  undef - some error happens
#####################################################################
sub get_new_sem($)
{
   my $semkey = shift @_;

   # Format: $semkey - key assosiated with the semaphore
   #         second parameter - number of the semaphores in the set
   #         S_IRWXU, S_IRWXG - access permitions to the semaphores for the owner and group
   #                                               set "read", "write" and "execute" permitions.
   #         IPC_CREAT - if semaphore with such name does not exist then create one.
   my $nsems = keys(%BASE_SEMS) + 1;
   my $sem = IPC::Semaphore->new($semkey, $nsems, S_IRWXU | S_IRWXG | IPC_CREAT);
   if (not defined $sem)
   {
      my $error_descr = $!;
      baselog($LOG_CRIT, "Cannot create semaphore ($error_descr).");
   }
   return $sem;
} ## end sub get_new_sem($)

#####################################################################
## @fn sub sem_up()
# @brief   Increases value of corresponding semaphores by one.
# @details The  set  of operations is performed atomically.
# semop command arguments: (int semid, struct sembuf *sops, unsigned nsops);
# The first is the semaphore number in the set, the second is the operation and
# the last is  flags value.
# 1) $nsem - number of semaphore in the set
# 2) $ZERO_OPERATION = 0 - "wait-for-zero" operation: if semaphore value is zero,
#                          the operation can immediately proceed
#    $INC_OPERATION = 1 - this operation adds "1" to the semaphore value
# 3) $flag - exit with error or wait "zero condition".
# @param   $semref - name of the semaphore or reference to list of semaphores.
# @param   $flag - exit with error or wait "zero condition"
# @param   $timeout - time for alarm
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_up(@)
{
   my ($semref, $flag, $timeout) = @_;
   my @sems = ();
   if (defined $semref)
   {
      @sems = get_array_from_ref($semref);
   }

   if ($flag eq $WAIT_FLAG)
   {
      if (not defined $timeout)
      {
         $timeout = 0;
      }
   } ## end if ($flag eq $WAIT_FLAG...
   else
   {
      $timeout = 0;
   }

   my @uplist = ($SEM_BLOCK, $ZERO_OPERATION, $WAIT_FLAG, $SEM_BLOCK, $INC_OPERATION, $WAIT_FLAG);

   foreach my $nsem (@sems)
   {
      push @uplist, ($BASE_SEMS{$nsem}, $ZERO_OPERATION, $flag, $BASE_SEMS{$nsem}, $INC_OPERATION, $flag);
   }
   my $sem = get_sem($SEMSET);

   my @error_descr;
   eval {
      local $SIG{ALRM} = sub {baselog($LOG_ERROR, "Semaphore was taken more then $timeout second."); die "alarm\n"};
      alarm $timeout;
      $sem->op(@uplist);
      if ($! ne "")
      {
         push(@error_descr, $!);
      }
      alarm 0;
   };
   if ($@ ne "")
   {
      push(@error_descr, $@);
   }
   if (scalar(@error_descr) > 0)
   {
      baselog($LOG_ERROR, "Cannot take semaphore.");
      foreach my $descr (@error_descr)
      {
         baselog($LOG_ERROR, "Error description: $descr.");
      }
      return $FALSE;
   } ## end if (scalar(@error_descr...

   return $TRUE;
} ## end sub sem_up(@)

#####################################################################
## @fn sub sem_down()
# @brief   Decreases value of corresponding semaphores by one.
# @details The  set  of operations is performed atomically.
# semop command arguments: (int semid, struct sembuf *sops, unsigned nsops);
# The first is the semaphore number in the set, the second is the operation and
# the last is  flags value.
# 1) $nsem - number of semaphore in the set
# 2) $ZERO_OPERATION = 0 - "wait-for-zero" operation: if semaphore value is zero,
#                          the operation can immediately proceed
#    $INC_OPERATION = 1 - this operation adds "1" to the semaphore value
#    $DEC_OPERATION = -1 - this operation decreases semaphore value by "1"
# 3) $flag - exit with error or wait "zero condition".
# @param   $semref - name of the semaphore or reference to list of semaphores.
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_down($)
{
   my $semref = shift @_;
   my @sems   = ();
   if (defined $semref)
   {
      @sems = get_array_from_ref($semref);
   }
   my @uplist = ($SEM_BLOCK, $ZERO_OPERATION, $WAIT_FLAG, $SEM_BLOCK, $INC_OPERATION, $WAIT_FLAG);

   foreach my $nsem (@sems)
   {
      push @uplist, ($BASE_SEMS{$nsem}, $DEC_OPERATION, $NOWAIT_FLAG);
   }
   my $sem    = get_sem($SEMSET);
   my $result = $sem->op(@uplist);

   my $error_descr = $!;
   if ($error_descr ne "")
   {
      baselog($LOG_ERROR, "Cannot release semaphore.\n", $TRUE);
      baselog($LOG_ERROR, "Command returned $result, Error description: $error_descr.");
      return $FALSE;
   } ## end if ($error_descr ne ""...

   return $TRUE;
} ## end sub sem_down($)

#####################################################################
## @fn sub sems_create_all()
# @brief   Creates default semaphores defined by %BASE_SEMS
# @param   none
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sems_create_all()
{
   my $retval = $TRUE;
   my $sem    = get_new_sem($SEMSET);
   if (defined $sem)
   {

      # change access permissions for nortel and root accounts.
      $sem->set(uid => $ROOT_UID, gid => $NORTEL_UID);
      my $uid = get_sem_attr("uid");
      my $gid = get_sem_attr("gid");
      if (   (not defined $uid)
          or ($uid ne $ROOT_UID)
          or (not defined $gid)
          or ($gid ne $NORTEL_UID))
      {
         baselog($LOG_ERROR, "Cannot set semaphore permissions.");
         $retval = $FALSE;
      } ## end if ((not defined $uid)...
   } ## end if (defined $sem)
   else
   {
      $retval = $FALSE;
   }
   return $retval;
} ## end sub sems_create_all()

#####################################################################
## @fn sub sems_delete_all()
# @brief   Deletes default semaphores defined by %BASE_SEMS
# @param   none
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sems_delete_all()
{
   my $retval = $TRUE;

   my $sem = get_sem($SEMSET);
   if (!defined($sem))
   {
      $retval = $FALSE;
   }
   else
   {
      $sem->remove();
   }

   $retval &= basedb_remove($SEMS_DB);
   return $retval;
} ## end sub sems_delete_all()

#####################################################################
## @fn sub DESTROY()
# @brief   Object destructor.
# @param   none
# @return  none
#####################################################################
sub DESTROY
{
   my $self = shift @_;

   # enforce release all taken semaphores.
   my @taken_semaphores = keys(%{$self->{'taken'}});
   $self->sem_give(\@taken_semaphores) unless (scalar(@taken_semaphores) == 0);
   return;
} ## end sub DESTROY

#--------------------------------------------------------------------
# Misc functions
#--------------------------------------------------------------------

#####################################################################
## @fn sub is_semname_valid()
# @brief   Checks if semaphore with defined name is in the list
#          of supported semaphores defined by %BASE_SEMS.
# @param   $semname - semaphore's name
# @return  TRUE  - semaphore supported
# @return  FALSE - semaphore does not supported
#####################################################################
sub is_semname_valid($)
{
   my $semname = shift @_;
   my $retval  = $FALSE;
   if (defined $semname)
   {
      if (!exists $BASE_SEMS{$semname})
      {
         baselog($LOG_ERROR, "Invalid name. Semaphore with $semname is not supported.\n", $TRUE);
      }
      else
      {
         $retval = $TRUE;
      }
   } ## end if (defined $semname)
   else
   {
      baselog($LOG_ERROR, "Semaphore name is not defined.\n", $TRUE);
   }
   return $retval;
} ## end sub is_semname_valid($)

#####################################################################
## @fn sub get_array_from_ref()
# @brief   Gets array from reference
# @param   $arref - reference to list or single variable
# @return  @array - array with single element or with a set of elements
# @return  undef - some error happens
#####################################################################
sub get_array_from_ref($)
{
   my $arref = shift @_;
   my @array;
   if (is_array($arref))
   {
      push @array, @$arref;
   }
   else
   {
      push @array, $arref;
   }
   return @array;
} ## end sub get_array_from_ref($)

#####################################################################
## @fn sub sem_get_val()
# @brief   Returns current value of the semaphore
# @param   $semname - name of semaphore
# @return  digit - value of the semaphore
# @return  undef - some error happens
#####################################################################
sub sem_get_val($)
{
   my $semname = shift @_;
   my $retval  = undef;
   if (defined $semname)
   {
      my $sem = get_sem($SEMSET);
      if (defined $sem)
      {

         #get value of the semaphore in the set
         $retval = $sem->getval($BASE_SEMS{$semname});
      } ## end if (defined $sem)
   } ## end if (defined $semname)
   else
   {
      baselog($LOG_ERROR, "Undefined semaphore name ($semname).");
   }
   return $retval;
} ## end sub sem_get_val($)

#####################################################################
## @fn sub sem_check_alive()
# @brief   Checks if process blocked semaphore is alive
# @details If ex-owner of the semaphore is dead then release semaphore
# @param   $semname - name of semaphore
# @return  TRUE - process is alive
# @return  FALSE - process is dead or error occurred
#####################################################################
sub sem_check_alive($)
{
   my $semname = shift @_;

   my $sem = get_sem($SEMSET);
   if (not defined $sem)
   {
      return $FALSE;
   }

   #get pid of the process that changed semaphore.
   my $sem_owner = $sem->getpid($BASE_SEMS{$semname});

   # $PS keys: select only process with specified pid and do not print title
   if (`$PS h --pid $sem_owner | $WC -l` > 0)
   {
      return $TRUE;
   }

   return $FALSE;
} ## end sub sem_check_alive($)

#####################################################################
## @fn sub is_sems_exists()
# @brief   Checks semaphore set exists
# @param   none
# @return  TRUE - exists
# @return  FALSE - does not exists
#####################################################################
sub is_sems_exists()
{
   my $retval = $TRUE;

   # only owner and group have an access to the semaphore
   my $sem = get_sem($SEMSET);
   if (not defined $sem)
   {
      $retval = $FALSE;
   }
   return $retval;
} ## end sub is_sems_exists()

#####################################################################
########################### Public Methods ##########################

#####################################################################
## @fn sub new()
# @brief   Object constructor. Object is hash.
# @param   $subsystem - name of subsystem
# @return  $self - reference to the hash with data
#####################################################################
sub new($)
{

   #initialization
   my $subsystem = shift @_;
   my %self = (
               'subsystem'  => $subsystem,  # subsystem: appinstall, patcher, etc
               'pid'        => "$$",        # pid of the process
               'hash_db'    => undef,       # data base hash
               'basedbDESC' => undef,       # db file descriptor
               'taken'      => {}           # list of semaphores taken by the object
              );

   bless \%self;                   #Declaring an object
   return \%self;

} ## end sub new($)

#####################################################################
## @fn sub sems_create()
# @brief   Creates semaphores set
# @param   $self - reference to the hash with data
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sems_create($)
{
   my $self   = shift @_;
   my $retval = sems_create_all();
   if (not $retval)
   {
      baselog($LOG_ERROR, "Semaphores creation failed.\n", $TRUE);
      return $retval;
   }

   if (semdb_create())
   {
      $retval = $self->semdb_init();
   }

   return $retval;
} ## end sub sems_create($)

#####################################################################
## @fn sub sems_delete()
# @brief   Removes semaphores set
# @param   $self - reference to the hash with data
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sems_delete($)
{
   my $self = shift @_;
   return sems_delete_all();
}

#####################################################################
## @fn sub sem_take()
# @brief   Gets semaphore(s) associated with the key(s) passed as parameter
# @param   $self - reference to the hash with data
# @param   $semref - name of semaphore or reference to list of
#          semaphores.
# @param   $flag  - one of supported flags: $NOWAIT_FLAG or $WAIT_FLAG
# @param   $timeout - time for semaphore taking
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_take(@)
{
   my ($self, $semref, $flag, $timeout) = @_;
   my @needlock = ();
   if (defined $semref)
   {
      @needlock = get_array_from_ref($semref);
      if (scalar(@needlock) < 1)
      {
         baselog($LOG_ERROR, "Nothing to take. Semaphore list is empty.");
         baselog($LOG_DEBUG, "Input parameter - $semref.");
         return $FALSE;
      } ## end if (scalar(@needlock) ...
   } ## end if (defined $semref)
   my $sem = get_sem($SEMSET, $TRUE);
   if (not defined $sem)
   {
      return $FALSE;
   }

   # if semaphore cannot be taken then check who blocked it.
   if (not sem_up($semref, $flag, $timeout))
   {
      foreach my $semname (@needlock)
      {
         if (sem_check_alive($semname))
         {

            # as semaphore is blocked by the other subsystem
            # then print information who block it, release taken semaphores
            # and exit with error.

            $self->who_blocks_sem($semname);
            return $FALSE;
         } ## end if (sem_check_alive($semname...
         else
         {
            $self->sem_give($semname);

            # updating semaphores DB
            my %dbrecord;
            $dbrecord{'PID'}       = "n/a";
            $dbrecord{'SUBSYSTEM'} = "n/a";
            $dbrecord{'BUSY'}      = '0';
            $self->sem_set_info($semname, \%dbrecord);
         } ## end else [ if (sem_check_alive($semname...
      } ## end foreach my $semname (@needlock...
      return $FALSE;
   } ## end if (not sem_up($semref...
   else
   {
      foreach my $semname (@needlock)
      {
         baselog($LOG_DEBUG, "$semname semaphore has been taken.");

         # updating semaphores DB
         my %dbrecord;
         if ($self->sem_get_info($semname, \%dbrecord))
         {
            $dbrecord{'PID'}       = $self->{'pid'};
            $dbrecord{'SUBSYSTEM'} = $self->{'subsystem'};
            $dbrecord{'BUSY'}      = '1';
            $self->sem_set_info($semname, \%dbrecord);
         } ## end if ($self->sem_get_info...

         #Adding semaphore to the semaphore list blocked by the object
         if (!exists $self->{'taken'}->{$semname})
         {
            $self->{'taken'}->{$semname} = "1";
         }
         else
         {
            baselog($LOG_WARNING, "Semaphore $semname already in semlist of the object.");
         }
      } ## end foreach my $semname (@needlock...
      release_block_sem();
   } ## end else [ if (not sem_up($semref...
   return $TRUE;
} ## end sub sem_take(@)

#####################################################################
## @fn sub sem_give()
# @brief   Releases semaphore(s) associated with the key(s) passed as parameter
# @param   $self - reference to the hash with data
# @param   $semref - name of semaphore or reference to list of
#          semaphores.
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_give(@)
{
   my ($self, $semref) = @_;
   my $retval   = $TRUE;
   my @needfree = ();
   if (defined $semref)
   {
      @needfree = get_array_from_ref($semref);
      if (scalar(@needfree) < 1)
      {
         baselog($LOG_ERROR, "Nothing to release. Semaphore list is empty.");
         baselog($LOG_DEBUG, "Input parameter - $semref.");
         return $FALSE;
      } ## end if (scalar(@needfree) ...
   } ## end if (defined $semref)

   my $sem = get_sem($SEMSET);
   if (not defined $sem)
   {
      return $FALSE;
   }
   foreach my $semname (@needfree)
   {
      if (sem_get_val($semname) eq "0")
      {
         baselog($LOG_WARNING, "Semaphore $semname is free.");
      }
      elsif (sem_down($semname))
      {

         # updating semaphores DB
         my %dbrecord;
         $dbrecord{'PID'}       = "n/a";
         $dbrecord{'SUBSYSTEM'} = "n/a";
         $dbrecord{'BUSY'}      = '0';
         $self->sem_set_info($semname, \%dbrecord);

         # Removing semaphore from the object list of the blocked semaphores
         if (defined $self->{'taken'}->{$semname})
         {
            delete $self->{'taken'}->{$semname};
         }
         else
         {
            baselog($LOG_ERROR, "Semaphore is not in the object's list.");
         }
         baselog($LOG_DEBUG, "Semaphore $semname released.");
         release_block_sem();
      } ## end elsif (sem_down($semname)...
      else
      {
         return $FALSE;
      }
   } ## end foreach my $semname (@needfree...
   return $TRUE;
} ## end sub sem_give(@)

#####################################################################
## @fn sub sem_test()
# @brief   Checks if semaphore is busy
# @param   $semname - name of semaphore
# @return  TRUE  - semaphore is free (not busy)
# @return  FALSE - semaphore is busy
# @return  undef - some error happened
#####################################################################
sub sem_test($)
{
   my $semname = shift @_;
   my $semval  = sem_get_val($semname);

   if (!defined $semval)
   {
      return undef;
   }

   if ($semval)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub sem_test($)

#####################################################################
## @fn sub is_subsystem_active()
# @brief   Checks if the given subsystem locks now some semaphores
# @param   $subsysname - name of subsystem
# @param   @semnames   - optional names of semaphores to check for
# @return  TRUE  - this subsystem doesn't lock any semaphores
# @return  FALSE - this subsystem locking some semaphores now
#####################################################################
sub is_subsystem_active(@)
{
   my ($subsysname, @semnames) = @_;
   my $self      = new('SEMCTL');
   my %semstatus = %sem_fields;
   my $retval    = $FALSE;

   @semnames = keys(%BASE_SEMS) unless @semnames;
   foreach my $semname (@semnames)
   {
      if (!sem_test($semname) and sem_check_alive($semname))
      {
         if ($self->sem_get_info($semname, \%semstatus) == $TRUE)
         {
            $retval = $TRUE if ($semstatus{'SUBSYSTEM'} eq $subsysname);
         }
      } ## end if (!sem_test($semname...
   } ## end foreach my $semname (@semnames...

   undef $self;
   return $retval;
} ## end sub is_subsystem_active(@)

#####################################################################
## @fn sub sem_show()
# @brief   Prints information about semaphore set
# @param   none
# @return  TRUE  - operation successful
# @return  FALSE - some error happens
#####################################################################
sub sem_show()
{
   my $retval = $FALSE;
   my %res = (
              uid   => '',
              gid   => '',
              cuid  => '',
              cgid  => '',
              mode  => '',
              ctime => '',
              otime => '',
              nsems => ''
             );
   if (sem_get_stat(\%res))
   {
      foreach my $key (keys %res)
      {
         print "  - $key: $res{$key}\n";
      }
      $retval = $TRUE;
   } ## end if (sem_get_stat(\%res...
   return $retval;
} ## end sub sem_show()

#####################################################################
## @fn sub sem_print_base_status()
# @brief   Prints current  status of all Linux Base semaphores
# @param   $self - reference to the hash with data
# @return  none
#####################################################################
sub sem_print_base_status($)
{
   my $self = shift @_;
   if (not is_sems_exists())
   {
      print "Semaphore set is not configured.\n";
      return;
   }

   if (take_block_sem())
   {
      my $strformat = "|  %-15s|  %-9s|  %-9s|  %-9s|  %-20s  |\n";
      print "\nLinux Base semaphores status: \n";
      print "-" x 80;
      print "\n";
      printf($strformat, "SEMNAME", "SEMID", "STATUS", "PID", "SUBSYSTEM");
      print "-" x 80;
      print "\n";

      foreach my $semname (keys(%BASE_SEMS))
      {

         # SEMNAME  SEMID BUSY  PID  SUBSYSTEM
         my %semstatus = %sem_fields;
         if ($self->sem_get_info($semname, \%semstatus) == $TRUE)
         {
            my $semvalue = sem_get_val($semname);
            my $busy = ($semstatus{'BUSY'} ? "busy" : "free") . " ($semvalue)";
            printf($strformat, $semname, $semstatus{'SEMID'}, $busy, $semstatus{'PID'}, $semstatus{'SUBSYSTEM'});
         } ## end if ($self->sem_get_info...
         else
         {
            printf($strformat, $semname, "n/a", "n/a", "n/a", "n/a");
         }
      } ## end foreach my $semname (keys(%BASE_SEMS...
      print "-" x 80;
      print "\n";
      release_block_sem();
   } ## end if (take_block_sem())
   else
   {
      print "Internal semaphore is busy. Try later.";
   }
} ## end sub sem_print_base_status($)

#####################################################################
## @fn sub get_sem_attr()
# @brief   Gets one single attribute of semaphore
# @param   $semattr - attribute to retrieve
# @return  $res{$semattr} - value of requested attribute
# @return  undef   - some error happened
#####################################################################
sub get_sem_attr($)
{
   my $semattr = shift @_;
   my %res;

   # Create hash for request
   $res{$semattr} = "";

   if (sem_get_stat(\%res))
   {
      return $res{$semattr};
   }
   else
   {
      return undef;
   }
} ## end sub get_sem_attr($)

#####################################################################
## @fn sub sem_get_stat()
# @brief   Returns output of stat() function
# @param   $res - reference to resulting hash
# @return  TRUE - resulting hash is filled
# @return  FALSE - some error happened
#####################################################################
sub sem_get_stat($)
{
   my $res = shift @_;

   unless (defined $res)
   {
      baselog($LOG_ERROR, 'Undefined reference to resulting hash.');
      return $FALSE;
   }

   my $sem = get_sem($SEMSET, $TRUE);
   if (not defined $sem)
   {
      return $FALSE;
   }

   my $semstat = $sem->stat();
   foreach my $key (keys(%{$res}))
   {
      $res->{$key} = $semstat->$key;
   }
   return $TRUE;
} ## end sub sem_get_stat($)

1;

__END__

=head1 NAME

base_semctl - Linux Base critical section manager object class

=head1 SYNOPSIS

    use base_semctl;
    
    my $busy = base_semctl::sem_test(semname);

    my $sem = base_semctl::new(<SUBSYSTEM>);

    $sem->sems_create();

    $sem->sem_take(semname/semlist,flag);

    $sem->sem_give(semname/semlist, flag);

    $sem->sem_show(semname);
    $sem->sem_show();

    $attr_value = $sem->get_sem_attr(semid, attrname);

    $sem->sem_get_stat(semid, resulthash);

    $sem->sem_print_base_status();


=head1 DESCRIPTION

A class providing an object based interface to SysV IPC semaphores.
Class manages a set of predefined semaphores to protect critical
operations.

=head1 METHODS

=over 4

=item new ( "base_semctl" , SUBSYSTEM )

Create a new object - semaphore manager associated with C<SUBSYSTEM>. C<SUBSYSTEM> is the
user-friendly string describing subsystem that operates on Linux Base semaphores.

=over 4

=item *

C<SUBSYSTEM> could be "PATCHING", "BACKUP", "APPINSTALL" and etc.

=back

=item sems_create

Creates default semaphores defined by %SEM_BASE_LIST list.
The only users that have an access to created semaphores are "root" and "norel";
"root" is the owner
"nortel" is the group that has a full access to semaphore operations.

=item sem_take(semref, flag)

Gets a semaphore(s) defined by semref. Semref could be a semaphore name of a
reference to a list with set of semaphores that should be blocked.
flag could be set to NO_WAIT.

=item sem_give(semref)

Releases semaphores defined by semref. Semref could be a semaphore name of a
reference to a list with set of semaphores that should be released.

=item sem_test(semname)

Returns status of the semaphore: 1 - free, 0 - busy.

=item sem_show

Prints information about semaphore.
If semaphore name is passed as a parameter then only information about
specified semaphore is printed.

Output format is the following:
*** SEM_NAME:
  - key1: value
  - key2: value
  - key3: value

Supported keys are:
  uid
  gid
  cuid
  cgid
  mode
  ctime
  otime
  nsems

=item get_sem_attr

=item sem_get_stat

=item sem_print_base_status

Prints current  status of all Linux Base semaphores.
Print out is the table with the following format:

SEM_NAME   SEMID   BUSY   PID   SUBSYSTEM

Where:
   SEM_NAME    - semaphore name in user friendly form as defined
                 key in %SEM_BASE_LIST
   SEMID       - semaphore ID as represented by kernel
   BUSY        - semaphore status (busy or free)
   PID         - pid of the process that took the semaphore.
                 If the semaphore is free then this field has
                 "not set" status.
   SUBSYSTEM   - subsystem that took the semaphore.
                 If the semaphore is free then this field has
                 "not set" status.


=head1 AUTHOR

Kate Bereznyakova
