## @file base_snmp.pm
# @brief   This module provides interfaces for SNMP traps sending
# @details This module provides a database of SNMP traps which used by CS1000
#          Linux Base, and interfaces for SNMP traps sending
# @author  Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - added OSM0018 - OSM0029 messages for cpu/memory/disk by Sergey Ivin
# @version 1.02 - Two colons were removed before shared functions and variables.
# @version 1.03 - Modified under the perlcheck warnings cleanup activity.
# @version 1.04 - added OSM0030, OSM0031 messages
# @version 1.05 - added SEC0109 - SEC0110 messages
# @version 1.06 - added OSM0032 - OSM0033 messages
# @note
package base_snmp;
use lib "/opt/nortel/base/lib";

use strict;
use warnings;
use common_variables;
use base_system;
use baselog;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(%SNMP_MSGS
  $SNMP_SEV_CRITICAL
  $SNMP_SEV_MAJOR
  $SNMP_SEV_MINOR
  $SNMP_SEV_WARNING
  $SNMP_SEV_INFO
  $SNMP_SEV_INDETERMINATE
  $SNMP_SEV_CLEARED
  $SNMP_TYP_COMM
  $SNMP_TYP_QOS
  $SNMP_TYP_PROC
  $SNMP_TYP_EQUIP
  $SNMP_TYP_SEC
  $SNMP_TYP_OPER
  $SNMP_TYP_DEBUG
  $SNMP_TYP_UNKNOWN
  $SNMP_COMP_MGMT
  $SNMP_COMP_SS
  $SNMP_COMP_CS
  $SNMP_CAUSE_LOSS_OF_SIGNAL
  $SNMP_CAUSE_LOSS_OF_FRAME
  $SNMP_CAUSE_FRAMING_ERROR
  $SNMP_CAUSE_LOCAL_TRANSMISSION_ERROR
  $SNMP_CAUSE_REMOTE_TRANSMISSION_ERROR
  $SNMP_CAUSE_CALL_ESTABLISHMENT_ERROR
  $SNMP_CAUSE_DEGRADED_SIGNAL
  $SNMP_CAUSE_COMM_SUBSYSTEM_FAILURE
  $SNMP_CAUSE_COMM_PROTOCOL_ERROR
  $SNMP_CAUSE_LAN_ERROR
  $SNMP_CAUSE_DTE_DCE_INTERFACE_ERROR
  $SNMP_CAUSE_RESPONSE_TIME_EXCESSIVE
  $SNMP_CAUSE_QUEUE_SIZE_EXCEEDED
  $SNMP_CAUSE_BANDWIDTH_REDUCED
  $SNMP_CAUSE_RETRANSMISSION_RATE_REDUCED
  $SNMP_CAUSE_THRESHOLD_CROSSED
  $SNMP_CAUSE_PERFORMANCE_DEGRADED
  $SNMP_CAUSE_CONGESTION
  $SNMP_CAUSE_AT_OR_NEAR_CAPACITY
  $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM
  $SNMP_CAUSE_VERSION_MISMATCH
  $SNMP_CAUSE_CORRUPT_DATA
  $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED
  $SNMP_CAUSE_SOFTWARE_ERROR
  $SNMP_CAUSE_SOFTWARE_PROGRAM_ERROR
  $SNMP_CAUSE_SOFTWARE_PROGRAM_TERMINATION
  $SNMP_CAUSE_FILE_ERROR
  $SNMP_CAUSE_OUT_OF_MEMORY
  $SNMP_CAUSE_UNDERLYING_RESOURCE_UNAVAIL
  $SNMP_CAUSE_APPLICATION_SUBSYSTEM_FAILURE
  $SNMP_CAUSE_CONFIGURATION_ERROR
  $SNMP_CAUSE_POWER_PROBLEM
  $SNMP_CAUSE_TIMING_PROBLEM
  $SNMP_CAUSE_PROCESSOR_PROBLEM
  $SNMP_CAUSE_DATASET_MODEM_ERROR
  $SNMP_CAUSE_MULTIPLEXOR_PROBLEM
  $SNMP_CAUSE_RECEIVER_FAILURE
  $SNMP_CAUSE_TRANSMITTER_FAILURE
  $SNMP_CAUSE_OUTPUT_DEVICE_ERROR
  $SNMP_CAUSE_INPUT_DEVICE_ERROR
  $SNMP_CAUSE_IO_DEVICE_ERROR
  $SNMP_CAUSE_EQUIPMENT_FAILURE
  $SNMP_CAUSE_ADAPTER_ERROR
  $SNMP_CAUSE_DUPLICATE_INFO
  $SNMP_CAUSE_INFO_MISSING
  $SNMP_CAUSE_INFO_MODIFICATION
  $SNMP_CAUSE_INFO_OUT_OF_SEQUENCE
  $SNMP_CAUSE_UNEXPECTED_INFO
  $SNMP_CAUSE_DENIAL_OF_SERVICE
  $SNMP_CAUSE_OUT_OF_SERVICE
  $SNMP_CAUSE_PROCEDURAL_ERROR
  $SNMP_CAUSE_OTHER_OPERATIONAL
  $SNMP_CAUSE_CABLE_TAMPER
  $SNMP_CAUSE_INTRUSION_DETECTION
  $SNMP_CAUSE_OTHER_PHYSICAL
  $SNMP_CAUSE_AUTHENTICATION_FAILURE
  $SNMP_CAUSE_BREACH_OF_CONFIDENCE
  $SNMP_CAUSE_NON_REPUDIATION_FAILURE
  $SNMP_CAUSE_UNAUTHORIZED_MAX_ACCESS
  $SNMP_CAUSE_OTHER_SECURITY_SERVICE
  $SNMP_CAUSE_DELAYED_INFO
  $SNMP_CAUSE_KEY_EXPIRED
  $SNMP_CAUSE_OUT_OF_HOURS_ACTIVITY
  $SNMP_CAUSE_OPERATIONAL_CONDITION
  $SNMP_CAUSE_DEBUGGING
  $SNMP_CAUSE_UNKNOWN
  $SNMP_CAUSE_INACTIVE_VIRTUAL_CIRCUIT
  $SNMP_CAUSE_NETWORK_SERVER_INTERVENTION
  &send_snmp_trap
  &send_snmp_trap_raw
  );
our $VERSION = '1.06';

our $SNMP_COMP_MGMT = 'MGMT';
our $SNMP_COMP_SS   = 'SS';
our $SNMP_COMP_CS   = 'CS';

our $SNMP_SEV_CRITICAL      = 1;
our $SNMP_SEV_MAJOR         = 2;
our $SNMP_SEV_MINOR         = 3;
our $SNMP_SEV_WARNING       = 4;
our $SNMP_SEV_INFO          = 5;
our $SNMP_SEV_INDETERMINATE = 6;
our $SNMP_SEV_CLEARED       = 7;

my $SNMP_SEV_MIN = $SNMP_SEV_CRITICAL;
my $SNMP_SEV_MAX = $SNMP_SEV_CLEARED;

our $SNMP_TYP_COMM    = 1;
our $SNMP_TYP_QOS     = 2;
our $SNMP_TYP_PROC    = 3;
our $SNMP_TYP_EQUIP   = 4;
our $SNMP_TYP_SEC     = 5;
our $SNMP_TYP_OPER    = 6;
our $SNMP_TYP_DEBUG   = 7;
our $SNMP_TYP_UNKNOWN = 8;

my $SNMP_TYP_MIN = $SNMP_TYP_COMM;
my $SNMP_TYP_MAX = $SNMP_TYP_UNKNOWN;

# these constant are got from the common trap MIB
# if you are going to change anything in this area please contact to SNMP team
# TODO: it make sense to export these (and other constants) from common trap
#       MIB on the loadbuild time
our $SNMP_CAUSE_LOSS_OF_SIGNAL                = 0;
our $SNMP_CAUSE_LOSS_OF_FRAME                 = 1;
our $SNMP_CAUSE_FRAMING_ERROR                 = 2;
our $SNMP_CAUSE_LOCAL_TRANSMISSION_ERROR      = 3;
our $SNMP_CAUSE_REMOTE_TRANSMISSION_ERROR     = 4;
our $SNMP_CAUSE_CALL_ESTABLISHMENT_ERROR      = 5;
our $SNMP_CAUSE_DEGRADED_SIGNAL               = 6;
our $SNMP_CAUSE_COMM_SUBSYSTEM_FAILURE        = 7;
our $SNMP_CAUSE_COMM_PROTOCOL_ERROR           = 8;
our $SNMP_CAUSE_LAN_ERROR                     = 9;
our $SNMP_CAUSE_DTE_DCE_INTERFACE_ERROR       = 10;
our $SNMP_CAUSE_RESPONSE_TIME_EXCESSIVE       = 20;
our $SNMP_CAUSE_QUEUE_SIZE_EXCEEDED           = 21;
our $SNMP_CAUSE_BANDWIDTH_REDUCED             = 22;
our $SNMP_CAUSE_RETRANSMISSION_RATE_REDUCED   = 23;
our $SNMP_CAUSE_THRESHOLD_CROSSED             = 24;
our $SNMP_CAUSE_PERFORMANCE_DEGRADED          = 25;
our $SNMP_CAUSE_CONGESTION                    = 26;
our $SNMP_CAUSE_AT_OR_NEAR_CAPACITY           = 27;
our $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM      = 40;
our $SNMP_CAUSE_VERSION_MISMATCH              = 41;
our $SNMP_CAUSE_CORRUPT_DATA                  = 42;
our $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED     = 43;
our $SNMP_CAUSE_SOFTWARE_ERROR                = 44;
our $SNMP_CAUSE_SOFTWARE_PROGRAM_ERROR        = 45;
our $SNMP_CAUSE_SOFTWARE_PROGRAM_TERMINATION  = 46;
our $SNMP_CAUSE_FILE_ERROR                    = 47;
our $SNMP_CAUSE_OUT_OF_MEMORY                 = 48;
our $SNMP_CAUSE_UNDERLYING_RESOURCE_UNAVAIL   = 49;
our $SNMP_CAUSE_APPLICATION_SUBSYSTEM_FAILURE = 50;
our $SNMP_CAUSE_CONFIGURATION_ERROR           = 51;
our $SNMP_CAUSE_POWER_PROBLEM                 = 60;
our $SNMP_CAUSE_TIMING_PROBLEM                = 61;
our $SNMP_CAUSE_PROCESSOR_PROBLEM             = 62;
our $SNMP_CAUSE_DATASET_MODEM_ERROR           = 63;
our $SNMP_CAUSE_MULTIPLEXOR_PROBLEM           = 64;
our $SNMP_CAUSE_RECEIVER_FAILURE              = 65;
our $SNMP_CAUSE_TRANSMITTER_FAILURE           = 66;
our $SNMP_CAUSE_OUTPUT_DEVICE_ERROR           = 67;
our $SNMP_CAUSE_INPUT_DEVICE_ERROR            = 68;
our $SNMP_CAUSE_IO_DEVICE_ERROR               = 69;
our $SNMP_CAUSE_EQUIPMENT_FAILURE             = 70;
our $SNMP_CAUSE_ADAPTER_ERROR                 = 71;
our $SNMP_CAUSE_DUPLICATE_INFO                = 80;
our $SNMP_CAUSE_INFO_MISSING                  = 81;
our $SNMP_CAUSE_INFO_MODIFICATION             = 82;
our $SNMP_CAUSE_INFO_OUT_OF_SEQUENCE          = 83;
our $SNMP_CAUSE_UNEXPECTED_INFO               = 84;
our $SNMP_CAUSE_DENIAL_OF_SERVICE             = 90;
our $SNMP_CAUSE_OUT_OF_SERVICE                = 91;
our $SNMP_CAUSE_PROCEDURAL_ERROR              = 92;
our $SNMP_CAUSE_OTHER_OPERATIONAL             = 93;
our $SNMP_CAUSE_CABLE_TAMPER                  = 100;
our $SNMP_CAUSE_INTRUSION_DETECTION           = 101;
our $SNMP_CAUSE_OTHER_PHYSICAL                = 102;
our $SNMP_CAUSE_AUTHENTICATION_FAILURE        = 110;
our $SNMP_CAUSE_BREACH_OF_CONFIDENCE          = 111;
our $SNMP_CAUSE_NON_REPUDIATION_FAILURE       = 112;
our $SNMP_CAUSE_UNAUTHORIZED_MAX_ACCESS       = 113;
our $SNMP_CAUSE_OTHER_SECURITY_SERVICE        = 114;
our $SNMP_CAUSE_DELAYED_INFO                  = 120;
our $SNMP_CAUSE_KEY_EXPIRED                   = 121;
our $SNMP_CAUSE_OUT_OF_HOURS_ACTIVITY         = 122;
our $SNMP_CAUSE_OPERATIONAL_CONDITION         = 200;
our $SNMP_CAUSE_DEBUGGING                     = 201;
our $SNMP_CAUSE_UNKNOWN                       = 202;
our $SNMP_CAUSE_INACTIVE_VIRTUAL_CIRCUIT      = 203;
our $SNMP_CAUSE_NETWORK_SERVER_INTERVENTION   = 204;

# database of SNMP messages which used in CS1000 Linux Base code
our %SNMP_MSGS = (
   "PCH0110" => {
                 'msg'        => 'PATCH %patchID% ACTIVATED. Name=%MPLR%, PRS#=%CR%, File=%filename%',
                 'severity'   => $SNMP_SEV_MINOR,
                 'alarm_type' => $SNMP_TYP_OPER,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OPERATIONAL_CONDITION
                },
   "PCH0111" => {
                 'msg'        => 'PATCH %patchID% DEACTIVATED. Name=%MPLR%, PRS#=%CR%, File=%filename%',
                 'severity'   => $SNMP_SEV_MINOR,
                 'alarm_type' => $SNMP_TYP_OPER,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OPERATIONAL_CONDITION
                },
   "SEC0060" => {
                 'msg'        => 'SSH source filtering is enabled.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052901,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0061" => {
                 'msg'        => 'SSH source filtering is disabled.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052901,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0062" => {
                 'msg'        => 'The host %host% is added to the list of allowed hosts for SSH connection.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0063" => {
                 'msg'        => 'The host %host% is removed from allowed hosts for SSH connection.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0064" => {
                 'msg'        => 'The host %host% is added to the list of denied hosts for SSH connection.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0065" => {
                 'msg'        => 'The host %host% is removed from the list of denied hosts for SSH connection.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0066" => {
                 'msg'        => 'The Linux Audit daemon is started.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052902,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0067" => {
                 'msg'        => 'The Linux Audit daemon is stopped.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052902,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0068" => {
                 'msg'        => 'The maximum number of days a password may be used is set to the new value %maxdays%.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0069" => {
                 'msg'        => 'The minimum number of days allowed between password changes is set to the new value %mindays%.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0070" => {
                 'msg'        => 'The FTP service is permitted.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052903,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0071" => {
                 'msg'        => 'The FTP service is forbidden.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052903,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0072" => {
                 'msg'        => 'The FTP service is started.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052904,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0073" => {
                 'msg'        => 'The FTP service is stopped.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052904,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0074" => {
                 'msg'        => 'Core dumps files are permitted.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052905,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0075" => {
                 'msg'        => 'Core dumps files are forbidden.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052905,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0076" => {
                 'msg'        => 'Core files size is changed to %coresize%.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0077" => {
                 'msg'        => 'The TFTP service is permitted.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052906,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0078" => {
                 'msg'        => 'The TFTP service is forbidden.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052906,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0079" => {
                 'msg'        => 'The TFTP service is started.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052907,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },

   "SEC0080" => {
                 'msg'        => 'The TFTP service is stopped.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052907,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0081" => {
                 'msg'        => 'The Telnet service is permitted.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052908,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0082" => {
                 'msg'        => 'The Telnet service is forbidden.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052908,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0083" => {
                 'msg'        => 'The Telnet service is started.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052909,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0084" => {
                 'msg'        => 'The Telnet service is stopped.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052909,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0085" => {
                 'msg'        => 'The pre-login banners are turned off.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052910,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0091" => {
                 'msg'        => 'The pre-login banners are turned on.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052910,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0086" => {
                 'msg'        => 'The text of the pre-login banner is changed.',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 0,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0092" => {
                 'msg'        => 'Network tools are disabled.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052911,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0093" => {
                 'msg'        => 'Network tools are enabled.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008052911,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0095" => {
                 'msg'        => 'rlogin to the Call Server by external client is permitted.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008090401,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0096" => {
                 'msg'        => 'rlogin to the Call Server by external client is forbidden.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008090401,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0109" => {
                 'msg'        => 'The firewall is enabled.',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2009032401,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "SEC0110" => {
                 'msg'        => 'The firewall is disabled.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_SEC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2009032402,
                 'cause'      => $SNMP_CAUSE_OTHER_SECURITY_SERVICE
                },
   "OSM0018" => {
                 'msg'        => 'Physical Memory usage has passed the %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062901,
                 'cause'      => $SNMP_CAUSE_OUT_OF_MEMORY
                },
   "OSM0019" => {
                 'msg'        => 'Physical Memory usage has dropped below the %THRESHOLD%%. Memory utilization threshold and is below high usage levels. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062901,
                 'cause'      => $SNMP_CAUSE_OUT_OF_MEMORY
                },
   "OSM0020" => {
                 'msg'        => 'Physical Memory usage has passed the %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_CRITICAL,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062902,
                 'cause'      => $SNMP_CAUSE_OUT_OF_MEMORY
                },
   "OSM0021" => {
                 'msg'        => 'Physical Memory usage has dropped below the %THRESHOLD%% Memory utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062902,
                 'cause'      => $SNMP_CAUSE_OUT_OF_MEMORY
                },
   "OSM0022" => {
                 'msg'        => 'CPU Utilization has passed the %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062903,
                 'cause'      => $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED
                },
   "OSM0023" => {
                 'msg'        => 'CPU Utilization has dropped below the %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062903,
                 'cause'      => $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED
                },
   "OSM0024" => {
                 'msg'        => 'CPU Utilization has passed the %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_CRITICAL,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062904,
                 'cause'      => $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED
                },
   "OSM0025" => {
                 'msg'        => 'CPU Utilization has dropped below %THRESHOLD%% utilization threshold. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062904,
                 'cause'      => $SNMP_CAUSE_CPU_CYCLES_LIMIT_EXCEEDED
                },
   "OSM0026" => {
                 'msg'        => '\'%PARTITION%\' disk is getting full and has passed the %THRESHOLD%% threshold. %REST%',
                 'severity'   => $SNMP_SEV_WARNING,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062905,
                 'cause'      => $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM
                },
   "OSM0027" => {
                 'msg'        => '\'%PARTITION%\' disk usage has dropped below the %THRESHOLD%% threshold. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062905,
                 'cause'      => $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM
                },
   "OSM0028" => {
                 'msg'        => '\'%PARTITION%\' disk Is Full. %REST%',
                 'severity'   => $SNMP_SEV_CRITICAL,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062906,
                 'cause'      => $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM
                },
   "OSM0029" => {
                 'msg'        => '\'%PARTITION%\' disk usage has dropped below the %THRESHOLD%% threshold. %REST%',
                 'severity'   => $SNMP_SEV_CLEARED,
                 'alarm_type' => $SNMP_TYP_PROC,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2008062906,
                 'cause'      => $SNMP_CAUSE_STORAGE_CAPACITY_PROBLEM
                },
   "OSM0030" => {
                 'msg'        => '%MEMORY_SIZE% MB does not meet the minimum system memory requirement of %MIN_MEMORY_SIZE% MB',
                 'severity'   => $SNMP_SEV_CRITICAL,
                 'alarm_type' => $SNMP_TYP_EQUIP,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2009031100,
                 'cause'      => $SNMP_CAUSE_EQUIPMENT_FAILURE
                },
   "OSM0031" => {
		 'msg'        => '%HDD_SIZE% MB does not meet the minimum HDD requirement of %MIN_HDD_SIZE% MB',
		 'severity'   => $SNMP_SEV_CRITICAL,
		 'alarm_type' => $SNMP_TYP_EQUIP,
		 'component'  => $SNMP_COMP_MGMT,
		 'notif_id'   => 2009031101,
		 'cause'      => $SNMP_CAUSE_EQUIPMENT_FAILURE
		},
   "OSM0032" => {
		 'msg'        => 'IP address is not assigned for %ETH%.',
		 'severity'   => $SNMP_SEV_MAJOR,
		 'alarm_type' => $SNMP_TYP_EQUIP,
		 'component'  => $SNMP_COMP_MGMT,
		 'notif_id'   => 2009040201,
		 'cause'      => $SNMP_CAUSE_LAN_ERROR
		},
   "OSM0033" => {
                 'msg'        => 'NIC settings for %ETH% do not comply with the minimal system requirements.',
                 'severity'   => $SNMP_SEV_MAJOR,
                 'alarm_type' => $SNMP_TYP_EQUIP,
                 'component'  => $SNMP_COMP_MGMT,
                 'notif_id'   => 2009040202,
                 'cause'      => $SNMP_CAUSE_LAN_ERROR
                }
);

my $pattern_regex = '[a-zA-Z0-9_]+';

## Declarations
## @skipline
sub fill_template($$);
## @skipline
sub build_snmp_message($$);
## @skipline
sub send_snmp_trap($;$);
## @skipline
sub send_snmp_trap_raw(@);

###############################################################################
## @fn sub fill_template($template, $content)
# @brief   Fills a template string with some content
# @details This subroutine parses a template string in order to find names
#          (patterns) of elements from the hash %content and replaced found
#          patterns by values of these elements. If the string contains no
#          patterns it returns without changes.
# @param   $template - a template string where patterns are marked as
#                      %SomePattern%, e.g. "This line is number %Number%".
# @param   $content  - reference to a hash describing which values should be
#                      used for replacement of found patterns.
#                      E.g. {"Number"=>21}.
# @return  $res_str - result of substitution.
# @return  undef    - if the template contains a pattern which is not defined in
#                     the hash
sub fill_template($$)
{
   my ($template, $content) = @_;

   my $res_str = $template;

   baselog($LOG_DEBUG, "The template is '$template'");

   my $pattern;
   my $value;
   my $quit = 0;

   while (not $quit)
   {

      # Look for a pattern
      if ($res_str =~ /%($pattern_regex)%/)
      {
         $pattern = $1;
         baselog($LOG_DEBUG, "Found the pattern '$pattern'");

         # Check if the pattern is defined in the hash with values
         if (defined $content->{$pattern})
         {

            #replace pattern with value
            $value = $content->{$pattern};
            $res_str =~ s/%$pattern%/$value/g;
         } ## end if (defined $content->...
         else
         {
            $quit = 2;
            baselog($LOG_ERROR, "No value provided for the pattern '$pattern'");
         }
      } ## end if ($res_str =~ /%($pattern_regex)%/...
      else
      {

         # Template contain no patterns. No modification is needed.
         $quit = 1;
      } ## end else [ if ($res_str =~ /%($pattern_regex)%/...
   } ## end while (not $quit)

   if ($quit == 1)
   {
      baselog($LOG_DEBUG, "The message is '$res_str'");
   }
   else
   {
      $res_str = undef;
   }

   return $res_str;
} ## end sub fill_template($$)

###############################################################################
## @fn sub build_snmp_message($snmp_id, $content)
# @brief   Fills a template string with some content
# @details This subroutine gets the parameters of a SNMP message from the SNMP
#          database by its ID. If the message string is an template the
#          subroutine fill_template is used in order to prepare resulting
#          message string.
# @param   $snmp_id - unique ID which is used to find the SNMP message in the
#                     SNMP database
# @param   $content - reference to a hash describing which values should be used
#                     for preparing the message string from the message template.
# @return  the list of parameters describing the SNMP message
# @return  empty list if the message string cannot be not prepared from the
#          template
# @note This is the internal subroutine and it does not validate input
#       parameters and data from the SNMP database.
sub build_snmp_message($$)
{
   my ($snmp_id, $content) = @_;

   baselog($LOG_DEBUG, "Try to find the message '$snmp_id'");

   my $snmp_msg = fill_template($SNMP_MSGS{$snmp_id}->{'msg'}, $content);

   # Some error happens during the message preparation
   if (not defined $snmp_msg)
   {
      return ();
   }

   my $severity        = $SNMP_MSGS{$snmp_id}->{'severity'};
   my $alarm_type      = $SNMP_MSGS{$snmp_id}->{'alarm_type'};
   my $component       = $SNMP_MSGS{$snmp_id}->{'component'};
   my $notification_id = $SNMP_MSGS{$snmp_id}->{'notif_id'};
   my $probable_cause  = $SNMP_MSGS{$snmp_id}->{'cause'};

   return ($severity, $snmp_id, $alarm_type, $snmp_msg, $component, $notification_id, $probable_cause);
} ## end sub build_snmp_message($$)

###############################################################################
## @method sub send_snmp_trap_raw($severity, $msg_id, $alarm_type, $msg, $component, $notification_id, $probable_cause)
# @brief   Sends SNMP trap by using external command.
# @details This method validates input parameters and tries to use the
#          external command in order to send SNMP trap with usage of the input
#          parameters.
# @param   $severity - Trap Severity. Numeric value which denotes the
#                      severity of the trap.
# @param   $msg_id  - Error code to be sent in trap.
# @param   $alarm_type - Alarm Type: A numeric value which defines the type of
#                        the alarm.
# @param   $msg - A string which defines the description of alarm.
# @param   $component - A string which denotes a component.
# @param   $notification_id - A numeric value which denotes the unique ID for
#                             each generated traps
# @param   $probable_cause - A numeric value which gives the probable cause for
#                            the alarm
# @return  TRUE - message is sent
# @return  FALSE - some error happens
sub send_snmp_trap_raw(@)
{
   my ($severity, $msg_id, $alarm_type, $msg, $component, $notification_id, $probable_cause) = @_;

   if ((not defined $severity) or ($severity < $SNMP_SEV_MIN) or ($severity > $SNMP_SEV_MAX))
   {
      baselog($LOG_ERROR, "SNMP message severity number is not correct.");
      return $FALSE;
   }

   if ((not defined $alarm_type) or ($alarm_type < $SNMP_TYP_MIN) or ($alarm_type > $SNMP_TYP_MAX))
   {
      baselog($LOG_ERROR, "SNMP message alarm type is not correct.");
      return $FALSE;
   }

   if (not defined $msg_id)
   {
      baselog($LOG_ERROR, "SNMP message ID is not specified.");
      return $FALSE;
   }

   if (not defined $msg)
   {
      baselog($LOG_ERROR, "SNMP message string is not specified.");
      return $FALSE;
   }

   if (not defined $component)
   {
      baselog($LOG_ERROR, "SNMP message component is not specified.");
      return $FALSE;
   }

   if (not defined $notification_id)
   {
      baselog($LOG_ERROR, "SNMP message notification ID is not specified.");
      return $FALSE;
   }

   if (not defined $probable_cause)
   {
      baselog($LOG_ERROR, "Probably cause of SNMP message is not specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "The SNMP message is '" . join("' '", @_) . "'");

   baselog($LOG_NOTICE, "$msg_id: $msg Severity: $severity; Alarm type: $alarm_type; Component: '$component'; Probable cause: $probable_cause.");

   my $retval = $TRUE;

   # In gloomy times of RHEL5.1 migration it is necessary to handle situation
   # when SNMP trap tool can be not defined in Linux Base modules at all.
   if (defined $SENDSNMPTRAP)
   {

      # It is necessary to handle situation when a SNMP trap is tried to send
      # when SNMP application is not installed yet.
      if (-f $SENDSNMPTRAP)
      {
         if (not do_system([$SENDSNMPTRAP, $severity, $msg_id, $alarm_type, $msg, $component, $notification_id, $probable_cause]))
         {
            baselog($LOG_ERROR, "Cannot send SNMP trap.");
            $retval = $FALSE;
         }
      } ## end if (-f $SENDSNMPTRAP)
      else
      {
         baselog($LOG_ERROR, "'$SENDSNMPTRAP' is not found. Cannot send SNMP trap.");
         $retval = $FALSE;
      }
   } ## end if (defined $SENDSNMPTRAP...
   else
   {
      baselog($LOG_ERROR, "Command for sending SNMP is not found. Cannot send SNMP trap.");
      $retval = $FALSE;
   }

   return $retval;
} ## end sub send_snmp_trap_raw(@)

###############################################################################
## @method sub send_snmp_trap($snmp_id, $content)
# @brief   Gets a SNMP trap from the SNMP DB and sends it.
# @details This subroutine gets the parameters of a SNMP message from the SNMP
#          database by its ID and prepares it for sending by using the
#          subroutine build_snmp_message. If the message is prepared successfully
#          it is sent by using the subroutine send_snmp_trap_raw.
# @param   $snmp_id - unique ID which is used to find the SNMP message in the
#                     SNMP database
# @param   $content - reference to a hash describing which values should be used
#                     for preparing the message string from the message template
#                     stored in the database.
# @return  TRUE - the trap is sent
# @return  FALSE - some error happens
sub send_snmp_trap($;$)
{
   my ($snmp_id, $content) = @_;

   if (not defined $snmp_id)
   {
      baselog($LOG_ERROR, "SNMP message ID is not specified.");
      return $FALSE;
   }
   elsif (not exists $SNMP_MSGS{$snmp_id})
   {
      baselog($LOG_ERROR, "SNMP message with ID '$snmp_id' is not registered.");
      return $FALSE;
   }

   if (not defined $content)
   {
      baselog($LOG_DEBUG, "No extra content provided.");
      %{$content} = ();
   }

   baselog($LOG_DEBUG, "Try to send SNMP message '$snmp_id'");

   my $retval = $TRUE;

   my @snmp_msg = build_snmp_message($snmp_id, $content);

   if (scalar @snmp_msg)
   {
      $retval = send_snmp_trap_raw(@snmp_msg);
   }
   else
   {
      baselog($LOG_ERROR, "SNMP trap is not sent.");
      $retval = $FALSE;
   }

   return $FALSE;
} ## end sub send_snmp_trap($;$)

1;
