# ############################################################
# Description:
# This module is the LinuxBase interface for the system
# parameters changing. It deals with the XML DB files.
#
# Author:
# Alexander Kolotov
#
# History of changes:
#   1.00 - Initial version
#   1.10 - two subroutines were added get_param & set_param
#   1.11 - The get_referenced_value function was moved to XMLRef module
#   1.12 - Two colons were removed before shared functions and variables.
# ############################################################
package base_sysctl;

use lib "/opt/nortel/base/lib";

use common_variables;
use install_common;

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&backup_parameters
  &restore_parameters
  &apply_parameters
  &remove_sysctl_backup
  &show_parameters
  &set_param
  &get_param);
our $VERSION = 1.12;

use ifxml;
use baselog;
use XML::Writer;
use IO::File;
use Data::Dumper;
use XMLRef;
use warnings;
use strict;

#####################################################################
##################### Public Methods Declaration ####################

sub backup_parameters;
sub restore_parameters;
sub apply_parameters;
sub remove_sysctl_backup;
sub show_parameters;
sub set_param;
sub get_param;

#####################################################################
##################### Private Methods Declaration ###################

sub save_backup_file;
sub open_xml_file;
sub backup_exists;
sub do_sysctl;
sub is_rule_correct;
sub compact_backup_database;

#####################################################################
##################### Private Data Declaration #######################

# sysctl DB files
my $SYSCTL_FILE                 = "sysctl.xml";
my $SYSCTL_BACKUP_FILE          = "sysctl_bak.xml";
my $SYSCTL_FILE_FULLPATH        = qq|$SYSDB_ROOT/$SYSCTL_FILE|;
my $SYSCTL_BACKUP_FILE_FULLPATH = qq|$SYSDB_ROOT/$SYSCTL_BACKUP_FILE|;

# define section name and internal tag name
my $SECTION   = 'sysctl';
my $TAG       = 'rule';
my $XPATH_TAG = "/$TAG";

# sysctl actions
my $READ_SYSCTL  = 'read';
my $WRITE_SYSCTL = 'write';

# other constans
my $BASE_APPL          = 'base';
my @ALLOWED_SUBSYSTEMS = ('net');

format SYSCTL_HEADER =
Kernel parameters table
Parameter                Current value         Backup value          Application
.

my $SYSCTL_ROW_TEMPLATE = '
@<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<
';

# Hash to hold the system parameter & its value
my %RULE = ('param' => undef, 'value' => undef);

#####################################################################
########################### Private Methods #########################

#####################################################################
# This subroutine saves backup data in the file
#  INPUT: @data - backup data
# OUTPUT: $TRUE  - file is saved sucesssfully
#         $FALSE - some error happens
#####################################################################
sub save_backup_file
{
   my @data = @{shift @_};
   my $rule;

   my $dstfile = ">" . $SYSCTL_BACKUP_FILE_FULLPATH;
   my $output  = new IO::File($dstfile);
   my $xml     =
     new XML::Writer(
                     DATA_MODE   => 'true',
                     DATA_INDENT => 3,
                     OUTPUT      => $output
                    );
   if (!$xml)
   {
      baselog($LOG_ERROR, "Cannot save backup parameters.");
      return $FALSE;
   }

   $xml->startTag($SECTION);
   foreach $rule (@data)
   {
      $xml->emptyTag($TAG, %{$rule});
   }
   $xml->endTag($SECTION);
   $xml->end();

   return $TRUE;
} ## end sub save_backup_file

#####################################################################
# This subroutine opens XML file, check that the file contains rules
# and returns XML parser
#  INPUT: $fdir  - dir name where the XML file is situated
#         $fname - XML file name
# OUTPUT: XML parser or 'undef' if some error happens
#####################################################################
sub open_xml_file
{
   my $fdir  = shift @_;
   my $fname = shift @_;

   baselog($LOG_DEBUG, "open_xml_file: dir:'$fdir', file:'$fname'");

   # 1. Get XML parser
   my $parser = ifxml::new($fdir, $fname);
   if (!$parser)
   {
      baselog($LOG_ERROR, "Cannot read sysctl XML DB file: $fdir/$fname");
      return undef;
   }

   # 2. Check that the file contains rules
   my $rules = $parser->find_data_by_xpath($GETDATA, $XPATH_TAG);
   if ((!defined $rules) || (scalar(@{$rules}) == 0))
   {
      baselog($LOG_INFO, "There are no rules in XML DB file: $fdir/$fname");
      undef $parser;
   }

   return $parser;
} ## end sub open_xml_file

#####################################################################
# This subroutine looks for the parameter in XML DB.
#  INPUT: $parser - the reference to XML parser
#         $rule   - a reference to the hash containing the name of
#                   the parameter
# OUTPUT: $TRUE  - the parameter exists in XML DB. The backup value
#                    is in $param->{'value_bak'}
#         $FALSE - there is no such parameter in XML DB.
#####################################################################
sub backup_exists
{
   my $parser = shift @_;
   my $rule   = shift @_;

   my $param = $rule->{'param'};

   # Get data from the XML DB
   my $rules = $parser->find_data_by_xpath($GETDATA, $XPATH_TAG, $NOATTR, {param => $param});
   if ((!defined $rules) || (scalar(@{$rules}) == 0))
   {
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Got backup data: " . Data::Dumper::Dumper(\$rules));

   if (!is_rule_correct($rules->[0], {'value' => $TRUE}))
   {
      return $FALSE;
   }

   $rule->{'value_bak'} = $rules->[0]->{'value'};

   return $TRUE;
} ## end sub backup_exists

#####################################################################
# This subroutine is a wrapper for 'sysctl' command.
#  INPUT: $action - action: 'read' or 'write'
#         $rule   - a reference to the hash containing the name of
#                   the parameter and its value
# OUTPUT: $TRUE  - the new value is in $rule->{'value'}.
#                    For action='write', $rule->{'value'} is empty
#         $FALSE - some error happens
#####################################################################
sub do_sysctl
{
   my $action = shift @_;
   my $rule   = shift @_;

   my $param = $rule->{'param'};
   my $value = $rule->{'value'};

   if (!defined $value)
   {
      $value = '';
   }

   baselog($LOG_DEBUG, "do_sysctl: action:'$action', param:'$param', value='$value'");

   my $retval = $FALSE;

   my $cmd = '';

   if ($action eq $READ_SYSCTL)
   {
      $cmd = "-n $param";
   }
   elsif ($action eq $WRITE_SYSCTL)
   {
      $cmd = "-q -w $param='$value'";
   }
   else
   {
      baselog($LOG_WARNING, "Unknown action '$action'");
      $value = '';
   }

   if ($cmd)
   {
      baselog($LOG_DEBUG, "do_sysctl: cmd:'$cmd'");

      $value = `$SYSCTL $cmd 2>/dev/null`;
      if ($? >> 8)
      {
         $value = '';
      }
      else
      {
         chomp $value;
         $retval = $TRUE;
      }
   } ## end if ($cmd)

   $rule->{'value'} = $value;

   return $retval;
} ## end sub do_sysctl

#####################################################################
# This subroutine checks if all three mandatory attributes "param",
# "appl" and "value" are defined in the rule. The checking of "value"
# is turned off by default. It can be turned on if the key 'value' of
# %options hash is $TRUE.
# If all the attributes are defined this subroutine checks if the
# application is allowed to change the parameter in the rule. This
# check is turned off by default. It can be turned on if the key
# 'access' of %options hash is $TRUE.
#  INPUT: $rule        - a reference to the hash describing rule
#         $options->{'value'} - if it is $TRUE the "value" attribute
#                               of the rule will be checked. If it is
#                               undefined or $FALSE this attribute
#                               will not be checked
#         $options->{'access'} - if it is $TRUE the rule is checked
#                                if the application can control the
#                                parameter
## OUTPUT : $TRUE - all the attributes are defined
#          $FALSE - There is no one of the attributes
#####################################################################
sub is_rule_correct
{
   my $rule    = shift @_;
   my $options = shift @_;

   my $check_value = $FALSE;
   my $access      = $FALSE;

   if (defined $options->{'value'})
   {
      $check_value = $options->{'value'};
   }
   if (defined $options->{'access'})
   {
      $access = $options->{'access'};
   }

   my $retval = $TRUE;

   my $attr;

   my @attributes = ("param", "appl", "value");

   # 1. Check if attributes are defined
   foreach $attr (@attributes)
   {
      if (($attr eq "value") && (!$check_value))
      {
         last;
      }

      if (!defined $rule->{$attr})
      {
         baselog($LOG_ERROR, "There is no '$attr' attribute in the rule.");
         $retval = $FALSE;
         last;
      } ## end if (!defined $rule->{$attr...
   } ## end foreach $attr (@attributes)

   return $retval if (!$retval);

   # 2. Check the application is allowed to change this parameter
   if (($access) && ($rule->{'appl'} ne $BASE_APPL))
   {
      my $param = $rule->{'param'};

      $retval = $FALSE;

      # If the parameter is allowed, $retval will be $TRUE
      foreach my $subsys (@ALLOWED_SUBSYSTEMS)
      {
         if ($param =~ /^$subsys\..+/)
         {
            $retval = $TRUE;
            last;
         }
      } ## end foreach my $subsys (@ALLOWED_SUBSYSTEMS...

      if (!$retval)
      {
         baselog($LOG_EMERG, "Application '$rule->{'appl'}' has no permissions to change parameter '$rule->{'param'}'.");
         return $FALSE;
      }
   } ## end if (($access) && ($rule...

   return $TRUE;
} ## end sub is_rule_correct

#####################################################################
# This subroutine compacts backup DB - all rules which belong to the
# given application, are removed. The compacted DB is stored in
# $SYSCTL_BACKUP_FILE.
#  INPUT: $appl - the application name
# OUTPUT: @tmparr - an array with rules after compacting (it does not
#                   contain the rules for the given application)
#         ()      - if the backup DB does not exist or if there are
#                   no rules in this DB
#####################################################################
sub compact_backup_database
{
   my $appl = shift @_;

   baselog($LOG_DEBUG, "Try to compact backup parameters for application '$appl'.");

   my @tmparr = ();

   my ($bak_parser, $rules, $rule);

   # Check if the backup DB exists. It is necessary to work with DB
   # only if it exists.
   if (-f $SYSCTL_BACKUP_FILE_FULLPATH)
   {

      # Open the backup XML DB and build XML tree.
      $bak_parser = open_xml_file($SYSDB_ROOT, $SYSCTL_BACKUP_FILE);
      if (defined $bak_parser)
      {

         # Get the current state of XML DB
         $rules = $bak_parser->find_data_by_xpath($GETDATA, $XPATH_TAG);
         if ((defined $rules) && (scalar(@{$rules}) > 0))
         {
            foreach $rule (@{$rules})
            {
               baselog($LOG_DEBUG, "Got data: " . Data::Dumper::Dumper(\$rule));

               if (is_rule_correct($rule, {'value' => $TRUE}))
               {

                  # Do not add the rules for the given application to
                  # the result array
                  if ($rule->{'appl'} ne $appl)
                  {
                     push @tmparr, $rule;
                  }
               } ## end if (is_rule_correct($rule...
            } ## end foreach $rule (@{$rules})
         } ## end if ((defined $rules) &&...
         else
         {
            baselog($LOG_INFO, "There are no rules in backup XML DB for application '$appl'.");
         }

         undef $bak_parser;
      } ## end if (defined $bak_parser...

      if (scalar(@tmparr) == 0)
      {

         # Remove old backup XML DB
         remove_sysctl_backup();
      } ## end if (scalar(@tmparr) ==...
      elsif (save_backup_file(\@tmparr))
      {
         baselog($LOG_DEBUG, "DB with backup parameters purged.");
      }
   } ## end if (-f $SYSCTL_BACKUP_FILE_FULLPATH...
   else
   {
      baselog($LOG_INFO, "The file $SYSCTL_BACKUP_FILE_FULLPATH does not exist.");
   }

   return @tmparr;
} ## end sub compact_backup_database

#####################################################################
########################### Public Methods ##########################

#####################################################################
# This subroutine backups the current values of system paramteres
# listed in $SYSCTL_FILE file for given application and stored them
# in $SYSCTL_BACKUP_FILE.
# If there is no application provided, the subroutines backups all
# the parameters.
#  INPUT: $appl - the application name
# OUTPUT: $TRUE  - parameters are backed up sucesssfully
#         $FALSE - some error happens
#####################################################################
sub backup_parameters
{
   my $appl = shift @_;

   my %params = ();

   my ($parser, $rules, $rule);
   my $retval  = $FALSE;
   my $reshash = {};
   my $filter  = {};

   # 1. Remove old backup data
   # If the application is provided, all rules (they are outdated) for
   # this application are removed from backup XML DB.
   # If there is no application provided, all rules are outdated. The
   # backup XML DB is removed.
   if ((defined $appl) && ($appl ne ''))
   {

      # Remove the application's rules from the backup DB. The rest of
      # rules is stored in %params hash in order to check duplicates.
      foreach $rule (compact_backup_database($appl))
      {
         $params{$rule->{'param'}} = $rule;
      }

      $filter = {appl => $appl};
   } ## end if ((defined $appl) &&...
   else
   {

      # Remove old backup XML DB
      remove_sysctl_backup();

      $appl = '';
   } ## end else [ if ((defined $appl) &&...

   # 2. Open the XML file and build XML tree.
   $parser = open_xml_file($SYSDB_ROOT, $SYSCTL_FILE);
   if (!defined $parser)
   {
      return $FALSE;
   }

   # 3. Get the current system parameters
   $rules = $parser->find_data_by_xpath($GETDATA, $XPATH_TAG, $NOATTR, $filter);
   if ((defined $rules) && (scalar(@{$rules}) > 0))
   {
      $retval = $TRUE;

      foreach $rule (@{$rules})
      {
         baselog($LOG_DEBUG, "Got data: " . Data::Dumper::Dumper(\$rule));

         if (!is_rule_correct($rule))
         {
            if (!defined $rule->{'param'})
            {
               $rule->{'param'} = '';
            }
            baselog($LOG_ERROR, "The backup of parameter '$rule->{'param'}' for the application '$appl' will not be created.");
            $retval = $FALSE;
            next;
         } ## end if (!is_rule_correct($rule...

         if (!exists($params{$rule->{'param'}}))
         {

            # Get the value of the system parameter
            $reshash = {
                        param => $rule->{'param'},
                        appl  => $rule->{'appl'},
                        value => ''
                       };
            if (!do_sysctl($READ_SYSCTL, $reshash))
            {
               baselog($LOG_ERROR, "Cannot get the value of parameter '$rule->{'param'}'.");
               $retval = $FALSE;
            }
            else
            {

               # Store the value in the hash
               $params{$rule->{'param'}} = $reshash;

               baselog($LOG_DEBUG, "$rule->{'param'} = '$reshash->{'value'}'");

               $retval &= $TRUE;
            } ## end else [ if (!do_sysctl($READ_SYSCTL...
         } ## end if (!exists($params{$rule...
         else
         {
            baselog($LOG_ERROR, "The applications " . "'" . $params{$rule->{'param'}}->{'appl'} . "' and " . "'" . $rule->{'appl'} . "' " . "try to backup the same parameter '$rule->{'param'}'.");

            # Do not change $retval, as this is not a critical error.
         } ## end else [ if (!exists($params{$rule...
      } ## end foreach $rule (@{$rules})

      # 4. Save backup file.
      if ($retval)
      {
         baselog($LOG_INFO, "Backup data of the system parameters for application '$appl' are collected.");
         if (!save_backup_file([values %params]))
         {
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_DEBUG, "DB with backup parameters updated.");
         }
      } ## end if ($retval)
      else
      {
         baselog($LOG_ERROR, "Cannot collect backup data of the system parameters for application '$appl'");
      }
   } ## end if ((defined $rules) &&...
   else
   {
      baselog($LOG_INFO, "There are no rules in XML DB for application '$appl'.");
   }

   # 5. Free memory
   undef $parser;

   return $retval;
} ## end sub backup_parameters

#####################################################################
# This subroutine restores the system paramteres for given application
# from backup XML DB.
# If there is no application provided, the subroutines restores all
# the parameters.
#  INPUT: $appl - the application name
# OUTPUT: $TRUE  - parameters are restored sucesssfully
#         $FALSE - some error happens
#####################################################################
sub restore_parameters
{
   my $appl = shift @_;

   my ($rules, $rule);
   my $retval = $FALSE;
   my $filter = {};

   if ((defined $appl) && ($appl ne ''))
   {
      $filter = {appl => $appl};
   }
   else
   {
      $appl = '';
   }

   # 1. Open backup XML DB
   my $bak_parser = open_xml_file($SYSDB_ROOT, $SYSCTL_BACKUP_FILE);
   if (!defined $bak_parser)
   {

      # The critical message is generated basing on an assumption that
      # Linux Base always contains sysctl rules. If it is not right
      # extra checks should be introduced.
      baselog($LOG_CRIT, "The system parameters for application '$appl' cannot be restored.");

      # In the case if restore_parameters is invoked during 'appstartup
      # reload', the reload process will be aborted. Only reboot can
      # solve this issue.
      return $FALSE;
   } ## end if (!defined $bak_parser...

   # 2. Get rules
   $rules = $bak_parser->find_data_by_xpath($GETDATA, $XPATH_TAG, $NOATTR, $filter);

   if ((defined $rules) && (scalar(@{$rules}) > 0))
   {
      $retval = $TRUE;

      foreach $rule (@{$rules})
      {
         baselog($LOG_DEBUG, "Got backup data: " . Data::Dumper::Dumper(\$rule));

         if (!is_rule_correct($rule, {'access' => $TRUE}))
         {
            baselog($LOG_ERROR, "The parameter for '$appl' application will not be restored.");
            $retval = $FALSE;
            next;
         } ## end if (!is_rule_correct($rule...

         # 3. Apply the rule
         if ((defined $rule->{'value'}) && ($rule->{'value'} ne ''))
         {
            if (do_sysctl($WRITE_SYSCTL, $rule))
            {
               $retval &= $TRUE;
            }
            else
            {
               baselog($LOG_ERROR, "Cannot restore the value '$rule->{'value'}' for parameter '$rule->{'param'}'.");
               $retval = $FALSE;
            }
         } ## end if ((defined $rule->{'value'...
         else
         {
            baselog($LOG_ERROR, "There is no backup value for parameter '$rule->{'param'}'. The parameter for application '$rule->{'appl'}' will not be restored.");
            $retval = $FALSE;
         }
      } ## end foreach $rule (@{$rules})
      if ($retval)
      {
         baselog($LOG_INFO, "System parameters for application '$appl' are restored.");
      }
   } ## end if ((defined $rules) &&...
   else
   {
      baselog($LOG_INFO, "There are no rules in backup XML DB for application '$appl'.");
   }

   # 4. Free memory
   undef $bak_parser;

   return $retval;
} ## end sub restore_parameters

#####################################################################
# This subroutine reads XML DB files and applies the new values of
# system parameters for given application. If it is found that some
# value has to be dereferenced before applying, XMLRef class is used
# for this.
# If there is no application provided, the subroutines applies all
# the parameters.
#  INPUT: $appl - the application name
# OUTPUT: $TRUE  - parameters are applied sucessfully
#         $FALSE - some error happens
#####################################################################
sub apply_parameters
{
   my $appl = shift @_;

   my ($rules, $rule, $param);
   my $retval = $FALSE;
   my $filter = {};

   if ((defined $appl) && ($appl ne ''))
   {
      $filter = {appl => $appl};
   }
   else
   {
      $appl = '';
   }

   # 1. Backup parameters prior to apply them
   if (!backup_parameters($appl))
   {
      baselog($LOG_INFO, "There are no backup parameters for application '$appl'.");
      return $FALSE;
   }

   # 2. Open XML DB
   my $parser = open_xml_file($SYSDB_ROOT, $SYSCTL_FILE);
   if (!defined $parser)
   {
      return $FALSE;
   }

   # Create instance of XMLRef class.
   my $xmlref = XMLRef::new();

   # 3. Get rules
   $rules = $parser->find_data_by_xpath($GETDATA, $XPATH_TAG, $NOATTR, $filter);

   if ((defined $rules) && (scalar(@{$rules}) > 0))
   {

      $retval = $TRUE;

      foreach $rule (@{$rules})
      {
         baselog($LOG_DEBUG, "Got data: " . Data::Dumper::Dumper(\$rule));

         if (!is_rule_correct($rule, {'access' => $TRUE}))
         {
            baselog($LOG_ERROR, "The rule for application '$appl' will be skipped.");
            $retval = $FALSE;
            next;
         } ## end if (!is_rule_correct($rule...

         # 4. De-reference value
         my $val_tmp = $xmlref->get_referenced_value($rule->{'reference'}, $rule->{'value_ref'}, $rule->{'value'});
         if (defined $val_tmp)
         {
            $rule->{'value'} = $val_tmp;
         }

         # 5. Apply the rule
         if ((defined $rule->{'value'}) && ($rule->{'value'} ne ''))
         {
            if (!do_sysctl($WRITE_SYSCTL, $rule))
            {
               baselog($LOG_ERROR, "Cannot apply the new value '$rule->{'value'}' for parameter '$rule->{'param'}'.");
               $retval = $FALSE;
            }
         } ## end if ((defined $rule->{'value'...
         else
         {
            baselog($LOG_WARNING, "There is no value for parameter '$rule->{'param'}'. The rule for application '$rule->{'appl'}' will be skipped.");

            # Do not change $retval, as the  error is not fatal.
            # It should be handled on the design stage.
         } ## end else [ if ((defined $rule->{'value'...
      } ## end foreach $rule (@{$rules})
      if ($retval)
      {
         baselog($LOG_INFO, "System parameters for application '$appl' are applied.");
      }
   } ## end if ((defined $rules) &&...
   else
   {
      baselog($LOG_INFO, "There are no rules in XML DB for application '$appl'.");
   }

   # 6. Free memory
   undef $parser;
   undef $xmlref;

   return $retval;
} ## end sub apply_parameters

#####################################################################
# This subroutine removed bakup XML DB file
#  INPUT: None
# OUTPUT: $TRUE  - the file is remove
#         $FALSE - some error happens
#####################################################################
sub remove_sysctl_backup
{

   baselog($LOG_DEBUG, "Try to remove $SYSCTL_BACKUP_FILE_FULLPATH file");

   if (-f $SYSCTL_BACKUP_FILE_FULLPATH)
   {
      if (!unlink($SYSCTL_BACKUP_FILE_FULLPATH))
      {
         baselog($LOG_ERROR, "Cannot remove $SYSCTL_BACKUP_FILE_FULLPATH");
         return $FALSE;
      }
   } ## end if (-f $SYSCTL_BACKUP_FILE_FULLPATH...
   else
   {
      baselog($LOG_INFO, "The file $SYSCTL_BACKUP_FILE_FULLPATH does not exist.");
   }

   return $TRUE;
} ## end sub remove_sysctl_backup

#####################################################################
# This subroutine prints a table with the system parameters changed
# by applications
#  INPUT: None
# OUTPUT: $TRUE  - the file is remove
#         $FALSE - some error happens
#####################################################################
sub show_parameters
{
   my ($rules, $rule);
   my $retval = $TRUE;

   # 1. Open XML DBs
   # Open generic DB
   my $parser = open_xml_file($SYSDB_ROOT, $SYSCTL_FILE);

   # Open backup XML DB
   my $bak_parser = open_xml_file($SYSDB_ROOT, $SYSCTL_BACKUP_FILE);

   # Put empty line
   print "\n";

   if ((!defined $parser) && (!defined $bak_parser))
   {
      baselog($LOG_INFO, "Cannot read the list of system parameters configured.\n", $TRUE);
      return $FALSE;
   }

   # 2. Get rules
   if (defined $parser)
   {
      $rules = $parser->find_data_by_xpath($GETDATA, $XPATH_TAG);
   }
   else
   {
      $rules = $bak_parser->find_data_by_xpath($GETDATA, $XPATH_TAG);
   }

   if ((defined $rules) && (scalar(@{$rules}) > 0))
   {

      # Prepare FORMAT subsytem to print the table
      # Backup old values of formates
      my $old_fmt_row      = $~;
      my $old_fmt_top      = $^;
      my $old_fmt_formfeed = $^L;

      # Apply new formates
      $^ = "SYSCTL_HEADER";
      eval 'format SYSCTL_ROW =' . $SYSCTL_ROW_TEMPLATE . q/$rule->{'param'}, $rule->{'value'}, $rule->{'value_bak'}, $rule->{'appl'}/ . "\n.";
      $~  = "SYSCTL_ROW";
      $^L = "";

      foreach $rule (@{$rules})
      {
         baselog($LOG_DEBUG, "Got data: " . Data::Dumper::Dumper(\$rule));

         if (!is_rule_correct($rule))
         {
            baselog($LOG_ERROR, "The rule will be skipped.");
            $retval = $FALSE;
            next;
         } ## end if (!is_rule_correct($rule...

         # 3. Get the backup data
         if ((!defined $bak_parser) || (!backup_exists($bak_parser, $rule)))
         {
            $rule->{'value_bak'} = 'No backup';
         }

         # 4. Get the system data
         if ((!defined $parser) || (!do_sysctl($READ_SYSCTL, $rule)))
         {
            baselog($LOG_ERROR, "Cannot get the value of parameter '$rule->{'param'}'.");
            $rule->{'value'} = 'Unknown';
            $retval = $FALSE;
         } ## end if ((!defined $parser)...

         $retval &= $TRUE;

         # 5. Print one row of the table
         write;
      } ## end foreach $rule (@{$rules})

      # Restore old formates
      $^L = $old_fmt_formfeed;
      $^  = $old_fmt_top;
      $~  = $old_fmt_row;
   } ## end if ((defined $rules) &&...
   else
   {
      baselog($LOG_INFO, "No system parameters configured.\n", $TRUE);
   }

   # 6. Free memory
   undef $bak_parser;
   undef $parser;

   return $retval;
} ## end sub show_parameters

#####################################################################
# This subroutine set one system parameter
#  INPUT: $_[0]    - parameter name
#         $_[1]    - parameter value
# OUTPUT: $TRUE  - the parameter is set
#         $FALSE - some error happens
#####################################################################
sub set_param
{
   $RULE{'param'} = shift @_;
   $RULE{'value'} = shift @_;

   my $retval = $TRUE;
   unless (do_sysctl($WRITE_SYSCTL, \%RULE))
   {
      baselog($LOG_ERROR, "Cannot set parameter $RULE{'param'}.");
      $retval = $FALSE;
   }
   return $retval;
} ## end sub set_param

#####################################################################
# This subroutine returns the value of the system parameter given
#  INPUT: $_[0]    - parameter name
# OUTPUT: $retval  - parameter value or undef value if some error happens
#####################################################################
sub get_param
{
   $RULE{'param'} = shift @_;

   unless (do_sysctl($READ_SYSCTL, \%RULE))
   {
      $RULE{'value'} = undef;
      baselog($LOG_ERROR, "Cannot get parameter $RULE{'param'}.");
   }
   return $RULE{'value'};
} ## end sub get_param

1;
__END__

