## @file base_system.pm
# @brief Execute command, allow to get or redirect the output of the command
#   and log an error if occurred
# @details
# @author Vladimir Ivanov
# @author Ilya Kudelin
# @version  1.00 - Initial version
# @version  1.01 - $dsSTDOUT and $dsINTERACT added
# @version  1.02 - do_chdir and $dsCHDIR, $dsHOMEDIR added
# @version  1.03 - IPC::Run3 is used instead of IPC::Open3
# @version  1.10 - $dsSIGDEFAULT added, do_system function was reworked to
#   implement fork in do_system function.
# @version  1.11 - Two colons were removed before shared functions and variables.
# @version  1.12 - To rework pipes structure in do_system function. To replace two
#                  pipes (PROCESS_STATUS and READER_RETVAL) with one (PROCESS_STATUS).
#                  This pipe collects all service data about process status, retval and
#                  checksums for data pipes.
# @version  1.13 - do_sudo added
# @version  1.14 - Rename write_file to ds_write_file() to prevent conflict with
#                  function from common_functions.pm
# @version  1.15 - Modified under the perlcheck warnings cleanup activity.
# @version  1.20 - do_sys_recv and do_sys_send functions were implemented.
#                  Defensive code for reading/sending messages from/into pipes was implemented.
# @version  1.21 - message through the pipe is sent without changes.
# @note
package base_system;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use common_variables;
use common_functions;
use baselog;
use IPC::Run3;
use IO::File;
use POSIX;
use Cwd qw(getcwd chdir);

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&do_system
  &do_chdir
  &do_sudo
  $dsHOMEDIR
  $dsRETVAL
  $dsSTDOUT
  $dsOUT
  $dsOUTFILE
  $dsOUTMODE
  $dsERR
  $dsERRFILE
  $dsERRMODE
  $dsNOERROR
  $dsINTERACT
  $dsCHDIR
  $dsSIGDEFAULT
  );

our $VERSION = '1.21';

#####################################################################
########################## Public Data ##############################

# The reference to the variable to hold the value returned by system command
our $dsRETVAL = 'retval';

# To define whether it is necessary to redirect the output of the command
# to STDOUT. The value of this key in the hash is 'false' by default.
our $dsSTDOUT = 'stdout';

# The reference to the array to hold the output of the command
our $dsOUT = 'out';

# The file name or file descriptor to write the output in
our $dsOUTFILE = 'outfile';

# The string with the mode to open the output file (only for file name)
our $dsOUTMODE = 'outmode';

# The reference to the array to hold the error output of the command
our $dsERR = 'err';

# The file name or file descriptor to write the error output in
our $dsERRFILE = 'errfile';

# The string with the mode to open the error output file (only for file name)
our $dsERRMODE = 'errmode';

# To define the case when the non-zero exit status does not mean an error case
our $dsNOERROR = 'noerror';

# To define the case when the command has to be executed interactively.
# In this case the options $dsSTDOUT, $dsOUT, $dsOUTFILE, $dsERR and
# $dsERRFILE will be ignored, STDOUT and STDERR will be used to output
# information. The value of this key in the hash is 'false' by default.
our $dsINTERACT = 'interactive';

#To define the case when signals handlers must be re-set into default.
#The value of this key in the hash is 'false' by default.
our $dsSIGDEFAULT = 'sigdefault';

# To define whether it is necessary to change a directory before the
# command execution. If the value of this parameter is undefined, the
# current working directory will not be changed. If the value is an
# empty string ($dsHOMEDIR), the subroutine changes the directory
# specified by $ENV{HOME}.
our $dsCHDIR   = 'chdir';
our $dsHOMEDIR = '';

#####################################################################
########################## Private Data #############################

# Default mode to open file for writing (output or error output); means
# create new file or overwrite existing one.You can change the file open
# mode by using $dsOUTMODE & $dsERRMODE parameters hash keys when using
# the file name $dsOUTFILE & $dsERRFILE parameters accordingly
# or open the file by yourself and pass the file descriptor in those
# parameters (mode parameters are not handled in this case).
my $DEFAULT_MODE = '>';

# Hash to hold all parameters given into the command
my %dsPARAM_default = (
                       $dsRETVAL     => undef,
                       $dsSTDOUT     => $FALSE,
                       $dsOUT        => undef,
                       $dsOUTFILE    => undef,
                       $dsOUTMODE    => undef,
                       $dsERR        => undef,
                       $dsERRFILE    => undef,
                       $dsERRMODE    => undef,
                       $dsNOERROR    => undef,
                       $dsINTERACT   => $FALSE,
                       $dsCHDIR      => undef,
                       $dsSIGDEFAULT => $FALSE
                      );
my $PIPE_TMOUT = 50;  #timeout in seconds for waiting while message will be putted/received to/from pipe

#####################################################################
#################### Private Methods Declaration ####################

## @skipline
sub ds_write_file($$;$);
## @skipline
sub do_sys_send($$);
## @skipline
sub do_sys_recv($$);

#####################################################################
##################### Public Methods Declaration ####################

## @skipline
sub do_system($;$);
## @skipline
sub do_chdir($);
## @skipline
sub do_sudo($;$);

#####################################################################
########################## Private Methods ##########################

###------------------------------------------------------------------
# Description:
#  - Write an ARRAY into the file given by name or handle
###
#  INPUT :  1st - the reference to the ARRAY with data
#           2nd - the file name or handle
#           3d  - the mode to open the file with (optional)
#  OUTPUT:  $TRUE   - the data was saved into the file
#           $FALSE  - error case
###------------------------------------------------------------------
sub ds_write_file($$;$)
{
   my @data = @{$_[0]};
   my $fd   = $_[1];

   my $mode = (!defined $_[2]) ? $DEFAULT_MODE : $_[2];

   my $retval = $TRUE;

   if (ref $fd eq "GLOB")
   {
      baselog($LOG_DEBUG, "File descriptor is given '$fd'");

      # Dump the command output into the file
      print $fd @data;
   } ## end if (ref $fd eq "GLOB")
   elsif (ref $fd eq "")
   {
      baselog($LOG_DEBUG, "File name is given '$fd'");
      my $fname = $fd;
      $fd = undef;
      if (open($fd, $mode, $fname))
      {

         # Dump the command output into the file & close it
         print $fd @data;
         close($fd);
      } ## end if (open($fd, $mode, $fname...
      else
      {
         baselog($LOG_ERROR, "Cannot open file $fname.");
         $retval = $FALSE;
      }
   } ## end elsif (ref $fd eq "")
   else
   {
      baselog($LOG_ERROR, "Wrong file descriptor '$fd'.");
      $retval = $FALSE;
   }

   return $retval;
} ## end sub ds_write_file($$;$)

#####################################################################
########################## Public Methods ###########################

#######################################################################
## @fn do_chdir($directory)
# @brief Changes the working directory to $directory, if possible.
# @details Changes the working directory to $directory, if possible.
#          $ENV{PWD} will be kept up to date. If $directory is an empty
#          string, the subroutine changes the directory specified by
#          $ENV{HOME}.
# @param $directory - the directory for changing
# @return $ret - a string with the directory which was the current
#                before changing. If $directory is undefined, $ret will
#                be an empty line. If some error happens, $ret will be
#                undefined.
# @author
# @date
# @note
#######################################################################
sub do_chdir($)
{
   my $directory = shift @_;

   my $ret;

   if (defined $directory)
   {
      my $cwd = getcwd();

      if ($cwd)
      {
         baselog($LOG_DEBUG, "Change directory to '$directory'");

         if ($directory eq $dsHOMEDIR)
         {
            $directory = (getpwuid($<))[7];
            baselog($LOG_DEBUG, "The home directory for UID '$<' is '$directory'.");
         }

         if ($directory ne '')
         {
            if (chdir $directory)
            {
               $ret = $cwd;
            }
            else
            {
               baselog($LOG_ERROR, "Cannot change the directory to '$directory'.");
            }
         } ## end if ($directory ne '')
         else
         {
            baselog($LOG_ERROR, "Cannot get home directory for user with UID '$<'");
         }
      } ## end if ($cwd)
      else
      {
         baselog($LOG_ERROR, "Cannot get the current working directory.");
      }
   } ## end if (defined $directory...
   else
   {
      baselog($LOG_DEBUG, "The directory for changing is not specified.");
      $ret = '';
   }

   return $ret;
} ## end sub do_chdir($)

#######################################################################
## @fn do_system($rcmd, $rparam)
# @brief replacement for perl function system
# @details Execute command, allow to get or redirect the output of the
#          command and log an error if occurred
# @param $rcmd - the command to execute; may be a STRING or a reference
#                to ARRAY
# @param $rparam - optional parameter; may contain the reference
#                  to hash with additional parameters
# @return $TRUE   - the command has been executed successfully
# @return $FALSE  - error
#######################################################################
sub do_system($;$)
{
   my ($rcmd, $rparam) = @_;

   my %dsPARAM = %dsPARAM_default;

   # 0. Get the caller info for the function
   ###   For debug and error handling purposes
   my ($path, $line) = (caller)[1, 2];
   my $filename = substr $path, (1 + rindex($path, '/'));
   my $message  = "$filename($line)";

   # 1. Check parameters & fill out the $dsPARAM HASH
   ### 1st must be defined, 2nd must be a HASH if defined

   if (not defined $rcmd)
   {
      baselog($LOG_ERROR, "$message: First argument is undefined.");
      return $FALSE;
   }
   elsif (ref $rcmd eq "ARRAY")
   {
      if (scalar @{$rcmd} == 1 and !defined $rcmd->[0])
      {
         baselog($LOG_ERROR, "$message: First argument is the reference to undefined array.");
         return $FALSE;
      }
   } ## end elsif (ref $rcmd eq "ARRAY"...
   if (defined $rparam)
   {
      baselog($LOG_DEBUG, "$message: Parameters are defined.");
      unless (ref $rparam eq "HASH")
      {
         baselog($LOG_ERROR, "$message: Second argument must be a HASH.");
         return $FALSE;
      }
      foreach my $key (keys %{$rparam})
      {
         unless (exists $dsPARAM{$key})
         {
            baselog($LOG_ERROR, "$message: Key '$key' is not allowed in the parameters HASH.");
            return $FALSE;
         }
         $dsPARAM{$key} = $rparam->{$key};
      } ## end foreach my $key (keys %{$rparam...
   } ## end if (defined $rparam)

   # 2. Get the array with command (dereference an array reference
   ###   or split a string)

   my @command;
   if (ref $rcmd eq "ARRAY")
   {
      @command = @{$rcmd};
      baselog($LOG_DEBUG, "$message: command is an ARRAY.");
   }
   elsif (ref $rcmd eq "")
   {
      @command = split(/\s+/, $rcmd);
      baselog($LOG_DEBUG, "$message: command is a STRING.");
   }
   else
   {
      baselog($LOG_ERROR, "$message: Wrong first argument '$rcmd'");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "$message: Command: '@command'");

   my $ret;

   # 3. Execute the command[, capturing all outputs & errors]
   ###
   pipe(READER_OUT,       WRITER_OUT);
   pipe(READER_ERR,       WRITER_ERR);
   pipe(PROCESS_STATUS_R, PROCESS_STATUS_W);
   WRITER_OUT->autoflush(1);
   WRITER_ERR->autoflush(1);
   PROCESS_STATUS_W->autoflush(1);
   my @process_status = (undef, undef, undef, undef);  #service message: proc_condition|retval|stdout_chksum|stderr_chksum
   my @stdout         = ();
   my @stderr         = ();
   my $pid            = fork;

   if ($pid == 0)
   {                                                   #child process created successfully
      close READER_OUT;
      close READER_ERR;
      close PROCESS_STATUS_R;
      @process_status = ("$ERROR\n", "-1\n", "-1\n", "-1\n");
      my $old_cwd = do_chdir($dsPARAM{$dsCHDIR});

      # If some error happens during chdir
      unless (defined $old_cwd)
      {
         baselog($LOG_ERROR, "$message: The directory is not changed.");
         if (not do_sys_send(*PROCESS_STATUS_W, \@process_status))
         {
            baselog($LOG_ERROR, "$message: Cannot send data about proccess status.");
         }
      } ## end unless (defined $old_cwd)
      else
      {  #If directory is changed
         if ($dsPARAM{$dsSIGDEFAULT})
         {
            common_functions::signals_default();
         }
         my @streams = (\undef, \@stdout, \@stderr);
         if ($dsPARAM{$dsINTERACT})
         {
            @streams = (undef, undef, undef);
         }
         eval q#run3(\@command,@streams);#;
         if ($@ ne '')
         {  #if command not found or cannot be executed.
            baselog($LOG_ERROR, "$message: Cannot execute the command '@command': $@");
            if (not do_sys_send(*PROCESS_STATUS_W, \@process_status))
            {
               baselog($LOG_ERROR, "$message: Cannot send data about proccess status.");
            }
         } ## end if ($@ ne '')
         else
         {  #If transmitted to 'do_system' command is executed correctly
            $ret = WEXITSTATUS($?);
            @process_status = ("$OK\n", "$ret\n", scalar(@stdout) . "\n", scalar(@stderr) . "\n");
            if (do_sys_send(*PROCESS_STATUS_W, \@process_status))
            {
               unless ($dsPARAM{$dsINTERACT})
               {
                  foreach my $send_msg (@stdout)
                  {
                     if (not do_sys_send(*WRITER_OUT, $send_msg))
                     {
                        baselog($LOG_ERROR, "$message: Error was occured while sending STDOUT message from child.");
                        last;
                     }
                  } ## end foreach my $send_msg (@stdout...
                  foreach my $send_msg (@stderr)
                  {
                     if (not do_sys_send(*WRITER_ERR, $send_msg))
                     {
                        baselog($LOG_ERROR, "$message: Error was occured while sending STDERR message from child.");
                        last;
                     }
                  } ## end foreach my $send_msg (@stderr...
               } ## end unless ($dsPARAM{$dsINTERACT...
               do_chdir($old_cwd) if ($old_cwd);
            } ## end if (do_sys_send(*PROCESS_STATUS_W...
            else
            {
               baselog($LOG_ERROR, "$message: Cannot send data about proccess status.");
            }
         } ## end else [ if ($@ ne '')
      } ## end else

      close WRITER_OUT;
      close WRITER_ERR;
      close PROCESS_STATUS_W;

      #standard Perl function 'exit' executes all destructors(END section)
      #described in script and imported libraries. In our situation, we do not
      #want to execute destructors because this point is not exit from main script
      #but only from forked process which is full copy of parent process with
      #all variables, functions and destructors. _exit function from POSIX is used.
      _exit(0);
   } ## end if ($pid == 0)
   else
   {  #parent
      close WRITER_OUT;
      close WRITER_ERR;
      close PROCESS_STATUS_W;

      #waiting while child procces start to pass any data.
      my $rIN = '';
      my $rOUT;
      vec($rIN, fileno(PROCESS_STATUS_R), 1) = 1;
      select($rOUT = $rIN, undef, undef, undef);  #arguments: INfh,OUTfh,ERRfh,delay in seconds
      foreach my $field (@process_status)
      {
         if (not do_sys_recv(*PROCESS_STATUS_R, \$field))
         {
            baselog($LOG_ERROR, "$message: Pipe PROCESS_STATUS_R is broken.");
            close READER_OUT;
            close READER_ERR;
            close PROCESS_STATUS_R;
            return $FALSE;
         } ## end if (not do_sys_recv(*PROCESS_STATUS_R...
         unless (defined $field)
         {                                        #broken pipe
            baselog($LOG_ERROR, "$message: Cannot read data(process status) from child process. It seems that child proccess was killed.");
            close READER_OUT;
            close READER_ERR;
            close PROCESS_STATUS_R;
            return $FALSE;
         } ## end unless (defined $field)
         chomp $field;
      } ## end foreach my $field (@process_status...
      close PROCESS_STATUS_R;
      my $status = $process_status[0];
      $ret = $process_status[1];
      my $stdout_chksum = $process_status[2];
      my $stderr_chksum = $process_status[3];
      if ($status == $ERROR)
      {  #some error was occurred while transmitted to do_system command was executed.
         close READER_OUT;
         close READER_ERR;
         close PROCESS_STATUS_R;
         return $FALSE;
      } ## end if ($status == $ERROR)
         #3.1 collect the output data from executed command.
      unless ($dsPARAM{$dsINTERACT})
      {
         my $recv_msg;  #buffer for reading from pipe
         my $i;         #loop counter
         for ($i = 0 ; $i < $stdout_chksum ; $i++)
         {

            #logic is reverced. good case in 'else' branch.
            if (not do_sys_recv(*READER_OUT, \$recv_msg))
            {
               baselog($LOG_ERROR, "$message: Pipe READER_OUT is broken.");
               baselog($LOG_INFO,  "$message: '$i' of '$stdout_chksum' were received from READER_OUT.");
               close READER_OUT;
               close READER_ERR;
               return $FALSE;
            } ## end if (not do_sys_recv(*READER_OUT...
            elsif (not defined $recv_msg)
            {
               baselog($LOG_ERROR, "$message: Cannot read data(STDOUT) from child process. It seems that child proccess was killed.");
               baselog($LOG_INFO,  "$message: '$i' of '$stdout_chksum' were received from READER_OUT.");
               close READER_OUT;
               close READER_ERR;
               return $FALSE;
            } ## end elsif (not defined $recv_msg...
            else
            {
               push @stdout, $recv_msg;
            }
         } ## end for ($i = 0 ; $i < $stdout_chksum...
         for ($i = 0 ; $i < $stderr_chksum ; $i++)
         {
            if (not do_sys_recv(*READER_ERR, \$recv_msg))
            {
               baselog($LOG_ERROR, "$message: Pipe READER_ERR is broken.");
               baselog($LOG_INFO,  "$message: '$i' of '$stderr_chksum' were received from READER_ERR.");
               close READER_OUT;
               close READER_ERR;
               return $FALSE;
            } ## end if (not do_sys_recv(*READER_ERR...
            elsif (not defined $recv_msg)
            {
               baselog($LOG_ERROR, "$message: Cannot read data(STDERR) from child process. It seems that child proccess was killed.");
               baselog($LOG_INFO,  "$message: '$i' of '$stderr_chksum' were received from READER_ERR.");
               close READER_OUT;
               close READER_ERR;
               return $FALSE;
            } ## end elsif (not defined $recv_msg...
            else
            {
               push @stderr, $recv_msg;
            }
         } ## end for ($i = 0 ; $i < $stderr_chksum...
      } ## end unless ($dsPARAM{$dsINTERACT...
      close READER_OUT;
      close READER_ERR;
      waitpid($pid, 0);
   } ## end else [ if ($pid == 0)

   baselog($LOG_DEBUG, "$message: system returns exit status '$ret'");

   unless ($dsPARAM{$dsINTERACT})
   {

      # 4. Handle other parameters from $dsPARAM
      ###

      # 4.1 Handle the output
      if (defined $dsPARAM{$dsOUT} or defined $dsPARAM{$dsOUTFILE} or ($dsPARAM{$dsSTDOUT}))
      {
         if (@stdout)
         {
            baselog($LOG_DEBUG, "$message: The output is present.");

            # Send the output to STDOUT
            if ($dsPARAM{$dsSTDOUT})
            {
               baselog($LOG_DEBUG, "$message: The '$dsSTDOUT' parameter is '$dsPARAM{$dsSTDOUT}'");
               print @stdout;
            }

            # Return the output
            if (defined $dsPARAM{$dsOUT})
            {
               baselog($LOG_DEBUG, "$message: The '$dsOUT' parameter is defined");
               @{$dsPARAM{$dsOUT}} = @stdout;
            }

            # Write the output into the given file
            if (defined $dsPARAM{$dsOUTFILE})
            {
               baselog($LOG_DEBUG, "$message: The '$dsOUTFILE' parameter is defined");
               unless (ds_write_file(\@stdout, $dsPARAM{$dsOUTFILE}, $dsPARAM{$dsOUTMODE}))
               {
                  baselog($LOG_ERROR, "$message: Cannot save the output into the file.");
                  return $FALSE;
               }
            } ## end if (defined $dsPARAM{$dsOUTFILE...
         } ## end if (@stdout)
      } ## end if (defined $dsPARAM{$dsOUT...

      # 4.2 Handle ERROR output
      if (defined $dsPARAM{$dsERR} or defined $dsPARAM{$dsERRFILE})
      {
         if (@stderr)
         {
            baselog($LOG_DEBUG, "$message: The error output is present");

            # Return the error output
            if (defined $dsPARAM{$dsERR})
            {
               baselog($LOG_DEBUG, "$message: The '$dsERR' parameter is defined");
               @{$dsPARAM{$dsERR}} = @stderr;
            }

            # Write the error output into the given file
            if (defined $dsPARAM{$dsERRFILE})
            {
               baselog($LOG_DEBUG, "$message: The '$dsERRFILE' parameter is defined");
               unless (ds_write_file(\@stderr, $dsPARAM{$dsERRFILE}, $dsPARAM{$dsERRMODE}))
               {
                  baselog($LOG_ERROR, "$message: Cannot save the error output into the file.");
                  return $FALSE;
               }
            } ## end if (defined $dsPARAM{$dsERRFILE...
         } ## end if (@stderr)
      } ## end if (defined $dsPARAM{$dsERR...
   } ## end unless ($dsPARAM{$dsINTERACT...

   # Set return value for the subroutine (it can be changed later
   ### if the $dsNOERROR parameter is set to 'true'
   my $retval = ($ret == 0) ? $TRUE : $FALSE;

   # 4.3 Return the exit status of the command
   if (defined $dsPARAM{$dsRETVAL})
   {
      baselog($LOG_DEBUG, "$message: The '$dsRETVAL' parameter is defined");
      ${$rparam->{$dsRETVAL}} = $ret;
   }

   # 4.4 Handle $NOERROR parameter
   if ((defined $dsPARAM{$dsNOERROR}) and ($dsPARAM{$dsNOERROR} == $TRUE))
   {
      baselog($LOG_DEBUG, "$message: Return value was $retval. Setting to 'true'.");
      $retval = $TRUE;
   }
   elsif (!$retval)
   {
      baselog($LOG_ERROR, "$message: Command '" . join(' ', @command) . "' return value is $ret.");
   }
   return $retval;
} ## end sub do_system($;$)

###############################################################################
## @fn do_sudo($rcmd;$rparam)
# @brief   Check if the command needs to be executed through sudo and run it.
# @details This method check the current user and if it is not root it invokes
#          the command by using sudo. Otherwise it invokes the command directly.
#          To run the command (even through sudo) do_system is used, so the
#          method accepts the same parameters as do_system.
# @param   $rcmd - reference to an array which contain the command and its
#                  arguments
# @param   $rparam - reference to a hash with option which must be used for
#                    running the command in do_system (please refer to
#                    do_system method)
# @return  TRUE - the command has been executed successfully
# @return  FALSE - some error happens
# @note    This method does not accept the command as the string whereas
#          do_system does. The method assumes that the command is needed to be
#          performed under root account, it does not intended to invoke the
#          command through sudo under with user switching. It is in
#          responsibility of the developer to check that the current user is
#          allowed to invoke the command through sudo, the corresponding
#          configuration must be made in /etc/sudoers file
sub do_sudo($;$)
{
   my ($rcmd, $rparam) = @_;

   my ($path, $line) = (caller)[1, 2];
   my $filename = common_functions::get_basename($path);
   my $message  = "$filename($line)";

   if (not common_functions::is_array($rcmd))
   {
      baselog($LOG_ERROR, "$message: 'Command' argument is undefined.");
      return $FALSE;
   }

   if (scalar @{$rcmd} == 1 and !defined $rcmd->[0])
   {
      baselog($LOG_ERROR, "$message: 'Command' argument is the reference to undefined array.");
      return $FALSE;
   }

   if (common_functions::is_right_user($ROOT_ACCT) == $FALSE)
   {
      baselog($LOG_DEBUG, "$message: sudo is used for invocation '@{$rcmd}'.");
      $rcmd = [$SUDO, @{$rcmd}];
   }
   else
   {
      baselog($LOG_DEBUG, "$message: root is attempting to invoke '@{$rcmd}'.");
   }

   my $retval = do_system($rcmd, $rparam);

   if (not $retval)
   {
      baselog($LOG_ERROR, "$message: Failed attempt to invoke '@{$rcmd}'.");
   }

   return $retval;
} ## end sub do_sudo($;$)

#######################################################################
## @fn do_sys_send($FH, $m_send)
# @brief Send message to the specified pipe(or file)
# @param $FH - reference to the pipe
# @param $m_send - message which must be sent(can be reference to array).
# @return $TRUE   - message was sent successfully
# @return $FALSE  - error
#######################################################################
sub do_sys_send($$)
{
   my $FH     = shift @_;
   my $m_send = shift @_;
   my $arr    = [];
   if (not defined $FH)
   {
      baselog($LOG_ERROR, "Pipe name is missing.");
      return $FALSE;
   }

   #check for message to send.
   if (not defined $m_send)
   {
      baselog($LOG_ERROR, "Message for sending is missing.");
      return $FALSE;
   }
   elsif (common_functions::is_array($m_send))
   {
      $arr = $m_send;
   }
   else
   {
      push @{$arr}, $m_send;
   }

   #sending message.
   foreach my $mess (@{$arr})
   {
      eval {
         local $SIG{ALRM} = sub {die};
         alarm $PIPE_TMOUT;
         print $FH $mess;
         alarm 0;
      };
      if ($@)
      {
         baselog($LOG_ERROR, "Time is out for sending message in pipe.");
         return $FALSE;
      }
   } ## end foreach my $mess (@{$arr})
   return $TRUE;
} ## end sub do_sys_send($$)

#######################################################################
## @fn do_sys_recv($FH, $buff)
# @brief Receive messages from specified pipe(or file)
# @param $FH - reference to the pipe
# @param $m_recv - reference to scalar where will be stored received message.
# @return $TRUE   - message was received successfully
# @return $FALSE  - error
#######################################################################
sub do_sys_recv($$)
{
   my $FH     = shift @_;
   my $m_recv = shift @_;
   if (not defined $FH)
   {
      baselog($LOG_ERROR, "Pipe name is missed.");
      return $FALSE;
   }
   eval {
      local $SIG{ALRM} = sub {die};
      alarm $PIPE_TMOUT;
      $$m_recv = <$FH>;
      alarm 0;
   };
   if ($@)
   {
      baselog($LOG_ERROR, "Time is out for reading message from pipe.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub do_sys_recv($$)

1;

__END__

=head1 NAME

do_system - Executes command, allow to get or redirect the output of the  command and log an error if occurred

=head1 SYNOPSIS

use base_system;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
