#############################################################
## @file base_timedate.pm
# @brief Interface for time adjustment
# @details # This module is the LinuxBase interface
#            for the system clock parameters changing.
#            It deals with the XML DB files.
# @author Maxim Uvarov
# @date December 2006
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @note
#############################################################
package base_timedate;

use lib "/opt/nortel/base/lib";

use common_variables;
use install_common;

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
  &clock_parameters_restore
  &clock_adjustment_set
  &clock_adjustment_get
  &clock_file_save
  &print_clock_params
  &reset_clock_params
  $SEC_TICK
  $MSEC_PTICK);
our $VERSION = 1.01;

use ifxml;
use baselog;
use XML::Writer;
use IO::File;
use Data::Dumper;
use XMLRef;
use warnings;
use strict;

#####################################################################
##################### Public Methods Declaration ####################
sub clock_parameters_restore;
sub clock_adjustment_set($;$);
sub clock_adjustment_get;
sub clock_file_save($;$;$);
sub print_clock_params;

# timeadj DB file
my $CLOCK_XML_FILE_NAME = "adjtime.xml";
my $CLOCK_BKUP          = qq|$SYSDB_ROOT/$CLOCK_XML_FILE_NAME|;
my $CLOCK_SECTION       = 'timeadj';
my $CLOCK_TAG           = 'adjparam';

# frequency offset mask
my $ADJ_FREQUENCY = 0x0002;

# tick mask
my $ADJ_TICK = 0x4000;

# system call
my $SYS_adjtimex = 124;

# Increasing tick by 1 speeds up the system clock by about 8.64 sec/day
our $SEC_TICK = 8.639136;

# microseconds that should be added to the system time for each kernel tick interrupt
our $MSEC_PTICK = 10000;

my $CLOCK_BUF = "\0" x 1000;

my $PACK_TEMPLATE = "illllillllll";

my $TICK_PARM = 11;
my $FREQ_PARM = 2;

## @fn clock_parameters_restore
# @brief This subroutine reads system clock parameters saved in XML DB
# @details This subroutine reads system clock parameters saved in XML DB and apply them
#  on system clock
# @param None
# @return $TRUE - on success
# @return $FALSE - on error
# @author Maxim Uvarov
# @date May 2008
sub clock_parameters_restore
{

   # 1. Get XML parser
   my $parser = ifxml::new($SYSDB_ROOT, $CLOCK_XML_FILE_NAME);
   if (!$parser)
   {
      baselog($LOG_ERROR, "Cannot read timeadj XML DB file:$SYSDB_ROOT/$CLOCK_XML_FILE_NAME");
      return $TRUE;  # Return TRUE even if no such file for backward cap.
   }

   # 2. Get clock params
   my $params = $parser->find_data_by_xpath($GETDATA, "/$CLOCK_TAG");

   if ((!defined $params) || (scalar(@{$params}) == 0))
   {
      baselog($LOG_INFO, "There are no time adjustment params in XML DB file");
      undef $parser;
      return $FALSE;
   } ## end if ((!defined $params)...

   if ($params == 1)
   {
      my $value = @{$params}[0];
      if (clock_adjustment_set($value->{'tick'}, $value->{'freq'}) != $TRUE)
      {
         baselog($LOG_ERROR, "Failed setup time correction, check $CLOCK_BKUP");
         undef $parser;
         return $FALSE;
      } ## end if (clock_adjustment_set...
   } ## end if ($params == 1)
   undef $parser;
   return $TRUE;
} ## end sub clock_parameters_restore

## @fn clock_adjustment_set($;$)
# @brief This subroutine set system clock parameters
# @details This subroutine set system clock parameters
# @param None
# @return $TRUE - on success
# @return $FALSE - on error
# @author Maxim Uvarov
# @date May 2008
sub clock_adjustment_set($;$)
{
   my $argc = scalar(@_);

   if ($argc != 2)
   {
      baselog($LOG_ERROR, "Wrong number of arguments: $argc");
      return $FALSE;
   }

   my ($adj_tick, $adj_freq) = @_;

   # Pack ticks and frequency values to buffer
   $CLOCK_BUF = pack($PACK_TEMPLATE, ($ADJ_TICK | $ADJ_FREQUENCY), 0, $adj_freq, 0, 0, 0, 0, 0, 0, 0, 0, $adj_tick);

   # Adjust system clock
   if (syscall($SYS_adjtimex, $CLOCK_BUF) == -1)
   {
      baselog($LOG_ERROR, "SYS_adjtimes call failed\n");
      return $FALSE;
   }

   return $TRUE;
} ## end sub clock_adjustment_set($;$)

## @fn clock_adjustment_get
# @brief This subroutine gets system clock parameters
# @details This subroutine gets system clock parameters
# @param None
# @return tick
# @return freq
# @author Maxim Uvarov
# @date May 2008
sub clock_adjustment_get
{
   my @systime;

   # Read system time
   if (syscall($SYS_adjtimex, $CLOCK_BUF) == -1)
   {
      baselog($LOG_ERROR, "SYS_adjtimes call failed\n");
      return (-1, -1);
   }
   @systime = unpack $PACK_TEMPLATE, $CLOCK_BUF;

   return ($systime[$TICK_PARM], $systime[$FREQ_PARM]);
} ## end sub clock_adjustment_get

## @fn clock_file_save($;$;$)
# @brief This subroutine saves backup data in the file
# @details This subroutine saves backup data in the file
# @param tick, freq, drift
# @return $TRUE  - file is saved sucesssfully
# @return $FALSE - some error happens
# @author Maxim Uvarov
# @date May 2008
sub clock_file_save($;$;$)
{
   my $argc  = scalar(@_);
   my $found = "";

   if ($argc != 3)
   {
      baselog($LOG_ERROR, "Wrong number of arguments: $argc");
      return $FALSE;
   }

   my ($save_tick, $save_freq, $save_drift) = @_;
   my $dstfile = ">" . $CLOCK_BKUP;
   my $output  = new IO::File($dstfile);
   my $xml     =
     new XML::Writer(
                     DATA_MODE   => 'true',
                     DATA_INDENT => 3,
                     OUTPUT      => $output
                    );
   if (!$xml)
   {
      baselog($LOG_ERROR, "Cannot save clock parameters.");
      return $FALSE;
   }

   $xml->startTag($CLOCK_SECTION);

   $xml->emptyTag(
                  $CLOCK_TAG,
                  'tick'  => "$save_tick",
                  'freq'  => "$save_freq",
                  'drift' => "$save_drift"
                 );

   $xml->endTag($CLOCK_SECTION);
   $xml->end();

   return $TRUE;
} ## end sub clock_file_save($;$;$)

## @fn print_clock_params
# @brief This subroutine print current system clock adjustment
# @details This subroutine print current system clock adjustment
# @param None
# @return $TRUE  - sucesss
# @return $FALSE - some error happens
# @author Maxim Uvarov
# @date May 2008
sub print_clock_params
{
   my ($current_tick, $current_freq) = clock_adjustment_get();

   if ($current_tick == -1 || $current_freq == -1)
   {
      printf "\nCannot get clock parameters\n";
      return $FALSE;
   }

   # Calculate current time adjustment (sec per day)
   my $current_time_correction = ($MSEC_PTICK - $current_tick) * $SEC_TICK - ($current_freq * ($SEC_TICK / 100)) / (2**16);

   printf "\nSystem clock parameters:\ncorrection: %.3f [sec]      tick: %d      freq: %d\n\n", $current_time_correction, $current_tick, $current_freq;

   return $TRUE;
} ## end sub print_clock_params

## @fn reset_clock_params()
# @brief This subroutine check entered drift value
# @details This subroutine reset system clock parameters and update xml db
# @param None
# @return $TRUE - on success
# @return $FALSE - on error
# @author Maxim Uvarov
# @date May 2008
sub reset_clock_params()
{

   # Make the correction
   if (clock_adjustment_set($MSEC_PTICK, 0) != $TRUE)
   {
      baselog($LOG_ERROR, "Failed setup time correction", $TRUE);
      return $FALSE;
   }

   ## Save parameter in xml to survive reboot
   if (clock_file_save($MSEC_PTICK, 0, 0) != $TRUE)
   {
      baselog($LOG_ERROR, "Save parameters failed", $TRUE);
      return $FALSE;
   }

   return $TRUE;
} ## end sub reset_clock_params()

1;
__END__
