#!/usr/bin/perl

## @file base_validate.pm
# @brief This module contains basic validation functions.
# @details
# @author Ekaterina Bazhanova 
# @date   October 2008
# @version 1.00 - Initial version


package base_validate;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;

our @ISA    = qw(Exporter);

our @EXPORT =
  qw(&validateHostname &validateFQDN &validateDomain);

our $VERSION = "1.00";

## @skipline
sub validateHostname($);
## @skipline
sub validateFQDN($);
## @skipline
sub validateDomain($);

###############################################################################
## @fn    validateHostname
# @brief  Makes sure the user entered hostname follows the
#             defined hostname rules (taken from network.py in
#             the usr/lib/anaconda directory)
# @param  $hostname - hostname to check
# @return $TRUE  - hostname is correct
# @return $FALSE - hostname is not correct
sub validateHostname($)
{
   my $hostname = shift(@_);

   # Hostname must start with a valid character in the range 'a-z' or 'A-Z'
   if ($hostname !~ /^[a-zA-Z]/)
   {
      return $FALSE;
   }

   # Hostnames can only contain the characters 'a-z', 'A-Z', '0-9', '-'
   if ($hostname =~ /[^a-zA-Z-0-9]/)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub validateHostname()

###############################################################################
## @fn    validateFQDN
# @brief  Makes sure the user entered fqdn follows the
#         defined hostname rules (taken from network.py in
#         the usr/lib/anaconda directory) It must have at
#         least one dots inside.
# @param  $fqdn  - fqdn to check
# @return $TRUE  - fqdn is correct
# @return $FALSE - fqdn is not correct
sub validateFQDN($)
{
   my $fqdn = shift(@_);
   my $hostname = undef;
   my $domain   = undef;
   if ($fqdn =~ /([^.]*)\.(.*)/)
   {
      $hostname = $1;
      $domain   = $2;
   }
   else
   {
      return $FALSE;
   }   
   unless (validateHostname($hostname))
   {
      return $FALSE;    
   }
   unless (validateDomain($domain))
   {
      return $FALSE;    
   }  
   return $TRUE;
} ## end sub validateFQDN()

###############################################################################
## @fn    validateDomain
# @brief  Makes sure the user entered domain follows the
#             defined hostname rules (taken from network.py in
#             the usr/lib/anaconda directory)
# @param  $domain - domain to check
# @return $TRUE   - domain is correct
# @return $FALSE  - domain is not correct
sub validateDomain($)
{
   my $domain = shift(@_);

   # Domain must start with a valid character in the range 'a-z' or 'A-Z' or digits
   if ($domain !~ /^[a-zA-Z0-9]/)
   {
      return $FALSE;
   }

   # domain can only contain the characters 'a-z', 'A-Z', '0-9', '-', or '.'
   if ($domain =~ /[^a-zA-Z-\.0-9]/)
   {
      return $FALSE;
   }
   my @parts = split(/\./, $domain);

   # There should be at least one dot in domain
   if (scalar(@parts) < 2)
   {
      return $FALSE;
   }

   if ($domain =~ /\.\./)
   {
      return $FALSE;
   }

   return $TRUE;
} ## end sub validateDomain()

