###############################################################################
## @file   base_xml.pm
# @brief   Linux Base binding for libxml
# @details This module implements the class providing the actions for for
#          reading and changing XML files.
# @author  Ekaterina Bazhanova
# @author  Alexander Kolotov
# @author  Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - Added handling 2 sitaution when hardening storage does not exist and
#                 when hardening storage element does not exist
# @version 1.03 - 'getAttrsForElement' has been added.
# @version 1.04 - Empty XML tags text is now '', not undef.
# @version 1.05 - "if (-w $out_file)" has been added before toFile($out_file)

package base_xml;

use strict;
use warnings;
use common_variables;
use common_functions;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw($FILE_DOES_NOT_EXIST $ELEMENT_DOES_NOT_EXIST);
our $VERSION = '1.05';

use File::Basename;
use XML::LibXML;
use baselog;

our $FILE_DOES_NOT_EXIST    = 'file_does_not_exist';
our $ELEMENT_DOES_NOT_EXIST = 'element_does_not_exist';

## @skipline
sub new($$;$$);
## @skipline
sub save($);
## @skipline
sub addElements($$$$$);
## @skipline
sub deleteElement($$$);
## @skipline
sub findElements($$;$);
## @skipline
sub changeAttrs($$$$);
## @skipline
sub changeText($$$$);
## @skipline
sub getText($$$);
## @skipline
sub getAttrs($$);
## @skipline
sub element_exists($$);
## @skipline
sub getFileName($);
## @skipline
sub getOutFileName($);
## @skipline
sub getAttrsforallElements($$);
## @skipline
sub getAttrsForElement($$);

my $ELEMENT_NODE = '1';
my $TEXT_NODE    = '3';

###############################################################################
## @cmethod sub new($class, $xml_file, $out_file)
# @brief   Constructor of base_xml
# @details This class method creates an object and fills necessary object fields.
# @param   $class    - class name
# @param   $xml_file - full name of input xml file, if no file name is
#                      specified, a new XML document will be created in the
#                      memory.
# @param   $out_file - full name of xml file where new data can be written. It
#                      is optional, if it is not defined, the changes will be
#                      put into input xml file.
# @return  $self - reference to object
# @return  undef - indicates an error condition.
sub new($$;$$)
{
   my ($class, $xml_file, $out_file, $extra_options) = @_;

   my $doc;

   my $self = {};

   if (not defined $class)
   {
      baselog($LOG_ERROR, 'Class is undefined.');
      return undef;
   }

   if (not defined $extra_options)
   {
      $extra_options = {};
   }

   if (not is_hash($extra_options))
   {
      baselog($LOG_ERROR, 'Extra options container is not correct');
      return undef;
   }

   $self->{'extra_options'} = $extra_options;

   if (defined $xml_file)
   {
      if (not -f $xml_file)
      {
         if (defined $self->{'extra_options'}->{$FILE_DOES_NOT_EXIST})
         {

            # Try to handle the situation when the file does not exist.
            # The handler needs to operate with the document descriptor

            baselog($LOG_DEBUG, "File '$xml_file' not found, trying to invoke handler");

            if (invoke_subroutine_safely($self->{'extra_options'}->{$FILE_DOES_NOT_EXIST}, [\$doc]) != $TRUE)
            {
               baselog($LOG_ERROR, "Cannot find the file '$xml_file'");
               baselog($LOG_ERROR, "Handler '$FILE_DOES_NOT_EXIST' failed.");
               return undef;
            } ## end if (invoke_subroutine_safely...
         } ## end if (defined $self->{'extra_options'...
         else
         {
            baselog($LOG_ERROR, "Cannot find the file '$xml_file'");
            return undef;
         }
      } ## end if (not -f $xml_file)
      else
      {
         if (not -r $xml_file)
         {
            baselog($LOG_ERROR, "The file '$xml_file' is not readable");
            return undef;
         }
      } ## end else [ if (not -f $xml_file)
   } ## end if (defined $xml_file)
   else
   {
      $xml_file = "";
   }

   if (not defined $out_file)
   {
      if ($xml_file ne "")
      {
         $out_file = $xml_file;
      }
      else
      {
         baselog($LOG_NOTICE, "Both files IN and OUT are undefined.");
         $out_file = "";
      }
   } ## end if (not defined $out_file...
   else
   {
      if (not basename($out_file))
      {
         baselog($LOG_ERROR, "Could not write '$out_file'.");
         return undef;
      }
   } ## end else [ if (not defined $out_file...

   baselog($LOG_DEBUG, "Trying to create an instance of base_xml class: read from '$xml_file', write to '$out_file'.");

   #There is an input XML file
   if (not defined $doc)
   {
      if ($xml_file ne "")
      {
         my $parser = XML::LibXML->new();
         if (not defined $parser)
         {
            baselog($LOG_ERROR, 'Cannot create parser object.');
            return undef;
         }

         baselog($LOG_DEBUG, "Trying to parse the XML file.");

         eval q{$doc = $parser->parse_file($xml_file)};
         if ($@)
         {
            baselog($LOG_ERROR, "Cannot parse the XML file: $@");
            undef $parser;
            return undef;
         } ## end if ($@)

         $self->{'new_document'} = $FALSE;
      } ## end if ($xml_file ne "")
      else
      {
         baselog($LOG_DEBUG, "New document is created.");
         $doc = XML::LibXML->createDocument;

         $self->{'new_document'} = $TRUE;
      } ## end else [ if ($xml_file ne "")
   } ## end if (not defined $doc)

   $self->{'file'} = $xml_file;
   $self->{'doc'}  = $doc;
   $self->{'out'}  = $out_file;

   bless($self, $class);

   return $self;
}  ##end sub new($$)

###############################################################################
## @cmethod sub DESTROY($self)
# @brief   Object destructor.
# @details This method destroies an instance of the base_xml class. Prior to
#          destroying the class member $self->{'doc'} containing XML Document
#          is destroyed as well.
# @param   $self  - reference to object.
# @return  None
sub DESTROY
{
   my $self = shift @_;

   baselog($LOG_DEBUG, "Destroing the instanse of base_xml");

   undef $self->{'doc'};

   return;
}  ##end sub DESTROY

###############################################################################
## @method sub save($self)
# @brief   Saves XML data.
# @details Save XML data to the file defined as outfile during the instance
#          initialization.
# @param   $self  - reference to object.
# @return  TRUE - data are saved.
# @return  FALSE - some error happens
sub save($)
{
   my $self = shift @_;

   #will add ignorable whitespaces, so the nodes content is easier to read.
   #Existing text nodes will not be altered
   my $fileFormat = '1';

   my $retval = $FALSE;

   my $out_file = $self->{'out'};

   baselog($LOG_DEBUG, "Trying to save XML data into file '$out_file'.");

   if ((defined $out_file) and ($out_file ne ""))
   {
      my $out_dir = dirname($out_file);

      #Check if it is possible to write to the directory.
      if (-d $out_dir)
      {
         if (-w $out_dir)
         {
            if ((-e $out_file) && (not -w $out_file))
            {
               baselog($LOG_ERROR, "It will be impossible to save '$out_file' as this file is not writable.");
            }
            else
            {
               eval q{$self->{'doc'}->toFile($out_file, $fileFormat)};
               if (not $@)
               {
                  $retval = $TRUE;
               }
               else
               {
                  baselog($LOG_ERROR, "Cannot write the XML to file '$out_file': $@");
               }
            } ## end else [ if ((-e $out_file) && ...
         } ## end if (-w $out_dir)
         else
         {
            baselog($LOG_ERROR, "It will be impossible to save '$out_file' as the '$out_dir' is not writable.");
         }
      } ## end if (-d $out_dir)
      else
      {
         baselog($LOG_ERROR, "It will be impossible to save '$out_file' as the '$out_dir' does not exist.");
      }
   } ## end if ((defined $out_file...
   else
   {
      baselog($LOG_ERROR, "The output file is not specified.");
   }

   return $retval;
}  ##end sub save($)

###############################################################################
## @method sub addElements($self, $parents, $parentattrs, $child, $childattrs)
# @brief   Adds element to parent nodes.
# @details This method adds child element into every found parent elements.
#          XPath is used to find all parents. If a hash which describes required
#          attributes of the parent elements is provided, the corresponding
#          parents attributes are checking and if they coincide the child
#          element is added to it .
# @param   $self    - reference to object
# @param   $parents - XPath to parent nodes
# @param   $parentattrs - reference to attribute hash describing the parent
#                         node (can be undefined)
# @param   $child   - name of the added node
# @param   $childattrs - reference to hash describing child's attributes
# @return  TRUE    - if some elements were added
# @return  FALSE   - indicates an error condition.
sub addElements($$$$$)
{
   my ($self, $parents, $parentattrs, $child, $childattrs) = @_;

   if ((not defined $parents) or ($parents eq ""))
   {
      baselog($LOG_ERROR, 'XPath to parent nodes is not specified.');
      return $FALSE;
   }

   if (($self->{'new_document'}) and ($parents ne "/"))
   {
      baselog($LOG_ERROR, 'It is a new document. The root element does not exist.');
      return $FALSE;
   }

   if ((defined $parentattrs) and (not is_hash($parentattrs)))
   {
      baselog($LOG_ERROR, 'Parent attributes are in invalid form.');
      return $FALSE;
   }

   if ((not defined $child) or ($child eq ""))
   {
      baselog($LOG_ERROR, 'The name of child node is not specified.');
      return $FALSE;
   }

   if (defined $childattrs)
   {
      if (not is_hash($childattrs))
      {
         baselog($LOG_ERROR, 'Child attributes are in invalid form.');
         return $FALSE;
      }
   } ## end if (defined $childattrs...

   #For new document it is neccessary just add root element
   if (not $self->{'new_document'})
   {
      baselog($LOG_DEBUG, "Trying to add new element '$child' to all parents '$parents'.");

      # find parent elements by XPath and required attributes
      my $elements = findElements($self, $parents, $parentattrs);
      if (not defined $elements)
      {
         baselog($LOG_ERROR, "Cannot add any elements");
         return $FALSE;
      }

      my $node;

      #add element into every of the parent elements
      foreach my $elem (@{$elements})
      {
         $node = $self->{'doc'}->createElement($child);

         if (defined $childattrs)
         {
            foreach my $key (keys %{$childattrs})
            {
               $node->setAttribute($key, $childattrs->{$key});
            }
         } ## end if (defined $childattrs...
         $elem->addChild($node);

         baselog($LOG_DEBUG, "Added '" . $node->nodePath() . "' element.");
      } ## end foreach my $elem (@{$elements...
   } ## end if (not $self->{'new_document'...
   else
   {
      baselog($LOG_DEBUG, "Trying to add root element '$child'.");

      my $root = $self->{'doc'}->createElement($child);
      if (defined $childattrs)
      {
         foreach my $key (keys %{$childattrs})
         {
            $root->setAttribute($key, $childattrs->{$key});
         }
      } ## end if (defined $childattrs...
      $self->{'doc'}->setDocumentElement($root);

      $self->{'new_document'} = $FALSE;
   } ## end else [ if (not $self->{'new_document'...

   return $TRUE;
}  ##end sub addElem

###############################################################################
## @method sub deleteElement($self, $XPath, $attrs)
# @brief   Deletes elements
# @details This method deletes elements which were specified by XPath
#          name and the hash describing attributes which the the deleted
#          elements should have
# @param   $self     - reference to object
# @param   $XPath    - XPath to nodes
# @param   $attrs    - reference to attribute hash
# @return  TRUE     - if some elements were deleted
# @return  FALSE    - indicates an error condition.
sub deleteElement($$$)
{
   my ($self, $XPath, $attrs) = @_;

   my $parent;

   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return $FALSE;
   }

   if (!defined $XPath)
   {
      baselog($LOG_ERROR, 'Path for node search is undefined.');
      return $FALSE;
   }

   #the list of the elements for deletion
   my $childs = findElements($self, $XPath, $attrs);
   if (!defined $childs)
   {
      baselog($LOG_ERROR, "Cannot delete any elements");
      return $FALSE;
   }

   foreach my $child (@$childs)
   {
      $parent = $child->parentNode();
      $parent->removeChild($child);
   }
   return $TRUE;
} ## end sub deleteElement($$$)

###############################################################################
## @method sub findElements($self, $XPath, $attrs)
# @brief   Looks for elements
# @details This method looks for XML elements (by XPath) which have the same
#          attributes as the hash $attrs
# @param   $self  - reference to object
# @param   $XPath - XPath to elements
# @param   $attrs - reference to attribute hash
# @return  \\\@findElements - reference to the list of found elements
# @return  undef          - indicates an error condition.
sub findElements($$;$)
{
   my ($self, $XPath, $attrs) = @_;

   my @findElements = ();

   if ((not defined $XPath) or ($XPath eq ""))
   {
      baselog($LOG_ERROR, 'Path to elements is not defined.');
      return undef;
   }

   if ((defined $attrs) and (not is_hash($attrs)))
   {
      baselog($LOG_ERROR, "The reference is not reference to HASH with attributes.");
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to find elements with XPath '$XPath'.");

   if ($self->{'new_document'})
   {
      baselog($LOG_WARNING, "The document is empty.");
      return \@findElements;
   }

   #Extract parten elements from XPath
   my @Parents = split('/', $XPath);
   @Parents = reverse @Parents;

   baselog($LOG_DEBUG, "XPath members: '@Parents'.");

   #if the first parent was not provide (XPath is "/")
   #then starting with root element of the xml file
   my $first_parent = shift @Parents;
   if (not $first_parent)
   {
      my $root = $self->{'doc'}->documentElement();
      $first_parent = $root->nodeName();
   }

   foreach my $elem ($self->{'doc'}->getElementsByTagName($first_parent))
   {
      baselog($LOG_DEBUG, "'" . $elem->nodePath() . "' is found.");

      #check list of the element parents
      my $rootParent     = $elem;
      my $parentNode     = $rootParent->parentNode();
      my $correctParents = $TRUE;
      foreach my $subParent (@Parents)
      {

         #if parent node is not an element break the cycle
         if ($parentNode->nodeType ne $ELEMENT_NODE)
         {
            last;
         }

         #Check if XPath are the same
         my $parentName = $parentNode->nodeName;
         if ($parentName eq $subParent)
         {

            #get next parent node
            $parentNode = $parentNode->parentNode();
         } ## end if ($parentName eq $subParent...
         else
         {

            #XPaths are different
            $correctParents = $FALSE;
            last;
         } ## end else [ if ($parentName eq $subParent...
         $correctParents = $TRUE;
      } ## end foreach my $subParent (@Parents...

      #if sequence of the parents are correct for current element
      #add element to the list of the required elements
      if (not $correctParents)
      {
         baselog($LOG_DEBUG, "XPath is not matched.");
         next;
      }

      if (defined $attrs)
      {

         #checking attributes
         my $correctAttr = $TRUE;
         foreach my $key (keys %{$attrs})
         {

            #if an attribute does not exist in the XML element, mthe element is
            #considered as non-mached
            my $xml_attribute = $elem->getAttribute($key);
            if (not $xml_attribute)
            {
               baselog($LOG_DEBUG, "It does not contain the attribute '$key'.");
               $correctAttr = $FALSE;
               last;
            } ## end if (not $xml_attribute...

            #if value for the current input attribute does not exist
            #then this attribure is skipped
            if (not defined $attrs->{$key})
            {
               baselog($LOG_DEBUG, "The attribute '$key' found but skipped.");
               next;
            }

            if ($attrs->{$key} ne $xml_attribute)
            {
               baselog($LOG_DEBUG, "The attribute '$key' does not match: '$attrs->{$key}' ne '$xml_attribute'");
               $correctAttr = $FALSE;
               last;
            } ## end if ($attrs->{$key} ne ...
         }  ##end foreach my $key...

         # If input attributes does not match with current element, check next element
         if (not $correctAttr) {next;}
      } ## end if (defined $attrs)
      else
      {
         baselog($LOG_DEBUG, "Attributes are not defined. They are not checked.");
      }

      baselog($LOG_DEBUG, "The element is OK.");
      push @findElements, $elem;
   }  ##end foreach my $elem

   if (not scalar @findElements)
   {
      baselog($LOG_DEBUG, "Cannot find elements");
   }

   #return the list of the required elements
   return \@findElements;
} ## end sub findElements($$;$)

###############################################################################
## @method sub changeAttrs($self, $XPath, $oldattrs, $newattrs)
# @brief   Changes or/and adds attributes
# @details This method allows changing existed attributes or to add new
#          attributes.
#          If the actual number of attributes of XML element is not equal the
#          number of attributes requested to be changed, then
#        \li the method adds new attribute if there is not such attribute in
#            the element,
#        \li the method changes the attribute if it is already in element and
#            it is in the hash with new valued of attributes,
#        \li the method keeps the attribute as is in the element if it exists
#            only in the element but not in the hash.
# @param   $self     - reference to object
# @param   $XPath    - XPath to node for changing of attributes
# @param   $oldattrs - reference to the old attribute hash of the node
# @param   $newattrs - reference to the new attribute hash of the node
# @return  TRUE  - if attributes were changed
# @return  FALSE - indicates an error condition.
sub changeAttrs($$$$)
{
   my ($self, $XPath, $oldattrs, $newattrs) = @_;

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return undef;
   }

   if ((not defined $XPath) or ($XPath eq ""))
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return $FALSE;
   }

   if ((defined $oldattrs) and (not is_hash($oldattrs)))
   {
      baselog($LOG_ERROR, "The reference is not reference to HASH with old attributes.");
      return $FALSE;
   }

   if ((defined $newattrs) and (not is_hash($newattrs)))
   {
      baselog($LOG_ERROR, "The reference is not reference to HASH with new attributes.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to change attributes for the node '$XPath'");

   #list of required elements for changing attributes
   my $elements = findElements($self, $XPath, $oldattrs);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot change attributes for the node '$XPath'.");
      return $FALSE;
   }

   if (not scalar(@$elements))
   {
      baselog($LOG_INFO, "The node '$XPath' is not found.");
      return $FALSE;
   }

   foreach my $elem (@$elements)
   {
      baselog($LOG_DEBUG, "Changing attributes for '" . $elem->nodePath() . "' node.");

      foreach my $key (keys %{$newattrs})
      {

         #if this attribute exists than change value else add attribute
         if ($elem->hasAttribute($key))
         {
            baselog($LOG_DEBUG, "Attribute name: '$key', old value: '" . $elem->getAttribute($key) . "', new value: '$newattrs->{$key}'");

            my $attr = $elem->getAttributeNode($key);
            $attr->setValue($newattrs->{$key});
         } ## end if ($elem->hasAttribute...
         else
         {
            baselog($LOG_DEBUG, "Attribute '$key' does not exist. Creating...");

            $elem->setAttribute($key, $newattrs->{$key});
         } ## end else [ if ($elem->hasAttribute...
      }  #end foreach my $key...
   }  #end foreach my $elem

   return $TRUE;
}  ##end sub changeAttr($$$$)

###############################################################################
## @method sub changeText($self, $XPath, $attrs, $newText)
# @brief   Changes text belonging of XML tags
# @details This method changes text nodes value of the required elements
#          or add text node if it was not exist.
# @param   $self     - reference to object
# @param   $XPath    - full path to node for changing of child text node value
# @param   $attrs    - reference to the attribute hash of the node
# @param   $newText  - new value for the text node
# @return  TRUE     -  if the text was changed successfully
# @return  FALSE    -  indicates an error condition.
sub changeText($$$$)
{
   my ($self, $XPath, $attrs, $newText) = @_;
   my $elements;

   if (!defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return $FALSE;
   }

   if (!defined $XPath)
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return $FALSE;
   }

   if (!defined $newText)
   {
      baselog($LOG_ERROR, 'New text is undefined.');
      return $FALSE;
   }

   #list of required elements for the text changing
   $elements = findElements($self, $XPath, $attrs);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot change text for the node '$XPath'.");
      return $FALSE;
   }

   if (not scalar(@$elements))
   {
      baselog($LOG_INFO, "The node '$XPath' is not found.");
      return $FALSE;
   }

   #change text of the each child text node of the current element
   foreach my $elem (@$elements)
   {
      my @childnodes = $elem->childNodes;
      my $addedText  = $FALSE;
      foreach my $child (@childnodes)
      {
         if ($child->nodeType() eq $TEXT_NODE)
         {

            #replace old text data to new text data
            $child->setData($newText);
            $addedText = $TRUE;
         } ## end if ($child->nodeType()...
      } ## end foreach my $child (@childnodes...

      #if where was no text node at current element add the new text node
      unless ($addedText)
      {
         $elem->appendText($newText);
      }
   }  ##end foreach my $elem...
   return $TRUE;
} ## end sub changeText($$$$)

###############################################################################
## @method sub getText($self, $XPath, $attrs)
# @brief   Gets text of tag
# @details This method returns text of provided node if it was found
# @param   $self     - reference to object
# @param   $XPath    - full path to node to search
# @param   $attrs    - reference to the attribute hash of the node
# @return  text      - found banner text
# @return  undef     - indicates an error condition.
sub getText($$$)
{
   my ($self, $XPath, $attrs) = @_;
   my $elements;

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return undef;
   }

   if (not defined $XPath)
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return undef;
   }

   #list of required elements for the text changing
   $elements = findElements($self, $XPath, $attrs);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot get text from the node '$XPath'.");
      return undef;
   }
   if (not scalar(@$elements))
   {
      baselog($LOG_INFO, "The node '$XPath' is not found.");
      return undef;
   }

   #get text of the child text node of the current element
   my $text = '';
   foreach my $elem (@$elements)
   {
      my @childnodes = $elem->childNodes;
      foreach my $child (@childnodes)
      {
         if ($child->nodeType() eq $TEXT_NODE)
         {
            return $child->data;
         }
      } ## end foreach my $child (@childnodes...
   }  ##end foreach my $elem...
   return $text;
} ## end sub getText($$$)

###############################################################################
## @method sub getAttrs($self, $XPath)
# @brief   Gets attributes of the XML node
# @details This method returns a hash reference with values of attributes found
#          for the provided node. If several nodes are found by the XPath only
#          the first one is used.
# @param   $self  - reference to object
# @param   $XPath - XPath to node, which contains required attributes
# @return  hash reference - if attributes were found
# @return  undef          - some error happens
sub getAttrs($$)
{
   my ($self, $XPath) = @_;

   my %ret_hash = ();

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return undef;
   }

   if ((not defined $XPath) or ($XPath eq ""))
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to get attributes for the node '$XPath'");

   #list of required elements
   my $elements = $self->findElements($XPath);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the node '$XPath'.");
      return undef;
   }

   if (not scalar(@$elements))
   {
      if (defined $self->{'extra_options'}->{$ELEMENT_DOES_NOT_EXIST})
      {

         baselog($LOG_DEBUG, "The node '$XPath' not found, trying to invoke handler");

         if (invoke_subroutine_safely($self->{'extra_options'}->{$ELEMENT_DOES_NOT_EXIST}, [$self, $XPath, \$elements]) != $TRUE)
         {
            baselog($LOG_ERROR, "The node '$XPath' is not found.");
            baselog($LOG_ERROR, "Handler '$ELEMENT_DOES_NOT_EXIST' failed.");
            return undef;
         } ## end if (invoke_subroutine_safely...
      } ## end if (defined $self->{'extra_options'...
      else
      {
         baselog($LOG_ERROR, "The node '$XPath' is not found.");
         return undef;
      }
   } ## end if (not scalar(@$elements...

   my $elem = $elements->[0];
   my $nodeName;

   baselog($LOG_DEBUG, "Getting attributes for '" . $elem->nodePath() . "' node.");
   my @attributelist = $elem->attributes();
   if (not scalar @attributelist)
   {
      baselog($LOG_INFO, "There are no attributes for '" . $elem->nodePath() . "' node.");
   }
   else
   {
      foreach my $name (@attributelist)
      {
         $nodeName = $name->nodeName;
         $ret_hash{$nodeName} = $elem->getAttribute($nodeName);
         baselog($LOG_DEBUG, "attr name: '$nodeName', attr value: '$ret_hash{$nodeName}'");
      } ## end foreach my $name (@attributelist...
   } ## end else [ if (not scalar @attributelist...

   return \%ret_hash;
} ## end sub getAttrs($$)

###############################################################################
## @method sub element_exists($self, $XPath)
# @brief   Checks if the node is in the XML structure
# @details This method uses findElements subroutine and checks if the node
#          exists in the XML structure. If it is, the count of such nodes is
#          returned.
# @param   $self  - reference to object
# @param   $XPath - XPath to node, which contains required attributes
# @return  $retval > 0 - the node exists
# @return  $retval = 0 - the node does not exist
# @return  $retval < 0 - some error happens
sub element_exists($$)
{
   my ($self, $XPath) = @_;

   my $retval = -1;

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return $retval;
   }

   if ((not defined $XPath) or ($XPath eq ""))
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return $retval;
   }

   baselog($LOG_DEBUG, "Trying to find node '$XPath'");

   #list of required elements
   my $elements = $self->findElements($XPath);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot get find the node '$XPath'.");
      return $retval;
   }

   if (scalar(@$elements))
   {
      $retval = scalar(@$elements);
   }
   else
   {
      $retval = 0;
      baselog($LOG_DEBUG, "The node '$XPath' does not exists.");
   }

   return $retval;
} ## end sub element_exists($$)

###############################################################################
## @method sub getFileName($self)
# @brief   Provides the name of the work file
# @details This method returns a value of {'file'} memeber of the object
# @param   $self  - reference to object
# @return  full path to the work file
# @return  empty string - if the method executes without reference to the object
sub getFileName($)
{
   my $self = shift @_;

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return "";
   }

   baselog($LOG_DEBUG, qq/IN-file: '$self->{"file"}'/);

   return $self->{'file'};
} ## end sub getFileName($)

###############################################################################
## @method sub getOutFileName($self)
# @brief   Provides the name of the file where XML data should be saved
# @details This method returns a value of {'out'} memeber of the object
# @param   $self  - reference to object
# @return  full path to the out file
# @return  empty string - if the method executes without reference to the object
sub getOutFileName($)
{
   my $self = shift @_;

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return "";
   }

   baselog($LOG_DEBUG, qq/OUT-file: '$self->{"out"}'/);

   return $self->{'out'};
} ## end sub getOutFileName($)

###############################################################################
## @method sub getAttrsforallElements($self, $XPath)
# @brief   Gets attributes of the XML node
# @details This method returns a hash reference with values of attributes found
#          for the given nodes.
#
#          NOTE: It concatenates attributes of all elements found, e.g if there are
#          \code <tag1 attr='123'/> \endcode and
#          \code <tag1 attr2='345'/> \endcode - the method returns
#          \code {'attr'=>'123', 'attr2'=>'345'} \endcode
# @param   $self  - reference to object
# @param   $XPath - XPath to node, which contains required attributes
# @return  array reference - if attributes were found
# @return  undef          - some error happens
sub getAttrsforallElements($$)
{
   my ($self, $XPath) = @_;

   my @ret_arr = ();

   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Object is undefined.');
      return undef;
   }

   if ((not defined $XPath) or ($XPath eq ""))
   {
      baselog($LOG_ERROR, 'Path for search is undefined.');
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to get attributes for the node '$XPath'");

   #list of required elements
   my $elements = $self->findElements($XPath);
   if (not defined $elements)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the node '$XPath'.");
      return undef;
   }
   if (not scalar(@$elements))
   {
      baselog($LOG_DEBUG, "The node '$XPath' is not found.");
      return \@ret_arr;
   }
   else
   {
      my $nodeName;
      my $i = 0;
      foreach my $elem (@$elements)
      {
         baselog($LOG_DEBUG, "Getting attributes for '" . $elem->nodePath() . "' node.");
         my @attributelist = $elem->attributes();
         if (not scalar @attributelist)
         {
            baselog($LOG_INFO, "There are no attributes for '" . $elem->nodePath() . "' node.");
         }
         else
         {
            foreach my $name (@attributelist)
            {
               $nodeName = $name->nodeName;
               my %tmp_hash;
               $tmp_hash{$nodeName} = $elem->getAttribute($nodeName);
               $ret_arr[$i] = \%tmp_hash;
               baselog($LOG_DEBUG, "attr name: '$nodeName', attr value: '$tmp_hash{$nodeName}'");
               $i++;
            } ## end foreach my $name (@attributelist...
         } ## end else [ if (not scalar @attributelist...
      } ## end foreach my $elem (@$elements...
   } ## end else [ if (not scalar(@$elements...

   return \@ret_arr;
} ## end sub getAttrsforallElements($$)

###############################################################################
## @method sub getAttrsForElement($self, $element)
# @brief   returns reference to the hash with attributes of the element.
# @param   $self - reference to object
# @param   $elem - XML::LibXML::Element object
# @return  reference to a hash with attributes
sub getAttrsForElement($$)
{
   my ($self, $elem) = @_;
   my $nodeName;
   my @attributelist = $elem->attributes();
   my %tmp_hash      = ();
   if (not scalar @attributelist)
   {
      baselog($LOG_INFO, "There are no attributes for '" . $elem->nodePath() . "' node.");
   }
   else
   {
      foreach my $name (@attributelist)
      {
         $nodeName = $name->nodeName;
         $tmp_hash{$nodeName} = $elem->getAttribute($nodeName);
      }
   } ## end else [ if (not scalar @attributelist...
   return \%tmp_hash;
} ## end sub getAttrsForElement($$)

sub getReferenceToXMLParser($)
{
   my $self = shift @_;

   return $self->{'doc'};
}

sub setReferenceToXMLParser($$)
{
   my ($self, $new_doc) = @_;

   $self->{'doc'} = $new_doc;
}

1;
__END__
