## @file    basedb.pm
# @brief    LinuxBase Database module
# @details  This module is LinuxBase interface to the most common
#           action with Berkley DB.
# @author   Petr Kascheev
# @version  1.00 - Initial version
# @version  1.01 - Two colons were removed before shared functions and variables.
# @version  1.02 - Now basedb_create using change_permissions_and_ownership proc
#                  instead of system() calls for 'CHMOD' and 'CHOWN'
# @version  1.03 -  New function is added: basedb_sync().

################################################################
package basedb;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;

use DB_File;
use Fcntl qw (:flock);
use baselog;

# Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&basedb_lock &basedb_unlock &basedb_create &basedb_remove $PATH_DB &basedb_sync);

our $VERSION = 1.03;

our $PATH_DB = "/var/opt/nortel/base/databases";

INIT
{
   unless (-e $PATH_DB)
   {
      unless (mkdir $PATH_DB, 0700)
      {
         baselog($LOG_ERROR, "$PATH_DB can not be created: $!\n", $TRUE);
         exit($ERROR);
      }
      if (chown($ROOT_UID, $NORTEL_GID, $PATH_DB) != 1)
      {
         baselog($LOG_ERROR, 'Cannot change owner for DB directory.');
         exit($ERROR);
      }
      if (chmod(0770, $PATH_DB) != 1)
      {
         baselog($LOG_ERROR, 'Cannot change mode for DB directory.');
         exit($ERROR);
      }
   } ## end unless (-e $PATH_DB)
} ## end INIT

##################################################################
####################### Public Methods ###########################

##################################################################
# Function: basedb_lock
# Description: Tries to open db file,
# lock it and tie it with hash.
#
# INPUT:  $_[0] - db file to lock
#
# OUTPUT: file descriptor - db file locked
#         Hashreference associated with basedb file
#         OR
#         undef - can not open/lock/tie file
###################################################################
sub basedb_lock
{
   my $db = shift @_;
   my ($desc, %hash_db) = undef;

   unless (open($desc, "+<$db"))
   {
      baselog($LOG_ERROR, "Can not open $db db");
      return undef;
   }

   unless (flock($desc, LOCK_EX))
   {
      baselog($LOG_ERROR, "Can not lock $db db");
      close($desc);
      return undef;
   } ## end unless (flock($desc, LOCK_EX...

   unless (tie(%hash_db, 'DB_File', $db, O_RDWR, 0666))
   {
      baselog($LOG_ERROR, "Cannot tie $db: $!");
      close($desc);
      return undef;
   } ## end unless (tie(%hash_db, 'DB_File'...

   return $desc, \%hash_db;
} ## end sub basedb_lock

##################################################################
# Function: basedb_unlock
# Description: Tries to untie hash, which was associated with
# db file,  then unlock db file
#
# INPUT:  $_[0] - file descriptor
#         $_[1] - hash reference
#
# OUTPUT: $TRUE  - file unlocked
#         $FALSE - can not unlock/untie file
###################################################################
sub basedb_unlock
{
   my ($desc, $hash_db) = @_;

   my $ret = $FALSE;

   unless (untie($hash_db))
   {
      baselog($LOG_ERROR, "Cannot untie hash");
      return $ret;
   }

   $ret = flock($desc, LOCK_UN);
   close($desc);

   return $ret;
} ## end sub basedb_unlock

########################################################################
## @fn basedb_create($db_file, $owner, $group, $perms)
# @brief   Create database file.
# @details This subroutine checks if database file exists if it does not,
#          create it.
#
# @param  $db_file - full name of new database file
# @param  $owner   - owner of the new database file (optional).
#                    If parameter is omitted then file will have the
#                    same owner as the running process (caller).
# @param   $group  - group of the new database file (optional).
#                    If parameter is omitted then file will have the
#                    same group as the running process (caller).
# @param   $perms  - permissions of the new database file (optional).
#                    If parameter is omitted then file will have '600'
#                    permissions mode.
#
# @return $TRUE  - database file already exists or has been created
#                  successfully
# @return $FALSE - can not create database file
########################################################################
sub basedb_create
{
   my ($db_file, $owner, $group, $perms) = @_;
   unless (defined $perms)
   {
      $perms = 0600;  # by default just r/w for owner
   }

   # Create database file
   unless (-e $db_file)
   {
      system($TOUCH, $db_file);

      if (($? >> 8) == $ERROR)
      {
         baselog($LOG_ERROR, "Can not create $db_file");
         return $FALSE;
      }
      baselog($LOG_DEBUG, "$db_file is created");
      if (change_permissions_and_ownership($perms, $owner, $group, [$db_file], undef, $NO_LOG))
      {
         baselog($LOG_DEBUG, "Permissions successfully changed to $perms for $db_file.");
         if (defined $owner)
         {
            baselog($LOG_DEBUG, "Owner successfully changed to $owner for $db_file.");
         }
         if (defined $group)
         {
            baselog($LOG_DEBUG, "Group successfully changed to $group for $db_file.");
         }
      } ## end if (change_permissions_and_ownership...
      else
      {
         baselog($LOG_ERROR, "Cannot change owner and permissions for $db_file.");
         return $FALSE;
      }
   } ## end unless (-e $db_file)

   return $TRUE;
} ## end sub basedb_create

########################################################################
# This subroutine waits for unlocking the DB file and then remove it
#  INPUT: $_[0] - db file
# OUTPUT: $TRUE  - database file is removed successfully
#         $FALSE - database file is not removed
########################################################################
sub basedb_remove
{
   my $db   = shift @_;
   my $desc = undef;

   unless (open($desc, "+<$db"))
   {
      baselog($LOG_ERROR, "Can not open $db db");
      return $FALSE;
   }

   unless (flock($desc, LOCK_EX))
   {
      baselog($LOG_ERROR, "Can not lock $db db");
      close($desc);
      return $FALSE;
   } ## end unless (flock($desc, LOCK_EX...

   close($desc);

   if (system($RM, '-r', $db))
   {
      baselog($LOG_ERROR, "Can not remove $db db");
      return $FALSE;
   }

   return $TRUE;
} ## end sub basedb_remove

########################################################################
## @fn basedb_sync($hash_db)
# @brief   This function enforces flush changes to the disk.
# @param  $hash_db - hash reference associated with database file.
# @return $TRUE  - sync operation applied.
# @return $FALSE - error condition
########################################################################
sub basedb_sync($)
{
   my $hash_db = shift @_;

   if (not is_hash($hash_db))
   {
      baselog($LOG_ERROR, 'Database hash is not defined or input type is invalid.');
      return $FALSE;
   }

   # enforce flush cached changes to disk to decrease timeout between changes in memory and
   # actual changes in the database file.
   # do not check return status as it is not documented in the perldoc.
   tied(%$hash_db)->sync();

   return $TRUE;
} ## end sub basedb_sync($)
__END__

=head1 NAME

basedb - common data for data base management tools.

=head1 SYNOPSIS

use basedb;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

