####################################################################
#
## @file baselog.pm
# @brief LinuxBase base log module.
# @details This module is LinuxBase interface for standard
#          Linux syslog providing some more functionality
# @author Alex Arkhipov
# @date December 2006
# @version 1.00 - Initial version
# @version 2.00 - Rewrited to be based on syslog_config.pm
#                 by Alexander Khoroshilov
# @version 2.01 - Two colons were removed before shared functions and variables.
# @version 2.02 - Remove common_functions.pm from include list
# @version 2.03 - baselog() function has been changed - PID has been added into log message
####################################################################
package baselog;

use warnings;
use strict;

use lib "/opt/nortel/base/lib";

use common_variables;

# Access management
use Exporter;
our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(&baselog &set_subsystem &set_logfile
  $LOG_NONE $LOG_EMERG $LOG_ALERT $LOG_CRIT
  $LOG_ERROR $LOG_WARNING $LOG_NOTICE $LOG_INFO $LOG_DEBUG
  );

use Sys::Syslog qw(:DEFAULT setlogsock);
use File::Basename;
use syslog_config;

our $VERSION = "2.03";

# Define SYSLOG levels
our $LOG_NONE    = -1;
our $LOG_EMERG   = 0;
our $LOG_ALERT   = 1;
our $LOG_CRIT    = 2;
our $LOG_ERROR   = 3;
our $LOG_WARNING = 4;
our $LOG_NOTICE  = 5;
our $LOG_INFO    = 6;
our $LOG_DEBUG   = 7;

# Hash containing syslog levels and their names
my %log_levels = (
                  $LOG_NONE    => 'NONE',
                  $LOG_EMERG   => 'EMERG',
                  $LOG_ALERT   => 'ALERT',
                  $LOG_CRIT    => 'CRIT',
                  $LOG_ERROR   => 'ERROR',
                  $LOG_WARNING => 'WARNING',
                  $LOG_NOTICE  => 'NOTICE',
                  $LOG_INFO    => 'INFO',
                  $LOG_DEBUG   => 'DEBUG'
                 );

# Constants
my $LINUXBASE_PROCESS         = 'linuxbase';  # linux base process
my $BASELOG_DEFAULT_FACILITY  = 'local7';     # baselog default facility
my $BASELOG_DEFAULT_SUBSYSTEM = 'Base';       # baselog default subsystem
my $BASELOG_DEFAULT_LEVEL     = $LOG_INFO;    # baselog default level

# Variables
my $facility         = undef;                   # baselog facility
my $cur_subsys       = undef;                   # current subsystem
my $cur_level        = $BASELOG_DEFAULT_LEVEL;  # current level
my $stage            = undef;                   # execution stage: install - runtime
my $install_log_file = undef;                   # log file for installation stage

INIT
{
   init();
}

###################################################################
########################### Private Methods #######################
###################################################################

## @method init()
# @brief Module initialization.
# @details Do the initialization of the baselog subsystem.
# @param None
# @return None
# @author Alex Arkhipov
# @date December 2006
sub init()
{
   $facility   = $BASELOG_DEFAULT_FACILITY;
   $cur_subsys = $BASELOG_DEFAULT_SUBSYSTEM;

   get_stage();

   if ($stage eq "install")
   {

      #preinstall stage and postinstall stage
      if (baselog_is_debug_mode("/mnt/source/extra/nortel-linuxbase-info", "/admin/instfiles/extra/nortel-linuxbase-info"))
      {

         #set baselog level to DEBUG
         $cur_level = $LOG_DEBUG;
      } ## end if (baselog_is_debug_mode...
   } ## end if ($stage eq "install"...
   else
   {
      my $file = new syslog_config($SYSLOG_CFG_FILE);
      unless (defined($file))
      {
         baselog($LOG_ERROR, "Can not create syslog_config object for $SYSLOG_CFG_FILE\n", $TRUE);
         return;
      }
      my $fac = $file->get_facility($LINUXBASE_PROCESS);
      if (defined($fac))
      {
         $facility = 'local' . $fac;
      }
      else
      {
         baselog($LOG_ERROR, "Can not get facility for $LINUXBASE_PROCESS\n", $TRUE);
      }
      my $level = $file->get_level($LINUXBASE_PROCESS, $cur_subsys);
      if (defined($level))
      {
         $cur_level = $level;
      }
      else
      {
         baselog($LOG_ERROR, "Can not get level for $cur_subsys subsystem\n", $TRUE);
      }
   } ## end else [ if ($stage eq "install"...
} ## end sub init()

## @method get_stage()
# @brief Get stage.
# @details Determine the execution stage by reading kernel
#  parameter and set $stage variable to one of the following values:
#  "install" - means that log is written to file
#  "runtime" - means that syslog is used
# @param None
# @return None
# @author Alex Arkhipov
# @date December 2006
sub get_stage()
{

   return if (defined($stage));  # has already defined before
   my $cmdLine;
   $stage = "runtime";
   unless (open($cmdLine, "<", "/proc/cmdline"))
   {
      print "Can not open /proc/cmdline file for reading\n";
      $stage = "install";        # install is more "safe"
      return;
   } ## end unless (open($cmdLine, "<"...
   my @file = <$cmdLine>;
   close($cmdLine);
   foreach my $line (@file)
   {
      if ($line =~ /installation_stage/)
      {
         $stage = "install";
         last;
      }
   } ## end foreach my $line (@file)
} ## end sub get_stage()

###################################################################
########################### Public Methods ########################
###################################################################

## @method set_logfile($)
# @brief Set logfile.
# @details Set logfile. Is used during installation stage only.
# @param $_[0] - logfile name
# @return None
# @author Alex Arkhipov
# @date December 2006
sub set_logfile($)
{
   if ($stage ne "install")
   {
      print "baselog::set_logfile() can be invoked ";
      print "during 'install' stage only\n";
      baselog($LOG_WARNING, "baselog::set_logfile() is invoked");
   } ## end if ($stage ne "install"...
   $install_log_file = shift(@_);
   baselog($LOG_DEBUG, "Set logfile - $install_log_file");
} ## end sub set_logfile($)

## @method set_subsystem($)
# @brief Set subsystem.
# @details Set current subsystem.
# @param $_[0] - subsystem name (should belong to $SYSLOG_INI_FILE)
# @return None
# @author Alex Arkhipov
# @date December 2006
sub set_subsystem($)
{
   my $subsys = shift(@_);

   my $file = new syslog_config($SYSLOG_CFG_FILE);
   unless (defined($file))
   {
      baselog($LOG_ERROR, "Can not create syslog_config object for $SYSLOG_CFG_FILE\n", $TRUE);
      return;
   }
   my $level = $file->get_level($LINUXBASE_PROCESS, $subsys);
   unless (defined($level))
   {
      baselog($LOG_ERROR, "Incorrect subsystem provided: $subsys\n", $TRUE);
      return;
   }

   $cur_subsys = $subsys;
   $cur_level  = $level;
   baselog($LOG_DEBUG, "Subsystem set: $cur_subsys, level: $cur_level");
} ## end sub set_subsystem($)

## @method baselog($$;$)
# @brief Log message.
# @details Log message to the appropriate location which
#  depends on $stage variable.
# @param $_[0] - priority (has to be one of syslog levels)
# @param $_[1] - message to log
# @param $_[2] - flag whether print message to the screen
#                $TRUE  - print message to the screen also
#                $FALSE - log message only, but not print
# @return None
# @author Alex Arkhipov
# @date December 2006
sub baselog($$;$)
{
   my ($pri, $message, $is_print) = @_;
   chomp($message);
   $is_print = $FALSE if (!defined($is_print));

   # Print message to the screen
   print "$message\n" if ($is_print);

   if ((!defined $pri) or ($pri =~ /\D/))
   {
      print "baselog: Incorrect priority provided\n";
      return;
   }

   # Check for priority
   if ($pri > $cur_level or $pri < $LOG_EMERG)
   {
      return;
   }

   # Get file and line of the caller script
   my ($path, $line) = (caller)[1, 2];

   # Get file name without path
   my $filename    = basename($path);
   my $pri_str     = $log_levels{$pri};
   my $log_message = "($pri_str) $cur_subsys: $filename($line): PID[$$]: $message";

   if ($stage eq 'install')
   {
      if (!defined($install_log_file))
      {
         print "Install log file is undefined. Call set_logfile() first.\n";
         return;
      }

      # Open log file for append (exit if error)
      my $file;
      unless (open($file, ">>", $install_log_file))
      {
         print "Can not open " . $install_log_file . " file for writing\n";
         return;
      }

      # Turn on auto flushing
      select((select($file), $| = 1)[0]);

      # Write message to the file (including end of line)
      my $time = localtime;
      print $file "$time $LINUXBASE_PROCESS: $log_message\n";

      # Close file
      close($file);
   } ## end if ($stage eq 'install'...
   else  # runtime stage
   {
      setlogsock('unix');
      openlog($LINUXBASE_PROCESS, 'ndelay', $facility);
      $pri_str = lc($pri_str);
      $pri_str =~ s/error/err/;
      syslog($pri_str, $log_message);
      closelog();
   } ## end else [ if ($stage eq 'install'...
} ## end sub baselog($$;$)

## @fn baselog_is_debug_mode
# @brief Function checks if the debug mode should be used or not
# @details Function was originaly copied from common_functions.pm module
#          to not include it in baselog.pm (to prevent inter including)
# @param  with out parameter search in file - $LBINFO
# @param   file1  - file to search the string "DEBUG_MODE=1"
#          file2
#           ...
#
# @result $TRUE  - the searching string found at least in one file / debug mode
# @result $FALSE - no string found in files / usual mode
sub baselog_is_debug_mode
{
   my (@files) = @_;
   my $retval = $FALSE;

   if (!@files)
   {
      open INFO, "<$LBINFO" or die "Can't open $LBINFO file: $!";
      $retval = $TRUE if (grep /^$DEBUG_MODE$/, <INFO>);
      close INFO;
   } ## end if (!@files)
   else
   {
      foreach my $file (@files)
      {
         open INFO, "<$file" or next;
         $retval = $TRUE if (grep /^$DEBUG_MODE$/, <INFO>);
         close INFO;
      } ## end foreach my $file (@files)
   } ## end else [ if (!@files)

   return $retval;

} ## end sub baselog_is_debug_mode

1;

__END__

=pod

=head1 NAME

B<baselog.pm>

=head1 SYNOPSIS

B<use baselog.pm;>

=head1 DESCRIPTION

This module is LinuxBase interface for standard
Linux syslog providing some more functionality.

=head1 SEE ALSO

B<syslog_config.pm> package,
B<syslogShow>, B<syslogFacilitySet> and B<syslogLevelSet> commands.

=cut

