###############################################################################
## @file bios_upgrade.pm
# @brief Contains functions for work with BIOS ROM
# @details Today it is the wrapper for flashrom utility
# @author Vladimir Ivanov
# @date March 2009
# @version 1.00 - Initial version
# @note TODO: DEBUG support should be added to save the output of the flashrom
#
package bios_upgrade;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use common_variables;
use common_functions;
use CommonInstallUtils;
use base_semctl;
use base_system;
use baselog;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
  check_image
  read_image
  write_image
  take_biosupgrade_semaphore
  release_biosupgrade_semaphore
  );
our $VERSION = '1.00';

########################## Private Data #######################################

my $IMAGE_SIZE_512K  = 524288;                              # bytes (1024 * 1024 / 2)
my $IMAGE_SIZE_1024K = 1048576;                             # bytes (1024 * 1024)
my $BIOS_IMAGE_FILE  = "/tmp/bios.rom.image.$$";
my $LAYOUT_FILE      = "$BASE_DATA_DIR/cppm_v1_rom.layout";

# time to wait for semaphore
my $TIME_TO_WAIT = 5;

#################### Private Functions Declaration ############################

## @skipline
sub check_rom_image_file($);
## @skipline
sub read_rom_image_into_file($);
## @skipline
sub write_rom_image_from_file($);
## @skipline
sub get_file_size_in_bytes($);
## @skipline
sub update_rom_image_file($);
## @skipline
sub cleanup();

#################### Public Functions Declaration #############################

## @skipline
sub check_image($);
## @skipline
sub read_image($);
## @skipline
sub write_image($);
## @skipline
sub take_biosupgrade_semaphore();
## @skipline
sub release_biosupgrade_semaphore($);

############################ Private Functions ################################

## @fn check_rom_image_file($image)
# @brief   Check the BIOS image file
# @details Check the supported image size only
# @param $image - name of the image file
# @return  $TRUE - the image file passed the check and can be applied
# @return  $FALSE - the image file cannot be applied
sub check_rom_image_file($)
{
   my $image = shift @_;

   my $retval = $FALSE;
   if (not defined $image)
   {
      baselog($LOG_ERROR, "BIOS image file is not defined");
   }
   elsif (not -f $image)
   {
      baselog($LOG_ERROR, "The image file '$image' does not exist");
   }
   else
   {
      my $image_size = get_file_size_in_bytes($image);
      if ($image_size < 0)
      {
         baselog($LOG_ERROR, "Cannot get size of the image file '$image'.");
      }
      elsif ($image_size == $IMAGE_SIZE_512K)
      {
         $retval = $TRUE;
      }
      elsif ($image_size == $IMAGE_SIZE_1024K)
      {
         $retval = $TRUE;
      }
      else
      {
         baselog($LOG_ERROR, "Size of the image '$image' must be equal to $IMAGE_SIZE_512K or $IMAGE_SIZE_1024K bytes.");
      }
   } ## end else [ if (not defined $image...
   return $retval;
} ## end sub check_rom_image_file($)

## @fn  read_rom_image_into_file($image)
# @brief  Read flash ROM contents and save them into the given file.
# @details Check the existatnce of the image target directory
# @param $image - the image file to read the BIOS ROM into
# @return  $TRUE - flash ROM image is saved successfully
# @return  $FALSE - an error occurred
# @note This function must be protected by SEM_BIOSUPGRADE semaphore
sub read_rom_image_into_file($)
{
   my $image  = shift @_;
   my $retval = $FALSE;
   if (not defined $image)
   {
      baselog($LOG_ERROR, "BIOS image file parameter is not defined");
   }
   else
   {
      my $image_path = get_realpath($image);
      my $image_dir  = get_dirname($image_path);
      if (not -d $image_dir)
      {
         baselog($LOG_ERROR, "Cannot create BIOS image file in the non-existing directory '$image_dir'");
      }
      elsif (-f $image_path)
      {
         baselog($LOG_ERROR, "Cannot overwrite file '$image_path'");
      }
      else
      {
         if (not do_system([$FLASHROM, '--read', $image_path]))
         {
            baselog($LOG_ERROR, "Cannot read BIOS ROM into the file '$image_path'");
         }
         elsif (not change_permissions_and_ownership(0660, $ROOT_UID, $NORTEL_GID, [$image_path]))
         {
            baselog($LOG_ERROR, "Cannot change permissions for the file '$image_path'");
         }
         else
         {
            $retval = $TRUE;
         }
      } ## end else [ if (not -d $image_dir)
   } ## end else [ if (not defined $image...
   return $retval;
} ## end sub read_rom_image_into_file($)

## @fn write_rom_image_from_file($image)
# @brief  Write the ROM image into the BIOS ROM
# @details Generate the default ROM layout file, update BIOS ROM image with
#          zeros to 1M size, write the image and clean up
# @param  $image - name of the image file
# @return  $TRUE - the image is written successfully
# @return  $FALSE - an error occurred
# @note This function must be protected by SEM_BIOSUPGRADE semaphore
sub write_rom_image_from_file($)
{
   my $image  = shift @_;
   my $retval = $FALSE;
   if (not defined $image)
   {
      baselog($LOG_ERROR, "BIOS ROM image file is not defined");
   }
   else
   {
      if (not update_rom_image_file($image))
      {
         baselog($LOG_ERROR, "Cannot update the BIOS ROM image file '$image'");
      }
      else
      {
         if (not do_system([$FLASHROM, '--write', '--layout', $LAYOUT_FILE, '--image', 'normal', $BIOS_IMAGE_FILE]))
         {
            baselog($LOG_ERROR, "Cannot write the image file '$image' into the BIOS ROM");
         }
         else
         {
            $retval = $TRUE;
         }
         cleanup();
      } ## end else [ if (not update_rom_image_file...
   } ## end else [ if (not defined $image...
   return $retval;
} ## end sub write_rom_image_from_file($)

## @fn  get_file_size_in_bytes($f_name)
# @brief  Return total size of file, in bytes
# @details  Use stat() function to get the file info, perform a set of checks
#           to make sure this file exists.
# @param  $f_name - the name of the file
# @return  $f_size - total size of file, in bytes
# @return  -1 - an error occurred (see log file for detailes)
# @note This subroutine
sub get_file_size_in_bytes($)
{
   my $f_name = shift @_;
   my $f_size = -1;
   if (not defined $f_name)
   {
      baselog($LOG_ERROR, "file parameter is not defined.");
   }
   elsif (not -e $f_name)
   {
      baselog($LOG_ERROR, "File '$f_name' does not exist.");
   }
   elsif (not -f $f_name)
   {
      baselog($LOG_ERROR, "'$f_name' is not a plain file.");
   }
   else
   {
      my @stat_info = stat($f_name);
      if (scalar @stat_info == 0)
      {
         baselog($LOG_ERROR, "stat function failed for '$f_name'.");
      }
      else
      {
         $f_size = $stat_info[7];
      }
      baselog($LOG_DEBUG, "File '$f_name' size is '$f_size'.");
   } ## end else [ if (not defined $f_name...
   return $f_size;
} ## end sub get_file_size_in_bytes($)

## @fn  update_rom_image_file($image)
# @brief Update the BIOS ROM image file
# @details Stuff the image file with 512K of zeros to make it 1M size.
#          If update is not needed, just copy the image
# @param  $image - name of the image file
# @return  $TRUE - the image file is updated successfully
# @return  $FALSE - an error occurred
sub update_rom_image_file($)
{
   my $image = shift @_;

   my $retval = $TRUE;
   if (not defined $image)
   {
      baselog($LOG_ERROR, "BIOS ROM image file is not defined");
      $retval = $FALSE;
   }
   else
   {
      if (get_file_size_in_bytes($image) == $IMAGE_SIZE_512K)
      {
         if (not do_system([$DD, "if=$image", "of=$BIOS_IMAGE_FILE", 'bs=512K', 'count=1', 'seek=1']))
         {
            baselog($LOG_ERROR, "Cannot create the updated BIOS ROM image file '$BIOS_IMAGE_FILE'");
            $retval = $FALSE;
         }
      } ## end if (get_file_size_in_bytes...
      else
      {
         if (not do_system([$CP, $image, $BIOS_IMAGE_FILE]))
         {
            baselog($LOG_ERROR, "Cannot copy image '$image' into the file '$BIOS_IMAGE_FILE'");
            $retval = $FALSE;
         }
      } ## end else [ if (get_file_size_in_bytes...
   } ## end else [ if (not defined $image...
   return $retval;
} ## end sub update_rom_image_file($)

## @fn  cleanup()
# @brief Clean up
# @details Delete the temporary file
# @param  none
# @return  none
sub cleanup()
{
   unlink $BIOS_IMAGE_FILE if (-e $BIOS_IMAGE_FILE);
   return;
}

############################# Public Functions ################################

###############################################################################
## @fn check_image($image)
# @brief Check the given image file
# @details none
# @param $image - image file
# @return $TRUE - it looks like a valid BIOS ROM image
# @return $FALSE - it is not a valid BIOS ROM image
sub check_image($)
{
   my $image = shift @_;

   print "Checking image... ";
   if (not check_rom_image_file($image))
   {
      print "Not ok\n";
      print "\nThe image file '$image' is not a valid BIOS ROM image.\n";
      return $FALSE;
   } ## end if (not check_rom_image_file...

   print "Ok\n\nThe image '$image' looks like a valid BIOS ROM image.\n";
   return $TRUE;
} ## end sub check_image($)

###############################################################################
## @fn read_image($image)
# @brief Save current BIOS ROM contents into the given file
# @details none
# @param $image - image file
# @return $TRUE - BIOS ROM contents is saved into the given file
# @return $FALSE - an error occurred during reading
sub read_image($)
{
   my $image = shift @_;

   print "Please wait. Reading the BIOS ROM contents... ";

   my $retval = $TRUE;
   if (not read_rom_image_into_file($image))
   {
      print "Not ok\n";
      print "\nCannot read the BIOS ROM contents into the file '$image'.\n";
      $retval = $FALSE;
   } ## end if (not read_rom_image_into_file...
   else
   {
      print "Ok\n\nThe BIOS ROM contents are saved into the file '$image'\n";
   }

   return $retval;
} ## end sub read_image($)

###############################################################################
## @fn write_image($image)
# @brief Write the given image file contents into the BIOS ROM
# @details none
# @param $image - image file
# @return $TRUE - BIOS ROM is upgraded successfully
# @return $FALSE - an error occurred during writing
sub write_image($)
{
   my $image = shift @_;

   print "\n\nReboot is required to complete BIOS ROM upgrade.\n";
   print "Furthermore, to complete the upgrade BIOS settings must be changed to defaults.\n";
   print "Please refer to the documentation for more information.\n";

   my $retval = $TRUE;
   if (getYNResponse("Do you want to continue BIOS upgrade?", "N"))
   {
      print "\nPlease wait. Writing the BIOS ROM contents... ";
      if (not write_rom_image_from_file($image))
      {
         print "Not ok\n";
         print "\nCannot write the image '$image' into the BIOS ROM.\n";
         $retval = $FALSE;
      } ## end if (not write_rom_image_from_file...
      else
      {
         print "Ok\n\nThe BIOS ROM is upgraded successfully.\n";
      }
   } ## end if (getYNResponse("Do you want to continue BIOS upgrade?"...
   else
   {
      print "BIOS upgrade is cancelled. BIOS ROM contents are not changed.\n";
   }

   return $retval;
} ## end sub write_image($)

# @fn take_biosupgrade_semaphore()
# @brief takes SEM_BIOSUPGRADE semaphore
# @return $semmgr - the reference to a semaphore manager object
# @return undef   - the semaphore cannot be taken
# @note
sub take_biosupgrade_semaphore()
{
   my $semmgr = base_semctl::new($SBS_BIOSUPGRADE);
   if (!defined($semmgr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
   }
   elsif (!$semmgr->sem_take([$SEM_BIOSUPGRADE], $WAIT_FLAG, $TIME_TO_WAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphore $SEM_BIOSUPGRADE.");
      undef $semmgr;
   }
   return $semmgr;
} ## end sub take_biosupgrade_semaphore()

# @fn release_biosupgrade_semaphore($semmgr)
# @brief releases SEM_BIOSUPGRADE semaphore
# @param $semmgr - the reference to semaphore manager object
# @return $TRUE - the semaphore is released successfully
# @return $FALSE - the semaphore cannot be released
# @note
sub release_biosupgrade_semaphore($)
{
   my $semmgr = shift @_;
   if (!defined $semmgr)
   {
      baselog($LOG_ERROR, "Undefined parameter.");
      return $FALSE;
   }

   my $retval = $TRUE;
   if (!$semmgr->sem_give($SEM_BIOSUPGRADE))
   {
      baselog($LOG_ERROR, "Cannot release semaphore $SEM_BIOSUPGRADE.");
      $retval = $FALSE;
   }
   undef $semmgr;
   return $retval;
} ## end sub release_biosupgrade_semaphore($)

1;

__END__

=head1 NAME

bios_upgrade  - the LinuxBase wrapper for flashrom tool

=head1 SYNOPSIS

use bios_upgrade;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

