## @file    cbcCrypto.pm
# @brief    LinuxBase cbcCrypto module
# @details  Module for providing a encryption and decryption
#           API for password management.  This module
#           provides full password managemement support
#           including securely storing the passwords.
# @author   K. Agarwal
# @version  1.00 - Initial version
# @version  1.01 - Two colons were removed before shared functions and variables.
################################################################
package cbcCrypto;

use lib "/opt/nortel/base/lib";
use common_variables;

our $VERSION = 1.01;

use strict;
use warnings;

use English;
use baselog;

use Crypt::CBC;
use Crypt::Blowfish;

##################################################################
# Access Management
##################################################################
use Exporter;

our @ISA = qw(Exporter);  # public Exporter

# Items to export into the callers name space
our @EXPORT = qw(&cbcEncrypt &cbcDecrypt);

##################################################################
# Cipher set up Parameters
##################################################################
my $ALGORITHM = "Blowfish";
my $HEADER = "none";
my $IV = "N0rte1nT";

##################################################################
# Cipher files
##################################################################
my $pfile1 = "/var/opt/nortel/base/nortel_cs1000_1";
my $pfile2 = "/var/opt/nortel/base/nortel_cs1000_2";

##################################################################
# Function: charToAscii 
# Description: Convers a char string to an Ascii string (separated
#              by ':' 
#
# INPUT:  $char_str - char string to convert
#
# OUTPUT: $ascii_str  - converted ascii string
##################################################################
sub charToAscii
{
   my $char_str = shift(@_);
   my $ascii_str = "";

   my @char_array = split('', $char_str);
   foreach my $str (@char_array)
   {
      $ascii_str .= ord($str).":";
   }

   return $ascii_str;
}

##################################################################
# Function: asciiToChar
# Description: Convers an ascii string to an Char string 
#
# INPUT:  $ascii_str - ascii string to convert
#
# OUTPUT: $char_str  - converted char string
##################################################################
sub asciiToChar
{
   my $ascii_str = shift(@_);
   my $char_str = "";

   my @ascii_array = split(':', $ascii_str);
   foreach my $str (@ascii_array)
   {
      $char_str .= chr($str);
   }
   return $char_str;
}

##################################################################
# Function: readFile
# Description: Read the specified file and returns the contents
#              in an array
#
# INPUT:  $file - path of the file to be read
#
# OUTPUT: @str  - contents of the file stored in an array
##################################################################
sub readFile
{
   my $file = shift(@_);

   my $fh;
   system ($CHMOD, "400", $file);
   unless (open $fh, '<', $file)
   {
      baselog($LOG_ERROR, "Could not open the password file!");
      return undef;
   }

   my @str = <$fh>;
   close $fh;
   system($CHMOD, "000", $file);

   return @str;
}

##################################################################
# Function: writeFile
# Description: Write the given data to the specified file
#
# INPUT:  $file - path of the file to write to
#         @out_str - data to be written
#
# OUTPUT: $TRUE - operation completed successfully
#         $FALSE - failure (either could not open the file for
#                    writing or could not rename the file)
##################################################################
sub writeFile
{
   my ($file, @out_str) = @_;

   my $tmpfile = $file . $$;
   my $fh;
   unless (open $fh, '>', $tmpfile)
   {
      baselog($LOG_ERROR, "Cannot open temporary password file!");
      return $FALSE;
   }

   print $fh @out_str;
   close $fh;

   system($CHMOD, "--reference=$file", $tmpfile);
   unless(rename($tmpfile, $file))
   {
      baselog($LOG_ERROR, "Could not update the password file!");
      system($RM, "-f", $tmpfile);
      return $FALSE;
   }

   return $TRUE;
}

##################################################################
# Function: createCipher
# Description: Generate the cipher used to encrypt and decypt 
#              using the prespecified parameters and the given key
#
# INPUT:  $key - the key used to generate the cipher
#
# OUTPUT: $cipher - the cipher object used to encrypt/decrypt
##################################################################
sub createCipher
{
   my $key = shift(@_);
   my $cipher = Crypt::CBC->new(-key => $key,
                                -cipher => $ALGORITHM,
                                -header => $HEADER,
                                -iv => $IV);
   return $cipher;
}

##################################################################
# Function: getNewString
# Description: Generate a new string, excluding the line containing
#              the specified key
#
# INPUT:  $file - path of the file to be read
#         $key - the key to be excluded
#
# OUTPUT: @new_str  - contents of the file (excluding the specified key)
#                     stored in an array
##################################################################
sub getNewString
{
   my $file = shift(@_);
   my $key = shift(@_);
   my @old_str = readFile($file);

   my @new_str = "";
   foreach my $str (@old_str)
   {
      my @data = split('==', $str);
      if ($data[0] ne $key)
      {
         push @new_str, $str;
      }
   }

   return @new_str
}

##################################################################
# Function: findKeyStr
# Description: Reads the specified file and finds the specified
#              key
#
# INPUT:  $file - path of the file to be read
#         $key - the key need to be found
#
# OUTPUT: @data - array containing the content associated with the
#                 specified key or empty array if nothing was found
##################################################################
sub findKeyStr
{
   my $file = shift(@_);
   my $key = shift(@_);

   my @passwd_str = readFile($file);

   my @data = "";
   foreach my $str (@passwd_str)
   {
      if ($str =~ $key)
      {
         @data = split('==', $str);
         if ($data[0] eq $key)
         {
            return @data;
         }
      }
   }

   $data[0] = "";
   return @data;
}

##################################################################
# Function: genRandStr
# Description: Generates a random string, based on a specified string
#
# INPUT:  $digest - string to use in the random generator
#
# OUTPUT: $str - random string
##################################################################
sub genRandStr
{
   my $digest = shift(@_);

   my @chars = ('c'..'l','a','m','b','n','o'..'z','1'..'9','A'..'H','I','M','K','L','J'..'Z');

   my $str = "";
   for (0..rand(100))
   {
      $str .= $chars[int rand @chars];
   }

   return crypt($digest, $str);
}

##################################################################
# Function: encryptPassword
# Description: Preforms the main task of encryption and storing
#              the password
#
# INPUT:  $key - the key associated with the password
#         $data - the plaintext password
#
# OUTPUT: $TRUE - password successfully encrypted and stored
#         $FALSE - encrypted password not stored
##################################################################
sub encryptPasswd
{
   my $key = shift(@_);
   my $data = shift(@_);

   my $passwd_key = genRandStr($key);
   my $encryption_key = genRandStr($key);

   my $key_name = genRandStr($key);
   my @key_data = findKeyStr($pfile2, $key);
   if ($key_data[0] ne "") 
   {
      my $cipher_key_name = $key_data[1];
      $cipher_key_name = asciiToChar($cipher_key_name);
      my $cipher = createCipher($key);
      $key_name = $cipher->decrypt($cipher_key_name);
   }

   my $cipher = createCipher($key);
   my $cipher_key_name = $cipher->encrypt($key_name);
   $cipher_key_name = charToAscii($cipher_key_name);

   $cipher = createCipher($encryption_key);
   my $cipher_pass_key = $cipher->encrypt($passwd_key);
   $cipher_pass_key = charToAscii($cipher_pass_key);

   $cipher = createCipher($key.$key_name);
   my $cipher_encrypt_key = $cipher->encrypt($encryption_key);
   $cipher_encrypt_key = charToAscii($cipher_encrypt_key);

   $cipher = createCipher($passwd_key);
   my $ciphertext = $cipher->encrypt($data);
   $cipher = createCipher($encryption_key);
   $ciphertext = $cipher->encrypt($ciphertext);
   $cipher = createCipher($key_name.$key);
   $ciphertext = $cipher->decrypt($ciphertext);
   $ciphertext = charToAscii($ciphertext);

   my @new_key_str = getNewString($pfile2, $key);
   push @new_key_str, $key, "==", $cipher_key_name, "==", $cipher_pass_key, "\n";
   
   my @new_passwd_str = getNewString($pfile1, $key_name);
   push @new_passwd_str, $key_name, "==", $cipher_encrypt_key, "==", $ciphertext, "\n";

   if (writeFile($pfile2, @new_key_str))
   {
      return (writeFile($pfile1, @new_passwd_str));
   }
   else {return $FALSE;}
}

##################################################################
# Function: decryptPasswd
# Description: Preforms the main task of decrypting the ciphertext
#
# INPUT:  $key - the key associated with the password
#
# OUTPUT: $password - the decrypted password
#         undef - error - could not find the specified key
##################################################################
sub decryptPasswd
{
   my $key = shift(@_);

   my @key_data = findKeyStr($pfile2, $key);
   if ($key_data[0] eq "") {return undef;}
   
   my $cipher_key_name = $key_data[1];
   my $cipher_pass_key = $key_data[2];
   chomp $cipher_pass_key;

   $cipher_key_name = asciiToChar($cipher_key_name);
   my $cipher = createCipher($key);
   my $key_name = $cipher->decrypt($cipher_key_name);

   my @passwd_data = findKeyStr($pfile1, $key_name);
   if ($passwd_data[0] eq "") {return undef;} 

   my $cipher_encrypt_key = $passwd_data[1];
   my $ciphertext = $passwd_data[2];
   chomp $ciphertext;

   $cipher_encrypt_key = asciiToChar($cipher_encrypt_key);
   $cipher = createCipher($key.$key_name);
   my $encryption_key = $cipher->decrypt($cipher_encrypt_key);

   $cipher_pass_key = asciiToChar($cipher_pass_key);
   $cipher = createCipher($encryption_key);
   my $passwd_key = $cipher->decrypt($cipher_pass_key);
   
   $ciphertext = asciiToChar($ciphertext);
   $cipher = createCipher($key_name.$key);
   $ciphertext = $cipher->encrypt($ciphertext);
   $cipher = createCipher($encryption_key);
   $ciphertext = $cipher->decrypt($ciphertext);
   $cipher = createCipher($passwd_key);
   return $cipher->decrypt($ciphertext);
}

##################################################################
# Function: cbcEncrypt 
# Description: The publicly viewable interface to Encrypt
#
# INPUT:  $key - the key associated with the password
#         $data - the plaintext password
#
# OUTPUT: $TRUE - the password was successfuly encrypted and stored
#         $FALSE - the password was not stored
##################################################################
sub cbcEncrypt
{
   my $key = shift(@_);
   my $data = shift(@_);

   my $retval = encryptPasswd($key, $data);

   return $retval;
}

##################################################################
# Function: cbcDecrypt
# Description: The publicly viewable interface to Decrypt
#              This also generates a default password for the specified
#              key if that key does not exist in the password database
#
# INPUT:  $key - the key associated with the password
#
# OUTPUT: $passwd - the plaintext password if it exists
#         $default - the default password if the key does not exist
#                    in the database
##################################################################
sub cbcDecrypt
{
   my $key = shift(@_);
   my $passwd = decryptPasswd($key);

   if (defined $passwd)
   {
       return $passwd;
   }
   else
   {
      #If the password does not exist for the specified key then generate
      #a default password and store that in the database
      my $default = `/opt/nortel/base/lib/dpg NTlpm $key 16`;

      cbcEncrypt($key, $default);

      return $default;
   }
}
