#############################################################
## @file common_functions.pm
# @brief Common functions for linuxbase are stored in this library.
# @details
# @autor
# @version 1.00  - Initial version
# @version 1.10  - New functions were added:
# \li get_basename - returns the last level of a filepath
# \li warning_if_not_nortel_grp - prints error message if invoker of
#      the script does not belong to nortel group
# \li warning_if_not_root_uid - prints error message if invoker of
#      the script is root
# \li is_right_group - Check whenever user belong to on of the allowed
#     groups
# @version 1.20  - New functions were moved from hardening_common.pm:
# \li write_file - safely write file
# \li get_tmp_filename - get temporary file name
# \li backup_and_replace - backup original file and move new one
# \li replace_by_new - just replace original one without backup
# @version 1.21  - change_permissions_and_ownership function was moved
# \li from hardening_common.pm, is_valid_group from base_accounts.pm
# @version 1.22  - get_mntdevice function has been added.
# @version 1.23  - write_file function was modify, work with hash reference
#                  as file content was added.
# @version 1.24  - new variable $RM has been added into system_installation_init function
# @version 1.25  - signals_ignore and signals_default functions were enhanced
# @version 1.26  - get_hostname and get_dirname functions have been added.
# @version 1.27  - added function set_parameter
# @version 1.28  - 'get_mount_counter' has been moved from the base_ftp.pm module
# @version 1.29  - added function recover_env_path
# @version 1.30  - added function is_array_unique
# @version 1.31  - added functions delete_line_from_file, make_regexp_pattern_save
# @version 1.32  - XML validation has been added, added function parseFiles
# @version 1.33  - added function list2hash
# @version 1.34  - the fix for the 'write_function' added. (if the hash is passed as an
#                  argument and value by one the keys is undefined error is printed)
# @version 1.35  - new function 'is_integer' has been added.
# @version 1.37  - get_realpath() was added
# @version 1.38  - function is_package_installed is added
# @version 1.39  - ssh_hang_on_exit_bug() was added
# @version 1.40  -  signals_ignore() is updated to support extended list of signals to ignore.
package common_functions;

use strict;
use warnings;

use lib '/opt/nortel/base/lib';

use common_variables;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&system_installation_init &parse_configs &use_confver &is_right_user &do_co_stdin
  &is_debug_mode &status_success &status_failure &add_sudo_cmd &prepare_command_to_run_as
  &print_pod_usage &is_hash &is_array &is_code &invoke_subroutine_safely &is_user_exist
  &signals_ignore &signals_default &cmp_hashes &is_install_stage &pos_in_array &is_right_group
  &get_basename &warning_if_not_nortel_grp &warning_if_not_root_uid &write_file &get_tmp_filename
  &change_permissions_and_ownership &is_valid_group &get_mntdevice &get_hostname &get_dirname &set_parameter
  &get_mount_counter &recover_env_path &is_array_unique &delete_line_from_file &parseFile
  &make_regexp_pattern_save &list2hash &is_package_installed &is_integer &get_realpath &ssh_hang_on_exit_bug);

use XML::Parser;
use Pod::Usage;
use File::Basename;
use Sys::Hostname;
use base_system;
use baselog;
use XML::LibXML;
use Cwd qw(realpath);

our $VERSION = '1.40';

## @skipline
sub is_install_stage();
## @skipline
sub system_installation_init();
## @skipline
sub parse_configs($);
## @skipline
sub use_confver($);
## @skipline
sub is_right_user(@);
## @skipline
sub is_right_group(@);
## @skipline
sub do_co_stdin($);
## @skipline
sub is_debug_mode(@);
## @skipline
sub status_success(;$);
## @skipline
sub status_failure(;$);
## @skipline
sub add_sudo_cmd();
## @skipline
sub prepare_command_to_run_as($;$$$);
## @skipline
sub print_pod_usage(;$);
## @skipline
sub is_hash($);
## @skipline
sub is_array($);
## @skipline
sub is_code($);
## @skipline
sub invoke_subroutine_safely($;$);
## @skipline
sub is_user_exist($);
## @skipline
sub signals_ignore();
## @skipline
sub signals_default();
## @skipline
sub cmp_hashes($$);
## @skipline
sub pos_in_array($$);
## @skipline
sub get_basename($);
## @skipline
sub get_dirname($);
## @skipline
sub warning_if_not_nortel_grp();
## @skipline
sub warning_if_not_root_uid();
## @skipline
sub get_tmp_filename($;$);
## @skipline
sub get_backup_filename($;$);
## @skipline
sub backup_and_replace($$$);
## @skipline
sub replace_by_new($$);
## @skipline
sub write_file($$;$$$);
## @skipline
sub delete_line_from_file($$);
## @skipline
sub make_regexp_pattern_save($);
## @skipline
sub change_permissions_and_ownership($$$$;$$);
## @skipline
sub is_valid_group($);
## @skipline
sub get_mntdevice($);
## @skipline
sub get_hostname(;$);
## @skipline
sub recover_env_path();
## @skipline
sub set_parameter($$$);
## @skipline
sub get_mount_counter($);
## @skipline
sub is_array_unique($);
## @skipline
sub parseFile($;$);
## @skipline
sub list2hash($;$);

###############################################################################
# @fn is_install_stage()
# @brief Determines if we are running from the installation stage
# @details
# @param none
# @return $TRUE - the installation stage
# @return $FALSE - runtime
# @note
sub is_install_stage()
{
   my $retval = $FALSE;
   open(CMDLINE, "< /proc/cmdline")
     or die print "Can not open /proc/cmdline file for reading\n";
   while (<CMDLINE>)
   {
      if ($_ =~ /installation_stage/)
      {
         $retval = $TRUE;
         last;
      }
   } ## end while (<CMDLINE>)
   close(CMDLINE);
   return $retval;
} ## end sub is_install_stage()

###############################################################################
# @fn system_installation_init()
# @brief Re-initialize command variables for installation stage
# @details
# @param none
# @return none
# @note
sub system_installation_init()
{
   if (is_install_stage())
   {
      $CAT             = "cat";
      $CLEAR           = "clear";
      $CP              = "cp";
      $DATE            = "date";
      $DD              = "dd";
      $DF              = "df";
      $E2LABEL         = "e2label";
      $FDISK           = "fdisk";
      $FSCK            = "fsck";
      $GREP            = "grep";
      $GRUBBY          = "/sbin/grubby";
      $GRUBINST        = "grub-install";
      $HWCLOCK         = "hwclock";
      $LS              = "ls";
      $MKDIR           = "mkdir";
      $MKNOD           = "mknod";
      $MOUNT           = "mount";
      $MV              = "mv";
      $NANO            = "/usr/bin/nano";
      $PERL            = "perl";
      $PARTED          = "parted";
      $REBOOT          = "reboot";
      $RM              = "rm";
      $STTY            = "stty";
      $TAR             = "tar";
      $TAIL            = "tail";
      $TOUCH           = "touch";
      $TTY             = "tty";
      $UMOUNT          = "umount";
      $WC              = "wc";
      $WHOAMI          = "whoami";
      $ADMIN_PARTITION = "/tmp/admin";
   } ## end if (is_install_stage()...
   return;
} ## end sub system_installation_init()

###----------------------------------------------------------------------------
# @fn parse_configs($pkg_name)
# @brief This function looks for configure.xml files in $APP_CFG_ROOT.
#               All found files are parsed by XML parser and info from their
#               tags are passed to specified package.
# @param  $pkg_name - A package. The package needs to have functions equal to
#          configure.xml file tags
# @return $FALSE if file list is empty. Overwise $TRUE;
###----------------------------------------------------------------------------
sub parse_configs($)
{

   #get package name
   my $pkg_name = shift @_;

   #Looking for configure.xml files
   my @filelist = ();
   @filelist =
     split(/\n/, qx!$FIND $APP_CFG_ROOT -name "configure.xml" 2>/dev/null!);
   if (!scalar(@filelist))
   {
      return $FALSE;
   }

   foreach my $entry (@filelist)
   {
      if (not exists $XML_PATHS{$entry})
      {
         if (not parseFile($entry))
         {
            print "Invalid XML syntax of $entry file\n";
            next;
         }
      } ## end if (not exists $XML_PATHS...
      else
      {
         next if (not $XML_PATHS{$entry});
      }
      my $confparser = new XML::Parser(Style => 'Subs', Pkg => $pkg_name);
      $confparser->parsefile($entry);
      $confparser = 0;  #Calling destructor
   } ## end foreach my $entry (@filelist...
   return $TRUE;
} ## end sub parse_configs($)

###############################################################################
## @fn use_confver($config)
# @brief Returns the reference to hash with values of variables from the configuration file
# @param $config - full path to the configuration file for reading
# @return reference - reference to hash
# @return undef - configuration file does not exist
sub use_confver($)
{
   my $config = shift @_;
   unless (defined($config))
   {
      baselog($LOG_WARNING, "Configuration file is not specified.");
      return undef;
   }
   unless (-e $config)
   {

      # it is a valid condition when Nortel Applications are not installed.
      baselog($LOG_INFO, "$config does not exist.");
      return undef;
   } ## end unless (-e $config)
   my %params = map {
      s/#.*//;          #Remove comments
      s/^\s+//;         #Remove extra spaces
      s/\s+$//;         #
      split /\s*=\s*/;  #Split the line
   } split /\n/, qx/$CAT $config/;

   return \%params;
} ## end sub use_confver($)

###############################################################################
# @fn is_right_user(@allowed_users)
# @brief Determines if we are running from the installation stage
# @details
# @param @allowed_users - list of accounts that permitted to use the script
# @param none - checks for script's owner
# @return $TRUE -  user matches to a permitted user
# @return $FALSE - invoker is not a permitted user
# @note
sub is_right_user(@)
{
   my @allowed_users = @_;
   my $retval        = $FALSE;
   my $real_user     = (getpwuid($<))[2];  # get uid of the invoker
   if (scalar(@allowed_users) > 0)
   {
      for (my $i = 0 ; $i < scalar(@allowed_users) ; $i++)
      {
         if (getpwnam($allowed_users[$i]) eq $real_user)
         {
            $retval = $TRUE;
            last;
         }
      } ## end for (my $i = 0 ; $i < scalar...
   } ## end if (scalar(@allowed_users...
   else
   {
      my $default_user = (stat($0))[4];  # get uid of owner of the script
      if ($real_user eq $default_user)
      {
         $retval = $TRUE;
      }
   } ## end else [ if (scalar(@allowed_users...

   return $retval;
} ## end sub is_right_user(@)

###############################################################################
## @fn is_right_group(@allowed_groups)
# @brief   Check whenever user belong to on of the allowed groups
# @details Checks primary and supplementary groups of the user.
# @param  @allowed_groups - list of allowed groups (group names)
# @return $TRUE - user belongs to one of the allowed groups
# @return $FALSE - user does not belong to one of the allowed groups
sub is_right_group(@)
{
   my @allowed_groups = @_;
   my $retval         = $FALSE;
   my $real_user      = (getpwuid($<))[0];  # get uid of the invoker
   my $real_group     = (getgrgid($())[0];  # get gid of the invoker
   my @users_in_group = ();
   if (scalar(@allowed_groups) > 0)
   {
      for (my $i = 0 ; $i < scalar(@allowed_groups) ; $i++)
      {
         @users_in_group = split(/ /, (getgrnam($allowed_groups[$i]))[3]);
         if ((pos_in_array(\@users_in_group, $real_user) >= 0) || ($real_group eq $allowed_groups[$i]))
         {
            $retval = $TRUE;
            last;
         }
      } ## end for (my $i = 0 ; $i < scalar...
   } ## end if (scalar(@allowed_groups...
   else
   {
      @users_in_group = split(/ /, (getgrnam($NORTEL_GRP))[3]);
      if ((pos_in_array(\@users_in_group, $real_user) >= 0) || ($real_group eq $NORTEL_GRP))
      {
         $retval = $TRUE;
      }
   } ## end else [ if (scalar(@allowed_groups...

   return $retval;
} ## end sub is_right_group(@)

###############################################################################
## @fn do_co_stdin($arg)
# @brief Disable or enable STDIN to prevent unwanted input
# @details
# @param $arg - action
#   @li  "open"  - reopen STDIN, $SAVESTDIN should contain old STDIN
#   @li  "close" - disable STDIN by reopening it to /dev/null
# @return $TRUE - STDIN is disabled/enabled successfully
# @return $FALSE - fail to disable/enable
sub do_co_stdin($)
{
   my $arg    = shift @_;
   my $retval = $FALSE;
   if ($arg eq 'close')
   {
      if (!$SAVESTDIN)
      {
         unless (open $SAVESTDIN, "<&STDIN")
         {
            baselog($LOG_WARNING, "Can not duplicate STDIN: $!\n", $TRUE);
            $retval = $FALSE;
         }
         else
         {
            unless (open STDIN, ">", "/dev/null")
            {
               baselog($LOG_WARNING, "Can not open /dev/null for reading: $!\n", $TRUE);
               $retval = $FALSE;
            }
            else
            {
               baselog($LOG_DEBUG, "STDIN was re-opened to /dev/null");
               $retval = $TRUE;
            }
         } ## end else
      } ## end if (!$SAVESTDIN)
      else {$retval = $FALSE;}
   } ## end if ($arg eq 'close')
   elsif ($arg eq 'open')
   {
      if ($SAVESTDIN)
      {
         unless (open STDIN, "<&", $SAVESTDIN)
         {
            baselog($LOG_WARNING, "Can not reopen STDIN: $!\n", $TRUE);
            $retval = $FALSE;
         }
         else
         {
            $SAVESTDIN = undef;
            baselog($LOG_DEBUG, "STDIN was restored");
            $retval = $TRUE;
         } ## end else
      } ## end if ($SAVESTDIN)
      else
      {
         if (fileno(STDIN) ne "0")
         {
            baselog($LOG_WARNING, "SAVESTDIN is not defined\n", $TRUE);
         }
         else
         {
            baselog($LOG_WARNING, "STDIN is already open.\n");
         }
         $retval = $FALSE;
      } ## end else [ if ($SAVESTDIN)
   } ## end elsif ($arg eq 'open')

   return $retval;
} ## end sub do_co_stdin($)

###############################################################################
# @fn is_debug_mode(@files)
# @brief Checks if the debug mode should be used or not
# @details
# @param @files - files to search the string "DEBUG_MODE=1" (optional,
#    by default it searches in the file $LBINFO)
# @return $TRUE - the searching string found at least in one file (debug mode)
# @return $FALSE - no string found in files (usual mode)
# @note
sub is_debug_mode(@)
{
   my @files  = @_;
   my $retval = $FALSE;

   if (scalar @files == 0)
   {
      open INFO, "<$LBINFO" or die "Can't open $LBINFO file: $!";
      $retval = $TRUE if (grep /^$DEBUG_MODE$/, <INFO>);
      close INFO;
   } ## end if (scalar @files == 0...
   else
   {
      foreach my $file (@files)
      {
         open INFO, "<$file" or next;
         $retval = $TRUE if (grep /^$DEBUG_MODE$/, <INFO>);
         close INFO;
      } ## end foreach my $file (@files)
   } ## end else [ if (scalar @files == 0...

   return $retval;
} ## end sub is_debug_mode(@)

###############################################################################
# @fn status_success($msg)
# @brief Prints success status
# @details Uses the library which contains the code common for all standard
#   Linux services in order to display successful status of service executing
# @param $msg - message string for printing in the log (optional)
# @return none
# @note
sub status_success(;$)
{
   my $msg = shift @_;
   $msg = "" if (not defined $msg);

   print " ";
   system(qq/. $FUNCTIONS_LIB; success "$msg"/);
   print "\n";
} ## end sub status_success(;$)

###############################################################################
# @fn status_failure($msg)
# @brief Prints failure status
# @details Uses the library which contains the code common for all standard
#   Linux services in order to display successful status of service executing
# @param $msg - message string for printing in the log (optional)
# @return none
# @note
sub status_failure(;$)
{
   my $msg = shift @_;
   $msg = "" if (not defined $msg);

   print " ";
   system(qq/. $FUNCTIONS_LIB; failure "$msg"/);
   print "\n";
} ## end sub status_failure(;$)

#########################################################
# Description: this func determines to add or not SUDO
# INPUT:  none
# OUTPUT: the same command in case of root user and
#         the command with SUDO in case of others users
#########################################################
sub add_sudo_cmd()
{
   if (is_right_user($ROOT_ACCT) == $TRUE)
   {
      return " ";
   }
   return $SUDO . " ";
} ## end sub add_sudo_cmd()

###############################################################################
## @fn prepare_command_to_run_as($command, $user, $nologin, $unpriv_user)
# @brief This subroutine prepares an array in order to use it in system command
# @details Usage of 'su' command in LinuxBase is obsoleted. If it is necessary
#   to change user prior to execution some system command (or script), this
#   subroutine has to be called before the command 'system' (or do_system)
#   invocation. It uses 'runuser' command that performs the same actions
#   as 'su', but passes over PAM.
# @param $command - the command to be run
# @param $user - login name of user which is used for the command invoking.
# @param $nologin - if the current user is the same as $user, to not establish
#   environment by invoking initialization scripts ('false' by default).
# @param $unpriv_user - this flag is used in order to determine it is necessary
#   to check that the current user is root or not. If this flag is 'true',
#   a developer can execute 'runuser' from non root user, e.g. when the process
#   has CAP_SETUID capability set ('false' by default).
# @return $res - a reference to array. It begins with runuser command if it is
#   necessary to run command from another user. This array contains $command
#   at least. If the array is empty, some error happened.
sub prepare_command_to_run_as($;$$$)
{
   my ($command, $user, $nologin, $unpriv_user) = @_;

   my @res = ();

   my %initialize = ($RUNUSER => '-', $BASH => '-l');

   if (not defined $user)
   {
      baselog($LOG_NOTICE, "The user is not defined");
      $user = $NORTEL_ACCT;
   }

   # Set default values
   if (not defined $nologin)
   {
      $nologin = $FALSE;
   }
   if (not defined $unpriv_user)
   {
      $unpriv_user = $FALSE;
   }

   # Get login name of the current user
   my $cur_user = getpwuid($<);

   # Check that the current user is root
   if (($cur_user ne $ROOT_ACCT) and (not $unpriv_user))
   {
      baselog($LOG_ERROR, "The user '$cur_user' has no permission to change user ID");
      return \@res;
   }

   if ($command eq "")
   {
      baselog($LOG_ERROR, "The command is not specified");
      return \@res;
   }

   # Prepare the array for the command invoking
   if ($cur_user ne $user)
   {
      push @res, $RUNUSER;
      if (not $nologin)
      {
         push @res, $initialize{$RUNUSER};
      }
      push @res, $user, "-c", $command;
   } ## end if ($cur_user ne $user...
   else
   {
      if (not $nologin)
      {
         push @res, $BASH, $initialize{$BASH}, "-c";
      }
      push @res, $command;
   } ## end else [ if ($cur_user ne $user...

   baselog($LOG_DEBUG, "Current user: '$cur_user', User required: '$user', command: '$command', options: ($nologin, $unpriv_user)");
   baselog($LOG_DEBUG, "Prepared array: " . join(',', @res));

   return \@res;
} ## end sub prepare_command_to_run_as($;$$$)

#########################################################
# Description: prints the usage, generated from the pod
#              section of the file; optionally, prints
#              the string, given as s first parameter
# INPUT:  $msg  - this text precedes the usage message
#                 (optional)
# OUTPUT: none
#########################################################
sub print_pod_usage(;$)
{
   my %options;
   $options{'-exitval'} = "NOEXIT";
   $options{'-message'} = shift @_ if defined($_[0]);
   $options{'-verbose'} = 1;
   pod2usage(\%options);
} ## end sub print_pod_usage(;$)

###############################################################################
## @fn is_hash($ref)
# @brief Check if the reference is a HASH-reference
# @details This method uses 'ref' function in order to check if the reference
#          is a HASH-reference
# @param $ref - reference
# @return TRUE - the reference is a HASH-reference
# @return FALSE - the reference either is not defined or is not a HASH-reference
sub is_hash($)
{
   my $ref = shift @_;

   if (not defined $ref)
   {
      baselog($LOG_DEBUG, "The reference is not defined.");
   }
   elsif (ref($ref) eq "HASH")
   {
      return $TRUE;
   }

   baselog($LOG_DEBUG, "The reference is not reference to HASH.");
   return $FALSE;
} ## end sub is_hash($)

###############################################################################
## @fn is_array($ref)
# @brief Check if the reference is a ARRAY-reference
# @details This method uses 'ref' function in order to check if the reference
#          is a ARRAY-reference
# @param $ref - reference
# @return TRUE - the reference is a ARRAY-reference
# @return FALSE - the reference either is not defined or is not a ARRAY-reference
sub is_array($)
{
   my $ref = shift @_;

   if (not defined $ref)
   {
      baselog($LOG_DEBUG, "The reference is not defined.");
   }
   elsif (ref($ref) eq "ARRAY")
   {
      return $TRUE;
   }

   baselog($LOG_DEBUG, "The reference is not reference to ARRAY.");
   return $FALSE;
} ## end sub is_array($)

###############################################################################
## @fn is_code($ref)
# @brief Check if the reference is a CODE-reference
# @details This method uses 'ref' function in order to check if the reference
#          is a CODE-reference
# @param $ref - reference
# @return TRUE - the reference is a CODE-reference
# @return FALSE - the reference either is not defined or is not a CODE-reference
sub is_code($)
{
   my $ref = shift @_;

   if (not defined $ref)
   {
      baselog($LOG_DEBUG, "The reference is not defined.");
   }
   elsif (ref($ref) eq "CODE")
   {
      return $TRUE;
   }

   baselog($LOG_DEBUG, "The reference is not reference to CODE.");
   return $FALSE;
} ## end sub is_code($)

###############################################################################
## @fn invoke_subroutine_safely($sub_ref, $parameters)
# @brief   Invoke an subroutine by using a reference
# @details This method tries to invoke an subroutine by a reference.
#          If the subroutine is undefined, the error message is generated.
# @param   $sub_ref - reference to a subroutine
# @param   $parameters - reference to an array with extra parameters
# @return  result of the action invocation
# @return  FALSE - if some error happened
sub invoke_subroutine_safely($;$)
{
   my ($sub_ref, $parameters) = @_;

   # Get file and line of the caller script
   my ($path, $line) = (caller)[1, 2];

   # Get file name without path
   my $filename = substr $path, 1 + rindex($path, '/');

   # Get subroutine of caller
   my $full_subroutine = (caller(1))[3];
   my $subroutine      = (defined $full_subroutine) ? (split /::/, $full_subroutine)[-1] : "";

   baselog($LOG_DEBUG, "Try to invoke the subroutine from '$subroutine' ($filename, line $line)");

   if (not is_code($sub_ref))
   {
      baselog($LOG_ERROR, "Undefined subroutine cannot be invoked from '$subroutine' ($filename, line $line)");
      return $FALSE;
   }

   my $retval = $FALSE;

   my @params = ();

   if ((is_array($parameters)) and (scalar @{$parameters}))
   {
      @params = @{$parameters};
   }

   eval q/$retval = $sub_ref->(@params)/;

   if ($@)
   {
      baselog($LOG_ERROR, "Cannot invoke the subroutine from '$subroutine' ($filename, line $line): $@");
      $retval = $FALSE;
   }

   return $retval;
} ## end sub invoke_subroutine_safely($;$)

## @fn is_user_exist($username)
# @brief this functions checks existence of the user with specified name
# @details This function reads /etc/passwd file and check that specified user
#          does not exist.
# @param $_[0] - user's name
# @return $TRUE  - if specified user exists.
# @return $FALSE - if specified user does not exist, or user's name is not specified.
# @author Evgeny Lazarev
# @date October 2008
sub is_user_exist($)
{
   my $username = shift @_;
   my $retval   = $FALSE;
   unless ($username)
   {
      baselog($LOG_ERROR, "User's name is not specified.");
      return $FALSE;
   }
   if (getpwnam($username))
   {
      $retval = $TRUE;
   }
   return $retval;
} ## end sub is_user_exist($)

## @fn signals_ignore()
# @brief Allows to ignore some signals
# @details The %SIG hash contains names of user-installed
#          signal handlers
#          Signals:
#                  INT - Ctrl+C
#                  TSTP - Ctrl+Z
#                  QUIT - Ctrl+\
#                  TERM - kill sends this signal by default
#                  HUP - is generated by the Kernel when user
#                        closes the controlling terminal
#                  PIPE - is generated by the Kernel when broken
#                         pipe issue is detected.
# @param None
# @return None
# @author Kate Pestova
# @date February 2008
# @note
sub signals_ignore()
{
   if ($SIGNALS_IGNORE_COUNTER == 0)
   {
      $SIG{'INT'}  = 'IGNORE';
      $SIG{'TERM'} = 'IGNORE';
      $SIG{'HUP'}  = 'IGNORE';
      $SIG{'TSTP'} = 'IGNORE';
      $SIG{'QUIT'} = 'IGNORE';

      # get the first signal. Log the event. And ignore the other signals of this type.
      $SIG{'PIPE'} = sub {
         baselog($LOG_INFO, "Got the SIGPIPE signal. Ignoring it.");
         $SIG{'PIPE'} = 'IGNORE';
         return;
        }

   } ## end if ($SIGNALS_IGNORE_COUNTER...

   $SIGNALS_IGNORE_COUNTER++;
} ## end sub signals_ignore()

## @fn signals_default()
# @brief Allows to set default value for some signals
# @details In order to set signals to default state,
#          you must call signals_default() as many times
#          as signals_ignore() was called.
#
#          The %SIG hash contains names of user-installed
#          signal handlers
#          Signals:
#                  INT - Ctrl+C
#                  TSTP - Ctrl+Z
#                  QUIT - Ctrl+\
#                  TERM - kill sends this signal by default
#                  HUP - is generated by the Kernel when user
#                        closes the controlling terminal
#                  PIPE - is generated by the Kernel when broken
#                         pipe issue is detected.
# @param None
# @return None
# @author Kate Pestova
# @date February 2008
# @note
sub signals_default()
{
   if ($SIGNALS_IGNORE_COUNTER != 0)
   {
      $SIGNALS_IGNORE_COUNTER--;
      if ($SIGNALS_IGNORE_COUNTER == 0)
      {
         $SIG{'INT'}  = 'DEFAULT';
         $SIG{'TERM'} = 'DEFAULT';
         $SIG{'HUP'}  = 'DEFAULT';
         $SIG{'TSTP'} = 'DEFAULT';
         $SIG{'QUIT'} = 'DEFAULT';
         $SIG{'PIPE'} = 'DEFAULT';
      } ## end if ($SIGNALS_IGNORE_COUNTER...
   } ## end if ($SIGNALS_IGNORE_COUNTER...

} ## end sub signals_default()

# @fn cmp_hashes
# @brief compares two hashes
#
# @param hash1 - hash reference
# @param hash2 - hash reference
#
# @return $TRUE if hashes are equal. Otherwise it returns $FALSE
sub cmp_hashes($$)
{
   my ($hash1, $hash2) = @_;

   if ((not is_hash($hash1)) or (not is_hash($hash2)))
   {
      baselog($LOG_ERROR, "Wrong parameter type");
      return $FALSE;
   }

   # It is done to get rid of "perlcheck" warning about "Implicit scalar context"
   my %h2 = %$hash2;

   if (scalar(keys(%$hash1)) != scalar(keys(%h2)))
   {
      return $FALSE;
   }

   for my $hash1key (keys %$hash1)
   {

      if ((!exists($h2{$hash1key})) or ($hash1->{$hash1key} ne $h2{$hash1key}))
      {
         return $FALSE;
      }
   } ## end for my $hash1key (keys ...
   return $TRUE;
}  ##end sub cmp_hashes

########################################################
# @fn pos_in_array($arr_ref, $item)
# @brief checks whether specified element is present in the array and returns
#        position of the first found element
# @param $arr_ref - reference to array for check
# @param $item - element for check
# @return number - the first position of the element in the array. (counting from 0)
# @return -1 - specified element does not present in the array
sub pos_in_array($$)
{
   my ($arr_ref, $item) = (@_);
   if (!is_array($arr_ref))
   {
      baselog($LOG_ERROR, "Not a array reference passed.");
   }
   else
   {
      for (my $pos = 0 ; $pos < scalar(@$arr_ref) ; $pos++)
      {
         if ($item eq $$arr_ref[$pos])
         {
            return $pos;
         }
      } ## end for (my $pos = 0 ; $pos...
   } ## end else [ if (!is_array($arr_ref...
   return -1;
} ## end sub pos_in_array($$)

########################################################
# @fn get_basename($fullpath)
# @brief returns the last level of a filepath
# @param $fullpath - full filepath
# @return returns the last level of a filepath
# @return empty string if path is not specified
sub get_basename($)
{
   my $fullpath = shift(@_);
   my $filename = '';
   unless ($fullpath)
   {
      baselog($LOG_ERROR, "Path is not specified.");
   }
   else
   {
      $filename = fileparse($fullpath);
   }
   return $filename;
} ## end sub get_basename($)

########################################################
# @fn get_dirname($fullpath)
# @brief returns the directory name of the file at
#        $fullpath
# @param $fullpath - full filepath
# @return returns the dirname
# @return empty string if path is not specified
sub get_dirname($)
{
   my $fullpath = shift(@_);
   my $dirname  = '';
   unless ($fullpath)
   {
      baselog($LOG_ERROR, "Path is not specified.");
   }
   else
   {
      $dirname = dirname($fullpath);
   }
   return $dirname;
} ## end sub get_dirname($)

###############################################################################
## @fn get_realpath($file)
# @brief Return the real path to the file or directory, given
# @details Symbolic links and relative-path components ("." and "..") are
#          resolved to return the canonical pathname
# @param $file - the file relative path
# @return $realpath - real path to the file
# @return empty string - if file is not specified
sub get_realpath($)
{
   my $file     = shift @_;
   my $realpath = '';
   unless ($file)
   {
      baselog($LOG_ERROR, "The file is not specified");
   }
   else
   {
      $realpath = realpath($file);
   }
   return $realpath;
} ## end sub get_realpath($)

########################################################
# @fn warning_if_not_nortel_grp()
# @brief prints error message if invoker of the script does not belong to nortel group
# @param None
# @return $TRUE - message has been printed
# @return $FALSE - otherwise
sub warning_if_not_nortel_grp()
{
   my $retval = $FALSE;
   if (!is_right_group($NORTEL_GRP))
   {
      print "You are not authorized to run '" . get_basename($0) . "' command.\n";
      baselog($LOG_INFO, "User '" . (getpwuid($<))[0] . "' is trying to execute '" . get_basename($0) . "' command.");
      $retval = $TRUE;
   } ## end if (!is_right_group($NORTEL_GRP...
   return $retval;
} ## end sub warning_if_not_nortel_grp()

########################################################
# @fn warning_if_not_root_uid()
# @brief prints error message if invoker of the script is root
# @param None
# @return $TRUE - message has been printed
# @return $FALSE - otherwise
sub warning_if_not_root_uid()
{
   my $retval = $FALSE;
   if (!is_right_user($ROOT_ACCT))
   {
      print "You are not authorized to run '" . get_basename($0) . "' command.\n";
      baselog($LOG_INFO, "User '" . (getpwuid($<))[0] . "' is trying to execute '" . get_basename($0) . "' command.");
      $retval = $TRUE;
   } ## end if (!is_right_user($ROOT_ACCT...
   return $retval;
} ## end sub warning_if_not_root_uid()

###############################################################################
## @fn sub get_tmp_filename($filename)
# @brief   Provide temporary file name for the give file name.
# @details Temporary file name will be like: 'file.555.tmp' where 555 is PID
# @param   $filename - a file name.
# @param   $extra_postfix - additional postfix (optional)
# @return  empty string if $filename is not defined otherwise new file name.
sub get_tmp_filename($;$)
{
   my ($filename, $extra_postfix) = @_;

   if ((not defined $filename) or ($filename eq ""))
   {
      baselog($LOG_ERROR, 'The filename is not specified.');
      return '';
   }

   if ((not defined $extra_postfix) or ($extra_postfix eq ''))
   {
      $extra_postfix = 'tmp';
   }

   baselog($LOG_DEBUG, "Preparing temporary file name for the file '$filename'.");

   $filename = $filename . '.' . $$ . '.' . $extra_postfix;

   baselog($LOG_DEBUG, "The name for temporary file is '$filename'.");

   return $filename;
} ## end sub get_tmp_filename($;$)

###############################################################################
## @fn sub get_backup_filename($filename, $extra_postfix)
# @brief   Provide backup name for the give file name.
# @details This method uses the subroutine transform_file_name in order to get
#          file name suitable to be backup the file name.
# @param   $filename - a file name.
# @param   $extra_postfix - additional postfix (optional).
# @return  empty string if $filename is not defined otherwise new file name.
sub get_backup_filename($;$)
{
   my ($filename, $extra_postfix) = @_;

   if ((not defined $filename) or ($filename eq ''))
   {
      baselog($LOG_ERROR, 'The filename is not specified.');
      return '';
   }

   if ((not defined $extra_postfix) or ($extra_postfix eq ''))
   {
      $extra_postfix = '.bak';
   }

   baselog($LOG_DEBUG, "Preparing backup name for the file '$filename'.");

   $filename = $filename . $extra_postfix;

   baselog($LOG_DEBUG, "Backup name is '$filename'.");

   return $filename;
} ## end sub get_backup_filename($;$)

###############################################################################
## @fn sub backup_and_replace($original, $backup, $new)
# @brief   Backup a file and replace it by a new one.
# @details This method backups a file $original into another file $backup,
#          after this the original file is replaced by a new one with
#          permission and ownership preserving.
#
#          If some error happens, new file is removed.
# @param   $original - file name of an original file
# @param   $backup   - name of file which is used for backing up
# @param   $new      - name of a new file
# @return  TRUE - all operations are performed successfully
# @return  FALSE - some error happens
sub backup_and_replace($$$)
{
   my ($original, $backup, $new) = @_;

   if (   base_system::do_system([$CHMOD, "--reference=$original", $new])
       && base_system::do_system([$CHOWN, "--reference=$original", $new])
       && base_system::do_system([$MV, '-f', $original, $backup])
       && base_system::do_system([$MV, '-f', $new,      $original]))
   {
      return $TRUE;
   } ## end if (base_system::do_system...
   else
   {
      base_system::do_system([$RM, '-f', $new]);
      return $FALSE;
   }
} ## end sub backup_and_replace($$$)

###############################################################################
## @fn sub replace_by_new($original, $new)
# @brief   Replace a file by a new one.
# @details This method replaces the original file by a new one with permission
#          and ownership preserving.
# @param   $original - file name of an original file
# @param   $new      - name of a new file
# @return  $TRUE  - all operations are performed successfully
# @return  $FALSE - some error happens
# @note    If some error happens, new file is removed.
sub replace_by_new($$)
{
   my ($original, $new) = @_;

   my $retval = $FALSE;

   #If the original file exists, the new one will inherits its permissions,
   #before replacing
   if (-f $original)
   {
      if (   base_system::do_system([$CHMOD, "--reference=$original", $new])
          && base_system::do_system([$CHOWN, "--reference=$original", $new])
          && base_system::do_system([$MV,    '-f',                    $new, $original]))
      {
         $retval = $TRUE;
      } ## end if (base_system::do_system...
   } ## end if (-f $original)
   else
   {

      #If the original file does not exist, just replace it
      $retval = base_system::do_system([$MV, '-f', $new, $original]);
   } ## end else [ if (-f $original)

   if (not $retval)
   {
      base_system::do_system([$RM, '-f', $new]);
   }

   return $retval;
} ## end sub replace_by_new($$)

###############################################################################
## @fn sub write_file($file, $content, $not_backup)
# @brief Write content to a file.
# @details This method writes some data to a temporary file, and then replace
#          the original file by this temporary one. Prior to replacing, it
#          tries to backup the original file.
# @param $file            - file name
# @param $content         - new content of the file. It can be string variable, link
#                           to array or hash.
#                           In case if it is
#                             *string variable - value of this variable will be written
#                             *ref to array - elements of array will be written in array
#                                context(without '\n' symbols between elements)
#                             *ref to hash - strings in next format will be written into file:
#                                KEY1=$HASH{KEY1}
#                                KEYn=$HASH{KEYn}
# @param $not_backup      - (optional) if it is true, the file will not be backed up
# @param $get_tmp_file    - (optional) ref to sub that creates temporary file name
#                           if not se then default will be used (get_tmp_filename)
# @param $get_backup_file - (optional) ref to sub that creates backup file name
#                           if not se then default will be used (get_backup_filename)
# @return $TRUE  - the content is written to the file.
# @return $FALSE - some error happens
sub write_file($$;$$$)
{
   my ($file, $content, $not_backup, $get_tmp_func, $get_backup_func) = @_;

   if (not defined $file)
   {
      baselog($LOG_ERROR, 'File should be defined.');
      return $FALSE;
   }
   if (not defined $content)
   {
      baselog($LOG_ERROR, 'Content should be defined.');
      return $FALSE;
   }
   baselog($LOG_DEBUG, "Trying to write the file '$file'...");

   if (not defined $not_backup)
   {
      $not_backup = $FALSE;
   }

   if (not defined $get_tmp_func)
   {
      $get_tmp_func = \&get_tmp_filename;
   }

   if (not defined $get_backup_func)
   {
      $get_backup_func = \&get_backup_filename;
   }

   my $tmp_name = $get_tmp_func->($file);

   if ($tmp_name eq '')
   {
      return $FALSE;
   }

   my $fd;
   if (not open($fd, '>', $tmp_name))
   {
      baselog($LOG_ERROR, "Cannot create new file '$file': $!.");
      return $FALSE;
   }

   if (is_array($content))
   {
      print $fd @$content;
   }
   elsif (is_hash($content))
   {
      foreach my $key (keys %{$content})
      {
         if (defined($content->{$key}))
         {
            print $fd "$key=" . $content->{$key} . "\n";
         }
         else
         {
            baselog($LOG_WARNING, "The value for key '$key' is not defined.");
         }
      } ## end foreach my $key (keys %{$content...
   } ## end elsif (is_hash($content))
   else
   {
      print $fd $content;
   }

   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
      return $FALSE;
   }

   my $retval = $TRUE;

   my $backup_name = $get_backup_func->($file);

   # Try to prepare the very first backup. When a file exists, but its backup does not
   if ((not $not_backup) and (-f $file) and (not -f $backup_name))
   {
      baselog($LOG_DEBUG, "Original file: '$file', backup file: '$backup_name', temporary file: '$tmp_name'");
      $retval = backup_and_replace($file, $backup_name, $tmp_name);
   }
   else
   {
      baselog($LOG_DEBUG, "Original file: '$file', temporary file: '$tmp_name'. No backup.");
      $retval = replace_by_new($file, $tmp_name);
   }

   return $retval;
} ## end sub write_file($$;$$$)

###############################################################################
## @fn     delete_line_from_file($file, $pattern)
# @brief   Delete lines from file, which includes pattern string
# @details This method uses the perl "grep" function to find lines with
#          given pattern and delete thouse lines from file.
# @param   $file       - file name
# @param   $pattern    - string, which should be found in file.
# @return  TRUE        - all operations are performed successfully or nothing is
#                        changed
# @return  FALSE       - some error happens
sub delete_line_from_file($$)
{
   my ($file, $pattern) = @_;

   if (not defined $file)
   {
      baselog($LOG_ERROR, "File should be defined");
      return $FALSE;
   }

   if (not defined $pattern)
   {
      baselog($LOG_ERROR, "Pattern should be defined.");
      return $FALSE;
   }

   if (!-e "$file")
   {
      baselog($LOG_ERROR, "File '$file' doesn't exist");
      return $FALSE;
   }

   my $fd;
   if (not open($fd, '<', $file))
   {
      baselog($LOG_ERROR, "Can't open file: '$file'");
      return $FALSE;
   }

   my @file_content = <$fd>;

   $pattern = make_regexp_pattern_save($pattern);
   my @grep_results = grep (!/$pattern/, @file_content);

   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
      return $FALSE;
   }

   return write_file($file, \@grep_results);

} ## end sub delete_line_from_file($$)

###############################################################################
## @fn     make_regexp_pattern_save($pattern)
# @brief   Replace all special characters
# @details This method replace all special characters in given string
# @param   $pattern    - string, which contains special characters
# @return              - string with replaced special characters
sub make_regexp_pattern_save($)
{
   my $pattern       = shift @_;
   my @split_pattern = split(//, $pattern);
   my @tmp           = ();
   foreach my $letter (@split_pattern)
   {
      if ($letter =~ /[\\\^\.\$\|\(\)\[\]\*\+\?\{\}]/)
      {
         push @tmp, '\\';
      }
      push @tmp, $letter;
   } ## end foreach my $letter (@split_pattern...

   return join('', @tmp);
} ## end sub make_regexp_pattern_save($)

###############################################################################
## @fn change_permissions_and_ownership($mode, $owner, $group, $file_list, $prefix, $no_log)
# @brief   Change permissions and ownership of the files
# @details This method uses the perl subroutines chown and chmod in order
#          to change permissions and ownership for the list of files.
# @param $mode - new permissions (undef means do not change)
# @param $owner - numeric ID of owner (undef means do not change)
# @param $group - numeric ID of group (undef means do not change)
# @param $file_list - reference to the file list
# @param $prefix - prefix which will be provided with info message (optional)
# @param $no_log - do not log INFO message, describing successfull changes (optional)
# @return  $TRUE - all opperation are performed successfully or nothing is changed
# @return  $FALSE - some error happened
sub change_permissions_and_ownership($$$$;$$)
{
   my ($mode, $owner, $group, $file_list, $prefix, $no_log) = @_;

   if ((defined $prefix) and ($prefix ne ""))
   {
      $prefix = $prefix . ": ";
   }
   else
   {
      $prefix = "";
   }

   if (not defined $no_log)
   {
      $no_log = $FALSE;
   }

   # -1 is standard constant for chown perl function if it is unnecessary to
   # change the owner
   if (not defined $owner)
   {
      $owner = -1;
   }

   # -1 is standard constant for chown perl function if it is unnecessary to
   # change the group
   if (not defined $group)
   {
      $group = -1;
   }

   my $retval = $TRUE;

   if ((not defined $mode) and ($owner == -1) and ($group == -1))
   {
      baselog($LOG_INFO, "Nothing is to be changed.");
      return $retval;
   }

   if (defined $file_list)
   {

      foreach my $file (@{$file_list})
      {
         my $object = "";
         if (-d $file)
         {
            $object = "directory";
         }
         elsif (-f $file)
         {
            $object = "file";
         }

         if ($object ne "")
         {
            my $res = $TRUE;

            #Change the permissions
            if ((defined $mode) and (chmod($mode, $file) == 0))
            {
               $res = $FALSE;
            }

            #Change the owner
            if ((($owner != -1) or ($group != -1)) and (chown($owner, $group, $file) == 0))
            {
               $res = $FALSE;
            }

            if ($res)
            {

               #To not log INFO message if $no_log was provided.
               my $log_level = $LOG_DEBUG;
               if ($no_log == $FALSE)
               {
                  $log_level = $LOG_INFO;
               }
               baselog($log_level, $prefix . "the permissions and owner of the $object '$file' changed successfully.");
            } ## end if ($res)
            else
            {
               $retval = $FALSE;
            }
         } ## end if ($object ne "")
         else
         {
            baselog($LOG_ERROR, $prefix . "the file system object '$file' does not exist.");
            $retval = $FALSE;
         }
      } ## end foreach my $file (@{$file_list...
   } ## end if (defined $file_list...
   else
   {
      baselog($LOG_ERROR, "List of file is not provided.");
      $retval = $FALSE;
   }

   return $retval;
} ## end sub change_permissions_and_ownership($$$$;$$)

## @fn sub is_valid_group($grp)
# @brief Checks if group is in the groups list.
# @param $grp - groups's name or groups's id for check.
# @return $TRUE  - if specified group is in the groups list
# @return $FALSE - otherwise
# @author Evgeny Lazarev
# @date June 2008
# @note
sub is_valid_group($)
{
   my $grp = shift(@_);
   my @groups_list = (keys %GROUPS, values %GROUPS);
   return (pos_in_array(\@groups_list, $grp) >= 0);
}

## @fn sub get_mntdevice($mntpoint)
# @brief Get mounted device name by mount point.
# @param $mntpoint - mount point.
# @return $device  - mounted device name.
# @return undef - otherwise
# @author Ekaterina Bazhanova
# @date October 2008
# @note
sub get_mntdevice($)
{
   my $mntpoint = shift @_;
   my $file     = "/etc/mtab";
   my $device   = undef;

   #Check if the device is a mount point
   if (-d $mntpoint)
   {
      my $is_mountpoint_exist = $FALSE;
      my $fh                  = undef;
      unless (open($fh, '<', $file))
      {
         baselog($LOG_ERROR, "Cannot open file for reading [ $file ]");
         return undef;
      }
      my @mountpoints = <$fh>;
      unless (close($fh))
      {
         baselog($LOG_ERROR, "Cannot close file [ $file ]");
         return undef;
      }
      foreach my $line (@mountpoints)
      {
         if ($line !~ /^\s*\#/)
         {
            if ($line =~ /\/dev\/(\w+)\s+$mntpoint\s+/)
            {
               $is_mountpoint_exist = $TRUE;
               $device              = $1;
               last;
            } ## end if ($line =~ /\/dev\/(\w+)\s+$mntpoint\s+/...
         } ## end if ($line !~ /^\s*\#/)
      } ## end foreach my $line (@mountpoints...
      unless ($is_mountpoint_exist)
      {
         baselog($LOG_WARNING, "There is no such mount point: $mntpoint");
         return undef;
      }
   } ## end if (-d $mntpoint)
   return $device;
} ## end sub get_mntdevice($)

## @fn sub get_hostname(;$long)
# @brief returns hostname of the current system
# @param $long - if TRUE, return FQDN, else just a short hostname version.
# @return $hostname - hostname or undef if an error occurred
# @author Ivanov Vladimir
# @date November 2008
# @note
sub get_hostname(;$)
{
   my $long     = shift @_;
   my $hostname = hostname();
   if ((not defined $long or not $long) and $hostname =~ /([^.]+)/)
   {
      $hostname = $1;
   }
   return $hostname;
} ## end sub get_hostname(;$)

## @fn sub recover_env_path()
# @brief   this procedure overwrite global $EVN{'PATH'} variable
# @details place /bin and /usr/bin in the $ENV{'PATH'} and so make these
#          directories accessible for child processes (e.g. 'tar xzf' can use /bin/gzip)
#          Useful under restricted shell environment.
#          This subroutine is secure to be invoked from our Perl scripts: it just implements
#          flexibility to run command without full path from third party scripts/command
#          without changing them. If a user could modify or substitute some script it means
#          that this user can modify this script to use full pathes in it.
#          Also, environment is not changed for parent process, so, the restricted shell is not extended.
# @date January 2009
# @note
sub recover_env_path()
{
   $ENV{'PATH'} .= ':/bin:/usr/bin';
}

## @fn sub set_parameter($filename, $parameter, $param_value)
# @brief Set the specified parameter to the specified value in the
#        specified file
# @param $filename
# @param $parameter
# @param $param_value
# @return $TRUE  - Parameter set successfully.
# @return $FALSE - Parameter not set.
# @author Kushal Agarwal
# @date December 2008
# @note
sub set_parameter($$$)
{
   my ($filename, $parameter, $param_value) = @_;
   my $retval = $TRUE;

   my $fdread;
   if (not open($fdread, "<", $filename))
   {
      baselog($LOG_ERROR, "Cannot read '$filename': $!");
      $retval = $FALSE;
   }
   else
   {

      #Check if it is already configured
      my $found = $FALSE;

      my $content = "";

      #copy content from original file
      foreach my $line (<$fdread>)
      {

         #comment out the lines
         if ($line !~ /^\s*#/)
         {
            if ($line =~ /^\s*$parameter\s*/)
            {
               if ($line =~ /^\s*$parameter\s*$param_value\s*$/)
               {

                  #If it is already configured correctly
                  return $FALSE;
               } ## end if ($line =~ /^\s*$parameter\s*$param_value\s*$/...
               else
               {
                  $line = "";
               }
            } ## end if ($line =~ /^\s*$parameter\s*/...
         } ## end if ($line !~ /^\s*#/)
         $content .= $line;
      } ## end foreach my $line (<$fdread>...

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         $retval = $FALSE;
      }
      else
      {
         $content .= "\n$parameter $param_value\n";
         if (write_file($filename, $content))
         {
            baselog($LOG_INFO, "The file '$filename' changed successfully.");
         }
         else
         {
            $retval = $FALSE;
         }
      } ## end else [ if (not close $fdread)
   } ## end else [ if (not open($fdread, ...
   return $retval;
} ## end sub set_parameter($$$)

###############################################################################
## @fn     get_mount_counter($mount_path)
# @brief   It checks how many mount entries have this mount point in use.
# @param   $mount_path - "entry point" path
# @return   $mount_counter - number of mount entries
sub get_mount_counter($)
{
   my $mount_path = shift @_;

   my $mount_counter = 0;
   if ((!defined $mount_path) or ($mount_path eq ''))
   {
      baselog($LOG_ERROR, 'Mount path is undefined.');
      return 0;
   }

   # if mount path has last "/" symbol it is better to cut it out
   if ($mount_path ne "/")
   {
      $mount_path =~ s/[\/]*$//;
   }

   # if mount path has double "/" it is better to cut one of it out
   $mount_path =~ s/\/{2,}/\//g;
   my @mounts = ();
   base_system::do_system([$GREP, $mount_path, '/proc/mounts'], {$base_system::dsOUT => \@mounts, $base_system::dsNOERROR => $TRUE});
   chomp(@mounts);
   $mount_counter = scalar(@mounts);
   baselog($LOG_DEBUG, "There are '$mount_counter' mount records for '$mount_path'.");
   if ($mount_counter > 1)
   {
      baselog($LOG_INFO, "There are '$mount_counter' mount records for '$mount_path'.");
   }
   return $mount_counter;
} ## end sub get_mount_counter($)

###############################################################################
# @fn is_array_unique($array_ref)
# @brief Checks, that all elements in the array are unique
# @param $array_ref - reference to array for check
# @return $TRUE - all array elements are unique
# @return $FALSE - otherwise
# @note
sub is_array_unique($)
{
   my $array_ref = shift @_;
   if (not is_array($array_ref))
   {
      baselog($LOG_ERROR, "Not an array reference passed.");
      return $FALSE;
   }

   my %seen    = ();
   my @tmp_arr = grep {!$seen{$_}++} @{$array_ref};

   return (scalar @{$array_ref} == scalar @tmp_arr) ? $TRUE : $FALSE;
} ## end sub is_array_unique($)

######################################################
# @fn parseFiles($xmlpath, $err)
# @brief  parses an XML file
# @param  $xmlpath - path to the XML file
# @param  $err - for error description (optional)
# @return LibXML document object if no errors, undef - otherwise
# @author Pavel Sergeev
# @date   February 2009
######################################################
sub parseFile($;$)
{
   my $xmlpath = shift @_;
   my $err     = shift @_;
   my $xmldoc;  #dom object
   if (defined($xmlpath))
   {
      if (!(-e $xmlpath))
      {
         baselog($LOG_ERROR, "\"$xmlpath\" file does not exist");
         return undef;
      }
   } ## end if (defined($xmlpath))
   else
   {
      baselog($LOG_ERROR, "Undefined parameter\n");
      return undef;
   }
   my $xmlparser = XML::LibXML->new;

   #enable line numbers
   $xmlparser->line_numbers($TRUE);
   eval {$xmldoc = $xmlparser->parse_file($xmlpath);};
   if ($@)
   {
      $@ =~ s/at .*?$//g;  #delete filepath printing
      $@ =~ s/\^//g;       #delete ^ symbols
      if (defined $err)
      {
         $$err = $@;
      }
      $@ =~ s/\n+//g;      #remove new line symbols
      baselog($LOG_ERROR, "Invalid XML syntax: $@");
      $XML_PATHS{$xmlpath} = $FALSE;
      return undef;
   } ## end if ($@)
   $XML_PATHS{$xmlpath} = $TRUE;
   return $xmldoc;
} ## end sub parseFile($;$)

###############################################################################
## @fn     list2hash($list, $val)
# @brief   Converts the given list to a hash (each value is the key for the
#          resulting hash
# @param   $list - reference to a list
# @param   $val - default value associated with each key ($TRUE if not defined)
# @return  %hash - resulting hash, () if not array reference present
# @author  Nikolay Panov
# @date    February 2009
# @note
sub list2hash($;$)
{
   my ($list, $val) = @_;
   my %hash = ();
   unless (is_array($list))
   {
      baselog($LOG_ERROR, "Not an array reference passed.");
      return %hash;
   }
   $val = $TRUE unless defined $val;
   foreach my $elem (@$list)
   {
      $hash{$elem} = $val;
   }

   return %hash;
} ## end sub list2hash($;$)

###############################################################################
## @fn     is_integer($string)
# @brief   Checks if passed argument is an integer number.
# @param   $string - string for check
# @return  $TRUE - passed string is an integer number
# @return  $FALSE - passed string is not an integer number
# @author  Evgeny Lazarev
# @date    February 2009
sub is_integer($)
{
   my $string = shift(@_);
   my $retval = $FALSE;
   if (!defined($string))
   {
      baselog($LOG_ERROR, "Passed argument is not defined.");
      return $retval;
   }
   if ($string =~ /^-?\d+$/)
   {
      $retval = $TRUE;
   }
   return $retval;
} ## end sub is_integer($)

############################################################################
## @fn     is_package_installed($package_path)
# @brief   Checks that an rpm package is installed
# @param   $package_path - path to the rpm package
# @return  $TRUE - the package is installed.
# @return  $FALSE - in case of any error or if the package is not installed.
# @author  Pavel Sergeev
# @date    March 2009
sub is_package_installed($)
{
   my $package_path = shift @_;
   if (not defined $package_path)
   {
      baselog($LOG_ERROR, "Undefined parameter");
      return $FALSE;
   }
   my @output = ();
   my $ret    = $FALSE;

   #get package version from package file
   if (not do_system([$RPM, '-qp', '--qf', '%{NAME} %{VERSION}-%{RELEASE}\n', '--nosignature', $package_path], {$base_system::dsOUT => \@output}))
   {
      baselog($LOG_ERROR, "Cannot get information for the package '$package_path'.");
      return $FALSE;
   }
   else
   {
      if (scalar(@output) > 1)
      {
         baselog($LOG_ERROR, "Incorrect format of information for the package '$package_path'.");
      }
      else
      {
         my $line = $output[0];

         #split rpm name and version
         my ($name, $version) = split /\s+/, $line;
         @output = ();

         #get rpm version from rpm database
         do_system([$RPM, '-q', '--queryformat', '%{VERSION}-%{RELEASE}\n', '--nosignature', $name],
                   {$base_system::dsRETVAL => \$ret, $base_system::dsNOERROR => $TRUE, $base_system::dsOUT => \@output});
         if ($ret)
         {
            return $FALSE;
         }

         #compare installed package version and package version from file
         foreach my $installed_version (@output)
         {
            chomp $installed_version;
            return $TRUE if ($version eq $installed_version);
         }
      } ## end else [ if (scalar(@output) > ...
   } ## end else [ if (not do_system([$RPM...
   return $FALSE;
} ## end sub is_package_installed($)

###############################################################################
## @fn     ssh_hang_on_exit_bug()
# @brief   SSH Hang on exit workaround
# @param   none
# @return  $OK - passed
# @return  $ERROR - error detected
# @author
# @date    April 2009

sub ssh_hang_on_exit_bug()
{
   my $ppid   = getppid();
   my $TMP_PL = "/tmp/tmp.$$.pl";

   baselog($LOG_INFO, "DIE DIE PPID=$ppid, PID=$$");

   # Create the tmp perl script
   my $FD;

   unless (open(FD, ">", $TMP_PL))
   {
      baselog($LOG_ERROR, "Unable to open file to create script to kill PPID=$ppid (PID=$$)");
      baselog($LOG_ERROR, "- \"$ppid sshd: nortel\@notty\" may be stuck until a reboot");
      return $ERROR;
   } ## end unless (open(FD, ">", $TMP_PL...
   print FD "#!/usr/bin/perl\n";
   print FD "use lib \"/opt/nortel/base/lib\";\n";
   print FD "use warnings;\n";
   print FD "use strict;\n";
   print FD "use common_variables;\n";
   print FD "use common_functions;\n";
   print FD "use baselog;\n";

   #
   # Log to the fact that the "hung" ssh tty session is about to be
   # killed
   # - delay the killing for a few seconds to let us exit gracefully :-|
   #
   print FD "sleep 2;\n";
   print FD "baselog($LOG_INFO,\"DIE DIE PPID=$ppid, PID=$$\");\n";
   print FD "kill 1, $ppid;\n";
   print FD "unlink \"$TMP_PL\";\n";
   unless (close FD)
   {
      baselog($LOG_ERROR, "Unable to close $TMP_PL file handle");
      return $ERROR;
   }

   #
   # Execute the above script and exit
   # - if the ppid is truly gone then this is an inconsequential action
   #
   unless (chmod 0770, $TMP_PL)
   {
      baselog($LOG_ERROR, "Unable to chmod $TMP_PL");
      return $ERROR;
   }
   system("$TMP_PL &");  # stuff into background and get out of here

   return $OK;
}  # ssh_hang_on_exit_bug

1;

__END__

=head1 NAME

common_functions - common functions for all base tools.

=head1 SYNOPSIS

use common_functions;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

