#! /usr/bin/perl

## @file dmiParser.pm
# @brief dmidecode output parser
# @details This PM module is an API, which provides
#          possibility to store HW system information
#          in the nortel-linuxbase-info file.
#          It is created under the "HW platform type
#          recognition" activity.
# @version 1.00 - initial creation
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - Logic of checking CP-PM, in case CMOS is not set to defaults, is added.
# @version 1.03 - Subroutine dmi_get_bios_version, dmi_get_system_serial
#                 and dmi_get_dmi_property are added by Alex Arkhipov
# @version 1.04 - get_hw_type func is changed to not show warning banner about reseting 
#                 CMOS setting to default always, correction in do_system.
# @author Yulya Bezrukova
# @date April, 2008
# @note To use subroutines, defined in this module, root permissions are required

## @namespace dmiParser
package dmiParser;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;

use base_system;
use baselog;

use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(&get_hw_type &dmi_get_bios_version &dmi_get_system_serial &dmi_get_dmi_property);

our $VERSION = '1.04';

## @var \RET_TYPE
# @brief It is the array of known platform types.
# @details For each platform type there is its small unique representation.
#          Note: Due to BUPM and CPPM has "Nortel" in its names there is a
#                possibility that CPPM will be identified as BUPM platform.
#                We might perform several check on the line, but it is not
#                universal way. It was decided to use regular expressions
#                in platform type names.
#                ^ - means start of string.
#                \\Z - means end of string.
#                Using these two anchors allows us to identify platforms
#                explicitly and avoid complexity of the code.
#                Ex.: "^Nortel\\Z" means that there is no any other symbol
#                     except "Nortel" in the "Product Name" line of the
#                     dmidecode output for the BUPM platform.
my %RET_TYPE = (
                "^IBM eServer 306m"        => "IBM X306M",
                "^IBM System x3350"        => "IBM X3350",
                "^ProLiant DL320 G4"       => "HP DL320G4",
                "^ProLiant DL360 G5"       => "HP DL360G5",
                "^Nortel\\Z"               => "KONTRON BUPM",
                "^Nortel CP-PM"            => "Nortel CPPMv1",
                "^Nortel CPPMv2"           => "Nortel CPPMv2",
                "^PowerEdge R300\\Z"       => "Dell R300",
                "^VMware Virtual Platform" => "VMware",
                "^Virtual Machine"         => "VirtualPC"
               );

## @var $NO_SERIAL
# @brief this line is returned by/usr/sbin/dmidecode -s  dmidecode if there is no serial number set
my $NO_SERIAL = 'No System Serial Number';

## @var $DMI_CMD
# @brief path to dmidecode
my $DMI_CMD = '/usr/sbin/dmidecode';

## @var $DMI_OPTION
# @brief option to query property of dmidecode
my $DMI_OPTION = '-s';

# Property names
my $PROP_SYSTEM_PRODUCT_NAME  = 'system-product-name';
my $PROP_BIOS_VERSION         = 'bios-version';
my $PROP_SYSTEM_SERIAL_NUMBER = 'system-serial-number';

## @skipline
sub dmi_get_dmi_property($);

## @fn get_hw_type()
# @brief provides the name of HW platform type
# @details The function uses /usr/sbin/dmidecode tool in order to get
#          unique platform identification.
#          The whole parsing is based on regular expressions.
# @param None
# @retval name - in case it is identified
# @retval "UNKNOWN" - in case the name of hw platform
#         cannot be identified.
sub get_hw_type()
{
   my $ProductName = dmi_get_dmi_property($PROP_SYSTEM_PRODUCT_NAME);

   if ($ProductName)
   {
      for my $i (keys %RET_TYPE)
      {
         if ($ProductName =~ /$i/)
         {
            return $RET_TYPE{$i};
         }
      } ## end for my $i (keys %RET_TYPE...

      baselog($LOG_ERROR, "Cannot find HW platform type in the pre-defined list.");
      return "UNKNOWN";
   } ## end if ($ProductName)
   else
   {
      if (do_system([$GREP, 'Intel(R) Pentium(R) M processor', '/proc/cpuinfo']))
      {
         return $RET_TYPE{"^Nortel CP-PM"};
      } ## end if (!do_system("grep 'Intel(R) Pentium(R) M processor' /proc/cpuinfo"...
   } ## end else [ if ($ProductName)

   baselog($LOG_ERROR, "Cannot get HW platform type --- coded as UNKNOWN.");
   return "UNKNOWN";
} ## end sub get_hw_type()

## @fn dmi_get_bios_version()
# @brief provides the bios version
# @details To get bios version this function uses /usr/sbin/dmidecode tool
# @param None
# @retval bios_ver
sub dmi_get_bios_version()
{
   my $bios_ver = dmi_get_dmi_property($PROP_BIOS_VERSION);

   baselog($LOG_DEBUG, "BIOS version is $bios_ver");
   return $bios_ver;
} ## end sub dmi_get_bios_version()

## @fn dmi_get_system_serial()
# @brief provides the system serial number
# @details To get serial number this function uses /usr/sbin/dmidecode tool
# @param None
# @retval $serial
# @retval '' - if serial is not available
sub dmi_get_system_serial()
{
   my $serial = dmi_get_dmi_property($PROP_SYSTEM_SERIAL_NUMBER);

   # Check for unknown serial
   if ($serial eq $NO_SERIAL)
   {
      baselog($LOG_WARNING, 'No serial detected');
      $serial = '';
   }
   baselog($LOG_DEBUG, "Serial is $serial");
   return $serial;
} ## end sub dmi_get_system_serial()

## @fn dmi_get_dmi_property()
# @brief return property value of dmiDecode -s "property name"
# @details This function chomp value before returning
# @param $property - property name
# @retval $value - property value
# @retval '' - in any errors found
sub dmi_get_dmi_property($)
{
   my $property = shift(@_);
   return '' if (not defined($property));
   my $value = '';
   my @out;
   my $rc;

   my $cmd = [$DMI_CMD, $DMI_OPTION, $property];

   if (not do_system($cmd, {$dsRETVAL => \$rc, $dsOUT => \@out}))
   {
      baselog($LOG_ERROR, 'Error executing command: ' . join(' ', @$cmd));
	  return '';
   }

   chomp(@out);

   if (scalar(@out) == 0)
   {
      baselog($LOG_WARNING, 'Return value is empty');
	  return '';
   }

   if (scalar(@out) != 1)
   {
      baselog($LOG_WARNING, 'More than one line returned: ' . scalar(@out));
   }

   # Return only first line
   $value = shift(@out);
   return $value;
}

1;

__END__
