###############################################################################
## @file 00OS_harden_ctrlaltdel.pm
# @brief This module performs hardening of /etc/inittab
# @details This module is part of LinuxBase OS hardening and performs hardening
#          of /etc/inittab.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 00OS_harden_ctrlaltdel
package hardening::basic::00OS_harden_ctrlaltdel;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use hardening::hardening_common;
use common_variables;
use baselog;

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (ctrlaltdel)";

my $inittab = "/etc/inittab";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens /etc/inittab file
# @details This method checks if it is necessary to comments out the line in
#          /etc/inittab file which configures Ctrl+Alt+Del handler and do such
#          changes.
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");
   
   my $fdread;
   if (not open($fdread, "<", $inittab))
   {
      baselog($LOG_ERROR, "Cannot read '$inittab': $!");
      $retval = $FALSE;
   }
   else
   {
      my $found = $FALSE;
      my $content = "";
      
      #copy content from original file
      foreach my $line (<$fdread>)
      {
         #comment out the line
         if (($line !~ /^\s*#/) and ($line =~ /(ca::ctrlaltdel.*)/))
         {
            baselog($LOG_DEBUG, "The handler for Ctrl+Alt+Del is found in the file '$inittab'.");
            $found = $TRUE;
            $line = "#" . $1; 
         }
         $content .= $line;
      }

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         $retval = $FALSE;      
      }
      else
      {
         if (($found) and (write_harden_file($inittab, $content)))
         {
            baselog($LOG_INFO, "$DODitem: the file '$inittab' changed successfully.");
         }
         elsif (not $found)
         {
            baselog($LOG_INFO, "It seems that the handler for Ctrl+Alt+Del is already disabled.");
         }
         else
         {
            $retval = $FALSE;
         }
      }
   }
   
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
