###############################################################################
## @file 00OS_harden_grub.pm
# @brief This module performs hardening of GRUB configuration file
# @details This module is part of LinuxBase OS hardening and performs hardening
#          of GRUB.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 00OS_harden_grub
package hardening::basic::00OS_harden_grub;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (GRUB)";

my $grub_conf = "/boot/grub/grub.conf";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens GRUB configuration file
# @details This method checks if it is necessary to harden some options in GRUB
#          configuration file:
#      \li - hiddenmenu option is configured
#      \li - timeout is set in 0
#      \li - GRUB access password is set
#      \li - splash image is turned off
#
#          If it not so, the methods changes the file.
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $fdread;
   if (not open($fdread, "<", $grub_conf))
   {
      baselog($LOG_ERROR, "Cannot read '$grub_conf': $!");
      $retval = $FALSE;
   }
   else
   {
      #Get root password in order to put it to the grub.conf
      my (undef, $rootpwd) = getpwuid($ROOT_UID);

      my $found = $FALSE;
      my $content = "";
      
      #copy content from original file
      foreach my $line (<$fdread>)
      {
         #comment out the lines
         if ($line !~ /^\s*#/)
         {
            #Anaconda sets "=", we replace them by " ". It is also correct and
            #will help us to distinguish modified options.
            if (($line =~ /^default=(\d+)/))
            {
               baselog($LOG_DEBUG, "The options 'hiddenmenu' is set.");
               $line = "hiddenmenu\n" . "default " . $1 . "\n"; 
               $found = $TRUE;
            }
            elsif (($line =~ /^timeout=(\d+)/))
            {
               baselog($LOG_DEBUG, "Zero timeout and access password are set.");
               $line = "timeout 0\n" . "password --md5 " . $rootpwd . "\n"; 
               $found = $TRUE;
            }
            elsif (($line =~ /^(splashimage)/))
            {
               baselog($LOG_DEBUG, "Splash image is turned off.");
               $line = "#" . $1 . "\n"; 
               $found = $TRUE;
            }
            elsif (($line =~ /^(terminal .*--timeout=)(\d+)(.*)/))
            {
               baselog($LOG_DEBUG, "Zero timeout for terminal option is set.");
               $line = $1 . "0" . $3 . "\n"; 
               $found = $TRUE;
            }
         }
         $content .= $line;
      }

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         $retval = $FALSE;
      }
      else
      {
         if (($found) and (write_harden_file($grub_conf, $content)))
         {
            baselog($LOG_INFO, "$DODitem: the file '$grub_conf' changed successfully.");
         }
         elsif (not $found)
         {
            baselog($LOG_INFO, "It seems that GRUB is already hardened.");
         }
         else
         {
            $retval = $FALSE;
         }
      }
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
