###############################################################################
## @file 00OS_harden_initd_dir.pm
# @brief This module hardens init scripts of Linux services
# @details This module is part of LinuxBase OS hardening and hardens init
#          scripts of Linux services situated in the directory /etc/rc.d/init.d/
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - added changing ownership to root:root for all files under /etc/init.d 
#                 and this module invocation during appinstall 

## @class 00OS_harden_initd_dir
package hardening::basic::00OS_harden_initd_dir;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.02';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (init.d content)";

my $init_dir = "/etc/rc.d/init.d";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens init scripts of Linux services
# @details This method looks for init scripts of Linux services in the
#          directory /etc/rc.d/init.d/ and if they are more permissive than 750
#          changes their permissions. The file in the directory
#          /etc/rc.d/init.d/functions is treated separately in order to be
#          readable for non-root accounts (Nortel applications should have the
#          ability to source it).
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   if ((not -e $init_dir) or (not -d $init_dir))
   {
      baselog($LOG_ERROR, "The directory '$init_dir' does not exists.");
      $retval = $FALSE;
   }
   else
   {
      my $found = $FALSE;

      my $filename;
      #permissions should be 0754
      my $func_perms = (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH);
      #permissions should be 0750
      my $serv_perms = (S_IRWXU | S_IRGRP | S_IXGRP);
      #find all ser
      find(
         sub
         {
            my $filename = $File::Find::name;
            if ($filename eq $FUNCTIONS_LIB)
            {
               #permissions should be 0754
               if (not permissions_less_or_equal($filename, $func_perms))
               {
                  $found = $TRUE;
                  $retval &= change_permissions_and_ownership($func_perms, undef, undef, [$filename], $DODitem);
               }
            }
            else
            {
               #to not check symlinks and direcories
               if ((not -l $filename) and (not -d $filename))
               {
                  #permissions should be 0750
                  if (not permissions_less_or_equal($filename, $serv_perms))
                  {
                     $found = $TRUE;
                     $retval &= change_permissions_and_ownership($serv_perms, $ROOT_UID, $ROOT_GID, [$filename], $DODitem);
                  }
               }
            }
         }, $init_dir);

      if (not $found)
      {
         baselog($LOG_INFO, "All services init scripts are OK.");
      }
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
