###############################################################################
## @file 00OS_harden_securetty.pm
# @brief This module hardens /etc/securetty
# @details This module is part of LinuxBase OS hardening and hardens
#          /etc/securetty in line of the current COM port (TTYS)
#          and KVM (TTY1) usage policy
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - Added support for root login via TTY1 (KVM)
## @class 00OS_harden_securetty
package hardening::basic::00OS_harden_securetty;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.02';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use baseParams;

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = (
               $H_IF_APPLY => {
                               'sub'     => \&apply,
                               'visible' => $FALSE
                              },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_PATCHING => {
                                       'sub'     => \&apply,
                                       'visible' => $FALSE
                                      }
              );

my $DODitem = "OS hardening (COM, KVM)";

my $cmdline   = "/proc/cmdline";
my $securetty = "/etc/securetty";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens /etc/securetty
# @details This method tries to determine the current policy of COM port and KVM usage:
#      \li   Com port is configured through the kernel command line. This is
#            due to:
#           -#  COM-port based installation
#           -#  run-time system
#
#            In both these cases COM-port data is extracted from the kernel
#            command line.
#      \li   Com port is not configured through the kernel command line. This
#            is due to:
#           -#  KVM based installation - COM-port data is extracted from
#                 userinfo.txt.
#           -#  run-time system without console port redirection - no COM
#                 port data is needed. Root is not allowed to directly login to
#                 the system at all.
#
#          Depending on configured COM port usage, the file /etc/securetty is
#          changed - only configured COM port is allowed for root login.
#          Additionally root login is also allowed via TTY1 (using KVM).
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #Get the current console from the kernel command line.
   my $fdread;
   if (not open($fdread, "<", $cmdline))
   {
      baselog($LOG_ERROR, "Cannot read '$cmdline': $!");
      $retval = $FALSE;
   }
   else
   {

      my $console = "";

      foreach my $line (<$fdread>)
      {
         if ($line =~ /console=(ttyS\d+)/)
         {
            $console = $1;
         }
      } ## end foreach my $line (<$fdread>...

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      }

      #Even the system was configured to use KVM, console=ttySXX was set to
      #grub.conf. Such option should not be only incase of no console port
      #redirection chosen.
      if ($console eq "")
      {

         #It is necessary to determine if we are on installation stage and get
         #console data from userinfo.txt file. On run-time we will be here only
         #in case when no console port redirection configured, so, it is
         #determined from userinfo.txt - consolePort will be none.
         baselog($LOG_NOTICE, "Cannot get configured COM port from the kernel command line.");

         #Get information about configured COM port from userinfo.txt.
         my %userinfodata = ('consolePort' => '');
         if (not getParams(\%userinfodata))
         {
            baselog($LOG_ERROR, "Cannot get LinuxBase data");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_DEBUG, "consolePort = '$userinfodata{'consolePort'}'");

            #If there is console port redirection, use this data.
            if ($userinfodata{'consolePort'} =~ /ttyS\d+/)
            {
               $console = $userinfodata{'consolePort'};
            }
         } ## end else [ if (not getParams(\%userinfodata...
      } ## end if ($console eq "")

      if ($retval)
      {
         baselog($LOG_DEBUG, "The console '$console' will be used.");

         #Check if the configured /etc/securetty the same
         if (not open($fdread, "<", $securetty))
         {
            baselog($LOG_ERROR, "Cannot read '$securetty': $!");
            $retval = $FALSE;
         }
         else
         {
            my $foundException = $FALSE;
            my $foundTTYS      = $FALSE;
            my $foundTTY1      = $FALSE;

            #look at lines in /etc/securetty
            #If some line does not match with the configured tty mark the file as
            #incorrect
            foreach my $line (<$fdread>)
            {
               chomp $line;
               if ($line eq $console)
               {
                  $foundTTYS = $TRUE;
                  baselog($LOG_DEBUG, "The console '$console' is permitted.");
               }
               elsif ($line eq $TTY1)
               {
                  $foundTTY1 = $TRUE;
                  baselog($LOG_DEBUG, "The console '$TTY1' is permitted.");
               }
               else
               {
                  $foundException = $TRUE;
                  baselog($LOG_DEBUG, "The console '$line' was permitted.");
               }
            } ## end foreach my $line (<$fdread>...

            if (not close $fdread)
            {
               baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
               $retval = $FALSE;
            }
            else
            {

               #Change the file /etc/securetty
               if (((not $foundTTYS) or (not $foundTTY1) or ($foundException)) and (write_harden_file($securetty, $console . "\n" . $TTY1 . "\n")))
               {
                  baselog($LOG_INFO, "$DODitem: Root account is configured to have limited access.");
               }
               elsif (($foundTTYS) and ($foundTTY1) and (not $foundException))
               {
                  baselog($LOG_INFO, "Root account is already configured to have limited access.");
               }
               else
               {
                  $retval = $FALSE;
               }
            } ## end else [ if (not close $fdread)
         } ## end else [ if (not open($fdread, ...
      } ## end if ($retval)
   } ## end else [ if (not open($fdread, ...

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");
   }

   return $retval;
} ## end sub apply()

1;
__END__
