###############################################################################
## @file 00OS_harden_services_file.pm
# @brief This module hardens /etc/services file.
# @details This module is part of LinuxBase OS hardening and hardens
#          /etc/services file.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class 00OS_harden_services_file
package hardening::basic::00OS_harden_services_file;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (/etc/services)";

my $etc_services = "/etc/services";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens the file /etc/services
# @details This method checks if the file /etc/services has the attribute
#          IMMUTABLE set, if it is not, the attribute is set to the file.
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @out = ();
   if (not do_system([$LSATTR, $etc_services], {$dsOUT=>\@out}) or (scalar(@out) != 1))
   {
      baselog($LOG_ERROR, "Cannot get information about extended attributes of the file '$etc_services'");
      $retval = $FALSE;
   }
   else
   {

      #check attributes and change if it is necessary
      my ($attrs) = split(/\s+/, $out[0]);
      if (substr($attrs, 4, 1) eq "-")
      {
         if (not do_system([$CHATTR, "+i", $etc_services]))
         {
            baselog($LOG_ERROR, "Cannot change extended attributes for the file '$etc_services'");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_INFO, "$DODitem: Extended attributes for the file '$etc_services' are changed.");
         }
      }
      else
      {
         baselog($LOG_INFO, "It seems that extended attributes for the file '$etc_services' are already changed.");
      }
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
