###############################################################################
## @file 00OS_harden_ssh.pm
# @brief This module hardens SSHD
# @details This module is part of LinuxBase OS hardening and hardens
#          SSH daemon in order to disallow root login.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 00OS_harden_ssh
package hardening::basic::00OS_harden_ssh;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;

# Declarations
## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (SSH)";

my $sshd_config = "/etc/ssh/sshd_config";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens the SSH daemon
# @details This method checks /etc/ssh/sshd_config file if root login by SSH is
#          permitted and changes the file in order to forbid the access. Even
#          if root login is not forbidden explicitely in the file, the method
#          do this as well.
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $fdread;
   if (not open($fdread, "<", $sshd_config))
   {
      baselog($LOG_ERROR, "Cannot read '$sshd_config': $!");
      $retval = $FALSE;
   }
   else
   { 
      #Check if it is already configured
      my $found = $FALSE;
      
      my $content = "";
      
      #copy content from original file
      foreach my $line (<$fdread>)
      {
         #comment out the lines
         if ($line !~ /^\s*#/)
         {
            if (($line =~ /^\s*PermitRootLogin\s*no\s*$/))
            {
               #If it is already configured
               $found = $TRUE;
            }
            elsif (($line =~ /^\s*PermitRootLogin\s*yes\s*$/))
            {
               #If permit is configured
               baselog($LOG_WARNING, "Found SSH access for root permitted.");
               $line = "";
            }
         }
         $content .= $line;
      }

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         $retval = $FALSE;
      }
      else
      {
         if (not $found)
         {
            $content .= "\n#Forbid SSH access for root explicitly\nPermitRootLogin no\n";
         }

         if ((not $found) and (write_harden_file($sshd_config, $content)))
         {
            baselog($LOG_INFO, "$DODitem: the file '$sshd_config' changed successfully.");
         }
         elsif ($found)
         {
            baselog($LOG_INFO, "SSH access for root is already hardened.");
         }
         else
         {
            $retval = $FALSE;
         }
      }
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
