###############################################################################
## @file 00OS_harden_suid.pm
# @brief This module hardens SUID/SGID commands
# @details This module is part of LinuxBase OS hardening and clears SUID/SGID
#          bits from the files.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 00OS_harden_suid
package hardening::basic::00OS_harden_suid;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (SUID/SGID)";

my @suid_command = ($PASSWD, $SUDO, $SU, $USERHELPER, $PING, $PING6, $RLOGIN);
my $suid_list_file = "/root/suid.lst";
my $sgid_list_file = "/root/sgid.lst";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Hardens SUID/SGID commands
# @details This method looks for SUID/SGID commands, checks if the command is
#          in the list of allowed SUID/SGID commands and removes the SUID/SGID
#          bits if it is not. The list of all the found SUID commands is append
#          to /root/suid.lst file and the list of all the found SGID commands
#          is append to /root/sgid.lst.
#
#          Since /usr/sbin/sudo and /usr/sbin/sudoedit commands have the same
#          inode number in RHEL5.1, the inode of /usr/sbin/sudo is changed in
#          order to prevent SUID bit clearing during /usr/sbin/sudoedit
#          treatment.
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #special handler for RHEL5.1
   #seperation hard link between sudo and sudoedit must be done before removal
   #of suid bit from sudo in next step
   #otherwise the suid bit will be stripped from sudo
   if (-f $SUDO)
   {
      if (-f $SUDOEDIT)
      {
         #Get inodes of the files
         my (undef, $sudo_ino) = stat($SUDO);
         my (undef, $sudoedit_ino) = stat($SUDOEDIT);
         if ($sudo_ino == $sudoedit_ino)
         {
            if (do_system([$CP, "-p", $SUDO, $SUDO . ".bak"]) and 
                do_system([$RM, "-f", $SUDO]) and 
                do_system([$MV, "-f", $SUDO . ".bak", $SUDO]))
            {
               baselog($LOG_DEBUG, "'$SUDO' and '$SUDOEDIT' are unbound.");
            }
            else
            {
               baselog($LOG_ERROR, "Cannot unbind '$SUDO' and '$SUDOEDIT'.");
            }
         }
         else
         {
            baselog($LOG_DEBUG, "'$SUDO' and '$SUDOEDIT' are not tied.");
         }
      }
      else
      {
         baselog($LOG_NOTICE, "There is no '$SUDOEDIT'");
      }
   }
   else
   {
      baselog($LOG_ERROR, "Cannot found '$SUDO'");
   }

   my $found = $FALSE;
   
   my @suid_found = ();
   my @sgid_found = ();

   #Find all suid/sgid files/directories
   my $filename;
   #find all ser
   finddepth(
      sub
      {
         $filename = $File::Find::name;
         #If file but not symlink
         if ((-f $filename) and (not -l $filename))
         {
            #If suid or sgid are set
            if ((-u $filename) or (-g $filename))
            {
               my $allowed = $FALSE;

               baselog($LOG_DEBUG, "Found $filename.");
               
               #Check if the file is allowed to have SUID/SGID
               foreach my $suid_file (@suid_command)
               {
                  if ($filename eq $suid_file)
                  {
                     baselog($LOG_DEBUG, "'$suid_file' is allowed to be SUID/SGID program.");
                     $allowed = $TRUE;
                  }
               }

               if (-u $filename)
               {
                  #if SUID is set  
                  push @suid_found, $filename;
               }
               if (-g $filename)
               {
                  #if SGID is set
                  push @sgid_found, $filename;
               }
               
               #To not change allowed commands
               if (not $allowed)
               {
                  $retval &= remove_permissions((S_ISUID | S_ISGID), [$filename], $DODitem);
                  $found = $TRUE;
               }
            }
         }
      }, "/");

   my $curtime = localtime();
   
   #To log all the changed SUID files to the special file
   if ((scalar @suid_found) > 0)
   {
      my $fd;
      if (not open($fd, ">>", $suid_list_file))
      {
         #The file updates failed is not critical. So, do not change $retval;
         baselog($LOG_ERROR, "Cannot update '$suid_list_file': $!");
      }
      else
      {
         print $fd "<--- $curtime\n";
         foreach my $line (@suid_found)
         {
            print $fd $line . "\n";
         }
         
         if (not close $fd)
         {
            baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         }
      }
   }

   #To log all the changed SGID files to the special file
   if ((scalar @sgid_found) > 0)
   {
      my $fd;
      if (not open($fd, ">>", $sgid_list_file))
      {
         #The file updates failed is not critical. So, do not change $retval;
         baselog($LOG_ERROR, "Cannot update '$sgid_list_file': $!");
      }
      else
      {
         print $fd "<--- $curtime\n";
         foreach my $line (@sgid_found)
         {
            print $fd $line . "\n";
         }
         
         if (not close $fd)
         {
            baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         }
      }
   }

   if (not $found)
   {
      baselog($LOG_INFO, "There is no SUID and SGID programs changed.");
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

1;
__END__
