###############################################################################
## @file 50GEN000020.pm
# @brief This module addresses the US DoD requirement GEN000020
# @details This module addresses the US DoD requirement GEN000020 ("The unix
#          host is bootable in single user mode without a password"). In order
#          to comply this requirement the corresponding line is added to
#          /etc/inittab file.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN000020
package hardening::basic::50GEN000020;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

# Declarations
## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN000020";
my $inittab = "/etc/inittab";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN000020
# @details This method adds the line "~:S:wait:/sbin/sulogin" in /etc/inittab
#          if it is needed in order to close the US DoD requirement GEN000020
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      my $fdread;
      if (not open($fdread, "<", $inittab))
      {
         baselog($LOG_ERROR, "Cannot read '$inittab': $!");
         return $FALSE;
      }

      my $content = "";
      
      #copy content from original file to temporary
      foreach my $line (<$fdread>)
      {
         $content .= $line;
         #put the line after initdefault line
         if (($line !~ /^\s*#/) and ($line =~ /id:\d:initdefault:/))
         {
            $content .= "\n~:S:wait:/sbin/sulogin\n";
         }
      }
      
      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         return $FALSE;      
      }

      if (write_harden_file($inittab, $content))
      {
         baselog($LOG_INFO, "$DODitem: the file '$inittab' changed successfully.");
         $retval = $TRUE;
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformance for GEN000020
# @details This method looks at /etc/inittab and checks if the corresponding
#          line exists in the file.
# @return TRUE - the line exists
# @return FALSE - the line does not exist
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$inittab'");

   my $fd;
   if (not open($fd, "<", $inittab))
   {
      baselog($LOG_ERROR, "Cannot read '$inittab': $!");
      return $H_ERROR;
   }
   my @inittab_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;
   }

   my @res = grep(/~:S:wait:\/sbin\/sulogin/, grep(!/^#/, @inittab_content));
   
   if (@res)
   {
      baselog($LOG_INFO, "Password in single user mode is already enabled.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "Password in single user mode is not enabled.");
   }

   return $retval;
}

1;
__END__
