###############################################################################
## @file 50GEN000920.pm
# @brief This module addresses the US DoD requirement GEN000920
# @details This module addresses the US DoD requirement GEN000920 ("The root
#          account home directory (other than /) is more permissive than 700").
#          In order to comply this requirement the permissions of the directory 
#          /root are changed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN000920
package hardening::basic::50GEN000920;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE}
              );

my $DODitem = "GEN000920";
my $rootdir = "/root";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN000920
# @details This method changes permissions of the directory /root if it is
#          needed in order to close the US DoD requirement GEN000920
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      #S_IRWXU = 0700
      $retval = change_permissions_and_ownership(S_IRWXU, undef, undef, [$rootdir], $DODitem);
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformance for GEN000920
# @details This method looks at the directory /root and checks if it is
#          permissive enough.
# @return TRUE - the directory is permissive enough
# @return FALSE - the directory is more permissive than 700
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$rootdir'");

   if ((-e $rootdir) and (-d $rootdir))
   {
      #handle only mode rather than file type
      my $mode = (stat($rootdir))[2] & 07777;
      
      #(S_IRWXG | S_IRWXO) = 0077
      if (($mode & (S_IRWXG | S_IRWXO)) > 0)
      {
         my $form = sprintf("%04o", $mode);  
         
         baselog($LOG_DEBUG, "The current permissions of '$rootdir' are $form.");
      }
      else
      {
         baselog($LOG_INFO, "The root account home directory '$rootdir' permssions are set as per $DODitem specifications.");
         $retval = $TRUE;
      }
   }
   else
   {
      baselog($LOG_ERROR, "The root account home directory does not exist.");
      $retval = $H_ERROR;
   }

   return $retval;
}

1;
__END__
