###############################################################################
## @file 50GEN001220.pm
# @brief This module addresses the US DoD requirements GEN001220 and GEN001240
# @details This module addresses the US DoD requirements GEN001220 ("System
#          files, programs, and directories are not owned by a system account")
#          and GEN001240 ("The group owner of system files, programs, or
#          directories is not a system group"). In order to comply these
#          requirements, permissions and owner of applictions configuration
#          files configure.xml are changed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class 50GEN001220
package hardening::basic::50GEN001220;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},    
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}              );

my $DODitem = "GEN001220 and GEN001240";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN001220 and GEN001240
# @details This method checks if there are configure.xml files with incorrect
#          permissions and ownership and changes these attributes to correct
#          ones.
# @return TRUE - the requirements is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   my $issue_exist = check(\@list);
   if ($issue_exist == $FALSE)
   {
      #Change the permissions and owner
      #S_IRUSR | S_IWUSR | S_IRGRP = 0640
      $retval = change_permissions_and_ownership(S_IRUSR | S_IWUSR | S_IRGRP, $ROOT_UID, $NORTEL_UID, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");
   }   

   return $retval;
}

###############################################################################
## @fn sub check($list)
# @brief Check existence of non-conformance for GEN001220 and GEN001240
# @details This method looks for configure.xml file in /etc/opt/nortel/
#          directory by using the subroutine File::Find::finddepth. If a file
#          is more permissive than 640 and his onwers are not root:nortel, it
#          is added to the array $list.
# @param $list - reference to the list of configure.xml files
# @return TRUE - all files are good enough
# @return FALSE - permissions of files in $list are not OK
# @return H_ERROR - an error happened
sub check($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking configure.xml files...");

   if ((not -e $APP_CFG_ROOT) or (not -d $APP_CFG_ROOT))
   {
      baselog($LOG_ERROR, "The directory '$APP_CFG_ROOT' does not exists.");
      return $H_ERROR;
   }

   @{$list} = ();

   #find all configure.xml files
   finddepth(
      sub
      {
         #$_ contains the file name rather than fullpath
         if ($_ =~ /^$APPLCFG$/)
         {
            my (undef, undef, $mode, undef, $uid, $gid) = stat($_);
            #handle only mode rather than file type
            $mode = $mode & 07777;

            #uid and gid should be "root:nortel", permissions should be 0640
            if ((($mode & (S_IXUSR | S_IWGRP | S_IXGRP | S_IRWXO)) > 0) or ($uid != $ROOT_UID) or ($gid != $NORTEL_UID))
            {
               my $form = sprintf("%04o", $mode);

               baselog($LOG_DEBUG, "The current permissions of '$File::Find::name' are $form, owners are '$uid:$gid'.");
               push @{$list}, $File::Find::name;
            }
            else
            {
               baselog($LOG_INFO, "The current permissions of '$File::Find::name' are good enough.");
            }
         }
      }, $APP_CFG_ROOT);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of files in the directory $APP_CFG_ROOT are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

1;
__END__
