###############################################################################
## @file 50GEN001260.pm
# @brief This module addresses the US DoD requirements GEN001260
# @details This module addresses the US DoD requirements GEN001260 ("System log
#          file permissions are more permissive than 640."). In order to
#          comply this requirement permissions of log files in /var/log changed.
#          Special action are added into /etc/rc.d/rc.sysinit and
#          /etc/logrotate.conf files in order to prevent permissions changing
#          for /var/log/wtmp.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - $log_perms variable was changed.

## @class 50GEN001260
package hardening::basic::50GEN001260;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.02';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check_log_files($);
## @skipline
sub check_rc_sysinit($);
## @skipline
sub check_logrotate($);
## @skipline
sub check_sa_file($$);
my %actions = (
               $H_IF_APPLY => {
                               'sub'     => \&apply,
                               'visible' => $FALSE
                              },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_APPINSTALL => {
                                         'sub'     => \&apply,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_PATCHING => {
                                       'sub'     => \&apply,
                                       'visible' => $FALSE
                                      }
              );

my $DODitem        = "GEN001260";
my $logdir         = "/var/log";
my $log_perms      = (S_IRUSR | S_IWUSR | S_IRGRP);
my $rc_sysinit     = "/etc/rc.d/rc.sysinit";
my $logrotate_conf = "/etc/logrotate.conf";
my @sa_filenames   = ("/usr/lib/sa/sa1", "/usr/lib/sa/sa2");
###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN001260
# @details This method checks if there are log files with incorrect permissions
#          and changes these attributes to correct ones. The files
#          /etc/rc.d/rc.sysinit and /etc/logrotate.conf are changed in order
#          to prevent permissions changing for /var/log/wtmp.
# @return TRUE - the requirements is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   #Check permissions of log files and correct them
   my $issue_exist = check_log_files(\@list);
   if ($issue_exist == $FALSE)
   {

      #Change the permissions
      $retval = change_permissions_and_ownership($log_perms, undef, undef, \@list, $DODitem);

   } ## end if ($issue_exist == $FALSE...
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   my $content = "";
   $issue_exist = check_rc_sysinit(\$content);
   if ($issue_exist == $FALSE)
   {
      if (write_harden_file($rc_sysinit, $content))
      {
         baselog($LOG_INFO, "$DODitem: the file '$rc_sysinit' changed successfully.");
         $retval &= $TRUE;
      }
   } ## end if ($issue_exist == $FALSE...
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   elsif ($issue_exist == $H_ERROR)
   {
      $retval = $FALSE;
   }
   foreach my $myfile (@sa_filenames)
   {
      my $content = "";
      $issue_exist = check_sa_file(\$content, $myfile);
      if ($issue_exist == $FALSE)
      {
         if (write_harden_file($myfile, $content))
         {
            baselog($LOG_INFO, "$DODitem: the file '$myfile' changed successfully.");
            $retval = $TRUE;
         }
      } ## end if ($issue_exist == $FALSE...

      elsif ($issue_exist == $TRUE)
      {
         $retval &= $TRUE;
      }
      elsif ($issue_exist == $H_ERROR)
      {
         $retval = $FALSE;
      }
   } ## end foreach my $myfile (@sa_filenames...
   $content     = "";
   $issue_exist = check_logrotate(\$content);
   if ($issue_exist == $FALSE)
   {
      if (write_harden_file($logrotate_conf, $content))
      {
         baselog($LOG_INFO, "$DODitem: the file '$logrotate_conf' changed successfully.");
         $retval &= $TRUE;
      }
   } ## end if ($issue_exist == $FALSE...
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   elsif ($issue_exist == $H_ERROR)
   {
      $retval = $FALSE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");
   }

   return $retval;
} ## end sub apply()

###############################################################################
## @fn sub check_log_files($list)
# @brief Check existence of non-conformance for system log files
# @details This method looks for system log files in the directory /var/log/
#          except logs from the directory /var/log/nortel by using the
#          subroutine File::Find::finddepth. If a log file is more permissive
#          than 644, it is added to the array $list.
# @param $list - reference to the list of log files
# @return TRUE - all log files are good enough
# @return FALSE - permissions of logfiles in $list are not OK
# @return H_ERROR - some error happens
sub check_log_files($)
{
   my $list = shift @_;

   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Checking system log files...");

   if ((not -e $logdir) or (not -d $logdir))
   {
      baselog($LOG_ERROR, "The directory '$logdir' does not exists.");
      return $H_ERROR;
   }

   @{$list} = ();

   my $filename;

   #find all log files
   finddepth(
      sub {

         #get name of the current file
         $filename = $File::Find::name;

         #check only files
         if ((-f $filename) and ($filename !~ /^\/var\/log\/nortel/))
         {
            if (not permissions_less_or_equal($filename, $log_perms))
            {
               push @{$list}, $File::Find::name;
            }
         } ## end if ((-f $filename) and...
      },
      $logdir
            );

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of log files are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
} ## end sub check_log_files($)

###############################################################################
## @fn sub check_rc_sysinit($content)
# @brief Check if /etc/rc.d/rc.sysinit is configured to set unsecure permissons
# @details This method inspects the file /etc/rc.d/rc.sysinit and checks if it
#          sets unsecure permissions for /var/log/wtmp. If it is so, new content
#          of the script is prepared.
# @param $content - reference to the content of the file /etc/rc.d/rc.sysinit
# @return TRUE - /etc/rc.d/rc.sysinit does not set unsecure permissions
# @return FALSE - /etc/rc.d/rc.sysinit is configured to set unsecure permissions
# @return H_ERROR - some error happens
sub check_rc_sysinit($)
{
   my $content = shift @_;

   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Checking '$rc_sysinit' file...");

   if (not -f $rc_sysinit)
   {
      baselog($LOG_ERROR, "The file '$rc_sysinit' does not exists.");
      return $H_ERROR;
   }

   ${$content} = "";

   my $fdread;
   if (not open($fdread, "<", $rc_sysinit))
   {
      baselog($LOG_ERROR, "Cannot read '$rc_sysinit': $!");
      $retval = $H_ERROR;
   }
   else
   {
      my $found = $FALSE;

      #read content from original file
      foreach my $line (<$fdread>)
      {

         #skip comment out the lines
         if ($line !~ /^\s*#/)
         {

            #rc.sysinit sets 664 permissions for /var/log/wtmp explicitly
            if (($line =~ /chmod 0664 \/var\/run\/utmp \/var\/log\/wtmp/))
            {
               baselog($LOG_DEBUG, "The permissions of /var/log/wtmp are changed in 0664 by $rc_sysinit.");
               $line  = "chmod 0664 /var/run/utmp\nchmod 0640 /var/log/wtmp\n";
               $found = $TRUE;
            } ## end if (($line =~ /chmod 0664 \/var\/run\/utmp \/var\/log\/wtmp/...
         } ## end if ($line !~ /^\s*#/)
         ${$content} .= $line;
      } ## end foreach my $line (<$fdread>...

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         $retval = $H_ERROR;
      }
      else
      {
         if (not $found)
         {
            baselog($LOG_INFO, "'$rc_sysinit' manages log files permissions properly.");
            $retval = $TRUE;
         }
      } ## end else [ if (not close $fdread)
   } ## end else [ if (not open($fdread, ...

   return $retval;
} ## end sub check_rc_sysinit($)

###############################################################################
## @fn sub check_logrotate($content)
# @brief Check if /etc/logrotate.conf is configured to set unsecure permissons
# @details This method inspects the file /etc/logrotate.conf and checks if it
#          sets unsecure permissions for /var/log/wtmp. If it is so, new content
#          of the configuration file is prepared.
# @param $content - reference to the content of the file /etc/logrotate.conf
# @return TRUE - /etc/logrotate.conf does not set unsecure permissions
# @return FALSE - /etc/logrotate.conf is configured to set unsecure permissions
# @return H_ERROR - some error happens
sub check_logrotate($)
{
   my $content = shift @_;

   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Checking '$logrotate_conf' file...");

   if (not -f $logrotate_conf)
   {
      baselog($LOG_ERROR, "The file '$logrotate_conf' does not exists.");
      return $H_ERROR;
   }

   ${$content} = "";

   my $fdread;
   if (not open($fdread, "<", $logrotate_conf))
   {
      baselog($LOG_ERROR, "Cannot read '$logrotate_conf': $!");
      $retval = $H_ERROR;
   }
   else
   {
      my $found = $FALSE;

      #read content from original file
      foreach my $line (<$fdread>)
      {

         #skip comment out the lines
         if ($line !~ /^\s*#/)
         {

            #logrotate.conf sets 664 permissions for /var/log/wtmp explicitly
            if (($line =~ /^\s*create 0664 root utmp\s*$/))
            {
               baselog($LOG_DEBUG, "The permissions of /var/log/wtmp are changed in 0664 by $logrotate_conf.");
               $line =~ s/0664/0640/;
               $found = $TRUE;
            } ## end if (($line =~ /^\s*create 0664 root utmp\s*$/...
         } ## end if ($line !~ /^\s*#/)
         ${$content} .= $line;
      } ## end foreach my $line (<$fdread>...

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         $retval = $H_ERROR;
      }
      else
      {
         if (not $found)
         {
            baselog($LOG_INFO, "'$logrotate_conf' manages log files permissions properly.");
            $retval = $TRUE;
         }
      } ## end else [ if (not close $fdread)
   } ## end else [ if (not open($fdread, ...

   return $retval;
} ## end sub check_logrotate($)

###############################################################################
## @fn check_sa_file($content, $myfile)
# @brief Check if umask in /usr/lib/sa/ files is configured to set unsecure permissons
# @details This method inspects files from /usr/lib/sa directory and checks if they
#          have unsecure umask for /var/log/sa/. If it is so, new content
#          of the configuration file is prepared.
# @param $content - reference to the content of the file /usr/lib/sa/sa
#        $myfile - name of file in /usr/lib/sa directory
# @return TRUE - files in /usr/lib/sa/ does not set unsecure permissions
# @return FALSE - files in /usr/lib/sa/ are configured to set unsecure permissions
# @return H_ERROR - some error happens
sub check_sa_file($$)
{
   my ($content, $myfile) = @_;

   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Checking '$myfile' file...");

   if (not -f $myfile)
   {
      baselog($LOG_INFO, "The file '$myfile' does not exists.");
      return $TRUE;
   }

   ${$content} = "";

   my $fdread;
   if (not open($fdread, "<", $myfile))
   {
      baselog($LOG_ERROR, "Cannot read '$myfile': $!");
      $retval = $H_ERROR;
   }
   else
   {
      my $found = $FALSE;

      #read content from original file
      foreach my $line (<$fdread>)
      {

         #skip comment out the lines
         if ($line !~ /^\s*#/)
         {
            if (($line =~ /umask\s*\d*$/))
            {
               if (($line =~ /umask 0027$/))
               {
                  $found = $FALSE;
               }
               else
               {
                  baselog($LOG_DEBUG, "The umask of /var/log/sa are changed to 0027 by $myfile.");
                  $line =~ s/umask\s*\d*/umask 0027/;
                  $found = $TRUE;
               } ## end else [ if (($line =~ /umask 0027$/...
            } ## end if (($line =~ /umask/s...
         } ## end if ($line !~ /^\s*#/)
         ${$content} .= $line;
      } ## end foreach my $line (<$fdread>...

      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         $retval = $H_ERROR;
      }
      else
      {
         if (not $found)
         {
            baselog($LOG_INFO, "'$myfile' manages log files permissions properly.");
            $retval = $TRUE;
         }
      } ## end else [ if (not close $fdread)
   } ## end else [ if (not open($fdread, ...

   return $retval;
} ## end sub check_sa_file($$)

1;
__END__
