###############################################################################
## @file 50GEN001280.pm
# @brief This module addresses the US DoD requirements GEN001280
# @details This module addresses the US DoD requirements GEN001280 ("Manual
#          page file permissions are more permissive than 644"). In order to
#          comply this requirement permissions of man pages and all files in
#          /usr/share/doc are changed
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN001280
package hardening::basic::50GEN001280;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check_man_pages($);
## @skipline
sub check_doc_files($);
## @skipline
sub check_doc_dirs($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},    
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}              );

my $DODitem = "GEN001280";
my $docsdir = "/usr/share/doc";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN001280
# @details This method checks if there are man pages and documentation
#          files/directories with incorrect permissions and changes these
#          attributes to correct ones.
# @return TRUE - the requirements is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   #Check permissions of man pages and correct them
   my $issue_exist = check_man_pages(\@list);
   if ($issue_exist == $FALSE)
   {
      #Change the permissions and owner
      #S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH = 0644
      $retval = change_permissions_and_ownership(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH, undef, undef, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   #Check permissions of document files and correct them
   @list = ();
   $issue_exist = check_doc_files(\@list);
   if ($issue_exist == $FALSE)
   {
      #Remove the permissions
      #S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH = 0133
      #new permisssion should not be more permissive than 0644
      $retval &= remove_permissions(S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   elsif ($issue_exist == $H_ERROR)
   {
      $retval = $FALSE;
   }
   
   #Check permissions of document directories and correct them   
   @list = ();
   $issue_exist = check_doc_dirs(\@list);
   if ($issue_exist == $FALSE)
   {
      #Remove the permissions
      #S_IWGRP | S_IWOTH = 0022
      #new permisssion should not be more permissive than 0755
      $retval &= remove_permissions(S_IWGRP | S_IWOTH, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   elsif ($issue_exist == $H_ERROR)
   {
      $retval = $FALSE;
   }
   
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");
   }
    
   return $retval;
}

###############################################################################
## @fn sub check_man_pages($list)
# @brief Check existence of non-conformance with man pages
# @details This method looks for man pages in the directories provided by the
#          command 'man --path' by using the subroutine File::Find::finddepth.
#          If a man page is more permissive than 644, it is added to the array
#          $list.
# @param $list - reference to the list of man pages
# @return TRUE - all man pages are good enough
# @return FALSE - permissions of man pages in $list are not OK
# @return H_ERROR - an error happened
sub check_man_pages($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking man pages...");

   my @tmp_arr = ();
   if (not do_system([$MAN, "--path"], {$dsOUT => \@tmp_arr}))
   {
      baselog($LOG_ERROR, "Cannot get list of directories containing man pages.");
      return $H_ERROR;
   }
   
   my @man_paths = ();
   if (scalar(@tmp_arr) == 1)
   {
      my $dirs = $tmp_arr[0];
      chomp $dirs;
      baselog($LOG_DEBUG, "Trying to extract man directories from '$dirs'.");
      @man_paths = split(/:/, $dirs);
   }
   else
   {
      baselog($LOG_ERROR, "Unexpected data are returned by '$MAN --path' (" . scalar(@tmp_arr) . " lines)");
      return $H_ERROR;
   }

   if (scalar(@man_paths) == 0)
   {
      baselog($LOG_NOTICE, "There are no directories containing man pages.");
   }

   #Add Linux Base man pages for checking explicitly
   #It is needed to check the dir on the Linux Base install stage
   push @man_paths, "/opt/nortel/base/man";

   @{$list} = ();

   #find all man pages
   finddepth(
      sub
      {
         #$_ contains the file name rather than fullpath
         #check only files
         if (-f $_)
         {
            my (undef, undef, $mode) = stat($_);
            #handle only mode rather than file type
            $mode = $mode & 07777;

            #permissions should be 0644
            if (($mode & (S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH)) > 0)
            {
               my $form = sprintf("%04o", $mode);

               baselog($LOG_DEBUG, "The current permissions of '$File::Find::name' are $form.");
               push @{$list}, $File::Find::name;
            }
         }
      }, @man_paths);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of man pages are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

###############################################################################
## @fn sub check_doc_files($list)
# @brief Check existence of non-conformance with documentation files
# @details This method looks for files in /use/share/doc by using the
#          subroutine File::Find::finddepth. If a file is more permissive than
#          744, it is added to the array $list.
# @param $list - reference to the list of files
# @return TRUE - all files are good enough
# @return FALSE - permissions of files in $list are not OK
# @return H_ERROR - an error happened
sub check_doc_files($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking files in '$docsdir'...");

   if ((not -e $docsdir) or (not -d $docsdir))
   {
      baselog($LOG_ERROR, "The directory '$docsdir' does not exists.");
      return $H_ERROR;
   }

   @{$list} = ();

   #find all documentation files
   finddepth(
      sub
      {
         my $filename = $File::Find::name;
         
         #the following check only for files
         if ((-f $filename) and (not -l $filename))
         {
            #permissions should be 0644
            if (not permissions_less_or_equal($filename, (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)))
            {
               push @{$list}, $filename;
            }
         }
      }, $docsdir);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of documentation files are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

###############################################################################
## @fn sub check_doc_dirs($list)
# @brief Check existence of non-conformance with documentation directories
# @details This method looks for directories in /use/share/doc by using the
#          subroutine File::Find::finddepth. If a file/directory is more
#          permissive than 755, it is added to the array $list.
# @param $list - reference to the list of files
# @return TRUE - all directories are good enough
# @return FALSE - permissions of directories in $list are not OK
# @return H_ERROR - an error happened
sub check_doc_dirs($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking directories in '$docsdir'...");

   if ((not -e $docsdir) or (not -d $docsdir))
   {
      baselog($LOG_ERROR, "The directory '$docsdir' does not exists.");
      return $H_ERROR;
   }

   @{$list} = ();

   #find all documentation directories
   finddepth(
      sub
      {
         #$_ contains the file name rather than fullpath
         #the following check only for directories
         if (-d $_)
         {
            my (undef, undef, $mode) = stat($_);
            #handle only mode rather than file type
            $mode = $mode & 07777;

            #permissions should be 0755
            if (($mode & (S_IWGRP | S_IWOTH)) > 0)
            {
               my $form = sprintf("%04o", $mode);

               baselog($LOG_DEBUG, "The current permissions of '$File::Find::name' are $form.");
               push @{$list}, $File::Find::name;
            }
         }
      }, $docsdir);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of documentation directories are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

1;
__END__
