###############################################################################
## @file 50GEN001480.pm
# @brief This module addresses the US DoD requirement GEN001480
# @details This module addresses the US DoD requirement GEN001480 ("User home
#          directories have permissions greater than 750"). In order to comply
#          this requirement the permissions of the directory /var/lib/monit are
#          changed and the user's shell is changed to forbid direct login for
#          this user (accounts less than 500 is considered as system non-login
#          accounts).
#
#          There is a SIPLine requirement that an account with UID 100 is
#          reserved for SIPLine purposes. Since monit account has the UID 100
#          which is set during installation of monit RPM package and monit has
#          no strict requirements that its account UID should be 100, UID and
#          GID are changed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN001480
package hardening::basic::50GEN001480;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "GEN001480";
my $monit_account = "monit";
my $no_login_shell = "/sbin/nologin";
my @forbidden_uids = (100);
my $new_monit_uid = 480;

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN001480
# @details This method changes credentials of the monit account (uid and shell)
#          and permissions of the monit home directory if all these are needed
#          in order to close the US DoD requirement GEN001480
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $uid = $FALSE;

   my $issue_exist = check(\$uid);
   if ($issue_exist == $FALSE)
   {
      my (undef, undef, $monit_uid, undef, undef, undef, undef, $monit_dir) = getpwnam($monit_account);
      
      my @comm = ();

      push @comm, $USERMOD;
      
      if ($uid)
      {
         $monit_uid = $new_monit_uid;
         push @comm, "-u", $new_monit_uid;
      }

      push @comm, "-s", $no_login_shell, $monit_account;

      if (not do_system(\@comm))
      {
         baselog($LOG_ERROR, "Cannot update the monit account credentials.");
      }
      else
      {
         baselog($LOG_INFO, "$DODitem: the monit account credentials are updated.");
         
         #(S_IRWXU | S_IRGRP | S_IXGRP) = 0750
         $retval = change_permissions_and_ownership((S_IRWXU | S_IRGRP | S_IXGRP), $monit_uid, undef, [$monit_dir], $DODitem);
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check($uid)
# @brief Check existence of non-conformance for GEN001480
# @details This method looks at the credentials of monit account and checks if
#          - the monit home directory has permissions less or equal 750
#          - the monit UID is not in forbidden list
#          - the monit in non-login account
# @param $uid - reference to a flag which denotes changing of uid. 
# @return TRUE - all requirements are compied
# @return FALSE - some of the requirements are not compled. If the requirement
#                 is about UID, the flag $uid is set.
# @return H_ERROR - some error happens
sub check($)
{
   my ($uid) = @_;

   ${$uid} = $FALSE;
   
   my $retval = $FALSE;

   my (undef, undef, $monit_uid, undef, undef, undef, undef, $monit_dir, $monit_shell) = getpwnam($monit_account);

   baselog($LOG_DEBUG, "Checking monit account...");

   if ((not defined $monit_uid) or (not defined $monit_dir) or (not defined $monit_shell))
   {
      baselog($LOG_ERROR, "Some troubles with getting the monit account credentials.");
      return $H_ERROR;      
   }

   baselog($LOG_DEBUG, "The monit account: UID='$monit_uid', DIR='$monit_dir', SHELL='$monit_shell'");

   baselog($LOG_DEBUG, "Checking monit home directory...");
   
   if ((-e $monit_dir) and (-d $monit_dir))
   {
      #(S_IRWXU | S_IRGRP | S_IXGRP) = 0750
      if (permissions_less_or_equal($monit_dir, (S_IRWXU | S_IRGRP | S_IXGRP)))
      {
         baselog($LOG_INFO, "The monit home directory '$monit_dir' permssions are set as per $DODitem specifications.");
         $retval = $TRUE;
      }
   }
   else
   {
      baselog($LOG_ERROR, "The monit home directory does not exist.");
      $retval = $H_ERROR;
   }

   if ($retval != $H_ERROR)
   {
      baselog($LOG_DEBUG, "Checking monit UID...");

      # FALSE if uid is not in the forbidden list
      my $found = $FALSE;
      
      foreach my $uid (@forbidden_uids)
      {
         if ($monit_uid == $uid)
         {
            $found = $TRUE;
            last;
         }
      }

      # if uid is in the forbidden list
      if ($found)
      {
         ${$uid} = $TRUE;
      }
      else
      {
         baselog($LOG_INFO, "The monit account complies SIPLine requirements.");
         $retval = $TRUE;
      }

      baselog($LOG_DEBUG, "Checking that monit is non login account...");

      if ($monit_shell ne $no_login_shell)
      {
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "The monit account is already configured as a non login account.");
         $retval = $TRUE;
      }
   }

   return $retval;
}

1;
__END__
