###############################################################################
## @file 50GEN002420.pm
# @brief This module addresses the US DoD requirement GEN002420
# @details This module addresses the US DoD requirement GEN002420 ("User file
#          systems, removable media, or remote file systems are not mounted
#          with the nosuid option invoked"). In order to comply this requirement
#          the corresponding lines are changed in /etc/fstab file.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN002420
package hardening::basic::50GEN002420;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN002420";
my $fstab = "/etc/fstab";

my %valid_fs = ('type' => ["tmpfs", "devpts", "sysfs", "proc", "swap"],
                'mount_point' => ["/", "/usr", "/opt", "/var", "/tmp"]);

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN002420
# @details This method adds the option "nosuid" to filesystems configured in
#          /etc/fstab if it is needed in order to close the US DoD requirement.
#          If this hardening action is done on a run-time system, actual
#          non-conformance closure will happen only after reboot.
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   my $issue_exist = check(\@list);
   if ($issue_exist == $FALSE)
   {
      my $content = "";
      
      #prepare new content for the file
      foreach my $elem (@list)
      {
         my $line = $elem->[1];
         
         #check if the line was marked as invalid
         if (not $elem->[0])
         {
            $line =~ s/(^.*\s*defaults)/$1,nosuid/;
         }

         $content .= $line . "\n";
      }

      #if this hardening action is done on a run-time system, the changes will
      #be applied for all filesystem only after reboot

      if (write_harden_file($fstab, $content))
      {
         baselog($LOG_INFO, "$DODitem: the file '$fstab' changed successfully.");
         $retval = $TRUE;
      }   
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check($list)
# @brief Check existence of non-conformance for GEN002420
# @details This method looks at /etc/fstab and checks if it contain filesystems
#          with not configured nosuid option. Some filesystems like /, /usr,
#          /opt, /var, /tmp are not checked as well as filesystem with type
#          tmpfs, devpts, sysfs, proc and swap.
# @param $list - reference to array where thecontent of /etc/fstab file will be
#                returned.
# @return TRUE - the file contains invalid filesystems
# @return FALSE - all filesystems in the filw are OK
# @return H_ERROR - an error happened
sub check($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;

   @{$list} = ();
   
   baselog($LOG_DEBUG, "Checking '$fstab'");

   my $fd;
   if (not open($fd, "<", $fstab))
   {
      baselog($LOG_ERROR, "Cannot read '$fstab': $!");
      return $H_ERROR;
   }
   my @fstab_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;
   }
   
   my $num = 0;
   my $found = $FALSE;

   foreach my $line (@fstab_content)
   {
      chomp $line;
      
      my $valid = $TRUE;
      
      if ($line !~ /^\s*#/)
      {
         $valid = $FALSE;
         my (undef, $mpoint, $type, $opts) = split(/\s+/, $line);

         #Check if nosuid haas been already added
         if ($opts !~ /nosuid/)
         {
            #Check mount point
            foreach my $elem (@{$valid_fs{"mount_point"}})
            {
               if ($mpoint eq $elem)
               {
                  $valid = $TRUE;
               }
            }
            #Check type if mount point is OK
            if (not $valid)
            {
              foreach my $elem (@{$valid_fs{"type"}})
               {
                  if ($type eq $elem)
                  {
                     $valid = $TRUE;
                  }
               }
            }
         }
         else
         {
            baselog($LOG_DEBUG, "The filesystem in the mount point '$mpoint' has 'nosuid' option.");
            $valid = $TRUE;
         }

         #If the line is not valid, mark flag $found in TRUE
         if (not $valid)
         {
            $found = $TRUE;
            baselog($LOG_DEBUG, "The filesystem in the mount point '$mpoint' is not configured to have 'nosuid' option.");
         }
      }
      
      #Put the line in array together with its validation status
      push @{$list}, [$valid, $line];
   }
   
   if (not $found)
   {
      baselog($LOG_INFO, "All filesystems in the file '$fstab' are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

1;
__END__
