###############################################################################
## @file 50GEN002480.pm
# @brief This module addresses the US DoD requirement GEN002480
# @details This module addresses the US DoD requirement GEN002480 ("There are
#          world writeable files or directories that are not public
#          directories"). In order to comply this requirement SELinux is
#          disabled.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class 50GEN002480
package hardening::basic::50GEN002480;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use File::Basename;
use base_system;

# Declarations
## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN002480";
my $selinux_config = "/etc/selinux/config";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Performs the actions to close GEN002480
# @details This method removes disable SELinux by adding the line
#          "SELINUX=disabled" in /etc/selinux/config if it is needed in order
#          to close the US DoD requirements GEN002480
# @return TRUE - the requirements are closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #Check /etc/selinux/config file
   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      my $error = $FALSE;
      
      #Create the directory if it is necessary
      my $selinux_dir = dirname($selinux_config);
      if (not -e $selinux_dir)
      {
         baselog($LOG_DEBUG, "The directory '$selinux_dir' does not exist.");
         if (not do_system([$MKDIR, $selinux_dir]))
         {
            baselog($LOG_ERROR, "Cannot create the directory '$selinux_dir'.");
            $error = $TRUE;
         }
      }

      if (not $error)
      {
         my $content = "";
         
         #Not to try read file if it does not exist
         if (not -f $selinux_config)
         {
            baselog($LOG_DEBUG, "The file '$selinux_config' does not exist.");
         }
         else
         {
            my $fd;
            if (not open($fd, "<", $selinux_config))
            {
               baselog($LOG_ERROR, "Cannot read '$selinux_config': $!");
               $error = $TRUE;
            }

            #copy content from original file
            foreach my $line (<$fd>)
            {
               #comment lines are not checked
               if ($line !~ /^\s*#/)
               {
                  #If SELinux is enabled explicitely - remove such line
                  if (($line =~ /^\s*SELINUX=/) and ($line !~ /[^#]*disabled/))
                  {
                     baselog($LOG_WARNING, "SELinux is enabled.");
                     $line = "";
                  }
               }
               $content .= $line;
            }

            if (not close $fd)
            {
               baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
               $error = $TRUE;
            }
         }
      
         if (not $error)
         {
            $content .= "\n#SELinux is disabled in CS1000 Linux Base\nSELINUX=disabled\n";
            
            if (write_harden_file($selinux_config, $content))
            {
               baselog($LOG_INFO, "$DODitem: the file '$selinux_config' changed successfully.");
               if (change_permissions_and_ownership(0600, $ROOT_UID, $ROOT_GID, [$selinux_config], $DODitem))
               {
                  $retval = $TRUE;
               }
            }
         }
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Checks existence of non-conformance for GEN002480.
# @details This method inspects /etc/selinux/config and checks if SELinux
#          policy is disabled in this file.
# @return TRUE - SELinux is disabled
# @return FALSE - SELinux is not disabled
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$selinux_config'");

   if (not -f $selinux_config)
   {
      baselog($LOG_DEBUG, "The file '$selinux_config' does not exist.");
      return $retval;
   }

   my $fd;
   if (not open($fd, "<", $selinux_config))
   {
      baselog($LOG_ERROR, "Cannot read '$selinux_config': $!");
      return $H_ERROR;
   }
   my @selinux_config_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;   
   }

   my @res = grep(/SELINUX=disabled/, grep(!/^\s*#/, @selinux_config_content));
   
   if (@res)
   {
      baselog($LOG_INFO, "SELinux is already disabled.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "SELinux is not disabled.");
   }

   return $retval;
}

1;
__END__
