###############################################################################
## @file 50GEN003080.pm
# @brief This module addresses the US DoD requirements GEN003080
# @details This module addresses the US DoD requirements GEN003080 ("Crontab
#          files are more permissive than 600. 700 on some linux systems."). In
#          order to comply this requirement permissions of cron tables and cron
#          jobs are changed
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class 50GEN003080
package hardening::basic::50GEN003080;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use File::Find;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check_crontabs($);
## @skipline
sub check_cronjobs($);
## @skipline
sub check_crondirs($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},    
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}              );

my $DODitem = "GEN003080";
my $crontabs_dir = "/etc/cron.d";
my $crontab = "/etc/crontab";
my $cron_spool = "/var/spool/cron";

my @crontabs_paths = ($crontabs_dir, $cron_spool, $crontab);
my @cronjobs = ("/etc/cron.hourly", "/etc/cron.daily", "/etc/cron.weekly", "/etc/cron.monthly");

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN003080
# @details This method checks if there are cron tables and cron jobs with
#          incorrect permissions and changes these attributes to correct ones.
# @return TRUE - the requirements is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   #Check permissions of cron tables and correct them
   my $issue_exist = check_crontabs(\@list);
   if ($issue_exist == $FALSE)
   {
      #Change the permissions
      #S_IRUSR | S_IWUSR = 0600
      $retval = change_permissions_and_ownership(S_IRUSR | S_IWUSR, undef, undef, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   #Check permissions of cron jobs and correct them
   @list = ();
   $issue_exist = check_cronjobs(\@list);
   if ($issue_exist == $FALSE)
   {
      #Change the permissions
      #S_IRWXU = 0700
      $retval &= change_permissions_and_ownership(S_IRWXU, undef, undef, \@list, $DODitem);

   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   
   #Check permissions of cron directories and correct them   
   @list = ();
   $issue_exist = check_crondirs(\@list);
   if ($issue_exist == $FALSE)
   {
      #Change the permissions
      #S_IRWXU = 0700
      $retval &= change_permissions_and_ownership(S_IRWXU, undef, undef, \@list, $DODitem);
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");
   }
    
   return $retval;
}

###############################################################################
## @fn sub check_crontabs($list)
# @brief Check existence of non-conformance with tables for driving cron
# @details This method looks for tables for driving cron (/etc/crontab and in
#          the directories /etc/cron.d/ and /var/spool/cron) by using the
#          subroutine File::Find::finddepth. If a cron table is more permissive
#          than 600, it is added to the array $list.
# @param $list - reference to the list of cron tables
# @return TRUE - all cron tables are good enough
# @return FALSE - permissions of cron tables in $list are not OK
# @return H_ERROR - an error happened
sub check_crontabs($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking tables for driving cron...");

   @{$list} = ();

   #find all tables for driving cron
   finddepth(
      sub
      {
         #$_ contains the file name rather than fullpath
         #check only files
         if (-f $_)
         {
            my (undef, undef, $mode) = stat($_);
            #handle only mode rather than file type
            $mode = $mode & 07777;

            #permissions should be 0600
            #S_IXUSR | S_IRWXG | S_IRWXO = 177
            if (($mode & (S_IXUSR | S_IRWXG | S_IRWXO)) > 0)
            {
               my $form = sprintf("%04o", $mode);

               baselog($LOG_DEBUG, "The current permissions of '$File::Find::name' are $form.");
               push @{$list}, $File::Find::name;
            }
         }
      }, @crontabs_paths);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of cron tables are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

###############################################################################
## @fn sub check_cronjobs($list)
# @brief Check existence of non-conformance with cron jobs
# @details This method looks for cron jobs in /etc/cron.hourly, /etc/cron.daily,
#          /etc/cron.weekly and /etc/cron.monthly by using the subroutine
#          File::Find::finddepth. If a cron job is more permissive than 700, it
#          is added to the array $list.
# @param $list - reference to the list of files
# @return TRUE - all cron jobs are good enough
# @return FALSE - permissions of cron jobs in $list are not OK
# @return H_ERROR - an error happened
sub check_cronjobs($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking cron jobs in '" . join(", ", @cronjobs). "'...");

   @{$list} = ();

   #find all cron jobs
   finddepth(
      sub
      {
         #$_ contains the file name rather than fullpath
         #the following check only for files
         if (-f $_)
         {
            my (undef, undef, $mode) = stat($_);
            #handle only mode rather than file type
            $mode = $mode & 07777;

            #permissions should be 0700
            #S_IRWXG | S_IRWX = 077
            if (($mode & (S_IRWXG | S_IRWXO)) > 0)
            {
               my $form = sprintf("%04o", $mode);

               baselog($LOG_DEBUG, "The current permissions of '$File::Find::name' are $form.");
               push @{$list}, $File::Find::name;
            }
         }
      }, @cronjobs);

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of cron jobs are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

###############################################################################
## @fn sub check_crondirs($list)
# @brief Check existence of non-conformance with cron directories
# @details This method checks the directories /etc/cron.d, /etc/cron.hourly,
#          /etc/cron.daily, /etc/cron.weekly and /etc/cron.monthly and if a
#          directory is more permissive than 700, it is added to the array $list.
# @param $list - reference to the list of directories
# @return TRUE - all cron directories are good enough
# @return FALSE - permissions of directories in $list are not OK
# @return H_ERROR - an error happened
sub check_crondirs($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking directories with cron files...");

   #inspect all all cron directories except "spool"...
   my @crondirs = (@cronjobs, $crontabs_dir);

   @{$list} = ();

   foreach my $dir (@crondirs)
   {
      #$_ contains the file name rather than fullpath
      #the following check only for files
      if (-d $dir)
      {
         my (undef, undef, $mode) = stat($dir);
         #handle only mode rather than file type
         $mode = $mode & 07777;

         #permissions should be 0700
         #S_IRWXG | S_IRWX = 077
         if (($mode & (S_IRWXG | S_IRWXO)) > 0)
         {
            my $form = sprintf("%04o", $mode);

            baselog($LOG_DEBUG, "The current permissions of '$dir' are $form.");
            push @{$list}, $dir;
         }
      }
      else
      {
         baselog($LOG_NOTICE, "The directory '$dir' is not found.");
      }
   }

   if (not @{$list})
   {
      baselog($LOG_INFO, "Permissions of cron directories are set as per $DODitem specifications.");
      $retval = $TRUE;
   }

   return $retval;
}

1;
__END__
