###############################################################################
## @file 50GEN003300.pm
# @brief This module addresses the US DoD requirements GEN003300, GEN003320 and
#        GEN003340
# @details This module addresses the US DoD requirement GEN003300 ("The at.deny
#          file exists and is empty"),GEN003320 ("Default accounts are
#          listed in the at.allow file") and GEN003340 ("The at.allow or
#          at.deny file(s) is more permissive than 600."). In order to comply
#          these requirements the account 'root' is added to /etc/at.allow file,
#          its permissions are changed and /etc/at.deny is removed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN003300
package hardening::basic::50GEN003300;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check_allow();
## @skipline
sub check_deny();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN003300, GEN003320 and GEN003340";
my $at_allow = "/etc/at.allow";
my $at_deny = "/etc/at.deny";

my @allowed_users = ($ROOT_ACCT);

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN003300, GEN003320 and GEN003340
# @details This method removes /etc/at.deny, puts 'root' to /etc/at.allow and
#          changes the permissions of this file if all these are needed in
#          order to close the US DoD requirements GEN003300, GEN003320 and
#          GEN003340
# @return TRUE - the requirements are closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #Check /etc/at.allow file
   my $issue_exist = check_allow();
   if ($issue_exist == $FALSE)
   {
      my $allowed_users_string = join "\n", @allowed_users;
      
      if (write_harden_file($at_allow, $allowed_users_string, $NO_BACKUP))
      {
         baselog($LOG_INFO, "$DODitem: the file '$at_allow' changed successfully.");
         if (change_permissions_and_ownership(0600, $ROOT_UID, $ROOT_GID, [$at_allow], $DODitem))
         {
            $retval = $TRUE;
         }
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   #Check /etc/at.deny file
   $issue_exist = check_deny();
   if ($issue_exist == $FALSE)
   {
      if (do_system([$RM, "-f", $at_deny]))
      {
         baselog($LOG_INFO, "$DODitem: the file '$at_deny' removed.");
         $retval &= $TRUE;
      }
      else
      {
         $retval = $FALSE;
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }
   
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check_allow()
# @brief Check existence of non-conformances for GEN003320 and GEN003340.
# @details This method 
#          -# looks at /etc/at.allow and checks if only root is in the file;
#          -# check if the file is readable and writable only for root.
# @return TRUE - the file is OK
# @return FALSE - the file does not comply the requirements
# @return H_ERROR - an error happened
sub check_allow()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$at_allow'");

   if (not -f $at_allow)
   {
      baselog($LOG_DEBUG, "The file '$at_allow' does not exist.");
      return $retval;
   }

   my $fd;
   if (not open($fd, "<", $at_allow))
   {
      baselog($LOG_ERROR, "Cannot read '$at_allow': $!");
      return $H_ERROR;
   }
   my @at_allow_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;
   }
   
   #the file is not configured by the Hardening
   if (scalar(@at_allow_content) != scalar(@allowed_users))
   {
      baselog($LOG_DEBUG, "It seems that the file '$at_allow' is not configured by the Hardening.");
   }
   else
   {
      #compare lists of accounts which can use at, assuming that the file
      #can be modified only by the Hardening
      my $flag = $TRUE;
      
      for (my $i = 0; $i < scalar(@allowed_users); $i++)
      {
         my $line = $at_allow_content[$i];
         chomp $line;
         baselog($LOG_DEBUG, "Accounts from '$at_allow': '$line'.");

         if ($line ne $allowed_users[$i])
         {
            $flag = $FALSE;
         }
      }
      
      #Check if the lists are equal
      if ($flag)
      {
         $retval = $TRUE;

         #Check permissions of the file
         my (undef, undef, $mode, undef, $uid, $gid) = stat($at_allow);
         #handle only mode rather than file type
         $mode = $mode & 07777;

         #uid and gid should be "root:root", permissions should be 0600
         if ((($mode & (S_IXUSR | S_IRWXG | S_IRWXO)) > 0) or ($uid != $ROOT_UID) or ($gid != $ROOT_UID))
         {
            my $form = sprintf("%04o", $mode);

            baselog($LOG_DEBUG, "The current permissions of '$at_allow' are $form, owners are '$uid:$gid'.");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_INFO, "The content of '$at_allow' and the current permissions are set as per $DODitem specifications.");
         }
      }
   }
   
   return $retval;
}

###############################################################################
## @fn sub check_deny()
# @brief Check existence of non-conformances for GEN003300.
# @details This method check if /etc/at.deny file does not exist
# @return TRUE - the file does not exist
# @return FALSE - the file exists
# @return H_ERROR - an error happened
sub check_deny()
{
   baselog($LOG_DEBUG, "Checking '$at_deny'");

   if (-f $at_deny)
   {
      baselog($LOG_DEBUG, "The file '$at_deny' exists.");
      return $FALSE;
   }

   baselog($LOG_INFO, "The file '$at_deny' does not exists. It complies $DODitem specifications.");
   
   return $TRUE;
}

1;
__END__
