###############################################################################
## @file 50GEN003300.pm
# @brief This module addresses the US DoD requirement GEN003620
# @details This module addresses the US DoD requirement GEN003620 ("Separate
#          file system partitions are not used for /home, /export/home, and
#          /var and is not justified and documented with the IAO."). In order 
#          to comply this requirement /export/home is removed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN003620
package hardening::basic::50GEN003620;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN003620";
my $export_home = "/export/home";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN003620
# @details This method removes /export/home if it is needed in order to close
#          the US DoD requirement GEN003620
# @return TRUE - the requirements are closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #Check /export/home file
   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      if (do_system([$RM, "-rf", $export_home]))
      {
         baselog($LOG_INFO, "$DODitem: the file '$export_home' removed.");
         $retval = $TRUE;
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }
   
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformances for GEN003620.
# @details This method check if /export/home directory does not exist
# @return TRUE - the directory does not exist
# @return FALSE - the directory exists
# @return H_ERROR - an error happened
sub check()
{
   baselog($LOG_DEBUG, "Checking '$export_home'");

   if (-d $export_home)
   {
      baselog($LOG_DEBUG, "The directory '$export_home' exists.");
      return $FALSE;
   }

   baselog($LOG_INFO, "The file '$export_home' does not exists. It is OK.");
   
   return $TRUE;
}

1;
__END__
