###############################################################################
## @file 50GEN003740.pm
# @brief This module addresses the US DoD requirement GEN003740
# @details This module addresses the US DoD requirement GEN003740 ("The
#          inetd.conf file permissions are more permissive than 440. The linux
#          xinetd.d is more permissive than 755"). In order to comply this
#          requirement the permissions of the file /etc/xinetd.conf and the
#          directory /etc/xinetd.d are changed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN003740
package hardening::basic::50GEN003740;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check_conf();
## @skipline
sub check_dir();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}
              );

my $DODitem = "GEN003740";
my $xinetd_conf = "/etc/xinetd.conf";
my $xinetd_dir = "/etc/xinetd.d";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN003740
# @details This method changes permissions of the file /etc/xinetd.conf if it
#          is needed in order to close the US DoD requirement
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   #Check /etc/xinetd.conf file
   my $issue_exist = check_conf();
   if ($issue_exist == $FALSE)
   {
      #Change the permissions and owner
      #S_IRUSR | S_IRGRP = 0440
      $retval = change_permissions_and_ownership(S_IRUSR | S_IRGRP, $ROOT_UID, $ROOT_GID, [$xinetd_conf], $DODitem);
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   #Check /etc/xinetd.d directory
   $issue_exist = check_dir();
   if ($issue_exist == $FALSE)
   {
      #Change the permissions and owner
      #S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH = 0755
      $retval &= change_permissions_and_ownership(S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH, $ROOT_UID, $ROOT_GID, [$xinetd_dir], $DODitem);
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval &= $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check_conf()
# @brief Check existence of non-conformance for GEN003740
# @details This method looks at the file /etc/xinetd.conf and checks if it is
#          permissive enough.
# @return TRUE - the file is permissive enough
# @return FALSE - the file is more permissive than 440
# @return H_ERROR - an error happened
sub check_conf()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$xinetd_conf'");

   if (-f $xinetd_conf)
   {
      #handle only mode rather than file type
      my $mode = (stat($xinetd_conf))[2] & 07777;
      
      #(S_IWUSR | S_IXUSR | S_IWGRP | S_IXGRP | S_IRWXO) = 0337
      if (($mode & (S_IWUSR | S_IXUSR | S_IWGRP | S_IXGRP | S_IRWXO)) > 0)
      {
         my $form = sprintf("%04o", $mode);  
         
         baselog($LOG_DEBUG, "The current permissions of '$xinetd_conf' are $form.");
      }
      else
      {
         baselog($LOG_INFO, "The file '$xinetd_conf' permssions are set as per $DODitem specifications.");
         $retval = $TRUE;
      }
   }
   else
   {
      baselog($LOG_ERROR, "The file '$xinetd_conf' does not exist.");
      $retval = $H_ERROR;
   }

   return $retval;
}

###############################################################################
## @fn sub check_conf()
# @brief Check existence of non-conformance for GEN003740
# @details This method looks at the directory /etc/xinetd.d and checks if it is
#          permissive enough.
# @return TRUE - the file is permissive enough
# @return FALSE - the file is more permissive than 755
# @return H_ERROR - an error happened
sub check_dir()
{
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Checking '$xinetd_dir'");

   if ((-e $xinetd_dir) and (-d $xinetd_dir))
   {
      #handle only mode rather than file type
      my $mode = (stat($xinetd_dir))[2] & 07777;

      #(S_IWGRP | S_IWOTH) = 0022
      if (($mode & (S_IWGRP | S_IWOTH)) > 0)
      {
         my $form = sprintf("%04o", $mode);

         baselog($LOG_DEBUG, "The current permissions of '$xinetd_dir' are $form.");
      }
      else
      {
         baselog($LOG_INFO, "The directory '$xinetd_dir' permssions are set as per $DODitem specifications.");
         $retval = $TRUE;
      }
   }
   else
   {
      baselog($LOG_ERROR, "The directory does not exist.");
      $retval = $H_ERROR;
   }

   return $retval;
}

1;
__END__
