###############################################################################
## @file 50GEN004640.pm
# @brief This module addresses the US DoD requirement GEN004640
# @details This module addresses the US DoD requirement GEN004640 ("The
#          sendmail decode command is not disabled."). In order to comply this
#          requirement the alias decode is commented out in the file 
#          /etc/aliases.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class 50GEN004640
package hardening::basic::50GEN004640;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN004640";
my $aliases = "/etc/aliases";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN004640
# @details This method comments out  "decode" alias in /etc/aliases
#          if it is needed in order to close the US DoD requirement.
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      my $fdread;
      if (not open($fdread, "<", $aliases))
      {
         baselog($LOG_ERROR, "Cannot read '$aliases': $!");
         return $FALSE;
      }

      my $content = "";
      
      #read content from original file
      foreach my $line (<$fdread>)
      {
         #comment out decode alias
         if (($line !~ /^\s*#/) and ($line =~ /^(\s*decode\s*:.*)/))
         {
            $line = "#" . $1 . "\n";
         }
         $content .= $line;
      }
      
      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
         return $FALSE;      
      }

      if (write_harden_file($aliases, $content))
      {
         #rebuild the data base for the mail aliases file
         if (do_system([$NEWALIASES]))
         {
            baselog($LOG_INFO, "$DODitem: the alias decode is comment out in '$aliases' successfully.");
            $retval = $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "Cannot re-build the alias data base.");
         }
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformance for GEN004640
# @details This method looks at /etc/aliases and checks if the alias decode 
#          exists in the file.
# @return TRUE - the alias exists
# @return FALSE - the alias does not exist
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$aliases'");

   my $fd;
   if (not open($fd, "<", $aliases))
   {
      baselog($LOG_ERROR, "Cannot read '$aliases': $!");
      return $H_ERROR;
   }
   my @aliases_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;   
   }
   
   #Find all lines with decode (not commented out)
   my @res = grep(/^\s*decode\s*:/, grep(!/^\s*#/, @aliases_content));
   
   if (not @res)
   {
      baselog($LOG_INFO, "The active alias decode is not found in '$aliases'.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "The alias decode is found.");
   }

   return $retval;
}

1;
__END__
