###############################################################################
## @file 50GEN006600.pm
# @brief This module addresses the US DoD requirement GEN006600
# @details This module addresses the US DoD requirement GEN006600 ("The access
#          control program does not log each system access attempt"). In order
#          to comply this requirement the logging option is added to
#          /etc/sysconfig/xinetd file.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50GEN006600
package hardening::basic::50GEN006600;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}    
              );

my $DODitem = "GEN006600";
my $xinetd_config = "/etc/sysconfig/xinetd";
my $xinetd_config_opts_line="EXTRAOPTIONS";
my $xinetd_config_log_opts="-syslog";
my $xinetd_config_log_level="local7";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close GEN006600
# @details This method adds the logging option to xinetd options stored in
#          /etc/sysconfig/xinetd if it is needed in order to close the US DoD
#          requirement.
#          Restart of the xinetd is needed to the configuration becomes
#          effective. But since some telnet/tftp sessions could be active in
#          this time and basic hardening is not allowed (at least it is
#          reasonable) to take down them, this restart lays to a end user
#          shoulders.
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      my $fdread;
      if (not open($fdread, "<", $xinetd_config))
      {
         baselog($LOG_ERROR, "Cannot read '$xinetd_config': $!");
         return $FALSE;
      }
     
      my $logging_set = $FALSE;
      my $content = "";
      
      #read content from original file
      foreach my $line (<$fdread>)
      {
         if (not $logging_set)
         {
            #add logging to EXTRAOPTIONS
            if (($line !~ /^\s*#/) and ($line =~ /(^\s*$xinetd_config_opts_line\s*=\")(.*\".*$)/))
            {
               $line = $1 . "$xinetd_config_log_opts $xinetd_config_log_level " . $2 . "\n";
               $logging_set = $TRUE;
            }
         }
         $content .= $line;
      }
      
      if (not close $fdread)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
      }
      else
      {
         if (write_harden_file($xinetd_config, $content))
         {
            baselog($LOG_INFO, "$DODitem: the file '$xinetd_config' changed successfully.");
            $retval = $TRUE;
         }
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem handled incorrectly.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformance for GEN006600
# @details This method looks for defenitions of the variable EXTRAOPTIONS in
#          /etc/sysconfig/xinetd and checks if logging option for xinetd is
#          enabled in this variable
# @return TRUE - logging option is enabled
# @return FALSE - logging option is not enabled
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$xinetd_config'");

   if (not -f $xinetd_config)
   {
      baselog($LOG_ERROR, "The file '$xinetd_config' does not exist.");
      return $H_ERROR;      
   }

   my $fd;
   if (not open($fd, "<", $xinetd_config))
   {
      baselog($LOG_ERROR, "Cannot read '$xinetd_config': $!");
      return $H_ERROR;
   }
   my @xinetd_config_content = <$fd>;
   
   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
      return $H_ERROR;   
   }

   #Find the line with options
   my @options = grep(/^\s*$xinetd_config_opts_line\s*=/, grep(!/^\s*#/, @xinetd_config_content));
   if (scalar @options)
   {
      #Check if logging is configured (it does not matter for which syslog
      #facility)
      my @res = grep(/^\s*$xinetd_config_opts_line\s*=.*\".*$xinetd_config_log_opts\s.*\"/, @options);
   
      if (@res)
      {
         baselog($LOG_INFO, "Logging is already configured for xinet daemon.");
         $retval = $TRUE;
      }
      else
      {
         baselog($LOG_DEBUG, "Logging is not configured for xinet daemon.");
      }
   }
   else
   {
      baselog($LOG_ERROR, "Unknown format of '$xinetd_config'.");
      $retval = $H_ERROR;      
   }

   return $retval;
}

1;
__END__
