###############################################################################
## @file 50LNX00320.pm
# @brief This module addresses the US DoD requirements LNX00320 and LNX00340
# @details This module addresses the US DoD requirements LNX00320 ("Special
#          privileged accounts such as shutdown and halt have not been
#          deleted") and LNX00340 ("Unnecessary accounts (e.g., games, news)
#          and associated software have not been deleted"). In order to comply
#          this requirement such accounts as halt, shutdown, games, news and
#          gopher are removed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50LNX00320
package hardening::basic::50LNX00320;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check($);

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_APPINSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}
              );

my $DODitem = "LNX00320 and LNX00340";
my @accounts = ("halt", "shutdown", "games", "news", "gopher");

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close LNX00320 and LNX00340
# @details This method looks for the prohibited accounts and removes them in
#             order to close the US DoD requirements
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my @list = ();

   #Check prohibited accounts
   my $issue_exist = check(\@list);
   if ($issue_exist == $FALSE)
   {
      foreach my $acc (@list)
      {
         #Delete account
         if (not do_system([$USERDEL, $acc]))
         {
            baselog($LOG_ERROR, "Cannot remove the account '$acc'.");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_INFO, "The account '$acc' is removed.");
            $retval = $TRUE;
         }
      }
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }
 
   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check($list)
# @brief Check existence of non-conformance for LNX00320 and LNX340
# @details This method looks for existence of the prohibited accounts. If an
#          account exists it is added to the array $list.
# @param $list - reference to the list of the prohibited accounts
# @return TRUE - no prohibited accounts in the system
# @return FALSE - prohibited accounts in $list are not OK
# @return H_ERROR - an error happened
sub check($)
{
   my $list = shift @_;
   
   my $retval = $FALSE;

   @{$list} = ();

   foreach my $acc (@accounts)
   {
      baselog($LOG_DEBUG, "Checking account '$acc'");

      if (defined getpwnam($acc))
      {
         baselog($LOG_DEBUG, "The account '$acc' exists.");
         push @{$list}, $acc;
      }
      else
      {
         baselog($LOG_INFO, "The account '$acc' does not exist.");
      }
   }
 
   if (not @{$list})
   {
      $retval = $TRUE;
   }

   return $retval;
}

1;
__END__
