###############################################################################
## @file 50LNX00440.pm
# @brief This module addresses the US DoD requirement LNX00440
# @details This module addresses the US DoD requirement LNX00440 ("The
#          /etc/login.access or /etc/security/access.conf file is more
#          permissive than 640").  In order to comply this requirement the
#          permissions of the file /etc/security/access.conf are changed.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 50LNX00440
package hardening::basic::50LNX00440;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;
use hardening::hardening_common;
use baselog;
use base_system;
use Fcntl ':mode';

## @skipline
sub actions();
## @skipline
sub apply();
## @skipline
sub check();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                          'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                          'visible' => $FALSE}
              );

my $DODitem = "LNX00440";
my $accessconf = "/etc/security/access.conf";

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Perform the actions to close LNX00440
# @details This method changes permissions of the file
#          /etc/security/access.conf if it is needed in order to close the US
#          DoD requirement
# @return TRUE - the requirement is closed
# @return FALSE - some error happened
sub apply()
{
   my $retval = $FALSE;

   baselog($LOG_INFO, "Handling $DODitem...");

   my $issue_exist = check();
   if ($issue_exist == $FALSE)
   {
      #Change the permissions and owner
      #S_IRUSR | S_IWUSR | S_IRGRP = 0640
      $retval = change_permissions_and_ownership(S_IRUSR | S_IWUSR | S_IRGRP, undef, undef, [$accessconf], $DODitem);
   }
   elsif ($issue_exist == $TRUE)
   {
      $retval = $TRUE;
   }

   if (not $retval)
   {
      baselog($LOG_ERROR, "$DODitem cannot be handled.");         
   }
   
   return $retval;
}

###############################################################################
## @fn sub check()
# @brief Check existence of non-conformance for LNX00440
# @details This method looks at the file /etc/security/access.conf and checks
#          if it is permissive enough.
# @return TRUE - the file is permissive enough
# @return FALSE - the file is more permissive than 640
# @return H_ERROR - an error happened
sub check()
{
   my $retval = $FALSE;
   
   baselog($LOG_DEBUG, "Checking '$accessconf'");

   if (-f $accessconf)
   {
      #handle only mode rather than file type
      my $mode = (stat($accessconf))[2] & 07777;
      
      #(S_IXUSR | S_IWGRP | S_IXGRP | S_IRWXO) = 0137
      if (($mode & (S_IXUSR | S_IWGRP | S_IXGRP | S_IRWXO)) > 0)
      {
         my $form = sprintf("%04o", $mode);  
         
         baselog($LOG_DEBUG, "The current permissions of '$accessconf' are $form.");
      }
      else
      {
         baselog($LOG_INFO, "The file '$accessconf' permssions are set as per $DODitem specifications.");
         $retval = $TRUE;
      }
   }
   else
   {
      baselog($LOG_ERROR, "The file '$accessconf' does not exist.");
      $retval = $H_ERROR;
   }

   return $retval;
}

1;
__END__
