###############################################################################
## @file hardening_common.pm
# @brief Hardening subsystem common interfaces
# @details This module contains constants and function common for whole
#          hardening subsystem.
# @author Alexander Kolotov, Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - Fix get_status_linux_service for incorrect status of xinetd
#                 services.
# @version 1.03 - New action constant $H_IF_POST_LB_STATUS and subroutines for
#                 post-installation banner formatting and printing were added.
# @version 1.04 - Move functions write_file, backup_and_replace, replace_by_new
#                 to common_functions.pm
# @version 1.05 - Move change_permissions_and_ownership function
#                 to common_functions.pm, $NO_LOG to common_variables.pm
# @version 1.06 - The following functions have been added in order to control
#                 TELNET settings - service_telnet_start, service_telnet_stop,
#                 open_telnet_firewall, close_telnet_firewall.
#                 The following functions have been changed to include semaphore
#                 using: service_ftp_start, service_ftp_stop, service_tftp_start,
#                 service_tftp_stop;
#                 enable_service_firewall, disable_service_firewall,
#                 change_firewall_port_status are moved here from base_ftp module.
# @version 1.07 - The issue with incorrect log error has been fixed
# @version 1.08 - New action - init is supported. is_valid_backup_cookie sub is changed.
# @version 1.09 - introduced delay between sending the signal and checking the dump file in
#                 get_active_xinet_services
#
## @class hardening_common
package hardening::hardening_common;

use warnings;
use strict;

use lib "/opt/nortel/base/lib";
use common_variables;
use common_functions;
use install_common;
use recovery::backup qw($backup_cookie);

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
  $PATH_TO_BASIC_HARDENING_MODULES
  $PATH_TO_ENHANCED_HARDENING_MODULES
  $PATH_TO_HARDENING_XML_FILE
  $HARDENING_XML
  $HARDENING_ROOT_ELEM
  $HARDENING_STATUS_ATTR
  $HARDENING_STATUS_ON
  $HARDENING_STATUS_OFF
  $H_IF_APPLY
  $H_IF_POST_LB_INSTALL
  $H_IF_POST_LB_RECOVERY
  $H_IF_INIT
  $H_IF_SEND_SNMP
  $H_IF_POST_LB_STATUS
  $H_IF_RECOVERY
  $H_IF_POST_APPINSTALL
  $H_IF_POST_PATCHING
  $H_ERROR
  $TELNET
  $FTP
  $TFTP
  &prepare_action_hash
  &do_help_action
  &do_basic_hardening_action
  &do_enhanced_hardening_action
  &do_enhanced_hardening_action_for_module
  &get_backup_name
  &get_tmp_name
  &permissions_less_or_equal
  &remove_permissions
  &write_harden_file
  &start_and_enable_linux_service
  &stop_and_disable_linux_service
  &activate_xinet_service
  &deactivate_xinet_service
  &get_status_linux_service
  $NOT_LSB_SERVICE
  $HANDLE_XINET_SERVICE
  &enable_linux_service
  &disable_linux_service
  &start_linux_service
  &stop_linux_service
  &service_ftp_start
  &service_ftp_stop
  &service_tftp_start
  &service_tftp_stop
  &service_telnet_start
  &service_telnet_stop
  &open_telnet_firewall
  &close_telnet_firewall
  &is_option_permitted
  &open_hardening_storage
  &save_hardening_storage
  &get_xml_status_for_option
  &set_xml_status_for_option
  &set_new_hardening_status
  $FORCE_TO_CHANGE_STATUS
  $CHANGE_SYSTEM_AFTER_XML
  &show_post_installation_banner
  &prepare_module_status_for_banner
  &enable_service_firewall
  &disable_service_firewall
  &change_firewall_port_status
  &hardening_special_baselog
  );

our $VERSION = '1.09';

use Pod::Usage;
use File::Basename;
use POSIX;
use baselog;
use base_system;
use plugin_mgr;
use base_xml;
use base_semctl;

#Declarations
## @skipline
sub prepare_action_hash(@);
## @skipline
sub do_help_action($);
## @skipline
sub do_basic_hardening_action($);
## @skipline
sub do_enhanced_hardening_action($);
## @skipline
sub do_enhanced_hardening_action_for_module($$);
## @skipline
sub transform_file_name($;$);
## @skipline
sub get_backup_name($;$);
## @skipline
sub get_tmp_name($);
## @skipline
sub permissions_less_or_equal($$);
## @skipline
sub remove_permissions($$;$);
## @skipline
sub write_harden_file($$;$);
## @skipline
sub get_active_xinet_services();
## @skipline
sub start_and_enable_linux_service($);
## @skipline
sub stop_and_disable_linux_service($);
## @skipline
sub activate_xinet_service($);
## @skipline
sub deactivate_xinet_service($);
## @skipline
sub get_status_linux_service($;$$);
## @skipline
sub enable_linux_service($;$);
## @skipline
sub disable_linux_service($;$);
## @skipline
sub start_linux_service($);
## @skipline
sub stop_linux_service($);
## @skipline
sub service_ftp_start();
## @skipline
sub service_ftp_stop();
## @skipline
sub service_tftp_start();
## @skipline
sub service_tftp_stop();
## @skipline
sub is_option_permitted($);
## @skipline
sub get_xml_status_for_option($);
## @skipline
sub set_xml_status_for_option($$);
## @skipline
sub is_hardening_option_valid($);
## @skipline
sub create_hardening_storage();
## @skipline
sub open_hardening_storage();
## @skipline
sub save_hardening_storage($);
## @skipline
sub set_new_hardening_status($$;$$);
## @skipline
sub show_post_installation_banner();
## @skipline
sub prepare_module_status_for_banner($$);
## @skipline
sub open_telnet_firewall();
## @skipline
sub close_telnet_firewall();
## @skipline
sub enable_service_firewall($);
## @skipline
sub disable_service_firewall($);
## @skipline
sub change_firewall_port_status($$);
## @skipline
sub handle_storage_does_not_exist($);
## @skipline
sub handle_option_does_not_exist($$$);
## @skipline
sub is_recovery_or_init_stage();
## @skipline
sub hardening_special_baselog($$;$);
## @skipline
sub is_special_restore_case();

our $PATH_TO_BASIC_HARDENING_MODULES    = "/opt/nortel/base/lib/hardening/basic/";
our $PATH_TO_ENHANCED_HARDENING_MODULES = "/opt/nortel/base/lib/hardening/plugins/";

our $PATH_TO_HARDENING_XML_FILE    = $SYSCONFIG_ROOT;
our $HARDENING_XML                 = "hardening.xml";
our $HARDENING_STORAGE             = $PATH_TO_HARDENING_XML_FILE . "/" . $HARDENING_XML;
our $HARDENING_STORAGE_TMP         = $HARDENING_STORAGE . ".tmp";
our $HARDENING_STORAGE_PERMISSIONS = 0600;
our $HARDENING_STORAGE_OWNER_UID   = $ROOT_UID;
our $HARDENING_STORAGE_OWNER_GID   = $ROOT_GID;

our $HARDENING_ROOT_ELEM   = "harden";
our $HARDENING_XML_VERSION = "1.0";

our $HARDENING_VERSION_ATTR = "version";
our $HARDENING_STATUS_ATTR  = "applied";
our $HARDENING_STATUS_ON    = "on";
our $HARDENING_STATUS_OFF   = "off";

# Hardening InterFaces
# 'apply' action for Basic modules
our $H_IF_APPLY = "apply";

# performed after linux base installation
our $H_IF_POST_LB_INSTALL = "post_lb_install";

# performed as part of recovery process after linux base installation
our $H_IF_POST_LB_RECOVERY = "post_lb_recovery";

# performed in order to show Enhanced Hardening items status after linux base
# installation
our $H_IF_POST_LB_STATUS = "post_lb_status";

# performed on post rpm installing stage
our $H_IF_INIT = "init";

# performed in order to send snmp trap in accordance with the current options
# state
our $H_IF_SEND_SNMP = "send_snmp";

# performed as part of recovery process on run-time system
our $H_IF_RECOVERY = "recovery";

# performed after applications installation
our $H_IF_POST_APPINSTALL = "post_appinstall";

# performed after patches activation/deactivation
our $H_IF_POST_PATCHING = "post_patching";

# Hardening error
our $H_ERROR = -1;

our $NOT_LSB_SERVICE         = $TRUE;
our $HANDLE_XINET_SERVICE    = $TRUE;
our $FORCE_TO_CHANGE_STATUS  = 1;
our $CHANGE_SYSTEM_AFTER_XML = 2;

our $XINET_SERVICE   = "xinetd";
our $XINET_DUMP_FILE = "/var/run/xinetd.dump";

my $HARDENING_POSTFIX = ".hardening";

our $TELNET = 'telnet';
our $FTP    = 'ftp';
our $TFTP   = 'tftp';

my $CURRENT_HARDENING_ACTION = '';

##############################################################################
## @fn sub prepare_action_hash($module, $action, @params)
# @brief   Prepare a hash to use it with an instance of the Plug-in manager.
# @details This method takes the passed parameters and fills the hash. This
#          hash could be used with such method of the Plug-in manager as
#          do_action and try_action.
# @param   $module - name of module/plug-in. Could be undefined.
# @param   $action - action.  Could be undefined.
# @param   @params - all extra parameters are interpreted as parameters of the
#                    action and stored into the corresponding field of the hash
#                    as a reference to this array.
# @return  \%hash  - reference to the action hash
sub prepare_action_hash(@)
{
   my %hash = ();

   $hash{'module'}  = shift @_;
   $hash{'action'}  = shift @_;
   $hash{'params'}  = \@_;
   $hash{'err_msg'} = '';

   baselog($LOG_DEBUG, "Module: $hash{'module'}")        if (defined $hash{'module'});
   baselog($LOG_DEBUG, "Action: $hash{'action'}")        if (defined $hash{'action'});
   baselog($LOG_DEBUG, "Parameters: @{$hash{'params'}}") if (scalar @_);

   return \%hash;
} ## end sub prepare_action_hash(@)

###############################################################################
## @fn sub do_help_action($module)
# @brief   Outputs manual page embedded into a perl module.
# @details This method looks into a file which this method is invoked from,
#          and prepare manual page by using the tool /usr/bin/pod2man. After
#          that this page is displayed by using the tool /usr/bin/less.
# @param   $module - the name of module which will be displayed in the manual
#                    page
# @return  TRUE - the manual page is displayed successfully
# @return  FALSE - some error happens
sub do_help_action($)
{
   my $module = shift @_;

   if ((not defined $module) or ($module eq ""))
   {
      baselog($LOG_ERROR, "Module name is not specified");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to prepare manual page for the module '$module'.");

   my $retval = $FALSE;

   my ($path) = (caller)[1];

   if ((defined $path) and (-f $path))
   {

      #By some undiscovered reason we cannot merely use perldoc (Pod::Usage)
      #under root account.
      system(qq#$POD2MAN -c "$MAN_CENTER" -n "harden $module" -r "$MAN_RELEASE" $path | $NROFF -man 2>/dev/null#);

      my $res = (WEXITSTATUS($?) == 0) ? $TRUE : $FALSE;

      if ($res)
      {
         $retval = $TRUE;
      }
      else
      {
         baselog($LOG_ERROR, "Cannot display manual page.\n", $TRUE);
      }
   } ## end if ((defined $path) and...

   return $retval;
} ## end sub do_help_action($)

###############################################################################
## @fn sub do_basic_hardening_action($action)
# @brief   Perform the action for Basic hardening items.
# @details This method uses the Plug-in manager to perform the provided action
#          for all Basic hardening modules.
# @param   $action - action.
# @return  TRUE - the action is performed
# @return  FALSE - some error happens
sub do_basic_hardening_action($)
{
   my $action = shift @_;

   my $retval = $FALSE;

   if (not $action)
   {
      baselog($LOG_ERROR, "The action is not specified.");
      return $retval;
   }

   $CURRENT_HARDENING_ACTION = $action;

   baselog($LOG_DEBUG, "Try to create the Basic Hardening manager.");

   my $basic_harden_mgr = plugin_mgr->new($PATH_TO_BASIC_HARDENING_MODULES);
   if (defined $basic_harden_mgr)
   {

      #Perform the action for all elements - the first paremeter is undefined.
      my $status = $basic_harden_mgr->try_action(prepare_action_hash(undef, $action));

      baselog($LOG_DEBUG, "The Basic Hardening manager returns '$status'.");

      undef $basic_harden_mgr;

      $retval = $TRUE;
   } ## end if (defined $basic_harden_mgr...
   else
   {
      baselog($LOG_ERROR, "The Basic Hardening manager is not created.");
   }

   return $retval;
} ## end sub do_basic_hardening_action($)

###############################################################################
## @fn sub do_enhanced_hardening_action($action)
# @brief   Perform the action for Enhanced hardening items.
# @details This method uses the Plug-in manager to perform the provided action
#          for all Enhanced hardening modules (including help and status).
# @param   $action - action.
# @return  TRUE - the action is performed
# @return  FALSE - some error happens
sub do_enhanced_hardening_action($)
{
   my $action = shift @_;

   my $retval = $FALSE;

   if (not $action)
   {
      baselog($LOG_ERROR, "The action is not specified.");
      return $retval;
   }

   $CURRENT_HARDENING_ACTION = $action;

   baselog($LOG_DEBUG, "Try to create the Enhanced Hardening manager.");

   my $enhanced_harden_mgr = plugin_mgr->new($PATH_TO_ENHANCED_HARDENING_MODULES);
   if (defined $enhanced_harden_mgr)
   {

      #Perform the action for all elements - the first paremeter is undefined.
      my $status = $enhanced_harden_mgr->try_action(prepare_action_hash(undef, $action));

      baselog($LOG_DEBUG, "The Enhanced Hardening manager returns '$status'.");

      undef $enhanced_harden_mgr;

      $retval = $TRUE;
   } ## end if (defined $enhanced_harden_mgr...
   else
   {
      baselog($LOG_ERROR, "The Enhanced Hardening manager is not created.");
   }

   return $retval;
} ## end sub do_enhanced_hardening_action($)

###############################################################################
## @fn sub do_enhanced_hardening_action_for_module($module, $action)
# @brief   Perform the action for Enhanced hardening items.
# @details This method uses the Plug-in manager to perform the provided action
#          for one Enhanced hardening module.
# @param   $module - module.
# @param   $action - action.
# @return  TRUE - the action is performed
# @return  FALSE - some error happens
sub do_enhanced_hardening_action_for_module($$)
{
   my ($module, $action) = @_;

   my $retval = $FALSE;

   if (not $module)
   {
      baselog($LOG_ERROR, "The module is not specified.");
      return $retval;
   }

   if (not $action)
   {
      baselog($LOG_ERROR, "The action is not specified.");
      return $retval;
   }

   $CURRENT_HARDENING_ACTION = $action;

   baselog($LOG_DEBUG, "Try to create the Enhanced Hardening manager.");

   my $enhanced_harden_mgr = plugin_mgr->new($PATH_TO_ENHANCED_HARDENING_MODULES);
   if (defined $enhanced_harden_mgr)
   {

      #Perform the action for all elements - the first paremeter will be undefined.
      $retval = $enhanced_harden_mgr->try_action(prepare_action_hash($module, $action));

      baselog($LOG_DEBUG, "The Enhanced Hardening manager returns '$retval'.");

      undef $enhanced_harden_mgr;
   } ## end if (defined $enhanced_harden_mgr...
   else
   {
      baselog($LOG_ERROR, "The Enhanced Hardening manager is not created.");
   }

   return $retval;
} ## end sub do_enhanced_hardening_action_for_module($$)

###############################################################################
## @fn sub transform_file_name($filename, $extra_postfix)
# @brief   Transformation of the file name.
# @details This method adds standard hardening postfix plus extra postfix to
#          the given file name and provides resulting file name.
# @param   $filename - a file name.
# @param   $extra_postfix - additional postfix (optional).
# @return  empty string if $filename is not defined otherwise new file name.
sub transform_file_name($;$)
{
   my ($filename, $extra_postfix) = @_;

   if (not defined $extra_postfix)
   {
      $extra_postfix = "";
   }
   else
   {
      $extra_postfix = ".$extra_postfix";
   }

   $filename = $filename . $HARDENING_POSTFIX . $extra_postfix;

   return $filename;
} ## end sub transform_file_name($;$)

###############################################################################
## @fn sub get_backup_name($filename, $extra_postfix)
# @brief   Provide backup name for the give file name.
# @details This method uses the subroutine transform_file_name in order to get
#          file name suitable to be backup the file name.
# @param   $filename - a file name.
# @param   $extra_postfix - additional postfix (optional).
# @return  empty string if $filename is not defined otherwise new file name.
sub get_backup_name($;$)
{
   my ($filename, $extra_postfix) = @_;

   if ((not defined $filename) or ($filename eq ""))
   {
      baselog($LOG_ERROR, "The filename is not specified.");
      return "";
   }

   baselog($LOG_DEBUG, "Preparing backup name for the file '$filename'.");

   $filename = transform_file_name($filename, $extra_postfix);

   baselog($LOG_DEBUG, "Backup name is '$filename'.");

   return $filename;
} ## end sub get_backup_name($;$)

###############################################################################
## @fn sub get_tmp_name($filename)
# @brief   Provide temporary file name for the give file name.
# @details This method uses the subroutine transform_file_name() with the extra
#          postfix "tmp" in order to get the name for temporary file.
# @param   $filename - a file name.
# @return  empty string if $filename is not defined otherwise new file name.
sub get_tmp_name($)
{
   my $filename = shift @_;

   if ((not defined $filename) or ($filename eq ""))
   {
      baselog($LOG_ERROR, "The filename is not specified.");
      return "";
   }

   baselog($LOG_DEBUG, "Preparing temporary file name for the file '$filename'.");

   $filename = transform_file_name($filename, "tmp");

   baselog($LOG_DEBUG, "The name for temporary file is '$filename'.");

   return $filename;
} ## end sub get_tmp_name($)

###############################################################################
## @fn sub permissions_less_or_equal($filename, $needmode)
# @brief   Check permissions of the file
# @details This method checks the file permissions if they are less or equal
#          the requested once.
# @param   $filename - name of file
# @param   $needmode - max permissions which the file can have
# @return  TRUE - the file permissions are less or equal the requested once
# @return  FALSE - the file permissions are greater than the requested once
sub permissions_less_or_equal($$)
{
   my ($filename, $needmode) = @_;

   #handle only last 4 octets
   $needmode = $needmode & 07777;

   my (undef, undef, $mode) = stat($filename);

   if (not defined $mode)
   {
      baselog($LOG_DEBUG, "Cannot get stat for '$filename'");
      return $TRUE;
   }

   #handle only mode rather than file type
   $mode = $mode & 07777;

   my $form     = sprintf("%04o", $mode);
   my $needform = sprintf("%04o", $needmode);
   baselog($LOG_DEBUG, "The current permissions of '$filename' are $form (needed less or equal $needform).");

   #Since we are looking for the files which permissions are greater than given
   #we cannot use "==", that is why we are using (curmode & ~need > 0)
   if (($mode & (~$needmode)) > 0)
   {
      return $FALSE;
   }
   else
   {
      return $TRUE;
   }
} ## end sub permissions_less_or_equal($$)

###############################################################################
## @fn sub remove_permissions($mode, $file_list, $prefix)
# @brief   Remove permissions of the files
# @details This method uses the perl subroutines chmod to remove permissions
#          $mode the list of files.
# @param   $mode     - new permissions, if it is undefined, permissions will
#                      not be changed
# @param   $file_list - list of files
# @param   $prefix   - prefix which will be provided with info message
# @return  TRUE - all operations are performed successfully or nothing is
#                 changed
# @return  FALSE - some error happens
sub remove_permissions($$;$)
{
   my ($mode, $file_list, $prefix) = @_;

   if (defined $prefix)
   {
      $prefix = $prefix . ": ";
   }
   else
   {
      $prefix = "";
   }

   my $retval = $TRUE;

   if (not defined $mode)
   {
      baselog($LOG_INFO, "Nothing is to be changed.");
      return $retval;
   }

   if (defined $file_list)
   {

      foreach my $file (@{$file_list})
      {
         my $object = "";
         if (-d $file)
         {
            $object = "directory";
         }
         elsif (-f $file)
         {
            $object = "file";
         }

         if ($object ne "")
         {

            #get the current permissions of the file
            my (undef, undef, $cur_mode) = stat($file);

            #handle only mode rather than file type
            $cur_mode = $cur_mode & 07777;

            #get new permission
            #e.g. if current permissions 0666 and we need 0640,
            #it will be 0640 = 0666 & ~027
            my $new_mode = $cur_mode & ~$mode;

            #Change the permissions
            if (chmod($new_mode, $file) == 0)
            {
               $retval = $FALSE;
            }
            else
            {
               baselog($LOG_INFO, $prefix . "the permissions of the $object '$file' changed successfully.");
            }
         } ## end if ($object ne "")
         else
         {
            baselog($LOG_ERROR, $prefix . "the file system object '$file' does not exist.");
            $retval = $FALSE;
         }
      } ## end foreach my $file (@{$file_list...
   } ## end if (defined $file_list...
   else
   {
      baselog($LOG_ERROR, "List of file is not provided.");
      $retval = $FALSE;
   }

   return $retval;
} ## end sub remove_permissions($$;$)

###############################################################################
## @fn sub write_harden_file($file, $content, $not_backup)
# @brief Write content to a file.
# @details This method writes some data to a temporary file, and then replace
#          the original file by this temporary one. Prior to replacing, it
#          tries to backup the original file.
# @param $file       - file name
# @param $content    - new content of the file
# @param $not_backup - (optional) if it is true, the file will not be backed up
# @return TRUE - the content is written to the file.
# @return  FALSE - some error happens
sub write_harden_file($$;$)
{
   my ($file, $content, $not_backup) = @_;

   return write_file($file, $content, $not_backup, \&get_tmp_name, \&get_backup_name);
}

###############################################################################
## @fn sub start_and_enable_linux_service($service)
# @brief   Starts and enable some Linux service
# @details This method uses the subroutines enable_linux_service and
#          start_linux_service in order to enable some Linux service and start
#          it at once.
#
#          At first, a service is being tried to start If it is done, it
#          is enabled.
# @param   $service - the name of the service
# @return  TRUE - the service is enabled and started successfully
# @return  FALSE - some error happens
sub start_and_enable_linux_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   if (start_linux_service($service) and enable_linux_service($service))
   {
      return $TRUE;
   }

   return $FALSE;
} ## end sub start_and_enable_linux_service($)

###############################################################################
## @fn sub stop_and_disable_linux_service($service)
# @brief   Stops and disables some Linux service
# @details This method uses the subrutines disable_linux_service and
#          stop_linux_service in order to disable some Linux service and stop
#          it at once.
#
#          At first, a service is being tried to stop. If it is done, it
#          is disabled.
# @param   $service - the name of the service
# @return  TRUE - the service is disabled and stopped successfully
# @return  FALSE - some error happens
sub stop_and_disable_linux_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   if (stop_linux_service($service) and disable_linux_service($service))
   {
      return $TRUE;
   }

   return $FALSE;
} ## end sub stop_and_disable_linux_service($)

###############################################################################
## @fn sub activate_xinet_service($service)
# @brief   Activate some xinet based service
# @details This method enables some xinet based service and if it is necessary,
#          run xinet daemon.
# @param   $service - the name of the service
# @return  TRUE - the service is activated successfully
# @return  FALSE - some error happens
sub activate_xinet_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to activate the xinet based service '$service'.");

   #Enable xinet based service
   if (not enable_linux_service($service, $HANDLE_XINET_SERVICE))
   {
      baselog($LOG_ERROR, "Cannot activate the xinet based service '$service'.");
      return $FALSE;
   }

   #Check if xinetd is running, if not - run it.
   #Assuming (due to security reason) that xinet based services will be
   #requested to be run either after *every* Linux boot up or during run time,
   #xinetd service is running but not enabled.

   #xinetd does not follow LSB recommendation that is why its status should be
   #defined through the service's output parsing.
   my $xinetd_status = get_status_linux_service($XINET_SERVICE, $NOT_LSB_SERVICE);
   if ($xinetd_status == $H_ERROR)
   {
      baselog($LOG_ERROR, "Cannot get the status of xinet service.");
      return $FALSE;
   }
   elsif ($xinetd_status == $FALSE)
   {
      if (not start_linux_service($XINET_SERVICE))
      {
         baselog($LOG_ERROR, "Cannot run xinet service.");
         return $FALSE;
      }
   } ## end elsif ($xinetd_status == ...
   else
   {
      baselog($LOG_DEBUG, "The xinetd service is already started.");
   }

   return $TRUE;
} ## end sub activate_xinet_service($)

###############################################################################
## @fn sub get_active_xinet_services()
# @brief   Get a list of active xinet based services
# @details This method uses xinetd's dump files generated by xinetd after
#          receiving SIGUSR1 signal in order to get the list of active xinet
#          based services.
#
#          This file does not contain inactive services at all. The section
#          describing every active service starts with 'Service = '.
#          This method does not check if xinetd service is running. So, it is
#          necessary to do it before invoking of this method.
# @return  \\\@retarr - reference to array containing the list of active xinet
#                     based services. This list will be empty if there are no
#                     services found.
# @return  undef    - some error happens
sub get_active_xinet_services()
{
   my @retarr = ();

   baselog($LOG_DEBUG, "Trying to found active xinet based services.");

   #Since dump files accumulates the information it is necessary to remove all
   #previous collected information
   if (not do_system([$RM, "-f", $XINET_DUMP_FILE]))
   {
      baselog($LOG_ERROR, "Cannot remove old information about the xinet based service.");
      return undef;
   }

   #Send signal to the xinetd process. Since it is assumed that only one
   #xinetd service can be run at one moment, /usr/bin/killall command is used.
   if (not do_system([$KILLALL, "-SIGUSR1", $XINET_SERVICE]))
   {
      baselog($LOG_ERROR, "Cannot collect information about the xinet based service.");
      return undef;
   }

   #Try to check dump file if there is no such file - check it again with some delay (max delay - 3 secs)
   for (my $i = 0 ; ; $i++)
   {
      if (-f $XINET_DUMP_FILE)
      {
         last;
      }
      elsif ($i == 3)
      {
         baselog($LOG_ERROR, "Cannot get information about the xinet based service.");
         return undef;
      }
      sleep 1;
   } ## end for (my $i = 0 ; ; $i++...

   my $fd;
   if (not open($fd, "<", $XINET_DUMP_FILE))
   {
      baselog($LOG_ERROR, "Cannot open file with information about the xinet based service: $!");
      return undef;
   }

   #Read the file and look for the line starting with 'Service = '
   foreach my $line (<$fd>)
   {
      if ($line =~ /^Service = (.*)$/)
      {
         baselog($LOG_DEBUG, "Active xinet based service: '$1'.");
         push @retarr, $1;
      }
   } ## end foreach my $line (<$fd>)

   if (not close $fd)
   {
      baselog($LOG_ERROR, "Cannot close the file descriptor: $!");
   }

   #Remove dump file
   do_system([$RM, "-f", $XINET_DUMP_FILE]);

   if (not scalar @retarr)
   {
      baselog($LOG_DEBUG, "No active xinet based service found.");
   }

   return \@retarr;
} ## end sub get_active_xinet_services()

###############################################################################
## @fn sub deactivate_xinet_service($service)
# @brief   Deactivate some xinet based service
# @details This method disables some xinet based service and if there are no
#          other active xinet based services, the method stops xinet daemon.
# @param   $service - the name of the service
# @return  TRUE - the service is deactivated
# @return  FALSE - some error happens
sub deactivate_xinet_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to deactivate the xinet based service '$service'.");

   #Disable xinet based service
   if (not disable_linux_service($service, $HANDLE_XINET_SERVICE))
   {
      baselog($LOG_ERROR, "Cannot deactivate the xinet based service '$service'.");
      return $FALSE;
   }

   #Check if xinetd is running
   #xinetd does not follow LSB recommendation that is why its status should be
   #defined through the service's output parsing.
   my $xinetd_status = get_status_linux_service($XINET_SERVICE, $NOT_LSB_SERVICE);
   if ($xinetd_status == $H_ERROR)
   {
      baselog($LOG_ERROR, "Cannot get the status of xinet service.");
      return $FALSE;
   }
   elsif ($xinetd_status == $FALSE)
   {

      #The xinetd service is already stopped, no action required to stop it.
      baselog($LOG_DEBUG, "The xinetd service is already stopped.");
      return $TRUE;
   } ## end elsif ($xinetd_status == ...

   my $retval = $FALSE;

   #Check if there are other xinet based services enabled
   my $services = get_active_xinet_services();
   if (not defined $services)
   {
      baselog($LOG_ERROR, "Cannot get the status of xinet based services.");
   }
   elsif (scalar @$services)
   {
      baselog($LOG_DEBUG, "Active xinet services found. Xinetd will not be stopped.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "No active xinet services found. Xinetd will be stopped.");

      #Assuming (due to security reason) that xinet based services will be
      #requested to be run either after *every* Linux boot up or during
      #run time, xinetd service is stopped and explicitly disabled.
      $retval = stop_and_disable_linux_service($XINET_SERVICE);
   } ## end else [ if (not defined $services...

   return $retval;
} ## end sub deactivate_xinet_service($)

###############################################################################
## @fn sub get_status_linux_service($service, $parse_output, $xinet_service)
# @brief   Get status of some Linux service
# @details This method allows getting status of generic linux services and
#          xinet based services.
#
#          The status of generic linux services is determined by invoking
#          /sbin/service command:
#         \li 0 - the service is running,
#         \li 1-3 - the service is stopped.
#
#          Since some services do not follow Linux Standard Base Core
#          Specification, they return not standard exit codes, that is why a
#          dirty solution - parsing output of /bin/service command is used:
#         \li "... is running" - the service is running,
#         \li "... is stopped" - the service is stopped.
#
#          In order to use this parsing the second parameter of this
#          subroutine should be set in TRUE.
#
#          The status of xinet based services is checked only if the third
#          argument is set in TRUE. In this case the status of xinetd is
#          checked if it is stopped the requested service is considered as
#          stopped as well. If the xinetd is running, it is necessary to parse
#          output of /sbin/chkconfig command:
#         \li "service        on" - the service is active
#         \li "service        off" - the service is not active
# @param   $service       - the name of the service
# @param   $parse_output  - not rely on the return value, parse the output.
# @param   $xinet_service - the specified service is a xinet based service.
#                           $parse_output and $xinet_service should not be used
#                           together.
# @return  TRUE    - the service is running
# @return  FALSE   - the service is stopped
# @return  H_ERROR - some error happens
sub get_status_linux_service($;$$)
{
   my ($service, $parse_output, $xinet_service) = @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $H_ERROR;
   }

   #If both arguments are set - it is illegal usage
   if (($parse_output) and ($xinet_service))
   {
      baselog($LOG_ERROR, "Illegal subroutine usage.");
      return $H_ERROR;
   }

   if (not defined $parse_output)
   {
      $parse_output = $FALSE;
   }

   if (not defined $xinet_service)
   {
      $xinet_service = $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to get status of the service '$service'.");

   my $retval = $H_ERROR;

   if (not $xinet_service)
   {
      my $service_status = 255;
      my @service_output = ();

      if (not do_system([$SERVICE, $service, $SRV_STATUS], {$dsRETVAL => \$service_status, $dsOUT => \@service_output, $dsNOERROR => $TRUE}))
      {
         baselog($LOG_ERROR, "Invocation of $SERVICE for the service '$service' failed");
      }
      else
      {
         baselog($LOG_DEBUG, "The status of the service '$service' is '$service_status'.");

         #According to Linux Standard Base Core Specification the status action
         #has to return:
         #0 - program is running or service is OK
         #1 - program is dead and /var/run pid file exists
         #2 - program is dead and /var/lock lock file exists
         #3 - program is not running
         #4 - program or service status is unknown
         #5-99 - reserved for future LSB use
         #100-149 - reserved for distribution use
         #150-199 - reserved for application use
         #200-254 - reserved
         #So, 0     - is considered as the service is running
         #    1-3   - is considered as the service is not running
         #    4-... - error
         if ($service_status == 0)
         {

            #for services which does not comply to LSB Core Specification it is
            #necessary to double check output :(
            #"... is running" is considered as the service is running
            #"... is stopped" is considered as the service is not running
            if ($parse_output)
            {
               foreach my $line (@service_output)
               {
                  chomp $line;

                  if ($line =~ /is running/)
                  {
                     baselog($LOG_DEBUG, "The status of the service '$service' is '$line'.");
                     $retval = $TRUE;
                     last;
                  } ## end if ($line =~ /is running/...
                  elsif ($line =~ /is stopped/)
                  {
                     baselog($LOG_DEBUG, "The status of the service '$service' is '$line'.");
                     $retval = $FALSE;
                     last;
                  } ## end elsif ($line =~ /is stopped/...
               } ## end foreach my $line (@service_output...
               if ($retval == $H_ERROR)
               {
                  baselog($LOG_ERROR, "Status of the service '$service' returns '" . join("/", @service_output) . "'.");
               }
            } ## end if ($parse_output)
            else
            {
               $retval = $TRUE;
            }
         } ## end if ($service_status ==...
         elsif ($service_status <= 3)
         {
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_ERROR, "Status of the service '$service' returns '$service_status'.");
         }
      } ## end else [ if (not do_system([$SERVICE...
   } ## end if (not $xinet_service...
   else
   {

      #Get the status of xinet based service

      #Check if xinetd is running
      #xinetd does not follow LSB recommendation that is why its status should be
      #defined through the service's output parsing.
      my $xinetd_status = get_status_linux_service($XINET_SERVICE, $NOT_LSB_SERVICE);
      if ($xinetd_status == $TRUE)
      {

         #Xinetd is running, get the list of active xinet based services
         my $services = get_active_xinet_services();
         if (not defined $services)
         {
            baselog($LOG_ERROR, "Cannot get the status of xinet based services.");
         }
         elsif (scalar(@$services) == 0)
         {

            #No running services found
            baselog($LOG_DEBUG, "The status of the service '$service' is 'off'.");
            $retval = $FALSE;
         } ## end elsif (scalar(@$services)...
         else
         {

            #Look for the service int the list active xinet based services
            $retval = $FALSE;

            foreach my $active_serv (@$services)
            {
               if ($active_serv eq $service)
               {
                  baselog($LOG_DEBUG, "The status of the service '$service' is 'on'.");
                  $retval = $TRUE;
               }
            } ## end foreach my $active_serv (@$services...

            if (not $retval)
            {
               baselog($LOG_DEBUG, "The status of the service '$service' is 'off'.");
            }
         } ## end else [ if (not defined $services...
      } ## end if ($xinetd_status == ...
      elsif ($xinetd_status == $FALSE)
      {

         #Since xinetd is disabled, we can consider any xinet based service is
         #disabled as well.
         baselog($LOG_DEBUG, "The status of the service '$service' is 'off'.");
         $retval = $FALSE;
      } ## end elsif ($xinetd_status == ...
   } ## end else [ if (not $xinet_service...

   return $retval;
} ## end sub get_status_linux_service($;$$)

###############################################################################
## @fn sub enable_linux_service($service, $xinet_service)
# @brief   Enable some Linux service
# @details This method uses /sbin/chkconfig command in order to enable some
#          Linux service (to allow running this service automatically as a
#          part of Linux bootup sequence). Since xinet based services have
#          different syntax for enabling, $xinet_service flag can be used to
#          specify that the service is a xinet based service.
# @param   $service - the name of the service
# @param   $xinet_service - the specified service is a xinet based service
# @return  TRUE - the service is enabled successfully
# @return  FALSE - some error happens
sub enable_linux_service($;$)
{
   my ($service, $xinet_service) = @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   if (not defined $xinet_service)
   {
      $xinet_service = $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to enable the service '$service'.");

   my $retval = $TRUE;

   if (not $xinet_service)
   {
      if (not do_system([$CHKCONFIG, "--level", "345", $service, $SRV_ON]))
      {
         baselog($LOG_ERROR, "Invocation of $CHKCONFIG for the service '$service' failed");
         $retval = $FALSE;
      }
   } ## end if (not $xinet_service...
   else
   {
      if (not do_system([$CHKCONFIG, $service, $SRV_ON]))
      {
         baselog($LOG_ERROR, "Invocation of $CHKCONFIG for the service '$service' failed");
         $retval = $FALSE;
      }
   } ## end else [ if (not $xinet_service...

   if ($retval)
   {
      baselog($LOG_DEBUG, "The service '$service' is enabled.");
   }

   return $retval;
} ## end sub enable_linux_service($;$)

###############################################################################
## @fn sub disable_linux_service($service, $xinet_service)
# @brief   Disable some Linux service
# @details This method uses /sbin/chkconfig command in order to disable some
#          Linux service (to disallow running this service automatically as a
#          part of Linux bootup sequence). Since xinet based services have
#          different syntax for disabling, $xinet_service flag can be used to
#          specify that the service is a xinet based service.
# @param   $service - the name of the service
# @param   $xinet_service - the specified service is a xinet based service
# @return  TRUE - the service is disabled successfully
# @return  FALSE - some error happens
sub disable_linux_service($;$)
{
   my ($service, $xinet_service) = @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   if (not defined $xinet_service)
   {
      $xinet_service = $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to disable the service '$service'.");

   my $retval = $TRUE;

   if (not $xinet_service)
   {
      if (not do_system([$CHKCONFIG, "--level", "345", $service, $SRV_OFF]))
      {
         baselog($LOG_ERROR, "Invocation of $CHKCONFIG for the service '$service' failed");
         $retval = $FALSE;
      }
   } ## end if (not $xinet_service...
   else
   {
      if (not do_system([$CHKCONFIG, $service, $SRV_OFF]))
      {
         baselog($LOG_ERROR, "Invocation of $CHKCONFIG for the service '$service' failed");
         $retval = $FALSE;
      }
   } ## end else [ if (not $xinet_service...

   if ($retval)
   {
      baselog($LOG_DEBUG, "The service '$service' is disabled.");
   }

   return $retval;
} ## end sub disable_linux_service($;$)

###############################################################################
## @fn sub start_linux_service($service)
# @brief   Start some Linux service
# @details This method uses /sbin/service command in order to start some
#          Linux service.
# @param   $service - the name of the service
# @return  TRUE - the service is started successfully
# @return  FALSE - some error happens
sub start_linux_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to start the service '$service'.");

   my $retval = $TRUE;

   if (not do_system([$SERVICE, $service, $SRV_START], {$dsNOERROR => $TRUE}))
   {
      baselog($LOG_ERROR, "Invocation of $SERVICE for the service '$service' failed");
      $retval = $FALSE;
   }
   else
   {
      baselog($LOG_DEBUG, "The service '$service' is started.");
   }

   return $retval;
} ## end sub start_linux_service($)

###############################################################################
## @fn sub stop_linux_service($service)
# @brief   Stop some Linux service
# @details This method uses /sbin/service command in order to stop some Linux
#          service.
# @param   $service - the name of the service
# @return  TRUE - the service is stopped successfully
# @return  FALSE - some error happens
sub stop_linux_service($)
{
   my $service = shift @_;

   if (not defined $service)
   {
      baselog($LOG_ERROR, "There is no service specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to stop the service '$service'.");

   my $retval = $TRUE;

   if (not do_system([$SERVICE, $service, $SRV_STOP], {$dsNOERROR => $TRUE}))
   {
      baselog($LOG_ERROR, "Invocation of $SERVICE for the service '$service' failed");
      $retval = $FALSE;
   }
   else
   {
      baselog($LOG_DEBUG, "The service '$service' is stopped.");
   }

   return $retval;
} ## end sub stop_linux_service($)

###############################################################################
## @fn sub service_ftp_start()
# @brief   Starts FTP service through the hardening module ftp.pm
# @details This method uses the Hardening Manager in order to perform start
#          action of ftp.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_ftp_start()
{
   baselog($LOG_DEBUG, "Starting FTP service...");

   my $retval = $TRUE;

   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "FTP service will not be enabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('ftp', 'start'))
   {
      baselog($LOG_ERROR, "Cannot start FTP service");
      $retval = $FALSE;
   }
   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;
   baselog($LOG_INFO, "FTP service is started.");
   return $retval;

} ## end sub service_ftp_start()

###############################################################################
## @fn sub service_ftp_stop()
# @brief   Stops FTP service through the hardening module ftp.pm
# @details This method uses the Hardening Manager in order to perform stop
#          action of ftp.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_ftp_stop()
{
   baselog($LOG_DEBUG, "Stopping of FTP service...");

   my $retval = $TRUE;

   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "FTP service will not be disabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('ftp', 'stop'))
   {
      baselog($LOG_ERROR, "Cannot stop FTP service");
      $retval = $FALSE;
   }

   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;
   baselog($LOG_INFO, "FTP service is stopped.");
   return $retval;
} ## end sub service_ftp_stop()

###############################################################################
## @fn sub service_tftp_start()
# @brief   Starts TFTP service through the hardening module ftp.pm
# @details This method uses the Hardening Manager in order to perform start
#          action of tftp.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_tftp_start()
{
   baselog($LOG_DEBUG, "Activation of TFTP service...");

   my $retval = $TRUE;

   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "TFTP service will not be enabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('tftp', 'start'))
   {
      baselog($LOG_ERROR, "Cannot activated TFTP service");
      $retval = $FALSE;
   }

   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;
   baselog($LOG_INFO, "TFTP service is activated.");
   return $retval;
} ## end sub service_tftp_start()

###############################################################################
## @fn sub service_tftp_stop()
# @brief   Stops TFTP service through the hardening module tftp.pm
# @details This method uses the Hardening Manager in order to perform stop
#          action of tftp.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_tftp_stop()
{
   baselog($LOG_DEBUG, "Deactivation of TFTP service...");

   my $retval = $TRUE;

   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "TFTP service will not be disabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('tftp', 'stop'))
   {
      baselog($LOG_ERROR, "Cannot deactivate TFTP service");
      $retval = $FALSE;
   }

   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;
   baselog($LOG_INFO, "TFTP service is deactivated.");
   return $retval;

} ## end sub service_tftp_stop()

##############################################################################
## @fn sub service_telnet_start()
# @brief   Starts TELNET service through the hardening module telnet.pm
# @details This method uses the Hardening Manager in order to perform start
#          action of telnet.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_telnet_start()
{
   baselog($LOG_DEBUG, "Starting TELNET service...");
   my $retval = $TRUE;

   #initialize semaphore manager
   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "TELNET service will not be enabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('telnet', 'start'))
   {
      baselog($LOG_ERROR, "Cannot start TELNET service");
      $retval = $FALSE;
   }
   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;

   baselog($LOG_INFO, "TELNET service is started.");

   return $retval;
} ## end sub service_telnet_start()

###############################################################################
## @fn sub service_telnet_stop()
# @brief   Stops TELNET service through the hardening module telnet.pm
# @details This method uses the Hardening Manager in order to perform stop
#          action of telnet.pm hardening module
# @return  TRUE    - The service is started
# @return  FALSE   - The service is not started
sub service_telnet_stop()
{
   baselog($LOG_DEBUG, "Stopping of TELNET service...");

   my $retval  = $TRUE;
   my $semmngr = base_semctl::new($SEM_HARD);
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $FALSE;
   }
   if (!$semmngr->sem_take($SEM_HARD, IPC_NOWAIT))
   {
      baselog($LOG_ERROR, "Cannot take semaphores.");
      baselog($LOG_ERROR, "TELNET service will not be disabled.\n", $TRUE);
      undef $semmngr;
      return $FALSE;
   } ## end if (!$semmngr->sem_take...

   if (not do_enhanced_hardening_action_for_module('telnet', 'stop'))
   {
      baselog($LOG_ERROR, "Cannot stop TELNET service");
      $retval = $FALSE;
   }

   if (!$semmngr->sem_give($SEM_HARD))
   {
      baselog($LOG_WARNING, "Cannot release $SEM_HARD semaphores.");
   }
   undef $semmngr;
   baselog($LOG_INFO, "TELNET service is stopped.");
   return $retval;
} ## end sub service_telnet_stop()
###############################################################################
## @fn sub is_option_permitted($hardening_option)
# @brief   Returns status of an enhanced hardening option
# @details This method checks if en enhanced hardening option is permitted or
#          forbidden
# @param   $hardening_option - the enhanced hardening option
# @return  TRUE    - the ehnanced hardenign option is in the status 'on'
# @return  FALSE   - the enhanced hardening option is in the status 'off'
# @return  H_ERROR - some error happens
sub is_option_permitted($)
{
   my $hardening_option = shift @_;

   baselog($LOG_DEBUG, "Is the option '$hardening_option' permitted?");

   my $xml_status = get_xml_status_for_option($hardening_option);

   my $retval = $H_ERROR;

   if (not defined $xml_status)
   {
      baselog($LOG_ERROR, "Cannot get the status for the option '$hardening_option'.");
   }
   else
   {
      if ($xml_status eq $HARDENING_STATUS_OFF)
      {
         baselog($LOG_DEBUG, "The option '$hardening_option' is forbidden.");
         $retval = $FALSE;
      }
      elsif ($xml_status eq $HARDENING_STATUS_ON)
      {
         baselog($LOG_DEBUG, "The option '$hardening_option' is permitted.");
         $retval = $TRUE;
      }
      else
      {

         #It is valid situation when the status of the hardening module is
         #not set on recovery stage or in stage of new installation - there
         #is no hardening storage, or it exists but some options are not
         #initialized there.
         if (not is_recovery_or_init_stage())
         {
            baselog($LOG_ERROR, "Wrong status for the option '$hardening_option'.");
         }
      } ## end else [ if ($xml_status eq $HARDENING_STATUS_OFF...
   } ## end else [ if (not defined $xml_status...

   return $retval;
} ## end sub is_option_permitted($)

###############################################################################
## @fn sub get_xml_status_for_option($hardening_option)
# @brief   Get status of the Enhanced hardening option (on or off)
# @details This method reads Hardening XML storage and returns a value of the
#          attribute 'applied ' for the provided Enhanced hardening option
# @param   $hardening_option  - Name of Hardening option in XML file
# @return  The current status (on or off)
# @return  undef  - some error happens
sub get_xml_status_for_option($)
{
   my $hardening_option = shift @_;

   if (not defined $hardening_option)
   {
      baselog($LOG_ERROR, "Name of Hardening option is not defined.");
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to get the status for the option '$hardening_option'.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return undef;
   }

   my $hash_ref = $parser->getAttrs(qq{/$HARDENING_ROOT_ELEM/$hardening_option});
   if (not defined $hash_ref)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the option '$hardening_option'.");
      undef $parser;
      return undef;
   } ## end if (not defined $hash_ref...
   undef $parser;

   my %tmphash = %{$hash_ref};

   if (not exists($tmphash{$HARDENING_STATUS_ATTR}))
   {
      baselog($LOG_ERROR, "The option '$hardening_option' does not contain the attribute '$HARDENING_STATUS_ATTR'.");
      return undef;
   }

   baselog($LOG_DEBUG, "The status for the option '$hardening_option' is '$tmphash{$HARDENING_STATUS_ATTR}'.");

   return $tmphash{$HARDENING_STATUS_ATTR};
} ## end sub get_xml_status_for_option($)

###############################################################################
## @fn sub set_xml_status_for_option($hardening_option, $status)
# @brief   Set status of the Enhanced hardening option (on or off)
# @details This method writes new value for the attribute 'applied' of the
#          provided Enhanced hardening option in the Hardening XML storage.
# @param   $hardening_option - Name of Hardening option in XML file
# @param   $status           - new status (on/off)
# @return  TRUE  - The status is changed
# @return  FALSE - some issue happens
sub set_xml_status_for_option($$)
{
   my ($hardening_option, $status) = @_;

   my $retval = $FALSE;

   if (not defined $hardening_option)
   {
      baselog($LOG_ERROR, "Name of Hardening option is not defined.");
      return $FALSE;
   }

   if ((not defined $status) or ($status !~ /^(on|off)$/))
   {
      baselog($LOG_ERROR, "Status is not valid.");
      return $FALSE;
   }

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }

   if (not $parser->changeAttrs(qq{/$HARDENING_ROOT_ELEM/$hardening_option}, undef, {$HARDENING_STATUS_ATTR => $status}))
   {
      baselog($LOG_ERROR, "Cannot set new status for '$hardening_option'.");
   }
   else
   {
      save_hardening_storage($parser);
      $retval = $TRUE;
   }

   undef $parser;
   return $retval;
} ## end sub set_xml_status_for_option($$)

###############################################################################
## @fn sub is_hardening_option_valid($option)
# @brief   Check if a name of Hardening option is valid
# @details This subroutine uses the Hardening Manager in order to get
#          information if the name of the provided Hardening option matches
#          with an existent Enhanced Hardening module.
# @param   $option - name of an Enhanced hardening option
# @return  TRUE  - the name of the option and the Enhanced Hardening module
#                  are matched.
# @return  FALSE - either the name is not valid or some error happens
sub is_hardening_option_valid($)
{
   my $option = shift @_;

   if ((not defined $option) or ($option eq ""))
   {
      baselog($LOG_ERROR, "Enhanced Hardening option name is not valid.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Is the Hardening option '$option' supported?");

   # create plugin_mrg object
   my $harden_mgr = plugin_mgr->new($PATH_TO_ENHANCED_HARDENING_MODULES);
   if (not defined $harden_mgr)
   {
      baselog($LOG_ERROR, "Cannot create Hardening Manager.");
      return $FALSE;
   }

   my $retval = $FALSE;

   if (not $harden_mgr->module_exists($option))
   {
      baselog($LOG_ERROR, "Invalid name of Hardening option '$option'");
   }
   else
   {
      $retval = $TRUE;
   }

   undef $harden_mgr;

   return $retval;
} ## end sub is_hardening_option_valid($)

###############################################################################
## @fn sub create_hardening_storage()
# @brief   Prepare the basic XML structure for new Hardening storage
# @details This subroutine creates an instance of base_xml class in order to
#          create the basic XML structure of new Hardening storage.
#          A reference to the instance is returned.
# @return  $parser - reference to the instance of base_xml class
# @return  undef   - some error happens
sub create_hardening_storage()
{
   baselog($LOG_DEBUG, "Trying to initialize the Hardening XML storage.");

   my $parser = base_xml->new(undef, $HARDENING_STORAGE_TMP);
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot create Hardening XML storage.");
      return undef;
   }

   #add root element
   if (not $parser->addElements("/", undef, $HARDENING_ROOT_ELEM, {$HARDENING_VERSION_ATTR => $HARDENING_XML_VERSION}))
   {
      baselog($LOG_ERROR, "Cannot add the root element '$HARDENING_ROOT_ELEM' to the Hardening XML storage.");
      undef $parser;
      return undef;
   } ## end if (not $parser->addElements...

   return $parser;
} ## end sub create_hardening_storage()

###############################################################################
## @fn sub open_hardening_storage()
# @brief   Open the Hardening storage
# @details This subroutine creates an instance of base_xml class with the
#          Hardening storage and checks if the storage has proper structure
# @return  $parser - reference to the instance of base_xml class
# @return  undef   - some error happens
sub open_hardening_storage()
{
   baselog($LOG_DEBUG, "Trying to open the Hardening XML storage.");

   #pass into constructor two files
   #the first file - the current hardening storage
   #the second file - a temporary storage which is used for more safely data saving
   my $parser;

   if (is_recovery_or_init_stage())
   {
      $parser = base_xml->new($HARDENING_STORAGE, $HARDENING_STORAGE_TMP, {$FILE_DOES_NOT_EXIST => \&handle_storage_does_not_exist, $ELEMENT_DOES_NOT_EXIST => \&handle_option_does_not_exist});
   }
   else
   {
      $parser = base_xml->new($HARDENING_STORAGE, $HARDENING_STORAGE_TMP);
   }
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return undef;
   }

   #Check if the option exists in XML file
   my $tmp = $parser->element_exists(qq{/$HARDENING_ROOT_ELEM});
   if (not($tmp > 0))
   {
      baselog($LOG_ERROR, "The XML file does not contain Hardening data.");
      undef $parser;
      return undef;
   } ## end if (not($tmp > 0))

   #Get all the attributes of the tag
   my $hash_ref = $parser->getAttrs(qq{/$HARDENING_ROOT_ELEM});
   if (not defined $hash_ref)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the tag '$HARDENING_ROOT_ELEM'.");
      undef $parser;
      return undef;
   } ## end if (not defined $hash_ref...

   my %tmphash = %{$hash_ref};

   #check 'version' of the XML data
   if ((not exists $tmphash{$HARDENING_VERSION_ATTR}) or ($tmphash{$HARDENING_VERSION_ATTR} ne $HARDENING_XML_VERSION))
   {
      baselog($LOG_WARNING, "The Hardening XML storage has wrong version.");
   }

   return $parser;
} ## end sub open_hardening_storage()

###############################################################################
## @fn sub save_hardening_storage($parser)
# @brief   Save the Hardening storage
# @details This subroutine saves content of instance of base_xml class and
#          changes the permissions to this file.
# @param   $parser - reference to the instance of base_xml class
# @return  TRUE  - data are saved
# @return  FALSE - some error happens
sub save_hardening_storage($)
{
   my $parser = shift @_;

   if (not defined $parser)
   {
      baselog($LOG_ERROR, "The storage is undefined.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to save the Hardening XML storage.");

   my $retval = $FALSE;

   my $out_file = $parser->getOutFileName();

   #try to save data
   if ($parser->save())
   {

      #try to change permissions and replace the old hardening storage by the
      #temporary file
      if (    (change_permissions_and_ownership($HARDENING_STORAGE_PERMISSIONS, $HARDENING_STORAGE_OWNER_UID, $HARDENING_STORAGE_OWNER_GID, [$out_file], undef, $NO_LOG))
          and (do_system([$MV, "-f", $out_file, $HARDENING_STORAGE])))
      {
         baselog($LOG_DEBUG, "The Hardening XML storage is saved successfully.");
         $retval = $TRUE;
      } ## end if ((change_permissions_and_ownership...
   } ## end if ($parser->save())

   if (not $retval)
   {
      baselog($LOG_ERROR, "Cannot save the Hardening XML storage.");
   }

   return $retval;
} ## end sub save_hardening_storage($)

###############################################################################
## @fn sub set_new_hardening_status($option, $newstatus, $sub_change_status, $invocation_mode)
# @brief   Change status of Hardening option
# @details This subroutine reads the Hardening storage (creates it if it is
#          not exist), performs actual changes on OS-level for the Hardening
#          option by invoking the provided callback subroutine if it is
#          necessary and save changed XML data.
# @param   $option            - name of an Enhanced hardening option
# @param   $newstatus         - new value of the status
# @param   $sub_change_status - (optional) reference to the callback subroutine
#                               which performs actual changes on OS-level.
# @param   $invocation_mode   - (optional) this flag controls subroutine
#                               invocation:
#                          \li  0 - the subroutine is invoked prior to XML
#                                   changing, XML is changed only if the
#                                   subroutine returns TRUE
#                          \li  1 - the subroutine is invoked prior to XML
#                                   changing, XML is changed even if the
#                                   subroutine returns FALSE
#                          \li  2 - the subroutine is invoked after XML
#                                   changing, only if XML was changed
#                                   successfully.
# @return  TRUE    - the changes are perfromed successfully
# @return  FALSE   - either the changes are unnecessary or the callback
#                    subroutine returns FALSE
# @return  H_ERROR - some error happens
sub set_new_hardening_status($$;$$)
{
   my ($option, $newstatus, $sub_change_status, $invocation_mode) = @_;

   baselog($LOG_DEBUG, "Trying to change status of an Enhanced Hardening option.");

   if ((not defined $option) or (not is_hardening_option_valid($option)))
   {
      baselog($LOG_ERROR, "Unknown Enhanced Hardening option.");
      return $H_ERROR;
   }

   if ((not defined $newstatus) or ($newstatus eq "") or ($newstatus !~ /^(on|off)$/))
   {
      baselog($LOG_ERROR, "New status is not specified.");
      return $H_ERROR;
   }

   if (not defined $invocation_mode)
   {
      $invocation_mode = 0;
   }

   baselog($LOG_DEBUG, "The option: '$option', the new status: '$newstatus'.");

   my $parser;
   my $XPath_root = qq{/$HARDENING_ROOT_ELEM};
   my $XPath      = qq{/$HARDENING_ROOT_ELEM/$option};

   #Hardening storage does not exists
   if (not -e $HARDENING_STORAGE)
   {

      #prepare new XML document
      baselog($LOG_INFO, "The Hardening XML storage does not exist.");
      $parser = create_hardening_storage();
      if (not defined $parser)
      {
         return $H_ERROR;
      }

      #add new tag describing the option
      if (not $parser->addElements($XPath_root, undef, $option, {$HARDENING_STATUS_ATTR => ""}))
      {
         baselog($LOG_ERROR, "Cannot add the element '$option' to the Hardening XML storage.");
         undef $parser;
         return $H_ERROR;
      } ## end if (not $parser->addElements...
   } ## end if (not -e $HARDENING_STORAGE...
   else
   {

      #Try to get the current XML status of the option
      #open storage
      $parser = open_hardening_storage();
      if (not defined $parser)
      {
         return $H_ERROR;
      }

      #Check if the option exists in XML file
      my $tmp = $parser->element_exists($XPath);
      if ($tmp < 0)
      {
         baselog($LOG_ERROR, "Cannot get attributes for the option '$XPath'.");
         undef $parser;
         return $H_ERROR;
      } ## end if ($tmp < 0)

      #no such tag in the XML structure
      if ($tmp == 0)
      {

         #add the tag in XML
         baselog($LOG_INFO, "The Hardening XML storage does not contain configuration for the option '$option'.");
         if (not $parser->addElements($XPath_root, undef, $option, {$HARDENING_STATUS_ATTR => ""}))
         {
            baselog($LOG_ERROR, "Cannot add the element '$option' to the Hardening XML storage.");
            undef $parser;
            return $H_ERROR;
         } ## end if (not $parser->addElements...
      } ## end if ($tmp == 0)
      else
      {

         #Get all the attributes of the tag
         my $hash_ref = $parser->getAttrs($XPath);
         if (not defined $hash_ref)
         {
            baselog($LOG_ERROR, "Cannot get attributes for the option '$option'.");
            undef $parser;
            return $H_ERROR;
         } ## end if (not defined $hash_ref...

         my %tmphash = %{$hash_ref};

         #if there is no such attribute in the tag
         if (not exists $tmphash{$HARDENING_STATUS_ATTR})
         {

            #add the attribute
            baselog($LOG_INFO, "The Enhanced Hardening option '$option' does not contain the attribute '$HARDENING_STATUS_ATTR'.");
            if (not $parser->changeAttrs($XPath, undef, {$HARDENING_STATUS_ATTR => ""}))
            {
               baselog($LOG_ERROR, "Cannot add the attribute '$HARDENING_STATUS_ATTR' to the element '$XPath'.");
               undef $parser;
               return $H_ERROR;
            } ## end if (not $parser->changeAttrs...
         } ## end if (not exists $tmphash...
         else
         {
            if ($tmphash{$HARDENING_STATUS_ATTR} eq $newstatus)
            {
               baselog($LOG_NOTICE, "The Enhanced Hardening option '$option' is already in the status '$newstatus'.\n", $TRUE);
               undef $parser;
               return $FALSE;
            } ## end if ($tmphash{$HARDENING_STATUS_ATTR...
         } ## end else [ if (not exists $tmphash...
      } ## end else [ if ($tmp == 0)
   } ## end else [ if (not -e $HARDENING_STORAGE...

   my $retval = $TRUE;

   if ((not $invocation_mode) or ($invocation_mode == $FORCE_TO_CHANGE_STATUS))
   {

      #perform actual status changing
      if (defined $sub_change_status)
      {
         baselog($LOG_DEBUG, "Performing actual changes...");
         $retval = invoke_subroutine_safely($sub_change_status, undef);
      }
      else
      {
         baselog($LOG_INFO, "No OS-level changes will be performed.");
      }
   } ## end if ((not $invocation_mode...

   #Change the status in XML file and save the file
   if (($invocation_mode == $FORCE_TO_CHANGE_STATUS) or ($retval))
   {

      #add the attribute
      if (not $parser->changeAttrs($XPath, undef, {$HARDENING_STATUS_ATTR => $newstatus}))
      {
         baselog($LOG_ERROR, "Cannot add the attribute '$HARDENING_STATUS_ATTR' to the element '$XPath'.");
         undef $parser;
         return $H_ERROR;
      } ## end if (not $parser->changeAttrs...
      baselog($LOG_INFO, "The Enhanced Hardening option '$option' is set to new status '$newstatus'.");
      $retval = $TRUE;

      if (not save_hardening_storage($parser))
      {
         $retval = $H_ERROR;
      }
   } ## end if (($invocation_mode ...

   undef $parser;

   if (($retval) and ($invocation_mode == $CHANGE_SYSTEM_AFTER_XML))
   {

      #perform actual status changing
      if (defined $sub_change_status)
      {
         baselog($LOG_DEBUG, "Performing actual changes...");
         $retval = invoke_subroutine_safely($sub_change_status, undef);

         #TODO: Incase of FALSE it is necessary to restore the previous value in
         #XML or default one. Not realized due to time constraints.
      } ## end if (defined $sub_change_status...
   } ## end if (($retval) and ($invocation_mode...

   return $retval;
} ## end sub set_new_hardening_status($$;$$)

###############################################################################
## @fn sub show_post_installation_banner()
# @brief   Shows Post-Installation Hardening banner
# @details This subroutine dispays the Post-Installation Hardening banner which
#          consists of a header and data from Enhanced Hardening modules
#          accessible on Post-Installation stage (i.e. applications owned
#          modules are not accessible at this moment). In order to get statuses
#          from the modules, they are asked for performing action $H_IF_POST_LB_STATUS
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return  a result of invocation do_enhanced_hardening_action (see description
#          of this subroutine)
sub show_post_installation_banner()
{
   baselog($LOG_DEBUG, "Try to output Post-Installation Hardening banner.");

   print "\r
##################################################################\r
#                Status of Linux Hardening items                 #\r
##################################################################\r\n";

   my $ret = do_enhanced_hardening_action($H_IF_POST_LB_STATUS);

   return $ret;
} ## end sub show_post_installation_banner()

###############################################################################
## @fn sub prepare_module_status_for_banner($module_name, $status_line)
# @brief   Pepare a formatted string for Post-Installation Banner
# @details This subroutine prepares a formatted string containing status of
#          Hardening module. This string is intended to be included in the
#          Post-Installation Hardening banner.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @param   $module_name - name of an Enhanced hardening module
# @param   $status_line - line describing status of the module configuration
# @return  formatted string
sub prepare_module_status_for_banner($$)
{
   my ($module_name, $status_line) = @_;

   if ((not defined $module_name) or (not is_hardening_option_valid($module_name)))
   {
      baselog($LOG_ERROR, "Unknown Enhanced Hardening module.");
      $module_name = "(unknown)";
   }

   if ((not defined $status_line) or ($status_line eq ""))
   {
      baselog($LOG_ERROR, "Status line is not defined.");
      $status_line = "";
   }

   return sprintf("%-12s", $module_name) . ": " . $status_line . "\r\n";
} ## end sub prepare_module_status_for_banner($$)

###############################################################################
## @mfn open_telnet_firewall()
# @brief    It opens TELNET firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub open_telnet_firewall()
{
   my $retval = $FALSE;

   if (!enable_service_firewall($TELNET))
   {
      baselog($LOG_ERROR, 'Cannot enable firewall settings for TELNET service.');
   }
   else
   {
      baselog($LOG_INFO, 'TELNET service is enabled.');
      $retval = $TRUE;
   }

   return $retval;
} ## end sub open_telnet_firewall()

###############################################################################
## @fn  close_telnet_firewall()
# @brief    It closes TELNET firewall ports
# @details  It changes xml file port settings. Firewall restart requires to
#           changes take effect.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens

sub close_telnet_firewall()
{
   my $retval = $TRUE;

   if (disable_service_firewall($TELNET))
   {
      baselog($LOG_DEBUG, 'TELNET firewall disabled.');
   }
   else
   {
      baselog($LOG_DEBUG, 'Cannot disable TELNET firewall.');
      $retval = $FALSE;
   }

   return $retval;
} ## end sub close_telnet_firewall()

###############################################################################
## @fn  enable_service_firewall($service)
# @brief    Change the dynamic basefirewall configuration file to enable rule
#           for the service. If configuration file does not exist then create
#           default one.
# @param    $service  - Service to enable.
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.

sub enable_service_firewall($)
{
   my $service = shift @_;
   my $retval  = $TRUE;

   if (!-e $FIREWALL_RULES_FILE)
   {
      unless (open(CFG, '>', $FIREWALL_RULES_FILE))
      {
         baselog($LOG_ERROR, "Cannot create firewall rules ('$FIREWALL_RULES_FILE').");
         return $FALSE;
      }
      print CFG <<HERE;
<iptables>
<firewall>
<ftp>off</ftp>
<tftp>off</tftp>
<telnet>off</telnet>
</firewall>
</iptables>
HERE
      if (!close(CFG))
      {
         baselog($LOG_ERROR, "Cannot close '$FIREWALL_RULES_FILE'.");
         return $FALSE;
      }
   } ## end if (!-e $FIREWALL_RULES_FILE...
   $retval = change_firewall_port_status($service, 'on');

   return $retval;
} ## end sub enable_service_firewall($)

###############################################################################
## @fn  disable_service_firewall($service)
# @brief    Change the dynamic basefirewall configuration file to disable rule for
#           the service.
# @details  If there is no configuration file the service is disabled by default.
# @param    $service  - Service to disable.
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.
# @author   Kate Bereznaykova

sub disable_service_firewall($)
{
   my $service = shift @_;
   my $retval  = $TRUE;
   if (!-e $FIREWALL_RULES_FILE)
   {
      return $TRUE;
   }

   return change_firewall_port_status($service, 'off');
} ## end sub disable_service_firewall($)

###############################################################################
## @fn  change_firewall_port_status($service, $status)
# @brief    It updates FTP service status in configuration file
# @param    $service  - Service to change the status.
# @param    $status - new status
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.

sub change_firewall_port_status($$)
{
   my ($service, $status) = @_;
   my $retval = $TRUE;

   if ($service !~ /^($FTP|$TFTP|$TELNET)$/)
   {
      baselog($LOG_ERROR, "Unknown service ('$service').");
      return $FALSE;
   }

   my $doc = base_xml->new($FIREWALL_RULES_FILE);
   unless ($doc)
   {
      baselog($LOG_ERROR, 'Cannot create base_xml object.');
      return $FALSE;
   }

   if ($doc->changeText($service, undef, $status))
   {
      if ($doc->save())
      {
         baselog($LOG_INFO, "'$service' section in '$FIREWALL_RULES_FILE' file has been changed to '$status' successfully.");
      }
      else
      {
         baselog($LOG_ERROR, "Cannot save changes ('$status') in '$service' section at '$FIREWALL_RULES_FILE' file");
         $retval = $FALSE;
      }
   } ## end if ($doc->changeText($service...
   undef $doc;
   return $retval;
} ## end sub change_firewall_port_status($$)

###############################################################################
## @fn  handle_storage_does_not_exist($parser)
# @brief    It creates hardening XML storage structure
# @param    $parser - XML parser
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.

sub handle_storage_does_not_exist($)
{
   my $xml_parser = shift @_;

   my $parser = create_hardening_storage();
   if (not defined $parser)
   {
      return $H_ERROR;
   }

   $$xml_parser = $parser->getReferenceToXMLParser();

   undef $parser;

   return $TRUE;
} ## end sub handle_storage_does_not_exist($)

###############################################################################
## @fn  handle_option_does_not_exist($basexml, $XPath, $elements)
# @brief    It creates appropriate tags in hardening XML storage
# @param    $parser - XML parser
# @param    $XPath - path
# @param    $elements - elements
# @return   $TRUE  - operation successful.
# @return   $FALSE - some error happens.

sub handle_option_does_not_exist($$$)
{
   my ($basexml, $XPath, $elements) = @_;

   if (not $basexml)
   {
      baselog($LOG_ERROR, "basexml parser is not specified");
      return $H_ERROR;
   }
   if (not $XPath)
   {
      baselog($LOG_ERROR, "XPath is not specified");
      return $H_ERROR;
   }
   if (not $elements)
   {
      baselog($LOG_ERROR, "elements are not specified");
      return $H_ERROR;
   }

   my $option = "";

   #get the last entry in XPath
   $XPath =~ m/.*\/(.*)/;
   if ($1)
   {
      $option = $1;
   }
   else
   {
      baselog($LOG_ERROR, "Incorrect XPath provided");
      return $H_ERROR;
   }

   #add new tag describing the option
   if (not $basexml->addElements("/$HARDENING_ROOT_ELEM", undef, $option, {$HARDENING_STATUS_ATTR => ""}))
   {
      baselog($LOG_ERROR, "Cannot add the element '$option' to the Hardening XML storage.");
      return $H_ERROR;
   }

   $$elements = $basexml->findElements($XPath);

   return $TRUE;
} ## end sub handle_option_does_not_exist($$$)

###############################################################################
## @fn     is_recovery_or_init_stage()
# @brief   Determine if recovery or init stage is used
# @return  TRUE  - recovery/init stage
# @return  FALSE - not recover/init stage
sub is_recovery_or_init_stage()
{
   my $retval = $FALSE;

   if ($CURRENT_HARDENING_ACTION eq $H_IF_INIT)
   {
      return $TRUE;
   }
   if (($CURRENT_HARDENING_ACTION eq $H_IF_POST_LB_RECOVERY) or ($CURRENT_HARDENING_ACTION eq $H_IF_RECOVERY))
   {
      baselog($LOG_DEBUG, "The recovery stage for hardening is recognized");

      # to determine if valid file is used
      $retval = is_special_restore_case();
   } ## end if (($CURRENT_HARDENING_ACTION...

   return $retval;
} ## end sub is_recovery_or_init_stage()

###############################################################################
## @fn     is_special_restore_case()
# @brief   Determine if backup cookie file from 5.0 or 5.5 release
# @details The situation when a backup archive doesn't contain the Hardening
#          storage is valid for these releases, so,
#          special handlers for absence of this file is not needed, except
#          when fresh installation is performed.
# @return  TRUE  - file from 5.0 or 5.5 release or 6.0 and backup is not used
# @return  FALSE - backup file from 6.0 and it is used

sub is_special_restore_case()
{
   my $retval      = $FALSE;
   my $cookie_file = $backup_cookie;
   my ($version, $fd);
   if (not -e $cookie_file)
   {
      baselog($LOG_DEBUG, "No such file $cookie_file.");
      return $TRUE;
   }
   if (not -r $cookie_file)
   {
      baselog($LOG_ERROR, "Cannot read the file $cookie_file.");
      return $retval;
   }
   if (not open($fd, "<$cookie_file"))
   {
      baselog($LOG_ERROR, "Cannot open the file $cookie_file.");
      return $retval;
   }
   while (<$fd>)
   {
      next if (/^\s*\#/);  # skip comments
      if (/^\s*baseversion\s*=\s*(.+)/)
      {
         $version = $1;
         last;
      }
   } ## end while (<$fd>)
   if (not close($fd))
   {
      baselog($LOG_ERROR, "Cannot close the file $cookie_file.");
   }
   if (defined $version)
   {

      #check version in backup cookie and check base recovery flag file for existing
      #if recovery flag does not exist (fresh installation) and it is LinuxBase post install recovery stage - error messages are disabled
      if ($version =~ /^(5.0\d|5.5\d)/)
      {
         $retval = $TRUE;
      }
      else
      {
         if ((!-e $BASERECOVERY_FLAG) and ($CURRENT_HARDENING_ACTION eq $H_IF_POST_LB_RECOVERY))
         {
            $retval = $TRUE;
         }
      } ## end else [ if ($version =~ /^(5.0\d|5.5\d)/...
      baselog($LOG_DEBUG, "Backup version is defined as '$version'");

   } ## end if (defined $version)
   return $retval;

} ## end sub is_special_restore_case()

###############################################################################
## @fn sub hardening_special_baselog($$;$)
# @brief   Determine to invoke baselog function on recovery stage
# @param   $level  - log level
# @param   $msg  - message
# @param   $on_term - TRUE or FALSE
# @return  TRUE  - baselog functions returns TRUE
# @return  FALSE - baselog functions returns FALSE

sub hardening_special_baselog($$;$)
{
   my ($level, $msg, $on_term) = @_;

   if (not is_recovery_or_init_stage())
   {
      baselog($level, $msg, $on_term);
   }
   else
   {
      baselog($LOG_INFO, $msg, $on_term);
   }
} ## end sub hardening_special_baselog($$;$)

1;
__END__
