###############################################################################
## @file audit.pm
# @brief Provides hardening actions for manage the Linux Audit daemon
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against the Linux Audit daemon and the Enhanced
#          Hardening module 'audit' are requested.
#
#\li          It is responsible for the CLI actions:
#\li          harden audit
#\li          harden audit on
#\li          harden audit off
#\li          harden audit status
#\li          harden audit help
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before common functions and variables
# @version 1.02 - audit rules are improved, the issues with logrotation and
#                 audit disabling ares fixed
# @version 1.03 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage

=pod

=head1 NAME

=over 4

=item B<harden audit> - command to manage the Linux Audit daemon.

=back

=head1 SYNOPSIS

=over 10

=item B<harden audit> I<on> | I<off> | I<status> | I<help>

=back

=head1 DESCRIPTION

B<harden audit> module allows managing the Linux Audit daemon.

=head1 OPTIONS

=over 4

=item B<on>

Applies default configuration of the Linux Audit daemon and run it. The SNMP
trap is sent to track this event by SNMP manager.

=item B<off>

Turns off (stops) the Linux Audit daemon. The SNMP trap is sent to track this
event by SNMP manager.

=item B<status>

Outputs the actual status whether the Linux Audit daemon is running or not.

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden audit> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden audit on>

 The Linux Audit daemon is turned on.

=item B<$ harden audit off>

 The Linux Audit daemon is turned off.

=item B<$ harden audit status>

 The Linux Audit daemon is running.

=back

=head1 DEFAULT CONFIGURATION

Every time when B<harden audit on> is invoked it re-writes the following files
F</etc/auditd.conf> and F</etc/audit.rules> in order to apply recommended
configuration rules.

=head1 AUDIT LOG FILES

The Linux Audit daemon puts its audit log files into the F</val/log/audit>
directory. The current working log file is F</val/log/audit/audit.log>. Old
log files have names like F</val/log/audit/audit.log.XX>, where I<XX> is
digits - the most new files will have the lowest numbers.

=head1 LOG ROTATION

Every time when B<harden audit on> is invoked it re-applies
F</etc/cron.daily/audit> file. This file allows enabling daily rotation for the
Linux Audit log files.

The Linux Audit log files will be rotated in accordance to the following rules:

=over 4

1. When the Linux Audit detects that its working log file has reached the max
file size limit, it will invoke the log rotation routine.

2. The audit's log rotation job performed by cron will immediately rotate the
logs even the max file size limit has not been reached.

=back

B<NOTE>: The Linux Audit daemon is configured to not overwrite old log files,
so, the System Administrator is responsible for backing up log files from the
Audit's logs directory and for timely clearing of this directory.

=head1 CAVEATS

B<harden audit> must be run only under a high privileges account.

=head1 SEE ALSO

B<auditd>, B<auditd.conf>, B<auditctl>, B<aureport>, B<ausearch>

=cut

## @class audit
package hardening::plugins::audit;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";
use common_variables;
use common_functions;
use base_system;
use baselog;
use CommonInstallUtils;
use hardening::hardening_common;
use base_snmp;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);

our $VERSION = '1.03';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub start();
## @skipline
sub stop();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_audit_state();
## @skipline
sub recovery();

#variables
my $auditd      = "auditd";
my $module_name = "audit";

my $conf_file  = "/etc/audit/auditd.conf";
my $rules_file = "/etc/audit/audit.rules";
my $cronjob    = "/etc/cron.daily/audit";

###############################################################################
# Content of file with audit rules
###############################################################################
my $rules_file_data = q{
# This file contains the auditctl rules that are loaded
# whenever the audit daemon is started via the initscripts.

# First rule - delete all
-D

# Feel free to add below this line. See auditctl man page

# Increase the buffers to survive stress events
-b 320

# audit failed attempts to access files and programs, except attempts to open
# non-existent file
-a exit,always -S open -F success!=1 -F exit!=-2

# audit files and programs deleted by the user
-a exit,always -S unlink -S rmdir

# audit all administrative, privileged, and security actions
-a exit,always -S stime -S acct -S reboot -S swapon -S settimeofday -S setrlimit -S setdomainname -S sched_setparam
# special treatment for attempts to invoke sched_setscheduler with invalid
# parameters (issue found in jboss process)
-a exit,always -S sched_setscheduler -F exit!=-22
-w } . $conf_file . q{ -k fk_auconf -p rwa
-w } . $rules_file . q{ -k fk_auruls -p rwa

# audit all discretionary access control permission modifications
-a exit,always -S chmod -S fchmod -S chown -S chown32 -S fchown -S fchown32 -S lchown -S lchown32

-e 1
};

###############################################################################
# Content of configuration file
###############################################################################
my $conf_file_data = q{
#
# This file controls the configuration of the audit daemon
#

log_file = /var/log/audit/audit.log
log_format = RAW
priority_boost = 3
flush = INCREMENTAL
freq = 20
# num_logs specifies the number of log files to keep. This number must be 99
# or less. If this options is not specified directly, it takes the default
# value (0) - which means no rotation.
num_logs = 99
max_log_file = 10
max_log_file_action = KEEP_LOGS
space_left = 75
space_left_action = SYSLOG
action_mail_acct = root
admin_space_left = 50
admin_space_left_action = SUSPEND
disk_full_action = SUSPEND
disk_error_action = SUSPEND
};

###############################################################################
# Content of log rotate cron job
###############################################################################
my $cronjob_data = q{
#!/bin/sh

##########
# This script can be installed to get a daily log rotation
# based on a cron job.
##########

/sbin/service auditd rotate
EXITVALUE=$?
if [ $EXITVALUE != 0 ]; then
    /usr/bin/logger -t auditd "ALERT exited abnormally with [$EXITVALUE]"
fi
exit 0
};

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_audit_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method ouptuts the actual status of the Linux Audit daemon.
# @return TRUE - if the status is got.
# @return FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status the Linux Audit daemon...");

   my $run_message = "The Linux Audit daemon is ";

   #get info from XML file
   my $hard_status = is_option_permitted($module_name);

   #get actual state of the Audit daemon
   my $run_status = get_status_linux_service($auditd, $NOT_LSB_SERVICE);

   my $retval = $TRUE;

   if ($hard_status == $H_ERROR)
   {
      baselog($LOG_ERROR, "Hardening status of 'audit' option is unknown.\n", $TRUE);
      $retval = $FALSE;
   }

   if ($run_status == $H_ERROR)
   {
      baselog($LOG_ERROR, "Status of the Linux Audit daemon is unknown.\n", $TRUE);
      $retval = $FALSE;
   }
   elsif ($run_status == $TRUE)
   {
      if ($hard_status == $FALSE)
      {
         baselog($LOG_WARNING, $run_message . "turned off but actually running.\n", $TRUE);
      }
      else
      {
         baselog($LOG_INFO, $run_message . "running.\n", $TRUE);
      }
   } ## end elsif ($run_status == $TRUE...
   else
   {
      if ($hard_status == $TRUE)
      {
         baselog($LOG_WARNING, $run_message . "turned on but actually stopped.\n", $TRUE);
      }
      else
      {
         baselog($LOG_INFO, $run_message . "stopped.\n", $TRUE);
      }
   } ## end else [ if ($run_status == $H_ERROR...

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parses POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub start()
# @brief Runs the Linux Audit daemon
# @details This method applies default configuration option and daily cron job,
#          then it runs the Linux Audit daemon.
# @return TRUE - the service is running sucessfully
# @return FALSE - some error happens
sub start()
{
   baselog($LOG_DEBUG, "Re-applying default data...");

   #new audit rules
   #made the rules file operable only for root
   if (not change_permissions_and_ownership(0600, $ROOT_UID, $ROOT_GID, [$rules_file], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the file '$rules_file'");
      return $FALSE;
   }

   #re-apply new content
   if (not write_harden_file($rules_file, $rules_file_data))
   {
      baselog($LOG_ERROR, "Cannot re-apply default the Linux Audit rules.");
      return $FALSE;
   }

   #new audit configuration
   #made the rules file operable only for root
   if (not change_permissions_and_ownership(0600, $ROOT_UID, $ROOT_GID, [$conf_file], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the file '$conf_file'");
      return $FALSE;
   }

   #re-apply new content
   if (not write_harden_file($conf_file, $conf_file_data))
   {
      baselog($LOG_ERROR, "Cannot re-apply default the Linux Audit configuration.");
      return $FALSE;
   }

   #daily cron job - apply content
   #Since the file does not exist in normal case. It is created before
   #permissions changing
   if (not write_harden_file($cronjob, $cronjob_data, $NO_BACKUP))
   {
      baselog($LOG_ERROR, "Cannot register new daily cron job for the Linux Audit log rotation.");
      return $FALSE;
   }

   #made the job executable
   if (not change_permissions_and_ownership(0700, $ROOT_UID, $ROOT_GID, [$cronjob], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the file '$cronjob'");
      do_system([$RM, "-f", $cronjob]);
      return $FALSE;
   } ## end if (not change_permissions_and_ownership...

   baselog($LOG_DEBUG, "Starting the Linux Audit daemon...");

   #Try to start the service
   if (not start_and_enable_linux_service($auditd))
   {
      baselog($LOG_ERROR, "Cannot start the Linux Audit daemon");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "The Linux Audit daemon is started.");

   return $TRUE;
} ## end sub start()

###############################################################################
## @fn sub stop()
# @brief Stops the Linux Audit daemon
# @details This method stops the Linux Audit daemon and removes daily cron job.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub stop()
{
   baselog($LOG_DEBUG, "Stopping the Linux Audit daemon...");

   #1. In order to be sure that the service will not be started automaticaly
   #during Linux bootup sequence, the service is disabled and stopped.
   #2. Due to a bug in RedHat's audit init script we need to turn off audit
   #explicitly.
   if ((not stop_and_disable_linux_service($auditd)) or (not do_system([$AUDITCTL, "-e", "0"])))
   {
      baselog($LOG_ERROR, "Cannot stop the Linux Audit daemon");
      return $FALSE;
   }

   do_system([$RM, "-f", $cronjob]);

   baselog($LOG_DEBUG, "The Linux Audit daemon service is stopped.");

   return $TRUE;
} ## end sub stop()

###############################################################################
## @fn sub off()
# @brief Turns off the Linux Audit daemon
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, and stops the Linux Audit daemon.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to turn off the Linux Audit daemon...");

   my $retval = $FALSE;

   if (is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'Audit daemon' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (is_option_permitted...

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&stop, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The Linux Audit daemon is turned off.\n", $TRUE);

      send_snmp_trap("SEC0067");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn off the Linux Audit daemon.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief Turns on the Linux Audit daemon
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, configures and runs the Linux Audit daemon. It sends
#          the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to turn on the Linux Audit daemon...");

   my $retval = $FALSE;

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&start);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The Linux Audit daemon is turned on.\n", $TRUE);

      send_snmp_trap("SEC0066");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn on the Linux Audit daemon.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief Apply default value of the Hardening option relating to the Linux
#        Audit daemon
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it turn off the Linux Audit daemon due to
#          possible performance impact.
#          Default value is applied in the case when it is impossible to detect
#          the current audit usage policy. Assuming that this is the cases when
#          either the system is fresh installed or there are no restored
#          Hardening data, usage of the Linux Audit daemon is turned off
#          explicitly.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to turn off the Linux Audit daemon...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   #Default value is applied in the case when it is impossible to detect the
   #current audit usage policy. Assuming that this is the cases when either the
   #system is fresh installed or there are no restored Hardening data, usage of
   #the Linux Audit daemon is turned off explicitly.
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&stop, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The Linux Audit daemon is turned off.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn off the Linux Audit daemon.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of Linux Audit daemon is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "The Linux Audit daemon is enabled.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "The Linux Audit daemon is disabled.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of the Linux Audit daemon should be turned on or not. First,
#          the case when there is some problem with the Hardening storage -
#          default state of the option is applied. Otherwise, the system is
#          changed in accordance to the state from the Hardening storage.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of enabling of the Linux Audit daemon from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of enabling of the Linux Audit daemon is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {
      $retval = start();
   }
   else
   {
      $retval = stop();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_audit_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          the Linux Audit daemon is turned on or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_audit_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of enabling of the Linux Audit daemon.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of enabling of the Linux Audit daemon is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0066");
   }
   else
   {
      $retval = send_snmp_trap("SEC0067");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_audit_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if the Linux Audit daemon should be turned on or not, and performs
#          all necessary actions to reflect the new state on the system (e.g.
#          send the corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of the Linux Audit daemon...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_audit_state();

   return $retval;
} ## end sub recovery()

1;
__END__
