###############################################################################
## @file banners.pm
# @brief Provides hardening actions for CS1000 pre-login banners
#        changing
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 pre-login banners
#          and the Enhanced  Hardening module 'banners' are requested.
#
# \li          It is responsible for the CLI actions:
# \li          harden banners
# \li          harden banners on
# \li          harden banners off
# \li          harden banners set -path path_to_file
# \li          harden banners status
# \li          harden banners help
# \li          harden banners default
# @author Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage
# @version 1.03 - the macros for release version in the banner text is supported.

=pod

=head1 NAME

=over 4

=item B<harden banners> - command to manage CS1000 pre-login banners.

=back

=head1 SYNOPSIS

=over 10

=item B<harden banners> I<on> | I<off> | I<status> | I<help>

=item B<harden banners> I<set> -path <path_to_file_with_banner_text>  

=item B<harden banners> I<default> 

=back

=head1 DESCRIPTION

B<harden banners> module allows changing of CS1000 pre-login banners.
The module changes the pre-login banner for com-port connections,
FTP, SSH and TELNET connections.

=head1 OPTIONS

=over 4

=item B<on>

This command enables configured pre-login banners. 
The command synchronizes the system pre-login banners with hardening storage banners. 
The SNMP trap is sent to track this event by SNMP manager.

=item B<off>

This command disables the pre-login banners. 
The SNMP trap is sent to track this event by SNMP manager.

=item B<status>

Outputs the status of pre-login banners and the text of banner. 

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden banners> is
invoked without parameters.

=item B<set -path path_to_file_with_banner_text>

This command sets the content from provided file as text of pre-login banners.

=item B<default>

This command sets default pre-login banners. 

=back

=head1 EXAMPLES

=over 4

=item B<$ harden banners on>

 The hardening status of banners is set to 'on' state. The banners are enabled.

=item B<$ harden banners off>

 The hardening status of banners is set to 'off' state. The banners are disabled. 

=item B<$ harden banners status>

 The hardening status of banners is set to 'on'.

=item B<$ harden banners set -path /home/nortel/banner_text>
 
 New text of pre-login banners is set up.

=item B<$ harden banners default>
  
 The status of pre-login banners is set to off. Default text will be applied after changing the hardening status to 'on'.

=back

=head1 CAVEATS

B<harden banners> must be run only under a high privileges account.

=head1 SEE ALSO

B</etc/issue>, B</etc/issue.net> 

=cut

## @class banners
package hardening::plugins::banners;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";
use common_variables;
use CommonInstallUtils;
use base_system;
use Getopt::Long;
use baselog;
use hardening::hardening_common;
use base_snmp;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.03';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub set();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_passwd_days_state();
## @skipline
sub recovery();
## @skipline
sub parse_command_line_arguments($);
## @skipline
sub set_customized_text($);
## @skipline
sub validate_input_params($);
## @skipline
sub set_banner_to_system($);
## @skipline
sub setup_banner_for_comport($);
## @skipline
sub setup_banner_for_ssh();
## @skipline
sub setup_banner_for_ftp();
## @skipline
sub setup_banner_for_telnet();
## @skipline
sub set_default_banner();
## @skipline
sub banners_turn_off();
## @skipline
sub synch_files_with_storage();
## @skipline
sub get_banner_from_storage();
## @skipline
sub set_banner_to_storage($);
## @skipline
sub get_text_from_file($);
## @skipline
sub get_base_version();
## @skipline
sub default();

#variables
my $module_name = "banners";

#base version macros
my $base_version_macros = "##BASE_VERSION##";

#default banner
my $default_banner_text = "
               Nortel Networks Linux Base  $base_version_macros";
$default_banner_text .= q{
The software and data stored on this system are the property of,
or licensed to, Nortel Networks and are lawfully available only
to authorized users for approved purposes. Unauthorized access
to any software or data on this system is strictly prohibited and
punishable under appropriate laws. If you are not an authorized
user then do not try to login. This system may be monitored for 
operational purposes at any time.

};

#com-port banner
my $com_banner_path = "/etc/issue";

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'default' => {
                             'sub'     => \&default,
                             'visible' => $TRUE
                            },
               'set' => {
                         'sub'             => \&set_customized_text,
                         'validate_params' => \&validate_input_params,
                         'visible'         => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_banners_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine
#          pre-login banners value, and outputs this status.
# @return TRUE - if the status of the Hardening option is got.
# @return FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status of banners...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "The hardening status of banners is set to 'on'.\n", $TRUE);
      my $banner = get_banner_from_storage();
      $retval = $TRUE;
      if (not defined $banner)
      {
         baselog($LOG_ERROR, "Cannot get text of pre-login banners from hardening storage.");
         return $FALSE;
      }
      print "Text of pre-login banners is:\n" . $banner . "\n";
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "The hardening status of pre-login banners is set to 'off'.\n", $TRUE);

      my $banner = get_banner_from_storage();
      $retval = $TRUE;
      if (not defined $banner)
      {
         baselog($LOG_ERROR, "Cannot get text of pre-login banners from hardening storage.");
         return $FALSE;
      }
      print "Previously applied text of pre-login banners is:\n" . $banner . "\n";

   } ## end elsif ($status == $FALSE)
   else
   {
      baselog($LOG_ERROR, "The hardening status of banners is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return  FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub off()
# @brief   It reapplies pre-login banners
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform changing text of pre-login banners
#          and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to disable pre-login banners...");

   my $retval = $FALSE;

   if (is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'Pre-login banners' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (is_option_permitted...

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&banners_turn_off);

   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The hardening status of banners is set to 'off' state. The banners are disabled.\n", $TRUE);

      send_snmp_trap("SEC0085");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot disable pre-login banners.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief  It reapplies pre-login banners
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to apply configured text of pre-login banners...");

   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&synch_files_with_storage, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The hardening status of banners is set to 'on' state. The banners are enabled.\n", $TRUE);

      send_snmp_trap("SEC0091");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change hardening status of pre-login banners.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub default()
# @brief  It reapplies hardening storage
# @details This method changes the text of pre-login banners to the default
#          in the hardening storage, and sends the corresponding SNMP trap.
# @return TRUE - the default text is set
# @return FALSE - some error happens
sub default()
{
   baselog($LOG_DEBUG, "Trying to apply default text of pre-login banners to hardening storage...");

   my $retval = $FALSE;

   if (not set_banner_to_storage($default_banner_text))
   {
      baselog($LOG_ERROR, "Cannot set default text of banners to hardening storage.\n", $TRUE);
      return $FALSE;
   }
   my $status = is_option_permitted($module_name);
   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "Default text of pre-login banners is set up.\n", $TRUE);
      send_snmp_trap("SEC0086");
      $retval = $TRUE;
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_WARNING, "The status of pre-login banners is set to off. Default text will be applied after changing the hardening status to 'on'.\n", $TRUE);

      $retval = $TRUE;
   } ## end elsif ($status == $FALSE)
   else
   {
      baselog($LOG_ERROR, "The status of pre-login banners is unknown.\n", $TRUE);
      $retval = $FALSE;
   }

   return $retval;
} ## end sub default()

###############################################################################
## @fn sub set_customized_text($params)
# @brief  It sets provided text as pre-login banners
# @details The method makes the following actions in order to apply banners text:
# \li      1. get text from provided file
# \li      2. set banner text to the system
# \li      3. set banner text to the hardening storage
# \li      4. print status
# @param  $params - command line arguments
# @return TRUE -  new text of banners is set up
# @return FALSE - some error happens
sub set_customized_text($)
{
   my $params = \@_;

   baselog($LOG_DEBUG, "Trying to handle parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      baselog($LOG_ERROR, "Incorrect syntax.");
      return $FALSE;
   }

   my $banner = get_text_from_file($parsed_params->{path});
   if (not defined $banner)
   {
      baselog($LOG_ERROR, "Cannot get text from provided file '$parsed_params->{path}'.", $TRUE);
      return $FALSE;
   }
   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $TRUE)
   {
      if (not set_banner_to_system($banner))
      {
         baselog($LOG_ERROR, "Cannot set text of banners to the system.\n", $TRUE);
         return $FALSE;
      }
      if (not set_banner_to_storage($banner))
      {
         baselog($LOG_ERROR, "Cannot set text of banners to hardening storage.\n", $TRUE);
         return $FALSE;
      }

      baselog($LOG_INFO, "New text of pre-login banners is set up.\n", $TRUE);

      send_snmp_trap("SEC0086");

      $retval = $TRUE;

   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      if (not set_banner_to_storage($banner))
      {
         baselog($LOG_ERROR, "Cannot set text of banners to hardening storage.\n", $TRUE);
         return $FALSE;
      }

      baselog($LOG_WARNING, "The status of pre-login banners is set to off. New text will be applied after changing the hardening status to 'on'.\n", $TRUE);

      $retval = $TRUE;
   } ## end elsif ($status == $FALSE)
   else
   {
      baselog($LOG_ERROR, "The status of pre-login banners is unknown.\n", $TRUE);

      $retval = $FALSE;
   } ## end else [ if ($status == $TRUE)
   return $retval;
} ## end sub set_customized_text($)

###############################################################################
## @fn sub parse_command_line_arguments($params)
# @brief Parses command line parameters.
# @details This module parses command line parameters by GetOptions and fills
#          the hash with parsed data.
# @param $params - reference to array with command line parameters
# @return \%parsed_params - reference to hash with parsed data
# @return undef - illegal command line parameters (e.g. wrong syntax)
sub parse_command_line_arguments($)
{
   my $params = shift @_;

   local @ARGV = @$params;

   my %parsed_params = ('path' => undef);

   my $res = GetOptions(\%parsed_params, 'path=s');
   if ((not $res) or (scalar(@ARGV) > 0))
   {
      return undef;
   }

   return \%parsed_params;
} ## end sub parse_command_line_arguments($)

###############################################################################
## @fn sub validate_input_params($params)
# @brief Parses command line parameters.
# @details This module parses command line parameters by GetOptions and fills
#          the hash with parsed data.
# @param $params - reference to array with command line parameters
# @return \%parsed_params - reference to hash with parsed data
# @return undef - illegal command line parameters (e.g. wrong syntax)
sub validate_input_params($)
{

   my $params = \@_;
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Trying to verify parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      print STDERR "Incorrect syntax. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   if (not defined $parsed_params->{path})
   {
      print STDERR "Parameters parsing error. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   my $path = $parsed_params->{path};
   if ($path =~ m|^.*/|)
   {
      if (-e $path and -r $path and -T $path)
      {
         $retval = $TRUE;
      }
      else
      {
         print STDERR "The file should be accessible for reading and should be a text file.\n";
         $retval = $FALSE;
      }
   } ## end if ($path =~ m|^.*/|)
   else
   {
      print STDERR "The path to banners file should be defined.\n";
      $retval = $FALSE;
   }

   return $retval;
} ## end sub validate_input_params($)

###############################################################################
## @fn sub set_banner_to_system($banner)
# @brief It sets text of banner to the system
# @details The method sets pre-login banner text for the following connections:
#          com-port, SSH, FTP, TELNET.
# @param $banner - string with text of banner
# @return TRUE - text of banner is set up to the system successfully
# @return FALSE - some error happens
sub set_banner_to_system($)
{
   my $banner = shift;
   if (not defined $banner)
   {
      baselog($LOG_ERROR, "Parameters should be defined.");
      return $FALSE;
   }

   my $retval = $TRUE;

   #replace macros (if it exists) by the actual LinuxBase version
   my $version = get_base_version();
   $banner =~ s/$base_version_macros/$version/sg;

   # banner for com-port should be set first, because it's conf file is used as banner storage.
   $retval |= setup_banner_for_comport($banner);
   $retval |= setup_banner_for_ssh();
   $retval |= setup_banner_for_ftp();
   $retval |= setup_banner_for_telnet();

   return $retval;
} ## end sub set_banner_to_system($)

###############################################################################
## @fn sub setup_banner_for_comport($banner)
# @brief   It sets banner text for com port connections
# @details The method rewrites com-port configuration file
# @param $banner - text of banner
# @return TRUE - banner is set up successfully
# @return FALSE - some error happens
sub setup_banner_for_comport($)
{
   my $banner = shift;
   if (not defined $banner)
   {
      baselog($LOG_ERROR, "Parameter should be defined.");
      return $FALSE;
   }
   return write_harden_file($com_banner_path, $banner);
} ## end sub setup_banner_for_comport($)

###############################################################################
## @fn sub setup_banner_for_ssh()
# @brief   It sets banner text for SSH
# @details The method parses SSH configuration file in order to set up banner file
#          path.
# @return TRUE  - configuration file is changed successfully
# @return FALSE - some error happens
sub setup_banner_for_ssh()
{
   my $ssh_conf_file = "/etc/ssh/sshd_config";

   baselog($LOG_DEBUG, "Reapplying '$ssh_conf_file' file.");

   #get content of $ssh_conf_file file
   my $file;
   if (not open($file, '<', $ssh_conf_file))
   {
      baselog($LOG_ERROR, "Cannot open $ssh_conf_file file: $!");
      return $FALSE;
   }
   my @orig_content = <$file>;
   if (not close($file))
   {
      baselog($LOG_ERROR, "Cannot close $ssh_conf_file file: $!");
   }

   my @new_content = ();
   my $flag        = $FALSE;

   #change banner parameter
   my $ssh_banner_param = "Banner";

   foreach my $str (@orig_content)
   {
      if ($str =~ /^\s*$ssh_banner_param\s*/)
      {
         push @new_content, $ssh_banner_param . "  " . $com_banner_path . "\n";
         $flag = $TRUE;
         next;
      } ## end if ($str =~ /^\s*$ssh_banner_param\s*/...
      push @new_content, $str;
   } ## end foreach my $str (@orig_content...

   #if there is no '$ssh_banner_param' parameter - add it
   if (not $flag)
   {
      push @new_content, $ssh_banner_param . "  " . $com_banner_path . "\n";
   }

   #some checking
   if ((not scalar(@new_content)) or (not defined $new_content[0]))
   {
      baselog($LOG_ERROR, "Cannot get information from '$ssh_conf_file' configuration file.");
      return $FALSE;
   }

   return write_harden_file($ssh_conf_file, \@new_content);

} ## end sub setup_banner_for_ssh()

###############################################################################
## @fn sub setup_banner_for_ftp ()
# @brief   It sets banner text for FTP
# @details The method parses FTP configuration file in order to set up banner file
#          path.
# @return TRUE  - configuration file is changed successfully
# @return FALSE - some error happens
sub setup_banner_for_ftp()
{
   my $ftp_conf_file = "/etc/vsftpd/vsftpd.conf";
   baselog($LOG_DEBUG, "Reapplying '$ftp_conf_file' file.");

   #get content of $ftp_conf_file file
   my $file;
   if (not open($file, '<', $ftp_conf_file))
   {
      baselog($LOG_ERROR, "Cannot open $ftp_conf_file file: $!");
      return $FALSE;
   }
   my @orig_content = <$file>;
   if (not close($file))
   {
      baselog($LOG_ERROR, "Cannot close $ftp_conf_file file: $!");
   }

   my @new_content = ();
   my $flag        = $FALSE;

   #change banner parameter
   my $ftp_banner_param = "banner_file";

   foreach my $str (@orig_content)
   {
      if ($str =~ /^\s*$ftp_banner_param\s*/)
      {
         push @new_content, $ftp_banner_param . "=" . $com_banner_path . "\n";
         $flag = $TRUE;
         next;
      } ## end if ($str =~ /^\s*$ftp_banner_param\s*/...
      push @new_content, $str;
   } ## end foreach my $str (@orig_content...

   #if there is no '$ftp_banner_param' parameter - add it
   if (not $flag)
   {
      push @new_content, $ftp_banner_param . "=" . $com_banner_path . "\n";
   }

   #some checking
   if ((not scalar(@new_content)) or (not defined $new_content[0]))
   {
      baselog($LOG_ERROR, "Cannot get information from '$ftp_conf_file' configuration file.");
      return $FALSE;
   }

   return write_harden_file($ftp_conf_file, \@new_content);

} ## end sub setup_banner_for_ftp()

###############################################################################
## @fn sub setup_banner_for_telnet()
# @brief   It sets banner text for TELNET connections
# @details The method creates symbol link to the com-port configuration file
# @return TRUE - Link is created successfully
# @return FALSE - some error happens
sub setup_banner_for_telnet()
{

   #create symbol link to $com_banner_path file
   my $file_path = "/etc/issue.net";
   if (-e $file_path)
   {
      return $FALSE if (not do_system([$RM, "-f", $file_path]));
   }
   return symlink $com_banner_path, $file_path;
} ## end sub setup_banner_for_telnet()

###############################################################################
## @fn sub set_default_banner()
# @brief   It sets default pre-login banner to the system.
# @details The method invokes the function which reapplies banner text,
#          passing default banner text as argument. Then sets default banner to
#          the hardening storage.
# @return  TRUE - The banenr text is reapplied successfully
# @return  FALSE - some error happens
sub set_default_banner()
{
   if (not set_banner_to_system($default_banner_text))
   {
      baselog($LOG_ERROR, "Cannot set text of parameters to conf file.");
      return $FALSE;
   }
   if (not set_banner_to_storage($default_banner_text))
   {
      baselog($LOG_ERROR, "Cannot set text of parameters to conf file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub set_default_banner()

###############################################################################
## @fn sub banners_turn_off()
# @brief   It turns off pre-login banner to the system.
# @details The method invokes the function which reapplies banner text,
#          passing empty string as argument.
# @return  TRUE - The banenr text is reapplied successfully
# @return  FALSE - some error happens
sub banners_turn_off()
{
   if (not set_banner_to_system(""))
   {
      baselog($LOG_ERROR, "Cannot set text of parameters to conf file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub banners_turn_off()

###############################################################################
## @fn sub sub synch_files_with_storage()
# @brief   It synchronizes configuration file with text from hardening storage
# @details The method perfomes the following actions in order synchronize
#          configuration files with the hardening storage
# \li      1. open hardening storage
# \li      2. get text of parameters from storage
# \li      3. change configuration file according to text form storage
# \li      4. if it is impossible to get banner from storage - apply default one
# @return  TRUE - synchronization is performed successfully
# @return   FALSE - some error happens
sub synch_files_with_storage()
{
   my $banner = get_banner_from_storage();
   if (not defined $banner)
   {
      baselog($LOG_INFO, "Cannot get text of pre-login banners from hardening storage.");
      baselog($LOG_INFO, "Applying default banner.");

      $banner = $default_banner_text;

      if (not set_banner_to_storage($banner))
      {
         baselog($LOG_ERROR, "Cannot set text of pre-login banners to storage.");
         return $FALSE;
      }
   } ## end if (not defined $banner...

   if (not set_banner_to_system($banner))
   {
      baselog($LOG_ERROR, "Cannot set text of pre-login banners to system.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub synch_files_with_storage()

###############################################################################
## @fn sub get_banner_from_storage()
# @brief It gets banner text from hardening storage
# @details The method opens hardening storage, gets corresponding text of banners.
# @return  $text - text of pre-login banners
# @return  undef - some error happens
sub get_banner_from_storage()
{
   baselog($LOG_DEBUG, "Trying to get text of pre-login banners.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return undef;
   }

   my $text = $parser->getText(qq{/$HARDENING_ROOT_ELEM/$module_name});
   if (not defined $text)
   {
      baselog($LOG_ERROR, "Cannot get text for the option '$module_name'.");
      undef $parser;
      return undef;
   } ## end if (not defined $text)
   undef $parser;
   return $text;
} ## end sub get_banner_from_storage()

###############################################################################
## @fn sub set_banner_to_storage($banner)
# @brief It sets text of banner to the hardening storage
# @details The method opens hardening storage, changes corresponding entry and
#          saves the storage.
# @param  $banner  - text of pre-login banners
# @return TRUE - the new text is stored in the storage
# @return FALSE - some error happens
sub set_banner_to_storage($)
{
   my $banner = shift;

   if (not defined $banner)
   {
      baselog($LOG_ERROR, "Banner text is not specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to set text of pre-login banners to hardening storage.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }

   my $retval = $FALSE;
   if (not $parser->changeText(qq{/$HARDENING_ROOT_ELEM/$module_name}, undef, $banner))
   {
      baselog($LOG_ERROR, "Cannot set text of pre-login banners.");
   }
   else
   {
      baselog($LOG_DEBUG, "New text for pre-login banners is set.");

      save_hardening_storage($parser);
      $retval = $TRUE;
   } ## end else [ if (not $parser->changeText...
   undef $parser;
   return $retval;
} ## end sub set_banner_to_storage($)

###############################################################################
## @fn sub get_text_from_file($file_path)
# @brief It gets text from provided file
# @details The method check the provided file, opens it and returns it's content
#          as scalar.
# @param  $file_path - file path
# @return $text - text of file
# @return undef - some error happens
sub get_text_from_file($)
{
   my $file_path = shift;
   if (not defined $file_path)
   {
      baselog($LOG_ERROR, "File is not specified.");
      return undef;
   }
   if ((not -e $file_path) or (not -r $file_path))
   {
      baselog($LOG_ERROR, "Cannot get text of file.");
      return undef;
   }
   my $file;
   if (not open($file, '<', $file_path))
   {
      baselog($LOG_ERROR, "Cannot open '$file_path': $!");
      return undef;
   }
   my @arr_text = <$file>;
   my $text;
   $text .= $_ foreach (@arr_text);
   if (not defined $text)
   {
      baselog($LOG_ERROR, "File is empty.");
      return undef;
   }
   if (not close($file))
   {
      baselog($LOG_ERROR, "Cannot close $file_path file: $!");
      return undef;
   }
   return $text;
} ## end sub get_text_from_file($)

###############################################################################
## @fn sub get_base_version()
# @brief Get base application version
# @details This method returns first 3 digits of base version(e.g. 5.80).
# @return version if cannot define version - return 6.00 by default
# @todo   FIXME - it is necessary to move this function from this module to some
#               common place and replace all places, where base version needs
#               to be defined by this function

sub get_base_version()
{

   my $default_version = "6.00";

   my $conf_file = $BASE_CONFIG_DIR . "/configure.xml";

   my $parser = base_xml->new($conf_file);
   if (not defined $parser)
   {
      baselog($LOG_ERROR,   "Cannot read $conf_file file.");
      baselog($LOG_WARNING, "Using default base version.");
      return $default_version;
   } ## end if (not defined $parser...

   my $hash_ref = $parser->getAttrs("application");

   if (not defined $hash_ref)
   {
      undef $parser;
      baselog($LOG_ERROR,   "Cannot defined base version.");
      baselog($LOG_WARNING, "Using default base version.");
      return $default_version;
   } ## end if (not defined $hash_ref...

   undef $parser;

   if ($hash_ref->{version} =~ /^(\d+\.\d+)\./)
   {
      return $1;
   }
   else
   {
      return $default_version;
   }

} ## end sub get_base_version()

###############################################################################
## @fn sub apply_default()
# @brief Apply default text of banners parameters
# @details This method changes the status of the Hardening option in accordance
#          to the default state -
#          Default value is applied in the case when it is impossible to detect
#          the current banners usage policy.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to apply default text of pre-login banners...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&set_default_banner, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "banners status is off.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change pre-login banners.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got successfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of pre-login banners are unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "The pre-login banners are enabled.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "The pre-login banners are disabled.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in accordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine
#          the text value of pre-login banners.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of banners from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of banners is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {
      $retval = synch_files_with_storage();
   }
   else
   {
      $retval = banners_turn_off();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_banners_state()
# @brief Send SNMP trap in accordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of banners service is permitted or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_banners_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of banners.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of banners is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0091");
   }
   else
   {
      $retval = send_snmp_trap("SEC0085");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_banners_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          text of pre-login banners, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of banners...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_banners_state();

   return $retval;
} ## end sub recovery()

1;
__END__
