###############################################################################
## @file basic.pm
# @brief Manages the Basic Hardening items
# @details This module uses the Basic Hardening manager to manage the Basic
#          Hardening items
#
#      @li It is responsible for the CLI actions:
#      @li    harden basic 
#      @li    harden basic help
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


=pod

=head1 NAME

=over 4

=item B<harden basic> - command to manage the Basic Hardening items.

=back

=head1 SYNOPSIS

=over 10

=item B<harden basic>

=item B<harden basic> I<help>

=back

=head1 DESCRIPTION

B<harden basic> module allows managing the Basic Hardening items. If the module
is used without parameters it re-applied all Basic Hardening items.

=head1 OPTIONS

=over 4

=item B<help>

Outputs this manual page.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden basic>

 All Basic Hardening items successfully applied.

=back

=head1 CAVEATS

B<harden basic> must be run only under a high privileges account.

=cut


## @class basic
package hardening::plugins::basic;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use strict;
use warnings;

use lib "/opt/nortel/base/lib";

use common_variables;
use hardening::hardening_common;
use baselog;

## @skipline 
sub actions();
## @skipline
sub reapply();
## @skipline
sub status();

my $module_name = "basic";

my %actions = ('reapply' => {'sub'   => \&reapply,
                             'visible' => $FALSE,
                             'default' => $TRUE},
               'status'  => {'sub'   => \&status,
                             'visible' => $TRUE},
               'help'    => {'sub'   => \&help,
                             'visible' => $TRUE}
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions()

###############################################################################
## @fn sub status()
# @brief Prints status of Basic Hardening items  
# @details Should output status of Basic Hardening items. It is just a stub for
#          future design
# @return  always TRUE
sub status()
{
  
  baselog($LOG_INFO, "Getting status of Basic Hardening items is not supported.\n", $TRUE);
  
  return $TRUE;
} ##end sub get_status()

###############################################################################
## @fn sub reapply()
# @brief Re-apply the Basic Hardening items
# @details This method uses the Basic Hardening manager (instance of the
#          Plug-in manager) in order to re-apply the Basic Hardening items.
# @return  TRUE  - the items are reapplied
# @return  FALSE - some error happens
sub reapply()
{
  
  baselog($LOG_DEBUG, "Try to re-apply the Basic Hardening items.");
  
  my $retval = do_basic_hardening_action($H_IF_APPLY);

  if ($retval)
  {
     baselog($LOG_INFO, "All Basic Hardening items successfully applied.\n", $TRUE);
  }
  else
  {
     baselog($LOG_INFO, "Basic Hardening items were applied with errors.\n", $TRUE);
  }

  return $retval;
} ##end sub add_host()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
} ##end sub get_help()


1;
__END__
