###############################################################################
## @file coredumps.pm
# @brief Provides hardening actions for core files
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against the Linux core files capability and the
#          Enhanced Hardening module 'coredumps' are requested.
#
#\li          It is responsible for the CLI actions:
#\li           harden coredumps
#\li           harden coredumps on
#\li           harden coredumps off
#\li           harden coredumps set -size value
#\li           harden coredumps status
#\li           harden coredumps help
#
#         The configuration of core file size is placed in
#         /etc/opt/nortel/base/corefile_limits which is sourced by the profiles
#         scripts linuxbase.sh and linuxbase.csh. If you changes something in
#         this subroutine please reflect the chanages in those files.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage

=pod

=head1 NAME

=over 4

=item B<harden coredumps> - command to manage the ability of core files
creation.

=back

=head1 SYNOPSIS

=over 10

=item B<harden coredumps> I<on> | I<off> | I<status> | I<help>

=item B<harden coredumps> I<set> -size <size>

=back

=head1 DESCRIPTION

B<harden coredumps> module allows managing the ability of core files creation.

=head1 OPTIONS

=over 4

=item B<on>

Turns on the ability of core files creation. It sets up max size of core files
and changes permissions on F</var/coredump> directory to allow all users in the
group I<nortel> to create core files. If the size has not been configured
before, the default size in 1GB is applied. The SNMP trap is sent to track this
event by SNMP manager.

=item B<off>

Turns off the ability of core files creation. Besides this, the permissions of
the directory F</var/coredump> is changed to disallow writing core files. The
SNMP trap is sent to track this event by SNMP manager.

=item B<set>

Allows configuring new max size of core files in kilobytes. New size should be
greater than 0 and less than 1048576 kilobytes (1GB). If the size is changed
successfully, the SNMP trap is sent to track this event by SNMP manager.

=item B<status>

Outputs the actual status whether the ability of core files creation is turned
on or not. If it is turned on, the curent maximum size of core files is output
as well.

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden coredumps>
is invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden coredumps on>

 The ability to prepare core files is turned on.

 WARNING: In order for a new core file limit becomes effective, please either reboot
 the system or re-login (log off and log in) to the terminal and restart the
 applications.

=item B<$ harden coredumps off>

 The ability of core files creation is turned off.

 WARNING: In order for a new core file limit becomes effective, please either reboot
 the system or re-login (log off and log in) to the terminal and restart the
 applications.

=item B<$ harden coredumps set -size 5000>

 New maximum core file size is set to '5000' kilobytes.

 WARNING: In order for a new core file limit becomes effective, please either reboot
 the system or re-login (log off and log in) to the terminal and restart the
 applications.

=item B<$ harden audit coredumps>

 The current maximum size of core files can be '5000' kilobytes.

=back

=head1 CAVEATS

B<harden coredumps> must be run only under a high privileges account.

In order to new core file limit becomes effective, either reboot
the system should be performed or the current terminal session should be
re-initialized (log off and log in) and restart of the applications should be
invoked.

There is a special daily cron job which cleans core files directories in
accordance to the following rules:

=over 4

1. It deletes old core files, if the size of the directory larger than
B<10Gb>.

2. It deletes old core files, except the most recent five files.

3. It deletes all core files which are older than 2 weeks.

4. It deletes all files with size greater than configured limit.

=back

=cut

## @class coredumps
package hardening::plugins::coredumps;

use strict;
use warnings;

use common_variables;
use common_functions;
use CommonInstallUtils qw(getYNResponse);

use Getopt::Long;
use base_system;
use baselog;
use hardening::hardening_common;
use base_snmp;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.02';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub get_core_files_size_from_storage();
## @skipline
sub set_core_files_size_to_storage();
## @skipline
sub enable_corefiles(;$);
## @skipline
sub disable_corefiles();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_coredumps_state();
## @skipline
sub recovery();
## @skipline
sub parse_command_line_arguments($);
## @skipline
sub validate_size_params(@);
## @skipline
sub set_new_size(@);
## @skipline
sub print_warning();
## @skipline

#variables
my $auditd      = "auditd";
my $module_name = "coredumps";
my $size_attr   = "size";

my $max_core_size       = 1048576;
my $unlimited_core_size = "unlimited";
my $disabled_core       = "0";

my $corefiles_dir = "/var/coredump";

# The configuration of core file size is placed in
# /etc/opt/nortel/base/corefile_limits which is sourced by the profiles
# scripts linuxbase.sh and linuxbase.csh. If you changes something in
# this subroutine please reflect the chanages in those files.
my $core_limits     = $BASE_CONFIG_DIR . "/corefile_limits";
my $core_soft_limit = "CORESOFTLIMIT=";
my $core_hard_limit = "COREHARDLIMIT=";

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'set' => {
                         'sub'             => \&set_new_size,
                         'validate_params' => \&validate_size_params,
                         'visible'         => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_coredumps_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current configuration of core files
# @details This method ouptuts configuration of core files.
# @return TRUE - if configuration is got.
# @return FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get configuration of core files...");

   #Check if core files is permitted
   my $hard_status = is_option_permitted($module_name);

   my $retval = $TRUE;

   if ($hard_status == $H_ERROR)
   {
      baselog($LOG_ERROR, "Hardening status of '$module_name' option is unknown.\n", $TRUE);
      $retval = $FALSE;
   }
   elsif (not $hard_status)
   {
      baselog($LOG_INFO, "The ability to prepare core files is turned off.\n", $TRUE);

      #Get configured max size of core files
      my $size = get_core_files_size_from_storage();
      if (defined $size)
      {
         baselog($LOG_INFO, "The previous maximum size of core files was '$size' kilobytes. \n", $TRUE);
      }
   } ## end elsif (not $hard_status)
   else
   {

      #Get configured max size of core files
      my $size = get_core_files_size_from_storage();
      if (not defined $size)
      {
         baselog($LOG_ERROR, "The current maximum size of core files is unknown.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "The current maximum size of core files is '$size' kilobytes. \n", $TRUE);
      }

   } ## end else [ if ($hard_status == $H_ERROR...

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parses POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub get_core_files_size_from_storage()
# @brief Gets configured size of core files
# @details This method examines the Hardening storage and get configured size
#          of core files.
# @return number - configured size of core files
# @return undef - some error happens
sub get_core_files_size_from_storage()
{
   baselog($LOG_DEBUG, "Trying to get configured maximum size of core files.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return undef;
   }

   my $hash_ref = $parser->getAttrs(qq{/$HARDENING_ROOT_ELEM/$module_name});
   if (not defined $hash_ref)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the option '$module_name'.");
      undef $parser;
      return undef;
   } ## end if (not defined $hash_ref...
   undef $parser;

   if (not exists $hash_ref->{$size_attr})
   {
      hardening_special_baselog($LOG_ERROR, "The option '$module_name' does not contain the attribute '$size_attr'.");
      return undef;
   }

   baselog($LOG_DEBUG, "Configured maximum size of core files is '$hash_ref->{$size_attr}'.");

   return $hash_ref->{$size_attr};
} ## end sub get_core_files_size_from_storage()

###############################################################################
## @fn sub set_core_files_size_to_storage($size)
# @brief Stores new size of core files in the Hardening storage
# @details This method stores new size of core files in the Hardening storage.
#          It is assumed that all XML tree is already created in the storage,
#          so, the method does not created all parent nodes, just change/add
#          'size' attribute.
# @return TRUE - the new value is stored in the storage
# @return FALSE - some error happens
sub set_core_file_size_to_storage($)
{
   my $size = shift @_;

   if ((not defined $size) or ($size eq ""))
   {
      baselog($LOG_ERROR, "Maximum size of core files is not specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to set new maximus size of core files.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }

   my $retval = $FALSE;

   if (not $parser->changeAttrs(qq{/$HARDENING_ROOT_ELEM/$module_name}, undef, {$size_attr => $size}))
   {
      baselog($LOG_ERROR, "Cannot set new maximum size of core files.");
   }
   else
   {
      baselog($LOG_DEBUG, "New maximum size '$size' of core files is stored.");

      save_hardening_storage($parser);
      $retval = $TRUE;
   } ## end else [ if (not $parser->changeAttrs...

   undef $parser;
   return $retval;
} ## end sub set_core_file_size_to_storage($)

###############################################################################
## @fn sub enable_corefiles($size)
# @brief Configures the system to apply new core file limits
# @details This method applies core file limits from the Hardening storage to
#          the system (or the passed value). If the size does not defined in
#          the storage, default maximum size is applied.
#
#          The configuration of core file size is placed in
#          /etc/opt/nortel/base/corefile_limits which is sourced by the profiles
#          scripts linuxbase.sh and linuxbase.csh. If you changes something in
#          this subroutine please reflect the chanages in those files.
# @param $size - (optional) new limit. If it not defined, it will be got from
#                the Hardening storage.
# @return TRUE - new core file limits are applied
# @return FALSE - some error happens
sub enable_corefiles(;$)
{
   my $size = shift @_;

   baselog($LOG_DEBUG, "Applying core file limits...");

   if (not defined $size)
   {

      #Get configured max size of core files
      $size = get_core_files_size_from_storage();

      #Size is not defined - add default max size to the Hardening storage
      if (not defined $size)
      {
         if (not set_core_file_size_to_storage($max_core_size))
         {
            baselog($LOG_ERROR, "Cannot set default maximum size to the Hardening storage.");
            return $FALSE;
         }

         $size = $max_core_size;
      } ## end if (not defined $size)
   } ## end if (not defined $size)

   my $limits_content = $core_soft_limit . $size . "\n" . $core_hard_limit . $unlimited_core_size . "\n";

   #apply new limits
   if (not write_harden_file($core_limits, $limits_content, $NO_BACKUP))
   {
      baselog($LOG_ERROR, "Cannot apply new core file limits to the system.");
      return $FALSE;
   }

   #made the limits readable for all
   if (not change_permissions_and_ownership(0644, $ROOT_UID, $ROOT_GID, [$core_limits], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the file '$core_limits'");
      do_system([$RM, "-f", $core_limits]);
      return $FALSE;
   } ## end if (not change_permissions_and_ownership...

   #made core files directory writable for root and 'nortel' accounts.
   #'sticky' bit is set up, in order to prevents other corefiles modification
   if (not change_permissions_and_ownership(01770, $ROOT_UID, $NORTEL_GID, [$corefiles_dir], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the core files directory '$corefiles_dir'");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "New core file limits are set to '$size' kilobytes.");

   return $TRUE;
} ## end sub enable_corefiles(;$)

###############################################################################
## @fn sub disable_corefiles()
# @brief Configures the system to disable the ability of core file creation
# @details This method applies new core file limits which turn of the ability
#          of core file creation.
#
#          The configuration of core file size is placed in
#          /etc/opt/nortel/base/corefile_limits which is sourced by the profiles
#          scripts linuxbase.sh and linuxbase.csh. If you changes something in
#          this subroutine please reflect the chanages in those files.
# @return TRUE - core file disabled
# @return FALSE - some error happens
sub disable_corefiles()
{
   baselog($LOG_DEBUG, "Disabling the ability of core file creation...");

   my $limits_content = $core_soft_limit . $disabled_core . "\n" . $core_hard_limit . $disabled_core . "\n";

   #apply new limits
   if (not write_harden_file($core_limits, $limits_content, $NO_BACKUP))
   {
      baselog($LOG_ERROR, "Cannot disable the ability of core file creation.");
      return $FALSE;
   }

   #made the limits readable for all
   if (not change_permissions_and_ownership(0644, $ROOT_UID, $ROOT_GID, [$core_limits], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the file '$core_limits'");
      do_system([$RM, "-f", $core_limits]);

      #Since the default behavior of profile script is to disable core files
      #if the limits file does not exists, FALSE is not returned.
   } ## end if (not change_permissions_and_ownership...

   #made core files directory writable only for root.
   #'sticky' bit is preserved
   if (not change_permissions_and_ownership(01700, $ROOT_UID, $NORTEL_GID, [$corefiles_dir], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of the core files directory '$corefiles_dir'");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "New core file limits to disable the ability of core file creation are set.");

   return $TRUE;
} ## end sub disable_corefiles()

###############################################################################
## @fn sub off()
# @brief Turns off the ability of core files creation
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, and turns of the ability of core files creation
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to turn off the ability of core files creation...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&disable_corefiles, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The ability of core files creation is turned off.\n", $TRUE);

      send_snmp_trap("SEC0075");

      #since disabling core files includes changing permissions for
      #/var/coredump directory, after this moment only root will have the
      #ability of core files creation with old limits. Once root re-login, it
      #could not create core files as well.
      print_warning();

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn off the ability of core files creation.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief Turns on the ability of core files creation
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, configures maximum size of core files. It sends the
#          corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to turn on the ability of core files creation...");

   my $retval = $FALSE;

   if (not is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'Core dump files' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (not is_option_permitted...

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&enable_corefiles, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The ability to prepare core files is turned on.\n", $TRUE);

      send_snmp_trap("SEC0074");

      print_warning();

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn on the ability to prepare core files.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief Apply default value of maximum core files size
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it turns on the ability of core files
#          creation.
#          Default value is applied in the case when it is impossible to detect
#          the current core files creation policy, i.e. there is no
#          corresponding info in the Hardening storage. Therefore the actual
#          applying of deafult maximum size to the Hardening storage should
#          happen only after the corresponding XML nodes creation.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to turn on the ability of core files creation...");

   my $retval = $FALSE;

   #Set new status to value 'on'
   #Default value is applied in the case when it is impossible to detect the
   #current core files creation policy, i.e. there is no corresponding info in
   #the Hardening storage. Therefore the actual applying of deafult maximum
   #size to the Hardening storage should happen only after the corresponding
   #XML nodes creation.
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&enable_corefiles, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The ability to prepare core files is turned on.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot turn on the ability to prepare core files.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The coredumps configuration status is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "The ability to create core files is permitted.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "The ability to create core files is forbidden.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          the ability of core files creation should be turned on or not. First,
#          the case when there is some problem with the Hardening storage -
#          default state of the option is applied. Otherwise, the system is
#          changed in accordance to the state from the Hardening storage.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of the ability of core files creation from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of the ability of core files creation is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {
      $retval = enable_corefiles();
   }
   else
   {
      $retval = disable_corefiles();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_audit_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          the ability of core files creation is turned on or not, and send the
#          corresponding SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_coredumps_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of the ability of core files creation.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of the ability of core files creation is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0074");
   }
   else
   {
      $retval = send_snmp_trap("SEC0075");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_coredumps_state()

###############################################################################
## @fn sub recovery()
# @brief Applies the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if the ability of core files creation should be turned on or not, and
#          performs all necessary actions to reflect the new state on the system
#          (e.g. send the corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of the ability of core files creation...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_coredumps_state();

   return $retval;
} ## end sub recovery()

###############################################################################
## @fn sub parse_command_line_arguments($params)
# @brief Parses command line parameters.
# @details This module parses command line parameters by GetOptions and fills
#          the hash with parsed data.
# @param $params - reference to array with command line parameters
# @return \%parsed_params - reference to hash with parsed data
# @return undef - illegal command line parameters (e.g. wrong syntax)
sub parse_command_line_arguments($)
{
   my $params = shift @_;

   local @ARGV = @$params;

   my %parsed_params = ('size' => undef);

   my $res = GetOptions(\%parsed_params, '-size=s');
   if ((not $res) or (scalar(@ARGV) > 0))
   {
      return undef;
   }

   return \%parsed_params;
} ## end sub parse_command_line_arguments($)

###############################################################################
## @fn sub validate_size_params($params)
# @brief Validates command line parameters.
# @details This module parses command line parameters and validates their
#          correctness.
#          This method is intvoked by the Hardening Manager for checking
#          command line parameters prior to performing of the action 'set'.
# @param $params - reference to array with command line parameters
# @return TRUE - parameters are valid
# @return FALSE - illegal command line parameters (e.g. wrong syntax)
sub validate_size_params(@)
{
   my $params = \@_;

   baselog($LOG_DEBUG, "Trying to verify parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      print STDERR "Incorrect syntax. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   if (not defined $parsed_params->{'size'})
   {
      print STDERR "Parameters parsing error. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   my $size = $parsed_params->{'size'};

   my $retval = $FALSE;

   if ($size =~ /^\d+$/)
   {
      if (($disabled_core < $size) and ($size <= $max_core_size))
      {
         $retval = $TRUE;
      }
      else
      {
         print STDERR "Size must be greater than $disabled_core and less or equal than $max_core_size kilobytes.\n";
      }
   } ## end if ($size =~ /^\d+$/)
   else
   {
      print STDERR "Size must has a numeric value.\n";
   }

   return $retval;
} ## end sub validate_size_params(@)

###############################################################################
## @fn sub set_new_size($params)
# @brief Changes the current limit of core files size.
# @details This module parses command line parameters, changes the current
#          limit of core files size if it is possible and sends a SNMP trap.
#          This method is intended to handle the action 'set'.
# @param $params - reference to array with command line parameters
# @return TRUE - the limit is changed
# @return FALSE - some error happens
sub set_new_size(@)
{
   my $params = \@_;

   baselog($LOG_DEBUG, "Trying to handle parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      baselog($LOG_ERROR, "Incorrect syntax.");
      return $FALSE;
   }

   if (not defined $parsed_params->{'size'})
   {
      baselog($LOG_ERROR, "Parameters parsing error");
      return $FALSE;
   }

   my $retval = $FALSE;

   my $size = $parsed_params->{'size'};

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of the ability of core files creation is unknown.");
   }
   elsif ($status == $TRUE)
   {
      if (not set_core_file_size_to_storage($size))
      {
         baselog($LOG_ERROR, "Cannot set new maximum core file size to the Hardening storage.");
      }
      else
      {
         $retval = enable_corefiles($size);
         baselog($LOG_INFO, "New maximum core file size is set to '$size' kilobytes.\n", $TRUE);

         send_snmp_trap("SEC0076", {'coresize' => $size});

         print_warning();
      } ## end else [ if (not set_core_file_size_to_storage...
   } ## end elsif ($status == $TRUE)
   else
   {
      baselog($LOG_WARNING, "The ability of core file creation is turned off. New maximum core file size cannot be set.\n", $TRUE);
      $retval = $TRUE;
   }

   return $retval;
} ## end sub set_new_size(@)

###############################################################################
## @fn sub print_warning()
# @brief Prints the warning message.
# @details This module prints the warning message about additional acctions
#          which it is necessary to perform in order to the new limits become
#          effective.
# @return TRUE
sub print_warning()
{
   print "
WARNING: In order for a new core file limit becomes effective, please either reboot
the system or re-login (log off and log in) to the terminal and restart the
applications.
";

   return $TRUE;
} ## end sub print_warning()

1;
__END__
