###############################################################################
## @file ftp.pm
# @brief Provides hardening actions for CS1000 FTP service
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 FTP service and the Enhanced
#          Hardening module 'ftp' are requested.
#
#\li          It is responsible for the CLI actions:
#\li           harden ftp
#\li           harden ftp on
#\li           harden ftp off
#\li           harden ftp status
#\li           harden ftp help
#\li           It also provides the API in order to start and stop vsftpd
#\li           service in accordance with the current status of the
#\li           corresponding Hardening rule.
# @author Petr Kascheev
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - default behaviour is changed to ftp service is permitted
# @version 1.10 - Managing firewall ports functionality added.
# @version 1.11 - Two colons were deleted before shared functions and variables.
# @version 1.12 - fixed a bug with appearance of an error with undefined
#                 variable when the action 'stop' is run from CLI
# @version 1.13 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage

=pod

=head1 NAME

=over 4

=item B<harden ftp> - command to manage usage of CS1000 FTP service.

=back

=head1 SYNOPSIS

=over 10

=item B<harden ftp> I<on> | I<off> | I<status> | I<help>

=back

=head1 DESCRIPTION

B<harden ftp> module allows managing usage of CS1000 FTP service.

=head1 OPTIONS

=over 4

=item B<on>

Permits usage of CS1000 FTP service, starts FTP service and opens the ports. 
The SNMP trap is sent to track this event by SNMP manager.

B<NOTE>: This action performs an actual starting of FTP service and opens 
         firewalls. It also sets a permit flag to use the service in the future.

=item B<off>

Forbids usage of CS1000 FTP service. This action immediately tries to shutdown
all FTP activities and close FTP ports. The warning message notifies about this 
and a user is asked to abort this. Even if the FTP activites are not shut down, 
CS1000 FTP service is forbidden for future usage. The SNMP trap is sent to track
this event
by SNMP manager.

=item B<status>

Outputs the status whether usage of CS1000 FTP service is permitted or not. If
the service is permitted the actual status (whether the service is running or
not) is output as well.

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden ftp> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden ftp on>

 You are trying to set Hardening policy 'FTP service' in less secure state.
 Do you want to proceed? (Y/N) [Y]? Y
 FTP service is permitted.
 Updating iptables rules:                                   [  OK  ]

=item B<$ harden ftp off>

 FTP service is forbidden.
 Updating iptables rules:                                   [  OK  ]

=item B<$ harden ftp status>

 FTP service is permitted.
 FTP service is running.

=back

=head1 CAVEATS

B<harden ftp> must be run only under a high privileges account.

=head1 SEE ALSO

B<harden tftp>

=cut

## @class ftp
package hardening::plugins::ftp;

use base_ftp;

use common_variables;
use base_system;
use baselog;
use hardening::hardening_common;
use base_snmp;
use base_firewall;
use CommonInstallUtils;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.13';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub start();
## @skipline
sub stop();
## @skipline
sub stop_ftp();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_FTP_state();
## @skipline
sub recovery();
## @skipline

#variables
my $module_name = "ftp";

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'start' => {
                           'sub'     => \&start,
                           'visible' => $FALSE
                          },
               'stop' => {
                          'sub'     => \&stop,
                          'visible' => $FALSE
                         },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_FTP_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine if
#          usage of FTP service is permitted or not, and outputs this status.
#          If FTP service is permitted, it ouptuts if the service is actually
#          running or not.
# @return TRUE - if the status of the Hardening option is got.
# @return FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status FTP service...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "FTP service is permitted.\n", $TRUE);
      my $status = get_status_linux_service($FTP_SRV);
      if ($status == $TRUE)
      {
         baselog($LOG_INFO, "FTP service is running.\n", $TRUE);
      }
      elsif ($status == $FALSE)
      {
         baselog($LOG_INFO, "FTP service is stopped.\n", $TRUE);
      }
      else
      {
         baselog($LOG_ERROR, "Status of FTP service is unknown.\n", $TRUE);
      }
      $retval = $TRUE;
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "FTP service is forbidden.\n", $TRUE);
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Hardening status of FTP service is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub start()
# @brief Runs Very Secure FTP (vsftp) server
# @details This method examines the Hardening storage in order to determine if
#          usage of FTP service is permitted or not. If the service is permitted,
#          the method tries to run Very Secure FTP (vsftp) server and sends the
#          corresponding SNMP trap. It also changes firewall rules to open FTP
#          ports and mounts points for extended FTP access.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is running sucessfully
# @return FALSE - either the usage FTP service is forbidden, or some error happens
sub start()
{
   baselog($LOG_DEBUG, "Starting FTP service...");

   #get info from the hardening storage file
   if (is_option_permitted($module_name) == $FALSE)
   {

      #warning message if FTP service is forbidden
      baselog($LOG_WARNING, "Usage of FTP service is forbidden. Contact your Security Administrator for assistance.");
      return $FALSE;
   } ## end if (is_option_permitted...

   #Try to start the service
   if (not start_linux_service($FTP_SRV))
   {
      baselog($LOG_ERROR, "Cannot start FTP service.");
      return $FALSE;
   }

   baselog($LOG_INFO, "FTP service is started.");

   # Try to change firewall rules to open FTP ports
   if (base_ftp::open_ftp_firewall())
   {
      baselog($LOG_INFO, 'Activate mount points.');
      if (not base_ftp::activate_ftp_mounts())
      {
         baselog($LOG_ERROR, 'Cannot activate extended FTP access.');
         return $FALSE;
      }
   } ## end if (base_ftp::open_ftp_firewall...
   else
   {
      baselog($LOG_ERROR, 'Cannot enable firewall ports.');
      return $FALSE;
   }

   send_snmp_trap("SEC0072");

   return $TRUE;
} ## end sub start()

###############################################################################
## @fn sub stop()
# @brief Stops Very Secure FTP (vsftp) server
# @details This method tries to stop Very Secure FTP (vsftp) server and sends
#          the corresponding SNMP trap.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub stop()
{
   if (stop_ftp())
   {
      send_snmp_trap("SEC0073");
      return $TRUE;
   }

   return $FALSE;
} ## end sub stop()

###############################################################################
## @fn sub stop_ftp()
# @brief Stops Very Secure FTP (vsftp) server
# @details This method tries to stop Very Secure FTP (vsftp) server and
#         changes the firewall rules to close FTP ports. It also tries
#         to unmount extended points of FTP access.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub stop_ftp()
{
   my $retval = $TRUE;
   baselog($LOG_DEBUG, "Stopping of FTP service...");

   # Try to change firewall rules to close FTP ports
   if (not base_ftp::close_ftp_firewall())
   {
      baselog($LOG_ERROR, 'Cannot disable FTP ports.');
      $retval = $FALSE;
   }

   # Removing extended FTP access points.
   baselog($LOG_INFO, 'Deactivate mount points.');
   if (not base_ftp::deactivate_ftp_mounts())
   {
      baselog($LOG_ERROR, 'Cannot deactivate extended FTP access.');
      $retval = $FALSE;
   }

   #In order to be sure that the service will not be started automaticaly during
   #Linux bootup sequence, the service is disabled and stopped.
   if (not stop_and_disable_linux_service($FTP_SRV))
   {
      baselog($LOG_ERROR, 'Cannot stop FTP service.');
      $retval = $FALSE;
   }

   baselog($LOG_INFO, 'FTP service is stopped.');

   return $retval;
} ## end sub stop_ftp()

###############################################################################
## @fn sub off()
# @brief Forbid usage of FTP service
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform stop of CS1000 FTP service
#          and sends the corresponding SNMP trap. It also restarts the firewall
#          to apply new rules to close FTP ports.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to forbid FTP service...");

   my $retval = $FALSE;

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&stop, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "FTP service is forbidden.\n", $TRUE);

      send_snmp_trap("SEC0071");

      # Try to restart firewall
      if (not base_firewall::firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         $retval = $TRUE;
      }
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change FTP service permissions.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief Permit usage of FTP service
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to permit FTP service...");

   my $retval = $FALSE;

   if (not is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'FTP service' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (not is_option_permitted...

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&start, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "FTP service is permitted.\n", $TRUE);

      send_snmp_trap("SEC0070");

      # Try to restart firewall
      if (not base_firewall::firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         $retval = $TRUE;
      }

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change FTP service permissions.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief Apply default state of usage of FTP service
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it permits usage of FTP service.
#          Default value is applied in the case when it is impossible to detect
#          the current FTP usage policy.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to forbid FTP service...");
   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, undef);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "FTP service is forbidden.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change FTP service permissions.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of FTP service use is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "Use of FTP service is permitted.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Use of FTP service is forbidden.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of FTP service is permitted or not. Since there are no actual
#          system changes for FTP permitting, only a case when there is some
#          problem with the Hardening storage - default state of the option is
#          applied.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of FTP service from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of usage of FTP service is unknown. Default status is applied.");
      $retval = apply_default();
   }
   else
   {

      #There are no actual system changes in order to set status of FTP service
      #If new status is "off" the current FTP activities are  not taken down.
      $retval = $TRUE;
   } ## end else [ if ($status == $H_ERROR...

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_FTP_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of FTP service is permitted or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_FTP_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of FTP service.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of usage of FTP service is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0070");
   }
   else
   {
      $retval = send_snmp_trap("SEC0071");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_FTP_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if usage of FTP service is permitted or not, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
# @note FTP service will not be started.
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of FTP service...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_FTP_state();

   return $retval;
} ## end sub recovery()

1;
__END__
