###############################################################################
## @file nettools.pm
# @brief Provides hardening actions for CS1000 network tools management.
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 network tools and the Enhanced
#          Hardening module 'nettools' are requested.
#
#\li          It is responsible for the CLI actions:
#\li           harden nettools
#\li           harden nettools on
#\li           harden nettools off
#\li           harden nettools status
#\li           harden nettools help
# @author Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage
# @version 1.03 - fix for the issue when 'rpm -q' queries causes appearing of
#                 error messages in the log
# @version 1.04 - post patching actions are provided.

=pod

=head1 NAME

=over 4

=item B<harden nettools> - command to manage usage of CS1000 network tools. 
The following network tools are affected by this command: trace commands (tracepath, traceroute), tethereal, tcpdump, wireshark.

=back

=head1 SYNOPSIS

=over 10

=item B<harden nettools> I<on> | I<off> | I<status> | I<help>

=back

=head1 DESCRIPTION

B<harden nettools> module allows managing usage of CS1000 network tools.

=head1 OPTIONS

=over 4

=item B<on>

Enable usage of CS1000 network tools. The SNMP trap is sent to track this event
by SNMP manager.

=item B<off>

Disable usage of CS1000 network analysis tools. 
The SNMP trap is sent to track this event by SNMP manager.

=item B<status>

Outputs the status whether usage of CS1000 nettools service is enabled or not. 

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden nettools> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden nettools on>

 Network tools are enabled. 

=item B<$ harden nettools off>

 Network tools are disabled. 

=item B<$ harden nettools status>

 Network tools are enabled.

=back

=head1 CAVEATS

B<harden nettools> must be run only under a high privileges account.

=cut

## @class nettools
package hardening::plugins::nettools;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use CommonInstallUtils qw(getYNResponse);
use base_system;
use baselog;
use hardening::hardening_common;
use base_snmp;
use File::Find;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.04';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_passwd_days_state();
## @skipline
sub recovery();
## @skipline
sub nettools_disable();
## @skipline
sub nettools_enable();
## @skipline
sub disable_trace_commands();
## @skipline
sub enable_trace_commands();
## @skipline
sub enable_rpm($);
## @skipline
sub disable_rpm($);
## @skipline
sub is_rpm_installed($);
## @skipline
sub find_rpm_in_storage($$);
## @skipline
sub install_rpm($$;$);
## @skipline
sub get_last_version_rpm($$);
## @skipline
sub uninstall_rpm($);
## @skipline
sub prepare_to_install($);
## @skipline
sub prepare_to_uninstall($);
## @skipline
sub post_uninstall($);
## @skipline
sub post_install($);

#variables
my $module_name           = "nettools";
my $hardening_rpm_storage = "/var/opt/nortel/base_harden/";

#tethereal tool, it is symbolic link to tshark
my $tethereal = "/usr/sbin/tethereal";

# RPMS packages in install order
my %rpms = (
            '1' => {
                    'name'    => 'wireshark',
                    'command' => '/usr/sbin/tshark'
                   },
            '2' => {
                    'name'    => 'wireshark-gnome',
                    'command' => '/usr/sbin/wireshark'
                   },
            '3' => {
                    'name'    => 'tcpdump',
                    'command' => '/usr/sbin/tcpdump'
                   },
            '4' => {'name' => 'appcrit'}
           );

# RPMS packages in uninstall order
my %un_rpms = (
   '1' => {
           'name'    => 'wireshark-gnome',
           'command' => '/usr/sbin/wireshark'
          },

   '2' => {
           'name'    => 'wireshark',
           'command' => '/usr/sbin/tshark'
          },
   '3' => {
           'name'    => 'tcpdump',
           'command' => '/usr/sbin/tcpdump'
          },
   '4' => {'name' => 'appcrit'}
              );

my $restricted_data = q{
#!/bin/sh  

# This file is auto-generated script. 

echo "Warning: The Security Administrator restricted this tool.";
echo "Contact your Security Administrator for assistance.";

exit 1; 
};

my %trace_commands = (
                      'path'   => '/bin/tracepath',
                      'path6'  => '/bin/tracepath6',
                      'route'  => '/bin/traceroute',
                      'route6' => '/bin/traceroute6',
                     );

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_nettools_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               $H_IF_POST_PATCHING => {
                                       'sub'     => \&post_patching,
                                       'visible' => $FALSE
                                      },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief   Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return  %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief   Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine if
#          network tools are enabled or not, and outputs this status.
# @return  TRUE - if the status of the Hardening option is got.
# @return  FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status of network tools..");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "Network tools are enabled.\n", $TRUE);
      $retval = $TRUE;
   }
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "Network tools are disabled.\n", $TRUE);
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Hardening status of network tools is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief   Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return  TRUE - if the page is generated.
# @return  FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub nettools_disable()
# @brief   Disable network tools
# @details This method tries to disable network tools.
# @return  TRUE - the network tools are disabled successfully
# @return  FALSE - some error happens
sub nettools_disable()
{
   baselog($LOG_DEBUG, "Trying to disable network tools...");

   my $retval = $TRUE;

   if (not disable_trace_commands())
   {
      baselog($LOG_ERROR, "Cannot disable trace[path|route] commands.");
      return $FALSE;
   }

   foreach my $rpm (sort keys %un_rpms)
   {
      if (not disable_rpm($rpm))
      {
         baselog($LOG_ERROR, "Cannot disable RPMS packages.");
         $retval = $FALSE;
         next;
      } ## end if (not disable_rpm($rpm...
   } ## end foreach my $rpm (sort keys ...

   return $retval;  # return value must be a scalar value
} ## end sub nettools_disable()

###############################################################################
## @fn sub disable_trace_commands()
# @brief   Disable /bin/trace* commands for non root users
# @details The method changes parmissions of trace commands.
# @return  TRUE - commands are disabled successfully
# @return  FALSE - some error happens
sub disable_trace_commands()
{
   baselog($LOG_DEBUG, "Trying to disable /bin/trace* commands...");

   if (not change_permissions_and_ownership(0700, undef, undef, [$trace_commands{path}, $trace_commands{path6}], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of trace commands.");
      return $FALSE;
   }
   if (not change_permissions_and_ownership(04700, undef, undef, [$trace_commands{route}, $trace_commands{route6}], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of trace commands.");
      return $FALSE;
   }
   return $TRUE;  # return value must be a scalar value
} ## end sub disable_trace_commands()

###############################################################################
## @fn sub enable_trace_commands()
# @brief   Enable /bin/trace* commands for non root users
# @details The method changes permissions of trace commands.
# @return  TRUE - commands are enabled successfully
# @return  FALSE - some error happens
sub enable_trace_commands()
{
   baselog($LOG_DEBUG, "Trying to enable /bin/trace* commands...");

   if (not change_permissions_and_ownership(0755, undef, undef, [$trace_commands{path}, $trace_commands{path6}], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of trace commands.");
      return $FALSE;
   }
   if (not change_permissions_and_ownership(04755, undef, undef, [$trace_commands{route}, $trace_commands{route6}], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of trace commands.");
      return $FALSE;
   }
   return $TRUE;  # return value must be a scalar value
} ## end sub enable_trace_commands()

###############################################################################
## @fn sub enable_rpm($rpm)
# @brief   It Installs RPM package
# @details The method performes the following actions in order to install RPM package:
#\li          1. check if it is already installed
#\li          2. check rpm source package available
#\li          3. check binary file
#\li          4. Try to install rpm
# @param   $rpm  - key of 'rpms' hash;
# @return  TRUE - rpms are enabled
# @return  FALSE - some error happens
sub enable_rpm($)
{
   my $rpm = shift @_;

   if ((not defined $rpm) or ($rpm eq ""))
   {
      baselog($LOG_ERROR, "rpm package index should be defined.");
      return $FALSE;
   }

   my $rpm_name = $rpms{$rpm}->{'name'};

   baselog($LOG_DEBUG, "Trying to install '$rpm_name' RPM...");

   my $rpm_installed = is_rpm_installed($rpm_name);

   if ($rpm_installed == $H_ERROR)
   {
      baselog($LOG_ERROR, "Cannot check if RPM '$rpm name' is installed.");
      return $FALSE;
   }
   elsif ($rpm_installed == $TRUE)
   {
      baselog($LOG_INFO, "RPM '$rpm_name' is already installed.");
      return $TRUE;
   }

   my $retval = $FALSE;

   if (prepare_to_install($rpm_name))
   {
      if (install_rpm($rpm_name, $hardening_rpm_storage))
      {
         if (post_install($rpm_name))
         {
            $retval = $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "Cannot make RPM post-installation actions.");
         }
      } ## end if (install_rpm($rpm_name...
   } ## end if (prepare_to_install...
   else
   {
      baselog($LOG_ERROR, "Cannot make RPM pre-installation actions.");
   }

   return $retval;
} ## end sub enable_rpm($)

###############################################################################
## @fn sub disable_rpm ($rpm)
# @brief   Uninstall RPM package
# @details The method performes the following actions in order to uninstall RPM package:
# \li         1. check if it is already uninstalled
# \li         3. check binary file
# \li         4. Try to uninstall rpm
# @param   $rpm - key of 'un_rpms' hash
# @return  TRUE - rpm is uninstalled
# @return  FALSE - some error happens
sub disable_rpm($)
{
   my $rpm = shift @_;

   if ((not defined $rpm) or ($rpm eq ""))
   {
      baselog($LOG_ERROR, "rpm package index should be defined.");
      return $FALSE;
   }

   my $rpm_name = $un_rpms{$rpm}->{'name'};
   my $command  = '';
   if (exists $un_rpms{$rpm}->{'command'})
   {
      $command = $un_rpms{$rpm}->{'command'};
   }

   baselog($LOG_DEBUG, "Trying to uninstall '$rpm_name' RPM...");

   my $rpm_installed = is_rpm_installed($rpm_name);

   if ($rpm_installed == $H_ERROR)
   {
      baselog($LOG_ERROR, "Cannot check if RPM '$rpm_name' is installed.");
      return $FALSE;
   }
   elsif ($rpm_installed == $FALSE)
   {
      baselog($LOG_INFO, "RPM '$rpm_name' is not installed.");

      my $ret = $TRUE;
      if ($command ne '')
      {
         $ret = post_uninstall($command);
      }

      if (not $ret)
      {
         baselog($LOG_ERROR, "Cannot make RPM post-uninstallation actions.");
      }

      return $ret;
   } ## end elsif ($rpm_installed == ...

   my $retval = $FALSE;

   if (prepare_to_uninstall($rpm_name))
   {
      if (uninstall_rpm($rpm_name))
      {
         $retval = $TRUE;

         if ($command ne '')
         {
            if (not post_uninstall($command))
            {
               baselog($LOG_ERROR, "Cannot make RPM post-uninstallation actions.");
               $retval = $FALSE;
            }
         } ## end if ($command ne '')
      } ## end if (uninstall_rpm($rpm_name...
   } ## end if (prepare_to_uninstall...
   else
   {
      baselog($LOG_ERROR, "Cannot make RPM pre-uninstallation actions.");
   }

   return $retval;
} ## end sub disable_rpm($)

##############################################################################
## @fn sub is_rpm_installed($rpm)
# @brief   Check if provided rpm is installed
# @details The method checks if provided RPM package is installed by using
#          'rpm -q' query.
# @param   $rpm - RPM name
# @return  TRUE    - RPM is installed on the system
# @return  FALSE   - not installed
# @return  H_ERROR - some error happens
sub is_rpm_installed($)
{
   my $rpm = shift @_;

   if (not defined $rpm)
   {
      baselog($LOG_ERROR, "RPM name should be defined.");
      return $H_ERROR;
   }
   baselog($LOG_DEBUG, "Check if RPM is installed.");

   my $retval = $FALSE;

   if (not do_system([$RPM, "-q", $rpm], {$dsNOERROR => $TRUE, $dsRETVAL => \$retval}))
   {
      baselog($LOG_ERROR, "RPM query cannot be performed for the package '$rpm'.");
      return $H_ERROR;
   }

   if ($retval == $OK)
   {
      baselog($LOG_DEBUG, "RPM '$rpm' is installed.");
      return $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "RPM '$rpm' is not installed.");
      return $FALSE;
   }
} ## end sub is_rpm_installed($)

##############################################################################
## @fn sub find_rpm_in_storage ($name, $path_rpm)
# @brief   Find all rpms with the given name in hardening RPM storage
# @details The method looks for all rpms files in hardening RPM storage. The
#          name of found RPM files is similar to the given one.
# @param   $name - RPM name
# @param   $path_rpm - path to the file storage
# @return  reference to a hash which contain pathes to RPM files and the version
#          of these packages
# @return  undef - RPM is not in the storage
sub find_rpm_in_storage($$)
{
   my ($name, $path_rpm) = @_;

   if ((not defined $name) or ($name eq ''))
   {
      baselog($LOG_ERROR, "RPM name should be defined.");
      return undef;
   }

   if ((not defined $path_rpm) or ($path_rpm eq ''))
   {
      baselog($LOG_ERROR, "Path to the RPM should be defined.");
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to get RPM '$name' from dir '$path_rpm'.");

   my %rpm_path_list = ();
   my @output        = ();
   my $fullpath;
   my $line;
   my $package_version;
   my $package_name;

   #find all RPM files which name contains the needed string
   finddepth(
      sub {

         #$_ contains the file name rather than fullpath
         if ($_ =~ /^$name.*rpm$/)
         {
            baselog($LOG_DEBUG, "RPM '$File::Find::name' containing '$name' in the filename");

            $fullpath = $File::Find::name;

            @output = ();
            if (not do_system([$RPM, '-qp', '--qf', '%{NAME} %{VERSION}-%{RELEASE}\n', '--nosignature', $fullpath], {$dsOUT => \@output}))
            {
               baselog($LOG_ERROR, "Cannot get information for the package '$fullpath'.");
            }
            else
            {

               #Handle only first line (appearing of other lines is considered
               #as an error)
               if (scalar(@output) > 1)
               {
                  baselog($LOG_ERROR, "Incorrect format of information for the package '$fullpath'.");
               }
               else
               {

                  #Check if the name from RPM header is the same as the required name
                  $line = $output[0];
                  chomp $line;

                  baselog($LOG_DEBUG, "The output for RPM query is: '$line'");

                  ($package_name, $package_version) = split /\s+/, $line;
                  if ($package_name eq $name)
                  {
                     $rpm_path_list{$fullpath}->{'name'}    = $package_name;
                     $rpm_path_list{$fullpath}->{'version'} = $package_version;

                     baselog($LOG_DEBUG, "'$fullpath' provides the package '$name' with version '$package_version'");
                  } ## end if ($package_name eq $name...
                  else
                  {
                     baselog($LOG_DEBUG, "'$fullpath' does not provide the package '$name' - skipped");
                  }
               } ## end else [ if (scalar(@output) > ...
            } ## end else [ if (not do_system([$RPM...
         } ## end if ($_ =~ /^$name.*rpm$/...
      },
      $path_rpm
            );

   if (not scalar(keys %rpm_path_list))
   {
      baselog($LOG_ERROR, "Cannot find RPM for '$name' in the path '$path_rpm'.");
      return undef;
   }

   return \%rpm_path_list;
} ## end sub find_rpm_in_storage($$)

##############################################################################
## @fn sub install_rpm($name , $path_rpm, $is_post_patching)
# @brief   It installs RPM package
# @details The method gets the last version of provided RPM from the provided
#          file storage and installs this RPM.
# @param   $name - rpm name
# @param   $path_rpm - path which contain rpm package
# @param   $is_post_patching -  if is specified and $TRUE current hardening stage is post patching.
# @return  TRUE - RPM is installed
# @return  FALSE - RPM is not installed
sub install_rpm($$;$)
{
   my ($name, $path_rpm, $is_post_patching) = @_;
   if (not defined $name)
   {
      baselog($LOG_ERROR, "RPM name should be defined.");
      return $FALSE;
   }

   if (not defined $path_rpm)
   {
      baselog($LOG_ERROR, "Path to the RPM storage should be defined.");
      return $FALSE;
   }
   my $rpm = get_last_version_rpm($name, $path_rpm);
   if (not defined $rpm)
   {
      baselog($LOG_ERROR, "Cannot get RPM package file for RPM '$name'.");
      return $FALSE;
   }

   if (is_package_installed($rpm))
   {
      if (not defined $is_post_patching or not $is_post_patching)
      {
         baselog($LOG_INFO, "The package '$rpm' is already installed");
      }
      return $TRUE;
   } ## end if (is_package_installed...

   baselog($LOG_DEBUG, "Installing RPM '$name'.");

   if (not do_system([$RPM, "-U", "--oldpackage", $rpm]))
   {
      baselog($LOG_ERROR, "Cannot install RPM '$rpm'.");
      return $FALSE;
   }

   baselog($LOG_INFO, "RPM '$rpm' is installed successfully.");
   return $TRUE;
} ## end sub install_rpm($$;$)

##############################################################################
## @fn sub get_last_version_rpm($name, $path)
# @brief   Get RPM package path
# @details If directory provided contains several RPM packages with the same name,
#          this subroutine will return path of newest version of RPM package.
# @param   $name - RPM name
# @param   $path - path to file storage
# @return  path  - path to rpm found
# @return  undef - if some error happens
sub get_last_version_rpm($$)
{
   my ($name, $path) = @_;

   if (not defined $name)
   {
      baselog($LOG_ERROR, "RPM name should be defined.");
      return undef;
   }

   if (not defined $path)
   {
      baselog($LOG_ERROR, "Path to the RPM should be defined.");
      return undef;
   }

   baselog($LOG_DEBUG, "Trying to get last version of RPM package '$name' from dir '$path'.");

   my $rpm_path_list = find_rpm_in_storage($name, $path);
   if (not defined $rpm_path_list)
   {
      return undef;
   }

   #sort packages by the version and get the package path of the last rpm version.
   my @versions_list = sort {$rpm_path_list->{$b}->{'version'} cmp $rpm_path_list->{$a}->{'version'}} keys(%{$rpm_path_list});
   my $ret_rpm       = $versions_list[0];

   baselog($LOG_DEBUG, "The newest version of the RPM package '$name' is in the file '$ret_rpm'.");

   return $ret_rpm;
} ## end sub get_last_version_rpm($$)

##############################################################################
## @fn sub uninstall_rpm($name)
# @brief   Un-install RPM package
# @details The method uninstalls provided RPM package
# @param   $name - RPM name
# @return  TRUE -  RPM is uninstalled
# @return  FALSE - RPM is not uninstalled
sub uninstall_rpm($)
{
   my $name = shift @_;
   if (not defined $name)
   {
      baselog($LOG_ERROR, "RPM name should be defined.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Uninstalling RPM '$name'.");

   if (not do_system([$RPM, "-e", $name]))
   {
      baselog($LOG_ERROR, "Cannot uninstall RPM '$name'.");
      return $FALSE;
   }

   baselog($LOG_INFO, "RPM '$name' is uninstalled successfully.");
   return $TRUE;
} ## end sub uninstall_rpm($)

##############################################################################
## @fn sub prepare_to_install($cmd)
# @brief   It makes preparation actions before installing RPM
# @details The method removes provided file.
# @param   $cmd - command path
# @return  TRUE - file is removed successfully
# @return  FALSE - some error happens
sub prepare_to_install($)
{
   my $cmd = shift;
   if (not defined $cmd)
   {
      baselog($LOG_ERROR, "command should be defined.");
      return $FALSE;
   }

   #  if command exists - remove it
   if (-e $cmd)
   {
      if (not unlink($cmd))
      {
         baselog($LOG_ERROR, "Cannot remove '$cmd' tool.");
         return $FALSE;
      }
   } ## end if (-e $cmd)

   # if provided command is /usr/sbin/tshark
   if ($cmd eq $rpms{1}->{command})
   {
      if (not unlink $tethereal)
      {
         baselog($LOG_ERROR, "Cannot remove '$tethereal' tool.");
         return $FALSE;
      }
   } ## end if ($cmd eq $rpms{1}->...
   return $TRUE;
} ## end sub prepare_to_install($)

##############################################################################
## @fn sub prepare_to_uninstall($rpm)
# @brief   Make preparation actions before uninstalling RPM
# @param  $rpm - rpm name
# @return  TRUE - operation is performed successfully
# @return  FALSE - some error happens
# @todo    to fill up
sub prepare_to_uninstall($)
{

   #TODO to fill up
   my $rpm = shift;
   return $TRUE;

} ## end sub prepare_to_uninstall($)

##############################################################################
## @fn sub post_uninstall ($cmd)
# @brief   It makes actions after uninstalling RPM
# @details The method restricts using of provided command. It changes the content
#          according to $restricted_data variable and changes permissions of the file
#          to make it executable.
# @param   $cmd - file path;
# @return  TRUE - operation is performed successfully
# @return  FALSE - some error happens
sub post_uninstall($)
{
   my $cmd = shift;
   if (not defined $cmd)
   {
      baselog($LOG_ERROR, "command should be defined.");
      return $FALSE;
   }

   #  create specific command
   if (not write_harden_file($cmd, $restricted_data))
   {
      baselog($LOG_ERROR, "Cannot create default '$cmd' command.");
      return $FALSE;
   }
   if (not change_permissions_and_ownership(0755, undef, undef, [$cmd], undef, $NO_LOG))
   {
      baselog($LOG_ERROR, "Cannot change permissions of '$cmd' command.");
      return $FALSE;
   }

   # if provided command is /usr/sbin/tshark
   if ($cmd eq $rpms{1}->{command})
   {
      if (not do_system([$LN, "-s", $cmd, $tethereal]))
      {
         baselog($LOG_ERROR, "Cannot create symbolic link '$tethereal' to '$cmd' command.");
      }
   } ## end if ($cmd eq $rpms{1}->...
   return $TRUE;
} ## end sub post_uninstall($)

##############################################################################
## @fn sub post_install ($rpm)
# @brier   It makes actions after installing RPM
# @param $rpm - rpm name
# @return  TRUE - operation is performed successfully
# @return  FALSE - some error happens
# @todo    to fill up
sub post_install($)
{

   #TODO to fill up
   my $rpm = shift;
   return $TRUE;
} ## end sub post_install($)

###############################################################################
## @fn sub nettools_enable()
# @brief   Enable network tools
# @details This method tries to enable network tools.
# @return  TRUE - the network tools are enabled successfully
# #return  FALSE - some error happens
sub nettools_enable()
{
   baselog($LOG_DEBUG, "Trying to enable network tools...");

   my $retval = $TRUE;
   if (not enable_trace_commands())
   {
      baselog($LOG_ERROR, "Cannot enable trace[path|route] commands.");
      return $FALSE;
   }
   foreach my $rpm (sort keys %rpms)
   {
      if (not enable_rpm($rpm))
      {
         baselog($LOG_ERROR, "Cannot enable '$rpms{$rpm}->{name}' tool.");
         $retval = $FALSE;
         next;
      } ## end if (not enable_rpm($rpm...
   } ## end foreach my $rpm (sort keys ...

   return $retval;  # return value must be a scalar value
} ## end sub nettools_enable()

###############################################################################
## @fn sub off()
# @brief   Disable usage of network tools
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform disable function of nettools
#          and sends the corresponding SNMP trap.
# @return  TRUE - the new status is set
# @return  FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to disable network tools..");

   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&nettools_disable, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Network tools are disabled.\n", $TRUE);

      send_snmp_trap("SEC0092");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot disable network tools.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief   Enable usage of network tools
# @details This method tries to change the status of the Hardening option to "on"
#          if it is necessary, then enables network tools and then
#          sends the corresponding SNMP trap.
# @return  TRUE - the new status is set
# @return  FALSE - some error happens or cancel by user
sub on()
{
   baselog($LOG_DEBUG, "Trying to enable network tools..");

   my $retval = $FALSE;
   if (not is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'network tools' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (not is_option_permitted...
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&nettools_enable, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Network tools are enabled.\n", $TRUE);

      send_snmp_trap("SEC0093");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot enable network tools.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief   Apply default state of usage of network tools
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it disables usage of network tools.
#          Default value is applied in the case when it is impossible to detect
#          the current network tools usage policy.
#          When either the system is fresh installed or there are no restored
#          Hardening data, usage of network tools are disabled.
# @return  TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&nettools_disable, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Network tools are disabled.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change the status of network tools.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is intended to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got successfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of Network Analysis Tools is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "Network Analysis Tools are permitted.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Network Analysis Tools are forbidden.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in accordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of nettools is enabled or not.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of nettools service from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of usage of nettools service is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status eq $TRUE)
   {
      $retval = nettools_enable();
   }
   else
   {
      $retval = nettools_disable();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_nettools_state()
# @brief Send SNMP trap in accordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of nettools service is enabled or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_nettools_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of nettools service.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of usage of nettools service is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0093");
   }
   else
   {
      $retval = send_snmp_trap("SEC0092");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_nettools_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if usage of nettools service is enabled or not, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored successfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of network tools...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_nettools_state();

   return $retval;
} ## end sub recovery()

###############################################################################
## @fn post_patching()
# @brief Provides post patching actions
# @details Installs last rpms versions if they are not installed yet after patching.
# @return TRUE - Network tools are updated successfully
# @return FALSE - some issues happened
sub post_patching()
{
   my $retval = $TRUE;

   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of Network Analysis Tools is unknown.");
      $retval = $FALSE;
   }
   elsif ($status)
   {
      my $rpm_name = "";
      foreach my $rpm (keys %rpms)
      {
         $rpm_name = $rpms{$rpm}->{'name'};
         if (not install_rpm($rpm_name, $hardening_rpm_storage, $TRUE))
         {
            baselog($LOG_ERROR, "Error installing rpm '$rpm_name'");
            $retval = $FALSE;
         }
      } ## end foreach my $rpm (keys %rpms...
   } ## end elsif ($status)

   return $retval;
} ## end sub post_patching()

1;
__END__
