###############################################################################
## @file passwd_days.pm
# @brief Provides hardening actions for CS1000 password lifetime parameters
#        changing
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 passwd_days (password lifetime
#          parameters) and the Enhanced  Hardening module 'passwd_days'
#          are requested.
#
# \li         It is responsible for the CLI actions:
# \li          harden passwd_days
# \li          harden passwd_days on
# \li          harden passwd_days off
# \li          harden passwd_days set -max value
# \li          harden passwd_days set -min value
# \li          harden passwd_days status
# \li          harden passwd_days help
# @author Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage
# @version 1.03 - fixed small bug with lchage command(added full path)
# @version 1.04 - changing password expiration options for all accounts
#                 (from base_accounts) has been added, change_values_for_accounts
#                 function has been changed; get_non_block_accounts and
#                 get_block_accounts functions are added

=pod

=head1 NAME

=over 4

=item B<harden passwd_days> - command to manage changing of CS1000 password lifetime parameters.

=back

=head1 SYNOPSIS

=over 10

=item B<harden passwd_days> I<on> | I<off> | I<status> | I<help>

=item B<harden passwd_days> I<set> -max|-min <value>  

=back

=head1 DESCRIPTION

B<harden passwd_days> module allows changing of CS1000 password lifetime parameters.
Next parameters are affected by this command: PASS_MAX_DAYS - maximum number of days a password may be used.
PASS_MIN_DAYS - minimum number of days allowed between password changes. 

=head1 OPTIONS

=over 4

=item B<on>

Synchronize system values of password lifetime parameters with values from haredning storage. 
Allow values of password lifetime parameters have the previously configured values by B<harden set> command.  
The SNMP trap is sent to track this event by SNMP manager.

B<NOTE>: This action performs system changes of password lifetime parameters. If there were
not previously configured parameters, it applies default values of password lifetime parameters.

=item B<off>

It changes values of password lifetime parameters to the default.
The SNMP trap is sent to track this event by SNMP manager.

B<NOTE>: This action performs system changes of password lifetime parameters.

=item B<status>

Outputs the status of password lifetime parameters. 

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden passwd_days> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden passwd_days on>

 The hardening status of passwd_days is set to 'on' state. 

=item B<$ harden passwd_days off>

 The hardening status of passwd_days is set to 'off' state. 

=item B<$ harden passwd_days status>

 Hardening status of passwd_days is set to 'off'.
 The values of password lifetime parameters are set to PASS_MAX_DAYS->'89' days, PASS_MIN_DAYS->'2' days.

=item B<$ harden passwd_days set -max 78> 

 The status of password lifetime parameters is set to off. New values will be applied after changing the status to 'on'.

=item B<$ harden passwd_days set -min 3>

 New value of the minimum number of days allowed between password changes is set to '3' days.

=item B<$ harden passwd_days set -min 8 -max 45>

 New values of password lifetime parameters are set to PASS_MAX_DAYS->'45' days, PASS_MIN_DAYS->'8' days.

=back

=head1 CAVEATS

B<harden passwd_days> must be run only under a high privileges account.

=head1 SEE ALSO

B</etc/login.defs>

=cut

## @class passwd_days
package hardening::plugins::passwd_days;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";

use common_variables;
use base_system;
use Getopt::Long;
use baselog;
use hardening::hardening_common;
use base_snmp;
use base_accounts;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.04';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub set();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_passwd_days_state();
## @skipline
sub recovery();
## @skipline
sub parse_command_line_arguments($);
## @skipline
sub set_customized_values($);
## @skipline
sub validate_input_params($);
## @skipline
sub set_values_to_conf_file($$);
## @skipline
sub change_values_for_accounts($$);
## @skipline
sub set_default_values_to_system();
## @skipline
sub set_default_values_to_system_and_storage();
## @skipline
sub synch_conf_file_with_storage();
## @skipline
sub get_values_from_storage();
## @skipline
sub set_values_to_storage($$);

#variables
my $module_name     = "passwd_days";
my $login_defs_file = "/etc/login.defs";
my $pass_max_days   = "PASS_MAX_DAYS";
my $pass_min_days   = "PASS_MIN_DAYS";
my $default_max     = 90;
my $default_min     = 1;

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'set' => {
                         'sub'             => \&set_customized_values,
                         'validate_params' => \&validate_input_params,
                         'visible'         => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_passwd_days_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine
#          passworf lifetime parameters values, and outputs this status.
# @return TRUE - if the status of the Hardening option is got.
# @return FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status of passwd_days...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "Hardening status of passwd_days is set to 'on'.\n", $TRUE);
      my ($max, $min) = get_values_from_storage();
      if ((not defined $max) or (not defined $min))
      {
         baselog($LOG_ERROR, "Cannot get values from hardening storage.");
         return $FALSE;
      }
      baselog($LOG_INFO, "The values of password lifetime parameters are set to PASS_MAX_DAYS->'$max' days, PASS_MIN_DAYS->'$min' days.\n", $TRUE);

      $retval = $TRUE;
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "Hardening status of passwd_days is set to 'off'.\n", $TRUE);
      my ($max, $min) = get_values_from_storage();
      if ((not defined $max) or (not defined $min))
      {
         baselog($LOG_ERROR, "Cannot get values from hardening storage.");
         return $FALSE;
      }
      baselog($LOG_INFO, "The values of password lifetime parameters are set to PASS_MAX_DAYS->'$max' days, PASS_MIN_DAYS->'$min' days.\n", $TRUE);

      $retval = $TRUE;
   } ## end elsif ($status == $FALSE)
   else
   {
      baselog($LOG_ERROR, "Hardening status of passwd_days is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub off()
# @brief   It reapplies password lifetime parameters
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform changing values of password lifetime
#          parameters and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to apply default values of password lifetime parameters...");

   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&set_default_values_to_system);

   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The hardening status of passwd_days is set to 'off' state.\n", $TRUE);

      send_snmp_trap("SEC0068", {'maxdays' => $default_max});
      send_snmp_trap("SEC0069", {'mindays' => $default_min});

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change password lifetime parameters.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief  It reapplies password lifetime parameters
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to apply configured values of password lifetime parameter...");

   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&synch_conf_file_with_storage, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "The hardening status of passwd_days is set to 'on' state.\n", $TRUE);

      my ($max, $min) = get_values_from_storage();
      if (not defined $max or not defined $min)
      {
         baselog($LOG_ERROR, "Cannot get values from storage.");
         return $FALSE;
      }
      send_snmp_trap("SEC0068", {'maxdays' => $max});
      send_snmp_trap("SEC0069", {'mindays' => $min});

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change values of password lifetime parameters.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub set_customized_values($params)
# @brief  It sets provided values of password lifetime parameters
# @details The method makes the following actions in order to setup customized
#          values of password lifetime parameters
# \li      1. check hardening storage values and provided values
# \li      2. update configuration file with customized values
#          of PASS_MAX_DAYS and PASS_MIN_DAYS params
# \li      3. update harending storage with customized values
#          of PASS_MAX_DAYS and PASS_MIN_DAYS params
# \li      4. print status
# $params  $params - command line argumetns
# @return TRUE -  new values are applied successfully
# @return FALSE - some error happens
sub set_customized_values($)
{
   my $params = \@_;

   baselog($LOG_DEBUG, "Trying to handle parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      baselog($LOG_ERROR, "Incorrect syntax.");
      return $FALSE;
   }

   my ($max,      $min);
   my ($max_flag, $min_flag) = ($TRUE, $TRUE);

   if (not defined $parsed_params->{max})
   {
      ($max, undef) = get_values_from_storage();
      $min      = $parsed_params->{min};
      $max_flag = $FALSE;
   } ## end if (not defined $parsed_params...
   elsif (not defined $parsed_params->{min})
   {
      (undef, $min) = get_values_from_storage();
      $max      = $parsed_params->{max};
      $min_flag = $FALSE;
   } ## end elsif (not defined $parsed_params...
   else
   {
      $max = $parsed_params->{max};
      $min = $parsed_params->{min};
   }

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $TRUE)
   {
      if (not set_values_to_conf_file($max, $min))
      {
         baselog($LOG_ERROR, "Cannot set values of parameters to conf file.\n", $TRUE);
         return $FALSE;
      }
      if (not set_values_to_storage($max, $min))
      {
         baselog($LOG_ERROR, "Cannot set values of parameters to hardening storage.\n", $TRUE);
         return $FALSE;
      }
      if (not $min_flag)
      {
         baselog($LOG_INFO, "New value of the maximum number of days a password may be used is set to '$max' days.\n", $TRUE);
         send_snmp_trap("SEC0068", {'maxdays' => $max});

      } ## end if (not $min_flag)
      elsif (not $max_flag)
      {
         baselog($LOG_INFO, "New value of the minimum number of days allowed between password changes is set to '$min' days.\n", $TRUE);
         send_snmp_trap("SEC0069", {'mindays' => $min});

      } ## end elsif (not $max_flag)
      else
      {
         baselog($LOG_INFO, "New values of password lifetime parameters are set to PASS_MAX_DAYS->'$max' days, PASS_MIN_DAYS->'$min' days.\n", $TRUE);
         send_snmp_trap("SEC0068", {'maxdays' => $max});
         send_snmp_trap("SEC0069", {'mindays' => $min});

      } ## end else [ if (not $min_flag)
      $retval = $TRUE;

   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      if (not set_values_to_storage($max, $min))
      {
         baselog($LOG_ERROR, "Cannot set values of parameters to hardening storage.");
         return $FALSE;
      }

      baselog($LOG_WARNING, "The status of password lifetime parameters is set to off. New values will be applied after changing the status to 'on'.\n", $TRUE);
      $retval = $TRUE;
   } ## end elsif ($status == $FALSE)
   else
   {
      baselog($LOG_ERROR, "The status of password lifetime parameters is unknown.\n", $TRUE);
      $retval = $FALSE;

   } ## end else [ if ($status == $TRUE)

   return $retval;
} ## end sub set_customized_values($)

###############################################################################
## @fn sub parse_command_line_arguments($params)
# @brief Parses command line parameters.
# @details This module parses command line parameters by GetOptions and fills
#          the hash with parsed data.
# @param $params - reference to array with command line parameters
# @return \%parsed_params - reference to hash with parsed data
# @return  undef - illegal command line parameters (e.g. wrong syntax)
sub parse_command_line_arguments($)
{
   my $params = shift @_;

   local @ARGV = @$params;

   my %parsed_params = ('max' => undef, 'min' => undef);

   my $res = GetOptions(\%parsed_params, 'max=i', 'min=i');
   if ((not $res) or (scalar(@ARGV) > 0))
   {
      return undef;
   }

   return \%parsed_params;
} ## end sub parse_command_line_arguments($)

###############################################################################
## @fn sub validate_input_params($params)
# @brief Parses command line parameters.
# @details This module parses command line parameters by GetOptions and fills
#          the hash with parsed data.
# @param $params - reference to array with command line parameters
# @return \%parsed_params - reference to hash with parsed data
# @return  undef - illegal command line parameters (e.g. wrong syntax)
sub validate_input_params($)
{

   my $params = \@_;
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Trying to verify parameters: '@{$params}'");

   my $parsed_params = parse_command_line_arguments($params);
   if (not defined $parsed_params)
   {
      print STDERR "Incorrect syntax. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   if ((not defined $parsed_params->{max}) and (not defined $parsed_params->{min}))
   {
      print STDERR "Parameters parsing error. Please read 'harden $module_name help'.\n";
      return $FALSE;
   }

   if (defined $parsed_params->{max})
   {
      if ($parsed_params->{max} <= $default_max and $parsed_params->{max} > 0)
      {
         $retval = $TRUE;
      }
      else
      {
         print STDERR "The maximum number of days a password may be used must be greater than 0 and less or equal than $default_max days.\n";
         $retval = $FALSE;
      }
   } ## end if (defined $parsed_params...

   if (defined $parsed_params->{min})
   {
      if ($parsed_params->{min} >= $default_min)
      {
         $retval = $TRUE;
      }
      else
      {
         print STDERR "The minimum number of days allowed between password changes be greater than $default_min days.\n";
         $retval = $FALSE;
      }
   } ## end if (defined $parsed_params...

   return $retval;
} ## end sub validate_input_params($)
###############################################################################
## @fn sub set_values_to_conf_file($max_value, $min_value)
# @brief Update configuration file with provided values
# @details The method makes the following actions in order to reapply configuration
#          file (/etc/login.defs) with customized values of password lifetime
#          parameters
#      \li 1. open file
#      \li 2. replace content of /etc/login.defs file with customized values
#      \li of PASS_MAX_DAYS and PASS_MIN_DAYS params
#      \li 3. save file
# @param   $max_value - value of PASS_MAX_DAYS parameter
# @param   $min_value - value of PASS_MIN_DAYS parameter
# @return TRUE - configuration file is updated successfully
# @return FALSE - some error happens
sub set_values_to_conf_file($$)
{
   my ($max, $min) = @_;
   if ((not defined $max) or (not defined $min))
   {
      baselog($LOG_ERROR, "Parameters should be defined.");
      return $FALSE;
   }
   my $file;
   my @orig_content = ();

   baselog($LOG_DEBUG, "Reappling '$login_defs_file' file.");

   #get content of $login_defs_file file
   if (not open($file, '<', $login_defs_file))
   {
      baselog($LOG_ERROR, "Cannot open $login_defs_file file: $!");
      return $FALSE;
   }
   @orig_content = <$file>;
   if (not close($file))
   {
      baselog($LOG_ERROR, "Cannot close $login_defs_file file: $!");
   }

   my @new_content = ();
   my $max_flag    = $FALSE;
   my $min_flag    = $FALSE;

   #replace values of PASS_MAX_DAYS and PASS_MIN_DAYS params
   foreach my $str (@orig_content)
   {
      if ($str =~ /^\s*$pass_max_days\s*/)
      {
         push @new_content, $pass_max_days . "  " . $max . "\n";
         $max_flag = $TRUE;
         next;
      } ## end if ($str =~ /^\s*$pass_max_days\s*/...
      if ($str =~ /^\s*$pass_min_days\s*/)
      {
         push @new_content, $pass_min_days . "  " . $min . "\n";
         $min_flag = $TRUE;
         next;
      } ## end if ($str =~ /^\s*$pass_min_days\s*/...
      push @new_content, $str;
   } ## end foreach my $str (@orig_content...

   #if there is no pass_max_days string - add it
   if (not $max_flag)
   {
      push @new_content, $pass_max_days . "  " . $max . "\n";
   }

   #if there is no pass_min_days string - add it
   if (not $min_flag)
   {
      push @new_content, $pass_min_days . "  " . $min . "\n";
   }

   #some checking
   if ((not scalar(@new_content)) or (not defined $new_content[0]))
   {
      baselog($LOG_ERROR, "Cannot get information from '$login_defs_file' file.");
      return $FALSE;
   }

   #save new $login_defs_file
   if (not write_harden_file($login_defs_file, \@new_content))
   {
      baselog($LOG_ERROR, "Cannot write '$login_defs_file' file.");
      return $FALSE;
   }

   #re-apply new values for all existent login account
   if (not change_values_for_accounts($max, $min))
   {
      return $FALSE;
   }
   return $TRUE;
} ## end sub set_values_to_conf_file($$)

## @fn get_non_block_accounts()
# @brief returns reference to list names of Linux_FTP_User and
#        Linux_FTP_SFTP_User and Linux_FTP_SFTP_Users users
# @return reference to list
sub get_non_block_accounts()
{
   return [@{get_accounts_by_type($LINUX_FTP_USER)}, @{get_accounts_by_type($LINUX_FTP_SFTP_USER)}, @{get_accounts_by_type($LINUX_SFTP_USER)}];
}

## @fn get_block_accounts()
# @brief returns reference to list names of Linux_FTP_User and
#        Linux_FTP_SFTP_User and Linux_FTP_SFTP_Users users
# @return reference to list
sub get_block_accounts()
{
   return [@{get_accounts_by_type($UCM_USER)}, @{get_accounts_by_type($LINUX_USER)}];
}

###############################################################################
## @fn sub change_values_for_accounts($max, $min)
# @brief It changes password lifetime parameters values for accounts
# @details  The method applies specified password lifetime parameters values
#           for all accounts from base_accounts + root and nortel.
# @param $max - value of PASS_MAX_DAYS parameter
# @param $min - value of PASS_MIN_DAYS parameter
# @return TRUE -  values of parameters are applied successfully
# @return FALSE - some error happens
sub change_values_for_accounts($$)
{

   my ($max, $min) = @_;
   if ((not defined $max) or (not defined $min))
   {
      baselog($LOG_ERROR, "Parameters should be defined.");
      return $FALSE;
   }
   my $non_block_accounts = get_non_block_accounts();
   my $block_accounts     = get_block_accounts();

   #security vulnerability - password of nortel account never expires;
   #this should be considered as exception;
   push @$non_block_accounts, $NORTEL_ACCT;

   #password of root accout expires as usual
   push @$block_accounts, $ROOT_ACCT;

   #apply new values of password lifetime parameters
   foreach my $account (@$non_block_accounts)
   {
      if (not do_system([$LCHAGE, '-M', $PASS_UNLIM_VALUE, '-m', $min, $account]))
      {
         baselog($LOG_ERROR, "Reapplying values for account '$account' falied.");
         return $FALSE;
      }
   } ## end foreach my $account (@$non_block_accounts...
   foreach my $account (@$block_accounts)
   {
      if (not do_system([$LCHAGE, '-M', $max, '-m', $min, $account]))
      {
         baselog($LOG_ERROR, "Reapplying values for account '$account' falied.");
         return $FALSE;
      }
   } ## end foreach my $account (@$block_accounts...

   return $TRUE;
} ## end sub change_values_for_accounts($$)

###############################################################################
## @fn sub set_default_values_to_system()
# @brief   It sets default values of password lifetime parameters to the
#          configuration file.
# @details The method replaces content of /etc/login.defs configuration file
#          with defualt values of PASS_MAX_DAYS and PASS_MIN_DAYS params
# @return  TRUE - values of parameters are reapplied successfully
# @return  FALSE - some error happens
sub set_default_values_to_system()
{
   if (not set_values_to_conf_file($default_max, $default_min))
   {
      baselog($LOG_ERROR, "Cannot set values of parameters to conf file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub set_default_values_to_system()

###############################################################################
## @fn sub set_default_values_to_system_and_storage()
# @brief   It sets default values of password lifetime parameters to the
#          configuration file and to the hardening storage.
# @details The method replaces content of /etc/login.defs configuration file
#          with defualt values of PASS_MAX_DAYS and PASS_MIN_DAYS params, The same
#          values are presented in the hardening storage.
# @return  TRUE - values of parameters are reapplied successfully
# @return  FALSE - some error happens
sub set_default_values_to_system_and_storage()
{
   if (not set_values_to_conf_file($default_max, $default_min))
   {
      baselog($LOG_ERROR, "Cannot set values of parameters to conf file.");
      return $FALSE;
   }

   if (not set_values_to_storage($default_max, $default_min))
   {
      baselog($LOG_ERROR, "Cannot set values of parameters to hardening storage.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub set_default_values_to_system_and_storage()

###############################################################################
## @fn sub synch_conf_file_with_storage()
# @brief   It synchronizes configuration file with values from hardening storage
# @details The method makes the following actions in order to synchronizes
#          the configuration file with values from hardening storage:
# \li      1. open hardening storage
# \li      2. get values of password lifetime parameters from storage
# \li      3. change configuration file according to values form storage
# \li      4. if it is impossible to get values from storage - apply default values to
#          storage and to system
# @return  TRUE - synchronization is performed successfully
# @return  FALSE - some error happens
sub synch_conf_file_with_storage()
{
   my ($max, $min) = get_values_from_storage();
   if ((not defined $max) or (not defined $min))
   {
      baselog($LOG_INFO, "Cannot get values of parameters from hardening storage.");
      baselog($LOG_INFO, "Applying default values of password lifetime parameters.");
      ($max, $min) = ($default_max, $default_min);

      if (not set_values_to_storage($max, $min))
      {
         baselog($LOG_ERROR, "Cannot set values of parameters to hardening storage.");
         return $FALSE;
      }
   } ## end if ((not defined $max)...
   if (not set_values_to_conf_file($max, $min))
   {
      baselog($LOG_ERROR, "Cannot set values of parameters to conf file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub synch_conf_file_with_storage()

###############################################################################
## @fn sub get_values_from_storage()
# @brief  It returns values of parameters from the hardening storage
# @details The method makes the following actions in order to get the values
#          of password lifetime parameters.
#  \li     1. open hardening storage
#  \li     2. get corresponding attributes
# @return  ($max_days, $min_days) - values of password lifetime parameters
# @return  undef - some error happens
sub get_values_from_storage()
{
   baselog($LOG_DEBUG, "Trying to get values of password lifetime parameters.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return undef;
   }

   my $hash_ref = $parser->getAttrs(qq{/$HARDENING_ROOT_ELEM/$module_name});
   if (not defined $hash_ref)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the option '$module_name'.");
      undef $parser;
      return undef;
   } ## end if (not defined $hash_ref...
   undef $parser;

   if (not exists $hash_ref->{$pass_max_days})
   {
      baselog($LOG_ERROR, "The option '$module_name' does not contain the attribute '$pass_max_days'.");
      return undef;
   }
   if (not exists $hash_ref->{$pass_min_days})
   {
      baselog($LOG_ERROR, "The option '$module_name' does not contain the attribute '$pass_min_days'.");
      return undef;
   }

   return ($hash_ref->{$pass_max_days}, $hash_ref->{$pass_min_days});
} ## end sub get_values_from_storage()

###############################################################################
## @fn sub set_values_to_storage($max, $min)
# @brief   It sets values the the hardening storage
# @details The method makes the following actions in order to set the values
#          of password lifetime parameters to the storage.
#  \li     1. open hardening storage
#  \li     2. change corresponding attributes
#  \li     3. save storage
# @param  $max  - value of pass_max_days parameter
# @param  $min  - value of pass_min_days parameter
# @return TRUE - the new values is stored in the storage
# @return FALSE - some error happens
sub set_values_to_storage($$)
{
   my ($max, $min) = @_;

   if ((not defined $max) or (not defined $min))
   {
      baselog($LOG_ERROR, "Values are not specified.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Trying to set values of password lifetime parameters to hardening storage.");

   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }

   my $retval = $FALSE;

   if (not $parser->changeAttrs(qq{/$HARDENING_ROOT_ELEM/$module_name}, undef, {$pass_max_days => $max, $pass_min_days => $min}))
   {
      baselog($LOG_ERROR, "Cannot set values of password lifetime parameters.");
   }
   else
   {
      baselog($LOG_DEBUG, "New values are set for $pass_max_days - $max, for $pass_min_days - $min.");

      save_hardening_storage($parser);
      $retval = $TRUE;
   } ## end else [ if (not $parser->changeAttrs...
   undef $parser;
   return $retval;
} ## end sub set_values_to_storage($$)

###############################################################################
## @fn sub apply_default()
# @brief Apply default values of passwd_days parameters
# @details This method changes the status of the Hardening option in accordance
#          to the default state -
#          Default value is applied in the case when it is impossible to detect
#          the current passwd_days values.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to apply default values of password lifetime parameters...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&set_default_values_to_system_and_storage, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "passwd_days status is off.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change password lifetime parameters.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of password lifetime parameters is unknown."), $TRUE);
      $retval = $FALSE;
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Password lifetime parameters are configured.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine
#          values of password lifetime parameters.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of passwd_days from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of passwd_days is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {
      $retval = synch_conf_file_with_storage();
   }
   else
   {
      $retval = set_default_values_to_system();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_passwd_days_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of passwd_days service is permitted or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_passwd_days_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of passwd_days.");

   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of passwd_days is unknown.");
   }
   elsif ($status == $TRUE)
   {

      my ($max, $min) = get_values_from_storage();
      if (not defined $max or not defined $min)
      {
         baselog($LOG_ERROR, "Cannot get values from storage.");
         return $FALSE;
      }
      $retval |= send_snmp_trap("SEC0068", {'maxdays' => $max});
      $retval |= send_snmp_trap("SEC0069", {'mindays' => $min});

   } ## end elsif ($status == $TRUE)
   else
   {
      $retval |= send_snmp_trap("SEC0068", {'maxdays' => $default_max});
      $retval |= send_snmp_trap("SEC0069", {'mindays' => $default_min});
   }

   return $retval;
} ## end sub send_snmp_with_hardening_passwd_days_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          values of password lifetime parameters, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of passwd_days...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_passwd_days_state();

   return $retval;
} ## end sub recovery()

1;
__END__
