###############################################################################
## @file ssh_filter.pm
# @brief Provides hardening actions for CS1000 SSH filtration service
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 SSH filtration and the Enhanced
#          Hardening module 'ftp' are requested.
#
#          It is responsible for the CLI actions:
#    \li      harden ssh_filter
#    \li       harden ssh_filter on
#    \li       harden ssh_filter off
#    \li       harden ssh_filter status
#    \li       harden ssh_filter help
#    \li       harden ssh_filter add [-allow|-deny] -[ip|subnet|host] [value]
#    \li       harden ssh_filter del [-allow|-deny] -[ip|subnet|host] [value]
#    \li       harden ssh_filter del [-allow|-deny] -num [value]
# @author Petr Kascheev
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage
# @version 1.03 - validateSubnet func is modified to allow subnet in the form of n.n.n.n/m.m.m.m

=pod

=head1 NAME

=over 4

=item B<harden ssh_filter> - command to manage usage of CS1000 SSH filtration.

=back

=head1 SYNOPSIS

=over 10

=item B<harden ssh_filter> I<on> | I<off> | I<status> | I<help> 

=item B<harden ssh_filter> I<add> -allow|-deny -ip|-subnet|-host <address>

=item B<harden ssh_filter> I<del> -deny|-allow -host|-ip|-subnet <address> 

=item B<harden ssh_filter> I<del> -allow|-deny -num <number>

=back

=head1 DESCRIPTION

B<harden ssh_filter> module allows managing usage of CS1000 SSH filtration.

=head1 OPTIONS

=over 4

=item B<on>

Turn on the SSH host filtration. SSH access to the server will be avaliable
only for hosts, which are included into the allowable list.
The SNMP trap is sent to track this event by SNMP manager.

B<NOTE>: If there were not previously configured hosts, 
the warning message is shown indicating that access to the server is restricted 
by SSH. 

=item B<off>

Turn off the SSH host filtration. Access to the server will be avaliable 
for any hosts.
The SNMP trap is sent to track this event by SNMP manager.

=item B<status>

Outputs the status whether CS1000 SSH filtration is turned on/off.
Shows the list of the names of the hosts which are allowed/disallowed connecting
CS1000 system by SSH.

=item B<add>

Add host IP to the allowable or deny list. Connecting access from this host will be permitted 
if SSH filtration is turned on.  

=over 4

=item B<-allow>

This options specifies which list will be used while adding the host.
Host will be added to the allowable list.
  
=item B<-deny>
     
This options specifies which list will be used while adding the host.
Host will be added to the deny list.

=item B<-ip> I<IP_address>

The specified IP address must be reachable. 

=item B<-host> I<host_address>

Host address means hostname or FQDN. 

=item B<-subnet> I<subnet_address>

The specified subnet address must be reachable. The module uses the standard hosts.allow/deny 
mechanizm so the end user is responsible for entering subnet in correct format.
Subnet can be specified by two ways:

1. A string that consist of one, two or three groups of digits separated by a '.' character. 
A host address is matched if its first numeric fields match  the  given  string.   
For  example,  the  pattern '131.155' matches the address of (almost) every host on the Eindhoven University network (131.155.x.x).

2. An expression of the form 'n.n.n.n/m.m.m.m' is interpreted as a 'net/mask' pair. 
An IPv4 host address is matched if 'net' is equal to the bitwise AND of the address and the 'mask'. 
For example, the net/mask pattern '131.155.72.0/255.255.254.0' matches every address in the range '131.155.72.0' through '131.155.73.255'.

=back

=back

=over 4
   
=item B<del> 

Delete host IP from the allowable or deny list. 
Connecting access from this host will be restricted if SSH filtration is turned on.

=over 4

=item B<-allow>

This options specifies which list will be used while deleting the host.
Host will be added to the allowable list.

=item B<-deny>

This options specifies which list will be used while deleting the host.
Host will be added to the deny list.

=item B<-ip> I<IP_address>

The specified IP address must be reachable.

=item B<-host> I<host_address>

Host address means hostname or FQDN.

=item B<-subnet> I<subnet_address>

The specified subnet address must be reachable. The module uses the standard hosts.allow/deny 
mechanizm so the end user is responsible for entering subnet in correct format.

=item B<-num> I<number>

Also del command supports deleting host by number, which B<harden ssh_filter status> prints.
Number is a sequnce number of XML entry. 
 
=back

=back

=over 4
 
=item B<help>

Outputs this manual page. This actions is being assumed if B<harden ssh_filter> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden ssh_filter on>

 SSH filtration is turned on. 

=item B<$ harden ssh_filter off>

 SSH filtration is turned off. 

=item B<$ harden ssh_filter status>

 SSH filtration is turned on.
 SSH allowable list consists of:
 1. IP=192.168.24.56
 2. SUBNET=192.168.23.
 Warning: Please make sure that ECM hosts are added to SSH filtration allowable list.
 
=item B<$ harden ssh_filter add -allow -ip 192.168.35.90>

 Host '192.168.35.90' is successfully added to the SSH list.
 
=item B<$ harden ssh_filter del -allow -ip 192.168.35.90>
 
 Host '192.168.35.90' is successfully deleted from the SSH list.               
 
=item B<$ harden ssh_filter add -allow -subnet 131.155.72.0/255.255.254.0>

 Host '131.155.72.0/255.255.254.0' is successfully added to the SSH list.
                
=back

=head1 CAVEATS B<harden ssh_filter> 

must be run only under a high privileges account.

=head1 SEE ALSO  B</etc/hosts.allow>, B</etc/hosts.deny> 

=cut

## @class ssh_filter
package hardening::plugins::ssh_filter;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";

use common_variables;
use baseParams;
use CommonInstallUtils qw(validateIP getYNResponse);
use base_system;
use baselog;
use base_xml;
use hardening::hardening_common;
use base_snmp;

use Exporter;
use Getopt::Long;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.03';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub add();
## @skipline
sub del();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_ssh_filter_state();
## @skipline
sub recovery();
## @skipline
sub parse_command_line_arguments();
## @skipline
sub check_allow_deny_option();
## @skipline
sub parse_del_command_line_arguments();
## @skipline
sub del_param_validate();
## @skipline
sub add_param_validate();
## @skipline
sub get_hardening_storage_list();
## @skipline
sub is_host_in_the_list($$);
## @skipline
sub check_adding_params();
## @skipline
sub check_deleting_params(;$$);
## @skipline
sub is_allow_list();
## @skipline
sub is_deny_list();
## @skipline
sub get_params();
## @skipline
sub get_host(;$);
## @skipline
sub add_elem_to_hardening_storage();
## @skipline
sub delete_elem_from_hardening_storage(;$$);
## @skipline
sub find_host_by_num($$);
## @skipline
sub print_ssh_lists();
## @skipline
sub ssh_filtration_disabled();
## @skipline
sub ssh_filtration_enabled();
## @skipline
sub update_hosts_files(;$$);
## @skipline
sub clear_files();
## @skipline
sub get_orig_host_list($);
## @skipline
sub validateHostName($);
## @skipline
sub validateSubnet($);
## @skipline
sub validateNum($);
## @skipline
sub validateIPaddress($$);

#constants
my $DODitem     = "GEN005540";
my $module_name = "ssh_filter";
my $xml_node    = "/harden/ssh_filter";
my $allow_host  = "host_allow";
my $deny_host   = "host_deny";
my $allow_file  = "/etc/hosts.allow";
my $deny_file   = "/etc/hosts.deny";
my $sshd        = "sshd";

#global hash is used to store input parameters
my %params = (
              'allow'  => undef,
              'deny'   => undef,
              'host'   => undef,
              'ip'     => undef,
              'subnet' => undef,
              'num'    => undef,
             );

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'add' => {
                         'sub'             => \&add,
                         'validate_params' => \&add_param_validate,
                         'visible'         => $TRUE
                        },
               'del' => {
                         'sub'             => \&del,
                         'validate_params' => \&del_param_validate,
                         'visible'         => $TRUE
                        },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_ssh_filter_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Print SSH list
# @details It gets status from XML storage; Then gets SSH allowable and deny list
#          and print these lists; if there is no hosts in the list warning message is shown.
# @return  TRUE - ssh filtration list is displayed
# @return  FALSE - some error happens
sub status()
{

   baselog($LOG_DEBUG, "Trying to get status of SSH filtration...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $TRUE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "SSH filtration is turned on.\n", $TRUE);

      # if there are hosts in the list print them
      baselog($LOG_DEBUG, "Displaying SSH filtration list.");
      print_ssh_lists();

      # print warning message about ECM hosts
      print "Warning: Please make sure that ECM hosts are added to SSH filtration allowable list.\n";
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "SSH filtration is turned off.\n", $TRUE);
   }
   else
   {
      baselog($LOG_ERROR, "SSH filteration status is unknown.\n", $TRUE);
      $retval = $FALSE;
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief   Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return  TRUE - if the page is generated.
# @return  FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub parse_command_line_arguments()
# @brief This method sets up input arguments
# @details It uses GetOptions function in order to fill up 'params' hash with
#          input parameters; global hash %params will contain input parameters;
#          Then some validation functions are invoked in order to check the input
#          parameters;
# @return  TRUE -  input arguments are correct.
# @return  FALSE - incorrect arguments provided.
sub parse_command_line_arguments()
{
   baselog($LOG_DEBUG, "Triyng to get input parameters.");
   my $res = GetOptions(\%params, 'allow', 'deny', 'host=s', 'ip=s', 'subnet=s');
   if (not $res)
   {
      print "Incorrect use of command.\n";
      return $FALSE;
   }
   baselog($LOG_DEBUG, "Checking input parameters.");
   if (not check_allow_deny_option())
   {
      return $FALSE;
   }
   if ((not defined $params{host}) and (not defined $params{ip}) and (not defined $params{subnet}))
   {
      print "Please specify hots|ip|subnet options\n";
      return $FALSE;
   }
   if ((defined $params{host}) and (defined $params{ip}))
   {
      print "Please specify one of the hots|ip|subnet options\n";
      return $FALSE;
   }
   if ((defined $params{subnet}) and (defined $params{host}))
   {
      print "Please specify one of the hots|ip|subnet options\n";
      return $FALSE;
   }
   if ((defined $params{subnet}) and (defined $params{ip}))
   {
      print "Please specify one of the hots|ip|subnet options\n";
      return $FALSE;
   }
   if (defined $params{ip} and not validateIPaddress($params{ip}, "ip"))
   {
      print "Incorrect IP address provided.\n";
      return $FALSE;
   }
   if (defined $params{subnet} and not validateIPaddress($params{subnet}, "subnet"))
   {
      print "Incorrect Subnet address provided.\n";
      return $FALSE;
   }
   if (defined $params{host} and not validateHostName($params{host}))
   {
      print "Incorrect Host address provided.\n";
      return $FALSE;
   }
   return $TRUE;
} ## end sub parse_command_line_arguments()

###############################################################################
## @fn sub check_allow_deny_option()
# @brief   This method checks input argument - allow/deny
# @details This method checks if allow or deny options are defined;
#          It prins error message on the screen in case of error;
# @return  TRUE if one of the option(allow/deny) is defined;
# @return  FALSE if both or none options(allow/deny) are defined.
sub check_allow_deny_option()
{
   if ((defined $params{allow}) and (defined $params{deny}))
   {
      print "Please specify allow or deny option(-allow|-deny).\n";
      return $FALSE;
   }
   if ((not defined($params{allow})) and (not defined($params{deny})))
   {
      print "Please specify allow or deny option(-allow|-deny).\n";
      return $FALSE;
   }
   return $TRUE;
} ## end sub check_allow_deny_option()

###############################################################################
## @fn sub parse_del_command_line_arguments()
# @brief   This method parses input arguments
# @details This method is used when deleting host by number CLI command is invoked;
#          It uses global %params hash to store input parameters;
# @return  TRUE  - input arguments are correct
# @return  FALSE - incorrect arguments provided
sub parse_del_command_line_arguments()
{
   my $res = GetOptions(\%params, 'allow', 'deny', 'num=i', 'host=s', 'ip=s', 'subnet=s');
   if (not $res)
   {
      print "Incorrect use of command.\n";
      return $FALSE;
   }
   my $retval = check_allow_deny_option();
   return $retval;
} ## end sub parse_del_command_line_arguments()

###############################################################################
## @fn sub add_param_validate()
# @brief   Validate input parameters when adding host CLI command is invoked;
# @details It invokes parse_command_line_arguments() function to perform input parameter
#          validation;
# @return  TRUE - add command parameters are correct
# @return  FALSE - incorrect parameters provided
sub add_param_validate()
{

   #checking input parameters
   my $retval = parse_command_line_arguments();
   return $retval;
} ## end sub add_param_validate()

###############################################################################
## @fn sub del_param_validate()
# @brief   Perform input parameters validation for delete command
# @details At first it invokes parse_del_command_line_arguments() function to check
#          if input parameters contains 'num' option; So if input parameters contains
#          this option - it means CLI command looks like:
#    \li   '.. ssh_filter del -[allow|deny] -num=2';
#          If there is no 'num' parameter, it means that CLI command looks like:
#    \li  '.. ssh_filter del -[allow|deny] -[ip|host|subnet] xxx.yyy...'
# @return  TRUE - del command parameters are correct
# @return  FALSE - incorrect parameters provided
sub del_param_validate()
{
   my $retval = parse_del_command_line_arguments();
   if (not $retval)
   {
      return $FALSE;
   }
   if (defined $params{num})
   {
      baselog($LOG_DEBUG, "'$params{num}' is a sequence number of XML entity for deletion.");
      if (not validateNum($params{num}))
      {
         print "Incorrect '$params{num}' number for deletion provided.\n";
         return $FALSE;
      }
      return $TRUE;
   } ## end if (defined $params{num...
   else
   {
      $retval = parse_command_line_arguments();
   }
   return $retval;
} ## end sub del_param_validate()

###############################################################################
## @fn sub add()
# @brief   Adds hosts to configuration files and XML storage;
# @details This method performs the following actions in order to add host
#          to the allowable or deny SSH filtration list.
#     \li  1. compare input parameters with values from XML
#     \li  2. update XML
#     \li  3. update hosts files if hardening option is permitted
#     \li  4. send snmp trap
# @return  TRUE - host is added successfully
# @return  FALSE - some error happens
sub add()
{
   baselog($LOG_INFO, "Adding host to /etc/hosts.[allow/deny] files.");
   if (not check_adding_params())
   {
      baselog($LOG_ERROR, "Input parameters are incorrect.\n", $TRUE);
      return $FALSE;
   }

   # update hardening storage
   if (not add_elem_to_hardening_storage())
   {
      baselog($LOG_ERROR, "Cannot update hardening XML storage.\n", $TRUE);
      return $FALSE;
   }
   my $host = get_host();
   if (not defined $host)
   {
      return $FALSE;
   }

   if (is_option_permitted($module_name) == $FALSE)
   {

      print "Host '$host' is successfully added to the SSH list.\n";
      print "SSH filtration is turned off.\n";
      print "The system behavior will be changed after turning on SSH filtration.\n";
      return $TRUE;
   } ## end if (is_option_permitted...
   else
   {
      if (not update_hosts_files())
      {
         baselog($LOG_ERROR, "Cannot update /etc/hosts.[allow|deny] files.\n", $TRUE);
         return $FALSE;
      }
      print "Host '$host' is successfully added to the SSH list.\n";
   } ## end else [ if (is_option_permitted...

   my $msg;
   $msg = "SEC0062" if (is_allow_list());
   $msg = "SEC0064" if (is_deny_list());

   return send_snmp_trap($msg, {'host' => $host});
} ## end sub add()

###############################################################################
## @fn sub del()
# @brief   It deletes provided host from SSH list
# @details This method makes the following actions in order to delete provided host
#          from the SSH filtration list.
#     \li  1. compare input parameters with values from XML
#     \li  2. update XML
#     \li  3. update hosts configuration files if hardening option is permitted
#     \li  4. send snmp trap
# @return  TRUE - host is deleted successfully
# @return  FALSE - some error happens
sub del()

{
   baselog($LOG_INFO, "Trying to delete host...");

   my ($allow, $deny) = get_hardening_storage_list();
   if ((not defined $allow) or (not defined $deny))
   {
      baselog($LOG_ERROR, "Cannot get SSH XML list.");
      return $FALSE;
   }
   if (not check_deleting_params($allow, $deny))
   {
      baselog($LOG_ERROR, "Input parameters are incorrect.\n", $TRUE);
      return $FALSE;
   }

   # update hardening storage
   if (not delete_elem_from_hardening_storage($allow, $deny))
   {
      baselog($LOG_ERROR, "Cannot update hardening XML storage.\n", $TRUE);
      return $FALSE;
   }

   my $host;
   if (defined $params{num})
   {
      if (is_allow_list())
      {
         $host = get_host(find_host_by_num($allow, $params{num}));
      }
      else
      {
         $host = get_host(find_host_by_num($deny, $params{num}));
      }

   } ## end if (defined $params{num...
   else
   {
      $host = get_host();
   }
   if (not defined $host)
   {
      baselog($LOG_ERROR, "Cannot get host.");
      return $FALSE;
   }

   if (is_option_permitted($module_name) == $FALSE)
   {
      print "Host '$host' is successfully deleted from the SSH list.\n";
      print "SSH filtration is turned off.\n";
      print "The system behavior will be changed after turning on SSH filtration.\n";
      return $TRUE;
   } ## end if (is_option_permitted...
   else
   {
      if (not update_hosts_files())
      {
         baselog($LOG_ERROR, "Cannot update /etc/hosts.[allow|deny] files.\n", $TRUE);
         return $FALSE;
      }
      print "Host '$host' is successfully deleted from the SSH list.\n";
   } ## end else [ if (is_option_permitted...
   my $msg;
   $msg = "SEC0063" if (is_allow_list());
   $msg = "SEC0065" if (is_deny_list());

   return send_snmp_trap($msg, {'host' => $host});
} ## end sub del()

###############################################################################
## @fn sub off()
# @brief   Turns off the SSH filtration
# @details This method disable the SSH host filtration.
#          It changes the status of the Hardening option to "off" if it
#          is necessary, and disable SSH filtration - any hosts are avaliable
#          to connect to the system by SSH.
# @return  TRUE - the new status is set, SSH filtration is turned off.
# @return  FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to turn off SSH filtration...");

   my $retval = $FALSE;

   if (is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'SSH filtration' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (is_option_permitted...

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&ssh_filtration_disabled);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "SSH filtration is turned off.\n", $TRUE);

      $retval = send_snmp_trap("SEC0061");

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_INFO, "Cannot turn off SSH filtration.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief   Turn on SSH filtration
# @details It changes the status of the Hardening option to "on" if it
#          is necessary, and turn on SSH filtration; The only hosts, which are
#          presented in the allowable list are enabled to connect to the system by SSH.
# @return  TRUE - the new status is set. SSH filtration is turned on.
# @return  FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to turn on SSH filtration...");

   my $retval = $FALSE;

   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&ssh_filtration_enabled);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "SSH filtration is turned on.\n", $TRUE);
      print_ssh_lists();

      $retval = send_snmp_trap("SEC0060");
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_INFO, "Cannot turn on SSH filtration.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub get_hardening_storage_list()
# @brief   get SSH list
# @details This method makes the following actions in order to get allowable and
#          deny lists from hardening storage:
#     \li  1. open hardening storage
#     \li  2. get attributes for host entry
#     \li  3. return hash references
# @return  (undef, undef) if error happens
# @return  2 hash reference with allowable and deny SSH lists
#          If lists are empty, return empty lists(not undef)
#          (see implementation of getAttrsforallElements)
sub get_hardening_storage_list()
{
   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return (undef, undef);
   }

   my $allow_hosts = $parser->getAttrsforallElements($xml_node . "/" . $allow_host);
   if (not defined $allow_hosts)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the option $xml_node/$allow_host.");
      undef $parser;
      return (undef, undef);
   } ## end if (not defined $allow_hosts...

   my $deny_hosts = $parser->getAttrsforallElements($xml_node . "/" . $deny_host);
   if (not defined $allow_hosts)
   {
      baselog($LOG_ERROR, "Cannot get attributes for the option $xml_node/$deny_host.");
      undef $parser;
      return (undef, undef);
   } ## end if (not defined $allow_hosts...

   undef $parser;
   return ($allow_hosts, $deny_hosts);

} ## end sub get_hardening_storage_list()

###############################################################################
## @fn sub is_host_in_the_list($list, $hash_ref)
# @brief   Check if host provided in the list
# @details This method performs checking if host in the list
# @param   $list - hash reference with list to search
# @param   $hash_ref - hash reference with host
# @return  TRUE - host is in the list
# @return  FALSE - no host in the list
sub is_host_in_the_list($$)
{
   my ($list, $hash_ref) = @_;

   if (not defined $list or not defined $hash_ref)
   {
      baselog($LOG_ERROR, "List and hash should be defined.");
      return undef;
   }
   baselog($LOG_DEBUG, "Check if provided host in the list.");
   foreach my $elem (@$list)
   {
      foreach my $key (keys(%{$elem}))
      {
         if (defined $hash_ref->{$key})
         {
            if ($elem->{$key} eq $hash_ref->{$key})
            {
               baselog($LOG_DEBUG, " '$hash_ref->{$key}' address is in the provided list.");
               return $TRUE;
            }
         } ## end if (defined $hash_ref->...
      } ## end foreach my $key (keys(%{$elem...
   } ## end foreach my $elem (@$list)
   return $FALSE;
} ## end sub is_host_in_the_list($$)

###############################################################################
## @fn sub check_adding_params()
# @brief   Check input parameters
# @details This method checks if input parameters are valid or not,
#          when add CLI command is used;
# @return  TRUE - parameters are valid
# @return  FALSE - parameters are incorrect
sub check_adding_params()
{
   my ($allow, $deny) = get_hardening_storage_list();
   if (not defined $allow or not defined $deny)
   {
      baselog($LOG_ERROR, "Cannot get SSH hosts list");
      return $FALSE;
   }
   my $params = get_params();
   if (not defined $params)
   {
      baselog($LOG_ERROR, "Cannot get input parameters.");
      return $FALSE;
   }

   #check if such host is already in the allowable list
   if (is_host_in_the_list($allow, $params))
   {
      print "Error: Duplicated address already is in the allowable list\n";
      return $FALSE;
   }

   #check if such host is already in the deny list
   if (is_host_in_the_list($deny, $params))
   {
      print "Error: Duplicated address already is in the deny list\n";
      return $FALSE;
   }
   if (is_deny_list())
   {
      if (defined $params{subnet})
      {
         print "Limitation: Subnet cannot be added to deny list.\n";
         return $FALSE;
      }
   } ## end if (is_deny_list())
   return $TRUE;
} ## end sub check_adding_params()

###############################################################################
## @fn sub check_deleting_params()
# @brief   Check input parameters
# @details This method checks if input parameters are valid or not,
#          when del CLI command is used;
# @return  TRUE - parameters are valid
# @return  FALSE - parameters are incorrect
sub check_deleting_params(;$$)
{
   my ($allow, $deny) = @_;
   if ((not defined $allow) or (not defined $deny))
   {
      ($allow, $deny) = get_hardening_storage_list();
      if ((not defined $allow) or (not defined $deny))
      {
         baselog($LOG_ERROR, "Cannot get SSH hosts list");
         return $FALSE;
      }
   } ## end if ((not defined $allow...
   my $list = undef;
   if (is_allow_list())
   {
      $list = $allow;
   }
   elsif (is_deny_list())
   {
      $list = $deny;
   }
   else
   {
      baselog($LOG_ERROR, "Cannot define which list is used(allow or deny list).");
      return $FALSE;
   }
   if (defined $params{num})
   {
      if (is_allow_list())
      {
         if ($params{num} > scalar @$allow)
         {
            print "Error: Incorrect '$params{num}' number provided.\n";
            return $FALSE;
         }
      } ## end if (is_allow_list())
      else
      {
         if ($params{num} > scalar @$deny)
         {
            print "Error: Incorrect '$params{num}' number provided.\n";
            return $FALSE;
         }
      } ## end else [ if (is_allow_list())
   } ## end if (defined $params{num...
   else
   {
      my $params = get_params();
      if (not defined $params)
      {
         baselog($LOG_ERROR, "Cannot get input parameters.");
         return $FALSE;
      }

      #check if host is in the corresponding list;
      if (not is_host_in_the_list($list, $params))
      {
         print "Error: There is no such host to delete.\n";
         return $FALSE;
      }
   } ## end else [ if (defined $params{num...
   return $TRUE;
} ## end sub check_deleting_params(;$$)

###############################################################################
## @fn sub is_allow_list()
# @brief   Check if allowable list is used
# @details It checks %params hash in order to define if allow key is defined or not.
#          If it is defined it means that allowable SSH list is used.
# @return  TRUE if allow list is used
# @return  FALSE if not allow list is used
sub is_allow_list()
{
   return $TRUE if (defined $params{allow});
   return $FALSE;
}

###############################################################################
## @fn sub is_deny_list()
# @brief   Check if deny list is used
# @details It checks %params hash in order to define if deny key is defined or not.
#          If it is defined it means that deny SSH list is used.
# @return  TRUE if deny list is used
# @return  FALSE if not deny list is used
sub is_deny_list()
{
   return $TRUE if (defined $params{deny});
   return $FALSE;
}

###############################################################################
## @fn sub get_params()
# @brief   This method returns hash reference with input parameters;
# @details %params hash contains 'undef' value; it is necessary to
#          get rid of undef value in order to use a hash in XML element;
# @return  hash reference
# @return  undef if error happens
sub get_params()
{

   baselog($LOG_DEBUG, "Parsing input parameters hash.");
   my %attrs = ();

   if (defined $params{ip})
   {
      $attrs{ip} = $params{ip};
   }
   elsif (defined $params{subnet})
   {
      $attrs{subnet} = $params{subnet};
   }
   elsif (defined $params{host})
   {
      $attrs{ip} = $params{host};
   }
   else
   {
      baselog($LOG_ERROR, "Cannot define hash parameters.");
      return undef;
   }
   return \%attrs;

} ## end sub get_params()

###############################################################################
## @fn sub get_host($hosts)
# @brief   This method returns value of host address;
# @details It parses hash, and gets defined value of host;
# @param   $hosts - hash reference to find host
# @return  host address
# @return  undef if error happens
sub get_host(;$)
{
   my $hosts = shift;
   if (not defined $hosts)
   {
      $hosts = get_params();
   }
   if (not defined $hosts)
   {
      baselog($LOG_ERROR, "Cannot get hash parameters.");
      return undef;
   }
   my $host;
   $host = $hosts->{$_} foreach (keys(%{$hosts}));
   if (not defined $host)
   {
      baselog($LOG_ERROR, "Cannot get host.");
      return undef;
   }
   return $host;
} ## end sub get_host(;$)

###############################################################################
## @fn sub add_elem_to_hardening_storage()
# @brief   Update hardening storage
# @details This method makes the following actions in order to add element to the
#          hardening storage.
#     \li     1. Opean hardening storage;
#     \li     2. update XML file (add/delete elements)
#     \li     3. Save hardening storage;
# @return TRUE - hardening storage is updated.
# @return FALSE - some error happens
sub add_elem_to_hardening_storage()
{
   my $retval = $TRUE;
   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }
   my $node = undef;
   if (is_allow_list())
   {
      $node = $allow_host;
   }
   elsif (is_deny_list())
   {
      $node = $deny_host;
   }
   else
   {
      baselog($LOG_ERROR, "Cannot defined ssh list");
      return $FALSE;
   }
   if (not $parser->addElements($xml_node, undef, $node, get_params()))
   {
      baselog($LOG_ERROR, "Cannot add element to XML storage");
      return $FALSE;
   }
   if (not save_hardening_storage($parser))
   {
      baselog($LOG_ERROR, "Cannot save Hardening XML storage.");
      return $FALSE;
   }
   return $retval;
} ## end sub add_elem_to_hardening_storage()

###############################################################################
## @fn sub delete_elem_from_hardening_storage($allow, $deny)
# @brief   Update hardening storage
# @details This method makes the following actions in order to delete element
#          from the hardening storage.
#  \li        1. Opean hardening storage;
#  \li        2. update XML file (delete elements)
#  \li        3. Save hardening storage;
# @return TRUE - hardening storage is updated.
# @return FALSE - some error happens
sub delete_elem_from_hardening_storage(;$$)
{

   my $retval = $TRUE;
   my ($allow, $deny) = @_;
   my $parser = open_hardening_storage();
   if (not defined $parser)
   {
      baselog($LOG_ERROR, "Cannot read Hardening XML storage.");
      return $FALSE;
   }
   my $node = undef;
   if (is_allow_list())
   {
      $node = $allow_host;
   }
   elsif (is_deny_list())
   {
      $node = $deny_host;
   }
   else
   {
      baselog($LOG_ERROR, "Cannot define ssh list");
      return $FALSE;
   }
   my $attrs = ();

   if (defined $params{num})
   {
      baselog($LOG_DEBUG, "Deleting host with '$params{num}' number.");

      if ((not defined $allow) or (not defined $deny))
      {
         baselog($LOG_DEBUG, "SSH XML host list is undefined. Trying to get it.");
         ($allow, $deny) = get_hardening_storage_list();
         if ((not defined $allow) or (not defined $deny))
         {
            baselog($LOG_ERROR, "Cannot get SSH hosts list");
            return $FALSE;
         }
      } ## end if ((not defined $allow...
      if (is_allow_list())
      {
         $attrs = find_host_by_num($allow, $params{num});
      }
      else
      {
         $attrs = find_host_by_num($deny, $params{num});
      }
   } ## end if (defined $params{num...
   else
   {
      $attrs = get_params();
   }
   if (not defined $attrs)
   {
      return $FALSE;
   }
   if (not $parser->deleteElement($xml_node . "/" . $node, $attrs))
   {
      baselog($LOG_ERROR, "Cannot delete element from XML storage");
      return $FALSE;
   }
   if (not save_hardening_storage($parser))
   {
      baselog($LOG_ERROR, "Cannot save Hardening XML storage.");
      return $FALSE;
   }

   return $retval;
} ## end sub delete_elem_from_hardening_storage(;$$)

###############################################################################
## @fn sub find_host_by_num($list_ref, $number)
# @brief   It finds host by number
# @details This method is used when deleting host by number;
#          Each host entry has sequnce number;
#          It compares each host entry with number provided;
# @param   $list_ref - hash reference with list to search
# @param   $number - number
# @return  hash reference - if number is equal to host entry
# @return  undef - if not found;
sub find_host_by_num($$)
{

   my ($list_ref, $number) = @_;
   if ((not defined $list_ref) or (not defined $number))
   {
      baselog($LOG_ERROR, "list or number is undefined.");
      return undef;
   }

   my $ret_hash = ();
   my $i        = 1;
   foreach my $elem (@$list_ref)
   {
      foreach my $key (keys(%{$elem}))
      {
         if ($i eq $number)
         {
            $ret_hash->{$key} = $elem->{$key};
            baselog($LOG_DEBUG, "Host is found : $key -> $ret_hash->{$key}");
            last;
         } ## end if ($i eq $number)
      } ## end foreach my $key (keys(%{$elem...
      $i++;
   } ## end foreach my $elem (@$list_ref...
   return $ret_hash;
} ## end sub find_host_by_num($$)

###############################################################################
## @fn sub print_ssh_lists()
# @brief   prints SSH lists
# @details This method prints allow and deny lists of host;
# @return  TRUE - printing was successfully;
# @return  FALSE - some error happens;
sub print_ssh_lists()
{
   my ($allow, $deny) = get_hardening_storage_list();
   if (not defined $allow or not defined $deny)
   {
      baselog($LOG_ERROR, "Cannot get SSH hosts list");
      return $FALSE;
   }

   if (scalar(@$allow) > 0)
   {
      print "SSH allowable list consists of:\n";
      my $i = 1;
      foreach my $elem (@$allow)
      {
         foreach my $key (keys(%{$elem}))
         {
            print $i . ". " . uc($key) . "=" . $elem->{$key} . "\n";
            $i++;
         }
      } ## end foreach my $elem (@$allow)
   } ## end if (scalar(@$allow) > ...

   # if there is no such hosts print warning message
   else
   {
      print "Warning: Access to the server is restricted by SSH because there are no configured hosts allowed connecting.\n";
   }
   if (scalar(@$deny) > 0)
   {
      print "SSH deny list consists of:\n";
      my $i = 1;
      foreach my $elem (@$deny)
      {
         foreach my $key (keys(%{$elem}))
         {
            print $i. ". " . uc($key) . "=" . $elem->{$key} . "\n";
            $i++;
         }
      } ## end foreach my $elem (@$deny)
   } ## end if (scalar(@$deny) > 0...
   return $TRUE;
} ## end sub print_ssh_lists()

###############################################################################
## @fn sub ssh_filtration_disabled()
# @brief   Turn off SSH filtration
# @details This method is used when 'ssh_filter off' command is invoked;
#          The method removes the corresponding settings from configuration file.
# @return  TRUE - SSH filtration is turned off succesfully
# @return  FALSE - if error happens
sub ssh_filtration_disabled()
{
   my $retval = clear_files();
   return $retval;
}

###############################################################################
## @fn sub ssh_filtration_enabled()
# @brief   Turn on SSH filtration
# @details This method is used when 'ssh_filter on' command is invoked;
#          The method defines the corresponding settings of the configuration file.
# @return  TRUE - successfully turn on SSH filtration
# @return  FALSE - some error happens
sub ssh_filtration_enabled()
{
   my $retval = update_hosts_files();
   return $retval;
}

###############################################################################
## @fn sub update_hosts_files($allow_list, $deny_list)
# @brief   Reapply hosts allow and deny files
# @details It synchronizes /etc/hosts.allow and /etc/hosts.deny files acoording to
#          appropriate section of hardening XML storage
# @param   $allow_list - hash reference with allow SSH list
# @param   $deny_list - hash reference with deny SSH list
# @return  TRUE - the file is synchronized successfully
# @return  FALSE - some error happens
sub update_hosts_files(;$$)
{
   my ($allow_list, $deny_list) = @_;
   if ((not defined $allow_list) or (not defined $deny_list))
   {
      ($allow_list, $deny_list) = get_hardening_storage_list();
      if ((not defined $allow_list) or (not defined $deny_list))
      {
         baselog($LOG_ERROR, "Cannot get SSH hosts list");
         return $FALSE;
      }
   } ## end if ((not defined $allow_list...

   my $allow_data = get_orig_host_list($allow_file);
   if (not defined $allow_data)
   {
      baselog($LOG_ERROR, "Cannot get content of $allow_file file.");
      return $FALSE;
   }

   my $deny_data = get_orig_host_list($deny_file);
   if (not defined $deny_data)
   {
      baselog($LOG_ERROR, "Cannot get content of $deny_file file.");
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Synchronizing $allow_file file.");

   # putting hosts in the allow file in order to give them access;
   # the syntax is 'sshd: hosts1 .dom1.dom2'
   push @$allow_data, $sshd . ": ";
   foreach my $elem (@$allow_list)
   {
      foreach my $ip (keys(%{$elem}))
      {
         push @$allow_data, " " . $elem->{$ip};
         push @$allow_data, "." if (($ip eq 'subnet') and ($elem->{$ip} !~ /\//));
         baselog($LOG_DEBUG, "Synchronizing $ip -> $elem->{$ip}.");
      } ## end foreach my $ip (keys(%{$elem...
   } ## end foreach my $elem (@$allow_list...

   if (scalar @$deny_list > 0)
   {

      #putting EXCEPT key word to deny access for all hosts in the deny list
      push @$allow_data, " EXCEPT ";

      #putting hosts after key word EXCEPT (from deny list)
      foreach my $elem (@$deny_list)
      {
         foreach my $ip (keys(%{$elem}))
         {
            push @$allow_data, " " . $elem->{$ip};
            baselog($LOG_DEBUG, "Synchronizing $ip -> $elem->{$ip}.");
         }
      } ## end foreach my $elem (@$deny_list...
   } ## end if (scalar @$deny_list...
   push @$allow_data, "\n";

   # deny file will contain 'sshd: ALL' string
   baselog($LOG_DEBUG, "Synchronizing $deny_file file.");
   push @$deny_data, $sshd . ": ALL \n";

   #save /etc/hosts.allow file
   if (not write_harden_file($allow_file, $allow_data))
   {
      baselog($LOG_ERROR, "Cannot save $allow_file file.");
      return $FALSE;
   }

   #save /etc/hosts.deny file
   if (not write_harden_file($deny_file, $deny_data))
   {
      baselog($LOG_ERROR, "Cannot save $deny_file file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub update_hosts_files(;$$)

###############################################################################
## @fn sub clear_files()
# @brief   Clear /etc/hosts.allow/deny files from sshd strings
# @details This method makes the following actions in order
#          to remove the SSH settings from configuration files.
#    \li      1. get context of hosts files
#    \li      2. remove sshd strings
#    \li      3. save hosts files
# @return  TRUE configuration files are updated successfully
# @return  FALSE if error happens;
sub clear_files()
{

   #1. get /etc/hosts.(allow/deny) files context
   #2. clear sshd string content
   #3. save files
   my $allow_data = get_orig_host_list($allow_file);
   if (not defined $allow_data)
   {
      baselog($LOG_ERROR, "Cannot get content of $allow_file file.");
      return $FALSE;
   }

   my $deny_data = get_orig_host_list($deny_file);
   if (not defined $deny_data)
   {
      baselog($LOG_ERROR, "Cannot get content of $deny_file file.");
      return $FALSE;
   }

   if (not write_harden_file($allow_file, $allow_data))
   {
      baselog($LOG_ERROR, "Cannot save $allow_file file.");
      return $FALSE;
   }

   if (not write_harden_file($deny_file, $deny_data))
   {
      baselog($LOG_ERROR, "Cannot save $deny_file file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub clear_files()

###############################################################################
## @fn sub get_orig_host_list($file)
# @brief   get content of file provided;
# @details The method makes the following actions in order to get the content of
#          the SSH configuration file.
#    \li      1. Open file
#    \li      2. Find sshd string
#    \li      3. remove this string
#    \li      4. return content of file without sshd string
# @param  $file - SSH configuration file;
# @return TRUE - content is returned successfully
# @return FALSE - some error happens
sub get_orig_host_list($)
{
   my $input_file = shift;
   if (not defined $input_file)
   {
      baselog($LOG_ERROR, "File has to be defined.");
      return $FALSE;
   }
   my $file;
   my @orig_arr = ();
   my @ret_arr  = ();

   if (not open($file, '<', $input_file))
   {
      baselog($LOG_ERROR, "Cannot open $input_file: $!");
      return $FALSE;
   }
   @orig_arr = <$file>;
   if (not close $file)
   {
      baselog($LOG_ERROR, "Cannot close file: $!");
   }

   #remove $sshd string
   foreach my $str (@orig_arr)
   {
      if ($str !~ /^\s*$sshd\s*/)
      {
         push @ret_arr, $str;
      }
   } ## end foreach my $str (@orig_arr)
   if ((not scalar(@ret_arr)) or (not defined $ret_arr[0]))
   {
      baselog($LOG_ERROR, "Cannot get host information from $input_file.");
      return undef;
   }
   return \@ret_arr;
} ## end sub get_orig_host_list($)

###############################################################################
## @fn sub validateHostName($hostname)
# @brief   Validate host name
# @details The method uses two validation functions - validateFQDN and validateHostname;
#          The valid host name is the FQDN or simple host name.
# @param  $hostname - host name
# @return TRUE - host name is correct
# @return FALSE - host name is not correct
sub validateHostName($)
{
   my $hostname = shift;

   if ((not baseParams::validateFQDN($hostname)) and (not baseParams::validateHostname($hostname)))
   {
      return $FALSE;
   }
   return $TRUE;
} ## end sub validateHostName($)

###############################################################################
## @fn sub validateSubnet($subnet)
# @brief   Validate subnet
# @details The method validates provided subnet.
# @param  $subnet - subnet;
# @return TRUE - subnet is correct
# @return FALSE - subnet is not correct
sub validateSubnet($)
{
   my $subnet = shift;

   if (not defined $subnet)
   {
      return $FALSE;
   }

   #$ip_addr_expr looks like 192.168.23.103
   my $ip_addr_expr = '(\d{1,3}(\.\d{1,3}){3})';
   if ($subnet =~ /^$ip_addr_expr\/$ip_addr_expr$/)
   {

      #the pair of net/mask for e.g. 192.168.23.103/255.255.255.0
      my ($net, $mask) = split /\//, $subnet;
      if (not validateIP($net) or not validateIP($mask))
      {
         return $FALSE;
      }

   } ## end if ($subnet =~ /^$ip_addr_expr\/$ip_addr_expr$/...
   else
   {

      # check that in right dot notation format
      if ($subnet !~ /^(\d{1,3}(\.\d{1,3}){1,2})$/)
      {
         return $FALSE;
      }

      # Check that bytes are in range
      my $zcnt = 0;

      for (split /\./, $subnet)
      {
         $zcnt++ if ($_ == 0);  # keep track of the # of 0's
         return $FALSE if $_ < 0 or $_ > 255;
      }

      #
      # don't allow an ip with only 0's in it
      #
      if ($zcnt == 4)
      {
         return $FALSE;
      }
   } ## end else [ if ($subnet =~ /^$ip_addr_expr\/$ip_addr_expr$/...
   return $TRUE;
} ## end sub validateSubnet($)

###############################################################################
## @fn sub validateIPaddress($IP, $option)
# @brief   Validate IP address
# @details The method validates IP address and invokes
#          appropriate function(validateIP or validateSubnet) to validate address
#          according to second argument
# @param  $ip  - IP address;
# @param  $option - ip or subnet
# @return TRUE - ip is correct
# @return FALSE - ip  is not correct
# @todo   FIXME - it is necessary to move this function from this module to some
#               common place (e.g. base_validation.pm)
sub validateIPaddress($$)
{
   my ($IP, $option) = @_;
   if ((not defined $IP) or (not defined $option))
   {
      baselog($LOG_ERROR, "Parameters are not defined.");
      return $FALSE;
   }

   if ($IP =~ /^0+/ or $IP =~ /^\.0+/)
   {
      return $FALSE;
   }

   #some common checking should be here
   #
   my $ret = $FALSE;

   if ($option eq "ip")
   {
      $ret = validateIP($IP);
   }
   elsif ($option eq "subnet")
   {
      $ret = validateSubnet($IP);
   }
   else
   {
      baselog($LOG_ERROR, "Incorrect option provided.");
      return $FALSE;
   }
   return $ret;
} ## end sub validateIPaddress($$)
###############################################################################
## @fn sub validateNum($num)
# @brief   It validates input parameter 'num'
# @details The number should be sequence number of XML entity
# @param   $num -  number to validate;
# @return TRUE - number is correct
# @return FALSE - number is not correct
sub validateNum($)
{
   my $num = shift;
   return $FALSE if (not defined $num);
   return $FALSE if ($num !~ /^\d+$/ or $num eq 0);
   return $TRUE;
} ## end sub validateNum($)

###############################################################################
## @fn sub apply_default()
# @brief   Apply default state of SSH filtration
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it turns off SSH filtration.
#          Default value is applied in the case when it is impossible to detect
#          the current SSH filtration turning on/off policy.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to turn off SSH filtration...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&ssh_filtration_disabled, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "SSH filtration is turned off.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change the status os SSH filtration.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of Host-based SSH filtration lists is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "Host-based SSH filtration is enabled.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Host-based SSH filtration is disabled");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief   Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          SSH filtration is turned on or off.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of SSH filtration from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of SSH filtration is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {
      $retval = ssh_filtration_enabled();
   }
   else
   {
      $retval = ssh_filtration_disabled();
   }

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_ssh_filter_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          SSH filtration is turned on or off, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_ssh_filter_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of SSH filtration.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of SSH filtration is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0060");
   }
   else
   {
      $retval = send_snmp_trap("SEC0061");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_ssh_filter_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if SSH filtration is turned on or off, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of SSH filtration...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_ssh_filter_state();

   return $retval;
} ## end sub recovery()
1;
__END__
