###############################################################################
## @file   status.pm
# @brief   Gets statuses of each enhanced hardening modules
# @details This module invokes the action 'status' for all enhanced hardening
#          modules.
# @author  Petr Kascheev
# @author  Alexander Kolotov 
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.


## @class status
package hardening::plugins::status;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use plugin_mgr;
use baselog;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

## @skipline
sub actions();
## @skipline
sub get_status();
## @skipline
sub help();

my %actions = ('get'     => {'sub'     => \&get_status,
                             'visible' => $FALSE,
                             'default' => $TRUE},
              );


###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub get_status()
# @brief  It gets statuses of all hardening modules. 
# @details The method creates plugin manager object, then gets list of avaliable 
#          modules, and than invokes status action for each modules. 
# @return FALSE - some error happens  
# @return TRUE - operation is performed successfully
sub get_status()
{
   my $retval = $TRUE;
   my @modules = ();
   my $manager;
   my $status = 0;
   
   $manager  = plugin_mgr->new($PATH_TO_ENHANCED_HARDENING_MODULES);
   if (not defined $manager)
   {
       baselog($LOG_ERROR, "Cannot create Hardening manager instance.");
       print "Status option is not avaliable.\n";
       return $FALSE;
   }
   
   @modules = $manager->get_modules();
   if ((scalar @modules eq 0) or (not defined $modules[0]))
   {
       baselog($LOG_ERROR, "Cannot get list of modules.");
       print "Status option is not avaliable.\n";
       $retval = $FALSE;
   }
   else
   {
      foreach my $module (@modules)
      {  
         #Run status for for all modules except the current one (status) 
         next if ($module =~ /^status$/);      
         
         baselog($LOG_DEBUG, "Try to get status for the module '$module'.");
         
         my $hash = prepare_action_hash($module, 'status');
         print "Status of '$module' module:\n";
         $status = $manager->try_action($hash);
         print "\n";
         
         if (not $status)
         {
            $retval = $FALSE;
         }   
      }
   }

   undef $manager;
   
   return $retval; 
} ##end sub get_status()

1;
__END__
