###############################################################################
## @file telnet.pm
# @brief Provides hardening actions for CS1000 Telnet service
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 Telnet service and the Enhanced
#          Hardening module 'telnet' are requested.
#
#\li          It is responsible for the CLI actions:
#\li           harden telnet
#\li           harden telnet on
#\li           harden telnet off
#\li           harden telnet status
#\li           harden telnet help
#\li           It also provides the API in order to activate and deactivate xinet
#              based telnet service in accordance with the current status of the
#              corresponding Hardening rule.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - fixed a bug with appearance of an error with undefined
#                 variable when the action 'stop' is run from CLI
# @version 1.03 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage
# @version 1.04 - The functions on, off, start, stop
#                 have been modified in order to support TELNET firewall settings

=pod

=head1 NAME

=over 4

=item B<harden telnet> - command to manage usage of CS1000 Telnet service.

=back

=head1 SYNOPSIS

=over 10

=item B<harden telnet> I<on> | I<off> | I<status> | I<help>

=back

=head1 DESCRIPTION

B<harden telnet> module allows managing usage of CS1000 Telnet service.

=head1 OPTIONS

=over 4

=item B<on>

Permits usage of CS1000 Telnet service. The SNMP trap is sent to track this event
by SNMP manager.

B<NOTE>: This action does not perform actual activation of Telnet service. It just
permits to use the service in the future.

=item B<off>

Forbids usage of CS1000 Telnet service. This action immediately tries to shutdown
all Telnet activities. The warning message notifies about this and a user is asked
to abort this. Even if the Telnet activites are not shut down, CS1000 Telnet service
is forbidden for future usage. The SNMP trap is sent to track this event
by SNMP manager.

=item B<status>

Outputs the status whether usage of CS1000 Telnet service is permitted or not. If
the service is permitted the actual status (whether the service is running or
not) is output as well.

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden telnet> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden telnet on>

 Telnet service is permitted.

=item B<$ harden telnet off>

 Telnet service is forbidden.

=item B<$ harden telnet status>

 Telnet service is permitted.
 Telnet service is stopped.

=back

=head1 CAVEATS

B<harden telnet> must be run only under a high privileges account.

=cut

## @class telnet
package hardening::plugins::telnet;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";
use common_variables;
use CommonInstallUtils qw(getYNResponse);
use base_system;
use baselog;
use hardening::hardening_common;
use base_snmp;
use base_firewall qw(firewall_restart);

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.04';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub start();
## @skipline
sub stop();
## @skipline
sub deactivate_telnet();
## @skipline
sub emergency_stop();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_telnet_state();
## @skipline
sub recovery();

#variables
my $module_name  = "telnet";
my $xinet_telnet = "telnet";

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'start' => {
                           'sub'     => \&start,
                           'visible' => $FALSE
                          },
               'stop' => {
                          'sub'     => \&stop,
                          'visible' => $FALSE
                         },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_telnet_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine if
#          usage of Telnet service is permitted or not, and outputs this status.
#          If Telnet service is permitted, it ouptuts if the service is actually
#          running or not.
# @return TRUE - if the status of the Hardening option is got.
# @return        FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status Telnet service...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "Telnet service is permitted.\n", $TRUE);
      my $status = get_status_linux_service($xinet_telnet, undef, $HANDLE_XINET_SERVICE);
      if ($status == $TRUE)
      {
         baselog($LOG_INFO, "Telnet service is running.\n", $TRUE);
      }
      elsif ($status == $FALSE)
      {
         baselog($LOG_INFO, "Telnet service is stopped.\n", $TRUE);
      }
      else
      {
         baselog($LOG_ERROR, "Status of Telnet service is unknown.\n", $TRUE);
      }
      $retval = $TRUE;
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "Telnet service is forbidden.\n", $TRUE);
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Hardening status of Telnet service is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub start()
# @brief Activate xinet based telnet service
# @details This method examines the Hardening storage in order to determine if
#          usage of Telnet service is permitted or not. If the service is permitted,
#          the method tries to activate xinet based telnet service and sends the
#          corresponding SNMP trap.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is running sucessfully
# @return FALSE - either the usage Telnet service is forbidden, or some error happens
sub start()
{
   baselog($LOG_DEBUG, "Activation of Telnet service...");

   #get info from the hardening storage file
   if (is_option_permitted($module_name) == $FALSE)
   {

      #warning message if Telnet service is forbidden
      baselog($LOG_WARNING, "Usage of Telnet service is forbidden. Contact your Security Administrator for assistance.\n", $TRUE);
      return $FALSE;
   } ## end if (is_option_permitted...

   #Try to activate the service
   if (not activate_xinet_service($xinet_telnet))
   {
      baselog($LOG_ERROR, "Cannot activated Telnet service");
      return $FALSE;
   }

   # Try to open firewall ports
   if (not open_telnet_firewall())
   {
      baselog($LOG_ERROR, 'Cannot enable TELNET firewall ports.');
      return $FALSE;
   }

   baselog($LOG_INFO, "Telnet service is activated.");

   send_snmp_trap("SEC0083");

   return $TRUE;
} ## end sub start()

###############################################################################
## @fn sub stop()
# @brief Deactivate xinet based telnet service
# @details This method tries to deactivate xinet based telnet service and sends
#          the corresponding SNMP trap.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub stop()
{

   #Deactivate the service
   if (deactivate_telnet())
   {
      send_snmp_trap("SEC0084");
      return $TRUE;
   }

   return $FALSE;
} ## end sub stop()

###############################################################################
## @fn sub deactivate_telnet()
# @brief Deactivate telnet service
# @details This method tries to deactivate xinet based telnet service.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub deactivate_telnet()
{
   baselog($LOG_DEBUG, "Deactivation of Telnet service...");
   my $retval = $TRUE;

   if (not deactivate_xinet_service($xinet_telnet))
   {
      baselog($LOG_ERROR, "Cannot deactivate Telnet service");
      $retval = $FALSE;
   }

   # Try to close TELNET ports
   if (not close_telnet_firewall())
   {
      baselog($LOG_ERROR, 'Cannot disable TELNET ports.');
      return $FALSE;
   }

   baselog($LOG_INFO, "Telnet service is deactivated.") if $retval;

   return $retval;
} ## end sub deactivate_telnet()

###############################################################################
## @fn sub emergency_stop()
# @brief Stops CS1000 Telnet service
# @details This method tries to shutdown CS1000 Telnet service.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub emergency_stop()
{
   baselog($LOG_DEBUG, "Trying to stop all Telnet activities...");

   #FIXME: It must be replaced by the corresponding methods which will be
   #       intorduced under Secure Shell management activity.
   stop();

   return $TRUE;  # return value must be a scalar value
} ## end sub emergency_stop()

###############################################################################
## @fn sub off()
# @brief Forbid usage of Telnet service
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform emergency stop of CS1000 Telnet service
#          and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to forbid Telnet service...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   #Emergency stop must be performed as actual system changes after Telnet
   #forbidding, thats is why the third argument is the reference to
   #emergency_stop.
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&emergency_stop, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Telnet service is forbidden.\n", $TRUE);

      send_snmp_trap("SEC0082");

      # Try to restart firewall
      if (not firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         $retval = $TRUE;
      }

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change Telnet service permissions.\n", $TRUE);
      if (is_option_permitted($module_name))
      {

         # Try to close TELNET ports
         baselog($LOG_ERROR, 'Cannot disable TELNET ports.') if (!close_telnet_firewall());

         # Try to restart firewall - The security admin must have the ability to always
         # close the corresponding traffic activity if he/she wants.
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE) if (!firewall_restart());
      } ## end if (is_option_permitted...

   } ## end else [ if ($result == $TRUE)

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief Permit usage of Telnet service
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to permit Telnet service...");

   my $retval = $FALSE;

   if (not is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'TELNET service' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (not is_option_permitted...

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&start, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Telnet service is permitted.\n", $TRUE);

      send_snmp_trap("SEC0081");

      # Try to restart firewall
      if (not firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         $retval = $TRUE;
      }

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change Telnet service permissions.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief Apply default state of usage of Telnet service
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it forbids usage of Telnet service.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to forbid Telnet service...");

   my $retval = $FALSE;

   #Set new status to value 'off'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&deactivate_telnet, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "Telnet service is forbidden.");

      $retval = $TRUE;
   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change Telnet service permissions.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of Telnet service use is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "Use of Telnet service is permitted.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Use of Telnet service is forbidden.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of Telnet service is permitted or not. First, the case when
#          there is some problem with the Hardening storage - default state of
#          the option is applied. Otherwise, the system is changed in accordance
#          to the state from the Hardening storage.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of Telnet service from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of usage of Telnet service is unknown. Default status is applied.");
      $retval = apply_default();
   }
   elsif ($status)
   {

      #Handle "on" state
      $retval = $TRUE;
   } ## end elsif ($status)
   else
   {

      #Handle "off" state - shutdown telnet service if it is running
      $retval = deactivate_telnet();
   } ## end else [ if ($status == $H_ERROR...

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_telnet_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of Telnet service is permitted or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_telnet_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of Telnet service.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of usage of Telnet service is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0081");
   }
   else
   {
      $retval = send_snmp_trap("SEC0082");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_telnet_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if usage of Telnet service is permitted or not, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of Telnet service...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of usage of Telnet service is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval &= start();
   }
   $retval &= send_snmp_with_hardening_telnet_state();

   return $retval;
} ## end sub recovery()

1;
__END__
