###############################################################################
## @file tftp.pm
# @brief Provides hardening actions for CS1000 TFTP service
# @details This module provides API used by the Hardening Manager when any
#          hardening actions against CS1000 TFTP service and the Enhanced
#          Hardening module 'tftp' are requested.
#
#\li          It is responsible for the CLI actions:
#\li           harden tftp
#\li           harden tftp on
#\li           harden tftp off
#\li           harden tftp status
#\li           harden tftp help
#              It also provides the API in order to activate and deactivate xinet
#              based tftp service in accordance with the current status of the
#              corresponding Hardening rule.
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - default behaviour is changed tftp service is permitted
# @version 1.10 - Managing firewall ports functionality added.
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.12 - fixed a bug with appearance of an error with undefined
#                 variable when the action 'stop' is run from CLI
# @version 1.13 - add a special action 'post_lb_status' in order to display
#                 status of option on Linux Base post-installation stage

=pod

=head1 NAME

=over 4

=item B<harden tftp> - command to manage usage of CS1000 TFTP service.

=back

=head1 SYNOPSIS

=over 10

=item B<harden tftp> I<on> | I<off> | I<status> | I<help>

=back

=head1 DESCRIPTION

B<harden tftp> module allows managing usage of CS1000 TFTP service.

=head1 OPTIONS

=over 4

=item B<on>

Permits usage of CS1000 TFTP service. The SNMP trap is sent to track this event
by SNMP manager.

B<NOTE>: This action performs actual activation of TFTP service. It 
permits to use the service in the future, opens firewall port and 
starts the service.

=item B<off>

Forbids usage of CS1000 TFTP service. This action immediately tries to shutdown
all TFTP activities and close firewall port. The warning message notifies about 
this and a user is asked to abort this. Even if the TFTP activites are not shut 
down, CS1000 TFTP service is forbidden for future usage. The SNMP trap is sent 
to track this event by SNMP manager. 

=item B<status>

Outputs the status whether usage of CS1000 TFTP service is permitted or not. If
the service is permitted the actual status (whether the service is running or
not) is output as well.

=item B<help>

Outputs this manual page. This actions is being assumed if B<harden tftp> is
invoked without parameters.

=back

=head1 EXAMPLES

=over 4

=item B<$ harden tftp on>

 TFTP service is permitted.

=item B<$ harden tftp off>

 TFTP service is forbidden.

=item B<$ harden tftp status>

 TFTP service is permitted.
 TFTP service is stopped.

=back

=head1 CAVEATS

B<harden tftp> must be run only under a high privileges account.

=head1 SEE ALSO

B<harden ftp>

=cut

## @class tftp
package hardening::plugins::tftp;

use strict;
use warnings;

use lib "/opt/nortel/base/lib";
use common_variables;
use CommonInstallUtils qw(getYNResponse);
use base_firewall;
use base_ftp;
use base_system;
use baselog;
use hardening::hardening_common;
use base_snmp;

use Exporter;
our @ISA     = qw(Exporter);
our @EXPORT  = qw(&actions);
our $VERSION = '1.13';

## @skipline
sub actions();
## @skipline
sub status();
## @skipline
sub help();
## @skipline
sub start();
## @skipline
sub stop();
## @skipline
sub deactivate_tftp();
## @skipline
sub on();
## @skipline
sub off();
## @skipline
sub apply_default();
## @skipline
sub post_install_status();
## @skipline
sub synchronize_storage_and_system();
## @skipline
sub send_snmp_with_hardening_TFTP_state();
## @skipline
sub recovery();

#variables
my $module_name = "tftp";
my $xinet_tftp  = "tftp";

my %actions = (
               'on' => {
                        'sub'     => \&on,
                        'visible' => $TRUE
                       },
               'off' => {
                         'sub'     => \&off,
                         'visible' => $TRUE
                        },
               'start' => {
                           'sub'     => \&start,
                           'visible' => $FALSE
                          },
               'stop' => {
                          'sub'     => \&stop,
                          'visible' => $FALSE
                         },
               $H_IF_POST_LB_INSTALL => {
                                         'sub'     => \&apply_default,
                                         'visible' => $FALSE
                                        },
               $H_IF_POST_LB_RECOVERY => {
                                          'sub'     => \&synchronize_storage_and_system,
                                          'visible' => $FALSE
                                         },
               $H_IF_POST_LB_STATUS => {
                                        'sub'     => \&post_install_status,
                                        'visible' => $FALSE
                                       },
               $H_IF_SEND_SNMP => {
                                   'sub'     => \&send_snmp_with_hardening_TFTP_state,
                                   'visible' => $FALSE
                                  },
               $H_IF_RECOVERY => {
                                  'sub'     => \&recovery,
                                  'visible' => $FALSE
                                 },
               'status' => {
                            'sub'     => \&status,
                            'visible' => $TRUE
                           },
               'help' => {
                          'sub'     => \&help,
                          'visible' => $TRUE,
                          'default' => $TRUE
                         }
              );

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
}  ##end sub actions ()

###############################################################################
## @fn sub status()
# @brief Gets the current status of the Hardening option
# @details This method examines the Hardening storage in order to determine if
#          usage of TFTP service is permitted or not, and outputs this status.
#          If TFTP service is permitted, it ouptuts if the service is actually
#          running or not.
# @return TRUE - if the status of the Hardening option is got.
# @return          FALSE - some error happens
sub status()
{
   baselog($LOG_DEBUG, "Trying to get status TFTP service...");

   #get info from XML file
   my $status = is_option_permitted($module_name);

   my $retval = $FALSE;

   if ($status == $TRUE)
   {
      baselog($LOG_INFO, "TFTP service is permitted.\n", $TRUE);
      my $status = get_status_linux_service($xinet_tftp, undef, $HANDLE_XINET_SERVICE);
      if ($status == $TRUE)
      {
         baselog($LOG_INFO, "TFTP service is running.\n", $TRUE);
      }
      elsif ($status == $FALSE)
      {
         baselog($LOG_INFO, "TFTP service is stopped.\n", $TRUE);
      }
      else
      {
         baselog($LOG_ERROR, "Status of TFTP service is unknown.\n", $TRUE);
      }
      $retval = $TRUE;
   } ## end if ($status == $TRUE)
   elsif ($status == $FALSE)
   {
      baselog($LOG_INFO, "TFTP service is forbidden.\n", $TRUE);
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Hardening status of TFTP service is unknown.\n", $TRUE);
   }

   return $retval;
}  ##end sub get_status()

###############################################################################
## @fn sub help()
# @brief Output manual page describing the usage syntax
# @details This method parse POD section of the module and prepares manual
#          page describing the module.
# @return TRUE - if the page is generated.
# @return FALSE - some error happens
sub help()
{
   return do_help_action($module_name);
}  ##end sub get_help()

###############################################################################
## @fn sub start()
# @brief Activate xinet based tftp service
# @details This method examines the Hardening storage in order to determine if
#          usage of TFTP service is permitted or not. If the service is permitted,
#          the method tries to activate xinet based tftp service, changes
#          firewall rule and sends the  corresponding SNMP trap.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is running sucessfully
# @return FALSE - either the usage TFTP service is forbidden, or some error happens
sub start()
{
   baselog($LOG_DEBUG, "Activation of TFTP service...");

   #get info from the hardening storage file
   if (is_option_permitted($module_name) == $FALSE)
   {

      #warning message if TFTP service is forbidden
      baselog($LOG_WARNING, 'Usage of TFTP service is forbidden. Contact your Security Administrator for assistance.');
      return $FALSE;
   } ## end if (is_option_permitted...

   #Try to activate the service
   if (not activate_xinet_service($xinet_tftp))
   {
      baselog($LOG_ERROR, "Cannot activated TFTP service");
      return $FALSE;
   }

   # Try to open firewall ports
   if (not base_ftp::open_tftp_firewall())
   {
      baselog($LOG_ERROR, 'Cannot enable TFTP firewall ports.');
      return $FALSE;
   }

   baselog($LOG_INFO, 'TFTP service is activated.');

   send_snmp_trap("SEC0079");

   return $TRUE;
} ## end sub start()

###############################################################################
## @fn sub stop()
# @brief Deactivate xinet based tftp service
# @details This method tries to deactivate xinet based tftp service and sends
#          the corresponding SNMP trap.
#          This method is not provided as CLI command. It is intended in order
#          to run the service from other parts of code.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub stop()
{

   #Deactivate the service
   if (deactivate_tftp())
   {
      send_snmp_trap("SEC0080");
      return $TRUE;
   }

   return $FALSE;
} ## end sub stop()

###############################################################################
## @fn sub deactivate_tftp()
# @brief Deactivate tftp service
# @details This method tries to deactivate xinet based tftp service and change
#          firewall rule.
# @return TRUE - the service is stopped sucessfully
# @return FALSE - some error happens
sub deactivate_tftp()
{
   my $retval = $TRUE;
   baselog($LOG_DEBUG, "Deactivation of TFTP service...");

   if (not deactivate_xinet_service($xinet_tftp))
   {
      baselog($LOG_ERROR, "Cannot deactivate TFTP service");
      $retval = $FALSE;
   }

   baselog($LOG_INFO, 'TFTP service is deactivated.');

   # Try to close FTP ports
   if (not base_ftp::close_tftp_firewall())
   {
      baselog($LOG_ERROR, 'Cannot disable TFTP ports.');
      $retval = $FALSE;
   }

   return $retval;
} ## end sub deactivate_tftp()

###############################################################################
## @fn sub off()
# @brief Forbid usage of TFTP service
# @details This method changes the status of the Hardening option to "off" if it
#          is necessary, tries to perform stop of CS1000 TFTP service
#          and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub off()
{
   baselog($LOG_DEBUG, "Trying to forbid TFTP service...");

   my $retval = $FALSE;

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_OFF, \&stop, $FORCE_TO_CHANGE_STATUS);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "TFTP service is forbidden.\n", $TRUE);

      send_snmp_trap("SEC0078");

      # Try to restart firewall
      if (not base_firewall::firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
         $retval = $FALSE;
      }
      else
      {
         $retval = $TRUE;
      }

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change TFTP service permissions.\n", $TRUE);
   }

   return $retval;
} ## end sub off()

###############################################################################
## @fn sub on()
# @brief Permit usage of TFTP service
# @details This method changes the status of the Hardening option to "on" if it
#          is necessary, and sends the corresponding SNMP trap.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub on()
{
   baselog($LOG_DEBUG, "Trying to permit TFTP service...");

   my $retval = $FALSE;

   if (not is_option_permitted($module_name))
   {
      if (not getYNResponse("You are trying to set Hardening policy 'TFTP service' in less secure state.\nDo you want to proceed?", 'Y'))
      {
         print "Cancelled by user.\n";
         return $FALSE;
      }
   } ## end if (not is_option_permitted...

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, \&start, $CHANGE_SYSTEM_AFTER_XML);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "TFTP service is permitted.\n", $TRUE);

      send_snmp_trap("SEC0077");

      # Try to restart firewall
      if (not base_firewall::firewall_restart())
      {
         baselog($LOG_ERROR, "Cannot restart firewall.\n", $TRUE);
      }
      else
      {
         $retval = $TRUE;
      }

   } ## end if ($result == $TRUE)
   else
   {
      baselog($LOG_ERROR, "Cannot change TFTP service permissions.\n", $TRUE);
   }

   return $retval;
} ## end sub on()

###############################################################################
## @fn sub apply_default()
# @brief Apply default state of usage of TFTP service
# @details This method changes the status of the Hardening option in accordance
#          to the default state - it permits usage of TFTP service.
#          Default value is applied in the case when it is impossible to detect
#          the current TFTP usage policy.
# @return TRUE - the new status is set
# @return FALSE - some error happens
sub apply_default()
{
   baselog($LOG_DEBUG, "Trying to permit TFTP service...");

   my $retval = $FALSE;

   #Set new status to value 'on'
   my $result = set_new_hardening_status($module_name, $HARDENING_STATUS_ON, undef);
   if ($result == $TRUE)
   {
      baselog($LOG_INFO, "TFTP service is permitted.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Cannot change TFTP service permissions.");
   }

   return $retval;
} ## end sub apply_default()

###############################################################################
## @fn sub post_install_status()
# @brief Gets and shows the current status of the Hardening option
# @details This method changes gets the status of the Hardening option from the
#          Hardening storage and shows it.
#          This method is inteded to be invoked on post Linux Base installation
#          stage when all statuses applied for Enhanced Hardening options need
#          to be shown.
#          Since on Post-Installation stage all lines should contain carrier
#          return character (due to specific terminal settings), "\r" is added.
# @return TRUE - the status is got sucessfully
# @return FALSE - some error happens
sub post_install_status()
{
   my $retval = $TRUE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, prepare_module_status_for_banner($module_name, "The status of TFTP service use is unknown."), $TRUE);
      $retval = $FALSE;
   }
   elsif ($status)
   {
      print prepare_module_status_for_banner($module_name, "Use of TFTP service is permitted.");
   }
   else
   {
      print prepare_module_status_for_banner($module_name, "Use of TFTP service is forbidden.");
   }

   return $retval;
} ## end sub post_install_status()

###############################################################################
## @fn sub synchronize_storage_and_system()
# @brief Change the system in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of TFTP service is permitted or not. Since there are no actual
#          system changes for TFTP permitting, only a case when there is some
#          problem with the Hardening storage - default state of the option is
#          applied.
#          This method is intended to be used on the stage of recovery Linux
#          Base data after Linux Base installation. When there are no Nortel
#          applications installed yet. That is why SNMP traps are not sent by
#          this method.
# @return TRUE - data recovered or default applied
# @return FALSE - some issue happens
sub synchronize_storage_and_system()
{
   baselog($LOG_DEBUG, "Trying to synchronize state of TFTP service from the Hardening storage with the system.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      hardening_special_baselog($LOG_ERROR, "The status of usage of TFTP service is unknown. Default status is applied.");
      $retval = apply_default();
   }
   else
   {

      #There are no actual system changes in order to set status of TFTP service
      #If new status is "off" the current TFTP activities are  not taken down.
      $retval = $TRUE;
   } ## end else [ if ($status == $H_ERROR...

   return $retval;
} ## end sub synchronize_storage_and_system()

###############################################################################
## @fn sub send_snmp_with_hardening_TFTP_state()
# @brief Send SNMP trap in acordance with the state from the Hardening storage
# @details This module examines the Hardening storage in order to determine if
#          usage of TFTP service is permitted or not, and send the corresponding
#          SNMP trap.
#          This method is intended to be used on the stage when Linux Base is
#          doing the very first boot up, after Nortel SNMP application
#          installation.
# @return TRUE - SNMP trap is sent
# @return FALSE - some issue happens
sub send_snmp_with_hardening_TFTP_state()
{
   baselog($LOG_DEBUG, "Trying to send SNMP trap with the current state of TFTP service.");

   my $retval = $FALSE;

   #get info from the hardening storage file
   my $status = is_option_permitted($module_name);
   if ($status == $H_ERROR)
   {
      baselog($LOG_ERROR, "The status of usage of TFTP service is unknown.");
   }
   elsif ($status == $TRUE)
   {
      $retval = send_snmp_trap("SEC0077");
   }
   else
   {
      $retval = send_snmp_trap("SEC0078");
   }

   return $retval;
} ## end sub send_snmp_with_hardening_TFTP_state()

###############################################################################
## @fn sub recovery()
# @brief Apply the state of the Hardening option from restored Hardening storage.
# @details This module examines restored Hardening storage in order to determine
#          if usage of TFTP service is permitted or not, and performs all necessary
#          action to reflect the new state on the system (e.g. send the
#          corresponding SNMP trap).
#          This method is intended to be used when applications data restoration
#          is being performed on run-time system.
# @return TRUE - data restored sucessfully
# @return FALSE - some issue happens
sub recovery()
{
   baselog($LOG_DEBUG, "Trying to recovery state of TFTP service...");

   my $retval = $TRUE;

   $retval &= synchronize_storage_and_system();
   $retval &= send_snmp_with_hardening_TFTP_state();

   return $retval;
} ## end sub recovery()

1;
__END__
