################################################################
# @file    hostConfig.pm
# @brief   This module is the LinuxBase interface to manage
#          user configurations to /etc/hosts.
#          It stores user configured hosts (file hosts_usercfg.xml)
#          and provides interface to manipulate them.
#          Methods: host->add host->del host->show host->sync
# @author  Kushal Agarwal
# @date:   August 2007
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - the print_user_hosts() function has been changed to use printf command
 

package hostControl;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use install_common;

our $VERSION = 1.02;

### Used modules

use warnings;
use strict;

use English;
use Data::Dumper;
use XML::Writer;
use IO::File;
use baselog;
use ifxml;

## @skipline
sub read_host_xml;
## @skipline
sub save_xml_file;
## @skipline
sub update_etc_hosts;
## @skipline
sub new;
## @skipline
sub add;
## @skipline
sub del($$);
## @skipline
sub show(;$);
## @skipline
sub sync;

#####################################################################
########################## Private Data #############################

my $hostxml_name = "hosts_usercfg.xml";                    #Host DB XML file
my $host_xml     = "$SYSCONFIG_ROOT/$hostxml_name";
my $host_xml_bak = "$SYSCONFIG_ROOT/$hostxml_name" . ".bak";
my $section      = "hosts";                                #main tag in host xml file
my $tag          = "host";                                 #tag in host xml file

my $etc_host_file = "/etc/hosts";

###############################################################################
## @fn    INIT
# @brief  This subroutine checks to see if the host_xml file exists,
#         if not creates it!
# @return hosts_usercfg.xml - file exists or was created successfully
# @return $ERROR - file could not be created
INIT
{
   unless (-e $host_xml)
   {
      baselog($LOG_INFO, "'$host_xml' file is not exist. Creating.");
      my $fh;
      unless (open $fh, ">", $host_xml)
      {
         baselog($LOG_ERROR, "Could not create '$host_xml' file\n", $TRUE);
         exit($ERROR);
      }
      print $fh "<$section></$section>\n";
      close $fh;
   } ## end unless (-e $host_xml)
} ## end INIT

#####################################################################
########################## Private Methods ##########################

###############################################################################
## @method read_host_xml()
# @brief   This subroutine reads the host xml file and creates array of hashes
#          using the object of ifxml class.
# @return  $ret - array reference
# @return  undef - if error
sub read_host_xml
{
   my $ret;

   #Create parser
   my $xml_parser = ifxml::new($SYSCONFIG_ROOT, $hostxml_name);

   #Read xml file
   if ($xml_parser)
   {
      $ret = $xml_parser->find_data_by_xpath($GETDATA, "/$section/$tag");
   }
   else {return undef;}
   undef $xml_parser;
   return $ret;

} ## end sub read_host_xml

###############################################################################
## @method save_xml_file()
# @brief   This subroutine overwrites host xml file
# @param   @data - array reference with data to be written to xml file
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub save_xml_file
{
   my @data = @{shift @_};
   my ($output, $xml);

   $output = new IO::File(">$host_xml");
   unless (defined $output)
   {
      baselog($LOG_ERROR, "Cannot open $host_xml file");
      return $FALSE;
   }
   $xml = new XML::Writer(DATA_MODE => 'true', DATA_INDENT => 3, OUTPUT => $output);
   if (!$xml)
   {
      baselog($LOG_ERROR, "Cannot get XML::Writer");
      $output->close();
      return $FALSE;
   } ## end if (!$xml)

   $xml->startTag($section);  #OPEN section
   foreach my $hash (@data)
   {
      if (defined($hash))
      {
         $xml->emptyTag($tag, %{$hash});
      }
   } ## end foreach my $hash (@data)
   $xml->endTag($section);    #CLOSE section

   $xml->end();
   $output->close();

   return $TRUE;

} ## end sub save_xml_file

#####################################################################
########################## Public Methods ###########################

###############################################################################
## @method new()
# @brief   This subroutine is object constructor. Object is an array of hashes
# @return  $self - reference to object
# @return  undef - indicates an error condition.
sub new
{

   #initialization
   my $class = shift;
   my $self  = [];    #Class instance

   #Class variables initialization
   $class = "hostControl" unless $class;

   $self = read_host_xml();
   unless ($self)
   {
      baselog($LOG_ERROR, "Could not read '$host_xml'\n", $TRUE);
      return undef;
   }

   bless $self, $class;  #Declaring an object

   return $self;
} ## end sub new

###############################################################################
## @method backup_host_xml()
# @brief   This subroutine creates a backup copy of the hosts xml file
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub backup_host_xml
{
   unless (system($CP, "-p", $host_xml, $host_xml_bak) == 0)
   {
      baselog($LOG_ERROR, "Could not backup the '$host_xml' file\n");
      return $FALSE;
   }
   return $TRUE;
} ## end sub backup_host_xml

###############################################################################
## @method restore_host_xml()
# @brief   This subroutine rstores the hosts xml file from the backup copy
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub restore_host_xml
{
   unless (rename($host_xml_bak, $host_xml))
   {
      baselog($LOG_ERROR, "Could not restore the '$host_xml' file\n");
      return $FALSE;
   }
   return $TRUE;
} ## end sub restore_host_xml

###############################################################################
## @method delete_host_xml_backup()
# @brief   This function deletes the host xml backup file
# @return  none

sub delete_host_xml_backup
{
   unless (system($RM, "-f", $host_xml_bak) == 0)
   {
      baselog($LOG_WARNING, "Could not delete $host_xml_bak file.");
   }
}

###############################################################################
## @method add($data, $add)
# @brief   This subroutine creates a backup copy of the hosts xml file
# @param   $data - hash reference with data
# @param   $add  - hash reference with data to be added
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub add
{
   my ($data, $add) = @_;

   foreach my $dat (@$data)
   {
      if ($add->{ip} eq $dat->{ip})
      {
         if ($add->{host} eq $dat->{host})
         {
            if ($add->{domain} eq $dat->{domain})
            {
               print "Could not add host: Duplicate Host\n";
               return $FALSE;
            }
         } ## end if ($add->{host} eq $dat...
      } ## end if ($add->{ip} eq $dat...
   } ## end foreach my $dat (@$data)

   # Add host
   push @$data, $add;

   # Save hosts
   unless (backup_host_xml())
   {
      print "Could not backup the host database file, cannot revert changes in an error.\n";
      return $FALSE;
   }
   unless (save_xml_file($data))
   {
      baselog($LOG_ERROR, "Could not save the '$host_xml' file\n", $TRUE);
      print "Could not update the host database file. This change will not be saved.\n";
      return $FALSE;
   } ## end unless (save_xml_file($data...

   unless (sync($data))
   {
      if (!restore_host_xml())
      {
         print "Could not update the system host database file.\nCould not restore the host database file to its original state.\nThe host database file has been modified.\n";
      }
      else
      {
         print "Could not update the system host database file.\nAll changes will be disregarded.\n";
      }
      baselog($LOG_WARNING, "Could not add host: $add->{host} to '$etc_host_file' file\n", $TRUE);
      return $FALSE;
   } ## end unless (sync($data))

   delete_host_xml_backup();
   print "Host succesfully added\n";
   return $TRUE;
} ## end sub add

###############################################################################
## @method del($data, $add)
# @brief   This subroutine deletes host from array of hashes of hosts and
# invokes sync and save_xml_file functions
# @param   $data - hash reference with data
# @param   $add  - hash reference with data to be deleted
#                  OR array index of data to be deleted
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub del($$)
{
   my ($data, $del) = @_;
   my $ind = undef;

   if (is_hash($del))
   {
      my $i = 0;

      # define array index that will be deleted
      foreach my $dat (@$data)
      {
         if (($dat->{ip} eq $del->{ip}))
         {
            if (($dat->{host} eq $del->{host}))
            {
               if (($dat->{domain} eq $del->{domain}))
               {
                  $ind = $i;
                  last;
               }
            } ## end if (($dat->{host} eq $del...
         } ## end if (($dat->{ip} eq $del...

         $i++;
      } ## end foreach my $dat (@$data)
      unless (defined $ind)
      {
         baselog($LOG_ERROR, "Could not find host that should be deleted\n", $TRUE);
         return $FALSE;
      }
   } ## end if (ref($del) eq "HASH"...
   else
   {
      $ind = $del - 1;
   }
   if ((defined $data->[$ind]) && ($ind >= 0))  
   {
      my $del_data = $data->[$ind];

      #delete host
      delete $data->[$ind];

      #delete undef value in array
      @{$data} = grep {defined $_} @{$data};

      unless (backup_host_xml())
      {
         print "Could not backup the host database file, cannot revert changes in an error.\n";
         return $FALSE;
      }
      unless (save_xml_file($data))
      {
         baselog($LOG_ERROR, "Could not save the '$host_xml' file\n", $TRUE);
         print "Could not update the host database file. This change will not be saved.\n";
         return $FALSE;
      } ## end unless (save_xml_file($data...

      unless (sync($data))
      {
         if (!restore_host_xml())
         {
            print "Could not update the system host database file.\nCould not restore the host database file to its original state.\nThe host databaase file has been modified.\n";
         }
         else
         {
            print "Could not update the system host database file.\nAll changes will be disregarded.\n";
         }
         baselog($LOG_WARNING, "Could not delete host: $del_data->{host} from '$etc_host_file' file\n", $TRUE);
         return $FALSE;
      } ## end unless (sync($data)...
   } ## end if (defined $data->[$ind...
   else
   {
      print "Cannot find host: $del\n";
      return $FALSE;
   }

   delete_host_xml_backup();
   print "Host succesfully deleted\n";
   return $TRUE;
} ## end sub del

###############################################################################
## @method show($data)
# @brief   This subroutine prints hosts
# @param   $data - hash reference with data
# @param   $long - option which allows to show hosts with long hostnames.
#          This parameter is optional. 
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub show (;$)
{
   my ($data, $long) = @_;

   if (scalar(@$data) > 0)
   {
      my $i = 1;
      my $tmp_length = 26;
      if ($long)
      {
         $tmp_length = 0;
         foreach my $dat (@$data)
         {
            if (length($dat->{host}) > $tmp_length)
            {
              $tmp_length = length( $dat->{host});
            }
         }
      }      
      foreach my $dat (@$data)
      {
         print_user_hosts($i, $dat->{ip}, $dat->{host}, $dat->{domain}, $tmp_length);
         $i++;
      }
   } ## end if (scalar(@$data) > 0...
   else {print "The system has no user configured hosts.\n";}

   return $TRUE;
} ## end sub show

###############################################################################
## @method sync($data)
# @brief   This subroutine synchronizes the /etc/hosts file with the xml file
# @param   $data - hash reference with data
# @return  $TRUE - no errors
# @return  $FALSE - erorrs
sub sync
{
   my $data = shift(@_);

   #update /etc/hosts file
   my $fh;
   unless (open $fh, '<', $etc_host_file)
   {
      baselog($LOG_ERROR, "Could not open $etc_host_file: $!");
      return $FALSE;
   }

   my @host_str = <$fh>;
   close $fh;

   my @new_host_str = "";
   foreach my $str (@host_str)
   {
      push @new_host_str, $str;
      if ($str =~ "User Configured Hosts") {last;}
   }

   baselog($LOG_DEBUG, "Synchronizing $etc_host_file file with $hostxml_name");
   foreach my $dat (@$data)
   {
      if ($dat->{domain} ne "")
      {
         push @new_host_str, $dat->{ip}, "    ", $dat->{host}, ".", $dat->{domain}, "   ", $dat->{host}, "\n";
      }
      else
      {
         push @new_host_str, $dat->{ip}, "    ", $dat->{host}, "\n";
      }
   }

   my $tmpfile      = $etc_host_file . $$;
   unless (open $fh, '>', $tmpfile)
   {
      baselog($LOG_ERROR, "Cannot open $tmpfile file for writing: $!");
      return $FALSE;
   }

   print $fh @new_host_str;
   close $fh;

   system($CHMOD, "--reference=$etc_host_file", $tmpfile);
   unless (rename($tmpfile, $etc_host_file))
   {
      baselog($LOG_ERROR, "The $etc_host_file file could not be updated");
      print "Synchronizing the system host database file";
      status_failure("Synchronizing the system host database file");

      system($RM, "-f", $tmpfile);
      return $FALSE;
   }
   baselog($LOG_INFO, "The $etc_host_file file is updated");

   print "Synchronizing the system host database file";
   status_success("Synchronizing the system host database file");
   return $TRUE;
} ## end sub sync

###############################################################################
## @method print_user_hosts(@)
# @brief   This sub prints data in the specified format
# @param   @_ - data to print
# @return  none
sub print_user_hosts(@)
{
   my $host_header = "Hosts Table (user configured hosts)\n"; 
   my ($host_num, $ip, $host, $domain, $length) = @_;
   if ($host =~ /(\S{1,$length})/)
   {
      $host = $1;
   } 
   if ($host_num == 1)
   {
      print $host_header;
      printf ("%-5s   %-16s  %-${length}s  %-17s\n", 'HOST#', 'IP Address', 'Hostname', 'Domain' );
   }
   printf ("%-3d     %-16s  %-${length}s  %-17s\n",$host_num, $ip, $host, $domain);
} ## end sub print_user_hosts(@)

###############################################################################
## @method DESTROY
# @brief   Object destructor is used to reduce warnings output.
#          This function name is defined in Perl and must not be changed.
# @param   none
# @return  none
sub DESTROY
{
   return;
}

1;
__END__

