#! /usr/bin/perl

## @file hwRecognition.pm
# @brief allows applications get information about
#        HW platform type
# @details This PM module is a public interface
#          based on the C API (hwRecognition.c).
#         This API parses returns value according
#         to the following table:
# @author Yulya Bezrukova
# @date March, 2008
# @version 1.00 - Initial version
# @version 1.01 - The following functions were added: get_memory_stat(),get_hd_type(),get_hd_size()
# @version 2.00 - Modified to support more HW information (PEC, serial, etc...)
# @version 2.01 - new argument has been added to the 'new' fucntion
# @version 2.02 - R6.0 -> R7.0 upgrade - dummy is_cpdc() and is_cpmg() functions added.
# @note All subroutines from this module uses the functions declared in the hwRecognition.c

## @namespace hwRecognition
package hwRecognition;

use ExtUtils::testlib;  # adds blib/* directories to @INC
use Math::BigInt;

use warnings;
use strict;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
  &get_memory_stat
  &get_hd_type
  &get_hd_size
  &initialize_hw_recognition
  &sysPlatformNameGet
  &get_pec
  &get_serial
  &get_slot
  &get_bios_version
  &generate_hwinfo_file
  &is_cppm
  &is_cpdc
  &is_cpmg
  $HWINFO_FILE
  $HW_PLATFORM
  $HW_HDD_SIZE
  $HW_MEM_SIZE
  $HW_PEC
  $HW_SERIAL
  $HW_SLOT
  );

use baselog;
use base_system;
use dmiParser;
use Hw;

our $VERSION = '2.02';

our $HWINFO_FILE = '/etc/opt/nortel/base/hwInfo.dat';

our $HW_PLATFORM = 'PLATFORM';
our $HW_HDD_SIZE = 'HDD_SIZE';
our $HW_MEM_SIZE = 'MEM_SIZE';
our $HW_PEC      = 'PEC';
our $HW_SERIAL   = 'SERIAL';
our $HW_SLOT     = 'SLOT';
our $HW_MSP430   = 'MSP430VER';
our $HW_BIOS_VER = 'BIOS_VER';

# No need to export. There is a special function is exported: is_cppm()
my $IS_CPPM = 'IS_CPPM';
my $CPPM    = 'CPPM';

# $HWINFO_FILE permissions
my $HWINFO_PERMISSIONS = 0644;

my %HW_FIELDS = (
                 $HW_PLATFORM => \&sysPlatformNameGet,
                 $HW_HDD_SIZE => \&get_hd_size,
                 $HW_MEM_SIZE => \&get_memory_stat,
                 $HW_PEC      => \&get_pec,
                 $HW_SERIAL   => \&get_serial,
                 $HW_SLOT     => \&get_slot,
                 $HW_MSP430   => \&get_msp430_version,
                 $HW_BIOS_VER => \&get_bios_version
                );

my $PLATFORM_DL320G4 = 'HP DL320G4';
my $PLATFORM_X306M   = 'IBM X306M';
my $PLATFORM_X3350   = 'IBM X3350';
my $PLATFORM_R300    = 'Dell R300';

my %PEC_CODES = (
                 $PLATFORM_DL320G4 => 'NTDK97AAE5',
                 $PLATFORM_X306M   => 'NTDK99AAE5',
                 $PLATFORM_X3350   => 'NTDW40AAE5',
                 $PLATFORM_R300    => 'NTDW41AAE6'
                );

## @skipline
sub get_memory_stat();
## @skipline
sub get_hd_type();
## @skipline
sub get_hd_size();
## @skipline
sub sysPlatformTypeGet();
## @skipline
sub sysPlatformNameGet();
## @skipline
sub new($;$);
## @skipline
sub read_from_file($);
## @skipline
sub print_hw_info($);
## @skipline
sub get_hw_data($);
## @skipline
sub is_cppm(;$);
## @skipline
sub is_cpdc(;$);
sub is_cpmg(;$);
## @skipline
sub is_hp_dl320g4(;$$);
## @skipline
sub is_hp(;$$);
## @skipline
sub is_ibm_x306m(;$$);
## @skipline
sub is_ibm_x3350(;$$);
## @skipline
sub is_ibm(;$$);
## @skipline
sub is_dell_r300(;$$);
## @skipline
sub is_dell(;$$);
## @skipline
sub getHWPlatformName();
## @skipline
sub get_cppm_slot();
## @skipline
sub get_cppm_msp430_version();
## @skipline
sub get_cppm_pec_serial();
## @skipline
sub set_cppm_pec_serial($);
## @skipline
sub get_bios_version();
## @skipline
sub get_pec();
## @skipline
sub get_serial();
## @skipline
sub get_slot();
## @skipline
sub get_msp430_version();
## @skipline
sub get_slot();
## @skipline
sub get_hw_data($);
## @skipline
sub generate_hwinfo_file($);

## @fn sysPlatformTypeGet
# @brief Returns the digit corresponding to the platform
#        type.
# @details This function returns values from 1 to 6 (see the table above)
# @param None
# @retval - depends on Platform type. (For more details look
#           into the file hwRecognition.c.
# @note This function uses C API
sub sysPlatformTypeGet()
{
   my $type = Hw::sysPlatformTypeGet();
   return $type;
}

## @fn sysPlatformNameGet()
# @brief   Provides the HW platform name
# @retval $name - the name of the HW platform type.
#         It can be empty in case some error occures.
# @note This function uses C API
sub sysPlatformNameGet()
{
   my $name = Hw::sysPlatformNameGet();
   return $name;
}

## @fn       get_memory_stat()
# @brief     This subroutine tries to calculate the size of memory installed.
# @details   None.
# @return    $mbytes    - value of the total system memory in MB
# @return    undef      - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      May 2008
# @note      The subroutine always round the number of available memory
# @note      to be approximated to the the highest number of Gigabytes.
sub get_memory_stat()
{
   my $addrs    = undef;
   my $name     = undef;
   my $start    = undef;
   my $end      = undef;
   my $decstart = undef;
   my $decend   = undef;
   my $fd       = undef;
   my $iomem    = "/proc/iomem";
   my $sum      = Math::BigInt->new();

   unless (open($fd, "<", $iomem))
   {
      baselog($LOG_ERROR, "Cannot open file for reading [$iomem]");
      return undef;
   }
   foreach my $line (<$fd>)
   {
      chomp $line;
      ($addrs, $name) = split(/ : /, $line);
      if ($line =~ /(System RAM)/)
      {
         ($start, $end) = split(/-/, $addrs);
         $start =~ s/(\w+)/0x$1/;
         $end   =~ s/(\w+)/0x$1/;

         #the standard perl function hex() cannot be used here
         #in order to avoid integer overflow in hexadecimal number
         $decstart = Math::BigInt->new($start);
         $decend   = Math::BigInt->new($end);
         $decend->bsub($decstart);
         $sum->badd($decend);  #sum in bytes
      } ## end if ($line =~ /(System RAM)/...
   } ## end foreach my $line (<$fd>)
   $sum->bdiv('1024');         #size of installed system memory in Kb
   $sum->bdiv('1024');         #size of installed system memory in MB before rounding it up
   unless ($sum)
   {
      baselog($LOG_ERROR, "Unable to get memory size: $!");
   }
   my $realsum = $sum->copy();
   $sum->bdiv('1024');         #size of installed system memory in Gb before rounding it up
   my $roundsum = $sum->copy();
   $roundsum->bmul('1024');
   if ($realsum > $roundsum)   #if it's need to be rounding up to the next Gb
   {
      $sum->binc();            #rounding up to the next Gb
   }
   $sum->bmul('1024');         #size of installed system memory in MB after rounding it up
   return $sum;
} ## end sub get_memory_stat()

## @fn       get_hd_type()
# @brief     defines type of hard disk
# @details   None.
# @return    $type      - type of hard disk
# @return    undef      - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      May 2008
sub get_hd_type()
{
   my $type = undef;
   my $fh   = undef;

   unless (open($fh, '<', $FSTAB))
   {
      baselog($LOG_ERROR, "Cannot open file $FSTAB for reading: $!");
      return undef;
   }

   while (my $fstab_line = <$fh>)
   {

      # define type of hard disk
      if ($fstab_line =~ /\/dev\/([^\s\d]*)\d*\s*\/\s*/)
      {
         $type = $1;
         last;
      }
   } ## end while (my $fstab_line = <$fh>...
   return $type;
} ## end sub get_hd_type()

## @fn       get_hd_size()
# @brief     gets size of hard disk
# @details   None.
# @return    $size      - size of hard disk in MB
# @return    undef      - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      May 2008
sub get_hd_size()
{
   my $fh     = undef;
   my $size   = undef;
   my $HDtype = get_hd_type();
   unless ($HDtype)
   {
      baselog($LOG_ERROR, "Cannot define hard disk type");
      return undef;
   }

   unless (open($fh, '<', $PARTITIONS))
   {
      baselog($LOG_ERROR, "Cannot open file $PARTITIONS for reading: $!");
      return undef;
   }

   while (my $partitions_line = <$fh>)
   {

      # define type of hard disk
      if ($partitions_line =~ /\s*\d*\s*\d*\s*(\d*)\s*$HDtype\W*/)
      {
         $size = int($1 / 1024 + 0.5);
         last;
      }
   } ## end while (my $partitions_line...
   return $size;
} ## end sub get_hd_size()

## @fn       initialize_hw_recognition()
# @brief     find out the platform type
# @details   None.
# @return    TRUE         - Write the platform-type into LBINFO file
# @return    FALSE        - Can not wirte the type into LBINFO file
# @date      Oct 2008
# @note      This subrutine is only intended to be invoked on
#            post-installation stage.
#            Do not invoke it on run-time system
#            (since it changes nortel-linuxbase-info file)
sub initialize_hw_recognition()
{
   my $platform = get_hw_type();
   if ($platform eq "UNKNOWN")
   {

      # If the platform is UNKNOWN, giving a warning
      print "\r
     ------------------------------------------------------------\r
     -  Warning: The platform type of this machine is UNKNOWN   -\r
     -                  Precautions are required                -\r
     ------------------------------------------------------------\r\n";
   } ## end if ($platform eq "UNKNOWN"...
   my $fh;
   if (not open($fh, ">>", $LBINFO))
   {
      baselog($LOG_ERROR, "Can't open $LBINFO file: $!");
      return $FALSE;
   }
   print $fh "SYSTEM_HW_PLATFORM=$platform\n";
   close $fh;
   return $TRUE;
} ## end sub initialize_hw_recognition()

## @fn new($class, $regenerate)
# @brief Creates new object
# @param $class - class name
# @param $regenerate - do not read data from the $HWINFO_FILE file. Get data
#        from the system (by default read from the file).
# @return $self - created object
# @return none - if any errors occurs
sub new($;$)
{
   my ($class, $regenerate) = @_;
   my $self = {};

   if (not defined $class)
   {
      baselog($LOG_ERROR, 'Class is undefined.');
      return undef;
   }
   bless $self, $class;

   if (!defined($regenerate) || !$regenerate)
   {

      # Initialize values from the $HWINFO_FILE file
      $self->read_from_file();
   } ## end if (!defined($regenerate...

   # Get platform and is_cppm first
   # This is necessary because is_cppm field is used in functions get_pec and get_serial
   if (not exists($self->{$HW_PLATFORM}))
   {
      $self->{$HW_PLATFORM} = getHWPlatformName();
   }
   if (not exists($self->{$IS_CPPM}))
   {
      $self->{$IS_CPPM} = $self->is_cppm($self->{$HW_PLATFORM});
   }
   if ($self->is_cppm())
   {

      # Set PEC and SERIAL number
      $self->set_cppm_pec_serial();
   } ## end if ($self->is_cppm())

   # The following keys will not be populated as they are already done:
   #    PLATFORM
   #    IS_CPPM - not in %HW_FIELDS
   #    PEC    - cppm only
   #    SERIAL - cppm only
   foreach my $key (keys(%HW_FIELDS))
   {
      if (not exists($self->{$key}))
      {
         $self->{$key} = $HW_FIELDS{$key}->();
      }
   } ## end foreach my $key (keys(%HW_FIELDS...

   return $self;
} ## end sub new($;$)

## @fn read_from_file($)
# @brief Reads data from hw_info.dat file
# @details This function parses file and fills in hash object
# @param $self - reference to the class object
# @return none
sub read_from_file($)
{
   my $self = shift(@_);
   return if (not defined($self));
   if (!-f $HWINFO_FILE)
   {
      return;
   }
   my $fh;
   if (not open($fh, '<', $HWINFO_FILE))
   {
      baselog($LOG_WARNING, "Cannot open $HWINFO_FILE file");
      return;
   }
   my @content = <$fh>;
   if (not close($fh))
   {
      baselog($LOG_ERROR, "Error closing $HWINFO_FILE file");
   }
   foreach my $line (@content)
   {
      next if ($line =~ /^#/);
      my ($key, $value) = split(/=/, $line);
      if (not defined($value))
      {
         baselog($LOG_WARNING, "Unrecognized line: $line");
         next;
      }

      # Trimming
      $key   =~ s/^\s*(.+)\s*$/$1/;
      $value =~ s/^\s*(.+)\s*$/$1/;

      if (exists($HW_FIELDS{$key}))
      {
         $self->{$key} = $value;
      }
      else
      {
         baselog($LOG_WARNING, "Unknown key: $key = $value");
      }
   } ## end foreach my $line (@content)
} ## end sub read_from_file($)

## @fn print_hw_info
# @brief Prints current data from object
# @details This function was created for debug purposes
# @param $self - reference to the class object
# @return none
sub print_hw_info($)
{
   my $self = shift(@_);
   print "HW INFO\n-------\n";
   return if (not defined($self));
   foreach my $key (keys(%$self))
   {
      print $key . '=' . $self->{$key} . "\n";
   }
} ## end sub print_hw_info($)

## @fn is_cppm(;$$)
# @brief  Return if this platform is CPPM or not
# @param $self - reference to class object (optional)
# @return $TRUE  - platform is CPPM
# @return $FALSE - platform is not CPPM
# @author Alex Arkhipov
# @date October 2008
sub is_cppm(;$)
{
   my $self = shift(@_);
   my $ret  = $FALSE;
   if (is_install_stage())
   {
      my $hwType = get_hw_type();
      $ret = ($hwType =~ /$CPPM/) ? $TRUE : $FALSE;
   }
   elsif (not defined($self))
   {
      $ret = (Hw::hwIsCppm() == 1) ? $TRUE : $FALSE;
   }
   else
   {
      $ret = (defined $self->{$IS_CPPM}) ? $self->{$IS_CPPM} : $FALSE;
   }

   return $ret;
} ## end sub is_cppm(;$)

## Dummy functions to accommondate R6.0 -> R7.0 System Upgrades
sub is_cpdc(;$) { return $FALSE; }
sub is_cpmg(;$) { return $FALSE; }


## @fn is_hp_dl320g4(;$$)
# @brief  Return if this platform is HP DL320G4 or not
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is HP DL320G4
# @return $FALSE - platform is not HP DL320G4
# @author Alex Arkhipov
# @date October 2008
sub is_hp_dl320g4(;$$)
{
   my ($self, $name) = @_;
   if (not defined($self) and not defined($name))
   {
      $name = getHWPlatformName();
   }
   elsif (not defined($name))
   {

      # only object is provided
      $name = $self->{$HW_PLATFORM};
   } ## end elsif (not defined($name)...

   return (($name eq $PLATFORM_DL320G4) ? $TRUE : $FALSE);
} ## end sub is_hp_dl320g4(;$$)

## @fn is_hp(;$$)
# @brief  Return if this platform is any HP platform
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is HP
# @return $FALSE - platform is not HP
# @author Dan DeAraujo
# @date November 2008
sub is_hp(;$$)
{
   if (is_hp_dl320g4($_))
   {
      return $TRUE;
   }
   else
   {
      return $FALSE;
   }
} ## end sub is_hp(;$$)

## @fn is_ibm_x306m(;$$)
# @brief  Return if this platform is IBM x306m or not
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is IBM x306m
# @return $FALSE - platform is not IBM x306m
# @author Alex Arkhipov
# @date October 2008
sub is_ibm_x306m(;$$)
{
   my ($self, $name) = @_;
   if (not defined($self) and not defined($name))
   {
      $name = getHWPlatformName();
   }
   elsif (not defined($name))
   {

      # only object is provided
      $name = $self->{$HW_PLATFORM};
   } ## end elsif (not defined($name)...

   return (($name eq $PLATFORM_X306M) ? $TRUE : $FALSE);
} ## end sub is_ibm_x306m(;$$)

## @fn is_ibm_x3350(;$$)
# @brief  Return if this platform is IBM x3350 or not
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is IBM x3350
# @return $FALSE - platform is not IBM x3350
# @author Alex Arkhipov
# @date October 2008
sub is_ibm_x3350(;$$)
{
   my ($self, $name) = @_;
   if (not defined($self) and not defined($name))
   {
      $name = getHWPlatformName();
   }
   elsif (not defined($name))
   {

      # only object is provided
      $name = $self->{$HW_PLATFORM};
   } ## end elsif (not defined($name)...

   return (($name eq $PLATFORM_X3350) ? $TRUE : $FALSE);
} ## end sub is_ibm_x3350(;$$)

## @fn is_ibm(;$$)
# @brief  Return if this platform is any IBM platform
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is IBM
# @return $FALSE - platform is not IBM
# @author Dan DeAraujo
# @date November 2008
sub is_ibm(;$$)
{
   if (is_ibm_x306m($_) || is_ibm_x3350($_))
   {
      return $TRUE;
   }
   else
   {
      return $FALSE;
   }
} ## end sub is_ibm(;$$)

## @fn is_dell_r300(;$$)
# @brief  Return if this platform is DELL R300 or not
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is DELL R300
# @return $FALSE - platform is not DELL R300
# @author Alex Arkhipov
# @date October 2008
sub is_dell_r300(;$$)
{
   my ($self, $name) = @_;
   if (not defined($self) and not defined($name))
   {
      $name = getHWPlatformName();
   }
   elsif (not defined($name))
   {

      # only object is provided
      $name = $self->{$HW_PLATFORM};
   } ## end elsif (not defined($name)...

   return (($name eq $PLATFORM_R300) ? $TRUE : $FALSE);
} ## end sub is_dell_r300(;$$)

## @fn is_dell(;$$)
# @brief  Return if this platform is any Dell platform
# @param $self - reference to class object (optional)
# @param $name - name to parse (optional)
# @return $TRUE  - platform is Dell
# @return $FALSE - platform is not Dell
# @author Dan DeAraujo
# @date November 2008
sub is_dell(;$$)
{
   if (is_dell_r300($_))
   {
      return $TRUE;
   }
   else
   {
      return $FALSE;
   }
} ## end sub is_dell(;$$)

## @fn getHWPlatformName()
# @brief  Provides the HW platform name from nortel-cs1000-info file
# @return $name - the name of the HW platform type.
#         It can be empty in case some error occures.
# @note This function uses C API
sub getHWPlatformName()
{
   return Hw::hwPlatformNameGet();
}

## @fn get_cppm_clot()
# @brief Provides cppm slot number
# @return $slot - cppm slot number
# @note This function uses C API. Must be used for CPPM only
sub get_cppm_slot()
{
   return Hw::hwCppmSlotGet();
}

## @fn get_cppm_msp430_version()
# @brief Provides msp430 version for CPPM
# @return $msp430 - msp430 version
# @note This function uses C API. Must be used for CPPM only
sub get_cppm_msp430_version()
{
   return Hw::hwMsp430VersionGet();
}

## @fn get_cppm_pec_serial()
# @brief Provides PECSerial for CPPM
# @return $p_s - PECSerial for CPPM
# @note This function uses C API. Must be used for CPPM only
sub get_cppm_pec_serial()
{
   return Hw::hwPecSerialGet();
}

## @fn set_cppm_pec_serial()
# @brief reads pec and serial, split it by ' ' and sets PEC and Serial
# @param $self - reference to the object
# @return none
sub set_cppm_pec_serial($)
{
   my $self = shift(@_);
   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Incorrect invocation.');
      return;
   }

   # For CPPM only
   if (not $self->is_cppm())
   {
      return;
   }
   my $pec_serial = get_cppm_pec_serial();
   my ($pec, $serial) = split(/ /, $pec_serial);
   $self->{$HW_PEC}    = (defined($pec))    ? $pec    : '';
   $self->{$HW_SERIAL} = (defined($serial)) ? $serial : '';
} ## end sub set_cppm_pec_serial($)

## @fn get_bios_version()
# @brief Return bios version
# @details Uses dmiDecode to get version
# @return $bios_ver - bios version
# @author Alex Arkhipov
# @date October 2008
sub get_bios_version()
{
   return dmi_get_bios_version();
}

## @fn get_pec()
# @brief  Return pec for the platform
# @details For CPPM reads pec from BIOS
#          For other platforms uses hardcoded values
# @return $pec - platform's PEC
# @author Alex Arkhipov
# @date October 2008
sub get_pec()
{
   my $pec           = '';                  # Default value
   my $platform_name = getHWPlatformName();
   if (is_cppm())
   {
      my $pec_serial = get_cppm_pec_serial();
      my $serial;
      ($pec, $serial) = split(/ /, $pec_serial);
      $pec = (defined($pec)) ? $pec : '';
   } ## end if (is_cppm())
   elsif (exists($PEC_CODES{$platform_name}))
   {
      $pec = $PEC_CODES{$platform_name};
   }
   else
   {
      baselog($LOG_ERROR, 'Unknown PEC for platform' . $platform_name);
   }
   return $pec;
} ## end sub get_pec()

## @fn get_serial()
# @brief  Return serial for the platform
# @details For CPPM reads pec from BIOS
#          For other platforms uses dmidecode tool
# @return $serial - platform's serial
# @author Alex Arkhipov
# @date October 2008
sub get_serial()
{
   my $serial = '';
   if (is_cppm())
   {
      my $pec_serial = get_cppm_pec_serial();
      my $pec;
      ($pec, $serial) = split(/ /, $pec_serial);
      $serial = (defined($serial)) ? $serial : '';
   } ## end if (is_cppm())
   else
   {
      $serial = dmi_get_system_serial();
   }
   return $serial;
} ## end sub get_serial()

## @fn get_slot()
# @brief  Return slot for the platform
# @details For CPPM reads pec from BIOS
#          For other platforms uses 0
# @return $slot - platform's slot
# @author Alex Arkhipov
# @date October 2008
sub get_slot()
{
   my $slot = 0;  # Default one
   if (is_cppm())
   {
      $slot = get_cppm_slot();
   }
   return $slot;
} ## end sub get_slot()

## @fn get_msp430_version()
# @brief  Return msp for the platform
# @details For CPPM reads msp from BIOS
#          For other platforms uses 0
# @return $msp430 - platform's slot
# @author Alex Arkhipov
# @date October 2008
sub get_msp430_version()
{
   my $msp = 0;  # Default one
   if (is_cppm())
   {
      $msp = get_cppm_msp430_version();
   }
   return $msp;
} ## end sub get_msp430_version()

## @fn get_hw_info($)
# @brief Return HW info
# @param $hw_key - key of data to return. Must belong to HWINFO_FIELDS hash
# @return $value - hw data value
# @return undef  - if no such hw data
# @author Alex Arkhipov
# @date October 2008
sub get_hw_data($)
{
   my ($self, $hw_key) = @_;
   if (not defined $self or not defined $hw_key)
   {
      baselog($LOG_ERROR, 'Insufficient parameters provided');
      return undef;
   }
   if (not exists($self->{$hw_key}))
   {
      baselog($LOG_ERROR, 'Unknown hw_key provided:' . $hw_key);
      return undef;
   }
   return $self->{$hw_key};
} ## end sub get_hw_data($)

## @fn generate_hwinfo_file(;$)
# @brief generate $HWINFO_FILE file
# @param $self - reference to hash with values
# @return $TRUE  - file was written successfully
# @return $FALSE - error writing file
# @author Alex Arkhipov
# @date October 2008
sub generate_hwinfo_file($)
{
   my $self = shift(@_);
   if (not defined $self)
   {
      baselog($LOG_ERROR, 'Insufficient parameters provided');
      return $FALSE;
   }
   my $content = '';
   foreach my $key (keys(%HW_FIELDS))
   {
      if (exists($self->{$key}))
      {
         $content .= $key . '=' . $self->{$key} . "\n";
      }
      else
      {
         baselog($LOG_ERROR, "The $key key doesn't exist");
      }
   } ## end foreach my $key (keys(%HW_FIELDS...
   if (write_file($HWINFO_FILE, $content, $NO_BACKUP))
   {
      chmod $HWINFO_PERMISSIONS, $HWINFO_FILE;
      return $TRUE;
   }
   return $FALSE;
} ## end sub generate_hwinfo_file($)

1;

__END__
