#####################################################
## @file   ifxml.pm
# @brief   Reads data from xml.
# @details This module reads data from XML files and
#          allows an access to them through the autoloading subroutine
# @author   Ilya Teterev, Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - bugfix in the AUTOLOAD subroutine. The support of
#          the baselog module is added.
# @version 1.02 - the find_data_by_xpath subroutine is added.  new,
#          DESTROY, AUTOLOAD, xmlraw methods are deleted
#          from export list.
# @version 1.03 - Two colons were removed before shared functions and variables.
# @version 1.04 - bugfix in $compress code(value 0 was proccessed like undefined).
# @version 1.05 - Changed the xml_find method to use perl find rather than
#          the shell command.
# @version 1.06 - xml validating supported
#

package ifxml;

### Access management
use Exporter;
our @ISA    = qw(Exporter);  # : public Exporter;
our @EXPORT = qw(&filelist
  $GETTEXT
  $GETDATA
  $GETCOUNT
  $NOATTR
  );
our $VERSION = 1.05;

### Used modules
use lib "/opt/nortel/base/lib";
use common_variables;
use common_functions;
use XML::Parser;
use XML::Parser::EasyTree;
use Data::Dumper;
use File::Find;
use baselog;
use warnings;

### Private vars
BEGIN
{

   #Min number of configure.xml files required to be found
   our $xml_files_min = 1;
   our $retval        = $FALSE;           #Return value
   our $class         = "main";           #Class name
   our $search_root   = ".";              #Search parh
   our $search_name   = "configure.xml";  #XML file
   our %search_types = (
                        GETDATA => 'get',
                        COUNT   => 'count'
                       );
   our $GETTEXT  = 'text';
   our $GETDATA  = 'data';
   our $GETCOUNT = 'count';
   our $NOATTR   = '';
} ## end BEGIN

### Private functions

#Description: This function looks for tag name in XMLRAW
#Input:  $where - array ref to XML three node
#        $what  - search pattern
#Output: undef if the array is empty
#        overwise array ref to fetched data
our $cutsearch = sub {
   local $self   = shift @_;
   local $retval = undef;
   local ($where, $what) = @_;

   foreach $block (@$where)
   {
      if (   defined($block->{'type'})
          && $block->{'type'} eq "t"
          && !defined($what))
      {

         #prepare text blocks
         my $text = $block->{content};

         #check for empty content
         $text =~ s/[\s\n\r]*//g;
         push @{$retval->{'T'}}, {'text', $block->{content}} if ($text ne '');

      } ## end if (defined($block->{'type'...
      elsif (   (defined($block->{'type'}) && ($block->{'type'} eq "e") && !defined($what))
             || (defined($block->{'name'}) && ($block->{'name'} eq $what)))
      {
         my $tempblock = undef;
         $tempblock = $block->{'attrib'} if (%{$block->{'attrib'}});
         $tempblock->{'_tagname'} = $block->{'name'} if (defined($block->{'name'}));
         if (@{$block->{'content'}})
         {

            #collecting parameters if any
            my $tempval = $self->$cutsearch($block->{'content'}, undef);

            if (defined($tempval->{'T'}))
            {
               foreach $textline (@{$tempval->{'T'}})
               {
                  foreach (keys(%$textline))
                  {
                     $tempblock->{$_} .= $textline->{$_};
                  }
               } ## end foreach $textline (@{$tempval...
            } ## end if (defined($tempval->...
            else
            {
               $tempblock->{'_xmltag'} = $tempval->{'E'} if $tempval->{'E'};
            }
         } ## end if (@{$block->{'content'...
         push @{$retval->{'E'}}, $tempblock;
      } ## end elsif ((defined($block->{...
      elsif (@{$block->{'content'}})
      {

         #collecting parameters if any
         my $tempval = $self->$cutsearch($block->{'content'}, $what);
         push @{$retval->{'E'}}, @{$tempval->{'E'}} if $tempval->{'E'};
      } ## end elsif (@{$block->{'content'...
   } ## end foreach $block (@$where)
   return $retval;
};  ## end sub cutsearch

#Description: This function prepares final output
#Input:       $tree - hash ref to the tree
#             $filter - ref to tag list
#Output:      Final output
our $compress = sub {
   local $self   = shift @_;
   local $tree   = shift @_;
   local $filter = shift @_;
   my $tmparr = ();
   my $item;
   my $elem;

   $atag = shift @$filter;
   foreach $item (@$tree)
   {
      if (ref($item) eq "HASH")
      {
         push @$tmparr, ($item->{$atag}) if (defined $item->{$atag});

         #Looking for inner elements
         if ($item->{'_xmltag'})
         {
            foreach $elem (@{$item->{'_xmltag'}})
            {
               push @$tmparr, $elem if (defined($elem->{'_tagname'}) && $elem->{'_tagname'} eq $atag);
            }
         } ## end if ($item->{'_xmltag'}...
      } ## end if (ref($item) eq "HASH"...
      if (ref($item) eq "ARRAY")
      {
         foreach $elem (@{$item})
         {
            push @$tmparr, $elem if ($elem->{'_tagname'} eq $atag);
         }
      } ## end if (ref($item) eq "ARRAY"...
   } ## end foreach $item (@$tree)
   $tmparr = $self->$compress($tmparr, $filter) if scalar(@$filter);
   return $tmparr;
};

#Autoloader
#Description: This function performs a delay before function calls
#             to create a new function if this function name is
#             undefined before
#Input:  hash ref to filter data
#Output: array ref to retrieved data or undef if error/empty array
#This function name is defined in Perl and must not be changed.
sub AUTOLOAD
{
   local $self   = shift @_;
   local $retarr = undef;
   my $stype    = undef;     #Search action
   my $funcname = $AUTOLOAD;
   my @fparms   = ();        #[0] - tag, [1] - fields
   my $cond     = shift @_;
   my $found    = 0;
   my $atag     = undef;     #active tag

   $funcname =~ s/^${class}:://;

   while (my ($key, $value) = each %search_types)
   {
      ($funcname =~ /^$value\_.*$/) and ($stype = $key);
   }

   unless (defined $stype)
   {
      baselog($LOG_WARNING, "Undefined function is called: $AUTOLOAD");

      #undef is used to return an error status, because () == $FALSE;
      return undef;
   } ## end unless (defined $stype)

   #query format:
   #get_<field>_<param>({condition=>value,...})
   #
   $funcname =~ s/$search_types{$stype}\_//;
   @fparms = split /_/, $funcname;

   #Step 1: preparing a cut
   $atag = shift @fparms;
   if ($atag)
   {
      local $refarray = $self->xmlraw;

      $retarr = $self->$cutsearch($refarray, $atag);
      if (!defined $retarr)
      {
         baselog($LOG_WARNING, "The '$atag' tag is not found in XML sources");
         return undef;
      }

      $retarr = $retarr->{'E'};
   } ## end if ($atag)

   #Step 2: checking received conditions
   if ($cond and ref($cond) eq "HASH")
   {
      local @tmparr = ();
      foreach $item (@$retarr)
      {
         if (ref($item) eq "HASH")
         {
            $found = 1;
            foreach $key (keys(%$cond))
            {
               if ((!defined $item->{$key}) || ($item->{$key} ne $cond->{$key}))
               {
                  $found = 0;
                  last;
               }
            } ## end foreach $key (keys(%$cond))
            delete $item->{'_xmltag'} if !scalar(@fparms);
            push @tmparr, ($item) if $found;
         } ## end if (ref($item) eq "HASH"...
      } ## end foreach $item (@$retarr)
      @$retarr = @tmparr;
   } ## end if ($cond and ref($cond...
   elsif ($cond and ref($cond) ne "HASH")
   {
      baselog($LOG_WARNING, "Function parameter is skipped");
   }

   #Step 3: getting required parameters
   if (scalar(@fparms))
   {
      $retarr = $self->$compress($retarr, \@fparms);
   }

   #Cleanup
   if (defined(@$retarr))
   {
      @$retarr = map {

         #remove system data
         delete $_->{'_xmltag'};
         delete $_->{'_tagname'};
         $_;
      } @$retarr;
   } ## end if (defined(@$retarr))
   $retarr = scalar(@$retarr) if ($stype eq "COUNT");
   return $retarr;
} ## end sub AUTOLOAD

# @fn xml_find
# @brief This function fills an array with paths to XML files
# @param none
# @return $FALSE if the array is empty
my $xml_find = sub {
   no warnings File::Find;
   local $self = shift @_;

   @{$self->{FILELIST}} = ();
   if (-e $search_root)
   {
      find(
         sub {
            if ($_ eq $search_name)
            {
               if (not exists $XML_PATHS{$File::Find::name})
               {
                  if (parseFile($File::Find::name))
                  {
                     push(@{$self->{FILELIST}}, $File::Find::name);
                  }
                  else
                  {
                     print "Invalid XML syntax of $File::Find::name file\n";
                  }
               } ## end if (not exists $XML_PATHS...
               else
               {
                  push(@{$self->{FILELIST}}, $File::Find::name) if ($XML_PATHS{$File::Find::name});
               }
            } ## end if ($_ eq $search_name...
         },
         $search_root
          );
   } ## end if (-e $search_root)
   else
   {
      return $FALSE;
   }

   if (scalar(@{$self->{FILELIST}}) < $xml_files_min)
   {
      return $FALSE;
   }
   return $TRUE;
};

#Description: This function parses each configure file in @filelist
#
#Input:  none
#Output: none
my $xml_parse = sub {
   local $self = shift @_;
   foreach $file (@{$self->filelist})
   {
      my $parser = XML::Parser->new(Style => 'EasyTree');
      my $tree = $parser->parsefile($file);
      $self->xmlraw($tree);  #Store fetched data
      undef $parser;         #Calling parser destructor
   } ## end foreach $file (@{$self->filelist...
   return;
};

### Public functions
#Object constructor
#Input: 1. path to XML files
#       2. XML file name
#Output: class instance
sub new
{

   #Parser initialization.
   ($search_root, $search_name, $class) = @_;

   my $self = {};

   #Class variables initialization
   $class = "ifxml" unless $class;
   $self->{FILELIST} = [];  #List of configure.xml files
   $self->{XMLRAW}   = [];  #Raw data

   bless $self, $class;     #Declaring an object

   $retval = $self->$xml_find();  #Building file list
   if ($retval eq $FALSE)
   {
      return $FALSE;
   }
   $self->$xml_parse();           #Parsing XML file

   return $self;
} ## end sub new

#Get/set @FILELIST function
sub filelist
{
   local $self = shift @_;
   @{$self->{FILELIST}} = @_ if scalar(@_);
   return $self->{FILELIST};
}

#Get/set @XMLRAW function
sub xmlraw
{
   local $self = shift @_;
   push @{$self->{XMLRAW}}, map {@$_} @_;
   return $self->{XMLRAW};
}

#####################################################################
# This method allows to get XML data by using XPath
#  INPUT: $mode   - which kind of data should be got
#         $xpath  - path to the tag (XPath) in XML file in the
#                   form: Tag1/Tag2/.../TagN
#         $attr   - attribute which value should be got if mode
#                   is 'attr'.
#         $filter - a hash with attributes which will be used for
#                   filtering
# OUTPUT: undef   - if some error happens
#         $retval - result of request
#####################################################################
sub find_data_by_xpath
{
   my $self   = shift @_;
   my $mode   = shift @_;
   my $xpath  = shift @_;
   my $attr   = shift @_;
   my $filter = shift @_;

   my ($method, $expression, $refval);

   # Replace slash in XPath notation by underscore
   if ((!defined $xpath) || ($xpath eq ''))
   {
      baselog($LOG_ERROR, "The XPath can not be empty");
      return undef;
   }

   # Check that method name does not contain unsupported characters
   if ($xpath =~ /[-_\*\.]/)
   {
      baselog($LOG_ERROR, 'Unsupported characters in XPATH: ' . $xpath);
      return undef;
   }

   if ($xpath !~ /^\/.+/)
   {
      baselog($LOG_ERROR, "The slash character ('/') must be at the beginning of XPath '$xpath'.");
      return undef;
   }

   $method = $xpath;
   $method =~ tr|/|_|;

   if (!defined $mode)
   {
      baselog($LOG_ERROR, "The mode is undefined");
      return undef;
   }

   if ($mode eq $GETCOUNT)
   {
      $method = "count" . $method;
   }
   else
   {
      $method = "get" . $method;
   }

   # Create full request
   if ($mode eq $GETTEXT)
   {
      $method .= '_text';
   }
   elsif ($mode eq $GETDATA)
   {
      if ((defined $attr) && ($attr ne ''))
      {
         $method .= "_$attr";
      }
   } ## end elsif ($mode eq $GETDATA)
   elsif ($mode ne $GETCOUNT)
   {
      baselog($LOG_ERROR, "The '$mode' mode is incorrect");
      return undef;
   }

   if (!defined $attr)
   {
      $attr = '';
   }
   if (!defined $filter)
   {
      $filter = '';
   }
   baselog($LOG_DEBUG, "find_data_by_xpath: mode='$mode', XPath='$xpath', attr='$attr', filter=" . Data::Dumper::Dumper($filter));

   # Perform request
   $expression = '$retval = $self->' . $method . '($filter);';
   eval $expression;

   baselog($LOG_DEBUG, "Expression for eval: '$expression'; retval = " . Data::Dumper::Dumper($retval));

   return $retval;
} ## end sub find_data_by_xpath

#Object destructor is used to reduce warnings output.
#This function name is defined in Perl and must not be changed.
sub DESTROY
{
   return;
}

1;
__END__

