###############################################################################
## @file   install_common.pm
# @brief   This module contains common data for application installation scripts
# @details
# @author  Ilya Teterev
# @author  Evgeny Lazarev
# @version 1.00 - Initial version
# @version 1.01 - Function "check_scripts" and variable $CONTACT_MESSAGE are
#                 added.
# @version 1.02 - Function "is_app_installed" is changed.
# @version 1.03 - Semaphores introduced: $SEMMNGR object declaration and
#                 semaphore list @SEMLIST are added.
# @version 1.04 - Function "is_app_installed" is changed. $is_su was added to
#                 handle the version check for SU patches
# @version 1.05 - new functions: isBaseApplication, getApplicationsList,
#                 getBaseApplicationsList and getNortelApplicationsList added.
# @version 1.06 - new semaphore $SEM_HARD added to the semaphores list.
#          1.06 - new function "getInstalledPackagesList" is added.
# @version 1.07  - "check_scripts" function changed to accept list of
#                  applications to check. New "check_apps_scripts" is added.
#                  Function 'uninstall_rpms' is changed. RPM packages are
#                  uninstalled in one transaction.
# @version 1.08 - added support for alphanumeric characters in rpm names
#                 added support for CF as a Media Device, for appinstall
# @version 1.09 - fixed bug with CF as media device.
# @version 1.10 - Get rid of the global variables $APP_INSTALLED and $APP_VERSION
# @version 1.11 - two new variables $APPINSTALL_OUTFILE and $APPINSTALL_ERRFILE have been added.
# @version 1.12 - Two colons were removed before shared functions and variables.
#                 Shared functions and variables were exported.
# @version 1.13 - Two new functions was added: is_rpm_database_corrupted(),
#                 update_app_installation_status(status).
# @version 1.14 - Modified to use is_cppm() function to define is CP-PM or not.
# @version 1.15 - is_rpm_database_corrupted() function updated to exclude temp files from checking
# @version 1.16 - $SEM_ACCOUNTS has been added to the @SEMLIST
# @version 1.17 - $APPSTARTWAIT_SEM_WAIT_TIME has been added
# @version 1.18 - Text banners, which should be written to motd file now added to motd over open/close
#                 file functions. New banner with guide after LinuxBase installation was added.
# @version 1.19 - isNortelApplication function added
# @version 1.20 - XML validation has been added
# @version 1.21 - New variables are defined:
# @version 1.22 - Now RPMUF error raised in case of rpm uninstallation issues
# @version 1.23 - $MEDIADEV is removed. getMediaDev() is implemented. 
#                 $XML_DUMP_DEBUG is removed, directly invocation of is_debug_mode is implemented.
# @li  $APPINSTALL_REPORTFILE - appinstall report file for Nortel Applications.
# @li $APPINSTALL_BASEAPPS_OUTFILE - appinstall stdout file for Base Applications.
# @li $APPINSTALL_BASEAPPS_ERRFILE - appinstall error output file for Base Applications.
# @li $APPINSTALL_BASEAPPS_REPORTFILE - appinstall report file for Base Applications.
package install_common;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use base_semctl;
use CommonInstallUtils;
use baselog;
use Data::Dumper;
use warnings;
use base_xml;
use base_system;
use hwRecognition;
use xmlvalid;

our $VERSION = "1.23";

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&is_app_installed &print_header &print_xml_dump &uninstall_rpms
  &configure_xml_update &check_scripts &getBaseApplicationsList &getNortelApplicationsList
  &isBaseApplication &isNortelApplication &base_app_install &getInstalledPackagesList &is_rpm_database_corrupted
  &update_app_installation_status &getMediaDev
  $APP_BIN_ROOT $SYSDB_ROOT $SYSCONFIG_ROOT $MNTPOINT $ISPDEV $LICENSAPP $INSTLOG
  $STARTLOG $IPTABLES_CFG $INIT_CFG $baseAppsmode $MONITSTARTUP_SEM_WAIT_TIME $APPSTARTWAIT_SEM_WAIT_TIME
  $MIN_CFGVER $DINFO $APPSTARTUP_LOCK_FILE $IPTABLES_USERCFG @DBSECTIONS $LASTERR
  $INSTALL_TYPE $INSTVER $SEMMNGR @SEMLIST $INSTALL_USERNAME $INSTALL_FOLDER
  $PRIORITY_DEFAULT @SCHED_POLICIES $SCHED_POLICY_DEFAULT @NICE_LIMITS @RT_LIMITS $NAIMNTPOINT
  $CONTACT_MESSAGE $APPINSTALL_OUTFILE $APPINSTALL_ERRFILE $INTERACTIVE
  $APPINSTALL_REPORTFILE $APPINSTALL_BASEAPPS_OUTFILE $APPINSTALL_BASEAPPS_ERRFILE
  $APPINSTALL_BASEAPPS_REPORTFILE
  );

##################
#     PATHS      #
##################

our $APP_BIN_ROOT   = "/opt/nortel";                                   #application binaries
our $SYSDB_ROOT     = "/var/opt/nortel/base/appconfig";                #application DB
our $SYSCONFIG_ROOT = "/var/opt/nortel/base/appconfig/appcustconfig";  #system-wide configs directory
our $MNTPOINT       = "/tmp/media";                                    #media mount point
our $ISPDEV         = "/dev/hdc";                                      #ISP1100 media device
our $LICENSAPP      = "/license/apps/";                                #application licenses directory
our $NAIMNTPOINT    = "/mnt/nai";                                      #nai media mount point

my $SCRIPT_FILES = "/opt/nortel/base/appinstall";                      #installation scripts location

# Log info
our $INSTLOG  = "appinst.log";
our $STARTLOG = "appstart.log";

# Config files
our $IPTABLES_CFG = "iptables.xml";                                    #base_firewall DB file
our $INIT_CFG     = "init.xml";                                        #appstartup DB file
our $MIN_CFGVER   = 1.1;                                               #install.xml minimum version
our $DINFO        = ".diskinfo";                                       #file with md5sums on CD

# Basefirewallconfig DB file
our $IPTABLES_USERCFG = "iptables_usercfg.xml";

# Sections that needs to be saved as DB files
our @DBSECTIONS = ("iptables", "ipSec", "init", "sysctl");

# Log files for Deploy Manager application
our $APPINSTALL_LOGDIR     = '/var/log/nortel/appinstall';
our $APPINSTALL_OUTFILE    = "$APPINSTALL_LOGDIR/appinstall_stdout.log";
our $APPINSTALL_ERRFILE    = "$APPINSTALL_LOGDIR/appinstall_stderr.log";
our $APPINSTALL_REPORTFILE = "$APPINSTALL_LOGDIR/appinstall_report";

our $APPINSTALL_BASEAPPS_OUTFILE    = "$APPINSTALL_LOGDIR/appinstall_baseapps_stdout.log";
our $APPINSTALL_BASEAPPS_ERRFILE    = "$APPINSTALL_LOGDIR/appinstall_baseapps_stderr.log";
our $APPINSTALL_BASEAPPS_REPORTFILE = "$APPINSTALL_LOGDIR/appinstall_baseapps_report";

##################
#  COMMON VARS   #
##################

our $baseAppsmode     = undef;
our $INTERACTIVE      = undef;
our $LASTERR          = "NULL";                                                                              #Latest error code (check errors.pm)
our $INSTALL_TYPE     = "install";                                                                           #Install type (install, upgrade, re-install)
our $INSTALL_USERNAME = $ROOT_ACCT;
our $INSTALL_FOLDER   = "";
our $INSTVER          = "";                                                                                  #path to selected install.xml
our $CONTACT_MESSAGE  = "Please contact technical support for help.";
our $SEMMNGR          = undef;                                                                               #semaphore manager global object.
our @SEMLIST          = ($SEM_BACKUP, $SEM_PATCH, $SEM_APPINSTALL, $SEM_APPSTART, $SEM_HARD, $SEM_ACCOUNTS);

#time of semaphore taking for monitappstartup
our $MONITSTARTUP_SEM_WAIT_TIME = 180;

#time of semaphore taking for appstartwait
our $APPSTARTWAIT_SEM_WAIT_TIME = 300;
###############################
#  Default startup parameters #
###############################
our $PRIORITY_DEFAULT = 0;
our @SCHED_POLICIES   = ('SCHED_OTHER', '-o', 'SCHED_RR', '-r', 'SCHED_FF', '-f');

our $SCHED_POLICY_DEFAULT = $SCHED_POLICIES[0];
our @NICE_LIMITS          = (-20 .. 19);
our @RT_LIMITS            = (1 .. 99);

##################
#  COMMON FCNS   #
##################

###############################################################################
## @fn is_app_installed($app_name, $app_ver, $is_su)
# @brief This sub checks if application is installed or not.
# @param  $app_name - application name
# @param  $app_ver - application version (optional)
# @param  $is_su - SU flag (optional)
# @return $TRUE - application is installed
# @return $FALSE - application is NOT installed
sub is_app_installed($;$$)
{
   my ($app_name, $app_ver, $is_su) = @_;
   my $ret = $FALSE;
   my $in_line;  # variable for input info

   if (!(defined($app_name)))
   {
      return $ret;
   }

   my $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
   if (!$parser)
   {
      baselog($LOG_ERROR, "No configuration file ($APPLCFG) found.\n");
      return $FALSE;
   }

   my $version = $parser->get_application_version({name => $app_name});
   if (scalar(@{$version}))
   {
      if (defined $app_ver)
      {
         $version = ${$version}[0];

         #It is necessary to check the first three groups of digits for SU only.
         #For other types of patches, the version of patch must be the same
         # as the version of installed application.
         if ($is_su)
         {
            baselog($LOG_DEBUG, "SU version '$version' vs application '$app_name:$app_ver'.");
            $version =~ s/^(\d+\.\d+\.\d+).*/$1/;
            $app_ver =~ s/^(\d+\.\d+\.\d+).*/$1/;
         } ## end if ($is_su)
         if ($version eq $app_ver)
         {
            $ret = $TRUE;
         }
      } ## end if (defined $app_ver)

      else
      {
         $ret = $TRUE;
      }
   } ## end if (scalar(@{$version}...

   undef $parser;
   return $ret;
} ## end sub is_app_installed($;$$)

###############################################################################
## @fn print_header($data)
# @brief It clears screen and prints data in the specified format.
# @param  $data - data to print
sub print_header($)
{
   my $data = shift(@_);
   my $old  = $~;
   $~ = "MMHEADER";
   system($CLEAR);
   print " #####################################################################\n";
   write;
   print " #####################################################################\n";
   format MMHEADER =
 #^||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||#
 $data
 #^|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||~#
 $data
 #^|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||~#
 $data
.
   $~ = $old;
} ## end sub print_header($)

###############################################################################
## @fn print_xml_dump($log_file)
# @brief This sub prints xml info to log file if debug mode turned on.
# @param  $log_file - log file to print xml dump (LOGDIR/log_file)
#         all other args  - messages to print to log file
#                 \li open log file
#                 \li print to this file
#                 \li close log file
sub print_xml_dump(@)
{
   my $log_file = shift @_;
   my (@mesg) = @_;
   if (is_debug_mode())
   {
      system("$MKDIR -p $LOGDIR");
      open DUMPLOG, ">>", $LOGDIR . "/" . $log_file
        || baselog($LOG_ERROR, "Can't open $LOGDIR/$log_file");
      print DUMPLOG $_ foreach (@mesg);
      close DUMPLOG;
   } ## end if ($XML_DUMP_DEBUG)
   else
   {
      return;
   }
} ## end sub print_xml_dump(@)

###############################################################################
## @fn uninstall_rpms(@rpmlist)
# @brief Removes previously installed RPMs.
# @param  @rpmlist - list of rpms to remove.
# @return    $TRUE if uninstallation pass
# @return    $FALSE if some error was occurred
sub uninstall_rpms(@)
{
   my @rpmlist  = @_;
   my @packages = ();

   print_xml_dump($INSTLOG, "RPMs uninstallation order:\n", Data::Dumper::Dumper(\@rpmlist));

   #Preparing RPMs to be uninstalled
   foreach my $file (@rpmlist)
   {
      unlink $RPMBACKUP . "/" . $file;
      $file =~ s/\.rpm$//g;
      my $pkg = qx/$RPMQUERY $file/;
      chomp $pkg;
      if ($pkg !~ /is not installed$/)
      {
         push @packages, $pkg;
      }
      else
      {
         baselog($LOG_WARNING, "WARNING: $pkg");
      }
   } ## end foreach my $file (@rpmlist)
   print "removing packages:\n" . join("\n", @packages) . "\n";

   #Uninstalling RPMs
   my $ret = $OK;
   if (!$INTERACTIVE)
   {
      do_system([$RPM, '-e', @packages], {$dsOUTMODE => '>>', $dsERRMODE => '>>', $dsERRFILE => $APPINSTALL_ERRFILE, $dsOUTFILE => $APPINSTALL_OUTFILE, $dsRETVAL => \$ret});
   }
   else
   {
      do_system([$RPM, '-e', @packages], {$dsSTDOUT => $TRUE, $dsRETVAL => \$ret});
   }
   if ($ret == $OK)
   {
      return $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "Cannot uninstall some RPMs.");
      $LASTERR = "RPMUF";
      return $FALSE;
   }
} ## end sub uninstall_rpms(@)

###############################################################################
## @fn configure_xml_update($app, $version)
# @brief This sub creates configure.xml file if it is missed.
# It also adds "version" attribute to application "configure.xml" files
# if it is missed.
# @param  $app - application name
# @param  $version - application version
# @return $TRUE - file was updated
# @return $FALSE - error occurred
sub configure_xml_update($$)
{
   my ($app, $version) = @_;
   my $app_cfg_dir    = "$APP_CFG_ROOT/$app";
   my $configure_file = "$app_cfg_dir/$APPLCFG";
   my $file;

   baselog($LOG_DEBUG, "app=$app\tversion=$version");

   # If file is not exist then create a dummy one
   if (!-e $configure_file)
   {
      if (system($MKDIR, "-p", $app_cfg_dir) != 0)
      {
         baselog($LOG_WARNING, "mkdir -p $app_cfg_dir returned $?");
      }
      unless (open $file, ">", $configure_file)
      {
         baselog($LOG_ERROR, "Can not create $configure_file\n");
         return $FALSE;
      }
      print $file qq{<application name="$app" version="$version"/>\n};
      close($file);
      baselog($LOG_INFO, "New file is created: $configure_file");
      system($CHOWN, "-R", "$NORTEL_ACCT:$NORTEL_ACCT", $app_cfg_dir);
      return $TRUE;
   }  # if (!-e $configure_file)
   my $rewrite = 0;  # Flag if it is necessary to rewrite file or not

   unless (open $file, "<", $configure_file)
   {
      baselog($LOG_ERROR, "Can not open $configure_file for writing\n");
      return $FALSE;
   }
   my @file_content = <$file>;
   close $file;
   foreach my $line (@file_content)
   {
      if ($line =~ /^<application/)
      {
         if (!($line =~ /\s+version=\"(\d[\d\.-]*\d)\"/ || $line =~ /\s+version=\"(\d[\d\.-]*[A-Z])\"/))
         {
            $line    = qq{<application name="$app" version="$version">\n};
            $rewrite = 1;
            baselog($LOG_WARNING, "configure.xml for $app application does not contain version attribute");
         } ## end if (!($line =~ /\s+version=\"(\d[\d\.-]*\d)\"/...
         last;
      } ## end if ($line =~ /^<application/...
   } ## end foreach my $line (@file_content...
   if ($rewrite)
   {
      my $configure_bak = $configure_file . ".bak";

      # Update configure.xml file
      unless (open $file, ">", $configure_bak)
      {
         baselog($LOG_ERROR, "Can not open $configure_bak for writing\n");
         baselog($LOG_ERROR, "configure.xml for APP $app was not updated\n");
         return $FALSE;
      } ## end unless (open $file, ">", $configure_bak...
      print $file @file_content;
      close $file;
      rename($configure_bak, $configure_file);
      baselog($LOG_INFO, "File $configure_file was updated successfully\n");
   } ## end if ($rewrite)

   #check xml file using xml scheme
   checkXmlFiles([$configure_file], $CONFIGURE_XML_SCHEME_PATH, $FALSE);

   return $TRUE;
} ## end sub configure_xml_update($$)

###############################################################################
## @fn check_app_scripts($section_name, $parser)
# @brief   This function checks presence and permissions of the scripts
#          available in the parsed document.
# @details
# @param   $section_name - script section name from configure.xml
# @param   $parser - reference to the parsed document (parser).
# @return  $TRUE  - if all scripts for specified section name exist and have
#                     right permissions
# @return  $FALSE - some error happened or one of the checks failed.
sub check_apps_scripts($$)
{
   my ($section_name, $parser) = @_;
   my $retval   = $TRUE;
   my $tag_info = undef;
   my $exp      = undef;

   $exp      = '$parser->get_' . "$section_name" . '_script()';
   $tag_info = eval $exp;

   if (!defined $tag_info || !scalar @{$tag_info})
   {
      baselog($LOG_DEBUG, "There is no $section_name section in configure.xml");
   }
   else
   {
      foreach my $script (@{$tag_info})
      {
         my $cfile = $script->{'file'};
         if (!defined $cfile)
         {
            baselog($LOG_ERROR, "Section $section_name is not specified in configure.xml file.\n", $TRUE);
            $retval = $FALSE;
         }
         else
         {

            # remove execution script with sudo
            if ($cfile =~ s/.*sudo\s+(.*)/$1/)
            {
               baselog($LOG_WARNING, "Script $cfile is executed as sudo.\n");
            }
            if (!-e $cfile)
            {
               baselog($LOG_ERROR, "Script $cfile does not exist.\n", $TRUE);
               $retval = $FALSE;
            }  ## if(! -e $cfile)
            elsif (!-x $cfile)
            {
               baselog($LOG_ERROR, "Script $cfile has wrong permissions.\n", $TRUE);
               $retval = $FALSE;
            }
         } ## end else [ if (!defined $cfile)
      } ## end foreach my $script (@{$tag_info...
   } ## end else [ if (!defined $tag_info...
   return $retval;
} ## end sub check_apps_scripts($$)

###############################################################################
## @fn check_scripts($section_name, $applist)
# @brief   This function initialize a parser with required configuration file and
#          calls check_apps_scripts() function to check available scripts in
#          defined configuration files.
# @details If application list is not defined, then parser is initialized with
#          all availbale application's configuration files.
# @param   $section_name - script section name from configure.xml
# @param   $applist - reference to list of applications to check.
# @return  $TRUE  - if all scripts for specified section name exist and have
#                     right permissions
# @return  $FALSE - some error happened or one of the checks failed.
sub check_scripts($;$)
{
   my ($section_name, $applist) = @_;
   my $retval = $TRUE;
   my $parser;

   if (!defined $applist)
   {
      $parser = ifxml::new("$APP_CFG_ROOT", "$APPLCFG");
      if (!$parser)
      {

         # at least base configure.xml file should be available.
         baselog($LOG_INFO, 'Cannot initialize parser.');
         return $FALSE;
      } ## end if (!$parser)
      $retval = check_apps_scripts($section_name, $parser);
      undef $parser;
   } ## end if (!defined $applist)
   else
   {
      foreach my $app (@$applist)
      {
         $parser = ifxml::new("$APP_CFG_ROOT/$app", "$APPLCFG");
         if (!$parser)
         {
            baselog($LOG_INFO, "Cannot initialize parser for '$app'.");
            $retval = $FALSE;
         }
         else
         {
            baselog($LOG_DEBUG, "Check '$app' scripts.");
            $retval &= check_apps_scripts($section_name, $parser);
            undef $parser;
         } ## end else [ if (!$parser)
      } ## end foreach my $app (@$applist)
   } ## end else [ if (!defined $applist)
   return $retval;
} ## end sub check_scripts($;$)

###############################################################################
## @fn getApplicationsList($xml_file, $config)
# @brief This subroutine returns list of the Applications from
#        specified configuration file and configuration name.
# @details configuration file is found in the $ARCHDIR directory.
# @param $xml_file - name of the file for parsing.
# @param $config - configuration's name
# @return reference to the list of the applications
# @author Evgeny Lazarev
# @date May 2008
# @note
sub getApplicationsList($$)
{
   my ($xml_file, $config) = (@_);
   my @applist = ();

   unless ($xml_file)
   {
      baselog($LOG_ERROR, "XML file is not specified.");
      return undef;
   }
   unless ($config)
   {
      baselog($LOG_ERROR, "Configuration name is not specified.");
      return undef;
   }

   my $parser = ifxml::new($ARCHDIR, $xml_file);
   if ($parser)
   {
      my $res = $parser->get_supportedConfig_application({'name' => $config});
      for my $app (@$res)
      {
         if (defined($app->{name}))
         {
            push @applist, $app->{name};
         }
      } ## end for my $app (@$res)
   } ## end if ($parser)
   return \@applist;
} ## end sub getApplicationsList($$)

###############################################################################
## @fn getBaseApplicationsList()
# @brief This subroutine returns list of the installed Base Applications
# @details This function parses baseapps-install.xml and returns list
#          of the Base Applications of the installed configuration.
# @return reference to a list of installed Base Applications
# @author Evgeny Lazarev
# @date May 2008
# @note
sub getBaseApplicationsList()
{
   my $params = use_confver($BASEAPPCONFVER);
   if (!defined($params))
   {
      baselog($LOG_WARNING, "Base Applications are not installed. System is not installed properly.");
      return [];
   }
   my $applist = getApplicationsList($BASEAPPINSTCFG, $params->{'APP_INSTALLED'});
   unless (defined $applist)
   {
      $applist = [];
   }
   return $applist;
} ## end sub getBaseApplicationsList()

###############################################################################
## @fn getNortelApplicationsList()
# @brief This subroutine returns list of the installed Nortel Applications
# @details This function parses install.xml and returns list
#          of the Nortel Applications of the installed configuration.
# @return reference to a list of installed Nortel Applications
# @author Evgeny Lazarev
# @date May 2008
sub getNortelApplicationsList()
{
   my $params = use_confver($CONFVER);
   if (!defined($params))
   {
      baselog($LOG_INFO, "Nortel Applications are not installed.");
      return [];
   }
   my $applist = getApplicationsList($INSTCFG, $params->{'APP_INSTALLED'});
   unless (defined $applist)
   {
      $applist = [];
   }
   return $applist;
} ## end sub getNortelApplicationsList()

###############################################################################
## @fn getMediaDev()
# @brief This subroutine returns mount point related to the HW type
# @return $ret - default media mount point
# @author Ilya Kudelin
# @date May 2008
sub getMediaDev()
{
   my $ret = (is_cppm()) ? "/dev/hdc1" :  "/dev/cdrom";
   return $ret;
}
###############################################################################
## @fn isBaseApplication($app_name)
# @brief This subroutine checks if specified application is Base Application
# @details If the specified application is in baseapps-install.xml then it is
#          considered to be Base Application.
# @param $app_name - name of the application for check
# @return $TRUE - if specified application is the Base Application
# @return $FALSE - if specified application is not the Base Application
# @author Evgeny Lazarev
# @date May 2008
# @note
sub isBaseApplication($)
{
   my $app_name = shift(@_);
   if (!defined($app_name))
   {
      baselog($LOG_WARNING, "Applications name is not specified.");
      return $FALSE;
   }

   my $base_applications = getBaseApplicationsList();
   if (scalar(@$base_applications))
   {
      for my $app (@$base_applications)
      {
         if ($app_name eq $app)
         {
            return $TRUE;
         }
      } ## end for my $app (@$base_applications...
   } ## end if (scalar(@$base_applications...
   return $FALSE;

} ## end sub isBaseApplication($)

###############################################################################
## @fn isNortelApplication($app_name)
# @brief This subroutine checks if specified application is a Nortel Application
# @details If the specified application is in the install.xml then it is
#          considered to be a Nortel Application.
# @param $app_name - name of the application to check for
# @return $TRUE - if specified application is a Nortel Application
# @return $FALSE - if specified application is not a Nortel Application
# @date Jan 2009
# @note
sub isNortelApplication($)
{
   my $app_name = shift(@_);
   if (!defined($app_name))
   {
      baselog($LOG_WARNING, "Applications name is not specified.");
      return $FALSE;
   }

   my $applications = getNortelApplicationsList();
   if (scalar(@$applications))
   {
      for my $app (@$applications)
      {
         if ($app_name eq $app)
         {
            return $TRUE;
         }
      } ## end for my $app (@$applications...
   } ## end if (scalar(@$applications...
   return $FALSE;

} ## end sub isNortelApplication($)

###############################################################################
## @fn getInstalledPackagesList()
# @brief This subroutine returns list of the installed Packages
# @details This function parses install.xml and returns list
#          of the installed packages.
# @return list of installed Packages
# @author Ekaterina Bazhanova
sub getInstalledPackagesList()
{
   my @packages = ();
   unless (-e $ARCHVER)
   {
      baselog($LOG_WARNING, "Cannot find $ARCHVER.");
      return [];
   }
   my $parser = base_xml->new($ARCHVER);
   unless ($parser)
   {
      baselog($LOG_ERROR, "Cannot create base_xml object.");
      return undef;
   }
   my %hash = ('deployed' => $TRUE);
   my $packagenodes = $parser->findElements('/install/package', \%hash);
   unless ($packagenodes)
   {
      baselog($LOG_ERROR, "Cannot find any installed packages in $ARCHVER.");
      return undef;
   }
   foreach my $package (@$packagenodes)
   {
      push @packages, $package->getAttribute('name');
   }
   return \@packages;
} ## end sub getInstalledPackagesList()

###############################################################################
## @fn base_app_install()
# @brief This function installs Base Applications on the first bootup
# @details Checks for existence $BASEFIRSTBOOTUP. If it is exist,
#          then start appinstall with specified parameter -base-app and in
#          case successfully completion remove it.
# @return $TRUE    - Applications installed successfully
# @return $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date April 2008
sub base_app_install()
{
   my $retval = $TRUE;

   # Open motd file to write
   my $fd;
   if (not open($fd, '>', $MOTDFILE))
   {
      baselog($LOG_ERROR, "Can't open file: '$MOTDFILE'");
   }
   else
   {

      # Write to motd file
      print $fd "LinuxBase Applications are installing now.\n";

      # Close motd file
      if (not close $fd)
      {
         baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
      }
   } ## end else [ if (not open($fd, '>',...

   if (do_system([$APPINSTALL, '-base-app']))
   {
      do_system([$WALL, 'LinuxBase Applications successfully installed.']);

      # Delete text banner from motd file
      delete_line_from_file($MOTDFILE, "LinuxBase Applications are installing now");

      # Add new text banner into motd file
      # Open motd file to append
      if (not open($fd, '>>', $MOTDFILE))
      {
         baselog($LOG_ERROR, "Can't open file: '$MOTDFILE'");
      }
      else
      {

         # Write to motd file
         print $fd "$FIRST_LOGIN_GUIDE\n";

         # Close motd file
         if (not close $fd)
         {
            baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         }
      } ## end else [ if (not open($fd, '>>'...
      baselog($LOG_INFO, "LinuxBase Applications successfully installed.");
   } ## end if (do_system([$APPINSTALL...
   else
   {
      baselog($LOG_ERROR, "Installation of the LinuxBase Applications failed.");
      do_system([$WALL, 'Installation of the LinuxBase Applications failed.']);

      # Delete text banner from motd file
      delete_line_from_file($MOTDFILE, "LinuxBase Applications are installing now");

      # Add new text banner into motd file
      # Open motd file to append
      if (not open($fd, '>>', $MOTDFILE))
      {
         baselog($LOG_ERROR, "Can't open file: '$MOTDFILE'");
      }
      else
      {

         # Write to motd file
         print $fd "Installation of the LinuxBase Applications failed.\n";

         # Close motd file
         if (not close $fd)
         {
            baselog($LOG_ERROR, "Cannot close the file descriptor: $!.");
         }
      } ## end else [ if (not open($fd, '>>'...
      $retval = $FALSE;
   } ## end else [ if (do_system([$APPINSTALL...
   return $retval;
} ## end sub base_app_install()

###############################################################################
## @fn is_rpm_database_corrupted()
# @brief This function check system rpm database
# @details  Use /usr/lib/rpm/rpmdb_verify utility.
# @return $TRUE    - Database is corrupted.System must be reinstalled!
# @return $FALSE - Database is valid.
sub is_rpm_database_corrupted()
{
   my $corrupt  = $FALSE;
   my @rpm_base = ();

   if (not do_system([$FIND, $RPMDATABASEDIR, '-type', 'f'], {$dsOUT => \@rpm_base}))
   {
      baselog($LOG_ERROR, "Cannot find files under '$RPMDATABASEDIR' directory.");
      return $TRUE;
   }

   chomp(@rpm_base);

   foreach my $file (@rpm_base)
   {
      my $ret;
      my @output = ();
      my @out    = ();

      do_system([$FILE, $file], {$dsOUT => \@out});
      if ((defined $out[0]) and ($out[0] =~ / Berkeley DB/))
      {
         baselog($LOG_DEBUG, "Checking Berkeley DB: '$file.'");
         do_system([$RPMDB_VERIFY, $file], {$dsRETVAL => \$ret, $dsOUT => \@output});
         if ($ret)
         {
            $corrupt = $TRUE;
            baselog($LOG_INFO,    "'$ret' was returned while checking $file. output: '@output'.");
            baselog($LOG_WARNING, "RPM database is corrupted. Try to recover...");

            @rpm_base = grep {m/__db/} @rpm_base;
            if (scalar(@rpm_base))
            {
               if (not do_system([$RM, "-f", @rpm_base]))
               {
                  baselog($LOG_ERROR, 'Cannot remove __db files.');
               }
            } ## end if (scalar(@rpm_base))

            do_system([$RPM, "--rebuilddb"], {$dsRETVAL => \$ret});
            unless ($ret)
            {
               $corrupt = $FALSE;
               baselog($LOG_INFO, "RPM database recovered.");
            }
            else
            {
               write_file($MOTDFILE, "RPM database is corrupted.\nSystem must be reinstalled.");
            }

            last;
         } ## end if ($ret)
      } ## end if ((defined $out[0]) ...
   } ## end foreach my $file (@rpm_base...
   return $corrupt;
} ## end sub is_rpm_database_corrupted()

###############################################################################
## @fn update_app_installation_status($status)
# @brief Update status of application installation process.
# @details Update STATUS field in CONFVER file.
# @param $file            - status
# @return $TRUE  - success.
# @return $FALSE - some error happens.
sub update_app_installation_status($)
{
   my $status = shift @_;
   unless ((grep {m/$status/} (values %CONFVER_STATES)) and (defined $status))
   {
      baselog($LOG_ERROR, "Unsupported application state '$status' is provided. See the CONFVER_STATES hash.");
      return $FALSE;
   }

   my $confver_content = use_confver($CONFVER);
   if (is_hash($confver_content))
   {
      $confver_content->{'STATUS'} = $status;
      return write_file($CONFVER, $confver_content, $TRUE);
   }
   return $FALSE;
} ## end sub update_app_installation_status($)

1;
__END__

=head1 NAME

install_common - installation common variables and functions.

=head1 SYNOPSIS

use install_common;

=head1 LIMITATIONS

=head1 SEE ALSO

