## @file    jarwarlib.pm
# @brief
# @details  This file contains common declarations for
#           jar and war patch manipulation scripts
#
#           Before inclusion of this lib the
#           patches_target.pm has to be included
# @author
# @version  1.00 - Initial version
# @version  1.01 - To export shared functions and variables.
# @version  1.02 - Remove 'Base_Patch' subsystem setting, add warnings & strict.
# @version  1.03 - Rework cleanup function. add patchutil::patches_public::create_patch_dir()
#                  function into set_variables.
# @version  1.04 - Using get_basename() instead of `basename`
# @version  1.05 - get_files_to_array() added

package jarwarlib;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use baselog;
use base_system;
use common_variables;
use common_functions;
use patchutil::patches_public;
use patchutil::patches_target;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&set_variables &get_files &is_conflict &cleanup $my_patch_dir
  $my_tmp_dir $pinfo_file $mod_file $add_file $files_to_extract $patch_name
  $jar $del_file &get_files_to_array
  );

our $patch_name       = undef;
our $my_patch_dir     = undef;
our $my_tmp_dir       = undef;
our $pinfo_file       = undef;
our $mod_file         = undef;
our $add_file         = undef;
our $files_to_extract = undef;
our $jar              = undef;
our $del_file         = undef;

## @fn sub cleanup()
# @brief Cleanup procedure
# @details Deletes the tmp directory $my_tmp_dir if defined
# @param none
# @return $TRUE - always
# @author
# @date
# @note
sub cleanup()
{

   # Remove the temp directory
   unless (remove_patch_dir())
   {
      baselog($LOG_ERROR, "Temporary directory was not cleanuped.");
   }
   return $TRUE;
} ## end sub cleanup()

## @fn sub get_jar_path($jar)
# @brief Defines path to the latest jdk utility
# @details Returns path to "jar" utility depending on latest "jdk" rpm installed
#    Note that path to jar file is returned through the first function argument
# @param $jar - the variable to return the latest jdk utility
# @return $TRUE  - success
# @return $FALSE - no "jdk" rpm installed
# @author
# @date
# @note
sub get_jar_path($)
{
   my ($jdk, $jdk_version);
   my @jdk_list;
   my $java_dir = "/usr/java";

   chomp($jdk = `$ECHO $java_dir/jdk*`);

   # Some checking
   if ($jdk eq "$java_dir/jdk*")
   {
      baselog($LOG_ERROR, "Cannot find jdk package installed", $TRUE);
      return $FALSE;
   }

   @jdk_list = split(" ", $jdk);

   # Find the latest jdk version
   my ($latest_jdk, $latest_jdk_version);

   foreach $jdk (@jdk_list)
   {
      $jdk =~ /jdk(.*)/;
      $jdk_version = $1;
      unless (defined($latest_jdk))
      {
         $latest_jdk         = $jdk;
         $latest_jdk_version = $jdk_version;
      }
      else
      {
         if ($jdk_version ge $latest_jdk_version)
         {
            $latest_jdk         = $jdk;
            $latest_jdk_version = $jdk_version;
         }
      } ## end else

   } ## end foreach $jdk (@jdk_list)

   $_[0] = $latest_jdk . "/bin/jar";
   baselog($LOG_DEBUG, "jar_path=$_[0]");

   return $TRUE;
} ## end sub get_jar_path($)

## @fn sub set_variables($patch, $ext)
# @brief Sets common variables
# @details
# @param $patch - the patch name
# @param $ext   - the patch type (could be jar/war)
# @return $TRUE  - success
# @return $FALSE - error
# @author
# @date
# @note
sub set_variables($$)
{
   my ($patch, $ext) = @_;

   # system utilities
   unless (get_jar_path($jar))
   {
      return $FALSE;
   }

   unless (create_patch_dir())
   {
      baselog($LOG_ERROR, "Cannot create temporary directory.");
      return $FALSE;
   }

   # Variable definition
   $patch_name       = get_basename($patch);
   $my_patch_dir     = "$PATCHDIR/$ext/";
   $my_tmp_dir       = "$TMP_DIR$ext/$patch_name/";
   $pinfo_file       = "META-INF/patch-info";
   $mod_file         = "META-INF/modified-files";
   $add_file         = "META-INF/added-files";
   $del_file         = "META-INF/deleted-files";
   $files_to_extract = "$mod_file $add_file";
   return $TRUE;
} ## end sub set_variables($$)

## @fn sub get_files($file, $filelist)
# @brief Returns the file list, read from the configuration file
# @details Returns the file list from modified-files, deleted-files and
#    added-files sections of the configuration file
# @param $file     - the configuration file to read from
# @param $filelist - the file list (could be empty)
# @return $TRUE  - success
# @return $FALSE - error
# @author
# @date
# @note
sub get_files($$)
{
   my $file     = $_[0];
   my $filelist = "";
   $_[1] = "";

   unless (open(File, $file))
   {
      baselog($LOG_ERROR, "Cannot open the '$file' file");
      return $FALSE;
   }

   while (my $line = <File>)
   {
      chomp($line);
      if ($line =~ /^\[.*\]$/)
      {
         $line =~ s/(\[|\])//g;
         $filelist .= " $line";
      }
   } ## end while (my $line = <File>)
   close(File);

   $_[1] = $filelist;
   return $TRUE;
} ## end sub get_files($$)

## @fn get_files_to_array($file)
# @brief Returns a reference to a list of files, read from the configuration file
# @details Returns the file list from modified-files, deleted-files and
#    added-files sections of the configuration file
# @param $file     - the configuration file to read from
# @return reference to a list of files
# @return undef if some error happened
sub get_files_to_array($)
{
   my $file = shift @_;

   my $array = [];

   unless (open(File, $file))
   {
      baselog($LOG_ERROR, "Cannot open the '$file' file");
      return undef;
   }

   while (my $line = <File>)
   {
      chomp($line);
      # To find lines in the follwoing form
      #[some/path/to/file]
      # and extract the path to file from them (remove '[' and ']')
      if ($line =~ /^\[.*\]$/)
      {
         $line =~ s/(\[|\])//g;
         push @{$array}, $line;
      }
   } ## end while (my $line = <File>)
   close(File);

   return $array;
} ## end sub get_files_to_array($)

## @fn sub is_conflict($file, $filelist)
# @brief Checks if there are conflicts
# @details
# @param $file     - file where to search patched files
# @param $filelist - filelist to compare
# @return $TRUE  - there is a conflict
# @return $FALSE - no conflicts
# @author Vladimir Ivanov
# @date Sept 2008
# @note Remember, $TRUE means there is a conflict
sub is_conflict($$)
{
   my $file     = shift @_;
   my $filelist = shift @_;

   baselog($LOG_DEBUG, "Params: file='$file', filelist='$filelist'");

   unless (open(File, $file))
   {
      baselog($LOG_ERROR, "Cannot open file '$file'");
      return $TRUE;
   }

   while (my $line = <File>)
   {
      chomp($line);
      if ($line =~ /^\[.*\]$/)
      {
         $line =~ s/(\[|\])//g;

         #Check for conflict
         if ($filelist =~ /\b$line\b/)
         {
            baselog($LOG_ERROR, "Conflict detected. The common patched element is: $line");
            close(File);
            return $TRUE;
         } ## end if ($filelist =~ /\b$line\b/...
      } ## end if ($line =~ /^\[.*\]$/...
   } ## end while (my $line = <File>)
   close(File);
   return $FALSE;
} ## end sub is_conflict($$)

1;

# This was added for the by-passing of "rpm -i"
# dependency check on target-machine
