# ############################################################
# @file    master_firewall.pm
# @details This module is the LinuxBase interface for checking master firewall
#          rules. It reads data from the database (file master_firewall_control.xml)
# @author  Tim Costa
# @author  Kushal Agarwal
# @date    December 2008
# @version 1.00 - Initial version
# ############################################################

package master_firewall;

use lib "/opt/nortel/base/lib";

### Access management
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw($ACCEPT $REJECT);

our $ACCEPT = "accept";
our $REJECT = "reject";

use common_variables;
use common_functions;
use install_common;
use base_system;
use baselog;
use warnings;
use strict;

sub new();
sub service_check($$);
sub service_find($$);
sub read_iptables_cfg();
sub read_master_cfg();
sub save_master_cfg($);

#####################################################################
## @fn new()
# @brief   This subroutine is object constructor. Object is an array of hashes
# @return  $self - reference to object
# @return  undef - indicates an error condition.
sub new()
{

   #initialization
   my $class = shift;
   my $self  = [];    #Class instance

   #Class variables initialization
   $class = "master_firewall" unless $class;

   $self = read_master_cfg();
   unless ($self)
   {
      baselog($LOG_ERROR, "Could not get the IPTable rules from the $MASTERCFG file!\n", $TRUE);
      return undef;
   }

   bless $self, $class;  #Declaring an object

   return $self;
} ## end sub new()

## @fn service_check($$)
# @brief   This subroutine checks to see if a specified service is "accept" them
# @param   $_[0]  - service to be searched for
# @return  $TRUE - service is "accept"
# @return  $FALSE - otherwise
sub service_check($$)
{
   my ($rules, $service) = @_;
   my $position = undef;

   unless (defined($rules) and scalar(@$rules) != 0)
   {
      baselog($LOG_ERROR, "Cannot get master file rules");
      return $FALSE;
   }

   $position = service_find($rules, $service);

   unless (defined $position)
   {
      baselog($LOG_ERROR, "Service does not exist");
      return $FALSE;
   }

   if (defined(${@{$rules}}[$position]{'action'}) and (${@{$rules}}[$position]{'action'} eq $ACCEPT))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub service_check($$)

## @fn service_find($$)
# @brief   This subroutine returns the location of a specified service in the masterfile
# @param   $_[0]  - service to be searched for
# @return  $position - position of service
# @return  undef - if service can't be found
sub service_find($$)
{
   my ($rules, $service) = @_;
   my $position = 0;

   unless (defined($rules) and scalar(@$rules) != 0)
   {
      baselog($LOG_ERROR, "Cannot get master file rules");
      return undef;
   }

   foreach my $rule (@{$rules})
   {
      if (defined($rule->{'service'}) and ($rule->{'service'} eq $service))
      {
         return $position;
      }
      $position++;
   } ## end foreach my $rule (@{$rules}...
   return undef;
} ## end sub service_find($$)

## @fn sub read_iptables_cfg()
# @brief   Read the iptables configuration file
# @details Parse the iptables.xml configuration file, and store all iptables rules
#          into the variable $iptables_rules
# @param
#          NONE
# @return
#          $TRUE  - no errors during reading configuration data
#          $FALSE - an error occured
#
# @author  Tim Costa
# @date    August 2008
sub read_iptables_cfg()
{
   my $iptables_rules = undef;

   if (-e "$SYSDB_ROOT/$IPTABLES_CFG")
   {
      $iptables_rules = base_firewall::get_iptables($SYSDB_ROOT, $IPTABLES_CFG);
      unless ($iptables_rules)
      {
         baselog($LOG_ERROR, "Could not read the $IPTABLES_CFG file.\n");
         return undef;
      }
   } ## end if (-e "$SYSDB_ROOT/$IPTABLES_CFG"...
   else
   {
      baselog($LOG_ERROR, "File $IPTABLES_CFG does not exist.");
      return undef;
   }
   return $iptables_rules;
} ## end sub read_iptables_cfg()

## @fn sub read_master_cfg()
# @brief   Read the masterfirewall configuration file
# @details Parse the master_firewall_control.xml configuration file, and store all iptables
#          rules into the $master_rules variable
# @param
#          NONE
# @return
#          $TRUE  - no errors during reading configuration data
#          $FALSE - an error occured
#
# @author  Tim Costa
# @date    August 2008
sub read_master_cfg()
{
   my $rules = undef;
   if (-e "$MASTER_CFG_ROOT/$MASTERCFG")
   {
      $rules = base_firewall::get_iptables($MASTER_CFG_ROOT, $MASTERCFG);
      unless ($rules)
      {
         baselog($LOG_ERROR, "Could not get the IPTable rules from the $MASTERCFG file!\n", $TRUE);
         return undef;
      }
   } ## end if (-e "$MASTER_CFG_ROOT/$MASTERCFG"...
   else
   {
      baselog($LOG_ERROR, "$MASTERCFG file does not exist!\n", $TRUE);
      return undef;
   }

   return $rules;
} ## end sub read_master_cfg()

## @fn sub save_master_cfg($)
# @brief   Save any changes to the masterfirewall configuration file
# @details Rewrite all $master_rules to the master_firewall_control.xml file
# @param   master_rules - the list of firewall rules to save
# @return  $TRUE  - no errors during saving configuration data
# @return  $FALSE - an error occured
# @author  Tim Costa
# @date    August 2008
sub save_master_cfg($)
{
   my $master_rules = shift @_;

   my $retval = base_firewall::save_iptables($master_rules, $MASTER_CFG_ROOT, $MASTERCFG);
   unless ($retval)
   {
      baselog($LOG_ERROR, "Could not save the $MASTERCFG file.");
      return $FALSE;
   }
   return $TRUE;
} ## end sub save_master_cfg($)

