## @file patch.pm
# @brief this class is representative of the one single patch
# @details none
# @author Alex Arkhipov
# @author Evgeny Lazarev
# @author Vladimir Ivanov
# @date May 2008
# @version 1.00 - Initial version
# @version 1.10 - new class variable 'backupdir' is added. Support patchind of
#               the Base Applications is added.
# @version 1.20 - Those methods are added due to the pstat and plis commands reworking:
#          pstat, pstat_l and plis (to show statistics);
#          get_elements_str, get_specins_str, get_created_time_str, get_pins_data_day_str,
#          get_pins_data_str, get_status_text_str, get_status_date_str, get_date_plis_str,
#          get_mark_in_service_str, is_in_service, is_patch, is_update.
# @version 1.21 - Patch file MD5 checksum check added (db integrity check).
# @version 1.22 - Two colons were removed before shared functions and variables.
# @version 1.27 - check_md5sum is modified to check md5 sums for each member in the patch
#                     - %patchinfo and %membersinfo hashes have been removed
#                     - is_rpm_patch, get_patchmembers_info, check_patch those methods are removed
#                     - method get_patchheader_info now working with XML
# @version 1.28 - plis and pstat_l changed to new format
# @version 1.29 - get_patchheader_info and isPATCHNAMEvalid methods are modified.
# @version 1.30 - issp method added
# @version 1.31 - Move get_full_patch_path to patch.pm from patches_target.pm. Modify new_patch_from_file to set PATCHPATH and modify call to get_full_patch_file to use patch object
# @version 1.32 - parameters checking added in initialize() function
# @version 1.33 - bugfix and change to handling of temp directories
# @version 1.34 - fix to Q01968202
# @version 1.36 - Fix for Q01964287 - A FRU patch can be put in service that creates a new file.
#               - method check_patch_files added
# @version 1.37 - Fix for Q01966780 - Patch is getting loaded if the name or extension is changed.
# @version 1.39 - get_patchheader_info function modified
#		- define_app_level removed
# @version 1.40 - Fix for Q01976406 - Incorrect Error message from Base API
#               - check_md5sum() - get_patch_members_extracted() return value validation
# @version 1.41 - Fix for Q01981522 - Can pins a SU with the same rpm version as is already installed
# @version 1.42 - Fix for Q01992350
# @version 1.43 - Add MIN_RPM_VER and MUTEX fields.
# @version 1.44 - Fix for Q01998722 -  Add "noauto" for REMOVABLE validation.
# @version 1.45 - Fix for Q01999918 -  Validate piControl dir for pi-control patches.
# @version 1.46 - Fox for Q02007605 - Allow spload to continue if a patch failed to be loaded.
# @version 1.47 - Fix for Q01982785 -  Support OS and deployment fru patches.
# @version 1.48 - Fix for Q02007834 - more appropriate pstat_l output
# @version 1.49 - Fix for - plis() started/stopped list display reversed
# @version 1.50 - Fix for Q02027226 - CS RPM/FW Distributions failing Linux FRU Patching
# @version 1.51 - Fix for Q02031389 - Ploading a Patch fails if SU is in a loaded state
# @version 1.52 - check_patch_files() modified to ensure that the rpm file list is
#                 defined prior to checking that the patched files belong to the rpm
#                 (will allow patching framework to introduce new files)
# @note

package patchutil::patch;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;

use install_common;
use patchutil::header_xml;
use patchutil::patches_public;
use patchutil::patches_target;
use File::Basename;

### Access management
use Exporter;
our @ISA    = qw(Exporter);  # : public Exporter;
our @EXPORT = qw($VERSION);

use File::Path;
use Term::ANSIColor;
use English;

use basedb;
use baselog;
use patchutil::patch_db;
use base_system;

our $VERSION = "1.52";

################################
# Local lineiables definitions
################################
my $class = "patchutil::patch";

my %install = (
               'RPM' => "$PERL /opt/nortel/base/patchutil/vulnerability/patch_rpm -in %s",
               'JAR' => "$PERL /opt/nortel/base/patchutil/jar/patch_jar -in %s",
               'WAR' => add_sudo_cmd() . "/opt/nortel/base/patchutil/war/patch_war -in %s",
               'BIN' => "$PERL /opt/nortel/base/patchutil/binary/binpatchutil -in %s",
               'FRU' => add_sudo_cmd() . "/opt/nortel/base/patchutil/fru/patch_fru -in %s"
              );
my %uninstall = (
                 'RPM' => "$PERL /opt/nortel/base/patchutil/vulnerability/patch_rpm -out  %s",
                 'JAR' => "$PERL /opt/nortel/base/patchutil/jar/patch_jar -out %s",
                 'WAR' => add_sudo_cmd() . "/opt/nortel/base/patchutil/war/patch_war -out %s",
                 'BIN' => "$PERL /opt/nortel/base/patchutil/binary/binpatchutil -out %s",
                 'FRU' => add_sudo_cmd() . "/opt/nortel/base/patchutil/fru/patch_fru -out %s"
                );

my $restartapp = 'You must restart target application.';

################################
# Public methods
################################
sub is_kernel_patch($);
sub is_service_update($);
sub kernel_update_prep($);
sub check_md5sum($);
sub is_broken($);
sub get_patch_status();
sub check_patch_files($$$);
sub is_PIcontrol_patch($);
sub is_OS_patch($);
sub is_deployment_patch($);
################################
# End of public methods
################################

################################
# Private methods
################################
sub new_patch_from_file($$);
sub initialize($$);
sub isPATCHNAMEvalid($);
sub get_patchheader_info($);
sub get_full_patch_file($);
################################
# End of private methods
################################

## @method new($par)
# @brief   Patch object constructor.
# @details None
# @param   $par - reference to the patch. If 'file' parameter is specified then
#          object is created based on the specified patch file, otherwise create
#          patch object based on the information provided.
# @return  $self - reference to object
# @return  undef - indicates an error condition.
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @date    May 2008
# @note    None.
sub new($)
{
   my $par  = shift(@_);
   my $self = {};
   bless $self,
     $class;

   recover_env_path();

   $self->{'is_kernel'} = $FALSE;
   $self->{'backupdir'} = $RPMBACKUP;
   $self->{'is_broken'} = $FALSE;
   if (defined($par->{'file'}))
   {
      baselog($LOG_DEBUG, "Creating patch object from file: $par->{'file'}");

      # create object based on file
      if (!($self->new_patch_from_file($par->{'file'})))
      {
         $self = undef;
      }
   } ## end if (defined($par->{'file'...
   else
   {

      # create patch object based on provided information
      if (!($self->initialize($par)))
      {
         $self = undef;
      }
   } ## end else [ if (defined($par->{'file'...

   if (defined($self))
   {
      unless ($self->{'is_broken'})
      {

         #check patch md5 check sum
         unless ($self->check_md5sum())
         {
            $self->{'is_broken'} = $TRUE;
            baselog($LOG_ERROR, "Patch object creation checksum failure");
         }
         else
         {
            my @members = @{$self->{'MEM_INFO'}};
            foreach my $name (@members)
            {
               if ($name =~ /(.+)-(.+)-(.+)\.(.+)\.rpm$/)
               {
                  $self->{'is_kernel'} = $TRUE if $name =~ /^kernel(-smp)?$/;
               }
            } ## end foreach my $name (@members)
            baselog($LOG_DEBUG, "'$class' object created");
         } ## end else
      } ## end unless ($self->{'is_broken'...
      else
      {
         baselog($LOG_ERROR, "Patch object created. Patch file is corrupted.");
      }
   } ## end if (defined($self))
   return $self;
} ## end sub new($)

#######################################
# Delete object
# Input: None
# Output: None
#######################################
sub DESTROY
{
   my $self = shift(@_);
   baselog($LOG_DEBUG, "'$class' object destroyed");
}

## @method new_patch_from_file($self, $file)
# @brief   Create new object based on patch file
# @details None
# @param   $self - reference to object
# @param   $file - file name
# @return  $TRUE - success
# @return  $FALSE - error reading patch
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @author  Vladimir Ivanov
# @date    Aug 2008
# @note    None.
sub new_patch_from_file($$)
{
   my ($self, $file) = @_;
   my $par;
   if (!-e $file)
   {
      baselog($LOG_ERROR, "File '$file' does not exist");
      return $FALSE;
   }
   $par = get_patchheader_info($file);
   return $FALSE unless defined($par);

   # define some patching parameters
   $par->{'STATUS'}    = $OUT_SERV;
   $par->{'PLOAD'}     = '';
   $par->{'PINS'}      = '';
   $par->{'POOS'}      = '';
   $par->{'backupdir'} = $RPMBACKUP;

   # patch ID will be assigned later
   $self->{'PID'} = undef;

   # initialize
   unless ($self->initialize($par))
   {
      baselog($LOG_ERROR, "Cannot initialize patching parameters");
      return $FALSE;
   }
   return $TRUE;
} ## end sub new_patch_from_file($$)

## @method initialize($self, $par)
# @brief   Create new object based on info provided.
# @details All necessary fields must be provided.
# @param   $self - reference to object
# @param   $par - reference to hash with info
# @return  $TRUE - success
# @return  $FALSE - not enough info provided
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @date    May 2008
# @note    None.
sub initialize($$)
{
   my ($self, $par) = @_;
   $par->{'backupdir'} = $RPMBACKUP;

   # patch object can also be created from hash of parameters
   # so need to check if all necessary fileds were passed
   foreach my $key (keys(%PATCH_FIELDS))
   {
      unless (exists $par->{$key})
      {
         baselog($LOG_ERROR, "Field $key doesn't exist in provided parameters. Can't create patch object.", $TRUE);
         return $FALSE;
      }
   } ## end foreach my $key (keys(%PATCH_FIELDS...
   foreach my $key (keys(%{$par}))
   {
      $self->{$key} = $par->{$key};
   }

   $self->{'members_files'} = undef;  # Extracted patch members
   $self->{'extracted'}     = $FALSE;
   return $TRUE;
} ## end sub initialize($$)

##############################################
# Create hash with patch header information
# Input: $_[0] - patch file
# Output: $par - reference to the hash of patch info
#         undef - error reading file
##############################################
sub get_patchheader_info($)
{
   my $file = shift(@_);

   # check if patch file is valid
   if (!do_system([$TAR, '-tf', $file, $PATCHINFO_FILE]))
   {
      baselog($LOG_ERROR, "$file is not a Linux patch or SU.\n", $TRUE);
      if (do_system([$UNZIP, '-t', $file, $PATCHINFO_FILE]))
      {
         baselog($LOG_ERROR, "It appears to be a service pack bundle. Move the file to $SRV_PACK_DIR and use spload.", $TRUE);
      }
      return undef;
   } ## end if (!do_system([$TAR, ...

   # read patch information from file
   baselog($LOG_DEBUG, "Trying to get patch header info");
   system("$TAR xzf $file --to-stdout $PATCHINFO_FILE 2>/dev/null > $TMP_DIR/$PATCHINFO_FILE");
   if ($?)
   {
      baselog($LOG_ERROR, "Cannot extract $PATCHINFO_FILE from $file.", $TRUE);
      return undef;
   }

   my $header = patchutil::header_xml::new("$TMP_DIR/$PATCHINFO_FILE");
   unless (defined $header)
   {
      baselog($LOG_ERROR, "Cannot parse $PATCHINFO_FILE");
      return undef;
   }
   if (($header->{'type'} ne "PATCH") and ($header->{'type'} ne "SU"))
   {
      baselog($LOG_ERROR, "Patch object can't be created on $header->{type}.");
      return undef;
   }

   # need to get patchname and patchpath
   my $path      = dirname($file);
   my $filename  = basename($file);
   my $patchname = $header->getHeaderValue($HEADER_FIELDS{'PATCH_NAME'});
   if (not defined $patchname)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'PATCH_NAME' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }

   ## get patchname from filename removing $PATCHEXT
   if ($filename =~ /^(.*?)\.$PATCHEXT$/)
   {
      my $pname = $1;
      if ($pname ne $patchname)
      {
         baselog($LOG_ERROR, "Invalid file name. File name $filename doesn't match patch name $patchname inside $PATCHINFO_FILE.\n", $TRUE);
         return undef;
      }
   } ## end if ($filename =~ /^(.*?)\.$PATCHEXT$/...
   else
   {
      baselog($LOG_ERROR, "Invalid patch extension. The patch extension must be .$PATCHEXT.\n", $TRUE);
      return undef;
   }

   # prepare hash $par with info
   my %patch_info = ();

   # 'TYPE' field is already verified.
   $patch_info{'TYPE'} = $header->{'type'};
   my $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'GENERIC'});
   if (not defined $tmp_value)
   {

      # 'GENERIC' field is not used. But it can mean that something wrong with header.xml
      baselog($LOG_ERROR, "Cannot retrieve 'GENERIC' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   } ## end if (not defined $tmp_value...
   $patch_info{'GENERIC'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'MACHINE'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'MACHINE' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'MACHINE'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'REMOVABLE'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'REMOVABLE' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($tmp_value !~ /^yes|no|noauto$/i)
   {
      baselog($LOG_ERROR, "'REMOVABLE' value of $PATCHINFO_FILE can be only 'yes', 'no' or 'noauto'. Retrieved value is: $tmp_value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'REMOVABLE'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'PRE_SCRIPT'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'PRE_SCRIPT' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($tmp_value !~ /^yes|no$/i)
   {
      baselog($LOG_ERROR, "'PRE_SCRIPT' value of $PATCHINFO_FILE can be only 'yes' or 'no'. Retrieved value is: $tmp_value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'PRE_SCRIPT'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'POST_SCRIPT'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'POST_SCRIPT' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($tmp_value !~ /^yes|no$/i)
   {
      baselog($LOG_ERROR, "'POST_SCRIPT' value of $PATCHINFO_FILE can be only 'yes' or 'no'. Retrieved value is: $tmp_value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'POST_SCRIPT'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'SPECINST'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'SPECINST' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($tmp_value !~ /^yes|no$/i)
   {
      baselog($LOG_ERROR, "'SPECINST' value of $PATCHINFO_FILE can be only 'yes' or 'no'. Retrieved value is: $tmp_value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'SPECINST'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'REBOOT'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'REBOOT' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($tmp_value !~ /^yes|no$/i)
   {
      baselog($LOG_ERROR, "'REBOOT' value of $PATCHINFO_FILE can be only 'yes' or 'no'. Retrieved value is: $tmp_value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'REBOOT'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'AUTHOR'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'AUTHOR' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'AUTHOR'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'CREATE_TIME'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'CREATE_TIME' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'CREATE_TIME'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'RPM_NAME'});

   # this to check if 'RPM_NAME' tag is present in the header.xml file. The 'RPM_NAME' value will be checked later.
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'RPM_NAME' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'RPM_NAME'} = $tmp_value;

   $tmp_value = $header->getHeaderValue($HEADER_FIELDS{'CR'});
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'CR' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'CR'} = $tmp_value;

   # need other variable as it used during 'META_DATA retrieval
   my $subtype = $header->getHeaderValue($HEADER_FIELDS{'SUBTYPE'});
   if (not defined $subtype)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'SUBTYPE' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   elsif ($subtype !~ /^war|jar|bin|fru|rpm$/i)
   {
      baselog($LOG_ERROR, "'SUBTYPE' value one of following: bin, fru, war, jar, rpm. Retrieved value is: $subtype. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'SUBTYPE'} = $subtype;

   $tmp_value = $header->getDependency();

   # 'DEPENDENT' tag can has no value. But if it's un-defined then mostly seems that tag is missing in header.xml.
   if (not defined $tmp_value)
   {
      baselog($LOG_ERROR, "Cannot retrieve 'DEPENDENT' value. Please check $PATCHINFO_FILE file.", $TRUE);
      return undef;
   }
   $patch_info{'DEPENDENT'} = $tmp_value;
   $patch_info{'PATCHNAME'} = $patchname;
   $patch_info{'FILENAME'}  = $filename;
   $patch_info{'PATCHPATH'} = $path;

   $patch_info{'APPS'}      = [$header->getApps()];
   $patch_info{'OBSOLETES'} = [$header->getObsoletes()];
   $patch_info{'APP_START'} = [$header->getAppStart()];
   $patch_info{'APP_STOP'}  = [$header->getAppStop()];

   # product release field exists for both PATCH and SU
   $patch_info{'PRODUCT_RLS'} = [$header->getReleases()];

   unless (isPATCHNAMEvalid(\%patch_info))
   {
      baselog($LOG_ERROR, "Field 'PATCHNAME' is too long");
   }

   #getting patch/su members from the header xml file

   my @members  = $header->getMembers();
   my @md5_sums = $header->getMD5Sums();
   if (scalar(@members) == 0)
   {
      baselog($LOG_ERROR, "Cannot get patch members info.", $TRUE);
      return undef;
   }

   if (scalar(@md5_sums) == 0)
   {
      baselog($LOG_ERROR, "Cannot get patch members md5 sums.", $TRUE);
      return undef;
   }

   $patch_info{'MEM_INFO'} = [@members];
   $patch_info{'MEM_MD5'}  = [@md5_sums];

   my @data = $header->getMetaData();
   if ($subtype =~ /^fru|jar|war|rpm$/i)
   {
      $patch_info{'META_DATA'} = [@data];
      $patch_info{'EXEC'}      = "";
   }
   elsif ($subtype =~ /^bin$/i)
   {
      my @libs = ();
      my @exec = ();
      foreach my $entry (@data)
      {
         push @libs, $entry->{'repl'};
         push @exec, $entry->{'exec'};
         $patch_info{$entry->{'exec'}} = $entry->{'funcs'};
      } ## end foreach my $entry (@data)
      $patch_info{'META_DATA'} = [@libs];
      $patch_info{'EXEC'}      = [@exec];
   } ## end elsif ($subtype =~ /^bin$/i...

   my @min_rpm_ver = ();
   my @mutex       = ();
   if ($subtype =~ /^fru$/i)
   {
      @min_rpm_ver = $header->getMinRpmVer();
      @mutex       = $header->getMutex();
   }
   $patch_info{'MIN_RPM_VER'} = [@min_rpm_ver];
   $patch_info{'MUTEX'}       = [@mutex];

   baselog($LOG_DEBUG, "Patch header info retrieved.");
   undef $header;

   baselog($LOG_INFO, "header_xml object released.");
   return \%patch_info;

} ## end sub get_patchheader_info($)

#########################################################
#Description: get full patch file path be patch name
#INPUT:  $_[0] - patch object
#OUTPUT: $filename - full patch file path
#########################################################
sub get_full_patch_file($)
{
   my $patch = shift(@_);
   return ($patch->{'PATCHPATH'} . '/' . $patch->{'FILENAME'});
}

############################################################
#Description: validate whether PATCHNAME field is valid
#INPUT:  $par - hash with patch info
#OUTPUT: $TRUE  - the field PATCHNAME is valid
#        $FALSE - the filed PATCHNAME is not valid
############################################################
sub isPATCHNAMEvalid($)
{

   my $par     = shift(@_);
   my $len     = length($par->{'PATCHNAME'});
   my $max_len = ($par->{'TYPE'} eq 'SU') ? $SU_PATCHNAME_MAX : $GENERIC_PATCHNAME_MAX;
   return ($len <= $max_len);
} ## end sub isPATCHNAMEvalid($)

##############################################
# Get patch member list
# Input: $_[0] - reference to the object
# Output: @members - list of patch members
##############################################
sub get_patch_members($)
{
   my $self = shift(@_);
   return @{$self->{'MEM_INFO'}};
}

##############################################
#
# Description: Get patch's status
# Input: $_[0] - reference to the object
# Output: patch's status
#
##############################################
sub get_patch_status()
{
   my $self = shift(@_);
   return $self->{'STATUS'};
}

###############################################
#
# Description: Get patch's id
# Input: $_[0] - reference to the object
# Output: patch's id
#
##############################################
sub get_patch_ID()
{
   my $self = shift(@_);
   return $self->{'ID'};
}

#############################################
# Get kernel RPM version inside patch
# Input: $_[0] - reference to the object
# Output: $ver - kernel RPM version
#         undef - error or not kernel RPM
##############################################
sub get_kernel_version($)
{
   my $self = shift(@_);
   my $ver  = undef;
   return undef unless ($self->is_service_update());
   foreach my $member (@{$self->{'MEM_INFO'}})
   {
      if ($member =~ /kernel-(smp-)?(.*)\.(.*)\.rpm/)
      {
         $ver = $1 . 'smp';
         last;
      }
   } ## end foreach my $member (@{$self...
   $ver =~ s/smp//gi if (defined $ver);
   return $ver;
} ## end sub get_kernel_version($)

##############################################
# Extract all patch members into $TMP_DIR
# Input: $_[0] - reference to the object
# Output: $TRUE  - success
#         $FALSE - error extracting
##############################################
sub extract_patch_members($)
{
   my $self = shift(@_);
   my $temp_dir;

   # If patch file does not exist, then there is no use in continuing
   my $file = get_full_patch_file($self);
   unless (-e $file)
   {
      baselog($LOG_ERROR, "Cannot find patch \"$file\"\n ");
      return $FALSE;
   }

   # return if already extracted
   return $TRUE if ($self->{'extracted'});

   # extract patch members
   my @members = @{$self->{'MEM_INFO'}};

   if (defined $self->{'PID'})
   {
      $temp_dir = $TMP_DIR . $self->{'PID'} . "/";
   }
   else
   {
      $temp_dir = $TMP_DIR;
   }

   #Create sub dir for this patches members
   if (!do_system([$MKDIR, '-p', $temp_dir]))
   {
      baselog($LOG_ERROR, "Cannot create patch member temporary directory $temp_dir\n", $TRUE);
      return $FALSE;
   }
   my @out;
   foreach my $member (@members)
   {
      if (!do_system([$TAR, 'zxf', $file, '-C', $temp_dir, $member], {$dsOUT => \@out}))
      {
         baselog($LOG_ERROR, "Cannot extract patch members (@out).");
         return $FALSE;
      }
   } ## end foreach my $member (@members...

   # check that all patch members are extracted
   my @members_files = ();
   foreach my $member (@members)
   {
      my $mem_file = $temp_dir . $member;
      if (not -e $mem_file)
      {
         baselog($LOG_ERROR, 'File ' . $mem_file . ' does not exists');
         $self->remove_patch_members();
         return $FALSE;
      } ## end if (not -e $mem_file)
      push(@members_files, $mem_file);
   } ## end foreach my $member (@members...
   $self->{'members_files'} = \@members_files;
   $self->{'extracted'}     = $TRUE;
   return $TRUE;
} ## end sub extract_patch_members($)

##############################################
# Remove patch members extracted before
# Input:  $_[0] - reference to the object
# Output: $TRUE  - success
#         $FALSE - error
##############################################
sub remove_patch_members($)
{
   my $self = shift(@_);

   # return if not extracted
   return $TRUE if (!$self->{'extracted'});

   # remove patch members extracted
   foreach my $file (@{$self->{'members_files'}})
   {
      if (-e $file)
      {

         # remove it
         if (system($RM, '-rf', $file) != $OK)
         {
            baselog($LOG_ERROR, "Cannot remove file: $file");
            next;
         }
         baselog($LOG_DEBUG, "File $file is removed");
      } ## end if (-e $file)
   } ## end foreach my $file (@{$self->...
   $self->{'members_files'} = undef;
   $self->{'extracted'}     = $FALSE;
} ## end sub remove_patch_members($)

################################################
## @method  check_patch_files
# @brief checks if patch member's files belong to rpm
#        specified in 'RPM_NAME' field of patch.
#        The function checks if the RPM is installed
#        on the target first, if not it then tries to 
#        find a SU in the loaded state that could be used.
# @details
# @param $self   - reference to patch object
# @param $action - command issued
# @param $pDB    - reference to the patch database
# @return $TRUE if no new files found, otherwise $FALSE
# @author
# @date
sub check_patch_files($$$)
{
   my ($self, $action, $pDB) = @_;
   unless (defined $self)
   {
      baselog($LOG_ERROR, "Patch object is not defined.");
      return $FALSE;
   }
   return $TRUE if ($self->{'SUBTYPE'} =~ /^rpm|jar$/i);

   # patch members should be extracted.
   if (not $self->{'extracted'})
   {
      baselog($LOG_ERROR, "Patch members are not extracted.", $TRUE);
      return $FALSE;
   }

   my @rpm_filelist;

   # if pload command, get file list base on full rpm name
   if ($action eq $PLOAD)
   {
      my $query_rpm_name;
      # if CS RPM then do not check last part of file name
      if ($self->{'RPM_NAME'} =~ '^(nortel-cs1000-cs-\d\.\d{2}\.\w)')
      {
         $query_rpm_name = $1 . '*';
      }
      else
      {
         $query_rpm_name = $self->{'RPM_NAME'};
      }
      #
      # Check if required RPM is installed on the target
      #
      if (not do_system([$RPM, '-ql', $query_rpm_name], {$dsOUT => \@rpm_filelist}))
      {
         baselog($LOG_WARNING, "Package $query_rpm_name is not installed on the target.", $TRUE);
         #
         # Check if there is a SU in a loaded state with the associated RPM
         #
         unless (defined $pDB)
         {
            baselog($LOG_ERROR, "Patch database object is not defined.");
            return $FALSE;
         }
         #
         # Determine if there are any SUs in a loaded state with the required Patch RPM
         # 
         my @poos_patches = $pDB->get_rpm_patches_by_name($query_rpm_name, $OUT_SERV, 'SU');
         if (scalar(@poos_patches) == 0)
         {
            baselog($LOG_INFO, "Patch $self->{'PATCHNAME'} doesn't have an associated SU/RPM $self->{RPM_NAME} loaded", $TRUE);
            return $FALSE;
         }
         else
         {  #
            # The assumption here is that there should be no more one relevant SU in a loaded state.
            if (scalar(@poos_patches) > 1)
            {
               baselog($LOG_INFO, "Patch $self->{'PATCHNAME'} has more than one associated SU/RPM $self->{RPM_NAME} loaded", $TRUE);
               return $FALSE;
            }
            #
            # Get the list of files in the loaded SU RPM
            #
            baselog($LOG_INFO, "Patch $self->{'PATCHNAME'} has an associated SU $poos_patches[0]->{'PATCHNAME'} loaded", $TRUE);

            if (not $self->get_su_rpm_filelist($poos_patches[0], \@rpm_filelist))
            {
               baselog($LOG_ERROR, "Failed to get filelist of SU $poos_patches[0]->{'PATCHNAME'}.", $TRUE);
               return $FALSE;
            }

            baselog($LOG_DEBUG,"$poos_patches[0]->{'PATCHNAME'} filelist:");
            foreach my $myfile (@rpm_filelist)
            {
               baselog($LOG_DEBUG,"... $myfile");
            }
         }
      }
   } ## end if ($action eq $PLOAD)
   else
   {

      # if spload get file list base on rpm name (without version),
      # since current patch it could be against same installed version or different installed version.

      if (!$self->get_rpm_filelist(\@rpm_filelist))
      {
         baselog($LOG_ERROR, "Package $self->{RPM_NAME} is not installed.", $TRUE);
         return $FALSE;
      }
   } ## end else [ if ($action eq $PLOAD)
   chomp(@rpm_filelist);

   my @patch_files = ();

   # if condition is needed because there is different checking depending on patch subtype
   if ($self->{'SUBTYPE'} =~ /^war$/i)
   {
      foreach my $member (@{$self->{'members_files'}})
      {
         my $war_file = `$UNZIP -p $member META-INF/patch-info | $GREP 'FilePath' | $AWK '{print \$3}'`;
         if ($war_file eq "")
         {
            baselog($LOG_ERROR, "Can't get WAR file from $member.", $TRUE);
            return $FALSE;
         }
         chomp($war_file);
         push @patch_files, $war_file;
      } ## end foreach my $member (@$self...
   } ## end if ($self->{'SUBTYPE'}...
   elsif ($self->{'SUBTYPE'} =~ /^fru$/i)
   {

      #for PI-control and deployment patches restrict the number of fru's patched to one.
      if ((is_PIcontrol_patch($self)) || (is_deployment_patch($self)))
      {
         if ((@{$self->{'members_files'}}) > 1)
         {
            baselog($LOG_ERROR, "There is more than one fru in this patch.\n", $TRUE);
            return $FALSE;
         }
      } ## end if ((is_PIcontrol_patch...
      my $members_ok = $TRUE;

      # firstly need to get file list of all files in FRU patch.
      foreach my $member (@{$self->{'members_files'}})
      {
         my @member_files;
         if (not do_system([$TAR, "tf", $member], {$dsOUT => \@member_files}))
         {
            baselog($LOG_ERROR, "Can't list files of $member.", $TRUE);
            return $FALSE;
         }
         if ((is_OS_patch($self)) || (is_deployment_patch($self)))
         {
            if (!check_member_files($self, $member, \@member_files))
            {
               $members_ok = $FALSE;
               last;
            }
         } ## end if ((is_OS_patch($self...
         else
         {
            chomp(@member_files);
            push @patch_files, @member_files;
         }
      } ## end foreach my $member (@$self...

      # For OS or deployment patch only, return true or false base on
      # file_exists flag.
      if ((is_OS_patch($self)) || (is_deployment_patch($self)))
      {
         if (!$members_ok)
         {
            return $FALSE;
         }
         return $TRUE;
      } ## end if ((is_OS_patch($self...
   } ## end elsif ($self->{'SUBTYPE'}...
   elsif ($self->{'SUBTYPE'} =~ /^bin$/i)
   {

      # need to check if executable specified in 'ORIG_PATH' belongs to the rpm
      foreach my $member (@{$self->{'members_files'}})
      {
         my @tar_files;
         if (not do_system([$TAR, "tf", $member], {$dsOUT => \@tar_files}))
         {
            baselog($LOG_ERROR, "Can't list members of file: $member", $TRUE);
            return $FALSE;
         }
         my @cntl = grep(/.*\.so\.cntl$/, @tar_files);
         chomp(@cntl);
         if (not scalar(@cntl))
         {
            baselog($LOG_ERROR, "There is no 'cntl' file in member $member", $TRUE);
            return $FALSE;
         }
         elsif (scalar(@cntl) > 1)
         {
            baselog($LOG_ERROR, "More than one (" . scalar(@cntl) . ") 'cntl' files found in member $member.", $TRUE);
            return $FALSE;
         }
         my $exec_file = `$TAR xf $member --to-stdout $cntl[0] | $GREP 'ORIG_PATH' | $AWK '{print \$2}'`;
         if ($?)
         {
            baselog($LOG_ERROR, "Can't retrieve executable file name from $member.", $TRUE);
            return $FALSE;
         }
         chomp($exec_file);
         push @patch_files, $exec_file;
      } ## end foreach my $member (@$self...
   } ## end elsif ($self->{'SUBTYPE'}...
   else
   {
      baselog($LOG_ERROR, "Unknown patch subtype: $self->{SUBTYPE}.", $TRUE);
      return $FALSE;
   }

   # if pi-control patch only
   if (is_PIcontrol_patch($self))
   {

      if ((scalar(@patch_files)) == 5)
      {

         # verify if patch member matches pi-control directory
         if ($patch_files[3] eq $PI_CONTROL_DIR)
         {
            baselog($LOG_DEBUG, "Validation for PI patch $self->{'PATCHNAME'} succeed.\n");
            return $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "Member path $patch_files[3] for pi-control xml file inside patch $self->{'PATCHNAME'} is incorrect. The path specified should be $PI_CONTROL_DIR\n.", $TRUE);
            return $FALSE;
         }
      } ## end if ((scalar(@patch_files...
      else
      {
         baselog($LOG_ERROR, "Incorrect member path for pi-control xml file inside patch $self->{'PATCHNAME'}. The path specified should be $PI_CONTROL_DIR\n.", $TRUE);
         return $FALSE;
      }
   } ## end if (is_PIcontrol_patch...
   #
   # Determine if the patch is against a file(s) being delivered by the specified RPM
   #
   if (not scalar(@rpm_filelist))
   {
      baselog($LOG_ERROR, "RPM $self->{RPM_NAME} file list is empty.", $TRUE);
      return $FALSE;
   }

   my $retval = $TRUE;
   foreach my $member (@patch_files)
   {
      my @grep_file = grep(/$member/, @rpm_filelist);

      # if @grep_file is empty then member doesn't belong to the patched rpm
      if (not scalar(@grep_file))
      {
         baselog($LOG_ERROR, "Member $member of patch $self->{PATCHNAME} doesn't belong to the rpm $self->{RPM_NAME}.", $TRUE);
         $retval = $FALSE;
         last;
      } ## end if (not scalar(@grep_file...
   } ## end foreach my $member (@patch_files...
   return $retval;
} ## end sub check_patch_files($$$)

################################################
## @method check_member_files
# @brief - checks if file member is valid
# @details - base on passed parameters obtain the path and
# @          filename of the specific member and check
# @          if the file belongs to any installed rpm.
# @          if that is the case, the file member is invalid.
# @          In addition, also validate that:
# @          In case of OS fru:
# @          1. the path member includes anything except piControl and app_loads directories.
# @          In case of deployment fru patch:
# @          1. the path member matches an existing application load subdir under app_loads dir
# @          2. the same path is used for all members
# @          3. that mandatory files members are included inside the fru
# @             (install.xml and load_deletion_lock files)
# @          4. that install.xml file is present on target
# @          5. that fru can deliver only new rpms and cannot update existing rpms.
# @          6. that only one new rpm per application is included in the fru
# @param $self - reference to the patch object
# @param $member - fru member
# @param $member_files - reference to the file members inside fru
# @return $TRUE  - the file member is valid
# @return $FALSE - the file member is not valid
################################################
sub check_member_files($$$)
{
   my ($self, $member, $member_files) = @_;
   my $retval                   = $TRUE;
   my $load_deletion_lock_file  = "load_deletion_lock";
   my $install_xml_file         = 'install.xml';
   my @app_loads_subdir_content = ();
   my $app_loads_subdir;
   my $found_xml_file = $FALSE;
   my $found_dm_flag  = $FALSE;
   my @rpms_in_fru    = ();

   foreach my $file (@$member_files)
   {
      my $path     = dirname($file);
      my $filename = basename($file);
      if (length($filename) gt 1)
      {
         chomp($file);

         # validate path member for OS or deployment patch
         if (!check_path_member($self, $path, $file, $member, \$app_loads_subdir, \@app_loads_subdir_content))
         {
            $retval = $FALSE;
            last;
         }
         if (is_deployment_patch($self))
         {

            # validate that install_xml_file exists on target
            chomp($filename);
            if ($filename eq $install_xml_file)
            {
               $found_xml_file = $TRUE;

               # validate that $install_xml_file exists on target.
               if (!-e "/$file")
               {
                  baselog($LOG_ERROR, "Cannot update file $file because doesn't exist on target.\n", $TRUE);
                  $retval = $FALSE;
                  last;
               } ## end if (!-e "/$file")
            } ## end if ($filename eq $install_xml_file...

            # if file is an rpm (Example: nortel-cs1000-vtrk-6.00.01-00.i386.rpm
            elsif ($filename =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)\.(rpm)$/)
            {

               # check duplicated rpm's in the tar file with different versions
               foreach my $rpm_file (@rpms_in_fru)
               {
                  if ($1 eq $rpm_file)
                  {
                     baselog($LOG_ERROR, "There are different versions of rpm name $1 inside the fru tar file.\n", $TRUE);
                     $retval = $FALSE;
                     last;
                  } ## end if ($1 eq $rpm_file)
               } ## end foreach my $rpm_file (@rpms_in_fru...
               if (!$retval)
               {
                  last;
               }

               # accumulate fru names found in the tar file to check duplicated rpms.
               push(@rpms_in_fru, $1);

               # check if rpm name already exists on target
               my $new_rpm_name = $1;
               foreach my $rpm_file_name (@app_loads_subdir_content)
               {

                  # check if rpm already exist on target
                  if ($filename eq $rpm_file_name)
                  {
                     baselog($LOG_ERROR, "RPM $filename already exist on target. Cannot update same file.\n", $TRUE);
                     $retval = $FALSE;
                     last;
                  } ## end if ($filename eq $rpm_file_name...
                  elsif ($rpm_file_name =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)\.(rpm)$/)
                  {

                     # check if another version of the same rpm exist on target.
                     my $found_rpm_name = $1;
                     if ($new_rpm_name eq $found_rpm_name)
                     {
                        baselog($LOG_ERROR, "Cannot deploy rpm $filename. A different version for the same rpm already exists on target ($rpm_file_name).\n", $TRUE);
                        $retval = $FALSE;
                        last;
                     } ## end if ($new_rpm_name eq $found_rpm_name...
                  } ## end elsif ($rpm_file_name =~ ...
               } ## end foreach my $rpm_file_name (...
               if (!$retval)
               {
                  last;
               }
            } ## end elsif ($filename =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)\.(rpm)$/...

            # validate that the file is a the dm flag file ($load_deletion_lock)
            elsif ($filename eq $load_deletion_lock_file)
            {
               $found_dm_flag = $TRUE;
            }
            else  # other files different than $install_xml, rpms and dm flag are errors.
            {
               baselog($LOG_ERROR, "Invalid file member $file.\n", $TRUE);
               $retval = $FALSE;
               last;
            } ## end else [ if ($filename eq $install_xml_file...
         } ## end if (is_deployment_patch...

         # check if full file name belong to any installed rpm.
         my $rpm_installed = undef;
         my $rpm_retval;
         do_system([$RPM, "-qf", "/$file", '--nosignature', '--qf', '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\n'], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$rpm_retval});
         if (not $rpm_retval)
         {
            if (defined(@$rpm_installed))
            {
               my $rpm_inst = @$rpm_installed[0];
               chomp($rpm_inst);
               baselog($LOG_ERROR, "Invalid file member $file under fru $member. This file member belongs to the $rpm_inst rpm.\n", $TRUE);
               $retval = $FALSE;
               last;
            } ## end if (defined(@$rpm_installed...
         } ## end if (not $rpm_retval)
      } ## end if (length($filename) ...
   } ## end foreach my $file (@$member_files...

   # validate that mandatory files ($install_xml_file and $load_deletion_lock_file)  for deployment patches are present
   if ((is_deployment_patch($self)) && ($retval))
   {
      if (!$found_xml_file)
      {
         baselog($LOG_ERROR, "The file $install_xml_file is missing inside the fru $member.\n", $TRUE);
         return $FALSE;
      }
      if (!$found_dm_flag)
      {
         baselog($LOG_ERROR, "The file $load_deletion_lock_file is missing inside the fru $member.\n", $TRUE);
         return $FALSE;
      }
   } ## end if ((is_deployment_patch...

   return $retval;
} ## end sub check_member_files($$$)

################################################
## @method  get_rpm_filelist
# @brief Obtain the file list of the rpm package that
#        is installed on the system or belong to an
#        SU that is in loaded state.
# @details 1. First check if the patch rpm name is installed.
# @        2. If it is, compare the full rpm name installed with
# @        the full rpm name of the patch.
# @        3. If they don't match, search for an SU that has
# @        been loaded and has same rpm name that the patch.
# @        4. If found full rpm name install or an SU loaded with full
# @        rpm name equal to patch rpm name, obtain the list of
# @        files for the specific rpm.
# @param $self - reference to patch object
# @param $action - command issued
# @param $ref_filelist - reference to the obtained rpm files list
# @return $TRUE  - successful: rpm file list is obtained
# @return $FALSE - failure: cannot obtain rpm file list
# @author
# @date
################################################
sub get_rpm_filelist($$)
{
   my ($self, $ref_filelist) = @_;
   my @fields_RPM = ();
   my $rpm_installed;
   my $ret;
   my $found_rpm = $FALSE;
   my $su_name;
   my $tmp_dir      = $PATCHDIR . "/tmp";
   my @rpm_filelist = ();

   @{$ref_filelist} = @rpm_filelist;

   if (!patchutil::patching::extract_RPM_name(undef, $self->{'RPM_NAME'}, \@fields_RPM))
   {
      baselog($LOG_ERROR, "Cannot obtain RPM name from patch rpm $self->{'RPM_NAME'}.\n", $TRUE);
      return $FALSE;
   }
   elsif (defined($fields_RPM[0]))  # if rpm name extracted
   {
      do_system([$RPM, '-q', $fields_RPM[0]], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$ret});
      if ((not $ret) && (defined(@$rpm_installed)))
      {
         $rpm_installed = @$rpm_installed[0];
         chomp($rpm_installed);

         # if CS RPM then do not check last part of file name
         my $rpm_installed_save = $rpm_installed;
         if ($rpm_installed =~ s/^(nortel-cs1000-cs-\d\.\d{2}\.\w\.)\d{3}(-\d{2})/${1}000${2}/)
         {
            baselog($LOG_INFO, "CS RPM. Do not check last version part");
            $fields_RPM[1] =~ s/(\d\.\d{2}\.\w\.)\d{3}/${1}000/;
         }

         # compose rpm name with version and release only
         my $rpm_name_with_ver_rls = $fields_RPM[0] . '-' . $fields_RPM[1] . '-' . $fields_RPM[2];

         # if rpm installed is equal to rpm from patch header, the patch is against RPM installed.
         # Otherwise, validate if patch is against SU to be installed
         if ($rpm_name_with_ver_rls ne $rpm_installed)
         {

            #obtain list of SUS from SP header
            my $h_obj = patchutil::header_xml::new("$SPLOAD_DIR/$PATCHINFO_FILE");
            if (!defined($h_obj))
            {
               baselog($LOG_ERROR, "Cannot create header_xml object\n", $TRUE);
               return $FALSE;
            }
            my @sus_list = $h_obj->getSPContent("SUS");
            foreach my $SU_item (@sus_list)
            {
               my @fields_SU = ();
               if (!patchutil::patching::extract_RPM_name(undef, $SU_item, \@fields_SU))
               {
                  baselog($LOG_ERROR, "Cannot extract RPM name from SU $SU_item\n", $TRUE);
               }
               elsif (defined($fields_SU[0]) and defined($fields_SU[1]) and defined($fields_SU[2]) and defined($fields_SU[3]))
               {

                  # compose rpm_name base on $SU_item without MPL issue and extension
                  my $rpm_name_for_SU = $fields_SU[0] . "-" . $fields_SU[1] . "-" . $fields_SU[2] . "." . $fields_SU[3];
                  if ($self->{'RPM_NAME'} eq $rpm_name_for_SU)
                  {

                     # this patch is against an su that is still loaded.
                     # untar SU to PATCHDIR dir
                     if (-e "$PATCHDIR/$SU_item")
                     {

                        # create $tmp_dir to untar rpm for SU
                        if (!patchutil::patching::spec_dir_prep($tmp_dir))
                        {
                           return $FALSE;
                        }
                        if (do_system([$TAR, 'xvzf', "$PATCHDIR/$SU_item", '-C', $tmp_dir]) == 0)
                        {
                           baselog($LOG_ERROR, "Cannot extract $SU_item under $PATCHDIR.\n", $TRUE);
                           return $FALSE;
                        }
                        $found_rpm = $TRUE;
                        $su_name   = $SU_item;
                        last;
                     } ## end if (-e "$PATCHDIR/$SU_item"...
                     else
                     {
                        baselog($LOG_ERROR, "Found SU $SU_item inside SP that matches rpm for patch $self->{'PATCHNAME'}, but SU failed to be loaded.\n", $TRUE);
                        last;
                     }
                  } ## end if ($self->{'RPM_NAME'...
               } ## end elsif (defined($fields_SU...
            } ## end foreach my $SU_item (@sus_list...

            # if not found_rpm, it means that the package rpm is not installed or found in any SU
            if (!$found_rpm)
            {
               baselog($LOG_ERROR, "Could not found rpm $self->{'RPM_NAME'} in any loaded SU.\n", $TRUE);
               return $FALSE;
            }
            else
            {

               #obtain the files inside the rpm package found on the SU
               do_system([$RPM, "-qpl", "$tmp_dir/$self->{'RPM_NAME'}.rpm"], {$dsNOERROR => $TRUE, $dsOUT => \@rpm_filelist, $dsRETVAL => \$ret});
               if (($ret) || (!scalar(@rpm_filelist)))
               {
                  baselog($LOG_DEBUG, "Cannot get rpm file list of RPM name $self->{'RPM_NAME'} found on SU $su_name for patch $self->{'PATCHNAME'}.\n");

                  # cleanup $tmp_dir
                  if (!do_system([$RM, '-rf', $tmp_dir]))
                  {
                     baselog($LOG_ERROR, "Can not remove $tmp_dir\n", $TRUE);
                  }
                  return $FALSE;
               } ## end if (($ret) || (!scalar...

               # cleanup $tmp_dir
               if (!do_system([$RM, '-rf', $tmp_dir]))
               {
                  baselog($LOG_ERROR, "Can not remove $tmp_dir\n", $TRUE);
                  return $FALSE;
               }
            } ## end else [ if (!$found_rpm)
         } ## end if ($rpm_name_with_ver_rls...
         else
         {
            my $ret_val;

            # this patch is against an installed rpm, so i can safety obtain the files list base on rpm name only
            do_system([$RPM, '-ql', $rpm_installed_save], {$dsNOERROR => $TRUE, $dsOUT => \@rpm_filelist, $dsRETVAL => \$ret_val});
            if (($ret_val) || (!scalar(@rpm_filelist)))
            {
               baselog($LOG_ERROR, "Cannot get rpm file_list of RPM name installed $self->{'RPM_NAME'} for patch $self->{'PATCHNAME'}.\n");
               return $FALSE;
            }
         } ## end else [ if ($rpm_name_with_ver_rls...

      } ## end if ((not $ret) && (defined...
      else
      {
         baselog($LOG_DEBUG, "RPM $fields_RPM[0] is not installed at all.\n");
         return $FALSE;
      }
   } ## end elsif (defined($fields_RPM...
   @{$ref_filelist} = @rpm_filelist;
   return $TRUE;
} ## end sub get_rpm_filelist($$)

################################################
## @method is_PIcontrol_patch
# @brief - checks if patch is a PI patch.
# @details - check if patch is an FRU and if full
# @pi-control rpm name includes $PI_CONTROL_RPM name.
# @param $self   - reference to patch object
# @return $TRUE  - patch is PI patch
# @return $FALSE - patch is not PI patch.
################################################
sub is_PIcontrol_patch($)
{
   my $self = shift(@_);

   # check if subtype is FRU and if
   # full rpm name includes PI_CONTROL_RPM name
   if (($self->{'SUBTYPE'} =~ /^fru$/i) && ($self->{'RPM_NAME'} =~ /^$PI_CONTROL_RPM/))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_PIcontrol_patch($)

################################################
## @method is_OS_patch
# @brief - checks if patch is an OS patch.
# @details - check if patch is an FRU and if full
# @rpm name includes $OS_RPM name.
# @param $self   - reference to patch object
# @return $TRUE  - patch is OS patch
# @return $FALSE - patch is not OS patch.
################################################
sub is_OS_patch($)
{
   my $self = shift(@_);

   # check if subtype is FRU and if
   # full rpm name includes OS_RPM name
   if (($self->{'SUBTYPE'} =~ /^fru$/i) && ($self->{'RPM_NAME'} =~ /^$OS_RPM/))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_OS_patch($)

################################################
## @method is_deployment_patch
# @brief - checks if patch is a deployment patch.
# @details - check if patch is an FRU and if full
# @rpm name includes $DEPLOYMENT_RPM name.
# @param $self   - reference to patch object
# @return $TRUE  - patch is deployment patch
# @return $FALSE - patch is not deployment patch.
################################################
sub is_deployment_patch($)
{
   my $self = shift(@_);

   # check if subtype is FRU and if
   # full rpm name includes OS_RPM name
   if (($self->{'SUBTYPE'} =~ /^fru$/i) && ($self->{'RPM_NAME'} =~ /^$DEPLOYMENT_RPM/))
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_deployment_patch($)

################################################
## @method check_path_member
# @brief - validate the path for an OS or deployment fru member
# @details - In case of OS fru validate:
# @          1. the path member includes anything except piControl and app_loads directories.
# @          In case of deployment fru patch:
# @          1. the path member matches an existing application load subdir under app_loads dir
# @          2. the same path is used for all members
# @param $self - reference to the patch object
# @param $path - path of the fru member
# @param $file - file member inside fru
# @param $member - fru name
# @param $subdir - reference to the subdirectory used for deployment of all the fru members
# @param $subdir_content - reference to an array with the files already deployed on target.
# @return $TRUE  - the path for the fru member/s is valid
# @return $FALSE - the path of an fru member is invalid
################################################
sub check_path_member($$$$$$)
{
   my ($self, $path, $file, $member, $subdir, $subdir_content) = @_;
   my $app_loadspath    = 'var/opt/nortel/deployment/app_loads';
   my $app_loads_subdir = ${$subdir};

   # make sure that OS patches don't include any fru member with the $PI_CONTROL_DIR dir or $app_loadspath dir
   if (is_OS_patch($self))
   {
      if (($path =~ /^($PI_CONTROL_DIR)/) || ($path =~ /^($app_loadspath)/))
      {
         baselog($LOG_ERROR, "The path for fru member $file inside $member is invalid.\n", $TRUE);
         return $FALSE;
      }
   } ## end if (is_OS_patch($self)...

   if (is_deployment_patch($self))
   {

      #if member path includes $app_loadspath and application load subdir
      if ($path =~ /^($app_loadspath)\/([0-9|A-Z|a-z|\.|\-]+)$/)
      {
         if (!-d "/$path")
         {
            baselog($LOG_ERROR, "The path for the fru member $file doesn't exist on target.\n", $TRUE);
            return $FALSE;
         }
         if (!defined($app_loads_subdir))
         {
            ${$subdir} = $2;
            my $dir;

            # get content of $app_loadspath dir
            unless (opendir($dir, "/$path"))
            {

               baselog($LOG_ERROR, "Cannot open $2 for reading.\n");
               return $FALSE;
            } ## end unless (opendir($dir, "/$path"...

            # this array will be set only once when processing the first member
            @{$subdir_content} = readdir($dir);
            closedir($dir);
         } ## end if (!defined($app_loads_subdir...
         else
         {

            # check that all the members of the fru include same
            # subdirectory under $app_loadspath
            if ($2 ne $app_loads_subdir)
            {
               baselog($LOG_ERROR, "There are different paths among the fru members. All fru members should be including same application load subdirectory under $app_loadspath directory.\n", $TRUE);
               return $FALSE;
            }
         } ## end else [ if (!defined($app_loads_subdir...
      } ## end if ($path =~ /^($app_loadspath)\/([0-9|A-Z|a-z|\.|\-]+)$/...
      else
      {
         baselog($LOG_ERROR, "Invalid path for fru member $file. The path member should match an application load subdirectory under the $app_loadspath directory.\n", $TRUE);
         return $FALSE;
      }
   } ## end if (is_deployment_patch...
   return $TRUE;
} ## end sub check_path_member($$$$$$)

###########################################################
# Description: extract patch members if necessary
#              and returns array of extracted files
# Input:     $_[0] - reference to the patch object
# Output:    \@array - refereance to array of patch members
#            undef - error
###########################################################
sub get_patch_members_extracted($)
{
   my $self = shift(@_);
   return undef if (not $self->extract_patch_members());
   return $self->{'members_files'};
}

###########################################################
# Description: test if the patch is kernel.
# Input:     $_[0] - reference to the patch object
# Output:    $TRUE  - the patch is a kernel patch
#            $FALSE - the patch is NOT a kernel patch
###########################################################
sub is_kernel_patch($)
{
   my $self = shift(@_);
   return $self->{'is_kernel'};
}

## @method is_broken($self)
# @brief   Test if the patch is broken.
# @details
# @param   $self - reference to the patch object
# @return  $TRUE  - the patch is broken
# @return  $FALSE - the patch is NOT broken
# @author  Vladimir Ivanov
# @date    Aug 2008
# @note    None.
sub is_broken($)
{
   my $self = shift(@_);
   return $self->{'is_broken'};
}

##########################################################
#Description: check if the patch is a service update.
#INPUT:     $_[0] - the object reference
#OUTPUT:    $TRUE  - the patch is a service update
#           $FALSE - the patch is NOT a service update
##########################################################
sub is_service_update($)
{
   my $self = shift(@_);
   return ($self->{'TYPE'} eq 'SU') ? $TRUE : $FALSE;
}

##############################################################
# Description: return external installation utility
# Input:  $_[0] - the object reference
# Output: install utility based on patch member subtype
##############################################################
sub get_install_utility()
{
   my $self = shift(@_);
   return $install{$self->{'SUBTYPE'}};
}

##############################################################
# Description: return external un-installation utility
# Input:  $_[0] - the object reference
# Output: un-install utility based on patch member subtype
##############################################################
sub get_uninstall_utility()
{
   my $self = shift(@_);
   return $uninstall{$self->{'SUBTYPE'}};
}

##################################################################
# Description: install all patch members using
#              according external utility
# Input:  $_[0] - the object reference
#         $_[1] - member to install (parameter for ext. utility)
# Output: $TRUE  - success
#         $FALSE - error
##################################################################
sub install_member($$)
{
   my ($self, $param) = @_;
   my $execstr = sprintf($self->get_install_utility(), $param);
   baselog($LOG_DEBUG, $execstr);

   if (system($execstr) ne $OK)
   {
      baselog($LOG_ERROR, 'The installation of ' . $param . ' has been failed');
      return $FALSE;
   }
   return $TRUE;
} ## end sub install_member($$)

#####################################################################
# Description: un-install all patch members using
#              according external utility
# Input:  $_[0] - the object reference
#         $_[1] - member to un- install (parameter for ext. utility)
# Output: $TRUE  - success
#         $FALSE - error
#####################################################################
sub uninstall_member($$)
{
   my ($self, $param) = @_;
   my $execstr = sprintf($self->get_uninstall_utility(), $param);
   baselog($LOG_DEBUG, $execstr);
   if (system($execstr) ne $OK)
   {
      baselog($LOG_ERROR, 'The un-installation of ' . $param . ' has been failed');
      return $FALSE;
   }
   return $TRUE;
} ## end sub uninstall_member($$)

############################################################
# Description: install all patch members using
#             according external utility
# Input:  $_[0] - the object reference
#         $_[1] - interactivity flag ($TRUE/$FALSE)
# Output: $TRUE  - success
#         $FALSE - error
############################################################
sub install($$)
{
   my ($self, $interact) = @_;
   my $subtype = $self->{'SUBTYPE'};

   print "\nPerforming the installation:\n";

   # For RPM there is a special subroutine
   if ($subtype eq 'RPM')
   {
      if (not $self->installRPM($interact))
      {
         baselog($LOG_ERROR, "SU installation failed\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($subtype eq 'RPM')
   else
   {
      my $members = $self->get_patch_members_extracted();
      if (not defined($members))
      {
         baselog($LOG_ERROR, "Cannot get patch members for $self->{'FILENAME'}\n", $TRUE);
         return $FALSE;
      }
      my @members = @{$members};
      baselog($LOG_DEBUG, "Patch members: " . join(' ', @members));
      my @rlback = ();

      # For each member invoke install procedure
      foreach my $member (@members)
      {
         my $ret = $self->install_member($member);
         if ($ret)
         {
            baselog($LOG_INFO, 'The patch member ' . $member . " has been installed successfully.\n");
            push @rlback, $member;
         }
         else
         {
            print "Installation of $member failed\n";

            # performing an uninstallation process for all patch members
            # that have already been installed during this session
            foreach my $rb (@rlback)
            {
               $self->uninstall_member($rb);
            }
            baselog($LOG_ERROR, "Error activating patch.\n", $TRUE);
            return $FALSE;
         } ## end else [ if ($ret)
      } ## end foreach my $member (@members...
   } ## end else [ if ($subtype eq 'RPM')
   print 'The patch ' . $self->{'PID'} . " has been installed successfully.\n";
   if (($subtype eq 'JAR') || ($subtype eq 'WAR'))
   {
      print "$restartapp\n";
   }

   baselog($LOG_INFO, "In service: $self->{'TYPE'} $self->{'PATCHNAME'} $self->{'PID'}");
   return $TRUE;
} ## end sub install($$)

############################################################
# Description: install all RPM patch members at once
# Input:  @_[0] - the object reference
#         @_[1] - interactivity flag ($TRUE/$FALSE)
# Output: $TRUE  - success
#         $FALSE - error
############################################################
sub installRPM($$)
{
   my ($self, $interact) = @_;
   my $execstr;

   my $members = $self->get_patch_members_extracted();
   if (not defined($members))
   {
      baselog($LOG_ERROR, "Cannot get patch members for $self->{'FILENAME'}\n", $TRUE);
      return $FALSE;
   }
   my @members = @{$members};
   my $rpms = join(' ', @members);
   baselog($LOG_INFO, "Installing: " . $self->{'RPM_NAME'} . "\n", $TRUE);
   $rpms = $self->{'backupdir'} . ' ' . $rpms;

   $execstr = sprintf($self->get_install_utility(), $rpms);

   # for patch reapplying and SU installation we do not need interactivity
   if (!$interact || $self->is_service_update())
   {
      $execstr = $execstr . "<<EOF\n";
      foreach (@members) {$execstr = $execstr . "y\n";}
      $execstr = $execstr . 'EOF';
   } ## end if (!$interact || $self...

   baselog($LOG_DEBUG, $execstr);
   if (system($execstr) ne $OK)
   {
      baselog($LOG_ERROR, 'Installation was not completed');
      return $FALSE;
   }
   baselog($LOG_INFO, 'SU installation completed');
   return $TRUE;
} ## end sub installRPM($$)

##############################################################
# Description: un-install all patch members using
#             according external utility
# Input:  $_[0] - the object reference
#         $_[1] - interactivity flag ($TRUE/$FALSE)
#         $_[2] - SU file name if SU is in service for
#                 the patched application (it is needed
#                 for RPM '--oldpackage' operation) or undef
# Output: $TRUE  - success
#         $FALSE - error
##############################################################
sub uninstall($$)
{
   my ($self, $interact) = @_;
   my $subtype = $self->{'SUBTYPE'};

   print "Performing the uninstallation:\n";

   if ($subtype eq 'RPM')
   {
      if (not $self->uninstallRPM($interact))
      {
         baselog($LOG_ERROR, "SU uninstallation failed\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($subtype eq 'RPM')
   else
   {
      my $members = $self->get_patch_members_extracted();
      if (not defined $members)
      {
         baselog($LOG_ERROR, "Cannot get patch members for $self->{'FILENAME'}\n", $TRUE);
         return $FALSE;
      }
      my @members = @{$members};
      baselog($LOG_DEBUG, "Patch members: " . join(' ', @members));
      foreach my $member (@members)
      {
         my @rlback = ();
         my $ret    = $self->uninstall_member($member);
         if ($ret)
         {
            baselog($LOG_INFO, "The patch member $member has been uninstalled successfully.");
            push @rlback, $member;
         }
         else
         {
            print "Uninstallation of $member failed\n";

            # performing an installation process for all patch members
            # that have already been un-installed during this session
            foreach my $rb (@rlback)
            {
               $self->install_member($rb);
            }
            baselog($LOG_ERROR, "Error deactivating patch.\n", $TRUE);
            return $FALSE;
         } ## end else [ if ($ret)
      } ## end foreach my $member (@members...
   } ## end else [ if ($subtype eq 'RPM')
   print 'The patch ' . $self->{'PID'} . " has been uninstalled successfully.\n";
   if (($subtype eq 'JAR') || ($subtype eq 'WAR'))
   {
      print "$restartapp\n";
   }
   baselog($LOG_INFO, "Out service: $self->{'TYPE'} $self->{'PATCHNAME'} $self->{'PID'}");
   return $TRUE;
} ## end sub uninstall($$)

########################################################
## @method uninstallRPM($self, $interact, $su)
# @brief   un-install all RPM patch members at once
# @details None
# @param   $self - reference to object
# @param   $interact - interactivity flag ($TRUE/$FALSE)
# @param   $su - SU file name if SU is in service for the patched application
#          (it is needed for RPM '--oldpackage' operation) or undef
# @return  $TRUE - success
# @return  $FALSE - error
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @date    May 2008
# @note    None.
########################################################
sub uninstallRPM($$)
{
   my ($self, $interact) = @_;
   my $execstr;
   my $env = '';
   my $ret = $TRUE;

   my $members = $self->get_patch_members_extracted();
   if (not defined($members))
   {
      baselog($LOG_ERROR, "Cannot get patch members for $self->{'FILENAME'}\n", $TRUE);
      return $FALSE;
   }
   my @members = @{$members};
   my $rpms = join(' ', @members);
   baselog($LOG_INFO, "Uninstalling: " . $self->{'RPM_NAME'} . "\n", $TRUE);
   $rpms = $self->{'backupdir'} . ' ' . $rpms;

   $execstr = sprintf($self->get_uninstall_utility(), $rpms);

   my $su_path = get_full_patch_file($self);
   system("$MKDIR \"$self->{'backupdir'}/SU\" ; $TAR -zxf $su_path -C \"$self->{'backupdir'}/SU\" ");
   if ($? != $OK)
   {
      baselog($LOG_ERROR, 'SU rpm directory creation failed.');
      return $FALSE;
   }
   baselog($LOG_DEBUG, "SU rpm directory $self->{'backupdir'}/SU has been created.");

   # Set environment variable
   $env     = 'SURPMBACKUP=' . $TRUE . ' ';  # one 'space' behind
   $execstr = $env . $execstr;
   baselog($LOG_DEBUG, $execstr);

   if (system($execstr) ne $OK)
   {
      baselog($LOG_ERROR, 'Uninstallation was not completed', $TRUE);
      $ret = $FALSE;
   }
   else
   {
      baselog($LOG_INFO, 'SU uninstallation completed');
   }

   # Remove SU directory
   if ($env ne '')
   {
      rmtree "$self->{'backupdir'}/SU";
      baselog($LOG_DEBUG, "SU rpm directory $self->{'backupdir'}/SU has been removed.");
   }
   return $ret;
} ## end sub uninstallRPM($$)

## @method  get_elements_str
# @brief get member elements
# @details
# @param $_[0] - reference to patch object
# @param $_[1] - element name if not defined than search all members
# @return $ret - formatted string with patch member elements
# @author
# @date
sub get_elements_str($)
{
   my $self      = shift;
   my $elem_name = (scalar(@_)) ? shift(@_) : undef;

   my $ret = "";

   $self->get_patch_members_extracted() unless ($self->{'members_files'});
   foreach my $file (@{$self->{'members_files'}})
   {
      foreach my $elem (patchutil::patch_db::get_member_elements($file))
      {
         foreach (@{$self->{'MEM_INFO'}})
         {
            if ($elem_name)
            {
               $ret .= "\t\t" . $elem . "\n" if ($file =~ /$elem_name$/);
               last;
            }
            else
            {
               $ret .= "\n\t\t" . $elem if ($file =~ /$_$/);
            }
         } ## end foreach (@{$self->{'MEM_INFO'...
      } ## end foreach my $elem (patchutil::patch_db::get_member_elements...
   } ## end foreach my $file (@{$self->...
   return $ret;
} ## end sub get_elements_str($)

## @method  get_specins_str
# @brief get special instruction mark
# @details
# @param $_[0] - reference to patch object
# @return "yes" - if patch has special instructions, otherwise "no"
# @author
# @date
sub get_specins_str($)
{
   my $self = shift;
   return $self->{'SPECINST'};
}

## @method  get_created_time_str
# @brief get patch creation time
# @details
# @param $_[0] - reference to patch object
# @return formatted time (e.g. "Thu Dec 20 16:04:30 2007")
# @author
# @date
sub get_created_time_str($)
{
   my $self = shift;
   return localtime($self->{'CREATE_TIME'});
}

## @method  get_pload_data_day_str
# @brief get patch load time(day, month, year)
# @details
# @param $_[0] - reference to patch object
# @return formatted time (e.g. "12/12/08")
# @author
# @date
sub get_pload_data_day_str($)
{
   my $self = shift;

   if ($self->{'PLOAD'})
   {
      my (undef, undef, undef, $day, $mon, $year, undef, undef, undef) = localtime($self->{'PLOAD'});
      return sprintf("%02d", $day) . "/" . sprintf("%02d", ++$mon) . "/" . sprintf("%02d", ($year + 1900) % 100);
   }
   return "";
} ## end sub get_pload_data_day_str($)

## @method  get_pins_data_day_str
# @brief get patch in-service time(day, month, year)
# @details
# @param $_[0] - reference to patch object
# @return formatted time (e.g. "12/12/08")
# @author
# @date
sub get_pins_data_day_str($)
{
   my $self = shift;

   if ($self->{'PINS'})
   {
      my (undef, undef, undef, $day, $mon, $year, undef, undef, undef) = localtime($self->{'PINS'});
      return sprintf("%02d", $day) . "/" . sprintf("%02d", ++$mon) . "/" . sprintf("%02d", ($year + 1900) % 100);
   }
   return "";
} ## end sub get_pins_data_day_str($)

## @method  get_pins_data_str
# @brief get patch in-service time(sec, min, hour, day, month, year)
# @details
# @param $_[0] - reference to patch object
# @return formatted time (e.g. "12/12/08 12:14:04")
# @author
# @date
sub get_pins_data_str($)
{
   my $self = shift;

   if ($self->{'PINS'})
   {
      my ($sec, $min, $hour, undef, undef, undef, undef, undef, undef) = localtime($self->{'PINS'});
      return $self->get_pins_data_day_str() . " " . sprintf("%02d", $hour) . ":" . sprintf("%02d", $min) . ":" . sprintf("%02d", $sec);
   }
   return "";
} ## end sub get_pins_data_str($)

## @method  get_status_text_str
# @brief get status text
# @details
# @param $_[0] - reference to patch object
# @return string (e.g. "Patch is out-of-service")
# @author
# @date
sub get_status_text_str($)
{
   my $self = shift;

   if ($self->{'STATUS'} == $OUT_SERV)
   {
      return "Patch is out-of-service";
   }
   elsif ($self->{'STATUS'} == $IN_SERV_REBOOT)
   {
      return "Patch will be in service after restart";
   }
   elsif ($self->{'STATUS'} == $OUT_SERV_REBOOT)
   {
      return "Patch will be out of service after restart";
   }
   elsif ($self->{'STATUS'} == $IN_SERV)
   {
      return "Patch is in-service";
   }
   return "";
} ## end sub get_status_text_str($)

## @method  get_status_date_str
# @brief get status date text
# @details
# @param $_[0] - reference to patch object
# @return string (e.g. "Last in-service date: 12/12/08 12:14:04")
# @author
# @date
sub get_status_date_str($)
{
   my $self = shift;

   if ($self->{'STATUS'} == $OUT_SERV)
   {
      return "\n\tLast in-service date: " . $self->get_pins_data_str() if ($self->{'PINS'});
   }
   elsif ($self->{'STATUS'} == $IN_SERV_REBOOT)
   {
      return "\n\tLast in-service date: " . $self->get_pins_data_str() if ($self->{'PINS'});
   }
   elsif ($self->{'STATUS'} == $OUT_SERV_REBOOT)
   {
      return "\n\tIn-service date: " . $self->get_pins_data_str();
   }
   elsif ($self->{'STATUS'} == $IN_SERV)
   {
      return "\n\tIn service date: " . $self->get_pins_data_str();
   }
   return "";
} ## end sub get_status_date_str($)

## @method  get_status_date_str
# @brief get status date text for plis command
# @details
# @param $_[0] - reference to patch object
# @return string (e.g. "Out of service date: Thu Dec 27 12:39:59 2007")
# @author
# @date
sub get_date_plis_str($)
{
   my $self = shift;
   my $ret  = "";

   if ($self->{'STATUS'} == $IN_SERV)
   {
      $ret = "In-service date: " . localtime($self->{'PINS'}) . "\n";
      my ($sec, $min, $hour, $mday) = (gmtime(time() - $self->{'PINS'}));
      $ret .= "In-service for   " . ($mday - 1) . " days, " . $hour . " hours, " . $min . " mins";
      return $ret;

      #print("In-service number of init(s): 0", "\n");  #TBD
   } ## end if ($self->{'STATUS'} ...
   else
   {
      $ret = "Out of service date: ";
      $ret .= ($self->{'POOS'}) ? localtime($self->{'POOS'}) . "\n" : "N/A";
      if ($self->{'POOS'})
      {
         my ($sec, $min, $hour, $mday) = gmtime((time() - $self->{'POOS'}));
         $ret .= "Out of service for " . ($mday - 1) . " days, " . $hour . " hours, " . $min . " mins";
      }
   } ## end else [ if ($self->{'STATUS'} ...
   return $ret;
} ## end sub get_date_plis_str($)

## @method  get_mark_in_service_str
# @brief get patch mark
# @details
# @param $_[0] - reference to patch object
# @return "*" if patch is in service, otherwise - ""
# @author
# @date
sub get_mark_in_service_str($)
{
   my $self = shift;
   return "*" if ($self->is_in_service());
   return "";
}

## @method  get_text_in_service_str
# @brief get yes/no response to in service request
# @details
# @param $_[0] - reference to patch object
# @return "Yes" if patch is in service, otherwise - "No"
# @author
# @date
sub get_text_in_service_str($)
{
   my $self = shift;
   return "Yes" if ($self->is_in_service());
   return "No";
}

## @method  is_in_service
# @brief  check if the patch is in-service
# @details
# @param $_[0] - reference to patch object
# @return $TRUE - if patch is in-service, otherwise $FALSE
# @author
# @date
sub is_in_service($)
{
   my $self = shift;
   return $FALSE unless (ref $self eq $class);
   return $TRUE if ($self->{'STATUS'} eq $IN_SERV
                    or ($self->{'STATUS'} eq $IN_SERV_REBOOT));
   return $FALSE;
} ## end sub is_in_service($)

## @method  is_patch
# @brief check if the patch has a type 'patch'
# @details
# @param $_[0] - reference to patch object
# @return $TRUE - if patch is 'patch', otherwise $FALSE
# @author
# @date
sub is_patch($)
{
   my $self = shift;
   return $FALSE unless (ref $self eq $class);
   return $TRUE if ($self->{'TYPE'} eq $PATCHTYPE{'PATCH'});
   return $FALSE;
} ## end sub is_patch($)

## @method  pstat_l
# @brief print patch statistics
# @details long format
# @param $_[0] - reference to patch object
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub pstat_l($)
{
   my $self = shift;
   return $FALSE unless (ref $self eq $class);

   my @show_list = ($self->{'PID'});
   push @show_list, $self->{'PATCHNAME'} if ($self->is_patch());
   push @show_list, $self->get_text_in_service_str;
   push @show_list, $self->get_pins_data_day_str() if ($self->is_in_service());
   push @show_list, $self->get_pload_data_day_str() if (not $self->is_in_service());
   push @show_list, $self->get_specins_str();
   push @show_list, $self->{'SUBTYPE'} if ($self->is_patch());

   push @show_list, $self->{'RPM_NAME'}  if ($self->is_patch());
   push @show_list, $self->{'REMOVABLE'} if ($self->is_service_update());
   push @show_list, $self->{'PATCHNAME'} if ($self->is_service_update());

   $FORMAT_FORMFEED = "";

   my $head_name = $self->{'TYPE'} . '_HEADER';
   my $body_name = $self->{'TYPE'} . '_CON';

   my ($head, $body);
   if ($self->is_patch())
   {
      $head = "PATCH#  NAME       IN_SERVICE   DATE      SPECINS  TYPE   RPM";
      $body = "@<<     @<<<<<<<<  @<<<         @<<<<<<<  @<<<     @<<<<  @*";
   }
   elsif ($self->is_service_update())
   {
      $head = "PATCH#  IN_SERVICE   DATE       SPECINS   REMOVABLE   NAME";
      $body = "@<<     @<<<         @<<<<<<<   @<<<      @<<<        @*";
   }

   if ($^ ne $head_name)
   {
      $- = 0;
      eval "format " . $head_name . " = \n" . $head . "\n.";
      eval "format " . $body_name . " = \n" . $body . "\n" . '@show_list' . "\n.";
   } ## end if ($^ ne $head_name)

   $FORMAT_TOP_NAME = $head_name;
   $FORMAT_NAME     = $body_name;

   write;
   return $TRUE;
} ## end sub pstat_l($)

## @method pstat
# @brief print patch statistics
# @details usual format
# @param $_[0] - reference to patch object
# @return $TRUE if success, otherwis $FALSE
# @author
# @date
sub pstat
{
   my $self = shift;
   return $FALSE unless (ref $self eq $class);
   return $FALSE unless (exists $self->{'PATCHNAME'});

   $- = 0;

   #format name
   my $body_name = $self->{'PATCHNAME'} . "_PSTAT";

   my $body =
       "format $body_name = \n"
     . "Patch handle "
     . $self->{'PID'}
     . $self->get_mark_in_service_str()
     . "\n\tPatch name: "
     . $self->{'PATCHNAME'}
     . "\n\tType: "
     . $self->{'TYPE'}
     . "\n\tReference number: "
     . $self->get_status_text_str()
     . $self->get_status_date_str()
     . "\n\tPatch special instructions: "
     . $self->get_specins_str()
     . "\n\tPatch member type: "
     . $self->{'SUBTYPE'}
     . "\n\tPatch members: "
     . $self->{'TYPE'};

   eval $body . "\n.";
   $~ = $body_name;
   write;

   return $TRUE;

} ## end sub pstat

## @method plis
# @brief print patch statistics
# @details for plis command
# @param $_[0] - reference to patch object
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub plis
{
   my $self = shift;
   return $FALSE unless (ref $self eq $class);
   return $FALSE unless (exists $self->{'PATCHNAME'});

   my $mark      = $self->get_mark_in_service_str();
   my $loaded    = localtime($self->{'PLOAD'});
   my $status    = $self->get_status_text_str();
   my $date      = $self->get_date_plis_str();
   my $appstarts = join(',', @{$self->{'APP_START'}});
   my $appstops  = join(',', @{$self->{'APP_STOP'}});

   print <<PLIS;
Handle: \t$self->{'PID'}$mark
Type: \t\t$self->{'TYPE'}
Filename: \t$self->{'FILENAME'}
Dependency: \t$self->{'DEPENDENT'}  
CR number: \t$self->{'CR'}
Engineer: \t$self->{'AUTHOR'}
Created: \t$self->{'CREATE_TIME'}
Loaded: \t$loaded
$status
$date
Special Instructions: \t$self->{'SPECINST'}
Applications stopped: \t$appstops
Applications started: \t$appstarts
Requires reboot: \t$self->{'REBOOT'}
PrePatch script: \t$self->{'PRE_SCRIPT'}
PostPatch script: \t$self->{'POST_SCRIPT'}
Patch member type: \t$self->{'SUBTYPE'}
RPM: \t$self->{'RPM_NAME'}
PLIS
   return $TRUE;

} ## end sub plis

## @method check_md5sum($self)
# @brief  check patch MD5 checksum
# @details calculate the md5 checksum of the patch file and compare it
#         with the value from the patch DB.
# @param   $self - reference to the patch object
# @return  $TRUE  - MD5 checksum calculated is equal to the value from the DB.
# @return  $FALSE - values are not equal.
# @author  Vladimir Ivanov
# @date    Aug 2008
# @note    None.
sub check_md5sum($)
{
   my $self     = shift(@_);
   my @mem_sums = @{$self->{'MEM_MD5'}};
   my $num      = 0;

   my $members = $self->get_patch_members_extracted();
   if (not defined($members))
   {
      baselog($LOG_ERROR, "Cannot get patch members for $self->{'FILENAME'}\n", $TRUE);
      return $FALSE;
   }
   my @members = @{$members};

   foreach my $member (@members)
   {
      my $md5 = get_patch_md5sum($member);
      unless (defined($md5) and $md5 eq $mem_sums[$num])
      {
         baselog($LOG_ERROR, "MD5SUM check failed for member: $member of patch $self->{PATCHNAME}", $TRUE);
         return $FALSE;
      }
      $num++;
   } ## end foreach my $member (@members...
   return $TRUE;

} ## end sub check_md5sum($)

## @method  issp
# @brief list rpm, su and patch details
# @details
# @param $_[0] - reference to patch object
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub issp($)
{
   my $self      = shift;
   my $appstarts = join(',', @{$self->{'APP_START'}});
   my $appstops  = join(',', @{$self->{'APP_STOP'}});
   return $FALSE unless (ref $self eq $class);
   return $FALSE unless ($self->is_in_service());

   $FORMAT_FORMFEED = "";

   $- = 0;

   my $body_name = 'patch' . $self->{'PID'} . '_ISSP';

   my $body = "format $body_name = \n" . $self->{'PATCHNAME'} . "|Start:" . $appstarts . "|Stop:" . $appstops . "|Reboot:" . $self->{'REBOOT'} . "|Spec:" . $self->get_specins_str() . "\n";

   eval $body . ".";
   $~ = $body_name;

   write;
   return $TRUE;
} ## end sub issp($)



################################################
## @method  get_su_rpm_filelist
# @brief Obtain the file list of the rpm package that
# @      of the specified SU 
# @details
# @param $self  - reference to patch object
# @param $patch - SU for which to obtain RPM filelist
# @param $ref_filelist - reference to the obtained rpm files list
# @return $TRUE  - successful: rpm file list is obtained
# @return $FALSE - failure: cannot obtain rpm file list
# @author
# @date
################################################

sub get_su_rpm_filelist($$$)
{
   my ($self, $patch, $ref_filelist) = @_;

   my $retval       = $FALSE;
   my $tmp_dir      = $PATCHDIR . "/tmp";
   my @rpm_filelist = ();

   @{$ref_filelist} = @rpm_filelist;

   # create $tmp_dir to untar rpm for SU
   if (!patchutil::patching::spec_dir_prep($tmp_dir))
   {
      baselog($LOG_ERROR, "Failed to create $tmp_dir", $TRUE);
      return $FALSE;
   }
   #
   # untar SU to PATCHDIR/tmp dir in order to get the list of files in the 
   # associated RPM member
   #
   if (-e "$PATCHDIR/$patch->{'FILENAME'}")
   {
      if (do_system([$TAR, 'xvzf', "$PATCHDIR/$patch->{'FILENAME'}", '-C', $tmp_dir]) == 0)
      {
         baselog($LOG_ERROR, "Cannot extract $patch->{'FILENAME'} under $PATCHDIR.\n", $TRUE);
         $retval =  $FALSE;
      }
      else
      {
         #
         # Obtain the list of files inside the rpm package found in the SU
         #
         my $ret;
         do_system([$RPM, "-qpl", "$tmp_dir/$patch->{'RPM_NAME'}.rpm"], {$dsNOERROR => $TRUE, $dsOUT => \@rpm_filelist, $dsRETVAL => \$ret});
         if (($ret) || (!scalar(@rpm_filelist)))
         {
            baselog($LOG_INFO, "Cannot get file list of RPM $patch->{'RPM_NAME'} found in SU $patch->{'PATCHNAME'}.", $TRUE);
            $retval =  $FALSE;
         } ## end if (($ret) || (!scalar...
         else
         {  #
            # Return the filelist extracted from the SU
            #
            @{$ref_filelist} = @rpm_filelist;
            $retval =  $TRUE;
         }
      }
   } ## end if (-e "$PATCHDIR/$patch->{'PATCHNAME'}"...
   else
   {
      baselog($LOG_ERROR, "Patch $patch->{'PATCHNAME'} not found.\n", $TRUE);
      $retval =  $FALSE;
   }

   # cleanup $tmp_dir
   if (!do_system([$RM, '-rf', $tmp_dir]))
   {
      baselog($LOG_ERROR, "Cannot remove $tmp_dir\n", $TRUE);
   }
   return $retval;
} ## end sub get_su_rpm_filelist($$)


1;

