## @file patch_db.pm
# @brief this module incapsulates all operations with patch DataBase
# @details none
# @author Alex Arkhipov
# @author Vladimir Ivanov
# @author Sergey Urlin
# @author Artem Shilin
# @date Aug 2008
# @version 1.00 - Initial version
# @version 1.01 - is_patch_loaded_byID is added.
# @version 1.02 - Patch file MD5 checksum check added (db integrity check).
# @version 1.03 - Two colons were removed before shared functions and variables.
# @version 1.06 - get_patch_id_by_name is added.
#                     - check_patches_dependent_on is added.
#                     - is_patch_in_service is added.
#                     - is_patch_in_service_byID is added.
#                     - new fields implemented in get_patch_params, add_patch_to_DB and delete_patch_from_DB
# @version 1.09 - get_rpm_patches, check_SU_conflict are added,
#		- get_app_patches is removed,
#		- get_kernel_update, check_patch_conflict, check_elements_conflict are modified,
#		- Fields 'CATEGORY', 'GEN_AVAIL_TIME', 'ISS' are removed
# @version 1.10 - Fields 'APPS', 'FILENAME', 'PATCHPATH' added
# @version 1.11 - Modify get_full_patch_file to accept patch object instead of patchname parameter.
# @version 1.12 - get_rpm_patches modified to returns SUs first.
# @version 1.13 - get_rpm_patches_by_name added
# @version 1.14 - check_patch_obsolete method added; check_patches_dependent_on modified
# @version 1.24 - check_element_conflict, add_patch_to_DB, delete_patch_from_DB, add_element_record, remove_element_record functions are modified
#		- check_functions_conflict added
#		- check_patch_conflict function removed
# @version 1.25 - Fix for Q01969344 - PSTAT ID shows incorrect status for patches that requires a reboot after install
# @version 1.26 - Added MIN_RPM_VER and MUTEX fields.
# @version 1.27 - Fix for Q01999918 - Drop issue number when checking mutex inside check_patch_mutex.
# @version 1.28 - Fix for Q02002104 - Fail to detect conflict when activating FRU patch with BIN patch in service
# @version 1.29 - Fix for Q02029805 - "BVWTT PM patching fails". Auto-flush action is added to put cached data to a disk
#                 after each change in a patching database.
# @note

package patchutil::patch_db;

use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use patchutil::patches_public;
use patchutil::patches_target;
use common_variables;

### Access management
use Exporter;
our @ISA    = qw(Exporter);  # : public Exporter;
our @EXPORT = qw($VERSION);

use basedb;
use baselog;
use patchutil::patch;

our $VERSION = "1.29";

################################
# Local variables definitions
################################
my $class             = "patchutil::patch_db";
my $PATCH_DB          = $HANDLES;
my $PATCH_ELEMENTS_DB = $HANDLES_ELEMENTS;
my $MAX_PATCHES       = 1000;

# Patch information

################################
# Public methods
################################
sub new();
sub DESTROY();

sub pload_patch($$);
sub pins_patch($$);
sub poos_patch($$);
sub pout_patch($$);
sub pout_patch_id($$);

sub get_patch($$);
sub is_patch_loaded($$);
sub is_patch_loaded_byID($$);
sub get_rpm_patches($$$;$);
sub get_rpm_patches_by_name($$$;$);
sub get_patch_params($$);
sub get_kernel_update($$);
sub check_functions_conflict($$$);
sub check_element_conflict($$$);
sub put_patch_in_service($$);
sub put_patch_out_service($$);
sub put_patch_in_service_after_reboot($$);
sub put_patch_out_service_after_reboot($$);
sub set_status($$$);
sub check_SU_conflict($;$);
sub add_element_record($$);
sub remove_element_record($$);
sub is_patch_in_service($$);
sub is_patch_in_service_byID($$);
sub get_patch_id_by_name($$);
sub check_patches_dependent_on($$);
sub check_patch_obsolete($$);
sub check_patch_mutex($$);
################################
# End of public methods
################################

################################
# Private methods
################################
sub get_free_id($);
sub add_patch_to_DB($$);
sub delete_patch_from_DB($$);
sub get_member_elements($);
sub put_patch_in_service_byID($$);
sub put_patch_out_service_byID($$);
sub set_status_byID($$$);
################################
# End of private methods
################################

#####################################################
## @method new()
# @brief Object constructor
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch ID
# @return reference to the new created object
#	undef - if error occurred (e.g. can not lock patch DB)
# @date
sub new()
{
   my $self = {};
   bless $self, $class;

   # Create DB is not exists
   if (not -e $PATCH_DB)
   {
      system($TOUCH, $PATCH_DB);
   }
   if (not -e $PATCH_ELEMENTS_DB)
   {
      system($TOUCH, $PATCH_ELEMENTS_DB);
   }

   # Lock patch db
   my ($db_descr, $hash_db) = basedb_lock($PATCH_DB);
   if (!defined($db_descr))
   {
      return undef;
   }

   # save data base info
   $self->{'db_descr'} = $db_descr;
   $self->{'hash_db'}  = $hash_db;

   #check db integrity
   unless ($self->check_db_integrity())
   {
      baselog($LOG_ERROR, "'$class' object not created");
      $self = undef;
   }
   else
   {
      baselog($LOG_DEBUG, "'$class' object created");
   }
   return $self;
} ## end sub new()

#####################################################
## @method DESTROY()
# @brief Object destructor
# @details
# @date
sub DESTROY()
{
   my $self = shift(@_);

   # unlock patch data base
   basedb_unlock($self->{'db_descr'}, $self->{'hash_db'});
   baselog($LOG_DEBUG, "'$class' object destroyed");
} ## end sub DESTROY()

#####################################################
## @method pload_patch($self, $patch)
# @brief Loads patch in patch data base
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch ID
# @return $TRUE - on success, $FALSE - if patch already loaded
# @date
sub pload_patch($$)
{
   my ($self, $patch) = @_;

   # update data base
   if (!$self->add_patch_to_DB($patch))
   {
      baselog($LOG_ERROR, "Cannot add patch to database", $TRUE);
      return $FALSE;
   }
   baselog($LOG_INFO, "Patch $patch->{'PATCHNAME'} loaded with handle $patch->{'PID'}", $TRUE);
   return $TRUE;
} ## end sub pload_patch($$)

#####################################################
## @method get_patch_params($self, $pid)
# @brief Check whether patch is loaded or not by patch file name
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch ID
# @return $ret - reference to the patch parameter hash
#         undef - patch is not in DB
# @date
sub get_patch_params($$)
{
   my ($self, $pid) = @_;
   my $ret = {};
   $ret->{'PID'} = $pid;
   my $p_hash = $self->{'hash_db'};
   my @p_i = split(/\t/, $p_hash->{$pid});
   $ret->{'GENERIC'}     = $p_i[$PATCH_FIELDS{'GENERIC'}];
   $ret->{'MACHINE'}     = $p_i[$PATCH_FIELDS{'MACHINE'}];
   $ret->{'TYPE'}        = $p_i[$PATCH_FIELDS{'TYPE'}];
   $ret->{'SUBTYPE'}     = $p_i[$PATCH_FIELDS{'SUBTYPE'}];
   $ret->{'REMOVABLE'}   = $p_i[$PATCH_FIELDS{'REMOVABLE'}];
   $ret->{'AUTHOR'}      = $p_i[$PATCH_FIELDS{'AUTHOR'}];
   $ret->{'CREATE_TIME'} = $p_i[$PATCH_FIELDS{'CREATE_TIME'}];
   $ret->{'RPM_NAME'}    = $p_i[$PATCH_FIELDS{'RPM_NAME'}];
   $ret->{'PATCHNAME'}   = $p_i[$PATCH_FIELDS{'PATCHNAME'}];
   $ret->{'PRODUCT_RLS'} = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'PRODUCT_RLS'}})];
   $ret->{'PRE_SCRIPT'}  = $p_i[$PATCH_FIELDS{'PRE_SCRIPT'}];
   $ret->{'POST_SCRIPT'} = $p_i[$PATCH_FIELDS{'POST_SCRIPT'}];
   $ret->{'SPECINST'}    = $p_i[$PATCH_FIELDS{'SPECINST'}];
   $ret->{'REBOOT'}      = $p_i[$PATCH_FIELDS{'REBOOT'}];
   $ret->{'CR'}          = $p_i[$PATCH_FIELDS{'CR'}];
   $ret->{'APPS'}        = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'APPS'}})];
   $ret->{'PATCHPATH'}   = $p_i[$PATCH_FIELDS{'PATCHPATH'}];
   $ret->{'FILENAME'}    = $p_i[$PATCH_FIELDS{'FILENAME'}];
   $ret->{'APP_START'}   = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'APP_START'}})];
   $ret->{'APP_STOP'}    = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'APP_STOP'}})];
   $ret->{'DEPENDENT'}   = $p_hash->{$pid . $PATCH_FIELDS{'DEPENDENT'}};
   $ret->{'OBSOLETES'}   = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'OBSOLETES'}})];
   $ret->{'MEM_INFO'}    = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'MEM_INFO'}})];
   $ret->{'MEM_MD5'}     = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'MEM_MD5'}})];
   $ret->{'STATUS'}      = $p_hash->{$pid . $PATCH_FIELDS{'STATUS'}};
   $ret->{'PLOAD'}       = $p_hash->{$pid . $PATCH_FIELDS{'PLOAD'}};
   $ret->{'PINS'}        = $p_hash->{$pid . $PATCH_FIELDS{'PINS'}};
   $ret->{'POOS'}        = $p_hash->{$pid . $PATCH_FIELDS{'POOS'}};
   $ret->{'META_DATA'}   = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'META_DATA'}})];
   $ret->{'MIN_RPM_VER'} = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'MIN_RPM_VER'}})];
   $ret->{'MUTEX'}       = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'MUTEX'}})];

   if (defined $p_hash->{$pid . $PATCH_FIELDS{'EXEC'}})
   {
      $ret->{'EXEC'} = [split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'EXEC'}})];
      foreach my $executable (split(/\t/, $p_hash->{$pid . $PATCH_FIELDS{'EXEC'}}))
      {
         $ret->{$executable} = [split(/\t/, $p_hash->{$pid . "_" . $executable})];
      }
   } ## end if (defined $p_hash->{...
   else
   {
      $ret->{'EXEC'} = "";
   }

   unless (defined $ret->{'MEM_INFO'})
   {
      $ret->{'MEM_INFO'}  = [];
      $ret->{'is_broken'} = $TRUE;
   }
   return $ret;
} ## end sub get_patch_params($$)

#####################################################
## @method get_patch($self, $pid)
# @brief Gets reference to the patch object by patch ID
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch ID
# @return $patch - reference to patch object
# 	  undef - if patch is not in database
# @date
sub get_patch($$)
{
   my ($self, $pid) = @_;

   if (!defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->get_patch($pid);
   }

   # check if patch is present
   return undef unless exists($self->{'hash_db'}->{$pid});

   # create patch object
   my $params = $self->get_patch_params($pid);
   return patchutil::patch::new($params);
} ## end sub get_patch($$)

#####################################################
## @method get_rpm_patches($self, $rpm; $status, $type)
# @brief Check whether patch is loaded or not by patch file name
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $rpm - rpm name
# @param $status (optional parameter) - status of patches ($ALL_PATCHES (default), $ALL_IN_SERVICE, $OUT_SERV, $IN_SERV, ...)
# @param $type (optional parameter) - Patch or SU, if undef then both will be returned
# @return @patches - array of patches which have requested status and belong to rpm provided
# 	  undef - if error occurred
# @date
sub get_rpm_patches($$$;$)
{
   my ($self, $rpm, $status, $type) = @_;
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->get_rpm_patches($rpm, $status, $type);
   }
   $status = $ALL_PATCHES unless defined $status;

   my @patches = ();
   my %db      = %{$self->{'hash_db'}};

   # Get keys from database
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {
      next unless exists $db{$i};
      my @patch_info = split(/\t/, $db{$i});
      my ($rpm_name, $patch_type) = (@patch_info[$PATCH_FIELDS{'RPM_NAME'}], @patch_info[$PATCH_FIELDS{'TYPE'}]);
      if (    ((not defined $rpm) or ((defined $rpm) and ($rpm_name eq $rpm)))
          and ((not defined $type) or ((defined $type) and ($patch_type eq $type))))
      {
         my $pstatus = $db{$i . $PATCH_FIELDS{'STATUS'}};
         if (   ($status == $ALL_PATCHES)
             or ($pstatus eq $status)
             or (($status == $ALL_IN_SERVICE)  and (($pstatus eq $IN_SERV)  or ($pstatus eq $IN_SERV_REBOOT)))
             or (($status == $ALL_OUT_SERVICE) and (($pstatus eq $OUT_SERV) or ($pstatus eq $OUT_SERV_REBOOT))))
         {
            my $patch = patchutil::patch::new($self->get_patch_params($i));
            push(@patches, $patch);
         } ## end if (($status == $ALL_PATCHES...
      } ## end if (((not defined $rpm...
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   if (defined $type)
   {
      return @patches;
   }
   else
   {
      my @patchsort = sort {$$b{TYPE} cmp $$a{TYPE}} @patches;
      return @patchsort;
   }
} ## end sub get_rpm_patches($$$;$)

#####################################################
## @method get_rpm_patches_by_name($self, $rpm; $status, $type)
# @brief Gets array of patch objects that are against the same rpm name and match the supplied status.
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $rpm - rpm name
# @param $status - status of patches ($ALL_PATCHES, $ALL_IN_SERVICE, $OUT_SERV, $IN_SERV, ...)
# @param $type (optional parameter) - Patch or SU, if undef then both will be returned
# @return @patches - array of patches which have requested status and belong to rpm provided
# 	  undef - if error occurred
# @date
sub get_rpm_patches_by_name($$$;$)
{
   my ($self, $rpm, $status, $type) = @_;
   my @fields_SU_passed;

   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->get_rpm_patches_by_name($rpm, $status, $type);
   }
   $status = $ALL_PATCHES unless defined $status;

   my @patches = ();
   my %db      = %{$self->{'hash_db'}};

   if (defined $rpm and !patchutil::patching::extract_RPM_name(undef, $rpm, \@fields_SU_passed))
   {
      baselog($LOG_ERROR, "Cannot obtain RPM name from $rpm\n", $TRUE);
      return @patches;  # you've got problems, return an empty array
   }

   # Get keys from database
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {
      my @fields_SU_DB;
      next unless exists $db{$i};

      my @patch_info = split(/\t/, $db{$i});
      my ($rpm_name, $patch_type) = (@patch_info[$PATCH_FIELDS{'RPM_NAME'}], @patch_info[$PATCH_FIELDS{'TYPE'}]);
      if (!patchutil::patching::extract_RPM_name(undef, $rpm_name, \@fields_SU_DB))
      {
         baselog($LOG_ERROR, "Cannot obtain RPM name from $rpm_name\n", $TRUE);
      }
      elsif (defined($fields_SU_DB[0]))  # SU name extracted
      {
         if (    ((not defined $rpm) or ((defined $rpm) and ($fields_SU_DB[0] eq $fields_SU_passed[0])))
             and ((not defined $type) or ((defined $type) and ($patch_type eq $type))))
         {
            my $pstatus = $db{$i . $PATCH_FIELDS{'STATUS'}};
            if (   ($status == $ALL_PATCHES)
                or ($pstatus eq $status)
                or (($status == $ALL_IN_SERVICE)  and (($pstatus eq $IN_SERV)  or ($pstatus eq $IN_SERV_REBOOT)))
                or (($status == $ALL_OUT_SERVICE) and (($pstatus eq $OUT_SERV) or ($pstatus eq $OUT_SERV_REBOOT))))
            {
               my $patch = patchutil::patch::new($self->get_patch_params($i));
               push(@patches, $patch);
            } ## end if (($status == $ALL_PATCHES...
         } ## end if (((not defined $rpm...
      } ## end elsif (defined($fields_SU_DB...
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   if (defined $type)
   {
      return @patches;
   }
   else
   {
      my @patchsort = sort {$$b{TYPE} cmp $$a{TYPE}} @patches;
      return @patchsort;
   }
} ## end sub get_rpm_patches_by_name($$$;$)

#####################################################
## @method check_SU_conflict($self)
# @brief Checks if there are several versions of SU for one rpm exists in database
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $rpm (optional) - rpm name
# @return @su - array of duplicated SUs
#	or empty array
# @date
sub check_SU_conflict($;$)
{
   my ($self, $rpm) = @_;
   my (%t1, %t2) = ((), ());
   return grep {!$t2{$_->{'RPM_NAME'}++}} (grep {$t1{$_->{'RPM_NAME'}++}} $self->get_rpm_patches($rpm, $ALL_OUT_SERVICE, 'SU'));
}
#####################################################
## @method is_patch_loaded($self, $patchname)
# @brief Check whether patch is loaded or not by patch file name
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch file name
# @return $TRUE - patch is loaded, $FALSE - patch is not loaded
# @date
sub is_patch_loaded($$)
{
   my ($self, $patchname) = @_;
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->is_patch_loaded($patchname);
   }

   # check if patch is loaded
   my $pid = $self->get_patch_id_by_name($patchname);
   if (defined $pid)
   {
      baselog($LOG_DEBUG, "Patch found: $pid");
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_patch_loaded($$)

#####################################################
## @method is_patch_in_service_byID($self, $pid)
# @brief Checks whether patch is in service or not by patch ID
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $pid - patch ID
# @return $TRUE - patch is in service, $FALSE - patch is out of service
# @date
sub is_patch_in_service_byID($$)
{
   my ($self, $pid) = @_;
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->is_patch_in_service($pid);
   }

   # check if patch is in service
   return $TRUE if (exists($self->{'hash_db'}->{$pid})
                    and ($self->{'hash_db'}->{$pid . $PATCH_FIELDS{'STATUS'}} == $IN_SERV));
   return $FALSE;
} ## end sub is_patch_in_service_byID($$)

#####################################################
## @method is_patch_in_service($self, $patchname)
# @brief Checks whether patch is in service or not by patch name
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $patchname - patch file name
# @return $TRUE - patch is in service, $FALSE - patch is out of service
# @date
sub is_patch_in_service($$)
{
   my ($self, $patchname) = @_;

   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->is_patch_in_service($patchname);
   }

   my $pid = $self->get_patch_id_by_name($patchname);
   return $self->is_patch_in_service_byID($pid) if (defined $pid);
   return $FALSE;
} ## end sub is_patch_in_service($$)

#####################################################
## @method check_patches_dependent_on($self, $patchname)
# @brief Checks if there are any patches which dependent on patch provided
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $patchname - patch file name
# @return informational string about dependent strings or undef if there is no one
# @date
sub check_patches_dependent_on($$)
{
   my ($self, $patchname) = @_;
   my @retval = ();
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->get_patches_dependent_on($patchname);
   }

   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {

      # Run over all possible PIDs and check if there are patches which depend on the patch that we are going to POOS.
      # First, check if PID exist in DB
      # Second, check if the current patch is dependent on patch which we are going to POOS.
      # Third, check if the current patch is IN SERVICE state.
      # If all conditions above are TRUE then we have a problem with our patch, because there is some patch in service
      # which need our patch to be in service too. It means that our patch can't be POOSed now.
      if (    exists($self->{'hash_db'}->{$i})
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'DEPENDENT'}} =~ /$patchname/)
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'STATUS'}} == $IN_SERV))
      {
         my @patch_info = split(/\t/, $self->{'hash_db'}->{$i});
         my $pname = @patch_info[$PATCH_FIELDS{'PATCHNAME'}];
         push @retval, "$pname (POOS $i)";
      } ## end if (exists($self->{'hash_db'...
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   return @retval;
} ## end sub check_patches_dependent_on($$)

#####################################################
## @method check_patch_obsolete($self, $patchname)
# @brief Checks if there are any patches which obsolete patch provided
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $patchname - patch file name
# @return name of patch which obsoletes the given patch or undef if there is no such patch
# @date
sub check_patch_obsolete($$)
{
   my ($self, $patchname) = @_;
   my @retval = ();
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->check_patch_obsolete($patchname);
   }

   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {

      # Run over all possible PIDs and check if there are patches which obsoletes the given patch.
      # First, check if PID exist in DB
      # Second, check if the current patch contains patch which we are going to PINS in the obsolete list.
      # Third, check if the current patch is IN SERVICE state.
      # If all conditions above are TRUE then we have a patch in service which obsoteles the given patch
      # So the given patch can't be PINS'ed
      if (    exists($self->{'hash_db'}->{$i})
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'OBSOLETES'}} =~ /$patchname/)
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'STATUS'}} == $IN_SERV))
      {
         my @patch_info = split(/\t/, $self->{'hash_db'}->{$i});
         my $pname = @patch_info[$PATCH_FIELDS{'PATCHNAME'}];
         push @retval, $pname;
      } ## end if (exists($self->{'hash_db'...
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   return @retval;
} ## end sub check_patch_obsolete($$)

#####################################################
## @method check_patch_mutex($self, $patchname)
# @brief Checks if there are any patches which mutex patch provided
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $patchname - patch file name
# @return name of patch which are mutually exclusive with the given patch or undef if there is no such patch
# @date
sub check_patch_mutex($$)
{
   my ($self, $patchname) = @_;
   my @retval = ();
   unless (defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->check_patch_mutex($patchname);
   }

   my $p_name = undef;

   # get $patchname without issue.
   if ($patchname =~ /^(.*?)\_([0-9]{1,2})$/)
   {
      $p_name = $1;
   }
   if (!defined($p_name))
   {
      baselog($LOG_ERROR, "Cannot obtain patchname without issue from $patchname\n", $TRUE);
      return $FALSE;
   }
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {

      # Run over all possible PIDs and check if there are patches which include inside MUTEX list the given patch.
      # 1. check if PID exist in DB
      # 2. check if the current patch contains patch which we are going to PINS in the mutex list.
      # 3  check if the current patch is IN SERVICE state.
      # If all conditions above are TRUE then we have a patch in service and include the given patch in the mutex list
      # So the given patch can't be PINS'ed
      if (    exists($self->{'hash_db'}->{$i})
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'MUTEX'}} =~ /$p_name/)
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'STATUS'}} == $IN_SERV))
      {
         my @patch_info = split(/\t/, $self->{'hash_db'}->{$i});
         my $pname = @patch_info[$PATCH_FIELDS{'PATCHNAME'}];
         push @retval, $pname;

         # print a warning message if the pachname issue contained inside the mutex list of current patch is not the same as the issue of the patchname passed as parameter.
         unless ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'MUTEX'}} =~ /$patchname/)
         {
            baselog($LOG_WARNING,
                    "Patch $patchname is mutually exclusive with patch $pname, but a different patch issue is in the MUTEX list of $pname. Please take appropriate action to update the patches.\n",
                    $TRUE);
         } ## end unless ($self->{'hash_db'}...
      } ## end if (exists($self->{'hash_db'...

   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   return @retval;
} ## end sub check_patch_mutex($$)

#####################################################
## @method get_patch_id_by_name($self, $patchname)
# @brief Gets patch ID (PID) by given patchname
# @details
# @param $self - reference to patch_db object, if "undef" then temporary object is created
# @param $patchname - patch name
# @return PID of patch if patch exists in database, otherwise undef
# @date
sub get_patch_id_by_name($$)
{
   my ($self, $patchname) = @_;
   if (!defined $self)
   {
      my $pDB = patchutil::patch_db::new();
      return $pDB->get_patch_id_by_name($patchname);
   }
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {
      return $i if (exists($self->{'hash_db'}->{$i})
                    and ($self->{'hash_db'}->{$i} =~ /\t($patchname)\t/));
   }
   return undef;
} ## end sub get_patch_id_by_name($$)

#####################################################
## @method is_patch_loaded_byID ($self, $pid)
# @brief check if pid provided exists or not
# @details
# @param $self - reference to patch_db object
# @param $pid - patch PID
# @return $TRUE if patch pid exists, otherwise $FALSE
# @author Peter Kascheev
# @date
sub is_patch_loaded_byID($$)
{
   my ($self, $pid) = @_;

   # check if patch exists
   return $TRUE if (exists($self->{'hash_db'}->{$pid}));
   return $FALSE;
} ## end sub is_patch_loaded_byID($$)

#####################################################
## @method pout_patch($self, $patch)
# @brief Removes patch from DB by patch ID
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to the patch object
# @return $TRUE on success, $FALSE - if patch is not loaded
# @date
sub pout_patch($$)
{
   my ($self, $patch) = @_;
   return $self->pout_patch_id($patch->{'PID'});
}

#####################################################
## @method pout_patch_id($self, $pid)
# @brief Removes patch from DB by patch ID
# @details
# @param $self - reference to patch_db object
# @param $patchname - patch ID
# @return $TRUE on success, $FALSE - if patch is not loaded
# @date
sub pout_patch_id($$)
{
   my ($self, $pid) = @_;
   return $self->delete_patch_from_DB($pid);
}

#####################################################
## @method get_free_id($self)
# @brief Gets first 'free' patch handle in DB
# @details
# @param $self - reference to patch_db object
# @return $id - first 'free' ID, undef if there is no 'free' ID in DB
# @date
sub get_free_id($)
{
   my $self        = shift(@_);
   my $free_handle = undef;
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {
      if (!exists($self->{'hash_db'}->{$i}))
      {
         $free_handle = $i;
         last;
      }
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   if (!defined($free_handle))
   {
      baselog($LOG_ERROR, "Patch number has reached the limit ($MAX_PATCHES)", $TRUE);
      return undef;
   }
   baselog($LOG_DEBUG, "Free handle reserved: $free_handle");
   return $free_handle;
} ## end sub get_free_id($)

#####################################################
## @method add_patch_to_DB ($self, $patch)
# @brief Adds patch into patch DB. Updates pload time
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to the patch object
# @return $TRUE on success, otherwise $FALSE
# @date
sub add_patch_to_DB($$)
{
   my ($self, $patch) = @_;
   my $new_id = $self->get_free_id();
   if (!defined($new_id))
   {
      baselog($LOG_ERROR, "Cannot aquire free patch handle");
      return $FALSE;
   }

   # Update patch's PID
   $patch->{'PID'} = $new_id;
   baselog($LOG_INFO, "New patch ID = " . $new_id);

   # Write_record_to_DB
   $self->{'hash_db'}->{$new_id} =
       $patch->{'GENERIC'} . "\t"
     . $patch->{'MACHINE'} . "\t"
     . $patch->{'TYPE'} . "\t"
     . $patch->{'SUBTYPE'} . "\t"
     . $patch->{'REMOVABLE'} . "\t"
     . $patch->{'AUTHOR'} . "\t"
     . $patch->{'CREATE_TIME'} . "\t"
     . $patch->{'RPM_NAME'} . "\t"
     . $patch->{'PATCHNAME'} . "\t"
     . $patch->{'PRE_SCRIPT'} . "\t"
     . $patch->{'POST_SCRIPT'} . "\t"
     . $patch->{'SPECINST'} . "\t"
     . $patch->{'REBOOT'} . "\t"
     . $patch->{'CR'} . "\t"
     . $patch->{'PATCHPATH'} . "\t"
     . $patch->{'FILENAME'} . "\t";

   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'APP_START'}} = join("\t", @{$patch->{'APP_START'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'APP_STOP'}}  = join("\t", @{$patch->{'APP_STOP'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'DEPENDENT'}}   = $patch->{'DEPENDENT'};
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'OBSOLETES'}}   = join("\t", @{$patch->{'OBSOLETES'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'PRODUCT_RLS'}} = join("\t", @{$patch->{'PRODUCT_RLS'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'APPS'}}        = join("\t", @{$patch->{'APPS'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'MEM_INFO'}}    = join("\t", @{$patch->{'MEM_INFO'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'MEM_MD5'}}     = join("\t", @{$patch->{'MEM_MD5'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'META_DATA'}}   = join("\t", @{$patch->{'META_DATA'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'MIN_RPM_VER'}} = join("\t", @{$patch->{'MIN_RPM_VER'}});
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'MUTEX'}}       = join("\t", @{$patch->{'MUTEX'}});

   if ($patch->{'EXEC'} ne "")
   {
      my @exec = @{$patch->{'EXEC'}};
      foreach my $executable (@exec)
      {
         $self->{'hash_db'}->{$new_id . "_" . $executable} = join("\t", @{$patch->{$executable}});
      }
      $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'EXEC'}} = join("\t", @exec);
   } ## end if ($patch->{'EXEC'} ne...
   else
   {
      $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'EXEC'}} = "";
   }

   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'STATUS'}} = $patch->{'STATUS'};
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'PLOAD'}}  = time();
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'PINS'}}   = $patch->{'PINS'};
   $self->{'hash_db'}->{$new_id . $PATCH_FIELDS{'POOS'}}   = $patch->{'POOS'};

   basedb_sync($self->{'hash_db'});

   baselog($LOG_INFO, "Patch DB was updated successfully");

   return $TRUE;
} ## end sub add_patch_to_DB($$)

#####################################################
## @method delete_patch_from_DB ($self, $pid)
# @brief Removes patch info from the patch DB
# @details
# @param $self - reference to patch_db object
# @param $pid - patch ID to remove
# @return $TRUE on success, otherwise $FALSE (if path is not in database)
# @date
sub delete_patch_from_DB($$)
{
   my ($self, $pid) = @_;
   my $db = $self->{'hash_db'};

   if (not exists($db->{$pid}))
   {
      baselog($LOG_ERROR, "The patch $pid is not found in DB");
      return $FALSE;
   }

   # Delete record from DB
   delete($db->{$pid . $PATCH_FIELDS{'STATUS'}});
   delete($db->{$pid . $PATCH_FIELDS{'PLOAD'}});
   delete($db->{$pid . $PATCH_FIELDS{'PINS'}});
   delete($db->{$pid . $PATCH_FIELDS{'POOS'}});
   delete($db->{$pid . $PATCH_FIELDS{'MEM_MD5'}});
   delete($db->{$pid . $PATCH_FIELDS{'APP_START'}});
   delete($db->{$pid . $PATCH_FIELDS{'APP_STOP'}});
   delete($db->{$pid . $PATCH_FIELDS{'DEPENDENT'}});
   delete($db->{$pid . $PATCH_FIELDS{'OBSOLETES'}});
   delete($db->{$pid . $PATCH_FIELDS{'PRODUCT_RLS'}});
   delete($db->{$pid . $PATCH_FIELDS{'APPS'}});
   delete($db->{$pid . $PATCH_FIELDS{'MIN_RPM_VER'}});
   delete($db->{$pid . $PATCH_FIELDS{'MUTEX'}});

   delete($db->{$pid});

   delete($db->{$pid . $PATCH_FIELDS{'META_DATA'}});
   my @exec = split("\t", $db->{$pid . $PATCH_FIELDS{'EXEC'}});
   foreach my $executable (@exec)
   {
      delete($db->{$pid . "_" . $executable});
   }

   delete($db->{$pid . $PATCH_FIELDS{'EXEC'}});
   delete($db->{$pid . $PATCH_FIELDS{'MEM_INFO'}});

   basedb_sync($self->{'hash_db'});

   baselog($LOG_INFO, "Patch DB was updated successfully");

   return $TRUE;
} ## end sub delete_patch_from_DB($$)

#####################################################
## @method get_member_elements ($filename)
# @brief Gets patch member elements' list
# @details
# @param $filename - patch member filename
# @return @list - list of elements belong to patch
# @date
sub get_member_elements($)
{
   my $filename = shift(@_);
   my @list     = ();
   if ($filename =~ /FRU$/i)  #FRU file
   {
      @list = `$TAR tf $filename`;
      chomp @list;

      # parse only patch elements, excluding upper directories
      my @list_tmp;
      my $prev_elem = $list[0];
      $list[scalar(@list)] = '';
      foreach my $cur_elem (@list)
      {
         if ($cur_elem !~ /^$prev_elem/) {push @list_tmp, $prev_elem;}
         $prev_elem = $cur_elem;
      }
      @list = @list_tmp;
   } ## end if ($filename =~ /FRU$/i...
   elsif ($filename =~ /JAR$/i)  #JAR file
   {
      my @classes = `$UNZIP -l -qq  $filename | $GREP -v 'META-INF'| $AWK '{print \$1" "\$4}'| $SED 's/\\//./g'`;

      # removing folder from list
      foreach my $line (@classes)
      {
         my ($size, $name) = split(' ', $line);
         push @list, $name if ($size ne '0');
      }
   } ## end elsif ($filename =~ /JAR$/i...
   elsif ($filename =~ /WAR$/i)  #WAR file
   {
      my $war_file = `$UNZIP -p $filename META-INF/patch-info | $GREP 'FilePath' | $AWK '{print \$3}'`;
      chomp($war_file);
      @list = `for i in \`$UNZIP -l -qq $filename | $GREP -P -v META-INF | $AWK '{print \$4}' \` ; do $ECHO \"$war_file:\$i\" ; done`;
      chomp(@list);
   } ## end elsif ($filename =~ /WAR$/i...
   elsif ($filename =~ /BIN$/i)  #BIN file
   {
      my $bin_file = `$TAR xf $filename --to-stdout patch_file_list.txt | $GREP ORIG | $AWK '{print \$2}' | $SED 's/"//g'`;
      chomp($bin_file);
      @list = $bin_file;
   } ## end elsif ($filename =~ /BIN$/i...
   elsif ($filename =~ /RPM$/i)  #RPM file
   {
      my $pkg_name = `$RPM_COMMAND -qp --nosignature --qf \%{Name}\ $filename`;
      chomp($pkg_name);
      @list = $pkg_name;
   } ## end elsif ($filename =~ /RPM$/i...
   chomp @list;
   return @list;
} ## end sub get_member_elements($)

#####################################################
## @method get_kernel_update ($self, $status)
# @brief Returns kernel updates installed based on status
# @details
# @param $self - reference to patch_db object
# @param $status - status of kernel update (all patches if 'undef')
# @return array of kernel updates on success, otherwise undef
# @date
sub get_kernel_update($$)
{
   my ($self, $status) = @_;
   my @ker_patches = ();
   for (my $i = 0 ; $i < $MAX_PATCHES ; $i++)
   {
      next unless exists $self->{'hash_db'}->{$i};
      my $member = $self->{'hash_db'}->{$i . $PATCH_FIELDS{'MEM_INFO'}};
      if (    ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'MEM_INFO'}} =~ /^.+kernel(-smp)?$/)
          and ($self->{'hash_db'}->{$i . $PATCH_FIELDS{'STATUS'}} eq $status))
      {
         push(@ker_patches, $self->get_patch($i));
      }
   } ## end for (my $i = 0 ; $i < $MAX_PATCHES...
   return @ker_patches;
} ## end sub get_kernel_update($$)

#####################################################
## @method check_functions_conflict ($self, $patch, $quiet)
# @brief   Checks functions conflicts
# @details The conflict checking ensures that the current BIN patch
#          does not update/replace the functions for the specified
#          executable which are changed by other BIN patch
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @param $quiet - $TRUE or $FALSE, shows wether or not to print conflict
#	          patch names on the screen
# @return $TRUE - if the patch is not in conflict with other patches
#	          that are in service, otherwise $FALSE (if exists at least one conflict patch)
# @date
sub check_functions_conflict($$$)
{
   my ($self, $p, $quiet) = @_;
   my @in_conflict;
   my $no_conflicts = $TRUE;

   # if new SU is going to be installed then all the patches for the same rpm have to be removed
   return $TRUE if ($p->{'SUBTYPE'} ne 'BIN');
   return $FALSE if ((not defined $p->{'EXEC'}) or (not defined $p->{'META_DATA'}));

   my @patches = $self->get_rpm_patches($p->{'RPM_NAME'}, $IN_SERV, "PATCH");

   my @bin_patches = ();

   # getting 'BIN' patches only
   foreach my $patch (@patches)
   {
      if ($patch->{'SUBTYPE'} eq 'BIN')
      {
         push @bin_patches, $patch;
      }
   } ## end foreach my $patch (@patches...

   # Search all patches in service
   my @p_libs = @{$p->{'META_DATA'}};
   my @p_exec = @{$p->{'EXEC'}};
   foreach my $patch (@bin_patches)
   {

      # check if different patches do not try to load the same library name into the ORIG_PATCH specified executable memory space
      # there can be duplicated entries
      if (defined $patch->{'META_DATA'})
      {
         foreach my $lib (@{$patch->{'META_DATA'}})
         {
            if (scalar(grep(/$lib/, @p_libs)))
            {
               push @in_conflict, $patch->{'PATCHNAME'};
            }
         } ## end foreach my $lib (@{$patch->...
      } ## end if (defined $patch->{'META_DATA'...
      if (defined $patch->{'EXEC'})
      {

         # loop over all patch executables
         foreach my $executable (@{$patch->{'EXEC'}})
         {
            if (scalar(grep (/^$executable$/, @p_exec)))
            {

               # some patch patched the same executable as current patch. Need to check functions
               # check if new patch does not update/replace the functions indicated by the INCLUDE tags for the specified executable as indicated by the ORIG_PATH
               foreach my $func (@{$patch->{$executable}})
               {
                  if (scalar(grep (/^$func$/, @{$p->{$executable}})))
                  {
                     push @in_conflict, $patch->{'PATCHNAME'};
                     last;
                  }
               } ## end foreach my $func (@{$patch->...
            } ## end if (scalar(grep (/^$executable$/...
         } ## end foreach my $executable (@{$patch...
      } ## end if (defined $patch->{'EXEC'...
   } ## end foreach my $patch (@bin_patches...

   # Patch conflict
   if (scalar(@in_conflict))
   {

      #remove duplicate elements
      my %seen;
      my @temp = @in_conflict;
      @in_conflict = grep(!$seen{$_}++, @temp);
      baselog($LOG_ERROR, "Patch $p->{PATCHNAME} conflicts with " . join(' ', @in_conflict) . " which is/are already installed.", $quiet);

      $no_conflicts = $FALSE;
   } ## end if (scalar(@in_conflict...

   return $no_conflicts;
} ## end sub check_functions_conflict($$$)

#####################################################
## @method check_element_conflict ($self, $patch, $quiet)
# @brief checks patch elements conflict
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @param $quiet - $TRUE or $FALSE, shows wether or not to print conflict
#	patch names on the screen
# @return $TRUE - if the patch is not in conflict with other patches
#	that are in service, otherwise $FALSE (if exists at least one conflict patch)
# @date
sub check_element_conflict($$$)
{
   my ($self, $p, $quiet) = @_;
   my @conflict_patches = ();
   my @temp;
   my $ret = $TRUE;

   my ($descr_elem, $hash_db_elements) = basedb_lock($PATCH_ELEMENTS_DB);
   if (!defined($hash_db_elements))
   {
      baselog($LOG_ERROR, "Can not get information from patch elements DB\n", $TRUE);
      return $FALSE;
   }
   my $pid = $p->{'PID'};
   if (defined($self->{'hash_db'}->{$pid . $PATCH_FIELDS{'META_DATA'}}))
   {
      foreach my $element (split(/\t/, $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'META_DATA'}}))
      {
         if (defined($hash_db_elements->{$element}))  #conflict element is found, store it
         {

            baselog($LOG_ERROR, 'Conflict detected: ' . $element . ' - ' . $hash_db_elements->{$element});
            push(@conflict_patches, $hash_db_elements->{$element});
         } ## end if (defined($hash_db_elements...
      } ## end foreach my $element (split(...
   } ## end if (defined($self->{'hash_db'...
                                                      # for BIN patches also check ORIG_PATH member for conflicts
   if ($p->{'SUBTYPE'} =~ /^BIN$/)
   {
      my @exec = @{$p->{'EXEC'}};
      foreach my $executable (@exec)
      {

         # remove the leading "/"
         $executable =~ s/^\/*//;
         if (exists $hash_db_elements->{$executable})  #conflict element is found, store it
         {
            baselog($LOG_ERROR, 'Conflict detected: ' . $executable . ' - ' . $hash_db_elements->{$executable});
            push(@conflict_patches, $hash_db_elements->{$executable});
         }
      } ## end foreach my $executable (@exec...
   } ## end if ($p->{'SUBTYPE'} =~...

   basedb_unlock($descr_elem, $hash_db_elements);

   # Patch conflict
   if (scalar(@conflict_patches))
   {

      #remove duplicate elements
      my %seen;
      @temp             = @conflict_patches;
      @conflict_patches = grep(!$seen{$_}++, @temp);

      my $error = "\nError: the patch conflicts with the following installed patch(es):\n";
      $error .= "\t$_\n" foreach (@conflict_patches);
      print($error) unless $quiet;
      baselog($LOG_ERROR, "Conflict elements: " . join(' ', @conflict_patches), $quiet);
      $ret = $FALSE;
   } ## end if (scalar(@conflict_patches...
   return $ret;
} ## end sub check_element_conflict($$$)

#####################################################
## @method put_patch_in_service ($self, $patch)
# @brief Puts patch in service
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return none
# @date
sub put_patch_in_service($$)
{
   my ($self, $p) = @_;
   my $time   = time();
   my $status = $IN_SERV;
   $status = $IN_SERV_REBOOT if ($p->{'REBOOT'} =~ m/^yes$/i);
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'STATUS'}} = $status;
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'PINS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   # Update patch info
   $p->{'STATUS'} = $status;
   $p->{'PINS'}   = $time;

} ## end sub put_patch_in_service($$)

#####################################################
## @method put_patch_in_service_byID ($self, $pid)
# @brief Puts patch in service by patch ID
# @details
# @param $self - reference to patch_db object
# @param $pid - patch ID
# @return $TRUE - on success, $FALSE - if record doesn't exist in database
# @date
sub put_patch_in_service_byID($$)
{
   my ($self, $pid) = @_;
   my $time = time();
   unless (exists $self->{'hash_db'}->{$pid})
   {
      baselog($LOG_ERROR, "Patch with pid $pid doesn't exist in database.");
      return $FALSE;
   }
   $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'STATUS'}} = $IN_SERV;
   $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'PINS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   return $TRUE;
} ## end sub put_patch_in_service_byID($$)

#####################################################
## @method put_patch_out_service ($self, $patch)
# @brief Puts patch out of service
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return none
# @date
sub put_patch_out_service($$)
{
   my ($self, $p) = @_;
   my $time   = time();
   my $status = $OUT_SERV;
   $status = $OUT_SERV_REBOOT if ($p->{'REBOOT'} =~ m/^yes$/i);
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'STATUS'}} = $status;
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'POOS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   # Update patch info
   $p->{'STATUS'} = $status;
   $p->{'POOS'}   = $time;

} ## end sub put_patch_out_service($$)

#####################################################
## @method put_patch_out_service_byID ($self, $pid)
# @brief Puts patch out of service by patch ID
# @details
# @param $self - reference to patch_db object
# @param $pid - patch ID
# @return $TRUE - on success, $FALSE - if record doesn't exist in database
# @date
sub put_patch_out_service_byID($$)
{
   my ($self, $pid) = @_;
   my $time = time();
   unless (exists $self->{'hash_db'}->{$pid})
   {
      baselog($LOG_ERROR, "Patch with pid $pid doesn't exist in database.");
      return $FALSE;
   }
   $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'STATUS'}} = $OUT_SERV;
   $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'POOS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   return $TRUE;
} ## end sub put_patch_out_service_byID($$)

#####################################################
## @method set_status ($self, $patch, $status)
# @brief Sets patch status
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @param $status - patch status
# @return none
# @date
sub set_status($$$)
{
   my ($self, $p, $status) = @_;
   if ($status == $IN_SERV)
   {
      return $self->put_patch_in_service($p);
   }
   elsif ($status == $OUT_SERV)
   {
      return $self->put_patch_out_service($p);
   }
   else
   {
      $self->set_status_byID($p->{'PID'}, $status);
      $p->{'STATUS'} = $status;
   }

   return;
} ## end sub set_status($$$)

#####################################################
## @method set_status_byID ($self, $patch, $status)
# @brief Sets patch status by patch ID
# @details
# @param $self - reference to patch_db object
# @param $pid - patch ID
# @param $status - patch status
# @return none
# @date
sub set_status_byID($$$)
{
   my ($self, $pid, $status) = @_;
   $self->{'hash_db'}->{$pid . $PATCH_FIELDS{'STATUS'}} = $status;

   basedb_sync($self->{'hash_db'});

} ## end sub set_status_byID($$$)

#####################################################
## @method add_element_record ($self, $patch)
# @brief Adds record into the patch element DB
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return $TRUE - on success, $FALSE - if error occurred
# @date
sub add_element_record($$)
{
   my ($self, $p) = @_;

   my ($descr_elem, $hash_db_elements) = basedb_lock($PATCH_ELEMENTS_DB);
   if (!defined($hash_db_elements))
   {
      baselog($LOG_ERROR, "Cannot update elements DB", $TRUE);
      return $FALSE;
   }

   # Write info into element DB
   my @files = @{$p->{'META_DATA'}};
   foreach my $file (@files)
   {

      # remove the leading "/"
      $file =~ s/^\/*//;
      $hash_db_elements->{$file} = $p->{'PATCHNAME'};
      baselog($LOG_DEBUG, 'Adding element: ' . $file . ' - ' . $hash_db_elements->{$file});
   } ## end foreach my $file (@files)
   if ($p->{'SUBTYPE'} =~ 'BIN')
   {
      my @exec = @{$p->{'EXEC'}};
      foreach my $executable (@exec)
      {

         # remove the leading "/"
         $executable =~ s/^\/*//;
         if (exists $hash_db_elements->{$executable})
         {
            if ($hash_db_elements->{$executable} !~ /$p->{'PATCHNAME'}/)
            {
               $hash_db_elements->{$executable} .= "\t" . $p->{'PATCHNAME'};
               baselog($LOG_DEBUG, 'Changing element: ' . $executable . ' - ' . $hash_db_elements->{$executable});
            }
         } ## end if (exists $hash_db_elements...
         else
         {
            $hash_db_elements->{$executable} = $p->{'PATCHNAME'};
            baselog($LOG_DEBUG, 'Adding element: ' . $executable . ' - ' . $hash_db_elements->{$executable});
         }
      } ## end foreach my $executable (@exec...
   } ## end if ($p->{'SUBTYPE'} =~...

   basedb_unlock($descr_elem, $hash_db_elements);
   return $TRUE;
} ## end sub add_element_record($$)

#####################################################
## @method remove_element_record ($self, $patch)
# @brief Removes patch elements from DB
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return $TRUE - on success, $FALSE - if error occurred
# @date
sub remove_element_record($$)
{
   my ($self, $p) = @_;

   my ($descr_elem, $hash_db_elements) = basedb_lock($PATCH_ELEMENTS_DB);
   if (!defined($hash_db_elements))
   {
      baselog($LOG_ERROR, "Cannot update elements DB", $TRUE);
      return $FALSE;
   }

   my @files = @{$p->{'META_DATA'}};
   foreach my $file (@files)
   {

      # remove the leading "/"
      $file =~ s/^\/*//;
      if (exists $hash_db_elements->{$file})
      {
         baselog($LOG_INFO, 'Removing element: ' . $file . ' - ' . $hash_db_elements->{$file});
         delete($hash_db_elements->{$file});
      }
   } ## end foreach my $file (@files)
   if ($p->{'SUBTYPE'} =~ /^bin$/i)
   {
      my @exec        = @{$p->{'EXEC'}};
      my @patch_names = ();
      foreach my $executable (@exec)
      {

         # removing the leading "/"
         $executable =~ s/^\/*//;
         if (exists $hash_db_elements->{$executable})
         {

            @patch_names = split("\t", $hash_db_elements->{$executable});
            for (my $n = 0 ; $n < scalar(@patch_names) ; $n++)
            {
               if ($patch_names[$n] eq $p->{'PATCHNAME'})
               {
                  splice(@patch_names, $n, 1);
                  last;
               }
            } ## end for (my $n = 0 ; $n < scalar...
            if (not scalar(@patch_names))
            {
               baselog($LOG_INFO, 'Removing element: ' . $executable . ' - ' . $hash_db_elements->{$executable});
               delete($hash_db_elements->{$executable});
            }
            else
            {
               $hash_db_elements->{$executable} = join("\t", @patch_names);
            }
         } ## end if (exists $hash_db_elements...
      } ## end foreach my $executable (@exec...
   } ## end if ($p->{'SUBTYPE'} =~...

   basedb_unlock($descr_elem, $hash_db_elements);
   return $TRUE;
} ## end sub remove_element_record($$)

#####################################################
## @method check_db_integrity
# @brief Check the patch database integrity
# @details Check each patch in the db, if it is broken or cannot be created;
#          prints the errors statistics if present
# @param $self   - reference to patch_db object
# @return $TRUE  - database integrity check passed
# @return $FALSE - otherwise
# @author Vladimir Ivanov
# @date March 2008
# @note
sub check_db_integrity($)
{
   my $self = shift @_;

   my $retval = $TRUE;
   my $errmsg = "";
   my @failed;

   foreach my $patch ($self->get_rpm_patches(undef, $ALL_PATCHES))
   {
      unless (defined $patch)
      {
         $errmsg = "Error: Some patches failed to be created\n\n";
      }
      else
      {
         if ($patch->is_broken())
         {
            $retval = $FALSE;
            push(@failed, $patch);
         }
      } ## end else
   } ## end foreach my $patch ($self->get_rpm_patches...

   if (scalar @failed)
   {
      $errmsg = $errmsg . "Error: The following patches are detected to be corrupted or missing:\n";
      foreach my $patch (@failed)
      {
         $errmsg = $errmsg . patchutil::patch::get_full_patch_file($patch) . "\n";
      }

      $errmsg = $errmsg . "\nPlease reload them from the Nortel Patch Library.\n";
   } ## end if (scalar @failed)

   print $errmsg unless ($retval);
   return $retval;
} ## end sub check_db_integrity($)

#####################################################
## @method put_patch_in_service_after_reboot ($self, $patch)
# @brief Puts patch in service
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return none
# @date
sub put_patch_in_service_after_reboot($$)
{
   my ($self, $p) = @_;
   my $time   = time();
   my $status = $IN_SERV;
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'STATUS'}} = $status;
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'PINS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   # Update patch info
   $p->{'STATUS'} = $status;
   $p->{'PINS'}   = $time;
} ## end sub put_patch_in_service_after_reboot($$)

#####################################################
## @method put_patch_out_service_after_reboot ($self, $patch)
# @brief Puts patch out of service
# @details
# @param $self - reference to patch_db object
# @param $patch - reference to patch object
# @return none
# @date
sub put_patch_out_service_after_reboot($$)
{
   my ($self, $p) = @_;
   my $time   = time();
   my $status = $OUT_SERV;

   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'STATUS'}} = $status;
   $self->{'hash_db'}->{$p->{'PID'} . $PATCH_FIELDS{'POOS'}}   = $time;

   basedb_sync($self->{'hash_db'});

   # Update patch info
   $p->{'STATUS'} = $status;
   $p->{'POOS'}   = $time;
} ## end sub put_patch_out_service_after_reboot($$)

1;

