## @file patches_public.pm
# @brief this file contains common variables and functions for patching subsystem.
# @details none
# @author
# @date
# @version 1.00 - Initial version
# @version 1.10 - Actions related with TPM_DIR initialize and temporaty directory 
#                 creation were moved into create_patch_dir function. Code from END 
#                 section(cleanup temp dir) was moved into remove_patch_dir function.   
# @note

package patchutil::patches_public;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use common_variables;


use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&is_spec_ins &check_body &is_match_type &remove_patch_dir &create_patch_dir
  $PATCHDIR $PREPATCH $POSTPATCH $PATCHINFO_FILE $OS_RELEASE $FIRST_VER $README $TMP_DIR $MPLR_MAX  
  $TYPE_MAX $GENERIC_PATCHNAME_MAX $SU_PATCHNAME_MAX $GENERIC_MAX $RLS_MAX
  $ISS_MAX $AUTHOR_MAX $CR_MAX $MACHINE_MAX $QTY_MAX $SUBTYPE_MAX $FILENAME_MAX $APPS_MAX
  $MD5_MAX $HEADER_COUNT_STRING $ELEMENT_COUNT_STRING %PATCHTYPE $TYPE_WAR $TYPE_JAR
  $TYPE_WAR $TYPE_JAR $TYPE_FRU $TYPE_BIN
  );

our $VERSION = 1.10;


###########
#public data for patch management
###########
our $PATCHDIR       = '/var/opt/nortel/patch';
our $PREPATCH       = 'prePatch.pl';           #Pre patch script filename
our $POSTPATCH      = 'postPatch.pl';          #Post patch script filename
our $PATCHINFO_FILE = "header.xml";             #filename with patch information
our $OS_RELEASE     = "el5";
our $FIRST_VER      = "2.6.9-34.EL";  #first installed kernel version
our $README         = "readme";       #file name with path user patch information
our $TMP_DIR        = "/tmp/patch.$$/";  

#supported types for JAR and WAR patches (*.war and *.jar)
our $TYPE_WAR = "war";
our $TYPE_JAR = "jar";
our $TYPE_FRU = "fru";
our $TYPE_BIN = "bin";

###########
#patch header fields limitation
###########
our $MPLR_MAX              = 5;
our $TYPE_MAX              = 10;
our $GENERIC_PATCHNAME_MAX = 10;   # pXXXXXX_XX - 10 characters
our $SU_PATCHNAME_MAX      = 50;
our $GENERIC_MAX           = 20;
our $RLS_MAX               = 10;
our $AUTHOR_MAX            = 30;
our $CR_MAX                = 12;
our $MACHINE_MAX           = 3;
our $QTY_MAX               = 3;
our $SUBTYPE_MAX           = 3;
our $FILENAME_MAX          = 256;
our $APPS_MAX              = 1024;
our $MD5_MAX               = 128;

our $HEADER_COUNT_STRING  = 11;
our $ELEMENT_COUNT_STRING = 4;

our %PATCHTYPE = (
                  PATCH => "PATCH",
                  SU    => "SU"
                 );

sub create_patch_dir();
sub remove_patch_dir();

##################################################
# Checks if patch includes special instructions
# INPUT: patch file name
# OUTPUT: TRUE - includes
#         FALSE - otherwise
##################################################
sub is_spec_ins(@)
{
   my $filename = shift @_;

   system("$TAR tzfO $PATCHDIR/$filename $README 2>/dev/null");
   if ($? eq 0)
   {
      return $TRUE;
   }
   return $FALSE;
} ## end sub is_spec_ins(@)

##################################################
# Checks type of patch is in PATCHTYPE hash
# Input
#    arg1 - type of patch
# Output TRUE - match
#        FLALSE - unmatch
##################################################
sub is_match_type(@)
{
   my $word = shift @_;

   foreach (keys(%PATCHTYPE))
   {
      if ($word eq $PATCHTYPE{$_})
      {
         return $TRUE;
      }
   } ## end foreach (keys(%PATCHTYPE))
   return $FALSE;
} ## end sub is_match_type(@)

##################################################
# Checks type of patch body
# Input
#    $filename - patch body filename
#    $type - type of patch body (RPM,JAR,WAR,BIN,MOD,FRU)
# Output
#    0 ($FALSE) - unknown or broken patch body
#    1 ($TRUE)  - correct
##################################################
sub check_body($$)
{
   my ($filename, $type, $type_patch);
   $filename = $_[0];
   $type     = $_[1];

   $type_patch = `file $filename | /bin/awk \'\{print \$2\}\'`;
   chomp($type_patch);

   #print "$type_patch,$type\n";
   if (($type_patch =~ /RPM/) && ($type_patch eq $type))  #RPM
   {
      return $TRUE;
   }
   elsif (   ($type_patch =~ /Zip/)
          && (($type eq 'JAR') || ($type eq 'WAR')))      #ZIP(JAR/WAR)type.
   {
      return $TRUE;
   }
   elsif (($type_patch =~ /ELF/) && ($type eq 'MOD'))     #kernel module
   {
      return $TRUE;
   }
   elsif (($type_patch =~ /POSIX/) && ($type eq 'BIN'))   #binary patch
   {
      return $TRUE;
   }
   elsif (($type_patch =~ /POSIX/) && ($type eq 'FRU'))   # file-replacement patch
   {
      return $TRUE;
   }
   else
   {
      return $FALSE;                                      #unknown or broken patch body
   }
} ## end sub check_body($$)

##################################################
# @fn create_patch_dir()
# @brief    create work directory for patch manipulating
# @details  to initialize TMP_DIR variable.
# @param    none
# @return   $TRUE  - success
# @return   $FALSE - directory not created.
##################################################
sub create_patch_dir()
{
   #cleanup
   unless (remove_patch_dir())
   {
      return $FALSE;
   }
   
   #create new
   if (system($MKDIR, $TMP_DIR)>>8 != 0)
   {
      undef $TMP_DIR;
      return $FALSE;
   }
   return $TRUE;
}

##################################################
# @fn remove_patch_dir()
# @brief    cleanup patching staff 
# @details  
# @param    none
# @return   $TRUE  - success
# @return   $FALSE - directory cannot be cleanuped
##################################################
sub remove_patch_dir()
{
   if(-e $TMP_DIR)
   {   
      if (system($RM, "-rf", $TMP_DIR)>>8 !=0)
      {
         undef $TMP_DIR;
         return $FALSE;                        
      }
   }
   return $TRUE;
}

1;
__END__

=head1 NAME

patches_public - common data for all patch management tools.

=head1 SYNOPSIS

use patches_public;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut

