##############################################################################
## @file   plugin_mgr.pm
# @brief   the Plug-in manager class module
# @details This module provides the class Plug-in manager. The Plug-in Manager
#          allows using the concept of plug-in modules. So, new functionality
#          can be increased by adding of new plug-ins (separate module in the
#          file system) without of changing existent code.
# @author  Ekaterina Bazhanova
# @author  Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @version 1.02 - fixed a bug with uninitialized 'visible' attribute

package plugin_mgr;

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use common_functions;

### Access management
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw($VERSION);

use baselog;

our $VERSION = "1.02";

## @skipline 
sub find_modules($$$);
## @skipline
sub get_all_loaded_packages($$);
## @skipline
sub new($$;$);
## @skipline
sub load_module($$);
## @skipline
sub get_modules($);
## @skipline
sub module_exists($$);
## @skipline
sub module_loaded($$);
## @skipline
sub get_fullname_of_package($$);
## @skipline
sub get_all_actions_for_module($$);
## @skipline
sub get_action($$$;$);
## @skipline
sub get_default_action($$$);
## @skipline
sub load_module($$);
## @skipline
sub do_action($$);
## @skipline
sub try_action($$);


##############################################################################
############################## Private Methods ###############################

###############################################################################
## @fn sub  find_modules($path, $extension, $modules_list)
# @brief   finds modules in specified directory
# @details This function gets modules with specified extension from the directory 
# @param   $path         - full path of the directory
# @param   $extension    - extension of the functional modules
# @param   $modules_list - reference where the list of the functional modules
#                          will be stored
# @return  TRUE  - the list of the modules is prepared sucessfully
# @return  FALSE - some error happened
sub find_modules($$$)
{
   my ($path, $extension, $modules_list) = @_;
   
   baselog($LOG_DEBUG, "Try to look for '$extension' modules in '$path'");
   
   my $retval = $TRUE; 
   
   my $dir;
   if (opendir($dir, $path))
   {
      
      # Get the list of all file objects in the directory
      my @dir_list = readdir($dir);
      if (scalar @dir_list)
      {
         #look through the found file objects
         foreach my $module (@dir_list)
         {
            my $fmname = "$path/$module";
            # the module must be a plain file or a symlink
            # the module must end by the extenstion
            if (((-f $fmname) or (-l $fmname)) and ($module =~ /(.*)$extension$/))
            {
               push @$modules_list, $1;
            }
         }

         baselog($LOG_DEBUG, "The list of modules: '@$modules_list'");
      }
      else
      {
         baselog($LOG_NOTICE, "No modules found in the directory '$path'");
      }
   }
   else
   {
      baselog($LOG_ERROR, "Cannot open '$path' directory: $!");
      $retval = $FALSE;
   }
   
   return $retval;
}

###############################################################################
## @fn sub get_all_loaded_packages($cur_package, $ref_arr)
# @brief   Provides list of all loaded packages
# @details This method implement the approach described in "Advanced Perl
#          Programming", 2nd edition by Simon Cozens, Publisher: O'Reilly
#          (Section 1.3) in order to get the information about all used (loaded)
#          packages. This approach looks through the symbol table (just a hash)
#          and finds glob names by looking at the keys of the hash.
# @param   $cur_package - the name of the current package - for very first
#                         iteration it must be "main".
# @param   $ref_arr     - reference to array where the packages name will
#                         be accumulated.
# @return  None
sub get_all_loaded_packages($$)
{
   no strict 'refs';

   my ($cur_package, $ref_arr) = @_;
   
   my %symbols_of_package = %{$cur_package."::"};
   my @symbs = grep s/::$//, keys %symbols_of_package;
   if ($cur_package ne "main")
   {
      my $tmp_pack = $cur_package;
      $tmp_pack =~ s/^main:://;
      push @{$ref_arr}, $tmp_pack;
   }
   foreach my $symbol (@symbs)
   {
      if (($cur_package ne "main") or ($symbol ne "main"))
      {
         get_all_loaded_packages($cur_package."::".$symbol, $ref_arr);
      }
   }
}

##############################################################################
############################## Public Methods ################################

###############################################################################
## @cmethod sub new ($class, $modules_dir, $extension)
# @brief   Plug-in manager constructor
# @details This class method creates an instance of the class and looks for
#          accessible modules (plug-ins).
# @param   $class       - name of objects class
# @param   $modules_dir - full path of the functional modules directory
# @param   $extension   - extension of the functional modules. It is the
#                         optional parameter. ".pm" is default value.
# @return  $self - reference to object
# @return  undef - if some error happened
sub new($$;$)
{
   my ($class, $modules_dir, $extension)  = @_;
   if (not $extension)
   {
      $extension = '.pm';
   }
      
   my $self        = {};
   
   baselog($LOG_DEBUG, "Instance of Plug-in manager is being created.");
   
   if (not $modules_dir)
   {
      baselog($LOG_ERROR, "The modules directory is not specified.");
      return undef;
   }
   
   my @modules_list  = ();
   if (not find_modules($modules_dir, $extension, \@modules_list))
   {
      return undef;
   }

   $self->{'directory'} = $modules_dir;
   $self->{'extension'} = $extension;
   
   $self->{'modules'} = {};
   foreach my $module (@modules_list)
   {
      $self->{'modules'}->{$module} = {'package'=>"", 'loaded'=>$FALSE};
   }

   bless ($self, $class);
   
   return $self;    
} ##end sub new($;$)

###############################################################################
## @cmethod sub DESTROY($self)
# @brief   Plug-in manager destructor
# @details This methods destroys an instance of the Plug-in manager class. 
# @param   $self - reference to object
# @return  None
sub DESTROY
{
   baselog($LOG_DEBUG, "Destroing the instanse of Plug-in manager");

   return; 
} ##end sub DESTROY

###############################################################################
## @method sub get_fullname_of_package($self, $module)
# @brief   Provides full name of the module
# @details This method tries to determine the full name of the package by using
#          only its last component (e.g. "hardning::modules::ssh_filter" by
#          "ssh_filter"). Only packages intended to be a plug-in are checked -
#          the exported method "actions" is being detected.
# @param   $self   - reference to object
# @param   $module - the last component in the package name
# @return  $retval - the full name of the package. It is empty if the module is
#                    not loaded or is not recognized as plug-in.
sub get_fullname_of_package($$)
{
   my ($self, $module) = @_;

   my $retval = "";
   
   baselog($LOG_DEBUG, "Try to get full name of the module '$module'.");
   
   if (not $self->module_loaded($module))
   {
   
      my @loaded_packages = ();
      get_all_loaded_packages("main", \@loaded_packages);
      
      foreach my $loaded_package (@loaded_packages)
      {
         if ($loaded_package =~ /::$module$/)
         {
            baselog($LOG_DEBUG, "Check the package '$loaded_package' to be a plug-in.");

            #"can" checks to see if its object has a method called "METHOD", if it does
            #then a reference to the sub is returned, if it does not then undef is
            #returned.
            #Object should be the full name of the package. 
            my $is_actions_provided = $loaded_package->can("actions");
      
            if (not defined $is_actions_provided)
            {
               baselog($LOG_INFO, "The package '$loaded_package' does not seem to be a plug-in with supportable format");
            }
            else
            {
               baselog($LOG_DEBUG, "The package '$loaded_package' is recognized as plug-in.");
               
               $self->{'modules'}->{$module}->{'package'} = $loaded_package;
               $retval = $loaded_package;
               last;
            }
         }
      }
   }
   else
   {
      $retval = $self->{'modules'}->{$module}->{'package'};
   }
   
   if ($retval eq "")
   {
      baselog($LOG_ERROR, "Cannot get fullname of the module '$module'.");
   }

   return $retval;
}

###############################################################################
## @method sub load_module ($self, $ref_module)
# @brief   Load the appropriate module
# @details The method uses 'require' perl-operation in order to load the module
# @param   $self - reference to object
# @param   $ref_module - reference to a string with name of functional module.
#          The name of the corresponding package will be returned in this
#          parameter
# @return  TRUE - the module was loaded. $ref_module contain a reference to
#                 a string with the name of the corresponding package.
# @return  FALSE - the module was not loaded
sub load_module($$)
{
   my ($self, $ref_module) = @_;

   my $retval = $FALSE;
   my $module = ${$ref_module};
   
   baselog($LOG_DEBUG, "Try to load the module '$module'");
   
   if (not $self->module_exists($module))
   {
      return $retval;
   }

   if (not $self->{'modules'}->{$module}->{'loaded'})
   {

      my $module_full_path = $self->{'directory'}."/".$module.$self->{'extension'};
      
      baselog($LOG_DEBUG, qq{Try to 'require "$module_full_path"'});
      eval qq{require "$module_full_path"};

      if ($@)
      {
         baselog($LOG_ERROR, "Cannot load '$module' from directory \'$self->{directory}\': $@");
      }
      else
      {
         baselog($LOG_DEBUG, "The module is loaded, but not checked - not marked as loaded.");

         my $package_name = $self->get_fullname_of_package($module);
         if ($package_name eq "")
         {
            baselog($LOG_ERROR, "The module '$module' does not seem to be a plug-in with supportable format");
         }
         else
         {
            baselog($LOG_DEBUG, "The module is loaded and marked as loaded.");
            
            $self->{'modules'}->{$module}->{'loaded'} = $TRUE;
            ${$ref_module} = $package_name;
            $retval = $TRUE;
         }
      }
   }
   else
   {
      ${$ref_module} = $self->get_fullname_of_package($module);
      $retval = $TRUE;
   }

   return $retval;
} ##end sub load_module($$)

###############################################################################
## @method sub get_modules($self)
# @brief   Provides the list of avilable modules
# @details This method returns value of the 'modules' field of the object
# @param   $self - reference to object
# @return  Value of the objec field - 'modules'
sub get_modules($)
{
   my $self = shift @_;
   
   #One of the initial requirements to have the list of found modules sorted
   my @tmp_list = sort(keys %{$self->{'modules'}});
   
   return @tmp_list;
}

###############################################################################
## @method sub module_exists($self, $module)
# @brief   Checks if the module is available
# @details This function looks for the module in the list of available modules 
# @param   $self - reference to object
# @param   $module - the name for the the sought-for module
# @return  TRUE  - the module is accessible
# @return  FALSE - there is no such module in the list
sub module_exists($$)
{
   my ($self, $module) = @_;
   
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Is the module '$module' available?");
   
   if (exists $self->{'modules'}->{$module})
   {
      baselog($LOG_DEBUG, "The module '$module' is found.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_ERROR, "The module '$module' is not found.");
   }

   return $retval;
}

###############################################################################
## @method sub module_loaded($self, $module)
# @brief   Checks if the module is already loaded
# @details This function checks the attribute 'loaded' for the module 
# @param   $self - reference to object
# @param   $module - the module name
# @return  TRUE  - the module is loaded
# @return  FALSE - the module is no loaded
sub module_loaded($$)
{
   my ($self, $module) = @_;
   
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Is the module '$module' already loaded?");

   if (not $self->module_exists($module))
   {
      return $retval;
   }

   if ($self->{'modules'}->{$module}->{'loaded'})
   {
      baselog($LOG_DEBUG, "The module '$module' is marked as loaded.");
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_DEBUG, "The module '$module' is not marked as loaded.");
   }

   return $retval;
}

###############################################################################
## @method sub get_all_actions_for_module($self, $module)
# @brief   Provaides actions of the given module
# @details This method loads the appropriate functional module and gets a hash
#          of exported actions for this module.
# @param   $self - reference to object
# @param   $module -  name of functional module
# @return  %hash - hash of exported actions for the module. If the hash is
#          undefined, some error happened
sub get_all_actions_for_module($$)
{
   my ($self, $module) = @_;
   
   my %hash = ();   

   baselog($LOG_DEBUG, "Try to get all actions provided by the module '$module'");
   
   #After successful invokation of load_module, $module will contain the full
   #name of the corresponding package.
   if (load_module($self, \$module))
   {
      #Get the hash of exported actions
      %hash = $module->actions();
   }
   
   return %hash;
}

###############################################################################
## @method sub get_action ($self, $module, $action, $options)
# @brief   Gets a hash describing the given action for the given module
# @details This method gets all the accessible actions for the module and looks
#          for the needed. If the action is found it returns the hash
#          describing this action.
# @param   $self - reference to object
# @param   $module - name of functional module
# @param   $action - sought-for action
# @param   $options - (optional) reference to the hash with options:
#                     \li \em silent - to not produce an error if the module
#                                      does not provide the action (e.g. for
#                                      try_module)
# @return  %act - hash describing the action. If the hash is undefined, some
#          error happened
sub get_action($$$;$)
{
   my ($self, $module, $action, $options) = @_;

   my $silent = $FALSE;

   if (defined $options)
   {
      if ((defined $options->{'silent'}) and ($options->{'silent'}))
      {
         baselog($LOG_DEBUG, "The option 'silent' is set to TRUE.");
         
         $silent = $TRUE;
      }
   }

   my %act = ();

   baselog($LOG_DEBUG, "Try to look for the action '$action' in the module '$module'");
   
   #Get the hash of actions for the module
   my %actions = $self->get_all_actions_for_module($module);
   if (not %actions)
   {
      baselog($LOG_ERROR, "The module '$module' cannot be used.");
      return %act;
   }
   
   #Check if the action is exported by the module
   my $action_exists = $FALSE;
   foreach my $action_from_list (keys %actions)
   {
      if ($action eq $action_from_list)
      {
         $action_exists = $TRUE;
         last;
      }
   }
   if (not $action_exists)
   {
       if (not $silent)
       {
          baselog($LOG_ERROR, "The module '$module' does not provide the action '$action'.");
       }
       else
       {
          baselog($LOG_INFO, "The module '$module' does not provide the action '$action'.");
       }
       return %act;
   }

   baselog($LOG_DEBUG, "The action '$action' is provided by the module '$module'");
   
   %act = %{$actions{$action}};
   return %act;
}

###############################################################################
## @method sub get_default_action ($self, $module, $ref_action)
# @brief   Gets a hash describing default action of the given module
# @details This method gets all the accessible actions for the module and looks
#          for the default action. If the action is found it returns the hash
#          describing this action.
# @param   $self - reference to object
# @param   $module - name of functional module
# @param   $ref_action - reference to the action where the name of default actions
#                        will be stored
# @return  %act - hash describing the action. If the hash is undefined, some
#          error happened
sub get_default_action($$$)
{
   my ($self, $module, $ref_action) = @_;

   my %act = ();
   
   baselog($LOG_DEBUG, "Try to look for default action in the module '$module'");
   
   #Get the hash of actions for the module
   my %actions = $self->get_all_actions_for_module($module);
   if (not %actions)
   {
      baselog($LOG_ERROR, "The module '$module' cannot be used.");
      return %act;
   }
   
   #Look for default action
   my $default_action = "";
   foreach my $action_from_list (keys %actions)
   {
      if ((exists $actions{$action_from_list}{'default'}) and ($actions{$action_from_list}{'default'} == $TRUE))
      {
         $default_action = $action_from_list;
         #Extract only the first default action
         last;
      }
   }
   if ($default_action eq "")
   {
       baselog($LOG_ERROR, "The module '$module' does not provide any default action.");
       return %act;
   }

   baselog($LOG_DEBUG, "The action '$default_action' is provided by the module '$module' as default.");
   
   %act = %{$actions{$default_action}};
   ${$ref_action} = $default_action;
   return %act;
}

###############################################################################
## @method sub do_action($self, $hash_ref)
# @brief   Performs the action for module(s)
# @details This method performs the specified action for the module(s). This
#          method is intended to be used from CLI commands, that is why it
#          provides the checking if the action is accessible for users or not.
# @param   $self - reference to object
# @param   $hash_ref - reference to the hash containing:
#              \li \em $module  - name of the module (plug-in). If it is
#                                 undefined the action will be perfromed for all
#                                 accessible modules
#              \li \em $action  - action. If it is undefined, the default
#                                 action will be performed. If several modules
#                                 are asked and one of them does not support
#                                 the action, the module is skipped.
#              \li \em \@params  - extra parameters needed for the action
#              \li \em $err_msg - the error message which will be retuned by
#                                 the manager 
# @return  TRUE  - the action performed successfully
# @return  FALSE - some error happened, $err_msg can be used for getting more
#                  info about error 
sub do_action($$)
{ 
   my ($self, $hash_ref) = @_;

   my $retval = $FALSE;

   #List of modules for which perfrom the action
   my @list_of_modules = ();

   #Check if it is necessary to perform actions only for one module
   if (defined $hash_ref->{'module'})
   {
      my $module = $hash_ref->{'module'};
      
      if (not $self->module_exists($module))
      {
         $hash_ref->{'err_msg'} = "ERROR: The module '$module' is not found."; 
         return $retval;
      }

      #do action only for one module 
      @list_of_modules = ($hash_ref->{'module'});
   }
   else
   {
      #do the action for all modules
      @list_of_modules = $self->get_modules()
   }

   baselog($LOG_DEBUG, "Try to do action for '@list_of_modules'");
   
   $retval = $TRUE;
   foreach my $module (@list_of_modules)
   {
      my $action = "";
      my %act_hash = ();

      my $is_default = $FALSE;
      
      #Check if the action is specified.
      if (defined $hash_ref->{'action'})
      {
         $action = $hash_ref->{'action'};
         
         %act_hash = $self->get_action($module, $action);
         if (not %act_hash)
         {
            #only last message will be accounted
            $hash_ref->{'err_msg'} = "ERROR: The action '$action' cannot be used with the module '$module'.";
            #accumulate all falses
            $retval &= $FALSE;
            next;            
         }
      }
      else
      {
         #Try to use default action

         %act_hash = $self->get_default_action($module, \$action);
         if (not %act_hash)
         {
            #only last message will be accounted
            $hash_ref->{'err_msg'} = "ERROR: There is no the default action provided by the module '$module'.";
            #accumulate all falses
            $retval &= $FALSE;
            next;            
         }

         $is_default = $TRUE;
      }
       
      #Check if the actions can be accessed through CLI
      #If the action is invisible but default - allow execution (e.g. harden
      #status - will invoke a special action which is not visible for the user
      #but should be invoked)
      #if 'visible' attribute is not defined - the action is considered as
      #invisible
      if (((not defined $act_hash{'visible'}) or ($act_hash{'visible'} == $FALSE)) and (not $is_default))
      {
         #only last message will be accounted
         $hash_ref->{'err_msg'} = "ERROR: The module '$module' does not allow using the action '$action'.";
         #but all the fails will be accounted in the log file.
         baselog($LOG_ERROR, "Cannot access to the action '$action'.");
         #accumulate all falses
         $retval &= $FALSE;
         next;
      }       
      
      #perform parameters validation
      if (exists $hash_ref->{'params'})
      {
         if (defined $act_hash{'validate_params'})
         {
            baselog($LOG_DEBUG, "Try to validate parameters.");
            
            if (not invoke_subroutine_safely($act_hash{'validate_params'}, $hash_ref->{'params'}))
            {
               $hash_ref->{'err_msg'} = "ERROR: Invalid parameter is provided with the action '$action'.";
               baselog($LOG_ERROR, "Invalid parameter is provided with the action '$action'.");  
               $retval &= $FALSE;
               next;
            }

            baselog($LOG_DEBUG, "Validation is sucessful");
         }
         elsif (scalar @{$hash_ref->{'params'}})
         {   
            #If parameters exist but there is no subroutine for their validation -
            #the action needs no parameters.
            $hash_ref->{'err_msg'} = "ERROR: Extra parameters are unnecessary for the action '$action'.";
            baselog($LOG_ERROR, "Parameters are provided but unnecessary for the action '$action'.");  
            $retval &= $FALSE;
            next;
         }
         my $ret = invoke_subroutine_safely($act_hash{'sub'}, $hash_ref->{'params'});
         baselog($LOG_DEBUG, "The action '$action' returns '$ret'.");

         $retval &= $ret;
      }
      else
      {
         if (defined $act_hash{'validate_params'})
         {
            $hash_ref->{'err_msg'} = "ERROR: No parameters specified for the action '$action'.";
            baselog($LOG_ERROR, "No parameters specified for the action '$action'.");  
            $retval &= $FALSE;
            next;
         }
         my $ret = invoke_subroutine_safely($act_hash{'sub'});
         baselog($LOG_DEBUG, "The action '$action' returns '$ret'.");
         
         $retval &= $ret;
      }
   }

   return $retval;
} 

###############################################################################
## @method sub try_action($self, $hash_ref)
# @brief   Tries to performs the action for module(s) gently
# @details This method performs the specified action for the module(s). The
#          actions parameter is mandatory. This method is intended to be used
#          from other subsystems/modules. It does not return an error message
#          for invoked code.
# @param   $self - reference to object
# @param   $hash_ref - reference to the hash containing:
#              \li \em $module  - name of the module (plug-in). If it is
#                                 undefined the ation will be perfromed for all
#                                 accessible modules
#              \li \em $action  - action. If it is undefined, the method
#                                 returns FALSE and exit. If several modules
#                                 are asked and one of them does not support
#                                 the action, the module is skipped.
#              \li \em \@params  - extra parameters needed for the action
# @return  TRUE  - the action performed successfully
# @return  FALSE - some error happened 
sub try_action($$)
{ 
   my ($self, $hash_ref) = @_;

   my $retval = $FALSE;

   #Check if the action is specified.
   if (not defined $hash_ref->{'action'})
   {
      baselog($LOG_ERROR, "The action is not provided.");
      return $retval;
   }

   #List of modules for which perfrom the action
   my @list_of_modules = ();

   #Check if it is necessary to perform actions only for one module
   if (defined $hash_ref->{'module'})
   {
      my $module = $hash_ref->{'module'};
      
      if (not $self->module_exists($module))
      {
         return $retval;
      }

      #do action only for one module 
      @list_of_modules = ($hash_ref->{'module'});
   }
   else
   {
      #do the action for all modules
      @list_of_modules = $self->get_modules()
   }

   baselog($LOG_DEBUG, "Try action for '@list_of_modules'");
   
   $retval = $TRUE;
   foreach my $module (@list_of_modules)
   {
      my $action = $hash_ref->{'action'};
      my %act_hash = ();

      #If some module does not provide the action it is not considered as
      #false, the module just skipped
      %act_hash = $self->get_action($module, $action, {'silent' => $TRUE});
      if (not %act_hash)
      {
         next;            
      }
    
      #perform parameters validation
      if (exists $hash_ref->{'params'})
      {
         if (defined $act_hash{'validate_params'})
         {
            baselog($LOG_DEBUG, "Try to validate parameters.");
            
            if (not invoke_subroutine_safely($act_hash{'validate_params'}, $hash_ref->{'params'}))
            {
               baselog($LOG_ERROR, "Invalid parameter is provided with the action '$action'.");  
               $retval &= $FALSE;
               next;
            }

            baselog($LOG_DEBUG, "Validation is sucessful");
         }
         elsif (scalar @{$hash_ref->{'params'}})
         {   
            #If parameters exist but there is no subroutine for their validation -
            #the action needs no parameters.
            baselog($LOG_ERROR, "Parameters are provided but unnecessary for the action '$action'.");  
            $retval &= $FALSE;
            next;
         }
         my $ret = invoke_subroutine_safely($act_hash{'sub'}, $hash_ref->{'params'});
         baselog($LOG_DEBUG, "The action '$action' returns '$ret'.");

         $retval &= $ret;
      }
      else
      {
         if (defined $act_hash{'validate_params'})
         {
            baselog($LOG_ERROR, "No parameters specified for the action '$action'.");  
            $retval &= $FALSE;
            next;
         }
         my $ret = invoke_subroutine_safely($act_hash{'sub'});
         baselog($LOG_DEBUG, "The action '$action' returns '$ret'.");
         
         $retval &= $ret;
      }
   }

   return $retval;
} 

1;
__END__
