## @file base_deploy
# @brief This module is a generic interface to work with "property" files that
#        have data in format: "parameter = value".
# @details none
# @author Ekaterina Bazhanova
# @date April 2008
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables. 
# @note

package property;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw( );

use lib "/opt/nortel/base/lib";

use strict;
use warnings;
use common_variables;
use baselog;
use base_system;
our $VERSION = '1.01';

###############################################################################
## @fn       set_property_value($file, $tag, $newval)
# @brief     Change value of selected variable in baseOs.properties file.
# @details   None.
# @param     $file  - full path to file which will be parsed
# @param     $tag   - name of the variable which value will be changed
# @param     $newval- new value for variable $tag
# @return    $TRUE - if variable value nas been changed successfully
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
# @note      None.
###############################################################################
sub set_property_value($$$)
{
   my ($file, $tag, $newval) = @_;
   my $fh;
   my $changed      = $FALSE;
   my @changed_file = ();
   my $tmp_file     = "$file.tmp";

   unless (open($fh, '<', $file))
   {
      baselog($LOG_ERROR, "Cannot open file for reading [ $file ]");
      return $FALSE;
   }
   @changed_file = <$fh>;
   foreach my $line (@changed_file)
   {
      if ($line =~ s/(^\s*$tag\s*=)(.*)/$1$newval/)
      {
         $changed = $TRUE;
         last;
      }
   } ## end foreach my $line (@changed_file...
   unless (close($fh))
   {
      baselog($LOG_ERROR, "Cannot close file [ $file ]");
      return $FALSE;
   }

   if ($changed)
   {
      unless (open($fh, '>', $tmp_file))
      {
         baselog($LOG_ERROR, "Cannot open temporary file for writing [ $tmp_file ]");
         return $FALSE;
      }

      baselog($LOG_INFO, "Overwrite temporary file  $tmp_file");

      print $fh @changed_file;

      unless (close($fh))
      {
         baselog($LOG_ERROR, "Cannot close file [ $tmp_file ]");
         return $FALSE;
      }

      if (rename($tmp_file, $file))
      {
         baselog($LOG_INFO, "Overwrite file  $file : the new value \'$newval\' was set for variable \'$tag\'");
      }
      else
      {
         baselog($LOG_ERROR, "Cannot overwrite file  $file");
         return $FALSE;
      }
   } ## end if ($changed)
   else
   {
      baselog($LOG_WARNING, "File $file has not been changed");
   }

   return $TRUE;
}  ##end sub  set_property_value($$$)

###############################################################################
## @fn       get_property_value($file, $tag)
# @brief     Get value of the selected variable in baseOs.properties file.
# @details   None.
# @param     $file  - full path to file which will be parsed
# @param     $tag   - name of the variable which value function try to get
# @return    $value - value of the selected variable
# @return    undef   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      April 2008
# @note      None.
###############################################################################
sub get_property_value($$)
{
   my ($file, $tag) = @_;
   my $fh;
   my $value;

   unless (open($fh, '<', $file))
   {
      baselog($LOG_ERROR, "Cannot open file for reading [ $file ]");
      return undef;
   }

   while (my $line = <$fh>)
   {
      $line =~ /^\s*$tag\s*=\s*(.*)\s*$/;
      $value = $1;
   }
   unless (close($fh))
   {
      baselog($LOG_ERROR, "Cannot close file [ $file ]");
      return undef;
   }
   return $value;
} ## end sub get_property_value($$)

1;

__END__
