## @file    backup.pm
# @brief    LinuxBase backup module.
# @details  This module holds general backup subroutines and variables.
# @param    none
# @author   Kate Bereznyakova
# @date     February 2007
# @version  1.00 - Initial version
# @version  1.10 - sudo_nortel's key and arguments were added
# @version  1.20 - new function do_backup was added
# @version  1.21 - usage of 'su' command is replaced by invoking
#          prepare_command_to_run_as subroutine.
# @version  1.30 - new function  is_restore_exists() is added.
# @version  1.31 - the mount option "nosuid" added
# @version  1.32 - New variable homes_data_archive and function create_base_backup_dir added.
# @version  1.33 - get rid of global variables $APP_INSTALLED and $APP_VERSION
# @version  1.34 - Two colons were removed before shared functions and variables.
# @version  1.35 - new USB detection functions are added.
# @version  1.36 - Two variables are added($CRON_MARK $cronsftp_identity)
# @version  1.37 - Three variables are added($appconfig $appconfdir $loadedBackupArchive)
# @version  1.40 - Backup scripts error handling was added.
#  @li do_backup function was updated.
#  @li execute_section_scripts function was updated
#  @li $NONINTERACTIVE variable was added
#  @li $BACKUP_ERR_MODE variable was added.
#  @li clear_backup_dir is added.
# @version  1.41 - Now USB flash cards are mounted under $UID instead of $NORTEL_ACCT

package recovery::backup;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;
use CommonInstallUtils;

use warnings;
use strict;
use English;
use base_xml;
use baselog;
use base_system;

# Access management
use Exporter;

our @ISA = qw(Exporter);  # public Exporter;

# Items to export into callers name space
our @EXPORT = qw(
  &remove_tmp_backup_archive &getBackupConfiguration &checkUSBBackupDevice
  &needFormat &getUSBDevName &execute_section_scripts &do_backup
  &create_base_backup_dir &removeUpgradeBackup
  $recovery_util_path $backup_command  $backup_util $fsbackup_config_script $configBase
  $tmpMountDir $backupScriptPath $BACKUP_CRON_MNGR
  $udevNodeScriptPath $SYSPARM_GEN $cfg_cache_dir $backupArchive $restoreArchive
  $basebackup_dir $userinfo_sync_file $patches_data_archive $homes_data_archive
  $sftp_identity $CRON_MARK $cronsftp_identity
  &get_USB_SN_list &get_USB_node_path_by_SN
  $EXECUTE_SCRIPTS $RESTORE_SCRIPTS $BACKUP_SCRIPTS $VERSION &is_restore_exists
  $appconfig $appconfdir $loadedBackupArchive $backup_cookie $NONINTERACTIVE
  $BACKUP_ERR_MODE
  );

## @skipline
sub execute_section_scripts($);
## @skipline
sub do_backup($$);
## @skipline
sub is_restore_exists();
## @skipline
sub create_base_backup_dir();
## @skipline
sub get_usbstorage_devices();
## @skipline
sub get_USB_SN_list();
## @skipline
sub get_USB_node_path_by_SN($);
## @skipline
sub clear_backup_dir();

our $VERSION = 1.41;

###############
# Target data #
###############
# Constants
our $recovery_util_path     = "/opt/nortel/base/recovery";
our $backup_command         = "sysbackup_task";
our $backup_util            = "$recovery_util_path/bin/fsbackup.pl";
our $fsbackup_config_script = "/etc/opt/nortel/base/recovery/fsbackup";
our $configBase             = "/etc/opt/nortel/base/recovery/basebackuppredef.conf";
our $tmpMountDir            = "$recovery_util_path/usb/mnt";
our $backupScriptPath       = "$recovery_util_path/bin/$backup_command";
our $BACKUP_CRON_MNGR       = "/opt/nortel/base/recovery/bin/cron_backup_scheduler";
our $udevNodeScriptPath     = "/etc/opt/nortel/base/recovery/usb";
our $SYSPARM_GEN            = "$recovery_util_path/sysparm_gen";
our $cfg_cache_dir          = "/var/opt/nortel/base/RecoveryCache";
our $backupArchive          = "backup.tar.gz";
our $loadedBackupArchive    = "/admin/recovery/backup_archive.tar.gz";
our $restoreArchive         = "/admin/nortel/restore/recover.tar.gz";
our $basebackup_dir         = "/admin/nortel/backup/base";
our $userinfo_sync_file     = "$basebackup_dir/userinfo.txt";
our $patches_data_archive   = "patchesinfo.tar.gz";
our $sftp_identity          = "nortel_cs1000_linuxbase_sftp";
our $cronsftp_identity      = "nortel_cs1000_linuxbase_cronsftp";
our $homes_data_archive     = "homes.tar.gz";
our $appconfig              = "$basebackup_dir/appconfig.tar.gz";
our $appconfdir             = "/var/opt/nortel/base/appconfig/appcustconfig";
our $backup_cookie          = "$basebackup_dir/.backup_cookie";

# cron mark added to the name of the configuration file
our $CRON_MARK = 'CRON';

#sudo_nortel keys
our $EXECUTE_SCRIPTS = "execute_scripts";

#arguments for execution scripts
our $RESTORE_SCRIPTS = "restore";
our $BACKUP_SCRIPTS  = "backup";

our $NONINTERACTIVE  = $FALSE;
our $BACKUP_ERR_MODE = $FALSE;

# keys and parameters to filter USB device on HAL scan
my $USBDEVICEDRIVERNAME = 'usb-storage';
my $DRIVERKEY           = 'info.linux.driver';
my $USBSERIALKEYFILTER  = 'usb.serial';

###############################################################################
## @fn is_restore_exists()
# @brief     This function checks if there is system upgrade data present on
#            the system or not.
# @details
# @return   $TRUE  - there are system backup data on the system
# @return   $FALSE - system backup data is not found.
# @author    Kate Bereznyakova
# @date      May 2008
sub is_restore_exists()
{
   my $retval = $FALSE;
   if (-e $UPGRADE_RESTORE)
   {
      baselog($LOG_INFO, 'Restore(system upgrade) data has been detected.');
      $retval = $TRUE;
   }
   else
   {
      baselog($LOG_INFO, 'There is no restore (system upgrade) data.');
   }
   return $retval;
} ## end sub is_restore_exists()

###############################################################################
## @fn needFormat($devUSBName, $USBPartitionName, $MountPoint)
# @brief     This function triggers USB disk format operation.
# @details   The device defined as not a basebackup storage.
#            To use it user must format it first.
# @param    $devUSBName - USB device node name
# @param    $USBPartitionName
# @param    $MountPoint - mount point to use to get an access to the device.
# @return   TRUE  - operation successful
# @return   FALSE - some error happens
# @author   Kate Bereznyakova
# @date     2007
sub needFormat(@)
{
   my ($devUSBName, $USBPartitionName, $MountPoint) = @_;
   print "This disk must be formatted to use it as a backup storage.\n";
   if (getYNResponse("Do you want to format it", 'Y'))
   {
      if (formatUSB("$devUSBName") == $FALSE)
      {
         return $FALSE;
      }

      # need to wait for UDEV to recreate the USB Partition node...
      my $attempts = 5;
      until ((-e "/dev/$USBPartitionName") or !(--$attempts))
      {
         sleep 1;
         baselog($LOG_WARNING, "$attempts wait attempts remain for /dev/$USBPartitionName to come back...");
      }

      # the US DoD requires to use "nosuid" mount points for removable media
      if (do_system("$SUDO $MOUNT -w -o uid=$UID,nosuid /dev/$USBPartitionName $MountPoint"))
      {
         system("$MKDIR -p $MountPoint/sysinfo/");
         system("$ECHO 'Do not remove this file.\n' > $MountPoint/.backupMedia");
         if ((!-d "$MountPoint/sysinfo/") or (!-e "$MountPoint/.backupMedia"))
         {
            baselog($LOG_ERROR, "Couldn't create cookies on USB device.");
            umountUSB("/dev/$USBPartitionName");
            return $FALSE;
         } ## end if ((!-d "$MountPoint/sysinfo/"...
         umountUSB("/dev/$USBPartitionName");
      } ## end if (do_system("$SUDO $MOUNT -w -o uid=$UID,nosuid /dev/$USBPartitionName $MountPoint"...
      else
      {
         baselog($LOG_ERROR, "Failed to mount USB device after format");
         return $FALSE;
      }
   } ## end if (getYNResponse("Do you want to format it"...
   else
   {
      system($CLEAR);
      print "This device cannot be used as backup storage. Try to use another one.\n";
      return $FALSE;
   } ## end else [ if (getYNResponse("Do you want to format it"...
   return $TRUE;
} ## end sub needFormat(@)

###############################################################################
## @fn checkUSBBackupDevice($devUSBName, $USBPartitionName, $MountPoint)
# @brief     This function checks if this device can be used as backup storage.
# @details
# @param    $devUSBName - USB device node name
# @param    $USBPartitionName
# @param    $MountPoint - mount point to use to get an access to the device.
# @return   $TRUE  - device can be used as a backup storage.
# @return   $FALSE - device cannot be used as a backup storage.
# @author   Kate Bereznyakova
# @date     2007
sub checkUSBBackupDevice(@)
{
   my ($devUSBName, $USBPartitionName, $MountPoint) = @_;

   if ((`$SUDO $PARTED -s $devUSBName print | $GREP \"msdos\\\|fat32\" | $WC -l`) != "2")
   {
      baselog($LOG_WARNING, "USB device is not in the correct format.");
      return $FALSE;
   }

   # the US DoD requires to use "nosuid" mount points for removable media
   do_system("$SUDO $MOUNT -o uid=$UID,nosuid /dev/$USBPartitionName $MountPoint");
   if ((!-d "$MountPoint/sysinfo/") or (!-e "$MountPoint/.backupMedia"))
   {
      baselog($LOG_ERROR, "Missing cookies on USB device.");
      umountUSB("/dev/$USBPartitionName");
      return $FALSE;
   } ## end if ((!-d "$MountPoint/sysinfo/"...
   umountUSB("/dev/$USBPartitionName");
   return $TRUE;
} ## end sub checkUSBBackupDevice(@)

###############################################################################
## @fn getBackupConfiguration($archive)
# @brief     This function gets an information about backup configuration
# @details
# @param    $archive - full path to backup archive file.
# @return   $APP_INSTALLED and $APP_VERSION: configuration and
#           version system where backup was made
# @return   undef and undef: error condition.
# @author   Kate Bereznyakova
# @date     2007
sub getBackupConfiguration
{
   my $archive = shift @_;
   if (!-e $archive)
   {
      baselog($LOG_ERROR, "Backup archive $archive does not exist.");
      return (undef, undef);
   }

   #get backup configuration file
   chomp(my $config_file = `$TAR -ztf $archive | $GREP $INSTALLEDCONFIG`);
   if ($config_file eq "")
   {
      baselog($LOG_ERROR, "Configuration file $INSTALLEDCONFIG is not found in the backup archive.");
      return (undef, undef);
   }

   if (system("cd  /; $TAR -Pxzf $archive $config_file 2>/dev/null") != 0)
   {
      baselog($LOG_ERROR, "Cannot extract file with backed up configuration data from archive.\n", $TRUE);
      return (undef, undef);
   }
   $config_file = "/" . $config_file;
   my $params = use_confver($config_file);

   if (system($RM, "-rf", $config_file) != 0)
   {
      baselog($LOG_WARNING, "Cannot remove $config_file file.");
   }
   unless (defined($params))
   {
      return (undef, undef);
   }
   return ($params->{'APP_INSTALLED'}, $params->{'APP_VERSION'});
} ## end sub getBackupConfiguration

###############################################################################
## @fn remove_tmp_backup_archive()
# @brief     This function removes backup archive generated by fsbackup.
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     2007
sub remove_tmp_backup_archive()
{
   if (-e "$cfg_cache_dir/$backupArchive")
   {
      if (system($RM, "-rf", "$cfg_cache_dir/$backupArchive") != 0)
      {
         baselog($LOG_ERROR, "Cannot delete temporary backup archive.\n");
         return $FALSE;
      }
   } ## end if (-e "$cfg_cache_dir/$backupArchive"...
   return $TRUE;
} ## end sub remove_tmp_backup_archive()

###############################################################################
## @fn execute_section_scripts($section_name)
# @brief     This function executes scripts, which are defined in the
#            "configure.xml" files.
# @details  It s called only from "sudo_nortel" script with root's privileges.
# @param    $section_name - name of section where scripts are defined
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     2007
sub execute_section_scripts($)
{
   my $section_name       = shift(@_);
   my $retval             = $TRUE;
   my @file_list          = ();
   my %script_exit_status = ();

   #create list of configure.xml
   if (not do_system([$FIND, $APP_CFG_ROOT, '-name', $APPLCFG], {$dsOUT => \@file_list}))
   {
      baselog($LOG_ERROR, "Cannot execute find operation in '$APP_CFG_ROOT'.");
      return $FALSE;
   }

   #execute script from $section_name if one or more configure files was found
   if (scalar(@file_list) < 1)
   {

      #no one configure file was found. exit with TRUE.
      baselog($LOG_ERROR, "There is no '$APPLCFG' in '$APP_CFG_ROOT'.");
      return $FALSE;
   } ## end if (scalar(@file_list)...
   foreach my $config (@file_list)
   {
      chomp $config;
      my $parser = base_xml->new($config);
      if (not defined $parser)
      {
         baselog($LOG_ERROR, "Cannot create xml parser for '$config'.");

         #if parser cannot be created for some configure.xml, Then user will verify
         #list of applications which was backed up.
         $retval = $FALSE;
         next;
      } ## end if (not defined $parser...

      #collect application name from configure.xml
      my $app_name_obj = $parser->getAttrs('/application');
      my $app_name     = $app_name_obj->{'name'};

      if (not defined $app_name)
      {
         baselog($LOG_ERROR, "'$config' is invalide. Attribute 'name' is not defined at 'application' tag.");
         undef $parser;
         next;
      } ## end if (not defined $app_name...

      #collect script name from configure.xml
      my $section = "/application/$section_name/script";
      my $scripts = $parser->findElements($section, undef);
      if (scalar(@{$scripts}) == 0)
      {
         baselog($LOG_WARNING, "'$section_name' section is empty for '$app_name'.");
         undef $parser;
         next;
      } ## end if (scalar(@{$scripts}...
      $script_exit_status{$app_name} = $TRUE;
      foreach my $script (@{$scripts})
      {

         #collect script name from configure.xml
         my $file = $script->getAttribute('file');
         my $as   = $script->getAttribute('as');
         $as = $NORTEL_ACCT if (not defined $as);
         if ($file =~ s/.*sudo\s+(.*)/$1/)
         {
            baselog($LOG_WARNING, "Script $file is executed as sudo.\n");
         }
         my @cmdarr = @{prepare_command_to_run_as($file, $as)};
         if (scalar @cmdarr != 0)
         {
            my $cmd_retval;
            if (not do_system(\@cmdarr, {$dsRETVAL => \$cmd_retval, $dsINTERACT => !$NONINTERACTIVE}))
            {

               #script cannot be executed.
               baselog($LOG_ERROR, "command " . join(' ', @cmdarr) . " cannot be executed or returns '$cmd_retval'.");
               $script_exit_status{$app_name} = $FALSE;
               $retval = $FALSE;
            } ## end if (not do_system(\@cmdarr...
         } ## end if (scalar @cmdarr != ...
         else
         {
            baselog($LOG_ERROR, "Script '$file' can not be executed.\n", $TRUE);
            $script_exit_status{$app_name} = $FALSE;
            $retval = $FALSE;
         } ## end else [ if (scalar @cmdarr != ...
      } ## end foreach my $script (@{$scripts...
      undef $parser;
   } ## end foreach my $config (@file_list...

   #to confirm actions if some backup scripts fails
   if ($retval == $FALSE)
   {
      if ($NONINTERACTIVE == $FALSE)
      {

         #ask user what to do if interactive mode
         print "\nOperation status per application:\n";
         foreach my $_app (keys %script_exit_status)
         {
            my $status = "\tFAILED";
            if ($script_exit_status{$_app} == $TRUE)
            {
               $status = "\tOK";
            }
            print "$_app: $status\n";
         } ## end foreach my $_app (keys %script_exit_status...
         $retval = $TRUE if (getYNResponse("Some backup scripts failed. Do you want to continue?", 'Y'));
      } ## end if ($NONINTERACTIVE ==...
      elsif ($BACKUP_ERR_MODE == $FALSE)
      {

         #if noninteractive check for sysbackup input parameter --backup_err_mode
         #if it set backup operation will be interrupted.
         $retval = $TRUE;
      } ## end elsif ($BACKUP_ERR_MODE ==...
   } ## end if ($retval == $FALSE)
   return $retval;
} ## end sub execute_section_scripts($)

###############################################################################
## @fn do_backup($config, $arg1)
# @brief     This function performs a backup operation.
# @details   Backup operation includes the following steps:
# @details   1. Clean up backup directory.
# @details   2. Execute backup scripts.
# @details   3. Execute script, that copies archive to destination storage.
# @details   4. Clean up backup directory again.
# @details   5. Perform upgrade, if required.
# @param    $config - name of the configuration file for backup.
# @param    $arg1 - '--upgrade' option, if upgrade is required,
#                    otherwise - "" (empty string).
# @return   $TRUE  - operation successful
# @return   $FALSE - some error happens
# @author   Kate Bereznyakova
# @date     2007
sub do_backup($$)
{
   my $config = shift(@_);
   my $arg1   = shift(@_);
   my $retval = $TRUE;

   if (!defined $config)
   {
      baselog($LOG_ERROR, "Configuration file for backup is not specified.");
      return $FALSE;
   }
   if (!defined $arg1)
   {
      baselog($LOG_ERROR, "No option for backup is specified.");
      return $FALSE;
   }

   # clean up to avoid permissions and data mixing problems
   return $FALSE if (not clear_backup_dir());

   # filling the backup directory with applications data
   if (not do_system([$SUDO, $SUDO_NORTEL, $EXECUTE_SCRIPTS, $BACKUP_SCRIPTS, $NONINTERACTIVE, $BACKUP_ERR_MODE], {$dsINTERACT => !$NONINTERACTIVE}))
   {
      baselog($LOG_ERROR, "Some backup scripts failed.");
      clear_backup_dir();
      return $FALSE;
   } ## end if (not do_system([$SUDO...

   # make a backup archive and put it to backup storage
   if (not do_system([$backup_util, $config, $arg1], {$dsINTERACT => !$NONINTERACTIVE}))
   {
      baselog($LOG_ERROR, "Cannot not create backup archive.");
      clear_backup_dir();
      return $FALSE;
   } ## end if (not do_system([$backup_util...

   # no backup data under /admin should present to prevent backup/restore data mixing
   if (not clear_backup_dir())
   {
      baselog($LOG_WARNING, "Post backup clean up failed.");
   }

   # upgrade keeps default backup archive copy under /admin
   if ($arg1 eq "--upgrade")
   {
      if (-e "$cfg_cache_dir/$backupArchive")
      {
         if (not do_system([$MV, '--force', "$cfg_cache_dir/$backupArchive", $UPGRADE_BACKUP]))
         {
            baselog($LOG_ERROR, "Cannot put upgrade archive to /admin partition.\n", $TRUE);
            remove_tmp_backup_archive();
            removeUpgradeBackup();
            $retval = $FALSE;
         } ## end if (not do_system([$MV...
      } ## end if (-e "$cfg_cache_dir/$backupArchive"...
      else
      {
         baselog($LOG_ERROR, "Generated backup archive not found.\n", $TRUE);
         $retval = $FALSE;
      }
   } ## end if ($arg1 eq "--upgrade"...
   return $retval;
} ## end sub do_backup($$)

###############################################################################
## @fn       removeUpgradeBackup()
# @brief     Remove default uppgrade backup archive from /admin partition.
# @details   None.
# @param     None.
# @return    $TRUE - successfull operation
# @return    $FALSE   - error condition
# @author    Kate Bereznyakova
# @date      May 2008
# @note      None.
sub removeUpgradeBackup()
{
   if (-e $UPGRADE_RESTORE)
   {
      unless (unlink($UPGRADE_RESTORE))
      {
         baselog($LOG_ERROR, 'Cannot delete default upgrade/backup archive.');
         return $FALSE;
      }
   } ## end if (-e $UPGRADE_RESTORE...
   return $TRUE;
} ## end sub removeUpgradeBackup()

## @fn sub create_base_backup_dir()
# @brief If LinuxBase backup directory is not exist, then it is created
#        and proper attributes are set.
# @details
# @param None
# @return
#   - $TRUE  - directory is successfully created with right permissions.
#   - $FALSE - some errors happened.
# @author Evgeny Lazarev
# @date February 2008
# @note
sub create_base_backup_dir()
{
   my $retval = $TRUE;
   if (!-d $basebackup_dir)
   {
      if (!(do_system([$MKDIR, '-p', '-m', '770', $basebackup_dir]) && do_system([$CHOWN, '-R', "$NORTEL_ACCT:$NORTEL_ACCT", $basebackup_dir])))
      {
         baselog($LOG_ERROR, "Can not create $basebackup_dir directory.");
         $retval = $FALSE;
      }
   } ## end if (!-d $basebackup_dir...
   return $retval;
} ## end sub create_base_backup_dir()

## @fn get_USB_SN_list()
# @brief Get list of serial numbers of USB storage devices registered on the
# the system.
# @details
# @return @sn - reference to the list with found USB storage serial numbers.
# @return undef - indicates an error condition or no USB devices found.

sub get_USB_SN_list()
{
   my $deviceList = get_usbstorage_devices();
   my @sn         = ();

   if (!is_array($deviceList))
   {
      baselog($LOG_ERROR, 'USB device is not recognized.');
      return undef;
   }

   # check each device (by udi) to get USB device serial number
   foreach my $device (@$deviceList)
   {
      baselog($LOG_DEBUG, "USB device udi: '$device'.");
      my @snumber = ();

      # this command gets a USB serial key value associated with provided udi
      if (!do_system([$GETPROPERTY, '--udi', $device, '--key', $USBSERIALKEYFILTER], {$dsOUT => \@snumber}))
      {
         baselog($LOG_ERROR, "Cannot check serial number for '$device' device.");
      }
      else
      {
         if (!is_array(\@snumber))
         {
            baselog($LOG_WARNING, "Getpropery returned not an array. Skip '$device' device.");
            next;
         }

         chomp(@snumber);
         if ((defined $snumber[0]) and ($snumber[0] ne ''))
         {
            baselog($LOG_DEBUG, "USB serial number recognized: '@snumber'.");
            push @sn, @snumber;
         }
         else
         {
            baselog($LOG_ERROR, "USB serial number for udi '$device' is not recognized.");
         }
      } ## end else [ if (!do_system([$GETPROPERTY...
   } ## end foreach my $device (@$deviceList...
   return \@sn;
} ## end sub get_USB_SN_list()

## @fn get_usbstorage_devices()
# @brief Gets all available usb storage devices.
# @details The function checks HAL database and retrieves udi (uniq identification)
# for each usb storage device registered in the system at current moment. Check is
# based on the device driver name - "usb-storage". Device driver name is a subject
# to change.
# @return @deviceList - reference to the list of udi's of the found devices.
# @return undef - indicates an error condition or no USB devices found.
# @return 0 - if no USB devices found.

sub get_usbstorage_devices()
{
   my @deviceList = ();

   # this command gets all the udi's associated with usb-storage driver
   if (not do_system([$FINDBYPROP, '--key', $DRIVERKEY, '--string', $USBDEVICEDRIVERNAME], {$dsOUT => \@deviceList}))
   {
      baselog($LOG_ERROR, 'Cannot get USB storage devices.');
      return undef;
   }
   elsif (scalar(@deviceList) > 0)
   {
      chomp(@deviceList);
   }
   else
   {
      baselog($LOG_WARNING, 'No USB devices found.', $TRUE);
      return 0;
   }
   return \@deviceList;
} ## end sub get_usbstorage_devices()

## @fn get_USB_node_path_by_SN($sn)
# @brief Gets node name associated with provided disk serial number.
# @details Udev creates symlink to the real nodes using the rule defined in
# 50-udev.rules file under /etc/udev/rules.d/directory (it is part of
# standard udev package).
# The rule looks like the following:
#   KERNEL=="sd*[0-9]|dasd*[0-9]", ENV{ID_SERIAL}=="?*",
#   SYMLINK+="disk/by-id/$env{ID_BUS}-$env{ID_SERIAL}-part%n"
# It is possible to get real node name by the symlink that keeps the serial
# number of the device in the file name.
# @param $sn - uniq serial number of the device.
# @return $path - path to the node name under /dev.
#                 Return value is a full path.
# @return $first_part - path to the first partition node name under /dev.
#                 Return value is a full path.
# @return undef - indicates an error condition.
sub get_USB_node_path_by_SN($)
{
   my $sn         = shift @_;
   my $path       = undef;
   my $first_part = undef;

   if ((!defined $sn) or ($sn eq ''))
   {
      baselog($LOG_ERROR, 'Serial number is undefined.');
      return undef;
   }

   # get link to device node.
   my $dir_handler;
   my $base_dir = '/dev/disk/by-id/';
   unless (opendir($dir_handler, $base_dir))
   {
      baselog($LOG_ERROR, "Cannot open: $base_dir!");
      return undef;
   }

   # get list of files in the directory
   my @file_list = readdir($dir_handler);
   unless (closedir($dir_handler))
   {
      baselog($LOG_ERROR, "Cannot close $base_dir: $!");
      return undef;
   }

   if (scalar(@file_list) < 1)
   {
      baselog($LOG_ERROR, 'No information in udev database.', $TRUE);
      return undef;
   }

   my $sn_regexp = qr/$sn$/;

   foreach my $file (@file_list)
   {
      if (grep /$sn_regexp/, $file)
      {
         baselog($LOG_DEBUG, "Match found for '$file'.");

         # link to the real node.
         my $link = readlink($base_dir . $file);
         if ((defined $link) and ($link ne ''))
         {

            #need only last part
            (undef, undef, $path) = split(/\//, $link);
            $path       = '/dev/' . $path;
            $first_part = $path . '1';
            baselog($LOG_DEBUG, "USB node name is detected: '$path'.");
            baselog($LOG_DEBUG, "USB first partition is: '$first_part'.");
         } ## end if ((defined $link) and...
         else
         {
            baselog($LOG_ERROR, "Failed to get USB node name by link: '$file'.");
         }
         last;
      } ## end if (grep /$sn_regexp/,...
   } ## end foreach my $file (@file_list...

   return ($path, $first_part);
} ## end sub get_USB_node_path_by_SN($)

## fn clear_backup_dir()
# @brief Clear backup archive directory
# @details
# @return $FALSE - in error case, else $TRUE.
sub clear_backup_dir()
{
   my $retval = $TRUE;
   if (not do_system([$SUDO, $SUDO_NORTEL, $CLEAR_BACKUP]))
   {
      baselog($LOG_ERROR, "Cannot clear backup directory");
      $retval = $FALSE;
   }
   return $retval;
} ## end sub clear_backup_dir()

1;
__END__

=head1 NAME

backup - common data and functions for backup management tool.

use backup;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
