###############################################################################
## @file   BootLoader.pm
# @brief   The boot loader recovery functional module.
# @details This module introduces common variables
#          and functions for checking or repairing of the configuration file
#          and the boot loader reinstallation.
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version
# @version 1.01 - Modified to use is_cppm() function.
# @version 1.02 - Modified to use grub_cf.conf.bak configuration file for CPPM.

package recovery_pm::BootLoader;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use hwRecognition; 
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.02';

## @var $bootDir
#  /boot partition mount point
my $bootDir   = "/boot";

## @var $bootDev
#  /boot partition 
my $bootDev    = undef;

## @var $Dev
#  File name of device
my $Dev        = undef;

## @var $exit_mess
#  Message before return to the root menu
my $exit_mess  = "Press <Enter> to continue:";


## @var my $rootDev
#  /root partition
my $rootDev    = undef;

## @var $shadow
#  Shadow file path
my $shadow     = $ROOTDIR . $SHADOWFILE;

## @var $template_grub_file
#  Full path to the configuration file template
my $template_grub_file = undef;
## @var $menu_str
#  Name of the option for the recovery console main menu
my $menu_str = "Boot loader recovery";
my $grub_setup = undef;

my %actions = ('menu'     => {'string' => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );

## @skipline
sub DefaultEntryPoint();
## @skipline
sub grub_check();
## @skipline
sub grub_repair();
## @skipline
sub change_digest($);
## @skipline
sub grub_install();
## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to boot loader recovery module.
# @details This function allows creating of the boot loader recovery menu.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008


sub DefaultEntryPoint()
{
   $rootDev   = defDev('/');
   $bootDev   = defDev('/boot');
   unless ($rootDev)
   {
      print "Error: Cannot define device for / partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   unless ($bootDev)
   {
      print "Error: Cannot define device for /boot partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   if (is_cppm())
   {
      $template_grub_file = "/tmp/cf/scripts/recovery_pm/grub/grub_cf.conf.bak";
      $grub_setup = "/tmp/cf/scripts/recovery_pm/grub/grub.setup";
   }
   else
   {
      $template_grub_file = "/mnt/source/scripts/recovery_pm/grub/grub.conf.bak";
      $grub_setup = "/mnt/source/scripts/recovery_pm/grub/grub.setup";

   }
   if ($rootDev =~ /(\/dev\/\D+)\d+/) 
   {
      $Dev = $1;
   }
   else
   {
      print "Error: Cannot define file device name \n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   unless ( -d $RECDIR)
   {
      unless (mkdir($RECDIR))    { return $FALSE;}
   }   
   unless (-d $bootDir)
   {
      unless (mkdir($bootDir))  { return $FALSE;}
   }
   unless (-d $ROOTDIR)
   {
      unless (mkdir($ROOTDIR))  { return $FALSE;}
   }
   unless (system("$MOUNT", "$rootDev", "$ROOTDIR") == 0)
   {
      return $FALSE;
   }
   unless (system("$MOUNT", "$bootDev", "$bootDir") == 0)
   {
      system "$UMOUNT", "$ROOTDIR";
      return $FALSE;
   }

   while ($TRUE)
   {

      #Clear screen
      clear();
      print "$menu_str:\n";
      print <<HERE;
 You may choose to do one of the following: 
 1) Check configuration files
 2) Repair configuration files
 3) Reinstall Grub
 4) Exit
HERE
      print "Select an option (1-4):";
      my $choice = <STDIN>;
      print "\n";
      chomp($choice);
      if ($choice =~ /^\s*(\d+)\s*$/)
      {
         $choice = $1; 
      }
      else
      {
         next;
      }
      #Checking configuration files
      if ($choice == 1)
      {
         clear();     
         if (grub_check())
         {
            print "Configuration files have been checked. No error reported.\n";
         }
         else
         {
            print "You have to repair configuration files\n";
         }
         print $exit_mess;
         <STDIN>;
      } ## end if ($choice == 1)

      #Repaire configuration files if files are incorrect
      elsif ($choice == 2)
      {
         clear();         
         if (grub_check())
         {
            print "You have not to repair configuration files: all files are correct\n";
         }
         elsif (grub_repair())
         {
            print "The configuration file has been repaired successfully\n";
         } 
         else
         {
            print "The configuration file has not been repaired\n";
         }
         print $exit_mess ;
         <STDIN>;
      } ## end elsif ($choice == 2)

      #reinstall grub loader
      elsif ($choice == 3)
      {
         clear();        
         if (grub_install())
         {
            print "Installation was finished successfully.\n";
         } 
         else
         {
            print "Installation was broken\n";
         }
         print $exit_mess ;
         <STDIN>;
      } ## end elsif ($choice == 3)

      # exit from boot loader recovery
      elsif ($choice == 4)
      {
         last;
      }
   } ## end while ($TRUE)
   system "$UMOUNT", "$bootDir";
   system "$UMOUNT", "$ROOTDIR";
} ## end sub DefaultEntryPoint

###############################################################################
## @fn sub grub_check()
# @brief   Checking configuration files.
# @details The function check if grub.conf file and all required fields are
#          exist and not empty then configuration files are able.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008


sub grub_check()
{
   my @kernel  = ();
   my @initrd  = ();
   my $default = $FALSE;
   my $timeout = $FALSE;
   my $title   = $FALSE;
   my $exit    = $TRUE;
   my $grub_handler;

   unless (-s $GRUBCONF)  #file exists and not empty
   {     
      print "File $GRUBCONF does not exist or empty.\n";
      return $FALSE;
   }

      unless (open($grub_handler, '<', $GRUBCONF))
      {
         print STDERR "Cannot open file $GRUBCONF: $!\n";
         return $FALSE;
      } ## end unless (open($grub_handler...
      while (my $grub_str = readline($grub_handler))
      {

         if ($grub_str =~ /\A[^#]*kernel\s+(\/[^\s]+).*$/)
         {
            @kernel = (@kernel, $1);
         }
         elsif ($grub_str =~ /\A[^#]*initrd\s+(\/[^\s]+).*$/)
         {
            @initrd = (@initrd, $1);
         }
         elsif ($grub_str =~ /\A\s*default.*/)
         {
            $default = $TRUE;
         }
         elsif ($grub_str =~ /\A\s*timeout.*/)
         {
            $timeout = $TRUE;
         }
         elsif ($grub_str =~ /\A\s*title.*/)
         {
            $title = $TRUE;
         }
      }  #end while (my $grub_str ...
      if ((scalar @kernel == 0) || (scalar @initrd == 0))
      {
         print "There are no any kernel or initrd files\n";
         $exit = $FALSE;
      }
      unless (close($grub_handler))
      {
         print STDERR "Cannot close file $GRUBCONF: $!\n";
         return $FALSE;
      }

      foreach my $kernel_index (@kernel)
      {
         my $check_file = join('', $bootDir, $kernel_index);
         if (not -s $check_file)
         {
            print "Kernel file $kernel_index does not exist or empty\n";
            $exit = $FALSE;
         } ## end if (not -s $check_file...
      } ## end foreach my $kernel_index...

      foreach my $initrd_index (@initrd)
      {
         my $check_file = join('', $bootDir, $initrd_index);
         if (not -s $check_file)
         {
            print "Initrd file $initrd_index does not exist or empty\n";
            $exit = $FALSE;
         } ## end if (not -s $check_file...
      } ## end foreach my $initrd_index...

      unless ($default && $timeout && $title)  #Not all of required fields exist
      {
         print "Not all of required fields exist\n";
         $exit = $FALSE;
      }

   return $exit;
}  #end grub_check


###############################################################################
## @fn sub grub_repair()
# @brief   Repaire grub.conf file.
# @details Repairing via usig of the "/sbin/grubby" utility.
# @param   None
# @return  TRUE as the end of function
# @author  Ekaterina Bazhanova
# @date    February  2008


sub grub_repair()
{
   my $kernelName = 'vmlinuz';
   my $initrd     = 'initrd';
   my $config     = 'config';
   my $map        = 'System.map';
   my @version    = ();
   my @boot_cont  = ();
   my @template_version   = ('2.6.18-53.el5');
   my $dir;

   #change grub.conf for suitable template
   if (not -d $GRUBDIR)
   {
      system "$MKDIR", "$GRUBDIR";
   }
   if (not -e $GRUBCONF)
   {
      system "$TOUCH", "$GRUBCONF";
   }
   system "$CP", "$template_grub_file","$GRUBCONF";
   if ($?)
   {
      return $FALSE;
   }
   #find which kernel files are exist
   unless (opendir($dir, $bootDir))
   {
    print STDERR "Cannot open $bootDir directiry: $!\n";
   }

   @boot_cont = readdir($dir);
   foreach my $boot_index (@boot_cont)
   {
      if ($boot_index =~ /.* $kernelName-([^\s]*).*$/)
      {
         unless (-e "$bootDir/$initrd-$1.img")
         {
            next;
         }
         @version = (@version, $1);
      } ## end if ($boot_index =~ ...
   } ## end foreach my $boot_index ...

   foreach my $version_index (@version) 
   {
      my $template = 0;
      foreach my $template_index (@template_version )
      {
         if ($version_index eq $template_index)
         {
            $template = 1;
         }
      } ## end foreach my $template_index...
      if (not $template)
      {
         system ("$GRUBBY --add-kernel=$bootDir/$kernelName-$version[$version_index] --initrd $bootDir/$initrd-$version[$version_index].img --make-default --title \"Red Hat Enterprise Linux ($version[$version_index])\" --args=\"root=$rootDev\"");
         if ($?)
         {
            return $FALSE;
         }
      } ## end if (not $template)
   } ## end foreach my $version_index ...

   #setup actual root password hash
   change_digest(scalar @version);

   return $TRUE;
}  #end sub grub_repair

###############################################################################
## @fn sub change_digest($kernel_count)
# @brief   Setup actual root password hash.
# @details Function get actual hash from shadow file.
# @param   $kernel_count - count of linux kernels in system
# @return  TRUE as the end of function or FALSE in case of an error
# @author  Ekaterina Bazhanova
# @date    February  2008


sub change_digest($)
{
   my $digest;
   my @grub = ();
   my $kernel_count;
   my $shadow_handler;
   my $grub_handler;
   $kernel_count = shift;
   unless (open($shadow_handler, '<', $shadow))
   {
      print STDERR "Cannot open file  $shadow: $!\n";
      return $FALSE;
   }
   while (my $shadow_str = readline($shadow_handler))
   {
      if ($shadow_str  =~ /\A(root):(\$[^:]*):(.*)/)
      {
         $digest = $2;
      }
   } ## end while (my $shadow_str...
   unless (close($shadow_handler))
   {
      print STDERR "Cannot close file $shadow: $!\n";
      return $FALSE;
   }

   unless (open($grub_handler, '<', $GRUBCONF))
   {
      print STDERR "Cannot open file $GRUBCONF: $!\n";
      return $FALSE;
   }
   while (my $grub_str = readline($grub_handler))
   {
      $grub_str =~ s/(password[^\$]*)(\$[^\s]*)(.*)/$1$digest$3\n/;
      if ($kernel_count > 2)
      {
         $grub_str =~ s/(\A\s*)(timeout=0)(.*)/$1timeout=5$3/;
      }
      push @grub, $grub_str;
   } ## end while (my $grub_str...
   unless (close($grub_handler))
   {
      print STDERR "Cannot close file $GRUBCONF: $!\n";
      return $FALSE;
   }

   unless (open($grub_handler, '>', $GRUBCONF))
   {
      print STDERR "Cannot open file $GRUBCONF: $!\n";
      return $FALSE;
   }
   print $grub_handler @grub;
   unless (close($grub_handler))
   {
      print STDERR "Cannot close file $GRUBCONF: $!\n";
      return $FALSE;
   }
   return $TRUE;
}  #end sub change_digest

###############################################################################
## @fn sub grub_install()
# @brief   Reinstall grub loader.
# @details Uses /sbin/grub-install. Allows user to repair configuration file
#          if it is necessary.
# @param   None
# @return  TRUE - successful installation, or FALSE in case of an error
#          or breaking installation 
# @author  Ekaterina Bazhanova
# @date    February  2008


sub grub_install()
{
   my $repair;
   my $rep_query  = 'Would you like to repair it right now?';
   my $inst_query = 'Would you like to reinstall GRUB right now?';

   $repair = $FALSE; #do you need to repair configuration files? FALSE - No, TRUE - yes
   unless (grub_check())
   {
      print "You have to repair configuration files\n";
      $repair = $TRUE;

      if (getYNResponse($rep_query,'N'))
      {
         if (grub_repair())
         {
            print "The files have been repaired\n";
            $repair = $FALSE; 
         }
         else
         {
            print "The repairing was broken\n";
         } 
      } 
      else
      {
         print "The files have not been repaired\n";
      }

   } ## end unless (grub_check())

   if ($repair==$TRUE)
   {
      print "GRUB cannot be reinstalled: configuration files have not been repaired\n";
      return $FALSE;    
   }
   
   if (getYNResponse($inst_query,'N'))
   {
      system ("$GRUBINST --just-copy >/dev/null 2>&1");
      if ($?)
      {
         return $FALSE;
      } 
      system ("$GRUB --batch < $grub_setup >/dev/null 2>&1");
      if ($?)
      {
         return $FALSE;
      } 

   }
   else
   {
      return $FALSE; 
   }
   return $TRUE;
} ## end sub grub_install

1;
__END__
