###############################################################################
## @file   ChangePassword.pm
# @brief   Password recovery module.
# @details This module introduces common variables and functions for
#          softly changing of the user password
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version
# @version 1.01 - Modified to support only root and nortel users

package recovery_pm::ChangePassword;

use 5.008005;
use strict;
use warnings;
use common_variables;
use Digest::MD5;
use CommonInstallUtils;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

## @var  $adminDev
#  /admin partition 
my $adminDev   = undef;

## @var $bootDev
#  /boot partition 
my $bootDev    = undef;

## @var $rootDev
#  /root partition 
my $rootDev    = undef;

## @var $exit_mess
#  Message before return to the root menu
my $exit_mess  = "Press <Enter> to continue:";

## @var $grub_file
#  Configuration file path 
my $grub_file  = $RECDIR . $GRUBCONF;


## @var $shadow
#  Shadow file path
my $shadow     = $ROOTDIR . $SHADOWFILE;

## @var $usrinf_file
#  User information file path
my $usrinf_file = $RECDIR . $USERINFO;
## @var $menu_str
#  Name of the option for the recovery console main menu
my $menu_str   = "Password recovery";

my %actions = ('menu'     => {'string'     => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );
## @skipline
sub MenuString();
## @skipline
sub DefaultEntryPoint();
## @skipline
sub request_user_params($);
## @skipline
sub change_user_passwd($$);
## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to password recovery module.
# @details This function contains preparation work for the password changing.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008

sub DefaultEntryPoint()
{

   $rootDev   = defDev('/');
   $adminDev  = defDev('/admin');
   $bootDev   = defDev('/boot');

   unless ($rootDev)
   {
      print "Error: Cannot define device for / partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   unless ($adminDev)
   {
      print "Error: Cannot define device for /admin partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   unless ($bootDev)
   {
      print "Error: Cannot define device for /boot partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }

   #Clear screen
   clear();
   print "$menu_str:\n";
   unless ( -d $RECDIR)
   {
      unless (mkdir($RECDIR))    { return $FALSE;}
   }   
   unless (-d $ROOTDIR)
   {
      unless (mkdir($ROOTDIR))  { return $FALSE;}
   }
   unless (-d $BOOTDIR)
   {
      unless (mkdir($BOOTDIR))  { return $FALSE;}
   }
   unless (-d $ADMINDIR)
   {
      unless (mkdir($ADMINDIR)) { return $FALSE;}
   }
   unless (system("$MOUNT", "$rootDev", "$ROOTDIR") == 0)
   {
      return $FALSE;
   }
   unless (system("$MOUNT", "$bootDev", "$BOOTDIR") == 0)
   {
      system "$UMOUNT", "$ROOTDIR";
      return $FALSE;
   }

   unless (system("$MOUNT", "$adminDev", "$ADMINDIR") == 0)
   {
      system "$UMOUNT", "$ROOTDIR";
      system "$UMOUNT", "$BOOTDIR";
      return $FALSE;
   }

   if (request_user_params($shadow))
   {
      print "\nUser password has been changed successfully!\n";
   } ## end if (request_user_params...
   else
   {
      print "\nUser password has not been changed!\n";
   } ## end else [ if (request_user_params...
   print $exit_mess;
   <STDIN>;

   system "$UMOUNT", "$ROOTDIR";
   system "$UMOUNT", "$BOOTDIR";
   system "$UMOUNT", "$ADMINDIR";
} ## end sub DefaultEntryPoint

###############################################################################
## @fn sub request_user_params($path)
# @brief   Request new security user password.
# @details Check password for stability.
# @param   $path - "shadow" file directory
# @return  $TRUE if password was changed successfully or $FALSE in other case
# @author  Ekaterina Bazhanova
# @date    February  2008


sub request_user_params($)
{
   my $login;
   my $path = shift;
   my $passwd;
   my $repeat_passwd;
   my @users = ();
   my $user_index;
   my $shadow_handler;

   unless (open($shadow_handler, '<', $path))
   {
      print STDERR "Cannot open file $path: $!\n";
      return $FALSE;
   }

   while ( my $shadow_str = readline($shadow_handler))
   {
      if ($shadow_str =~ /\A(nortel|root):(\$[^:]*):(.*)/)
      {
         push @users, $1;
      }
   } ## end while ( my $shadow_str ...

   unless (close($shadow_handler))
   {
      print STDERR "Cannot close file $path: $!\n";
      return $FALSE;
   }

   # print password recovery menu
   print " You may change password one of the following users:\n\n";
   $user_index = 0;
   foreach my $user (@users)
   {  
      $user_index++;
      print " ", $user_index, ") $user\n";
   }
   print " ", scalar @users + 1,") exit\n";
   print "\nSelect an option (1-", scalar @users + 1, "):";
   ## end print password recovery menu

   $login = <STDIN>;
   chomp($login);

   if ($login=~ /^\s*(\d+)\s*$/)
   {
      $login = $1;
   }
   else
   {
      print "Incorrect choice!\n";      
      return $FALSE;
   }

   if ($login == scalar @users + 1)  # if exit option?
   {
      return $FALSE;
   }
   
   if (($login <= scalar @users) && ($login > 0))
   {
      $login = $users[$login - 1];
   }      
   else 
   {
      print "Incorrect choice!\n";
      return $FALSE;
   }

   unless (getPassword($login, $FALSE))
   {
      print "\nFailed to get password!";
      return $FALSE;
   }
         
   unless (change_user_passwd($path, $login))
   {
      print "\nFailed to store password!";
      return $FALSE;
   }

   return $TRUE;
} ## end sub request_user_params


###############################################################################
## @fn sub change_user_passwd($path, $login)
# @brief   Changes user password.
# @details Reset old user password and creating new user password.
# @param   $path - "shadow" file directory
# @param   $login - user login
# @param   $passwd - new user password
# @return  $TRUE if password was changed successfully or $FALSE in other case
# @author  Ekaterina Bazhanova
# @date    February  2008


sub change_user_passwd($$)
{
   my ($path,$login) = @_;
   my $digest;
   my @shadow = ();
   my @usrinf = ();
   my @grub   = ();
   my $shadow_handler;
   my $user_handler;
   my $grub_handler;


   $digest = getCryptPW($login);

   unless (open($shadow_handler, '<', $path))
   {
      print STDERR "Cannot open file $path: $!\n";
      return $FALSE;
   }

   while ( my $shadow_str = readline($shadow_handler))
   {  
      $shadow_str =~ s/$login:([^:]*):(.*)/$login:$digest:$2/;
      push @shadow, $shadow_str;
   } ## end while ( my $shadow_str ...

   unless (close($shadow_handler))
   {
      print STDERR "Cannot close file $path: $!\n";
      return $FALSE;
   }

   unless (open($shadow_handler, '>', $path))
   {
      print STDERR "Cannot open file $path: $!\n";
      return $FALSE;
   }

   print $shadow_handler @shadow;

   unless (close($shadow_handler))
   {
      print STDERR "Cannot close file $path: $!\n";
      return $FALSE;
   }

   # change userinfo.txt file
   unless (open($user_handler, '<', $usrinf_file))
   {
      print STDERR "Cannot open file $usrinf_file: $!\n";
      return $FALSE;
   }

   while (my $user_str = readline($user_handler))
   {
      $user_str =~ s/$login(pw=)([^ ]*)(.*)/$login$1$digest$3\n/;
      push @usrinf, $user_str;
   } ## end while (my $user_str...
   unless (close($user_handler))
   {
      print STDERR "Cannot close file $usrinf_file: $!\n";
      return $FALSE;
   }

   unless (open($user_handler, '>', $usrinf_file))
   {
      print STDERR "Cannot open file $usrinf_file: $!\n";
      return $FALSE;
   }

   print $user_handler @usrinf;
   unless (close($user_handler))
   {
      print STDERR "Cannot close file $usrinf_file: $!\n";
      return $FALSE;
   }

   ## end change userinfo.txt file

   # change grub.conf
   if ($login eq 'root')
   {
      unless (open($grub_handler, '<', $grub_file))
      {
         print STDERR "Cannot open file $grub_file: $!\n";
         return $FALSE;
      }

      while ( my $grub_str = readline( $grub_handler ))
      {
         $grub_str =~ s/(password[^\$]*)(\$[^ ]*)(.*)/$1$digest$3\n/;
         push @grub, $grub_str;
      } ## end while ( my $grub_str ...
      unless (close($grub_handler))
      {
         print STDERR "Cannot close file $grub_file: $!\n";
         return $FALSE;
      }
      unless (open($grub_handler, '>', $grub_file))
      {
         print STDERR "Cannot open file $grub_file: $!\n";
         return $FALSE;
      }
      print $grub_handler @grub;

      unless (close($grub_handler))
      {
         print STDERR "Cannot close file $grub_file: $!\n";
         return $FALSE;
      }
   }  ##end if
##end change grub.conf
   return $TRUE;
} ## end sub change_user_passwd

1;
__END__
