###############################################################################
## @file   CheckMedia.pm
# @brief   Check media CD module.
# @details This module introduces common variables and functions
#          for checking media CD for reading
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version
# @version 1.01 - Modified to use is_cppm() function.

package recovery_pm::CheckMedia;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use hwRecognition;
use FileHandle;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

## @var $mediaDev
#  Media CD path
my $mediaDev   = undef;

## @var $bksize
#  Checked block size (bytes)
my $bksize = 1024;

## @var $bkcount
#  Count of checked blocks
my $bkcount  = 0;
## @var $menu_str
#  Name of the option for the recovery console main menu
my $menu_str   = "Check installation media";
my %actions = ('menu'     => {'string' => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );

## @skipline
sub DefaultEntryPoint();
## @skipline
sub BlockCount($);
## @skipline
sub actions();


###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to check media CD module.
# @details This function allows user to check media CD
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008


sub DefaultEntryPoint()
{
   my $getbks;
   my $seek;
   my $pbwidth = 50;
   my $exit = $TRUE;
   if (is_cppm())
   {
      $mediaDev = '/dev/hdc1';
   }
   else
   {
      $mediaDev = '/tmp/cdrom';
   }
   $bkcount = BlockCount($mediaDev);
   $getbks  = int($bkcount / 100);

   system  "$STTY", "ctlecho", "-echo"; # input from keyboard is unable
   #Clear screen
   clear();
   print "$menu_str\n";
   printf("\r% -${pbwidth}s% 10s", '#' x '0', "[ 0% ]");  
   for (my $perc = 0 ; $perc < 101 ; $perc++)
   {  #Get next block for checking
      $seek = $getbks * ($perc);
      system ("$DD if=$mediaDev skip=$seek bs=$bksize count=$getbks of=/dev/null >/dev/null 2>&1");

      #if error is in block print error message and interrupt checking CD
      if ($?)
      {
         my $numhashes = int($perc/2);
         printf("\r% -${pbwidth}s% 10s", '#' x $numhashes, "[ " . $perc . "% ]");       
         print "\nError: Bad installation media. $perc percents were checked.\n";
         $exit = $FALSE;
         last;
      } ## end if ($?)

      #show progress bar
      else
      {         	 
          my $numhashes = int($perc/2);
          printf("\r% -${pbwidth}s% 10s", '#' x $numhashes, "[ " . $perc . "% ]");         
      } ## end else [ if ($?)
   } ## end for (my $perc = 0 ; $perc < 101...
   if ($exit)
   {
      print "\nChecking installation media has been completed. No errors reported.\n";
   }    
   print "Press <Enter> to exit:";   
   system "$STTY", "-ctlecho", "echo";
   <STDIN>;
    
} ## end sub DefaultEntryPoint

###############################################################################
## @fn sub BlockCount($target)
# @brief   Gets count of blocks which will be checked.
# @details Uses 'df' utility.
# @param   $target - target device
# @param   None
# @return  $blockcount - count of blocks.
# @author  Ekaterina Bazhanova
# @date    February  2008


sub BlockCount($)
{
   my $target = shift;
   my $blockcount;
   my @df = ();
   @df = `$DF`;
   foreach my $df_str (@df)
   {
      if ($df_str =~ /\A$target\D*(\d*).*/)
      {
         $blockcount = $1;
      }
   } ## end foreach my $df_str...
   return $blockcount;
} ## end sub BlockCount
1;
__END__
