###############################################################################
## @file   fsck.pm
# @brief   Check file system module.
# @details This module allows the check of the appropriate disk partition.
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version

package recovery_pm::fsck;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.00';

## @var @able_dev
#  List of the available devices for checking
my @able_dev = ();

## @var $exit_mess
#  Message before return to the root menu
my $exit_mess = "Press <Enter> to continue:";
my $pos = 0;
my $dev_partition;
my $menu_str ="Check file system";
my %actions = ('menu'     => {'string' => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );

## @skipline
sub DefaultEntryPoint();
## @skipline
sub GetDev($);
## @skipline
sub Err_handling($);
## @skipline
sub mount_point($);
## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to check file system module.
# @details This function allows creating of the check file system menu.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008


sub DefaultEntryPoint()
{    
   my $rootDev   = defDev('/');
   my $dev       = undef;
   if ($rootDev =~ /(\/dev\/\D+)\d+/) 
   {
      $dev = $1;
   }
   else
   {
      print "Error: Cannot define device for / partition\n";
      print $exit_mess;
      <STDIN>;
      return $FALSE;
   }
   @able_dev     = GetDev($dev);
   while ($TRUE)
   {
      $pos = 0;
      #Clear screen
      clear();
      print "$menu_str:\n";
      print " You may choose to do one of the following:\n";
      my $oldformat = $~;
      $~   = 'STDOUT';

      foreach my $ab_dev ( @able_dev )
      {
         $dev_partition = `$E2LABEL $ab_dev 2>/dev/null`;
         unless ($? == 0) {$dev_partition = "undefined";}
         chomp($dev_partition);
         $pos++;
         write STDOUT;

         format STDOUT = 
 @#)  Checking partition  @<<<<<<<<      (@<<<<<<<<<)
 $pos,                    $dev_partition,   $able_dev[$pos-1]
.
      } ## end foreach my $ab_dev( @able_dev)...
      $~ = $oldformat;
      print "  ", scalar @able_dev + 1, ")  Exit \n\n";

      print "Select an option (1-", scalar @able_dev + 1, "):";
      my $choice = <STDIN>;
      print "\n";
      chomp($choice);
      if ($choice =~ /^\s*(\d+)\s*$/)
      {
         $choice = $1;
      }
      else
      {  
         print "Wrong choice. Please repeat\n";
         print $exit_mess;
         <STDIN>;
         next;
      }
      if (($choice < (scalar @able_dev + 1)) && ($choice > 0))
      {
         my $mnt_point = mount_point($able_dev[$choice-1]);
         if ($mnt_point)
         {
            unless (system("$UMOUNT  $mnt_point 2>/dev/null")==0)
            {
               print "Partition has not been checked: cannot umount  $able_dev[$choice-1] \n";
               print $exit_mess;
               <STDIN>;
               next;
            }
         }
         system("$FSCK", "-yf", "$able_dev[$choice-1]");
         print Err_handling($? >> 8), "\n$exit_mess";         
         <STDIN>;
      } ## end if (($choice < (scalar...
      elsif ($choice == (scalar @able_dev + 1))
      {
         last;
      } 
      else
      {
         print "Wrong choice. Press <Enter> to repeat\n";
         <STDIN>;
         next;
      }
   }  ##end while ($TRUE)

} ## end sub DefaultEntryPoint


###############################################################################
## @fn sub GetDev($device)
# @brief   Creating list of available device.
# @details Uses "fdisk" utility
# @param   $device - file name of the device
# @return  @fsck - list of available device.
# @author  Ekaterina Bazhanova
# @date    February  2008


sub GetDev($)
{
   my $device = shift;
   my @fsck     = ();
   my $swap     = "82";
   my $extended = "5";
   my @fdisk    = `$FDISK -l $device`;
   foreach my $fdisk_str ( @fdisk )
   {
      if ($fdisk_str =~ /(\/dev\/\w+) \D*\d*\D*\d*\D*\d*\D*(\d*).*/)
      {
         unless (($2 eq $swap) | ($2 eq $extended))
         {
            push @fsck, $1;
         }
      }
   } ## end foreach my $fdisk_str...
   return @fsck;
} ## end sub GetDev

###############################################################################
## @fn sub Err_handling($Err_code)
# @brief   Creating list of errors.
# @details Decoding of "fsck" return value to list of errors 
# @param   $Err_code  - error code which "fsck" utility returns.
#          The exit code returned by fsck is the sum of the following conditions:
#            0    - No errors
#            1    - File system errors corrected
#            2    - System should be rebooted
#            4    - File system errors left uncorrected
#            8    - Operational error
#            16   - Usage or syntax error
#            32   - Fsck canceled by user request
#            128  - Shared library error
#          The  exit  code  returned when multiple file systems are checked 
#          is the bit-wise OR of the exit codes for each file system
#          that is checked.
# @return  @Err_list - list of errors.
# @author  Ekaterina Bazhanova
# @date    February  2008


sub Err_handling($)
{

   my $Err_code = shift;
   my @Err_list = ();
   if ($Err_code == 0)
   {
      return "No File System errors.\n";
   }
   if ($Err_code >= 128)
   {
      $Err_code = $Err_code - 128;
      @Err_list = (@Err_list, "Error 128: Shared library error\n");
   }
   if ($Err_code >= 32)
   {
      $Err_code = $Err_code - 32;
      @Err_list = (@Err_list, "Error  32: Fsck canceled by user request\n");
   }
   if ($Err_code >= 16)
   {
      $Err_code = $Err_code - 16;
      @Err_list = (@Err_list, "Error  16: Usage or syntax error\n");
   }
   if ($Err_code >= 8)
   {
      $Err_code = $Err_code - 8;
      @Err_list = (@Err_list, "Error   8: Operational error\n");
   }
   if ($Err_code >= 4)
   {
      $Err_code = $Err_code - 4;
      @Err_list = (@Err_list, "Error   4: File system errors left uncorrected\n");
   }
   if ($Err_code >= 2)
   {
      $Err_code = $Err_code - 2;
      @Err_list = (@Err_list, "Error   2: System should be rebooted\n");
   }
   if ($Err_code >= 1)
   {
      $Err_code = $Err_code - 1;
      @Err_list = (@Err_list, "Error   1: File system errors corrected\n");
   }
   return @Err_list;
} ## end sub Err_handling

###############################################################################
## @fn sub mount_point($device)
# @brief   Getting mount point of the appropriate device
# @details None
# @param   $device - device
# @return  undef - in case if mount point cannot be defined
# @return  $mpoint - mount point
# @author  Ekaterina Bazhanova
# @date    February  2008

sub mount_point($)
{
   my $device    = shift @_;
   my @mount  = `$MOUNT`;
   my $mpoint = undef;
   foreach my $mount_line (@mount)
   {
      if ($mount_line =~ /$device\s*on\s*(.*)\s*.*/)
      {
         $mpoint =  $1;
      } 
   } 
   return $mpoint;
} ## end sub mount_point($)

1;
__END__

