####################################################################
#
## @class syslog_ini
# @brief Interface to syslog management functionality.
# @details The class provides interface for managing syslog parameters.
# @author Alexander Khoroshilov
# @date December 2007
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
#
####################################################################

=pod

=head1 NAME

B<syslog_config.pm>

=head1 SYNOPSIS

B<use syslog_config.pm;>

=head1 DESCRIPTION

The class provides interface for managing syslog parameters.

=head1 SEE ALSO

B<syslogShow>, B<syslogFacilitySet> and B<syslogLevelSet> commands.

=cut

package syslog_config;

use lib "/opt/nortel/base/lib";
use common_variables;

use strict;
use warnings;
use Fcntl qw (:flock);  # import LOCK_* constants

## @var $VERSION
# Module version
our $VERSION = "1.01";

# Constants

# Name of facilities section
my $facilities_section_name = "FACILITIES";
my $slFACILITY              = "FAC";
my $slPROCESS               = "PROC";

# Hash containing syslog facilities with their number
my %syslog_facilities = (
                         0 => 'LOCAL0',
                         1 => 'LOCAL1',
                         2 => 'LOCAL2',
                         3 => 'LOCAL3',
                         4 => 'LOCAL4',
                         5 => 'LOCAL5',
                         6 => 'LOCAL6',
                         7 => 'LOCAL7'
                        );

# Hash containing syslog levels with their number
my %syslog_levels = (
                     -1 => 'NONE',
                     0  => 'EMERG',
                     1  => 'ALERT',
                     2  => 'CRIT',
                     3  => 'ERROR',
                     4  => 'WARNING',
                     5  => 'NOTICE',
                     6  => 'INFO',
                     7  => 'DEBUG'
                    );

## @method new($file)
# @brief Constructor for syslog_ini.
# @details
# @param $_[0] - ini file
# @return None
# @author Alexander Khoroshilov
# @date December 2007
sub new($$)
{
   unless (scalar(@_) == 2)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.");
      return undef;
   }
   my $class     = shift(@_);
   my $file_name = shift(@_);
   unless (defined($file_name))
   {
      baselog::baselog($baselog::LOG_ERROR, "Undefined name of file.");
      return undef;
   }
   my $self = {};
   $self->{FILE_NAME} = $file_name;
   bless($self, $class);

   baselog::baselog($baselog::LOG_DEBUG, "Successful new().");
   return $self;
} ## end sub new($$)

## @method parse_data(\@raw_data)
# @brief Parse data from array to hash.
# @details
# @param $_[0] - reference to array
# @return \%data - on success
# @return undef - on error
# @author Alexander Khoroshilov
# @date December 2007
sub parse_data($$)
{
   unless (scalar(@_) == 2)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.");
      return undef;
   }
   my ($self, $raw_data) = (@_);
   my $cur_section = undef;
   my %data;
   %{$data{$slFACILITY}} = ();
   %{$data{$slPROCESS}}  = ();

   foreach my $str (@{$raw_data})
   {
      if ($str =~ /(^\s*\#)|(^\s*$)/)  # Skip comments and blank lines
      {
         next;
      }

      if ($str =~ /\[(.+)\]/)          # Name of section
      {
         $cur_section = $1;
         next;
      }

      unless (defined($cur_section))
      {
         baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure: $str");
         return undef;
      }

      unless ($str =~ /^\s*(\S+)\s*=\s*(-*\d+)\s*$/)
      {
         baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure: $str");
         return undef;
      }

      if ($cur_section eq $facilities_section_name)
      {

         # check if facility is valid
         unless (exists($syslog_facilities{$2}))
         {
            baselog::baselog($baselog::LOG_ERROR, "Incorrect syslog facility value: $str");
            return undef;
         }
         $data{$slFACILITY}{$1} = $2;
      } ## end if ($cur_section eq $facilities_section_name...
      else
      {

         # check if level is valid
         unless (exists($syslog_levels{$2}))
         {
            baselog::baselog($baselog::LOG_ERROR, "Incorrect syslog level value: $str");
            return undef;
         }
         $data{$slPROCESS}{$cur_section}{$1} = $2;
      } ## end else [ if ($cur_section eq $facilities_section_name...
   } ## end foreach my $str (@{$raw_data...

   baselog::baselog($baselog::LOG_DEBUG, "Successful parse_data().");
   return \%data;
} ## end sub parse_data($$)

## @method show(@processes)
# @brief Print list of applications with associated facilities and permitted syslog levels
#        for tasks per application.
# @details Print information for given processes or for all processes if they are not specified.
# @param $_[0]... - process (optional)
# @return $TRUE - on success
# @return $FALSE - on error
# @author Alexander Khoroshilov
# @date December 2007
sub show($;@)
{
   unless (scalar(@_) > 0)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.", $TRUE);
      return $FALSE;
   }
   my $self = shift(@_);
   my @list = @_;
   my $fd;
   unless (open($fd, "<", $self->{FILE_NAME}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot open $self->{FILE_NAME} file.", $TRUE);
      return $FALSE;
   }
   flock($fd, LOCK_SH);

   my @raw_data = <$fd>;

   flock($fd, LOCK_UN);
   close($fd);

   my $data = $self->parse_data(\@raw_data);
   unless (defined($data))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.", $TRUE);
      return $FALSE;
   }

   if (scalar(@list))
   {
      foreach my $process (@_)
      {
         unless (exists($data->{$slPROCESS}{$process}))
         {
            baselog::baselog($baselog::LOG_ERROR, "Incorrect process argument $process.", $TRUE);
            return $FALSE;
         }
      } ## end foreach my $process (@_)
   } ## end if (scalar(@list))
   else
   {

      # combine unique processes to list
      @list = keys %{{%{$data->{$slFACILITY}}, %{$data->{$slPROCESS}}}};
   } ## end else [ if (scalar(@list))

   my $facility;
   foreach my $pkey (sort @list)
   {
      if (exists($data->{$slFACILITY}{$pkey}))
      {
         $facility = $syslog_facilities{$data->{$slFACILITY}{$pkey}};
      }
      else
      {
         $facility = '';
      }
      print <<TOPLINES;

Process: $pkey
Facility: $facility
----------------------------
Task                 Level

TOPLINES
      foreach my $tkey (sort keys %{$data->{$slPROCESS}{$pkey}})
      {
         printf("%-20s %s\n", $tkey, $syslog_levels{$data->{$slPROCESS}{$pkey}{$tkey}});
      }
      print "\n";
   } ## end foreach my $pkey (sort @list...

   baselog::baselog($baselog::LOG_DEBUG, "Successful show().");
   return $TRUE;
} ## end sub show($;@)

## @method change_value(/@data, $section, $name, $value)
# @brief Update value for given section and name in array.
# @details
# @param $_[0] - reference to array
# @param $_[1] - section
# @param $_[2] - name
# @param $_[3] - value
# @return $TRUE - on success
# @return $FALSE - on error
# @author Alexander Khoroshilov
# @date December 2007
sub change_value($$$$$)
{
   unless (scalar(@_) == 5)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.");
      return $FALSE;
   }
   my $self = shift(@_);
   my ($raw_data, $section, $name, $value) = (@_);
   my $cur_section = undef;

   foreach my $str (@{$raw_data})
   {
      if ($str =~ /(^\s*\#)|(^\s*$)/)  # Skip comments and blank lines
      {
         next;
      }

      if ($str =~ /\[(.+)\]/)          # Name of section
      {
         $cur_section = $1;
         next;
      }

      unless (defined($cur_section))
      {
         baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure: $str");
         return $FALSE;
      }

      if ($cur_section eq $section)
      {
         unless ($str =~ /^\s*(\S+)\s*=\s*-*\d+\s*$/)
         {
            baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure: $str");
            return $FALSE;
         }

         if ($1 eq $name)
         {

            # change line to new value
            $str =~ s/^(\s*\S+\s*=\s*)-*\d+(\s*)$/$1$value$2/;
            baselog::baselog($baselog::LOG_DEBUG, "Successful change_value().");
            return $TRUE;
         } ## end if ($1 eq $name)
      } ## end if ($cur_section eq $section...
   } ## end foreach my $str (@{$raw_data...

   baselog::baselog($baselog::LOG_ERROR, "Cannot find $section section, $name name.");
   return $FALSE;
} ## end sub change_value($$$$$)

## @method set_facility($process, $fasility)
# @brief Update facility for given process in INI file.
# @details
# @param $_[0] - process
# @param $_[1] - facility
# @return $TRUE - on success
# @return $FALSE - on error
# @author Alexander Khoroshilov
# @date December 2007
sub set_facility($$$)
{
   unless (scalar(@_) == 3)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.", $TRUE);
      return $FALSE;
   }
   my $self = shift(@_);
   my ($process, $facility) = @_;

   my %facilities = reverse %syslog_facilities;
   $facility = uc($facility);
   unless (exists($facilities{$facility}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect facility argument.", $TRUE);
      return $FALSE;
   }
   $facility = $facilities{$facility};

   my $fd;
   unless (open($fd, "+<", $self->{FILE_NAME}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot open $self->{FILE_NAME} file.", $TRUE);
      return $FALSE;
   }
   flock($fd, LOCK_EX);

   my @raw_data = <$fd>;

   my $data = $self->parse_data(\@raw_data);
   unless ($data)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.", $TRUE);
      return $FALSE;
   }

   unless (exists($data->{$slFACILITY}{$process}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect process argument.", $TRUE);
      return $FALSE;
   }

   # Change facility in @raw_data
   unless ($self->change_value(\@raw_data, $facilities_section_name, $process, $facility))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.", $TRUE);
      return $FALSE;
   }

   # Rewrite new data in the file
   seek($fd, 0, 0);
   print $fd @raw_data;
   truncate($fd, tell($fd));
   flock($fd, LOCK_UN);
   close($fd);

   baselog::baselog($baselog::LOG_INFO, "To take effect $process may need restart.", $TRUE);
   baselog::baselog($baselog::LOG_DEBUG, "Successful set_facility().");
   return $TRUE;
} ## end sub set_facility($$$)

## @method set_level($process, $task, $level)
# @brief Update level for given process and task in INI file.
# @details
# @param $_[0] - process
# @param $_[1] - task
# @param $_[2] - level
# @return $TRUE - on success
# @return $FALSE - on error
# @author Alexander Khoroshilov
# @date December 2007
sub set_level($$$$)
{
   unless (scalar(@_) == 4)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.", $TRUE);
      return $FALSE;
   }
   my $self = shift(@_);
   my ($process, $task, $level) = @_;

   unless (exists($syslog_levels{$level}))
   {
      my %levels = reverse %syslog_levels;
      $level = uc($level);
      unless (exists($levels{$level}))
      {
         baselog::baselog($baselog::LOG_ERROR, "Incorrect level argument.", $TRUE);
         return $FALSE;
      }
      $level = $levels{$level};
   } ## end unless (exists($syslog_levels...

   my $fd;
   unless (open($fd, "+<", $self->{FILE_NAME}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot open $self->{FILE_NAME} file.", $TRUE);
      return $FALSE;
   }
   flock($fd, LOCK_EX);

   my @raw_data = <$fd>;

   my $data = $self->parse_data(\@raw_data);
   unless ($data)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.", $TRUE);
      return $FALSE;
   }

   unless (exists($data->{$slPROCESS}{$process}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect process argument.", $TRUE);
      return $FALSE;
   }

   unless (exists($data->{$slPROCESS}{$process}{$task}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect task argument.", $TRUE);
      return $FALSE;
   }

   # Change level in @raw_data
   unless ($self->change_value(\@raw_data, $process, $task, $level))
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.", $TRUE);
      return $FALSE;
   }

   # Rewrite new data in the file
   seek($fd, 0, 0);
   print $fd @raw_data;
   truncate($fd, tell($fd));
   flock($fd, LOCK_UN);
   close($fd);

   baselog::baselog($baselog::LOG_DEBUG, "Successful set_level().");
   return $TRUE;
} ## end sub set_level($$$$)

## @method get_facility($process)
# @brief Get facility for given process from INI file.
# @details
# @param $_[0] - process
# @return facility - on success
# @return undef - on error
# @author Alexander Khoroshilov
# @date December 2007
sub get_facility($$)
{
   unless (scalar(@_) == 2)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.");
      return undef;
   }
   my $self    = shift(@_);
   my $process = shift(@_);

   my $fd;
   unless (open($fd, "<", $self->{FILE_NAME}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot open $self->{FILE_NAME} file.");
      return undef;
   }
   flock($fd, LOCK_SH);

   my @raw_data = <$fd>;

   flock($fd, LOCK_UN);
   close($fd);

   my $data = $self->parse_data(\@raw_data);
   unless ($data)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.");
      return undef;
   }

   unless (exists($data->{$slFACILITY}{$process}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot find $facilities_section_name section, $process name.");
      return undef;
   }

   baselog::baselog($baselog::LOG_DEBUG, "Successful get_facility().");
   return $data->{$slFACILITY}{$process};
} ## end sub get_facility($$)

## @method get_level($process, $task)
# @brief Get level for given process and task from INI file.
# @details
# @param $_[0] - process
# @param $_[1] - task
# @return level - on success
# @return undef - on error
# @author Alexander Khoroshilov
# @date December 2007
sub get_level($$$)
{
   unless (scalar(@_) == 3)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect number of arguments.");
      return undef;
   }
   my $self = shift(@_);
   my ($process, $task) = @_;

   my $fd;
   unless (open($fd, "<", $self->{FILE_NAME}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot open $self->{FILE_NAME} file.");
      return undef;
   }
   flock($fd, LOCK_SH);

   my @raw_data = <$fd>;

   flock($fd, LOCK_UN);
   close($fd);

   my $data = $self->parse_data(\@raw_data);
   unless ($data)
   {
      baselog::baselog($baselog::LOG_ERROR, "Incorrect $self->{FILE_NAME} file structure.");
      return undef;
   }

   unless (exists($data->{$slPROCESS}{$process}{$task}))
   {
      baselog::baselog($baselog::LOG_ERROR, "Cannot find $process section, $task name.");
      return undef;
   }

   return $data->{$slPROCESS}{$process}{$task};
} ## end sub get_level($$$)

1;

__END__;
