################################################
# @file  xmlvalid.pm
#
# @brief
#   XML validating subroutines
#
# @details  This module contains subroutines used
#    for validating XML structure
#
# @author Pavel Sergeev
# @date December 2007
#
# @verion 1.0 - initial version
################################################

package xmlvalid;

use lib "/opt/nortel/base/lib";

use Exporter;
@ISA    = "Exporter";
@EXPORT = qw(
  isUnknownNode
  isRequiredNode isUnknownAttr isWrongHierarchy
  isRequiredAttr isDupAttr isSubsetAttr
  isWrongType isValidXml isValidConfig isDupNode
  printErrorList checkXmlFiles
  );

use strict;
use warnings;
use baselog;
use common_variables;
use common_functions;
use XML::LibXML;
use File::Spec;

our $error   = $TRUE;  #global variable for detecting errors
our $VERSION = '1.0';
my $LibXMLDoc = "XML::LibXML::Document";

#####################################################
# @fn isValidConfig($configdoc, $errstr)
# @brief
#   checks the configuration file structure.
# @param $configdoc - the document object
# @param $errstr - if any error found this variable will
#         contain a error description.
# @return
#   $TRUE - configuration file is valid
#   $FALSE - configuration file is invalid
# @author Pavel Sergeev
# @date February 2008
#####################################################
sub isValidConfig($$)
{
   my ($configdoc, $errstr) = @_;
   my (@nodes, @attrs) = ((), ());  #arrays for nodes and attributes in config file
   my $attr = '';
   my $node = '';
   my (%repnodenames, %repattrnames) = ((), ());
   my $rootcount = 0;

   $$errstr = '';
   if ((ref($configdoc) ne $LibXMLDoc) or (ref($errstr) ne "SCALAR"))
   {
      baselog($LOG_ERROR, "Argument is invalid.");
      return $FALSE;
   }

   if ((!($configdoc->getElementsByTagName('config')) or (!($configdoc->getElementsByTagName('xmlnodes')))))
   {
      $$errstr = "The 'config' or the 'xmlnodes' tag is absent.\n";
      return $FALSE;
   }
   if (@{$configdoc->getElementsByTagName('config')} > 1)  ##If count of the 'config' tags is more than one
   {
      $$errstr = "More than one 'config' tags.\n";
      return $FALSE;
   }
   if (@{$configdoc->getElementsByTagName('xmlnodes')} > 1)  ##if count of the 'xmlnodes' is more than one
   {
      $$errstr = "More than one 'xmlnodes' tags.\n";
      return $FALSE;
   }
   if (${$configdoc->getElementsByTagName('xmlnodes')}[0]->parentNode->nodeName ne 'config')  ##if parent of the 'xmlnodes' isn't the 'config' tag
   {
      $$errstr = "Wrong hierarchy of the 'xmlnodes'\n";
      return $FALSE;
   }
   if (!($configdoc->firstChild->isEqual(${$configdoc->getElementsByTagName('config')}[0])))
   {
      $$errstr = "Wrong root tag\n";
      return $FALSE;
   }
   foreach $node (${$configdoc->getElementsByTagName('config')}[0]->getElementsByTagName('*'))
   {
      if (   (($node->nodeName ne 'node') and ($node->parentNode->nodeName eq 'xmlnodes'))
          or (($node->nodeName ne 'attr')   and ($node->nodeName ne 'parent') and ($node->parentNode->nodeName eq 'node'))
          or (($node->nodeName ne 'unique') and ($node->nodeName ne 'source') and ($node->parentNode->nodeName eq 'attr'))
          or (($node->nodeName ne 'in') and ($node->parentNode->nodeName eq 'source'))
          or (($node->nodeName ne 'xmlnodes') and ($node->nodeName ne 'types') and ($node->parentNode->nodeName eq 'config'))
          or (($node->nodeName ne 'val') and (($node->parentNode->nodeName =~ /^invalid$|^valid$|^regexpinval$/))))
      {
         $$errstr = "An unknown node tag '" . $node->nodeName . "'\n";
         return $FALSE;
      } ## end if ((($node->nodeName ...
   } ## end foreach $node (${$configdoc...
     # get 'node' tags
   my $xmlnodes = ${$configdoc->getElementsByTagName('xmlnodes')}[0];
   if (@nodes = $xmlnodes->getElementsByTagName('node'))
   {
      foreach $node (@nodes)
      {
         foreach my $attr ($node->getAttributes)
         {

            # checking attributes in the 'node' tag
            if (($attr->nodeName ne 'name') and ($attr->nodeName ne 'required') and ($attr->nodeName ne 'text'))
            {
               $$errstr = "An unknown attribute '" . $attr->nodeName . "'\n";
               return $FALSE;
            }
         } ## end foreach my $attr ($node->getAttributes...
         if (!($node->attributes->getNamedItem('name')))  ##if the 'name' attribute isn't exist in the 'node'
         {
            $errstr = "The 'name' attribute is absent\n";
            return $FALSE;
         }

         # get 'attr' tags
         if (@attrs = $node->getElementsByTagName('attr'))
         {
            %repattrnames = ();                           ##delete previous attribute values
            foreach $attr (@attrs)
            {
               foreach my $attribute ($attr->getAttributes)  ##looking for unknown attributes
               {
                  if (($attribute->nodeName ne 'name') and ($attribute->nodeName ne 'required') and ($attribute->nodeName ne 'type') and ($attribute->nodeName ne 'defvalue'))
                  {
                     $$errstr = "An unknown attribute '" . $attr->nodeName . "'\n";
                     return $FALSE;
                  }
               } ## end foreach my $attribute ($attr...
               if (!($attr->attributes->getNamedItem('name')))  ##if the 'name' attribute isn't exists in the 'attr'
               {
                  $$errstr = "The 'name' attribute is absent\n";
                  return $FALSE;
               }
               ##fill 'name' values hash. If hash element exists this means duplicate attributes
               if (!($repattrnames{$attr->attributes->getNamedItem('name')->value}))
               {
                  $repattrnames{$attr->attributes->getNamedItem('name')->value} = '1';
               }
               else
               {
                  $$errstr = "Duplicate attributes. Attribute value: " . $attr->attributes->getNamedItem('name')->value . "\n";
                  return $FALSE;
               }
            } ## end foreach $attr (@attrs)
         } ## end if (@attrs = $node->getElementsByTagName...
         foreach my $parent_node ($node->getElementsByTagName('parent'))
         {
            foreach my $parent_attr ($parent_node->getAttributes)
            {
               if (($parent_attr->nodeName ne 'unique') and ($parent_attr->nodeName ne 'required'))
               {
                  $$errstr = "An unknown attribute '" . $parent_attr->nodeName . "'\n";
                  return $FALSE;
               }
            } ## end foreach my $parent_attr ($parent_node...
         } ## end foreach my $parent_node ($node...
         if (!($repnodenames{$node->attributes->getNamedItem('name')->value}))
         {
            $repnodenames{$node->attributes->getNamedItem('name')->value} = '1';
         }
         else
         {
            $$errstr = "Duplicate node names. Name: " . $node->attributes->getNamedItem('name')->value . "\n";
            return $FALSE;
         }
         if (!($node->getElementsByTagName('parent')))  #if the 'parent' tag is absent
         {
            $rootcount++;                               #count of  the 'parent' nodes
         }
      } ## end foreach $node (@nodes)
      if ($rootcount != 1)                              # a root node has to be the only one
      {
         $$errstr = "Wrong count of the 'root' nodes.\n";
         return $FALSE;
      }
   } ## end if (@nodes = $xmlnodes...
   return $TRUE;
} ## end sub isValidConfig($$)

########################################################
# @fn isUnknownNodes($confignodes, $xnodes, $errlist)
# @brief looks for the unknown nodes
# @param $confignodes - configuration file nodes
# @param $xmlnodes - xml file nodes
# @param $errlist - array reference for errors
# @return
#   $TRUE - an unknown node was detected
#   $FALSE - unknown nodes are absent
# @author Pavel Sergeev
# @date November 2007
########################################################
sub isUnknownNode($$$)
{
   my ($confignodes, $xnodes, $errlist) = @_;
   if (not is_array($errlist) or not is_array($confignodes) or not is_array($xnodes))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my $retval = $FALSE;
   foreach my $xnode (@$xnodes)
   {
      $error = $TRUE;
      foreach my $cnode (@$confignodes)
      {
         if ($xnode->nodeName eq $cnode->attributes->getNamedItem('name')->getValue)
         {
            $error = $FALSE;
         }
      } ## end foreach my $cnode (@$confignodes...
      if ($error == $TRUE)
      {
         push @$errlist,
           {
            "line"     => $xnode->line_number,
            "nodename" => $xnode->nodeName,
            "nodepath" => $xnode->nodePath
           };
         $retval = $TRUE;
      } ## end if ($error == $TRUE)
   } ## end foreach my $xnode (@$xnodes...
   return $retval;
} ## end sub isUnknownNode($$$)

##############################################
# @fn isRequiredNodes($xmldoc, $cnodes, $errlist)
# @brief looks for required nodes that are absent in the XML file
# @param $xmldoc - xml file documents object
# @param $cnodes - configuration file nodes
# @param $errlist - array reference for errors
# @return
#          $TRUE - required node is absent
#          $FALSE - no errors
# @author Pavel Sergeev
# @date November 2007
#############################################
sub isRequiredNode($$$)
{
   my ($xmldoc, $cnodes, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or (not is_array($errlist)) or (not is_array($cnodes)))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my @parentnodes = ();
   my $retval      = $FALSE;
   foreach my $cnode (@$cnodes)
   {
      if ($cnode->attributes->getNamedItem('required'))
      {
         @parentnodes = $cnode->getElementsByTagName('parent');
         foreach my $pnode (@parentnodes)
         {
            foreach my $xnode (@{$xmldoc->findnodes($pnode->textContent)})
            {
               if ($cnode->attributes->getNamedItem('required')->getValue)
               {
                  if (!($xnode->getElementsByTagName($cnode->attributes->getNamedItem('name')->getValue)))
                  {
                     push @$errlist,
                       {
                        "line"     => $xnode->line_number,
                        "nodename" => $cnode->attributes->getNamedItem('name')->getValue,
                        "nodepath" => $xnode->nodePath
                       };
                     $retval = $TRUE;
                  } ## end if (!($xnode->getElementsByTagName...
                  else
                  {
                     $error = $TRUE;
                     foreach my $node ($xnode->getElementsByTagName($cnode->attributes->getNamedItem('name')->getValue))
                     {
                        if ($node->parentNode->nodeName eq $xnode->nodeName)
                        {
                           $error = $FALSE;
                        }
                     } ## end foreach my $node ($xnode->getElementsByTagName...
                     if ($error == $TRUE)
                     {
                        push @$errlist,
                          {
                           "line"     => $xnode->line_number,
                           "nodename" => $cnode->attributes->getNamedItem('name')->getValue,
                           "nodepath" => $xnode->nodePath
                          };
                        $retval = $TRUE;
                     } ## end if ($error == $TRUE)
                  } ## end else [ if (!($xnode->getElementsByTagName...
               } ## end if ($cnode->attributes...
            } ## end foreach my $xnode (@{$xmldoc...
         } ## end foreach my $pnode (@parentnodes...
      } ## end if ($cnode->attributes...
      else
      {
         foreach my $pnode (@{$cnode->getElementsByTagName('parent')})
         {
            if (defined($pnode->attributes->getNamedItem('required')) and ($pnode->attributes->getNamedItem('required')->value))
            {
               foreach my $xnode (@{$xmldoc->findnodes($pnode->textContent)})
               {
                  if (!($xnode->getElementsByTagName($cnode->attributes->getNamedItem('name')->getValue)))
                  {
                     push @$errlist,
                       {
                        "line"     => $xnode->line_number,
                        "nodename" => $cnode->attributes->getNamedItem('name')->getValue,
                        "nodepath" => $xnode->nodePath
                       };
                     $retval = $TRUE;
                  } ## end if (!($xnode->getElementsByTagName...
                  else
                  {
                     $error = $TRUE;
                     foreach my $node ($xnode->getElementsByTagName($cnode->attributes->getNamedItem('name')->getValue))
                     {
                        if ($node->parentNode->nodeName eq $xnode->nodeName)
                        {
                           $error = $FALSE;
                        }
                     } ## end foreach my $node ($xnode->getElementsByTagName...
                     if ($error == $TRUE)
                     {
                        push @$errlist,
                          {
                           "line"     => $xnode->line_number,
                           "nodename" => $cnode->attributes->getNamedItem('name')->getValue,
                           "nodepath" => $xnode->nodePath
                          };
                        $retval = $TRUE;
                     } ## end if ($error == $TRUE)
                  } ## end else [ if (!($xnode->getElementsByTagName...
               } ## end foreach my $xnode (@{$xmldoc...
            } ## end if (defined($pnode->attributes...
         } ## end foreach my $pnode (@{$cnode...
      } ## end else [ if ($cnode->attributes...
   } ## end foreach my $cnode (@$cnodes...
   return $retval;
} ## end sub isRequiredNode($$$)

################################################
# @fn isWrongnHierarchy($xmldoc, $cnodes, $xnodes, $errlist)
# @brief detects a wrong nesting hierarchy
# @param $xmldoc - xml file documents object
# @param $cnodes - configuration file nodes
# @param $xnodes - xml file nodes
# @param $errlist - array reference for errors
# @return
#          $TRUE - wrong nesting hierarchy detected
#          $FALSE - no errors
# @author Pavel Sergeev
# @date November 2007
################################################
sub isWrongHierarchy($$$$)
{
   my ($xmldoc, $cnodes, $xnodes, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or not is_array($errlist) or not is_array($cnodes) or not is_array($xnodes))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my %parenthash = ();
   my $retval     = $FALSE;
   foreach my $config_node (@$cnodes)
   {
      $parenthash{$config_node->attributes->getNamedItem('name')->getValue} = $config_node->getElementsByTagName('parent');
   }
   foreach my $xnode (@$xnodes)
   {
      $error = $TRUE;
      if ($parenthash{$xnode->nodeName})
      {
         foreach my $cnode (@{$parenthash{$xnode->nodeName}})
         {
            foreach my $xpathnode (@{$xmldoc->findnodes($cnode->textContent)})  ##nodes found by XPath
            {
               if ($xnode->parentNode->isEqual($xpathnode))
               {
                  $error = $FALSE;
               }
            } ## end foreach my $xpathnode (@{$xmldoc...
         } ## end foreach my $cnode (@{$parenthash...
      } ## end if ($parenthash{$xnode...
      else
      {
         $error = $FALSE;
      }
      if ($error == $TRUE)
      {
         push @$errlist,
           {
            "line"      => $xnode->line_number,
            "nodename"  => $xnode->nodeName,
            "pnodename" => ($xnode->isEqual($xmldoc->documentElement)) ? "#Document" : $xnode->parentNode->nodeName,
            "nodepath"  => $xnode->nodePath
           };
         $retval = $TRUE;
      } ## end if ($error == $TRUE)
   } ## end foreach my $xnode (@$xnodes...
   return $retval;
} ## end sub isWrongHierarchy($$$$)

#################################################
# @fn isUnknownAttr($cnodes, $xnodes, $attrshash, $errlist)
# @brief looks for the unknown attributes
# @param $cnodes - configuration file nodes
# @param $xnodes - xml file nodes
# @param $attrshash - attributes hash
# @param $errlist - array reference for errors
# @return
#          $TRUE - an unknown attribute was found
#          $FALSE - no errors
# @author Pavel Sergeev
# @date December 2007
#################################################
sub isUnknownAttr($$$$)
{
   my ($cnodes, $xnodes, $attrshash, $errlist) = @_;
   if (not is_array($errlist) or not is_array($cnodes) or not is_array($xnodes) or not is_hash($attrshash))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my $retval = $FALSE;

   foreach my $xnode (@$xnodes)
   {
      if ($xnode->hasAttributes)
      {
         foreach my $attr ($xnode->attributes)  #attributes (if a node has an attributes)
         {
            $error = $TRUE;
            if ($$attrshash{$xnode->nodeName})
            {
               foreach my $attrtag (@{$$attrshash{$xnode->nodeName}})  #"attr" tags
               {
                  if ($attr->nodeName eq $attrtag->attributes->getNamedItem('name')->value)
                  {
                     $error = $FALSE;
                  }
               } ## end foreach my $attrtag (@{$$attrshash...
               if ($error == $TRUE)
               {
                  push @$errlist,
                    {
                     "line"     => $xnode->line_number,
                     "nodename" => $xnode->nodeName,
                     "attrname" => $attr->nodeName,
                     "nodepath" => $xnode->nodePath
                    };
                  $retval = $TRUE;
               } ## end if ($error == $TRUE)
            } ## end if ($$attrshash{$xnode...
            else
            {
               push @$errlist,
                 {
                  "line"     => $xnode->line_number,
                  "nodename" => $xnode->nodeName,
                  "attrname" => $attr->nodeName,
                  "nodepath" => $xnode->nodePath
                 };
               $retval = $TRUE;
            } ## end else [ if ($$attrshash{$xnode...
         } ## end foreach my $attr ($xnode->attributes...
      } ## end if ($xnode->hasAttributes...
   } ## end foreach my $xnode (@$xnodes...
   return $retval;
} ## end sub isUnknownAttr($$$$)

##################################################
# @fn isRequiredAttr($cnodes, $xnodes, $attrshash, $errlist)
# @brief searches the required attributes that are absent in the XML file
# @param $cnodes - configuration file nodes
# @param $xnodes - xml file nodes
# @param $attrshash - attributes hash
# @param $errlist - array reference for errors
# @return
#          $TRUE - required attribute is absent
#          $FALSE - no errors
# @author Pavel Sergeev
# @date November 2007
##################################################
sub isRequiredAttr($$$$)
{
   my ($cnodes, $xnodes, $attrshash, $errlist) = @_;
   if (not is_array($errlist) or not is_array($cnodes) or not is_array($xnodes) or not is_hash($attrshash))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my @attrlist = ();
   my $attrtag;
   my $retval = $FALSE;

   foreach my $xnode (@$xnodes)
   {
      if (($xnode->hasAttributes) and ($$attrshash{$xnode->nodeName}))
      {
         foreach $attrtag (@{$$attrshash{$xnode->nodeName}})
         {
            $error = $TRUE;
            if ($attrtag->attributes->getNamedItem('required'))
            {
               if (!($xnode->attributes->getNamedItem($attrtag->attributes->getNamedItem('name')->getValue))
                   and (!($attrtag->attributes->getNamedItem('required')->getValue)))
               {
                  $error = $FALSE;
               }
               if (
                   ($xnode->attributes->getNamedItem($attrtag->attributes->getNamedItem('name')->getValue))
                   and (   ($attrtag->attributes->getNamedItem('required')->getValue)
                        or (!($attrtag->attributes->getNamedItem('required')->getValue)))
                  )
               {
                  $error = $FALSE;
               } ## end if (($xnode->attributes...
            } ## end if ($attrtag->attributes...
            else
            {
               $error = $FALSE;
            }
            if ($error == $TRUE)
            {
               push @$errlist,
                 {
                  "line"     => $xnode->line_number,
                  "nodename" => $xnode->nodeName,
                  "attrname" => $attrtag->attributes->getNamedItem('name')->getValue,
                  "nodepath" => $xnode->nodePath
                 };
               $retval = $TRUE;
            } ## end if ($error == $TRUE)
         } ## end foreach $attrtag (@{$$attrshash...
      } ## end if (($xnode->hasAttributes...
      else
      {
         foreach $attrtag (@{$$attrshash{$xnode->nodeName}})
         {
            if ($attrtag->attributes->getNamedItem('required'))
            {
               if ($attrtag->attributes->getNamedItem('required')->value)
               {
                  push @$errlist,
                    {
                     "line"     => $xnode->line_number,
                     "nodename" => $xnode->nodeName,
                     "attrname" => $attrtag->attributes->getNamedItem('name')->getValue,
                     "nodepath" => $xnode->nodePath
                    };
                  $retval = $TRUE;
               } ## end if ($attrtag->attributes...
            } ## end if ($attrtag->attributes...
         } ## end foreach $attrtag (@{$$attrshash...
      } ## end else [ if (($xnode->hasAttributes...
   } ## end foreach my $xnode (@$xnodes...
   return $retval;
} ## end sub isRequiredAttr($$$$)

############################################
# @fn isDupAttr($xmldoc, $cnodes, $errlist)
# @brief checks for unique attributes
# @details looks for attributes that have duplicate values.
# If duplicate values exist $TRUE is returned, and information is stored in the $errlist.
# @param $xdoc - xml file documents object
# @param $cnodes - configuration file nodes
# @param $errlist - array reference for errors
# @return
#          $TRUE - an attribute is not unique
#          $FALSE - no errors
# @author Pavel Sergeev
# @date November 2007
###########################################
sub isDupAttr($$$)
{
   my ($xmldoc, $cnodes, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or not is_array($errlist) or not is_array($cnodes))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my %uniqhash = ();
   my @nodelist = ();
   my $retval   = $FALSE;
   foreach my $config_node (@$cnodes)
   {
      foreach my $uniq_node ($config_node->getElementsByTagName('unique'))  ##'unique' nodes
      {
         push @{$uniqhash{$uniq_node->parentNode->parentNode->attributes->getNamedItem('name')->getValue}}, $uniq_node;
      }
   } ## end foreach my $config_node (@$cnodes...
   foreach my $name (keys %uniqhash)                                        #enumerate node names
   {
      if ($uniqhash{$name})                                                 #if unique tags exist for current node ($name)
      {
         foreach my $cnode (@{$uniqhash{$name}})
         {
            foreach my $xnode (@{$xmldoc->findnodes($cnode->textContent)})  #get nodes (names in unique)
            {
               @nodelist = @{$xnode->getElementsByTagName($name)};
               for (my $i = 0 ; $i < @nodelist - 1 ; $i++)                  #get nodes needed to check
               {
                  for (my $j = $i + 1 ; $j < @nodelist ; $j++)
                  {
                     if (    defined($nodelist[$i]->attributes->getNamedItem($cnode->parentNode->attributes->getNamedItem("name")->value))
                         and defined($nodelist[$j]->attributes->getNamedItem($cnode->parentNode->attributes->getNamedItem("name")->value)))
                     {
                        if (
                            (
                             $nodelist[$i]->attributes->getNamedItem($cnode->parentNode->attributes->getNamedItem("name")->value)->value eq
                             $nodelist[$j]->attributes->getNamedItem($cnode->parentNode->attributes->getNamedItem("name")->value)->value
                            )
                            and ($nodelist[$i]->parentNode->nodeName eq $xnode->nodeName)
                            and ($nodelist[$j]->parentNode->nodeName eq $xnode->nodeName)
                           )
                        {
                           push @$errlist,
                             {
                              "line"      => $nodelist[$i]->line_number,
                              "nodename"  => $name,
                              "attrname"  => $cnode->parentNode->attributes->getNamedItem("name")->value,
                              "attrvalue" => $nodelist[$i]->attributes->getNamedItem($cnode->parentNode->attributes->getNamedItem("name")->value)->value,
                              "nodepath"  => $nodelist[$i]->nodePath
                             };
                           $retval = $TRUE;
                        } ## end if (($nodelist[$i]->attributes...
                     } ## end if (defined($nodelist[...
                  } ## end for (my $j = $i + 1 ; $j...
               } ## end for (my $i = 0 ; $i < @nodelist...
            } ## end foreach my $xnode (@{$xmldoc...
         } ## end foreach my $cnode (@{$uniqhash...
      } ## end if ($uniqhash{$name})
   } ## end foreach my $name (keys %uniqhash...
   return $retval;
} ## end sub isDupAttr($$$)

#####################################################
# @fn isSubsetAttr($xmldoc, $cnodes, $errlist)
# @brief checks attributes subsets
# @details This subroutine collects attribute values from one tag, then
# it compares these values with the set of the attribute values from another tag.
# @param $xdoc - xml file documents object
# @param $cnodes - configuration file nodes
# @param $errlist - array reference for errors
# @return
#          $TRUE - wrong attribute values subset
#          $FALSE - no errors
# @author Pavel Sergeev
# @date December 2007
#####################################################
sub isSubsetAttr($$$)
{
   my ($xmldoc, $cnodes, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or not is_array($errlist) or not is_array($cnodes))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my @attrvalues = ();
   my $attrname   = '';
   my $node       = '';
   my $retval     = $FALSE;
   foreach my $cnode (@$cnodes)
   {
      my $nodename = $cnode->attributes->getNamedItem('name')->value;
      foreach my $source (@{$cnode->getElementsByTagName('source')})  #"source" tags
      {
         $attrname = $source->parentNode->attributes->getNamedItem('name')->value;  #attribute name
         foreach my $node (@{$xmldoc->findnodes($source->attributes->getNamedItem('tag')->value)})
         {
            foreach my $xnode (@{$node->getElementsByTagName($cnode->attributes->getNamedItem('name')->value)})
            {
               if (($xnode->parentNode->isEqual($node) and defined($xnode->attributes->getNamedItem($attrname))))
               {
                  push @attrvalues, $xnode->attributes->getNamedItem($attrname)->value;
               }
            } ## end foreach my $xnode (@{$node->getElementsByTagName...
         } ## end foreach my $node (@{$xmldoc...
         foreach my $in (@{$source->getElementsByTagName('in')})
         {
            foreach my $innode (@{$xmldoc->findnodes($in->textContent)})
            {
               foreach my $subnode (@{$innode->getElementsByTagName($nodename)})
               {
                  $error = $TRUE;
                  foreach my $attr_value (@attrvalues)
                  {
                     if ($subnode->attributes->getNamedItem($attrname))
                     {
                        if ($subnode->attributes->getNamedItem($attrname)->value eq $attr_value)
                        {
                           $error = $FALSE;
                        }
                     } ## end if ($subnode->attributes...
                  } ## end foreach my $attr_value (@attrvalues...
                  if ($error == $TRUE)
                  {
                     if ($subnode->attributes->getNamedItem($attrname))
                     {
                        push @$errlist,
                          {
                           "line"      => $subnode->line_number,
                           "nodename"  => $subnode->nodeName,
                           "attrname"  => $attrname,
                           "attrvalue" => $subnode->attributes->getNamedItem($attrname)->value,
                           "nodepath"  => $subnode->nodePath
                          };
                        $retval = $TRUE;
                     } ## end if ($subnode->attributes...
                  } ## end if ($error == $TRUE)
               } ## end foreach my $subnode (@{$innode...
            } ## end foreach my $innode (@{$xmldoc...
         } ## end foreach my $in (@{$source->getElementsByTagName...
      } ## end foreach my $source (@{$cnode...
   } ## end foreach my $cnode (@$cnodes...
   return $retval;
} ## end sub isSubsetAttr($$$)

#####################################################
# @fn isWrongType($xmldoc, $configdoc, $cnodes, $attrshash, $errlist)
# @brief checks attribute values
# @param $xdoc - xml file documents object
# @param $configdoc - xml scheme document object
# @param $cnodes - configuration file nodes
# @param $attrshash - attributes hash reference
# @param $errlist - array reference for errors
# @return
#          $TRUE - wrong attribute value detected
#          $FALSE - no errors
# @author Pavel Sergeev
# @date January 2008
#####################################################
sub isWrongType($$$$$)
{
   my ($xmldoc, $configdoc, $cnodes, $attrshash, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or (ref($configdoc) ne $LibXMLDoc) or not is_array($cnodes) or not is_array($errlist) or not is_hash($attrshash))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my %attrhash         = ();
   my $regexp           = '';
   my $tagfortypes      = '';
   my @invalidexprstags = ();
   my (@invalidexprs, @validlist, @invalidlist);
   my @typetag   = ();
   my $succeeded = $FALSE;
   my (@invalidtags, @validtags) = ((), ());
   my $retval = $FALSE;

   $tagfortypes = ${$configdoc->getElementsByTagName('types')}[0];
   if ($tagfortypes)
   {
      foreach my $nodename (keys %$attrshash)
      {
         foreach my $xnode (@{$xmldoc->getElementsByTagName($nodename)})  #get nodes with name $nodename
         {
            if ($xnode->hasAttributes)                                    #if attributes are exist
            {
               foreach my $cattr (@{$$attrshash{$nodename}})              #getting attribute type information
               {
                  if ($cattr->attributes->getNamedItem('type'))
                  {
                     if (@typetag = $tagfortypes->getElementsByTagName($cattr->attributes->getNamedItem('type')->value))
                     {
                        @invalidlist  = ();
                        @validlist    = ();
                        $regexp       = '';
                        @invalidexprs = ();
                        if (@invalidtags = $typetag[0]->getElementsByTagName('invalid'))  #fill the invalid values
                        {
                           foreach my $inval_node (@{$invalidtags[0]->getElementsByTagName('val')})
                           {
                              push @invalidlist, $inval_node->textContent;
                           }
                        } ## end if (@invalidtags = $typetag...
                        if (@validtags = $typetag[0]->getElementsByTagName('valid'))      #fill the valid values
                        {
                           foreach my $val_node (@{$validtags[0]->getElementsByTagName('val')})
                           {
                              push @validlist, $val_node->textContent;
                           }
                        } ## end if (@validtags = $typetag...
                        if (@invalidexprstags = $typetag[0]->getElementsByTagName('regexpinval'))
                        {
                           foreach my $inval_regexp_node (@{$invalidexprstags[0]->getElementsByTagName('val')})
                           {
                              push @invalidexprs, $inval_regexp_node->textContent;
                           }
                        } ## end if (@invalidexprstags ...
                        if ($typetag[0]->getElementsByTagName('regexp'))
                        {
                           $regexp = ${$typetag[0]->getElementsByTagName('regexp')}[0]->textContent;
                        }
                        $succeeded = $FALSE;
                        if (my $xattr = $xnode->attributes->getNamedItem($cattr->attributes->getNamedItem('name')->value))
                        {
                           foreach my $invalid_value (@invalidlist)
                           {
                              if ($xattr->value eq $invalid_value)
                              {
                                 push @$errlist,
                                   {
                                    "line"      => $xnode->line_number,
                                    "nodename"  => $xnode->nodeName,
                                    "attrname"  => $xattr->nodeName,
                                    "attrvalue" => $xattr->value,
                                    "nodepath"  => $xnode->nodePath
                                   };
                                 $retval = $TRUE;
                              } ## end if ($xattr->value eq $invalid_value...
                           } ## end foreach my $invalid_value (...
                           foreach my $valid_value (@validlist)
                           {
                              if ($xattr->value eq $valid_value)
                              {
                                 (@invalidtags, @validtags, $regexp, @invalidexprs) = ((), (), '', ());
                                 $succeeded = $TRUE;
                              }
                           } ## end foreach my $valid_value (@validlist...
                             # if only valid values specified
                           if ((not $succeeded) and (@validlist) and (not(@invalidlist)) and (not(@invalidexprs)))
                           {
                              push @$errlist,
                                {
                                 "line"      => $xnode->line_number,
                                 "nodename"  => $xnode->nodeName,
                                 "attrname"  => $xattr->nodeName,
                                 "attrvalue" => $xattr->value,
                                 "nodepath"  => $xnode->nodePath
                                };
                              $retval = $TRUE;
                           } ## end if ((not $succeeded) and...
                           if (@invalidexprs)
                           {
                              foreach my $invalid_regexp_value (@invalidexprs)
                              {
                                 if ($xattr->value =~ m/$invalid_regexp_value/)
                                 {
                                    push @$errlist,
                                      {
                                       "line"      => $xnode->line_number,
                                       "nodename"  => $xnode->nodeName,
                                       "attrname"  => $xattr->nodeName,
                                       "attrvalue" => $xattr->value,
                                       "nodepath"  => $xnode->nodePath
                                      };
                                    $retval = $TRUE;
                                 } ## end if ($xattr->value =~ m/$invalid_regexp_value/...
                              } ## end foreach my $invalid_regexp_value...
                           } ## end if (@invalidexprs)
                           if ($regexp)
                           {
                              if (!($xattr->value =~ m/$regexp/))
                              {
                                 push @$errlist,
                                   {
                                    "line"      => $xnode->line_number,
                                    "nodename"  => $xnode->nodeName,
                                    "attrname"  => $xattr->nodeName,
                                    "attrvalue" => $xattr->value,
                                    "nodepath"  => $xnode->nodePath,
                                    "regexp"    => $regexp
                                   };
                                 $retval = $TRUE;
                              } ## end if (!($xattr->value =~...
                           } ## end if ($regexp)
                        } ## end if (my $xattr = $xnode...
                     } ## end if (@typetag = $tagfortypes...
                  } ## end if ($cattr->attributes...
               } ## end foreach my $cattr (@{$$attrshash...
            } ## end if ($xnode->hasAttributes...
         } ## end foreach my $xnode (@{$xmldoc...
      } ## end foreach my $nodename (keys ...
   } ## end if ($tagfortypes)
   return $retval;
} ## end sub isWrongType($$$$$)

###############################################
# @fn isDupNode($xmldoc, $cnodes, $errlist)
# @brief looks for the nodes that have duplicate names
# @param $xdoc - xml file documents object
# @param $cnodes - configuration file nodes
# @param $errlist - array reference for errors
# @return
#          $TRUE - there are duplicate node names
#          $FALSE - no errors
# @author Pavel Sergeev
# @date February 2008
##############################################
sub isDupNode($$$)
{
   my ($xmldoc, $cnodes, $errlist) = @_;
   if ((ref($xmldoc) ne $LibXMLDoc) or (ref($xmldoc) ne $LibXMLDoc) or not is_array($cnodes))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $TRUE;
   }

   my %uniqhash = ();
   my @nodelist = ();
   my $count    = 0;
   my $retval   = $FALSE;
   foreach my $cnode (@$cnodes)
   {
      foreach my $parent_node ($cnode->getElementsByTagName('parent'))  # 'unique' nodes
      {
         if (($parent_node->attributes->getNamedItem('unique')) and ($parent_node->attributes->getNamedItem('unique')->value))
         {
            push @{$uniqhash{$parent_node->parentNode->attributes->getNamedItem('name')->value}}, $parent_node->textContent;
         }
      } ## end foreach my $parent_node ($cnode...
   } ## end foreach my $cnode (@$cnodes...
   foreach my $nodename (keys %uniqhash)
   {
      foreach my $parnode (@{$uniqhash{$nodename}})
      {
         foreach my $node (@{$xmldoc->findnodes($parnode)})
         {
            @nodelist = $node->getElementsByTagName($nodename);
            $count    = 0;
            foreach my $xnode (@nodelist)
            {
               if ($xnode->parentNode->isEqual($node))
               {
                  $count++;
               }
            } ## end foreach my $xnode (@nodelist...
            if ($count > 1)
            {
               push @$errlist,
                 {
                  "line"     => $node->line_number,
                  "nodename" => $nodename,
                  "nodepath" => $node->nodePath
                 };
               $retval = $TRUE;
            } ## end if ($count > 1)
         } ## end foreach my $node (@{$xmldoc...
      } ## end foreach my $parnode (@{$uniqhash...
   } ## end foreach my $nodename (keys ...
   return $retval;
} ## end sub isDupNode($$$)

#####################################################################
# @fn isValidXml($xdoc, $cdoc, $cnodes, $attrhash, $errhash)
# @brief validates XML file
# @param $cdoc - configuration file document object
# @param $xdoc - xml file documents object
# @param $cnodes - reference to xml node objects from xml scheme
# @param $attrhash - reference to the hash of attributes
# @param $errhash - contains error descriptions
# @return
#          $TRUE - valid xml file
#          $FALSE - xml file is not valid
# @author Pavel Sergeev
# @date December 2007
######################################################################
sub isValidXml($$$$$)
{
   my ($xdoc, $cdoc, $cnodes, $attrhash, $errhash) = @_;
   if ((ref($cdoc) ne $LibXMLDoc) or (ref($xdoc) ne $LibXMLDoc) or not is_array($cnodes) or not is_hash($attrhash) or not is_hash($errhash))
   {
      baselog($LOG_ERROR, "Wrong type of the arguments.");
      return $FALSE;
   }
   if (defined($errhash) and (ref($errhash) ne "HASH"))
   {
      baselog($LOG_ERROR, "Third parameter must be a hash reference");
      return $FALSE;
   }
   my $errstr = '';

   if (not isValidConfig($cdoc, \$errstr))
   {
      if ($errstr)
      {
         baselog($LOG_ERROR, $errstr);
         return $FALSE;
      }
      return $FALSE;
   } ## end if (not isValidConfig(...

   my $retval = $TRUE;
   my @xnodes = $xdoc->getElementsByTagName('*');

   $errhash->{'unknownnode'} = ();
   if (isUnknownNode($cnodes, \@xnodes, \@{$errhash->{'unknownnode'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'reqnode'} = ();
   if (isRequiredNode($xdoc, $cnodes, \@{$errhash->{'reqnode'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'wrongh'} = ();
   if (isWrongHierarchy($xdoc, $cnodes, \@xnodes, \@{$errhash->{'wrongh'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'unknownattr'} = ();
   if (isUnknownAttr($cnodes, \@xnodes, $attrhash, \@{$errhash->{'unknownattr'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'reqattr'} = ();
   if (isRequiredAttr($cnodes, \@xnodes, $attrhash, \@{$errhash->{'reqattr'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'uniqnode'} = ();
   if (isDupNode($xdoc, $cnodes, \@{$errhash->{'uniqnode'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'uniqattr'} = ();
   if (isDupAttr($xdoc, $cnodes, \@{$errhash->{'uniqattr'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'subset'} = ();
   if (isSubsetAttr($xdoc, $cnodes, \@{$errhash->{'subset'}}))
   {
      $retval = $FALSE;
   }
   $errhash->{'wrongtype'} = ();
   if (isWrongType($xdoc, $cdoc, $cnodes, $attrhash, \@{$errhash->{'wrongtype'}}))
   {
      $retval = $FALSE;
   }
   return $retval;
} ## end sub isValidXml($$$$$)

#######################################################
# @fn printErrorList($errtypehash)
# @brief prints list of errors to terminal and to log file
# @param $errtypehash - hash reference
# @param $xmlfilepath - path to xml file
# @param $printerrors - $TRUE - prints errors on the terminal and log them
#                       $FALSE - only log errors
# @return none
# @author Pavel Sergeev
# @date May 2008
######################################################
sub printErrorList($$;$)
{
   my $errtypehash = shift @_;
   my $xmlfilepath = shift @_;
   my $printerrors = shift @_;
   my $errhash;

   if (not defined $printerrors)
   {
      $printerrors = $FALSE;
   }
   if (ref($errtypehash) ne "HASH")
   {
      baselog($LOG_ERROR, "Bad parameter");
      return;
   }

   foreach my $errtype (keys %$errtypehash)
   {
      if (@{$errtypehash->{$errtype}} > 0)
      {
         if ($errtype eq "unknownnode")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. An unknown node: line $errhash->{line}, node name is $errhash->{nodename}", $printerrors);
            }
         } ## end if ($errtype eq "unknownnode"...
         elsif ($errtype eq "reqnode")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. \"$errhash->{nodename}\" node is required.", $printerrors);
            }
         } ## end elsif ($errtype eq "reqnode"...
         elsif ($errtype eq "wrongh")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. Wrong hierarchy. Node:\"$errhash->{nodename}\" at line $errhash->{line}. Parent node:\"$errhash->{pnodename}\"", $printerrors);
            }
         } ## end elsif ($errtype eq "wrongh"...
         elsif ($errtype eq "unknownattr")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. An unknown attribute: Node \"$errhash->{nodename}\", attribute \"$errhash->{attrname}\" at line $errhash->{line}", $printerrors);
            }
         } ## end elsif ($errtype eq "unknownattr"...
         elsif ($errtype eq "reqattr")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. There is a required attribute \"$errhash->{attrname}\", node:\"$errhash->{nodename}\" at line $errhash->{line}", $printerrors);
            }
         } ## end elsif ($errtype eq "reqattr"...
         elsif ($errtype eq "uniqnode")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. \"$errhash->{nodename}\" node must be unique.", $printerrors);
            }
         } ## end elsif ($errtype eq "uniqnode"...
         elsif ($errtype eq "uniqattr")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO,
                       "File: $xmlfilepath. \"$errhash->{attrname}\" attribute must be unique. Node \"$errhash->{nodename}\" at line $errhash->{line}. Attribute value \"$errhash->{attrvalue}\"",
                       $printerrors);
            } ## end foreach $errhash (@{$errtypehash...
         } ## end elsif ($errtype eq "uniqattr"...
         elsif ($errtype eq "subset")
         {
            for $errhash (@{$errtypehash->{$errtype}})
            {
               baselog(
                   $LOG_INFO,
                   "File: $xmlfilepath. Wrong attribute value in subset. Node: \"$errhash->{nodename}\", attribute: \"$errhash->{attrname}\", value:\"$errhash->{attrvalue}\" at line $errhash->{line}",
                   $printerrors
               );
            } ## end for $errhash (@{$errtypehash...
         } ## end elsif ($errtype eq "subset"...
         elsif ($errtype eq "wrongtype")
         {
            foreach $errhash (@{$errtypehash->{$errtype}})
            {
               baselog($LOG_INFO, "File: $xmlfilepath. Invalid attribute value at line $errhash->{line}. Attribute \"$errhash->{attrname}\", value \"$errhash->{attrvalue}\"", $printerrors);
               if ($errhash->{'regexp'})
               {
                  baselog($LOG_INFO, "The $errhash->{attrname} value is not matched the following regular expression: $errhash->{regexp}", $printerrors);
               }
            } ## end foreach $errhash (@{$errtypehash...
         } ## end elsif ($errtype eq "wrongtype"...
      } ## end if (@{$errtypehash->{$errtype...
   } ## end foreach my $errtype (keys %$errtypehash...
   return;
} ## end sub printErrorList($$;$)

#########################################################
# @fn checkXmlFiles($xmlFileList, $cdocpath, $printerrors)
# @brief check number of xml files
# @param $xmlFileList - refernce to list of xml file names
# @param $cdocpath - the xml scheme file pathname
# @param $printerrors - $TRUE - print errors on terminal
#                       $FALSE - log errors without printing
# @return $FALSE if any errors, $TRUE if no errors
# @author Pavel Sergeev
# @date September 2008
#########################################################
sub checkXmlFiles($$;$)
{
   my $xmlFileList = shift @_;
   my $cdocpath    = shift @_;
   my $printerrors = shift @_;
   my $err         = "";
   my $xdoc;
   my %attrhash     = ();
   my %errors       = ();
   my $retval       = $TRUE;
   my $prevfilename = "";

   if (not defined $printerrors)
   {
      $printerrors = $TRUE;
   }
   if ((not defined $cdocpath) or (!-e $cdocpath))
   {
      return $FALSE;
   }

   my $cdoc = parseFile($cdocpath, \$err);
   if (not defined $cdoc)
   {
      return $FALSE;
   }
   my @cnodes = $cdoc->getElementsByTagName('node');
   foreach my $cnode (@cnodes)
   {
      $attrhash{$cnode->attributes->getNamedItem('name')->getValue} = $cnode->getElementsByTagName('attr');
   }

   foreach my $filepath (@$xmlFileList)
   {
      $xdoc = parseFile($filepath, \$err);
      if (not defined $xdoc)
      {
         $retval = $FALSE;
         print("XML syntax error in $filepath\n$err\n");
         next;
      } ## end if (not defined $xdoc)
      if (not isValidXml($xdoc, $cdoc, \@cnodes, \%attrhash, \%errors))
      {
         $retval = $FALSE;
         printErrorList(\%errors, $filepath, $printerrors);
      }
   } ## end foreach my $filepath (@$xmlFileList...
   return $retval;
} ## end sub checkXmlFiles($$;$)

1;
