#!/usr/bin/perl
#
## @file fsbackup.pl
# @brief archive manipulation script
# @details http://www.opennet.ru/dev/fsbackup/
#          Copyright (c) 2001-2002 by Maxim Chirkov. <mc@tyumen.ru>
#
#            -n - create new archive do not bothering about hash.
#            -f - full_backup - full backup to archive without hash.
#            -h - hash - generate hash only without saving files into archive.
#            -c - clean - clean backup storage and create new backup
#                 (for incremental backup).
# @date: May 2006
# @author: Kate Bereznyakova (MeraNN)
# @version 1.00 - Initial version.
# @version 1.10 - changes:
#                   change FTP on SFTP
#                   change local type of storage to USB storage
#                   reduce incremental functionality
#                   "nosuid" mount option is added as the US DoD requirement
# @version 1.11 - Two colons were removed before shared functions and variables.
# @version 1.12 - USB mount operation is changed to use device serial number.
# @version 1.13 - Change sequence of baselog and common_functions
# @version 1.14 - setup of error message to deployment configuration file has been added
# @version 1.15 - get_hostname() function is now used instead of Sys::Hostname
# @version 1.16 - RecoveryCache/ and RecoveryCache/* now will be created with 770 perms
# @version 1.17 - Now USB flash cards are mounted under $UID instead of $NORTEL_ACCT

use lib "/opt/nortel/base/lib";

#Cash size to place hase to the memory
use constant DB_DEF_CACHE_SIZE => 4096000;

use common_variables;
use common_functions;

use POSIX;
use English;
use File::Find;
use Digest::MD5 qw(md5_base64);
use Net::SFTP;
use DB_File;
use recovery::backup;
use baselog;
use base_system;
use base_deploy;
use CommonInstallUtils;

use constant VERB_SILENT => 0;  # Silent mode, suspend all output.
use constant VERB_ERROR  => 1;  # Output all errors and warnings.
use constant VERB_ALL    => 2;  # Output all the  available  data.

my $list_lines_cnt = 0;
my $del_lines_cnt  = 0;
my $cur_time       = time();
my %active_hash_last;
my %active_hash_new;
my $cfg_new_flag    = 0;
my $cfg_clean_flag  = 0;
my $config          = 0;
my $cur_backup_size = 1536;     # tar-block size
my $prog_pgp_filter;
my $prog_gzip_filter;
my $arc_ext;
my $ftp;
my $cur_increment_level;
my $cur_dir;
my $cur_path;
my $cur_file;
my $cur_pathitem;
my $file_fullpath;
my $file_fullpath_md5;
my $key;
my $dbobj_new;
my $dbobj_last;
my $db_hashinfo;
my $db_hashinfo2;
my $file;
my @volume_position = (0);
my @fs_path         = ();
my @fs_notpath      = ();
my @fs_mask         = ();
my @fs_filemask     = ();
my @fs_dirmask      = ();
my @fs_notmask      = ();
my @fs_notfilemask  = ();
my @fs_notdirmask   = ();
my $archive_name;
INIT
{
   $SIG{INT} = \&cleanup;
}

END
{
   my $retval = $?;

   if (cleanup_tmp())
   {
      $? = $retval;
   }
   else
   {
      $? = $ERROR;
   }
} ## end END

if (warning_if_not_nortel_grp())
{
   exit $ERROR;
}
my $hostname = get_hostname();
my $cfg_file = get_deploy_cfg_path($hostname);

# ------------- Parsing command line parameters
if (   $ARGV[0] eq "-n"
    || $ARGV[0] eq "-h"
    || $ARGV[0] eq "-f"
    || $ARGV[0] eq "-c")
{
   $cfg_new_flag = 1;
   $config       = $ARGV[1];
} ## end if ($ARGV[0] eq "-n" ||...
else
{
   $cfg_new_flag = 0;
   $config       = $ARGV[0];
}
if (!-f $config)
{
   die "Usage: fsbackup.pl [-n|-f|-h|-c] config_name";
}
require "$config";

unless (defined($cfg_cache_dir))
{
   baselog($LOG_ERROR, "\$cfg_cache_dir not defined. Set \$cfg_cache_dir variable in fsbackup.pl", $TRUE);
   exit($ERROR);
}
if (!-d $cfg_cache_dir)
{
   unless (do_system([$MKDIR, '-p', '-m', '770', $cfg_cache_dir]))
   {
      baselog($LOG_ERROR, "Cannot create cache dir '$cfg_cache_dir'.", $TRUE);
      exit($ERROR);
   }
   unless (-d $cfg_cache_dir)
   {
      baselog($LOG_ERROR, "\$cfg_cache_dir ($cfg_cache_dir) not found. Set appropriate \$cfg_cache_dir variable in fsbackup.pl", $TRUE);
      exit($ERROR);
   }
} ## end if (!-d $cfg_cache_dir...

$cfg_time_limit           *= 60 * 60 * 24;  # days to seconds
$cfg_size_limit           *= 1024;          # kilobytes to bytes.
$cfg_maximum_archive_size *= 1024;          # kilobytes to bytes.

if (-d $cfg_root_path)
{
   chdir($cfg_root_path);
}
else
{
   die "Invalid \$cfg_root_path path ($cfg_root_path)";
}

if ($ARGV[0] eq "-h")
{
   $cfg_backup_style = "hash";
}
if ($ARGV[0] eq "-f")
{
   $cfg_backup_style = "full_backup";
}

if ($ARGV[0] eq "-c")
{
   $cfg_clean_flag = 1;
}
else
{
   $cfg_clean_flag = 0;
}

#------------------- Check variables in config.file
if ($cfg_backup_name !~ /^[\w\d\_\-]+$/)
{
   die "Found illegal characters in $cfg_backup_name ($cfg_backup_name).";
}

if (!grep {$_ eq $cfg_checksum} ("md5", "timesize"))
{
   die "Unknown checksum method:\$cfg_checksum=$cfg_checksum (allowed md5 or timesize)";
}

if (!grep {$_ eq $cfg_backup_style} ("backup", "full_backup", "sync", "hash"))
{
   die "Unknown backup_style:\$cfg_backup_style=$cfg_backup_style";
}

if ($cfg_remote_ftp_mode != 1)
{
   $cfg_remote_ftp_mode = 0;
}

if ($cfg_backup_style eq "full_backup" || $cfg_backup_style eq "hash")
{
   $cfg_new_flag   = 1;
   $cfg_clean_flag = 1;
}

if (!grep {$_ eq $cfg_type} ("local", "remote_ssh", "remote_ftp", "upgrade"))
{
   die "Unknown backup target:\$cfg_type=$cfg_type";
}

if ($cfg_backup_style eq "backup")
{
   $backup_file_base = generateBackupName();
}
else
{
   $backup_file_base = "$cfg_backup_name";
}

print_msg("Creating $cfg_type $cfg_backup_style: $cfg_backup_name")
  if ($cfg_verbose == &VERB_ALL);

if ($cfg_pgp_userid ne "" && $prog_pgp ne "")
{
   print_msg("PGP: enabled") if ($cfg_verbose == &VERB_ALL);
   $prog_pgp_filter = "| $prog_pgp -v --batch -e -r $cfg_pgp_userid";
}
else
{
   $prog_pgp_filter = "";
}

if ($prog_gzip ne "")
{
   $prog_gzip_filter = "| $prog_gzip";
   $arc_ext          = ".gz";
}
else
{
   $prog_gzip_filter = "";
   $arc_ext          = "";
}

my $cfg_backup_full_name = "$cfg_cache_dir/$cfg_backup_name";
print_msg("Cache dir - $cfg_backup_full_name")
  if ($cfg_verbose == &VERB_ALL);
if (!-d $cfg_backup_full_name)
{
   unless (do_system([$MKDIR, "-p", "-m", "770", $cfg_backup_full_name]))
   {
      baselog($LOG_ERROR, "Cannot create backup directory '$cfg_backup_full_name'.", $TRUE);
      exit($ERROR);
   }
} ## end if (!-d $cfg_backup_full_name...

#----------- mounting a USB device if this is local backup
if ($cfg_type eq "local")
{
   if ((!defined $cfg_usb_sn) or ($cfg_usb_sn eq ''))
   {
      print_msg("USB serial number is undefined. Fatal error.");
      exit($ERROR);
   }

   #checking if node for the USB device exists
   if (!-e $tmpMountDir)
   {
      print_msg("USB mount point $tmpMountDir does not exist.");
      exit($ERROR);
   }

   #Try to check the first partition device node path
   my (undef, $usb_path) = get_USB_node_path_by_SN($cfg_usb_sn);

   if ((!defined $usb_path) or ($usb_path eq ''))
   {
      baselog($LOG_ERROR, 'Cannot define USB device node name.');
      exit($ERROR);
   }

   if (!-e $usb_path)
   {
      baselog($LOG_ERROR, "Node '$usb_path' does not exist.");
      exit($ERROR);
   }

   baselog($LOG_INFO, "USB device: '$usb_path'.");

   # US DoD requires to use "nosuid" mount option for removable media
   if (not do_system([$SUDO, $MOUNT, "-o", "uid=$UID,nosuid", $usb_path, $tmpMountDir]))
   {
      print_msg("Can not mount USB device. Fatal error.");
      exit($ERROR);
   }

} ## end if ($cfg_type eq "local"...

#----------- defining incremental level
if ($cfg_increment_level != 0 && $cfg_backup_style eq "backup")
{
   $cur_increment_level = 0;
   if ($cfg_type eq "local")
   {
      opendir(DIR, "$tmpMountDir/sysinfo");
      while ($cur_dir = readdir DIR)
      {
         if ($cur_dir =~ /\-0\.tar${arc_ext}$/)
         {
            $cur_increment_level++;
         }
      } ## end while ($cur_dir = readdir...
      closedir(DIR);
   } ## end if ($cfg_type eq "local"...
   elsif ($cfg_type eq "remote_ftp")
   {
      foreach $cur_dir ($ftp->ls())
      {
         if ($cur_dir =~ /\-0\.tar${arc_ext}$/)
         {
            $cur_increment_level++;
         }
      } ## end foreach $cur_dir ($ftp->ls(...
   } ## end elsif ($cfg_type eq "remote_ftp"...
   if ($cur_increment_level >= $cfg_increment_level)
   {
      $cfg_new_flag   = 1;
      $cfg_clean_flag = 1;
   }
   print_msg("Current increment number: $cur_increment_level")
     if ($cfg_verbose == &VERB_ALL);
} ## end if ($cfg_increment_level...

################################################
#----------- Read hash to memory.

#do not want to use icremental backup - remove hash
system("$RM -rf $cfg_cache_dir/$cfg_backup_name/.hash");

$db_hashinfo2 = new DB_File::HASHINFO;
$db_hashinfo2->{'cachesize'} = 100000;

# just for simplicity
my $cfg_backup_name_list  = "$cfg_cache_dir/$cfg_backup_name/$cfg_backup_name.list";
my $cfg_backup_name_lsize = "$cfg_cache_dir/$cfg_backup_name/$cfg_backup_name.lsize";
my $cfg_backup_name_dir   = "$cfg_cache_dir/$cfg_backup_name/$cfg_backup_name.dir";
my $cfg_backup_name_del   = "$cfg_cache_dir/$cfg_backup_name/$cfg_backup_name.del";

# Create list of files for backup.
foreach my $filename ($cfg_backup_name_list, $cfg_backup_name_lsize, $cfg_backup_name_dir, $cfg_backup_name_del)
{
   if (-e $filename)
   {
      unless (unlink $filename)
      {
         print_msg("Cannot delete old file '$filename'.");
      }
   } ## end if (-e $cfg_backup_name_list...
} ## end foreach my $filename ($cfg_backup_name_list...
open(LIST, '>', $cfg_backup_name_list)
  || print_msg("Can't create list file ($cfg_backup_name_list).");
flock(LIST, 2);

# Create list with files's sizes.
open(LIST_SIZE, '>', $cfg_backup_name_lsize)
  || print_msg("Can't create list file ($cfg_backup_name_lsize).");
flock(LIST_SIZE, 2);

# Create directory list withing archive.
open(DIRS, '>', $cfg_backup_name_dir)
  || print_msg("Can't create list file ($cfg_backup_name_dir).");
flock(DIRS, 2);

# Read backup list into memory.
while (<DATA>)
{
   chomp;
   $cur_path = $_;
   if ($cur_path =~ /^\!(.*)$/)
   {  #  !
      push @fs_notpath, $1;
   }
   elsif ($cur_path =~ /^\=\~(.*)$/)
   {  #  =~
      push @fs_mask, $1;
   }
   elsif ($cur_path =~ /^f\~(.*)$/)
   {  #  f~
      push @fs_filemask, $1;
   }
   elsif ($cur_path =~ /^d\~(.*)$/)
   {  #  d~
      push @fs_dirmask, $1;
   }
   elsif ($cur_path =~ /^\=\!(.*)$/)
   {  #  =!
      push @fs_notmask, $1;
   }
   elsif ($cur_path =~ /^f\!(.*)$/)
   {  #  f!
      push @fs_notfilemask, $1;
   }
   elsif ($cur_path =~ /^d\!(.*)$/)
   {  #  d!
      push @fs_notdirmask, $1;
   }
   elsif ($cur_path =~ /^#/ || $cur_path =~ /^\s*$/)
   {  #  comment
      next;
   }
   elsif ($cur_path =~ /[\/\w]+/)
   {  #  /dir[/file]
      if ($cur_path =~ /^$cfg_root_path/)
      {
         push @fs_path, "$cur_path";
      }
      else
      {
         push @fs_path, "$cfg_root_path$cur_path";
      }
   } ## end elsif ($cur_path =~ /[\/\w]+/...
   else
   {
      print_msg("Syntax error: $cur_path, ingnored.")
        if ($cfg_verbose >= &VERB_ALL);
   }
} ## end while (<DATA>)

#--------------------------------------------------------------------
# Go through directory list
foreach $cur_pathitem (@fs_path)
{
   print_msg("Adding $cur_pathitem....") if ($cfg_verbose == &VERB_ALL);
   find(\&add_to_backup, $cur_pathitem);
   chdir($cfg_root_path);  # go to the root dir as find use chdir.
   print_msg("Done") if ($cfg_verbose == &VERB_ALL);
} ## end foreach $cur_pathitem (@fs_path...
close(LIST);
close(LIST_SIZE);
close(DIRS);

#------------
# Create an archive and transmit it to backup storage.

open(DEL, '>', $cfg_backup_name_del)
  || print_msg("Can't create list file ($cfg_backup_name_del).");
flock(DEL, 2);
if ($cfg_backup_style ne "hash")
{
   while (($file, $key) = each(%active_hash_last))
   {
      $file =~ s/\'/\'\\\'\'/g;
      $file =~ s/^\/(.*)$/$1/;
      print DEL "$RM -rf '$file'\n";
      $del_lines_cnt++;
   } ## end while (($file, $key) = each...
} ## end if ($cfg_backup_style ...
close(DEL);
untie %active_hash_new;
untie %active_hash_last;

if ($list_lines_cnt == 0 && $del_lines_cnt == 0)
{
   print_msg("WARNING: Nothing to backup");
   if (!update_section($cfg_file, "/target/errorMsg", 'Nothing to backup', $KEEPOLDCNT))
   {
      baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
   }
   exit($ERROR);
} ## end if ($list_lines_cnt ==...

# Create backup archive in temprory local directory.
$archive_name = "$backup_file_base.tar${arc_ext}";
my $tmp_list_file = crate_tmp_list(0, $volume_position[0], $volume_position[1], "$cfg_cache_dir/$cfg_backup_name/$cfg_backup_name.list");

my $cmd = "$TAR -Pc -f - -T $tmp_list_file 2>/dev/null $prog_gzip_filter $prog_pgp_filter 2>/dev/null";
$cmd = $cmd . " > $cfg_cache_dir/$backupArchive 2>/dev/null";
if (system($cmd) != 0)
{
   baselog($LOG_ERROR, "$TAR failed: $?\n", $TRUE);
   exit($ERROR);
}
my $archive_size = (stat("$cfg_cache_dir/$backupArchive"))[7];
if ($backup_mode eq "interactive")
{
   print "Backup archive with name $archive_name and size $archive_size bytes was generated.\n";
   print "Backup operation may take a long time.\n";
   if (!getYNResponse("Do you want to continue", 'Y'))
   {
      baselog($LOG_INFO, "Backup canceled by user.\n");
      exit($ERROR);
   }
} ## end if ($backup_mode eq "interactive"...

if ($cfg_type eq "local")
{
   if (check_space($archive_size) == $TRUE)
   {
      print_msg("Storing local backup...") if ($cfg_verbose == &VERB_ALL);
      if (system("$CP $cfg_cache_dir/$backupArchive $tmpMountDir/sysinfo/$archive_name") != 0)
      {
         print_msg("Can not copy backup archive to local (USB) device.");
         exit($ERROR);
      }
   } ## end if (check_space($archive_size...
   else
   {
      print_msg("There is not enough space on the USB flash disk. Cleanup your storage and try again.");
      exit($ERROR);
   }

} ## end if ($cfg_type eq "local"...

elsif ($cfg_type eq "remote_ftp")
{
   if (system("$PING -q -c 3 $cfg_remote_host > /dev/null 2>&1") != 0)
   {
      baselog($LOG_ERROR, "SFTP server is not accessible via Network.\n", $TRUE);
      print_msg("Check network configuration.\n\n");
      if (!update_section($cfg_file, "/target/errorMsg", 'SFTP server is not accessible via Network', $KEEPOLDCNT))
      {
         baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
      }
      exit($ERROR);
   } ## end if (system("$PING -q -c 3 $cfg_remote_host > /dev/null 2>&1"...

   print "Storing remote ftp backup...\n" if ($cfg_verbose == &VERB_ALL);

   #ignore data input from user
   do_co_stdin('close');
   unless (ftp_connect())
   {
      baselog($LOG_ERROR, "Can't connect to ftp server - $cfg_remote_host.\n", $TRUE);
      do_co_stdin('open');
      if (!update_section($cfg_file, "/target/errorMsg", "Can't connect to SFTP server - $cfg_remote_host. Please, check your input access parameters.", $KEEPOLDCNT))
      {
         baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
      }
      exit($ERROR);
   } ## end unless (ftp_connect())
   print "Operation in progress. Please wait.\n";
   $ftp->put("$cfg_cache_dir/$backupArchive", "$cfg_remote_path/$archive_name", \&callback);
   print "\n";
   $attrRemote = $ftp->do_stat("$cfg_remote_path/$archive_name");
   if ($attrRemote->size() ne $archive_size)
   {
      baselog($LOG_ERROR, "Error during transferring file to SFTP server", $TRUE);
      print_msg("File maybe corrupt.");
      do_co_stdin('open');
      if (!update_section($cfg_file, "/target/errorMsg", 'Error during transferring file to SFTP server', $KEEPOLDCNT))
      {
         baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
      }
      exit($ERROR);
   } ## end if ($attrRemote->size(...
   do_co_stdin('open');
} ## end elsif ($cfg_type eq "remote_ftp"...

#Update deployment.xml file with new backup archive name$archive_name
if (!update_section($cfg_file, "/target/lastBackupName", $archive_name))
{
   baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
}

exit($OK);

########################################
##------------------------------------------------------------------
# Purpose:
#   Function to provide user info about progress of SFTP downloading
#
##-------------------------------------------------------------------
sub callback
{
   my ($sftp, $data, $offset, $size) = @_;
   print "\rRead $offset of $size bytes";
   baselog($LOG_DEBUG, "Read $offset of $size bytes \n");
}
###-------------------------------------------------------------------------
# Purpose:
#    Handler for "Ctrl+C"
###-------------------------------------------------------------------------
sub cleanup()
{
   return;
}

###-------------------------------------------------------------------------
# Purpose:
#   Remove temporary data
#
#  INPUT:   no
#  OUTPUT:  $TRUE  - successful operation
#           $FALSE - error condition
###-------------------------------------------------------------------------
sub cleanup_tmp
{
   my $retval = $TRUE;

   if ($cfg_type eq "local")
   {
      baselog($LOG_DEBUG, "Umount USB '$tmpMountDir'.");
      umountUSB($tmpMountDir);
   }

   if ($ARGV[1] eq "--upgrade")
   {
      if (removeUpgradeBackup() != $TRUE)
      {
         remove_tmp_backup_archive();
         $retval = $FALSE;
      }
   } ## end if ($ARGV[1] eq "--upgrade"...
   elsif (-e "$cfg_cache_dir/$backupArchive")
   {
      $retval = remove_tmp_backup_archive();
   }
   return $retval;
} ## end sub cleanup_tmp

###-------------------------------------------------------------------------
# Purpose:
#   Route message to appropriate place
#   For interactive mode - to the screen
#   For foreground mode - to the log file
#  INPUT:   $msg - message to print
#  OUTPUT:  no
###-------------------------------------------------------------------------
sub print_msg
{
   my ($msg) = shift @_;
   if ($backup_mode == "interactive")
   {
      print $msg. "\n";
   }
   else
   {
      baselog($LOG_INFO, $msg);
   }

} ## end sub print_msg
###-------------------------------------------------------------------------
# Purpose:
#   Check if there is enough space at local backup storge
#
#  INPUT:  $need - required size in bytes.
#  OUTPUT: $TRUE  (1) - enough space
#          $FALSE (0) - not enough space
###-------------------------------------------------------------------------
sub check_space($)
{
   my ($need) = shift @_;
   my $retval = $TRUE;
   @info = split(' ', `$DF $tmpMountDir | $GREP $tmpMountDir`);
   $need = (int($need / 1024) + 1);
   if ($need > $info[3])
   {
      print_msg("There is not enough space on the device.");
      print_msg("  Required: $need (Kb)\n  Avalable: $info[3] (Kb)\n");
      $retval = $FALSE;
   } ## end if ($need > $info[3])
   return $retval;
} ## end sub check_space($)

#################################
#
# fsbackup original functions
#
#################################
sub add_to_backup
{
   my ($file_name, $file_dir,  $md5_checksum_stat, $checksum_stat);
   my ($tmp,       $stat_mode, $stat_uid,          $stat_gid, $stat_size, $stat_mtime, $stat_time);

   $file_name     = $_;
   $file_fullpath = $File::Find::name;
   $file_dir      = $File::Find::dir;
   $file_fullpath =~ s/^$cfg_root_path/\.\//;
   my $file_fullpath_esc = $file_fullpath;
   $file_fullpath_esc =~ s/\'/\'\\\'\'/g;

   if (   (-d "$cfg_root_path/$file_fullpath")
       && (!-l "$cfg_root_path/$file_fullpath"))
   {
      if (check_path($file_dir, $file_name) == 1)
      {
         if ($cfg_backup_style ne "hash")
         {
            ($tmp, $tmp, $stat_mode, $tmp, $stat_uid, $stat_gid, $tmp, $stat_size, $tmp, $stat_mtime, $stat_time) = stat("$cfg_root_path/$file_fullpath");
            $stat_mode = sprintf("%04o", $stat_mode & 07777);
            $file_fullpath_esc =~ s/^\/(.*)$/$1/;
            $stat_uid = getpwuid($stat_uid) || $stat_uid;
            $stat_gid = getgrgid($stat_gid) || $stat_gid;
            my ($sec, $min, $hour, $mday, $mon, $year) = localtime($stat_time);
            $stat_time = sprintf("%4.4d%2.2d%2.2d%2.2d%2.2d.%2.2d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);
            print DIRS "mkdir '$file_fullpath_esc'\n";
            print DIRS "chmod $stat_mode '$file_fullpath_esc'\n";
            print DIRS "chown $stat_uid:$stat_gid '$file_fullpath_esc'\n";
            print DIRS "touch -t $stat_time '$file_fullpath_esc'\n";
            $cur_backup_size += int(length($file_fullpath) / 100.0 + 1) * 512;

            if (   $cfg_maximum_archive_size > 0
                && $cur_backup_size + 10240 >= $cfg_maximum_archive_size)
            {
               my $old_val = $cur_backup_size - $stat_size - int(length($file_fullpath) / 100.0 + 1) * 512;
               my $tmp_pos = $#volume_position + 1;
               print "Volume $tmp_pos Done. Size: $old_val\n"
                 if ($cfg_verbose == &VERB_ALL);
               $cur_backup_size = $stat_size + int(length($file_fullpath) / 100.0 + 1) * 512 + 1536;
               push @volume_position, $list_lines_cnt;
            } ## end if ($cfg_maximum_archive_size...
         } ## end if ($cfg_backup_style ...
      } ## end if (check_path($file_dir...
      else
      {
         if ($cfg_stopdir_prune == 1)
         {
            $File::Find::prune = 1;
            return;
         }
      } ## end else [ if (check_path($file_dir...
   } ## end if ((-d "$cfg_root_path/$file_fullpath"...
   elsif (   (-f "$cfg_root_path/$file_fullpath")
          || (-l "$cfg_root_path/$file_fullpath"))
   {
      if (check_path($file_dir, $file_name) == 1)
      {
         ($tmp, $tmp, $stat_mode, $tmp, $stat_uid, $stat_gid, $tmp, $stat_size, $tmp, $stat_mtime) = stat("$cfg_root_path/$file_fullpath");
         $checksum_stat = md5_base64("$stat_mtime/$stat_size/$stat_mode/$stat_uid/$stat_gid");

         # $file_fullpath_md5 = md5_base64($file_fullpath);
         $file_fullpath_md5 = $file_fullpath;
         if ($cfg_time_limit != 0 && $cur_time - $cfg_time_limit > $stat_mtime)
         {
            print "Time limit: $cur_time - $cfg_time_limit > $stat_mtime, file $file_fullpath ignored.\n"
              if ($cfg_verbose == &VERB_ALL);
            next;
         } ## end if ($cfg_time_limit !=...
         if ($cfg_size_limit != 0 && $cfg_size_limit < $stat_size)
         {
            print "Size limit: $cfg_size_limit < $stat_size, file $file_fullpath ignored.\n"
              if ($cfg_verbose == &VERB_ALL);
            next;
         } ## end if ($cfg_size_limit !=...
         if (($cfg_checksum eq "md5") && (!-l "$cfg_root_path/$file_fullpath"))
         {
            ($md5_checksum_stat, $tmp) =
              split(/\s+/, `$prog_md5sum '$cfg_root_path/$file_fullpath_esc'`);
            $active_hash_new{$file_fullpath_md5} = "$checksum_stat/$md5_checksum_stat";
            check_update($file_fullpath, "$checksum_stat/$md5_checksum_stat", $file_fullpath_md5, $stat_size);
         } ## end if (($cfg_checksum eq ...
         else
         {
            $active_hash_new{$file_fullpath} = $checksum_stat;
            check_update($file_fullpath, $checksum_stat, $file_fullpath, $stat_size);
         }
      } ## end if (check_path($file_dir...
   } ## end elsif ((-f "$cfg_root_path/$file_fullpath"...
} ## end sub add_to_backup
###############################################
sub check_update
{
   my ($file, $checksum, $filesum, $stat_size) = @_;

   if ($active_hash_last{$filesum} ne $checksum)
   {
      if ($cfg_backup_style ne "hash")
      {
         $file =~ s/^\/(.*)$/$1/;
         print LIST "$file\n";
         print LIST_SIZE "$stat_size\t$file\n";
         if (-l "/$file")
         {
            $stat_size = 0;
         }
         $cur_backup_size += $stat_size + int(length($file) / 100.0 + 1) * 512;

         #               print "$cur_backup_size:$stat_size:$file\n";
         if (   $cfg_maximum_archive_size > 0
             && $cur_backup_size + 10240 >= $cfg_maximum_archive_size)
         {
            my $old_val = $cur_backup_size - $stat_size - int(length($file) / 100.0 + 1) * 512;
            my $tmp_pos = $#volume_position + 1;
            print "Volume $tmp_pos Done. Size: $old_val\n"
              if ($cfg_verbose == &VERB_ALL);
            $cur_backup_size = $stat_size + int(length($file) / 100.0 + 1) * 512 + 1536;
            push @volume_position, $list_lines_cnt;
         } ## end if ($cfg_maximum_archive_size...

      } ## end if ($cfg_backup_style ...
      $list_lines_cnt++;
   } ## end if ($active_hash_last{...
   delete $active_hash_last{$filesum};
   if (defined $dbobj_last)
   {
      $dbobj_last->del($filesum);
   }
} ## end sub check_update
###############################################
sub check_path
{
   my ($dir_name, $file_name) = @_;
   my ($item,     $path);

   $path = "$dir_name/$file_name";

   foreach $item (@fs_notmask)
   {
      if ($path =~ /$item/)
      {
         return 0;
      }
   } ## end foreach $item (@fs_notmask)

   foreach $item (@fs_notfilemask)
   {
      if ($file_name =~ /$item/)
      {
         return 0;
      }
   } ## end foreach $item (@fs_notfilemask...

   foreach $item (@fs_filemask)
   {
      if ($file_name =~ /$item/)
      {
         return 1;
      }
   } ## end foreach $item (@fs_filemask...
   foreach $item (@fs_notdirmask)
   {
      if ($dir_name =~ /$item/)
      {
         return 0;
      }
   } ## end foreach $item (@fs_notdirmask...

   foreach $item (@fs_mask)
   {
      if ($path =~ /$item/)
      {
         return 1;
      }
   } ## end foreach $item (@fs_mask)
   foreach $item (@fs_dirmask)
   {
      if ($dir_name =~ /$item/)
      {
         return 1;
      }
   } ## end foreach $item (@fs_dirmask)

   foreach $item (@fs_notpath)
   {
      if (($dir_name eq $item) || ($path eq $item) || ($dir_name =~ /^$item\//))
      {
         return 0;
      }
   } ## end foreach $item (@fs_notpath)

   return 1;
} ## end sub check_path
#######################################################################
## @fn ftp_connect()
# @brief Connect to remote backup server.
# @details Connect to remote server via SFTP
# @param none
# @return $FALSE - if error occurred
# @return $TRUE  - if connect successfully established
# @author
# @date
# @note
#######################################################################
sub ftp_connect
{
   my ($retval) = $TRUE;
   if ($cfg_type eq "remote_ftp")
   {
      $args{user} = $cfg_remote_login;
      my $ret      = undef;
      my @tempPass = undef;
      if ($config =~ m/$CRON_MARK/)
      {  #scheduled sysbackup
         do_sudo([$SMCRYPT, $cronsftp_identity], {$dsOUT => \@tempPass, $dsRETVAL => \$ret});
      }
      else
      {  #manual sysbackup
         do_sudo([$SMCRYPT, $sftp_identity], {$dsOUT => \@tempPass, $dsRETVAL => \$ret});
      }
      if ($ret || (!defined($tempPass[0])))
      {
         baselog($LOG_ERROR, "Error retrieving password.\n", $TRUE);
         return $FALSE;
      }
      $args{password} = $tempPass[0];
      $args{debug}    = 0;
      $args{Timeout}  = 30;
      $args{Passive}  = $cfg_remote_ftp_mode;

      $ftp = eval {Net::SFTP->new($cfg_remote_host, %args)}
        || return $FALSE;
      if (!$ftp)
      {  # error occurred
         baselog($LOG_ERROR, "An error occurred while trying to access the supplied SFTP server.\n", $TRUE);
         if (!update_section($cfg_file, "/target/errorMsg", 'An error occurred while trying to access the supplied SFTP server.', $KEEPOLDCNT))
         {
            baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
         }
         $retval = $FALSE;
      } ## end if (!$ftp)
      else
      {
         if (!$ftp->do_stat($cfg_remote_path))
         {
            baselog($LOG_ERROR, "An error occurred while trying to access the supplied SFTP directory.\n", $TRUE);
            if (!update_section($cfg_file, "/target/errorMsg", 'An error occurred while trying to access the supplied sftp directory.', $KEEPOLDCNT))
            {
               baselog($LOG_ERROR, 'Cannot update deployment.xml file.');
            }
            $retval = $FALSE;
         } ## end if (!$ftp->do_stat($cfg_remote_path...
      } ## end else [ if (!$ftp)
   } ## end if ($cfg_type eq "remote_ftp"...
   return $retval;
} ## end sub ftp_connect
###############################################
sub crate_tmp_list
{
   my ($arc_block_level, $position1, $position2, $full_list_path) = @_;
   my ($tmp_list_path, $pos_counter);

   if ($arc_block_level == 0 && $position1 == 0 && $position2 eq '')
   {
      $tmp_list_path = $full_list_path;
   }
   else
   {
      $pos_counter   = 0;
      $tmp_list_path = "$full_list_path.$arc_block_level";
      open(FULL_LIST, "<$full_list_path")
        || die "Can't open full list $full_list_path\n";
      flock(FULL_LIST, 1);
      open(TMP_LIST, ">$tmp_list_path")
        || die "Can't create temp list $tmp_list_path\n";
      flock(TMP_LIST, 2);
      while (<FULL_LIST>)
      {

         if (   ($pos_counter >= $position1)
             && ($pos_counter < $position2 || $position2 eq ''))
         {
            print TMP_LIST $_;
         }
         $pos_counter++;
      } ## end while (<FULL_LIST>)
      close(TMP_LIST);
      close(FULL_LIST);
   } ## end else [ if ($arc_block_level ==...
   return $tmp_list_path;
} ## end sub crate_tmp_list
###############################################
###############################################

__END__

