#!/usr/bin/perl 
################ patch_recovery implementation

use lib '/opt/nortel/base/lib';

use warnings;

use common_variables;
use common_functions;
use baselog;
use basedb;
use install_common;
use ifxml;
use patchutil::patching;
use patchutil::patches_public;
use patchutil::patches_target;

use recovery::backup;

use base_system;

sub main();             #entry point
sub default_message();  #default help message

my $patch_recovery_source     = "$basebackup_dir/$patches_data_archive";
my $backup_baseversion_source = "$backup_cookie";

exit(main());

sub main()
{
   my ($DIR, $patch, @patches, $pins_time_tmp);
   my @tmp_list = ();
   my @patch_list = ();

   if ((!defined @ARGV) || ($ARGV[0] ne '--recover'))
   {
      default_message();
      return $ERROR;
   }
   baselog($LOG_INFO, "Patch recovery...\n");

   #
   # This function is only meant to be run in an re-install scenario to recover the original patches
   # prior to the re-install, thus if the first boot flag does not exist then we are in error.
   #
   if (!-e $BASEFIRSTBOOTUP)
   {
      baselog($LOG_ERROR,"Patch recovery function can only be called in a firstbootup scenario.", $TRUE);
      return $ERROR;
   }
   
   #
   # The assumption is that the Recovery.pl has already run as part of the PostInstall.pl script
   # and thus if any patches were backed up (as part of the upgrade) then a patch recovery archive
   # "/admin/nortel/backup/base/patchesinfo.tar.gz" file should exists at this point
   #   
   if (!-e $patch_recovery_source)
   {
      baselog($LOG_INFO, "Nothing to recover.\n", $TRUE);
      return $OK;
   }

   # Check the installed and backed up base version
   if (-e $backup_baseversion_source)
   {
      baselog($LOG_DEBUG, "Recovery source $backup_baseversion_source exists.");
      if (`$CAT $backup_baseversion_source | $GREP baseversion | $WC -l` != 1)
      {
         baselog($LOG_WARNING, "Backup does not have valid base version information.\n");
         return $ERROR;
      }
      else
      {
         my $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
         if (!$parser)
         {
            baselog($LOG_WARNING, "Installed base version undefined. Patch recovery skipped.");
            return $ERROR;
         }
         my $current_base_version = $parser->get_application_version({name => 'base'});

         #calling destructor;
         undef $parser;

         chomp(my $backup_version = `$CAT $backup_baseversion_source | $GREP baseversion`);
         baselog($LOG_DEBUG, "Backed up base version $backup_version.");

         @tmp_list = ();
         @patch_list = ();
         if ($backup_version ne "baseversion=@$current_base_version")
         {
            baselog($LOG_WARNING, "Backed up ($backup_version) and current (baseversion=@$current_base_version) base version are different.\n", $TRUE);

            baselog($LOG_WARNING, "Patches are neither retrieved nor installed from backup ($patch_recovery_source)", $TRUE);

            return $ERROR;

         } ## end if ($backup_version ne...
         else
         {  #
            # Retrieve all patches & SPs and the patch db since that is all that is required 
            # to re-install the patches in a Linux server "upgrade-reinstall" (in the current context).
            #
            baselog($LOG_INFO, "Retrieving backed-up patches for Linux $backup_version", $TRUE);

            # Retrieve the list of patches from the backup bundle
            if (!do_system([$TAR, "-ztf", $patch_recovery_source ], { $dsOUT => \@tmp_list, $dsNOERROR => $TRUE} ) )
            {
               baselog($LOG_ERROR, "Retrieving list of patches from the backup ($patch_recovery_source) failed", $TRUE);
               return $ERROR;
            }
            @tmp_list = grep(/.ntl$/, @tmp_list );
            chomp(@tmp_list);
            push @patch_list, @tmp_list;


            # Confirm that the patch db exists in the backup bundle
            @tmp_list = ();
            if (!do_system([$TAR, "-ztf", $patch_recovery_source, "\*\.handles" ], {$dsOUT => \@tmp_list}) )
            {
               baselog($LOG_ERROR, "Patch DB does not exist in backup ($patch_recovery_source)", $TRUE);
               return $ERROR;
            }
            chomp(@tmp_list);
            push @patch_list, @tmp_list;

            baselog($LOG_INFO, "... Patches Recovered from $patch_recovery_source \n", $TRUE);
            foreach my $pp (@patch_list)
            {
               baselog($LOG_INFO,"    ... $pp\n", $TRUE);
            }

            # Retrieve all files specified in the patch list 
            if (!do_system([$TAR, "-zPxf", $patch_recovery_source, @patch_list]) )
            {
               baselog($LOG_ERROR, "Failed to retrieve patches from backup ($patch_recovery_source)", $TRUE);
               return $ERROR;
            }
            else
            {
               baselog($LOG_INFO, "Retrieved patches from backup ($patch_recovery_source)", $TRUE);
            }
         }
      } ## end else [ if (`$CAT $backup_baseversion_source | $GREP baseversion | $WC -l`...
   } ## end if (-e $backup_baseversion_source...
   else
   {
      baselog($LOG_INFO, "No backup base version information ($backup_baseversion_source).\n", 1);
      return $ERROR;
   }
   #
   # Reapply all patches flagged in the original .handles db that was recovered
   # from the sysbackup archive file.
   #
   my $patching = patchutil::patching::new();
   if (not defined($patching))
   {
      baselog($LOG_ERROR, 'Cannot create patching object');
      return $ERROR;
   }
   else
   {
      if (not $patching->reapply())
      {
         baselog($LOG_ERROR, "Patch reapply failed");
         $patching = undef;
         return $ERROR;
      }
      $patching = undef;
      
   } ## end else [ if (not defined($patching...
   
   return $OK;

} ## end sub main()

#default message put on screen
sub default_message()
{
   $0 =~ s/.*\/(.*)$/$1/egi;
   print "Recover installed patches\n";
   print "Usage:\n";
   print " $0 --recover\n";
} ## end sub default_message()

#Ctrl-C handler
sub cleanup()
{
   return;
}
