## @file patching.pm
# @brief this module provides methods to manipulate patches
# @details none
# @author Alex Arkhipov
# @author Evgeny Lazarev
# @date May 2008
# @version 1.00 - Initial version
# @version 1.01 - the issue with SU removal was fixed: CR Q01675883
# @version 1.10 - changes for support Base Applications patching has been done.
#                 make_hash_rpms is changed.
# @version 1.11 - get rid of global variables $APP_INSTALLED and $APP_VERSION
# @version 1.20 - Next methods are added: show_pstat, show_pstat_extended, show_plis
# @version 1.21 - Two colons were removed before shared functions and variables
# @version 1.22 - add prePatch.pl script checking for pload/pins/poos/pout commands.
# @version 1.26 - Those methods are added to support obsolesence/dependencies checking
#               - poos_Dependency, check_Dependency, check_Obsoletes.
#               - pload_all, pout_all, poos_patches are added.
#               - support for 'all' and 'rpm' arguments is added to pins, poos, pout
# @version 1.27 - Changes to pstat, plis to support new format
# @version 1.28 - Add spload command for SP load and installation
# @version 1.29 - Add postPatch.pl script processing for pload/pins/poos/pout commands
# @version 1.32 - poos_Dependency, check_Dependency, check_Obsoletes are modified
#                 - app_stop and app_start methods changed
# @version 1.33 - remove_prev_rpm_record is removed
#               - All linkages for patches and SUs are changed from application to RPM.
# @version 1.34 - show_issp method added
# @version 1.35 - Changes on spload for validation of SUs and patches and update get_full_patch_file parameter for all the occurrences
# @version 1.36 - Removed semaphore usage
# @version 1.37 - Added spout
# @version 1.38 - -app parameter handling added for all commands
#                 - check_patch_preload and poos methods changed
#                 - get_app_by_rpm method removed
# @version 1.39 - Adding spins command
# @version 1.42 - system reboot after patch install/uninstall feature is added
#               - app stop/start functionality is optimazed
#               - app_stop and app_start were changed
# @version 1.43 - add spload and spins info file creation
# @version 1.44 - changes for spins command and added remove_all_patches
# @version 1.45 - Fix for Q01962578 - Patch validation against SU fix
# @version 1.46 - Fix for Q01962462
# @version 1.47 - Fix for Q01963161 - Adding reboot message printing for spins with -sp option
# @version 1.50 - Fix for Q01964161 - Avoid reboot  and apps stop/start if sp is already in service
# @version 1.51 - fix of Q01964190 Obsolete patch can be put in service if patch which obsoletes that patch is in service
# @version 1.52 - Fix for Q01964264 - appstart/appstop/reboot behaviour not consistent
# @version 1.53 - Fix for Q01964866
# @version 1.54 - fix of Q01964787 - Pre_pins of patch/SU should happen before app stop.
# @version 1.55 - Fix for Q01964287 - A FRU patch can be put in service that creates a new file
# @version 1.56 - Fix for Q01964169 - Add PREPATCH and POSTPATCH scripts for spout
# @version 1.57 - fix for Q01968202 - Can't pload patches with multiple members
# @version 1.58 - Fix for Q01966780 - Patch is getting loaded if the name or extension is changed.
# @version 1.59 - Fix for Q01964285, Q01964291 - appstart not functioning for --all options.
# @version 1.60 - Fix for Q01970812 - Print at the end of the spins which patches PID's failed to be installed.
# @version 1.61 - patch conflict checking related changes
# @version 1.62 - Fix for Q01970829 - SUs getting put in service if they fail
# @version 1.63 - Fix for Q01971843 - No appstop before taking old patch out of service
# @version 1.64 - Fix for Q01973726 - Patching Linux OS base - need more feedback to end user.
# @version 1.65 - Fix for Q01976321 - cSPins command execution fails
# @version 1.66 - Fix for Q01977711 - spins fails with error message: undefined value as an ARRAY reference.
# @version 1.67 - Fixes for Q01971324, Q01976824 - Multiple appstop during pins -all, incorrect re-start message when
# @               cancel pins and don't re-start apps when SU is kicking out patches that requires re-start.
# @version 1.68 - Fix for Q01981567 - pload is not checking for the Pre/Post scripts tags on header.
# @version 1.69 - Fix for Q01981522 - Can pins a SU with the same rpm version as is already installed
# @version 1.70 - Fix for Q01991510 - Patch backup/recovery during server reinstall not operational
# @version 1.71 - Add restore_patches function.
# @version 1.72 - Fix for Q01996711 - During spins don't remove patches have been marked non removable.
# @version 1.73 - Fix for Q01998001 - Support PI control patches.
# @version 1.74 - Fix for Q01999918 - Modify MUTEX validation and add /var/opt/nortel/piControl validation for pi patches.
# @version 1.75 - Fix for Q02005196 - Undeployment of CoRes system is not happening when ntl patches are in-service
#                                     -- Removed check for basic hardening since it can fail when undeplying applications
#               - Fix for Q02005987 - Linux application Patch loading (PLOAD) verbiage error
#                                     -- slight wording change to pload prompt
# @version 1.76 - Fix for Q02007605 - Allow spload to continue if a patch failed to be loaded.
# @version 1.77 - Print correct message after spins completion.
# @version 1.78 - Fix for Q02002104 - Fail to detect conflict when activating FRU patch with BIN patch in service
# @version 1.79 - Fix for Q02011748 - to move validate_min_rpm from pins_patch to check_patch_preload.
# @version 1.80 - Post patching actions are performed for enhanced hardening.
# @version 1.81 - Fix for Q02013365 - spload should not fail for non-installed rpms
#               - fix for SP app start/stop list ordering [collect_apps()]
# @version 1.82 - Fix for Q02015937 - several user messages were changed.
# @version 1.83 - Fix for Q02021120 - SP Failure due to PI FRU Patches in Bundle (PI FRU MIN_RPM_VER criteria not met)
# @version 1.84 - Fix for Q02005731 - pstat error - provide hint of possible access rights issue
# @version 1.85 - Fix for Q02022105 - GALTT: SP installation tries to stop/start non relevant application
# @version 1.86 - Fix for Q02027206 - CS RPM/FW Distributions failing Linux FRU Patching
# @version 1.87 - Fix for Q02028466 - SP loading fails when SP contains a Linux Base SU and a patch for the SU
# @version 1.88 - Fix for Q02031389 - Ploading a Patch fails if SU is in a loaded state
# @version 1.89 - Fix for Q02033504 - Undeployment failed and can not remove applications patches
# @version 1.90 - Fix for Q02031964 - UCM/PM currently does not handle the pout'ing of patches after target reboot
# @version 1.91 - Fix for Q02032791-01 - Allow to apply empty SP(no patches inside)
# @version 1.92 - Add changes to complete fix Q02031964 and to pout all patches automatically if "poos" is called from PM
# @version 1.93 - If poos was invoked from patch manager pout actions will be invoked automaticaly
# @version 1.94 - Workaround for poos/upgrade linuxbase SU is implemented(related to bad original spec filr for R6.0)
# @version 1.95 - fix for Q02046509-01 - PM - Misleading information does not identify errors found in patch activation
# @version 1.96 - spload modification to ignore loading a SU if the associated RPM is already installed on the target (no error)
#               -  pload modification to ignore loading a SU if the associated RPM is already installed on the target (error reported)
# @version 2.00 - Fix for Q02094833 - Nortel_Service_Pack_Linux_6.00_18_Nov_20.ntl does not load properly
# @version 2.01 - Fix for Enhancement CR Q02059485 - CS 1000 R6.0 Linux Patching Support of "REFRESH" SP Capability
# @version 2.02 - Fix for Q02077155 - Unable to loginto CoResMemberserver under nortel after installin linuxbase SU#21

# @note

package patchutil::patching;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;

use common_variables;
use common_functions;
use patchutil::patches_public;
use patchutil::patches_target;
use CommonInstallUtils;

use install_common;

### Access management
use Exporter;
our @ISA    = qw(Exporter);  # : public Exporter;
our @EXPORT = qw($VERSION);

use ifxml;
use basedb;
use baselog;
use patchutil::patch_db;
use patchutil::patch;
use base_snmp;
use base_semctl;
use base_system;
use patchutil::header_xml;
use xmlvalid;
use hardening::hardening_common;

our $VERSION = "2.02";

################################
# Local variables definitions
################################
my $class = 'patchutil::patching';

################################
# Public methods
################################
sub pload($$);
sub pins($$);
sub poos($$);
sub pout($$);
sub spload($$);
sub spout($$);
sub spins($$);

sub set_interact($$);
################################
# End of public methods
################################

################################
# Private methods
################################

################################
# End of private methods
################################

#######################################
# Create new object
# Input: None
# Output: reference to the new object
#         undef if error occurs
#######################################
sub new()
{
   unless (create_patch_dir())
   {
      baselog($LOG_ERROR, "Temporary directory cannot be created.");
      return undef;
   }
   my $self = {};
   $self->{'interact'} = $TRUE;
   $self->{'appstart'} = $TRUE;
   $self->{'appstop'}  = $TRUE;
   $self->{'action'}   = "none";
   $self->{'context'}  = '';

   recover_env_path();

   # create patch DB object
   my $pDB = patchutil::patch_db::new();
   if (!defined($pDB))
   {
      baselog($LOG_ERROR, 'The "patch_db" creation failed');
      return undef;
   }
   $self->{'DB'} = $pDB;
   bless $self, $class;
   $self->make_hash_rpms();
   baselog($LOG_DEBUG, "The '$class' object created");
   return $self;
} ## end sub new()

#######################################
# Delete object
# Input: None
# Output: None
#######################################
sub DESTROY($)
{
   my $self = shift(@_);

   #remove DB
   $self->{'DB'} = undef;
   unless (remove_patch_dir())
   {
      baselog($LOG_ERROR, "Temporary directory cannot be cleaned.");
   }
   baselog($LOG_DEBUG, "The '$class' object destroyed");
} ## end sub DESTROY($)

####################################################
# Set patching mode
# Input: $_[0] - object reference
#        $_[1] - interactive or non-interactive mode
#          $TRUE - interactive mode
#          (user will be prompted for confirmation)
#          $FALSE - non-interactive mode
#          (user will not be prompted for confirmation)
# Output: None
#####################################################
sub set_interact($$)
{
   my ($self, $mode) = @_;
   $self->{'interact'} = ($mode) ? $TRUE : $FALSE;
   baselog($LOG_DEBUG, "'interact' mode is set to: " . $mode);
}
########################################################
# Set sp flag
# Input: $_[0] - object reference
#        $_[1] - sp flag received from command line
#          $TRUE - if sp command invoked from GUI
#          (user will be prompted for confirmation)
#          $FALSE - if sp command not invoked from GUI
#          (user will not be prompted for confirmation)
# Output: None
########################################################
sub set_sp($$)
{
   my ($self, $sp_flag) = @_;
   $self->{'sp'} = ($sp_flag) ? $TRUE : $FALSE;
   baselog($LOG_DEBUG, "'sp' is set to: " . $sp_flag);
}
####################################################
# Description: This func sets flags: start and stop
#            by default app start and stop performs
#            anytime
#
# Input: $_[0] - object reference
#        $_[1] - flag appstart - start application
#                during pins and poos
#                $TRUE - start
#                $FALSE - not start
#        $_[2] - flag appstop - stop application
#                during pins and poos
#                $TRUE - stop
#                $FALSE - not stop
# Output: None
#####################################################
sub set_app_flags($$$)
{
   my ($self, $appstart, $appstop) = @_;
   $self->{'appstart'} = ((not defined $appstart) || ($appstart == $TRUE)) ? $TRUE : $FALSE;
   baselog($LOG_DEBUG, "'app start' mode is set to: " . $self->{'appstart'});
   $self->{'appstop'} = ((not defined $appstop) || ($appstop == $TRUE)) ? $TRUE : $FALSE;
   baselog($LOG_DEBUG, "'app stop' mode is set to: " . $self->{'appstop'});
} ## end sub set_app_flags($$$)

####################################################
# Load patch
# Input: $_[0] - reference to the object
#        $_[1] - reference to the parameters' hash
#          'filename' - filename to load
#          'all' - load all patches from patch directory
# Output: $TRUE - success
#         $FALSE - error during loading
#####################################################
sub pload($$)
{
   my $self   = shift(@_);
   my $par    = shift(@_);
   my $retval = $TRUE;

   $self->{'action'} = $PLOAD;
   if (defined $par->{'interact'})
   {
      $self->set_interact($par->{'interact'});
   }
   if (defined $par->{'sp'})
   {
      $self->set_sp($par->{'sp'});
   }
   if (exists $par->{'filename'})
   {
      $retval = $self->pload_file($par->{'filename'}, undef);
   }
   elsif (exists $par->{'all'})
   {
      $retval = $self->pload_all();
   }
   else
   {
      baselog($LOG_ERROR, "Incorrect parameters provided", $TRUE);
      return $FALSE;
   }
   return $retval;
} ## end sub pload($$)

####################################################
# Load patch based on file name
# Also the cleanup of the patch special directory
# needs to be added when any of these conditions occurr:
#      1. Load prepatch script fails
#      2. Load of readme file fails (special instructions)
#      3. Cannot load patch into database
# Input: $_[0] - reference to the object
#        $_[1] - filename of patch in patch dir
#        $_[2] - application name. If defined then
#          patch should be loaded only if belongs
#          to specified application.
# Output: $TRUE - success
#         $FALSE - error during loading. Set 'error' variable
#####################################################
sub pload_file($$$)
{
   my $self = shift(@_);
   my ($file, $app) = @_;
   my $cmd_retval;  # used to return script return value
   my $command = "pload";

   # make full path
   $file = $PATCHDIR . "/" . $file;
   baselog($LOG_DEBUG, "Pload file invoked: $file");

   # create patch object
   my $patch = patchutil::patch::new({'file' => $file});
   if (!defined($patch))
   {
      baselog($LOG_ERROR, "Failed to load patch: $file.\n", $TRUE);
      return $FALSE;
   }
   my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};

   if ($self->{'DB'}->is_patch_loaded($patch->{'PATCHNAME'}))
   {
      baselog($LOG_ERROR, "Patch $patch->{'PATCHNAME'} is already loaded\n", $TRUE);
      return $TRUE;
   }
   baselog($LOG_INFO, "Loading patch " . $file . "\n");
   unless ($self->check_patch_preload($patch, $file))
   {
      baselog($LOG_ERROR, "Check patch preload failed for patch: $file\n", $TRUE);
      return $FALSE;
   }
   baselog($LOG_DEBUG, 'Patch preload check is completed',);

   # Cleanup special directory for the patch
   unless (spec_dir_cleanup($spec_dir))
   {
      baselog($LOG_ERROR, "Cannot cleanup $spec_dir directory\n", $TRUE);
      return $FALSE;
   }

   # Check patch special instruction
   unless ($self->check_spec_ins($patch))
   {
      baselog($LOG_ERROR, "Cannot cleanup $spec_dir directory\n", $TRUE) unless spec_dir_cleanup($spec_dir);
      return $FALSE;
   }

   baselog($LOG_DEBUG, "SpecIns check is completed");

   my $prePatch_exists;
   my $prePatch_retval;

   #check PREPATCH script existence
   do_system([$TAR, 'tfz', $file, $PREPATCH], {$dsNOERROR => $TRUE, $dsRETVAL => \$prePatch_retval});
   if (not $prePatch_retval)
   {

      if (!$self->load_script($patch, $PREPATCH))  # load PREPATCH script
      {
         unless (spec_dir_cleanup($spec_dir))
         {
            baselog($LOG_ERROR, "Cannot cleanup $spec_dir directory\n", $TRUE);
         }
         baselog($LOG_ERROR, "Loading script failed for $patch->{'PATCHNAME'}", $TRUE);
         return $FALSE;
      } ## end if (!$self->load_script...
   } ## end if (not $prePatch_retval...

   # if PREPATCH script exists, validate that tags are set to yes, otherwise print error
   # if PREPATCH script doesn't exists, validate that tags are set to no, otherwise print error
   if (!$self->check_script($patch, $PREPATCH, \$prePatch_exists))  # if PREPATCH script exists
   {
      return $FALSE;
   }

   my $postPatch_exists;
   my $postPatch_retval;

   #check POSTPATCH script existence
   do_system([$TAR, 'tfz', $file, $POSTPATCH], {$dsNOERROR => $TRUE, $dsRETVAL => \$postPatch_retval});
   if (not $postPatch_retval)
   {
      if (!$self->load_script($patch, $POSTPATCH))                  # load POSTPATCH script
      {
         unless (spec_dir_cleanup($spec_dir))
         {
            baselog($LOG_ERROR, "Cannot cleanup $spec_dir directory\n", $TRUE);
         }
         baselog($LOG_ERROR, "Loading script failed for $patch->{'PATCHNAME'}", $TRUE);
         return $FALSE;
      } ## end if (!$self->load_script...
   } ## end if (not $postPatch_retval...

   # if POSTPATCH script exists, validate that tags are set to yes, otherwise print error
   # if POSTPATCH script doesn't exists, validate that tags are set to no, otherwise print error
   if (!$self->check_script($patch, $POSTPATCH, \$postPatch_exists))  # if POSTPATCH script exists
   {
      return $FALSE;
   }

   if ($prePatch_exists eq "yes")
   {

      # run patch PREPATCH script
      $cmd_retval = $self->run_script($patch, $PREPATCH, $command);
      if ($cmd_retval == $DONE)
      {
         return $TRUE;
      }
      elsif ($cmd_retval == $ERROR)
      {
         return $FALSE;
      }
   } ## end if ($prePatch_exists eq...

   # Load patch to the DB
   if (!$self->{'DB'}->pload_patch($patch))
   {
      baselog($LOG_ERROR, "Cannot cleanup $spec_dir directory\n", $TRUE) unless spec_dir_cleanup($spec_dir);
      return $FALSE;
   }

   if (exists $self->{'sp'} and not $self->{'sp'})
   {
      print "Patch handle is: $patch->{'PID'}\n\n";
   }

   #write protect the patch file
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, '440', $file]))
   {
      baselog($LOG_ERROR, "Unable to write-protect $file file\n", $TRUE);
   }

   if ($postPatch_exists eq "yes")
   {

      # run patch POSTPATCH script
      $cmd_retval = $self->run_script($patch, $POSTPATCH, $command);
      if ($cmd_retval == $DONE)
      {
         return $TRUE;
      }
      elsif ($cmd_retval == $ERROR)
      {
         return $FALSE;
      }
   } ## end if ($postPatch_exists ...

   return $TRUE;

} ## end sub pload_file($$$)

####################################################
# Load all patches from patch directory created for
# Input: $_[0] - reference to the object
# Output: $TRUE - success
#         $FALSE - error during loading
#####################################################
sub pload_all($)
{
   my $self = shift(@_);
   my $app  = shift(@_);
   my (@patches, $dir);

   baselog($LOG_DEBUG, 'Pload all invoked: ');

   # get sorted file list
   unless (opendir($dir, $PATCHDIR))
   {
      baselog($LOG_ERROR, "Cannot open $PATCHDIR for reading");
      return $FALSE;
   }
   @patches = readdir($dir);
   closedir($dir);

   # use only patches (.ntl)
   @patches = sort(grep(/$PATCHEXT$/, @patches));

   my $retval = $TRUE;

   # for each patch file (*.ntl) in patch directory
   foreach my $patch (@patches)
   {

      # try loading patch
      $retval &= $self->pload_file($patch, undef);
   } ## end foreach my $patch (@patches...
   return $retval;
} ## end sub pload_all($)

#####################################################
## @fn spload($self, $par)
# @brief Tries to pload the content of a SP
# @details Call load_bundle with the SP file name
# @        as parameter. This command assumes that
# @        the SP file is located under the SP directory.
# @param $self - reference to the object
# @param $par - reference to the parameters' hash
# @             'filename' - bundle to load
# @return TRUE - load_bundle succeeds
# @return FALSE - load_bundle fails
#####################################################
sub spload($$)
{
   my $self = shift(@_);
   my $par  = shift(@_);

   $self->{'action'} = $SPLOAD;
   if (defined $par->{'interact'})
   {
      $self->set_interact($par->{'interact'});
   }
   if (defined $par->{'sp'})
   {
      $self->set_sp($par->{'sp'});
   }
   if (!exists($par->{'filename'}))
   {
      baselog($LOG_ERROR, "Incorrect parameters provided");
      return $FALSE;
   }
   else
   {
      baselog($LOG_INFO, "This command will load all the SUs and patches contained in the SP ($par->{'filename'}).\n", $TRUE);
      if ($self->{'interact'})
      {
         unless (getYNResponse("Do you want to proceed?", 'N'))
         {
            baselog($LOG_INFO, "spload command cancelled for SP $par->{'filename'}.\n", $TRUE);
            return $TRUE;
         }
      } ## end if ($self->{'interact'...

      if (!$self->load_bundle($par->{'filename'}))
      {
         return $FALSE;
      }

   } ## end else [ if (!exists($par->{'filename'...
   return $TRUE;
} ## end sub spload($$)

############################################################
## @fn sub load_bundle($self, $bundle_fname)
# @brief Load bundle on the system
# @details 1. If bundle exists under SP dir, extract bundle
# @        under SPLOAD dir if SPLOAD dir doesn't exists
# @        2. If SPLOAD dir exists, print error, since another
# @        bundle need to be put in service or cancel previous
# @        spload.
# @        3. Create zero length file with bundle name
# @        4. Validate product Rls inside SP header
# @        5. Check for special instructions (readme)
# @        6. Run prepatch script if exists
# @        7. Set SP type
# @        8. Get list of obsolete SUs and patches from SP header
# @           (if it is a DELTA SP).
# @        9. Obtain list of SUs and patches required on the
# @        system, base on the SP header.
# @        10. pout all patches with status out of service.
# @        11. Load all SUs and patches
# @        12. Run POSTPATCH script if exists
# @return TRUE - load of the bundle succeds (try to load the
# @              required SUs and patches))
# @return FALSE - load of the bundle fails
#############################################################
sub load_bundle($$)
{
   my ($self, $bundle_fname) = @_;

   my $cmd_retval;  # used to return retval of pre and post script
   my $bundle_full_path;
   my @required_SUS             = ();
   my @required_patches         = ();
   my @obsolete_SUs_and_patches = ();
   my $pload_all_retval         = $TRUE;

   my $bundle_full_name = $SRV_PACK_DIR . "/" . $bundle_fname;

   # extract bundle under SPLOAD dir
   if (!$self->extract_bundle($bundle_full_name))
   {
      return $FALSE;
   }
   if (!create_spload_info_file($self, "$bundle_fname"))
   {
      baselog($LOG_ERROR, "Cannot create length zero bundle name.\n", $TRUE);
      return $FALSE;
   }
   if (!validate_product_rls($self))
   {
      return $FALSE;
   }

   # Check service pack special instructions
   unless ($self->check_SP_spec_ins($bundle_full_name))
   {
      return $FALSE;
   }

   # run SP PREPATCH script if exists
   $cmd_retval = $self->run_SP_script($PREPATCH, "spload", $SPLOAD_DIR);
   if ($cmd_retval == $DONE)
   {
      return $TRUE;
   }
   elsif ($cmd_retval == $ERROR)
   {
      return $FALSE;
   }

   # set SP type (i.e. DELTA_SP, SEARCH_SP, SP)
   if (not $self->set_SP_type())
   {
      baselog($LOG_ERROR, "Cannot determine SP type.", $TRUE);
      return $FALSE;
   }

   # extract list of obsolete SUs and patches from SP header
   if ($self->{'SP_type'} eq 'DELTA_SP')
   {
      if (not $self->obtain_obsoletes(\@obsolete_SUs_and_patches))
      {
         baselog($LOG_ERROR, "Cannot obtain obsolete SUs and patches from SP.", $TRUE);
         return $FALSE;
      }
   } ## end if ($self->{'SP_type'}...

   # extract list of SU and patches required from the SP
   if (not $self->obtain_SUS_and_patches(\@required_SUS, \@required_patches, \@obsolete_SUs_and_patches))
   {
      baselog($LOG_ERROR, "Cannot obtain needed SUs and patches from SP.", $TRUE);
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Outing patches with out of service status.\n", $TRUE);
   unless ($self->pout_all())
   {
      baselog($LOG_ERROR, "Cannot pout all the patches that have out of service status.\n", $TRUE);
      return $FALSE;
   }

   # save old sp and interact flags
   my $old_sp       = $self->{'sp'};
   my $old_interact = $self->{'interact'};

   # set sp and interact flags to pload all members
   $self->set_interact($FALSE);
   $self->set_sp($TRUE);

   my @sus_to_load = @required_SUS;

   #pload all SUS
   if (!pload_all_members($self, 'SU', \@sus_to_load))
   {
      $pload_all_retval = $FALSE;
   }

   #pload all patches
   if (!pload_all_members($self, 'PATCH', \@required_patches, \@required_SUS))
   {
      $pload_all_retval = $FALSE;
   }

   # recovered all values of sp and interact flags
   $self->set_sp($old_sp);
   $self->set_interact($old_interact);

   # run SP POSTPATCH script if exists
   $cmd_retval = $self->run_SP_script($POSTPATCH, "spload", $SPLOAD_DIR);

   if ((!$pload_all_retval) || ($cmd_retval == $ERROR))
   {
      my $status_string = patchutil::patches_target::get_fault_sp_str('spload', $bundle_fname);
      baselog($LOG_ERROR, "\n$status_string\n", $TRUE);
      return $FALSE;
   } ## end if ((!$pload_all_retval...
   if ($cmd_retval == $DONE)
   {
      return $TRUE;
   }
   baselog($LOG_INFO, "\nspload command for SP $bundle_fname completed successfully.\n", $TRUE);
   return $TRUE;
} ## end sub load_bundle($$)

#####################################################
## @fn spins($self, $par)
# @brief Put in service the content of a SP already loaded
# @details Call pins_bundle
# @param $self - reference to the object
# @return TRUE - install_bundle succeeds
# @return FALSE - install_bundle fails
#####################################################
sub spins($$)
{
   my ($self, $par) = @_;

   $self->{'action'} = $SPINS;
   if (defined($par->{'interact'}))
   {
      $self->set_interact($par->{'interact'});
   }
   if (defined($par->{'sp'}))
   {
      $self->set_sp($par->{'sp'});
   }
   if (!-d $SPLOAD_DIR)
   {
      return $FALSE;
   }

   #find the bundle name currently loaded in the system
   my @bundle_files = ();
   my ($dir, $bundle_name);

   # get sorted file list
   unless (opendir($dir, $SPLOAD_DIR))
   {
      baselog($LOG_ERROR, "Cannot open $SPLOAD_DIR directory for reading\n", $TRUE);
      return $FALSE;
   }
   @bundle_files = readdir($dir);
   closedir($dir);
   my $bundle_ext = ".bundle";
   my @sp_name_with_bundle_ext = grep(/$bundle_ext$/, @bundle_files);
   $bundle_ext = join(' ', @sp_name_with_bundle_ext);
   if ($bundle_ext =~ /^(.*?)\.bundle$/)
   {
      $bundle_name = $1;
   }
   if (!defined($bundle_name))
   {
      baselog($LOG_ERROR, "SP is missing from $SPLOAD_DIR directory\n", $TRUE);
      return $FALSE;

   } ## end if (!defined($bundle_name...

   baselog($LOG_INFO, "All SUs and patches within the SP $bundle_name, which are applicable to the target, will be put into service\n", $TRUE);
   baselog($LOG_INFO, "This command may cause service interruption.\n", $TRUE);
   my $ret_val = $self->install_bundle();
   if ($ret_val == $ERROR)
   {

      # spins command cli will print message for failure
      return $FALSE;
   } ## end if ($ret_val == $ERROR...
   elsif ($ret_val == $ABORT)
   {
      baselog($LOG_INFO, "\nspins command aborted for SP $bundle_name.\n", $TRUE);
   }
   elsif ($ret_val == $FAULT)
   {
      my $status_string = patchutil::patches_target::get_fault_sp_str('spins', $bundle_name);
      baselog($LOG_INFO, "\n$status_string\n", $TRUE);
      create_spins_info_file($self, "$bundle_name", $ret_val);
   } ## end elsif ($ret_val == $FAULT...
   elsif ($ret_val == $OK)
   {
      baselog($LOG_INFO, "\nspins command for SP $bundle_name completed successfully.\n", $TRUE);
      create_spins_info_file($self, "$bundle_name", $ret_val);
   }
   return $TRUE;
} ## end sub spins($$)
############################################################
## @fn sub install_bundle($self)
# @brief Install bundle on the system
# @details 1. Run SP prepatch script
# @        2. Get list of all sus and patches from database
# @        3. Set SP type
# @        4. Get list all sus and patches from SP header
# @        5. Get list of obsolete SUs and patches from SP header
# @           (if it is a DELTA SP).
# @        6. Create a list of patch object from all the sus and
# @           patches obtained from SP header
# @        7. Collect common info base on lists of patches and
# @           sus from DB and SP header. This means to collect:
# @           - list of patches to be in service (depends on SP type)
# @           - list of sus to be in service (depends on SP type)
# @           - list of patches to be out of service (depends on SP type)
# @           - list of sus to be out of service (depends on SP type)
# @           - list of application to stop for patches and sus
# @           - list of application to start for patches and sus
# @           - set reboot flag to TRUE if at least one of the patches or sus to be in service or out
# @             of service requires reboot.
# @        8. Prepare directories under SP before proceeding with spins processing
# @        9. Stop all applications required base on common_info collected
# @        10. Process patches out of service
# @           - If patch is in service, poos patch
# @           - If patch is out of servive, pout patch
# @           - Move patch to sp/revert/ directory
# @        11. Process sus out of service
# @           - If su is in service, poos patch
# @           - If su is out of servive, pout patch
# @           - Move su to sp/revert/ directory
# @        12. Put in service all required sus and patches
# @        13. Run SP postpatch script
# @        14. If reboot flag is not set, start all applications required base on common_info collected
# @        15. Otherwise, if reboot flag is set, reboot the system
# @        NOTE: If during processing patches and sus out of service or in service, there is a failure,
# @        an error is printed and next patch or su is processed.
# @return $OK - SP installation succeeds.
# @return $ERROR - SP installation fails. Database wasn't changed.
# @return $FAULT - During SP installation found a problem, but keep going.
# @return $ABORT - SP installation aborted.
#############################################################
sub install_bundle($)
{
   my $self = shift(@_);
   my (@required_files, @required_SUS, @required_patches, @obsolete_SUs_and_patches, @patches_list) = ((), (), (), (), ());
   my @dep_list = ();
   my $cmd_retval;
   my $reboot_flag;
   my $status_in_out_srv_reboot;
   my $ret_val        = $OK;
   my @failed_patches = ();     # list of filenames which don't have pid
   my $empty_SP       = $FALSE;

   # run SP PREPATCH script if exists for spins
   $cmd_retval = $self->run_SP_script($PREPATCH, "spins", $SPLOAD_DIR);
   if ($cmd_retval == $DONE)
   {
      return $OK;
   }
   elsif ($cmd_retval == $ERROR)
   {
      return $ERROR;
   }

   # get all sus and patches from database sorted by sus first and patches second
   my @files_in_db = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);
   if (not scalar(@files_in_db))
   {
      baselog($LOG_INFO, "Patch database is empty.", $TRUE);
      return $ERROR;
   }

   # set SP type (i.e. DELTA_SP, SEARCH_SP, SP)
   if (not $self->set_SP_type())
   {
      baselog($LOG_ERROR, "Cannot determine SP type.", $TRUE);
      return $ERROR;
   }

   # extract list of SUs and patches from SP header
   if (not $self->obtain_SUS_and_patches(\@required_SUS, \@required_patches))
   {
      baselog($LOG_ERROR, "Cannot obtain needed SUs and patches from SP.", $TRUE);
      return $ERROR;
   }

   # extract list of obsolete SUs and patches from SP header
   if ($self->{'SP_type'} eq 'DELTA_SP')
   {
      if (not $self->obtain_obsoletes(\@obsolete_SUs_and_patches))
      {
         baselog($LOG_ERROR, "Cannot obtain obsolete SUs and patches from SP.", $TRUE);
         return $ERROR;
      }
   } ## end if ($self->{'SP_type'}...

   # merge in one list all sus and patches required
   my @files_list = ();
   push(@files_list, @required_SUS);
   push(@files_list, @required_patches);

   #get list of required sus and patch objects
   @required_files = get_object_list($self, \@files_list, \@failed_patches);
   if (!scalar(@required_files))
   {
      baselog($LOG_WARNING, "SP only specifies a header. Non relevant SUs and Patches will be removed.\n", $TRUE);

      #it is an empty SP (Q02032791-01)
      $empty_SP = $TRUE;
   } ## end if (!scalar(@required_files...

   if ($self->validate_SUs_dup_on_SP(@required_files) == $ERROR)
   {
      return $ERROR;
   }

   if (not $self->check_obsolete_and_required_files(\@required_files, \@obsolete_SUs_and_patches))
   {
      baselog($LOG_ERROR, "There are SUs/Patches that are tagged as both obsolete & required in the SP header", $TRUE);
      return $ERROR;
   }

   #collect common info from patches and SUs in database
   my $common_info = $self->collect_common_info(\@files_in_db, \@required_files, \@obsolete_SUs_and_patches, \$reboot_flag, \$status_in_out_srv_reboot);
   if (not defined $common_info)
   {
      baselog($LOG_ERROR, "Cannot obtain common info from SP patches and SUs.\n", $TRUE);
      return $ERROR;
   }
   if (not defined $status_in_out_srv_reboot)
   {
      baselog($LOG_ERROR, "status_in_out_srv_reboot flag undefined\n.", $TRUE);
      return $ERROR;
   }
   baselog($LOG_DEBUG, "Flag status_in_out_srv_reboot is set to $status_in_out_srv_reboot.\n");

   # if there is any patch or SU that has status IN_SERV_REBOOT or OUT_SERV_REBOOT
   if (($status_in_out_srv_reboot))
   {
      baselog($LOG_ERROR,
             "There are some patches from database with status IN_SERV_REBOOT or OUT_SERV_REBOOT which require reboot to update their status. Please reboot the system before running spins command.\n",
             $TRUE);
      return $ERROR;
   } ## end if (($status_in_out_srv_reboot...
   if (not defined $reboot_flag)
   {
      baselog($LOG_ERROR, "reboot_flag undefined\n.", $TRUE);
      return $ERROR;
   }
   my $dependency_check = $OK;

   #sort sus and patches to be in service in order of dependency
   my @in_srv_lst = ();

   # merge in one list all sus and patches required
   push(@in_srv_lst, @{$common_info->{'sus_in_srv'}})     if ((scalar(@{$common_info->{'sus_in_srv'}})) > 0);
   push(@in_srv_lst, @{$common_info->{'patches_in_srv'}}) if ((scalar(@{$common_info->{'patches_in_srv'}})) > 0);

   my @in_srv_files = $self->sort_by_dependency(\@in_srv_lst, \$dependency_check);

   if (not defined $in_srv_files[0])
   {
      baselog($LOG_ERROR, "There are no required SUs and patches which must be installed.");
      return $ERROR;
   }
   if (not defined $dependency_check)
   {
      baselog($LOG_ERROR, "dependency_check flag undefined.\n", $TRUE);
      return $ERROR;
   }
   if ($dependency_check == $ERROR)
   {
      return $ERROR;
   }

   # Notify the user for those patches that failed to be loaded during spload command
   if (scalar(@failed_patches))
   {
      baselog(
              $LOG_WARNING,
              "\n\tThe following patches will not be put into service since they were not loaded by spload:  "
                . join(', ', @failed_patches)
                . " . \nPlease refer to the spload output for more details.\n\n",
              $TRUE
             );
   } ## end if (scalar(@failed_patches...

   my $auto_reboot = $FALSE;
   baselog($LOG_DEBUG, "The reboot_flag is set to $reboot_flag, interact flag is set to $self->{'interact'} an sp flag is set to $self->{'sp'}.\n");

   # if a patch or SU requires reboot and if the interact and sp flags have default values, which it
   # means that spins was issued without -r (default value TRUE)  and -sp (default value parameters
   if (   (defined $reboot_flag)
       && ($reboot_flag)
       && (exists $self->{'interact'})
       && ($self->{'interact'})
       && (exists $self->{'sp'})
       && (!$self->{'sp'}))
   {

      # if any applications need to be stopped print message to the user
      if (scalar(@{$common_info->{'stop'}}))
      {
         my $apps_to_stop = $self->shrink_apps($common_info->{'stop'});
         if (scalar(@{$apps_to_stop->{'base'}}))
         {
            baselog($LOG_WARNING, "All Nortel and " . join(' ', @{$apps_to_stop->{'base'}}) . " applications will be stopped.\n", $TRUE);
         }
         else
         {
            baselog($LOG_WARNING, "The following applications will be stopped: " . join(' ', @{$apps_to_stop->{'nortel'}}) . ".\n", $TRUE);
         }
      } ## end if (scalar(@{$common_info...
      baselog($LOG_WARNING, "Automatic reboot will happen after SP is installed.\n", $TRUE);
      $auto_reboot = $TRUE;
   } ## end if ((defined $reboot_flag...
   else
   {

      # if any applications need to be stopped print message to the user
      if (scalar(@{$common_info->{'stop'}}))
      {
         my $apps_to_stop = $self->shrink_apps($common_info->{'stop'});
         if (scalar(@{$apps_to_stop->{'base'}}))
         {
            baselog($LOG_WARNING, "All Nortel and " . join(' ', @{$apps_to_stop->{'base'}}) . " applications will be stopped.\n", $TRUE);
         }
         else
         {
            baselog($LOG_WARNING, "The following applications will be stopped: " . join(' ', @{$apps_to_stop->{'nortel'}}) . ".\n", $TRUE);
         }
      } ## end if (scalar(@{$common_info...

      # if any applications need to be started print message to the user
      if (scalar(@{$common_info->{'start'}}))
      {
         my $apps_to_start = $self->shrink_apps($common_info->{'start'});
         if (scalar(@{$apps_to_start->{'base'}}))
         {
            baselog($LOG_WARNING, "All Nortel and " . join(' ', @{$apps_to_start->{'base'}}) . " applications will be started.\n", $TRUE);
         }
         else
         {
            baselog($LOG_WARNING, "The following applications will be started: " . join(' ', @{$apps_to_start->{'nortel'}}) . ".\n", $TRUE);
         }
      } ## end if (scalar(@{$common_info...
   } ## end else [ if ((defined $reboot_flag...

   if ($self->{'interact'})
   {
      if (!getYNResponse("Do you want to proceed?", 'N'))
      {
         return $ABORT;
      }
   } ## end if ($self->{'interact'...

   # prepare directories under SP directory before processing spins
   if (!prepare_dirs_for_spins())
   {
      if (!revert_dirs_for_spins())
      {
         baselog($LOG_ERROR, "Cannot revert spins directories.\n", $TRUE);
      }
      else
      {
         baselog($LOG_DEBUG, "Revert spins directories successful.\n");
      }
      return $ERROR;
   } ## end if (!prepare_dirs_for_spins...

   # stop all the applications needed to install SP
   if (scalar(@{$common_info->{'stop'}}))
   {
      if (sp_apps_stop($self, \@{$common_info->{'stop'}}) == $ERROR)
      {
         $ret_val = $FAULT if ($ret_val != $FAULT);
      }
   } ## end if (scalar(@{$common_info...

   #for all the patches not present in the SP header, poos and pout them.
   if (scalar(@{$common_info->{'patches_out_db'}}))
   {
      if (process_patches_out($self, \@{$common_info->{'patches_out_db'}}) == $ERROR)
      {
         $ret_val = $FAULT if ($ret_val != $FAULT);
      }
   } ## end if (scalar(@{$common_info...

   #put all sus not present on SP header, poos and pout them
   if (scalar(@{$common_info->{'sus_out_db'}}))
   {
      if (process_patches_out($self, \@{$common_info->{'sus_out_db'}}) == $ERROR)
      {
         $ret_val = $FAULT if ($ret_val != $FAULT);
      }
   } ## end if (scalar(@{$common_info...

   #put all sus and patches present on SP header and sorted by dependency in service
   if (scalar(@in_srv_files))
   {

      #do not pins patches in case SP is empty
      unless ($empty_SP == $TRUE)
      {
         if (process_patches_in($self, \@in_srv_files) == $ERROR)
         {
            $ret_val = $FAULT if ($ret_val != $FAULT);
         }

      } ## end unless ($empty_SP == $TRUE...
   } ## end if (scalar(@in_srv_files...

   # run SP POSTPATCH script if exists for spins
   $cmd_retval = $self->run_SP_script($POSTPATCH, "spins", $SPINS_DIR);
   if ($cmd_retval == $DONE)
   {
      return $OK;
   }
   elsif ($cmd_retval == $ERROR)
   {
      baselog($LOG_ERROR, "Failed to execute postPatch script for SP.\n", $TRUE);
      $ret_val = $FAULT;
   }

   # Reapply basic hardening items
   baselog($LOG_INFO, "Reapplying basic hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $BASIC_HARDENIG_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying basic hardening items failed.\n", $TRUE);
   }

   # If the reboot_flag got set but no patches that required a reboot went into service, don't reboot
   if ($reboot_flag == $TRUE and $self->reboot_required() == $FALSE)
   {
      $reboot_flag = $FALSE;
   }

   if (!$reboot_flag)
   {
      if (scalar(@{$common_info->{'start'}}))
      {
         if (sp_apps_start($self, \@{$common_info->{'start'}}, $FALSE) == $ERROR)
         {
            $ret_val = $FAULT if ($ret_val != $FAULT);
         }
      } ## end if (scalar(@{$common_info...
   } ## end if (!$reboot_flag)
   else  # if reboot_flag is set to $TRUE
   {

      # if reboot is required, shutdown the server in these two cases:
      # 1. if $auto_reboot, which means spins command was issued without parameters
      # 2. if $!auto_reboot, spins command was issue with parameter,
      #    but only -r flag is present (it means interact mode is set to $FALSE)
      if (
          ($auto_reboot)
          || (   (!$auto_reboot)
              && (exists $self->{'sp'})
              && (!$self->{'sp'})
              && (exists $self->{'interact'})
              && (!$self->{'interact'}))
         )
      {
         if (system("$SUDO $SHUTDOWN -r +2 \"A Server REBOOT is being performed to bring patches into service.\" &") >> 8 == $OK)
         {
            $ret_val = $OK if ($ret_val != $FAULT);
         }
         else
         {
            baselog($LOG_WARNING, "Cannot perform Server REBOOT to bring patch(es) into service.\n", $TRUE);
            $ret_val = $FAULT;
         }
      } ## end if (($auto_reboot) || ...
      else
      {
         if (   (defined $reboot_flag)
             && ($reboot_flag))
         {
            baselog($LOG_WARNING, "A Server REBOOT is required to bring patch(es) into service.\n", $TRUE);
         }
         else
         {
            if (scalar(@{$common_info->{'start'}}))
            {

               #ignore shutdown system, if -sp flag is present
               if (sp_apps_start($self, \@{$common_info->{'start'}}, $FALSE) == $ERROR)
               {
                  $ret_val = $FAULT if ($ret_val != $FAULT);
               }
            } ## end if (scalar(@{$common_info...
         } ## end else [ if ((defined $reboot_flag...
      } ## end else [ if (($auto_reboot) || ...
   } ## end else [ if (!$reboot_flag)
   return $ret_val;
} ## end sub install_bundle($)
####################################################
# UnLoad patch
# Also include the cleanup of the patch special
# directory when the prepatch script return DONE, or
# if pout patch is successful.
#
# Input: $_[0] - reference to the object
#        $_[1] - reference to the parameters' hash
#           'pid' - patch ID to unload
#           'rpm' - unload all patches for the specified RPM
#           'all' - unload all patches for specified application.
# Output: $TRUE - success
#         $FALSE - error during unloading
#####################################################
sub pout($$)
{
   my $self   = shift(@_);
   my $par    = shift(@_);
   my $retval = $TRUE;
   my $cmd_retval;  # used to return the script return value

   $self->{'action'} = $POUT;

   # setting interactive mode if -r option was specified
   if (defined $par->{'interact'})
   {
      $self->set_interact($par->{'interact'});
   }

   # setting parameter sp to FALSE. pout command is invoked from CLI or PM, not from spins
   $self->{'sp'} = $FALSE;

   if (exists($par->{'pid'}))
   {
      my $patch = $self->{'DB'}->get_patch($par->{'pid'});
      if (not defined($patch))
      {
         print 'Patch ' . $par->{'pid'} . " does not exist\n";
         return $FALSE;
      }
      $retval = $self->pout_patches([$patch]);
   } ## end if (exists($par->{'pid'...
   elsif (exists($par->{'rpm'}))
   {
      my $rpm_patches = [$self->{'DB'}->get_rpm_patches($par->{'rpm'}, $OUT_SERV)];
      unless ((defined($rpm_patches)) and (scalar(@$rpm_patches)))
      {

         # There are no patches. It's not an error so just print info and do nothing.
         baselog($LOG_INFO, "There are no patches for " . $par->{'rpm'} . "\n", $TRUE);
         return $TRUE;
      } ## end unless ((defined($rpm_patches...
      $retval = $self->pout_patches($rpm_patches);
   } ## end elsif (exists($par->{'rpm'...
   elsif (exists($par->{'app'}))
   {

      # Put all application patches in the @patches list
      my @rpms = $self->get_RPMs_for_app($par->{'app'});
      my $papp = undef;
      foreach my $r (@rpms)
      {
         if ($r =~ /^(.*?)\.(?:rpm)$/)
         {
            push(@$papp, $self->{'DB'}->get_rpm_patches_by_name($1, $OUT_SERV));
         }
      } ## end foreach my $r (@rpms)
      unless ((defined($papp)) and (scalar(@$papp)))
      {

         # There are no patches. It's not an error so just print info and do nothing.
         baselog($LOG_INFO, "There are no patches for " . $par->{'app'} . " application\n", $TRUE);
         return $TRUE;
      } ## end unless ((defined($papp)) and...
      $retval = $self->pout_patches($papp);
   } ## end elsif (exists($par->{'app'...
   elsif (exists($par->{'all'}))
   {
      $retval = $self->pout_all();
   }
   else
   {
      baselog($LOG_ERROR, "Incorrect parameters provided\n", $TRUE);
      return $FALSE;
   }
   return $retval;
} ## end sub pout($$)

####################################################
# Remove patch info from DB based on patch ID
# Input: $_[0] - reference to the "patching" object
#        $_[1] - patch ID to unload
# Output: $TRUE - success
#         $FALSE - error during unloading
#####################################################
sub pout_patch_id($$)
{
   my ($self, $pid) = @_;

   # remove patch from DB
   return $self->{'DB'}->pout_patch_id($pid);
} ## end sub pout_patch_id($$)

####################################################
# Remove patches info from DB
# Input: $_[0] - reference to the "patching" object
#        $_[1] - reference to the list of patches
# Output: $TRUE - success
#         $FALSE - error during unloading
#####################################################
sub pout_patches($$)
{
   my $self    = shift(@_);
   my @patches = @{shift(@_)};
   my $retval  = $TRUE;
   my $cmd_retval;
   my $command = "pout";
   my $exist_flag;

   # remove patches from DB
   foreach my $patch (@patches)
   {
      my $pid  = $patch->{'PID'};
      my $file = $patch->{'FILENAME'};
      $file = $PATCHDIR . "/" . $file;

      # Check patch status
      if ($patch->{'STATUS'} eq $IN_SERV)
      {
         baselog($LOG_ERROR, "Cannot remove patch $patch->{PATCHNAME}, pid($pid) because it is in service.\n", $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot remove patch $patch->{PATCHNAME}, pid($pid) because it is in service.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{PATCHNAME}\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         $retval = $FALSE;
         next;
      } ## end if ($patch->{'STATUS'}...
      elsif ($patch->{'STATUS'} == $IN_SERV_REBOOT)
      {
         baselog($LOG_ERROR, "Cannot remove patch $patch->{PATCHNAME}, pid($pid) because it will be in service after reboot.\n", $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot remove patch $patch->{PATCHNAME}, pid($pid) because it will be in service after reboot.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{PATCHNAME}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         $retval = $FALSE;
         next;
      } ## end elsif ($patch->{'STATUS'}...
      elsif ($patch->{'STATUS'} == $OUT_SERV_REBOOT)
      {
         if ((exists $self->{'sp'}) and (!$self->{'sp'}))
         {
            baselog($LOG_ERROR, "Cannot remove patch $patch->{PATCHNAME}, pid($pid) because it will not be out of service until after reboot.\n", $TRUE);
            return $FALSE;
         }
         else
         {
            baselog($LOG_DEBUG, "Removing patch $patch->{PATCHNAME}, pid($pid) with status out of service reboot.\n");
         }
      } ## end elsif ($patch->{'STATUS'}...

      my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};

      #process PREPATCH script

      if (!$self->check_script($patch, $PREPATCH, \$exist_flag))  # if PREPATCH script exists
      {
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot remove patch $pid because $PREPATCH script validation failed.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         return $FALSE;
      } ## end if (!$self->check_script...
      if ($exist_flag eq "yes")
      {

         # run patch PREPATCH script
         $cmd_retval = $self->run_script($patch, $PREPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            return $TRUE;
         }
         elsif ($cmd_retval == $ERROR)
         {
            if ($self->{'action'} eq $SPINS)
            {
               my $msg = "Cannot remove patch $pid because $PREPATCH execution returned error.";
               if (!update_file($self, $SPINS_FILE, $msg))
               {
                  baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
               }
            } ## end if ($self->{'action'} ...
            return $FALSE;
         } ## end elsif ($cmd_retval == $ERROR...
      } ## end if ($exist_flag eq "yes"...
      if (not $self->pout_patch_id($pid))
      {
         baselog($LOG_ERROR, "Unable to remove patch with pid $pid.\n", $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot remove patch $pid because it is not loaded.\n";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         $retval = $FALSE;
         next;
      } ## end if (not $self->pout_patch_id...

      #remove write protection from the patch file
      if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, '660', $file]))
      {
         baselog($LOG_ERROR, "Unable to change write permissions on $file file.\n", $TRUE);
      }

      #process POSTPATCH script
      if (!$self->check_script($patch, $POSTPATCH, \$exist_flag))  # if POSTPATCH script exists
      {
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot remove patch $pid because $POSTPATCH script validation failed.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         return $FALSE;
      } ## end if (!$self->check_script...
      if ($exist_flag eq "yes")
      {

         # run patch POSTPATCH script
         $cmd_retval = $self->run_script($patch, $POSTPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            return $TRUE;
         }
         elsif ($cmd_retval == $ERROR)
         {
            if ($self->{'action'} eq $SPINS)
            {
               my $msg = "Cannot remove patch $pid because $POSTPATCH execution returned error.";
               if (!update_file($self, $SPINS_FILE, $msg))
               {
                  baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
               }
            } ## end if ($self->{'action'} ...
            return $FALSE;
         } ## end elsif ($cmd_retval == $ERROR...
      } ## end if ($exist_flag eq "yes"...
      baselog($LOG_INFO, "Patch $pid has been removed successfully.\n", $TRUE);
      if ($self->{'action'} eq $SPINS)
      {
         if (-e $SPINS_FILE)
         {
            my $msg = "$patch->{'FILENAME'}, pid $pid has been removed successfully.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if (-e $SPINS_FILE)
      } ## end if ($self->{'action'} ...
   } ## end foreach my $patch (@patches...
   return $retval;
} ## end sub pout_patches($$)

####################################################
# Deactivates patches in right order according their dependencies
# Input: $_[0] - reference to the "patching" object
#        $_[1] - reference to the list of patches
# Output: $TRUE - success
#         $FALSE - error during unloading
#####################################################
sub poos_patches($$)
{
   my $self          = shift(@_);
   my @patches       = @{shift(@_)};
   my $retval        = $TRUE;
   my $patch_counter = 0;
   my $is_last       = $FALSE;
   my @applist       = ();
   my %seen          = ();

   # sorting patches
   @patches = sort {($b->{'PINS'}) <=> ($a->{'PINS'})} @patches;

   if (not defined $self->{'reboot'})
   {
      $self->{'reboot'} = ();
   }

   # poos patches
   foreach my $p (@patches)
   {

      # if we're looking at the last element in the array, set $is_last to TRUE
      if (++$patch_counter == scalar(@patches))
      {
         $is_last = $TRUE;
      }

      if ($p->{'STATUS'} ne $IN_SERV)
      {

         # not an error. just keep going.
         baselog($LOG_INFO, "Patch $p->{'PID'} is already out of service.\n", $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "$p->{'FILENAME'}, pid $p->{'PID'} is already out of service.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $p->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         next;
      } ## end if ($p->{'STATUS'} ne ...

      push(@{$self->{'reboot'}}, $p) if ($p->{'REBOOT'} =~ m/^yes$/i);
      if (scalar(@{$p->{'APP_START'}}))
      {
         foreach my $appstart (@{$p->{'APP_START'}})
         {
            push(@applist, $appstart) unless ($seen{$appstart}++);
         }
      } ## end if (scalar(@{$p->{'APP_START'...
      if (not $self->poos_patch($p, $is_last, \@applist))
      {
         baselog($LOG_ERROR, "Cannot take out of service patch $p->{'PID'}.\n", $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            my $msg = "Cannot put out of service patch $p->{'FILENAME'}, pid $p->{'PID'}.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $p->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
         $retval = $FALSE;
      } ## end if (not $self->poos_patch...
      if ($self->{'action'} eq $SPINS)
      {
         my $msg = "$p->{'FILENAME'}, pid $p->{'PID'} has been taken out of service successfully.";
         if (-e $SPINS_FILE)
         {
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $p->{'FILENAME'}.\n", $TRUE);
            }
         } ## end if (-e $SPINS_FILE)
      } ## end if ($self->{'action'} ...
   } ## end foreach my $p (@patches)
   return $retval;
} ## end sub poos_patches($$)

####################################################
# UnLoad patches created for
# the specific application only
# Input: $_[0] - reference to the "patching" object
#        $_[1] - application name. A patch will
#           be unloaded only if it belongs to
#           this application
# Output: $TRUE - success
#         $FALSE - any errors during unloading
#####################################################
sub pout_all($)
{
   my $self = shift(@_);

   # get all patches
   my @patches = $self->{'DB'}->get_rpm_patches(undef, $OUT_SERV, undef);
   if (not scalar(@patches))
   {
      baselog($LOG_INFO, "There are no patches in out of service state.", $TRUE);
      return $TRUE;
   }

   # Remove patches
   return $self->pout_patches(\@patches);
} ## end sub pout_all($)

####################################################
# Put patch(es) in service
# Input: $_[0] - reference to the object
#        $_[1] - reference to the parameters' hash
#          'all' - load all patcher
#          'interact' == $FALSE - non-interactive mode
#          'pid' - patch ID to put in service
#          'rpm' - rpm to put in service all relaited patches
#          'appstart' == $FALSE - not start apps
#          'appstop' == $FALSE - not stop apps
# Output: $TRUE - success
#         $FALSE - error during activating
#####################################################
sub pins($$)
{
   my ($self, $par) = @_;
   my @applist = ();
   my %seen    = ();

   # setting the action to 'pins'
   $self->{'action'} = $PINS;
   my @patches = ();  # list of patches to install
   if ((defined $par->{'appstart'}) || (defined $par->{'appstop'}))
   {
      $self->set_app_flags($par->{'appstart'}, $par->{'appstop'});
   }
   if (defined $par->{'interact'})
   {
      $self->set_interact($par->{'interact'});
   }

   $self->{'sp'} = $par->{'sp'};

   # 'pid' parameter exists so, only one patch or SU needs to be inserted
   if (exists($par->{'pid'}))
   {

      # get patch object
      my $p = $self->{'DB'}->get_patch($par->{'pid'});
      if (!defined $p)
      {
         baselog($LOG_ERROR, 'Patch ' . $par->{'pid'} . " does not exist.", $TRUE);
         return $FALSE;
      }
      push @patches, $p;
   } ## end if (exists($par->{'pid'...
     # Parameter 'all' has been specified. So need to activate all patches and SUs from
     # patch database which status is out of service.
   elsif (exists($par->{'all'}))
   {

      # checking SU conflicts. There should not be two or more SU in out of service for the same rpm.
      my @conf_su = $self->{'DB'}->check_SU_conflict();
      if (scalar(@conf_su))
      {

         # getting rpm names
         my @rpm_names;
         foreach my $rpm_file (@conf_su)
         {
            push @rpm_names, $rpm_file->{'RPM_NAME'};
         }

         # such SU found ... then exit with error.
         baselog($LOG_ERROR, "There are several SUs for the following rpms: " . join(' ', @rpm_names), $TRUE);
         return $FALSE;
      } ## end if (scalar(@conf_su))

      # retrieving the list of what needs to be activated (SUs go first)
      my @p = $self->{'DB'}->get_rpm_patches(undef, $OUT_SERV, undef);
      if (not scalar(@p))
      {
         baselog($LOG_INFO, "There are no out of service patches. Nothing to activate.", $TRUE);
         return $TRUE;
      }
      push @patches, @p;

   } ## end elsif (exists($par->{'all'...
     # unknown parameter was entered
   else
   {
      baselog($LOG_ERROR, "Incorrect parameters provided");
      return $FALSE;
   }

   my $retval = $TRUE;

   # loop across the list of patches
   #foreach my $patch (@patches)
   my $is_last = $FALSE;
   $self->{'reboot'} = ();

   if (not get_apps_stop_conf($self, \@patches))
   {
      return $FALSE;
   }

   # initialize flags for apps that require stop
   init_keys_for_app_stop($self, \@patches);

   for (my $i = 0 ; $i < scalar(@patches) ; $i++)
   {

      $is_last = ($i == scalar(@patches) - 1);

      push(@{$self->{'reboot'}}, $patches[$i]) if ($patches[$i]->{'REBOOT'} =~ m/^yes$/i);

      # create list of applications to start, no duplicates allowed
      if (scalar(@{$patches[$i]->{'APP_START'}}))
      {
         foreach my $appstart (@{$patches[$i]->{'APP_START'}})
         {
            push(@applist, $appstart) unless ($seen{$appstart}++);
         }
      } ## end if (scalar(@{$patches[...

      # put patch in service
      if (not $self->pins_patch($patches[$i], $is_last, \@applist))
      {
         baselog($LOG_ERROR, "Failed to put in service patch with pid $patches[$i]->{'PID'}\n", $TRUE);
         $retval = $FALSE;
         last;
      } ## end if (not $self->pins_patch...
      baselog($LOG_INFO, 'The pins_patch return OK for patch ' . $patches[$i]->{'PID'});
   } ## end for (my $i = 0 ; $i < scalar...

   if ($retval == $FALSE)
   {
      if (defined $self->{'reboot'} and ($self->reboot_required() == $TRUE))
      {
         baselog($LOG_INFO, "There are patches in service that require a reboot to complete installation\n", $TRUE);
      }
      elsif (scalar(@applist) != 0)
      {

         # if any app was stopped
         if (scalar(keys(%{$self->{'app_stop'}})))
         {

            #find which apps really need to be started.
            my @start_apps = ();
            foreach my $ap_key (keys(%{$self->{'app_stop'}}))
            {
               if ($ap_key ne 'all_nortel')
               {
                  if ($self->{'app_stop'}->{$ap_key})
                  {
                     push @start_apps, $ap_key;
                  }
               } ## end if ($ap_key ne 'all_nortel'...
            } ## end foreach my $ap_key (keys(%{...

            #   my $appstart = join(' ',@applist);
            my $apps_to_start = $self->shrink_apps(\@start_apps);
            if (scalar(@{$apps_to_start->{'base'}}))
            {
               baselog($LOG_WARNING, "All Nortel and " . join(' ', @{$apps_to_start->{'base'}}) . " applications need to be restarted.\n", $TRUE);
            }
            elsif (scalar(@{$apps_to_start->{'nortel'}}))
            {

               baselog($LOG_WARNING, "The following applications need to be restarted: " . join(' ', @{$apps_to_start->{'nortel'}}) . ".\n", $TRUE);
            }
         } ## end if (scalar(keys(%{$self...
      } ## end elsif (scalar(@applist) !=...
      baselog($LOG_ERROR, "Patch activation failed.", $TRUE);
   } ## end if ($retval == $FALSE)

   # Reapply basic hardening items
   baselog($LOG_INFO, "Reapplying basic hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $BASIC_HARDENIG_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying basic hardening items failed.");
   }

   # Reapply enhanced hardening items
   baselog($LOG_INFO, "Reapplying enhanced hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $ENHANCED_HARDENING_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying enhanced hardening items failed.");
   }

   $self->{'app_stop'} = undef;
   return $retval;
} ## end sub pins($$)

####################################################
# Put patch in service
# Input: $_[0] - reference to the object
#        $_[1] - reference to the patch object
#	 $_[2] - TRUE/FALSE indicates if this is a last patch in the bundle
#        $_[3] - list of applications to start if this is the last patch
# Output: $TRUE - success
#         $FALSE - error during activating
#####################################################
sub pins_patch($$$$)
{
   my ($self, $patch, $is_last, $reflist) = @_;
   my $cmd_retval;  #used to return script return value
   my $command = "pins";
   my $exist_flag;
   my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};
   my @applist  = @$reflist;
   my $msg      = " ";

   print 'Patch handle: ' . $patch->{'PID'} . "\t\n";

   # Check patch status
   if ($patch->{'STATUS'} == $IN_SERV)
   {
      print 'The patch ' . $patch->{'PID'} . " is already in service.\n";
      return $TRUE;
   }
   elsif ($patch->{'STATUS'} == $IN_SERV_REBOOT)
   {
      print 'The patch ' . $patch->{'PID'} . " will be in service after restart.\n";
      if ($self->{'action'} eq $SPINS)
      {
         $msg = "$patch->{'PATCHNAME'} with patch handle $patch->{'PID'} will be in service after restart.";
         if (!update_file($self, $SPINS_FILE, $msg))
         {
            baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'PATCHNAME'}.\n", $TRUE);
         }
      } ## end if ($self->{'action'} ...
      return $FALSE;
   } ## end elsif ($patch->{'STATUS'}...

   # Extract patch members
   if (not $patch->extract_patch_members())
   {
      $msg = "Cannot extract patch members for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
      baselog($LOG_ERROR, "$msg\n", $TRUE);
      if ($self->{'action'} eq $SPINS)
      {
         if (!update_file($self, $SPINS_FILE, $msg))
         {
            baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'PATCHNAME'}.\n", $TRUE);
         }
      } ## end if ($self->{'action'} ...
      return $FALSE;
   } ## end if (not $patch->extract_patch_members...
   my $retval = $TRUE;
 PINS:
   {

      #check if there is a patch which obsoletes given patch
      unless ($self->check_Obsoleted($patch))
      {
         $msg = "Pins obsoleted check failed for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($self->check_Obsoleted...

      #check dependent patch
      unless ($self->check_Dependency($patch))
      {
         $msg = "Pins dependency check failed for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($self->check_Dependency...

      #check if given patch obsoletes other patches
      unless ($self->check_Obsoletes($patch))
      {
         $msg = "Pins obsolescence check failed for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($self->check_Obsoletes...

      unless ($self->validate_Mutex($patch))
      {
         $msg = "Pins mutual exclusive check failed for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($self->validate_Mutex...
      unless ($self->prepare_pins($patch))
      {
         $msg = "Pins preparation failed for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($self->prepare_pins...

      #process PREPATCH script

      if (!$self->check_script($patch, $PREPATCH, \$exist_flag))  # if PREPATCH script exists
      {
         if ($self->{'action'} eq $SPINS)
         {
            $msg = "$PREPATCH script validation fails for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         }
         $retval = $FALSE;
         last PINS;
      } ## end if (!$self->check_script...
      if ($exist_flag eq "yes")
      {

         # run patch PREPATCH script
         $cmd_retval = $self->run_script($patch, $PREPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            $retval = $TRUE;
            last PINS;
         }
         elsif ($cmd_retval == $ERROR)
         {
            if ($self->{'action'} eq $SPINS)
            {
               $msg = "$PREPATCH script execution returned error for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
            }
            $retval = $FALSE;
            last PINS;
         } ## end elsif ($cmd_retval == $ERROR...
      } ## end if ($exist_flag eq "yes"...
      unless ($patch->install($self->{'interact'}))
      {
         $msg = "Failed to put patch $patch->{'FILENAME'} in service with patch handle $patch->{'PID'}.";
         baselog($LOG_ERROR, "$msg\n", $TRUE);
         $retval = $FALSE;
         last PINS;
      } ## end unless ($patch->install($self...

      #process POSTPATCH script
      if (!$self->check_script($patch, $POSTPATCH, \$exist_flag))  # if POSTPATCH script exists
      {
         if ($self->{'action'} eq $SPINS)
         {
            $msg = "$POSTPATCH script validation fails for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
         }
         $retval = $FALSE;
         last PINS;
      } ## end if (!$self->check_script...
      if ($exist_flag eq "yes")
      {

         # run patch POSTPATCH script
         $cmd_retval = $self->run_script($patch, $POSTPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            $retval = $TRUE;
            last PINS;
         }
         elsif ($cmd_retval == $ERROR)
         {
            if ($self->{'action'} eq $SPINS)
            {
               $msg = "$POSTPATCH script execution returned error for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.";
            }
            $retval = $FALSE;
            last PINS;
         } ## end elsif ($cmd_retval == $ERROR...
      } ## end if ($exist_flag eq "yes"...

      if (not $self->post_pins($patch, $is_last, \@applist))
      {
         $msg = "Post pins operation failed for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.\n";
         baselog($LOG_ERROR, $msg, $TRUE);
         if ($self->{'action'} eq $SPINS)
         {
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'PATCHNAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
      } ## end if (not $self->post_pins...

   } ## end PINS:

   # Clean up
   $patch->remove_patch_members();
   if ($retval)
   {
      baselog($LOG_INFO, 'Patch ' . $patch->{'PID'} . ' was put in service successfully.');

      # send an snmp trap to alert the admin that the patch is now in service
      my $fullname = patchutil::patch::get_full_patch_file($patch);
      my $msg_parms = {
                       "patchID"  => "$patch->{PID}",
                       "MPLR"     => "$patch->{PATCHNAME}",
                       "CR"       => "$patch->{CR}",
                       "filename" => "$fullname"
                      };
      send_snmp_trap("PCH0110", $msg_parms);
   } ## end if ($retval)
   else
   {

      #update $SPINS_FILE if $retval is $FALSE
      if ($self->{'action'} eq $SPINS)
      {
         if (!update_file($self, $SPINS_FILE, $msg))
         {
            baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'PATCHNAME'}.\n", $TRUE);
         }
      } ## end if ($self->{'action'} ...
   } ## end else [ if ($retval)
   return $retval;
} ## end sub pins_patch($$$$)

############################################################
#Description: check whether dependent patches are in service
#INPUT: $_[0] - refernce to the object
#        $_[1] - reference to the patch object
#OUTPUT: $TRUE  - dependent patch is in service
#        $FALSE - dependent patch is not in service
############################################################
sub check_Dependency($$)
{
   my ($self, $p) = @_;

   my $retval = $TRUE;

   # patch/SU doesn't have dependent patch/SU. returning $TRUE
   if ($p->{'DEPENDENT'} eq '')
   {
      baselog($LOG_DEBUG, "Patch $p->{PATCHNAME} doesn't have dependent patch.");
      return $TRUE;
   }
   my $dep_pid = $self->{'DB'}->get_patch_id_by_name($p->{'DEPENDENT'});

   # can't retrieve PID of dependent patch from database. This means dependent patch/SU is not loaded at all.
   # output an error and return $FALSE.
   if (not defined $dep_pid)
   {
      baselog($LOG_ERROR, "Dependent patch $p->{DEPENDENT} is not loaded in database. Please load patch in database and put it in service before installing $p->{PATCHNAME}.", $TRUE);
      $retval = $FALSE;
   }

   # dependent patch/SU found in patch database, but currently is not in service state. This is error condition, need to put dependent
   # patch/SU in service first.
   elsif (not $self->{'DB'}->is_patch_in_service_byID($dep_pid))
   {
      if ((exists $self->{'sp'}) and ($self->{'sp'}))
      {

         # find patch object
         my $dep_p = patchutil::patch::new($self->{'DB'}->get_patch_params($dep_pid));
         if (!defined($dep_p))
         {
            baselog($LOG_ERROR, "Error getting patch object for patch with pid $dep_p.\n", $TRUE);
            $retval = $FALSE;
         }
         if ($dep_p->{'STATUS'} eq $IN_SERV_REBOOT)
         {
            baselog($LOG_DEBUG, "Dependent patch $p->{DEPENDENT} for $p->{'PATCHNAME'} has status in service reboot.\n");
         }
         else
         {
            baselog($LOG_ERROR, "Dependent patch $p->{DEPENDENT} is not in service. Before installing $p->{'PATCHNAME'} you need to PINS $dep_pid.\n", $TRUE);
            $retval = $FALSE;
         }
      } ## end if ((exists $self->{'sp'...
      else
      {
         baselog($LOG_ERROR, "Dependent patch $p->{DEPENDENT} is not in service.  Before installing $p->{'PATCHNAME'} you need to PINS $dep_pid.\n", $TRUE);
         $retval = $FALSE;
      }
   } ## end elsif (not $self->{'DB'}->is_patch_in_service_byID...

   # patch/SU has dependency and dependent patch/SU found in database and is in service.
   else
   {
      baselog($LOG_DEBUG, "Patch $p->{PATCHNAME} has dependent patch ($p->{DEPENDENT}) and this dependent patch is in service.");
   }
   return $retval;
} ## end sub check_Dependency($$)

#####################################################
# Check if patch is obsoleted by another patch which is in service
# Input: $_[0] - reference to the object
#        $_[1] - reference to the patch object
# Output: $TRUE  - current patch is not obsoleted by installed patch
#         $FALSE - there is a patch in service that obsoletes the given patch
#####################################################
sub check_Obsoleted($$)
{
   my ($self, $p) = @_;
   my @obs_patch = $self->{'DB'}->check_patch_obsolete($p->{'PATCHNAME'});
   if (scalar(@obs_patch))
   {
      if (scalar(@obs_patch) == 1)
      {
         baselog($LOG_ERROR, "Patch " . $obs_patch[0] . " obsoletes current patch $p->{PATCHNAME} and is in service.", $TRUE);
      }
      else
      {
         baselog($LOG_ERROR, "Patches " . join(' ', @obs_patch) . " obsolete current patch $p->{PATCHNAME} and are in service.", $TRUE);
      }
      return $FALSE;
   } ## end if (scalar(@obs_patch)...
   return $TRUE;
} ## end sub check_Obsoleted($$)

#########################################################################
# Check obsoletes patch(es) out of service
# Input: $_[0] - reference to the object
#        $_[1] - reference to the patch object
# Output: $TRUE  - there are no patches in service obsoleted by given patch
#         $FALSE - given patch obsoletes a patch which is in service
#########################################################################
sub check_Obsoletes($$)
{
   my ($self, $p) = @_;
   my @obs_list = @{$p->{'OBSOLETES'}};

   # patch provided has empty obsoletes list, so returning $TRUE.
   if (not scalar(@obs_list))
   {
      baselog($LOG_DEBUG, "Patch $p->{PATCHNAME} doesn't obsolete another patches.");
      return $TRUE;
   }
   my $retval = "";

   # loop for each patch in obsoletes list
   foreach my $patchname (@obs_list)
   {

      # form a string to output on the screen/in logs if there are obsolete patches in service.
      my $pid = $self->{'DB'}->get_patch_id_by_name($patchname);
      if (defined $pid)
      {
         $retval .= "\n\t$patchname (patch ID $pid)" if ($self->{'DB'}->is_patch_in_service_byID($pid));
      }
   } ## end foreach my $patchname (@obs_list...
     # check if string is not empty
   if ($retval ne "")
   {
      baselog($LOG_ERROR, "Patch $p->{PATCHNAME} obsoletes the following patches: " . $retval . "Please put them out of service first.", $TRUE);
      return $FALSE;
   }

   # otherwise all obsolete patches are not in service
   baselog($LOG_DEBUG, "Patch $p->{PATCHNAME} doesn't have obsolete patches in service.");
   return $TRUE;
} ## end sub check_Obsoletes($$)

########################################################################
## @method validate_Mutex
# @brief - if patch is a PI control patch, validate MUTEX.
# @details - check for PI control patch:
#            1. if patch is in mutex list of other in-srv patches
#            2. if there is any patch in srv which is mutually
#               exclusive with current patch
# @param $self   - reference to object
# @param $p      - reference to patch object
# @return $TRUE  - Mutex validation successful.
# @return $FALSE - Mutex validation fails.
########################################################################
sub validate_Mutex($$)
{
   my ($self, $p) = @_;

   # only for PI control patches do mutex checking
   if ($p->is_PIcontrol_patch())
   {

      # check if patch is in mutex list of other in-srv patches
      if (!$self->check_insrv_patches_mutex($p))
      {
         return $FALSE;
      }

      # check if there is any patch in srv which is mutually exclusive with current patch
      if (!$self->check_Mutex($p))
      {
         return $FALSE;
      }
   } ## end if ($p->is_PIcontrol_patch...
   return $TRUE;
} ## end sub validate_Mutex($$)

########################################################################
## @method check_insrv_patches_mutex
# @brief - check if patch is in mutex list of other in-srv patches
# @details - check if patch is in mutex list of other in-srv patches
# @param $self   - reference to object
# @param $p      - reference to patch object
# @return $TRUE  - current patch is not in the Mutex list
#                  of another installed patch
# @return $FALSE - current patch is in the Mutex list
#                  of another installed patch
########################################################################
sub check_insrv_patches_mutex($$)
{
   my ($self, $p) = @_;

   my @mutex_patch = $self->{'DB'}->check_patch_mutex($p->{'PATCHNAME'});
   if (scalar(@mutex_patch))
   {
      if (scalar(@mutex_patch) == 1)
      {
         baselog($LOG_ERROR, "Patch " . $mutex_patch[0] . " is mutually exclusive with current patch $p->{'PATCHNAME'} and is in service.\n", $TRUE);
      }
      else
      {
         baselog($LOG_ERROR, "Patches " . join(' ', @mutex_patch) . " are in service and are mutually exclusive with current patch $p->{'PATCHNAME'}.\n", $TRUE);
      }
      return $FALSE;
   } ## end if (scalar(@mutex_patch...
   return $TRUE;
} ## end sub check_insrv_patches_mutex($$)
########################################################################
## @method check_Mutex
# @brief - Check if there is any patch in service that is mutually exclusive
# @        with current patch.
# @details - Check if there is any patch in service that is mutually exclusive
# @          with current patch.
# @param $self   - reference to object
# @param $p      - reference to patch object
# @return $TRUE  - there are no patches in service
#                  that are in the mutex list of given patch.
# @return $FALSE - there are patches in service
#                  that are in the mutex list of given patch.
########################################################################
sub check_Mutex($$)
{
   my ($self, $p) = @_;
   my @mutex_list = @{$p->{'MUTEX'}};

   my $check_all_patches = $TRUE;

   # patch provided has empty mutex list, so returning $TRUE.
   if (not scalar(@mutex_list))
   {
      baselog($LOG_DEBUG, "Patch $p->{'PATCHNAME'} doesn't have mutually exclusive patches.");
      return $TRUE;
   }
   my $retval = "";

   my @patches_in_db = $self->{'DB'}->get_rpm_patches(undef, $IN_SERV, 'PATCH');
   if (scalar(@patches_in_db))
   {

      # loop for each patch in mutex list
      foreach my $patchname (@mutex_list)
      {

         # form a string to output on the screen/in logs if there are mutex patches in service.

         my $p_name          = undef;
         my $p_name_with_iss = undef;

         # get patch name and patch name with issue from $patchname
         if ($patchname =~ /^(.*?)\_([0-9]{1,2})\.($PATCHEXT)$/)
         {
            $p_name          = $1;
            $p_name_with_iss = "$1_$2";
         }
         if ((!defined($p_name)) || (!defined($p_name_with_iss)))
         {
            baselog($LOG_ERROR, "Parsing of $patchname filename in MUTEX list of $p->{'FILENAME'} failed.\n", $TRUE);
            $check_all_patches = $FALSE;
            last;
         } ## end if ((!defined($p_name)...
         foreach my $patch (@patches_in_db)
         {
            if ($patch->{'PATCHNAME'} =~ /$p_name/)
            {
               baselog($LOG_DEBUG, "Found in db patch $patch->{'PATCHNAME'} with pid $patch->{'PID'} that matches $p_name.\n");

               unless ($patch->{'PATCHNAME'} =~ /$p_name_with_iss/)
               {
                  baselog(
                     $LOG_WARNING,
                     "Patch $p->{'PATCHNAME'} is mutually exclusive with patch $patchname. A different patch issue $patch->{'PATCHNAME'} is in service. Please take appropriate action to update the patches.\n",
                     $TRUE
                  );
               } ## end unless ($patch->{'PATCHNAME'...
               $retval .= "\n\t$patch->{'PATCHNAME'} (patch ID $patch->{'PID'})";
            } ## end if ($patch->{'PATCHNAME'...
         } ## end foreach my $patch (@patches_in_db...
      } ## end foreach my $patchname (@mutex_list...
   } ## end if (scalar(@patches_in_db...
   if (!$check_all_patches)
   {
      return $FALSE;
   }

   # check if string is not empty
   if ($retval ne "")
   {
      baselog($LOG_ERROR, "Patch $p->{'PATCHNAME'} is mutually exclusive with the following patches: " . $retval . "\nPlease put them out of service first.", $TRUE);
      return $FALSE;
   }
   baselog($LOG_DEBUG, "Patch $p->{'PATCHNAME'} doesn't have mutually exclusive patches in service.");
   return $TRUE;
} ## end sub check_Mutex($$)

########################################################################
## @method validate_min_rpm_ver
# @brief - if patch is a PI control patch, validate MIN_RPM_VER tag..
# @details - check for PI control patch:
#            For each rpm listed in the MIN_RPM_VER list if the rpm name
#            is installed. If it is, check if the installed version is
#            greater or equal to the listed rpm version.
# @param $self   - reference to object
# @param $p      - reference to patch object
# @return $TRUE  - Minimum rpm version validation successful.
# @return $FALSE - Minimum rpm version validation fails.
########################################################################
sub validate_min_rpm_ver($$)
{
   my ($self, $p) = @_;
   my $retval = $TRUE;

   # only for PI control patches do mutex checking
   if ($p->is_PIcontrol_patch())
   {
      my @min_rpm_ver_list = @{$p->{'MIN_RPM_VER'}};
      baselog($LOG_INFO, "Starting minimum rpm version validation for patch $p->{'PATCHNAME'}.\n");
      foreach my $rpm (@min_rpm_ver_list)
      {
         my @fields_RPM = ();
         if (!extract_RPM_name($self, $rpm, \@fields_RPM))
         {
            baselog($LOG_ERROR, "Cannot obtain RPM name from rpm $rpm.\n", $TRUE);
            $retval = $FALSE;
            next;
         } ## end if (!extract_RPM_name(...
         elsif (defined($fields_RPM[0]))  # if rpm name extracted
         {
            baselog($LOG_DEBUG, "RPM name extracted for $rpm is $fields_RPM[0].\n");
            my $rpm_installed = undef;
            my $rpm_retval;
            do_system([$RPM, "-q", $fields_RPM[0], '--nosignature', '--qf', '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\n'], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$rpm_retval});
            if (not $rpm_retval)
            {
               if (!defined(@$rpm_installed))
               {
                  baselog($LOG_ERROR, "Cannot check rpm version installed for rpm $fields_RPM[0].\n", $TRUE);
                  $retval = $FALSE;
                  next;
               } ## end if (!defined(@$rpm_installed...
               else
               {
                  $rpm_installed = @$rpm_installed[0];
                  chomp($rpm_installed);

                  # if rpm installed is greater or equal to rpm listed in the mutex patch list
                  # minimum rpm version validation passed for this rpm.
                  if ($rpm_installed ge $rpm)
                  {
                     baselog($LOG_INFO, "Minimum rpm version validation succeeds for rpm $rpm. The installed rpm version is $rpm_installed.\n");
                     next;
                  }
                  else
                  {
                     baselog($LOG_ERROR, "Minimum rpm version required for $p->{'PATCHNAME'} is $rpm. The installed rpm version is $rpm_installed.\n", $TRUE);
                     $retval = $FALSE;
                     next;
                  } ## end else [ if ($rpm_installed ge ...
               } ## end else [ if (!defined(@$rpm_installed...
            } ## end if (not $rpm_retval)
            else
            {
               baselog($LOG_ERROR, "RPM $fields_RPM[0] is not installed in the system.\n", $TRUE);
               $retval = $FALSE;
               next;
            } ## end else [ if (not $rpm_retval)
         } ## end elsif (defined($fields_RPM...
      } ## end foreach my $rpm (@min_rpm_ver_list...
   } ## end if ($p->is_PIcontrol_patch...
   return $retval;
} ## end sub validate_min_rpm_ver($$)

#####################################################
# Returns array with RPMs for specified application
# Input: $_[0] - reference to the object
#        $_[1] - application name
# Output: @rpms  - array with rpms
#         empty array - if there are no rpms
#####################################################
sub get_RPMs_for_app($$)
{
   my ($self, $app) = @_;
   my @rpms = ();
   my $rpm_retval;
   my $system_rpm;

   foreach my $r (keys(%{$self->{'hash_rpms'}}))
   {
      if ($self->{'hash_rpms'}->{$r} eq $app)
      {
         do_system([$RPM, "-q", $r, '--nosignature', '--qf', '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}.rpm\n'], {$dsNOERROR => $TRUE, $dsOUT => \@$system_rpm, $dsRETVAL => \$rpm_retval});
         if (not $rpm_retval and defined $system_rpm)
         {
            push(@rpms, @$system_rpm);
         }
      } ## end if ($self->{'hash_rpms'...
   } ## end foreach my $r (keys(%{$self...
   return @rpms;
} ## end sub get_RPMs_for_app($$)

#####################################################
## @method get_app_from_rpm($$)
# @brief Supply a rpm and return the application associated with it
# @details
# @param $_[0] - reference to patching object
# @param $_[1] - rpm used for reference
# @return application name if success, otherwise undef
# @author Dave Cairns
# @date November 24, 2008
#####################################################
sub get_app_from_rpm($$)
{
   my ($self, $rpm) = @_;
   my $app = undef;
   my @fields_rpm;

   if (!extract_RPM_name($self, $rpm, \@fields_rpm))
   {
      baselog($LOG_ERROR, "Cannot obtain rpm name from $rpm\n");
   }
   elsif (defined($fields_rpm[0]))  # rpm name extracted
   {
      $app = $self->{'hash_rpms'}->{$fields_rpm[0]};
   }
   return $app;
} ## end sub get_app_from_rpm($$)

#####################################################
# Checks if there are any patches which depends on us in_service state
# Input: $_[0] - reference to the object
#        $_[1] - reference to the patch object
# Output: $TRUE  - success
#         $FALSE - error during deactivating
#####################################################
sub poos_Dependency($$)
{
   my ($self, $p) = @_;
   my @deps = $self->{'DB'}->check_patches_dependent_on($p->{'PATCHNAME'});
   if (scalar(@deps))
   {
      baselog($LOG_ERROR, "If you want to put patch $p->{'PATCHNAME'} out of service please put out of service the following patch(es)" . join(' ', @deps) . " first.", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub poos_Dependency($$)

####################################################
# Put patch(es) out of service
# Input: $_[0] - reference to the object
#        $_[1] - reference to the parameters' hash
#          'all' - put all patches out of service
#          'interact' == $FALSE - non-interactive mode
#          'pid' - patch ID to put out of service
#          'rpm' - put all patches for RPM in out of service
#          'appstart' == $FALSE - not start apps
#          'appstop' == $FALSE - not stop apps
# Output: $TRUE  - success
#         $FALSE - error during deactivating
#####################################################
sub poos($$)
{
   my ($self, $par) = @_;
   my @patches = ();  # list of patches to un-install
   my @applist = ();
   my %seen    = ();

   $self->{'action'} = $POOS;
   if (defined $par->{'context'})
   {
      $self->{'context'} = $par->{'context'};
   }
   if ((defined $par->{'appstart'}) || (defined $par->{'appstop'}))
   {
      $self->set_app_flags($par->{'appstart'}, $par->{'appstop'});
   }
   $self->set_interact($par->{'interact'}) if (exists($par->{'interact'}));

   $self->{'sp'} = $par->{'sp'};

   if (exists($par->{'pid'}))
   {

      # Get patch object
      my $p = $self->{'DB'}->get_patch($par->{'pid'});
      unless (defined $p)
      {
         baselog($LOG_ERROR, "Patch $par->{pid} does not exist.", $TRUE);
         return $FALSE;
      }
      push @patches, $p;
   } ## end if (exists($par->{'pid'...
   elsif (exists($par->{'app'}))
   {

      # Put all application patches in the @patches list
      my @rpms = $self->get_RPMs_for_app($par->{'app'});
      my @p    = ();
      foreach my $r (@rpms)
      {

         # RPM_NAME is without 'rpm' extension
         if ($r =~ /^(.*?)\.(?:rpm)$/)
         {
            push(@p, $self->{'DB'}->get_rpm_patches($1, $ALL_IN_SERVICE));
         }
      } ## end foreach my $r (@rpms)

      # performing checking if there are patches/SUs which state is IN_SERV_REBOOT
      # if such patches found then reporting an error, so user could complete previous
      # installation
      my @error;
      foreach my $t_patch (@p)
      {
         if ($t_patch->{'STATUS'} == $IN_SERV_REBOOT)
         {
            push @error, $t_patch->{'PATCHNAME'};
         }
      } ## end foreach my $t_patch (@p)
      if (scalar(@error))
      {
         baselog($LOG_ERROR, "There are existing patches that will be in service only after a reboot: " . join(' ', @error) . ". Please reboot to complete previous installation.", $TRUE);
         return $FALSE;
      }

      # Sort patches by pins time
      @p = sort {($b->{'PINS'}) <=> ($a->{'PINS'})} @p;
      push @patches, @p;

   } ## end elsif (exists($par->{'app'...
   elsif (exists($par->{'all'}))
   {

      # Put all RPM patches in the @patches list
      my @p = $self->{'DB'}->get_rpm_patches(undef, $ALL_IN_SERVICE);

      # performing checking if there are patches/SUs which state is IN_SERV_REBOOT
      # if such patches found then reporting an error, so user could complete previous
      # installation
      my @error;
      foreach my $t_patch (@p)
      {
         if ($t_patch->{'STATUS'} == $IN_SERV_REBOOT)
         {
            push @error, $t_patch->{'PATCHNAME'};
         }
      } ## end foreach my $t_patch (@p)
      if (scalar(@error))
      {
         baselog($LOG_ERROR, "There are existing patches that will be in service only after a reboot: " . join(' ', @error) . ". Please reboot to complete previous installation.", $TRUE);
         return $FALSE;
      }

      # Sort patches by pins time
      @p = sort {($b->{'PINS'}) <=> ($a->{'PINS'})} @p;
      push @patches, @p;
   } ## end elsif (exists($par->{'all'...
   else
   {
      baselog($LOG_ERROR, "Incorrect parameters provided", $TRUE);
      return $FALSE;
   }

   if (not scalar(@patches))
   {
      baselog($LOG_INFO, 'There are no in-service patches that match that criteria', $TRUE);
      return $TRUE;
   }

   my $retval  = $TRUE;
   my $is_last = $FALSE;

   $self->{'reboot'} = ();

   if (not get_apps_stop_conf($self, \@patches))
   {
      return $FALSE;
   }

   # initialize flags for apps that require stop
   init_keys_for_app_stop($self, \@patches);

   for (my $i = 0 ; $i < scalar(@patches) ; $i++)
   {
      $is_last = ($i == scalar(@patches) - 1);

      push(@{$self->{'reboot'}}, $patches[$i]) if ($patches[$i]->{'REBOOT'} =~ m/^yes$/i);

      if (scalar(@{$patches[$i]->{'APP_START'}}))
      {
         foreach my $appstart (@{$patches[$i]->{'APP_START'}})
         {
            push(@applist, $appstart) unless ($seen{$appstart}++);
         }
      } ## end if (scalar(@{$patches[...

      # Put patch out of service
      if (not $self->poos_patch($patches[$i], $is_last, \@applist))
      {
         baselog($LOG_ERROR, "Failed to take out of service patch with pid $patches[$i]->{'PID'}.", $TRUE);
         $retval = $FALSE;
         next;
      } ## end if (not $self->poos_patch...
      baselog($LOG_INFO, "Patch with pid $patches[$i]->{'PID'} was put out of service successfully.\n");
   } ## end for (my $i = 0 ; $i < scalar...

   # Reapply basic hardening items (if we are called during undeploy then hardening can fail)
   baselog($LOG_INFO, "Reapplying basic hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $BASIC_HARDENIG_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying basic hardening items failed.");
   }

   # Reapply enhanced hardening items
   baselog($LOG_INFO, "Reapplying enhanced hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $ENHANCED_HARDENING_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying enhanced hardening items failed.");
   }

   $self->{'app_stop'} = undef;
   return $retval;
} ## end sub poos($$)

####################################################
# Put patch out of service
# Input:  $_[0] - reference to the object
#         $_[1] - reference to the patch object
#         $_[2] - TRUE/FALSE indicates if this is a last patch
# Output: $TRUE  - success
#         $FALSE - error during deactivating
#####################################################
sub poos_patch($$$$)
{
   my ($self, $patch, $is_last, $reflist) = @_;
   my @cmd_script;
   my $cmd_retval;  # used to return script return value
   my $command = "poos";
   my $exist_flag;
   my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};
   my @applist  = @$reflist;

   baselog($LOG_INFO, 'Patch handle: ' . $patch->{'PID'} . "\t\n", $TRUE);

   #workaround for LB SU poos (Q20238325)
   # - this assumes that the linuxbase RPM reuninstall script has created a basic hardening module
   #  00LB_recover_perms.pm to restore folder/file ownership&permissions, based on the "perm_list"
   #  generated below, to workaround a LB SU issue.
   #
   #Generate permissions recovery list only when the linuxbase SU is uninstalled
   if ($patch->{'PATCHNAME'} =~ /^nortel-cs1000-linuxbase.+/)
   {

      #create file to hold the folder/file ownership and permission list
      my $perm_storage = '/var/tmp/perm_list';
      if (-e $perm_storage)
      {
         do_system([$RM, '-rf', $perm_storage]);
      }
      do_system([$TOUCH, $perm_storage]);

      #for each folder recursively capture the folder/file ownership&permissions
      #into the $perm_storage file to used by the basic hardening module.
      my @dir2protect = ('/opt/nortel/base', '/etc/opt/nortel/base', '/var/opt/nortel/base');
      foreach my $dir (@dir2protect)
      {
         if (not do_system("$FIND $dir -print -exec stat -c \%a:\%u:\%g {} \;", {$dsOUTFILE => $perm_storage, $dsOUTMODE => '>>'}))
         {
            baselog($LOG_ERROR, "Cannot collect permissions for '$dir'.");
         }
      } ## end foreach my $dir (@dir2protect...
   } ## end if ($patch->{'PATCHNAME'...

   # check patch status
   if ($patch->{'STATUS'} == $OUT_SERV)
   {

      # try to pout patch if poos is triggered by PM
      baselog($LOG_INFO, "Patch with pid $patch->{'PID'} is already out of service.", $TRUE);
      if ($self->{'context'} eq 'PM')
      {
         my %params = ('interact' => $FALSE, 'pid' => $patch->{'PID'}, 'all' => $FALSE, 'sp' => $FALSE, 'filename' => undef);
         if (not $self->pout(\%params))
         {
            baselog($LOG_ERROR, "Unable to pout patch '$patch->{'FILENAME'}'");
         }
      } ## end if ($self->{'context'}...
      else
      {
         baselog($LOG_ERROR, "Cannot take out of service patch with pid $patch->{'PID'} because it is already out of service.\n", $TRUE);
      }
      return $TRUE;
   } ## end if ($patch->{'STATUS'}...
   elsif ($patch->{'STATUS'} == $OUT_SERV_REBOOT)
   {
      baselog($LOG_ERROR, 'The patch ' . $patch->{'PID'} . " will be out of service after restart.\n", $TRUE);
      return $FALSE;
   }

   # Extract patch members
   if (not $patch->extract_patch_members())
   {
      baselog($LOG_ERROR, "Cannot extract patch members for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.\n", $TRUE);
      return $FALSE;
   }
   my $retval = $TRUE;
 POOS:
   {
      unless ($self->poos_Dependency($patch))
      {
         baselog($LOG_ERROR, "Poos dependency check failed for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.\n", $TRUE);
         $retval = $FALSE;
         last POOS;
      } ## end unless ($self->poos_Dependency...
      unless ($self->pre_poos($patch))
      {
         baselog($LOG_ERROR, "Poos preparation failed for $patch->{'PATCHNAME'} with pid $patch->{'PID'}.\n", $TRUE);
         $retval = $FALSE;
         last POOS;
      } ## end unless ($self->pre_poos($patch...
        #process PREPATCH script

      if (!$self->check_script($patch, $PREPATCH, \$exist_flag))  # if PREPATCH script exists
      {
         $retval = $FALSE;
         last POOS;
      }
      if ($exist_flag eq "yes")
      {

         # run patch PREPATCH script
         $cmd_retval = $self->run_script($patch, $PREPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            $retval = $TRUE;
            last POOS;
         }
         elsif ($cmd_retval == $ERROR)
         {
            $retval = $FALSE;
            last POOS;
         }
      } ## end if ($exist_flag eq "yes"...
      unless ($patch->uninstall($self->{'interact'}))
      {
         baselog($LOG_ERROR, "Patch $patch->{'PATCHNAME'} with pid $patch->{'PID'} cannot be put out of service.\n", $TRUE);
         $retval = $FALSE;
         last POOS;
      } ## end unless ($patch->uninstall(...

      #process POSTPATCH script
      if (!$self->check_script($patch, $POSTPATCH, \$exist_flag))  # if POSTPATCH script exists
      {
         $retval = $FALSE;
         last POOS;
      }
      if ($exist_flag eq "yes")
      {

         # run patch POSTPATCH script
         $cmd_retval = $self->run_script($patch, $POSTPATCH, $command);
         if ($cmd_retval == $DONE)
         {
            $retval = $TRUE;
            last POOS;
         }
         elsif ($cmd_retval == $ERROR)
         {
            $retval = $FALSE;
            last POOS;
         }
      } ## end if ($exist_flag eq "yes"...

      unless ($self->post_poos($patch, $is_last, \@applist))
      {
         baselog($LOG_ERROR, "Post poos operation failed for patch $patch->{'PATCHNAME'} with pid $patch->{'PID'}.\n", $TRUE);
      }

   } ## end POOS:

   # Cleanup
   $patch->remove_patch_members();
   if ($retval)
   {
      baselog($LOG_INFO, 'Patch ' . $patch->{'PID'} . ' has been taken out of service successfully.');

      # send an snmp trap to alert the admin that the patch is now out of service
      my $fullname = patchutil::patch::get_full_patch_file($patch);
      my $msg_parms = {
                       "patchID"  => "$patch->{PID}",
                       "MPLR"     => "$patch->{PATCHNAME}",
                       "CR"       => "$patch->{CR}",
                       "filename" => "$fullname"
                      };
      send_snmp_trap("PCH0111", $msg_parms);
   } ## end if ($retval)
   return $retval;
} ## end sub poos_patch($$$$)

############################################################
#Description: Perform some checking before loading:
#               File name
#               Patch members MD5 checksums
#               mercury module check
#               base and application version checking
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#        $_[2] - patch file name
#OUTPUT: $TRUE  - Success
#        $FALSE - In case any errors
############################################################
sub check_patch_preload($$$)
{
   my ($self, $patch, $filename) = @_;

   # Check patch name format
   my $fn;
   $fn = patchutil::patch::get_full_patch_file($patch);
   baselog($LOG_DEBUG, "filename=$filename, fn=$fn");
   if ($filename ne $fn)
   {
      baselog($LOG_ERROR, "File name $fn must match patch name $filename.\n", $TRUE);
      return $FALSE;
   }

   my $retval = $TRUE;
   unless ($patch->extract_patch_members())
   {
      baselog($LOG_ERROR, "Cannot extract patch members for $patch->{'PATCHNAME'}.\n", $TRUE);
      return $FALSE;
   }

   # check if patch data is correct (log messages will have already been printed)
   if ($patch->{'is_broken'})
   {
      return $FALSE;
   }

   #
   # Check if there are no new files contained in the patch
   # The function will check if the required RPM is installed on the target,
   # and if not it will check if there is an SU in a loaded state in the
   # patch database.
   #
   unless ($patch->check_patch_files($self->{'action'}, $self->{'DB'}))
   {
      baselog($LOG_ERROR, "Patch files checking failed.", $TRUE);
      return $FALSE;
   }

 CHECK:
   {
      my $parser = ifxml::new($APP_CFG_ROOT, $APPLCFG);
      if (!$parser)
      {
         baselog($LOG_ERROR, "No configuration file ($APPLCFG) found.");
         $retval = $FALSE;
         last CHECK;
      } ## end if (!$parser)

      # Linux Base release version
      my $params = use_confver($BASEAPPCONFVER);
      my $base_version;
      if ($params)
      {
         if ($params->{'APP_VERSION'})
         {
            $base_version = $params->{'APP_VERSION'};
         }
         else
         {
            baselog($LOG_ERROR, "No Product Release found.");
            $retval = $FALSE;
            last CHECK;
         } ## end else [ if ($params->{'APP_VERSION'...
      } ## end if ($params)

      # Check base version. product release is in format: 6.00.00.00
      # product release is a list of values (for release 6.0 only one release allowed)
      my @releases = @{$patch->{'PRODUCT_RLS'}};
      foreach my $rel (@releases)
      {
         unless ($base_version eq $rel)
         {
            baselog($LOG_ERROR, "Patch $patch->{'PATCHNAME'} is not intended for $base_version. Patch $patch->{'PATCHNAME'} is meant for @{$patch->{'PRODUCT_RLS'}}.\n", $TRUE);
            $retval = $FALSE;
            last CHECK;
         } ## end unless ($base_version eq $rel...
      } ## end foreach my $rel (@releases)

      # checking rpm name
      # if $app defined then rpm name specified in header.xml for the patch belong to the application
      my $app = $self->get_app_from_rpm($patch->{'RPM_NAME'});
      if (not defined $app)
      {

         # For the SP bundle scenario we can by-pass the check since the SP
         # Handler has already validated the SUs against the target RPMs
         if ($self->{'action'} eq $SPLOAD)
         {
            last CHECK;
         }
         baselog($LOG_INFO, "Can't get application by rpm name: $patch->{RPM_NAME}.");

      } ## end if (not defined $app)
      else
      {

         # getting application patched by the patch/SU
         my $entries = $parser->get_application({name => $app});

         # if there is only one entry found then application exists in configuration
         if (scalar @$entries != 1)
         {
            baselog($LOG_ERROR, "Too many or none configuration files ($APPLCFG) for application $app.");
            $retval = $FALSE;
            last CHECK;
         } ## end if (scalar @$entries !=...
      } ## end else [ if (not defined $app)
      #
      # Check if the RPM name, version & release (being delivered by the SU) is installed on the system.
      # - if so, then do not load the SU
      #
      my $rpm_retval;
      if ($patch->{'TYPE'} eq 'SU')
      {
         my @fields_SU;
         if (!extract_RPM_name($self, $patch->{'FILENAME'}, \@fields_SU))
         {
            baselog($LOG_ERROR, "Cannot obtain SU name from patch $patch->{'PATCHNAME'}\n", $TRUE);
            $retval = $FALSE;
            last CHECK;
         } ## end if (!extract_RPM_name(...
         elsif (defined($fields_SU[0]))  # SU name extracted
         {
            my $rpm_installed = undef;
            do_system([$RPM, "-q", $fields_SU[0]], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$rpm_retval});
            if ($rpm_retval)
            {
               $retval = $FALSE;
               baselog($LOG_ERROR, "RPM $fields_SU[0] is not installed.", $TRUE);
               last CHECK;
            } ## end if ($rpm_retval)
            else
            {
               my $rpm_installed_str = join(' ', @$rpm_installed);

               # Check if the RPM name, version & release is installed on the system.
               if ( $rpm_installed_str =~ "$fields_SU[0]-$fields_SU[1]-$fields_SU[2]" )
               {

                  # If the RPM (name, version & release) is already installed then we will skip loading the SU
                  baselog($LOG_WARNING, "Patch $patch->{'FILENAME'} not loaded (RPM already installed on system).\n", $TRUE);
                  $retval = $FALSE;
                  last CHECK;
               } ## end if ($rpm_installed_str...
               else
               {
                  baselog($LOG_INFO, "RPM $fields_SU[0] is installed.");

                  # RPM is installed but of different version/release than being delivered by the specific SU
                  # we will thus attempt to load/install it.
                  $retval = $TRUE;
               } ## end else [ if ($rpm_installed_str...
            } ## end else [ if ($rpm_retval)
         } ## end elsif (defined($fields_SU...
      } ## end if ($patch->{'TYPE'} eq...

      if (not $self->validate_min_rpm_ver($patch))
      {
         baselog($LOG_ERROR, "Pload minimum rpm version check failed for patch $patch->{'PATCHNAME'}", $TRUE);
         $retval = $FALSE;
         last CHECK;
      } ## end if (not $self->validate_min_rpm_ver...

      # rpm exists in the system.

      #calling destructor;
      undef $parser;

   } ## end CHECK:

   return $retval;
} ## end sub check_patch_preload($$$)

## @method make_hash_rpms($self)
# @brief   Make hash_rpms hash. It is filled with applications relaiting to RPMs
# @details Configuration file stored in $ARCHDIR are used: $INSTCFG and
#          $CONFVER for Nortel Applications, $BASEAPPINSTCFG and
#          $BASEAPPCONFVER for Base Applications.
# @param   $self  - reference to object.
# @return  $TRUE - success
# @return  $FALSE - error
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @date    May 2008
# @note    None
sub make_hash_rpms($)
{
   my $self = shift(@_);
   $self->{'hash_rpms'} = {};
   my %hash_rpms = ();
   my $params    = use_confver($CONFVER);

   # add RPMs of the Nortel applications to hash
   if ($params)
   {
      if (defined($params->{'APP_INSTALLED'}))
      {
         my $prs = ifxml::new($ARCHDIR, $INSTCFG);
         if (defined($prs))
         {
            my $entries = $prs->get_supportedConfig_application_name({'name' => $params->{'APP_INSTALLED'}});
            foreach my $app (@$entries)
            {
               foreach my $rpm (@{$prs->get_application_file_name({'name' => $app})})
               {
                  my @fields_rpm;
                  if (!extract_RPM_name($self, $rpm, \@fields_rpm))
                  {
                     baselog($LOG_ERROR, "Cannot obtain rpm name from $rpm\n");
                  }
                  elsif (defined($fields_rpm[0]))  # rpm name extracted
                  {
                     $hash_rpms{$fields_rpm[0]} = $app;
                  }
               } ## end foreach my $rpm (@{$prs->get_application_file_name...
            } ## end foreach my $app (@$entries)
            undef $prs;
         } ## end if (defined($prs))
         else
         {
            baselog($LOG_INFO, "$INSTCFG is not found.");
         }
      } ## end if (defined($params->{...
   } ## end if ($params)
   else
   {
      baselog($LOG_INFO, "$CONFVER is not found.");
   }

   # If it is Base applications installation then RPMs for Base applications were already
   # added in the first part of function and there is no need to add then once more
   if (!$baseAppsmode)
   {
      $params = use_confver($BASEAPPCONFVER);

      # add RPMs of the Base applications to hash
      if ($params)
      {
         if (defined($params->{'APP_INSTALLED'}))
         {
            my $prs = ifxml::new($ARCHDIR, $BASEAPPINSTCFG);
            if (defined($prs))
            {
               my $entries = $prs->get_supportedConfig_application_name({'name' => $params->{'APP_INSTALLED'}});
               foreach my $app (@$entries)
               {
                  foreach my $rpm (@{$prs->get_application_file_name({'name' => $app})})
                  {
                     my @fields_rpm;
                     if (!extract_RPM_name($self, $rpm, \@fields_rpm))
                     {
                        baselog($LOG_ERROR, "Cannot obtain rpm name from $rpm\n");
                     }
                     elsif (defined($fields_rpm[0]))  # rpm name extracted
                     {
                        if (exists($hash_rpms{$fields_rpm[0]}))
                        {
                           baselog($LOG_WARNING, "There is Base and Nortel application with name '$app'.");
                           next;
                        }
                        $hash_rpms{$fields_rpm[0]} = $app;
                     } ## end elsif (defined($fields_rpm...
                  } ## end foreach my $rpm (@{$prs->get_application_file_name...
               } ## end foreach my $app (@$entries)
               undef $prs;
            } ## end if (defined($prs))
            else
            {
               baselog($LOG_WARNING, "$BASEAPPINSTCFG is not found.");
            }
         } ## end if (defined($params->{...
      } ## end if ($params)
      else
      {
         baselog($LOG_INFO, "$BASEAPPCONFVER is not found.");
      }
   } ## end if (!$baseAppsmode)
   $self->{'hash_rpms'} = \%hash_rpms;
   return $TRUE;
} ## end sub make_hash_rpms($)

######################################################
# Check patch for embedded special instruction
# If yes, then print to the screen and ask
# user to continue.
# If non interactive mode then do not print anything
# Input: $_[0] - reference to the object
#        $_[1] - reference to the patch object
# Output: $TRUE - success
#         $FALSE - user has cancelled patch loading
#######################################################
sub check_spec_ins($$)
{
   my ($self, $patch) = @_;
   my $patch_file = patchutil::patch::get_full_patch_file($patch);

   if (   (defined($patch->{'SPECINST'}))
       && ($patch->{'SPECINST'} =~ m/^yes$/i))
   {

      # check if readme file exists inside patch
      if (not is_spec_ins($patch->{'FILENAME'}))
      {
         baselog($LOG_ERROR, "Special instructions set to yes inside header but readme file is not present for $patch->{'FILENAME'}.\n", $TRUE);
         return $FALSE;
      }
   } ## end if ((defined($patch->{...
   else
   {
      return $TRUE;  # not need to bothering
   }
   if ($self->{'interact'})
   {
      my @readme = `$TAR xzf $patch_file --to-stdout $README 2>/dev/null`;
      if ($? eq 0)
      {
         baselog($LOG_INFO, "Special instructions found in patch $patch_file.");
         print "\n";
         print @readme;
         unless (getYNResponse("Continue loading patch?", "N"))
         {
            baselog($LOG_INFO, "User has chosen to not proceed with special instructions for $patch->{'PATCHNAME'}.");
            return $FALSE;
         }
      } ## end if ($? eq 0)
      else
      {
         baselog($LOG_ERROR, "Cannot extract readme file for $patch->{'PATCHNAME'}.\n", $TRUE);
         return $FALSE;
      }
   }  # ($self->{'interact'})

   # Set $spec_dir
   my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};

   if (!-d $spec_dir)  # if patch special dir doesn't exist
   {

      # prepare patch special directory
      if (!spec_dir_prep($spec_dir))
      {
         return $FALSE;
      }
   } ## end if (!-d $spec_dir)

   # Untar existed readme to the SPECDIR
   if (do_system([$TAR, 'xvzf', $patch_file, '-C', $spec_dir, $README]) == 0)
   {
      baselog($LOG_ERROR, "Cannot extract readme file on $spec_dir.\n", $TRUE);
      return $FALSE;
   }
   baselog($LOG_DEBUG, "Readme file extracted successfully for $patch->{'PATCHNAME'}.");
   return $TRUE;

} ## end sub check_spec_ins($$)
######################################################
# Check bundle for embedded special instruction
# If yes, then print to the screen and ask
# user to continue.
# If non interactive mode then do not print anything
# Input: $_[0] - reference to the object
#        $_[0] - reference to the bundle object
# Output: $TRUE - success
#         $FALSE - user has cancelled sp command
#######################################################
sub check_SP_spec_ins($$)
{
   my ($self, $bundle_file) = @_;
   my $h_obj;
   my $header_file = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;
   my @readme;

   if (!get_header_obj($self, \$h_obj, $header_file))
   {
      return $FALSE;
   }

   # get SPECINST tag from SP header
   my $spec_ins_tag = $h_obj->getHeaderValue("HEADER/SPECINST");
   unless (defined($spec_ins_tag))
   {
      baselog($LOG_ERROR, "Cannot obtain special instructions tag from SP header for $bundle_file.\n", $TRUE);
      return $FALSE;
   }
   if ($spec_ins_tag =~ m/^no$/i)
   {
      baselog($LOG_DEBUG, "Special instructions tag is not set inside header for $bundle_file.");
      return $TRUE;
   }
   my $sp_readme = $SPLOAD_DIR . "/" . $README;

   # check if readme file exists
   if (!-e $sp_readme)
   {
      baselog($LOG_ERROR, "Special instructions tag is set inside SP header, but readme file doesn't exist for $bundle_file.\n");
      return $FALSE;
   }
   else  # readme file exists
   {
      if (!do_system([$CAT, "$SPLOAD_DIR/$README"], {$dsOUT => \@readme}))
      {
         baselog($LOG_ERROR, "Cannot extract readme file for SP $bundle_file.\n", $TRUE);
         return $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "Special instructions found in SP $bundle_file.");
         print "\n";
         print @readme;
         if ($self->{'interact'})
         {
            unless (getYNResponse("Continue installing SP?", "N"))
            {
               baselog($LOG_INFO, "User chose not to proceed with spec. ins. for SP $bundle_file.");
               return $FALSE;
            }
         } ## end if ($self->{'interact'...
      } ## end else [ if (!do_system([$CAT, ...
   } ## end else [ if (!-e $sp_readme)
   return $TRUE;
}  # end sub check_SP_spec_ins($$)
######################################################################
## @fn sub check_script($self, $patch, $script_name, $exist_flag)
# @brief Check if script exists for a given patch or SU
# @details Check if script flag is set to "yes" inside patch or SU header
# @        and if it does, check if script exists inside the patch or SU.
# @param $self - reference to the object
# @param $patch - reference to the patch object
# @param $script_name - reference to the script name
# @parm  $exist_flag - reference to flag that will be set to 'YES' if
#        script exists or 'NO' if scripts doesn't exists.
# @return $TRUE - script validation successful
#         $FALSE - script validation fails
######################################################################
sub check_script($$$$)
{
   my ($self, $patch, $script_name, $exist_flag) = @_;
   my $patch_file = patchutil::patch::get_full_patch_file($patch);
   my $script_tag = "";

   if ($script_name eq $PREPATCH)
   {
      $script_tag = $patch->{PRE_SCRIPT};
   }
   elsif ($script_name eq $POSTPATCH)
   {
      $script_tag = $patch->{POST_SCRIPT};
   }
   if ((defined($script_tag)) && ($script_tag =~ m/^no$/i))
   {
      baselog($LOG_DEBUG, "Script tag is not set inside header for $patch->{'PATCHNAME'}.\n");
      ${$exist_flag} = "no";
      return $TRUE;
   } ## end if ((defined($script_tag...
   $script_name = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'} . "/" . $script_name;
   if (!-e $script_name)
   {
      baselog($LOG_ERROR, "Script tag is set to yes inside header file, but script $script_name is not present inside $patch->{'PATCHNAME'}.\n", $TRUE);
      return $FALSE;
   }

   baselog($LOG_DEBUG, "Script $script_name exists for $patch->{'PATCHNAME'}.");
   ${$exist_flag} = "yes";
   return $TRUE;

} ## end sub check_script($$$$)

####################################################################
## @fn sub load_script($self, $patch)
# @brief Load script in the patch special directory
# @details Load script in the SPECDIR directory, change directory
#          permissions
# @param $self - reference to the object
# @param $patch - reference to the patch object
# @return TRUE - script is loaded
# @return FALSE - script is not loaded
####################################################################
sub load_script($$$)
{
   my ($self, $patch, $script_name) = @_;

   my $patch_file = patchutil::patch::get_full_patch_file($patch);
   my $spec_dir   = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};

   # prepare patch special directory
   if (!spec_dir_prep($spec_dir))
   {
      baselog($LOG_ERROR, "Cannot prepare patch special directory for patch $patch->{'PATCHNAME'}.\n", $TRUE);
      return $FALSE;
   }

   # Untar existed patch script
   if (do_system([$TAR, 'xvzf', $patch_file, '-C', $spec_dir, $script_name]) == 0)
   {
      baselog($LOG_ERROR, "Cannot extract file $script_name for patch $patch->{'PATCHNAME'}.\n", $TRUE);
      return $FALSE;
   }

   #Change permissions for prepatch script file
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, 'gu+x', "$spec_dir/$script_name"]))
   {
      baselog($LOG_ERROR, "Cannot change permissions for $script_name script for patch $patch->{'PATCHNAME'}.\n", $TRUE);
      return $FALSE;
   }

   return $TRUE;

} ## end sub load_script($$$)
######################################################################
## @fn sub spec_dir_cleanup($spec_dir)
# @brief Cleans up patch special directory
# @details Cleans up SPECDIR/patch directory
# @param $spec_dir - reference to the patch special directory
# @return TRUE - cleanup is successful
# @return FALSE - cleanup fails
######################################################################
sub spec_dir_cleanup($)
{
   my $spec_dir = shift @_;

   # Delete sub dir and their content for this special patch
   if (do_system([$RM, '-rf', $spec_dir]))
   {
      baselog($LOG_DEBUG, "Previous patch directory $spec_dir deleted.");
      return $TRUE;
   }
   return $FALSE;  #if cannot delete directory/files

} ## end sub spec_dir_cleanup($)

######################################################################
## @fn sub spec_dir_prep($spec_dir)
# @brief Prepare patch SPECDIR directory
# @details Create patch SPECDIR, update permissions and ownership
# @param $spec_dir - reference to the patch special directory
# @return TRUE - prepare special directory is successful
# @return FALSE - prepare special directory  fails
######################################################################
sub spec_dir_prep($)
{
   my $spec_dir = shift @_;

   #Create sub dir for this special patch
   if (!do_system([$MKDIR, '-p', $spec_dir]))
   {
      baselog($LOG_ERROR, "Cannot create $spec_dir directory.\n", $TRUE);
      return $FALSE;
   }

   #Changed permission of patch $spec_dir
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, 'gu+wx', $spec_dir]))
   {
      baselog($LOG_ERROR, "Cannot change permissions for $spec_dir directory.\n", $TRUE);
      return $FALSE;
   }

   # Change special dir ownership
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHOWN, $spec_dir]))
   {
      baselog($LOG_ERROR, "Cannot change $spec_dir directory ownerships.\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;

} ## end sub spec_dir_prep($)
######################################################################
## @fn sub run_script($self, $patch, $script_name, $command)
# @brief Run script for a given patch and command.
# @details Compose the script command base on interact and sp flags settings
# @        for the command and script_name based as parameters
# @        Run script and return a reference value from the script run
# @param $self - reference to the object
# @param $patch - reference to the patch object
# @param $retval_ref - reference to the return value of the script run
# @param $script_name - reference to the name of the script to be run
# @param $command -  reference to the command used to run the script
# @param $stage - prePatch or postPatch
# @return $OK    - script run for given command returned $OK
# @return $DONE  - script run for given command returned $DONE
# @return $ERROR - script run for given command returned $ERROR
######################################################################
sub run_script($$$$)
{
   my ($self, $patch, $script_name, $command) = @_;

   my @cmd_script;
   my $spec_dir = $SPECINS_DIR . "/" . $patch->{'PATCHNAME'};

   # check if interactive mode is set
   if ($self->{'interact'})
   {

      # if sp option was entered from command line, add sp option to run the script
      if (exists $self->{'sp'} and not $self->{'sp'})
      {
         push @cmd_script, "$spec_dir/$script_name", "--cmd=$command";
      }
      else
      {
         push @cmd_script, "$spec_dir/$script_name", "-sp", "--cmd=$command";
      }
   } ## end if ($self->{'interact'...
   else  #pass -r flag to script if issued by user to turn off interactive mode
   {

      # if sp option was entered from command line, add sp option to run the script
      if (exists $self->{'sp'} and not $self->{'sp'})
      {
         push @cmd_script, "$spec_dir/$script_name", "-r", "--cmd=$command";
      }
      else
      {
         push @cmd_script, "$spec_dir/$script_name", "-r", "-sp", "--cmd=$command";
      }
   } ## end else [ if ($self->{'interact'...

   # Check if the current user is root
   if (is_right_user($ROOT_ACCT) == $TRUE)
   {

      # ensure the script is run under 'nortel' account
      @cmd_script = @{prepare_command_to_run_as(join(' ', @cmd_script), $NORTEL_ACCT)};
      if (scalar(@cmd_script) == 0)
      {
         baselog($LOG_ERROR, "prepare_command_to_run_as() failed.");
         return $ERROR;
      }
   } ## end if (is_right_user($ROOT_ACCT...

   # run script
   my $ret_script;
   if (!do_system(\@cmd_script, {$dsRETVAL => \$ret_script, $dsSTDOUT => $TRUE}))
   {
      baselog($LOG_ERROR, "Failed to run $script_name for command $command.\n", $TRUE);
      return $ERROR;
   }
   if (!defined($ret_script))
   {
      return $ERROR;
   }
   if ($ret_script != $OK)
   {
      if ($ret_script == $ERROR)
      {
         if ($command eq "pload")
         {
            baselog($LOG_ERROR, "Running $command cannot delete $spec_dir directory.\n") unless spec_dir_cleanup($spec_dir);
            baselog($LOG_ERROR, "$command script for patch $patch->{PATCHNAME} returned error.", $TRUE);
         }
         else
         {
            baselog($LOG_ERROR, "$command script for patch $patch->{PATCHNAME} (pid: $patch->{PID}) failed.", $TRUE);
         }
         return $ERROR;  # the script returned ERROR
      } ## end if ($ret_script == $ERROR...
      elsif ($ret_script == $DONE)
      {
         if ($command eq "pload")
         {
            baselog($LOG_INFO, "Script returned $ret_script. No need to run $command for $patch->{'PATCHNAME'}.");
         }
         else
         {
            baselog($LOG_INFO, "Script returned $ret_script. No need to run $command for patch $patch->{'PID'}.");
         }
         return $DONE;
      } ## end elsif ($ret_script == $DONE...
   } ## end if ($ret_script != $OK...
   else
   {
      if ($command eq "pload")
      {
         baselog($LOG_DEBUG, "Script returned OK for patch $patch->{'PATCHNAME'}.");
      }
      else
      {
         baselog($LOG_DEBUG, "Script returned OK for patch $patch->{'PID'}.");
      }
   } ## end else [ if ($ret_script != $OK...
   return $OK;
} ## end sub run_script($$$$)

#########################################################################
## @fn sub run_SP_script($self, $sp_script_name, $command, $dir)
# @brief Run SP script given by $sp_script_name
# @details Check if script tag is set to "yes" inside SP header for the script
# @        passed as parameter. If it does, check if script exists under SP directory.
# @        Compose script command base on interact and sp flags settings
# @        Run SP script
# @param $self - reference to the object
# @param $sp_script_name - reference to the SP script name
# @param $command - command issued (spload, spins or spout)
# @param $dir - directory where the patchinfo file is located
# @return $ERROR - script run for given command returned $ERROR or
# @              - script tag inside SP header is set to "yes" but script
# @                is not present inside bundle.
# @return $OK    - script run for given command returned $OK or
# @              - script is tag inside SP header is set to "no" and script
# @                is not present inside bundle
# @return $DONE  - script run for given command returned $DONE
#########################################################################
sub run_SP_script($$$$)
{
   my ($self, $sp_script, $command, $dir) = @_;
   my @cmd_script;
   my $sp_script_tag = "";
   my $h_obj;

   # check if script tag in SP header is set to YES
   my $sp_header = $dir . "/" . $PATCHINFO_FILE;
   if (!get_header_obj($self, \$h_obj, $sp_header))
   {
      return $ERROR;
   }
   if ($sp_script eq $PREPATCH)
   {
      $sp_script_tag = $h_obj->getHeaderValue("HEADER/PRE_SCRIPT");
   }
   elsif ($sp_script eq $POSTPATCH)
   {
      $sp_script_tag = $h_obj->getHeaderValue("HEADER/POST_SCRIPT");
   }
   if (   (defined($sp_script_tag))
       && (($sp_script_tag eq "no") || ($sp_script_tag eq "NO")))
   {
      baselog($LOG_DEBUG, "Script tag is not set inside SP header.\n");
      return $OK;
   } ## end if ((defined($sp_script_tag...

   # if script tag is set to yes, run SP script if it exists
   my $sp_script_name = $dir . "/" . $sp_script;
   if (-e $sp_script_name)
   {

      #Change permissions for $sp_script_name file
      if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, 'gu+x', $sp_script_name]))
      {
         baselog($LOG_ERROR, "Cannot change permissions for bundle script $sp_script_name.\n", $TRUE);
         return $ERROR;
      }

      # check if interactive mode is set
      if ($self->{'interact'})
      {

         # check if sp flag was entered from command line and add sp option if it set to true
         if (exists $self->{'sp'} and not $self->{'sp'})
         {
            push @cmd_script, $sp_script_name, "--cmd=$command";
         }
         else
         {
            push @cmd_script, $sp_script_name, "-sp", "--cmd=$command";
         }
      } ## end if ($self->{'interact'...
      else  #pass -r flag to script if issued by user to turn off interactive mode
      {

         # check if sp flag was entered from command line and add sp option if it set to true
         if (exists $self->{'sp'} and not $self->{'sp'})
         {
            push @cmd_script, $sp_script_name, "-r", "--cmd=$command";
         }
         else
         {
            push @cmd_script, $sp_script_name, "-r", "-sp", "--cmd=$command";
         }
      } ## end else [ if ($self->{'interact'...

      my $ret_script;

      # run script
      if (!do_system(\@cmd_script, {$dsRETVAL => \$ret_script, $dsSTDOUT => $TRUE}))
      {
         baselog($LOG_ERROR, "Failed to run $sp_script_name.\n", $TRUE);
         return $ERROR;
      }
      if (!defined($ret_script))
      {
         return $ERROR;
      }
      if ($ret_script != $OK)
      {
         if ($ret_script == $ERROR)
         {
            baselog($LOG_ERROR, "Script $sp_script_name returned ERROR for $command.\n", $TRUE);
            return $ERROR;  # the script returned ERROR
         }
         elsif ($ret_script == $DONE)
         {
            baselog($LOG_INFO, "Script $sp_script_name returned $ret_script. No need to run $command.\n");
            return $DONE;
         }
      } ## end if ($ret_script != $OK...
      else
      {
         baselog($LOG_DEBUG, "Script $sp_script_name returned OK for $command.");
         return $OK;
      }
   } ## end if (-e $sp_script_name...
   else  # $sp_script_name doesn't exist inside bundle
   {
      baselog($LOG_ERROR, "Script tag is set to yes inside SP header, but script $sp_script_name is not present inside bundle.\n", $TRUE);
      return $ERROR;
   }
   return $OK;
} ## end sub run_SP_script($$$$)

######################################################################
## @fn sub create_spload_info_file($self, $bundle_name)
# @brief Create status file $SPLOAD_INFO_FILE.
# @details Create status file $SPLOAD_INFO_FILE with information
# @        relevant to the spload command and length zero file with
# @        bundle name + extension .bundle inside SPLOAD directory.
# @param $self - reference to the object
# @param $bundle_name - reference to the bundle name
# @return TRUE - creation of files was successful
# @return FALSE - creation of files failed
######################################################################

sub create_spload_info_file($$)
{
   my ($self, $bundle_name) = @_;
   my $user   = "";
   my $time   = "";
   my $retval = $TRUE;
   my $fd;

   $user = getlogin() || (getpwuid($<))[0];
   $time = localtime(time());

   unless (open($fd, ">$SPLOAD_INFO_FILE"))
   {
      baselog($LOG_ERROR, "Cannot create $SPLOAD_INFO_FILE file for writing\n", $TRUE);
      $retval = $FALSE;
   }
   else
   {

      # Write message to the file
      print $fd "BUNDLE_NAME::$bundle_name\n";
      print $fd "SPLOAD_USER::$user\n";
      print $fd "SPLOAD_DATE::$time\n";

      # Close file
      close($fd);

      #Changed permission of file
      if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, '660', $SPLOAD_INFO_FILE]))
      {
         baselog($LOG_ERROR, "Cannot change permissions for $SPLOAD_INFO_FILE file\n", $TRUE);
         $retval = $FALSE;
      }
   } ## end else

   $bundle_name = "$SPLOAD_DIR/$bundle_name.bundle";

   # create length zero file with bundle name and extension .bundle under spload dir
   if (!do_system([$TOUCH, $bundle_name]))
   {
      baselog($LOG_ERROR, "Unable to create $bundle_name undern the spload directory\n", $TRUE);
      return $FALSE;
   }

   #Changed permission of length zero bundle file
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, '660', $bundle_name]))
   {
      baselog($LOG_ERROR, "Cannot change permissions for $bundle_name file\n", $TRUE);
      return $FALSE;
   }
   return $retval;
} ## end sub create_spload_info_file($$)
######################################################################
## @fn sub create_spins_info_file($self, $bundle_name)
# @brief Create status file $SPINS_INFO_FILE.
# @details Create status file $SPINS_INFO_FILE with inforamtion
# @        relevant to the spins command.
# @param $self - reference to the object
# @param $bundle_name - reference to the bundle name
# @return TRUE - creation of file was successful
# @return FALSE - creation of file failed
######################################################################

sub create_spins_info_file($$$)
{
   my ($self, $bundle_name, $status) = @_;
   my $user = "";
   my $time = "";
   my $fd;

   $user = getlogin() || (getpwuid($<))[0];
   $time = localtime(time());

   unless (open($fd, ">$SPINS_INFO_FILE"))
   {
      baselog($LOG_ERROR, "Cannot create $SPINS_INFO_FILE file for writing\n", $TRUE);
      return $FALSE;
   }

   # Write message to the file
   print $fd "BUNDLE_NAME::$bundle_name\n";
   print $fd "SPINS_USER::$user\n";
   print $fd "SPINS_DATE::$time\n";
   if ($status == $FAULT)
   {
      print $fd "STATUS::spins command completed with some errors detected.\n";
   }
   else
   {
      print $fd "STATUS::spins command completed with no errors detected.\n";
   }

   # Close file
   close($fd);

   #Changed permission file
   if (!do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, '660', $SPINS_INFO_FILE]))
   {
      baselog($LOG_ERROR, "Cannot change permissions for $SPINS_INFO_FILE file\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub create_spins_info_file($$$)

######################################################################
## @fn sub extract_bundle($self, $sp_file)
# @brief Check if SPLOAD dir exists. If it does, it means another
# @      SP is already loaded and pending to be put in
# @      service, so SPLOAD command is not allowed.
# @      Otherwise, create SPLOAD dir and untar the bundle file
# @details
# @ 1. Create SPLOAD dir if doesn't exist
# @ 2. Untar bundle file under SPLOAD directory.
# @param $self - reference to the object
# @param $sp_file - reference to the full SP file name
# @return TRUE -  bundle is saved under SP/SPLOAD directory
# @return FALSE - bundle cannot be saved under SP/SPLOAD directory
######################################################################
sub extract_bundle($$)
{
   my ($self, $sp_file) = @_;
   my $zip_retval;
   do_system([$UNZIP, '-t', $sp_file, $PATCHINFO_FILE], {$dsNOERROR => $TRUE, $dsRETVAL => \$zip_retval});
   if ($zip_retval)
   {
      baselog($LOG_ERROR, "$sp_file is not a service pack bundle.\n", $TRUE);
      my $tar_retval;
      do_system([$TAR, '-tf', $sp_file, $PATCHINFO_FILE], {$dsNOERROR => $TRUE, $dsRETVAL => \$tar_retval});
      if (not $tar_retval)
      {
         baselog($LOG_ERROR, "It appears to be a patch or SU. Move the file to $PATCHDIR and use pload.", $TRUE);
      }
      return $FALSE;
   } ## end if ($zip_retval)
   if (!-d $SPLOAD_DIR)  # spload dir doesn't exists
   {

      # create SP/SPLOAD directory
      if (!spec_dir_prep($SPLOAD_DIR))
      {
         return $FALSE;
      }

      #Unzip bundle in the SPLOAD directory
      if (!do_system([$UNZIP, $sp_file, '-d', $SPLOAD_DIR]))
      {
         baselog($LOG_ERROR, "Cannot save bundle $sp_file under SPLOAD directory.\n", $TRUE);
         return $FALSE;
      }
   } ## end if (!-d $SPLOAD_DIR)
   return $TRUE;
}  # end sub extract_bundle($)

######################################################################
## @fn sub get_header_obj($self, $header_obj, $header_xml)
# @brief Base on the header_xml file base as parameter create
# @      a header object and return it.
# @details
# @ 1. Check if header file exists given by $header_xml parameter
# @ 2. If it does, create a header object and save his value in the passed
# @    $header_obj reference parameter
# @param $self - reference to the object
# @param $header_obj - reference to the header object obtained
# @param $header_xml - reference to the header file name
# @return TRUE -  header object created successfully
# @return FALSE - failed to create header object
######################################################################
sub get_header_obj($$$)
{
   my ($self, $header_obj, $header_xml) = @_;

   if (!-e $header_xml)
   {
      baselog($LOG_ERROR, "Header file doesn't exist inside SP.\n", $TRUE);
      return $FALSE;
   }
   ${$header_obj} = patchutil::header_xml::new($header_xml);
   if (!defined(${$header_obj}))
   {
      baselog($LOG_ERROR, "Cannot create header_xml object.\n", $TRUE);
      return $FALSE;
   }

   return $TRUE;
} ## end sub get_header_obj($$$)
######################################################################
## @fn sub get_prod_rls_from_SP_header($self, $SP_header_prod_rls)
# @brief Obtains product release from SP header
# @details
# @ 1. Get a header object for the SP header
# @ 2. Call getReleases to obtain product release value from SP header
# @param $self - reference to the object
# @param $SP_header_prod_rls - reference to the header product release
# @return TRUE -  obtained SP header prodcut release successfully
# @return FALSE - failed to obtain SP header product release
######################################################################
sub get_prod_rls_from_SP_header($$)
{
   my ($self, $SP_header_prod_rls) = @_;
   my $h_obj;
   my $sp_header = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;

   if (!get_header_obj($self, \$h_obj, $sp_header))
   {
      return $FALSE;
   }
   ${$SP_header_prod_rls} = $h_obj->getReleases();
   unless (defined(${$SP_header_prod_rls}))
   {
      baselog($LOG_ERROR, "Cannot obtain SP header product release\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub get_prod_rls_from_SP_header($$)
######################################################################
## @fn sub get_install_prod_rls($self, $inst_prod_rls)
# @brief Obtains product release from installation file
# @details
# @    Read installation file $BASEAPPCONFVER and obtain APP_VERSION
# @    value.
# @param $self - reference to the object
# @param $inst_prod_rls - reference to the installation product release
# @return TRUE -  obtained installation prodcut release successfully
# @return FALSE - failed to obtain installation product release
######################################################################
sub get_install_prod_rls($$)
{
   my ($self, $inst_prod_rls) = @_;

   my $params = use_confver($BASEAPPCONFVER);
   if (!defined($params))
   {
      baselog($LOG_ERROR, "Base Applications are not installed. System is not installed properly.\n", $TRUE);
      return $FALSE;
   }
   ${$inst_prod_rls} = $params->{'APP_VERSION'};
   unless (defined(${$inst_prod_rls}))
   {
      baselog($LOG_ERROR, "Cannot obtain product release installed\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub get_install_prod_rls($$)
######################################################################
## @fn sub validate_product_rls()
# @brief Validates that Product Release specified in the SP header
# @      matches Product Release specified in the installation file
# @details 1. Read SP header.xml and get the Product Release value
# @        2. Read the Product Release value specified in the installation
# @        file.
# @        3. Compare both Product Release values and return TRUE or FALSE
# @        depending on they match or not.
# @return TRUE - product release matches
# @return FALSE - product release doesn't match
######################################################################
sub validate_product_rls($)
{
   my ($self) = @_;
   my $SP_header_prod_rls;
   my $inst_prod_rls;

   # get product release from SP header
   if (!get_prod_rls_from_SP_header($self, \$SP_header_prod_rls))
   {
      return $FALSE;
   }

   # get product release from installation file
   if (!get_install_prod_rls($self, \$inst_prod_rls))
   {
      return $FALSE;
   }
   if ($SP_header_prod_rls ne $inst_prod_rls)
   {
      baselog($LOG_ERROR, "Product release validation fails. The product release from SP header $SP_header_prod_rls doesn't match the product release from installation file $inst_prod_rls.\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub validate_product_rls($)
######################################################################
## @fn sub obtain_SUS_and_patches($self, $SU_list, $patches_list, $obsolete_list)
# @brief Obtain the list of required SU's and patches from the SP
# @      header.xml file included inside the SP bundle.
# @details Read SP header.xml and get the list of required SUs
# @        specified by SUs tag and the list of patches specified by
# @        PATCHES tag.
# @param $self - reference to the object
# @param $SU_list - reference to the list of SU's required
# @param $patches_list - reference to the list of patches required
# @param $obsolete_list - reference to the list of obsolete SU's and patches
# @return TRUE - obtained list of SU's and patches successfully
# @return FALSE - cannot obtain list of SU's and/or patches
######################################################################
sub obtain_SUS_and_patches($$$;$)
{
   my ($self, $SU_list, $patches_list, $obsolete_list) = @_;
   my $h_obj;

   my $header_xml = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;

   if (-e $header_xml)
   {

      #obtain list of SU's that must be in SRV
      $h_obj = patchutil::header_xml::new($header_xml);
      if (!defined($h_obj))
      {
         baselog($LOG_ERROR, "Cannot create header_xml object\n", $TRUE);
         return $FALSE;
      }

      @$SU_list      = $h_obj->getSPContent("SUS");
      @$patches_list = $h_obj->getSPContent("PATCHES");
      if ((scalar(@$SU_list) == 0) && (scalar(@$patches_list) == 0))
      {
         baselog($LOG_ERROR, "The SP $header_xml file doesn't contain any SUs or patches. There is nothing to install.\n", $TRUE);
         return $FALSE;
      }
      if (scalar(@$SU_list) == 0)
      {
         baselog($LOG_DEBUG, "The SP $header_xml file doesn't contain SUs.\n");
      }
      if (scalar(@$patches_list) == 0)
      {
         baselog($LOG_DEBUG, "The SP $header_xml file doesn't contain patches.\n");
      }

      if ($self->{'action'} eq $SPLOAD)
      {
         my $su_index;
         if ($self->{'SP_type'} eq 'DELTA_SP')
         {

            # If installed SU with the same RPM name as RPM name of SU inside the SP header
            # is not tagged as obsolete, then do not remove such SU from the system
            if (not defined $obsolete_list)
            {
               baselog($LOG_ERROR, "List of obsolete SUs and patches is undefined.");
               return $FALSE;
            }
            for ($su_index = 0 ; $su_index < scalar(@$SU_list) ; $su_index++)
            {
               my @fields_SU = ();
               if (not extract_RPM_name($self, @$SU_list[$su_index], \@fields_SU))
               {
                  baselog($LOG_ERROR, "Cannot extract RPM name from SU @$SU_list[$su_index]");
                  return $FALSE;
               }
               if (defined($fields_SU[0]) and defined($fields_SU[1]) and defined($fields_SU[2]) and defined($fields_SU[3]))
               {

                  # put together RPM NAME base on SU name without MPL issue and extension
                  my $rpm_name = $fields_SU[0] . "-" . $fields_SU[1] . "-" . $fields_SU[2] . "." . $fields_SU[3];

                  my @db_patches = $self->{'DB'}->get_rpm_patches_by_name($rpm_name, $IN_SERV, 'SU');
                  if (scalar @db_patches)
                  {
                     my $index;
                     for ($index = 0 ; $index < scalar(@db_patches) ; $index++)
                     {
                        last if (pos_in_array($obsolete_list, $db_patches[$index]->{'FILENAME'}) != -1);
                     }
                     if ($index == scalar(@db_patches))
                     {
                        baselog($LOG_NOTICE, "'@$SU_list[$su_index]' SU cannot be installed by DELTA SP. On the system installed SU for the same RPM.");
                        splice(@$SU_list, $su_index, 1);
                        $su_index--;
                     } ## end if ($index == scalar(@db_patches...
                  } ## end if (scalar @db_patches...
               } ## end if (defined($fields_SU...
            } ## end for ($su_index = 0 ; $su_index...
         } ## end if ($self->{'SP_type'}...
         elsif ($self->{'SP_type'} eq 'SEARCH_SP')
         {

            # For search-based SP do not install any SU which has RPM version smaller than
            # version of RPM installed
            for ($su_index = 0 ; $su_index < scalar(@$SU_list) ; $su_index++)
            {
               my $ret = compare_SU_version($self, @$SU_list[$su_index]);
               if (not defined $ret)
               {
                  baselog($LOG_ERROR, "Cannot compare SU RPM version with installed RPM", $TRUE);
                  return $FALSE;
               }
               elsif ($ret == $FALSE)
               {
                  baselog($LOG_NOTICE, "'@$SU_list[$su_index]' SU cannot be installed by SEARCH SP.");
                  splice(@$SU_list, $su_index, 1);
                  $su_index--;
               } ## end elsif ($ret == $FALSE)
            } ## end for ($su_index = 0 ; $su_index...
         } ## end elsif ($self->{'SP_type'}...
      } ## end if ($self->{'action'} ...
   } ## end if (-e $header_xml)
   else
   {
      baselog($LOG_ERROR, "SP header.xml file does NOT exists\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;

}  # end sub obtain_SUS_and_patches($$)
######################################################################
## @fn sub move_member_to_patchdir($self, $member_file)
# @brief Move SU or patch file from SPLOAD to PATCHDIR directory
# @details
# @    Move SU or patch file from SPLOAD to PATCHDIR directory if
# @    file exists inside the bundle
# @    Update $CPLIST_FILE with file name moved
# @param $self - reference to the object
# @param $mameber_file - reference to the patch or SU name
# @return TRUE -  move completed successfully
# @return FALSE - move failed
######################################################################
sub move_member_to_patchdir($$)
{
   my ($self, $member_file) = @_;

   my $file = $SPLOAD_DIR . "/" . $member_file;

   # if member required exists on the bundle
   if (-e $file)
   {

      #mv $member_file to patch directory
      if (!do_system([$MV, $file, $PATCHDIR]))
      {

         baselog($LOG_ERROR, "Cannot move $member_file to patch directory\n", $TRUE);
         return $FALSE;
      } ## end if (!do_system([$MV, $file...
      else
      {

         # update SPLIST_FILE with name of patch or SU moved to PATCHDIR
         if (!update_file($self, $CPLIST_FILE, $member_file))
         {
            baselog($LOG_ERROR, "Failed to update $CPLIST_FILE for $member_file\n", $TRUE);
            return $FALSE;
         }
      } ## end else [ if (!do_system([$MV, $file...
   } ## end if (-e $file)
   else
   {
      baselog($LOG_ERROR, "$member_file doesn't exist in bundle\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub move_member_to_patchdir($$)
######################################################################
## @fn sub extract_RPM_name($self, $file, $fields_array)
# @brief Base on an RPM or SU name, extract each name component and
# @      set the @fields_array with all the fields found
# @details For an SU extract the following fields:
# @        $1 = #name $2 = #version  $3 = #release  $4 = #machine type $5 = #MPL issue $6 = #extension
# @        Example: nortel-cs1000-vtrk-5.90.10-01.i386.000.ntl
# @        Results: 1: nortel-cs1000-vtrk
# @                 2: 5.90.10
# @                 3: 01
# @                 4: i386
# @                 5: 000
# @                 6: ntl
# @        For an RPM extract the following fields:
# @        $1 = #name $2 = #version  $3 = #release  $4 = #machine type
# @        Example: nortel-cs1000-vtrk-5.90.10-01.i386
# @        Results: 1: nortel-cs1000-vtrk
# @                 2: 5.90.10
# @                 3: 01
# @                 4: i386
# @param $self - reference to the object
# @param $file - name of the RPM or SU file
# @param $fields_array - reference to an array with all the fields found for
# @      an RPM or SU
# @return TRUE - Cannot extract field components from SU or RPM
# @return FALSE - Extracted field components from SU or RPM successfully
######################################################################
sub extract_RPM_name($$$)
{
   my ($self, $file, $fields_array) = @_;

   if ($file =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)\.([0-9]+)\.(ntl)$/)  # SU
   {
      push @{$fields_array}, $1;                                                            #SU name
      push @{$fields_array}, $2;                                                            #SU version
      push @{$fields_array}, $3;                                                            #SU rls
      push @{$fields_array}, $4;                                                            #machine type
      push @{$fields_array}, $5;                                                            #MPL issue
      push @{$fields_array}, $6;                                                            #extension
   } ## end if ($file =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)\.([0-9]+)\.(ntl)$/...
   elsif ($file =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)$/)                # RPM
   {
      push @{$fields_array}, $1;                                                            #rpm name
      push @{$fields_array}, $2;                                                            #rpm version
      push @{$fields_array}, $3;                                                            #rpm rls
      push @{$fields_array}, $4;                                                            #machine type
   } ## end elsif ($file =~ /^([\w|\.|-]+)-([\w|\.]+)-([\w|\.]+)\.([0-9|A-Z|a-z]+)$/...
   else
   {
      baselog($LOG_ERROR, "Cannot find RPM pattern for $file\n", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub extract_RPM_name($$$)
########################################################################
## @fn sub get_RPM_name_from_header($self, $path_header, \$rpm_name)
# @brief Gets the RPM name from the patch or SU header
# @details Gets the RPM name from the patch or SU header given by the TAG
# @        RPM_NAME
# @param $self - reference to the object
# @param $patch_file - reference to patch file
# @param $rpm_name - reference to the name of the RPM for the patch or SU
# @return TRUE - obtained RPM name successfully from patch or SU header
# @return FALSE - failed to obtain RPM name from patch or SU header
######################################################################
sub get_RPM_name_from_header($$$)
{
   my ($self, $path_header, $rpm_name) = @_;
   my $h_obj;

   if (!get_header_obj($self, \$h_obj, "$path_header/$PATCHINFO_FILE"))
   {
      return $FALSE;
   }

   my $rpm_name_found = $h_obj->getHeaderValue("HEADER/RPM_NAME");
   unless (defined($rpm_name_found))
   {
      baselog($LOG_ERROR, "Cannot obtain RPM NAME from patch header", $TRUE);
      return $FALSE;
   }

   #remove temporarly dir where patch header was untar
   if (!do_system([$RM, '-rf', $path_header]))
   {
      baselog($LOG_ERROR, "Cannot cleanup temporarly dir $path_header\n", $TRUE);
      return $FALSE;
   }

   ${$rpm_name} = $rpm_name_found;

   return $TRUE;
} ## end sub get_RPM_name_from_header($$$)

######################################################################
## @fn sub validate_patch_against_SU($self, $rpm_pkg, $sus_list)
# @brief Check if there is an SU loaded in the database with the
# @      same name and version of the RPM from patch header
# @details 1. Find SU from @SU_list with same RPM_name passed as parameter
# @        (RPM associated to the patch)
# @        2. Check if the SU associated to the patch is loaded in the DB.
# @param $self - reference to the object
# @param $rpm_pkg - reference to RPM associated to the patch.
# @param $sus_list - reference to the list of SUs from SP header
# @return TRUE - found SU loaded in DB with same full RPM name
# @return FALSE - cannot find SU loaded in DB with same full RPM name
######################################################################
sub validate_patch_against_SU($$$)
{
   my ($self, $rpm_pkg, $sus_list) = @_;
   my $retval = $FALSE;
   my @fields_SU;
   my $rpm_name_for_SU;

   # find an SU from $sus_list that matches RPM name given by $rpm_pkg
   foreach my $SU_item (@{$sus_list})
   {
      @fields_SU = ();
      if (!extract_RPM_name($self, $SU_item, \@fields_SU))
      {
         baselog($LOG_ERROR, "Cannot extract RPM name from SU $SU_item\n", $TRUE);
      }
      elsif (defined($fields_SU[0]) and defined($fields_SU[1]) and defined($fields_SU[2]) and defined($fields_SU[3]))
      {

         # compose rpm_name base on $SU_item without MPL issue and extension
         $rpm_name_for_SU = $fields_SU[0] . "-" . $fields_SU[1] . "-" . $fields_SU[2] . "." . $fields_SU[3];
         if ($rpm_pkg eq $rpm_name_for_SU)
         {

            # compose the SU name associated to the patch
            # get pid by filename
            my $pid = $self->{'DB'}->get_patch_id_by_name($SU_item);
            if (not defined $pid)
            {
               baselog($LOG_ERROR, "Cannot get pid for $SU_item\n.", $TRUE);
            }
            else
            {

               # find patch object
               my $p = patchutil::patch::new($self->{'DB'}->get_patch_params($pid));
               if (!defined($p))
               {
                  baselog($LOG_ERROR, "Error getting patch object for : $SU_item\n", $TRUE);
               }
               elsif ($self->{'DB'}->is_patch_loaded($p->{'PATCHNAME'}))  # check if SU is loaded in DB
               {
                  baselog($LOG_DEBUG, "SU $SU_item is loaded in the system.");
                  $retval = $TRUE;
               }
               else
               {
                  baselog($LOG_ERROR, "The patched SU $SU_item is not loaded in the system\n", $TRUE);
               }
               last;
            } ## end else [ if (not defined $pid)
         } ## end if ($rpm_pkg eq $rpm_name_for_SU...
      } ## end elsif (defined($fields_SU...
   }  # end foreach

   return $retval;
} ## end sub validate_patch_against_SU($$$)
######################################################################
## @fn sub validate_SU($self, $SU_file)
# @brief Validates that an RPM with same name that the RPM name from
# @      SU header is installed on the system.
# @details 1. Extract SU fields from SU file name
# @        2. Get RPM name from SU header given by RPM_NAME tag
# @        3. Extract RPM fields from RPM_NAME
# @        4. Compare SU and RPM names extracted. If they match, check if RPM is
# @        5. installed in the system.
# @param $self - reference to the object
# @param $SU_file - reference to the SU file
# @return $OK    - RPM name associated to SU is installed in the system
# @return $FAULT - RPM name associated to SU in not installed in the system
# @return $ERROR - Error determined
######################################################################
sub validate_SU($$)
{
   my ($self, $SU_file) = @_;
   my @fields_SU  = ();
   my @fields_RPM = ();
   my $rpm_name;
   my $retval = $ERROR;
   my $ret;

   if (!extract_RPM_name($self, $SU_file, \@fields_SU))
   {
      baselog($LOG_ERROR, "Cannot obtain SU name from SU file $SU_file\n", $TRUE);
   }
   elsif (defined($fields_SU[0]))  # SU name extracted
   {

      # this function extract the RPM_NAME tag from SU header
      if (!get_RPM_name_from_header($self, "$SPLOAD_DIR/tmp", \$rpm_name))
      {
         baselog($LOG_ERROR, "Cannot obtain RPM name tag from SU header for SU $SU_file.\n", $TRUE);
      }
      elsif (defined($rpm_name))
      {
         if (!extract_RPM_name($self, $rpm_name, \@fields_RPM))
         {
            baselog($LOG_ERROR, "Cannot obtain RPM name without version for SU $SU_file.\n", $TRUE);
         }
         elsif (defined($fields_RPM[0]))  # RPM name extracted
         {

            # compare SU name extracted against RPM name extracted
            if ($fields_SU[0] eq $fields_RPM[0])
            {
               baselog($LOG_DEBUG, "RPM name extracted for SU $SU_file is $fields_SU[0]");
               my $rpm_installed = undef;
               do_system([$RPM, "-q", $fields_RPM[0]], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$ret});
               if ((not $ret) && (defined(@$rpm_installed)))
               {
                  my $rpm_installed_str = join(' ', @$rpm_installed);

                  # Check if the RPM name, version & release is installed on the system.
                  # - we are looking for the exact match
                  if ( $rpm_installed_str =~ "$fields_RPM[0]-$fields_RPM[1]-$fields_RPM[2]" )
                  {

                     # If the RPM (name, version & release) is already installed then we will skip loading the SU
                     baselog($LOG_WARNING, "Patch $SU_file not loaded (RPM already installed on system)\n", $TRUE);
                     $retval = $FAULT;
                  } ## end if ($rpm_installed_str...
                  else
                  {

                     # RPM is installed but of different version/release than being delivered by the specific SU
                     # we will thus attempt to load/install it.
                     $retval = $OK;
                  } ## end else [ if ($rpm_installed_str...
               } ## end if ((not $ret) && (defined...
               else
               {
                  baselog($LOG_WARNING, "RPM name $fields_RPM[0] not found on system (application probably not installed).\n - SU $SU_file not installed.\n", $TRUE);
                  $retval = $FAULT;
               }
            } ## end if ($fields_SU[0] eq $fields_RPM...
            else
            {
               baselog($LOG_ERROR, "The RPM name $fields_RPM[0] given inside the SU $SU_file doesn't match the SU name $fields_SU[0].\n", $TRUE);
            }
         } ## end elsif (defined($fields_RPM...
         else
         {
            baselog($LOG_ERROR, "RPM name extracted is not defined for SU $SU_file.\n", $TRUE);
         }
      } ## end elsif (defined($rpm_name)...
      else
      {
         baselog($LOG_ERROR, "The RPM name inside the SU header is undefined.\n", $TRUE);
      }
   } ## end elsif (defined($fields_SU...
   else
   {
      baselog($LOG_ERROR, "Undefined SU name.\n", $TRUE);
   }
   return $retval;
} ## end sub validate_SU($$)
################################################################
## @fn sub validate_patch($self, $patch_file, $su_list)
# @brief Validates that an RPM associated to patch is already
# @       installed or is associated to an SU from DB
# @details 1. Get RPM name from patch header given by RPM_NAME tag
# @        2. Extract fields from RPM_name
# @        3. Check if an RPM is installed with same RPM name obtained
# @        from patch header.
# @        4. If not, RPM is installed with same name, return FALSE.
# @        5. If RPM is installed with same name, compare version
# @        between RPM installed and RPM from patch header. If they match,
# @        patch is against RPM installed.
# @        6. If versioning issue doesn't match, validate if patch
# @        is against to an SU from database. If validation returns $TRUE,
# @        patch is against an SU.
# @param $self - reference to the object
# @param $patch_file - reference to the patch file
# @param $su_list - reference to the list of Sus
# @return $OK    - RPM associated to patch is installed in the system or
#                is associated to an SU in the database.
# @return $FAULT - RPM associated to patch in not installed in the system
#                 and is not associated to an SU in the database.
# @return $ERROR - An ERROR was determined
######################################################################
sub validate_patch($$$)
{
   my ($self, $patch_file, $su_list) = @_;
   my $rpm_name;
   my @fields_RPM = ();
   my $rpm_installed;
   my $retval = $ERROR;
   my $ret;

   # get RPM_NAME tag from patch header
   if (get_RPM_name_from_header($self, "$SPLOAD_DIR/tmp", \$rpm_name))
   {
      if (!extract_RPM_name($self, $rpm_name, \@fields_RPM))
      {
         baselog($LOG_ERROR, "Cannot obtain RPM name and version for $patch_file\n", $TRUE);
      }
      elsif (defined($fields_RPM[0]))  # if rpm name extracted
      {
         baselog($LOG_INFO, "RPM name extracted is $fields_RPM[0] and full rpm_name is $rpm_name for patch $patch_file.");

         do_system([$RPM, '-q', $fields_RPM[0]], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$ret});
         if ((not $ret) && (defined(@$rpm_installed)))
         {
            $rpm_installed = @$rpm_installed[0];
            chomp($rpm_installed);

            # if CS RPM then do not check last part of file name
            if ($rpm_installed =~ s/^(nortel-cs1000-cs-\d\.\d{2}\.\w\.)\d{3}(-\d{2})/${1}000${2}/)
            {
               baselog($LOG_INFO, "CS RPM. Do not check last version part");
               $fields_RPM[1] =~ s/(\d\.\d{2}\.\w\.)\d{3}/${1}000/;
            }

            # compose rpm name with version and release only
            my $rpm_name_with_ver_rls = $fields_RPM[0] . '-' . $fields_RPM[1] . '-' . $fields_RPM[2];

            # if rpm installed is equal to rpm from patch header, the patch is against RPM installed.
            # Otherwise, validate if patch is against SU to be installed
            if ($rpm_name_with_ver_rls ne $rpm_installed)
            {
               my @sus = @$su_list;
               if (!validate_patch_against_SU($self, $rpm_name, \@sus))
               {
                  baselog($LOG_ERROR, "Cannot find RPM $rpm_name associated to patch $patch_file.\n", $TRUE);
               }
               else
               {
                  baselog($LOG_DEBUG, "Patch $patch_file is against SU to be installed.");
                  $retval = $OK;
               }
            } ## end if ($rpm_name_with_ver_rls...
            else
            {
               baselog($LOG_DEBUG, "Patch $patch_file is against installed RPM $rpm_name.");
               $retval = $OK;
            }
         } ## end if ((not $ret) && (defined...
         else
         {
            baselog($LOG_WARNING, "RPM name $rpm_name not found on system (application probably not installed).\n - Patch $patch_file not installed.\n", $TRUE);
            $retval = $FAULT;
         }
      } ## end elsif (defined($fields_RPM...
      else
      {
         baselog($LOG_ERROR, "Failed to obtain rpm name and version for patch $patch_file.\n", $TRUE);
      }
   } ## end if (get_RPM_name_from_header...
   else
   {
      baselog($LOG_ERROR, "Failed to obtain rpm_name from patch header for $patch_file.\n", $TRUE);
   }
   return $retval;
} ## end sub validate_patch($$$)
#######################################################################
## @fn sub update_file($self, $file_name, $msg)
# @brief Update $file_name with message received as parameter $msgm
# @details 1. If $file_name exists, appends $msg to the existing file
# @        2. If $file_name doesn't exist, create and update file with $msg
# @param $self - reference to the object
# @param $file_name - reference to a file name
# @param $msg - reference to the message to be saved in the file
# @return TRUE - File update successful
# @return FALSE - File update failed
######################################################################
sub update_file($$$)
{
   my ($self, $file_name, $msg) = @_;

   my $mode;
   if (-e $file_name)
   {
      $mode = ">>";
   }
   else  #(!-e $file_name)
   {
      $mode = ">";
   }

   # Open log file for append
   my $file;
   unless (open($file, $mode, $file_name))
   {
      baselog($LOG_ERROR, "Can not open " . $file_name . " file for writing\n", $TRUE);
      return $FALSE;
   }

   # Write message to the file
   print $file "$msg\n";

   # Close file
   close($file);
   return $TRUE;
} ## end sub update_file($$$)
#######################################################################
## @fn sub pload_member_required($self, $member_req)
# @brief pload patch or SU pass as $member_req parameter
# @details 1. Check if patch or SU is already loaded. If that is the case,
# @        return $TRUE.
# @        2, Move file to PATCHDIR directory.
# @        3. It tries to pload Patch or SU in DB
# @        4. If it succeed, update the $SPLOAD_FILE with the
# @        pload a success message for patch or SU.
# @param $self - reference to the object
# @param $member_req - reference to the name of SU or patch to be loaded
# @return TRUE - patch or SU loaded successfuly or already loaded
#              - If PI FRU Patch fails min_rpm_ver validation we skip
#                loading the patch but still return true to allow the
#                spload to continue.
# @return FALSE - fail to load a patch or SU in the DB
######################################################################
sub pload_member_required($$)
{
   my ($self, $member_req) = @_;
   my $retval = $FALSE;

   baselog($LOG_DEBUG, "Inside pload_member_required for $member_req.\n");
   my $file = $SPLOAD_DIR . "/" . $member_req;

   #if patch or SU is already loaded return $TRUE
   my $p = patchutil::patch::new({'file' => $file});
   if (!defined($p))
   {
      baselog($LOG_ERROR, "Error getting object for $file.\n", $TRUE);
      return $FALSE;
   }
   elsif ($self->{'DB'}->is_patch_loaded($p->{'PATCHNAME'}))  # if SU or patch is loaded in DB return $TRUE
   {
      baselog($LOG_DEBUG, "$member_req is already loaded.\n");
      return $TRUE;
   }

   if (move_member_to_patchdir($self, $member_req))
   {
      baselog($LOG_INFO, "Trying to pload $member_req.");

      # Validate if this is a PI Patch and whether or not any specified min_rpm_ver
      # is installed. If not we skip loading this patch.
      if (not $self->validate_min_rpm_ver($p))
      {

         # We will not flag this as an error in the SP context, but will at least flag a warning.
         baselog($LOG_WARNING, "- Patch $p->{'PATCHNAME'} is not loaded\n", $TRUE);
         return $TRUE;
      } ## end if (not $self->validate_min_rpm_ver...

      # Continue loading the patch
      #
      if (!$self->pload_file($member_req, undef))
      {
         baselog($LOG_ERROR, "Failed to pload file $member_req.\n", $TRUE);
      }
      else
      {

         #update SPLOAD_FILE with success pload
         my $success_msg = "$member_req loaded successfully.";
         if (!update_file($self, $SPLOAD_FILE, $success_msg))
         {
            baselog($LOG_ERROR, "Failed to udpate $SPLOAD_FILE for $member_req.\n", $TRUE);
         }
         else
         {
            $retval = $TRUE;
         }
      } ## end else [ if (!$self->pload_file...
   } ## end if (move_member_to_patchdir...
   return $retval;
} ## end sub pload_member_required($$)
#######################################################################
## @fn sub extract_header_file($self, $file, $dir)
# @brief Extract the header file passed as parameter under
# @      $SPLOAD_DIR/tmp
# @details 1. Creates $tmp_dir
# @        2. Checks if $file received as parameter exists
# @        3. If it does, copy $file to $tmp_dir and untar header file
# @           for SU or patch under same directory
# @param $self - reference to the object
# @param $file - reference to the patch or SU file
# @param $dir - reference to directory where the patch or SU exists
# @return OK    - extracted header file successfully
# @return FAULT - SU/Patch not in SP bundle
# @return ERROR - cannot extract header file
######################################################################
sub extract_header_file($$$)
{
   my ($self, $file, $dir) = @_;
   my $tmp_dir = $SPLOAD_DIR . "/tmp";

   # create $tmp_dir to untar header file used for validation only
   if (!spec_dir_prep($tmp_dir))
   {
      baselog($LOG_ERROR, "Cannot create temporarly dir for $file.\n", $TRUE);
      return $ERROR;
   }

   # if SU or patch required exists inside the bundle
   if (-e "$dir/$file")
   {

      #cp $tmp_file to patch directory
      if (!do_system([$CP, "$dir/$file", $tmp_dir]))  # check what happens if already exists
      {

         # needs to be debug
         baselog($LOG_ERROR, "Cannot copy $file to temporary directory.\n", $TRUE);
         return $ERROR;
      } ## end if (!do_system([$CP, "$dir/$file"...
      else
      {

         # Untar existed header file

         if (do_system([$TAR, 'tfz', "$tmp_dir/$file", $PATCHINFO_FILE]) != 0)
         {
            if (do_system([$TAR, 'xvfz', "$tmp_dir/$file", '-C', $tmp_dir, $PATCHINFO_FILE]) == 0)
            {
               baselog($LOG_ERROR, "Cannot extract header file for $file.\n", $TRUE);
               return $ERROR;
            }
         } ## end if (do_system([$TAR, 'tfz'...
         else
         {
            baselog($LOG_ERROR, "$file doesn't contain header file.\n", $TRUE);
            return $ERROR;
         }
      } ## end else [ if (!do_system([$CP, "$dir/$file"...
   } ## end if (-e "$dir/$file")
   else
   {
      baselog($LOG_WARNING, "Patch $file not loaded (May not be relevant to target configuration)\n", $TRUE);
      baselog($LOG_WARNING, "File $file doesn't exist inside the SP bundle.\n");
      return $FAULT;
   } ## end else [ if (-e "$dir/$file")

   # extracted header file successfully
   return $OK;
} ## end sub extract_header_file($$$)

######################################################################
## @fn sub pload_all_members($self, $type, $req_members, $sus_list)
# @brief Validates and tries to pload each member listed in the SP header
# @details For each member received inside the passed @req_members parameter,
# @        validates member base on $type and tries to load it in the system
# @        if is not already loaded. If validation fails, call remove_all_patches to
# @        "spout" all the patches and SUS already loaded. Inside spout, it will delete
# @         SPLOAD dir and cleanup from PATCHDIR the patches and SUS
# @         listed inside cplist.txt file.
# @param $self - reference to the object
# @param $type - type of members trying to be loaded (SU" or "PATCH")
# @param $req_members - reference to the list of SUS or patches from SP header
# @param $sus_list - reference to the list of SUS needed for validation of patches
# @return TRUE - All SUS have been loaded successfully.
# @return FALSE - At least one SU cannot be loaded successfully.
######################################################################
sub pload_all_members($$$;$)
{
   my ($self, $type, $req_members, $sus_list) = @_;
   my $is_valid_member;
   my $retval = $TRUE;
   my $file_to_rm;

   baselog($LOG_DEBUG, "Calling pload_all_members for type = $type.\n");

   # pload every SU listed on the SP header.xml
   foreach my $member_on_SP_header (@$req_members)
   {
      my $err = extract_header_file($self, $member_on_SP_header, $SPLOAD_DIR);
      if ($err == $FAULT)
      {

         # continue loading next member
         # - in this context SU/Patch are not in the body of the SP and thus
         #   this could be a mini-bundle (will need to look at using the MIRCC tag.)
         next;
      } ## end if ($err == $FAULT)
      elsif ($err == $ERROR)
      {

         # continue loading next member (SU/Patch in error)
         $retval = $FALSE;
         next;
      } ## end elsif ($err == $ERROR)

      if ($type eq 'SU')
      {

         # check if an RPM for the SU is already installed
         $is_valid_member = validate_SU($self, $member_on_SP_header);
      } ## end if ($type eq 'SU')
      elsif ($type eq 'PATCH')
      {

         # check if patch belongs to an installed RPM or an SU in the bundle
         my @sus = @$sus_list;
         $is_valid_member = validate_patch($self, $member_on_SP_header, \@sus);
      } ## end elsif ($type eq 'PATCH')

      baselog($LOG_DEBUG, "Validation for $member_on_SP_header = $is_valid_member.\n");
      if ($is_valid_member == $OK)
      {
         baselog($LOG_DEBUG, "Validation successful for $member_on_SP_header.\n");
         if (!pload_member_required($self, $member_on_SP_header))  # if cannot load member
         {

            # remove patch that had issue from PATCHDIR
            $file_to_rm = $PATCHDIR . "/" . $member_on_SP_header;
            if (-e $file_to_rm)
            {
               if (!do_system([$RM, '-rf', $file_to_rm]))
               {
                  baselog($LOG_ERROR, "Cannot remove $file_to_rm.\n", $TRUE);
               }
            } ## end if (-e $file_to_rm)
            $retval = $FALSE;                                      # continue loading next member
            next;
         } ## end if (!pload_member_required...
      } ## end if ($is_valid_member ==...
      else
      {

         # Flag any error otherwise continue loading next patch
         # - that is we will ignore $FAULTs since they indicate, in this context,
         #   that a specific RPM for a SU/Patch in not on the target
         if ($is_valid_member == $ERROR)
         {
            $retval = $FALSE;
         }
      } ## end else [ if ($is_valid_member ==...
   } ## end foreach my $member_on_SP_header...

   @{$req_members} = ();
   return $retval;
} ## end sub pload_all_members($$$;$)
############################################################
## @fn sub validate_SUs_dup_on_SP($self, @patches_list)
# @brief Check if there are duplicated SUs on the SP
# @details Check if there is more than one SU in the SP
# @        with the same RPM_NAME.
# @param   $self  - reference to object.
# @param   @patches_list - list of SUs and patches from SP
# @return OK - didn't find duplications
# @return ERROR - found duplication or an error during search
#############################################################
sub validate_SUs_dup_on_SP($@)
{
   my ($self, @patches_list) = @_;
   my $ret_val  = $OK;
   my @sp_files = @patches_list;
   my @fields_rpm_a;
   my @fields_rpm_b;

   # the patches_list contains SU first and after patches
   foreach my $patch (@patches_list)
   {
      @fields_rpm_a = ();
      if (!extract_RPM_name($self, $patch->{'RPM_NAME'}, \@fields_rpm_a))
      {
         baselog($LOG_ERROR, "Cannot extract name from rpm $patch->{'RPM_NAME'} \n", $TRUE);
         $ret_val = $ERROR;
         last;
      } ## end if (!extract_RPM_name(...
      if (!defined($fields_rpm_a[0]))
      {
         baselog($LOG_ERROR, "RPM name undefined for SU $patch->{'FILENAME'}\n", $TRUE);
         $ret_val = $ERROR;
         last;
      } ## end if (!defined($fields_rpm_a...
      if ($patch->{'TYPE'} eq "SU")
      {
         my $sus_found = 0;
         foreach my $p (@sp_files)
         {
            if ($p->{'TYPE'} eq "SU")
            {
               if (($sus_found) > 1)
               {
                  last;
               }
               @fields_rpm_b = ();
               if (!extract_RPM_name($self, $p->{'RPM_NAME'}, \@fields_rpm_b))
               {
                  baselog($LOG_ERROR, "Cannot extract name from rpm $p->{'RPM_NAME'} \n", $TRUE);
                  $ret_val = $ERROR;
                  last;
               } ## end if (!extract_RPM_name(...
               if (!defined($fields_rpm_b[0]))
               {
                  baselog($LOG_ERROR, "RPM name undefined for SU $p->{'FILENAME'}\n", $TRUE);
                  $ret_val = $ERROR;
                  last;
               } ## end if (!defined($fields_rpm_b...
               $sus_found++ if ($fields_rpm_a[0] eq $fields_rpm_b[0]);

            } ## end if ($p->{'TYPE'} eq "SU"...
            else
            {
               last;
            }
         } ## end foreach my $p (@sp_files)
         if ($sus_found > 1)
         {
            baselog($LOG_ERROR, "Invalid SP. There is more than one SU in the SP for the same RPM $patch->{'RPM_NAME'}\n", $TRUE);
            $ret_val = $ERROR;
            last;
         } ## end if ($sus_found > 1)
      } ## end if ($patch->{'TYPE'} eq...
      else
      {
         last;
      }
   } ## end foreach my $patch (@patches_list...
   return $ret_val;
} ## end sub validate_SUs_dup_on_SP($@)
############################################################
## @fn sub prepare_dirs_for_spins()
# @brief Prepare SPINS directory and others to handle spins command
# @details 1. If SP/REVERT dir exists, delete dir content
# @        2. If REVERT dir exists, remove content
# @        3. If SPINS dir exists, move SP/REVERT directory content
# @        to SPINS dir. If SPINS doesn't exist, create SPINS dir
# @        4. If SPLOAD dir exists, move SPLOAD directory content
# @        to SPINS_dir and delete SPLOAD directory.
# @return TRUE - directory preparation for spins is successful
# @return FALSE - directory preparation for spins fails.
#############################################################
sub prepare_dirs_for_spins()
{

   # if SP revert dir exists, delete directory
   if (-d $SP_REVERT_DIR)
   {

      # remove all files inside sp revert dir
      if (!do_system([$RM, '-rf', $SP_REVERT_DIR]))
      {
         baselog($LOG_ERROR, "Cannot remove files from $SP_REVERT_DIR directory\n", $TRUE);
         return $FALSE;
      }
   } ## end if (-d $SP_REVERT_DIR)

   # if revert dir exists, delete files inside directory
   if (-d $REVERT_DIR)
   {

      # remove all files inside revert dir
      if (system("$RM -f $REVERT_DIR/*"))
      {
         baselog($LOG_ERROR, "Cannot remove files from $REVERT_DIR directory\n", $TRUE);
         return $FALSE;
      }
   } ## end if (-d $REVERT_DIR)

   #if SPINS dir exists, move SPINS_DIR to SP_REVERT_DIR
   if (-d $SPINS_DIR)
   {

      # move SPINS dir content to SP revert dir
      if (!do_system([$MV, '-f', $SPINS_DIR, $SP_REVERT_DIR]))
      {
         baselog($LOG_ERROR, "Cannot move files from $SPINS_DIR directory to $SP_REVERT_DIR\n", $TRUE);
         return $FALSE;
      }
   } ## end if (-d $SPINS_DIR)

   if (-d $SPLOAD_DIR)
   {

      # move SPLOAD dir content to SPINS dir
      if (!do_system([$MV, '-f', $SPLOAD_DIR, $SPINS_DIR]))
      {
         baselog($LOG_ERROR, "Cannot move files from $SPLOAD_DIR directory to $SPINS_DIR\n", $TRUE);
         return $FALSE;
      }

      # remove directory SPLOAD
      if (!do_system([$RM, '-rf', $SPLOAD_DIR]))
      {
         baselog($LOG_ERROR, "Cannot delete directory $SPLOAD_DIR\n", $TRUE);
         return $FALSE;
      }
   } ## end if (-d $SPLOAD_DIR)
   return $TRUE;
} ## end sub prepare_dirs_for_spins()
############################################################
## @fn sub revert_dirs_for_spins()
# @brief Restore SPLOAD directory and previous SPINS contents.
# @details 1. If SPLOAD dir doesn't exist, create dir.
# @        2. Otherwise, move SPINS dir content to SPLOAD dir.
# @        3. If SP/REVERT directory exists, move content to
# @           SPINS directory.
# @        4. Remove SPINS directory.
# @return TRUE - revert directories successful
# @return FALSE - revert directories fails.
#############################################################
sub revert_dirs_for_spins()
{
   if (!-d $SPLOAD_DIR)
   {

      # create SPLOAD directory
      if (!spec_dir_prep($SPLOAD_DIR))
      {
         return $FALSE;
      }
   } ## end if (!-d $SPLOAD_DIR)

   #if SPINS dir exists, move SPINS_DIR to SPLOAD_DIR
   if (-d $SPINS_DIR)
   {

      # move SPINS dir content to SPLOAD dir
      if (!do_system([$MV, '-f', $SPINS_DIR, $SPLOAD_DIR]))
      {
         baselog($LOG_ERROR, "Cannot move content from $SPINS_DIR directory to $SPLOAD_DIR\n", $TRUE);
         return $FALSE;
      }
      if (-d $SP_REVERT_DIR)
      {

         # move SP_REVERT_DIR dir content to SPINS dir
         if (!do_system([$MV, '-f', $SP_REVERT_DIR, $SPINS_DIR]))
         {
            baselog($LOG_ERROR, "Cannot move content from $SP_REVERT_DIR directory to $SPINS_DIR\n", $TRUE);
            return $FALSE;
         }
      } ## end if (-d $SP_REVERT_DIR)

      # remove directory SPINS
      if (!do_system([$RM, '-rf', $SPINS_DIR]))
      {
         baselog($LOG_ERROR, "Cannot delete directory $SPINS_DIR\n", $TRUE);
         return $FALSE;
      }
   } ## end if (-d $SPINS_DIR)
   return $TRUE;
} ## end sub revert_dirs_for_spins()
############################################################
## @fn sub get_object_list($self, $files_list)
# @brief Merge all sus and patches passed as parameter in
# @      required_files list and create a list of patches object
# @      based on merged list.
# @details 1. Merge list of sus and patches in required_files list
# @        2. Create list with patch objects base on the
# @        required_files list.
# @param   $self  - reference to object.
# @param   $files_list - reference to list of SUs and patches.
# @param   $failed_patches - reference to list of SUs and patches
# @        filenames which failed to be loaded.
# @return @obj_list - list of patch objects.
#############################################################
sub get_object_list($$$)
{
   my ($self, $files_list, $failed_patches) = @_;
   my @obj_list    = ();
   my @no_pid_list = ();

   foreach my $filename (@$files_list)
   {

      # get pid by filename
      my $pid = $self->{'DB'}->get_patch_id_by_name($filename);
      if (not defined $pid)
      {
         push(@no_pid_list, $filename);
         next;
      }

      # find patch object
      my $p = patchutil::patch::new($self->{'DB'}->get_patch_params($pid));
      if (!defined($p))
      {
         baselog($LOG_ERROR, "Error getting patch object for : $filename\n", $TRUE);
         next;
      }
      push(@obj_list, $p);
   } ## end foreach my $filename (@$files_list...
   @{$failed_patches} = @no_pid_list;

   return @obj_list;
} ## end sub get_object_list($$$)
#############################################################
## @fn sub find_member_in_SP_header($self, $patch, @required_members)
# @brief Search for a patch or su inside the list of required patches and sus
# @      obtained from the SP header
# @details Try to find a patch or su inside $required_members
# @        which matches the FILENAME attribute for the passed patch object.
# @param   $self  - reference to object.
# @param   $patch  - reference to patch object.
# @param   @required_members  - reference to list of patches and sus
# @return  $index  - location of member found
# @        undef   - otherwise
#############################################################
sub find_member_in_SP_header($$@)
{
   my ($self, $patch, @required_members) = @_;
   my $index = undef;
   if (scalar(@required_members) > 0)
   {
      for (my $i = 0 ; $i < scalar(@required_members) ; $i++)
      {
         if (($patch->{'FILENAME'}) eq ($required_members[$i]->{'FILENAME'}))
         {
            $index = $i;
            last;
         }
      } ## end for (my $i = 0 ; $i < scalar...

   } ## end if (scalar(@required_members...
   return $index;
} ## end sub find_member_in_SP_header($$@)
#############################################################
## @fn sub find_app_name($list, $app_name)
# @brief Find an app name from app list.
# @details Based on parameters passed, find an app name
#          in an app list and return its position in list.
# @param   $list  - reference to list of applications
# @param   $app_name  - reference to app name
# @return  $ret_pos  - position on $list where app
# @                    name was found
# @        undef     - Otherwise
#############################################################
sub find_app_name($$)
{
   my ($list, $app_name) = @_;
   my $ret_pos = undef;
   if (!is_array($list))
   {
      baselog($LOG_ERROR, "Not a array reference passed.\n", $ERROR);
   }
   else
   {
      for (my $pos = 0 ; $pos < scalar(@$list) ; $pos++)
      {
         if ($$list[$pos] eq $app_name)
         {
            ${$ret_pos} = $pos;
            last;
         }
      } ## end for (my $pos = 0 ; $pos...
   } ## end else [ if (!is_array($list))
   return $ret_pos;
} ## end sub find_app_name($$)
#############################################################
## @fn sub collect_apps($patch, $list, $tag_name, $in_sp_flag)
# @brief Obtain list of applications to stop or to start based
#        on tag_name parameter.
# @details 1. Obtain the application list to start or stop
# @           from patch object.
# @        2. Base on application list, create sorted list of
# @           applications to be stopped or started.
# @           - Check if application exists in sorted app list
# @           - If doesn't exists, get previous app name
# @             from the app list and find the position
# @             on the sorted list. Insert the new app on the sorted
# @             list, after that position.
# @param   $patch  - reference to patch object.
# @param   $list  - reference to a list of applications
# @param   $tag_name  - stop or start tag name
# @param   $in_sp_flag - flag to indicate if patch is in the SP header
# @        or not.
# @return $TRUE - process apps to start or stop for patch
#############################################################
sub collect_apps($$$$)
{
   my ($patch, $ref_list, $tag_name, $in_sp_flag) = @_;
   my $prev_app_pos;
   my $prev_app;
   my @apps = ();
   my @list = @$ref_list;
   if ($tag_name eq "APP_START")
   {

      #if patch is in the SP header
      if ($in_sp_flag)
      {
         if ($patch->{'STATUS'} != $IN_SERV)
         {
            @apps = @{$patch->{'APP_START'}};
         }
      } ## end if ($in_sp_flag)
      else
      {
         @apps = @{$patch->{'APP_START'}};
      }
   } ## end if ($tag_name eq "APP_START"...
   elsif ($tag_name eq "APP_STOP")
   {

      #if patch is in the SP header
      if ($in_sp_flag)
      {
         if ($patch->{'STATUS'} != $IN_SERV)
         {
            @apps = @{$patch->{'APP_STOP'}};
         }
      } ## end if ($in_sp_flag)
      else
      {
         @apps = @{$patch->{'APP_STOP'}};
      }
   } ## end elsif ($tag_name eq "APP_STOP"...
   if (scalar(@apps))
   {

      # reverse the patch start/stop list
      # - we want to parse the list in reverse
      @apps = reverse(@apps);

      for (my $i = 0 ; $i < scalar(@apps) ; $i++)
      {

         # if app is not found then add it to the ref list supplied
         my $pos = find_app_name(\@list, $apps[$i]);
         if (not defined $pos)
         {
            if (scalar(@list) > 0)
            {
               if ($i == 0)
               {

                  # first time through the list add any new app to the end of
                  # the ref list
                  push @list, $apps[$i];
               } ## end if ($i == 0)
               else
               {

                  # add new app before the prev app in the ref list
                  $prev_app = $apps[$i - 1];
                  $prev_app_pos = find_app_name(\@list, $prev_app);
                  if (defined $prev_app_pos)
                  {
                     splice(@list, ${$prev_app_pos}, 0, $apps[$i]);
                  }
               } ## end else [ if ($i == 0)
            } ## end if (scalar(@list) > 0)
            else
            {

               # add first element to the list
               push @list, $apps[$i];
            } ## end else [ if (scalar(@list) > 0)
         } ## end if (not defined $pos)
      } ## end for (my $i = 0 ; $i < scalar...
   } ## end if (scalar(@apps))
   @{$ref_list} = @list;
   return $TRUE;
} ## end sub collect_apps($$$$)
#############################################################
## @fn sub update_lists($patch, $member_list, $start_list, $stop_lists, $flag_in_header)
# @brief Update the member_list, start and stop lists, base
#        on patch object passed as parameter.
# @details 1.Add to patch object to the member list.
# @        2.Update list of apps that need to be stopped before installing
# @        patch based on patch object.
# @        3.Update list of apps that need to be started after installing
# @        patch based on patch object.
# @param   $patch  - reference to patch object.
# @param   $member_list  - reference to list of patches or sus
# @param   $start_list  - reference to list of application names.
# @param   $stop_list  - reference to list of application names.
# @param   $flag_in_header - flag to indicate if patch is in the SP header or not.
# @return $TRUE - Updated lists sucessfully
# @return $FALSE - Failed to update lists.
#############################################################
sub update_lists($$$$$)
{
   my ($patch, $member_list, $start_list_ref, $stop_list_ref, $flag_in_header) = @_;
   my @start_list = @$start_list_ref;
   my @stop_list  = @$stop_list_ref;

   push @$member_list, $patch;  # add object to member list
   if (!collect_apps($patch, \@start_list, "APP_START", $flag_in_header))
   {
      baselog($LOG_ERROR, "Cannot update start list for $patch->{'FILENAME'}\n", $TRUE);
      return $FALSE;
   }
   else
   {
      if (!collect_apps($patch, \@stop_list, "APP_STOP", $flag_in_header))
      {
         baselog($LOG_ERROR, "Cannot update stop list for $patch->{'FILENAME'}\n", $TRUE);
         return $FALSE;
      }
   } ## end else [ if (!collect_apps($patch...
   @$start_list_ref = @start_list;
   @$stop_list_ref  = @stop_list;
   return $TRUE;
} ## end sub update_lists($$$$$)
#########################################################################################
## @fn sub collect_common_info($self, $files_in_db, $required_files, $obsolete_files, $reboot_flag, $status_in_out_srv_reboot)
# @brief Process SP header sus and patches and create a list of
#        arrays with common data.
# @details 1.For all the patches and sus in database, if they must be
# @          in service, update the following lists:
# @          -  $list_of_arrays{'stop'} = [@stop];
# @          -  $list_of_arrays{'patches_in_srv'} = [@patch_in_srv];
# @          -  $list_of_arrays{'sus_in_srv'} = [@su_in_srv];
# @          -  $list_of_arrays{'start'} = [@start];
# @        2.For all the patches and sus in database, if they must be
# @          out of service, update the following lists:
# @          -  $list_of_arrays{'stop'} = [@stop];
# @          -  $list_of_arrays{'patches_out_srv'} = [@patch_out_srv];
# @          -  $list_of_arrays{'sus_out_srv'} = [@su_out_srv];
# @          -  $list_of_arrays{'start'} = [@start];
# @param   $self  - reference to object.
# @param   $files_in_db  - reference to list of patches and sus from db
# @param   $required_files  - reference to list of patches and sus required
# @                           to be installed.
# @param   $obsolete_files - reference to list of obsolete SUs and patches from SP header.xml
# @param   $reboot_flag - reference to the reboot flag.
# @param   $status_in_out_srv_reboot - reference to flag for patches or SUs with status
#          $IN_SERV_REBOOT or $OUT_SRV_REBOOT
# @return %list_of_arrays - list of arrays
#########################################################################################
sub collect_common_info($$$$$$)
{
   my ($self, $files_in_db, $required_files, $obsolete_files, $reboot_flag, $status_in_out_srv_reboot) = @_;
   my (@patch_in_srv, @patch_out_db, @su_in_srv, @su_out_db, @start, @stop) = ((), (), (), (), (), ());
   my %list_of_arrays = ();
   my $upd_ok;
   my $obs_index;

   ${$reboot_flag}              = $FALSE;
   ${$status_in_out_srv_reboot} = $FALSE;

   foreach my $p (@$files_in_db)
   {
      if (($p->{'STATUS'} == $IN_SERV_REBOOT) || ($p->{'STATUS'} == $OUT_SERV_REBOOT))
      {
         ${$status_in_out_srv_reboot} = $TRUE;
         last;
      }

      if ($self->{'SP_type'} eq 'DELTA_SP')
      {
         $obs_index = pos_in_array($obsolete_files, $p->{'FILENAME'});
      }

      # if SU/patch is found in the SP header
      my $index = $self->find_member_in_SP_header($p, @$required_files);
      if (defined $index)
      {

         # update in service lists
         if (defined($p->{'TYPE'}))
         {
            $upd_ok = $TRUE;
            if ($p->{'TYPE'} eq "PATCH")
            {
               if (!update_lists($p, \@patch_in_srv, \@start, \@stop, $TRUE))
               {
                  baselog($LOG_ERROR, "Failed to process patch $p->{'FILENAME'} present on SP header\n", $TRUE);
                  $upd_ok = $FALSE;
               }
            } ## end if ($p->{'TYPE'} eq "PATCH"...
            elsif ($p->{'TYPE'} eq "SU")
            {
               if (!update_lists($p, \@su_in_srv, \@start, \@stop, $TRUE))
               {
                  baselog($LOG_ERROR, "Failed to process su $p->{'FILENAME'} present on SP header\n", $TRUE);
                  $upd_ok = $FALSE;
               }
            } ## end elsif ($p->{'TYPE'} eq "SU"...
            else
            {
               baselog($LOG_ERROR, "Failed to process $p->{'FILENAME'} present on SP header. Type is different than SU or PATCH\n", $TRUE);
               $upd_ok = $FALSE;
            }
            if ((!${$reboot_flag}) && ($upd_ok))
            {
               if (   defined($p->{REBOOT})
                   && ($p->{REBOOT} =~ m/^yes$/i)
                   && ($p->{'STATUS'} != $IN_SERV))
               {
                  ${$reboot_flag} = $TRUE;
               } ## end if (defined($p->{REBOOT...
            } ## end if ((!${$reboot_flag})...
         } ## end if (defined($p->{'TYPE'...
         else
         {
            baselog($LOG_ERROR, "Failed to process $p->{'FILENAME'} present on SP header. Undefined type\n", $TRUE);
         }  # endif (defined($p->{TYPE})
      } ## end if (defined $index)
      elsif ((defined $obs_index) and ($obs_index == -1))  # if SU/patch is not found in SP header and is not obsolete. For DELTA SP only
      {
         baselog($LOG_DEBUG, "To leave $p->{'FILENAME'} on the system.");
         next;
      }
      elsif ($self->{'SP_type'} ne 'SEARCH_SP')            #if SU/patch is not found in SP header or is obsolete
      {

         # only update out of service lists for patches with removable flag set to 'yes'
         if ((defined($p->{'REMOVABLE'})) && ($p->{'REMOVABLE'} =~ m/^yes$/i))
         {

            # update out of service lists
            if (defined($p->{'TYPE'}))
            {
               $upd_ok = $TRUE;
               if ($p->{'TYPE'} eq "PATCH")
               {
                  if (!update_lists($p, \@patch_out_db, \@start, \@stop, $FALSE))
                  {
                     baselog($LOG_ERROR, "Failed to process patch $p->{'FILENAME'} NOT present on SP header\n", $TRUE);
                     $upd_ok = $FALSE;
                  }
               } ## end if ($p->{'TYPE'} eq "PATCH"...
               elsif ($p->{'TYPE'} eq "SU")
               {
                  if (!update_lists($p, \@su_out_db, \@start, \@stop, $FALSE))
                  {
                     baselog($LOG_ERROR, "Failed to process su $p->{'FILENAME'} NOT present on SP header\n", $TRUE);
                     $upd_ok = $FALSE;
                  }
               } ## end elsif ($p->{'TYPE'} eq "SU"...
               else
               {
                  baselog($LOG_ERROR, "Failed to process $p->{'FILENAME'} NOT present on SP header. Type is different than SU or PATCH\n", $TRUE);
                  $upd_ok = $FALSE;
               }
            } ## end if (defined($p->{'TYPE'...
            else
            {
               baselog($LOG_ERROR, "Failed to process $p->{'FILENAME'} NOT present on SP header. Undefined type\n", $TRUE);
            }
            if ((!${$reboot_flag}) && ($upd_ok))
            {
               if (   defined($p->{REBOOT})
                   && ($p->{REBOOT} =~ m/^yes$/i)
                   && ($p->{'STATUS'} != $OUT_SERV))
               {
                  ${$reboot_flag} = $TRUE;
               } ## end if (defined($p->{REBOOT...
            } ## end if ((!${$reboot_flag})...
         } ## end if ((defined($p->{'REMOVABLE'...
      } ## end elsif ($self->{'SP_type'}...
   } ## end foreach my $p (@$files_in_db...
   if (!${$status_in_out_srv_reboot})
   {

      #update $list_of_arrays
      $list_of_arrays{'stop'}           = [@stop];
      $list_of_arrays{'patches_out_db'} = [@patch_out_db];
      $list_of_arrays{'patches_in_srv'} = [@patch_in_srv];
      $list_of_arrays{'sus_out_db'}     = [@su_out_db];
      $list_of_arrays{'sus_in_srv'}     = [@su_in_srv];
      $list_of_arrays{'start'}          = [@start];
   } ## end if (!${$status_in_out_srv_reboot...
   return \%list_of_arrays;
} ## end sub collect_common_info($$$$$$)
############################################################
## @fn sub sp_apps_stop($self, $apps_stop_list)
# @brief Stop list of application passed as parameter
# @details 1. Obtain base and nortel applications based on
# @           app_stop_list parameter.
# @        2. If there are base applications to stop, stop first
# @           nortel apps and after base.
# @           Otherwise, stop only nortel apps.
# @param   $self  - reference to object.
# @param   $apps_stop_list  - reference to list of apps to stop.
# @OUTPUT: $OK  - stopped all apps successfully
# @        $ERROR - failed to stop all apps
#############################################################
sub sp_apps_stop($$)
{
   my ($self, $apps_stop_list) = @_;
   my $apps = $self->shrink_apps($apps_stop_list);

   my $old_value = $ENV{'APPSTART_IGNORE'};
   if (!defined($old_value))
   {
      $old_value = '';
   }

   # do not print error if applications is not installed
   $ENV{'APPSTART_IGNORE'} = $TRUE;

   # if we have 'base' application(s) among app_stop, so stopping all 'nortel' applications first
   if (scalar(@{$apps->{'base'}}))
   {
      baselog($LOG_DEBUG, "Stopping all nortel applications.");
      if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_STOP], {$dsSTDOUT => $TRUE}))
      {
         baselog($LOG_ERROR, "Nortel Applications stopping failed\n", $TRUE);
         $ENV{'APPSTART_IGNORE'} = $old_value;
         return $ERROR;
      } ## end if (!do_sudo([$SUDO_NORTEL...

      # now stopping those 'base' applications which are in list
      foreach my $base_app (@{$apps->{'base'}})
      {
         baselog($LOG_DEBUG, "Stopping $base_app base application.");
         if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $BASE_APPS, $SRV_STOP, $base_app], {$dsSTDOUT => $TRUE}))
         {
            baselog($LOG_ERROR, "Base Application $base_app stopping failed\n", $TRUE);
            $ENV{'APPSTART_IGNORE'} = $old_value;
            return $ERROR;
         } ## end if (!do_sudo([$SUDO_NORTEL...
      } ## end foreach my $base_app (@{$apps...
   } ## end if (scalar(@{$apps->{'base'...
     # don't have 'base' applications, need to stop only those 'nortel' applications which are in list
   else
   {
      foreach my $ntl_app (@{$apps->{'nortel'}})
      {
         baselog($LOG_DEBUG, "Stopping $ntl_app nortel application.");
         if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_STOP, $ntl_app], {$dsSTDOUT => $TRUE}))
         {
            baselog($LOG_ERROR, "Nortel Application $ntl_app stopping failed\n", $TRUE);
            $ENV{'APPSTART_IGNORE'} = $old_value;
            return $ERROR;
         } ## end if (!do_sudo([$SUDO_NORTEL...
      } ## end foreach my $ntl_app (@{$apps...
   } ## end else [ if (scalar(@{$apps->{'base'...
   $ENV{'APPSTART_IGNORE'} = $old_value;
   return $OK;
} ## end sub sp_apps_stop($$)

##################################################################
## @fn sub get_poos_and_pout_patches($self, $patches_out_list)
# @brief Separates applications on 'poos' and 'pout' groups.
# @details - Base on patch status IN_SERV or OUT_SERV, separate
# @          applications on 'poos' and 'pout' groups.
#INPUT:  $_[0] - the object reference
#        $_[1] - patches object list
#OUTPUT: hash with keys 'poos' and 'pout'; values are patch objects
#################################################################
sub get_poos_and_pout_patches($$)
{
   my ($self, $patches_out_list) = @_;
   return undef if (ref $patches_out_list ne 'ARRAY');

   # initializing arrays
   my (@poos, @pout) = ((), ());
   my %patches_status = ();
   foreach my $patch (@{$patches_out_list})
   {
      if ($patch->{'STATUS'} eq $IN_SERV)
      {
         push @poos, $patch;
      }
      else
      {
         if ($patch->{'STATUS'} eq $OUT_SERV || $patch->{'STATUS'} eq $OUT_SERV_REBOOT)
         {
            push @pout, $patch;
         }
      } ## end else [ if ($patch->{'STATUS'}...
   } ## end foreach my $patch (@{$patches_out_list...
   $patches_status{'poos'} = [@poos];
   $patches_status{'pout'} = [@pout];
   return \%patches_status;
} ## end sub get_poos_and_pout_patches($$)

############################################################
## @fn sub process_patches_out($self, $patches_out_list)
# @brief Poos and pout patches base on patch status
# @details 1. Base on parameter patches_out_list, obtain
# @           patches to poos
# @        2. Poos patches in service
# @        3. Base on parameter patches_out_list, obtain
# @           patches to pout
# @        4. Move patches to pout to sp/revert dir
# @        5. Pout patches
# @param   $self  - reference to object.
# @param   $patches_out_list  - reference to list of patches to
#          be out of service
# @OUTPUT: $OK  - poos and pout all patches required
# @        $ERROR - couldn't poos and pout all patches required
#############################################################
sub process_patches_out($$)
{
   my ($self, $patches_out_list) = @_;

   my $retval = $OK;

   # Save previous mode
   my $interact = $self->{'interact'};
   my $sp       = $self->{'sp'};
   my $appstop  = $self->{'appstop'};
   my $appstart = $self->{'appstart'};
   $self->set_app_flags($FALSE, $FALSE);
   $self->set_interact($FALSE);
   $self->set_sp($TRUE);

   #obtain poos list
   my $p_status = $self->get_poos_and_pout_patches(\@$patches_out_list);

   if (scalar(@{$p_status->{'poos'}}))
   {

      # Deactivate patches one by one
      unless ($self->poos_patches(\@{$p_status->{'poos'}}))
      {
         baselog($LOG_ERROR, "Error taking patches out of service.\n", $TRUE);
         $retval = $ERROR;
      }
   } ## end if (scalar(@{$p_status...

   # There is a pending issue: if poos patches, change status of a patch to OUT_OF_SERV_REBOOT,
   # those patches are not outed as expected.

   # update pout patches list after poos is done
   $p_status = $self->get_poos_and_pout_patches(\@$patches_out_list);
   if (scalar(@{$p_status->{'pout'}}))
   {

      # Remove patches
      unless ($self->pout_patches(\@{$p_status->{'pout'}}))
      {
         baselog($LOG_ERROR, "Patch removal failed\n", $TRUE);
         $retval = $ERROR;
      }
      foreach my $patch (@{$p_status->{'pout'}})
      {
         $patch = $PATCHDIR . "/" . $patch->{'FILENAME'};

         # move $patch to revert dir
         if (!do_system([$MV, $patch, $REVERT_DIR]))
         {
            baselog($LOG_ERROR, "Cannot move $patch to $REVERT_DIR\n", $TRUE);
            $retval = $ERROR;
         }
      } ## end foreach my $patch (@{$p_status...
   } ## end if (scalar(@{$p_status...

   # Restore previous mode
   $self->{'interact'} = $interact;
   $self->{'sp'}       = $sp;
   $self->set_app_flags($appstart, $appstop);
   return $retval;
} ## end sub process_patches_out($$)
############################################################
## @fn sub process_patches_in($self, $patches_in_list)
# @brief Pins patches base on patch_in_list parameter
# @details 1. Tries to pins every list member of the
# @           patches_in_list parameter
# @        2. Update SPINS file with spins results for each patch
# @param   $self  - reference to object.
# @param   $patches_in_list  - reference to list of patches to
# @        be in service.
# @OUTPUT: $OK - processing pins for each patch completed
# @        $ERROR - error during activating
#############################################################
sub process_patches_in($$)
{
   my ($self, $patches_in_list) = @_;
   my $retval = $OK;
   my $msg;

   # Save previous mode
   my $interact = $self->{'interact'};
   my $sp       = $self->{'sp'};
   my $appstop  = $self->{'appstop'};
   my $appstart = $self->{'appstart'};
   $self->set_app_flags($FALSE, $FALSE);
   $self->set_interact($FALSE);
   $self->set_sp($TRUE);

   my @patches     = @$patches_in_list;
   my @pids_failed = ();
   my @applist     = ();

   # loop across the list of patches
   foreach my $patch (@patches)
   {

      # Check patch status
      if ($patch->{'STATUS'} == $IN_SERV)
      {
         print 'The patch ' . $patch->{'PID'} . " is already in service.\n";
         if ($self->{'action'} eq $SPINS)
         {
            $msg = "$patch->{'PATCHNAME'} with patch handle $patch->{'PID'} is already in service.";
            if (!update_file($self, $SPINS_FILE, $msg))
            {
               baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'PATCHNAME'}.\n", $TRUE);
            }
         } ## end if ($self->{'action'} ...
      } ## end if ($patch->{'STATUS'}...
      elsif (not $self->pins_patch($patch, $FALSE, \@applist))  # put patch in service
      {
         $retval = $ERROR;
         push @pids_failed, $patch->{'PID'};
         next;
      } ## end elsif (not $self->pins_patch...
      elsif ($self->{'action'} eq $SPINS)
      {
         $msg = "$patch->{'FILENAME'} with patch handle $patch->{'PID'} was put in service successfully.";
         if (!update_file($self, $SPINS_FILE, $msg))
         {
            baselog($LOG_ERROR, "Failed to update $SPINS_FILE for $patch->{'FILENAME'}\n", $TRUE);
            $retval = $ERROR;
         }
      } ## end elsif ($self->{'action'} ...
   } ## end foreach my $patch (@patches...
   if (scalar(@pids_failed))
   {
      my $msg = "\n\tThe following patch IDs failed to be put into service: " . join(', ', @pids_failed) . ".\n\n";
      baselog($LOG_WARNING, $msg, $TRUE);
      if ($self->{'action'} eq $SPINS)
      {
         if (!update_file($self, $SPINS_FILE, $msg))
         {
            baselog($LOG_ERROR, "Failed to update $SPINS_FILE with pid failed summary.\n", $TRUE);
         }
      } ## end if ($self->{'action'} ...
   } ## end if (scalar(@pids_failed...

   # Restore previous mode
   $self->{'interact'} = $interact;
   $self->{'sp'}       = $sp;
   $self->set_app_flags($appstart, $appstop);

   return $retval;
} ## end sub process_patches_in($$)
############################################################
## @fn add_independent_patches($self, $list, $patches_list, $type)
# @brief Add independent SUs/patches to the list of dependent
#        SUs/patches.
# @param   $self  - reference to object.
# @param   $list  - reference to the sorted list of dependent
#                   SUs or patches
# @param   $patches_list  - reference to the list of independent
#                           SUs and patches that should be installed
# @param   $type  - reference to type of patch, SU or patch.
# @return  $TRUE  - operation successful
#          $FALSE - otherwise
############################################################
sub add_independent_patches($$$$)
{
   my ($self, $list, $patches_list, $type) = @_;

   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return $FALSE;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return $FALSE;
   }

   if ((not defined $type) or ($type !~ /^(SU|PATCH)$/))
   {
      baselog($LOG_ERROR, "Type should be SU or PATCH");
      return $FALSE;
   }

   foreach my $patch (@$patches_list)
   {
      if ($patch->{'TYPE'} eq $type)
      {
         push @$list, $patch;
      }
   } ## end foreach my $patch (@$patches_list...
   return $TRUE;
} ## end sub add_independent_patches($$$$)
############################################################
## @fn sub sort_by_dependency($self, $in_srv_lst, $dependency_check)
# @brief Sort the list of required sus and patches to be in srv
#        in order of dependency, based on dependent tag from SP header
# @details 1.Obtain list of sus and patches to be in srv
#          2.Obtain dependent list from SP header
#          3.Revert dependent list and obtain list with corresponding
#          patch object
#          4.Create full_patches_list of sus and patches to be in
#          service in order of dependency
# @param   $self  - reference to object.
# @param   $in_srv_lst  - reference to list of sus and patches to be in srv
# @param   $dependency_check - reference to dependency check results
# @return  $full_patches_list - list of sus and patches sorted
#          in order of dependency
#          Otherwise, undef
############################################################
sub sort_by_dependency($$$)
{
   my ($self, $in_srv_lst, $dependency_check) = @_;
   my (@dep_list, @patches_list, @rev_dep_list) = ((), (), ());
   my ($h_obj, $index);

   if (!scalar(@$in_srv_lst))
   {
      baselog($LOG_ERROR, "List for SUS and patches to be in service is empty\n", $TRUE);
      return undef;
   }

   my $header_xml        = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;
   my @full_patches_list = ();

   if (-e $header_xml)
   {
      $h_obj = patchutil::header_xml::new($header_xml);
      if (!defined($h_obj))
      {
         baselog($LOG_ERROR, "Cannot create header_xml object\n", $TRUE);
         return undef;
      }

      @dep_list = $h_obj->getDependency();
   } ## end if (-e $header_xml)
   else
   {
      baselog($LOG_ERROR, "Cannot find SP $header_xml file\n", $TRUE);
      return undef;
   }
   if (!scalar(@dep_list))
   {
      baselog($LOG_DEBUG, "There are not patches for dependency on the SP header $header_xml.\n");
      @full_patches_list = @$in_srv_lst;
   }
   else
   {
      baselog($LOG_DEBUG, "There are patches for dependency on the SP header $header_xml.\n");

      my (@SUs, @Patches) = ((), ());

      # @dep_list has patches first and after sus
      # reverse order of @dep_list to have sus first and after patches in order of dependency
      @dep_list = reverse(@dep_list);

      # initialize list of filenames which don't have pid
      my @failed_patches = ();

      @rev_dep_list = get_object_list($self, \@dep_list, \@failed_patches);
      foreach my $patch (@rev_dep_list)
      {
         $index = $self->find_member_in_SP_header($patch, @$in_srv_lst);
         if (defined $index)
         {

            # delete the patch from in_srv_lst array
            splice(@$in_srv_lst, $index, 1);
            if ($patch->{'TYPE'} eq 'SU')
            {
               push @SUs, $patch;
            }
            elsif ($patch->{'TYPE'} eq 'PATCH')
            {
               push @Patches, $patch;
            }
            else  # this shouldn't happen
            {
               baselog($LOG_ERROR, "Type $patch->{'TYPE'} for $patch->{'FILENAME'} is invalid. This patch will not be in service", $TRUE);
            }
         } ## end if (defined $index)
         else
         {
            baselog($LOG_ERROR, "Incorrect SP header. $patch->{'FILENAME'} is present on dependency list of SP header but not included in the SP", $TRUE);
            ${$dependency_check} = $ERROR;
         }
      } ## end foreach my $patch (@rev_dep_list...

      # add to SUs list the rest of SUs that don't have dependencies and are part of SP header
      $self->add_independent_patches(\@SUs, $in_srv_lst, 'SU');

      # add independent patches to the Patches list
      $self->add_independent_patches(\@Patches, $in_srv_lst, 'PATCH');

      # merge in full_patches_list sorted SUs and Patches
      push @full_patches_list, @SUs;
      push @full_patches_list, @Patches;
   } ## end else [ if (!scalar(@dep_list)...
   return @full_patches_list;
} ## end sub sort_by_dependency($$$)
############################################################
## @fn sub sp_apps_start($self, $apps_start_list, $is_last)
# @brief Start list of application passed as parameter
# @details 1. Obtain base and nortel applications based on
# @           app_start_list parameter.
# @        2. If there are base applications to start, start first
# @           nortel apps and after base.
# @           Otherwise, start only nortel apps.
# @param   $self  - reference to object.
# @param   $apps_start_list  - reference to list of apps to start.
# @param   $is_last - TRUE or FALSE to indicate if it is the last patch.
# @OUTPUT: $OK  - started apps successfully or choose no to start apps
# @        $ERROR - failed to start apps
#############################################################
sub sp_apps_start($$$)
{
   my ($self, $apps_start_list, $is_last) = @_;

   my $apps = $self->shrink_apps($apps_start_list);

   my $old_value = $ENV{'APPSTART_IGNORE'};
   if (!defined($old_value))
   {
      $old_value = '';
   }

   # do not print error if applications is not installed
   $ENV{'APPSTART_IGNORE'} = $TRUE;

   # if we have 'base' application(s) among app_start, so need to start all 'base' applications first
   if (scalar(@{$apps->{'base'}}))
   {
      if (($self->{'interact'}) && $is_last && (($self->{'action'} eq $PINS) || ($self->{'action'} eq $POOS)))
      {
         print "\nAll Nortel and " . join(' ', @{$apps->{'base'}}) . " applications will be started.\n";
         unless (getYNResponse("Do you want to continue", 'N'))
         {
            baselog($LOG_INFO, "Applications start operation cancelled.", $TRUE);
            $ENV{'APPSTART_IGNORE'} = $old_value;
            return $OK;
         } ## end unless (getYNResponse("Do you want to continue"...
      } ## end if (($self->{'interact'...
        # starting each 'base' application in list
      foreach my $base_app (@{$apps->{'base'}})
      {
         if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $BASE_APPS, $SRV_START, $base_app], {$dsSTDOUT => $TRUE}))
         {
            baselog($LOG_ERROR, "Base Application $base_app starting failed\n", $TRUE);
            $ENV{'APPSTART_IGNORE'} = $old_value;
            return $ERROR;
         } ## end if (!do_sudo([$SUDO_NORTEL...
      } ## end foreach my $base_app (@{$apps...
        # assuming the 'nortel' application were stopped
      if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_START], {$dsSTDOUT => $TRUE}))
      {
         baselog($LOG_ERROR, 'Nortel Applications starting failed', $TRUE);
         $ENV{'APPSTART_IGNORE'} = $old_value;
         return $ERROR;
      } ## end if (!do_sudo([$SUDO_NORTEL...
   } ## end if (scalar(@{$apps->{'base'...
        # don't have 'base' applications. starting only those 'nortel' applications in list
   elsif (scalar(@{$apps->{'nortel'}}))
   {
      if (($self->{'interact'}) && $is_last && (($self->{'action'} eq $PINS) || ($self->{'action'} eq $POOS)))
      {

         # asking for confirmation if interactive mode is set.
         print "\n" . join(' ', @{$apps->{'nortel'}}) . " applications will be started.\n";
         unless (getYNResponse("Do you want to continue", 'N'))
         {
            baselog($LOG_INFO, "Applications start operation cancelled.\n", $TRUE);
            $ENV{'APPSTART_IGNORE'} = $old_value;
            return $OK;
         } ## end unless (getYNResponse("Do you want to continue"...
      } ## end if (($self->{'interact'...
        # assuming that there is no dependencies between applications
      foreach my $ntl_app (@{$apps->{'nortel'}})
      {
         baselog($LOG_DEBUG, "Starting $ntl_app nortel application.");
         if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_START, $ntl_app], {$dsSTDOUT => $TRUE}))
         {
            $ENV{'APPSTART_IGNORE'} = $old_value;
            baselog($LOG_ERROR, "Nortel Application $ntl_app starting failed\n", $TRUE);
            return $ERROR;
         } ## end if (!do_sudo([$SUDO_NORTEL...

         # Due to a dbcom / mysql / nrssync / <nrsapp> race condition we need to delay
         # after a dbcom startup.
         if ($ntl_app eq "dbcom")
         {

            # After start dbcom give this particular app a 5 seconds break
            sleep(5);
         } ## end if ($ntl_app eq "dbcom"...

      } ## end foreach my $ntl_app (@{$apps...
   } ## end elsif (scalar(@{$apps->{'nortel'...
   $ENV{'APPSTART_IGNORE'} = $old_value;
   return $OK;
} ## end sub sp_apps_start($$$)
####################################################################
#Description: prepare the kernel upgrade, does the necessary checks
#INPUT:  #_[0] - the object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - error
####################################################################
sub kernel_update_prep($$)
{
   my ($self, $p) = @_;
   return $TRUE if (not $p->is_kernel_patch());
   my $pid = $p->{'PID'};

   #check if there are any kernel patches in service or in service after reboot
   my @ker_patches   = $self->{'DB'}->get_kernel_update(undef);
   my @ker_patch_out = $self->{'DB'}->get_kernel_update($OUT_SERV_REBOOT);
   my @ker_patch_in  = $self->{'DB'}->get_kernel_update($IN_SERV);
   push(@ker_patch_in, $self->{'DB'}->get_kernel_update($IN_SERV_REBOOT));

   if (scalar(@ker_patch_in))
   {
      my $pid = $ker_patch_in[0]->{'PID'};
      if ($self->{'interact'})
      {

         # ask user for confirmation
         my $answer;
         print "There is kernel patch (handle $pid) installed in the system.\nIt has to be uninstalled.\n";
         while ($TRUE)
         {
            print "Continue yes/no (y/n):";
            chomp($answer = <STDIN>);
            if ($answer =~ /^[Y|y](es)?$/)
            {
               print "ok\n";
               last;
            }
            elsif ($answer =~ /^[N|n](o)?$/)
            {
               baselog($LOG_INFO, "User cancels the patch installation\n", $TRUE);
               return $FALSE;
            }
         } ## end while ($TRUE)
      } ## end if ($self->{'interact'...

      # Remove previous kernel patch
      print "Un-installing the kernel update with handle $pid\n";

      unless ($p->uninstall($ker_patch_in[0]))
      {
         baselog($LOG_ERROR, "Kernel patch un-installation failed\n", $TRUE);
         return $FALSE;
      }

      # Set the status of the poosed kernel patch
      $self->set_kernel_stat_poos($ker_patch_in[0]);
   } ## end if (scalar(@ker_patch_in...

   my ($patch_ver, $loaded_ver);
   $loaded_ver = `$UNAME -r`;
   chomp($loaded_ver);
   foreach my $mname ($p->get_patch_members())
   {
      if ($mname =~ /kernel-(smp-)?(.*)\.(.*)\.rpm/)  #kernel update
      {
         if (defined($1))
         {
            $patch_ver = $2 . "smp";
         }
         else
         {
            $patch_ver = $2;
         }
         last;
      } ## end if ($mname =~ /kernel-(smp-)?(.*)\.(.*)\.rpm/...
   } ## end foreach my $mname ($p->get_patch_members...
   if ($patch_ver eq $loaded_ver)  #the upgrade of the current(loaded) kernel
   {
      unless ($self->stop_app_prompt())
      {
         baselog($LOG_INFO, "User has cancelled the installation\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($patch_ver eq $loaded_ver...

   return $TRUE;
} ## end sub kernel_update_prep($$)

####################################################
# Description: post kernel installation procedure.
#              Set the necessary attributes
# Input:  $_[0] - the object reference
#         $_[1] - patch object reference
# Output: $TRUE - success
####################################################
sub kernel_update_post($$)
{
   my ($self, $p) = @_;
   return $TRUE if (not $p->is_kernel_patch());

   # check the result of the patch installation,
   # "out service after reboot" patch will be in service anyway
   if ((-f $MERCURY_SIGN) || ($p->{'STATUS'} == $OUT_SERV_REBOOT))
   {
      unlink($MERCURY_SIGN);
      $self->{'DB'}->put_patch_in_service($p);
      print 'The kernel patch ' . $p->{'PID'} . " is in service\n";

      # mark "out of service" - previous poosed patch
      # or "out of service after reboot" patch, if they were
      #foreach my $h ($handle_ins, $handle_oos)
      #{
      #   if (defined($h))
      #   {
      #      $self->{'DB'}->put_patch_out_service($p);
      #   }
      #} ## end foreach my $h ($handle_ins,...
      # !!! TODO

   } ## end if ((-f $MERCURY_SIGN)...
   else  #non BASE kernel patch has been "poosed" or the patched mercury was busy.
   {
      $self->{'DB'}->set_status($p, $IN_SERV_REBOOT);
      print "The kernel patch $p->{'PID'} will be in service after restart.\n";
   }

   return $TRUE;
} ## end sub kernel_update_post($$)

#########################################################
#Description: find binary patches and ask user stop the
#             applications which are using them
#INPUT:  $_[0] - the object reference
#OUTPUT: $TRUE  - success
#        $FALSE - error
#########################################################
sub stop_app_prompt($)
{
   my $self = shift(@_);

   # No need to bother if non-interactive mode
   return $TRUE unless ($self->{'interact'});

   my @patches = $self->{'DB'}->get_rpm_patches(undef, $IN_SERV, 'PATCH');
   my @apps = ();
   foreach my $p (@patches)
   {
      if ($p->{'SUBTYPE'} eq 'BIN')
      {
         push @apps, $self->get_app_from_rpm($p->{'RPM_NAME'});
      }
   } ## end foreach my $p (@patches)

   # need to remove duplicate strings from an array if any
   my %s;
   my @temp = @apps;
   @apps = grep(!$s{$_}++, @temp);

   # If any applications found
   if (scalar(@apps))
   {
      my $answer;

      # extract and show the Application list which should be stopped.
      print "Please ensure that the following Applications or RPM Applications are stopped to put this patch in service without a reboot.\n";

      print "Otherwise, restart may be needed.\n";
      foreach (@apps)
      {
         print "\t$_\n";
      }
      while ($TRUE)
      {
         print "Continue patch installation yes/no (y/n):";
         chomp($answer = <STDIN>);
         if ($answer =~ /^[Y|y](es)?$/)
         {
            print "ok\n";
            last;
         }
         elsif ($answer =~ /^[N|n](o)?$/)
         {
            print "skipping\n";
            return $FALSE;
         }
      } ## end while ($TRUE)
   } ## end if (scalar(@apps))
   return $TRUE;
} ## end sub stop_app_prompt($)

#################################################################
#Description: Some pins preparation actions
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - in case any errors
#################################################################
sub prepare_pins($$)
{
   my ($self, $p) = @_;

   # this is kernel update, need to process separately.
   if ($p->is_kernel_patch())
   {
      unless ($self->kernel_update_prep($p))
      {
         baselog($LOG_ERROR, "Kernel update preparation failed\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($p->is_kernel_patch...
     # it's not a kernel update, so we have patch or SU
   else
   {

      if ($p->is_service_update())
      {
         unless ($self->pre_pins_SU($p))
         {
            baselog($LOG_ERROR, "Pins preparation for SU: $p->{PATCHNAME} failed.");
            return $FALSE;
         }
      } ## end if ($p->is_service_update...
      else
      {
         unless ($self->pre_pins_patch($p))
         {
            baselog($LOG_ERROR, "Pins preparation for patch: $p->{PATCHNAME} failed.");
            return $FALSE;
         }
      } ## end else [ if ($p->is_service_update...

      # common action for patch and SU - stopping the applications
      unless (exists $self->{'sp'} and $self->{'sp'})
      {
         return $FALSE unless $self->app_stop($p);
      }
   } ## end else [ if ($p->is_kernel_patch...
   return $TRUE;
} ## end sub prepare_pins($$)

##################################################################
#Description: Separates applications on 'base' and 'nortel' groups.
#INPUT:  $_[0] - the object reference
#        $_[1] - application list
#OUTPUT: hash with keys 'base' and 'nortel'; values are lists of application names
#        undef
#################################################################
sub shrink_apps($$)
{
   my ($self, $apps) = @_;
   return undef if (ref $apps ne 'ARRAY');

   # initializing arrays
   my (@nortel, @base) = ((), ());
   my %app_type = ();
   foreach my $app (@{$apps})
   {
      if (isBaseApplication($app))
      {
         push @base, $app;
      }
      else
      {
         push @nortel, $app;
      }
   } ## end foreach my $app (@{$apps})
   $app_type{'nortel'} = [@nortel];
   $app_type{'base'}   = [@base];
   return \%app_type;
} ## end sub shrink_apps($$)

##################################################################
#Description: Asks for user confirmation for applicatin stopping
#INPUT:  $_[0] - the object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - In case if user cancels operation or failure
##################################################################
sub app_stop($$)
{
   my ($self, $p) = @_;

   my $old_value = $ENV{'APPSTART_IGNORE'};
   if (!defined($old_value))
   {
      $old_value = '';
   }

   # do not print error if applications is not installed
   $ENV{'APPSTART_IGNORE'} = $TRUE;
   if ($self->{'appstop'} and scalar(@{$p->{'APP_STOP'}}))
   {
      my $apps = $self->shrink_apps($p->{'APP_STOP'});

      # we have 'base' application(s) among app_stop, so stopping all 'nortel' applications first
      if (scalar(@{$apps->{'base'}}))
      {
         if (not $self->{'app_stop'}->{'all_nortel'})
         {
            baselog($LOG_DEBUG, "Stopping all nortel applications.");
            if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_STOP], {$dsSTDOUT => $TRUE}))
            {
               baselog($LOG_ERROR, "Nortel Applications stopping failed\n", $TRUE);
            }

            # don't know which apps failed to stop, if any, so just going to set the flag.
            $self->{'app_stop'}->{'all_nortel'} = $TRUE;
         } ## end if (not $self->{'app_stop'...

         # now stopping those 'base' applications which are in list
         foreach my $base_app (@{$apps->{'base'}})
         {
            if (not $self->{'app_stop'}->{$base_app})
            {
               baselog($LOG_DEBUG, "Stopping $base_app base application.");
               if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $BASE_APPS, $SRV_STOP, $base_app], {$dsSTDOUT => $TRUE}))
               {
                  baselog($LOG_ERROR, "Base Application $base_app stopping failed\n", $TRUE);
               }
               else
               {

                  # setting 'app_stop' for the base app stopped
                  $self->{'app_stop'}->{$base_app} = $TRUE;
               } ## end else [ if (!do_sudo([$SUDO_NORTEL...
            } ## end if (not $self->{'app_stop'...

         } ## end foreach my $base_app (@{$apps...
      } ## end if (scalar(@{$apps->{'base'...

      # don't have 'base' applications, need to stop only those 'nortel' applications which are in list
      else
      {
         if (not $self->{'app_stop'}->{'all_nortel'})
         {
            foreach my $ntl_app (@{$apps->{'nortel'}})
            {
               if (not $self->{'app_stop'}->{$ntl_app})
               {
                  baselog($LOG_DEBUG, "Stopping $ntl_app nortel application.\n");
                  if (!do_sudo([$SUDO_NORTEL, $APP_SERVICE, $SRV_STOP, $ntl_app], {$dsSTDOUT => $TRUE}))
                  {
                     baselog($LOG_ERROR, "Nortel Application $ntl_app stopping failed\n", $TRUE);
                  }
                  else
                  {
                     $self->{'app_stop'}->{$ntl_app} = $TRUE;
                  }
               } ## end if (not $self->{'app_stop'...
            } ## end foreach my $ntl_app (@{$apps...
         } ## end if (not $self->{'app_stop'...

      } ## end else [ if (scalar(@{$apps->{'base'...
   } ## end if ($self->{'appstop'}...
   $ENV{'APPSTART_IGNORE'} = $old_value;
   return $TRUE;
} ## end sub app_stop($$)
############################################################
## @fn sub get_apps_stop_conf($self, $patches_list)
# @brief Ask for user confirmation about stopping apps for
# @      those patches passed as parameter.
# @details 1. Obtain apps that require stop base on
# @        patches_list passed as parameter
# @        2. Obtain  base and nortel applications based on
# @           obtained app_stop_list.
# @        3. Ask for user confirmation about stopping base
# @        or/and nortel apps.
# @param   $self  - reference to object.
# @param   $patches_list  - reference to list of patches
# @OUTPUT: $TRUE  - user confirm to stop apps
# @        $FALSE - user cancel to stop apps
#############################################################
sub get_apps_stop_conf($$)
{
   my ($self, $patches_list) = @_;

   my %seen          = ();
   my @app_stop_list = ();
   my @patches       = @$patches_list;
   for (my $i = 0 ; $i < scalar(@patches) ; $i++)
   {

      # create list of applications to stop, no duplicates allowed
      if (scalar(@{$patches[$i]->{'APP_STOP'}}))
      {
         foreach my $appstop (@{$patches[$i]->{'APP_STOP'}})
         {
            push(@app_stop_list, $appstop) unless ($seen{$appstop}++);
         }
      } ## end if (scalar(@{$patches[...
   } ## end for (my $i = 0 ; $i < scalar...
   if (scalar(@app_stop_list))
   {
      my $apps = $self->shrink_apps(\@app_stop_list);

      # if we have 'base' application(s) among app_stop, so stopping all 'nortel' applications first
      if (scalar(@{$apps->{'base'}}))
      {
         if ($self->{'interact'})
         {

            # asking for confirmation if interactive mode is set.
            print "\n All Nortel and " . join(' ', @{$apps->{'base'}}) . " applications will be stopped.\n";
            unless (getYNResponse("Do you want to continue", 'N'))
            {
               if ($self->{'action'} eq $PINS)
               {
                  baselog($LOG_INFO, "pins command cancelled.\n", $TRUE);
               }
               elsif ($self->{'action'} eq $POOS)
               {
                  baselog($LOG_INFO, "poos command cancelled.\n", $TRUE);
               }
               return $FALSE;
            } ## end unless (getYNResponse("Do you want to continue"...
         } ## end if ($self->{'interact'...
      } ## end if (scalar(@{$apps->{'base'...

      # don't have 'base' applications, need to stop only those 'nortel' applications which are in list
      else
      {
         if ($self->{'interact'})
         {

            # asking for confirmation if interactive mode is set.
            print "\n" . join(' ', @{$apps->{'nortel'}}) . " applications will be stopped.\n";
            unless (getYNResponse("Do you want to continue", 'N'))
            {
               if ($self->{'action'} eq $PINS)
               {
                  baselog($LOG_INFO, "pins command cancelled.\n", $TRUE);
               }
               elsif ($self->{'action'} eq $POOS)
               {
                  baselog($LOG_INFO, "poos command cancelled.\n", $TRUE);
               }
               return $FALSE;
            } ## end unless (getYNResponse("Do you want to continue"...
         } ## end if ($self->{'interact'...
      } ## end else [ if (scalar(@{$apps->{'base'...
   } ## end if (scalar(@app_stop_list...
   return $TRUE;
} ## end sub get_apps_stop_conf($$)
############################################################
## @fn sub init_keys_for_app_stop($self, $patches_ref)
# @brief Initialize to FALSE a flag for each of the apps that
# @      need to be stopped by the patching command.
# @details 1. Obtain apps that require stop base on
# @        patches_ref passed as parameter
# @        2. Initialize to FALSE the flags for those apps
# @        obtained.
# @        3. Initialize to FALSE the flag for 'all_nortel'
# @        key.
# @param   $self  - reference to object.
# @param   $patches_ref  - reference to list of patches
# @OUTPUT: $TRUE  - initialize succeeds.
############################################################
sub init_keys_for_app_stop($$)
{
   my ($self, $patches_ref) = @_;
   my %seen    = ();
   my @patches = @$patches_ref;

   # initialize $self->{'app_stop'} for all the apps that need to be stopped.
   my @appstop_list = ();
   for (my $i = 0 ; $i < scalar(@patches) ; $i++)
   {
      if (scalar(@{$patches[$i]->{'APP_STOP'}}))
      {
         foreach my $appstop (@{$patches[$i]->{'APP_STOP'}})
         {
            push(@appstop_list, $appstop) unless ($seen{$appstop}++);
         }
      } ## end if (scalar(@{$patches[...
   } ## end for (my $i = 0 ; $i < scalar...
   my $stop = ();
   foreach (@appstop_list)
   {
      $stop->{$_} = $FALSE;
   }
   $self->{'app_stop'} = $stop;
   $self->{'app_stop'}->{'all_nortel'} = $FALSE;
   return $TRUE;
} ## end sub init_keys_for_app_stop($$)

##########################################################
#Description: Checks conflicts between being installed patch and installed patches
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - errors
##########################################################
sub pre_pins_patch($$)
{
   my ($self, $p) = @_;
   my ($system_rpm, $rpm_retval);

   # Check for the patch conflicts
   unless ($self->{'DB'}->check_element_conflict($p, $TRUE))
   {
      baselog($LOG_ERROR, "Patch conflict detected");
      return $FALSE;
   }
   if ($p->{'SUBTYPE'} =~ /^BIN$/)
   {
      unless ($self->{'DB'}->check_functions_conflict($p, $TRUE))
      {
         baselog($LOG_ERROR, "Patch conflict detected");
         return $FALSE;
      }
   } ## end if ($p->{'SUBTYPE'} =~...

   my $query_rpm_name;

   # if CS RPM then do not check last part of file name
   if ($p->{'RPM_NAME'} =~ '^(nortel-cs1000-cs-\d\.\d{2}\.\w)')
   {
      $query_rpm_name = $1 . '*';
   }
   else
   {
      $query_rpm_name = $p->{'RPM_NAME'};
   }

   do_system([$RPM, "-q", $query_rpm_name], {$dsNOERROR => $TRUE, $dsOUT => \@$system_rpm, $dsRETVAL => \$rpm_retval});
   if (not $rpm_retval and defined $system_rpm)
   {
      baselog($LOG_INFO, "The patch $p->{'PATCHNAME'} is intended for the rpm @$system_rpm. @$system_rpm is installed on the system.");
   }
   else
   {
      baselog($LOG_ERROR, "RPM $p->{RPM_NAME} is not installed, cannot patch it.", $TRUE);
      return $FALSE;
   }
   return $TRUE;
} ## end sub pre_pins_patch($$)

##########################################################
#Description: Asks for user confirmation and remove
#             in service patches before SU installation
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - errors
##########################################################
sub pre_pins_SU($$)
{
   my ($self, $p) = @_;
   my $ret;

   # Save previous mode
   my $interact = $self->{'interact'};

   #See if you are attempting to put an RPM in service that is the same version as the current one.
   do_system([$RPM, '-q', $p->{'RPM_NAME'}], {$dsNOERROR => $TRUE, $dsRETVAL => \$ret});
   if ($ret == $OK)  # rpm query found RPM_NAME already installed on system
   {

      # check if there is a SU installed with this RPM_NAME. If so, we'll replace it, if not, error.
      my @temp_patchlist = $self->{'DB'}->get_rpm_patches($p->{'RPM_NAME'}, $IN_SERV, 'SU');
      if (not scalar(@temp_patchlist))
      {
         baselog($LOG_ERROR, "Cannot install a SU with the same RPM version as the system\n", $TRUE);
         return $FALSE;
      }
   } ## end if ($ret == $OK)

   # getting patches to be poos'ed
   my @poos_patches = $self->{'DB'}->get_rpm_patches_by_name($p->{'RPM_NAME'}, $IN_SERV, 'PATCH');
   push(@poos_patches, $self->{'DB'}->get_rpm_patches_by_name($p->{'RPM_NAME'}, $IN_SERV, 'SU'));
   if (not scalar(@poos_patches))
   {
      baselog($LOG_INFO, "RPM $p->{RPM_NAME} doesn't have patches, so nothing to deactivate");
   }
   else
   {
      baselog($LOG_DEBUG, 'Patches to take out of service: ' . join(' ', @poos_patches));
      print "\nThe following patches will be taken out of service and unloaded:\n";
      foreach my $patch (@poos_patches)
      {
         print $patch->{'PATCHNAME'} . "\n";
      }

      print "Taking patches out of service. Please wait...\n\n";

      # Change mode to automate patch removal
      $self->set_interact($FALSE);

      # If there are patches which should be out of service, then do not reboot
      # to take patches out of service. Reboot a server after 'pins' operation.
      if (scalar @poos_patches)
      {
         $self->{'do_not_reboot'} = $TRUE;
      }

      # Deactivate patches one by one
      unless ($self->poos_patches(\@poos_patches))
      {
         baselog($LOG_ERROR, "Error taking patches out of service.\n", $TRUE);
         return $FALSE;
      }

      baselog($LOG_INFO, "Patches have been deactivated successfully");
      @poos_patches = ();  # cleanup
      baselog($LOG_INFO, "All patches have been taken out of service successfully.\n", $TRUE);
   } ## end else [ if (not scalar(@poos_patches...
                           # Get patches to remove
   my @pout_patches = $self->{'DB'}->get_rpm_patches_by_name($p->{'RPM_NAME'}, $OUT_SERV, 'PATCH');
   push(@pout_patches, $self->{'DB'}->get_rpm_patches_by_name($p->{'RPM_NAME'}, $OUT_SERV, 'SU'));

   # I am out of service yet, Do not remove myself
   @pout_patches = grep($_->{'PID'} != $p->{'PID'}, @pout_patches);

   # I don't want to pout any patches with the same RPM_NAME as me, just different versions.
   @pout_patches = grep($_->{'RPM_NAME'} ne $p->{'RPM_NAME'}, @pout_patches);

   if (!scalar(@pout_patches))
   {
      baselog($LOG_DEBUG, "Nothing to remove");
   }
   else
   {
      baselog($LOG_DEBUG, 'Patches to remove: ' . join(' ', map {$_->{'PID'}} @pout_patches));

      # Remove patches
      baselog($LOG_ERROR, 'Patch removal failed') unless $self->pout_patches(\@pout_patches);
   } ## end else [ if (!scalar(@pout_patches...

   # Restore previous mode
   $self->{'interact'} = $interact;

   return $TRUE;
} ## end sub pre_pins_SU($$)

#############################################
#Description: Post installation procedures.
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
# 	 $_[2] - TRUE/FALSE indicates if this is a last patch
#        $_[3] - list of applications to start if this is the last patch
#OUTPUT: $TRUE  - success
#        $FALSE - error
#############################################
sub post_pins($$$$)
{
   my ($self, $p, $is_last, $reflist) = @_;
   my @applist = @$reflist;

   # Update elements database. For patches only.
   unless ($p->is_kernel_patch() or $p->is_service_update())
   {
      if ($self->{'DB'}->add_element_record($p) == $FALSE)
      {
         baselog($LOG_ERROR, "Unable to add element record for patch $p->{'pid'}");
         return $FALSE;
      }
   } ## end unless ($p->is_kernel_patch...

   # Some kernel post install actions
   unless ($self->kernel_update_post($p))
   {
      baselog($LOG_ERROR, 'Kernel post installation failed');
      return $FALSE;
   }

   # Update configure.xml file for RPM patch
   unless ($self->update_configure_xml($p))
   {
      baselog($LOG_ERROR, 'Error updating configure.xml file');
      return $FALSE;
   }

   # Update DB
   # for the kernel patch , status attribute is
   # changed in the kernel_update_post() function
   if (not $p->is_kernel_patch())
   {
      $self->{'DB'}->put_patch_in_service($p);
      baselog($LOG_DEBUG, 'Patch status changed');
   }

   # if reboot is required then need to notify user
   # will not start the applications

   if (defined $self->{'reboot'} and scalar(@{$self->{'reboot'}}) and $is_last)
   {
      my $do_reboot = $FALSE;
      if ($self->reboot_required() == $TRUE)  # check the database to make sure a reboot is really required
      {
         baselog($LOG_WARNING, "\nA Server REBOOT must be performed to bring patch(es) into service", $TRUE);
         if ($self->{'interact'})
         {

            # asking for confirmation if interactive mode is set.
            if (getYNResponse("Do you want to reboot now", 'N'))
            {

               # this is interactive mode (pins) and user choses reboot
               $do_reboot = $TRUE;
            } ## end if (getYNResponse("Do you want to reboot now"...
            else
            {
               return $TRUE;
            }
         } ## end if ($self->{'interact'...

         # if '-sp' option is specified (pins -r -sp) then spins manages reboot. So ignore reboot tag.
         elsif (exists $self->{'sp'} and not $self->{'sp'})
         {

            # non-interactive mode (pins -r), so rebooting automatically
            $do_reboot = $TRUE;
         } ## end elsif (exists $self->{'sp'...
         elsif (exists $self->{'sp'} and $self->{'sp'})
         {
            return $TRUE;
         }
      } ## end if ($self->reboot_required...

      if ($do_reboot)
      {

         # Reapply basic hardening items before rebooting
         baselog($LOG_INFO, "Reapplying basic hardening... Please wait a moment\n", $TRUE);
         if (not do_sudo([$SUDO_NORTEL, $BASIC_HARDENIG_REAPPLY_POST_PATCHING]))
         {
            baselog($LOG_WARNING, "Reapplying basic hardening items failed.");
         }

         if (system("$SUDO $SHUTDOWN -r +2 \"A Server REBOOT is being performed to bring patches into service\" &") >> 8 == $OK)
         {
            return $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "\nCould not initiate a REBOOT. You must manually REBOOT the server to put the patches in service", $TRUE);
            return $FALSE;
         }
      } ## end if ($do_reboot)

   } ## end if (defined $self->{'reboot'...

   # Ask user to start applications
   ## no reboot required, need to start apps if not in sp mode

   if (exists $self->{'sp'} and not $self->{'sp'} and $is_last and scalar(\@applist))
   {
      return $FALSE unless ($self->sp_apps_start(\@applist, $is_last) == $OK);
   }

   return $TRUE;
} ## end sub post_pins($$$$)

## @method update_configure_xml($self, $p)
# @brief   Update configure.xml file after RPM installation/un-installation
# @details Version is taken from $CONFVER file if patched Application is
#          Nortel. System RPMS and base application RPMS don't need to do
#          anything.
# @param   $self - reference to object
# @param   $p - patch object reference
# @return  $TRUE - success
# @return  $FALSE - error
# @author  Alex Arkhipov
# @author  Evgeny Lazarev
# @date    May 2008
# @note    None.
sub update_configure_xml($$)
{
   my ($self, $p) = @_;
   my $ver    = "";
   my $params = undef;

   # Check SU only
   return $TRUE if (not $p->is_service_update());

   my $appname = $self->get_app_from_rpm($p->{'RPM_NAME'});
   if (not defined $appname)
   {
      return $TRUE;  # this is a system RPM, not a Nortel application
   }

   if (isBaseApplication($appname))
   {
      my $configure_file = "$APP_CFG_ROOT/$appname/$APPLCFG";

      #check xml file using xml scheme
      checkXmlFiles([$configure_file], $CONFIGURE_XML_SCHEME_PATH, $FALSE);
      return $TRUE;
   } ## end if (isBaseApplication(...
   else
   {
      $params = use_confver($CONFVER);
      unless ($params)
      {
         baselog($LOG_ERROR, "Can not find application installed.");
         return $FALSE;
      }

      $ver = $params->{'APP_VERSION'};
      if ($ver =~ /(\d+\.\d+\.\d+)/)
      {
         $ver = $1;
      }
      else
      {
         baselog($LOG_ERROR, "Version ($ver) does not match format. Using default version 6.00.00");
         $ver = "6.00.00";
      }
   } ## end else [ if (isBaseApplication(...

   # Put application media version in configure.xml
   unless (configure_xml_update($appname, $ver))
   {
      baselog($LOG_WARNING, "Updating configure.xml failed");
      return $FALSE;
   }
   return $TRUE;
} ## end sub update_configure_xml($$)

#############################
# POOS related subroutines  #
#############################

#################################################################
#Description: Some poos preparation actions
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - in case any errors
#################################################################
sub pre_poos($$)
{
   my ($self, $p) = @_;

   #perform common action for patch and SU - stopping applications if any
   unless (exists($self->{'sp'}) and $self->{'sp'})
   {
      return $FALSE unless ($self->app_stop($p));
   }

   # this is SU and state is not equal 'pins'
   if ($p->is_service_update() and $self->{'action'} ne $PINS and $self->{'action'} ne $SPINS)
   {
      return $FALSE unless ($self->pre_poos_SU($p));
   }

   return $TRUE;
} ## end sub pre_poos($$)

#################################################################
# Description: prepare the kernel uninstallation
# Input:  $_[0] - object reference
#         #_[1] - patch object reference
# Output: $TRUE  - success
#         $FALSE - error
#################################################################
sub kernel_remove_prep($$)
{
   my ($self, $p) = @_;
   return $TRUE if (not $p->is_kernel_patch());

   # If patch is in service
   # the kernel version = BASE version and
   # there are binary patches in service
   # then ask user to stop the applications.
   if ($p->{'STATUS'} eq $IN_SERV)
   {
      my $ker_ver = $p->get_kernel_version();
      if ($ker_ver eq $FIRST_VER)
      {
         unless ($self->stop_app_prompt())
         {
            return $FALSE;
         }
      } ## end if ($ker_ver eq $FIRST_VER...
   } ## end if ($p->{'STATUS'} eq ...
   return $TRUE;
} ## end sub kernel_remove_prep($$)

##########################################################
#Description: Asks for user confirmation and remove
#             in service patches before SU un-installation
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#OUTPUT: $TRUE  - success
#        $FALSE - errors
##########################################################
sub pre_poos_SU($$)
{
   my ($self, $p) = @_;

   # Save previous mode
   my $interact = $self->{'interact'};

   # putting SU out of service, so need to deactivate all the patches for the same rpm
   my @poos_patches = $self->{'DB'}->get_rpm_patches($p->{'RPM_NAME'}, $IN_SERV, 'PATCH');

   if (not scalar(@poos_patches))
   {
      baselog($LOG_INFO, "Nothing to deactivate");
   }
   else
   {
      baselog($LOG_DEBUG, 'Patches to deactivate: ' . join(' ', map {$_->{'PID'}} @poos_patches));
      print "\nThe following patches will be taken out of service and removed:\n";
      foreach my $patch (@poos_patches)
      {
         print $patch->{'PATCHNAME'} . "\n";
      }

      print "Taking patches out of service. Please wait...\n\n";

      # Change operating mode to automate patch removal
      $self->set_interact($FALSE);

      # Deactivate patches one by one
      unless ($self->poos_patches(\@poos_patches))
      {
         baselog($LOG_ERROR, "Error taking patches out of service.\n", $TRUE);
         return $FALSE;
      }

      baselog($LOG_INFO, "All patches have been taken out of service successfully.\n", $TRUE);
      print "\n";
      @poos_patches = ();  # cleanup
   } ## end else [ if (not scalar(@poos_patches...

   # Restore previous mode
   $self->{'interact'} = $interact;

   return $TRUE;
} ## end sub pre_poos_SU($$)

#################################################################
#Description: Post patch un-installation procedures.
#INPUT:  $_[0] - object reference
#        $_[1] - patch object reference
#        $_[2] - TRUE/FALSE indicates if this is a last patch
#OUTPUT: $TRUE  - success
#        $FALSE - error
#################################################################
sub post_poos($$$$)
{
   my ($self, $p, $is_last, $reflist) = @_;
   my @applist     = @$reflist;
   my $need_reboot = $FALSE;

   # Update elements database
   if ((not $p->is_kernel_patch()) and (not $p->is_service_update()))
   {
      baselog($LOG_WARNING, "Can't remove elements for patch $p->{PATCHNAME}.") unless ($self->{'DB'}->remove_element_record($p));
   }

   # Update configure.xml file for RPM updates
   if (not $self->update_configure_xml($p))
   {
      baselog($LOG_ERROR, 'Error updating configure.xml file');
      return $FALSE;
   }

   # Update DB
   # for the kernel patch , status attribute is
   # changed in the kernel_update_post() function
   if (not $p->is_kernel_patch())
   {
      $self->{'DB'}->put_patch_out_service($p);
      baselog($LOG_DEBUG, 'Patch status changed');
   }

   # check if reboot required before database updates
   if (defined $self->{'reboot'} and scalar(@{$self->{'reboot'}}) and $is_last)
   {
      $need_reboot = $self->reboot_required();
   }

   # pout patch if poos is triggered by PM
   if (    (($p->{'STATUS'} == $OUT_SERV) or ($p->{'STATUS'} == $OUT_SERV_REBOOT))
       and ($self->{'context'} eq 'PM'))
   {
      my %params = ('interact' => $FALSE, 'pid' => $p->{'PID'}, 'all' => $FALSE, 'sp' => $FALSE, 'filename' => undef);

      #update status to OUT_SERV for all patches;
      $self->{'DB'}->set_status_byID($p->{'PID'}, $OUT_SERV);
      if (not $self->pout(\%params))
      {
         baselog($LOG_ERROR, "Unable to put out of service patch with pid '$p->{'PID'}'", $TRUE);
      }
   } ## end if ((($p->{'STATUS'} ==...

   # if reboot is required then need to notify user
   # will not start the applications
   if (defined $self->{'reboot'} and scalar(@{$self->{'reboot'}}) and $is_last)
   {
      my $do_reboot = $FALSE;
      if ($need_reboot)
      {
         baselog($LOG_WARNING, "\nA Server REBOOT must be performed to put patch(es) out of service", $TRUE);
         if ($self->{'interact'})
         {

            # asking for confirmation if interactive mode is set.
            if (getYNResponse("Do you want to reboot now", 'N'))
            {

               # this is interactive mode (poos) and user choses reboot
               $do_reboot = $TRUE;
            } ## end if (getYNResponse("Do you want to reboot now"...
            else
            {
               return $TRUE;
            }
         } ## end if ($self->{'interact'...

         # if '-sp' option is specified (poos -r -sp) then spins manages reboot. So ignore reboot tag.
         elsif (exists $self->{'sp'} and not $self->{'sp'})
         {

            # non-interactive mode (poos -r), so rebooting automatically
            $do_reboot = $TRUE;
         } ## end elsif (exists $self->{'sp'...
         elsif (exists $self->{'sp'} and $self->{'sp'})
         {
            return $TRUE;
         }
      } ## end if ($need_reboot)
      if ($do_reboot)
      {

         if (exists $self->{'do_not_reboot'} and $self->{'do_not_reboot'} == $TRUE)
         {
            baselog($LOG_INFO, "Do not reboot now. Reboot server after putting required patch(es) into service");
            return $TRUE;
         }
         if (system("$SUDO $SHUTDOWN -r +2 \"A Server REBOOT is being performed to put patches out of service\" &") >> 8 == $OK)
         {
            return $TRUE;
         }
         else
         {
            baselog($LOG_ERROR, "\nCould not initiate a REBOOT. You must manually REBOOT the server to put the patches out of service", $TRUE);
            return $FALSE;
         }
      } ## end if ($do_reboot)
   } ## end if (defined $self->{'reboot'...

   # don't re-start the apps if poos was done from pins command.
   if ($self->{'action'} ne $PINS)
   {
      ## not a reboot, try to start apps if not sp
      if (not $self->{'sp'} and $is_last and scalar(\@applist))
      {
         return $FALSE unless ($self->sp_apps_start(\@applist, $is_last) == $OK);
      }
   } ## end if ($self->{'action'} ...
   return $TRUE;
} ## end sub post_poos($$$$)

################################################
# Description: set the kernel patch status
#              after the patch un-installation
# Input:  $_[0] - object reference
#         $_[1] - patch object reference
# Output: $TRUE  - success
#         $FALSE - error
################################################
sub set_kernel_stat_poos($)
{
   my ($self, $p) = @_;
   return $TRUE if (not $p->is_kernel_patch());

   # Check the result of the patch uninstallation
   if (-f $MERCURY_SIGN)
   {
      unlink($MERCURY_SIGN);
      $self->{'DB'}->put_patch_out_service($p);
      print 'Patch ' . $p->{'PID'} . ' is out of service' . "\n";
   } ## end if (-f $MERCURY_SIGN)
   else  #non BASE kernel patch has been "poosed" or the patched mercury was busy.
   {
      if ($p->{'STATUS'} == $IN_SERV)
      {  #in_service => will be out  of service after reboot
         $self->{'DB'}->set_status($p, $OUT_SERV_REBOOT);
         print 'Patch ' . $p->{'PID'} . ' will be out of service after restart' . "\n";
      }
      elsif ($p->{'STATUS'} == $IN_SERV_REBOOT)
      {  #will be in service after reboot => out of service
         $self->{'DB'}->set_status($p, $OUT_SERV);
         print 'Patch ' . $p->{'PID'} . ' is out of service' . "\n";
      }
   } ## end else [ if (-f $MERCURY_SIGN)

   return $TRUE;
} ## end sub set_kernel_stat_poos($)

###########################
# END of POOS subroutines #
###########################

##########################
# Some service utilities #
##########################

#################################################################
# Description: Compare two lists
# Input:  $_[0] - reference to the list 1
#         $_[1] - reference to the list 2
# Output: $TRUE  - lists are equal
#         $FALSE - lists are not equal
#################################################################
sub compare_lists($$)
{
   my @list1 = sort(@{shift(@_)});
   my @list2 = sort(@{shift(@_)});
   my $len   = length(@list1);
   return $FALSE if ($len != length(@list2));
   for (my $i = 0 ; $i < $len ; $i++)
   {
      return $FALSE if ($list1[$i] ne $list2[$i]);
   }
   return $TRUE;
} ## end sub compare_lists($$)

###############################################################################
#
# Description: this function applies kernel patches after reboot. It will also change
# the state of patches/SUs that required reboot to activate/deactivate.
# This function is executed only from "/opt/nortel/base/patchutil/bin/startup_patch_mngmnt"
# script which is called by the init.d "/opt/nortel/base/patchutil/bin/patch_startup"
# script.
#
# Input:       $_[0]  - object reference
# Output:      $TRUE  - patches successfully applied
#              $FALSE - some errors happened
###############################################################################
sub apply_reboot_patches($)
{
   my $self       = shift @_;
   my $loaded_ver = "";
   my $pstatus    = "";

   chomp($loaded_ver = `$UNAME -r`);

   # receiving list of all kernel patches
   my @all_patches = $self->{'DB'}->get_kernel_update();

   foreach my $patch (@all_patches)
   {
      $pstatus = $patch->get_patch_status();
      if ($pstatus == $IN_SERV || $pstatus == $IN_SERV_REBOOT)
      {
         my $ver = $self->get_version($patch);
         if ($ver ne $loaded_ver)
         {
            if (not $self->poos({'pid' => $patch->get_patch_ID(), 'interact' => $FALSE}))
            {
               baselog($LOG_ERROR, "Failed to take patch out of service.\n", $TRUE);
               return $FALSE;
            }
            if ($pstatus == $IN_SERV)
            {
               $self->{'DB'}->put_patch_out_service_after_reboot($patch);
            }
         } ## end if ($ver ne $loaded_ver...
         elsif ($pstatus == $IN_SERV_REBOOT)
         {
            $self->{'DB'}->put_patch_in_service_after_reboot($patch);
         }
      } ## end if ($pstatus == $IN_SERV...
      elsif ($pstatus == $OUT_SERV_REBOOT)
      {
         $self->{'DB'}->put_patch_out_service_after_reboot($patch);
      }
   } ## end foreach my $patch (@all_patches...

   ## advance other patches and SUs to their correct state.
   @all_patches = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);
   foreach my $patch (@all_patches)
   {
      $pstatus = $patch->get_patch_status();
      if ($pstatus == $IN_SERV_REBOOT)
      {
         $self->{'DB'}->put_patch_in_service_after_reboot($patch);
      }
      elsif ($pstatus == $OUT_SERV_REBOOT)
      {
         $self->{'DB'}->put_patch_out_service_after_reboot($patch);
      }
   } ## end foreach my $patch (@all_patches...

   return $TRUE;
} ## end sub apply_reboot_patches($)

###############################################################################
#
# Description: this function returns kernel RPM's version from patch file in
#              same format as returns command 'uname -r'
# Input:       $_[0] - object reference
#              $_[1] - patch object reference
# Output:      kernel RPM's version or undef if this is not kernel RPM
#
##############################################################################
sub get_version($$)
{
   my ($self, $patch) = (@_);
   my $ver     = undef;
   my @members = @{$patch->{'MEM_INFO'}};
   foreach my $name (@members)
   {
      if ($name =~ /kernel/)
      {
         $ver = $name;
         $ver =~ /kernel-(smp-)?(.*)\.(.*)\.rpm/;
         $ver = $2 . ($1 ? "smp" : '');
      } ## end if ($name =~ /kernel/)
   } ## end foreach my $name (@members)
   return $ver;
} ## end sub get_version($$)
#################################################################
# @fn sub reapply
# @brief: This subroutine implements patch re-apply mechanism
#         during a re-install of the linux server.
#
# @details:
#    This subroutine reapplies all patches flagged by the original
#    .handles db that was recovered from the sysbackup archive file.
#
#    The assumption is that reapply() is only called during a server re-
#    install and thus the system has been wiped clean. The patch
#    reapply is thus done on a freshly installed system only.
#
# @param  none
#         - assumes that the (recovered) patch db exists and that all
#          (recovered) patches also reside in the patch folder
#
# @return $TRUE  - all in-service patches are in service
# @return $FALSE - at least one error occured in installing the
#                  recovered patches
#################################################################
sub reapply
{
   my $self = shift;
   my (@all_patches, @p, $patch);
   my $patch_okay    = $TRUE;  # Flag patch installation failure
   my $noerr_found   = $TRUE;  # Flag actual non-patch install errors
   my $patch_counter = 0;
   my $is_last       = $FALSE;
   my @applist       = ();
   my %seen          = ();

   #
   # This function is only meant to be run in a re-install scenario to recover the original patches
   # prior to the re-install, thus if the first boot flag does not exist then we are in error.
   #
   if (!-e $BASEFIRSTBOOTUP)
   {
      baselog($LOG_ERROR, "Patch reapply function can only be called in a firstbootup scenario", $TRUE);
      return $FALSE;
   }
   baselog($LOG_INFO, "Re-applying base patches.", $TRUE);

   #
   # Create a new .handle_elements db file
   #
   my $ret;  # ret value from do_system
   unless (do_system([$RM, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot remove $HANDLES_ELEMENTS ($ret)");
   }
   unless (do_system([$TOUCH, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot create '$HANDLES_ELEMENTS ($ret)");
   }
   unless (do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, "660", "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot change mode to nortel for '$HANDLES_ELEMENTS ($ret)");
   }
   unless (do_sudo([$SUDO_NORTEL, $PATCH_CHOWN, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot change owner to nortel for $HANDLES_ELEMENTS ($ret)");
   }

   #
   # At this point we should be retrieving the patch list from the originally backup
   # of the patch database to determine which patches to load and install
   #
   @p = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);

   #
   # No need to continue if for some reason the backed-up patch database identifies
   # no patches; we are thus done.
   if (scalar(@p) == 0)
   {
      baselog($LOG_WARNING, "No Patches found in restored patch database.", $TRUE);
      return $TRUE;
   }

   #
   # Sort patch list by pins time
   # - those that are only in a pload'ed state will bubble to the top of the list
   #   and thus the first patches to be pout'ed (as desired)
   #
   @p = sort {($a->{'PINS'}) cmp($b->{'PINS'})} @p;

   # Dump patch list for posterity sake.
   baselog($LOG_INFO, "\n", $TRUE);
   foreach $patch (@p)
   {
      baselog($LOG_INFO, "  p= $patch->{'PID'} ... $patch->{'PATCHNAME'} ... \t $patch->{'PINS'}\n", $TRUE);
   }

   push @all_patches, @p;

   #
   # Process all patches identified in the original patch db
   #
   foreach $patch (@all_patches)
   {
      baselog($LOG_INFO, "\nProcessing PID $patch->{'PID'} $patch->{'PATCHNAME'}\n", $TRUE);

      # if it's the last time through the loop, set $is_last to $TRUE
      if (++$patch_counter == scalar(@all_patches))
      {
         $is_last = $TRUE;
      }

      #
      # We will only consider putting patches that where originally in-service back into service.
      # All the other patches will remain unloaded (pout'ed)
      #
      # set sp and interact flags
      $self->set_app_flags($FALSE, $FALSE);
      $self->set_interact($FALSE);  # Do not prompt user
      $self->set_sp($TRUE);         # Ignore any app startup & server reboots

      if ($patch->{'STATUS'} eq $IN_SERV_REBOOT || $patch->{'STATUS'} eq $IN_SERV)
      {

         # Mark recovered patch as being OOS in patch db
         $self->{'DB'}->put_patch_out_service($patch);
         baselog($LOG_DEBUG, ' Recovered PID ' . $patch->{'PID'} . " is marked as out of service (poos)", 1);

         # Mark recovered patch as being POUTed from patch db
         $self->{'DB'}->pout_patch($patch);
         baselog($LOG_DEBUG, ' Recovered PID ' . $patch->{'PID'} . " is marked as being unloaded (pout)", 1);

         #
         # put patch in service if it's a base or LinuxOS patch (that is, not a Nortel app)
         # - there is no "clean" way of determining whether a patch is a Nortel Application Patch
         #
         #   In all likelyhood our first boot context implies that the Nortel applications will
         #   not have been installed as yet. The Nortel Appl SUs and Patches
         #   recovered will thus fail the  pload and/or pins attempts which we will promply ignore
         #   (but will report to the baselog file)
         #
         #   At the end of the day, only the LinuxOS and Base Applications will be left in service.
         #
         # Reload the original patch
         if (not $self->pload_file($patch->{'FILENAME'}))
         {
            baselog($LOG_WARNING, "Patch $patch->{'FILENAME'} failed to be loaded into db\n", $TRUE);

            #
            # We will ignore failures since any non-installed applications may cause failure
            # at this point. Log the failure and continue onto the next patch.
            $patch_okay = $FALSE;  # Flag a patch failure
            next;
         } ## end if (not $self->pload_file...

         # Get new PID of just loaded patch
         my $npid = $self->{'DB'}->get_patch_id_by_name($patch->{'FILENAME'});
         if (!defined $npid)
         {
            baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} pid does not exist in db\n", $TRUE);
            $patch_okay  = $FALSE;  # Flag a patch failure
            $noerr_found = $FALSE;  # Actual error detected
            next;
         } ## end if (!defined $npid)

         # Get patch of the new pid
         my $np = $self->{'DB'}->get_patch($npid);
         if (!defined $np)
         {
            baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} pid = $npid does not exist\n", $TRUE);
            $patch_okay  = $FALSE;  # Flag a patch failure
            $noerr_found = $FALSE;  # Actual error detected
            next;
         } ## end if (!defined $np)

         # Place patch into service (applist is empty in our context)
         if (not $self->pins_patch($np, $is_last, \@applist))
         {
            baselog($LOG_WARNING, "Patch $patch->{'FILENAME'} pid = $npid failed to be put into service\n", $TRUE);

            #
            # We will ignore failures since any non-installed applications may cause failure
            # at this point. Log the failure and continue onto the next patch.
            $patch_okay = $FALSE;   # Flag a patch failure
            next;
         } ## end if (not $self->pins_patch...
         else
         {
            baselog($LOG_INFO, "Patch $patch->{'FILENAME'} pid = $npid put into service\n", $TRUE);
         }
      } ## end if ($patch->{'STATUS'}...
      elsif ($patch->{'STATUS'} eq $OUT_SERV_REBOOT || $patch->{'STATUS'} eq $OUT_SERV)
      {                             #
                                    # All patches that were originally in a loaded state will be pout'ed
                                    #
                                    # Mark recovered PLOAD'ed patches as being POUTed from patchdb
         $self->{'DB'}->pout_patch($patch);
         baselog($LOG_INFO, " Recovered PID $patch->{'PID'} is marked as being unloaded (pout).", $TRUE);
      } ## end elsif ($patch->{'STATUS'}...
      else
      {

         # If we get here we are in error. Try to mark the patch as pout and hope for the best
         $self->{'DB'}->pout_patch($patch);
         baselog($LOG_ERROR, " Recovered PID $patch->{'PID'} in unknown status - marked as being unloaded (pout)", $TRUE);
         $patch_okay  = $FALSE;     # Flag a patch failure
         $noerr_found = $FALSE;     # Actual error detected
         next;
      } ## end else [ if ($patch->{'STATUS'}...

   } ## end foreach $patch (@all_patches...

   if ($patch_okay)
   {
      baselog($LOG_INFO, "\nRe-applying base patches completed.", $TRUE);
   }
   else
   {
      baselog($LOG_INFO, "\nRe-applying base patches completed (with error).", $TRUE);
   }
   return $noerr_found;
} ## end sub reapply
#################################################################
# Description: This subroutine re-apply patches based on patch
#              database restored during sysrestore operation.
#
# This function is called from patchreaply script, during system
# restore.
#
# Input:  none
#         - assumes that the patch db exists and that all patches
#           also reside in the patch folder
#
# Output: $TRUE  - all in-service patches are in service
#         $FALSE - at least one error occured during uninstall/
#                 install of patches.
#################################################################
sub restore_patches
{
   my $self = shift;
   my (@pout_list, @poos_list, @pins_list) = ((), (), ());
   my $patch;
   my $retval        = $TRUE;
   my $patch_counter = 0;
   my $is_last       = $FALSE;
   my @applist       = ();
   my %seen          = ();
   my $ret;  # ret value from do_system

   #
   # Create a new .handles_elements db file
   #
   unless (do_system([$RM, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot remove $HANDLES_ELEMENTS ($ret).", $TRUE);
   }
   unless (do_system([$TOUCH, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot create '$HANDLES_ELEMENTS ($ret).", $TRUE);
   }
   unless (do_sudo([$SUDO_NORTEL, $PATCH_CHMOD, "660", "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot change mode to nortel for '$HANDLES_ELEMENTS ($ret).", $TRUE);
   }
   unless (do_sudo([$SUDO_NORTEL, $PATCH_CHOWN, "$HANDLES_ELEMENTS"], {$dsRETVAL => \$ret}))
   {
      baselog($LOG_ERROR, "Cannot change owner to nortel for $HANDLES_ELEMENTS ($ret).", $TRUE);
   }

   #
   # At this point we should be retrieving the patch list from the originally backup
   # patch database to determine which patches to load and install
   #

   # Get list of patches that have status in SERV or IN_SERV_REBOOT
   @pins_list = $self->{'DB'}->get_rpm_patches(undef, $ALL_IN_SERVICE, undef);

   # set list of patches to put out of service
   push @poos_list, @pins_list;

   # sort pins_patches by pins time
   @pins_list = sort {($a->{'PINS'}) <=> ($b->{'PINS'})} @pins_list;

   #
   # set sp and interact flags
   $self->set_app_flags($FALSE, $FALSE);
   $self->set_interact($FALSE);  # Do not prompt user
   $self->set_sp($TRUE);         # Ignore any app startup & server reboots

   if (scalar(@poos_list))
   {
      baselog($LOG_INFO, "\nUninstalling patches...\n", $TRUE);
      print("\n");

      # Inside poos_patches, list of patches get sorted by pins time from the last one in service to the newest in service. So this will avoid dependency issues during poos operation. Also patches will be poos before SU's.
      unless ($self->poos_patches(\@poos_list))
      {
         baselog($LOG_ERROR, "Error taking patches out of service.\n", $TRUE);
         $retval = $FALSE;
      }
   } ## end if (scalar(@poos_list)...

   # get the patches with status $OUT_SERV and $OUT_SERV_REBOOT
   # the patches that were originally out of service (from backup DB) will remain unloaded
   @pout_list = $self->{'DB'}->get_rpm_patches(undef, $ALL_OUT_SERVICE, undef);
   if (scalar(@pout_list))
   {
      baselog($LOG_INFO, "\nRemoving patches...\n", $TRUE);
      print("\n");

      # Remove patches
      unless ($self->pout_patches(\@pout_list))
      {
         baselog($LOG_ERROR, "Patch removal failed.\n", $TRUE);
         $retval = $ERROR;
      }
   } ## end if (scalar(@pout_list)...

   # Process all patches identified in the original patch db with status in service or in service reboot

   if (scalar(@pins_list))
   {
      baselog($LOG_INFO, "\nRe-applying patches...\n", $TRUE);
      foreach $patch (@pins_list)
      {
         baselog($LOG_INFO, "\nProcessing PID $patch->{'PID'} $patch->{'PATCHNAME'}\n", $TRUE);

         # if it's the last time through the loop, set $is_last to $TRUE
         if (++$patch_counter == scalar(@pins_list))
         {
            $is_last = $TRUE;
         }

         #
         # We will only consider putting patches that where originally in-service back into service.
         # All the other patches will remain unloaded (pout'ed)
         #
         # if still there is a pid associated to the patch, something went wrong unloading or poosing the patch prior
         # to pins it.   NOT SURE IF THIS CHECK IS NEEDED
         if ($self->{'DB'}->is_patch_loaded($patch->{'PATCHNAME'}))
         {
            baselog($LOG_ERROR, "$patch->{'PATCHNAME'} restoring failed.\n", $TRUE);
            $retval = $FALSE;
            next;
         } ## end if ($self->{'DB'}->is_patch_loaded...
         else
         {

            # put patch in service
            # Reload the original patch
            if (not $self->pload_file($patch->{'FILENAME'}))
            {
               baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} failed to be loaded into db.\n", $TRUE);
               $retval = $FALSE;
               next;
            } ## end if (not $self->pload_file...

            # Get new PID of just loaded patch
            my $npid = $self->{'DB'}->get_patch_id_by_name($patch->{'FILENAME'});
            if (!defined $npid)
            {
               baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} pid does not exist in db.\n", $TRUE);
               $retval = $FALSE;
               next;
            } ## end if (!defined $npid)

            # Get patch of the new pid
            my $np = $self->{'DB'}->get_patch($npid);
            if (!defined $np)
            {
               baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} pid = $npid does not exist.\n", $TRUE);
               $retval = $FALSE;
               next;
            } ## end if (!defined $np)

            # Place patch into service (applist is empty in our context)
            if (not $self->pins_patch($np, $is_last, \@applist))
            {
               baselog($LOG_ERROR, "Patch $patch->{'FILENAME'} pid = $npid failed to be put into service.\n", $TRUE);
               $retval = $FALSE;
               next;
            } ## end if (not $self->pins_patch...
            else
            {
               baselog($LOG_INFO, "Patch $patch->{'FILENAME'} pid = $npid put into service\n", $TRUE);
            }
         } ## end else [ if ($self->{'DB'}->is_patch_loaded...
      } ## end foreach $patch (@pins_list)
   }  ## if (scalar(@pins_patches...
      #
      # Since patches are restored then we need to make sure that hardening is performed
      # thus reapply basic hardening items
      #
   baselog($LOG_INFO, "Reapplying basic hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $BASIC_HARDENIG_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying basic hardening items failed.\n");
   }

   # Reapply enhanced hardening items
   baselog($LOG_INFO, "Reapplying enhanced hardening... Please wait a moment\n", $TRUE);
   if (not do_sudo([$SUDO_NORTEL, $ENHANCED_HARDENING_REAPPLY_POST_PATCHING]))
   {
      baselog($LOG_WARNING, "Reapplying enhanced hardening items failed.");
   }

   if ($retval)
   {
      baselog($LOG_INFO, "\nPatch restore operation completed.", $TRUE);
   }
   else
   {
      baselog($LOG_INFO, "\nPatch restore operation completed (with error).", $TRUE);
   }

   return $retval;
} ## end sub restore_patches

##############################################################################
## @method reboot_required
# @brief checks if reboot required to fully poos/pins any patches
# @details this function checks if any patches are in OUT_SERV_REBOOT or
#              IN_SERV_REBOOT
# @param $_[0] - object reference
# @return $TRUE if there is a patch that requires reboot, otherwise $FALSE
# @author - Dave Cairns
# @date - December 4/08
##############################################################################
sub reboot_required($)
{
   my $self = shift;
   my @patch_list;
   @patch_list = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);
   if (not scalar(@patch_list))  # no patches are in the database, no reboot required
   {
      return $FALSE;
   }
   foreach my $patch (@patch_list)
   {
      if (($patch->{'STATUS'} == $IN_SERV_REBOOT) or ($patch->{'STATUS'} == $OUT_SERV_REBOOT))
      {
         return $TRUE;
      }
   } ## end foreach my $patch (@patch_list...
   return $FALSE;
} ## end sub reboot_required($)

## @method show_plis
# @brief implementation of plis command
# @details
# @param $_[0] - reference to patching object
# @param $_[1] - patch handle to show
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub show_plis($$)
{
   my $self = shift;
   my @patches;
   my $ret = $TRUE;
   my $pid = (scalar(@_)) ? shift(@_) : undef;

   $self->{'action'} = $PLIS;
   @patches = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);
   if (not scalar(@patches))
   {
      baselog($LOG_INFO, "In system patches:" . scalar(@patches) . "\n", $TRUE);
      return $TRUE;
   }

   if (defined $pid)
   {
      unless ($self->{'DB'}->is_patch_loaded_byID($pid))
      {
         baselog($LOG_ERROR, "There is no patch with '$pid' handle\n", $TRUE);
         return $FALSE;
      }

      foreach my $patch (@patches)
      {
         if ($pid == $patch->{'PID'})
         {
            unless ($patch->plis())
            {
               baselog($LOG_ERROR, "Can not get patch status\n", $TRUE);
               $ret = $FALSE;
               last;
            } ## end unless ($patch->plis())
            last;
         } ## end if ($pid == $patch->{'PID'...
      } ## end foreach my $patch (@patches...
   } ## end if (defined $pid)
   else
   {
      baselog($LOG_ERROR, "Patch handle should be defined");
      $ret = $FALSE;
   }

   #remove 'patch' objects
   undef $_ foreach (@patches);
   return $ret;
} ## end sub show_plis($$)

## @method show_pstat
# @brief implementation of pstat command
# @details
# @param $_[0] - reference to patching object
# @param $_[1] - patch handle to show, if not defined all patches are shown
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub show_pstat($$)
{
   my $self = shift;
   my @patches;
   my $ret = $TRUE;
   my $pid = (scalar(@_)) ? shift(@_) : undef;

   $self->{'action'} = $PSTAT;
   @patches = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);
   if (not scalar(@patches))
   {
      baselog($LOG_INFO, "In system patches:" . scalar(@patches) . "\n", $TRUE);
      return $TRUE;
   }

   if (defined $pid)
   {
      unless ($self->{'DB'}->is_patch_loaded_byID($pid))
      {
         baselog($LOG_ERROR, "There is no patch with '$pid' handle\n", $TRUE);
         return $FALSE;
      }

      foreach my $patch (@patches)
      {
         if ($pid == $patch->{'PID'})
         {
            unless ($patch->plis())  ## pstat and plis display format is common now
            {
               baselog($LOG_ERROR, "Can not get patch status\n", $TRUE);
               $ret = $FALSE;
               last;
            } ## end unless ($patch->plis())
            last;
         } ## end if ($pid == $patch->{'PID'...
      } ## end foreach my $patch (@patches...
   } ## end if (defined $pid)
   else
   {

      # Sort patches by pid
      @patches = sort {($a->{'PID'}) <=> ($b->{'PID'})} @patches;

      print "In system patches:" . scalar(@patches) . "\n";

      foreach my $patch (@patches)
      {
         unless ($patch->plis())  ## pstat and plis display format is common now
         {
            baselog($LOG_ERROR, "Can not get patch status\n", $TRUE);
            $ret = $FALSE;
         }
      } ## end foreach my $patch (@patches...
   } ## end else [ if (defined $pid)

   #remove 'patch' objects
   undef $_ foreach (@patches);
   return $ret;
} ## end sub show_pstat($$)

## @method show_pstat_extended
# @brief implementation of pstat command with -l option
# @details
# @param $_[0] - reference to patching object
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub show_pstat_extended($)
{

   my $self = shift;
   my @patches;
   my $ret         = $TRUE;
   my $app_version = 'undefined';

   $self->{'action'} = $PSTAT;
   my $params = use_confver($BASEAPPCONFVER);
   if ($params)
   {
      if ($params->{'APP_VERSION'})
      {
         $app_version = $params->{'APP_VERSION'};
      }
   } ## end if ($params)
   print "Product Release: $app_version\n";

   @patches = $self->{'DB'}->get_rpm_patches(undef, $ALL_PATCHES, undef);

   if (not scalar(@patches))
   {
      baselog($LOG_INFO, "There are no patches in the system\n", $TRUE);
      return $TRUE;
   }

   # Sort patches by pid
   @patches = sort {($a->{'PID'}) <=> ($b->{'PID'})} @patches;

   my ($num_patch, $num_su) = (0, 0);
   foreach my $patch (@patches)
   {
      $num_patch++ if ($patch->is_patch());
      $num_su++    if ($patch->is_service_update());
   }
   print "In system patches: $num_patch\n";
   foreach my $patch (@patches)
   {
      if ($patch->is_patch())
      {
         unless ($patch->pstat_l())
         {
            baselog($LOG_ERROR, "Can not get patch status\n", $TRUE);
            $ret = $FALSE;
         }
      } ## end if ($patch->is_patch()...
   } ## end foreach my $patch (@patches...

   print "\nIn System service updates: $num_su\n";
   foreach my $patch (@patches)
   {
      if ($patch->is_service_update())
      {
         unless ($patch->pstat_l())
         {
            baselog($LOG_ERROR, "Can not get patch status\n", $TRUE);
            $ret = $FALSE;
         }
      } ## end if ($patch->is_service_update...
   } ## end foreach my $patch (@patches...

   #remove 'patch' objects
   undef $_ foreach (@patches);
   return $ret;
} ## end sub show_pstat_extended($)

## @method show_issp
# @brief implementation of issp command
# @details
# @param $_[0] - reference to patching object
# @return $TRUE if success, otherwise $FALSE
# @author
# @date
sub show_issp($)
{

   my $self = shift;
   my @patches;
   my $ret         = $TRUE;
   my $app_version = 'undefined';
   my $machine_type;
   my $host_name;
   my @rpmlist;

   $self->{'action'} = $ISSP;
   open(STDOUT, "|tee " . $PATCHDIR . "/MIRCC.txt");
   my $params = use_confver($BASEAPPCONFVER);
   if ($params)
   {
      if ($params->{'APP_VERSION'})
      {
         $app_version = $params->{'APP_VERSION'};
      }
   } ## end if ($params)
   print "START:\n";
   print "####################\n";
   print "Release: $app_version\n";
   do_system([$UNAME, '-m'], {$dsOUT => \@$machine_type});
   print "Machine: @$machine_type";
   do_system([$HOSTNAME], {$dsOUT => \@$host_name});
   print "Hostname: @$host_name";  #
   print "####################\n";
   print "RPMs:\n";
   ## rpm -qa
   do_system([$RPM, '-qa', '--nosignature', '--qf', '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\n'], {$dsNOERROR => $TRUE, $dsOUT => \@rpmlist});
   foreach my $line (@rpmlist)
   {
      print $line;
   }

   @patches = $self->{'DB'}->get_rpm_patches(undef, $ALL_IN_SERVICE, undef);

   # Sort patches by pid
   @patches = sort {($a->{'PID'}) <=> ($b->{'PID'})} @patches;

   print "####################\n";
   print "SUs:\n";

   foreach my $patch (@patches)
   {
      if ($patch->is_service_update())
      {
         unless ($patch->issp())
         {
            baselog($LOG_ERROR, "Can not get status of SU $patch->{'PATCHNAME'}\n", $TRUE);
            $ret = $FALSE;
         }
      } ## end if ($patch->is_service_update...
   } ## end foreach my $patch (@patches...

   print "####################\n";
   print "Patches:\n";

   foreach my $patch (@patches)
   {
      if ($patch->is_patch())
      {
         unless ($patch->issp())
         {
            baselog($LOG_ERROR, "Can not get status of patch $patch->{'PATCHNAME'}\n", $TRUE);
            $ret = $FALSE;
         }
      } ## end if ($patch->is_patch()...
   } ## end foreach my $patch (@patches...

   print "####################\n";
   print "EOF:\n";
   close STDOUT;

   #remove 'patch' objects
   undef $_ foreach (@patches);
   return $ret;
} ## end sub show_issp($)

## @method remove_all_patches
# @brief implementation of spout command
# @details
# @param $_[0] - reference to patching object
# @return $TRUE if success, otherwise $FALSE
sub remove_all_patches($)
{
   my $self      = shift(@_);
   my $ret_val   = $TRUE;
   my $patchname = "";
   my $bundle_name;
   my $cmd_retval;

   baselog($LOG_DEBUG, "Processing remove_all_patches.\n");
   if (!-d $SPLOAD_DIR)
   {
      baselog($LOG_DEBUG, "There is no SP currently in loaded state.\n", $TRUE);
      return $TRUE;
   }

   # run SP PREPATCH script if exists
   $cmd_retval = $self->run_SP_script($PREPATCH, "spout", $SPLOAD_DIR);
   if ($cmd_retval == $DONE)
   {
      return $TRUE;
   }
   elsif ($cmd_retval == $ERROR)
   {
      return $FALSE;
   }
   baselog($LOG_INFO, "Removing patches loaded by current SP.\n", $TRUE);
   unless ($self->pout_all())
   {
      baselog($LOG_ERROR, "ERROR unloading patches, patch directory will not be cleaned.\n", $TRUE);
      $ret_val = $FALSE;
   }
   else
   {
      if (!-e $CPLIST_FILE)
      {
         baselog($LOG_DEBUG, "File $CPLIST_FILE doesn't exist. There is nothing to remove.\n");
      }
      else
      {
         unless (open(CPLIST, "$CPLIST_FILE"))
         {
            baselog($LOG_ERROR, "Can not open file  $CPLIST_FILE \n", $TRUE);
            $ret_val = $FALSE;
         }
         else
         {
            while (<CPLIST>)
            {
               chomp;
               if ($_ eq "")
               {
                  next;
               }

               $patchname = "$PATCHDIR/$_";
               unless (-e "$patchname")
               {
                  baselog($LOG_ERROR, "Can't find file $patchname to delete.\n", $TRUE);
               }
               else
               {
                  unlink("$patchname");
               }
            } ## end while (<CPLIST>)
            close(CPLIST);
         } ## end else
      } ## end else [ if (!-e $CPLIST_FILE)
   } ## end else

   # run spout POSTPATCH script
   $cmd_retval = $self->run_SP_script($POSTPATCH, "spout", $SPLOAD_DIR);
   if ($cmd_retval == $DONE)
   {
      unless (spec_dir_cleanup($SPLOAD_DIR))
      {
         baselog($LOG_ERROR, "Cannot cleanup spload directory.\n", $TRUE);
         return $FALSE;
      }
      return $TRUE;
   } ## end if ($cmd_retval == $DONE...
   elsif ($cmd_retval == $ERROR)
   {
      return $FALSE;
   }

   # remove SPLOAD_DIR
   unless (spec_dir_cleanup($SPLOAD_DIR))
   {
      baselog($LOG_ERROR, "Cannot cleanup spload directory.\n", $TRUE);
      return $FALSE;
   }
   return $ret_val;
} ## end sub remove_all_patches($)

## @method spout
# @brief implementation of spout command
# @details
# @param $_[0] - reference to patching object
# @return $TRUE if success, otherwise $FALSE
# @author Osmond Ryall
# @date Oct 19, 2008
sub spout($$)
{
   my $self    = shift(@_);
   my $par     = shift(@_);
   my $ret_val = $TRUE;

   $self->{'action'} = $SPOUT;
   if (defined $par->{'interact'})
   {
      $self->set_interact($par->{'interact'});
   }

   if (defined $par->{'sp'})
   {
      $self->set_sp($par->{'sp'});
   }

   unless (-d $SPLOAD_DIR)
   {
      baselog($LOG_INFO, "There is no SP currently in loaded state.\n", $TRUE);
      return $TRUE;
   }

   if ($self->{'interact'})
   {
      unless (getYNResponse("This will remove the currently loaded SP, continue?", 'N'))
      {
         baselog($LOG_INFO, "spout command cancelled .\n", $TRUE);
         return $TRUE;
      }
   } ## end if ($self->{'interact'...

   if (!remove_all_patches($self))
   {
      return $FALSE;
   }
   return $TRUE;
} ## end sub spout($$)

## @fn      set_SP_type($self)
# @brief    Set SP type by header.xml of loaded SP
#           It can be SP, DELTA_SP, SEARCH_SP.
# @param    $self - reference to the object
# @return   TRUE  - SP type is set successfully
# @return   FALSE - otherwise
# @author   Michael Samokhin
# @date     September 2009
sub set_SP_type($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return $FALSE;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return $FALSE;
   }

   my $h_obj;
   my $header_xml = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;
   if (-e $header_xml)
   {

      $h_obj = patchutil::header_xml::new($header_xml);
      if (not defined($h_obj))
      {
         baselog($LOG_ERROR, "Cannot create header_xml object", $TRUE);
         return $FALSE;
      }
      if (not defined $h_obj->{'type'})
      {
         baselog($LOG_ERROR, "Header object type is undefined.", $TRUE);
         return $FALSE;
      }
      $self->{'SP_type'} = $h_obj->{'type'};
   } ## end if (-e $header_xml)
   else
   {
      baselog($LOG_ERROR, "SP header.xml file does NOT exists", $TRUE);
      return $FALSE;
   }
   undef $h_obj;
   return $TRUE;
} ## end sub set_SP_type($)

## @fn      obtain_obsoletes($self, $obsolete_list)
# @brief    Obtain the list of obsolete SUs/patches names from the SP
#           header.xml file included inside the SP bundle.
# @details  Read SP header.xml and get the list of obsolete SUs/patches
#           names specified by OBSOLETE tag.
# @param    $self          - reference to the object
# @param    $obsolete_list - reference to the list of obsolete SU's and patches
# @return   TRUE  - list of obsolete SU's and patches is obtained successfully
# @return   FALSE - cannot obtain list of obsolete SU's and patches
# @author   Michael Samokhin
# @date     September 2009
sub obtain_obsoletes($$)
{
   my ($self, $obsolete_list) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return $FALSE;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return $FALSE;
   }

   my $h_obj;
   my $header_xml = $SPLOAD_DIR . "/" . $PATCHINFO_FILE;
   if (-e $header_xml)
   {

      $h_obj = patchutil::header_xml::new($header_xml);
      if (not defined($h_obj))
      {
         baselog($LOG_ERROR, "Cannot create header_xml object", $TRUE);
         return $FALSE;
      }

      @$obsolete_list = $h_obj->getObsoletes();
      if (scalar(@$obsolete_list) == 0)
      {
         baselog($LOG_DEBUG, "The SP $header_xml file does not contain obsolete SUs/patches");
      }
   } ## end if (-e $header_xml)
   else
   {
      baselog($LOG_ERROR, "SP header.xml file does NOT exists in '$SPLOAD_DIR' directory", $TRUE);
      return $FALSE;
   }
   undef $h_obj;
   return $TRUE;

} ## end sub obtain_obsoletes($$)

## @fn      check_obsolete_and_required_files($self, $required_files, $obsolete_files)
# @brief    Check that there are no any obsolete SUs/patches with the same
#           name as required SUs/patches into SP header.xml
# @param    $self  - reference to object
# @param    $required_files - reference to the list of required SUs and
#                             patches from SP header.xml
# @param    $obsolete_files - reference to the list of obsolete SUs and
#                             patches from SP header.xml
# @return   TRUE  - there are no obsolete files with the same name as
#                   required files into SP header.xml
# @return   FALSE - otherwise
# @author   Michael Samokhin
# @date     September 2009
sub check_obsolete_and_required_files($$$)
{
   my ($self, $required_files, $obsolete_files) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return $FALSE;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return $FALSE;
   }

   my ($req_file, $obs_file);
   if ((scalar(@$required_files) > 0) and (scalar(@$obsolete_files) > 0))
   {
      foreach $req_file (@$required_files)
      {
         foreach $obs_file (@$obsolete_files)
         {
            if ($obs_file eq $req_file->{'FILENAME'})
            {
               return $FALSE;
            }
         } ## end foreach $obs_file (@$obsolete_files...
      } ## end foreach $req_file (@$required_files...
   } ## end if ((scalar(@$required_files...
   return $TRUE;
} ## end sub check_obsolete_and_required_files($$$)

## @fn      compare_SU_version($self, $patch_name)
# @brief    Compare installed RPM version with RPM version of SU in SP.
# @param    $self  - reference to object
# @param    $patch_name - name of patch from header.xml
# @return   $retval - TRUE/FALSE - is installed RPM version smaller/greater than RPM version of SU in SP
# @return   $retval - undef - error case
# @author   Michael Samokhin
# @date     September 2009
sub compare_SU_version($$)
{
   my ($self, $patch_name) = @_;
   my $retval = undef;

   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return $retval;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return $retval;
   }

   my @fields_RPM = ();

   if (not extract_RPM_name($self, $patch_name, \@fields_RPM))
   {
      baselog($LOG_ERROR, "Cannot obtain RPM name from SU $patch_name", $TRUE);
   }
   elsif (defined $fields_RPM[0])
   {
      my $rpm_installed = undef;
      my $rpm_name      = $fields_RPM[0];
      my $rpm_retval;
      do_system([$RPM, '-q', $rpm_name], {$dsNOERROR => $TRUE, $dsOUT => \@$rpm_installed, $dsRETVAL => \$rpm_retval});
      if ($rpm_retval == $OK)
      {
         if (not defined(@$rpm_installed))
         {
            baselog($LOG_ERROR, "Cannot check RPM version installed for RPM $rpm_name.", $TRUE);
         }
         else
         {
            $rpm_installed = @$rpm_installed[0];
            chomp($rpm_installed);

            # if installed RPM is smaller than RPM listed into SP SU list
            # then return TRUE. Otherwise, FALSE.
            if ($rpm_installed lt $patch_name)
            {
               baselog($LOG_DEBUG, "RPM version '$rpm_installed' installed on the system is smaller than RPM version '$patch_name' into SP.");
               $retval = $TRUE;
            }
            else
            {
               baselog($LOG_NOTICE, "RPM version '$rpm_installed' installed on the system is greater or equal to RPM version '$patch_name' into SP.");
               $retval = $FALSE;
            }
         } ## end else [ if (not defined(@$rpm_installed...
      } ## end if ($rpm_retval == $OK...
      else
      {
         baselog($LOG_ERROR, "RPM $rpm_name is not installed on the system.", $TRUE);
      }
   } ## end elsif (defined $fields_RPM...

   return $retval;
} ## end sub compare_SU_version($$)

1;
